#include <agl.h>
#include <jawt.h>
#include <jawt_md.h>
#include "sra_gl4jun_GLjInterface.h"
#include "rendering_mode.h"
#include "version.h"

#ifdef __MACOS__
#include <MacMemory.h>
#include <QDOffscreen.h>
#define malloc(size) NewPtr(size)
#define free(ptr) DisposePtr(ptr)
#define GetPortBitMapForCopyBits(port) (&(port)->portBits)
#endif

/**
 * glj-mac.c:
 *
 * 	@author:	Hoshi Takanori
 * 	@version:	1.2
 * 	@created:	1999/09/06 (by Hoshi Takanori)
 *
 * 	$Id: glj-mac.c,v 1.6 2002/11/20 10:26:56 hoshi Exp $
 */

struct JUN_HANDLE {
	int mode;
	jobject component_or_peer;
	GrafPtr port;
	AGLContext context;
	GWorldPtr gworld;
	int width;
	int height;
	GrafPtr copyWindow;
	int x;
	int y;
	Boolean resize;
};

static const GLint singleBuffer[] = {
	AGL_RGBA,
	AGL_DEPTH_SIZE,	32,
	AGL_NONE
};

static const GLint doubleBuffer[] = {
	AGL_DOUBLEBUFFER,
	AGL_RGBA,
	AGL_DEPTH_SIZE,	32,
	AGL_NONE
};

static const GLint offscreenBuffer[] = {
	AGL_RGBA,
	AGL_PIXEL_SIZE,	32,
	AGL_DEPTH_SIZE,	32,
	AGL_OFFSCREEN,
	AGL_NONE
};

// ==================================================================
/**
 *
 */
JNIEXPORT jstring JNICALL Java_sra_gl4jun_GLjInterface_getVersion
  (JNIEnv *env, jclass cls)
{
	return (*env)->NewStringUTF(env, JUN4JAVA_JNI_VERSION);
}

// ==================================================================
/**
 *
 */
static jboolean rendering2image
  (jint *image, int width, int height, GLenum format)
{
	GLint swapbytes, lsbfirst, rowlength;
	GLint skiprows, skippixels, alignment;
	unsigned char *buffer, *pBuffer;
	unsigned char byte1, byte2, byte3, byte4;
	jint color;
	int x, y;

	// create buffer
	switch (format) {
	case GL_RGB:
		buffer = malloc(width * height * 3);
		break;
	case GL_RGBA:
		buffer = malloc(width * height * 4);
		break;
	default:
		buffer = NULL;
		break;
	}
	if (buffer == NULL) {
		return JNI_FALSE;
	}

	// copy buffer
	glGetIntegerv(GL_UNPACK_SWAP_BYTES, &swapbytes);
	glGetIntegerv(GL_UNPACK_LSB_FIRST, &lsbfirst);
	glGetIntegerv(GL_UNPACK_ROW_LENGTH, &rowlength);
	glGetIntegerv(GL_UNPACK_SKIP_ROWS, &skiprows);
	glGetIntegerv(GL_UNPACK_SKIP_PIXELS, &skippixels);
	glGetIntegerv(GL_UNPACK_ALIGNMENT, &alignment);
	glPixelStorei(GL_UNPACK_SWAP_BYTES, GL_FALSE);
	glPixelStorei(GL_UNPACK_LSB_FIRST, GL_FALSE);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
	glPixelStorei(GL_UNPACK_SKIP_ROWS, 0);
	glPixelStorei(GL_UNPACK_SKIP_PIXELS, 0);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glReadPixels(0, 0, width, height, format, GL_UNSIGNED_BYTE, buffer);
	glPixelStorei(GL_UNPACK_SWAP_BYTES, swapbytes);
	glPixelStorei(GL_UNPACK_LSB_FIRST, lsbfirst);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, rowlength);
	glPixelStorei(GL_UNPACK_SKIP_ROWS, skiprows);
	glPixelStorei(GL_UNPACK_SKIP_PIXELS, skippixels);
	glPixelStorei(GL_UNPACK_ALIGNMENT, alignment);

	// convert buffer
	switch (format) {
	case GL_RGB:
		for (y = height - 1; 0 <= y ; y--) {
			pBuffer = buffer + (width * 3 * y);
			for (x = 0; x < width; x++) {
				byte1 = *pBuffer++;
				byte2 = *pBuffer++;
				byte3 = *pBuffer++;
				color = byte3 | byte2 << 8 | byte1 << 16 | 0xFF << 24;
				*image++ = color;
			}
		}
		break;
	case GL_RGBA:
		for (y = height - 1; 0 <= y ; y--) {
			pBuffer = buffer + (width * 4 * y);
			for (x = 0; x < width; x++) {
				byte1 = *pBuffer++;
				byte2 = *pBuffer++;
				byte3 = *pBuffer++;
				byte4 = *pBuffer++;
				color = byte3 | byte2 << 8 | byte1 << 16 | byte4 << 24;
				*image++ = color;
			}
		}
		break;
	default:
		break;
	}

	// release buffer
	free(buffer);

	return JNI_TRUE;
}

JNIEXPORT jboolean JNICALL Java_sra_gl4jun_GLjInterface_gljReadImage
  (JNIEnv *env, jobject obj, jint jHandle, jintArray jImage, jboolean isRGBA)
{
	struct JUN_HANDLE *handle;
	jint *image;

	// get handle
	if (jHandle == 0) {
		return JNI_FALSE;
	}
	handle = (struct JUN_HANDLE*)jHandle;
	image = (*env)->GetIntArrayElements(env, jImage, 0);

	// copy buffer
	glFlush();
	if (isRGBA == JNI_TRUE) {
		rendering2image(image, handle->width, handle->height, GL_RGBA);
	} else {
		// hanged up when set RGB.
		rendering2image(image, handle->width, handle->height, GL_RGBA);
		//rendering2image(image, handle->width, handle->height, GL_RGB);
	}

	// release handle
	(*env)->ReleaseIntArrayElements(env, jImage, image, 0);

	return JNI_TRUE;
}

// ==================================================================
/**
 *
 */
static jboolean component2port
  (JNIEnv *env, jobject component, GrafPtr *port)
{
	/*
	JAWT awt;
	JAWT_DrawingSurface *ds;
	JAWT_DrawingSurfaceInfo *dsi;
	JAWT_MacDrawingSurfaceInfo *dsi_mac;
	jint lock;

	awt.version = JAWT_VERSION_1_3;
	if (JAWT_GetAWT(env, &awt) == JNI_FALSE) {
		return JNI_FALSE;
	}

	ds = awt.GetDrawingSurface(env, component);
	if (ds == NULL) {
		return JNI_FALSE;
	}

	lock = ds->Lock(ds);
	if ((lock & JAWT_LOCK_ERROR) != 0) {
		awt.FreeDrawingSurface(ds);
		return JNI_FALSE;
	}

	dsi = ds->GetDrawingSurfaceInfo(ds);
	if (dsi == NULL) {
		ds->Unlock(ds);
		awt.FreeDrawingSurface(ds);
		return JNI_FALSE;
	}

	dsi_mac = (JAWT_MacDrawingSurfaceInfo *) dsi->platformInfo;
	*port = dsi_mac->fQDPort;

	ds->Unlock(ds);
	awt.FreeDrawingSurface(ds);

	return JNI_TRUE;
	*/

	// always return false...
	return JNI_FALSE;
}

/**
 *
 */
static jboolean peer2port
  (JNIEnv *env, jobject peer, GrafPtr *port)
{
	jclass cls;
	jmethodID mid, lock, unlock, get_port;
	jobject dsi, pds;

	// get drawing surface info
	// java: DrawingSurfaceInfo dsi = peer.getDrawingSurfaceInfo();
	cls = (*env)->GetObjectClass(env, peer);
	if (cls == 0) { return JNI_FALSE; }
	mid = (*env)->GetMethodID(env, cls,
		"getDrawingSurfaceInfo", "()Lsun/awt/DrawingSurfaceInfo;");
	if (mid == 0) { return JNI_FALSE; }
	dsi = (*env)->CallObjectMethod(env, peer, mid);
	if (dsi == 0) { return JNI_FALSE; }

	// get physical drawing surface
	// java: dsi.lock();
	// java: PhysicalDrawingSurface pds = dsi.getSurface();
	// java: dsi.unlock();
	cls = (*env)->GetObjectClass(env, dsi);
	if (cls == 0) { return JNI_FALSE; }
	lock = (*env)->GetMethodID(env, cls, "lock", "()I");
	if (lock == 0) { return JNI_FALSE; }
	unlock = (*env)->GetMethodID(env, cls, "unlock", "()V");
	if (unlock == 0) { return JNI_FALSE; }
	mid = (*env)->GetMethodID(env, cls,
		"getSurface", "()Lsun/awt/PhysicalDrawingSurface;");
	if (mid == 0) { return JNI_FALSE; }
	(*env)->CallIntMethod(env, dsi, lock);
	pds = (*env)->CallObjectMethod(env, dsi, mid);
	(*env)->CallVoidMethod(env, dsi, unlock);
	if (pds == 0) { return JNI_FALSE; }

	// get graf port
	// java: dsi.lock();
	// java: *port = pds.getPort();
	// java: dsi.unlock();
	cls = (*env)->GetObjectClass(env, pds);
	if (cls == 0) { return JNI_FALSE; }
	get_port = (*env)->GetMethodID(env, cls, "getPort", "()I");
	if (get_port == 0) { return JNI_FALSE; }
	(*env)->CallIntMethod(env, dsi, lock);
	*port = (GrafPtr) (*env)->CallIntMethod(env, pds, get_port);
	(*env)->CallVoidMethod(env, dsi, unlock);

	return JNI_TRUE;
}

/**
 *
 */
static jboolean get_port
  (JNIEnv *env, jobject component_or_peer, GrafPtr *port)
{
	jclass cls;
	jmethodID mid;
	jobject peer;
	jboolean result = JNI_FALSE;

	cls = (*env)->FindClass(env, "java/awt/Component");
	if (cls == 0) { return result; }

	if ((*env)->IsInstanceOf(env, component_or_peer, cls)) {
		// new behavior (build 29 and later).

		// first, call component2port().
		result = component2port(env, component_or_peer, port);

		// if it fails, call peer2port().
		if (result == JNI_FALSE) {
			mid = (*env)->GetMethodID(env, cls,
				"getPeer", "()Ljava/awt/peer/ComponentPeer;");
			if (mid != 0) {
				peer = (*env)->CallObjectMethod(env, component_or_peer, mid);
				if (peer != 0) {
					result = peer2port(env, peer, port);
				}
			}

			// clear exception, if any.
			(*env)->ExceptionClear(env);
		}
	} else {
		// old behavior (build 28 and before).

		// just call peer2port().
		result = peer2port(env, component_or_peer, port);

		// don't clear exception...
	}

	return result;
}

/*
 *
 */
JNIEXPORT jint JNICALL Java_sra_gl4jun_GLjInterface_gljCreateContext__Ljava_lang_Object_2III
  (JNIEnv *env, jobject obj, jobject component_or_peer, jint width, jint height, jint mode)
{
	GrafPtr port = NULL, copyWindow = NULL;
	AGLPixelFormat format;
	AGLContext context;
	Rect bounds;
	GWorldPtr gworld;
	PixMapHandle pixmap;
	void *baseaddr;
	int rowbytes;
	struct JUN_HANDLE *handle;

	// create resource
	switch (mode) {
	case RENDER_SINGLE_BUFFER:
		if (get_port(env, component_or_peer, &port) == JNI_FALSE) {
			return 0;
		}
		format = aglChoosePixelFormat(NULL, 0, singleBuffer);
		context = aglCreateContext(format, NULL);
		aglSetDrawable(context, port);
		aglDestroyPixelFormat(format);
		break;
	case RENDER_DOUBLE_BUFFER:
		if (get_port(env, component_or_peer, &port) == JNI_FALSE) {
			return 0;
		}
		format = aglChoosePixelFormat(NULL, 0, doubleBuffer);
		context = aglCreateContext(format, NULL);
		aglSetDrawable(context, port);
		aglDestroyPixelFormat(format);
		break;
	case RENDER_DIRECT:
		if (get_port(env, component_or_peer, &copyWindow) == JNI_FALSE) {
			return 0;
		}
		// no break
	case RENDER_IMAGE:
		// create gworld
		SetRect(&bounds, 0, 0, width, height);
		if (NewGWorld(&gworld, 32, &bounds, NULL, NULL, 0) != noErr) {
			return 0;
		}
		format = aglChoosePixelFormat(NULL, 0, offscreenBuffer);
		context = aglCreateContext(format, NULL);
		pixmap = GetGWorldPixMap(gworld);
		LockPixels(pixmap);
		baseaddr = GetPixBaseAddr(pixmap);
		rowbytes = (**pixmap).rowBytes & 0x7fff;
		aglSetOffScreen(context, width, height, rowbytes, baseaddr);
		aglDestroyPixelFormat(format);
		break;
	default:
		return 0;
	}

	// set handle
	handle = malloc(sizeof(struct JUN_HANDLE));
	if (handle == NULL) {
		return 0;
	}
	handle->mode = mode;
	handle->component_or_peer = (component_or_peer != 0)
			? (*env)->NewGlobalRef(env, component_or_peer) : 0;
	handle->port = port;
	handle->context = context;
	handle->gworld = gworld;
	handle->width = width;
	handle->height = height;
	handle->copyWindow = copyWindow;
	handle->x = 0;
	handle->y = 0;
	handle->resize = 0;

	return (jint)handle;
}

/*
 *
 */
JNIEXPORT jboolean JNICALL Java_sra_gl4jun_GLjInterface_gljDeleteContext
  (JNIEnv *env, jobject obj, jint jHandle, jboolean isApplet)
{
	struct JUN_HANDLE *handle;

	// get handle
	if (jHandle == 0) {
		return JNI_FALSE;
	}
	handle = (struct JUN_HANDLE*)jHandle;

	// delete resource
	switch (handle->mode) {
	case RENDER_SINGLE_BUFFER:
	case RENDER_DOUBLE_BUFFER:
		aglDestroyContext(handle->context);
		break;
	case RENDER_DIRECT:
	case RENDER_IMAGE:
		aglDestroyContext(handle->context);
		UnlockPixels(GetGWorldPixMap(handle->gworld));
		DisposeGWorld(handle->gworld);
		break;
	}

	// release handle
	if (handle->component_or_peer != 0) {
		(*env)->DeleteGlobalRef(env, handle->component_or_peer);
	}

	return JNI_TRUE;
}

/*
 *
 */
JNIEXPORT jboolean JNICALL Java_sra_gl4jun_GLjInterface_gljFlushBuffer
  (JNIEnv *env, jobject obj, jint jHandle)
{
	struct JUN_HANDLE *handle;
	GrafPtr port;
	Rect r, portRect;

	// get handle
	if (jHandle == 0) {
		return JNI_FALSE;
	}
	handle = (struct JUN_HANDLE*)jHandle;

	// flush buffer
	switch (handle->mode) {
	case RENDER_DOUBLE_BUFFER:
		glFlush();
		aglSwapBuffers(handle->context);
		aglSetCurrentContext(NULL);
		break;
	case RENDER_SINGLE_BUFFER:
	case RENDER_IMAGE:
		glFlush();
		aglSetCurrentContext(NULL);
		break;
	case RENDER_DIRECT:
		glFlush();
		aglSetCurrentContext(NULL);
		// a magic to set curent port...
		get_port(env, handle->component_or_peer, &port);
		// copy bits
		SetRect(&r, 0, 0, handle->width, handle->height);
		portRect = r;
		OffsetRect(&portRect, handle->x, handle->y);
		CopyBits(GetPortBitMapForCopyBits(handle->gworld),
				GetPortBitMapForCopyBits(port),
				&r, &portRect, srcCopy, NULL);
		break;
	}

	return JNI_TRUE;
}

/*
 *
 */
JNIEXPORT jboolean JNICALL Java_sra_gl4jun_GLjInterface_gljMakeCurrent
  (JNIEnv *env, jobject obj, jint jHandle)
{
	struct JUN_HANDLE *handle;
	int width, height;
	GrafPtr port;
	AGLPixelFormat format;
	AGLContext context;
	Rect bounds;
	GWorldPtr gworld;
	PixMapHandle pixmap;
	void *baseaddr;
	int rowbytes;

	// get handle
	if (jHandle == 0) {
		return JNI_FALSE;
	}
	handle = (struct JUN_HANDLE*)jHandle;
	width = handle->width;
	height = handle->height;

	if (handle->resize == 0) {
		// make current
		if (handle->mode == RENDER_SINGLE_BUFFER
		 || handle->mode == RENDER_DOUBLE_BUFFER) {
			// a magic to set curent port...
			get_port(env, handle->component_or_peer, &port);
		}
		aglSetCurrentContext(handle->context);
		glViewport(0, 0, width, height);
	} else {
		switch (handle->mode) {
		case RENDER_SINGLE_BUFFER:
		case RENDER_DOUBLE_BUFFER:
			// a magic to set curent port...
			get_port(env, handle->component_or_peer, &port);
			aglSetDrawable(handle->context, port);
			// make current and set viewport
			aglSetCurrentContext(handle->context);
			glViewport(0, 0, width, height);
			break;
		case RENDER_IMAGE:
		case RENDER_DIRECT:
			// delete resource
			aglDestroyContext(handle->context);
			gworld = handle->gworld;
			pixmap = GetGWorldPixMap(gworld);
			UnlockPixels(pixmap);
			DisposeGWorld(gworld);

			// create context
			format = aglChoosePixelFormat(NULL, 0, offscreenBuffer);
			context = aglCreateContext(format, NULL);
			aglDestroyPixelFormat(format);

			// create gworld
			SetRect(&bounds, 0, 0, width, height);
			if (NewGWorld(&gworld, 32, &bounds, NULL, NULL, 0) == noErr) {
				pixmap = GetGWorldPixMap(gworld);
				LockPixels(pixmap);
				baseaddr = GetPixBaseAddr(pixmap);
				rowbytes = (**pixmap).rowBytes & 0x7fff;
				aglSetOffScreen(context, width, height, rowbytes, baseaddr);
			}

			// make current
			aglSetCurrentContext(context);
			glViewport(0, 0, width, height);

			// set handle
			handle->context = context;
			handle->gworld = gworld;
			break;
		}
		handle->resize = 0;
	}

	return JNI_TRUE;
}

/*
 *
 */
JNIEXPORT jboolean JNICALL Java_sra_gl4jun_GLjInterface_gljSetLocation
  (JNIEnv *env, jobject obj, jint jHandle, jint x, jint y)
{
	struct JUN_HANDLE *handle;

	// get handle
	if (jHandle == 0) {
		return JNI_FALSE;
	}
	handle = (struct JUN_HANDLE*)jHandle;

	// set location
	handle->x = x;
	handle->y = y;

	return JNI_TRUE;
}

/*
 *
 */
JNIEXPORT jboolean JNICALL Java_sra_gl4jun_GLjInterface_gljSetSize
  (JNIEnv *env, jobject obj, jint jHandle, jint width, jint height)
{
	struct JUN_HANDLE *handle;

	// get handle
	if (jHandle == 0) {
		return JNI_FALSE;
	}
	handle = (struct JUN_HANDLE*)jHandle;

	// set handle
	if (handle->width != width || handle->height != height) {
		handle->width = width;
		handle->height = height;
		handle->resize = 1;
	}

	return JNI_TRUE;
}
