package sra.gl4jun;

import java.awt.*;
import java.awt.event.*;

/**
 * GLjCanvas class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/08/19 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: GLjCanvas.java,v 6.0 2003/10/28 10:52:02 nisinaka Exp $
 */
public class GLjCanvas extends Canvas implements ComponentListener {
	//
	private int renderingMode = GLjRenderingMode.DOUBLE_BUFFER;
	//
	private transient GLjRenderingContext renderingContext;
	private transient boolean resizeRequest;

	/**
	 * GLjCanvas constructor comment.
	 */
	public GLjCanvas() {
		super();
		this.addComponentListener(this);
	}

	/**
	 * Invoked when the component has been made invisible.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void componentHidden(ComponentEvent e) {
	}

	/**
	 * Invoked when the component's position changes.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void componentMoved(ComponentEvent e) {
	}

	/**
	 * Invoked when the component's size changes.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void componentResized(ComponentEvent e) {
		resizeRequest = true;
	}

	/**
	 * Invoked when the component has been made visible.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void componentShown(ComponentEvent e) {
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param g java.awt.Graphics
	 */
	public void paint(Graphics g) {
		this.render(g);
	}

	/**
	 * DOCUMENT ME!
	 */
	public void removeNotify() {
		if (renderingContext != null) {
			Container c = getParent();

			while ((c != null) && (c instanceof java.applet.Applet == false)) {
				c = c.getParent();
			}

			renderingContext.gljDelete(c != null);
			renderingContext = null;
		}

		super.removeNotify();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aMode int
	 */
	public void setRenderingMode(int aMode) {
		switch (aMode) {
			case GLjRenderingMode.NOTHING :
			case GLjRenderingMode.IMAGE :
			case GLjRenderingMode.SINGLE_BUFFER :
			case GLjRenderingMode.DIRECT :
			case GLjRenderingMode.DOUBLE_BUFFER :
				renderingMode = aMode;
				break;
			default :
				//throw new PropertyVetoException(
				//	"setMode() change rejected:",
				//	new PropertyChangeEvent(this, "mode", new Integer(renderingMode), new Integer(aMode)));
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param g java.awt.Graphics
	 */
	public void update(Graphics g) {
		paint(g);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.gl4jun.GLjRenderingContext
	 */
	protected GLjRenderingContext defaultRenderingContext() {
		return new GLjRenderingContext(this);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param g java.awt.Graphics
	 */
	protected void render(Graphics g) {
		if (this.isShowing() == false) {
			return;
		}
		synchronized (GLjInterface.Current()) {
			if (renderingContext == null) {
				renderingContext = this.defaultRenderingContext();
				renderingContext.setMode(renderingMode);
				resizeRequest = false;
				renderingContext.gljInitialize();
			}
		}

		if (renderingContext.getMode() != GLjRenderingMode.NOTHING) {
			synchronized (renderingContext.OpenGLInterface()) {
				if (resizeRequest) {
					renderingContext.gljResize(this.getSize());
					resizeRequest = false;
				}

				renderingContext.gljCurrent();
				this.render(renderingContext);

				if (renderingContext.getMode() == GLjRenderingMode.IMAGE) {
					Image anImage = renderingContext.gljImageRGB();
					g.drawImage(anImage, 0, 0, null);
				}

				renderingContext.gljFlush();
			}
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param openGLInterface sra.gl4jun.GLjInterface
	 */
	protected void render(GLjInterface openGLInterface) {
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aRenderingContext sra.gl4jun.GLjRenderingContext
	 */
	protected void render(GLjRenderingContext aRenderingContext) {
		this.render(aRenderingContext.OpenGLInterface());
	}
}
