package sra.gl4jun;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * GLjJCanvas class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/09/01 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: GLjJCanvas.java,v 6.0 2003/10/28 10:52:02 nisinaka Exp $
 */
public class GLjJCanvas extends JComponent implements ComponentListener {
	//
	private int renderingMode;
	//
	private transient GLjRenderingContext renderingContext;
	private transient boolean resizeRequest;
	private transient boolean moveRequest;

	/**
	 * GLjJCanvas constructor comment.
	 */
	public GLjJCanvas() {
		super();
		this.setPreferredSize(new Dimension(256, 256));
		this.addComponentListener(this);
		this.setRenderingMode(GLjRenderingMode.IMAGE);
	}

	/**
	 * Invoked when the component has been made invisible.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void componentHidden(ComponentEvent e) {
	}

	/**
	 * Invoked when the component's position changes.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void componentMoved(ComponentEvent e) {
		moveRequest = true;
	}

	/**
	 * Invoked when the component's size changes.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void componentResized(ComponentEvent e) {
		resizeRequest = true;
	}

	/**
	 * Invoked when the component has been made visible.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void componentShown(ComponentEvent e) {
	}

	/**
	 * DOCUMENT ME!
	 */
	public void removeNotify() {
		if (renderingContext != null) {
			Container c = getParent();

			while ((c != null) && (c instanceof java.applet.Applet == false)) {
				c = c.getParent();
			}

			renderingContext.gljDelete(c != null);
			renderingContext = null;
		}

		super.removeNotify();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aMode int
	 */
	public void setRenderingMode(int aMode) {
		switch (aMode) {
			case GLjRenderingMode.NOTHING :
			case GLjRenderingMode.IMAGE :
				renderingMode = aMode;
				RepaintManager.currentManager(this).setDoubleBufferingEnabled(true);

				break;
			case GLjRenderingMode.DIRECT :
			case GLjRenderingMode.SINGLE_BUFFER :
			case GLjRenderingMode.DOUBLE_BUFFER :
				renderingMode = aMode;
				RepaintManager.currentManager(this).setDoubleBufferingEnabled(false);

				break;
			default :
				//throw new PropertyVetoException(
				//	"setMode() change rejected:",
				//	new PropertyChangeEvent(this, "mode", new Integer(renderingMode), new Integer(aMode)));
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.gl4jun.GLjRenderingContext
	 */
	protected GLjRenderingContext defaultRenderingContext() {
		Container w = this;

		while (!(w instanceof Window)) {
			w = w.getParent();
		}

		return new GLjRenderingContext(this, (JFrame) w);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param g java.awt.Graphics
	 */
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		this.render(g);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param g java.awt.Graphics
	 */
	protected void render(Graphics g) {
		if (this.isShowing() == false) {
			return;
		}
		synchronized (GLjInterface.Current()) {
			if (renderingContext == null) {
				renderingContext = this.defaultRenderingContext();
				renderingContext.setMode(renderingMode);
				resizeRequest = false;
				moveRequest = true;
				renderingContext.gljInitialize();
			}
		}

		if (renderingContext.getMode() != GLjRenderingMode.NOTHING) {
			synchronized (renderingContext.OpenGLInterface) {
				if (resizeRequest) {
					renderingContext.gljResize(this.getSize());
					resizeRequest = false;
				}

				if (moveRequest) {
					Point rootPoint = this.getRootPane().getLocationOnScreen();
					Point thisPoint = this.getLocationOnScreen();
					Point location = new Point(thisPoint.x - rootPoint.x, thisPoint.y - rootPoint.y);
					renderingContext.gljSetLocation(location);
					moveRequest = false;
				}

				renderingContext.gljCurrent();
				this.render(renderingContext);

				if (renderingContext.getMode() == GLjRenderingMode.IMAGE) {
					Image anImage = renderingContext.gljImageRGB();
					g.drawImage(anImage, 0, 0, null);
				}

				renderingContext.gljFlush();
			}
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aRenderingContext sra.gl4jun.GLjRenderingContext
	 */
	protected void render(GLjRenderingContext aRenderingContext) {
		this.render(aRenderingContext.OpenGLInterface);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param openGLInterface sra.gl4jun.GLjInterface
	 */
	protected void render(GLjInterface openGLInterface) {
	}
}
