package sra.jun.breps.basic;

import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.opengl.objects.JunOpenGL3dObject;
import sra.jun.topology.elements.JunBody;

/**
 * JunBreps class
 * 
 *  @author    nisinaka
 *  @created   1998/10/19 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunBreps.java,v 6.0 2003/10/28 10:52:02 nisinaka Exp $
 */
public class JunBreps extends sra.jun.solid.abstracts.JunSolid {
	protected JunBody body;

	/**
	 * Create a new instance of JunBreps.
	 * 
	 * @param aJunBody sra.jun.topology.elements.JunBody
	 */
	public JunBreps(JunBody aJunBody) {
		this.setBody_(aJunBody);
	}

	/**
	 * Create a new instance of JunBreps.
	 * 
	 * @param aJunBody sra.jun.topology.elements.JunBody
	 * 
	 * @return sra.jun.breps.basic.JunBreps
	 */
	public static JunBreps On_(JunBody aJunBody) {
		return new JunBreps(aJunBody);
	}

	/**
	 * Convert to JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		return body.asJunOpenGL3dObject();
	}

	/**
	 * Answer the body.
	 * 
	 * @return sra.jun.topology.elements.JunBody
	 */
	public JunBody body() {
		return body;
	}

	/**
	 * Answer true if the receiver contains the specified point, otherwise
	 * false.
	 * 
	 * @param aJun3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return boolean
	 */
	public boolean containsPoint_(Jun3dPoint aJun3dPoint) {
		return body.containsPoint_(aJun3dPoint);
	}

	/**
	 * Answer a value at the specified point.
	 * 
	 * @param aJun3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return double
	 */
	public double valueAt_(Jun3dPoint aJun3dPoint) {
		if (this.containsPoint_(aJun3dPoint)) {
			return 1;
		} else {
			return -1;
		}
	}

	/**
	 * Set the body.
	 * 
	 * @param aJunBody sra.jun.topology.elements.JunBody
	 */
	protected void setBody_(JunBody aJunBody) {
		body = aJunBody;
	}
}
