package sra.jun.collections.sequences;

import java.awt.*;
import java.io.*;
import sra.smalltalk.*;

/**
 * JunMatrix class
 * 
 *  @author    nisinaka
 *  @created   1998/10/09 (by nisinaka)
 *  @updated   2002/07/18 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMatrix.java,v 6.0 2003/10/28 10:52:02 nisinaka Exp $
 */
public class JunMatrix extends JunSequence {
	/** The size of the matrix's row. */
	protected int rowSize;

	/** The size of the matrix's column. */
	protected int columnSize;

	/** The array which holds the matrix's elements. */
	protected Object[][] _matrix;

	/**
	 * Create a new instance of matrix.
	 */
	public JunMatrix() {
		this(0);
	}

	/**
	 * Create a new instance of matrix.
	 * 
	 * @param squareSize int
	 */
	public JunMatrix(int squareSize) {
		this(squareSize, squareSize);
	}

	/**
	 * Create a new instance of matrix and initialize it with collection.
	 * 
	 * @param squareSize int
	 * @param collection java.lang.Object[]
	 */
	public JunMatrix(int squareSize, Object[] collection) {
		this(squareSize);
		this.collection_(collection);
	}

	/**
	 * Create a new instance of matrix.
	 * 
	 * @param rowSize int
	 * @param columnSize int
	 */
	public JunMatrix(int rowSize, int columnSize) {
		this.rowSize = rowSize;
		this.columnSize = columnSize;
		this._matrix = new Object[rowSize][columnSize];
	}

	/**
	 * Create a new instance of matrix and initialize it with collection.
	 * 
	 * @param rowSize int
	 * @param columnSize int
	 * @param collection java.lang.Object[]
	 */
	public JunMatrix(int rowSize, int columnSize, Object[] collection) {
		this(rowSize, columnSize);
		this.collection_(collection);
	}

	/**
	 * Anser the object at the position specified with rowNumber and
	 * columnNumber.
	 * 
	 * @param rowNumber int
	 * @param columnNumber int
	 * 
	 * @return java.lang.Object
	 */
	public Object _at(int rowNumber, int columnNumber) {
		return _matrix[rowNumber][columnNumber];
	}

	/**
	 * Put 'anObject' at the position specified with 'rowNumber' and
	 * 'columnNumber'.
	 * 
	 * @param rowNumber int
	 * @param columnNumber int
	 * @param anObject java.lang.Object
	 */
	public void _put(int rowNumber, int columnNumber, Object anObject) {
		_matrix[rowNumber][columnNumber] = anObject;
	}

	/**
	 * Convert this matrix as an array.
	 * 
	 * @return java.lang.Object[]
	 */
	public Object[] asArray() {
		Object[] anArray = new Object[rowSize * columnSize];
		int index = 0;

		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				anArray[index] = this._at(i, j);
				index++;
			}
		}

		return anArray;
	}

	/**
	 * Answer the array of objects in the specified column.
	 * 
	 * @param anInteger int
	 * 
	 * @return java.lang.Object[]
	 */
	public Object[] atColumn_(int anInteger) {
		Object[] column = new Object[rowSize];

		for (int i = 0; i < rowSize; i++) {
			column[i] = this._at(i, anInteger);
		}

		return column;
	}

	/**
	 * Answer the array of objects in the specified row.
	 * 
	 * @param anInteger int
	 * 
	 * @return java.lang.Object[]
	 */
	public Object[] atRow_(int anInteger) {
		Object[] row = new Object[columnSize];

		for (int j = 0; j < columnSize; j++) {
			row[j] = this._at(anInteger, j);
		}

		return row;
	}

	/**
	 * Set all elements of the matrix from 'collection'.
	 * 
	 * @param collection java.lang.Object[]
	 */
	public void collection_(Object[] collection) {
		int index = 0;

		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				_matrix[i][j] = collection[index];
				index++;
			}
		}
	}

	/**
	 * Answer the size of column.
	 * 
	 * @return int
	 */
	public int columnSize() {
		return columnSize;
	}

	/**
	 * Create an entire copy of this matrix.
	 * 
	 * @return sra.smalltalk.StObject
	 */
	public StObject copy() {
		JunMatrix newMatrix = (JunMatrix) _New(this.species(), new Integer(rowSize), new Integer(columnSize));

		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				newMatrix._put(i, j, this._at(i, j));
			}
		}

		return newMatrix;
	}

	/**
	 * Emunerate all the elements and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object do_(final StBlockClosure aBlock) {
		return this.doIJ_(new StBlockClosure() {
			public Object value_value_value_(Object v, Object i, Object j) {
				return aBlock.value_(v);
			}
		});
	}

	/**
	 * Emunerate all the elements and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object doIJ_(StBlockClosure aBlock) {
		int rowSize = this.rowSize();
		int columnSize = this.columnSize();

		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				Object result = aBlock.value_value_value_(_matrix[i][j], new Integer(i), new Integer(j));

				if (result != null) {
					return result;
				}
			}
		}

		return null;
	}

	/**
	 * Answer true if this matrix is square, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean isSquare() {
		return rowSize == columnSize;
	}

	/**
	 * Answer a point which represents this matrix size.
	 * 
	 * @return java.awt.Point
	 */
	public Point matrixSize() {
		return new Point(rowSize, columnSize);
	}

	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * 
	 * @throws IOException DOCUMENT ME!
	 */
	public void printOn_(Writer aWriter) throws IOException {
		aWriter.write("Matrix (");

		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				aWriter.write(_matrix[i][j].toString());

				if (j < (columnSize - 1)) {
					aWriter.write(" ");
				}
			}

			if (i < (rowSize - 1)) {
				aWriter.write(" , ");
			}
		}

		aWriter.write(")");
	}

	/**
	 * Calculate the product of this matrix and aMatrix.
	 * 
	 * @param aMatrix sra.jun.collections.sequences.JunMatrix
	 * 
	 * @return sra.jun.collections.sequences.JunMatrix
	 */
	public JunMatrix product_(JunMatrix aMatrix) {
		if (columnSize != aMatrix.rowSize) {
			throw SmalltalkException.Error("my column size is not equal to your row size.");
		}

		JunMatrix newMatrix = (JunMatrix) _New(this.species(), new Integer(rowSize), new Integer(aMatrix.columnSize));

		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < aMatrix.columnSize; j++) {
				newMatrix._put(i, j, _calculateProductElementAt(i, j, aMatrix));
			}
		}

		return newMatrix;
	}

	/**
	 * Anser the object at the position specified with rowNumber and
	 * columnNumber.
	 * 
	 * @param rowNumber int
	 * @param columnNumber int
	 * 
	 * @return java.lang.Object
	 */
	public Object row_column_(int rowNumber, int columnNumber) {
		return this._at(rowNumber, columnNumber);
	}

	/**
	 * Put 'anObject' at the position specified with 'rowNumber' and
	 * 'columnNumber'.
	 * 
	 * @param rowNumber int
	 * @param columnNumber int
	 * @param anObject java.lang.Object
	 */
	public void row_column_put_(int rowNumber, int columnNumber, Object anObject) {
		this._put(rowNumber, columnNumber, anObject);
	}

	/**
	 * Enumerate all the values at the specified row and evaluate the block.
	 * 
	 * @param anInteger int
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object rowIndex_doJ_(int anInteger, StBlockClosure aBlock) {
		for (int j = 0; j < columnSize; j++) {
			Object result = aBlock.value_value_(_matrix[anInteger][j], new Integer(j));

			if (result != null) {
				return result;
			}
		}

		return null;
	}

	/**
	 * Answer the size of row.
	 * 
	 * @return int
	 */
	public int rowSize() {
		return rowSize;
	}

	/**
	 * Answer the number of all elements.
	 * 
	 * @return int
	 */
	public int size() {
		return this.columnSize() * this.rowSize();
	}

	/**
	 * Calculate a product element at i & j with aMatrix.
	 * 
	 * @param i int
	 * @param j int
	 * @param aMatrix sra.jun.collections.sequences.JunMatrix
	 * 
	 * @return java.lang.Object
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	protected Object _calculateProductElementAt(int i, int j, JunMatrix aMatrix) {
		throw new SmalltalkException("don't know how to calculate a product element.");
	}
}
