package sra.jun.csg.abstracts;

import sra.jun.csg.atoms.JunTransformedCSG;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.boundaries.JunBorderGenerator;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.octree.basic.JunOctree;
import sra.jun.opengl.objects.JunOpenGL3dObject;
import sra.jun.solid.abstracts.JunSolid;
import sra.jun.topology.elements.JunBody;
import sra.smalltalk.*;

/**
 * JunCSG class
 * 
 *  @author    nisinaka
 *  @created   1998/10/18 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCSG.java,v 6.0 2003/10/28 10:52:03 nisinaka Exp $
 */
public abstract class JunCSG extends JunSolid {

	/**
	 * Convert to a JunBody.
	 * 
	 * @param xInterval sra.smalltalk.StInterval
	 * @param yInterval sra.smalltalk.StInterval
	 * @param zInterval sra.smalltalk.StInterval
	 * 
	 * @return sra.jun.topology.elements.JunBody
	 */
	public JunBody asJunBodyXInterval_yInterval_zInterval_(StInterval xInterval, StInterval yInterval, StInterval zInterval) {
		final JunCSG self = this;

		return JunBorderGenerator.ContourBodyFrom_at_sign_xInterval_yInterval_zInterval_interim_(new StBlockClosure() {
			public Object value_value_value_(Object x, Object y, Object z) {
				return new Double(self.valueAt_(new Jun3dPoint(((Number) x).doubleValue(), ((Number) y).doubleValue(), ((Number) z).doubleValue())));
			}
		}, 0, 1, xInterval, yInterval, zInterval, new StBlockClosure());
	}

	/**
	 * Convert as a boxel octree with the specified conditions.
	 * 
	 * @param anInteger int
	 * @param aJun3dBoundingBox sra.jun.geometry.basic.Jun3dBoundingBox
	 * 
	 * @return sra.jun.octree.basic.JunOctree
	 */
	public JunOctree asJunBoxelOctreeDepth_bounds_(int anInteger, Jun3dBoundingBox aJun3dBoundingBox) {
		JunOctree octree = JunOctree.Bounds_(aJun3dBoundingBox);
		final JunCSG self = this;
		octree.block_depth_(new StBlockClosure() {
			public Object value_(Object point) {
				return new Double(self.valueAt_((Jun3dPoint) point));
			}
		}, anInteger);

		return octree;
	}

	/**
	 * Convert as a b-reps octree with the specified conditions.
	 * 
	 * @param anInteger int
	 * @param aJun3dBoundingBox sra.jun.geometry.basic.Jun3dBoundingBox
	 * 
	 * @return sra.jun.octree.basic.JunOctree
	 */
	public JunOctree asJunBrepsOctreeDepth_bounds_(int anInteger, Jun3dBoundingBox aJun3dBoundingBox) {
		JunOctree octree = JunOctree.Bounds_(aJun3dBoundingBox);
		final JunCSG self = this;
		octree.brepsBlock_depth_(new StBlockClosure() {
			public Object value_(Object point) {
				return new Double(self.valueAt_((Jun3dPoint) point));
			}
		}, anInteger);

		return octree;
	}

	/**
	 * Convert to a JunOpenGL3dObject.
	 * 
	 * @param xInterval sra.smalltalk.StInterval
	 * @param yInterval sra.smalltalk.StInterval
	 * @param zInterval sra.smalltalk.StInterval
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject asJunOpenGL3dObjectXInterval_yInterval_zInterval_(StInterval xInterval, StInterval yInterval, StInterval zInterval) {
		return this.asJunBodyXInterval_yInterval_zInterval_(xInterval, yInterval, zInterval).asJunOpenGL3dObject();
	}

	/**
	 * Answer true if the receiver contains the specified point, otherwise
	 * false.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return boolean
	 */
	public boolean containsPoint_(sra.jun.geometry.basic.Jun3dPoint a3dPoint) {
		return this.valueAt_(a3dPoint) >= 0;
	}

	/**
	 * Answer the JunCSG which is applied the specified transformation.
	 * 
	 * @param aJun3dTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.csg.abstracts.JunCSG
	 */
	public JunCSG transform_(Jun3dTransformation aJun3dTransformation) {
		return JunTransformedCSG.On_transform_(this, aJun3dTransformation);
	}
}
