package sra.jun.delaunay.twoD;

import java.awt.*;
import java.util.*;
import sra.jun.geometry.basic.*;
import sra.smalltalk.*;

/**
 * Jun2dDelaunayProcessor class
 * 
 *  @author    Ryouichi Matsuda
 *  @created   2002/01/14 (by Ryouichi Matsuda)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: Jun2dDelaunayProcessor.java,v 6.0 2003/10/28 10:52:04 nisinaka Exp $
 */
public class Jun2dDelaunayProcessor extends StObject {
	// 1.0d-8
	public static final double DECIMAL_8 = 0.00000001d;
	// 1.0d-12
	public static final double DECIMAL_12 = 0.000000000001d;
	protected Jun2dDelaunayList loops;
	protected Jun2dDelaunayList vertices;
	protected Jun2dDelaunayList edges;
	protected Jun2dBoundingBox bounds;
	protected boolean constrained;

	/**
	 * Creates a new Jun2dDelaunayProcessor object.
	 */
	public Jun2dDelaunayProcessor() {
		super();
		this.initialize();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aJun2dBoundingBox sra.jun.geometry.basic.Jun2dBoundingBox
	 */
	public Jun2dDelaunayProcessor(Jun2dBoundingBox aJun2dBoundingBox) {
		super();
		this.setBounds_(aJun2dBoundingBox);
		this.initialize();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.util.Hashtable
	 */
	public static Hashtable _IdentityDictionary() {
		return new Hashtable() {
			Vector keys = _IdentitySet();
			Vector values = new Vector();

			public Object put(Object key, Object obj) {
				int idx = keys.indexOf(key);

				if (idx < 0) {
					keys.addElement(key);
					values.addElement(obj);
				} else {
					values.setElementAt(obj, idx);
				}

				return null;
			}

			public Object get(Object key) {
				int idx = keys.indexOf(key);

				if (idx < 0) {
					return null;
				} else {
					return values.elementAt(idx);
				}
			}

			public Enumeration keys() {
				return keys.elements();
			}
		};
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.util.Vector
	 */
	public static Vector _IdentitySet() {
		return new Vector() {
			public int indexOf(Object elem) {
				for (int i = 0; i < elementCount; i++) {
					if (elementData[i] == elem) {
						return i;
					}
				}

				return -1;
			}

			public void addElement(Object obj) {
				if (indexOf(obj) < 0) {
					super.addElement(obj);
				}
			}
		};
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aJun2dBoundingBox sra.jun.geometry.basic.Jun2dBoundingBox
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayProcessor
	 */
	public static Jun2dDelaunayProcessor Bounds_(Jun2dBoundingBox aJun2dBoundingBox) {
		return new Jun2dDelaunayProcessor(aJun2dBoundingBox);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.geometry.basic.Jun2dBoundingBox
	 */
	public static Jun2dBoundingBox DefaultBounds() {
		return Jun2dBoundingBox.Origin_corner_(new Jun2dPoint(-0.0001, -0.0001), new Jun2dPoint(0.0001, 0.0001));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aJun2dDelaunayEdge sra.jun.delaunay.twoD.Jun2dDelaunayEdge
	 */
	public void basicInsertEdge_(Jun2dDelaunayEdge aJun2dDelaunayEdge) {
		constrained = true;
		this.makeWayFrom_to_(aJun2dDelaunayEdge.vertex1(), aJun2dDelaunayEdge.vertex2());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aJun2dDelaunayVertex sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 * @param aJun2dDelaunayLoop sra.jun.delaunay.twoD.Jun2dDelaunayLoop
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 */
	public Jun2dDelaunayVertex basicInsertVertex_onLoop_(Jun2dDelaunayVertex aJun2dDelaunayVertex, Jun2dDelaunayLoop aJun2dDelaunayLoop) {
		Jun2dDelaunayHalfEdge hEdge2;
		Jun2dDelaunayHalfEdge hEdge3;
		Jun2dDelaunayEdge newEdge1;
		Jun2dDelaunayEdge newEdge2;
		Jun2dDelaunayEdge newEdge3;

		final Jun2dDelaunayHalfEdge hEdge1 = aJun2dDelaunayLoop.halfEdge();
		hEdge2 = hEdge1.next();
		hEdge3 = hEdge2.next();
		newEdge1 = this.newEdge();
		newEdge2 = this.newEdge();
		newEdge3 = this.newEdge();

		final Jun2dDelaunayLoop newLoop1 = this.newLoop();
		final Jun2dDelaunayLoop newLoop2 = this.newLoop();
		newEdge1.vertex1_(hEdge1.vertex());
		newEdge1.vertex2_(aJun2dDelaunayVertex);
		newEdge2.vertex1_(hEdge2.vertex());
		newEdge2.vertex2_(aJun2dDelaunayVertex);
		newEdge3.vertex1_(hEdge3.vertex());
		newEdge3.vertex2_(aJun2dDelaunayVertex);
		aJun2dDelaunayVertex.halfEdge_(newEdge3.halfEdge12());
		hEdge1.next_(newEdge1.halfEdge12());
		newEdge1.halfEdge12().next_(newEdge3.halfEdge21());
		newEdge3.halfEdge21().next_(hEdge1);
		hEdge2.next_(newEdge2.halfEdge12());
		newEdge2.halfEdge12().next_(newEdge1.halfEdge21());
		newEdge1.halfEdge21().next_(hEdge2);
		hEdge3.next_(newEdge3.halfEdge12());
		newEdge3.halfEdge12().next_(newEdge2.halfEdge21());
		newEdge2.halfEdge21().next_(hEdge3);
		hEdge1.do_(new StBlockClosure() {
			public Object value_(final Object hEdge_) {
				final Jun2dDelaunayHalfEdge hEdge = (Jun2dDelaunayHalfEdge) hEdge_;
				hEdge.loop_(hEdge1.loop());

				return null;
			}
		});
		hEdge2.do_(new StBlockClosure() {
			public Object value_(final Object hEdge_) {
				final Jun2dDelaunayHalfEdge hEdge = (Jun2dDelaunayHalfEdge) hEdge_;
				hEdge.loop_(newLoop1);

				return null;
			}
		});
		hEdge3.do_(new StBlockClosure() {
			public Object value_(final Object hEdge_) {
				final Jun2dDelaunayHalfEdge hEdge = (Jun2dDelaunayHalfEdge) hEdge_;
				hEdge.loop_(newLoop2);

				return null;
			}
		});
		hEdge1.loop().halfEdge_(hEdge1);
		hEdge2.loop().halfEdge_(hEdge2);
		hEdge3.loop().halfEdge_(hEdge3);
		vertices.add_(aJun2dDelaunayVertex);

		if (hEdge1.edge().isConstrained() && (hEdge1.areaWith_(aJun2dDelaunayVertex) < DECIMAL_12)) {
			hEdge1.edge().enableArrange();
			hEdge1.next().edge().disableArrange();
			hEdge1.prev().edge().disableArrange();
			hEdge1.edge().basicArrange();
		}

		if (hEdge2.edge().isConstrained() && (hEdge2.areaWith_(aJun2dDelaunayVertex) < DECIMAL_12)) {
			hEdge2.edge().enableArrange();
			hEdge2.next().edge().disableArrange();
			hEdge2.prev().edge().disableArrange();
			hEdge2.edge().basicArrange();
		}

		if (hEdge3.edge().isConstrained() && (hEdge3.areaWith_(aJun2dDelaunayVertex) < DECIMAL_12)) {
			hEdge3.edge().enableArrange();
			hEdge3.next().edge().disableArrange();
			hEdge3.prev().edge().disableArrange();
			hEdge3.edge().basicArrange();
		}

		hEdge1.edge().setDirty();
		hEdge2.edge().setDirty();
		hEdge3.edge().setDirty();

		return aJun2dDelaunayVertex;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void convexHullTrianglesDo_(final StBlockClosure aBlock) {
		loops.do_(new StBlockClosure() {
			public Object value_(Object loop_) {
				Jun2dDelaunayLoop loop = (Jun2dDelaunayLoop) loop_;

				if (loop.hasBoundaryVertex() == false) {
					aBlock.value_value_value_(loop.point1(), loop.point2(), loop.point3());
				}

				return null;
			}
		});
	}

	/**
	 * Display the receiver on aGraphics.
	 * 
	 * @param aGraphicsContext java.awt.Graphics
	 */
	public void displayOn_(final Graphics aGraphicsContext) {
		edges.do_(new StBlockClosure() {
			public Object value_(Object edge_) {
				Jun2dDelaunayEdge edge = (Jun2dDelaunayEdge) edge_;

				edge.displayOn_(aGraphicsContext);

				return null;
			}
		});
	}

	/**
	 * DOCUMENT ME!
	 */
	public void initialize() {
		Jun2dDelaunayVertex vertex00;
		Jun2dDelaunayVertex vertex10;
		Jun2dDelaunayVertex vertex11;
		Jun2dDelaunayVertex vertex01;
		Jun2dDelaunayLoop l1;
		Jun2dDelaunayLoop l2;
		Jun2dDelaunayEdge e1;
		Jun2dDelaunayEdge e2;
		Jun2dDelaunayEdge e3;
		Jun2dDelaunayEdge e4;
		Jun2dDelaunayEdge e5;

		if (bounds == null) {
			bounds = this.DefaultBounds();
		}

		vertices = new Jun2dDelaunayList();
		edges = new Jun2dDelaunayList();
		loops = new Jun2dDelaunayList();
		vertex00 = this.newBoundaryVertexAtX_y_(bounds.origin().x(), bounds.origin().y());
		vertex10 = this.newBoundaryVertexAtX_y_(bounds.corner().x(), bounds.origin().y());
		vertex11 = this.newBoundaryVertexAtX_y_(bounds.corner().x(), bounds.corner().y());
		vertex01 = this.newBoundaryVertexAtX_y_(bounds.origin().x(), bounds.corner().y());
		l1 = this.newLoop();
		l2 = this.newLoop();
		e1 = this.newEdge();
		e2 = this.newEdge();
		e3 = this.newEdge();
		e4 = this.newEdge();
		e5 = this.newEdge();
		e1.vertex1_(vertex00);
		e1.vertex2_(vertex10);
		e2.vertex1_(vertex10);
		e2.vertex2_(vertex11);
		e3.vertex1_(vertex11);
		e3.vertex2_(vertex01);
		e4.vertex1_(vertex01);
		e4.vertex2_(vertex00);
		e5.vertex1_(vertex00);
		e5.vertex2_(vertex11);
		e1.halfEdge12().next_(e2.halfEdge12());
		e2.halfEdge12().next_(e5.halfEdge21());
		e5.halfEdge21().next_(e1.halfEdge12());
		e3.halfEdge12().next_(e4.halfEdge12());
		e4.halfEdge12().next_(e5.halfEdge12());
		e5.halfEdge12().next_(e3.halfEdge12());
		e1.halfEdge21().next_(e4.halfEdge21());
		e2.halfEdge21().next_(e1.halfEdge21());
		e3.halfEdge21().next_(e2.halfEdge21());
		e4.halfEdge21().next_(e3.halfEdge21());
		e1.loop12_(l1);
		e2.loop12_(l1);
		e5.loop21_(l1);
		e3.loop12_(l2);
		e4.loop12_(l2);
		e5.loop12_(l2);
		l1.halfEdge_(e1.halfEdge12());
		l2.halfEdge_(e4.halfEdge12());
		vertex00.halfEdge_(e1.halfEdge21());
		vertex10.halfEdge_(e2.halfEdge21());
		vertex11.halfEdge_(e3.halfEdge21());
		vertex01.halfEdge_(e4.halfEdge21());
		constrained = false;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aJun2dDelaunayEdge sra.jun.delaunay.twoD.Jun2dDelaunayEdge
	 */
	public void insertEdge_(Jun2dDelaunayEdge aJun2dDelaunayEdge) {
		aJun2dDelaunayEdge.vertex1_(this.insertVertex_(aJun2dDelaunayEdge.vertex1()));
		aJun2dDelaunayEdge.vertex2_(this.insertVertex_(aJun2dDelaunayEdge.vertex2()));
		this.basicInsertEdge_(aJun2dDelaunayEdge);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param fromPoint java.awt.Point
	 * @param toPoint sra.jun.geometry.basic.JunPoint
	 */
	public void insertEdgeFrom_to_(JunPoint fromPoint, JunPoint toPoint) {
		Jun2dDelaunayVertex vertex1;
		Jun2dDelaunayVertex vertex2;
		Jun2dDelaunayEdge edge;

		vertex1 = this.insertPoint_(fromPoint);
		vertex2 = this.insertPoint_(toPoint);
		edge = new Jun2dDelaunayEdge();
		edge.vertex1_(vertex1);
		edge.vertex2_(vertex2);
		this.insertEdge_(edge);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aPoint sra.jun.geometry.basic.JunPoint
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 */
	public Jun2dDelaunayVertex insertPoint_(JunPoint aPoint) {
		Jun2dDelaunayVertex vertex;
		vertex = new Jun2dDelaunayVertex(aPoint);

		return this.insertVertex_(vertex);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aCollectionOfPoint sra.jun.geometry.basic.JunPoint[]
	 */
	public void insertPoints_(JunPoint[] aCollectionOfPoint) {
		for (int i = 0; i < aCollectionOfPoint.length; i++) {
			this.insertPoint_(aCollectionOfPoint[i]);
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param pointArray sra.jun.geometry.basic.JunPoint[]
	 */
	public void insertPolygon_(JunPoint[] pointArray) {
		this.insertPolygon_interim_(pointArray, null);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param pointArray sra.jun.geometry.basic.JunPoint[]
	 * @param aBlockOrNil sra.smalltalk.StBlockClosure
	 */
	public void insertPolygon_interim_(JunPoint[] pointArray, StBlockClosure aBlockOrNil) {
		Jun2dDelaunayVertex vertex2;

		vertex2 = this.insertPoint_(pointArray[pointArray.length - 1]);

		for (int i = 0; i < pointArray.length; i++) {
			JunPoint point = pointArray[i];
			Jun2dDelaunayVertex vertex1;
			Jun2dDelaunayEdge edge;
			vertex1 = this.insertPoint_(point);

			if ((vertex1 != null) && (vertex2 != null)) {
				edge = new Jun2dDelaunayEdge();
				edge.vertex1_(vertex1);
				edge.vertex2_(vertex2);
				this.basicInsertEdge_(edge);

				if (aBlockOrNil != null) {
					aBlockOrNil.value_(this);
				}
			}

			vertex2 = vertex1;
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aJun2dDelaunayVertex sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 */
	public Jun2dDelaunayVertex insertVertex_(Jun2dDelaunayVertex aJun2dDelaunayVertex) {
		double x;
		double y;
		Jun2dDelaunayVertex vertex;
		Jun2dDelaunayLoop loop;

		x = aJun2dDelaunayVertex.x();
		y = aJun2dDelaunayVertex.y();
		vertex = this.nearestVertexFromX_y_(x, y);

		if (vertex.containsX_y_(x, y)) {
			return vertex;
		}

		loop = this.loopAtX_y_(x, y);

		if (loop != null) {
			return this.insertVertex_onLoop_(aJun2dDelaunayVertex, loop);
		}

		return null;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aJun2dDelaunayVertex sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 * @param aJun2dDelaunayLoop sra.jun.delaunay.twoD.Jun2dDelaunayLoop
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 */
	public Jun2dDelaunayVertex insertVertex_onLoop_(Jun2dDelaunayVertex aJun2dDelaunayVertex, Jun2dDelaunayLoop aJun2dDelaunayLoop) {
		Jun2dDelaunayHalfEdge hEdge2;
		Jun2dDelaunayHalfEdge hEdge3;
		Jun2dDelaunayEdge newEdge1;
		Jun2dDelaunayEdge newEdge2;
		Jun2dDelaunayEdge newEdge3;

		final Jun2dDelaunayHalfEdge hEdge1 = aJun2dDelaunayLoop.halfEdge();
		hEdge2 = hEdge1.next();
		hEdge3 = hEdge2.next();
		newEdge1 = this.newEdge();
		newEdge2 = this.newEdge();
		newEdge3 = this.newEdge();

		final Jun2dDelaunayLoop newLoop1 = this.newLoop();
		final Jun2dDelaunayLoop newLoop2 = this.newLoop();
		newEdge1.vertex1_(hEdge1.vertex());
		newEdge1.vertex2_(aJun2dDelaunayVertex);
		newEdge2.vertex1_(hEdge2.vertex());
		newEdge2.vertex2_(aJun2dDelaunayVertex);
		newEdge3.vertex1_(hEdge3.vertex());
		newEdge3.vertex2_(aJun2dDelaunayVertex);
		aJun2dDelaunayVertex.halfEdge_(newEdge3.halfEdge12());
		hEdge1.next_(newEdge1.halfEdge12());
		newEdge1.halfEdge12().next_(newEdge3.halfEdge21());
		newEdge3.halfEdge21().next_(hEdge1);
		hEdge2.next_(newEdge2.halfEdge12());
		newEdge2.halfEdge12().next_(newEdge1.halfEdge21());
		newEdge1.halfEdge21().next_(hEdge2);
		hEdge3.next_(newEdge3.halfEdge12());
		newEdge3.halfEdge12().next_(newEdge2.halfEdge21());
		newEdge2.halfEdge21().next_(hEdge3);
		hEdge1.do_(new StBlockClosure() {
			public Object value_(final Object hEdge_) {
				final Jun2dDelaunayHalfEdge hEdge = (Jun2dDelaunayHalfEdge) hEdge_;
				hEdge.loop_(hEdge1.loop());

				return null;
			}
		});
		hEdge2.do_(new StBlockClosure() {
			public Object value_(final Object hEdge_) {
				final Jun2dDelaunayHalfEdge hEdge = (Jun2dDelaunayHalfEdge) hEdge_;
				hEdge.loop_(newLoop1);

				return null;
			}
		});
		hEdge3.do_(new StBlockClosure() {
			public Object value_(final Object hEdge_) {
				final Jun2dDelaunayHalfEdge hEdge = (Jun2dDelaunayHalfEdge) hEdge_;
				hEdge.loop_(newLoop2);

				return null;
			}
		});
		hEdge1.loop().halfEdge_(hEdge1);
		hEdge2.loop().halfEdge_(hEdge2);
		hEdge3.loop().halfEdge_(hEdge3);
		vertices.add_(aJun2dDelaunayVertex);

		if (hEdge1.edge().isConstrained() && (hEdge1.areaWith_(aJun2dDelaunayVertex) < DECIMAL_8)) {
			hEdge1.edge().enableArrange();
			hEdge1.next().edge().disableArrange();
			hEdge1.prev().edge().disableArrange();
		}

		if (hEdge2.edge().isConstrained() && (hEdge2.areaWith_(aJun2dDelaunayVertex) < DECIMAL_8)) {
			hEdge2.edge().enableArrange();
			hEdge2.next().edge().disableArrange();
			hEdge2.prev().edge().disableArrange();
		}

		if (hEdge3.edge().isConstrained() && (hEdge3.areaWith_(aJun2dDelaunayVertex) < DECIMAL_8)) {
			hEdge3.edge().enableArrange();
			hEdge3.next().edge().disableArrange();
			hEdge3.prev().edge().disableArrange();
		}

		hEdge1.edge().setDirty();
		hEdge2.edge().setDirty();
		hEdge3.edge().setDirty();
		hEdge1.edge().recursiveArrange();
		hEdge2.edge().recursiveArrange();
		hEdge3.edge().recursiveArrange();

		return aJun2dDelaunayVertex;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void insideTrianglesDo_(StBlockClosure aBlock) {
		final Hashtable distances = _IdentityDictionary(); // Loop @ Int
		final StValueHolder activeLoops = new StValueHolder(_IdentitySet()); // Loop

		vertices.do_(new StBlockClosure() {
			public Object value_(Object vertex_) {
				Jun2dDelaunayVertex vertex = (Jun2dDelaunayVertex) vertex_;

				if (vertex.isBoundaryVertex()) {
					vertex.loopsDo_(new StBlockClosure() {
						public Object value_(Object loop_) {
							Jun2dDelaunayLoop loop = (Jun2dDelaunayLoop) loop_;

							distances.put(loop, new Integer(0));
							((Vector) activeLoops.value()).addElement(loop);

							return null;
						}
					});
				}

				return null;
			}
		});

		while (!((Vector) activeLoops.value()).isEmpty()) {
			final Vector updatedLoops = _IdentitySet();

			for (int i = 0; i < ((Vector) activeLoops.value()).size(); i++) {
				Jun2dDelaunayLoop loop = (Jun2dDelaunayLoop) ((Vector) activeLoops.value()).elementAt(i);
				final int distance = ((Integer) distances.get(loop)).intValue();
				loop.halfEdgesDo_(new StBlockClosure() {
					public Object value_(Object halfEdge_) {
						Jun2dDelaunayHalfEdge halfEdge = (Jun2dDelaunayHalfEdge) halfEdge_;
						Jun2dDelaunayLoop neighbor;

						neighbor = halfEdge.pair().loop();

						if (neighbor != null) {
							int distanceFromMe;
							Integer currentDistance;

							distanceFromMe = halfEdge.edge().isConstrained() ? (distance + 1) : distance;
							currentDistance = (Integer) distances.get(neighbor);

							if ((currentDistance == null) || (distanceFromMe < currentDistance.intValue())) {
								distances.put(neighbor, new Integer(distanceFromMe));
								updatedLoops.addElement(neighbor);
							}
						}

						return null;
					}
				});
			}

			activeLoops.value_(updatedLoops);
		}

		for (Enumeration e = distances.keys(); e.hasMoreElements();) {
			Jun2dDelaunayLoop loop = (Jun2dDelaunayLoop) e.nextElement();
			int distance = ((Number) distances.get(loop)).intValue();

			if ((distance % 2) != 0) {
				aBlock.value_value_value_(loop.point1(), loop.point2(), loop.point3());
			}
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param baseVertex sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 * @param vertex2 sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 */
	public Jun2dDelaunayVertex makeCheckpointFrom_to_(Jun2dDelaunayVertex baseVertex, Jun2dDelaunayVertex vertex2) {
		Jun2dDelaunayHalfEdge baseHalfEdge;

		if (baseVertex.halfEdgeToward_(vertex2) != null) {
			return vertex2;
		}

		baseHalfEdge = this.intersectingHalfEdgeFrom_to_(baseVertex, vertex2);

		if (baseHalfEdge == null) {
			return null;
		}

		while (true) {
			baseHalfEdge = baseHalfEdge.pair();

			if (!baseHalfEdge.edge().isArrangeable()) {
				Jun2dDelaunayVertex intersection;

				intersection = baseHalfEdge.edge().intersectionWithLineSegmentFrom_to_(baseVertex, vertex2);
				this.insertVertex_onLoop_(intersection, baseHalfEdge.loop());

				return intersection;
			}

			if (baseHalfEdge.next().vertex().isOnLineSegmentFrom_to_(baseVertex, vertex2)) {
				return vertex2;
			}

			if (baseHalfEdge.next().intersectsWithLineSegmentFrom_to_(baseVertex, vertex2)) {
				baseHalfEdge = baseHalfEdge.next();
			} else {
				if (baseHalfEdge.prev().intersectsWithLineSegmentFrom_to_(baseVertex, vertex2)) {
					baseHalfEdge = baseHalfEdge.prev();
				} else {
					return null;
				}
			}
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param vertex1 sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 * @param vertex2 sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 */
	public Jun2dDelaunayVertex makeStepFrom_to_(Jun2dDelaunayVertex vertex1, Jun2dDelaunayVertex vertex2) {
		Jun2dDelaunayVertex startVertex;
		Jun2dDelaunayVertex goalVertex;
		Vector dirties; // IdentitySet
		Jun2dDelaunayHalfEdge halfEdge;

		startVertex = vertex1;
		goalVertex = this.makeCheckpointFrom_to_(startVertex, vertex2);

		if (goalVertex == null) {
			return null;
		}

		dirties = _IdentitySet();

		for (halfEdge = startVertex.halfEdgeTo_(goalVertex); halfEdge == null; halfEdge = startVertex.halfEdgeTo_(goalVertex)) {
			halfEdge = this.intersectingHalfEdgeFrom_to_(startVertex, goalVertex);

			if (halfEdge == null) {
				for (int i = 0; i < dirties.size(); i++) {
					Jun2dDelaunayEdge dirtyEdge = (Jun2dDelaunayEdge) dirties.elementAt(i);
					dirtyEdge.recursiveArrange();
				}

				return null;
			}

			halfEdge.edge().enableArrange();
			halfEdge.edge().basicArrange();
			halfEdge.edge().setDirty();
			dirties.addElement(halfEdge.edge());
			halfEdge.next().edge().setDirty();
			dirties.addElement(halfEdge.next().edge());
			halfEdge.prev().edge().setDirty();
			dirties.addElement(halfEdge.prev().edge());
			halfEdge.pair().next().edge().setDirty();
			dirties.addElement(halfEdge.pair().next().edge());
			halfEdge.pair().prev().edge().setDirty();
			dirties.addElement(halfEdge.pair().prev().edge());
		}

		halfEdge.edge().disableArrange();

		for (int i = 0; i < dirties.size(); i++) {
			Jun2dDelaunayEdge dirtyEdge = (Jun2dDelaunayEdge) dirties.elementAt(i);
			dirtyEdge.recursiveArrange();
		}

		return goalVertex;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param vertex1 sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 * @param vertex2 sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 */
	public void makeWayFrom_to_(Jun2dDelaunayVertex vertex1, Jun2dDelaunayVertex vertex2) {
		Jun2dDelaunayVertex checkpoint;
		Jun2dDelaunayVertex newCheckpoint;

		checkpoint = vertex1;

		while (true) {
			newCheckpoint = this.makeStepFrom_to_(checkpoint, vertex2);

			if ((newCheckpoint == null) || (newCheckpoint == vertex2)) {
				return;
			}

			checkpoint = newCheckpoint;
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param xNumber double
	 * @param yNumber double
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayBoundaryVertex
	 */
	public Jun2dDelaunayBoundaryVertex newBoundaryVertexAtX_y_(double xNumber, double yNumber) {
		return (Jun2dDelaunayBoundaryVertex) vertices.add_(new Jun2dDelaunayBoundaryVertex(xNumber, yNumber));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayEdge
	 */
	public Jun2dDelaunayEdge newEdge() {
		return (Jun2dDelaunayEdge) edges.add_(new Jun2dDelaunayEdge());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayLoop
	 */
	public Jun2dDelaunayLoop newLoop() {
		return (Jun2dDelaunayLoop) loops.add_(new Jun2dDelaunayLoop());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 */
	public Jun2dDelaunayVertex newVertex() {
		return (Jun2dDelaunayVertex) vertices.add_(new Jun2dDelaunayVertex());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param xNumber double
	 * @param yNumber double
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 */
	public Jun2dDelaunayVertex newVertexAtX_y_(double xNumber, double yNumber) {
		return (Jun2dDelaunayVertex) vertices.add_(new Jun2dDelaunayVertex(xNumber, yNumber));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param xNumber double
	 * @param yNumber double
	 * @param zNumber double
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 */
	public Jun2dDelaunayVertex newVertexAtX_y_z_(double xNumber, double yNumber, double zNumber) {
		return (Jun2dDelaunayVertex) vertices.add_(new Jun2dDelaunayVertex(xNumber, yNumber, zNumber));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void trianglesDo_(final StBlockClosure aBlock) {
		loops.do_(new StBlockClosure() {
			public Object value_(final Object loop_) {
				final Jun2dDelaunayLoop loop = (Jun2dDelaunayLoop) loop_;

				aBlock.value_value_value_(loop.point1(), loop.point2(), loop.point3());

				return null;
			}
		});
	}

	/**
	 * DOCUMENT ME!
	 */
	protected void checkLoops() {
		loops.do_(new StBlockClosure() {
			public Object value_(final Object loop_) {
				final Jun2dDelaunayLoop loop = (Jun2dDelaunayLoop) loop_;

				if (loop.area() < 0) {
					throw SmalltalkException.Halt("checkLoops");
				}

				return null;
			}
		});
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayList
	 */
	protected Jun2dDelaunayList edges() {
		return edges;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param xNumber double
	 * @param yNumber double
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayLoop
	 */
	protected Jun2dDelaunayLoop exhaustSearchForLoopAtX_y_(final double xNumber, final double yNumber) {
		return (Jun2dDelaunayLoop) loops.do_(new StBlockClosure() {
			public Object value_(final Object loop_) {
				final Jun2dDelaunayLoop loop = (Jun2dDelaunayLoop) loop_;

				if (loop.containsX_y_(xNumber, yNumber)) {
					return loop;
				}

				return null;
			}
		});
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param xNumber double
	 * @param yNumber double
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 */
	protected Jun2dDelaunayVertex exhaustSearchNearestVertexFromX_y_(final double xNumber, final double yNumber) {
		final StValueHolder theVertex = new StValueHolder(); // Jun2dDelaunayVertex
		final StValueHolder min = new StValueHolder(); // double

		theVertex.value_(vertices.first());
		min.value_(((Jun2dDelaunayVertex) theVertex.value()).squaredDistanceFromX_y_(xNumber, yNumber));
		vertices.do_(new StBlockClosure() {
			public Object value_(Object vertex_) {
				final Jun2dDelaunayVertex vertex = (Jun2dDelaunayVertex) vertex_;
				double d;

				d = vertex.squaredDistanceFromX_y_(xNumber, yNumber);

				if (d < min._doubleValue()) {
					min.value_(d);
					theVertex.value_(vertex);
				}

				return null;
			}
		});

		return (Jun2dDelaunayVertex) theVertex.value();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param vertex1 sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 * @param vertex2 sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayHalfEdge
	 */
	protected Jun2dDelaunayHalfEdge intersectingHalfEdgeFrom_to_(final Jun2dDelaunayVertex vertex1, final Jun2dDelaunayVertex vertex2) {
		return (Jun2dDelaunayHalfEdge) vertex1.halfEdgesDo_(new StBlockClosure() {
			public Object value_(Object hEdge_) {
				Jun2dDelaunayHalfEdge hEdge = (Jun2dDelaunayHalfEdge) hEdge_;

				if (hEdge.prev().intersectsWithLineSegmentFrom_to_(vertex1, vertex2)) {
					return hEdge.prev();
				}

				return null;
			}
		});
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param xNumber double
	 * @param yNumber double
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayLoop
	 */
	protected Jun2dDelaunayLoop loopAtX_y_(double xNumber, double yNumber) {
		return this.topologicalSearchForLoopAtX_y_(xNumber, yNumber);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return double
	 */
	protected double margin() {
		return 1.0d;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param xNumber double
	 * @param yNumber double
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayEdge
	 */
	protected Jun2dDelaunayEdge nearestEdgeFromX_y_(final double xNumber, final double yNumber) {
		final StValueHolder theEdge = new StValueHolder(); // Jun2dDelaunayEdge
		final StValueHolder min = new StValueHolder(); // double
		theEdge.value_(edges.first());
		min.value_(new Double(((Jun2dDelaunayEdge) edges.first()).squaredDistanceFromX_y_(xNumber, yNumber)));
		edges.do_(new StBlockClosure() {
			public Object value_(Object edge_) {
				Jun2dDelaunayEdge edge = (Jun2dDelaunayEdge) edge_;
				double d;

				d = edge.squaredDistanceFromX_y_(xNumber, yNumber);

				if (d < min._doubleValue()) {
					min.value_(d);
					theEdge.value_(edge);
				}

				return null;
			}
		});

		return (Jun2dDelaunayEdge) theEdge.value();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param xNumber double
	 * @param yNumber double
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 */
	protected Jun2dDelaunayVertex nearestVertexFromX_y_(double xNumber, double yNumber) {
		if (constrained) {
			return this.exhaustSearchNearestVertexFromX_y_(xNumber, yNumber);
		} else {
			return this.topologicalSearchNearestVertexFromX_y_(xNumber, yNumber);
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aJun2dBoundingBox sra.jun.geometry.basic.Jun2dBoundingBox
	 */
	protected void setBounds_(Jun2dBoundingBox aJun2dBoundingBox) {
		bounds = aJun2dBoundingBox;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param xNumber double
	 * @param yNumber double
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayLoop
	 */
	protected Jun2dDelaunayLoop topologicalSearchForLoopAtX_y_(final double xNumber, final double yNumber) {
		StBlockClosure areaBlock;
		Jun2dDelaunayLoop loop;

		areaBlock = new StBlockClosure() {
			public Object value_(Object hEdge_) {
				Jun2dDelaunayHalfEdge hEdge = (Jun2dDelaunayHalfEdge) hEdge_;
				Jun2dDelaunayVertex v1;
				Jun2dDelaunayVertex v2;

				v1 = hEdge.pair().vertex();
				v2 = hEdge.vertex();

				return new Double(((v1.x() - xNumber) * (v2.y() - yNumber)) - ((v1.y() - yNumber) * (v2.x() - xNumber)));
			}
		};

		loop = (Jun2dDelaunayLoop) loops.first();

		Jun2dDelaunayHalfEdge halfEdge1;
		Jun2dDelaunayHalfEdge halfEdge2;
		Jun2dDelaunayHalfEdge halfEdge3;
		double area1;
		double area2;
		double area3;
		double minArea;

		while (true) {
			halfEdge1 = loop.halfEdge();
			halfEdge2 = halfEdge1.next();
			halfEdge3 = halfEdge2.next();
			area1 = ((Double) areaBlock.value_(halfEdge1)).doubleValue();
			area2 = ((Double) areaBlock.value_(halfEdge2)).doubleValue();
			area3 = ((Double) areaBlock.value_(halfEdge3)).doubleValue();

			if (loop.area() < 0) {
				return null;
			}

			if (!((area1 < 0) || (area2 < 0) || area3 < 0)) {
				return loop;
			}

			loop = halfEdge1.pair().loop();
			minArea = area1;

			if (area2 < minArea) {
				minArea = area2;
				loop = halfEdge2.pair().loop();
			}

			if (area3 < minArea) {
				loop = halfEdge3.pair().loop();
			}

			if (loop == null) {
				return null;
			}
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param xNumber double
	 * @param yNumber double
	 * 
	 * @return sra.jun.delaunay.twoD.Jun2dDelaunayVertex
	 */
	protected Jun2dDelaunayVertex topologicalSearchNearestVertexFromX_y_(final double xNumber, final double yNumber) {
		Jun2dDelaunayVertex theVertex;

		theVertex = (Jun2dDelaunayVertex) vertices.first();

		while (true) {
			final StValueHolder min = new StValueHolder(); // double
			final StValueHolder nextVertex = new StValueHolder(); // Jun2dDelaunayVertex

			min.value_(theVertex.squaredDistanceFromX_y_(xNumber, yNumber));
			nextVertex.value_(null);
			theVertex.neighborsDo_(new StBlockClosure() {
				public Object value_(Object vertex_) {
					Jun2dDelaunayVertex vertex = (Jun2dDelaunayVertex) vertex_;
					double d;

					d = vertex.squaredDistanceFromX_y_(xNumber, yNumber);

					if (d < min._doubleValue()) {
						min.value_(d);
						nextVertex.value_(vertex);
					}

					return null;
				}
			});

			if (nextVertex.value() == null) {
				return theVertex;
			}

			theVertex = (Jun2dDelaunayVertex) nextVertex.value();
		}
	}
}
