package sra.jun.geometry.basic;

import java.awt.Rectangle;
import java.io.*;
import java.util.Vector;
import sra.jun.geometry.abstracts.*;
import sra.jun.geometry.curves.Jun2dLine;
import sra.jun.geometry.transformations.Jun2dTransformation;
import sra.smalltalk.*;

/**
 * Jun2dBoundingBox class
 * 
 *  @author    nisinaka
 *  @created   1998/10/01 (by nisinaka)
 *  @updated   1999/08/05 (by nisinaka)
 *  @updated   2000/01/06 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: Jun2dBoundingBox.java,v 6.0 2003/10/28 10:52:04 nisinaka Exp $
 */
public class Jun2dBoundingBox extends JunBoundingBox {
	/** The <i>origin</i> point. */
	protected Jun2dPoint origin;

	/** The <i>corner</i> point. */
	protected Jun2dPoint corner;

	/**
	 * Create a new instance of Jun2dBoundingBox.
	 * 
	 * @return sra.jun.geometry.basic.Jun2dBoundingBox
	 * @category Instance creation
	 */
	public Jun2dBoundingBox() {
		super();
	}

	/**
	 * Create a new instance of Jun2dBoundingBox and initialize it with the
	 * <code>aRectangle</code>.
	 * 
	 * @return sra.jun.geometry.basic.Jun2dBoundingBox
	 * @param aRectangle java.awt.Rectangle
	 * @category Instance creation
	 */
	public Jun2dBoundingBox(Rectangle aRectangle) {
		super();
		this.origin_extent_(new Jun2dPoint(aRectangle.x, aRectangle.y), new Jun2dPoint(aRectangle.width, aRectangle.height));
	}

	/**
	 * Create a new instance of Jun2dBoundingBox and initialize it with the
	 * <code>origin</code> point and the <code>corner</code> point.
	 * 
	 * @return sra.jun.geometry.basic.Jun2dBoundingBox
	 * @param originPoint sra.jun.geometry.basic.Jun2dPoint
	 * @param cornerPoint sra.jun.geometry.basic.Jun2dPoint
	 * @category Instance creation
	 */
	public static Jun2dBoundingBox Origin_corner_(Jun2dPoint originPoint, Jun2dPoint cornerPoint) {
		Jun2dBoundingBox aBoundingBox = new Jun2dBoundingBox();
		aBoundingBox.origin_corner_(originPoint, cornerPoint);

		return aBoundingBox;
	}

	/**
	 * Create a new instance of Jun2dBoundingBox and initialize it with the
	 * <code>origin</code> point and the <code>extent</code> point.
	 * 
	 * @return sra.jun.geometry.basic.Jun2dBoundingBox
	 * @param originPoint sra.jun.geometry.basic.Jun2dPoint
	 * @param extentPoint sra.jun.geometry.basic.Jun2dPoint
	 * @category Instance creation
	 */
	public static Jun2dBoundingBox Origin_extent_(Jun2dPoint originPoint, Jun2dPoint extentPoint) {
		Jun2dBoundingBox aBoundingBox = new Jun2dBoundingBox();
		aBoundingBox.origin_extent_(originPoint, extentPoint);

		return aBoundingBox;
	}

	/**
	 * Create a new instance of Jun2dBoundingBox and initialize it with the
	 * <code>vertex</code> points.
	 * 
	 * @return sra.jun.geometry.basic.Jun2dBoundingBox
	 * @param vertexPoint1 sra.jun.geometry.basic.Jun2dPoint
	 * @param vertexPoint2 sra.jun.geometry.basic.Jun2dPoint
	 * @category Instance creation
	 */
	public static Jun2dBoundingBox Vertex_vertex_(Jun2dPoint vertexPoint1, Jun2dPoint vertexPoint2) {
		Jun2dBoundingBox aBoundingBox = new Jun2dBoundingBox();
		aBoundingBox.vertex_vertex_(vertexPoint1, vertexPoint2);

		return aBoundingBox;
	}

	/**
	 * Answer my area size.
	 * 
	 * @return double
	 * @category accessing
	 */
	public double area() {
		return this.width() * this.height();
	}

	/**
	 * Answer the center point of the receiver.
	 * 
	 * @return sra.jun.geometry.basic.Jun2dPoint
	 * @category accessing
	 */
	public Jun2dPoint center() {
		return new Jun2dPoint((origin.x + corner.x) / 2.0d, (origin.y + corner.y) / 2.0d);
	}

	/**
	 * Answer true if the receiver contains the bounding box.
	 * 
	 * @return boolean
	 * @param boundingBox sra.jun.geometry.basic.Jun2dBoundingBox
	 * @category testing
	 */
	public boolean contains_(Jun2dBoundingBox boundingBox) {
		return origin.isLessThanOrEqualTo_(boundingBox.origin()) && boundingBox.corner().isLessThanOrEqualTo_(corner);
	}

	/**
	 * Answer true if the receiver contains or touches aPoint.
	 * 
	 * @return boolean
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * @category testing
	 */
	public boolean containsOrTouchesPoint_(Jun2dPoint aPoint) {
		return origin.isLessThanOrEqualTo_(aPoint) && aPoint.isLessThanOrEqualTo_(corner);
	}

	/**
	 * Answer true if the receiver contains aPoint.
	 * 
	 * @return boolean
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * @category testing
	 */
	public boolean containsPoint_(Jun2dPoint aPoint) {
		return origin.isLessThanOrEqualTo_(aPoint) && aPoint.isLessThan_(corner);
	}

	/**
	 * Answer the <code>corner</code> point.
	 * 
	 * @return sra.jun.geometry.basic.Jun2dPoint
	 * @category accessing
	 */
	public Jun2dPoint corner() {
		return this.corner;
	}

	/**
	 * Answer the depth of the receiver.
	 * 
	 * @return double
	 * @category accessing
	 */
	public double depth() {
		throw SmalltalkException.ShouldNotImplement();
	}

	/**
	 * Answer true if anObject is equal to this bounding box, otherwise false.
	 * 
	 * @return boolean
	 * @param anObject java.lang.Object
	 * @category comparing
	 */
	public boolean equals(Object anObject) {
		if (anObject instanceof Jun2dBoundingBox == false) {
			return false;
		}

		Jun2dBoundingBox aBoundingBox = (Jun2dBoundingBox) anObject;

		return this.origin.equals(aBoundingBox.origin) && this.corner.equals(aBoundingBox.corner);
	}

	/**
	 * Answer a copy of this object expanded by "delta".
	 * 
	 * @return sra.jun.geometry.basic.Jun2dBoundingBox
	 * @param delta double
	 * @category functions
	 */
	public Jun2dBoundingBox expandedBy_(double delta) {
		return Origin_corner_(this.origin.minus_(delta), this.corner.plus_(delta));
	}

	/**
	 * Answer a copy of this object expanded by "delta".
	 * 
	 * @return sra.jun.geometry.basic.Jun2dBoundingBox
	 * @param aBoundingBox sra.jun.geometry.basic.Jun2dBoundingBox
	 * @category functions
	 */
	public Jun2dBoundingBox expandedBy_(Jun2dBoundingBox aBoundingBox) {
		return Origin_corner_(this.origin.minus_(aBoundingBox.origin), this.corner.plus_(aBoundingBox.corner));
	}

	/**
	 * Answer a copy of this object expanded by "delta".
	 * 
	 * @return sra.jun.geometry.basic.Jun2dBoundingBox
	 * @param delta sra.jun.geometry.basic.Jun2dPoint
	 * @category functions
	 */
	public Jun2dBoundingBox expandedBy_(Jun2dPoint delta) {
		return Origin_corner_(this.origin().minus_(delta), this.corner().plus_(delta));
	}

	/**
	 * Answer the extent of the receiver.
	 * 
	 * @return sra.jun.geometry.basic.Jun2dPoint
	 * @category accessing
	 */
	public Jun2dPoint extent() {
		return corner.minus_(origin);
	}

	/**
	 * Answer the height of the receiver.
	 * 
	 * @return double
	 * @category accessing
	 */
	public double height() {
		return corner.y - origin.y;
	}

	/**
	 * Answer a copy of this object inset by "delta".
	 * 
	 * @return sra.jun.geometry.basic.Jun2dBoundingBox
	 * @param delta double
	 * @category functions
	 */
	public Jun2dBoundingBox insetBy_(double delta) {
		return Origin_corner_(this.origin.plus_(delta), this.corner.minus_(delta));
	}

	/**
	 * Answer a copy of this object inset by "delta".
	 * 
	 * @return sra.jun.geometry.basic.Jun2dBoundingBox
	 * @param aBoundingBox sra.jun.geometry.basic.Ju2ndBoundingBox
	 * @category functions
	 */
	public Jun2dBoundingBox insetBy_(Jun2dBoundingBox aBoundingBox) {
		return Origin_corner_(this.origin.plus_(aBoundingBox.origin), this.corner.minus_(aBoundingBox.corner));
	}

	/**
	 * Answer the intersection part with the Jun2dBoundingBox.
	 * 
	 * @return sra.jun.geometry.basic.Jun2dBoundingBox
	 * @param boundingBox sra.jun.geometry.basic.Jun2dBoundingBox
	 * @category functions
	 */
	public Jun2dBoundingBox intersect_(Jun2dBoundingBox boundingBox) {
		return Origin_corner_(origin.max_(boundingBox.origin), corner.min_(boundingBox.corner));
	}

	/**
	 * Answer true if the receiver intersects with the Jun2dBoundingBox.
	 * 
	 * @return boolean
	 * @param aBoundingBox sra.jun.geometry.basic.Jun2dBoundingBox
	 * @category testing
	 */
	public boolean intersects_(Jun2dBoundingBox aBoundingBox) {
		return origin.isLessThan_(aBoundingBox.corner) && aBoundingBox.origin.isLessThan_(corner) && origin.isLessThan_(corner) && aBoundingBox.origin.isLessThan_(aBoundingBox.corner);
	}

	/**
	 * Answer true if the receiver intersects or touches with aBoundingBox.
	 * 
	 * @return boolean
	 * @param aBoundingBox sra.jun.geometry.basic.Jun2dBoundingBox
	 * @category testing
	 */
	public boolean intersectsOrTouches_(Jun2dBoundingBox aBoundingBox) {
		return this.intersects_(aBoundingBox) || this.touches_(aBoundingBox);
	}

	/**
	 * Answer the intersecting points with line.
	 * 
	 * @return sra.jun.geometory.basic.Jun2dPoint[]
	 * @param aLine sra.jun.geometry.curves.Jun2dLine
	 * @category functions
	 */
	public Jun2dPoint[] intersectingPointsWithLine_(Jun2dLine aLine) {
		Jun2dLine theLine = new Jun2dLine(aLine.first(), aLine.last());
		Jun2dPoint p1 = this.origin();
		Jun2dPoint p2 = new Jun2dPoint(this.origin().x, this.corner().y());
		Jun2dPoint p3 = this.corner();
		Jun2dPoint p4 = new Jun2dPoint(this.corner().x, this.origin().y());

		Jun2dLine[] lineCollectionOfRectangle = new Jun2dLine[4];
		lineCollectionOfRectangle[0] = new Jun2dLine(p1, p2);
		lineCollectionOfRectangle[1] = new Jun2dLine(p2, p3);
		lineCollectionOfRectangle[2] = new Jun2dLine(p3, p4);
		lineCollectionOfRectangle[3] = new Jun2dLine(p4, p1);

		Vector intersectingPointCollection = new Vector();
		for (int i = 0; i < lineCollectionOfRectangle.length; i++) {
			Jun2dLine eachLine = lineCollectionOfRectangle[i];
			if (eachLine.isParallelWithLine_(theLine)) {
				double distance = eachLine.distanceFromPoint_(theLine.first());
				if (Math.abs(distance) < JunGeometry.Accuracy()) {
					intersectingPointCollection.add(eachLine.first());
					intersectingPointCollection.add(eachLine.last());
				}
			} else {
				Jun2dPoint intersectingPoint = eachLine.intersectingPointWithLine_(theLine);
				if (intersectingPoint != null) {
					intersectingPointCollection.add(intersectingPoint);
				}
			}
		}

		Vector aStream = new Vector();
		Jun2dPoint[] points = new Jun2dPoint[intersectingPointCollection.size()];
		intersectingPointCollection.copyInto(points);
		for (int i = 0; i < points.length; i++) {
			Jun2dPoint eachPoint = points[i];
			for (int j = 0; j < lineCollectionOfRectangle.length; j++) {
				Jun2dLine eachLine = lineCollectionOfRectangle[j];
				if (eachLine.lineSegmentContainsPoint_(eachPoint)) {
					aStream.add(eachPoint);
				}
			}
		}

		Jun2dPoint[] results = new Jun2dPoint[aStream.size()];
		aStream.copyInto(results);
		return results;
	}

	/**
	 * Answer true if the receiver is a 2d geometry element.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean is2d() {
		return true;
	}

	/**
	 * Merge the Jun2dBoundingBox and the receiver.
	 * 
	 * @return sra.jun.geometry.basic.Jun2dBoundingBox
	 * @param boundingBox sra.jun.geometry.basic.Jun2dBoundingBox
	 * @category functions
	 */
	public Jun2dBoundingBox merge_(Jun2dBoundingBox boundingBox) {
		return Origin_corner_(this.origin.min_(boundingBox.origin), this.corner.max_(boundingBox.corner));
	}

	/**
	 * Answer the <code>origin</code> point.
	 * 
	 * @return sra.jun.geometry.basic.Jun2dPoint
	 * @category accessing
	 */
	public Jun2dPoint origin() {
		return this.origin;
	}

	/**
	 * Set the <code>origin</code> point and the <code>corner</code> point.
	 * 
	 * @param originPoint sra.jun.geometry.basic.Jun2dPoint
	 * @param cornerPoint sra.jun.geometry.basic.Jun2dPoint
	 * @category accessing
	 */
	public void origin_corner_(Jun2dPoint originPoint, Jun2dPoint cornerPoint) {
		this.origin = originPoint;
		this.corner = cornerPoint;
	}

	/**
	 * Set the <code>origin</code> point and the <code>corner</code> point. The
	 * <code>corner</code> point is calculated with the <code>extent</code>.
	 * 
	 * @param originPoint sra.jun.geometry.basic.Jun2dPoint
	 * @param extentPoint sra.jun.geometry.basic.Jun2dPoint
	 * @category accessing
	 */
	public void origin_extent_(Jun2dPoint originPoint, Jun2dPoint extentPoint) {
		this.origin = originPoint;
		this.corner = originPoint.plus_(extentPoint);
	}

	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws IOException DOCUMENT ME!
	 * @category printing
	 */
	public void printOn_(Writer aWriter) throws IOException {
		aWriter.write("(");
		aWriter.write(this.origin.printString());
		aWriter.write(" corner: ");
		aWriter.write(this.corner.printString());
		aWriter.write(")");
	}

	/**
	 * Answer the copy of the receiver rotated by the specified angle.
	 * 
	 * @return sra.jun.geometry.abstracts.JunGeometry
	 * @param anAngle sra.jun.geometry.basic.JunAngle
	 * @category transforming
	 */
	public JunGeometry rotatedBy_(JunAngle anAngle) {
		return this.transform_(Jun2dTransformation.Rotate_(anAngle));
	}

	/**
	 * Answer the copy of the receiver scaled by the specified extent.
	 * 
	 * @return sra.jun.geometry.abstracts.JunGeometry
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * @category transforming
	 */
	public Jun2dBoundingBox scaledBy_(Jun2dPoint aPoint) {
		return (Jun2dBoundingBox) this.transform_(Jun2dTransformation.Scale_(aPoint));
	}

	/**
	 * Answer true if the receiver touches with aBoundingBox.
	 * 
	 * @return boolean
	 * @param aBoundingBox sra.jun.geometry.basic.Jun2dBoundingBox
	 * @category testing
	 */
	public boolean touches_(Jun2dBoundingBox aBoundingBox) {
		return origin.isLessThanOrEqualTo_(aBoundingBox.corner) && aBoundingBox.origin.isLessThanOrEqualTo_(corner) && origin.isLessThanOrEqualTo_(corner) && aBoundingBox.origin.isLessThanOrEqualTo_(aBoundingBox.corner);
	}

	/**
	 * Answer the copy of the receiver trasnalted by the specified extent.
	 * 
	 * @return sra.jun.geometry.abstracts.JunGeometry
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * @category transforming
	 */
	public Jun2dBoundingBox translatedBy_(Jun2dPoint aPoint) {
		return (Jun2dBoundingBox) this.transform_(Jun2dTransformation.Translate_(aPoint));
	}

	/**
	 * Set the <code>origin</code> point and the <code>corner</code> point. Two
	 * points are compared and the smaller one will be the
	 * <code>origin</code>.
	 * 
	 * @param aPoint1 sra.jun.geometry.basic.Jun2dPoint
	 * @param aPoint2 sra.jun.geometry.basic.Jun2dPoint
	 */
	public void vertex_vertex_(Jun2dPoint aPoint1, Jun2dPoint aPoint2) {
		this.origin = aPoint1.min_(aPoint2);
		this.corner = aPoint1.max_(aPoint2);
	}

	/**
	 * Answer the width of the receiver.
	 * 
	 * @return double
	 * @category accessing
	 */
	public double width() {
		return this.corner.x - this.origin.x;
	}
}
