package sra.jun.geometry.basic;

import sra.jun.geometry.abstracts.JunGeometry;

/**
 * JunPoint class
 * 
 *  @author    nisinaka
 *  @created   1998/09/29 (by nisinaka)
 *  @updated   2000/01/06 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPoint.java,v 6.0 2003/10/28 10:52:04 nisinaka Exp $
 */
public abstract class JunPoint extends JunGeometry {

	/**
	 * Answer the receiver as an array of double.
	 * 
	 * @return double[]
	 * 
	 * @category converting
	 */
	public abstract double[] asArray();

	/**
	 * Answer the number representing the ordering of the receiver in the
	 * generality hierarchy.
	 * 
	 * @return int
	 * 
	 * @category coercing
	 */
	public abstract int generality();

	/**
	 * Computes a hash code for this object.
	 * 
	 * @return DOCUMENT ME!
	 * 
	 * @category hash function
	 */
	public abstract int hashCode();

	/**
	 * Answer the length from the origin.
	 * 
	 * @return double
	 * 
	 * @category functions
	 */
	public double length() {
		return this.rho();
	}

	/**
	 * Answer the length from the origin.
	 * 
	 * @return double
	 * 
	 * @category compatibilities
	 */
	public double r() {
		return this.rho();
	}

	/**
	 * Answer the rho as of a polar coordinate.
	 * 
	 * @return double
	 * 
	 * @category polar coordinates
	 */
	public abstract double rho();

	/**
	 * Answer the x coordinate.
	 * 
	 * @return double
	 * 
	 * @category accessing
	 */
	public abstract double x();

	/**
	 * Answer the y coordinate.
	 * 
	 * @return double
	 * 
	 * @category accessing
	 */
	public abstract double y();
}
