package sra.jun.geometry.boundaries;

import sra.jun.geometry.basic.Jun2dPoint;

/**
 * Jun2dBoundingBall class
 * 
 *  @author    NISHIHARA Satoshi
 *  @created   2000/01/24 (by NISHIHARA Satoshi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: Jun2dBoundingBall.java,v 6.0 2003/10/28 10:52:05 nisinaka Exp $
 */
public class Jun2dBoundingBall extends JunBoundingBall {
	/** center point. */
	protected Jun2dPoint center;

	/**
	 * Create a new Jun2dBoundingBall.
	 * 
	 * @param aJun2dPoint
	 * @param aNumber
	 * 
	 * @return Jun2dBoundingBall
	 */
	public static Jun2dBoundingBall Center_radius_(Jun2dPoint aJun2dPoint, double aNumber) {
		Jun2dBoundingBall ball = new Jun2dBoundingBall();
		ball.center_(aJun2dPoint);
		ball.radius_(aNumber);

		return ball;
	}

	/**
	 * Create a new instance of Jun2dBoundingBall from anArrayOfJunPoint
	 * 
	 * @param anArrayOfJunPoint
	 * 
	 * @return Jun2dBoundingBall
	 */
	public static Jun2dBoundingBall FromPoints_(Jun2dPoint[] anArrayOfJunPoint) {
		Jun2dPoint center;
		double radius;

		if (anArrayOfJunPoint.length == 0) {
			return Jun2dBoundingBall.Center_radius_(Jun2dBoundingBall.PointClass().Zero(), 0.0d);
		}

		center = anArrayOfJunPoint[0];
		radius = 0.0d;

		for (int i = 1; i < anArrayOfJunPoint.length; i++) {
			Jun2dPoint point = anArrayOfJunPoint[i];

			if (center.distance_(point) > radius) {
				Jun2dPoint farPoint = (center.minus_(point.length_(radius))).plus_(center);
				center = point.plus_(farPoint).dividedBy_(2);
				radius = (farPoint.distance_(point)) / 2;
			}
		}

		return Jun2dBoundingBall.Center_radius_(center, radius);
	}

	/**
	 * return pointClass.
	 * 
	 * @return Jun2dPoint
	 */
	public static Jun2dPoint PointClass() {
		return Jun2dPoint.Zero();
	}

	/**
	 * functions: add_(JunPoint aJunPoint).
	 * 
	 * @param aJunPoint
	 * 
	 * @return Jun2dBoundingBall
	 */
	public Jun2dBoundingBall add_(Jun2dPoint aJunPoint) {
		if (this.contains_(aJunPoint)) {
			return this;
		} else {
			Jun2dPoint farPoint = center.minus_(aJunPoint).length_(radius).plus_(center);

			return Jun2dBoundingBall.Center_radius_(aJunPoint.plus_(farPoint).dividedBy_(2), farPoint.distance_(aJunPoint));
		}
	}

	/**
	 * answer the center.
	 * 
	 * @return aJun2dPoint
	 */
	public Jun2dPoint center() {
		return this.center;
	}

	/**
	 * set the center.
	 * 
	 * @param aJunPoint
	 */
	public void center_(Jun2dPoint aJunPoint) {
		this.center = aJunPoint;
	}

	/**
	 * Answer true if the receiver contains the point.
	 * 
	 * @param aJunPoint
	 * 
	 * @return boolean
	 */
	public boolean contains_(Jun2dPoint aJunPoint) {
		return this.center.distance_(aJunPoint) <= this.radius();
	}

	/**
	 * Answer true if the Object is equal to the receiver, otherwise false.
	 * 
	 * @param other java.lang.Object
	 * 
	 * @return boolean
	 */
	public boolean equals(Object other) {
		if (other.getClass() == this.getClass()) {
			Jun2dBoundingBall aJunBoundingBall = (Jun2dBoundingBall) other;

			return ((center.equals(aJunBoundingBall.center)) && (radius == aJunBoundingBall.radius));
		}

		return false;
	}

	/**
	 * functions: expandedBy_(double aNumber).
	 * 
	 * @param aNumber
	 * 
	 * @return Jun2dBoundingBall
	 */
	public Jun2dBoundingBall expandedBy_(double aNumber) {
		return Jun2dBoundingBall.Center_radius_(center, radius + aNumber);
	}

	/**
	 * functions: insetBy_(double aNumber).
	 * 
	 * @param aNumber
	 * 
	 * @return Jun2dBoundingBall
	 */
	public Jun2dBoundingBall insetBy_(double aNumber) {
		return Jun2dBoundingBall.Center_radius_(center, Math.abs(radius - aNumber));
	}

	/**
	 * Answer true if the receiver intersects the JunBoundingBall.
	 * 
	 * @param aJunBoundingBall
	 * 
	 * @return boolean
	 */
	public boolean intersects_(Jun2dBoundingBall aJunBoundingBall) {
		return this.center.distance_(aJunBoundingBall.center()) <= (this.radius() + aJunBoundingBall.radius());
	}

	/**
	 * answer is2d.
	 * 
	 * @return boolean
	 */
	public boolean is2d() {
		return true;
	}

	/**
	 * functions: merge_(Jun2dBoundingBall aJunBoundingBall).
	 * 
	 * @param aJunBoundingBall
	 * 
	 * @return Jun2dBoundingBall
	 */
	public Jun2dBoundingBall merge_(Jun2dBoundingBall aJunBoundingBall) {
		double distance = center.distance_(aJunBoundingBall.center());

		if ((distance + aJunBoundingBall.radius()) < this.radius()) {
			return (Jun2dBoundingBall) this.copy();
		}

		if ((distance + this.radius()) < aJunBoundingBall.radius()) {
			return (Jun2dBoundingBall) aJunBoundingBall.copy();
		}

		Jun2dPoint farPoint1 = this.center.minus_(aJunBoundingBall.center()).length_(this.radius()).plus_(this.center);
		Jun2dPoint farPoint2 = aJunBoundingBall.center.minus_(this.center()).length_(aJunBoundingBall.radius()).plus_(aJunBoundingBall.radius);

		return Jun2dBoundingBall.Center_radius_(farPoint1.plus_(farPoint2).dividedBy_(2), farPoint1.distance_(farPoint2));
	}

	/**
	 * return an rounded copy of the receiver.
	 * 
	 * @return Jun2dBoundingBall
	 */
	public Jun2dBoundingBall rounded() {
		return Jun2dBoundingBall.Center_radius_(center.rounded(), Math.round(this.radius));
	}

	/**
	 * return an truncated copy of the receiver.
	 * 
	 * @return Jun2dBoundingBall
	 */
	public Jun2dBoundingBall truncated() {
		double newRadius = (this.radius >= 0) ? Math.ceil(radius) : Math.floor(radius);

		return Jun2dBoundingBall.Center_radius_(center.truncated(), newRadius);
	}
}
