package sra.jun.geometry.boundaries;

import java.util.*;
import sra.jun.geometry.abstracts.JunGeometry;
import sra.jun.geometry.basic.Jun2dPoint;
import sra.jun.geometry.surfaces.Jun2dPolygon;
import sra.jun.opengl.objects.*;
import sra.smalltalk.*;

/**
 * Jun2dPolygonalBoundary class
 * 
 *  @author    nisinaka
 *  @created   1999/08/04 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: Jun2dPolygonalBoundary.java,v 6.0 2003/10/28 10:52:05 nisinaka Exp $
 */
public class Jun2dPolygonalBoundary extends JunGeometry {
	/** The collection of polygons which is sorted with their area. */
	protected StSortedCollection polygons = null;

	/**
	 * Defalut constructor.
	 */
	public Jun2dPolygonalBoundary() {
		this.initialize();
	}

	/**
	 * Create a new instance of Jun2dPolygonalBoundary and initialize it with
	 * the specified array of Jun2dPolygon.
	 * 
	 * @param anArrayOfJun2dPolygon sra.jun.geometry.surfaces.Jun2dPolygon[]
	 */
	public Jun2dPolygonalBoundary(Jun2dPolygon[] anArrayOfJun2dPolygon) {
		this();
		this.setPolygons_(anArrayOfJun2dPolygon);
	}

	/**
	 * Answer the default sorting block.
	 * 
	 * @return sra.smalltalk.StBlockClosure
	 */
	public static StBlockClosure SortingBlock() {
		return new StBlockClosure() {
			public Object value_value_(Object more, Object less) {
				return new Boolean(((Jun2dPolygon) more).area() > ((Jun2dPolygon) less).area());
			}
		};
	}

	/**
	 * Add the polygon to the receiver.
	 * 
	 * @param aJun2dPolygon sra.jun.geometry.surfaces.Jun2dPolygon
	 */
	public void add_(Jun2dPolygon aJun2dPolygon) {
		polygons.add_(aJun2dPolygon);
	}

	/**
	 * Add the polygon as a hole to the receiver.
	 * 
	 * @param aJun2dPolygon sra.jun.geometry.surfaces.Jun2dPolygon
	 */
	public void addHole_(Jun2dPolygon aJun2dPolygon) {
		this.add_(aJun2dPolygon.asNegativePolygon());
	}

	/**
	 * Add the polygon to the receiver.
	 * 
	 * @param aJun2dPolygon sra.jun.geometry.surfaces.Jun2dPolygon
	 */
	public void addPolygon_(Jun2dPolygon aJun2dPolygon) {
		this.add_(aJun2dPolygon.asPositivePolygon());
	}

	/**
	 * Convert the receiver as an array of polygons.
	 * 
	 * @return sra.jun.geometry.surfaces.Jun2dPolygon[]
	 */
	public Jun2dPolygon[] asArrayOfPolygons() {
		final Vector polygonStream = new Vector();
		this.asArrayOfPolygonsDo_(new StBlockClosure() {
			public Object value_(Object polygon) {
				polygonStream.addElement(polygon);

				return null;
			}
		});

		Jun2dPolygon[] anArrayOfPolygons = new Jun2dPolygon[polygonStream.size()];
		polygonStream.copyInto(anArrayOfPolygons);

		return anArrayOfPolygons;
	}

	/**
	 * Enumerate the array of polygons and evaluate the block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object asArrayOfPolygonsDo_(StBlockClosure aBlock) {
		Vector islandsCollection = new Vector();
		Vector holesCollection = new Vector();
		Object[] thePolygons = polygons._asArray();

		for (int i = 0; i < thePolygons.length; i++) {
			Jun2dPolygon polygon = (Jun2dPolygon) thePolygons[i];

			if (polygon.isPositive()) {
				islandsCollection.addElement(polygon);
			} else {
				holesCollection.addElement(polygon);
			}
		}

		Object[] islands = (new StSortedCollection(new StBlockClosure() {
			public Object value_value_(Object less, Object more) {
				return new Boolean(((Jun2dPolygon) less).area() < ((Jun2dPolygon) more).area());
			}
		}, islandsCollection))._asArray();
		Jun2dPolygon[] holes = new Jun2dPolygon[holesCollection.size()];
		holesCollection.copyInto(holes);

		Hashtable holesOfIslands = new Hashtable(islands.length);

		for (int i = 0; i < islands.length; i++) {
			holesOfIslands.put(islands[i], new Vector());
		}

		for (int i = 0; i < holes.length; i++) {
			Jun2dPolygon hole = holes[i];
			Jun2dPolygon theIsland = null;

			for (int j = 0; j < islands.length; j++) {
				Jun2dPolygon anotherIsland = (Jun2dPolygon) islands[j];

				if (anotherIsland.containsPoint_(hole.pointAt_(0))) {
					theIsland = anotherIsland;

					break;
				}
			}

			if (theIsland != null) {
				((Vector) holesOfIslands.get(theIsland)).addElement(hole);
			}
		}

		for (int i = 0; i < islands.length; i++) {
			Jun2dPolygon island = (Jun2dPolygon) islands[i];
			Vector holesOfTheIsland = (Vector) holesOfIslands.get(island);
			Jun2dPolygon theIsland = island;

			if (holesOfTheIsland.isEmpty() == false) {
				theIsland = this._polygonFrom_withHoles_(island, holesOfTheIsland);
			}

			Object result = aBlock.value_(theIsland);

			if (result != null) {
				return result;
			}
		}

		return null;
	}

	/**
	 * Enumerate all of the convex polygons and evaluate the block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object asConvexPolygonsDo_(final StBlockClosure aBlock) {
		return this.asArrayOfPolygonsDo_(new StBlockClosure() {
			public Object value_(Object p) {
				return ((Jun2dPolygon) p).asConvexPolygonsDo_(aBlock);
			}
		});
	}

	/**
	 * Convert the receiver to a JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		final Vector objects = new Vector();
		this.asJunOpenGL3dObjectsDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				objects.add(anObject);

				return null;
			}
		});

		return (objects.size() == 1) ? (JunOpenGL3dObject) objects.firstElement() : JunOpenGL3dCompoundObject.Components_(objects);
	}

	/**
	 * Enumerate all of the polygons as JunOpenGL3dObjects and evaluate the
	 * block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object asJunOpenGL3dObjectsDo_(final StBlockClosure aBlock) {
		return this.asConvexPolygonsDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				aBlock.value_(((Jun2dPolygon) anObject).asJunOpenGL3dObject());

				return null;
			}
		});
	}

	/**
	 * Initialize the receiver.
	 */
	public void initialize() {
		polygons = new StSortedCollection(SortingBlock());
	}

	/**
	 * Answer a copy of the receiver scaled by the specified extent.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * 
	 * @return sra.jun.geometry.boundaries.Jun2dPolygonalBoundary
	 */
	public Jun2dPolygonalBoundary scaledBy_(Jun2dPoint aPoint) {
		Object[] myPolygons = polygons._asArray();
		Jun2dPolygon[] scaledPolygons = new Jun2dPolygon[myPolygons.length];

		if ((aPoint.x() * aPoint.y()) > 0) {
			for (int i = 0; i < myPolygons.length; i++) {
				scaledPolygons[i] = ((Jun2dPolygon) myPolygons[i]).scaledBy_(aPoint);
			}
		} else {
			for (int i = 0; i < myPolygons.length; i++) {
				scaledPolygons[i] = ((Jun2dPolygon) myPolygons[i]).scaledBy_(aPoint).reversed();
			}
		}

		return new Jun2dPolygonalBoundary(scaledPolygons);
	}

	/**
	 * Answer a copy of the receiver translated by the specified extent.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * 
	 * @return sra.jun.geometry.boundaries.Jun2dPolygonalBoundary
	 */
	public Jun2dPolygonalBoundary translatedBy_(Jun2dPoint aPoint) {
		Object[] myPolygons = polygons._asArray();
		Jun2dPolygon[] scaledPolygons = new Jun2dPolygon[myPolygons.length];

		for (int i = 0; i < myPolygons.length; i++) {
			scaledPolygons[i] = ((Jun2dPolygon) myPolygons[i]).translatedBy_(aPoint);
		}

		return new Jun2dPolygonalBoundary(scaledPolygons);
	}

	/**
	 * Answer a polygon with holes.
	 * 
	 * @param aJun2dPolygon sra.jun.geometry.surfaces.Jun2dPolygon
	 * @param aCollectionOfJun2dPolygons java.util.Vector
	 * 
	 * @return sra.jun.geometry.surfaces.Jun2dPolygon
	 */
	protected Jun2dPolygon _polygonFrom_withHoles_(Jun2dPolygon aJun2dPolygon, Vector aCollectionOfJun2dPolygons) {
		Jun2dPolygon[] anArrayOfJun2dPolygons = new Jun2dPolygon[aCollectionOfJun2dPolygons.size()];
		aCollectionOfJun2dPolygons.copyInto(anArrayOfJun2dPolygons);

		Vector loops = new Vector(anArrayOfJun2dPolygons.length + 1);
		loops.addElement(aJun2dPolygon.points());

		for (int i = 0; i < anArrayOfJun2dPolygons.length; i++) {
			loops.addElement(anArrayOfJun2dPolygons[i].points());
		}

		while (loops.size() > 1) {
			Jun2dPoint[] loop = (Jun2dPoint[]) loops.lastElement();
			loops.removeElementAt(loops.size() - 1);

			int maxYIndex = 0;
			Jun2dPoint maxYPoint = loop[0];

			for (int index = 1; index < loop.length; index++) {
				Jun2dPoint point = loop[index];

				if (point.y() > maxYPoint.y()) {
					maxYPoint = point;
					maxYIndex = index;
				}
			}

			Jun2dPoint minYPoint = null;
			int minYPointIndex = -1;
			int minYLoopIndex = -1;

			for (int pairLoopIndex = 0; pairLoopIndex < loops.size(); pairLoopIndex++) {
				Jun2dPoint[] pairLoop = (Jun2dPoint[]) loops.elementAt(pairLoopIndex);

				for (int i = 0; i < pairLoop.length; i++) {
					Jun2dPoint p1 = pairLoop[i];
					Jun2dPoint p2 = pairLoop[(i + 1) % pairLoop.length];

					if ((p1.x() > p2.x()) && (p1.x() > maxYPoint.x()) && (p2.x() <= maxYPoint.x())) {
						double y = (((maxYPoint.x() - p1.x()) * (p2.y() - p1.y())) / (p2.x() - p1.x())) + p1.y();

						if ((maxYPoint.y() <= y) && (minYPoint == null || y <= minYPoint.y())) {
							minYPoint = new Jun2dPoint(maxYPoint.x(), y);
							minYPointIndex = i;
							minYLoopIndex = pairLoopIndex;
						}
					}
				}
			}

			if (minYPoint == null) {
				throw SmalltalkException.Error("internal error");
			}

			Jun2dPoint[] minYLoop = (Jun2dPoint[]) loops.elementAt(minYLoopIndex);
			loops.removeElementAt(minYLoopIndex);

			Vector newLoop = new Vector();

			for (int i = 0; i <= maxYIndex; i++) {
				newLoop.addElement(loop[i]);
			}

			if (minYPoint.equals(minYLoop[(minYPointIndex + 1) % minYLoop.length]) == false) {
				newLoop.addElement(minYPoint);
			}

			for (int i = minYPointIndex + 1; i < minYLoop.length; i++) {
				newLoop.addElement(minYLoop[i]);
			}

			for (int i = 0; i <= minYPointIndex; i++) {
				newLoop.addElement(minYLoop[i]);
			}

			if (minYPoint.equals(minYLoop[minYPointIndex]) == false) {
				newLoop.addElement(minYPoint);
			}

			for (int i = maxYIndex; i < loop.length; i++) {
				newLoop.addElement(loop[i]);
			}

			Jun2dPoint[] thePoints = new Jun2dPoint[newLoop.size()];
			newLoop.copyInto(thePoints);

			if (minYLoopIndex == 0) {
				loops.insertElementAt(thePoints, 0);
			} else {
				loops.addElement(thePoints);
			}
		}

		return new Jun2dPolygon((Jun2dPoint[]) loops.firstElement());
	}

	/**
	 * Set the polygons.
	 * 
	 * @param anArrayOfPolygon sra.jun.geometry.surfaces.Jun2dPolygon[]
	 */
	private void setPolygons_(Jun2dPolygon[] anArrayOfPolygon) {
		for (int i = 0; i < anArrayOfPolygon.length; i++) {
			this.add_(anArrayOfPolygon[i]);
		}
	}
}
