package sra.jun.geometry.boundaries;

import sra.jun.geometry.basic.Jun3dPoint;

/**
 * Jun3dBoundingBall class
 * 
 *  @author    NISHIHARA Satoshi
 *  @created   2000/01/24 (by NISHIHARA Satoshi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: Jun3dBoundingBall.java,v 6.0 2003/10/28 10:52:05 nisinaka Exp $
 */
public class Jun3dBoundingBall extends JunBoundingBall {
	/** center point. */
	protected Jun3dPoint center;

	/**
	 * Create a new Jun3dBoundingBall.
	 * 
	 * @param aJun3dPoint
	 * @param aNumber
	 * 
	 * @return Jun3dBoundingBall
	 */
	public static Jun3dBoundingBall Center_radius_(Jun3dPoint aJun3dPoint, double aNumber) {
		Jun3dBoundingBall ball = new Jun3dBoundingBall();
		ball.center_(aJun3dPoint);
		ball.radius_(aNumber);

		return ball;
	}

	/**
	 * Create a new instance of Jun3dBoundingBall from anArrayOfJunPoint
	 * 
	 * @param anArrayOfJunPoint
	 * 
	 * @return Jun3dBoundingBall
	 */
	public static Jun3dBoundingBall FromPoints_(Jun3dPoint[] anArrayOfJunPoint) {
		Jun3dPoint center;
		double radius;

		if (anArrayOfJunPoint.length == 0) {
			return Jun3dBoundingBall.Center_radius_(Jun3dBoundingBall.PointClass().Zero(), 0.0d);
		}

		center = anArrayOfJunPoint[0];
		radius = 0.0d;

		for (int i = 1; i < anArrayOfJunPoint.length; i++) {
			Jun3dPoint point = anArrayOfJunPoint[i];

			if (center.distance_(point) > radius) {
				//farPoint := (center - point length: radius) + center.
				Jun3dPoint farPoint = (center.minus_(point).length_(radius)).plus_(center);
				center = point.plus_(farPoint).dividedBy_(2);
				radius = farPoint.distance_(point) / 2;
			}
		}

		return Jun3dBoundingBall.Center_radius_(center, radius);
	}

	/**
	 * return pointClass.
	 * 
	 * @return Jun3dPoint
	 */
	public static Jun3dPoint PointClass() {
		return Jun3dPoint.Zero();
	}

	/**
	 * functions: add_(Jun3dPoint aJunPoint).
	 * 
	 * @param aJunPoint
	 * 
	 * @return JunBoundingBall
	 */
	public Jun3dBoundingBall add_(Jun3dPoint aJunPoint) {
		if (this.contains_(aJunPoint)) {
			return this;
		} else {
			Jun3dPoint farPoint = center.minus_(aJunPoint).length_(radius).plus_(center);

			return Jun3dBoundingBall.Center_radius_(aJunPoint.plus_(farPoint).dividedBy_(2), farPoint.distance_(aJunPoint));
		}
	}

	/**
	 * answer the center.
	 * 
	 * @return aJun3dPoint
	 */
	public Jun3dPoint center() {
		return this.center;
	}

	/**
	 * set the center.
	 * 
	 * @param aJunPoint
	 */
	public void center_(Jun3dPoint aJunPoint) {
		this.center = aJunPoint;
	}

	/**
	 * Answer true if the receiver contains the point.
	 * 
	 * @param aJunPoint
	 * 
	 * @return boolean
	 */
	public boolean contains_(Jun3dPoint aJunPoint) {
		return this.center.distance_(aJunPoint) <= this.radius();
	}

	/**
	 * Answer true if the Object is equal to the receiver, otherwise false.
	 * 
	 * @param other java.lang.Object
	 * 
	 * @return boolean
	 */
	public boolean equals(Object other) {
		if (other.getClass() == this.getClass()) {
			Jun3dBoundingBall aJunBoundingBall = (Jun3dBoundingBall) other;

			return ((center.equals(aJunBoundingBall.center)) && (radius == aJunBoundingBall.radius));
		}

		return false;
	}

	/**
	 * functions: expandedBy_(double aNumber).
	 * 
	 * @param aNumber
	 * 
	 * @return Jun3dBoundingBall
	 */
	public Jun3dBoundingBall expandedBy_(double aNumber) {
		return Jun3dBoundingBall.Center_radius_(center, radius + aNumber);
	}

	/**
	 * functions: insetBy_(double aNumber).
	 * 
	 * @param aNumber
	 * 
	 * @return Jun3dBoundingBall
	 */
	public Jun3dBoundingBall insetBy_(double aNumber) {
		return Jun3dBoundingBall.Center_radius_(center, Math.abs(radius - aNumber));
	}

	/**
	 * Answer true if the receiver intersects the JunBoundingBall.
	 * 
	 * @param aJunBoundingBall
	 * 
	 * @return boolean
	 */
	public boolean intersects_(Jun3dBoundingBall aJunBoundingBall) {
		return this.center.distance_(aJunBoundingBall.center()) <= (this.radius() + aJunBoundingBall.radius());
	}

	/**
	 * answer is3d.
	 * 
	 * @return boolean
	 */
	public boolean is3d() {
		return true;
	}

	/**
	 * functions: merge_(Jun3dBoundingBall aJunBoundingBall).
	 * 
	 * @param aJunBoundingBall
	 * 
	 * @return Jun3dBoundingBall
	 */
	public Jun3dBoundingBall merge_(Jun3dBoundingBall aJunBoundingBall) {
		double distance = center.distance_(aJunBoundingBall.center());

		if ((distance + aJunBoundingBall.radius()) < this.radius()) {
			return (Jun3dBoundingBall) this.copy();
		}

		if ((distance + this.radius()) < aJunBoundingBall.radius()) {
			return (Jun3dBoundingBall) aJunBoundingBall.copy();
		}

		Jun3dPoint farPoint1 = this.center.minus_(aJunBoundingBall.center()).length_(this.radius()).plus_(this.center);
		Jun3dPoint farPoint2 = aJunBoundingBall.center.minus_(this.center()).length_(aJunBoundingBall.radius()).plus_(aJunBoundingBall.radius);

		return Jun3dBoundingBall.Center_radius_(farPoint1.plus_(farPoint2).dividedBy_(2), farPoint1.distance_(farPoint2));
	}

	/**
	 * return an rounded copy of the receiver.
	 * 
	 * @return Jun3dBoundingBall
	 */
	public Jun3dBoundingBall rounded() {
		return Jun3dBoundingBall.Center_radius_(center.rounded(), Math.round(this.radius));
	}

	/**
	 * return an truncated copy of the receiver.
	 * 
	 * @return Jun3dBoundingBall
	 */
	public Jun3dBoundingBall truncated() {
		double newRadius = (this.radius >= 0) ? Math.ceil(radius) : Math.floor(radius);

		return Jun3dBoundingBall.Center_radius_(center.truncated(), newRadius);
	}
}
