package sra.jun.geometry.boundaries;

import java.awt.*;
import java.util.*;
import sra.jun.collections.linkedlist.JunDoubleLinkedList;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.surfaces.*;
import sra.jun.goodies.cursors.JunCursors;
import sra.jun.opengl.display.*;
import sra.jun.opengl.objects.*;
import sra.jun.topology.elements.*;
import sra.jun.topology.euleroperators.*;
import sra.smalltalk.*;

/**
 * JunBorderGenerator class
 * 
 *  @author    Hirotsugu Kondo
 *  @created   1999/1/8 (by Hirotsugu Kondo)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunBorderGenerator.java,v 6.0 2003/10/28 10:52:05 nisinaka Exp $
 */
public class JunBorderGenerator extends StObject {
	private static final StBlockClosure[] MarchingCubes = InitializeMarchingCubes();

	/**
	 * DOCUMENT ME!
	 * 
	 * @param evaluationBlock DOCUMENT ME!
	 * @param aNumber DOCUMENT ME!
	 * @param aNumber2 DOCUMENT ME!
	 * @param xInterval DOCUMENT ME!
	 * @param yInterval DOCUMENT ME!
	 * @param zInterval DOCUMENT ME!
	 * @param interimBlock DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static synchronized JunBody ContourBodyFrom_at_sign_xInterval_yInterval_zInterval_interim_(
		final StBlockClosure evaluationBlock,
		final double aNumber,
		final int aNumber2,
		final StInterval xInterval,
		final StInterval yInterval,
		final StInterval zInterval,
		final StBlockClosure interimBlock) {
		final int xSize = xInterval.size();
		final int ySize = yInterval.size();
		final int zSize = zInterval.size();
		final JunBody body = new JunBody();
		final Hashtable vertices = new Hashtable();
		final Vector backloops = new Vector();
		final boolean kIsDebug = false;
		long timeToRun = 0L;

		if (kIsDebug) {
			timeToRun = System.currentTimeMillis();
		}

		final StBlockClosure interpolationBlock = new StBlockClosure() {
			public Object value_(Object vertexObject) {
				double x1;
				double y1;
				double z1;
				double x2;
				double y2;
				double z2;
				double value1;
				double value2;
				double t;
				Jun3dPoint p1;
				Jun3dPoint p2;
				JunBorderVertex vertex = (JunBorderVertex) vertexObject;
				x1 = xInterval.at_((int) Math.min(Math.max(Math.round(vertex.vertex1().x()), 1), xSize - 1));
				y1 = yInterval.at_((int) Math.min(Math.max(Math.round(vertex.vertex1().y()), 1), ySize - 1));
				z1 = zInterval.at_((int) Math.min(Math.max(Math.round(vertex.vertex1().z()), 1), zSize - 1));
				x2 = xInterval.at_((int) Math.min(Math.max(Math.round(vertex.vertex2().x()), 1), xSize - 1));
				y2 = yInterval.at_((int) Math.min(Math.max(Math.round(vertex.vertex2().y()), 1), ySize - 1));
				z2 = zInterval.at_((int) Math.min(Math.max(Math.round(vertex.vertex2().z()), 1), zSize - 1));
				value1 = ((Double) (evaluationBlock.value_value_value_(new Double(x1), new Double(y1), new Double(z1)))).doubleValue();
				value2 = ((Double) (evaluationBlock.value_value_value_(new Double(x2), new Double(y2), new Double(z2)))).doubleValue();
				t = (value1 == value2) ? 0 : (vertex.label() - value1) / (value2 - value1);
				t = Math.min(Math.max(t, 0), 1);
				p1 = new Jun3dPoint(x1, y1, z1);
				p2 = new Jun3dPoint(x2, y2, z2);

				return (Jun3dPoint) (((p2.minus_(p1)).multipliedBy_(t)).plus_(p1));
			}
		};

		final StBlockClosure edgeCreationBlock = new StBlockClosure() {
			public Object value_value_(Object vertex1Object, Object vertex2Object) {
				JunVertex vertex1 = (JunVertex) vertex1Object;
				JunVertex vertex2 = (JunVertex) vertex2Object;

				if (!vertex1.hasEdgeToVertex_(vertex2)) {
					JunLoop loop = vertex1.commonLoopWithVertex_(vertex2);

					if (loop != null) {
						JunMEL mel = JunMEL.Body_loop_vertex_vertex_(body, loop, vertex1, vertex2);
						mel.doOperation();

						JunLoop returnLoop = mel.newLoop();
						backloops.addElement(returnLoop);

						return returnLoop;
					} else {
						JunLoop loop1;
						JunLoop loop2;
						int backloopsContainsCount = 0;
						JunLoop backloopsContains = null;
						JunLoop[] loops1 = vertex1.loops();

						for (int i = 0; i < loops1.length; i++) {
							if (backloops.contains(loops1[i])) {
								backloopsContains = (backloopsContainsCount == 0) ? loops1[i] : backloopsContains;
								backloopsContainsCount++;
							}
						}

						if (backloopsContainsCount != 1) {
							throw SmalltalkException.Error("self halt");
						}

						loop1 = backloopsContains;
						backloopsContainsCount = 0;

						JunLoop[] loops2 = vertex2.loops();

						for (int i = 0; i < loops2.length; i++) {
							if (backloops.contains(loops2[i])) {
								backloopsContains = (backloopsContainsCount == 0) ? loops2[i] : backloopsContains;
								backloopsContainsCount++;
							}
						}

						if (backloopsContainsCount != 1) {
							throw SmalltalkException.Error("self halt");
						}

						loop2 = backloopsContains;

						JunMEKL mekl = JunMEKL.Body_killLoop_aliveLoop_vertex_vertex_(body, loop1, loop2, vertex1, vertex2);
						mekl.doOperation();
						backloops.removeElement(loop1);

						return loop1;
					}
				}

				return null;
			}
		};

		final StBlockClosure triangleCreationBlock0 = new StBlockClosure() {
			public Object valueWithArguments_(Object[] ObjectArray) {
				/*
				*        aJunBorderVertex must be an <strong>Hashable.</strong>
				*        So you must implement the <code>hashCode</code> and
				*        the <code>equals</code> method.
				*
				* @see     java.util.Hashtable
				*/
				JunBorderVertex point1 = (JunBorderVertex) ObjectArray[0];
				JunBorderVertex point2 = (JunBorderVertex) ObjectArray[1];
				JunBorderVertex point3 = (JunBorderVertex) ObjectArray[2];
				JunVertex vertex1;
				JunVertex vertex2;
				JunVertex vertex3;
				JunMEVVL mevvl = JunMEVVL.Body_point_point_(body, (Jun3dPoint) interpolationBlock.value_(point1), (Jun3dPoint) interpolationBlock.value_(point2));
				mevvl.doOperation();
				vertices.put(point1, vertex1 = mevvl.newVertex1());
				vertices.put(point2, vertex2 = mevvl.newVertex2());

				JunLoop loop = mevvl.newLoop();
				JunMEV mev = JunMEV.Body_vertex_loop_point_(body, vertex2, loop, (Jun3dPoint) (interpolationBlock.value_(point3)));
				mev.doOperation();
				vertices.put(point3, vertex3 = mev.newVertex());

				JunMEL mel = JunMEL.Body_loop_vertex_vertex_(body, loop, vertex3, vertex1);
				mel.doOperation();

				JunLoop returnLoop = vertex1.loopToVertex_(vertex3);
				backloops.addElement(returnLoop);

				return returnLoop;
			}
		};

		final StBlockClosure triangleCreationBlock1 = new StBlockClosure() {
			public Object value_value_value_(Object vertex1Object, Object point2Object, Object point3Object) {
				JunVertex vertex1 = (JunVertex) vertex1Object;
				JunBorderVertex point2 = (JunBorderVertex) point2Object;
				JunBorderVertex point3 = (JunBorderVertex) point3Object;
				JunVertex vertex2;
				JunVertex vertex3;
				JunLoop loop = vertex1.detectLoop_ifNone_(new StBlockClosure() {
					public Object value_(Object anObject) {
						return new Boolean(backloops.contains((JunLoop) anObject));
					}
				}, new StBlockClosure());
				JunMEV mev = JunMEV.Body_vertex_loop_point_(body, vertex1, loop, (Jun3dPoint) (interpolationBlock.value_(point2)));
				mev.doOperation();
				vertices.put(point2, vertex2 = mev.newVertex());
				mev = JunMEV.Body_vertex_loop_point_(body, vertex2, loop, (Jun3dPoint) (interpolationBlock.value_(point3)));
				mev.doOperation();
				vertices.put(point3, vertex3 = mev.newVertex());

				JunMEL mel = JunMEL.Body_loop_vertex_vertex_(body, loop, vertex3, vertex1);
				mel.doOperation();
				backloops.addElement(mel.newLoop());

				JunLoop foreloop = vertex1.loopToVertex_(vertex2);

				if (foreloop.numberOfVertexes() > 3) {
					throw SmalltalkException.Error("self halt");
				}

				if (!foreloop.includesVertex_(vertex1)) {
					throw SmalltalkException.Error("self halt");
				}

				if (!foreloop.includesVertex_(vertex2)) {
					throw SmalltalkException.Error("self halt");
				}

				if (!foreloop.includesVertex_(vertex3)) {
					throw SmalltalkException.Error("self halt");
				}

				backloops.removeElement(foreloop);

				return foreloop;
			}
		};

		final StBlockClosure triangleCreationBlock2 = new StBlockClosure() {
			public Object value_value_value_(Object vertex1Object, Object vertex2Object, Object point3Object) {
				JunVertex vertex1 = (JunVertex) vertex1Object;
				JunVertex vertex2 = (JunVertex) vertex2Object;
				JunBorderVertex point3 = (JunBorderVertex) point3Object;
				JunVertex vertex3;
				edgeCreationBlock.value_value_(vertex1, vertex2);

				JunLoop loop = vertex1.loopToVertex_(vertex2);
				JunMEV mev = JunMEV.Body_vertex_loop_point_(body, vertex2, loop, (Jun3dPoint) (interpolationBlock.value_(point3)));
				mev.doOperation();
				vertices.put(point3, vertex3 = mev.newVertex());

				JunMEL mel = JunMEL.Body_loop_vertex_vertex_(body, loop, vertex3, vertex1);
				mel.doOperation();
				backloops.addElement(mel.newLoop());

				JunLoop foreloop = vertex1.loopToVertex_(vertex2);

				if (foreloop.numberOfVertexes() > 3) {
					throw SmalltalkException.Error("self halt");
				}

				backloops.removeElement(foreloop);

				return foreloop;
			}
		};

		final StBlockClosure triangleCreationBlock3 = new StBlockClosure() {
			public Object value_value_value_(Object vertex1Object, Object vertex2Object, Object vertex3Object) {
				JunVertex vertex1 = (JunVertex) vertex1Object;
				JunVertex vertex2 = (JunVertex) vertex2Object;
				JunVertex vertex3 = (JunVertex) vertex3Object;
				edgeCreationBlock.value_value_(vertex1, vertex2);

				JunLoop foreloop = vertex1.loopToVertex_(vertex2);
				edgeCreationBlock.value_value_(vertex2, vertex3);
				foreloop = vertex1.loopToVertex_(vertex2);

				if (foreloop != (vertex2.loopToVertex_(vertex3))) {
					throw SmalltalkException.Error("self halt");
				}

				if (!vertex3.hasEdgeToVertex_(vertex1)) {
					JunMEL mel = JunMEL.Body_loop_vertex_vertex_(body, foreloop, vertex3, vertex1);
					mel.doOperation();
					backloops.addElement(mel.newLoop());
				}

				foreloop = vertex1.loopToVertex_(vertex2);

				if (foreloop.numberOfVertexes() > 3) {
					throw SmalltalkException.Error("self halt");
				}

				if (!foreloop.includesVertex_(vertex1)) {
					throw SmalltalkException.Error("self halt");
				}

				if (!foreloop.includesVertex_(vertex2)) {
					throw SmalltalkException.Error("self halt");
				}

				if (!foreloop.includesVertex_(vertex3)) {
					throw SmalltalkException.Error("self halt");
				}

				backloops.removeElement(foreloop);

				return foreloop;
			}
		};

		final StBlockClosure triangleCreationBlock = new StBlockClosure() {
			public Object value_(Object triangleObject) {
				JunBorderVertex[] triangle = (JunBorderVertex[]) triangleObject;
				JunVertex vertex1 = (JunVertex) vertices.get(triangle[1 - 1]);
				JunVertex vertex2 = (JunVertex) vertices.get(triangle[2 - 1]);
				JunVertex vertex3 = (JunVertex) vertices.get(triangle[3 - 1]);

				if (vertex1 != null) {
					if (vertex2 != null) {
						if (vertex3 != null) {
							triangleCreationBlock3.value_value_value_(vertex1, vertex2, vertex3);
						} else {
							triangleCreationBlock2.value_value_value_(vertex1, vertex2, triangle[3 - 1]);
						}
					} else {
						if (vertex3 != null) {
							triangleCreationBlock2.value_value_value_(vertex3, vertex1, triangle[2 - 1]);
						} else {
							triangleCreationBlock1.value_value_value_(vertex1, triangle[2 - 1], triangle[3 - 1]);
						}
					}
				} else {
					if (vertex2 != null) {
						if (vertex3 != null) {
							triangleCreationBlock2.value_value_value_(vertex2, vertex3, triangle[1 - 1]);
						} else {
							triangleCreationBlock1.value_value_value_(vertex2, triangle[3 - 1], triangle[1 - 1]);
						}
					} else {
						if (vertex3 != null) {
							triangleCreationBlock1.value_value_value_(vertex3, triangle[1 - 1], triangle[2 - 1]);
						} else {
							triangleCreationBlock0.valueWithArguments_(triangle);
						}
					}
				}

				return null;
			}
		};

		final StBlockClosure trianglesWithBlock = new StBlockClosure() {
			public Object value_value_value_(Object xObject, Object yObject, Object zObject) {
				int x = (int) ((Double) xObject).doubleValue();
				int y = (int) ((Double) yObject).doubleValue();
				int z = (int) ((Double) zObject).doubleValue();

				if (((x >= 1) && (x < xSize)) && ((y >= 1) && (y < ySize)) && ((z >= 1) && (z < zSize))) {
					return evaluationBlock.value_value_value_(new Double(xInterval.at_(x)), new Double(yInterval.at_(y)), new Double(zInterval.at_(z)));
				} else {
					return new Double(aNumber);
				}
			}
		};

		for (int zIndex = 0; zIndex < zSize; zIndex++) {
			for (int yIndex = 0; yIndex < ySize; yIndex++) {
				for (int xIndex = 0; xIndex < xSize; xIndex++) {
					int result = __trianglesWith_at_sign_inUnitCubeAt_do_(trianglesWithBlock, aNumber, aNumber2, new Jun3dPoint(xIndex, yIndex, zIndex), triangleCreationBlock);
				}
			}

			interimBlock.value_(body);
		}

		if (!backloops.isEmpty()) {
			final String kErrStr = "backloops should be empty.";
			throw SmalltalkException.Error(kErrStr);
		}

		if (kIsDebug) {
			timeToRun = System.currentTimeMillis() - timeToRun;
			System.out.println(timeToRun);
		}

		return body;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param evaluationBlock DOCUMENT ME!
	 * @param aNumber DOCUMENT ME!
	 * @param aNumber2 DOCUMENT ME!
	 * @param xInterval DOCUMENT ME!
	 * @param yInterval DOCUMENT ME!
	 * @param zInterval DOCUMENT ME!
	 * @param interimBlock DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static synchronized JunPolygon[] ContourPolygonsFrom_at_sign_xInterval_yInterval_zInterval_interim_(
		final StBlockClosure evaluationBlock,
		final double aNumber,
		final int aNumber2,
		final StInterval xInterval,
		final StInterval yInterval,
		final StInterval zInterval,
		final StBlockClosure interimBlock) {
		final boolean kIsDebug = false;
		long timeToRun = 0L;

		if (kIsDebug) {
			timeToRun = System.currentTimeMillis();
		}

		final Vector polygons = new Vector();
		final StInterval xInterval_ = new StInterval(1, xInterval.size() - 1);
		final StInterval yInterval_ = new StInterval(1, yInterval.size() - 1);
		StInterval zInterval_ = new StInterval(1, zInterval.size() - 1);
		zInterval_.do_(new StBlockClosure() {
			public Object value_(Object valueObjectZ) {
				final double zIndex = ((Double) valueObjectZ).doubleValue();
				yInterval_.do_(new StBlockClosure() {
					public Object value_(Object valueObjectY) {
						final double yIndex = ((Double) valueObjectY).doubleValue();
						xInterval_.do_(new StBlockClosure() {
							public Object value_(Object valueObjectX) {
								final double xIndex = ((Double) valueObjectX).doubleValue();
								int value = __trianglesWith_at_sign_inUnitCubeAt_do_(new StBlockClosure() {
									public Object value_value_value_(Object xObject, Object yObject, Object zObject) {
										int x = (int) ((Double) xObject).doubleValue();
										int y = (int) ((Double) yObject).doubleValue();
										int z = (int) ((Double) zObject).doubleValue();

										return evaluationBlock.value_value_value_(new Double(xInterval.at_(x)), new Double(yInterval.at_(y)), new Double(zInterval.at_(z)));
									}
								}, aNumber, aNumber2, new Jun3dPoint(xIndex, yIndex, zIndex), new StBlockClosure() {
									public Object value_(Object valueObject) {
										JunBorderVertex[] triangle = (JunBorderVertex[]) valueObject;
										Vector points = new Vector();

										for (int i = 0; i < triangle.length; i++) {
											Jun3dPoint vertex1 = triangle[i].vertex1();
											double x1 = xInterval.at_((int) Math.round(vertex1.x()));
											double y1 = yInterval.at_((int) Math.round(vertex1.y()));
											double z1 = zInterval.at_((int) Math.round(vertex1.z()));
											Jun3dPoint vertex2 = triangle[i].vertex2();
											double x2 = xInterval.at_((int) Math.round(vertex2.x()));
											double y2 = yInterval.at_((int) Math.round(vertex2.y()));
											double z2 = zInterval.at_((int) Math.round(vertex2.z()));
											double value1 = ((Double) evaluationBlock.value_value_value_(new Double(x1), new Double(y1), new Double(z1))).doubleValue();
											double value2 = ((Double) evaluationBlock.value_value_value_(new Double(x2), new Double(y2), new Double(z2))).doubleValue();
											double t = (value1 == value2) ? 0.0 : (triangle[i].label() - value1) / (value2 - value1);
											t = Math.min(Math.max(t, 0), 1);

											Jun3dPoint p1 = new Jun3dPoint(x1, y1, z1);
											Jun3dPoint p2 = new Jun3dPoint(x2, y2, z2);
											p2 = p2.minus_(p1).multipliedBy_(t).plus_(p1);
											points.add(p2);
										}

										Jun3dPoint[] pointsArray = new Jun3dPoint[points.size()];

										for (int i = 0; i < points.size(); i++) {
											pointsArray[i] = (Jun3dPoint) points.elementAt(i);
										}

										JunPolygon polygon = JunPolygon.Points_(pointsArray);

										if (polygon != null) {
											polygons.add(polygon);
										}

										return polygons;
									}
								});

								return null;
							}
						});

						return null;
					}
				});
				interimBlock.value_(polygons);

				return null;
			}
		});

		if (kIsDebug) {
			timeToRun = System.currentTimeMillis() - timeToRun;
			System.out.println(timeToRun);
		}

		JunPolygon[] polygonsArray = new JunPolygon[polygons.size()];

		for (int i = 0; i < polygons.size(); i++) {
			polygonsArray[i] = (JunPolygon) polygons.elementAt(i);
		}

		return polygonsArray;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param evaluationBlock DOCUMENT ME!
	 * @param aNumber DOCUMENT ME!
	 * @param xInterval DOCUMENT ME!
	 * @param yInterval DOCUMENT ME!
	 * @param zInterval DOCUMENT ME!
	 * @param interimBlock DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static synchronized JunPolygon[] ContourPolygonsFrom_at_xInterval_yInterval_zInterval_interim_(
		final StBlockClosure evaluationBlock,
		final double aNumber,
		final StInterval xInterval,
		final StInterval yInterval,
		final StInterval zInterval,
		final StBlockClosure interimBlock) {
		return ContourPolygonsFrom_at_sign_xInterval_yInterval_zInterval_interim_(evaluationBlock, aNumber, 1, xInterval, yInterval, zInterval, interimBlock);
	}

	/**
	 * Create contour polylines.
	 * 
	 * @param evaluationBlock sra.smalltalk.StBlockClosure
	 * @param aNumber double
	 * @param xInterval sra.smalltalk.StInterval
	 * @param yInterval sra.smalltalk.StInterval
	 * @param interimBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return sra.jun.geometry.basic.Jun2dPoint[][]
	 */
	public static Jun2dPoint[][] ContourPolylinesFrom_at_xInterval_yInterval_interim_(final StBlockClosure evaluationBlock, final double aNumber, final StInterval xInterval, final StInterval yInterval, final StBlockClosure interimBlock) {
		final JunDoubleLinkedList danglingPolyedges = new JunDoubleLinkedList();
		final int maxX = xInterval.size();
		final int maxY = yInterval.size();
		final Rectangle border = new Rectangle(1, 1, maxX - 1, maxY - 1);
		final StBlockClosure onBorderBlock = new StBlockClosure() {
			public Object value_(Object anObject) {
				Vector polyedges = (Vector) anObject;
				JunBorderVertex firstVertex = (JunBorderVertex) polyedges.firstElement();
				JunBorderVertex lastVertex = (JunBorderVertex) polyedges.lastElement();

				return new Boolean(firstVertex.onBorderOfRectangle_(border) && lastVertex.onBorderOfRectangle_(border));
			}
		};

		final Vector polylines = new Vector();
		final StBlockClosure completedBlock = new StBlockClosure() {
			public Object value_(Object anObject) {
				Vector aVector = (Vector) anObject;
				JunBorderVertex[] polyedges = new JunBorderVertex[aVector.size()];
				aVector.copyInto(polyedges);

				Jun2dPoint[] polyline = new Jun2dPoint[polyedges.length];

				for (int i = 0; i < polyedges.length; i++) {
					JunBorderVertex vertex = polyedges[i];
					double x0 = xInterval.at_((int) vertex.vertex1().x());
					double x1 = xInterval.at_((int) vertex.vertex2().x());
					double y0 = yInterval.at_((int) vertex.vertex1().y());
					double y1 = yInterval.at_((int) vertex.vertex2().y());
					double value0 = ((Number) evaluationBlock.value_value_(new Double(x0), new Double(y0))).doubleValue();
					double value1 = ((Number) evaluationBlock.value_value_(new Double(x1), new Double(y1))).doubleValue();
					double interpolation = 0;

					if (value0 != value1) {
						interpolation = (vertex.label() - value0) / (value1 - value0);
					}

					Jun2dPoint aPoint = new Jun2dPoint(x1, y1);
					aPoint = aPoint.minus_(new Jun2dPoint(x0, y0));
					aPoint = aPoint.multipliedBy_(interpolation);
					aPoint = aPoint.plus_(new Jun2dPoint(x0, y0));
					polyline[i] = aPoint;
				}

				polylines.addElement(polyline);

				if (interimBlock != null) {
					interimBlock.value_(polylines);
				}

				return null;
			}
		};

		StBlockClosure edgeInsertionBlock = new StBlockClosure() {
			public Object value_value_(Object objFrom, Object objTo) {
				final JunBorderVertex from = (JunBorderVertex) objFrom;
				final JunBorderVertex to = (JunBorderVertex) objTo;
				final StValueHolder leadingPolyedgeHolder = new StValueHolder();
				final StValueHolder trailingPolyedgeHolder = new StValueHolder();
				danglingPolyedges.do_(new StBlockClosure() {
					public Object value_(Object anObject) {
						Vector polyedge = (Vector) anObject;

						if (from.equals(polyedge.lastElement())) {
							if (leadingPolyedgeHolder.value() == null) {
								leadingPolyedgeHolder.value_(polyedge);
							}
						}

						if (to.equals(polyedge.firstElement())) {
							if (trailingPolyedgeHolder.value() == null) {
								trailingPolyedgeHolder.value_(polyedge);
							}
						}

						return null;
					}
				});

				Vector leadingPolyedge = (Vector) leadingPolyedgeHolder.value();
				Vector trailingPolyedge = (Vector) trailingPolyedgeHolder.value();

				if (leadingPolyedge == null) {
					if (trailingPolyedge == null) {
						Vector newPolyedge = new Vector();
						newPolyedge.addElement(from);
						newPolyedge.addElement(to);

						if (((Boolean) onBorderBlock.value_(newPolyedge)).booleanValue()) {
							completedBlock.value_(newPolyedge);
						} else {
							danglingPolyedges.add_(newPolyedge);
						}
					} else {
						trailingPolyedge.insertElementAt(from, 0);

						if (((Boolean) onBorderBlock.value_(trailingPolyedge)).booleanValue()) {
							danglingPolyedges.remove_ifAbsent_(trailingPolyedge, new StBlockClosure());
							completedBlock.value_(trailingPolyedge);
						}
					}
				} else {
					if (trailingPolyedge == null) {
						leadingPolyedge.addElement(to);

						if (((Boolean) onBorderBlock.value_(leadingPolyedge)).booleanValue()) {
							danglingPolyedges.remove_ifAbsent_(leadingPolyedge, new StBlockClosure());
							completedBlock.value_(leadingPolyedge);
						}
					} else {
						if (leadingPolyedge == trailingPolyedge) {
							leadingPolyedge.addElement(to);
							danglingPolyedges.remove_ifAbsent_(leadingPolyedge, new StBlockClosure());
							completedBlock.value_(leadingPolyedge);
						} else {
							leadingPolyedge.addAll(trailingPolyedge);
							danglingPolyedges.remove_ifAbsent_(trailingPolyedge, new StBlockClosure());

							if (((Boolean) onBorderBlock.value_(leadingPolyedge)).booleanValue()) {
								danglingPolyedges.remove_ifAbsent_(leadingPolyedge, new StBlockClosure());
								completedBlock.value_(leadingPolyedge);
							}
						}
					}
				}

				return null;
			}
		};

		for (int xIndex = 1; xIndex < xInterval.size(); xIndex++) {
			for (int yIndex = 1; yIndex < yInterval.size(); yIndex++) {
				_edgesWith_at_sign_inUnitSquareAt_do_(new StBlockClosure() {
					public Object value_value_(Object objX, Object objY) {
						int x = ((Number) objX).intValue();
						int y = ((Number) objY).intValue();

						return evaluationBlock.value_value_(new Double(xInterval.at_(x)), new Double(yInterval.at_(y)));
					}
				}, aNumber, 1, new Point(xIndex, yIndex), edgeInsertionBlock);
			}
		}

		Jun2dPoint[][] anArrayOfPolylines = new Jun2dPoint[polylines.size()][];
		polylines.copyInto(anArrayOfPolylines);

		return anArrayOfPolylines;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static StBlockClosure EvaluateBlockForBrightness() {
		StBlockClosure evaluateBlock = new StBlockClosure() {
			public Object value_value_value_(Object anStImage, Object aJun3dPoint, Object aDouble) {
				StImage image = (StImage) anStImage;
				Jun3dPoint aPoint = (Jun3dPoint) aJun3dPoint;
				double scale = ((Double) aDouble).doubleValue();
				double result = ((StColorValue._GetBrightness(image.valueAtPoint_(new Point((int) Math.round(aPoint.x() - 1), (int) Math.round(image.height() - aPoint.y())))) * scale) - aPoint.z());

				return new Double(result);
			}
		};

		return evaluateBlock;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static StBlockClosure EvaluateBlockForHue() {
		StBlockClosure evaluateBlock = new StBlockClosure() {
			public Object value_value_value_(Object anStImage, Object aJun3dPoint, Object aDouble) {
				StImage image = (StImage) anStImage;
				Jun3dPoint aPoint = (Jun3dPoint) aJun3dPoint;
				double scale = ((Double) aDouble).doubleValue();
				double result = ((StColorValue._GetHue(image.valueAtPoint_(new Point((int) Math.round(aPoint.x() - 1), (int) Math.round(image.height() - aPoint.y())))) * scale) - aPoint.z());

				return new Double(result);
			}
		};

		return evaluateBlock;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static StBlockClosure EvaluateBlockForSaturation() {
		StBlockClosure evaluateBlock = new StBlockClosure() {
			public Object value_value_value_(Object anStImage, Object aJun3dPoint, Object aDouble) {
				StImage image = (StImage) anStImage;
				Jun3dPoint aPoint = (Jun3dPoint) aJun3dPoint;
				double scale = ((Double) aDouble).doubleValue();
				double result = ((StColorValue._GetSaturation(image.valueAtPoint_(new Point((int) Math.round(aPoint.x() - 1), (int) Math.round(image.height() - aPoint.y())))) * scale) - aPoint.z());

				return new Double(result);
			}
		};

		return evaluateBlock;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param image sra.smalltalk.StImage
	 *
	 * @return sra.jun.geometry.surfaces.JunPolygon[]
	 */
	public static JunPolygon[] LandscapeFromImage_(StImage image) {
		return LandscapeFromImage_on_(image, new JunOpenGLDisplayModel());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param image sra.smalltalk.StImage
	 * @param displayModel sra.jun.opengl.display.JunOpenGLDisplayModel
	 * 
	 * @return sra.jun.geometry.surfaces.JunPolygon[]
	 */
	public static JunPolygon[] LandscapeFromImage_on_(StImage image, JunOpenGLDisplayModel displayModel) {
		return LandscapeFromImage_with_on_(image, EvaluateBlockForBrightness(), displayModel);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param image sra.smalltalk.StImage
	 * @param evaluatingBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return sra.jun.geometry.surfaces.JunPolygon[]
	 */
	public static JunPolygon[] LandscapeFromImage_with_(StImage image, StBlockClosure evaluatingBlock) {
		return LandscapeFromImage_with_on_(image, evaluatingBlock, new JunOpenGLDisplayModel());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param image sra.smalltalk.StImage
	 * @param evaluatingBlock sra.smalltalk.StBlockClosure
	 * @param displayModel sra.jun.opengl.display.JunOpenGLDisplayModel
	 * 
	 * @return sra.jun.geometry.surfaces.JunPolygon[]
	 */
	public static JunPolygon[] LandscapeFromImage_with_on_(StImage anImage, StBlockClosure evaluatingBlock, JunOpenGLDisplayModel displayModel) {
		JunCursors cursor = new JunCursors(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		try {
			cursor._show();

			int width = anImage.width();
			int height = anImage.height();
			double scale = (Math.min(width, height) / 4);
			JunOpenGL3dCompoundObject dObject = new JunOpenGL3dCompoundObject();
			dObject.paint_(Color.blue);
			displayModel.displayObject_(dObject);
			displayModel.sightPoint_(new Jun3dPoint(width / 2, height / 2, scale / 2));
			displayModel.eyePoint_(new Jun3dPoint(width / 2, height * -1, height * 2));
			displayModel.upVector_(new Jun3dPoint(0, 1, 0));

			StValueHolder lastNumPolygons = StValueHolder.With_(new Integer(0));
			int[] xInterval = new int[width];

			for (int x = 0; x < width; x++) {
				xInterval[x] = x + 1;
			}

			int[] yInterval = new int[height];

			for (int y = 0; y < height; y++) {
				yInterval[y] = y + 1;
			}

			double[] zInterval = new double[7];
			int zCount = 0;
			double interval = scale / 4;

			for (double z = interval * -1; z <= (interval * 5); z = z + interval) {
				zInterval[zCount] = z;
				zCount = zCount + 1;
			}

			return JunBorderGenerator._contourPolygonsFrom_at_sign_xInterval_yInterval_zInterval_interim_(evaluatingBlock, anImage, height, scale, 0, 1, xInterval, yInterval, zInterval, lastNumPolygons, dObject, displayModel);
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Create a Jun2dPolygonalBoundary with the specified information.
	 * 
	 * @param evaluationBlock sra.smalltalk.StBlockClosure
	 * @param maxNumber double
	 * @param xInterval sra.smalltalk.StInterval
	 * @param yInterval sra.smalltalk.StInterval
	 * @param interimBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return sra.jun.geometry.boundaries.Jun2dPolygonalBoundary
	 */
	public static Jun2dPolygonalBoundary PolygonalBoundaryWith_max_xInterval_yInterval_interim_(final StBlockClosure evaluationBlock, double maxNumber, final StInterval xInterval, final StInterval yInterval, final StBlockClosure interimBlock) {
		final JunDoubleLinkedList danglingPolyedges = new JunDoubleLinkedList();
		final int maxX = xInterval.size();
		final int maxY = yInterval.size();
		final Jun2dPolygonalBoundary polygonalBoundary = new Jun2dPolygonalBoundary();

		final StBlockClosure interpolationBlock = new StBlockClosure() {
			public Object value_(Object anObject) {
				JunBorderVertex vertex = (JunBorderVertex) anObject;
				double x1 = xInterval.at_((int) vertex.vertex1().x());
				double y1 = yInterval.at_((int) vertex.vertex1().y());
				double x2 = xInterval.at_((int) vertex.vertex2().x());
				double y2 = yInterval.at_((int) vertex.vertex2().y());
				double value1 = ((Number) evaluationBlock.value_value_(new Double(x1), new Double(y1))).doubleValue();
				double value2 = ((Number) evaluationBlock.value_value_(new Double(x2), new Double(y2))).doubleValue();
				double t = 0;

				if (value1 != value2) {
					t = (vertex.label() - value1) / (value2 - value1);
				}

				t = Math.max(0, Math.min(t, 1));

				return new Double(t);
			}
		};

		final StBlockClosure completedBlock = new StBlockClosure() {
			public Object value_(Object anObject) {
				Vector aVector = (Vector) anObject;
				JunBorderVertex[] polyedges = new JunBorderVertex[aVector.size()];
				aVector.copyInto(polyedges);

				Jun2dPoint[] points = new Jun2dPoint[polyedges.length];

				for (int i = 0; i < polyedges.length; i++) {
					JunBorderVertex vertex = polyedges[i];
					double t = ((Number) interpolationBlock.value_(vertex)).doubleValue();
					Jun3dPoint v1 = vertex.vertex1();
					Jun3dPoint v2 = vertex.vertex2();
					Jun2dPoint p1 = new Jun2dPoint(xInterval.at_((int) v1.x()), yInterval.at_((int) v1.y()));
					Jun2dPoint p2 = new Jun2dPoint(xInterval.at_((int) v2.x()), yInterval.at_((int) v2.y()));
					Jun2dPoint aPoint = p2.minus_(p1);
					aPoint = aPoint.multipliedBy_(t);
					aPoint = aPoint.plus_(p1);
					points[i] = aPoint;
				}

				Jun2dPolygon polygon = new Jun2dPolygon(points);
				polygonalBoundary.add_(polygon);

				if (interimBlock != null) {
					interimBlock.value_(polygonalBoundary);
				}

				return null;
			}
		};

		StBlockClosure edgeInsertionBlock = new StBlockClosure() {
			public Object value_value_(Object objFrom, Object objTo) {
				final JunBorderVertex from = (JunBorderVertex) objFrom;
				final JunBorderVertex to = (JunBorderVertex) objTo;
				Vector leadingPolyedge = (Vector) danglingPolyedges.detect_ifNone_(new StBlockClosure() {
					public Object value_(Object anObject) {
						Vector polyedge = (Vector) anObject;

						return new Boolean(from.equals(polyedge.lastElement()));
					}
				}, new StBlockClosure());
				Vector trailingPolyedge = (Vector) danglingPolyedges.detect_ifNone_(new StBlockClosure() {
					public Object value_(Object anObject) {
						Vector polyedge = (Vector) anObject;

						return new Boolean(to.equals(polyedge.firstElement()));
					}
				}, new StBlockClosure());

				if (leadingPolyedge == null) {
					if (trailingPolyedge == null) {
						Vector newPolyedge = new Vector();
						newPolyedge.addElement(from);
						newPolyedge.addElement(to);
						danglingPolyedges.add_(newPolyedge);
					} else {
						trailingPolyedge.insertElementAt(from, 0);
					}
				} else {
					if (trailingPolyedge == null) {
						leadingPolyedge.addElement(to);
					} else {
						if (leadingPolyedge == trailingPolyedge) {
							danglingPolyedges.remove_ifAbsent_(leadingPolyedge, new StBlockClosure());
							completedBlock.value_(leadingPolyedge);
						} else {
							leadingPolyedge.addAll(trailingPolyedge);
							danglingPolyedges.remove_ifAbsent_(trailingPolyedge, new StBlockClosure());
						}
					}
				}

				return null;
			}
		};

		for (int xIndex = 1; xIndex < maxX; xIndex++) {
			for (int yIndex = 1; yIndex < maxY; yIndex++) {
				_edgesWith_at_sign_inUnitSquareAt_do_(new StBlockClosure() {
					public Object value_value_(Object objX, Object objY) {
						int x = ((Number) objX).intValue();
						int y = ((Number) objY).intValue();

						return evaluationBlock.value_value_(new Double(xInterval.at_(x)), new Double(yInterval.at_(y)));
					}
				}, maxNumber, -1, new Point(xIndex, yIndex), edgeInsertionBlock);
			}
		}

		final Vector vertexesOnBorderX0 = new Vector();
		final Vector vertexesOnBorderX1 = new Vector();
		final Vector vertexesOnBorderY0 = new Vector();
		final Vector vertexesOnBorderY1 = new Vector();
		danglingPolyedges.do_(new StBlockClosure() {
			public Object value_(Object anObject) {
				Vector polyedge = (Vector) anObject;
				JunBorderVertex firstVertex = (JunBorderVertex) polyedge.firstElement();
				JunBorderVertex lastVertex = (JunBorderVertex) polyedge.lastElement();

				if (firstVertex.onX_(1)) {
					double value = firstVertex.vertex1().y();
					value += ((Number) interpolationBlock.value_(firstVertex)).doubleValue();
					vertexesOnBorderX0.add(new Object[] { new Double(value), firstVertex });
				}

				if (firstVertex.onX_(maxX)) {
					double value = firstVertex.vertex1().y();
					value += ((Number) interpolationBlock.value_(firstVertex)).doubleValue();
					vertexesOnBorderX1.add(new Object[] { new Double(value), firstVertex });
				}

				if (firstVertex.onY_(1)) {
					double value = firstVertex.vertex1().x();
					value += ((Number) interpolationBlock.value_(firstVertex)).doubleValue();
					vertexesOnBorderY0.add(new Object[] { new Double(value), firstVertex });
				}

				if (firstVertex.onY_(maxY)) {
					double value = firstVertex.vertex1().x();
					value += ((Number) interpolationBlock.value_(firstVertex)).doubleValue();
					vertexesOnBorderY1.add(new Object[] { new Double(value), firstVertex });
				}

				if (lastVertex.onX_(1)) {
					double value = lastVertex.vertex1().y();
					value += ((Number) interpolationBlock.value_(lastVertex)).doubleValue();
					vertexesOnBorderX0.add(new Object[] { new Double(value), lastVertex });
				}

				if (lastVertex.onX_(maxX)) {
					double value = lastVertex.vertex1().y();
					value += ((Number) interpolationBlock.value_(lastVertex)).doubleValue();
					vertexesOnBorderX1.add(new Object[] { new Double(value), lastVertex });
				}

				if (lastVertex.onY_(1)) {
					double value = lastVertex.vertex1().x();
					value += ((Number) interpolationBlock.value_(lastVertex)).doubleValue();
					vertexesOnBorderY0.add(new Object[] { new Double(value), lastVertex });
				}

				if (lastVertex.onY_(maxY)) {
					double value = lastVertex.vertex1().x();
					value += ((Number) interpolationBlock.value_(lastVertex)).doubleValue();
					vertexesOnBorderY1.add(new Object[] { new Double(value), lastVertex });
				}

				return null;
			}
		});

		double cornerValue = ((Number) evaluationBlock.value_value_(new Double(xInterval.first()), new Double(yInterval.first()))).doubleValue();

		if (cornerValue <= maxNumber) {
			JunBorderVertex vertex = new JunBorderVertex(new Jun3dPoint(1, 1, 0), new Jun3dPoint(1, 1, 0), 0);
			vertexesOnBorderX0.add(new Object[] { new Double(1), vertex });
			vertexesOnBorderY0.add(new Object[] { new Double(1), vertex });
		}

		cornerValue = ((Number) evaluationBlock.value_value_(new Double(xInterval.last()), new Double(yInterval.first()))).doubleValue();

		if (cornerValue <= maxNumber) {
			JunBorderVertex vertex = new JunBorderVertex(new Jun3dPoint(maxX, 1, 0), new Jun3dPoint(maxX, 1, 0), 0);
			vertexesOnBorderX1.add(new Object[] { new Double(1), vertex });
			vertexesOnBorderY0.add(new Object[] { new Double(maxX), vertex });
		}

		cornerValue = ((Number) evaluationBlock.value_value_(new Double(xInterval.last()), new Double(yInterval.last()))).doubleValue();

		if (cornerValue <= maxNumber) {
			JunBorderVertex vertex = new JunBorderVertex(new Jun3dPoint(maxX, maxY, 0), new Jun3dPoint(maxX, maxY, 0), 0);
			vertexesOnBorderX1.add(new Object[] { new Double(maxY), vertex });
			vertexesOnBorderY1.add(new Object[] { new Double(maxX), vertex });
		}

		cornerValue = ((Number) evaluationBlock.value_value_(new Double(xInterval.first()), new Double(yInterval.last()))).doubleValue();

		if (cornerValue <= maxNumber) {
			JunBorderVertex vertex = new JunBorderVertex(new Jun3dPoint(1, maxY, 0), new Jun3dPoint(1, maxY, 0), 0);
			vertexesOnBorderX0.add(new Object[] { new Double(maxY), vertex });
			vertexesOnBorderY1.add(new Object[] { new Double(1), vertex });
		}

		Vector vertexesOnBorder = new Vector();
		vertexesOnBorder.addAll((new StSortedCollection(new StBlockClosure() {
			public Object value_value_(Object lessObj, Object moreObj) {
				Object[] less = (Object[]) lessObj;
				Object[] more = (Object[]) moreObj;
				double lessKey = ((Number) less[0]).doubleValue();
				double moreKey = ((Number) more[0]).doubleValue();
				double lessX = ((JunBorderVertex) less[1]).vertex1().x();
				double moreX = ((JunBorderVertex) more[1]).vertex1().x();

				return new Boolean((lessX < moreX) || (lessX == moreX && lessKey <= moreKey));
			}
		}, vertexesOnBorderY0))._asCollection());
		vertexesOnBorder.addAll((new StSortedCollection(new StBlockClosure() {
			public Object value_value_(Object lessObj, Object moreObj) {
				Object[] less = (Object[]) lessObj;
				Object[] more = (Object[]) moreObj;
				double lessKey = ((Number) less[0]).doubleValue();
				double moreKey = ((Number) more[0]).doubleValue();
				double lessY = ((JunBorderVertex) less[1]).vertex1().y();
				double moreY = ((JunBorderVertex) more[1]).vertex1().y();

				return new Boolean((lessY < moreY) || (lessY == moreY && lessKey <= moreKey));
			}
		}, vertexesOnBorderX1))._asCollection());
		vertexesOnBorder.addAll((new StSortedCollection(new StBlockClosure() {
			public Object value_value_(Object lessObj, Object moreObj) {
				Object[] less = (Object[]) lessObj;
				Object[] more = (Object[]) moreObj;
				double lessKey = ((Number) less[0]).doubleValue();
				double moreKey = ((Number) more[0]).doubleValue();
				double lessX = ((JunBorderVertex) less[1]).vertex1().x();
				double moreX = ((JunBorderVertex) more[1]).vertex1().x();

				return new Boolean((lessX < moreX) || (lessX == moreX && lessKey <= moreKey));
			}
		}, vertexesOnBorderY1))._asCollection());
		vertexesOnBorder.addAll((new StSortedCollection(new StBlockClosure() {
			public Object value_value_(Object lessObj, Object moreObj) {
				Object[] less = (Object[]) lessObj;
				Object[] more = (Object[]) moreObj;
				double lessKey = ((Number) less[0]).doubleValue();
				double moreKey = ((Number) more[0]).doubleValue();
				double lessY = ((JunBorderVertex) less[1]).vertex1().y();
				double moreY = ((JunBorderVertex) more[1]).vertex1().y();

				return new Boolean((lessY < moreY) || (lessY == moreY && lessKey <= moreKey));
			}
		}, vertexesOnBorderX0))._asCollection());

		int size = vertexesOnBorder.size();

		for (int i = 0; i < size; i += 2) {
			JunBorderVertex vertex1 = (JunBorderVertex) ((Object[]) vertexesOnBorder.elementAt(i))[1];
			JunBorderVertex vertex2 = (JunBorderVertex) ((Object[]) vertexesOnBorder.elementAt(i + 1))[1];
			edgeInsertionBlock.value_value_(vertex1, vertex2);
		}

		return polygonalBoundary;
	}

	/**
	 * Create a Jun2dPolygonalBoundary with the specified information.
	 * 
	 * @param evaluationBlock sra.smalltalk.StBlockClosure
	 * @param minNumber double
	 * @param maxNumber double
	 * @param xInterval sra.smalltalk.StInterval
	 * @param yInterval sra.smalltalk.StInterval
	 * @param interimBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return sra.jun.geometry.boundaries.Jun2dPolygonalBoundary
	 */
	public static Jun2dPolygonalBoundary PolygonalBoundaryWith_min_max_xInterval_yInterval_interim_(
		final StBlockClosure evaluationBlock,
		double minNumber,
		double maxNumber,
		final StInterval xInterval,
		final StInterval yInterval,
		final StBlockClosure interimBlock) {
		final JunDoubleLinkedList danglingPolyedges = new JunDoubleLinkedList();
		final int maxX = xInterval.size();
		final int maxY = yInterval.size();
		final Jun2dPolygonalBoundary polygonalBoundary = new Jun2dPolygonalBoundary();

		final StBlockClosure interpolationBlock = new StBlockClosure() {
			public Object value_(Object anObject) {
				JunBorderVertex vertex = (JunBorderVertex) anObject;
				double x1 = xInterval.at_((int) vertex.vertex1().x());
				double y1 = yInterval.at_((int) vertex.vertex1().y());
				double x2 = xInterval.at_((int) vertex.vertex2().x());
				double y2 = yInterval.at_((int) vertex.vertex2().y());
				double value1 = ((Number) evaluationBlock.value_value_(new Double(x1), new Double(y1))).doubleValue();
				double value2 = ((Number) evaluationBlock.value_value_(new Double(x2), new Double(y2))).doubleValue();
				double t = 0;

				if (value1 != value2) {
					t = (vertex.label() - value1) / (value2 - value1);
				}

				t = Math.max(0, Math.min(t, 1));

				return new Double(t);
			}
		};

		final StBlockClosure completedBlock = new StBlockClosure() {
			public Object value_(Object anObject) {
				Vector aVector = (Vector) anObject;
				JunBorderVertex[] polyedges = new JunBorderVertex[aVector.size()];
				aVector.copyInto(polyedges);

				Jun2dPoint[] points = new Jun2dPoint[polyedges.length];

				for (int i = 0; i < polyedges.length; i++) {
					JunBorderVertex vertex = polyedges[i];
					double t = ((Number) interpolationBlock.value_(vertex)).doubleValue();
					Jun3dPoint v1 = vertex.vertex1();
					Jun3dPoint v2 = vertex.vertex2();
					Jun2dPoint p1 = new Jun2dPoint(xInterval.at_((int) v1.x()), yInterval.at_((int) v1.y()));
					Jun2dPoint p2 = new Jun2dPoint(xInterval.at_((int) v2.x()), yInterval.at_((int) v2.y()));
					Jun2dPoint aPoint = p2.minus_(p1);
					aPoint = aPoint.multipliedBy_(t);
					aPoint = aPoint.plus_(p1);
					points[i] = aPoint;
				}

				Jun2dPolygon polygon = new Jun2dPolygon(points);
				polygonalBoundary.add_(polygon);

				if (interimBlock != null) {
					interimBlock.value_(polygonalBoundary);
				}

				return null;
			}
		};

		StBlockClosure edgeInsertionBlock = new StBlockClosure() {
			public Object value_value_(Object objFrom, Object objTo) {
				final JunBorderVertex from = (JunBorderVertex) objFrom;
				final JunBorderVertex to = (JunBorderVertex) objTo;
				Vector leadingPolyedge = (Vector) danglingPolyedges.detect_ifNone_(new StBlockClosure() {
					public Object value_(Object anObject) {
						Vector polyedge = (Vector) anObject;

						return new Boolean(from.equals(polyedge.lastElement()));
					}
				}, new StBlockClosure());
				Vector trailingPolyedge = (Vector) danglingPolyedges.detect_ifNone_(new StBlockClosure() {
					public Object value_(Object anObject) {
						Vector polyedge = (Vector) anObject;

						return new Boolean(to.equals(polyedge.firstElement()));
					}
				}, new StBlockClosure());

				if (leadingPolyedge == null) {
					if (trailingPolyedge == null) {
						Vector newPolyedge = new Vector();
						newPolyedge.addElement(from);
						newPolyedge.addElement(to);
						danglingPolyedges.add_(newPolyedge);
					} else {
						trailingPolyedge.insertElementAt(from, 0);
					}
				} else {
					if (trailingPolyedge == null) {
						leadingPolyedge.addElement(to);
					} else {
						if (leadingPolyedge == trailingPolyedge) {
							danglingPolyedges.remove_ifAbsent_(leadingPolyedge, new StBlockClosure());
							completedBlock.value_(leadingPolyedge);
						} else {
							leadingPolyedge.addAll(trailingPolyedge);
							danglingPolyedges.remove_ifAbsent_(trailingPolyedge, new StBlockClosure());
						}
					}
				}

				return null;
			}
		};

		for (int xIndex = 1; xIndex < maxX; xIndex++) {
			for (int yIndex = 1; yIndex < maxY; yIndex++) {
				_edgesWith_at_sign_inUnitSquareAt_do_(new StBlockClosure() {
					public Object value_value_(Object objX, Object objY) {
						int x = ((Number) objX).intValue();
						int y = ((Number) objY).intValue();

						return evaluationBlock.value_value_(new Double(xInterval.at_(x)), new Double(yInterval.at_(y)));
					}
				}, minNumber, 1, new Point(xIndex, yIndex), edgeInsertionBlock);
				_edgesWith_at_sign_inUnitSquareAt_do_(new StBlockClosure() {
					public Object value_value_(Object objX, Object objY) {
						int x = ((Number) objX).intValue();
						int y = ((Number) objY).intValue();

						return evaluationBlock.value_value_(new Double(xInterval.at_(x)), new Double(yInterval.at_(y)));
					}
				}, maxNumber, -1, new Point(xIndex, yIndex), edgeInsertionBlock);
			}
		}

		final Vector vertexesOnBorderX0 = new Vector();
		final Vector vertexesOnBorderX1 = new Vector();
		final Vector vertexesOnBorderY0 = new Vector();
		final Vector vertexesOnBorderY1 = new Vector();
		danglingPolyedges.do_(new StBlockClosure() {
			public Object value_(Object anObject) {
				Vector polyedge = (Vector) anObject;
				JunBorderVertex firstVertex = (JunBorderVertex) polyedge.firstElement();
				JunBorderVertex lastVertex = (JunBorderVertex) polyedge.lastElement();

				if (firstVertex.onX_(1)) {
					double value = firstVertex.vertex1().y();
					value += ((Number) interpolationBlock.value_(firstVertex)).doubleValue();
					vertexesOnBorderX0.add(new Object[] { new Double(value), firstVertex });
				}

				if (firstVertex.onX_(maxX)) {
					double value = firstVertex.vertex1().y();
					value += ((Number) interpolationBlock.value_(firstVertex)).doubleValue();
					vertexesOnBorderX1.add(new Object[] { new Double(value), firstVertex });
				}

				if (firstVertex.onY_(1)) {
					double value = firstVertex.vertex1().x();
					value += ((Number) interpolationBlock.value_(firstVertex)).doubleValue();
					vertexesOnBorderY0.add(new Object[] { new Double(value), firstVertex });
				}

				if (firstVertex.onY_(maxY)) {
					double value = firstVertex.vertex1().x();
					value += ((Number) interpolationBlock.value_(firstVertex)).doubleValue();
					vertexesOnBorderY1.add(new Object[] { new Double(value), firstVertex });
				}

				if (lastVertex.onX_(1)) {
					double value = lastVertex.vertex1().y();
					value += ((Number) interpolationBlock.value_(lastVertex)).doubleValue();
					vertexesOnBorderX0.add(new Object[] { new Double(value), lastVertex });
				}

				if (lastVertex.onX_(maxX)) {
					double value = lastVertex.vertex1().y();
					value += ((Number) interpolationBlock.value_(lastVertex)).doubleValue();
					vertexesOnBorderX1.add(new Object[] { new Double(value), lastVertex });
				}

				if (lastVertex.onY_(1)) {
					double value = lastVertex.vertex1().x();
					value += ((Number) interpolationBlock.value_(lastVertex)).doubleValue();
					vertexesOnBorderY0.add(new Object[] { new Double(value), lastVertex });
				}

				if (lastVertex.onY_(maxY)) {
					double value = lastVertex.vertex1().x();
					value += ((Number) interpolationBlock.value_(lastVertex)).doubleValue();
					vertexesOnBorderY1.add(new Object[] { new Double(value), lastVertex });
				}

				return null;
			}
		});

		double cornerValue = ((Number) evaluationBlock.value_value_(new Double(xInterval.first()), new Double(yInterval.first()))).doubleValue();

		if ((cornerValue >= minNumber) && (cornerValue <= maxNumber)) {
			JunBorderVertex vertex = new JunBorderVertex(new Jun3dPoint(1, 1, 0), new Jun3dPoint(1, 1, 0), 0);
			vertexesOnBorderX0.add(new Object[] { new Double(1), vertex });
			vertexesOnBorderY0.add(new Object[] { new Double(1), vertex });
		}

		cornerValue = ((Number) evaluationBlock.value_value_(new Double(xInterval.last()), new Double(yInterval.first()))).doubleValue();

		if ((cornerValue >= minNumber) && (cornerValue <= maxNumber)) {
			JunBorderVertex vertex = new JunBorderVertex(new Jun3dPoint(maxX, 1, 0), new Jun3dPoint(maxX, 1, 0), 0);
			vertexesOnBorderX1.add(new Object[] { new Double(1), vertex });
			vertexesOnBorderY0.add(new Object[] { new Double(maxX), vertex });
		}

		cornerValue = ((Number) evaluationBlock.value_value_(new Double(xInterval.last()), new Double(yInterval.last()))).doubleValue();

		if ((cornerValue >= minNumber) && (cornerValue <= maxNumber)) {
			JunBorderVertex vertex = new JunBorderVertex(new Jun3dPoint(maxX, maxY, 0), new Jun3dPoint(maxX, maxY, 0), 0);
			vertexesOnBorderX1.add(new Object[] { new Double(maxY), vertex });
			vertexesOnBorderY1.add(new Object[] { new Double(maxX), vertex });
		}

		cornerValue = ((Number) evaluationBlock.value_value_(new Double(xInterval.first()), new Double(yInterval.last()))).doubleValue();

		if ((cornerValue >= minNumber) && (cornerValue <= maxNumber)) {
			JunBorderVertex vertex = new JunBorderVertex(new Jun3dPoint(1, maxY, 0), new Jun3dPoint(1, maxY, 0), 0);
			vertexesOnBorderX0.add(new Object[] { new Double(maxY), vertex });
			vertexesOnBorderY1.add(new Object[] { new Double(1), vertex });
		}

		Vector vertexesOnBorder = new Vector();
		vertexesOnBorder.addAll((new StSortedCollection(new StBlockClosure() {
			public Object value_value_(Object lessObj, Object moreObj) {
				Object[] less = (Object[]) lessObj;
				Object[] more = (Object[]) moreObj;
				double lessKey = ((Number) less[0]).doubleValue();
				double moreKey = ((Number) more[0]).doubleValue();
				double lessX = ((JunBorderVertex) less[1]).vertex1().x();
				double moreX = ((JunBorderVertex) more[1]).vertex1().x();

				return new Boolean((lessX < moreX) || (lessX == moreX && lessKey <= moreKey));
			}
		}, vertexesOnBorderY0))._asCollection());
		vertexesOnBorder.addAll((new StSortedCollection(new StBlockClosure() {
			public Object value_value_(Object lessObj, Object moreObj) {
				Object[] less = (Object[]) lessObj;
				Object[] more = (Object[]) moreObj;
				double lessKey = ((Number) less[0]).doubleValue();
				double moreKey = ((Number) more[0]).doubleValue();
				double lessY = ((JunBorderVertex) less[1]).vertex1().y();
				double moreY = ((JunBorderVertex) more[1]).vertex1().y();

				return new Boolean((lessY < moreY) || (lessY == moreY && lessKey <= moreKey));
			}
		}, vertexesOnBorderX1))._asCollection());
		vertexesOnBorder.addAll((new StSortedCollection(new StBlockClosure() {
			public Object value_value_(Object lessObj, Object moreObj) {
				Object[] less = (Object[]) lessObj;
				Object[] more = (Object[]) moreObj;
				double lessKey = ((Number) less[0]).doubleValue();
				double moreKey = ((Number) more[0]).doubleValue();
				double lessX = ((JunBorderVertex) less[1]).vertex1().x();
				double moreX = ((JunBorderVertex) more[1]).vertex1().x();

				return new Boolean((lessX < moreX) || (lessX == moreX && lessKey <= moreKey));
			}
		}, vertexesOnBorderY1))._asCollection());
		vertexesOnBorder.addAll((new StSortedCollection(new StBlockClosure() {
			public Object value_value_(Object lessObj, Object moreObj) {
				Object[] less = (Object[]) lessObj;
				Object[] more = (Object[]) moreObj;
				double lessKey = ((Number) less[0]).doubleValue();
				double moreKey = ((Number) more[0]).doubleValue();
				double lessY = ((JunBorderVertex) less[1]).vertex1().y();
				double moreY = ((JunBorderVertex) more[1]).vertex1().y();

				return new Boolean((lessY < moreY) || (lessY == moreY && lessKey <= moreKey));
			}
		}, vertexesOnBorderX0))._asCollection());

		int size = vertexesOnBorder.size();

		for (int i = 0; i < size; i += 2) {
			JunBorderVertex vertex1 = (JunBorderVertex) ((Object[]) vertexesOnBorder.elementAt(i))[1];
			JunBorderVertex vertex2 = (JunBorderVertex) ((Object[]) vertexesOnBorder.elementAt(i + 1))[1];
			edgeInsertionBlock.value_value_(vertex1, vertex2);
		}

		return polygonalBoundary;
	}

	/**
	 * Create a Jun2dPolygonalBoundary with the specified information.
	 * 
	 * @param evaluationBlock sra.smalltalk.StBlockClosure
	 * @param minNumber double
	 * @param xInterval sra.smalltalk.StInterval
	 * @param yInterval sra.smalltalk.StInterval
	 * @param interimBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return sra.jun.geometry.boundaries.Jun2dPolygonalBoundary
	 */
	public static Jun2dPolygonalBoundary PolygonalBoundaryWith_min_xInterval_yInterval_interim_(final StBlockClosure evaluationBlock, double minNumber, final StInterval xInterval, final StInterval yInterval, final StBlockClosure interimBlock) {
		final JunDoubleLinkedList danglingPolyedges = new JunDoubleLinkedList();
		final int maxX = xInterval.size();
		final int maxY = yInterval.size();
		final Jun2dPolygonalBoundary polygonalBoundary = new Jun2dPolygonalBoundary();

		final StBlockClosure interpolationBlock = new StBlockClosure() {
			public Object value_(Object anObject) {
				JunBorderVertex vertex = (JunBorderVertex) anObject;
				double x1 = xInterval.at_((int) vertex.vertex1().x());
				double y1 = yInterval.at_((int) vertex.vertex1().y());
				double x2 = xInterval.at_((int) vertex.vertex2().x());
				double y2 = yInterval.at_((int) vertex.vertex2().y());
				double value1 = ((Number) evaluationBlock.value_value_(new Double(x1), new Double(y1))).doubleValue();
				double value2 = ((Number) evaluationBlock.value_value_(new Double(x2), new Double(y2))).doubleValue();
				double t = 0;

				if (value1 != value2) {
					t = (vertex.label() - value1) / (value2 - value1);
				}

				t = Math.max(0, Math.min(t, 1));

				return new Double(t);
			}
		};

		final StBlockClosure completedBlock = new StBlockClosure() {
			public Object value_(Object anObject) {
				Vector aVector = (Vector) anObject;
				JunBorderVertex[] polyedges = new JunBorderVertex[aVector.size()];
				aVector.copyInto(polyedges);

				Jun2dPoint[] points = new Jun2dPoint[polyedges.length];

				for (int i = 0; i < polyedges.length; i++) {
					JunBorderVertex vertex = polyedges[i];
					double t = ((Number) interpolationBlock.value_(vertex)).doubleValue();
					Jun3dPoint v1 = vertex.vertex1();
					Jun3dPoint v2 = vertex.vertex2();
					Jun2dPoint p1 = new Jun2dPoint(xInterval.at_((int) v1.x()), yInterval.at_((int) v1.y()));
					Jun2dPoint p2 = new Jun2dPoint(xInterval.at_((int) v2.x()), yInterval.at_((int) v2.y()));
					Jun2dPoint aPoint = p2.minus_(p1);
					aPoint = aPoint.multipliedBy_(t);
					aPoint = aPoint.plus_(p1);
					points[i] = aPoint;
				}

				Jun2dPolygon polygon = new Jun2dPolygon(points);
				polygonalBoundary.add_(polygon);

				if (interimBlock != null) {
					interimBlock.value_(polygonalBoundary);
				}

				return null;
			}
		};

		StBlockClosure edgeInsertionBlock = new StBlockClosure() {
			public Object value_value_(Object objFrom, Object objTo) {
				final JunBorderVertex from = (JunBorderVertex) objFrom;
				final JunBorderVertex to = (JunBorderVertex) objTo;
				Vector leadingPolyedge = (Vector) danglingPolyedges.detect_ifNone_(new StBlockClosure() {
					public Object value_(Object anObject) {
						Vector polyedge = (Vector) anObject;

						return new Boolean(from.equals(polyedge.lastElement()));
					}
				}, new StBlockClosure());
				Vector trailingPolyedge = (Vector) danglingPolyedges.detect_ifNone_(new StBlockClosure() {
					public Object value_(Object anObject) {
						Vector polyedge = (Vector) anObject;

						return new Boolean(to.equals(polyedge.firstElement()));
					}
				}, new StBlockClosure());

				if (leadingPolyedge == null) {
					if (trailingPolyedge == null) {
						Vector newPolyedge = new Vector();
						newPolyedge.addElement(from);
						newPolyedge.addElement(to);
						danglingPolyedges.add_(newPolyedge);
					} else {
						trailingPolyedge.insertElementAt(from, 0);
					}
				} else {
					if (trailingPolyedge == null) {
						leadingPolyedge.addElement(to);
					} else {
						if (leadingPolyedge == trailingPolyedge) {
							danglingPolyedges.remove_ifAbsent_(leadingPolyedge, new StBlockClosure());
							completedBlock.value_(leadingPolyedge);
						} else {
							leadingPolyedge.addAll(trailingPolyedge);
							danglingPolyedges.remove_ifAbsent_(trailingPolyedge, new StBlockClosure());
						}
					}
				}

				return null;
			}
		};

		for (int xIndex = 1; xIndex < maxX; xIndex++) {
			for (int yIndex = 1; yIndex < maxY; yIndex++) {
				_edgesWith_at_sign_inUnitSquareAt_do_(new StBlockClosure() {
					public Object value_value_(Object objX, Object objY) {
						int x = ((Number) objX).intValue();
						int y = ((Number) objY).intValue();

						return evaluationBlock.value_value_(new Double(xInterval.at_(x)), new Double(yInterval.at_(y)));
					}
				}, minNumber, 1, new Point(xIndex, yIndex), edgeInsertionBlock);
			}
		}

		final Vector vertexesOnBorderX0 = new Vector();
		final Vector vertexesOnBorderX1 = new Vector();
		final Vector vertexesOnBorderY0 = new Vector();
		final Vector vertexesOnBorderY1 = new Vector();
		danglingPolyedges.do_(new StBlockClosure() {
			public Object value_(Object anObject) {
				Vector polyedge = (Vector) anObject;
				JunBorderVertex firstVertex = (JunBorderVertex) polyedge.firstElement();
				JunBorderVertex lastVertex = (JunBorderVertex) polyedge.lastElement();

				if (firstVertex.onX_(1)) {
					double value = firstVertex.vertex1().y();
					value += ((Number) interpolationBlock.value_(firstVertex)).doubleValue();
					vertexesOnBorderX0.add(new Object[] { new Double(value), firstVertex });
				}

				if (firstVertex.onX_(maxX)) {
					double value = firstVertex.vertex1().y();
					value += ((Number) interpolationBlock.value_(firstVertex)).doubleValue();
					vertexesOnBorderX1.add(new Object[] { new Double(value), firstVertex });
				}

				if (firstVertex.onY_(1)) {
					double value = firstVertex.vertex1().x();
					value += ((Number) interpolationBlock.value_(firstVertex)).doubleValue();
					vertexesOnBorderY0.add(new Object[] { new Double(value), firstVertex });
				}

				if (firstVertex.onY_(maxY)) {
					double value = firstVertex.vertex1().x();
					value += ((Number) interpolationBlock.value_(firstVertex)).doubleValue();
					vertexesOnBorderY1.add(new Object[] { new Double(value), firstVertex });
				}

				if (lastVertex.onX_(1)) {
					double value = lastVertex.vertex1().y();
					value += ((Number) interpolationBlock.value_(lastVertex)).doubleValue();
					vertexesOnBorderX0.add(new Object[] { new Double(value), lastVertex });
				}

				if (lastVertex.onX_(maxX)) {
					double value = lastVertex.vertex1().y();
					value += ((Number) interpolationBlock.value_(lastVertex)).doubleValue();
					vertexesOnBorderX1.add(new Object[] { new Double(value), lastVertex });
				}

				if (lastVertex.onY_(1)) {
					double value = lastVertex.vertex1().x();
					value += ((Number) interpolationBlock.value_(lastVertex)).doubleValue();
					vertexesOnBorderY0.add(new Object[] { new Double(value), lastVertex });
				}

				if (lastVertex.onY_(maxY)) {
					double value = lastVertex.vertex1().x();
					value += ((Number) interpolationBlock.value_(lastVertex)).doubleValue();
					vertexesOnBorderY1.add(new Object[] { new Double(value), lastVertex });
				}

				return null;
			}
		});

		double cornerValue = ((Number) evaluationBlock.value_value_(new Double(xInterval.first()), new Double(yInterval.first()))).doubleValue();

		if (cornerValue >= minNumber) {
			JunBorderVertex vertex = new JunBorderVertex(new Jun3dPoint(1, 1, 0), new Jun3dPoint(1, 1, 0), 0);
			vertexesOnBorderX0.add(new Object[] { new Double(1), vertex });
			vertexesOnBorderY0.add(new Object[] { new Double(1), vertex });
		}

		cornerValue = ((Number) evaluationBlock.value_value_(new Double(xInterval.last()), new Double(yInterval.first()))).doubleValue();

		if (cornerValue >= minNumber) {
			JunBorderVertex vertex = new JunBorderVertex(new Jun3dPoint(maxX, 1, 0), new Jun3dPoint(maxX, 1, 0), 0);
			vertexesOnBorderX1.add(new Object[] { new Double(1), vertex });
			vertexesOnBorderY0.add(new Object[] { new Double(maxX), vertex });
		}

		cornerValue = ((Number) evaluationBlock.value_value_(new Double(xInterval.last()), new Double(yInterval.last()))).doubleValue();

		if (cornerValue >= minNumber) {
			JunBorderVertex vertex = new JunBorderVertex(new Jun3dPoint(maxX, maxY, 0), new Jun3dPoint(maxX, maxY, 0), 0);
			vertexesOnBorderX1.add(new Object[] { new Double(maxY), vertex });
			vertexesOnBorderY1.add(new Object[] { new Double(maxX), vertex });
		}

		cornerValue = ((Number) evaluationBlock.value_value_(new Double(xInterval.first()), new Double(yInterval.last()))).doubleValue();

		if (cornerValue >= minNumber) {
			JunBorderVertex vertex = new JunBorderVertex(new Jun3dPoint(1, maxY, 0), new Jun3dPoint(1, maxY, 0), 0);
			vertexesOnBorderX0.add(new Object[] { new Double(maxY), vertex });
			vertexesOnBorderY1.add(new Object[] { new Double(1), vertex });
		}

		Vector vertexesOnBorder = new Vector();
		vertexesOnBorder.addAll((new StSortedCollection(new StBlockClosure() {
			public Object value_value_(Object lessObj, Object moreObj) {
				Object[] less = (Object[]) lessObj;
				Object[] more = (Object[]) moreObj;
				double lessKey = ((Number) less[0]).doubleValue();
				double moreKey = ((Number) more[0]).doubleValue();
				double lessX = ((JunBorderVertex) less[1]).vertex1().x();
				double moreX = ((JunBorderVertex) more[1]).vertex1().x();

				return new Boolean((lessX < moreX) || (lessX == moreX && lessKey <= moreKey));
			}
		}, vertexesOnBorderY0))._asCollection());
		vertexesOnBorder.addAll((new StSortedCollection(new StBlockClosure() {
			public Object value_value_(Object lessObj, Object moreObj) {
				Object[] less = (Object[]) lessObj;
				Object[] more = (Object[]) moreObj;
				double lessKey = ((Number) less[0]).doubleValue();
				double moreKey = ((Number) more[0]).doubleValue();
				double lessY = ((JunBorderVertex) less[1]).vertex1().y();
				double moreY = ((JunBorderVertex) more[1]).vertex1().y();

				return new Boolean((lessY < moreY) || (lessY == moreY && lessKey <= moreKey));
			}
		}, vertexesOnBorderX1))._asCollection());
		vertexesOnBorder.addAll((new StSortedCollection(new StBlockClosure() {
			public Object value_value_(Object moreObj, Object lessObj) {
				Object[] less = (Object[]) lessObj;
				Object[] more = (Object[]) moreObj;
				double lessKey = ((Number) less[0]).doubleValue();
				double moreKey = ((Number) more[0]).doubleValue();
				double lessX = ((JunBorderVertex) less[1]).vertex1().x();
				double moreX = ((JunBorderVertex) more[1]).vertex1().x();

				return new Boolean((lessX < moreX) || (lessX == moreX && lessKey <= moreKey));
			}
		}, vertexesOnBorderY1))._asCollection());
		vertexesOnBorder.addAll((new StSortedCollection(new StBlockClosure() {
			public Object value_value_(Object moreObj, Object lessObj) {
				Object[] less = (Object[]) lessObj;
				Object[] more = (Object[]) moreObj;
				double lessKey = ((Number) less[0]).doubleValue();
				double moreKey = ((Number) more[0]).doubleValue();
				double lessY = ((JunBorderVertex) less[1]).vertex1().y();
				double moreY = ((JunBorderVertex) more[1]).vertex1().y();

				return new Boolean((lessY < moreY) || (lessY == moreY && lessKey <= moreKey));
			}
		}, vertexesOnBorderX0))._asCollection());

		int size = vertexesOnBorder.size();

		for (int i = 0; i < size; i += 2) {
			JunBorderVertex vertex1 = (JunBorderVertex) ((Object[]) vertexesOnBorder.elementAt(i))[1];
			JunBorderVertex vertex2 = (JunBorderVertex) ((Object[]) vertexesOnBorder.elementAt(i + 1))[1];
			edgeInsertionBlock.value_value_(vertex1, vertex2);
		}

		return polygonalBoundary;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param evaluationBlock DOCUMENT ME!
	 * @param aNumber DOCUMENT ME!
	 * @param aNumber2 DOCUMENT ME!
	 * @param aJun3dPoint DOCUMENT ME!
	 * @param aBlock DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private static final int __trianglesWith_at_sign_inUnitCubeAt_do_(StBlockClosure evaluationBlock, double aNumber, int aNumber2, Jun3dPoint aJun3dPoint, StBlockClosure aBlock) {
		final int x0 = (int) Math.round(aJun3dPoint.x());
		final int x1 = x0 + 1;
		final int y0 = (int) Math.round(aJun3dPoint.y());
		final int y1 = y0 + 1;
		final int z0 = (int) Math.round(aJun3dPoint.z());
		final int z1 = z0 + 1;
		final int sign = ((aNumber2 == 0) ? 0 : ((aNumber2 > 0) ? 1 : -1));
		final Double x0d = new Double(x0);
		final Double y0d = new Double(y0);
		final Double z0d = new Double(z0);
		final Double x1d = new Double(x1);
		final Double y1d = new Double(y1);
		final Double z1d = new Double(z1);
		final double x0dy0dz0d = ((Double) (evaluationBlock.value_value_value_(x0d, y0d, z0d))).doubleValue();
		final double x1dy0dz0d = ((Double) (evaluationBlock.value_value_value_(x1d, y0d, z0d))).doubleValue();
		final double x1dy1dz0d = ((Double) (evaluationBlock.value_value_value_(x1d, y1d, z0d))).doubleValue();
		final double x0dy1dz0d = ((Double) (evaluationBlock.value_value_value_(x0d, y1d, z0d))).doubleValue();
		final double x0dy0dz1d = ((Double) (evaluationBlock.value_value_value_(x0d, y0d, z1d))).doubleValue();
		final double x1dy0dz1d = ((Double) (evaluationBlock.value_value_value_(x1d, y0d, z1d))).doubleValue();
		final double x1dy1dz1d = ((Double) (evaluationBlock.value_value_value_(x1d, y1d, z1d))).doubleValue();
		final double x0dy1dz1d = ((Double) (evaluationBlock.value_value_value_(x0d, y1d, z1d))).doubleValue();
		int index = 0;

		if (((x0dy0dz0d - aNumber) * sign) > 0) {
			index += 1;
		}

		if (((x1dy0dz0d - aNumber) * sign) > 0) {
			index += 2;
		}

		if (((x1dy1dz0d - aNumber) * sign) > 0) {
			index += 4;
		}

		if (((x0dy1dz0d - aNumber) * sign) > 0) {
			index += 8;
		}

		if (((x0dy0dz1d - aNumber) * sign) > 0) {
			index += 16;
		}

		if (((x1dy0dz1d - aNumber) * sign) > 0) {
			index += 32;
		}

		if (((x1dy1dz1d - aNumber) * sign) > 0) {
			index += 64;
		}

		if (((x0dy1dz1d - aNumber) * sign) > 0) {
			index += 128;
		}

		if ((index < 1) || (index > 254)) {
			return index;
		}

		JunBorderVertex[][] array = (JunBorderVertex[][]) MarchingCubes[index - 1].value_value_(aJun3dPoint, new Double(aNumber));

		for (int i = 0; i < array.length; i++) {
			aBlock.value_(array[i]);
		}

		return index;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param evaluatingBlock DOCUMENT ME!
	 * @param triangle DOCUMENT ME!
	 * @param image DOCUMENT ME!
	 * @param height DOCUMENT ME!
	 * @param scale DOCUMENT ME!
	 * @param xInterval DOCUMENT ME!
	 * @param yInterval DOCUMENT ME!
	 * @param zInterval DOCUMENT ME!
	 * @param polygonStream DOCUMENT ME!
	 */
	private static void _1evaluateMainBlock_(StBlockClosure evaluatingBlock, JunBorderVertex[] triangle, StImage image, int height, double scale, int[] xInterval, int[] yInterval, double[] zInterval, Vector polygonStream) {
		Jun3dPoint[] points = new Jun3dPoint[triangle.length];

		for (int i = 0; i < triangle.length; i++) {
			JunBorderVertex vertex = (JunBorderVertex) triangle[i];
			int x1 = xInterval[(int) Math.round(vertex.vertex1().x()) - 1];
			int y1 = yInterval[(int) Math.round(vertex.vertex1().y()) - 1];
			double z1 = zInterval[(int) Math.round(vertex.vertex2().z()) - 1];
			int x2 = xInterval[(int) Math.round(vertex.vertex2().x()) - 1];
			int y2 = yInterval[(int) Math.round(vertex.vertex2().y()) - 1];
			double z2 = zInterval[(int) Math.round(vertex.vertex2().z()) - 1];
			double value1 = _evaluateEvaluationBlock_(evaluatingBlock, image, height, scale, x1, y1, z1);
			double value2 = _evaluateEvaluationBlock_(evaluatingBlock, image, height, scale, x2, y2, z2);
			double t;

			if (value1 == value2) {
				t = 0;
			} else {
				t = (vertex.label() - value1) / (value2 - value1);
			}

			t = Math.min(Math.max(t, 0), 1);

			Jun3dPoint p1 = new Jun3dPoint(x1, y1, z1);
			Jun3dPoint p2 = new Jun3dPoint(x2, y2, z2);
			points[i] = (Jun3dPoint) ((p2.minus_(p1)).multipliedBy_(t)).plus_(p1);
		}

		JunPolygon polygon = JunPolygon.Vertexes_(points);

		if (polygon != null) {
			polygonStream.addElement(polygon);
		}

		return;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param evaluatingBlock DOCUMENT ME!
	 * @param triangle DOCUMENT ME!
	 * @param image DOCUMENT ME!
	 * @param height DOCUMENT ME!
	 * @param scale DOCUMENT ME!
	 * @param xInterval DOCUMENT ME!
	 * @param yInterval DOCUMENT ME!
	 * @param zInterval DOCUMENT ME!
	 * @param polygonStream DOCUMENT ME!
	 */
	private static void _2evaluateMainBlock_(StBlockClosure evaluatingBlock, JunBorderVertex[] triangle, StImage image, int height, double scale, int[] xInterval, int[] yInterval, double[] zInterval, Vector polygonStream) {
		Jun3dPoint[] points = new Jun3dPoint[triangle.length];

		for (int i = 0; i < triangle.length; i++) {
			JunBorderVertex vertex = (JunBorderVertex) triangle[i];
			int x1 = xInterval[(int) Math.round(vertex.vertex1().x()) - 1];
			int y1 = yInterval[(int) Math.round(vertex.vertex1().y()) - 1];
			double z1 = (double) xInterval[(int) Math.round(vertex.vertex2().z()) - 1];
			int x2 = xInterval[(int) Math.round(vertex.vertex2().x()) - 1];
			int y2 = yInterval[(int) Math.round(vertex.vertex2().y()) - 1];
			double z2 = (double) xInterval[(int) Math.round(vertex.vertex2().z()) - 1];
			double value1 = _evaluateEvaluationBlock_(evaluatingBlock, image, height, scale, x1, y1, z1);
			double value2 = _evaluateEvaluationBlock_(evaluatingBlock, image, height, scale, x2, y2, z2);
			double t;

			if (value1 == value2) {
				t = 0;
			} else {
				t = (vertex.label() - value1) / (value2 - value1);
			}

			t = Math.min(Math.max(t, 0), 1);

			Jun3dPoint p1 = new Jun3dPoint(x1, y1, z1);
			Jun3dPoint p2 = new Jun3dPoint(x2, y2, z2);
			points[i] = (Jun3dPoint) ((p2.minus_(p1)).multipliedBy_(t)).plus_(p1);
		}

		JunPolygon polygon = JunPolygon.Vertexes_(points);

		if (polygon != null) {
			polygonStream.addElement(polygon);
		}

		return;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param evaluatingBlock DOCUMENT ME!
	 * @param image DOCUMENT ME!
	 * @param height DOCUMENT ME!
	 * @param scale DOCUMENT ME!
	 * @param aNumber DOCUMENT ME!
	 * @param aNumber2 DOCUMENT ME!
	 * @param xInterval DOCUMENT ME!
	 * @param yInterval DOCUMENT ME!
	 * @param zInterval DOCUMENT ME!
	 * @param lastNumPolygons DOCUMENT ME!
	 * @param dObject DOCUMENT ME!
	 * @param displayModel DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private static JunPolygon[] _contourPolygonsFrom_at_sign_xInterval_yInterval_zInterval_interim_(
		StBlockClosure evaluatingBlock,
		StImage image,
		int height,
		double scale,
		double aNumber,
		int aNumber2,
		int[] xInterval,
		int[] yInterval,
		double[] zInterval,
		StValueHolder lastNumPolygons,
		JunOpenGL3dCompoundObject dObject,
		JunOpenGLDisplayModel displayModel) {
		final StImage image_ = image;
		final int height_ = height;
		final double scale_ = scale;
		final Vector polygonStream = new Vector();
		final int[] xInterval_ = xInterval;
		final int[] yInterval_ = yInterval;
		final double[] zInterval_ = zInterval;

		for (int zIndex = 1; zIndex < zInterval.length; zIndex++) {
			for (int yIndex = 1; yIndex < yInterval.length; yIndex++) {
				for (int xIndex = 1; xIndex < xInterval.length; xIndex++) {
					_trianglesWith_at_sign_inUnitCubeAt_do_(evaluatingBlock, image, height, scale, aNumber, aNumber2, new Jun3dPoint(xIndex, yIndex, zIndex), xInterval, yInterval, zInterval, polygonStream);
				}
			}

			int polygonSize = polygonStream.size();
			JunPolygon[] polygonContents = new JunPolygon[polygonSize];

			for (int i = 0; i < polygonSize; i++) {
				polygonContents[i] = (JunPolygon) polygonStream.elementAt(i);
			}

			_evaluateInterrimBlock_(polygonContents, lastNumPolygons, dObject, displayModel);
		}

		int finalPolygonSize = polygonStream.size();
		JunPolygon[] polygonContents = new JunPolygon[finalPolygonSize];

		for (int i = 0; i < finalPolygonSize; i++) {
			polygonContents[i] = (JunPolygon) polygonStream.elementAt(i);
		}

		return polygonContents;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param evaluationBlock sra.smalltalk.StBlockClosure
	 * @param aNumber double
	 * @param aNumber2 int
	 * @param aPoint java.awt.Point
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	protected static void _edgesWith_at_sign_inUnitSquareAt_do_(StBlockClosure evaluationBlock, double aNumber, int aNumber2, Point aPoint, StBlockClosure aBlock) {
		int x0 = aPoint.x;
		int x1 = x0 + 1;
		int y0 = aPoint.y;
		int y1 = y0 + 1;
		int sign = (aNumber2 > 0) ? 1 : (aNumber2 < 0) ? (-1) : 0;
		int index = 0;
		double value = ((Number) evaluationBlock.value_value_(new Integer(x0), new Integer(y1))).doubleValue();

		if (((value - aNumber) * sign) >= 0) {
			index += 1;
		}

		value = ((Number) evaluationBlock.value_value_(new Integer(x1), new Integer(y1))).doubleValue();

		if (((value - aNumber) * sign) >= 0) {
			index += 2;
		}

		value = ((Number) evaluationBlock.value_value_(new Integer(x1), new Integer(y0))).doubleValue();

		if (((value - aNumber) * sign) >= 0) {
			index += 4;
		}

		value = ((Number) evaluationBlock.value_value_(new Integer(x0), new Integer(y0))).doubleValue();

		if (((value - aNumber) * sign) >= 0) {
			index += 8;
		}

		if ((index < 1) || (14 < index)) {
			return;
		}

		if (index == 1) {
			aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x0, y1, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x0, y1, 0), new Jun3dPoint(x1, y1, 0), aNumber));

			return;
		}

		if (index == 2) {
			aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x0, y1, 0), new Jun3dPoint(x1, y1, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x1, y0, 0), new Jun3dPoint(x1, y1, 0), aNumber));

			return;
		}

		if (index == 3) {
			aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x0, y1, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x1, y0, 0), new Jun3dPoint(x1, y1, 0), aNumber));

			return;
		}

		if (index == 4) {
			aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x1, y0, 0), new Jun3dPoint(x1, y1, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x1, y0, 0), aNumber));

			return;
		}

		if (index == 5) {
			if (true) {
				aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x0, y1, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x1, y0, 0), aNumber));
				aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x1, y0, 0), new Jun3dPoint(x1, y1, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x0, y1, 0), new Jun3dPoint(x1, y1, 0), aNumber));

				return;
			} else {
				aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x1, y0, 0), new Jun3dPoint(x1, y1, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x1, y0, 0), aNumber));
				aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x0, y1, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x0, y1, 0), new Jun3dPoint(x1, y1, 0), aNumber));

				return;
			}
		}

		if (index == 6) {
			aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x0, y1, 0), new Jun3dPoint(x1, y1, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x1, y0, 0), aNumber));

			return;
		}

		if (index == 7) {
			aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x0, y1, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x1, y0, 0), aNumber));

			return;
		}

		if (index == 8) {
			aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x1, y0, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x0, y1, 0), aNumber));

			return;
		}

		if (index == 9) {
			aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x1, y0, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x0, y1, 0), new Jun3dPoint(x1, y1, 0), aNumber));

			return;
		}

		if (index == 10) {
			if (true) {
				aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x1, y0, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x1, y0, 0), new Jun3dPoint(x1, y1, 0), aNumber));
				aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x0, y1, 0), new Jun3dPoint(x1, y1, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x0, y1, 0), aNumber));

				return;
			} else {
				aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x1, y0, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x0, y1, 0), aNumber));
				aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x0, y1, 0), new Jun3dPoint(x1, y1, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x1, y0, 0), new Jun3dPoint(x1, y1, 0), aNumber));

				return;
			}
		}

		if (index == 11) {
			aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x1, y0, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x1, y0, 0), new Jun3dPoint(x1, y1, 0), aNumber));

			return;
		}

		if (index == 12) {
			aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x1, y0, 0), new Jun3dPoint(x1, y1, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x0, y1, 0), aNumber));

			return;
		}

		if (index == 13) {
			aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x1, y0, 0), new Jun3dPoint(x1, y1, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x0, y1, 0), new Jun3dPoint(x1, y1, 0), aNumber));

			return;
		}

		if (index == 14) {
			aBlock.value_value_(new JunBorderVertex(new Jun3dPoint(x0, y1, 0), new Jun3dPoint(x1, y1, 0), aNumber), new JunBorderVertex(new Jun3dPoint(x0, y0, 0), new Jun3dPoint(x0, y1, 0), aNumber));

			return;
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBlock DOCUMENT ME!
	 * @param image DOCUMENT ME!
	 * @param height DOCUMENT ME!
	 * @param scale DOCUMENT ME!
	 * @param x DOCUMENT ME!
	 * @param y DOCUMENT ME!
	 * @param z DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private static double _evaluateEvaluationBlock_(StBlockClosure aBlock, StImage image, int height, double scale, int x, int y, double z) {
		Jun3dPoint aPoint = new Jun3dPoint(x, y, z);
		Double result = (Double) aBlock.value_value_value_(image, aPoint, new Double(scale));

		return result.doubleValue();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param polygons DOCUMENT ME!
	 * @param lastNumPolygons DOCUMENT ME!
	 * @param dObject DOCUMENT ME!
	 * @param displayModel DOCUMENT ME!
	 */
	private static void _evaluateInterrimBlock_(JunPolygon[] polygons, StValueHolder lastNumPolygons, JunOpenGL3dCompoundObject dObject, JunOpenGLDisplayModel displayModel) {
		int polygonSize = polygons.length;

		if (polygonSize == lastNumPolygons._intValue()) {
			return;
		}

		JunOpenGL3dObject[] objects = new JunOpenGL3dObject[polygonSize];

		for (int index = 0; index < polygonSize; index++) {
			objects[index] = polygons[index].asJunOpenGL3dObject();
		}

		dObject.components_(objects);
		displayModel.fitZoom();
		lastNumPolygons.value_(polygonSize);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param evaluatingBlock DOCUMENT ME!
	 * @param triangle DOCUMENT ME!
	 * @param image DOCUMENT ME!
	 * @param height DOCUMENT ME!
	 * @param scale DOCUMENT ME!
	 * @param xInterval DOCUMENT ME!
	 * @param yInterval DOCUMENT ME!
	 * @param zInterval DOCUMENT ME!
	 * @param polygonStream DOCUMENT ME!
	 */
	private static void _evaluateMainBlock_(StBlockClosure evaluatingBlock, JunBorderVertex[] triangle, StImage image, int height, double scale, int[] xInterval, int[] yInterval, double[] zInterval, Vector polygonStream) {
		Jun3dPoint[] points = new Jun3dPoint[triangle.length];

		for (int i = 0; i < triangle.length; i++) {
			JunBorderVertex vertex = (JunBorderVertex) triangle[i];
			int x1 = xInterval[(int) Math.round(vertex.vertex1().x()) - 1];
			int y1 = yInterval[(int) Math.round(vertex.vertex1().y()) - 1];
			double z1 = (double) xInterval[(int) Math.round(vertex.vertex2().z()) - 1];
			int x2 = xInterval[(int) Math.round(vertex.vertex2().x()) - 1];
			int y2 = yInterval[(int) Math.round(vertex.vertex2().y()) - 1];
			double z2 = (double) xInterval[(int) Math.round(vertex.vertex2().z()) - 1];
			double value1 = _evaluateEvaluationBlock_(evaluatingBlock, image, height, scale, x1, y1, z1);
			double value2 = _evaluateEvaluationBlock_(evaluatingBlock, image, height, scale, x2, y2, z2);
			double t;

			if (value1 == value2) {
				t = 0;
			} else {
				t = (vertex.label() - value1) / (value2 - value1);
			}

			t = Math.min(Math.max(t, 0), 1);

			Jun3dPoint p1 = new Jun3dPoint(x1, y1, z1);
			Jun3dPoint p2 = new Jun3dPoint(x2, y2, z2);
			points[i] = (Jun3dPoint) ((p2.minus_(p1)).multipliedBy_(t)).plus_(p1);
		}

		JunPolygon polygon = JunPolygon.Vertexes_(points);

		if (polygon != null) {
			polygonStream.addElement(polygon);
		}

		return;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param evaluatingBlock DOCUMENT ME!
	 * @param image DOCUMENT ME!
	 * @param height DOCUMENT ME!
	 * @param scale DOCUMENT ME!
	 * @param aNumber DOCUMENT ME!
	 * @param aNumber2 DOCUMENT ME!
	 * @param aJun3dPoint DOCUMENT ME!
	 * @param xInterval DOCUMENT ME!
	 * @param yInterval DOCUMENT ME!
	 * @param zInterval DOCUMENT ME!
	 * @param polygonStream DOCUMENT ME!
	 */
	private static void _trianglesWith_at_sign_inUnitCubeAt_do_(
		StBlockClosure evaluatingBlock,
		StImage image,
		int height,
		double scale,
		double aNumber,
		int aNumber2,
		Jun3dPoint aJun3dPoint,
		int[] xInterval,
		int[] yInterval,
		double[] zInterval,
		Vector polygonStream) {
		int x0 = (int) Math.round(aJun3dPoint.x());
		int x1 = x0 + 1;
		int y0 = (int) Math.round(aJun3dPoint.y());
		int y1 = y0 + 1;
		int z0 = (int) Math.round(aJun3dPoint.z());
		int z1 = z0 + 1;
		int sign;

		if (aNumber2 == 0) {
			sign = 0;
		} else {
			if (aNumber2 > 0) {
				sign = 1;
			} else {
				sign = -1;
			}
		}

		int index = 0;

		if ((((_evaluateEvaluationBlock_(evaluatingBlock, image, height, scale, x0, y0, z0)) - aNumber) * sign) > 0) {
			index = index + 1;
		}

		if ((((_evaluateEvaluationBlock_(evaluatingBlock, image, height, scale, x1, y0, z0)) - aNumber) * sign) > 0) {
			index = index + 2;
		}

		if ((((_evaluateEvaluationBlock_(evaluatingBlock, image, height, scale, x1, y1, z0)) - aNumber) * sign) > 0) {
			index = index + 4;
		}

		if ((((_evaluateEvaluationBlock_(evaluatingBlock, image, height, scale, x0, y1, z0)) - aNumber) * sign) > 0) {
			index = index + 8;
		}

		if ((((_evaluateEvaluationBlock_(evaluatingBlock, image, height, scale, x0, y0, z1)) - aNumber) * sign) > 0) {
			index = index + 16;
		}

		if ((((_evaluateEvaluationBlock_(evaluatingBlock, image, height, scale, x1, y0, z1)) - aNumber) * sign) > 0) {
			index = index + 32;
		}

		if ((((_evaluateEvaluationBlock_(evaluatingBlock, image, height, scale, x1, y1, z1)) - aNumber) * sign) > 0) {
			index = index + 64;
		}

		if ((((_evaluateEvaluationBlock_(evaluatingBlock, image, height, scale, x0, y1, z1)) - aNumber) * sign) > 0) {
			index = index + 128;
		}

		if ((index < 1) || (index > 254)) {
			return;
		}

		JunBorderVertex[][] array = (JunBorderVertex[][]) MarchingCubes[index - 1].value_value_(aJun3dPoint, new Double(aNumber));

		for (int i = 0; i < array.length; i++) {
			_evaluateMainBlock_(evaluatingBlock, array[i], image, height, scale, xInterval, yInterval, zInterval, polygonStream);
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return StBlockClosure[]
	 */
	private static final StBlockClosure[] InitializeMarchingCubes() {
		StBlockClosure[] MarchingCubes = new StBlockClosure[254];
		JunBorderGenerator00.InitializeMarchingCubes0_(MarchingCubes);
		JunBorderGenerator01.InitializeMarchingCubes1_(MarchingCubes);
		JunBorderGenerator02.InitializeMarchingCubes2_(MarchingCubes);
		JunBorderGenerator03.InitializeMarchingCubes3_(MarchingCubes);
		JunBorderGenerator04.InitializeMarchingCubes4_(MarchingCubes);
		JunBorderGenerator05.InitializeMarchingCubes5_(MarchingCubes);
		JunBorderGenerator06.InitializeMarchingCubes6_(MarchingCubes);
		JunBorderGenerator07.InitializeMarchingCubes7_(MarchingCubes);
		JunBorderGenerator08.InitializeMarchingCubes8_(MarchingCubes);
		JunBorderGenerator09.InitializeMarchingCubes9_(MarchingCubes);
		JunBorderGenerator10.InitializeMarchingCubes10_(MarchingCubes);
		JunBorderGenerator11.InitializeMarchingCubes11_(MarchingCubes);
		JunBorderGenerator12.InitializeMarchingCubes12_(MarchingCubes);
		JunBorderGenerator13.InitializeMarchingCubes13_(MarchingCubes);
		JunBorderGenerator14.InitializeMarchingCubes14_(MarchingCubes);
		JunBorderGenerator15.InitializeMarchingCubes15_(MarchingCubes);

		return MarchingCubes;
	}
}
