package sra.jun.geometry.boundaries;

import sra.smalltalk.*;
import sra.jun.goodies.images.support.JunImageProcessor;
import sra.jun.opengl.display.JunOpenGLDisplayModel;

/**
 * JunBorderGeneratorModel class
 * 
 *  @author    Hirotsugu Kondo
 *  @created   1999/1/13 (by Hirotsugu Kondo)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunBorderGeneratorModel.java,v 6.0 2003/10/28 10:52:06 nisinaka Exp $
 */
public class JunBorderGeneratorModel extends JunOpenGLDisplayModel {

	protected StSymbol indexMode = $("Brightness");
	protected StMenu _indexMenu;

	/**
	 * DOCUMENT ME!
	 */
	public void fromBrightnessValue() {
		this.indexMode_($("Brightness"));
		this.updateIndexMenuIndication();
	}

	/**
	 * DOCUMENT ME!
	 */
	public void fromHueValue() {
		this.indexMode_($("Hue"));
		this.updateIndexMenuIndication();
	}

	/**
	 * DOCUMENT ME!
	 */
	public void fromSaturationValue() {
		this.indexMode_($("Saturation"));
		this.updateIndexMenuIndication();
	}

	/**
	 * DOCUMENT ME!
	 */
	public void imageFromMask() {
		String input = StDialog.Request_("Set Halftone Value(0.0-1.0)");

		if (input == null) {
			return;
		}

		double value = new Double(input).doubleValue();

		if ((value < 0) || (value > 1)) {
			return;
		}

		JunBorderGenerator.LandscapeFromImage_with_on_(JunImageProcessor.Halftone_(value), this.valueBlock(), this);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public boolean isBrightnessValueMode() {
		return this.indexMode() == $("Brightness");
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public boolean isHueValueMode() {
		return this.indexMode() == $("Hue");
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public boolean isSaturationValueMode() {
		return this.indexMode() == $("Saturation");
	}

	/**
	 * DOCUMENT ME!
	 */
	public void openImageFile() {
		StImage image = JunImageProcessor.ImageFromFileDialog();

		if (image == null) {
			return;
		}

		JunBorderGenerator.LandscapeFromImage_with_on_(image, this.valueBlock(), this);
	}

	/**
	 * Update the index menu indication.
	 */
	public void updateIndexMenuIndication() {
		StMenu indexMenu = this._indexMenu();
		if (indexMenu == null) {
			return;
		}

		if (isHueValueMode()) {
			((StRadioButtonMenuItem) indexMenu.getItem("Hue")).setSelected(true);
		} else if (this.isSaturationValueMode()) {
			((StRadioButtonMenuItem) indexMenu.getItem("Saturation")).setSelected(true);
		} else {
			((StRadioButtonMenuItem) indexMenu.getItem("Brightness")).setSelected(true);
		}
	}

	/**
	 * Update the menu indications.
	 */
	public void updateMenuIndication() {
		super.updateMenuIndication();
		this.updateIndexMenuIndication();
	}

	/**
	 * Answer the default index menu for JunBorderGeneratorModel.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _defaultIndexMenu() {
		StMenu defaultIndexMenu = new StMenu("Index");

		StButtonGroup indexGroup = new StButtonGroup();
		defaultIndexMenu.add(createRadioButtonMenuItem("Hue", "fromHueValue", indexGroup));
		defaultIndexMenu.add(createRadioButtonMenuItem("Saturation", "fromSaturationValue", indexGroup));
		defaultIndexMenu.add(createRadioButtonMenuItem("Brightness", "fromBrightnessValue", indexGroup));

		return defaultIndexMenu;
	}

	/**
	 * Answer my index menu.
	 *
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _indexMenu() {
		if (_indexMenu == null) {
			_indexMenu = this._defaultIndexMenu();
		}
		return _indexMenu;
	}

	/**
	 * Create the menubar for JunBorderGeneratorModel.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 */
	protected StMenuModel defaultMenu() {
		StMenuModel rootMenu = new StMenuModel();

		rootMenu.add(this._fileMenu());
		rootMenu.add(this._editMenu());
		rootMenu.add(this._viewMenu());
		rootMenu.add(this._lightMenu());
		rootMenu.add(this._indexMenu());
		rootMenu.add(this._miscMenu());

		return rootMenu;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private StSymbol indexMode() {
		return this.indexMode;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param newMode DOCUMENT ME!
	 */
	private void indexMode_(StSymbol newMode) {
		this.indexMode = newMode;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private StBlockClosure valueBlock() {
		if (this.isHueValueMode()) {
			return JunBorderGenerator.EvaluateBlockForHue();
		}

		if (this.isSaturationValueMode()) {
			return JunBorderGenerator.EvaluateBlockForSaturation();
		}

		return JunBorderGenerator.EvaluateBlockForBrightness();
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "Border Generator";
	}
}
