package sra.jun.geometry.boundaries;

import java.awt.*;
import java.util.Vector;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.surfaces.JunPolygon;
import sra.jun.goodies.progress.JunCursorAnimator;
import sra.jun.opengl.display.JunOpenGLDisplayModel;
import sra.jun.opengl.objects.*;
import sra.jun.opengl.texture.JunOpenGLTexture;
import sra.jun.system.support.JunTestExamples;
import sra.smalltalk.*;

/**
 * JunBorderGeneratorTestExamples class
 * 
 *  @author    nisinaka
 *  @created   2002/01/21 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunBorderGeneratorTestExamples.java,v 6.0 2003/10/28 10:52:06 nisinaka Exp $
 */
public class JunBorderGeneratorTestExamples extends JunTestExamples {

	/**
	 * 2D examples: circle example
	 * 
	 * @return boolean
	 */
	public static boolean ExampleCircle() {
		final Jun2dPoint center = new Jun2dPoint(0, 0);
		StBlockClosure evaluationBlock = new StBlockClosure() {
			public Object value_value_(Object x, Object y) {
				return new Double(center.distance_(new Jun2dPoint((Number) x, (Number) y)));
			}
		};

		Jun2dPolygonalBoundary circle = JunBorderGenerator.PolygonalBoundaryWith_max_xInterval_yInterval_interim_(evaluationBlock, (double) 1, new StInterval(-1.1, 1.1, 0.1), new StInterval(-1.1, 1.1, 0.1), null);
		JunOpenGL3dObject circleObject = circle.asJunOpenGL3dObject();
		circleObject.paint_(Color.blue);
		circleObject.show();

		return true;
	}

	/**
	 * 2D examples: image contouring
	 * 
	 * @return DOCUMENT ME!
	 */
	public static boolean ExampleImageContouring() {
		final StImage anImage = StImage._FromUser();
		final int width = anImage.width();
		final int height = anImage.height();
		final double scale = Math.max(width, height);
		JunOpenGL3dCompoundObject displayObject = new JunOpenGL3dCompoundObject();
		JunOpenGL3dPolygon texturePolygon = new JunOpenGL3dPolygon(new Jun3dPoint[] { new Jun3dPoint(width * -1, height * -1, 0), new Jun3dPoint(width, height * -1, 0), new Jun3dPoint(width, height, 0), new Jun3dPoint(width * -1, height, 0), });
		JunOpenGLTexture texture = JunOpenGLTexture.Image_(anImage);
		texturePolygon.texture_(texture);
		texture.zPositiveParameters();
		displayObject.add_(texturePolygon);

		final JunOpenGLDisplayModel displayModel = displayObject.show();
		displayModel.sightPoint_(new Jun3dPoint(0, 0, scale / 2));
		displayModel.eyePoint_(new Jun3dPoint(0, scale * -4, scale * 2));
		displayModel.upVector_(new Jun3dPoint(0, 1, 0));
		displayModel.fitZoom();

		JunCursorAnimator.ClockCursors().showWhile_(new StBlockClosure() {
			public Object value() {
				for (int i = 0; i < 10; i++) {
					double brightness = i * 0.1;
					StBlockClosure aBlock = new StBlockClosure() {
						public Object value_value_(Object objX, Object objY) {
							int x = ((Number) objX).intValue();
							int y = ((Number) objY).intValue();

							return new Float(StColorValue._GetBrightness(anImage.valueAtPoint_(new Point(x, y))));
						}
					};

					Jun2dPoint[][] polylines = JunBorderGenerator.ContourPolylinesFrom_at_xInterval_yInterval_interim_(aBlock, brightness, new StInterval(0, width - 1), new StInterval(0, height - 1), null);

					for (int j = 0; j < polylines.length; j++) {
						Jun2dPoint[] points = polylines[j];
						Jun3dPoint[] vertexes = new Jun3dPoint[points.length];

						for (int k = 0; k < points.length; k++) {
							double x = points[k].x();
							double y = points[k].y();
							vertexes[k] = new Jun3dPoint((x * 2) - width + 3, height - (y * 2), brightness * scale);
						}

						((JunOpenGL3dCompoundObject) displayModel.displayObject()).add_(new JunOpenGL3dPolyline(vertexes, StColorValue.Brightness_(brightness)));
					}

					displayModel.fitZoom();
				}

				displayModel.eyePoint_(new Jun3dPoint(0, 0, scale * 2));
				displayModel.parallelProjection();
				displayModel.fitZoom();

				return null;
			}
		});

		displayModel.changed();

		return true;
	}

	/**
	 * 2D examples: negative circle example
	 * 
	 * @return boolean
	 */
	public static boolean ExampleNegativeCircle() {
		StBlockClosure evaluationBlock = new StBlockClosure() {
			public Object value_value_(Object x, Object y) {
				return new Double((new Jun2dPoint((Number) x, (Number) y)).r());
			}
		};

		Jun2dPolygonalBoundary circle = JunBorderGenerator.PolygonalBoundaryWith_min_xInterval_yInterval_interim_(evaluationBlock, (double) 1, new StInterval(-1.1, 1.1, 0.1), new StInterval(-1.1, 1.1, 0.1), null);
		JunOpenGL3dObject circleObject = circle.asJunOpenGL3dObject();
		circleObject.paint_(Color.blue);
		circleObject.show();

		return true;
	}

	/**
	 * 2D examples: text
	 * 
	 * @return DOCUMENT ME!
	 */
	public static boolean ExampleText() {
		String aString = StDialog.Request_("Type in your favorite phrase", "Be free!");

		if ((aString == null) || (aString.length() == 0)) {
			return true;
		}

		String weightString = StDialog.Request_("Enter weight of the font. (0.1 .. 0.9)", "0.5");
		double weight;

		try {
			weight = Double.parseDouble(weightString);
		} catch (NumberFormatException e) {
			return true;
		}

		weight = Math.max(0.1, Math.min(weight, 0.9));

		Font font = new Font("Courier", Font.PLAIN, 12);
		Point extent = SystemResourceSupport._getStringExtentFor_(aString, font);
		JunOpenGLDisplayModel displayModel = new JunOpenGLDisplayModel();
		displayModel.open();
		displayModel.displayObject_(new JunOpenGL3dCompoundObject());
		displayModel.sightPoint_(new Jun3dPoint(extent.x / 2, extent.y / 2, 0));
		displayModel.eyePoint_(new Jun3dPoint(extent.x / 2, extent.y / 2, extent.x));
		displayModel.upVector_(new Jun3dPoint(0, 1, 0));

		int xOffset = 0;

		for (int i = 0; i < aString.length(); i++) {
			String str = String.valueOf(aString.charAt(i));
			extent = SystemResourceSupport._getStringExtentFor_(str, font);

			final StImage image = new StImage(extent.x, extent.y);
			Graphics graphics = null;

			try {
				graphics = image.image().getGraphics();
				graphics.setColor(Color.black);
				graphics.fillRect(0, 0, extent.x, extent.y);
				graphics.setColor(Color.white);
				graphics.setFont(font);

				int y = SystemResourceSupport.getFontMetrics(font).getAscent();
				graphics.drawString(str, 1, y);
				image.image().flush();
			} finally {
				if (graphics != null) {
					graphics.dispose();
					graphics = null;
				}
			}

			StBlockClosure evaluationBlock = new StBlockClosure() {
				public Object value_value_(Object x, Object y) {
					Color color = image.valueAtPoint_(new Point(((Number) x).intValue(), ((Number) y).intValue()));
					float[] hsb = Color.RGBtoHSB(color.getRed(), color.getGreen(), color.getBlue(), null);

					return new Float(hsb[2]);
				}
			};

			Jun2dPolygonalBoundary polygonalBoundary =
				JunBorderGenerator.PolygonalBoundaryWith_min_max_xInterval_yInterval_interim_(evaluationBlock, 1 - weight, (double) 1, new StInterval(0, image.width() - 1), new StInterval(0, image.height() - 1), new StBlockClosure());
			polygonalBoundary = polygonalBoundary.scaledBy_(new Jun2dPoint(1, -1));
			polygonalBoundary = polygonalBoundary.translatedBy_(new Jun2dPoint(xOffset, image.height()));
			xOffset += image.width();
			((JunOpenGL3dCompoundObject) displayModel.displayObject()).add_(polygonalBoundary.asJunOpenGL3dObject());
			displayModel.fitZoom();
		}

		return true;
	}

	/**
	 * 3D examples: text
	 * 
	 * @return DOCUMENT ME!
	 */
	public static boolean ExampleText3d() {
		String aString = StDialog.Request_("Type in your favorite phrase", "Be free!");

		if ((aString == null) || (aString.length() == 0)) {
			return true;
		}

		String weightString = StDialog.Request_("Enter weight of the font. (0.1 .. 0.9)", "0.5");
		double weight;

		try {
			weight = Double.parseDouble(weightString);
		} catch (NumberFormatException e) {
			return true;
		}

		weight = Math.max(0.1, Math.min(weight, 0.9));

		Font font = new Font("Courier", Font.PLAIN, 12);
		Point extent = SystemResourceSupport._getStringExtentFor_(aString, font);
		final JunOpenGLDisplayModel displayModel = new JunOpenGLDisplayModel();
		displayModel.open();
		displayModel.displayObject_(new JunOpenGL3dCompoundObject());
		displayModel.sightPoint_(new Jun3dPoint(extent.x / 2, extent.y / 2, 0));
		displayModel.eyePoint_(new Jun3dPoint(extent.x / 2, extent.y / 2, extent.x));
		displayModel.upVector_(new Jun3dPoint(0, 1, 0));

		final int[] xOffset = new int[] { 0 };

		for (int i = 0; i < aString.length(); i++) {
			final JunOpenGL3dCompoundObject displayObject = new JunOpenGL3dCompoundObject();
			displayObject.paint_(Color.blue);
			((JunOpenGL3dCompoundObject) displayModel.displayObject()).add_(displayObject);

			final int[] lastNumPolygons = new int[] { 0 };

			String str = String.valueOf(aString.charAt(i));
			extent = SystemResourceSupport._getStringExtentFor_(str, font);

			final StImage image = new StImage(extent.x, extent.y);
			Graphics graphics = null;

			try {
				graphics = image.image().getGraphics();
				graphics.setColor(Color.black);
				graphics.fillRect(0, 0, extent.x, extent.y);
				graphics.setColor(Color.white);
				graphics.setFont(font);

				int y = SystemResourceSupport.getFontMetrics(font).getAscent();
				graphics.drawString(str, 1, y);
				image.image().flush();
			} finally {
				if (graphics != null) {
					graphics.dispose();
					graphics = null;
				}
			}

			StBlockClosure evaluationBlock = new StBlockClosure() {
				public Object value_value_value_(Object objX, Object objY, Object objZ) {
					int x = ((Number) objX).intValue();
					int y = ((Number) objY).intValue();
					int z = ((Number) objZ).intValue();
					Color color = image.valueAtPoint_(new Point(x, image.height() - y));
					float[] hsb = Color.RGBtoHSB(color.getRed(), color.getGreen(), color.getBlue(), null);
					double result = 1 - hsb[2];
					result = result * result;
					result += (z * z);
					result = Math.sqrt(result);

					return new Double(result);
				}
			};

			StBlockClosure interimBlock = new StBlockClosure() {
				public Object value_(Object obj) {
					Vector ps = (Vector) obj;

					if (ps.size() != lastNumPolygons[0]) {
						for (int j = 0; j < ps.size(); j++) {
							displayObject.add_(((JunPolygon) ps.get(j)).asJunOpenGL3dObject().translatedBy_(new Jun3dPoint(xOffset[0], 0, 0)));
						}

						displayModel.fitZoom();
						lastNumPolygons[0] = ps.size();
					}

					return null;
				}
			};

			JunBorderGenerator.ContourPolygonsFrom_at_sign_xInterval_yInterval_zInterval_interim_(evaluationBlock, weight, -1, new StInterval(0, image.width() - 1), new StInterval(1, image.height()), new StInterval(-1, 1), interimBlock);
			xOffset[0] += image.width();
		}

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * 
	 * @throws Exception DOCUMENT ME!
	 */
	public static void main(String[] args) throws Exception {
		new JunBorderGeneratorTestExamples();
	}
}
