package sra.jun.geometry.boundaries;

import sra.jun.geometry.basic.*;

/**
 * JunBorderVertex class
 * 
 *  @author    Hirotsugu Kondo
 *  @created   1999/1/8 (by Hirotsugu Kondo)
 *  @updated   2002/01/21 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunBorderVertex.java,v 6.0 2003/10/28 10:52:06 nisinaka Exp $
 */
public class JunBorderVertex extends sra.smalltalk.StObject {
	private Jun3dPoint vertex1;
	private Jun3dPoint vertex2;
	private double label;

	/**
	 * Create a new JunBorderVertex and initialize it.
	 * 
	 * @param aPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param aPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param aNumber double
	 */
	public JunBorderVertex(Jun3dPoint aPoint1, Jun3dPoint aPoint2, double aNumber) {
		this.vertex1_(aPoint1);
		this.vertex2_(aPoint2);
		this.label_(aNumber);
	}

	/**
	 * Create a new JunBorderVertex and initialize it.
	 * 
	 * @param aPoint1 sra.jun.geometry.basic.JunPoint
	 * @param aPoint2 sra.jun.geometry.basic.JunPoint
	 * @param aNumber double
	 * 
	 * @return sra.jun.geometry.boundaries.JunBorderVertex
	 */
	public static JunBorderVertex Vertex_vertex_label_(JunPoint aPoint1, JunPoint aPoint2, double aNumber) {
		return new JunBorderVertex((Jun3dPoint) aPoint1, (Jun3dPoint) aPoint2, aNumber);
	}

	/**
	 * Answer true if the Object is equal to the receiver, otherwise false.
	 * 
	 * @param other java.lang.Object
	 * 
	 * @return boolean
	 */
	public boolean equals(Object other) {
		if (other.getClass() == this.getClass()) {
			JunBorderVertex aJunBorderVertex = (JunBorderVertex) other;

			return ((vertex1.equals(aJunBorderVertex.vertex1)) && (vertex2.equals(aJunBorderVertex.vertex2)) && (label == aJunBorderVertex.label));
		}

		return false;
	}

	// Hash Function

	/**
	 * Computes a hash code for this object.
	 * 
	 * @return int
	 */
	public int hashCode() {
		return vertex1.hashCode();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return double
	 */
	public double label() {
		return label;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNumber
	 */
	public void label_(double aNumber) {
		label = aNumber;
	}

	/**
	 * Answer true if the vertex in on the border of the specified rectangle.
	 * 
	 * @param aRectangle java.awt.Rectangle
	 * 
	 * @return boolean
	 */
	public boolean onBorderOfRectangle_(java.awt.Rectangle aRectangle) {
		return (this.onX_(aRectangle.getX()) || this.onX_(aRectangle.getX() + aRectangle.getWidth()) || this.onY_(aRectangle.getY()) || this.onY_(aRectangle.getY() + aRectangle.getHeight()));
	}

	/**
	 * Answer true if the x values of the vertex equals to the number.
	 * 
	 * @param aNumber double
	 * 
	 * @return boolean
	 */
	public boolean onX_(double aNumber) {
		return (vertex1.x() == aNumber && vertex2.x() == aNumber);
	}

	/**
	 * Answer true if the y values of the vertex equals to the number.
	 * 
	 * @param aNumber double
	 * 
	 * @return boolean
	 */
	public boolean onY_(double aNumber) {
		return (vertex1.y() == aNumber && vertex2.y() == aNumber);
	}

	/**
	 * Answer true if the z values of the vertex equals to the number.
	 * 
	 * @param aNumber double
	 * 
	 * @return boolean
	 */
	public boolean onZ_(double aNumber) {
		return (vertex1.z() == aNumber && vertex2.z() == aNumber);
	}

	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * 
	 * @throws java.io.IOException DOCUMENT ME!
	 */
	public void printOn_(java.io.Writer aWriter) throws java.io.IOException {
		aWriter.write("JunBorderVertex (");
		aWriter.write(this.vertex1() + " ");
		aWriter.write(this.vertex2() + " ");
		aWriter.write(this.label() + ")");
	}

	/**
	 * Answer printable string of this object.
	 * 
	 * @return java.lang.String
	 * 
	 * @throws sra.smalltalk.SmalltalkException DOCUMENT ME!
	 */
	public String printString() {
		try {
			java.io.StringWriter aWriter = new java.io.StringWriter();
			this.printOn_(aWriter);

			return aWriter.toString();
		} catch (java.io.IOException e) {
			throw new sra.smalltalk.SmalltalkException(e);
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return Jun3dPoint
	 */
	public Jun3dPoint vertex1() {
		return vertex1;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aPoint
	 */
	public void vertex1_(Jun3dPoint aPoint) {
		vertex1 = aPoint;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return Jun3dPoint
	 */
	public Jun3dPoint vertex2() {
		return vertex2;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aPoint
	 */
	public void vertex2_(Jun3dPoint aPoint) {
		vertex2 = aPoint;
	}
}
