package sra.jun.geometry.coordinate;

import java.util.Vector;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.*;
import sra.smalltalk.*;

/**
 * Jun3dCoordinateSystem class
 * 
 *  @author    nisinaka
 *  @created   1998/10/05 (by nisinaka)
 *  @updated   2000/01/06 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: Jun3dCoordinateSystem.java,v 6.0 2003/10/28 10:52:06 nisinaka Exp $
 */
public class Jun3dCoordinateSystem extends JunCoordinateSystem {
	protected Jun3dPoint origin = null;
	protected Jun3dTransformation transformation = null;
	protected Jun3dTransformation invTransformation = null;
	protected Jun3dTransformation globalTransformation = null;
	protected Jun3dTransformation globalInvTransformation = null;
	protected Jun3dCoordinateSystem outerCoordinateSystem = null;
	protected Vector localCoordinateSystems = null;
	private Jun3dPoint xUnitVector;
	private Jun3dPoint yUnitVector;
	private Jun3dPoint zUnitVector;

	/**
	 * Create a new Jun3dCoordinateSystem with the specified scale.
	 * 
	 * @param aNumber double
	 * 
	 * @return sra.jun.geometry.coordinate.Jun3dCoordinateSystem
	 */
	public static Jun3dCoordinateSystem Scale_(double aNumber) {
		Jun3dTransformation aTransformation = Jun3dTransformation.Scale_(aNumber);

		return Transformation_(aTransformation);
	}

	/**
	 * Create a new Jun3dCoordinateSystem with the specified scale.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.geometry.coordinate.Jun3dCoordinateSystem
	 */
	public static Jun3dCoordinateSystem Scale_(Jun3dPoint aPoint) {
		Jun3dTransformation aTransformation = Jun3dTransformation.Scale_(aPoint);

		return Transformation_(aTransformation);
	}

	/**
	 * Create a new Jun3dCoordinateSystem with specified Jun3dTransformation.
	 * 
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.geometry.coordinate.Jun3dCoordinateSystem
	 */
	public static Jun3dCoordinateSystem Transformation_(Jun3dTransformation aTransformation) {
		Jun3dCoordinateSystem aCoordinateSystem = new Jun3dCoordinateSystem();
		aCoordinateSystem.transformation_(aTransformation);

		return aCoordinateSystem;
	}

	/**
	 * Create a new Jun3dCoordinateSystem with the specified translation.
	 * 
	 * @param aNumber double
	 * 
	 * @return sra.jun.geometry.coordinate.Jun3dCoordinateSystem
	 */
	public static Jun3dCoordinateSystem Translate_(double aNumber) {
		Jun3dTransformation aTransformation = Jun3dTransformation.Translate_(aNumber);

		return Transformation_(aTransformation);
	}

	/**
	 * Create a new Jun3dCoordinateSystem with the specified translation.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.geometry.coordinate.Jun3dCoordinateSystem
	 */
	public static Jun3dCoordinateSystem Translate_(Jun3dPoint aPoint) {
		Jun3dTransformation aTransformation = Jun3dTransformation.Translate_(aPoint);

		return Transformation_(aTransformation);
	}

	/**
	 * Convert the receiver as a local coordinate system of aCoordinateSystem.
	 * 
	 * @param aCoordinateSystem
	 *        sra.jun.geometry.coordinate.Jun3dCoordinateSystem
	 * 
	 * @return sra.jun.geometry.coordinate.Jun3dCoordinateSystem
	 */
	public Jun3dCoordinateSystem asLocalCoordinateSystemOf_(Jun3dCoordinateSystem aCoordinateSystem) {
		if (outerCoordinateSystem == aCoordinateSystem) {
			return this;
		}

		if (aCoordinateSystem == null) {
			return Transformation_(this.globalTransformation());
		}

		return aCoordinateSystem.newLocalCoordinateSystemTransformation_(this.globalTransformation().product_((Jun3dTransformation) aCoordinateSystem.globalInvTransformation()));
	}

	/**
	 * Answer the inverse of global transformation.
	 * 
	 * @return sra.jun.geometry.transformations.JunTransformation
	 */
	public JunTransformation globalInvTransformation() {
		if (globalInvTransformation == null) {
			globalInvTransformation = this.globalTransformation().inverse();
		}

		return globalInvTransformation;
	}

	/**
	 * Answer the global transformation.
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public Jun3dTransformation globalTransformation() {
		if (globalTransformation == null) {
			globalTransformation = (Jun3dTransformation) this.transformation();

			if (outerCoordinateSystem != null) {
				globalTransformation = globalTransformation.product_(outerCoordinateSystem.globalTransformation());
			}
		}

		return globalTransformation;
	}

	/**
	 * Create a new local coordinate system with a specified scale.
	 * 
	 * @param aNumber double
	 * 
	 * @return sra.jun.geometry.coordinate.JunCoordinateSystem
	 */
	public Jun3dCoordinateSystem newLocalCoordinateSystemScale_(double aNumber) {
		Jun3dCoordinateSystem aCoordinateSystem = Scale_(aNumber);
		this.privateAddLocalCoordinateSystem_(aCoordinateSystem);

		return aCoordinateSystem;
	}

	/**
	 * Create a new local coordinate system with a transformation.
	 * 
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.geometry.coordinate.Jun3dCoordinateSystem
	 */
	public Jun3dCoordinateSystem newLocalCoordinateSystemTransformation_(Jun3dTransformation aTransformation) {
		Jun3dCoordinateSystem aCoordinateSystem = Transformation_(aTransformation);
		this.privateAddLocalCoordinateSystem_(aCoordinateSystem);

		return aCoordinateSystem;
	}

	/**
	 * Do something after a shallow copy.  Subclasses may add functionality,
	 * but they should always do 'super.postCopy()' first.
	 * 
	 * @return sra.smalltalk.StObject
	 */
	public StObject postCopy() {
		super.postCopy();
		localCoordinateSystems = null;

		if (outerCoordinateSystem != null) {
			outerCoordinateSystem.privateAddLocalCoordinateSystem_(this);
		}

		return this;
	}

	/**
	 * Add aCoordinateSystem as a local coordinate system of the receiver.
	 * 
	 * @param aCoordinateSystem
	 *        sra.jun.geometry.coordinate.Jun3dCoordinateSystem
	 */
	public void privateAddLocalCoordinateSystem_(Jun3dCoordinateSystem aCoordinateSystem) {
		aCoordinateSystem.outerCoordinateSystem = this;

		if (localCoordinateSystems == null) {
			localCoordinateSystems = new Vector();
		}

		localCoordinateSystems.addElement(aCoordinateSystem);
	}

	/**
	 * Answer the transformation.
	 * 
	 * @return sra.jun.geometry.transformations.JunTransformation
	 */
	public JunTransformation transformation() {
		if (transformation == null) {
			this.computeTransformation();
		}

		return transformation;
	}

	/**
	 * Set the transformation.
	 * 
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public void transformation_(Jun3dTransformation aTransformation) {
		if (transformation != aTransformation) {
			transformation = aTransformation;
			invTransformation = null;
			this.clearGlobalTransformation();
			this.clearOriginAndVectors();
		}
	}

	/**
	 * Clear the cached global transformations.
	 */
	protected void clearGlobalTransformation() {
		if ((globalTransformation != null) || (globalInvTransformation != null)) {
			globalTransformation = null;
			globalInvTransformation = null;

			if (localCoordinateSystems != null) {
				for (int i = 0; i < localCoordinateSystems.size(); i++) {
					((Jun3dCoordinateSystem) localCoordinateSystems.elementAt(i)).clearGlobalTransformation();
				}
			}
		}
	}

	/**
	 * Clear the cached origin and vectors.
	 */
	protected void clearOriginAndVectors() {
		origin = null;
		xUnitVector = null;
		yUnitVector = null;
		zUnitVector = null;
	}

	/**
	 * Clear the cached transformations.
	 */
	protected void clearTransformations() {
		transformation = null;
		invTransformation = null;
		this.clearGlobalTransformation();
	}
}
