package sra.jun.geometry.coordinate;

import sra.smalltalk.*;
import sra.jun.geometry.abstracts.*;
import sra.jun.geometry.transformations.*;

/**
 * JunCoordinateSystem class
 * 
 *  @author    nisinaka
 *  @created   1998/09/28 (by nisinaka)
 *  @updated   2000/01/06 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCoordinateSystem.java,v 6.0 2003/10/28 10:52:06 nisinaka Exp $
 */
public abstract class JunCoordinateSystem extends JunGeometry {

	/**
	 * Compute the transformation and assign it to the instance variable.
	 */
	public void computeTransformation() {
		throw SmalltalkException.SubclassResponsibility();
	}

	/**
	 * Answer the inverse of global transformation.
	 * 
	 * @return sra.jun.geometry.transformations.JunTransformation
	 */
	public JunTransformation globalInvTransformation() {
		throw SmalltalkException.SubclassResponsibility();
	}

	/**
	 * Answer the JunGeometry object interpreted by this JunCoordinateSystem.
	 * 
	 * @param aJunGeometry sra.jun.geometry.abstracts.JunGeometry
	 * 
	 * @return sra.jun.geometry.abstracts.JunGeometry
	 */
	public JunGeometry interprete_(JunGeometry aJunGeometry) {
		return aJunGeometry.interpretedBy_(this);
	}

	/**
	 * Answer the transformation.
	 * 
	 * @return sra.jun.geometry.transformations.JunTransformation
	 */
	public JunTransformation transformation() {
		throw SmalltalkException.SubclassResponsibility();
	}

	/**
	 * Clear the cached origin and vectors.
	 */
	protected void clearOriginAndVectors() {
		throw SmalltalkException.SubclassResponsibility();
	}
}
