package sra.jun.geometry.coordinate;

import sra.jun.geometry.abstracts.*;

/**
 * JunLocalGeometry class
 * 
 *  @author    nisinaka
 *  @created   1998/09/28 (by nisinaka)
 *  @updated   2000/01/06 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunLocalGeometry.java,v 6.0 2003/10/28 10:52:06 nisinaka Exp $
 */
public class JunLocalGeometry extends JunGeometry {
	protected JunGeometry geometry = null;
	protected JunCoordinateSystem coordinateSystem = null;

	/**
	 * Create an local geometry with a specified geometry and a coordinate
	 * system.
	 * 
	 * @param aClass java.lang.Class
	 * @param aGeometry sra.jun.geometry.abstracts.JunGeometry
	 * @param aCoordinateSystem sra.jun.geometry.coordinate.JunCoordinateSystem
	 * 
	 * @return sra.jun.geometry.coordinate.JunLocalGeometry
	 */
	public static JunLocalGeometry Geometry_coordinateSystem_(Class aClass, JunGeometry aGeometry, JunCoordinateSystem aCoordinateSystem) {
		JunLocalGeometry coordinateGeometry = (JunLocalGeometry) _New(aClass);
		coordinateGeometry.geometry = aGeometry;
		coordinateGeometry.coordinateSystem = aCoordinateSystem;

		return coordinateGeometry;
	}

	/**
	 * Convert this as a global geometry.
	 * 
	 * @return sra.jun.geometry.abstracts.JunGeometry
	 * 
	 * @throws sra.smalltalk.SmalltalkException DOCUMENT ME!
	 */
	public JunGeometry asGlobalGeometry() {
		if (coordinateSystem == null) {
			return geometry;
		}

		try {
			return (JunGeometry) geometry.perform_with_("transform_", coordinateSystem.transformation());
		} catch (Exception e) {
			throw new sra.smalltalk.SmalltalkException(e);
		}
	}

	/**
	 * Convert this as a local geometry in aJunCoordinateSystem.
	 * 
	 * @param aCoordinateSystem sra.jun.geometry.coordinate.JunCoordinateSystem
	 * 
	 * @return sra.jun.geometry.coordinate.JunLocalGeometry
	 */
	public JunLocalGeometry asLocalGeometryIn_(JunCoordinateSystem aCoordinateSystem) {
		if (this.coordinateSystem.equals(aCoordinateSystem)) {
			return this;
		}

		return this.asGlobalGeometry().asLocalGeometryIn_(aCoordinateSystem);
	}

	/**
	 * Answer my coordinate system.
	 * 
	 * @return sra.jun.geometry.coordinate.JunCoordinateSystem
	 */
	public JunCoordinateSystem coordinateSystem() {
		return this.coordinateSystem;
	}

	/**
	 * Answer my geometry.
	 * 
	 * @return sra.jun.geometry.abstracts.JunGeometry
	 */
	public JunGeometry geometry() {
		return this.geometry;
	}

	/**
	 * Answer true if this is a kind of 2D geometry object, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean is2d() {
		return this.geometry.is2d();
	}

	/**
	 * Answer true if this is a kind of 3D geometry object, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean is3d() {
		return this.geometry.is3d();
	}

	/**
	 * Answer true if this is a kind of global geometry object, otherwise
	 * false.
	 * 
	 * @return boolean
	 */
	public boolean isGlobalGeometry() {
		return false;
	}
}
