package sra.jun.geometry.curves;

import sra.jun.geometry.abstracts.*;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.geometry.transformations.JunTransformation;
import sra.jun.opengl.objects.JunOpenGL3dNurbsCurve;
import sra.jun.opengl.objects.JunOpenGL3dObject;
import sra.smalltalk.StObject;

/**
 * JunNurbsCurve class
 * 
 *  @author    nisinaka
 *  @created   1998/09/29 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunNurbsCurve.java,v 6.0 2003/10/28 10:52:06 nisinaka Exp $
 */
public class JunNurbsCurve extends JunCurve {
	/** An array of control points. */
	protected Jun3dPoint[] controlPoints = null;

	/** An array of weights. */
	protected double[] weights;

	/** An array of knots. */
	protected double[] knotVector;

	/**
	 * Create a new JunNurbsCurve and initialize it.
	 * 
	 * @param points sra.jun.geometry.basic.Jun3dPoint[]
	 * 
	 * @return sra.jun.geometry.curves.JunNurbsCurve
	 */
	public static final JunNurbsCurve BezierControlPoints_(Jun3dPoint[] points) {
		return JunNurbsCurve.BezierControlPoints_(JunNurbsCurve.class, points);
	}

	/**
	 * Create a new JunNurbsCurve and initialize it.
	 * 
	 * @param aClass java.lang.Class
	 * @param points sra.jun.geometry.basic.Jun3dPoint[]
	 * 
	 * @return sra.jun.geometry.curves.JunNurbsCurve
	 */
	public static JunNurbsCurve BezierControlPoints_(Class aClass, Jun3dPoint[] points) {
		int numberOfPoints = points.length;
		double[] knotVector = new double[numberOfPoints * 2];

		for (int i = 0; i < numberOfPoints; i++) {
			knotVector[i] = 0;
			knotVector[i + numberOfPoints] = 1;
		}

		return BSplineControlPoints_knotVector_(aClass, points, knotVector);
	}

	/**
	 * Create a new JunNurbsCurve and initialize it.
	 * 
	 * @param points sra.jun.geometry.basic.Jun3dPoint[]
	 * @param weights double[]
	 * 
	 * @return sra.jun.geometry.curves.JunNurbsCurve
	 */
	public static final JunNurbsCurve BezierControlPoints_weights_(Jun3dPoint[] points, double[] weights) {
		return JunNurbsCurve.BezierControlPoints_weights_(JunNurbsCurve.class, points, weights);
	}

	/**
	 * Create a new JunNurbsCurve and initialize it.
	 * 
	 * @param aClass java.lang.Class
	 * @param points sra.jun.geometry.basic.Jun3dPoint[]
	 * @param weights double[]
	 * 
	 * @return sra.jun.geometry.curves.JunNurbsCurve
	 */
	public static JunNurbsCurve BezierControlPoints_weights_(Class aClass, Jun3dPoint[] points, double[] weights) {
		int numberOfPoints = points.length;
		double[] knotVector = new double[numberOfPoints * 2];

		for (int i = 0; i < numberOfPoints; i++) {
			knotVector[i] = 0;
			knotVector[i + numberOfPoints] = 1;
		}

		return ControlPoints_weights_knotVector_(aClass, points, weights, knotVector);
	}

	/**
	 * Create a new JunNurbsCurve and initialize it.
	 * 
	 * @param aClass java.lang.Class
	 * @param anArrayOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @param anArrayOfNumbers double[]
	 * 
	 * @return sra.jun.geometry.curves.JunNurbsCurve
	 */
	public static JunNurbsCurve BSplineControlPoints_knotVector_(Class aClass, Jun3dPoint[] anArrayOfPoints, double[] anArrayOfNumbers) {
		int numberOfPoints = anArrayOfPoints.length;
		double[] weights = new double[numberOfPoints];

		for (int i = 0; i < numberOfPoints; i++) {
			weights[i] = 1.0;
		}

		return ControlPoints_weights_knotVector_(aClass, anArrayOfPoints, weights, anArrayOfNumbers);
	}

	/**
	 * Create a new JunNurbsCurve and initialize it.
	 * 
	 * @param points sra.jun.geometry.basic.Jun3dPoint[]
	 * @param weights double[]
	 * @param knotVector double[]
	 * 
	 * @return sra.jun.geometry.curves.JunNurbsCurve
	 */
	public static JunNurbsCurve ControlPoints_weights_knotVector_(Jun3dPoint[] points, double[] weights, double[] knotVector) {
		JunNurbsCurve aNurbsCurve = new JunNurbsCurve();
		aNurbsCurve.controlPoints_weights_knotVector_(points, weights, knotVector);

		return aNurbsCurve;
	}

	/**
	 * Create a new JunNurbsCurve and initialize it.
	 * 
	 * @param aClass java.lang.Class
	 * @param points sra.jun.geometry.basic.Jun3dPoint[]
	 * @param weights double[]
	 * @param knotVector double[]
	 * 
	 * @return sra.jun.geometry.curves.JunNurbsCurve
	 */
	public static JunNurbsCurve ControlPoints_weights_knotVector_(Class aClass, Jun3dPoint[] points, double[] weights, double[] knotVector) {
		JunNurbsCurve aNurbsCurve = (JunNurbsCurve) _New(aClass);
		aNurbsCurve.controlPoints_weights_knotVector_(points, weights, knotVector);

		return aNurbsCurve;
	}

	/**
	 * Convert the receiver as a JunOpenGL3dNurbsCurve.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dNurbsCurve
	 */
	public JunOpenGL3dNurbsCurve asJunOpenGL3dNurbsCurve() {
		//return JunOpenGL3dNurbsCurve.ControlPoints_weights_knotVector_(controlPoints.copy(), weights.copy(), knotVector.copy());
		Jun3dPoint[] controlPointsCopy = new Jun3dPoint[controlPoints.length];

		for (int i = 0; i < controlPoints.length; i++) {
			controlPointsCopy[i] = controlPoints[i];
		}

		double[] weightsCopy = new double[weights.length];

		for (int i = 0; i < weights.length; i++) {
			weightsCopy[i] = weights[i];
		}

		double[] knotVectorCopy = new double[knotVector.length];

		for (int i = 0; i < knotVector.length; i++) {
			knotVectorCopy[i] = knotVector[i];
		}

		return JunOpenGL3dNurbsCurve.ControlPoints_weights_knotVector_(controlPointsCopy, weightsCopy, knotVectorCopy);
	}

	/**
	 * Convert the receiver as a JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		return this.asJunOpenGL3dNurbsCurve();
	}

	/**
	 * Convert the receiver as a nurbs curve.
	 * 
	 * @return sra.jun.geometry.curves.JunNurbsCurve
	 */
	public JunNurbsCurve asNurbsCurve() {
		return this;
	}

	/**
	 * Answer the control point at the specified index.
	 * 
	 * @param index int
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint controlPointAt_(int index) {
		return controlPoints[index];
	}

	/**
	 * Put a Jun3dPoint as a control point at the specified index.
	 * 
	 * @param index int
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public void controlPointAt_put_(int index, Jun3dPoint aPoint) {
		controlPoints[index] = aPoint;
	}

	/**
	 * Answer the array of the control points.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint[]
	 */
	public Jun3dPoint[] controlPoints() {
		return controlPoints;
	}

	/**
	 * Set the array of the control points.
	 * 
	 * @param anArray sra.jun.geometry.basic.Jun3dPoint[]
	 */
	public void controlPoints_(Jun3dPoint[] anArray) {
		controlPoints = anArray;
	}

	/**
	 * Answer the number of the control points.
	 * 
	 * @return int
	 */
	public int controlPointSize() {
		return controlPoints.length;
	}

	/**
	 * Answer the array that contains knots.
	 * 
	 * @return double[]
	 */
	public double[] knotVector() {
		return knotVector;
	}

	/**
	 * Answer the order number.
	 * 
	 * @return int
	 */
	public int order() {
		return knotVector.length - controlPoints.length;
	}

	/**
	 * Do an extra copy of the receiver.
	 * 
	 * @return sra.smalltalk.StObject
	 */
	public StObject postCopy() {
		super.postCopy();

		int size = controlPoints.length;
		Jun3dPoint[] newControlPoints = new Jun3dPoint[size];
		System.arraycopy(controlPoints, 0, newControlPoints, 0, size);
		controlPoints = newControlPoints;
		size = weights.length;

		double[] newWeights = new double[size];
		System.arraycopy(weights, 0, newWeights, 0, size);
		weights = newWeights;
		size = weights.length;

		double[] newKnotVector = new double[size];
		System.arraycopy(knotVector, 0, newKnotVector, 0, size);
		knotVector = newKnotVector;

		return this;
	}

	/**
	 * Answer the reversed JunNurbsCurve.
	 * 
	 * @return sra.jun.geometry.curves.JunNurbsCurve
	 */
	public JunNurbsCurve reversed() {
		int size = controlPoints.length;
		Jun3dPoint[] reversedControlPoints = new Jun3dPoint[size];

		for (int i = 0; i < size; i++) {
			reversedControlPoints[i] = controlPoints[size - i - 1];
		}

		size = weights.length;

		double[] reversedWeights = new double[size];

		for (int i = 0; i < size; i++) {
			reversedWeights[i] = weights[size - i - 1];
		}

		size = knotVector.length;

		double[] reversedKnotVector = new double[size];

		for (int i = 0; i < size; i++) {
			reversedKnotVector[i] = knotVector[size - 1] - knotVector[size - i];
		}

		return ControlPoints_weights_knotVector_(this.getClass(), reversedControlPoints, reversedWeights, reversedKnotVector);
	}

	/**
	 * Apply a transformation 'aJunTransformation' to the receiver.
	 * 
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.JunTransformation
	 * 
	 * @return sra.jun.geometry.abstracts.JunGeometry
	 */
	public JunGeometry transform_(JunTransformation aTransformation) {
		JunNurbsCurve theNurbsCurve = (JunNurbsCurve) this.copy();

		for (int i = 0; i < theNurbsCurve.controlPointSize(); i++) {
			Jun3dPoint aPoint = theNurbsCurve.controlPointAt_(i);
			Jun3dPoint transformedPoint = (Jun3dPoint) aPoint.transform_(aTransformation);
			theNurbsCurve.controlPointAt_put_(i, transformedPoint);
		}

		return theNurbsCurve;
	}

	/**
	 * Answer the weight at the specified index.
	 * 
	 * @param index int
	 * 
	 * @return double
	 */
	public double weightAt_(int index) {
		return weights[index];
	}

	/**
	 * Answer the array of the weights.
	 * 
	 * @return double[]
	 */
	public double[] weights() {
		return weights;
	}

	/**
	 * Set the array of the weights.
	 * 
	 * @param anArray double[]
	 */
	public void weights_(double[] anArray) {
		weights = anArray;
	}

	/**
	 * Set the instance variables.
	 * 
	 * @param points sra.jun.geometry.basic.Jun3dPoint[]
	 * @param weights double[]
	 * @param knotVector double[]
	 */
	private void controlPoints_weights_knotVector_(Jun3dPoint[] points, double[] weights, double[] knotVector) {
		this.controlPoints = points;
		this.weights = weights;
		this.knotVector = knotVector;
	}
}
