package sra.jun.geometry.surfaces;

import java.io.*;
import java.util.Vector;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.curves.*;
import sra.jun.opengl.objects.*;
import sra.smalltalk.*;

/**
 * JunPolygon class
 * 
 *  @author    nisinaka
 *  @created   1998/11/09 (by nisinaka)
 *  @updated   1999/08/04 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPolygon.java,v 6.0 2003/10/28 10:52:06 nisinaka Exp $
 */
public class JunPolygon extends JunPlane {
	/** The accuracy number for JunPolygon. */
	public static final double ACCURACY = 1.0e-8d;

	/** The parameter polygon. */
	protected Jun2dPolygon parameterPolygon = null;

	/** The bounding box. */
	protected Jun3dBoundingBox boundingBox = null;

	/** The normal vecotr. */
	protected Jun3dPoint normalVector = null;

	/**
	 * Create a new instance of JunPolygon with three Jun3dPoints.
	 * 
	 * @param aPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param aPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param aPoint3 sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunPolygon _On_on_on_(Jun3dPoint aPoint1, Jun3dPoint aPoint2, Jun3dPoint aPoint3) {
		if (aPoint1.equals(aPoint2)) {
			return null;
		}

		if (aPoint1.equals(aPoint3)) {
			return null;
		}

		if (aPoint2.equals(aPoint3)) {
			return null;
		}

		JunPolygon polygon = new JunPolygon();
		polygon._initialize(aPoint1, aPoint2, aPoint3);

		return polygon;
	}

	/**
	 * Create a new instance of JunPolygon with three Jun3dPoints.
	 * 
	 * @param aPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param aPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param aPoint3 sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunPolygon On_on_on_(Jun3dPoint aPoint1, Jun3dPoint aPoint2, Jun3dPoint aPoint3) {
		return Vertexes_(new Jun3dPoint[] { aPoint1, aPoint2, aPoint3 });
	}

	/**
	 * Create a new instance of JunPolygon.
	 * 
	 * @param aJun3dPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint3 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun2dPolygon sra.jun.geometry.surfaces.Jun2dPolygon
	 * 
	 * @return sra.jun.geometry.surfaces.JunPolygon
	 */
	public static JunPolygon Origin_uVector_vVector_parameterPolygon_(Jun3dPoint aJun3dPoint1, Jun3dPoint aJun3dPoint2, Jun3dPoint aJun3dPoint3, Jun2dPolygon aJun2dPolygon) {
		Jun3dPoint wVector = (Jun3dPoint) aJun3dPoint2.vectorProduct_(aJun3dPoint3).unitVector();
		Jun3dPoint vVector = (Jun3dPoint) wVector.vectorProduct_(aJun3dPoint2).unitVector();
		JunPolygon polygon = JunPolygon._On_on_on_(aJun3dPoint1, aJun3dPoint1.plus_(aJun3dPoint2), aJun3dPoint1.plus_(vVector));

		if (polygon == null) {
			return null;
		}

		polygon.setParameterPolygon_(aJun2dPolygon);

		return polygon;
	}

	/**
	 * Create a new instance of JunPolygon.
	 * 
	 * @param aJun3dPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint3 sra.jun.geometry.basic.Jun3dPoint
	 * @param anArrayOfJun3dPoint sra.jun.geometry.basic.Jun3dPoint[]
	 * 
	 * @return sra.jun.geometry.surfaces.JunPolygon
	 */
	public static JunPolygon Origin_uVector_vVector_points_(Jun3dPoint aJun3dPoint1, Jun3dPoint aJun3dPoint2, Jun3dPoint aJun3dPoint3, Jun3dPoint[] anArrayOfJun3dPoint) {
		Jun3dPoint wVector = aJun3dPoint2.vectorProduct_(aJun3dPoint3).unitVector();
		Jun3dPoint vVector = wVector.vectorProduct_(aJun3dPoint2).unitVector();
		JunPolygon polygon = JunPolygon._On_on_on_(aJun3dPoint1, aJun3dPoint1.plus_(aJun3dPoint2), aJun3dPoint1.plus_(vVector));

		if (polygon == null) {
			return null;
		}

		Jun2dPoint[] parameterPoints = new Jun2dPoint[anArrayOfJun3dPoint.length];

		for (int i = 0; i < anArrayOfJun3dPoint.length; i++) {
			parameterPoints[i] = polygon.parameterPointAtPoint_(anArrayOfJun3dPoint[i]);
		}

		polygon.setParameterPolygon_(new Jun2dPolygon(parameterPoints));

		return polygon;
	}

	/**
	 * Create a new instance of JunPolygon with an array of Jun3dPoint. This
	 * corresponds to the instance creation method "vertexes:".
	 * 
	 * @param anArrayOfJun3dPoint sra.jun.geometry.basic.Jun3dPoint[]
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunPolygon Points_(Jun3dPoint[] anArrayOfJun3dPoint) {
		return Vertexes_(anArrayOfJun3dPoint);
	}

	/**
	 * Create a new instance of JunPolygon with an array of Jun3dPoint.
	 * 
	 * @param anArrayOfJun3dPoint sra.jun.geometry.basic.Jun3dPoint[]
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunPolygon Vertexes_(Jun3dPoint[] anArrayOfJun3dPoint) {
		Jun3dPoint[] points = anArrayOfJun3dPoint;
		int size = points.length;

		if (size < 3) {
			return null;
		}

		Jun3dPoint origin = points[0];
		int uIndex = 0;
		double maxDistance = Double.NEGATIVE_INFINITY;

		for (int i = 1; i < points.length; i++) {
			double distance = points[i].distance_(origin);

			if (maxDistance < distance) {
				uIndex = i;
				maxDistance = distance;
			}
		}

		Jun3dPoint uPoint = points[uIndex];
		Jun3dPoint uVector = (Jun3dPoint) uPoint.minus_(origin).unitVector();
		Jun3dPoint wVector = Jun3dPoint.Zero();

		for (int i = 0; i < size; i++) {
			Jun3dPoint point1 = points[i];
			Jun3dPoint point2 = points[(i + 1) % size];
			Jun3dPoint point3 = points[(i + 2) % size];
			wVector = (Jun3dPoint) wVector.plus_(((Jun3dPoint) point2.minus_(point1)).vectorProduct_((Jun3dPoint) point3.minus_(point2)));
		}

		Jun3dPoint vVector = (Jun3dPoint) wVector.vectorProduct_(uVector).unitVector();

		return Origin_uVector_vVector_points_(origin, uVector, vVector, points);
	}

	/**
	 * Create a new instance of JunPolygon with an Vector of Jun3dPoint.
	 * 
	 * @param aCollectionOfJun3dPoint java.util.Vector of
	 *        sra.jun.geometry.basic.Jun3dPoint[]
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunPolygon Vertexes_(Vector aCollectionOfJun3dPoint) {
		Jun3dPoint[] anArrayOfJun3dPoint = new Jun3dPoint[aCollectionOfJun3dPoint.size()];
		aCollectionOfJun3dPoint.copyInto(anArrayOfJun3dPoint);

		return Vertexes_(anArrayOfJun3dPoint);
	}

	/**
	 * Create a new instance of JunPolygon with an array of Jun3dPoint.
	 * 
	 * @param anArrayOfJun3dPoint sra.jun.geometry.basic.Jun3dPoint[]
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunPolygon Vertices_(Jun3dPoint[] anArrayOfJun3dPoint) {
		return Vertexes_(anArrayOfJun3dPoint);
	}

	/**
	 * Create a new instance of JunPolygon with three Jun3dPoints.
	 * 
	 * @param aPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param aPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param aPoint3 sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunPolygon With_with_with_(Jun3dPoint aPoint1, Jun3dPoint aPoint2, Jun3dPoint aPoint3) {
		return Vertexes_(new Jun3dPoint[] { aPoint1, aPoint2, aPoint3 });
	}

	/**
	 * Answer my area.
	 * 
	 * @return double
	 */
	public double area() {
		return this.parameterPolygon().area() * this.uVector().length() * this.vVector().length();
	}

	/**
	 * Convert the receiver as an array of the convex polygons.
	 * 
	 * @return sra.jun.geometry.surfaces.JunPolygon[]
	 */
	public JunPolygon[] asArrayOfConvexPolygons() {
		final Vector convexPolygons = new Vector();
		this.asConvexPolygonsDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				convexPolygons.addElement(anObject);

				return null;
			}
		});

		JunPolygon[] anArrayOfConvexPolygons = new JunPolygon[convexPolygons.size()];
		convexPolygons.copyInto(anArrayOfConvexPolygons);

		return anArrayOfConvexPolygons;
	}

	/**
	 * Convert the receiver as an array of triangles.
	 * 
	 * @return sra.jun.geometry.surfaces.JunPolygon[]
	 */
	public JunPolygon[] asArrayOfTriangles() {
		int numberOfPoints = this.numberOfPoints();

		if (numberOfPoints == 3) {
			JunPolygon[] triangles = { this };

			return triangles;
		}

		JunPolygon[] convexPolygons = this.asArrayOfConvexPolygons();
		Vector collection = new Vector(numberOfPoints - 2);

		for (int i = 0; i < convexPolygons.length; i++) {
			JunPolygon convexPolygon = convexPolygons[i];

			for (int index = 1; index < (numberOfPoints - 1); index++) {
				Jun3dPoint[] points = new Jun3dPoint[3];
				points[0] = convexPolygon.pointAt_(0);
				points[1] = convexPolygon.pointAt_(index);
				points[2] = convexPolygon.pointAt_(index + 1);

				JunPolygon triangle = JunPolygon.Vertexes_(points);
				collection.addElement(triangle);
			}
		}

		JunPolygon[] triangles = new JunPolygon[collection.size()];
		collection.copyInto(triangles);

		return triangles;
	}

	/**
	 * Convert myself as a convex polygons, enumerate each polygons, and
	 * evaluate the block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object asConvexPolygonsDo_(final StBlockClosure aBlock) {
		final JunPolygon self = this;

		return this.parameterPolygon().asConvexPolygonsDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				Jun2dPolygon pPolygon = (Jun2dPolygon) anObject;
				JunPolygon convexPolygon = (JunPolygon) self.copy();
				convexPolygon.setParameterPolygon_(pPolygon);

				return aBlock.value_(convexPolygon);
			}
		});
	}

	/**
	 * Conver the receiver as a JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		if (this.isConvex()) {
			return JunOpenGL3dPolygon.Vertexes_(this.points());
		} else {
			JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
			JunPolygon[] polygons = this.asArrayOfConvexPolygons();

			for (int i = 0; i < polygons.length; i++) {
				compoundObject.add_(JunOpenGL3dPolygon.Vertexes_(polygons[i].points()));
			}

			return compoundObject;
		}
	}

	/**
	 * Answer the bounding box of the receiver.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dBoundingBox
	 */
	public Jun3dBoundingBox boundingBox() {
		if (boundingBox == null) {
			boundingBox = this.preferredBoundingBox();
		}

		return boundingBox;
	}

	/**
	 * Answer true if the receiver contains the Jun3dLine as a line segment,
	 * otherwise false.
	 * 
	 * @param aLine sra.jun.geometry.curves.Jun3dLine
	 * 
	 * @return boolean
	 */
	public boolean containsLineSegment_(Jun3dLine aLine) {
		return this.containsLineSegmentFrom_to_((Jun3dPoint) aLine.from(), (Jun3dPoint) aLine.to());
	}

	/**
	 * Answer true if the receiver contains the Jun3dLine as a line segment,
	 * otherwise false.
	 * 
	 * @param aPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param aPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return boolean
	 */
	public boolean containsLineSegmentFrom_to_(Jun3dPoint aPoint1, Jun3dPoint aPoint2) {
		if ((this.containsPoint_(aPoint1) && this.containsPoint_(aPoint2)) == false) {
			return false;
		}

		return this.parameterPolygon().absContainsLineSegmentFrom_to_(this.parameterPointAtPoint_(aPoint1), this.parameterPointAtPoint_(aPoint2));
	}

	/**
	 * Answer true if the receiver contains the Jun3dPoint, otherwise false.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return boolean
	 */
	public boolean containsPoint_(Jun3dPoint aPoint) {
		if (!super.containsPoint_(aPoint)) {
			return false;
		}

		return this.parameterPolygon().absContainsPoint_(this.parameterPointAtPoint_(aPoint));
	}

	/**
	 * Answer the Point which intersects with the Line.
	 * 
	 * @param aLine sra.jun.geometry.curves.Jun3dLine
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint intersectingPointWithLine_(Jun3dLine aLine) {
		Jun3dPoint intersectingPoint = super.intersectingPointWithLine_(aLine);

		if (intersectingPoint == null) {
			return null;
		}

		if (this.containsPoint_(intersectingPoint)) {
			return intersectingPoint;
		}

		return null;
	}

	/**
	 * Answer the intersecting point with the line segment Jun3dLine.
	 * 
	 * @param aLine sra.jun.geometry.curves.Jun3dLine
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint intersectingPointWithLineSegment_(Jun3dLine aLine) {
		if (this.boundingBox().intersectsOrTouches_(aLine.boundingBox())) {
			Jun3dPoint intersectingPoint = super.intersectingPointWithLineSegment_(aLine);

			if (intersectingPoint == null) {
				return null;
			}

			if (this.containsPoint_(intersectingPoint)) {
				return intersectingPoint;
			} else {
				return null;
			}
		} else {
			return null;
		}
	}

	/**
	 * Answer true if the receiver is consistent, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean isConsistent() {
		return this.parameterPolygon().isConsistent();
	}

	/**
	 * Answer true if the receiver is convex, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean isConvex() {
		return this.parameterPolygon().absIsConvex();
	}

	/**
	 * Enumerate every line segments and evaluage the StBlockClosre.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void lineSegmentsDo_(StBlockClosure aBlock) {
		int numberOfPoints = this.numberOfPoints();

		for (int index = 0; index < numberOfPoints; index++) {
			aBlock.value_(new Jun3dLine(this.pointAt_(index), this.pointAt_((index + 1) % numberOfPoints)));
		}
	}

	/**
	 * Answer the nearest point on the receiver from the Jun3dPoint.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint nearestPointFromPoint_(Jun3dPoint aPoint) {
		return this.pointAtParameterPoint_(this.parameterPolygon().asPositivePolygon().nearestPointFromPoint_(this.parameterPointAtPoint_(aPoint)));
	}

	/**
	 * Answer my normal vector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint normalVector() {
		if (normalVector == null) {
			normalVector = this.preferredNormalVector();
		}

		return normalVector;
	}

	/**
	 * Answer the total number of points.
	 * 
	 * @return int
	 */
	public int numberOfPoints() {
		return this.pointsSize();
	}

	/**
	 * Answer my origin point.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint origin() {
		return p1;
	}

	/**
	 * Enumerate every parameter line segments and evaluage the StBlockClosre.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void parameterLineSegmentsDo_(StBlockClosure aBlock) {
		int numberOfPoints = this.numberOfPoints();

		for (int index = 0; index < numberOfPoints; index++) {
			aBlock.value_(new Jun2dLine(this.parameterPointAt_(index), this.parameterPointAt_((index + 1) % numberOfPoints)));
		}
	}

	/**
	 * Answer the parameter at the specified index as a Jun2dPoint.
	 * 
	 * @param anInteger int
	 * 
	 * @return sra.jun.geometry.basic.Jun2dPoint
	 */
	public Jun2dPoint parameterPointAt_(int anInteger) {
		return this.parameterPolygon().pointAt_(anInteger);
	}

	/**
	 * Answer the parameter at the specified point as a Jun2dPoint.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.geometry.basic.Jun2dPoint
	 */
	public Jun2dPoint parameterPointAtPoint_(Jun3dPoint aPoint) {
		Jun3dPoint thePoint = (Jun3dPoint) aPoint.minus_(p1);
		Jun3dPoint uVector = this.uVector();
		Jun3dPoint vVector = this.vVector();
		double u = uVector.dotProduct_(thePoint) / uVector.dotProduct_(uVector);
		double v = vVector.dotProduct_(thePoint) / vVector.dotProduct_(vVector);

		return new Jun2dPoint(u, v);
	}

	/**
	 * Answer my parameter polygon.
	 * 
	 * @return sra.jun.geometry.surfaces.Jun2dPolygon
	 */
	public final Jun2dPolygon parameterPolygon() {
		return parameterPolygon;
	}

	/**
	 * Answer the Jun3dPoint specified with the index number.
	 * 
	 * @param anInteger int
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint pointAt_(int anInteger) {
		return this.pointAtParameterPoint_(this.parameterPolygon().pointAt_(anInteger));
	}

	/**
	 * Answer the array of all points.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint[]
	 */
	public final Jun3dPoint[] points() {
		int size = this.pointsSize();
		Jun3dPoint[] points = new Jun3dPoint[size];

		for (int i = 0; i < size; i++) {
			points[i] = this.pointAt_(i);
		}

		return points;
	}

	/**
	 * Enumerate every points and evaluate the block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object pointsDo_(StBlockClosure aBlock) {
		for (int i = 0; i < this.pointsSize(); i++) {
			Object result = aBlock.value_(this.pointAt_(i));

			if (result != null) {
				return result;
			}
		}

		return null;
	}

	/**
	 * Answer the number of points.
	 * 
	 * @return int
	 */
	public int pointsSize() {
		return this.parameterPolygon().pointsSize();
	}

	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * 
	 * @throws IOException DOCUMENT ME!
	 */
	public void printOn_(Writer aWriter) throws IOException {
		aWriter.write("JunPolygon (");

		Jun3dPoint[] points = this.points();

		for (int i = 0; i < points.length; i++) {
			points[i].printOn_(aWriter);
			aWriter.write(' ');
		}

		aWriter.write(")");
	}

	/**
	 * Convert the receiver as reversed.
	 * 
	 * @return sra.jun.geometry.surfaces.JunPolygon
	 */
	public JunPolygon reversed() {
		Jun3dPoint[] points = this.points();
		int size = points.length;
		Jun3dPoint[] rPoints = new Jun3dPoint[size];

		for (int i = 0; i < size; i++) {
			rPoints[i] = points[size - i - 1];
		}

		return (JunPolygon) _New(this.getClass(), rPoints);
	}

	/**
	 * Answer the U vector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint uVector() {
		return (Jun3dPoint) p2.minus_(p1);
	}

	/**
	 * Answer the V vector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint vVector() {
		return (Jun3dPoint) p3.minus_(p1);
	}

	/**
	 * Answer the area of my parameter polygon.
	 * 
	 * @return double
	 */
	protected double parameterArea() {
		return this.parameterPolygon().area();
	}

	/**
	 * Answer the Jun3dPoint specified with the parameter point.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	private Jun3dPoint pointAtParameterPoint_(Jun2dPoint aPoint) {
		return this.pointAtU_v_(aPoint.x(), aPoint.y());
	}

	/**
	 * Answer the Jun3dPoint specified with "u" and "v".
	 * 
	 * @param u double
	 * @param v double
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	private Jun3dPoint pointAtU_v_(double u, double v) {
		return (Jun3dPoint) p1.plus_(this.uVector().multipliedBy_(u)).plus_(this.vVector().multipliedBy_(v));
	}

	/**
	 * Answer the preferable bounding box of the receiver.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dBoundingBox
	 */
	protected Jun3dBoundingBox preferredBoundingBox() {
		Jun3dPoint[] points = this.points();
		double minX = Double.POSITIVE_INFINITY;
		double minY = Double.POSITIVE_INFINITY;
		double minZ = Double.POSITIVE_INFINITY;
		double maxX = Double.NEGATIVE_INFINITY;
		double maxY = Double.NEGATIVE_INFINITY;
		double maxZ = Double.NEGATIVE_INFINITY;

		for (int i = 0; i < points.length; i++) {
			Jun3dPoint p = points[i];
			minX = Math.min(minX, p.x());
			minY = Math.min(minY, p.y());
			minZ = Math.min(minZ, p.z());
			maxX = Math.max(maxX, p.x());
			maxY = Math.max(maxY, p.y());
			maxZ = Math.max(maxZ, p.z());
		}

		return Jun3dBoundingBox.Origin_corner_(new Jun3dPoint(minX, minY, minZ), new Jun3dPoint(maxX, maxY, maxZ));
	}

	/**
	 * Answer my preferable normal vector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	protected Jun3dPoint preferredNormalVector() {
		Jun3dPoint vector = Jun3dPoint.Zero();
		int size = this.pointsSize();

		for (int i = 0; i < size; i++) {
			Jun3dPoint point1 = this.pointAt_(i);
			Jun3dPoint point2 = this.pointAt_((i + 1) % size);
			Jun3dPoint point3 = this.pointAt_((i + 2) % size);
			vector = (Jun3dPoint) vector.plus_(((Jun3dPoint) point2.minus_(point1)).vectorProduct_((Jun3dPoint) point3.minus_(point2)));
		}

		return (Jun3dPoint) vector.unitVector();
	}

	/**
	 * Set the Jun2dPolygon as my parameter polygon.
	 * 
	 * @param aJun2dPolygon sra.jun.geometry.surfaces.Jun2dPolygon
	 */
	protected void setParameterPolygon_(Jun2dPolygon aJun2dPolygon) {
		if (aJun2dPolygon.isPositive()) {
			parameterPolygon = aJun2dPolygon;
		} else {
			parameterPolygon = aJun2dPolygon.transposed();

			Jun3dPoint oldP2 = p2;
			Jun3dPoint oldP3 = p3;
			p2 = oldP3;
			p3 = oldP2;
		}
	}
}
