package sra.jun.geometry.surfaces;

import sra.jun.geometry.abstracts.JunSurface;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.geometry.curves.Jun3dLine;

/**
 * JunSphereSurface class
 * 
 *  @author    nisinaka
 *  @created   1998/11/13 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunSphereSurface.java,v 6.0 2003/10/28 10:52:06 nisinaka Exp $
 */
public class JunSphereSurface extends JunSurface {
	/** The center point. */
	protected Jun3dPoint center = null;

	/** The radius. */
	protected double radius = 0.0d;

	/**
	 * Create a new instance of JunSphereSurface and initialize it.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aNumber double
	 * 
	 * @return sra.jun.geometry.surfaces.JunSphereSurface
	 */
	public static JunSphereSurface Center_radius_(Jun3dPoint aPoint, double aNumber) {
		JunSphereSurface sphere = new JunSphereSurface();
		sphere.center_(aPoint);
		sphere.radius_(aNumber);

		return sphere;
	}

	/**
	 * Answer the center point of the receiver.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint center() {
		return center;
	}

	/**
	 * Set the center point of the receiver.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public void center_(Jun3dPoint aPoint) {
		center = aPoint;
	}

	/**
	 * Answer the cross points on the receiver with the specified Line.
	 * 
	 * @param aLine sra.jun.geometry.curves.Jun3dLine
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint[]
	 */
	public Jun3dPoint[] crossPointsWithLine_(Jun3dLine aLine) {
		Jun3dPoint nearestPointOnLine = aLine.nearestPointFromPoint_(center);
		double distance = center.distance_(nearestPointOnLine);

		if (distance > radius) {
			return new Jun3dPoint[0];
		}

		if ((Math.abs(distance - radius) - ACCURACY) < 0) {
			Jun3dPoint[] points = { nearestPointOnLine };

			return points;
		}

		double shift = Math.sqrt((radius * radius) - (distance * distance));
		Jun3dPoint shiftVector = (Jun3dPoint) aLine.to().minus_(aLine.from()).unitVector().multipliedBy_(shift);
		Jun3dPoint[] points = {(Jun3dPoint) nearestPointOnLine.plus_(shiftVector), (Jun3dPoint) nearestPointOnLine.minus_(shiftVector)};

		return points;
	}

	/**
	 * Answer the radius of the receiver.
	 * 
	 * @return double
	 */
	public double radius() {
		return radius;
	}

	/**
	 * Set the radius of the receiver.
	 * 
	 * @param aNumber double
	 */
	public void radius_(double aNumber) {
		radius = aNumber;
	}
}
