package sra.jun.goodies.bulletinboard;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import sra.smalltalk.*;

/**
 * JunBulletinBoardViewAwt class
 * 
 *  @author    nisinaka
 *  @created   2002/11/27 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunBulletinBoardViewAwt.java,v 6.0 2003/10/28 10:52:07 nisinaka Exp $
 */
public class JunBulletinBoardViewAwt extends StViewCanvas implements JunBulletinBoardView {

	protected int displaySpeed;
	protected Point displayPoint;

	/**
	 * Create a new instance of JunBulletinBoardViewAwt and initialize it.
	 *
	 * @param aBulletinBoard sra.jun.goodies.bulletinboard.JunBulletinBoard
	 */
	public JunBulletinBoardViewAwt(JunBulletinBoard aBulletinBoard) {
		super(aBulletinBoard);
		this.initialize();
	}

	/**
	 * Create a new instance of JunBulletinBoardViewAwt on the model and open it.
	 *
	 * @return sra.jun.goodies.bulletinboard.JunBulletinBoardViewAwt
	 * @param aModel sra.jun.goodies.bulletinboard.JunBulletinBoard
	 */
	public static JunBulletinBoardViewAwt OpenOn_(JunBulletinBoard aModel) {
		final JunBulletinBoardViewAwt aView = new JunBulletinBoardViewAwt(aModel) {
			public Dimension getMinimumSize() {
				return DefaultWindowSize;
			}
		};

		Panel aPanel = StApplicationModel._PanelWithBorder();
		aPanel.setLayout(new BorderLayout());
		aPanel.add(aView, BorderLayout.CENTER);

		Frame aFrame = new Frame(DefaultWindowLabel);
		aFrame.setLayout(new BorderLayout());
		aFrame.add(aPanel, BorderLayout.CENTER);
		aFrame.addWindowListener(new WindowAdapter() {
			public void windowOpened(WindowEvent e) {
				((JunBulletinBoardController) aView.controller()).resume();
			}

			public void windowClosing(WindowEvent e) {
				((JunBulletinBoardController) aView.controller()).terminate();
				e.getWindow().dispose();
			}
		});
		aFrame.pack();
		StApplicationModel._ShowAtMousePoint(aFrame);

		return aView;
	}

	/**
	 * Display the current content.
	 */
	public void display() {
		Graphics aGraphics = this.getGraphics();
		if (aGraphics == null) {
			return;
		}

		try {
			this.displayOn_(aGraphics);
		} finally {
			aGraphics.dispose();
		}
	}

	/**
	 * Paints this component.
	 * 
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayOn_(Graphics aGraphics) {
		if (this.isShowing() == false) {
			return;
		}

		BufferedImage anImage = this.getBulletinBoard().imageLocation_width_(this.displayPoint().x, this.getBounds().width);
		int y = (this.getBounds().height - anImage.getHeight()) / 2;
		aGraphics.drawImage(anImage, 0, y, null);

		Thread.yield();
	}

	/**
	 * Answer my current display point.
	 *
	 * @return java.awt.Point
	 */
	public Point displayPoint() {
		int modelWidth = this.getBulletinBoard().width();
		int startLocation = this.getBounds().width * -1;
		if (displayPoint == null) {
			displayPoint = new Point(startLocation, 0);
		} else {
			displayPoint.x += this.displaySpeed();
		}
		if (displayPoint.x > modelWidth) {
			displayPoint = new Point(startLocation, 0);
		}
		return displayPoint;
	}

	/**
	 * Answer the current display speed.
	 *
	 * @return int
	 */
	public int displaySpeed() {
		return displaySpeed;
	}

	/**
	 * Set the new display speed.
	 *
	 * @param newDisplaySpeed int
	 */
	public void displaySpeed_(int newDisplaySpeed) {
		displaySpeed = Math.max(1, Math.min(newDisplaySpeed, this.getBulletinBoard().width()));
		System.out.println("displaySpeed is " + displaySpeed);
	}

	/**
	 * Answer my model as a JunBulletinBoard.
	 *
	 * @return sra.jun.goodies.bulletinboard.JunBulletinBoard
	 */
	public JunBulletinBoard getBulletinBoard() {
		return (JunBulletinBoard) this.model();
	}

	/**
	 * Set the bounds of the view.
	 * Also needs to change the font.
	 *
	 * @param x The new <i>x</i>-coordinate of this component.
	 * @param y The new <i>y</i>-coordinate of this component.
	 * @param width The new <code>width</code> of this component.
	 * @param height The new <code>height</code> of this component.
	 * @see java.awt.Component#setBounds(int, int, int, int)
	 */
	public void setBounds(int x, int y, int width, int height) {
		super.setBounds(x, y, width, height);

		JunBulletinBoard aBulletinBoard = this.getBulletinBoard();
		Rectangle oldModelBounds = aBulletinBoard.bounds();
		aBulletinBoard.fontHeight_(Math.round(height * 0.8f));
		Rectangle newModelBounds = aBulletinBoard.bounds();
		if (displayPoint != null) {
			displayPoint.x = displayPoint.x * newModelBounds.width / oldModelBounds.width;
		}

		// this.displaySpeed_(displaySpeed * newModelBounds.width / oldModelBounds.width);
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setSize(DefaultWindowSize.width, JunBulletinBoard.DefaultBulletinHeight);
		this.setBackground(this.getBulletinBoard().backgroundColor());
	}

	/**
	 * Answer my default controller. Subclasses may override this.
	 * 
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new JunBulletinBoardController();
	}

	/**
	 * Initialize the receiver.
	 */
	protected void initialize() {
		super.initialize();

		displaySpeed = 2;
		displayPoint = null;
	}
}
