package sra.jun.goodies.bulletinboard;

import java.awt.*;
import sra.smalltalk.*;

/**
 * JunBulletinElement class
 * 
 *  @author    nisinaka
 *  @created   2002/11/27 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunBulletinElement.java,v 6.0 2003/10/28 10:52:07 nisinaka Exp $
 */
public class JunBulletinElement extends StObject {

	protected StDisplayable visualEntity;
	protected Rectangle visualRectangle;
	protected Color visualColor;

	/**
	 * Create a new instance of JunBulletinElement and initialize it with the specified entity object.
	 *
	 * @param anEntityObject java.lang.Object
	 */
	public JunBulletinElement(Object anEntityObject) {
		this();
		this.entity_(anEntityObject);
	}

	/**
	 * Create a new instance of JunBulletinElement and initialize it.
	 */
	protected JunBulletinElement() {
		this.initialize();
	}

	/**
	 * Answer the color of the receiver.
	 *
	 * @return java.awt.Color
	 */
	public Color color() {
		return visualColor;
	}

	/**
	 * Set the new color of the receiver.
	 *
	 * @param newColor java.awt.Color
	 */
	public void color_(Color newColor) {
		visualColor = newColor;
	}

	/**
	 * Answer the entity of the reciever.
	 *
	 * @return sra.smalltalk.StDisplayable
	 */
	public StDisplayable entity() {
		return visualEntity;
	}

	/**
	 * Set the specified object as an entity of the receiver.
	 *
	 * @param anObject java.lang.Object
	 */
	public void entity_(Object anObject) {
		if (anObject instanceof StDisplayable) {
			visualEntity = (StDisplayable) anObject;
		} else {
			String aString;
			if (anObject instanceof String) {
				aString = JunBulletinBoard.ConvertToFlatString_((String) anObject);
			} else {
				aString = JunBulletinBoard.ConvertToFlatString_(anObject.toString());
			}
			StComposedText composedText = new StComposedText(aString, this.font());
			visualEntity = composedText;
		}
		this.rectangle_(visualEntity.bounds());
	}

	/**
	 * Answer the font of the receiver.
	 *
	 * @return java.awt.Font
	 */
	public Font font() {
		if (this.entity() instanceof StComposedText) {
			return ((StComposedText) this.entity()).font();
		}

		return JunBulletinBoard.Font();
	}

	/**
	 * Set the new font of the receiver.
	 * Replacement of the style: method with a TextAttribute in Smalltalk version.
	 *
	 * @param newFont java.awt.Font
	 */
	public void font_(Font newFont) {
		if (this.entity() instanceof StComposedText) {
			StComposedText composedText = new StComposedText(((StComposedText) this.entity()).string(), newFont);
			visualEntity = composedText;
			this.rectangle_(visualEntity.bounds());
		}
	}

	/**
	 * Answer the height of the receiver.
	 *
	 * @return int
	 */
	public int height() {
		return this.rectangle().height;
	}

	/**
	 * Answer the bounds of the receiver.
	 *
	 * @return java.awt.Rectangle
	 */
	public Rectangle rectangle() {
		return visualRectangle;
	}

	/**
	 * Set the new bounds of the receiver.
	 *
	 * @param newRectangle java.awt.Rectangle
	 */
	public void rectangle_(Rectangle newRectangle) {
		visualRectangle = newRectangle;
	}

	/**
	 * Answer the width of the receiver.
	 *
	 * @return int
	 */
	public int width() {
		return this.rectangle().width;
	}

	/**
	 * Initialize the receiver.
	 */
	protected void initialize() {
		visualEntity = null;
		visualRectangle = new Rectangle(0, 0, 0, 0);
		visualColor = Color.black;
	}
}
