package sra.jun.goodies.button;

import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;

/**
 * JunButtonController class
 * 
 *  @author    He Weijie
 *  @created   1998/08/27 (by He Weijie)
 *  @updated   1999/11/12 (by MATSUDA Ryouichi)
 *  @updated   2002/11/03 (by nisinaka)
 *  @updated   2003/03/24 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun473 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunButtonController.java,v 6.0 2003/10/28 10:52:07 nisinaka Exp $
 */
public class JunButtonController extends StController implements MouseListener {

	protected boolean _mousePressed = false;
	protected Thread _repeatThread;
	protected Graphics _graphics;

	/**
	 * Invoked when a mouse button has been pressed on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 */
	public void mousePressed(MouseEvent e) {
		if (this.getButtonModel().isActive() == false) {
			return;
		}

		this.getButtonModel()._isPressedWithShiftDown(e.isShiftDown());

		_mousePressed = true;
		_graphics = this.getButtonView().toComponent().getGraphics();
		if (this.getButtonModel().value() == false) {
			this.getButtonView().displayTrueOn_(_graphics);
		}
		_graphics.setColor(Color.black);
		Dimension size = this.getButtonView().toComponent().getSize();
		_graphics.drawRect(0, 0, size.width - 1, size.height - 1);

		final long time = System.currentTimeMillis();
		_repeatThread = new Thread() {
			public void run() {
				while (_mousePressed) {
					if (getButtonModel().repeatAction() && ((System.currentTimeMillis() - time) > getButtonModel().repeatTick())) {
						_doAction();
					}

					Thread.yield();
				}
			}
		};
		_repeatThread.setPriority(Thread.NORM_PRIORITY + 1);
		_repeatThread.start();
	}

	/**
	 * Invoked when a mouse button has been released on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 */
	public void mouseReleased(MouseEvent e) {
		if (this.getButtonModel().isActive() == false) {
			return;
		}

		_mousePressed = false;

		// wait for the termination of the repeat action.
		try {
			_repeatThread.join();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		_repeatThread = null;

		// update the display view.
		if (this.getButtonModel().value()) {
			this.getView().displayTrueOn_(_graphics);
		} else {
			this.getView().displayFalseOn_(_graphics);
		}

		// and do the action.
		_doAction();

		// finally dispose the grahpics.
		_graphics.dispose();
		_graphics = null;
	}

	/**
	 * Do an action.
	 */
	private synchronized void _doAction() {
		StBlockClosure action = this.getButtonModel().action();
		switch (action.numArgs()) {
			case 0 :
				action.value();
				break;
			case 1 :
				action.value_(this.getButtonModel());
				break;
			case 2 :
				action.value_value_(this.getButtonModel(), null);
				break;
		}
	}

	/**
	 * add listeners.
	 * 
	 * @param newView sra.mvctest.StView
	 */
	protected void buildListener(StView newView) {
		final Component aView = (Component) newView;
		aView.addMouseListener(this);
	}

	/**
	 * Answer my model as a JunButtonModel.
	 * 
	 * @return JunButtonModel
	 */
	protected JunButtonModel getButtonModel() {
		return (JunButtonModel) this.model();
	}

	/**
	 * Answer my view as a JunButtonView.
	 * 
	 * @return JunButtonView interface
	 */
	protected JunButtonView getButtonView() {
		return (JunButtonView) this.view();
	}

	/**
	 * Answer my model as a JunButtonModel.
	 * 
	 * @return JunButtonModel
	 * @deprecated since Jun379, use getButtonModel()
	 */
	protected JunButtonModel getModel() {
		return (JunButtonModel) this.model();
	}

	/**
	 * Answer my view as a JunButtonView.
	 * 
	 * @return JunButtonView interface
	 * @deprecated since Jun379, use getButtonView()
	 */
	protected JunButtonView getView() {
		return (JunButtonView) this.view();
	}
}
