package sra.jun.goodies.button;

import java.awt.*;
import sra.smalltalk.*;

/**
 * JunButtonModel class
 * 
 *  @author    He Weijie
 *  @created   1998/08/13 (by He Weijie)
 *  @updated   1999/11/12 (by MATSUDA Ryouichi)
 *  @updated   2002/10/31 (by nisinaka)
 *  @updated   2002/11/21 (by nisinaka)
 *  @updated   2003/03/24 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun472 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunButtonModel.java,v 6.0 2003/10/28 10:52:07 nisinaka Exp $
 */
public class JunButtonModel extends StApplicationModel {

	/** A boolean value as a button model. */
	protected boolean buttonValue;

	/** An object which will be displayed on an associated view. */
	protected StDisplayable displayObject;

	/** An action which will be evaluated when a button is pressed. */
	protected StBlockClosure buttonAction;

	/** A flag to allow a repeat action or not. */
	protected boolean repeatAction;

	/** A number in millisecond to control the repeat action. */
	protected int repeatTick;

	/** A flag whether the button is currently active or not. */
	protected boolean activeState;

	protected boolean _isPressedWithShiftDown;

	/**
	 * Create a new instance of JunButtonModel and initialize it.
	 */
	public JunButtonModel() {
		this.initialize();
	}

	/**
	 * Create a new instance of JunButtonModel and initialize it.
	 * 
	 * @param newValue boolean
	 * @param newVisual java.awt.Image
	 * @param newAction sra.smalltalk.StBlockClosure
	 */
	public JunButtonModel(boolean newValue, Object newVisual, StBlockClosure newAction) {
		this();
		this.value_(newValue);
		this.visual_(newVisual);
		this.action_(newAction);
	}

	/**
	 * Create a new instance of JunButtonModel.
	 * 
	 * @param aClass java.lang.Class
	 * @param aBoolean boolean
	 * @param anImage java.awt.Image
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 * 
	 * @deprecated since Jun357, use the constructor
	 */
	public static JunButtonModel Value_visual_action_(Class aClass, boolean aBoolean, Image anImage, StBlockClosure aBlock) {
		JunButtonModel buttonModel = (JunButtonModel) _New(aClass);
		buttonModel.value_(aBoolean);
		buttonModel.visual_(anImage);
		buttonModel.action_(aBlock);
		return buttonModel;
	}

	/**
	 * Create a new instance of JunButtonModel.
	 * 
	 * @param aClass java.lang.Class
	 * @param aBoolean boolean
	 * @param anObject java.lang.Object
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 * 
	 * @deprecated since Jun357, use the constructor
	 */
	public static JunButtonModel Value_visual_action_(Class aClass, boolean aBoolean, Object anObject, StBlockClosure aBlock) {
		JunButtonModel buttonModel = (JunButtonModel) _New(aClass);
		buttonModel.value_(aBoolean);
		buttonModel.visual_(anObject);
		buttonModel.action_(aBlock);
		return buttonModel;
	}

	/**
	 * Create a new instance of JunButtonModel.
	 * 
	 * @param aBoolean boolean
	 * @param anImage java.awt.Image
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 * 
	 * @deprecated since Jun357, use the constructor
	 */
	public static final JunButtonModel Value_visual_action_(boolean aBoolean, Image anImage, StBlockClosure aBlock) {
		return new JunButtonModel(aBoolean, anImage, aBlock);
	}

	/**
	 * Create a new instance of JunButtonModel.
	 * 
	 * @param aBoolean boolean
	 * @param anObject java.lang.Object
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 * 
	 * @deprecated since Jun357, use the constructor
	 */
	public static final JunButtonModel Value_visual_action_(boolean aBoolean, Object anObject, StBlockClosure aBlock) {
		return new JunButtonModel(aBoolean, anObject, aBlock);
	}

	/**
	 * Answer true if the mouse pressed with the shift key down, otherwise false.
	 *
	 * @return boolean
	 */
	public boolean _isPressedWithShiftDown() {
		return _isPressedWithShiftDown;
	}

	/**
	 * Set whether the shift key is down when the button is pressed with the mouse.
	 *
	 * @param aBoolean boolean
	 */
	public void _isPressedWithShiftDown(boolean aBoolean) {
		_isPressedWithShiftDown = aBoolean;
	}

	/**
	 * Return the current button action. If not assigned, create a fake action
	 * and return it.
	 * 
	 * @return sra.smalltalk.StBlockClosure
	 */
	public StBlockClosure action() {
		if (buttonAction == null) {
			this.action_(new StBlockClosure());
		}
		return buttonAction;
	}

	/**
	 * Set the action.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void action_(StBlockClosure aBlock) {
		buttonAction = aBlock;
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		JunButtonView aView;
		if (GetDefaultViewMode() == VIEW_AWT) {
			aView = new JunButtonViewAwt(this, new Dimension(100, 100));
		} else {
			aView = new JunButtonViewSwing(this, new Dimension(100, 100));
		}
		return aView;
	}

	/**
	 * Make the button disable.
	 */
	public void disable() {
		activeState = false;
		this.changed_($("visual"));
	}

	/**
	 * Make the button enable.
	 */
	public void enable() {
		activeState = true;
		this.changed_($("visual"));
	}

	/**
	 * Answer true if the button is currently active, otherwise false.
	 *
	 * @return boolean
	 */
	public boolean isActive() {
		return activeState;
	}

	/**
	 * Just return the repeatAction.
	 * 
	 * @return boolean
	 */
	public boolean repeatAction() {
		return repeatAction;
	}

	/**
	 * Set the repeatAction.
	 * 
	 * @param aBoolean boolean
	 */
	public void repeatAction_(boolean aBoolean) {
		repeatAction = aBoolean;
	}

	/**
	 * Answer the repeat tick in milliseconds.
	 * 
	 * @return int
	 */
	public int repeatTick() {
		return repeatTick;
	}

	/**
	 * Set the repeat tick in milliseconds.
	 * 
	 * @param milliseconds int
	 */
	public void repeatTick_(int milliseconds) {
		repeatTick = milliseconds;
	}

	/**
	 * Just return the current button value.
	 * 
	 * @return boolean
	 */
	public boolean value() {
		return buttonValue;
	}

	/**
	 * Set the new button value. If the value is changed, announce it.
	 * 
	 * @param aBoolean boolean
	 */
	public void value_(boolean aBoolean) {
		if (aBoolean != buttonValue) {
			buttonValue = aBoolean;
			this.changed_($("value"));
		}
	}

	/**
	 * Return the current display object. If not assigned yet, set a null
	 * string as a fake display object and return it.
	 * 
	 * @return sra.smalltalk.StDisplayable
	 */
	public StDisplayable visual() {
		if (displayObject == null) {
			this.visual_(new String());
		}
		return displayObject;
	}

	/**
	 * Set a new visual object. 
	 * If the visual object is changed, announce it.
	 * 
	 * @param anObject java.lang.Object
	 */
	public void visual_(Object anObject) {
		Object visual = anObject;
		if (visual != displayObject) {
			if (visual == null) {
				displayObject = new StComposedText(new String());
			} else if (visual instanceof StDisplayable) {
				displayObject = (StDisplayable) visual;
			} else if (visual instanceof Image) {
				displayObject = new StImage((Image) visual);
			} else {
				String aString = (anObject instanceof String) ? (String) anObject : anObject.toString();
				displayObject = new StComposedText(aString);
			}
			this.changed_($("visual"));
		}
	}

	/**
	 * Initialize the JunButtonModel.
	 */
	protected void initialize() {
		super.initialize();

		buttonValue = false;
		displayObject = null;
		buttonAction = null;
		repeatAction = false;
		repeatTick = 500;
		activeState = true;
		_isPressedWithShiftDown = false;
	}
}
