package sra.jun.goodies.button;

import java.awt.*;
import sra.smalltalk.*;

/**
 * JunButtonViewAwt class
 * 
 *  @author    He Weijie
 *  @created   1998/08/27 (by He Weijie)
 *  @updated   1999/11/12 (by MATSUDA Ryouichi)
 *  @updated   2002/11/21 (by nisinaka)
 *  @updated   2003/03/24 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun472 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunButtonViewAwt.java,v 6.0 2003/10/28 10:52:07 nisinaka Exp $
 */
public class JunButtonViewAwt extends StViewCanvas implements JunButtonView {

	/**
	 * Create a new instance of JunButtonViewAwt.
	 */
	public JunButtonViewAwt() {
		super();
	}

	/**
	 * Create a new instance of JunButtonViewAwt and initialize it.
	 * 
	 * @param aButtonModel sra.jun.goodies.button.JunButtonModel
	 */
	public JunButtonViewAwt(JunButtonModel aButtonModel) {
		super(aButtonModel);
		this.setSize(aButtonModel.visual().bounds().getSize());
	}

	/**
	 * Create a new instance of JunButtonViewAwt and initialize it.
	 *
	 * @param aButtonModel sra.jun.goodies.button.JunButtonModel
	 * @param aDimension java.awt.Dimension
	 */
	public JunButtonViewAwt(JunButtonModel aButtonModel, Dimension aDimension) {
		super(aButtonModel);
		this.setSize(aDimension);
	}

	/**
	 * Update the view when the value of the model is false.
	 * 
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayFalseOn_(Graphics aGraphics) {
		Rectangle box = new Rectangle(0, 0, this.getSize().width, this.getSize().height);
		aGraphics.clipRect(box.x, box.y, box.width, box.height);
		aGraphics.setColor(Gray70);
		aGraphics.fillRect(box.x, box.y, box.width, box.height);

		aGraphics.setColor(Color.black);
		StDisplayable visual = this.getButtonModel().visual();
		Rectangle bounds = visual.bounds();
		visual.displayOn_at_(aGraphics, new Point((box.width - bounds.width) / 2, (box.height - bounds.height) / 2));

		aGraphics.setColor(Gray80);
		aGraphics.drawLine(box.x, box.y, (box.x + box.width) - 1, box.y);
		aGraphics.drawLine(box.x, box.y, box.x, (box.y + box.height) - 1);
		aGraphics.setColor(Gray20);
		aGraphics.drawLine(box.x, (box.y + box.height) - 1, (box.x + box.width) - 1, (box.y + box.height) - 1);
		aGraphics.drawLine((box.x + box.width) - 1, box.y, (box.x + box.width) - 1, (box.y + box.height) - 1);

		if (this.getButtonModel().isActive() == false) {
			aGraphics.setColor(Gray90);
			Dimension extent = this.getSize();
			for (int y = 0; y < extent.height; y++) {
				for (int x = 0; x < extent.width; x++) {
					if ((x + y) % 2 == 1) {
						aGraphics.drawRect(x, y, 0, 0);
					}
				}
			}
		}
	}

	/**
	 * paint.
	 * 
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayOn_(Graphics aGraphics) {
		if (this.getButtonModel().value() == true) {
			this.displayTrueOn_(aGraphics);
		} else {
			this.displayFalseOn_(aGraphics);
		}
	}

	/**
	 * Update the view when the value of the model is true.
	 * 
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayTrueOn_(Graphics aGraphics) {
		Rectangle box = new Rectangle(0, 0, this.getSize().width, this.getSize().height);
		aGraphics.clipRect(box.x, box.y, box.width, box.height);
		aGraphics.setColor(Gray30);
		aGraphics.fillRect(box.x, box.y, box.width, box.height);

		aGraphics.setColor(Color.black);
		StDisplayable visual = this.getButtonModel().visual();
		Rectangle bounds = visual.bounds();
		visual.displayOn_at_(aGraphics, new Point((box.width - bounds.width) / 2, (box.height - bounds.height) / 2));

		aGraphics.setColor(Gray20);
		aGraphics.drawLine(box.x, box.y, (box.x + box.width) - 1, box.y);
		aGraphics.drawLine(box.x, box.y, box.x, (box.y + box.height) - 1);
		aGraphics.setColor(Gray80);
		aGraphics.drawLine(box.x, (box.y + box.height) - 1, (box.x + box.width) - 1, (box.y + box.height) - 1);
		aGraphics.drawLine((box.x + box.width) - 1, box.y, (box.x + box.width) - 1, (box.y + box.height) - 1);

		if (this.getButtonModel().isActive() == false) {
			aGraphics.setColor(Gray90);
			Dimension extent = this.getSize();
			for (int y = 0; y < extent.height; y++) {
				for (int x = 0; x < extent.width; x++) {
					if ((x + y) % 2 == 1) {
						aGraphics.drawRect(x, y, 0, 0);
					}
				}
			}
		}
	}

	/**
	 * Answer my model as JunButtonModel.
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 */
	public JunButtonModel getButtonModel() {
		return (JunButtonModel) this.model();
	}

	/**
	 * model property accessing.
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 * @deprecated since Jun379, use getButtonModel().
	 */
	public JunButtonModel getModel() {
		return (JunButtonModel) this.model();
	}

	/**
	 * value property accessing.
	 * 
	 * @return boolean
	 */
	public boolean getValue() {
		return this.getButtonModel().value();
	}

	/**
	 * Answer the visual object of the view.
	 * 
	 * @return sra.smalltalk.StDisplayable
	 */
	public StDisplayable getVisual() {
		return this.getButtonModel().visual();
	}

	/**
	 * model property accessing.
	 * 
	 * @param aModel sra.jun.goodies.button.JunButtonModel
	 * @deprecated since Jun379
	 */
	public void setModel(JunButtonModel aModel) {
		this.model_(aModel);
	}

	/**
	 * value property accessing.
	 * 
	 * @param aValue boolean
	 */
	public void setValue(boolean aValue) {
		this.getButtonModel().value_(aValue);
	}

	/**
	 * visual property accessing.
	 * 
	 * @param anObject java.lang.Object
	 */
	public void setVisual(Object anObject) {
		this.getButtonModel().visual_(anObject);
	}

	/**
	 * Update the receiver according to the change notification from the model.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if (evt.getAspect() == $("visual")) {
			this.repaint();
			return;
		}
		super.update_(evt);
	}

	/**
	 * Answer my default controller
	 * 
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new JunButtonController();
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		JunButtonModel aModel = new JunButtonModel();
		aModel.visual_("JunButtonModel");
		return aModel;
	}
}
