package sra.jun.goodies.calendar;

import java.awt.*;
import java.awt.event.*;

import javax.swing.JButton;

import sra.smalltalk.StSimpleDialogViewSwing;

/**
 * JunCalendarDateAndTimeDialogViewSwing class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun472 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateAndTimeDialogViewSwing.java,v 6.0 2003/10/28 10:52:07 nisinaka Exp $
 */
public class JunCalendarDateAndTimeDialogViewSwing
	extends StSimpleDialogViewSwing
	implements JunCalendarDateAndTimeView {
	protected JunCalendarDateAndTimeViewSwing dateAndTimeView;
	protected JButton acceptButton;
	protected JButton cancelButton;

	/**
	 * Create a new instance of JunCalendarDateAndTimeDialogViewSwing and initialize it.
	 *
	 * @param aCalendarDateAndTimeModel JunCalendarDateAndTimeModel
	 * @category instance creation
	 */
	public JunCalendarDateAndTimeDialogViewSwing(JunCalendarDateAndTimeDialog aCalendarDateAndTimeDialog) {
		super(aCalendarDateAndTimeDialog);
	}

	/**
	* Answer the default dialog.
	*
	* @return java.awt.Dialog
	* @category defaults
	*/
	public Dialog defaultDialog() {
		Dialog aDialog =
			new Dialog(
				new Frame(),
				this.getCalendarDateAndTimeDialog().windowTitle(),
				true);
		aDialog.add(this);
		aDialog.pack();
		return aDialog;
	}

	/**
	    * Answer my model as a JunCalendarDateAndTimeDialog.
	    * @return JunCalendarDateAndTimeDialog
	* @category accessing
	    */
	public JunCalendarDateAndTimeDialog getCalendarDateAndTimeDialog() {
		return (JunCalendarDateAndTimeDialog) this.model();
	}

	/**
	    * Answer the panel on accept/cancel buttons.
	    *
	    * @return java.awt.Panel
	    * @category actions
	    */
	protected Panel _acceptCancelButtons() {
		Panel panel = new Panel();
		panel.setLayout(new FlowLayout());
		panel.add(this.acceptButton());
		panel.add(this.cancelButton());
		return panel;
	}

	/**
	    * Answer my accept button.
	    * @return java.awt.JButton
	    * @category actions
	    */
	protected JButton acceptButton() {
		if (acceptButton == null) {
			acceptButton = new JButton("Accept");
			acceptButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarDateAndTimeDialog()
						.acceptButtonPressed())
						.booleanValue()) {
						getCalendarDateAndTimeDialog().accept().value_(true);
					}
				}
			});
		}
		return acceptButton;
	}

	/**
	 * Build this component.
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setLayout(new BorderLayout());
		this.add(this.dateAndTimeView(), BorderLayout.CENTER);
		this.add(this._acceptCancelButtons(), BorderLayout.SOUTH);
		defaultButton = this.acceptButton();
	}

	/**
	 * Answer my cancel button.
	 *
	 * @return java.awt.JButton
	 * @category actions
	 */
	protected JButton cancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton("Cancel");
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarDateAndTimeDialog()
						.cancelButtonPressed())
						.booleanValue()) {
						getCalendarDateAndTimeDialog().cancel().value_(true);
					}
				}
			});
		}
		return cancelButton;
	}

	/**
	    * Answer my calendar date view.
	    *
	    * @return JunCalendarDateAndTimeViewSwing
	    * @category accessing
	    */
	protected JunCalendarDateAndTimeViewSwing dateAndTimeView() {
		if (dateAndTimeView == null) {
			dateAndTimeView =
				new JunCalendarDateAndTimeViewSwing(
					this
						.getCalendarDateAndTimeDialog()
						.calendarDateAndTimeModel());
		}
		return dateAndTimeView;
	}
}
