package sra.jun.goodies.calendar;

import java.awt.Dialog;
import java.util.Calendar;

import sra.smalltalk.*;

/**
 * JunCalendarDateAndTimeIntervalDialog class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun472 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateAndTimeIntervalDialog.java,v 6.0 2003/10/28 10:52:07 nisinaka Exp $
 */
public class JunCalendarDateAndTimeIntervalDialog extends StSimpleDialog {
	protected JunCalendarDateAndTimeIntervalModel calendarDateAndTimeIntervalModel;
	protected StBlockClosure acceptBlock;
	protected StBlockClosure cancelBlock;

	/**
	 * Create new instance of JunCalendarDateAndTimeIntervalDialog.
	 * @category instance creation
	 */
	public JunCalendarDateAndTimeIntervalDialog() {
		super();
	}

	/**
	 * JunCalendarDateAndTimeIntervalDialog.Request()
	 * @return Calendar
	 * @category utilities
	 */
	public static JunCalendarDateAndTimeModel[] Request() {
		return JunCalendarDateAndTimeIntervalDialog.Request_(
			"Calendar Date And Time Interval Dialog");
	}

	/**
	 * JunCalendarDateAndTimeIntervalDialog.Request_("Select date.")
	 * @param messageString java.lang.String
	 * @return Calendar
	 * @category utilities
	 */
	public static JunCalendarDateAndTimeModel[] Request_(String messageString) {
		Calendar[][] dateArray =
			{ { Calendar.getInstance(), Calendar.getInstance()}, {
				Calendar.getInstance(), Calendar.getInstance()
				}
		};
		return JunCalendarDateAndTimeIntervalDialog.Request_initialDate_(
			messageString,
			dateArray);
	}

	/**
	 * JunCalendarDateAndTimeIntervalDialog.Request_initialDate_("Select date.", Calendar.getInstance());
	 * @param messageString java.lang.String
	 * @param aDate java.util.Calendar
	 * @return Calendar
	 * @category utilities
	 */
	public static JunCalendarDateAndTimeModel[] Request_initialDate_(
		String messageString,
		Calendar[][] dateArray) {
		return JunCalendarDateAndTimeIntervalDialog
			.Request_initialDate_onAccept_onCancel_(
			messageString,
			dateArray,
			null,
			null);
	}

	/**
	 * JunCalendarDateAndTimeIntervalDialog.Request_initialDate_onAccept_onCancel_(
	 * 	"Select date.",
	 *	Calendar.getInstance(),
	 *	null,
	 *	null)
	 * @param messageString java.lang.String
	 * @param aDate java.util.Calendar
	 * @param acceptBlock StBlockClosure
	 * @param cancelBlock StBlockClosure
	 * @return java.util.Calendar
	 * @category utilities
	 */
	public static JunCalendarDateAndTimeModel[] Request_initialDate_onAccept_onCancel_(
		String messageString,
		Calendar[][] dateArray,
		StBlockClosure anAcceptBlock,
		StBlockClosure aCancelBlock) {
		JunCalendarDateAndTimeIntervalDialog calendarDateAndTimeIntervalDialog =
			new JunCalendarDateAndTimeIntervalDialog();
		if (!(dateArray == null)) {
			if (dateArray.length > 1) {
				calendarDateAndTimeIntervalDialog
					.calendarDateAndTimeIntervalModel_(
					JunCalendarDateAndTimeIntervalModel.From_to_(
						dateArray[0],
						dateArray[dateArray.length - 1]));
			}
		}
		Dialog dialog = calendarDateAndTimeIntervalDialog.allButOpenDialog();
		if (!(messageString == null)) {
			dialog.setTitle(messageString);
		}
		if (!(anAcceptBlock == null)) {
			calendarDateAndTimeIntervalDialog.acceptBlock_(anAcceptBlock);
		}
		if (!(aCancelBlock == null)) {
			calendarDateAndTimeIntervalDialog.cancelBlock_(aCancelBlock);
		}
		_ShowAtMousePoint(dialog);
		if (calendarDateAndTimeIntervalDialog.accept()._booleanValue()) {
			return calendarDateAndTimeIntervalDialog
				.calendarDateAndTimeIntervalModel()
				.dateAndTimes();
		} else {
			return null;
		}
	}

	/**
	 * Set an action accept to a block..
	 * @param aBlock StBlockClosure
	 * @category accessing
	 */
	public void acceptBlock_(StBlockClosure aBlock) {
		acceptBlock = aBlock;
	}

	/**
	 * Return a JunCalendarDateAndTimeIntervalModel instance.
	 * @return JunCalendarDateAndTimeIntervalModel
	 * @category accessing
	 */
	public JunCalendarDateAndTimeIntervalModel calendarDateAndTimeIntervalModel() {
		if (calendarDateAndTimeIntervalModel == null) {
			calendarDateAndTimeIntervalModel =
				new JunCalendarDateAndTimeIntervalModel();
		}
		return calendarDateAndTimeIntervalModel;
	}

	/**
	 * Set new insntace of JunCalendarDateAndTimeIntervalModel.
	 * @param aCalendarDateAndTimeIntervalModel JunCalendarDateAndTimeIntervalModel
	 * @category accessing
	 */
	public void calendarDateAndTimeIntervalModel_(JunCalendarDateAndTimeIntervalModel aCalendarDateAndTimeIntervalModel) {
		calendarDateAndTimeIntervalModel = aCalendarDateAndTimeIntervalModel;
	}

	/**
	 * Set a calcel action to a block.
	 * @param aBlock StBlockClosure
	 * @category accessing
	 */
	public void cancelBlock_(StBlockClosure aBlock) {
		cancelBlock = aBlock;
	}

	/**
	 * Return result of accept button pressed.
	 * @return boolean
	 * @category actions
	 */
	public Object acceptButtonPressed() {
		if (!(acceptBlock == null)) {
			return acceptBlock.value_(this.calendarDateAndTimeIntervalModel());
		}
		return new Boolean(true);
	}

	/**
	 * Return result of cancel button pressed.
	 * @return boolean
	 * @category actions
	 */
	public Object cancelButtonPressed() {
		if (!(cancelBlock == null)) {
			return cancelBlock.value_(this.calendarDateAndTimeIntervalModel());
		}
		return new Boolean(true);
	}

	/**
	 * Answer a default view.
	 *
	 * @return sra.smalltalk.StView
	 * @category defaults
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunCalendarDateAndTimeIntervalDialogViewAwt(this);
		} else {
			return new JunCalendarDateAndTimeIntervalDialogViewSwing(this);
		}
	}

	/**
	 * Initialize the ApplicationModel when created.
	 * @category initialize release
	 */
	protected void initialize() {
		super.initialize();

		acceptBlock = null;
		cancelBlock = null;
	}

	/**
	 * Answer a window title.
	 *
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return "Calendar Date Dialog";
	}
}
