package sra.jun.goodies.calendar;

import java.awt.*;
import java.awt.event.*;

import javax.swing.JButton;

import sra.smalltalk.StSimpleDialogViewSwing;

/**
 * JunCalendarDateDialogViewSwing class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun472 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateDialogViewSwing.java,v 6.0 2003/10/28 10:52:07 nisinaka Exp $
 */
public class JunCalendarDateDialogViewSwing
	extends StSimpleDialogViewSwing
	implements JunCalendarDateDialogView {
	protected JunCalendarDateViewSwing calendarDateView;
	protected JButton acceptButton;
	protected JButton cancelButton;

	/**
	 * Create a new instance of JunCaledarDateDialogViewSwing and initialize it with the JunCalendarDateDialog.
	 * @param newCalendarDateAndTimeDialog JunCalendarDateDialog
	 * @category instance creation
	 */
	public JunCalendarDateDialogViewSwing(JunCalendarDateDialog newCalendarDateDialog) {
		super(newCalendarDateDialog);
	}

	/**
	 * Answer the default dialog.
	 *
	 * @return java.awt.Dialog
	 * @category defaults
	 */
	public Dialog defaultDialog() {
		Dialog aDialog =
			new Dialog(
				new Frame(),
				this.getCalendarDateDialog().windowTitle(),
				true);
		aDialog.add(this);
		aDialog.pack();
		return aDialog;
	}

	/**
	 * Answer my model as a JunCalendarDateDialog.
	 * @return JunCalendarDateDialog
	 * @category accessing
	 */
	public JunCalendarDateDialog getCalendarDateDialog() {
		return (JunCalendarDateDialog) this.model();
	}

	/**
	 * Answer the panel on accept/cancel buttons.
	 *
	 * @return java.awt.Panel
	 * @category accessing
	 */
	protected Panel _acceptCancelButtons() {
		Panel panel = new Panel();
		panel.setLayout(new FlowLayout());
		panel.add(this.acceptButton());
		panel.add(this.cancelButton());
		return panel;
	}

	/**
	 * Answer my accept button.
	 * @return javax.swing.jButton
	 * @category accessing
	 */
	protected JButton acceptButton() {
		if (acceptButton == null) {
			acceptButton = new JButton("Accept");
			acceptButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarDateDialog()
						.acceptButtonPressed())
						.booleanValue()) {
						getCalendarDateDialog().accept().value_(true);
					}
				}
			});
		}
		return acceptButton;
	}

	/**
	 * Build this component.
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setLayout(new BorderLayout());
		this.add(this.calendarDateView(), BorderLayout.CENTER);
		this.add(this._acceptCancelButtons(), BorderLayout.SOUTH);
		defaultButton = this.acceptButton();
	}

	/**
	 * Answer my cancel button.
	 *
	 * @return javax.swing.jButton
	 * @category accessing
	 */
	protected JButton cancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton("Cancel");
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarDateDialog()
						.cancelButtonPressed())
						.booleanValue()) {
						getCalendarDateDialog().cancel().value_(true);
					}
				}
			});
		}
		return cancelButton;
	}

	/**
	 * Answer my calendar date view.
	 *
	 * @return JunCalendarDateViewSwing
	 * @category accessing
	 */
	protected JunCalendarDateViewSwing calendarDateView() {
		if (calendarDateView == null) {
			calendarDateView =
				new JunCalendarDateViewSwing(
					this.getCalendarDateDialog().calendarDateModel());
		}
		return calendarDateView;
	}
}
