package sra.jun.goodies.calendar;

import java.awt.Dialog;
import java.util.Calendar;

import sra.smalltalk.*;

/**
 * JunCalendarDateIntervalDialog class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun472 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateIntervalDialog.java,v 6.0 2003/10/28 10:52:07 nisinaka Exp $
 */
public class JunCalendarDateIntervalDialog extends StSimpleDialog {
	protected JunCalendarDateIntervalModel calendarDateIntervalModel;
	protected StBlockClosure acceptBlock;
	protected StBlockClosure cancelBlock;

	/**
	 * Create new instance of JunCalendarDateIntervalDialog.
	 * @category instance creation
	 */
	public JunCalendarDateIntervalDialog() {
		super();
	}

	/**
	 * JunCalendarDateIntervalDialog.Request()
	 * @return Calendar
	 * @category utilities
	 */
	public static JunCalendarDateModel[] Request() {
		return JunCalendarDateIntervalDialog.Request_(
			"Calendar Date Interval Dialog");
	}

	/**
	 * JunCalendarDateIntervalDialog.Request_("Select date.")
	 * @param messageString java.lang.String
	 * @return Calendar
	 * @category utilities
	 */
	public static JunCalendarDateModel[] Request_(String messageString) {
		Calendar[] dateArray =
			{ Calendar.getInstance(), Calendar.getInstance()};
		return JunCalendarDateIntervalDialog.Request_initialDate_(
			messageString,
			dateArray);
	}

	/**
	 * JunCalendarDateIntervalDialog.Request_initialDate_("Select date.", Calendar.getInstance());
	 * @param messageString java.lang.String
	 * @param aDate java.util.Calendar
	 * @return Calendar
	 * @category utilities
	 */
	public static JunCalendarDateModel[] Request_initialDate_(
		String messageString,
		Calendar[] dateArray) {
		return JunCalendarDateIntervalDialog
			.Request_initialDate_onAccept_onCancel_(
			messageString,
			dateArray,
			null,
			null);
	}

	/**
	 * JunCalendarDateIntervalDialog.Request_initialDate_onAccept_onCancel_(
	 * 	"Select date.",
	 *	Calendar.getInstance(),
	 *	null,
	 *	null)
	 * @param messageString java.lang.String
	 * @param aDate java.util.Calendar
	 * @param acceptBlock StBlockClosure
	 * @param cancelBlock StBlockClosure
	 * @return java.util.Calendar
	 * @category utilities
	 */
	public static JunCalendarDateModel[] Request_initialDate_onAccept_onCancel_(
		String messageString,
		Calendar[] dateArray,
		StBlockClosure anAcceptBlock,
		StBlockClosure aCancelBlock) {
		JunCalendarDateIntervalDialog calendarDateIntervalDialog =
			new JunCalendarDateIntervalDialog();
		if (!(dateArray == null)) {
			if (dateArray.length > 1) {
				calendarDateIntervalDialog.calendarDateIntervalModel_(
					JunCalendarDateIntervalModel.From_to_(
						dateArray[0],
						dateArray[dateArray.length - 1]));
			}
		}
		Dialog dialog = calendarDateIntervalDialog.allButOpenDialog();
		if (!(messageString == null)) {
			dialog.setTitle(messageString);
		}
		if (!(anAcceptBlock == null)) {
			calendarDateIntervalDialog.acceptBlock_(anAcceptBlock);
		}
		if (!(aCancelBlock == null)) {
			calendarDateIntervalDialog.cancelBlock_(aCancelBlock);
		}
		_ShowAtMousePoint(dialog);
		if (calendarDateIntervalDialog.accept()._booleanValue()) {
			return calendarDateIntervalDialog
				.calendarDateIntervalModel()
				.dates();
		} else {
			return null;
		}
	}

	/**
	 * Set an action accept to a block..
	 * @param aBlock StBlockClosure
	 * @category accessing
	 */
	public void acceptBlock_(StBlockClosure aBlock) {
		acceptBlock = aBlock;
	}

	/**
	 * Return a JunCalendarDateIntervalModel instance.
	 * @return JunCalendarDateIntervalModel
	 * @category accessing
	 */
	public JunCalendarDateIntervalModel calendarDateIntervalModel() {
		if (calendarDateIntervalModel == null) {
			calendarDateIntervalModel = new JunCalendarDateIntervalModel();
		}
		return calendarDateIntervalModel;
	}

	/**
	 * Set new insntace of JunCalendarDateIntervalModel.
	 * @param aCalendarDateIntervalModel JunCalendarDateIntervalModel
	 * @category accessing
	 */
	public void calendarDateIntervalModel_(JunCalendarDateIntervalModel aCalendarDateIntervalModel) {
		calendarDateIntervalModel = aCalendarDateIntervalModel;
	}

	/**
	 * Set a calcel action to a block.
	 * @param aBlock StBlockClosure
	 * @category accessing
	 */
	public void cancelBlock_(StBlockClosure aBlock) {
		cancelBlock = aBlock;
	}

	/**
	 * Return result of accept button pressed.
	 * @return boolean
	 * @category actions
	 */
	public Object acceptButtonPressed() {
		if (!(acceptBlock == null)) {
			return acceptBlock.value_(this.calendarDateIntervalModel());
		}
		return new Boolean(true);
	}

	/**
	 * Return result of cancel button pressed.
	 * @return boolean
	 * @category actions
	 */
	public Object cancelButtonPressed() {
		if (!(cancelBlock == null)) {
			return cancelBlock.value_(this.calendarDateIntervalModel());
		}
		return new Boolean(true);
	}

	/**
	 * Answer a default view.
	 *
	 * @return sra.smalltalk.StView
	 * @category defaults
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunCalendarDateIntervalDialogViewAwt(this);
		} else {
			return new JunCalendarDateIntervalDialogViewSwing(this);
		}
	}

	/**
	 * Initialize the ApplicationModel when created.
	 * @category initialize release
	 */
	protected void initialize() {
		super.initialize();

		acceptBlock = null;
		cancelBlock = null;
	}

	/**
	 * Answer a window title.
	 *
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return "Calendar Date Dialog";
	}
}
