package sra.jun.goodies.calendar;

import java.awt.*;
import java.util.Vector;

import sra.jun.system.framework.JunApplicationModel;
import sra.smalltalk.*;

/**
 * JunCalendarDateViewAwt class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun472 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateViewAwt.java,v 6.0 2003/10/28 10:52:07 nisinaka Exp $
 */
public class JunCalendarDateViewAwt
	extends StViewCanvas
	implements JunCalendarDateView {
	/**
	 * frames.
	 */
	protected Vector frames;

	protected PopupMenu _popupMenu;

	/**
	 * Create a new instance of JunCalendarDateViewAwt.
	 * @category instance creation
	 */
	public JunCalendarDateViewAwt(JunApplicationModel aCalendarDateModel) {
		super(aCalendarDateModel);
	}

	/**
	 * Return a frame.
	 * @return java.util.Vector
	 * @category accessing
	 */
	public Vector frames() {
		if (frames == null) {
			this.makeFrames();
		}
		return frames;
	}

	/**
	 * Make frames.
	 * @category private
	 */
	private void makeFrames() {
		Vector collection = new Vector();
		int width = this.getSize().width / 7;
		int height = this.getSize().height / 8;
		StRectangle button = new StRectangle(0, 0, 12, 16);
		int x = width * 1;
		int y = 0;
		StRectangle box = new StRectangle(x, y, width * 2, height);
		Object[] set = new Object[2];
		set[0] = "year";
		set[1] = box;
		collection.add(set);
		button = button.align_with_(button.rightCenter(), box.leftCenter());
		set = new Object[3];
		set[0] = "button";
		set[1] = "yearDown";
		set[2] = button;
		collection.add(set);
		button = button.align_with_(button.leftCenter(), box.rightCenter());
		set = new Object[3];
		set[0] = "button";
		set[1] = "yearUp";
		set[2] = button;
		collection.add(set);
		x = width * 4;
		y = 0;
		box =
			StRectangle.Origin_extent_(
				new Point(x, y),
				new Point(width * 2, height));
		set = new Object[2];
		set[0] = "month";
		set[1] = box;
		collection.add(set);
		button = button.align_with_(button.rightCenter(), box.leftCenter());
		set = new Object[3];
		set[0] = "button";
		set[1] = "monthDown";
		set[2] = button;
		collection.add(set);
		button = button.align_with_(button.leftCenter(), box.rightCenter());
		set = new Object[3];
		set[0] = "button";
		set[1] = "monthUp";
		set[2] = button;
		collection.add(set);
		x = 0;
		y = height * 1;
		JunCalendarDateModel m = (JunCalendarDateModel) this.model();
		String[] weekdayNames = m.weekdayNames();
		int[][] aCalendarMatrix = m.calendarMatrix();
		for (int i = 0; i < weekdayNames.length; ++i) {
			box =
				StRectangle.Origin_extent_(
					new Point(x, y),
					new Point(width, height));
			set = new Object[3];
			set[0] = "weekday";
			set[1] = new Integer(i);
			set[2] = box;
			collection.add(set);
			x = x + width;
		}
		y = height * 2;
		height = (this.getBounds().height - y) / m.calendarMatrix.length;
		int i, j = 0;
		for (int k = 0; k < m.calendarMatrix.length; ++k) {
			x = 0;
			i = 0;
			for (int l = 0; l < m.calendarMatrix[k].length; ++l) {
				if (!(m.calendarMatrix[k][l] == 0)) {
					box =
						StRectangle.Origin_extent_(
							new Point(x, y),
							new Point(width, height));
					set = new Object[3];
					set[0] = "day";
					set[1] = new Point(i, j);
					set[2] = box;
					collection.add(set);
				}
				x = x + width;
				i = i + 1;
			}
			y = y + height;
			j = j + 1;
		}
		frames = collection;
	}

	/**
	 * paint.
	 * @param aGraphics java.awt.Graphics
	 * @category displaying
	 */
	public void displayOn_(Graphics aGraphics) {
		this.displayBordersOn_(aGraphics);
		this.displayObjectsOn_(aGraphics);
	}

	/**
	 * Paint borders.
	 * @param aGraphics java.awt.Graphics
	 * @category displaying
	 */
	public void displayBordersOn_(Graphics aGraphics) {
		String key;
		StRectangle box;
		StRectangle area;
		StRectangle clip;
		for (int i = 0; i < this.frames().size(); ++i) {
			Object[] array = (Object[]) this.frames().get(i);
			key = (String) array[0];
			box = (StRectangle) array[array.length - 1];
			clip = new StRectangle(aGraphics.getClipBounds());
			if (clip.intersects_(box)) {
				if (key.equals("year")
					|| key.equals("month")
					|| key.equals("day")) {
					aGraphics.setColor(veryLightGray);
					aGraphics.fillRect(
						box.x(),
						box.y(),
						box.width(),
						box.height());
					area = box.insetBy_(new StRectangle(1, 1, 2, 2));
					aGraphics.setColor(veryDarkGray);
					aGraphics.drawRect(
						area.x(),
						area.y(),
						area.width(),
						area.height());
					if (key.equals("year") || key.equals("month")) {
						area = box.insetBy_(new StRectangle(2, 2, 3, 3));
						aGraphics.setColor(Color.black);
						aGraphics.drawRect(
							area.x(),
							area.y(),
							area.width(),
							area.height());
					}
					if (key.equals("day")) {
						JunCalendarDateModel m =
							(JunCalendarDateModel) this.model();
						Point point = (Point) array[1];
						int day = m.atPoint_(point);
						if (m.day() == day) {
							area = box.insetBy_(new StRectangle(2, 2, 3, 3));
							aGraphics.setColor(Color.black);
							aGraphics.drawRect(
								area.x(),
								area.y(),
								area.width(),
								area.height());
						}
					}
				}
			}
		}
	}

	/**
	 * Paint objects.
	 * @param aGraphics java.awt.Graphics
	 * @category displaying
	 */
	public void displayObjectsOn_(Graphics aGraphics) {
		String key;
		StRectangle box;
		for (int i = 0; i < this.frames().size(); ++i) {
			Object[] array = (Object[]) this.frames().get(i);
			key = (String) array[0];
			box = (StRectangle) array[array.length - 1];
			if (key.equals("year")) {
				this.displayOn_year_clip_(aGraphics, key, box);
			}
			if (key.equals("month")) {
				this.displayOn_month_clip_(aGraphics, key, box);
			}
			if (key.equals("weekday")) {
				this.displayOn_weekday_clip_(
					aGraphics,
					(Integer) array[1],
					box);
			}
			if (key.equals("day")) {
				this.displayOn_day_clip_(aGraphics, (Point) array[1], box);
			}
			if (key.equals("button")) {
				this.displayOn_button_clip_(aGraphics, (String) array[1], box);
			}
		}
	}

	/**
	 * Paint year.
	 * @param aGraphics Graphics
	 * @param aSymbol String
	 * @param aRectangle rect
	 * @category displaying
	 */
	public void displayOn_year_clip_(
		Graphics aGraphics,
		String aSymbol,
		StRectangle aRectangle) {
		int year;
		StComposedText para;
		StRectangle rect, clip;
		year = this.getModel().year();
		para = new StComposedText(String.valueOf(year));
		StRectangle bounds = new StRectangle(para.bounds());
		rect = bounds.align_with_(bounds.center(), aRectangle.center());
		clip = new StRectangle(aGraphics.getClipBounds());
		if (clip.intersects_(rect)) {
			aGraphics.setColor(Color.black);
			para.displayOn_at_(aGraphics, rect.origin());
		}
	}

	/**
	 * Paint month.
	 * @param aGraphics Graphics
	 * @param aSymbol String
	 * @param aRectangle rect
	 * @category displaying
	 */
	public void displayOn_month_clip_(
		Graphics aGraphics,
		String aSymbol,
		StRectangle aRectangle) {
		int month;
		StComposedText para;
		StRectangle rect, clip;
		month = 1 + this.getModel().month();
		para = new StComposedText(String.valueOf(month));
		StRectangle bounds = new StRectangle(para.bounds());
		rect = bounds.align_with_(bounds.center(), aRectangle.center());
		clip = new StRectangle(aGraphics.getClipBounds());
		if (clip.intersects_(rect)) {
			aGraphics.setColor(Color.black);
			para.displayOn_at_(aGraphics, rect.origin());
		}
	}

	/**
	 * Paint day.
	 * @param aGraphics Graphics
	 * @param aSymbol String
	 * @param aRectangle rect
	 * @category displaying
	 */
	public void displayOn_day_clip_(
		Graphics aGraphics,
		Point aPoint,
		StRectangle aRectangle) {
		int day;
		StComposedText para;
		StRectangle rect, clip;
		day = this.getModel().atPoint_(aPoint);
		para = new StComposedText(String.valueOf(day));
		StRectangle bounds = new StRectangle(para.bounds());
		rect = bounds.align_with_(bounds.center(), aRectangle.center());
		clip = new StRectangle(aGraphics.getClipBounds());
		if (clip.intersects_(rect)) {
			aGraphics.setColor(Color.black);
			if (aPoint.x == 0) {
				aGraphics.setColor(Color.red);
			}
			if (aPoint.x == 6) {
				aGraphics.setColor(Color.blue);
			}
			para.displayOn_at_(aGraphics, rect.origin());
		}
	}

	/**
	 * Paint weekday.
	 * @param aGraphics Graphics
	 * @param aSymbol String
	 * @param aRectangle rect
	 * @category displaying
	 */
	public void displayOn_weekday_clip_(
		Graphics aGraphics,
		Integer anInteger,
		StRectangle aRectangle) {
		String weekday;
		StComposedText para;
		StRectangle rect, clip;
		weekday = this.getModel().weekdayNames()[anInteger.intValue()];
		para = new StComposedText(weekday);
		StRectangle bounds = new StRectangle(para.bounds());
		rect = bounds.align_with_(bounds.center(), aRectangle.center());
		clip = new StRectangle(aGraphics.getClipBounds());
		if (clip.intersects_(rect)) {
			aGraphics.setColor(Color.black);
			if (anInteger.intValue() == 0) {
				aGraphics.setColor(Color.red);
			}
			if (anInteger.intValue() == 6) {
				aGraphics.setColor(Color.blue);
			}
			para.displayOn_at_(aGraphics, rect.origin());
		}
	}

	/**
	 * Paint button.
	 * @param aGraphics Graphics
	 * @param aSymbol String
	 * @param aRectangle rect
	 * @category displaying
	 */
	public void displayOn_button_clip_(
		Graphics aGraphics,
		String aSymbol,
		StRectangle aRectangle) {
		Polygon polygon;
		StRectangle box = aRectangle;
		StRectangle clip = new StRectangle(aGraphics.getClipBounds());
		if (clip.intersects_(box)) {
			if (aSymbol.equals("yearDown") || aSymbol.equals("monthDown")) {
				polygon = new Polygon();
				polygon.addPoint(box.topRight().x, box.topRight().y);
				polygon.addPoint(box.bottomRight().x, box.bottomRight().y);
				polygon.addPoint(box.leftCenter().x, box.leftCenter().y);
				polygon.addPoint(box.topRight().x, box.topRight().y);
			} else {
				polygon = new Polygon();
				polygon.addPoint(box.topLeft().x, box.topLeft().y);
				polygon.addPoint(box.bottomLeft().x, box.bottomLeft().y);
				polygon.addPoint(box.rightCenter().x, box.rightCenter().y);
				polygon.addPoint(box.topLeft().x, box.topLeft().y);
			}
			aGraphics.setColor(veryLightGray);
			aGraphics.fillPolygon(polygon);
			box = box.insetBy_(new StRectangle(1, 1, 2, 2));
			if (aSymbol.equals("yearDown") || aSymbol.equals("monthDown")) {
				polygon = new Polygon();
				polygon.addPoint(box.topRight().x, box.topRight().y);
				polygon.addPoint(box.bottomRight().x, box.bottomRight().y);
				polygon.addPoint(box.leftCenter().x, box.leftCenter().y);
				polygon.addPoint(box.topRight().x, box.topRight().y);
			} else {
				polygon = new Polygon();
				polygon.addPoint(box.topLeft().x, box.topLeft().y);
				polygon.addPoint(box.bottomLeft().x, box.bottomLeft().y);
				polygon.addPoint(box.rightCenter().x, box.rightCenter().y);
				polygon.addPoint(box.topLeft().x, box.topLeft().y);
			}
			aGraphics.setColor(veryDarkGray);
			aGraphics.drawPolygon(polygon);
		}
	}

	/**
	 * Update event handling
	 * @param evt DependentEvent
	 * @category updating
	 */
	public void update_(DependentEvent evt) {
		this.makeFrames();
		if (evt.getAspect() == $("day")) {
			repaint();
		}
		if (evt.getAspect() == $("month")) {
			repaint();
		}
		if (evt.getAspect() == $("year")) {
			repaint();
		}
		super.update_(evt);
	}

	/**
	 * Build component.
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setSize(300, 200);
	}

	/**
	 * Return model.
	 * @return sra.jun.goodies.calenadr.JunCalendarDateModel
	 * @category accessing
	 */
	public JunCalendarDateModel getModel() {
		return (JunCalendarDateModel) this.model();
	}

	/**
	 * Return default controller.
	 * @return sra.smalltalk.StController
	 * @category controller accessing
	 */
	protected StController defaultController() {
		return new JunCalendarDateController();
	}

	/**
	 * Return default model.
	 * @return sra.smalltalk.StModel
	 * @category accessing
	 */
	protected StModel defaultModel() {
		return new JunCalendarDateModel();
	}

	/**
	 * Answer the popup menu.
	 *
	 * @return java.awt.PopupMenu
	 * @category menu accessing
	 */
	protected PopupMenu popupMenu() {
		if (_popupMenu == null) {
			_popupMenu = this._createPopupMenu(this.getModel().popupMenu());
			this.add(_popupMenu);
		}
		return _popupMenu;
	}
}
