package sra.jun.goodies.calendar;

import java.awt.*;
import java.awt.event.*;

import sra.smalltalk.StSimpleDialogViewAwt;

/**
 * JunCalendarTimeIntervalDialogViewAwt class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun472 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarTimeIntervalDialogViewAwt.java,v 6.0 2003/10/28 10:52:07 nisinaka Exp $
 */
public class JunCalendarTimeIntervalDialogViewAwt
	extends StSimpleDialogViewAwt
	implements JunCalendarTimeIntervalDialogView {
	protected JunCalendarTimeIntervalViewAwt calendarTimeIntervalView;
	protected Button acceptButton;
	protected Button cancelButton;

	/**
	 * Create a new instance of JunCaledarTimeIntervalDialogViewAwt and initialize it with the JunCalendarTimeAndTimeIntervalDialog.
	 * @param newCalendarTimeAndTimeIntervalDialog JunCalendarTimeAndTimeIntervalDialog
	 * @category instance creation
	 */
	public JunCalendarTimeIntervalDialogViewAwt(JunCalendarTimeIntervalDialog newCalendarTimeIntervalDialog) {
		super(newCalendarTimeIntervalDialog);
	}

	/**
	 * Answer the default dialog.
	 *
	 * @return java.awt.Dialog
	 * @category accessing
	 */
	public Dialog defaultDialog() {
		Dialog aDialog =
			new Dialog(
				new Frame(),
				this.getCalendarTimeIntervalDialog().windowTitle(),
				true);
		aDialog.add(this);
		aDialog.pack();
		return aDialog;
	}

	/**
	 * Answer my model as a JunCalendarTimeIntervalDialog.
	 * @return JunCalendarTimeIntervalDialog
	 * @category accessing
	 */
	public JunCalendarTimeIntervalDialog getCalendarTimeIntervalDialog() {
		return (JunCalendarTimeIntervalDialog) this.model();
	}

	/**
	 * Answer the panel on accept/cancel buttons.
	 *
	 * @return java.awt.Panel
	 * @category accessing
	 */
	protected Panel _acceptCancelButtons() {
		Panel panel = new Panel();
		panel.setLayout(new FlowLayout());
		panel.add(this.acceptButton());
		panel.add(this.cancelButton());
		return panel;
	}

	/**
	 * Answer my accept button.
	 * @return java.awt.Button
	 * @category accessing
	 */
	protected Button acceptButton() {
		if (acceptButton == null) {
			acceptButton = new Button("Accept");
			acceptButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarTimeIntervalDialog()
						.acceptButtonPressed())
						.booleanValue()) {
						getCalendarTimeIntervalDialog().accept().value_(true);
					}
				}
			});
		}
		return acceptButton;
	}

	/**
	 * Build this component.
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setLayout(new BorderLayout());
		this.add(this.calendarTimeIntervalView(), BorderLayout.CENTER);
		this.add(this._acceptCancelButtons(), BorderLayout.SOUTH);
		defaultButton = this.acceptButton();
	}

	/**
	 * Answer my cancel button.
	 *
	 * @return java.awt.Button
	 * @category accessing
	 */
	protected Button cancelButton() {
		if (cancelButton == null) {
			cancelButton = new Button("Cancel");
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarTimeIntervalDialog()
						.cancelButtonPressed())
						.booleanValue()) {
						getCalendarTimeIntervalDialog().cancel().value_(true);
					}
				}
			});
		}
		return cancelButton;
	}

	/**
	 * Answer my calendar date view.
	 *
	 * @return JunCalendarTimeIntervalViewAwt
	 * @category accessing
	 */
	protected JunCalendarTimeIntervalViewAwt calendarTimeIntervalView() {
		if (calendarTimeIntervalView == null) {
			calendarTimeIntervalView =
				new JunCalendarTimeIntervalViewAwt(
					this
						.getCalendarTimeIntervalDialog()
						.calendarTimeIntervalModel());
		}
		return calendarTimeIntervalView;
	}
}
