package sra.jun.goodies.calendar;

import java.awt.Toolkit;
import java.awt.datatransfer.StringSelection;
import java.util.Calendar;

import sra.smalltalk.*;

/**
 * JunCalendarTimeModel class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun472 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarTimeModel.java,v 6.0 2003/10/28 10:52:07 nisinaka Exp $
 */
public class JunCalendarTimeModel extends JunCalendarModel {
	protected Calendar calendarTime;
	protected Calendar originalTime;
	protected StMenu popupMenu;
	protected StBlockClosure validateBlock;

	/**
	 * JunCalendarTimeModel.Hours_minutes_seconds_(new Integer(23), new Integer(59), new Integer(59)) *
	 *
	 * @param hour int
	 * @param minute int
	 * @param second int
	 * @return JunCalendarTimeModel
	 * @category utilities
	 */
	public static JunCalendarTimeModel Hours_minutes_seconds_(
		int hour,
		int minute,
		int second) {
		Calendar aTime = Calendar.getInstance();
		aTime.set(
			aTime.get(Calendar.YEAR),
			aTime.get(Calendar.MONTH),
			aTime.get(Calendar.DATE),
			Math.max(0, Math.min(hour, 23)),
			Math.max(0, Math.min(minute, 59)),
			Math.max(0, Math.min(second, 59)));
		return JunCalendarTimeModel.Time_(aTime);
	}

	/**
	 * JunCalendarTimeModel.time_(Calendar.getInstance())
	 *
	 * @param aTime java.util.Calendar
	 * @return JunCalendarTimeModel
	 * @category utilities
	 */
	public static JunCalendarTimeModel Time_(Calendar aTime) {
		return new JunCalendarTimeModel(aTime);
	}

	/**
	 * Create instance.
	 * @category instance creation
	 */
	public JunCalendarTimeModel() {
		this.initialize();
	}

	/**
	 * Create instance and set time aTime.
	 *
	 * @param aTime java.util.Calendar
	 * @category instance creation
	 */
	public JunCalendarTimeModel(Calendar aTime) {
		this.initialize_(aTime);
	}

	/**
	 * Copy time.
	 * @category menu messages
	 */
	public void copyTime() {
		StringBuffer buf = new StringBuffer();
		buf.append(this.hours());
		buf.append(':');
		buf.append(this.minutes());
		buf.append(':');
		buf.append(this.seconds());
		buf.append(" pm");
		Toolkit.getDefaultToolkit().getSystemClipboard().setContents(
			new StringSelection(buf.toString()),
			null);
	}

	/**
	 * Return default view.
	 *
	 * @return JunCalendarTimeView
	 * @category defaults
	 */
	public StView defaultView() {
		JunCalendarTimeView aView;
		if (GetDefaultViewMode() == VIEW_AWT) {
			aView = new JunCalendarTimeViewAwt(this);
		} else {
			aView = new JunCalendarTimeViewSwing(this);
		}
		return aView;
	}

	/**
	 * Action hour down.
	 * @category menu messages
	 */
	public void hourDown() {
		Calendar newTime;
		newTime =
			JunCalendarTimeModel
				.Hours_minutes_seconds_(
					this.hours() - 1 < 0 ? 23 : this.hours() - 1,
					this.minutes(),
					this.seconds())
				.time();
		this.time_(newTime);
	}

	/**
	 * Return hour.
	 *
	 * @return int
	 * @category accessing
	 */
	public int hours() {
		return this.time().get(Calendar.HOUR)
			+ (this.time().get(Calendar.AM_PM) == 1 ? 12 : 0);
	}

	/**
	 * Set hours.
	 *
	 * @param hour int
	 * @category accessing
	 */
	public void hours_(int hour) {
		Calendar newTime;
		if (!(0 <= hour && hour <= 23)) {
			return;
		}
		newTime =
			JunCalendarTimeModel
				.Hours_minutes_seconds_(hour, this.minutes(), this.seconds())
				.time();
		this.time_(newTime);
	}

	/**
	 * Action hour up.
	 * @category menu messages
	 */
	public void hourUp() {
		Calendar newTime;
		newTime =
			JunCalendarTimeModel
				.Hours_minutes_seconds_(
					this.hours() + 1 > 23 ? 0 : this.hours() + 1,
					this.minutes(),
					this.seconds())
				.time();
		this.time_(newTime);
	}

	/**
	 * Initialize and set time today.
	 * @category initialize release
	 */
	public void initialize() {
		this.initialize_(Calendar.getInstance());
	}

	/**
	 * Initialize and set time aTime.
	 *
	 * @param aTime java.util.Calendar
	 * @category initialize release
	 */
	public void initialize_(Calendar aTime) {
		super.initialize();
		this.time_(aTime);
		originalTime = aTime;
	}

	/**
	 * Action minute down.
	 * @category menu messages
	 */
	public void minuteDown() {
		Calendar newTime;
		newTime =
			JunCalendarTimeModel
				.Hours_minutes_seconds_(
					this.minutes() - 1 < 0
						? (this.hours() - 1 < 0 ? 23 : this.hours() - 1)
						: this.hours(),
					this.minutes() - 1 < 0 ? 59 : this.minutes() - 1,
					this.seconds())
				.time();
		this.time_(newTime);
	}

	/**
	 * Return minutes.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int minutes() {
		return this.time().get(Calendar.MINUTE);
	}

	/**
	 * Set minutes.
	 *
	 * @param minute int
	 * @category accessing
	 */
	public void minutes_(int minute) {
		Calendar newTime;
		if (!(0 <= minute && minute <= 59)) {
			return;
		}
		newTime =
			JunCalendarTimeModel
				.Hours_minutes_seconds_(this.hours(), minute, this.seconds())
				.time();
		this.time_(newTime);
	}

	/**
	 * Action minute up.
	 * @category menu messages
	 */
	public void minuteUp() {
		Calendar newTime;
		newTime =
			JunCalendarTimeModel
				.Hours_minutes_seconds_(
					this.minutes() + 1 > 59
						? (this.hours() + 1 > 23 ? 0 : this.hours() + 1)
						: this.hours(),
					this.minutes() + 1 > 59 ? 0 : this.minutes() + 1,
					this.seconds())
				.time();
		this.time_(newTime);
	}

	/**
	 * Set to now time.
	 * @category menu messages
	 */
	public void nowTime() {
		this.time_(Calendar.getInstance());
	}

	/**
	 * Return Original time.
	 *
	 * @return java.util.Calendar
	 * @category menu messages
	 */
	public Calendar original() {
		return originalTime;
	}

	/**
	 * Set to original time.
	 */
	public void originalTime() {
		this.time_(this.original());
	}

	/**
	 * Return the popup menu.
	 * 
	 * @return sra.smalltalk.StMenu
	 * @category menu accessing
	 */
	public StMenu popupMenu() {
		if (popupMenu == null) {
			popupMenu = new StMenu("");
			popupMenu.add(createMenuItem("Copy", "copyTime"));
			popupMenu.add(createMenuItem("Now", "nowTime"));
			popupMenu.add(createMenuItem("Original time", "originalTime"));
			popupMenu.add(createMenuItem("Hour down", "hourDown"));
			popupMenu.add(createMenuItem("Hour up", "hourUp"));
			popupMenu.add(createMenuItem("Minute down", "minuteDown"));
			popupMenu.add(createMenuItem("Minute up", "minuteUp"));
			popupMenu.add(createMenuItem("Second down", "secondDown"));
			popupMenu.add(createMenuItem("Second up", "secondUp"));
		}
		return popupMenu;
	}

	/**
	 * Action second down.
	 * @category menu messages
	 */
	public void secondDown() {
		Calendar newTime;
		newTime =
			JunCalendarTimeModel
				.Hours_minutes_seconds_(
					this.seconds() - 1 < 0
						&& this.minutes() - 1 < 0
							? (this.hours() - 1 < 0 ? 23 : this.hours() - 1)
							: this.hours(),
					this.seconds() - 1 < 0
						? (this.minutes() - 1 < 0 ? 59 : this.minutes() - 1)
						: this.minutes(),
					this.seconds() - 1 < 0 ? 59 : this.seconds() - 1)
				.time();
		this.time_(newTime);
	}

	/**
	 * Return seconds.
	 *
	 * @return int
	 * @category accessing
	 */
	public int seconds() {
		return this.time().get(Calendar.SECOND);
	}

	/**
	 * Set seconds.
	 *
	 * @param second int
	 * @category accessing
	 */
	public void seconds_(int second) {
		Calendar newTime;
		if (!(0 <= second && second <= 59)) {
			return;
		}
		newTime =
			JunCalendarTimeModel
				.Hours_minutes_seconds_(this.hours(), this.minutes(), second)
				.time();
		this.time_(newTime);
	}

	/**
	 * Action second up.
	 * @category menu messages
	 */
	public void secondUp() {
		Calendar newTime;
		newTime =
			JunCalendarTimeModel
				.Hours_minutes_seconds_(
					this.seconds() + 1 > 59
						&& this.minutes() + 1 > 59
							? (this.hours() + 1 > 23 ? 0 : this.hours() + 1)
							: this.hours(),
					this.seconds() + 1 > 59
						? (this.minutes() + 1 > 59 ? 0 : this.minutes() + 1)
						: this.minutes(),
					this.seconds() + 1 > 59 ? 0 : this.seconds() + 1)
				.time();
		this.time_(newTime);
	}

	/**
	 * Set time.
	 *
	 * @param aTime java.util.Calendar
	 * @category accessing
	 */
	public void setTime_(Calendar aTime) {
		calendarTime = aTime;
	}

	/**
	 * Return time.
	 *
	 * @return java.util.Calendar
	 * @category accessing
	 */
	public Calendar time() {
		return calendarTime;
	}

	/**
	 * Set time.
	 *
	 * @param aTime java.util.Calendar
	 * @category accessing
	 */
	public void time_(Calendar aTime) {
		Calendar oldTime = Calendar.getInstance(), newTime;
		if (this.time() != null) {
			oldTime = (Calendar) this.time().clone();
		}
		newTime = (Calendar) this.validateBlock().value_value_(oldTime, aTime);
		if (newTime == null) {
			return;
		}
		calendarTime = newTime;
		if (oldTime == null) {
			this.changed_($("time"));
			return;
		}
		if (!(oldTime.get(Calendar.HOUR) == calendarTime.get(Calendar.HOUR))) {
			this.changed_($("hours"));
			return;
		}
		if (!(oldTime.get(Calendar.MINUTE)
			== calendarTime.get(Calendar.MINUTE))) {
			this.changed_($("minutes"));
			return;
		}
		if (!(oldTime.get(Calendar.SECOND)
			== calendarTime.get(Calendar.SECOND))) {
			this.changed_($("seconds"));
			return;
		}
	}

	/**
	 * Return current time info.
	 *
	 * @return java.lang.String
	 * @category printing
	 */
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append(this.hours());
		buf.append(":");
		buf.append(this.minutes());
		buf.append(":");
		buf.append(this.seconds());
		if (this.time().get(Calendar.AM_PM) == 1) {
			buf.append(" PM");
		} else {
			buf.append(" AM");
		}
		return buf.toString();
	}

	/**
	 * Return aBlock.
	 *
	 * @return sra.smalltalk.StBlockClosure
	 * @category accessing
	 */
	public StBlockClosure validateBlock() {
		if (validateBlock == null) {
			validateBlock = new StBlockClosure() {
				public Object value_value_(Object oldTime, Object newTime) {
					return newTime;
				}
			};
		}
		return validateBlock;
	}

	/**
	 * Set aBlock.
	 *
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @category accessing
	 */
	public void validateBlock_(StBlockClosure aBlock) {
		validateBlock = aBlock;
	}
}
