package sra.jun.goodies.calendar;

import java.awt.*;
import java.util.*;

import sra.smalltalk.*;

/**
 * JunCalendarTimeViewAwt class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun472 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarTimeViewAwt.java,v 6.0 2003/10/28 10:52:07 nisinaka Exp $
 */
public class JunCalendarTimeViewAwt
	extends StViewCanvas
	implements JunCalendarTimeView {
	protected Vector frames;
	protected PopupMenu _popupMenu;
	/**
	 * Create instance.
	 * 
	 * @param aCalendarTimeModel JunCalendarTimeModel
	 * @category instance creation
	 */
	public JunCalendarTimeViewAwt(JunCalendarTimeModel aCalendarTimeModel) {
		super(aCalendarTimeModel);
	}

	/**
	 * Return a frame.
	 * @return java.util.Vector
	 * @category accessing
	 */
	public Vector frames() {
		if (frames == null) {
			this.makeFrames();
		}
		return frames;
	}

	/**
	 * Make frames.
	 * @category private
	 */
	private void makeFrames() {
		Vector collection = new Vector();
		int width = this.getSize().width / 10;
		int height = this.getSize().height / 8;
		StRectangle button = new StRectangle(0, 0, 12, 16);
		int x = width * 0;
		int y = 0;
		StRectangle box = new StRectangle(x, y, width * 4, height);
		box = box.insetBy_(new StRectangle(8, 0, 0, 0));
		Object[] set = new Object[3];
		set[0] = "label";
		set[1] = "hours:";
		set[2] = box;
		collection.add(set);
		x = width * 5;
		y = height * 0;
		box =
			StRectangle.Origin_extent_(
				new Point(x, y),
				new Point(width * 5, height));
		box =
			box.insetBy_(
				new StRectangle(button.width(), 0, button.width() + 8, 0));
		set = new Object[2];
		set[0] = "hours";
		set[1] = box;
		collection.add(set);
		button = button.align_with_(button.rightCenter(), box.leftCenter());
		set = new Object[3];
		set[0] = "button";
		set[1] = "hourDown";
		set[2] = button;
		collection.add(set);
		button = button.align_with_(button.leftCenter(), box.rightCenter());
		set = new Object[3];
		set[0] = "button";
		set[1] = "hourUp";
		set[2] = button;
		collection.add(set);
		x = width * 0;
		y = height * 1;
		box =
			StRectangle.Origin_extent_(
				new Point(x, y),
				new Point(width * 4, height));
		box = box.insetBy_(new StRectangle(8, 0, 0, 0));
		set = new Object[3];
		set[0] = "label";
		set[1] = "minutes:";
		set[2] = box;
		collection.add(set);
		x = width * 5;
		y = height * 1;
		box =
			StRectangle.Origin_extent_(
				new Point(x, y),
				new Point(width * 5, height));
		box =
			box.insetBy_(
				new StRectangle(button.width(), 0, button.width() + 8, 0));
		set = new Object[2];
		set[0] = "minutes";
		set[1] = box;
		collection.add(set);
		button = button.align_with_(button.rightCenter(), box.leftCenter());
		set = new Object[3];
		set[0] = "button";
		set[1] = "minuteDown";
		set[2] = button;
		collection.add(set);
		button = button.align_with_(button.leftCenter(), box.rightCenter());
		set = new Object[3];
		set[0] = "button";
		set[1] = "minuteUp";
		set[2] = button;
		collection.add(set);
		x = width * 0;
		y = height * 2;
		box =
			StRectangle.Origin_extent_(
				new Point(x, y),
				new Point(width * 4, height));
		box = box.insetBy_(new StRectangle(8, 0, 0, 0));
		set = new Object[3];
		set[0] = "label";
		set[1] = "seconds:";
		set[2] = box;
		collection.add(set);
		x = width * 5;
		y = height * 2;
		box =
			StRectangle.Origin_extent_(
				new Point(x, y),
				new Point(width * 5, height));
		box =
			box.insetBy_(
				new StRectangle(button.width(), 0, button.width() + 8, 0));
		set = new Object[2];
		set[0] = "seconds";
		set[1] = box;
		collection.add(set);
		button = button.align_with_(button.rightCenter(), box.leftCenter());
		set = new Object[3];
		set[0] = "button";
		set[1] = "secondDown";
		set[2] = button;
		collection.add(set);
		button = button.align_with_(button.leftCenter(), box.rightCenter());
		set = new Object[3];
		set[0] = "button";
		set[1] = "secondUp";
		set[2] = button;
		collection.add(set);
		x = width * 0;
		y = height * 3;
		box =
			StRectangle.Origin_extent_(
				new Point(x, y),
				new Point(width * 10, height * 5));
		set = new Object[2];
		set[0] = "clock";
		set[1] = box;
		collection.add(set);
		frames = collection;
	}

	/**
	 * Return rotateBlock.
	 *
	 * @return StBlockClosure
	 * @category accessing
	 */
	public StBlockClosure rotateBlock() {
		StBlockClosure block = new StBlockClosure() {
			public Object value_value_(Object p, Object d) {
				double radians, sin, cos, x, y;
				Point point = (Point) p;
				radians =
					Math.toRadians(((java.lang.Double) d).doubleValue() * -1.0);
				sin = Math.sin(radians);
				cos = Math.cos(radians);
				x = point.x * cos + point.y * sin;
				y = -1.0 * point.x * sin + point.y * cos;
				return (Object) (new Point((int) x, (int) y));
			}
		};
		return block;
	}

	/**
	 * paint.
	 * @param aGraphics java.awt.Graphics
	 * @category displaying
	 */
	public void displayOn_(Graphics aGraphics) {
		this.displayBordersOn_(aGraphics);
		this.displayObjectsOn_(aGraphics);
	}

	/**
	 * Paint borders.
	 * @param aGraphics java.awt.Graphics
	 * @category displaying
	 */
	public void displayBordersOn_(Graphics aGraphics) {
		String key;
		StRectangle box;
		StRectangle area;
		StRectangle clip;
		for (int i = 0; i < this.frames().size(); ++i) {
			Object[] array = (Object[]) this.frames().get(i);
			key = (String) array[0];
			box = (StRectangle) array[array.length - 1];
			clip = new StRectangle(aGraphics.getClipBounds());
			if (clip.intersects_(box)) {
				if (key.equals("hours")
					|| key.equals("minutes")
					|| key.equals("seconds")
					|| key.equals("clock")) {
					aGraphics.setColor(veryLightGray);
					aGraphics.fillRect(
						box.x(),
						box.y(),
						box.width(),
						box.height());
					area = box.insetBy_(new StRectangle(1, 1, 2, 2));
					aGraphics.setColor(veryDarkGray);
					aGraphics.drawRect(
						area.x(),
						area.y(),
						area.width(),
						area.height());
					if (!key.equals("clock")) {
						area = box.insetBy_(new StRectangle(2, 2, 3, 3));
						aGraphics.setColor(Color.black);
						aGraphics.drawRect(
							area.x(),
							area.y(),
							area.width(),
							area.height());
					}
				}
			}
		}
	}

	/**
	 * Paint objects.
	 * @param aGraphics java.awt.Graphics
	 * @category displaying
	 */
	public void displayObjectsOn_(Graphics aGraphics) {
		String key;
		StRectangle box;
		for (int i = 0; i < this.frames().size(); ++i) {
			Object[] array = (Object[]) this.frames().get(i);
			key = (String) array[0];
			box = (StRectangle) array[array.length - 1];
			if (key.equals("hours")) {
				this.displayOn_hours_clip_(aGraphics, key, box);
			}
			if (key.equals("minutes")) {
				this.displayOn_minutes_clip_(aGraphics, key, box);
			}
			if (key.equals("seconds")) {
				this.displayOn_seconds_clip_(aGraphics, key, box);
			}
			if (key.equals("label")) {
				this.displayOn_label_clip_(aGraphics, (String) array[1], box);
			}
			if (key.equals("button")) {
				this.displayOn_button_clip_(aGraphics, (String) array[1], box);
			}
			if (key.equals("clock")) {
				this.displayOn_clock_clip_(aGraphics, key, box);
			}
		}
	}

	/**
	 * Paint hours.
	 * @param aGraphics Graphics
	 * @param aSymbol String
	 * @param aRectangle rect
	 * @category displaying
	 */
	public void displayOn_hours_clip_(
		Graphics aGraphics,
		String aSymbol,
		StRectangle aRectangle) {
		int hours = this.getModel().hours();
		StComposedText para;
		StRectangle rect, clip;
		String string = String.valueOf(hours);
		if (string.length() < 2) {
			string = "0".concat(string);
		}
		para = new StComposedText(string);
		StRectangle bounds = new StRectangle(para.bounds());
		rect = bounds.align_with_(bounds.center(), aRectangle.center());
		clip = new StRectangle(aGraphics.getClipBounds());
		if (clip.intersects_(rect)) {
			aGraphics.setColor(Color.black);
			para.displayOn_at_(aGraphics, rect.origin());
		}
	}

	/**
	 * Paint minutes.
	 * @param aGraphics Graphics
	 * @param aSymbol String
	 * @param aRectangle rect
	 * @category displaying
	 */
	public void displayOn_minutes_clip_(
		Graphics aGraphics,
		String aSymbol,
		StRectangle aRectangle) {
		int minutes = this.getModel().minutes();
		StComposedText para;
		StRectangle rect, clip;
		String string = String.valueOf(minutes);
		if (string.length() < 2) {
			string = "0".concat(string);
		}
		para = new StComposedText(string);
		StRectangle bounds = new StRectangle(para.bounds());
		rect = bounds.align_with_(bounds.center(), aRectangle.center());
		clip = new StRectangle(aGraphics.getClipBounds());
		if (clip.intersects_(rect)) {
			aGraphics.setColor(Color.black);
			para.displayOn_at_(aGraphics, rect.origin());
		}
	}

	/**
	 * Paint seconds.
	 * @param aGraphics Graphics
	 * @param aSymbol String
	 * @param aRectangle rect
	 * @category displaying
	 */
	public void displayOn_seconds_clip_(
		Graphics aGraphics,
		String aSymbol,
		StRectangle aRectangle) {
		int seconds = this.getModel().seconds();
		StComposedText para;
		StRectangle rect, clip;
		String string = String.valueOf(seconds);
		if (string.length() < 2) {
			string = "0".concat(string);
		}
		para = new StComposedText(string);
		StRectangle bounds = new StRectangle(para.bounds());
		rect = bounds.align_with_(bounds.center(), aRectangle.center());
		clip = new StRectangle(aGraphics.getClipBounds());
		if (clip.intersects_(rect)) {
			aGraphics.setColor(Color.black);
			para.displayOn_at_(aGraphics, rect.origin());
		}
	}

	/**
	 * Paint label.
	 * @param aGraphics Graphics
	 * @param aSymbol String
	 * @param aRectangle rect
	 * @category displaying
	 */
	public void displayOn_label_clip_(
		Graphics aGraphics,
		String aString,
		StRectangle aRectangle) {
		StComposedText para;
		StRectangle rect, clip;
		para = new StComposedText(aString);
		StRectangle bounds = new StRectangle(para.bounds());
		rect = bounds.align_with_(bounds.leftCenter(), aRectangle.leftCenter());
		clip = new StRectangle(aGraphics.getClipBounds());
		if (clip.intersects_(rect)) {
			aGraphics.setColor(Color.black);
			para.displayOn_at_(aGraphics, rect.origin());
		}
	}

	/**
	 * Paint button.
	 * @param aGraphics Graphics
	 * @param aSymbol String
	 * @param aRectangle rect
	 * @category displaying
	 */
	public void displayOn_button_clip_(
		Graphics aGraphics,
		String aSymbol,
		StRectangle aRectangle) {
		Polygon polygon;
		StRectangle box = aRectangle;
		StRectangle clip = new StRectangle(aGraphics.getClipBounds());
		if (clip.intersects_(box)) {
			if (aSymbol.equals("hourDown")
				|| aSymbol.equals("minuteDown")
				|| aSymbol.equals("secondDown")) {
				polygon = new Polygon();
				polygon.addPoint(box.topRight().x, box.topRight().y);
				polygon.addPoint(box.bottomRight().x, box.bottomRight().y);
				polygon.addPoint(box.leftCenter().x, box.leftCenter().y);
				polygon.addPoint(box.topRight().x, box.topRight().y);
			} else {
				polygon = new Polygon();
				polygon.addPoint(box.topLeft().x, box.topLeft().y);
				polygon.addPoint(box.bottomLeft().x, box.bottomLeft().y);
				polygon.addPoint(box.rightCenter().x, box.rightCenter().y);
				polygon.addPoint(box.topLeft().x, box.topLeft().y);
			}
			aGraphics.setColor(veryLightGray);
			aGraphics.fillPolygon(polygon);
			box = box.insetBy_(new StRectangle(1, 1, 2, 2));
			if (aSymbol.equals("hourDown")
				|| aSymbol.equals("minuteDown")
				|| aSymbol.equals("secondDown")) {
				polygon = new Polygon();
				polygon.addPoint(box.topRight().x, box.topRight().y);
				polygon.addPoint(box.bottomRight().x, box.bottomRight().y);
				polygon.addPoint(box.leftCenter().x, box.leftCenter().y);
				polygon.addPoint(box.topRight().x, box.topRight().y);
			} else {
				polygon = new Polygon();
				polygon.addPoint(box.topLeft().x, box.topLeft().y);
				polygon.addPoint(box.bottomLeft().x, box.bottomLeft().y);
				polygon.addPoint(box.rightCenter().x, box.rightCenter().y);
				polygon.addPoint(box.topLeft().x, box.topLeft().y);
			}
			aGraphics.setColor(veryDarkGray);
			aGraphics.drawPolygon(polygon);
		}
	}

	/**
	 * Paint clock.
	 *
	 * @param aGraphics Graphics
	 * @param aSymbol String
	 * @param aRectangle StRectangle
	 * @category displaying
	 */
	public void displayOn_clock_clip_(
		Graphics aGraphics,
		String aSymbol,
		StRectangle aRectangle) {
		this.displayOn_clockBackground_clip_(aGraphics, aSymbol, aRectangle);
		this.displayOn_clockShortHand_clip_(aGraphics, aSymbol, aRectangle);
		this.displayOn_clockLongHand_clip_(aGraphics, aSymbol, aRectangle);
		this.displayOn_clockTickHand_clip_(aGraphics, aSymbol, aRectangle);
	}

	/**
	 * Paint clock background.
	 *
	 * @param aGraphics Graphics
	 * @param aSymbol String
	 * @param aRectangle StRectangle
	 * @category displaying
	 */
	public void displayOn_clockBackground_clip_(
		Graphics aGraphics,
		String aSymbol,
		StRectangle aRectangle) {
		StRectangle clip, rect;
		Point center;
		int radius, degree;
		double x, y;
		StComposedText paragraph;
		clip = new StRectangle(aGraphics.getClipBounds());
		if (clip.intersects_(aRectangle)) {
			aGraphics.setColor(Color.black);
			center = aRectangle.center();
			radius =
				Math.min(aRectangle.width(), aRectangle.height()) / 2
					- (new StComposedText(" ")).height()
					- 8;
			rect = StRectangle.Origin_extent_(new Point(0, 0), new Point(2, 2));
			for (int number = 1; number <= 12; ++number) {
				paragraph = new StComposedText(String.valueOf(number));
				degree = (number - 3) * 30;
				x = Math.cos(Math.toRadians(degree));
				y = Math.sin(Math.toRadians(degree));
				paragraph.displayOn_at_(
					aGraphics,
					new Point(
						(int) (center.x
							+ 1
							+ x * radius
							- paragraph.extent().width / 2),
						(int) (center.y
							+ 1
							+ y * radius
							- paragraph.extent().height / 2)));
				aGraphics.fillRect(
					(int) (center.x + x * (radius - 10)),
					(int) (center.y + y * (radius - 10)),
					rect.width(),
					rect.height());
			}
			paragraph = new StComposedText("am");
			rect = new StRectangle(paragraph.bounds());
			rect =
				rect.align_with_(
					rect.bottomLeft(),
					aRectangle.insetBy_(8).bottomLeft());
			if (this.getModel().time().get(Calendar.AM_PM) != 0) {
				aGraphics.setColor(Color.lightGray);
			} else {
				aGraphics.setColor(Color.black);
			}
			paragraph.displayOn_at_(aGraphics, rect.origin());
			paragraph = new StComposedText("pm");
			rect = new StRectangle(paragraph.bounds());
			rect =
				rect.align_with_(
					rect.bottomRight(),
					aRectangle.insetBy_(8).bottomRight());
			if (this.getModel().time().get(Calendar.AM_PM) > 0) {
				aGraphics.setColor(Color.black);
			} else {
				aGraphics.setColor(Color.lightGray);
			}
			paragraph.displayOn_at_(aGraphics, rect.origin());
		}
	}

	/**
	 * Paint clock short hand.
	 *
	 * @param aGraphics Graphics
	 * @param aSymbol String
	 * @param aRectangle StRectangle
	 * @category displaying
	 */
	public void displayOn_clockShortHand_clip_(
		Graphics aGraphics,
		String aSymbol,
		StRectangle aRectangle) {
		StRectangle clip;
		Point center;
		double radius, x, y;
		int minute, hour;
		Polygon polygon = new Polygon();
		clip = new StRectangle(aGraphics.getClipBounds());
		if (clip.intersects_(aRectangle)) {
			center = aRectangle.center();
			radius =
				Math.min(aRectangle.width(), aRectangle.height()) / 2
					- (new StComposedText(" ")).height()
					- 8;
			minute = this.getModel().minutes();
			hour = this.getModel().hours() * 5 + minute / 12;
			x = Math.cos(Math.toRadians(hour * 6 - 90));
			y = Math.sin(Math.toRadians(hour * 6 - 90));
			Point p =
				(Point) (this
					.rotateBlock()
					.value_value_(
						(Object) (new Point((int) (x * 3), (int) (y * 3))),
						(Object) (new java.lang.Double(-180))));
			Point first = (Point) p.clone();
			polygon.addPoint((int) (center.x + p.x), (int) (center.y + p.y));
			p =
				(Point) (this
					.rotateBlock()
					.value_value_(
						(Object) (new Point((int) (x * 3), (int) (y * 3))),
						(Object) (new java.lang.Double(-90))));
			polygon.addPoint((int) (center.x + p.x), (int) (center.y + p.y));
			p =
				new Point(
					(int) (x * radius * 6 / 10),
					(int) (y * radius * 6 / 10));
			polygon.addPoint((int) (center.x + p.x), (int) (center.y + p.y));
			p =
				(Point) (this
					.rotateBlock()
					.value_value_(
						(Object) (new Point((int) (x * 3), (int) (y * 3))),
						(Object) (new java.lang.Double(90))));
			polygon.addPoint((int) (center.x + p.x), (int) (center.y + p.y));
			polygon.addPoint(
				(int) (center.x + first.x),
				(int) (center.y + first.y));
			aGraphics.setColor(Color.lightGray);
			((Graphics2D) aGraphics).setStroke(
				new BasicStroke(
					1,
					BasicStroke.CAP_BUTT,
					BasicStroke.JOIN_MITER));
			aGraphics.fillPolygon(polygon);
			aGraphics.setColor(Color.black);
			aGraphics.drawPolygon(polygon);
		}
	}

	/**
	 * Paint clock long hand.
	 *
	 * @param aGraphics Graphics
	 * @param aSymbol String
	 * @param aRectangle StRectangle
	 * @category displaying
	 */
	public void displayOn_clockLongHand_clip_(
		Graphics aGraphics,
		String aSymbol,
		StRectangle aRectangle) {
		StRectangle clip;
		Point center;
		double radius, x, y;
		int minute;
		Polygon polygon = new Polygon();
		clip = new StRectangle(aGraphics.getClipBounds());
		if (clip.intersects_(aRectangle)) {
			center = aRectangle.center();
			radius =
				Math.min(aRectangle.width(), aRectangle.height()) / 2
					- (new StComposedText(" ")).height()
					- 8;
			minute = this.getModel().minutes();
			x = Math.cos(Math.toRadians(minute * 6 - 90));
			y = Math.sin(Math.toRadians(minute * 6 - 90));
			Point p =
				(Point) (this
					.rotateBlock()
					.value_value_(
						(Object) (new Point((int) (x * 3), (int) (y * 3))),
						(Object) (new java.lang.Double(-180))));
			Point first = (Point) p.clone();
			polygon.addPoint(center.x + p.x, center.y + p.y);
			p =
				(Point) (this
					.rotateBlock()
					.value_value_(
						(Object) (new Point((int) (x * 3), (int) (y * 3))),
						(Object) (new java.lang.Double(-90))));
			polygon.addPoint(center.x + p.x, center.y + p.y);
			p =
				new Point(
					(int) (x * radius * 9 / 10),
					(int) (y * radius * 9 / 10));
			polygon.addPoint(center.x + p.x, center.y + p.y);
			p =
				(Point) (this
					.rotateBlock()
					.value_value_(
						(Object) (new Point((int) (x * 3), (int) (y * 3))),
						(Object) (new java.lang.Double(90))));
			polygon.addPoint(center.x + p.x, center.y + p.y);
			polygon.addPoint(center.x + first.x, center.y + first.y);
			aGraphics.setColor(Color.lightGray);
			((Graphics2D) aGraphics).setStroke(
				new BasicStroke(
					1,
					BasicStroke.CAP_BUTT,
					BasicStroke.JOIN_MITER));
			aGraphics.fillPolygon(polygon);
			aGraphics.setColor(Color.black);
			aGraphics.drawPolygon(polygon);
		}
	}

	/**
	 * Paint clock tick hand.
	 *
	 * @param aGraphics Graphics
	 * @param aSymbol String
	 * @param aRectangle StRectangle
	 * @category displaying
	 */
	public void displayOn_clockTickHand_clip_(
		Graphics aGraphics,
		String aSymbol,
		StRectangle aRectangle) {
		StRectangle clip;
		Point center;
		double radius, x, y;
		int seconds, length;
		clip = new StRectangle(aGraphics.getClipBounds());
		if (clip.intersects_(aRectangle)) {
			center = aRectangle.center();
			radius =
				Math.min(aRectangle.width(), aRectangle.height()) / 2
					- (new StComposedText(" ")).height()
					- 8;
			length = (int) (radius * 9.5 / 10);
			seconds = this.getModel().seconds();
			x = Math.cos(Math.toRadians(seconds * 6 - 90));
			y = Math.sin(Math.toRadians(seconds * 6 - 90));
			aGraphics.setColor(Color.lightGray);
			((Graphics2D) aGraphics).setStroke(
				new BasicStroke(
					1,
					BasicStroke.CAP_BUTT,
					BasicStroke.JOIN_MITER));
			aGraphics.setColor(Color.black);
			aGraphics.drawLine(
				center.x,
				center.y,
				(int) (center.x + x * length),
				(int) (center.y + y * length));
		}
	}

	/**
	 * Update.
	 *
	 * @param evt DependentEvent
	 * @category updating
	 */
	public void update_(DependentEvent evt) {
		this.makeFrames();
		if (evt.getAspect() == $("hours")) {
			repaint();
		}
		if (evt.getAspect() == $("minutes")) {
			repaint();
		}
		if (evt.getAspect() == $("seconds")) {
			repaint();
		}
		super.update_(evt);
	}

	/**
	 * Build component.
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setSize(150, 200);
	}

	/**
	 * Get model.
	 *
	 * @return JunCalendarTimeModel
	 * @category accessing
	 */
	public JunCalendarTimeModel getModel() {
		return (JunCalendarTimeModel) this.model();
	}

	/**
	 * Get default controller.
	 *
	 * @return JunCalendarTimeController
	 * @category accessing
	 */
	protected StController defaultController() {
		return new JunCalendarTimeController();
	}

	/**
	 * Get default model.
	 *
	 * @return sra.smalltalk.StModel
	 * @category accessing
	 */
	protected StModel defaultModel() {
		return new JunCalendarTimeModel();
	}

	/**
	 * Answer the popup menu.
	 *
	 * @return java.awt.PopupMenu
	 * @category menu accessing
	 */
	protected PopupMenu popupMenu() {
		if (_popupMenu == null) {
			_popupMenu = this._createPopupMenu(this.getModel().popupMenu());
			this.add(_popupMenu);
		}
		return _popupMenu;
	}
}
