package sra.jun.goodies.catalog;

import java.awt.*;
import java.io.*;
import sra.jun.goodies.cursors.JunCursors;
import sra.jun.goodies.images.streams.*;
import sra.jun.goodies.images.support.JunImageAdjuster;
import sra.jun.goodies.movie.JunMoviePlayer;
import sra.jun.system.support.JunSystem;
import sra.smalltalk.*;

/**
 * JunImageCatalog class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2003/05/21 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun462 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunImageCatalog.java,v 6.0 2003/10/28 10:52:08 nisinaka Exp $
 */
public class JunImageCatalog extends JunVisualCatalog {
	protected int[] visualClip;

	/**
	 * The default constructor.
	 */
	public JunImageCatalog() {
		super();
	}

	/**
	 * Create a new instance of JunImageCatalog and set a directory.
	 * 
	 * @param directoryFilename java.io.File
	 */
	public JunImageCatalog(File directoryFilename) {
		super(directoryFilename);
	}

	/**
	 * Create a new instance of JunImageCatalog and set a directory.
	 * 
	 * @param fromDirectoryFilename java.io.File
	 * @param toDirectoryFilename java.io.File
	 */
	public JunImageCatalog(File fromDirectoryFilename, File toDirectoryFilename) {
		super(fromDirectoryFilename, toDirectoryFilename);
	}

	/**
	 * Create a new instance of JunImageCatalog and dive to a directory.
	 * 
	 * @return sra.jun.goodies.catalog.JunVisualCatalog
	 * @param directoryFilename java.io.File
	 */
	public static JunVisualCatalog Dive_(File directoryFilename) {
		return _Dive_catalog_(directoryFilename, new JunImageCatalog());
	}

	/**
	 * Create a new instance of JunImageCatalog and dive to a directory.
	 * 
	 * @return sra.jun.goodies.catalog.JunVisualCatalog
	 * @param directoryFilename java.io.File
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public static JunVisualCatalog Dive_block_(File directoryFilename, StBlockClosure aBlock) {
		return _Dive_block_catalog_(directoryFilename, aBlock, new JunImageCatalog());
	}

	/**
	 * Answer the default title string.
	 * 
	 * @return java.lang.String
	 */
	public String defaultTitleString() {
		return "Image Catalog";
	}

	/**
	 * Answer the default visual catalog directory name.
	 * 
	 * @return java.lang.String
	 */
	public String defaultVisualCatalogDirectoryName() {
		return "ImageCatalog";
	}

	/**
	 * Answer the default visual clip directory name.
	 * 
	 * @return java.lang.String
	 */
	public String defaultVisualClipDirectoryName() {
		return "images";
	}

	/**
	 * Answer the image clip extent.
	 * 
	 * @return java.awt.Dimension
	 * @param anImage sra.smalltalk.StImage
	 */
	public Dimension imageClipExtent_(StImage anImage) {
		Dimension imageExtent = null;
		if (this.visualClip() == null) {
			imageExtent = anImage.bounds().getSize();
		} else {
			imageExtent = this.proportionalExtentOfImage_within_(anImage, new Point(this.visualClip()[0], this.visualClip()[1]));
		}
		return imageExtent;
	}

	/**
	 * Answer the visual clip.
	 * 
	 * @return int[]
	 */
	public int[] visualClip() {
		return visualClip;
	}

	/**
	 * Set the visual clip.
	 * 
	 * @param anArray int[]
	 */
	public void visualClip_(int[] anArray) {
		visualClip = anArray;
	}

	/**
	 * Set the visual clip.
	 * 
	 * @param anArray java.awt.Point
	 */
	public void visualClip_(Point anArray) {
		visualClip = new int[] { anArray.x, anArray.y };
	}

	/**
	 * Answer the visual clip files.
	 * 
	 * @return java.io.File[]
	 */
	public File[] visualClipFiles() {
		File[] originalVisualFiles = this.originalVisualFiles();
		File[] visualClipFiles = new File[originalVisualFiles.length];
		for (int i = 0; i < originalVisualFiles.length; i++) {
			File each = originalVisualFiles[i];
			String aString = each.getName();
			aString = aString.substring(0, aString.lastIndexOf('.')) + "." + JunSystem.DefaultImageExtension();
			File aFilename = new File(this.visualClipDirectory(), aString);
			visualClipFiles[i] = aFilename;
		}

		return visualClipFiles;
	}

	/**
	 * Initialize this when created.
	 */
	protected void initialize() {
		super.initialize();
		visualClip = null;
	}

	/**
	 * Make visual clip files.
	 */
	protected void makeVisualClipFiles() {
		progressValue.value_(0.0f);
		int anIndex = 1;
		File[] originalVisualFiles = this.originalVisualFiles();
		File[] visualClipFiles = this.visualClipFiles();
		JunCursors cursor = new JunCursors(JunCursors.WriteCursor());

		for (int i = 0; i < originalVisualFiles.length; i++) {
			String aString = "Image:" + " " + visualClipFiles[i].getName();
			progressMessage.value_(this._contractTo_(aString, this.defaultContractLength()));

			if (originalVisualFiles[i].equals(visualClipFiles[i]) == false) {
				StImage anImage = null;
				JunMoviePlayer aMoviePlayer = null;
				try {
					aMoviePlayer = new JunMoviePlayer(originalVisualFiles[i]);
					anImage = new StImage(aMoviePlayer.asImage());
				} finally {
					if (aMoviePlayer != null) {
						aMoviePlayer.release();
						aMoviePlayer = null;
					}
				}

				JunImageAdjuster anAdjuster = new JunImageAdjuster();
				Dimension imageExtent = this.imageClipExtent_(anImage);
				anAdjuster.adjust_extent_(anImage, imageExtent);

				JunImageStream writeStream = null;
				try {
					cursor._show();
					writeStream = JunJpegImageStream.On_(new BufferedOutputStream(new FileOutputStream(visualClipFiles[i].getPath())));
					writeStream.nextPutImage_(anImage);
				} catch (IOException e) {
					System.out.println(e.getMessage());
					e.printStackTrace();
				} finally {
					if (writeStream != null) {
						try {
							writeStream.flush();
							writeStream.close();
						} catch (IOException e) {
						} finally {
							writeStream = null;
						}
					}
					cursor._restore();
				}
			}

			progressValue.value_(((float) anIndex) / this.numberOfVisuals());
			anIndex = anIndex + 1;
		}
	}
}
