package sra.jun.goodies.catalog;

import java.awt.Point;
import java.io.File;
import sra.jun.goodies.files.JunFileDialog;
import sra.jun.goodies.progress.JunProgress;
import sra.jun.system.support.JunTestExamples;
import sra.smalltalk.*;

/**
 * JunImageCatalogTestExamples class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2003/05/21 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun462 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunImageCatalogTestExamples.java,v 6.0 2003/10/28 10:52:08 nisinaka Exp $
 */
public class JunImageCatalogTestExamples extends JunTestExamples {

	/**
	 * Example 1.
	 * 
	 * @return boolean
	 */
	public static boolean Example1() {
		JunImageCatalog imageCatalog = new JunImageCatalog();
		imageCatalog.make();
		imageCatalog.show();

		return true;
	}

	/**
	 * Example 2.
	 * 
	 * @return boolean
	 */
	public static boolean Example2() {
		File aDirectory = JunFileDialog.RequestDirectory("");
		if (aDirectory == null) {
			return true;
		}

		JunImageCatalog imageCatalog = new JunImageCatalog(aDirectory);
		imageCatalog.computeWhenProgressMessageChanged_(new StBlockClosure() {
			public Object value_(Object m) {
				System.out.println((String) m);
				return null;
			}
		});
		imageCatalog.make();
		imageCatalog.show();

		return true;
	}

	/**
	 * Example 3.
	 * 
	 * @return boolean
	 */
	public static boolean Example3() {
		File aDirectory = JunFileDialog.RequestDirectory("");
		if (aDirectory == null) {
			return true;
		}

		final JunImageCatalog imageCatalog = new JunImageCatalog(aDirectory);
		final JunProgress aProgress = new JunProgress();
		imageCatalog.computeWhenProgressMessageChanged_(new StBlockClosure() {
			public Object value_(Object m) {
				aProgress.message_((String) m);
				return null;
			}
		});
		imageCatalog.computeWhenProgressValueChanged_(new StBlockClosure() {
			public Object value_(Object v) {
				aProgress.value_(((Number) v).floatValue());
				return null;
			}
		});
		aProgress.do_(new StBlockClosure() {
			public Object value() {
				imageCatalog.make();
				return null;
			}
		});
		imageCatalog.show();

		return true;
	}

	/**
	 * Example 4.
	 * 
	 * @return boolean
	 */
	public static boolean Example4() {
		File aDirectory = JunFileDialog.RequestDirectory("");
		if (aDirectory == null) {
			return true;
		}

		final JunImageCatalog imageCatalog = new JunImageCatalog(aDirectory);
		imageCatalog.visualClip_(new Point(400, 350));
		imageCatalog.thumbnailImageSize_(32);
		imageCatalog.numberOfColumns_(3);
		imageCatalog.withFileName_(true);
		final JunProgress aProgress = new JunProgress();
		imageCatalog.computeWhenProgressMessageChanged_(new StBlockClosure() {
			public Object value_(Object m) {
				aProgress.message_((String) m);
				return null;
			}
		});
		imageCatalog.computeWhenProgressValueChanged_(new StBlockClosure() {
			public Object value_(Object v) {
				aProgress.value_(((Number) v).floatValue());
				return null;
			}
		});
		aProgress.do_(new StBlockClosure() {
			public Object value() {
				imageCatalog.make();
				return null;
			}
		});
		imageCatalog.show();

		return true;
	}

	/**
	 * Example 5.
	 * 
	 * @return boolean
	 */
	public static boolean Example5() {
		File aDirectory = JunFileDialog.RequestDirectory("");
		if (aDirectory == null) {
			return true;
		}

		JunVisualCatalog anImageCatalog = JunImageCatalog.Dive_(aDirectory);
		anImageCatalog.show();

		return true;
	}

	/**
	 * Example 6.
	 * 
	 * @return boolean
	 */
	public static boolean Example6() {
		File aDirectory = JunFileDialog.RequestDirectory("");
		if (aDirectory == null) {
			return true;
		}

		JunVisualCatalog anImageCatalog = JunImageCatalog.Dive_block_(aDirectory, new StBlockClosure() {
			public Object value_(Object anObject) {
				final JunImageCatalog imageCatalog = (JunImageCatalog) anObject;
				imageCatalog.visualClip_(new Point(128, 128));
				imageCatalog.thumbnailImageSize_(32);
				imageCatalog.numberOfColumns_(3);
				imageCatalog.withFileName_(true);
				final JunProgress aProgress = new JunProgress();
				imageCatalog.computeWhenProgressMessageChanged_(new StBlockClosure() {
					public Object value_(Object m) {
						aProgress.message_((String) m);
						return null;
					}
				});
				imageCatalog.computeWhenProgressValueChanged_(new StBlockClosure() {
					public Object value_(Object v) {
						aProgress.value_(((Number) v).floatValue());
						return null;
					}
				});
				aProgress.do_(new StBlockClosure() {
					public Object value() {
						imageCatalog.make();
						return null;
					}
				});
				return null;
			}
		});
		anImageCatalog.show();

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args an array of command-line arguments
	 */
	public static void main(java.lang.String[] args) {
		new JunImageCatalogTestExamples();
	}
}
