package sra.jun.goodies.catalog;

import java.io.*;
import sra.jun.goodies.cursors.JunCursors;
import sra.jun.goodies.movie.JunMoviePlayer;
import sra.jun.system.support.JunSystem;
import sra.smalltalk.*;

/**
 * JunMovieCatalog class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2003/05/21 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun462 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMovieCatalog.java,v 6.0 2003/10/28 10:52:08 nisinaka Exp $
 */
public class JunMovieCatalog extends JunVisualCatalog {
	protected double[] visualClip;

	/**
	 * The default constructor.
	 */
	public JunMovieCatalog() {
		super();
	}

	/**
	 * Create a new instance of JunMovieCatalog and set a directory.
	 * 
	 * @param directoryFilename java.io.File
	 */
	public JunMovieCatalog(File directoryFilename) {
		super(directoryFilename);
	}

	/**
	 * Create a new instance of JunMovieCatalog and set a directory.
	 * 
	 * @param fromDirectoryFilename java.io.File
	 * @param toDirectoryFilename java.io.File
	 */
	public JunMovieCatalog(File fromDirectoryFilename, File toDirectoryFilename) {
		super(fromDirectoryFilename, toDirectoryFilename);
	}

	/**
	 * Create a new instance of JunMovieCatalog and dive to a directory.
	 * 
	 * @return sra.jun.goodies.catalog.JunVisualCatalog
	 * @param directoryFilename java.io.File
	 */
	public static JunVisualCatalog Dive_(File directoryFilename) {
		return _Dive_catalog_(directoryFilename, new JunMovieCatalog());
	}

	/**
	 * Create a new instance of JunMovieCatalog and dive to a directory.
	 * 
	 * @return sra.jun.goodies.catalog.JunVisualCatalog
	 * @param directoryFilename java.io.File
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public static JunVisualCatalog Dive_block_(File directoryFilename, StBlockClosure aBlock) {
		return _Dive_block_catalog_(directoryFilename, aBlock, new JunMovieCatalog());
	}

	/**
	 * Answer the default title string.
	 * 
	 * @return java.lang.String
	 */
	public String defaultTitleString() {
		return "Movie Catalog";
	}

	/**
	 * Answer the default visual catalog directory name.
	 * 
	 * @return java.lang.String
	 */
	public String defaultVisualCatalogDirectoryName() {
		return "MovieCatalog";
	}

	/**
	 * Answer the default visual clip directory name.
	 * 
	 * @return java.lang.String
	 */
	public String defaultVisualClipDirectoryName() {
		return "movies";
	}

	/**
	 * Answer the visual clip.
	 * 
	 * @return double[]
	 */
	public double[] visualClip() {
		return visualClip;
	}

	/**
	 * Set the visual clip.
	 * 
	 * @param anArray double[]
	 */
	public void visualClip_(double[] anArray) {
		visualClip = anArray;
	}

	/**
	 * Answer the visual clip files.
	 * 
	 * @return java.io.File[]
	 */
	public File[] visualClipFiles() {
		File[] originalVisualFiles = this.originalVisualFiles();
		File[] visualClipFiles = new File[originalVisualFiles.length];
		for (int i = 0; i < originalVisualFiles.length; i++) {
			File each = originalVisualFiles[i];
			String aString = each.getName();
			aString = aString.substring(0, aString.lastIndexOf('.')) + "." + JunSystem.DefaultMovieExtension();
			File aFilename = new File(this.visualClipDirectory(), aString);
			visualClipFiles[i] = aFilename;
		}

		return visualClipFiles;
	}

	/**
	 * Initialize this when created.
	 */
	protected void initialize() {
		super.initialize();
		visualClip = null;
	}

	/**
	 * Make visual clip files.
	 */
	protected void makeVisualClipFiles() {
		progressValue.value_(0.0f);
		int anIndex = 1;
		File[] originalVisualFiles = this.originalVisualFiles();
		File[] visualClipFiles = this.visualClipFiles();

		for (int i = 0; i < originalVisualFiles.length; i++) {
			String aString = "Movie:" + " " + visualClipFiles[i].getName();
			progressMessage.value_(this._contractTo_(aString, this.defaultContractLength()));

			if (originalVisualFiles[i].equals(visualClipFiles[i]) == false) {
				JunMoviePlayer aMoviePlayer = null;
				try {
					aMoviePlayer = new JunMoviePlayer(originalVisualFiles[i]);
					aMoviePlayer.trackerModel().enableMarkers();
					if (this.visualClip() == null) {
						aMoviePlayer.trackerModel().lastMarker_(1.0d);
						aMoviePlayer.trackerModel().firstMarker_(0.0d);
					} else {
						aMoviePlayer.trackerModel().lastMarker_(this.visualClip()[1]);
						aMoviePlayer.trackerModel().firstMarker_(this.visualClip()[0]);
					}
					this.saveMovie_into_(aMoviePlayer, visualClipFiles[i]);
				} finally {
					if (aMoviePlayer != null) {
						aMoviePlayer.release();
						aMoviePlayer = null;
					}
				}
			}

			progressValue.value_(((float) anIndex) / this.numberOfVisuals());
			anIndex = anIndex + 1;
		}
	}

	/**
	 * Save movie into aFilename.
	 * 
	 * @param aMoviePlayer sra.jun.goodies.movie.JunMoviePlayer
	 * @param aFilename java.io.File
	 */
	protected void saveMovie_into_(JunMoviePlayer aMoviePlayer, File aFilename) {
		int start = 0;
		int duration = 0;
		if (aMoviePlayer.isIntervalNotEmpty()) {
			start = Math.round((float) (aMoviePlayer.trackerModel().firstMarker() * aMoviePlayer.movieHandle().duration()));
			duration = Math.round((float) ((aMoviePlayer.trackerModel().lastMarker() - aMoviePlayer.trackerModel().firstMarker()) * aMoviePlayer.movieHandle().duration()));
		} else {
			start = 0;
			duration = aMoviePlayer.movieHandle().duration();
		}

		int newMovie = 0;
		JunCursors cursor = new JunCursors(JunCursors.ExecuteCursor());
		try {
			cursor._show();
			aMoviePlayer.movieHandle().selection_duration_(start, duration);
			newMovie = aMoviePlayer.movieHandle().copyMovie();
		} finally {
			cursor._restore();
		}

		if (newMovie == 0) {
			return;
		}

		cursor = new JunCursors(JunCursors.WriteCursor());
		try {
			cursor._show();
			aMoviePlayer.movieHandle()._interface().flattenMovie_filename_(newMovie, aFilename);
		} finally {
			aMoviePlayer.movieHandle()._interface().qtDisposeMovie(newMovie);
			cursor._restore();
		}
	}
}
