package sra.jun.goodies.catalog;

import java.io.File;
import sra.jun.goodies.files.JunFileDialog;
import sra.jun.goodies.progress.JunProgress;
import sra.jun.system.support.JunTestExamples;
import sra.smalltalk.*;

/**
 * JunMovieCatalogTestExamples class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2003/05/21 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun462 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMovieCatalogTestExamples.java,v 6.0 2003/10/28 10:52:08 nisinaka Exp $
 */
public class JunMovieCatalogTestExamples extends JunTestExamples {

	/**
	 * Example 1.
	 * 
	 * @return boolean
	 */
	public static boolean Example1() {
		JunMovieCatalog movieCatalog = new JunMovieCatalog();
		movieCatalog.make();
		movieCatalog.show();

		return true;
	}

	/**
	 * Example 2.
	 * 
	 * @return boolean
	 */
	public static boolean Example2() {
		File aDirectory = JunFileDialog.RequestDirectory("");
		if (aDirectory == null) {
			return true;
		}

		JunMovieCatalog movieCatalog = new JunMovieCatalog(aDirectory);
		movieCatalog.visualClip_(new double[] { 0.249d, 0.253d });
		movieCatalog.keyFramePosition_(0.25d);
		movieCatalog.computeWhenProgressMessageChanged_(new StBlockClosure() {
			public Object value_(Object m) {
				System.out.println((String) m);
				return null;
			}
		});
		movieCatalog.make();
		movieCatalog.show();

		return true;
	}

	/**
	 * Example 3.
	 * 
	 * @return boolean
	 */
	public static boolean Example3() {
		File aDirectory = JunFileDialog.RequestDirectory("");
		if (aDirectory == null) {
			return true;
		}

		final JunMovieCatalog movieCatalog = new JunMovieCatalog(aDirectory);
		movieCatalog.visualClip_(new double[] { 0.24d, 0.28d });
		movieCatalog.keyFramePosition_(0.25d);
		final JunProgress aProgress = new JunProgress();
		movieCatalog.computeWhenProgressMessageChanged_(new StBlockClosure() {
			public Object value_(Object m) {
				aProgress.message_((String) m);
				return null;
			}
		});
		movieCatalog.computeWhenProgressValueChanged_(new StBlockClosure() {
			public Object value_(Object v) {
				aProgress.value_(((Number) v).floatValue());
				return null;
			}
		});
		aProgress.do_(new StBlockClosure() {
			public Object value() {
				movieCatalog.make();
				return null;
			}
		});
		movieCatalog.show();

		return true;
	}

	/**
	 * Example 4.
	 * 
	 * @return boolean
	 */
	public static boolean Example4() {
		File aDirectory = JunFileDialog.RequestDirectory("");
		if (aDirectory == null) {
			return true;
		}

		final JunMovieCatalog movieCatalog = new JunMovieCatalog(aDirectory);
		movieCatalog.visualClip_(new double[] { 0.24d, 0.28d });
		movieCatalog.keyFramePosition_(0.25d);
		movieCatalog.thumbnailImageSize_(32);
		movieCatalog.numberOfColumns_(3);
		movieCatalog.withFileName_(true);
		final JunProgress aProgress = new JunProgress();
		movieCatalog.computeWhenProgressMessageChanged_(new StBlockClosure() {
			public Object value_(Object m) {
				aProgress.message_((String) m);
				return null;
			}
		});
		movieCatalog.computeWhenProgressValueChanged_(new StBlockClosure() {
			public Object value_(Object v) {
				aProgress.value_(((Number) v).floatValue());
				return null;
			}
		});
		aProgress.do_(new StBlockClosure() {
			public Object value() {
				movieCatalog.make();
				return null;
			}
		});
		movieCatalog.show();

		return true;
	}

	/**
	 * Example 5.
	 * 
	 * @return boolean
	 */
	public static boolean Example5() {
		File aDirectory = JunFileDialog.RequestDirectory("");
		if (aDirectory == null) {
			return true;
		}

		JunVisualCatalog aMovieCatalog = JunMovieCatalog.Dive_(aDirectory);
		aMovieCatalog.show();

		return true;
	}

	/**
	 * Example 6.
	 * 
	 * @return boolean
	 */
	public static boolean Example6() {
		File aDirectory = JunFileDialog.RequestDirectory("");
		if (aDirectory == null) {
			return true;
		}

		JunVisualCatalog anmovieCatalog = JunMovieCatalog.Dive_block_(aDirectory, new StBlockClosure() {
			public Object value_(Object anObject) {
				final JunMovieCatalog movieCatalog = (JunMovieCatalog) anObject;
				movieCatalog.visualClip_(new double[] { 0.24d, 0.28d });
				movieCatalog.keyFramePosition_(0.25d);
				movieCatalog.thumbnailImageSize_(32);
				movieCatalog.numberOfColumns_(3);
				movieCatalog.withFileName_(true);
				final JunProgress aProgress = new JunProgress();
				movieCatalog.computeWhenProgressMessageChanged_(new StBlockClosure() {
					public Object value_(Object m) {
						aProgress.message_((String) m);
						return null;
					}
				});
				movieCatalog.computeWhenProgressValueChanged_(new StBlockClosure() {
					public Object value_(Object v) {
						aProgress.value_(((Number) v).floatValue());
						return null;
					}
				});
				aProgress.do_(new StBlockClosure() {
					public Object value() {
						movieCatalog.make();
						return null;
					}
				});
				return null;
			}
		});
		anmovieCatalog.show();

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args an array of command-line arguments
	 */
	public static void main(java.lang.String[] args) {
		new JunMovieCatalogTestExamples();
	}
}
