package sra.jun.goodies.catalog;

import java.awt.*;
import java.io.*;
import java.net.MalformedURLException;
import java.util.*;
import sra.jun.goodies.cursors.JunCursors;
import sra.jun.goodies.images.streams.*;
import sra.jun.goodies.images.support.JunImageAdjuster;
import sra.jun.goodies.movie.JunMoviePlayer;
import sra.jun.goodies.progress.JunProgress;
import sra.jun.goodies.utilities.JunStringUtility;
import sra.jun.goodies.url.JunURL;
import sra.jun.system.support.JunSystem;
import sra.smalltalk.*;

/**
 * JunVisualCatalog class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2003/05/19 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun506 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVisualCatalog.java,v 6.0 2003/10/28 10:52:08 nisinaka Exp $
 */
public class JunVisualCatalog extends StObject {
	public static final String DefaultIndexHtmlFileName = "index.html";
	public static final int DIVE_MAX_LEVEL = 8;
	public static final int THUMBNAIL_IMAGE_MIN_SIZE = 32;
	public static final int THUMBNAIL_IMAGE_MAX_SIZE = 256;

	protected File originalVisualDirectory;
	protected Vector originalVisualFiles;
	protected File visualCatalogDirectory;
	// protected Number[] visualClip;
	protected double keyFramePosition;
	protected Point thumbnailImageSize;
	protected int numberOfColumns;
	protected boolean withFileName;
	protected StValueHolder progressValue;
	protected StValueHolder progressMessage;

	/**
	 * The default constructor.
	 */
	public JunVisualCatalog() {
		super();
		this.initialize();
	}

	/**
	 * Create a new instance of JunVisualCatalog and set a directory.
	 * 
	 * @param directoryFilename java.io.File
	 */
	public JunVisualCatalog(File directoryFilename) {
		this();
		this.originalVisualDirectory_(directoryFilename);
	}

	/**
	 * Create a new instance of JunVisualCatalog and set a directory.
	 * 
	 * @param fromDirectoryFilename java.io.File
	 * @param toDirectoryFilename java.io.File
	 */
	public JunVisualCatalog(File fromDirectoryFilename, File toDirectoryFilename) {
		this();
		this.originalVisualDirectory_(fromDirectoryFilename);
		this.visualCatalogDirectory_(toDirectoryFilename);
	}

	/**
	 * Create a new instance of JunVisualCatalog and dive to a directory.
	 * 
	 * @return sra.jun.goodies.catalog.JunVisualCatalog
	 * @param directoryFilename java.io.File
	 */
	public static JunVisualCatalog Dive_(File directoryFilename) {
		return _Dive_catalog_(directoryFilename, new JunVisualCatalog());
	}

	/**
	 * Create a new instance of JunVisualCatalog and dive to a directory.
	 * 
	 * @return sra.jun.goodies.catalog.JunVisualCatalog
	 * @param directoryFilename java.io.File
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public static JunVisualCatalog Dive_block_(File directoryFilename, StBlockClosure aBlock) {
		return _Dive_block_catalog_(directoryFilename, aBlock, new JunVisualCatalog());
	}

	/**
	 * Answer separated strings.
	 *
	 * @return java.lang.String[]
	 * @param separateCollection java.lang.String
	 * @param dividerCollection char[]
	 */
	public static String[] Separate_dividers_(String separateCollection, char[] dividerCollection) {
		return JunStringUtility.Separate_dividers_(separateCollection, dividerCollection);
	}

	/**
	 * Create a new instance of JunVisualCatalog and dive to a directory.
	 * 
	 * @return sra.jun.goodies.catalog.JunVisualCatalog
	 * @param directoryFilename java.io.File
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @param visualCatalog sra.jun.goodies.catalog.JunVisualCatalog
	 */
	protected static JunVisualCatalog _Dive_block_catalog_(File directoryFilename, StBlockClosure aBlock, JunVisualCatalog visualCatalog) {
		Collection originalDirectoryNames = new Vector();
		Dive_into_(directoryFilename, originalDirectoryNames);
		originalDirectoryNames = new TreeSet(originalDirectoryNames);
		Vector targetDirectoryNames = new Vector(originalDirectoryNames.size());
		String directoryName = directoryFilename.getPath();
		if (directoryName.charAt(directoryName.length() - 1) == File.separatorChar) {
			directoryName = directoryName.substring(0, directoryName.length() - 1);
		}
		String baseString = new File(visualCatalog.defaultDirectory(), visualCatalog.defaultVisualCatalogDirectoryName()).getPath();
		String[] strings = new String[originalDirectoryNames.size()];
		originalDirectoryNames.toArray(strings);
		for (int i = 0; i < strings.length; i++) {
			String aString = directoryFilename.getParent();
			aString = strings[i].substring(aString.length() + 1, strings[i].length());
			File aFilename = new File(baseString, aString);
			aString = aFilename.getPath();
			targetDirectoryNames.add(aString);
		}

		Vector removeCollection = new Vector();
		for (int i = 0; i < targetDirectoryNames.size(); i++) {
			String targetDirectoryName = (String) targetDirectoryNames.get(i);
			JunVisualCatalog aVisualCatalog = (JunVisualCatalog) StObject._New(visualCatalog.getClass(), new File(strings[i]), new File(targetDirectoryName));
			aBlock.value_(aVisualCatalog);
			if (aVisualCatalog.isEmpty()) {
				removeCollection.add(targetDirectoryName);
			}
		}
		targetDirectoryNames.removeAll(removeCollection);

		BufferedWriter writer = null;
		JunCursors cursor = new JunCursors(JunCursors.WriteCursor());
		try {
			cursor._show();

			writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(visualCatalog.indexHtmlFile()), "MS932"));
			writer.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">");
			writer.newLine();
			writer.write("<HTML>");
			writer.newLine();
			writer.write("<HEAD>");
			writer.newLine();
			writer.write("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=Shift_JIS\">");
			writer.newLine();
			writer.write("<LINK REV=\"MADE\" HREF=\"mailto:jun@ktlsva.sra.co.jp\">");
			writer.newLine();
			writer.write("<LINK REL=\"INDEX\" HREF=\"./index.html\">");
			writer.newLine();
			writer.write("<TITLE>" + visualCatalog.defaultTitleString() + "s" + "</TITLE>");
			writer.newLine();
			writer.write("</HEAD>");
			writer.newLine();
			writer.write("<BODY BGCOLOR=\"#FFFFFF\" TEXT=\"#000000\" LINK=\"#0000FF\" VLINK=\"#663399\" ALINK=\"#FF0000\">");
			writer.newLine();
			writer.write("<H2>" + visualCatalog.defaultTitleString() + "s" + "</H2>");
			writer.newLine();
			writer.write("<OL>");
			writer.newLine();
			for (Enumeration enum = targetDirectoryNames.elements(); enum.hasMoreElements();) {
				String each = ((String) enum.nextElement());
				String aString = each.substring(baseString.length(), each.length());
				aString = aString.replace(File.separatorChar, '/');
				aString = "." + aString;
				writer.write("<LI><A HREF=\"");
				if (aString.charAt(aString.length() - 1) == '/') {
					writer.write(aString + visualCatalog.DefaultIndexHtmlFileName);
				} else {
					writer.write(aString + "/" + visualCatalog.DefaultIndexHtmlFileName);
				}
				writer.write("\">" + aString + "</A>");
				writer.newLine();
			}
			writer.write("</OL>");
			writer.newLine();
			writer.write("<HR>");
			writer.newLine();
			writer.write("This page was created by " + visualCatalog._className() + " on " + JunSystem.FullName());
			writer.newLine();
			writer.write("</BODY>");
			writer.newLine();
			writer.write("</HTML>");
			writer.newLine();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			if (writer != null) {
				try {
					writer.flush();
					writer.close();
				} catch (IOException e) {
				} finally {
					writer = null;
				}
			}
			cursor._restore();
		}

		return visualCatalog;
	}

	/**
	 * Create a new instance of JunVisualCatalog and dive to a directory.
	 * 
	 * @return sra.jun.goodies.catalog.JunVisualCatalog
	 * @param directoryFilename java.io.File
	 * @param visualCatalog sra.jun.goodies.catalog.JunVisualCatalog
	 */
	protected static JunVisualCatalog _Dive_catalog_(File directoryFilename, JunVisualCatalog visualCatalog) {
		return _Dive_block_catalog_(directoryFilename, new StBlockClosure() {
			public Object value_(Object anObject) {
				final JunVisualCatalog visualCatalog = (JunVisualCatalog) anObject;
				final JunProgress aProgress = new JunProgress();
				visualCatalog.computeWhenProgressMessageChanged_(new StBlockClosure() {
					public Object value_(Object m) {
						aProgress.message_((String) m);
						return null;
					}
				});
				visualCatalog.computeWhenProgressValueChanged_(new StBlockClosure() {
					public Object value_(Object v) {
						aProgress.value_(((Number) v).floatValue());
						return null;
					}
				});
				aProgress.do_(new StBlockClosure() {
					public Object value() {
						visualCatalog.make();
						return null;
					}
				});

				return null;
			}
		}, visualCatalog);
	}

	/**
	 * 
	 * 
	 * @return java.util.Collection
	 * @param directoryName java.io.File
	 * @param aCollection java.util.Collection
	 */
	protected static Collection Dive_into_(File directoryName, Collection aCollection) {
		return Dive_into_(directoryName.getPath(), aCollection);
	}

	/**
	 * 
	 * 
	 * @return java.util.Collection
	 * @param directoryName java.lang.String
	 * @param aCollection java.util.Collection
	 */
	protected static Collection Dive_into_(String directoryName, Collection aCollection) {
		File aFilename = new File(directoryName);
		int startLevel = Separate_dividers_(aFilename.getPath(), new char[] { File.separatorChar }).length;
		return Dive_into_start_(aFilename.getPath(), aCollection, startLevel);
	}

	/**
	 * 
	 * 
	 * @return java.util.Collection
	 * @param directoryName java.lang.String
	 * @param resultCollection java.util.Set
	 * @param startLevel int
	 */
	protected static Collection Dive_into_start_(String directoryName, Collection resultCollection, int startLevel) {
		File aFilename = new File(directoryName);
		int diveLevel = Separate_dividers_(aFilename.getPath(), new char[] { File.separatorChar }).length;
		if (diveLevel - startLevel > JunVisualCatalog.DIVE_MAX_LEVEL) {
			return resultCollection;
		}
		if (resultCollection.contains(aFilename.getPath())) {
			return resultCollection;
		}
		if (aFilename.exists() && aFilename.isDirectory() && aFilename.canRead()) {
			resultCollection.add(aFilename.getPath());
			String[] aCollection = aFilename.list();
			for (int i = 0; i < aCollection.length; i++) {
				String aString = null;
				if (directoryName.charAt(directoryName.length() - 1) == File.separatorChar) {
					aString = directoryName + aCollection[i];
				} else {
					aString = directoryName + File.separator + aCollection[i];
				}
				Dive_into_start_(aString, resultCollection, startLevel);
			}
			return resultCollection;
		}
		return resultCollection;
	}

	/**
	 * Compute when progress message changed.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void computeWhenProgressMessageChanged_(StBlockClosure aBlock) {
		progressMessage.compute_(aBlock);
	}

	/**
	 * Compute when progress value changed.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void computeWhenProgressValueChanged_(StBlockClosure aBlock) {
		progressValue.compute_(aBlock);
	}

	/**
	 * Answer the default contract length.
	 * 
	 * @return int
	 */
	public int defaultContractLength() {
		return 32;
	}

	/**
	 * Answer the default directory.
	 * 
	 * @return java.io.File
	 */
	public File defaultDirectory() {
		return new File(this.defaultDirectoryName());
	}

	/**
	 * Answer the default directory name.
	 * 
	 * @return java.lang.String
	 */
	public String defaultDirectoryName() {
		return (String) System.getProperties().get("user.dir");
	}

	/**
	 * Answer the default key frame position.
	 * 
	 * @return double
	 */
	public double defaultKeyFramePosition() {
		return 0.5d;
	}

	/**
	 * Answer the default number of columns.
	 * 
	 * @return int
	 */
	public int defaultNumberOfColumns() {
		return 5;
	}

	/**
	 * Answer the default thumbnail image directory name.
	 * 
	 * @return java.lang.String
	 */
	public String defaultThumbnailImageDirectoryName() {
		return "thumbnails";
	}

	/**
	 * Answer the default thumbnail image size.
	 * 
	 * @return java.awt.Point()
	 */
	public Point defaultThumbnailImageSize() {
		return new Point(72, 72);
	}

	/**
	 * Answer the default title string.
	 * 
	 * @return java.lang.String
	 */
	public String defaultTitleString() {
		return "Visual Catalog";
	}

	/**
	 * Answer the default visual catalog directory name.
	 * 
	 * @return java.lang.String
	 */
	public String defaultVisualCatalogDirectoryName() {
		return "VisualCatalog";
	}

	/**
	 * Answer the default visual clip directory name.
	 * 
	 * @return java.lang.String
	 */
	public String defaultVisualClipDirectoryName() {
		return "VisualClip";
	}

	/**
	 * Answer the index.html file.
	 * 
	 * @return java.io.File
	 */
	public File indexHtmlFile() {
		return new File(this.visualCatalogDirectory(), JunVisualCatalog.DefaultIndexHtmlFileName);
	}

	/**
	 * Answer true if this visuals is empty, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean isEmpty() {
		return this.numberOfVisuals() == 0;
	}

	/**
	 * Answer the key frame position.
	 * 
	 * @return double
	 */
	public double keyFramePosition() {
		return keyFramePosition;
	}

	/**
	 * Set the key frame position.
	 * 
	 * @param normalizedValue double
	 */
	public void keyFramePosition_(double normalizedValue) {
		keyFramePosition = normalizedValue;
	}

	/**
	 * Make visual clip files, thumenail image files and index.html file.
	 */
	public void make() {
		this.makeVisualClipFiles();
		this.makeThumbnailImageFiles();
		this.makeIndexHtmlFile();
	}

	/**
	 * Answer the number of columns.
	 * 
	 * @return int
	 */
	public int numberOfColumns() {
		return numberOfColumns;
	}

	/**
	 * Set the number of columns.
	 * 
	 * @param anInteger int
	 */
	public void numberOfColumns_(int anInteger) {
		numberOfColumns = anInteger;
	}

	/**
	 * Answer the number of visuals.
	 * 
	 * @return int
	 */
	public int numberOfVisuals() {
		return this.originalVisualFiles().length;
	}

	/**
	 * Answer the original visual directory.
	 * 
	 * @return java.io.File
	 */
	public File originalVisualDirectory() {
		return originalVisualDirectory;
	}

	/**
	 * Set the original visual directory.
	 * 
	 * @param aFilename java.io.File
	 */
	public void originalVisualDirectory_(File aFilename) {
		originalVisualDirectory = aFilename;
		originalVisualFiles = null;
	}

	/**
	 * Answer the original visual files.
	 * 
	 * @return java.io.File[]
	 */
	public File[] originalVisualFiles() {
		if (this.originalVisualDirectory() == null) {
			return new File[0];
		}

		originalVisualFiles = new Vector();
		String[] lists = this.originalVisualDirectory().list();
		for (int i = 0; i < lists.length; i++) {
			String each = lists[i];
			int pos = each.lastIndexOf('.');
			String extentString = (pos != -1) ? (each.substring(pos + 1)) : null;
			if (extentString != null) {
				extentString = extentString.toLowerCase();
				File aFilename = null;

				String[] extensions = JunSystem.DefaultImageExtensions();
				for (int j = 0; j < extensions.length; j++) {
					if (extentString.equals(extensions[j])) {
						aFilename = new File(this.originalVisualDirectory, each);
						originalVisualFiles.add(aFilename);
						break;
					}
				}
				if (aFilename == null) {
					extensions = JunSystem.DefaultMovieExtensions();
					for (int j = 0; j < extensions.length; j++) {
						if (extentString.equals(extensions[j])) {
							aFilename = new File(this.originalVisualDirectory, each);
							originalVisualFiles.add(aFilename);
							break;
						}
					}
				}
			}
		}

		File[] files = new File[originalVisualFiles.size()];
		originalVisualFiles.copyInto(files);
		return files;
	}

	/**
	 * Open this visual catlog with browser.
	 */
	public void show() {
		File aFilename = this.indexHtmlFile();
		if (aFilename.exists() == false) {
			return;
		}
		try {
			JunURL.Browse_(aFilename.toURL());
		} catch (MalformedURLException e) {
		}
	}

	/**
	 * Answer the thumbnail image directory.
	 * 
	 * @return java.io.File
	 */
	public File thumbnailImageDirectory() {
		return new File(this.visualCatalogDirectory(), this.defaultThumbnailImageDirectoryName());
	}

	/**
	 * Answer the thumbnail image files.
	 * 
	 * @return java.io.File[]
	 */
	public File[] thumbnailImageFiles() {
		File[] originalVisualFiles = this.originalVisualFiles();
		File[] thumbnailImageFiles = new File[originalVisualFiles.length];
		for (int i = 0; i < originalVisualFiles.length; i++) {
			File each = originalVisualFiles[i];
			String aString = each.getName();
			int pos = aString.lastIndexOf('.');
			aString = (pos != -1) ? (aString.substring(0, pos)) : aString;
			aString = aString + "." + JunSystem.DefaultImageExtension();
			File aFilename = new File(this.thumbnailImageDirectory(), aString);
			thumbnailImageFiles[i] = aFilename;
		}

		return thumbnailImageFiles;
	}

	/**
	 * Answer the thumbnail image size.
	 * 
	 * @return java.awt.Point
	 */
	public Point thumbnailImageSize() {
		return thumbnailImageSize;
	}

	/**
	 * Set the thumbnail image size.
	 * 
	 * @param sizeObject int
	 */
	public void thumbnailImageSize_(int sizeObject) {
		this.thumbnailImageSize_(new Point(sizeObject, sizeObject));
	}

	/**
	 * Set the thumbnail image size.
	 * 
	 * @param sizeObject java.awt.Point
	 */
	public void thumbnailImageSize_(Point sizeObject) {
		int x = Math.max(THUMBNAIL_IMAGE_MIN_SIZE, Math.min(sizeObject.x, THUMBNAIL_IMAGE_MAX_SIZE));
		int y = Math.max(THUMBNAIL_IMAGE_MIN_SIZE, Math.min(sizeObject.y, THUMBNAIL_IMAGE_MAX_SIZE));
		thumbnailImageSize = new Point(x, y);
	}

	/**
	 * URLs do a block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @exception java.io.IOException
	 */
	public void urlsDo_(StBlockClosure aBlock) throws IOException {
		String[] visualClipUrls = this.visualClipUrls();
		String[] thumbnailImageUrls = this.thumbnailImageUrls();
		Dimension[] thumbnailImageExtents = this.thumbnailImageExtents();
		for (int i = 0; i < this.numberOfVisuals(); i++) {
			Object result = aBlock.valueWithArguments_(new Object[] { visualClipUrls[i], thumbnailImageUrls[i], thumbnailImageExtents[i], new Integer(i)});
			if (result != null && result instanceof IOException) {
				throw (IOException) result;
			}
		}
	}

	/**
	 * Answer the visual catalog directory.
	 * 
	 * @return java.io.File
	 */
	public File visualCatalogDirectory() {
		if (visualCatalogDirectory == null) {
			File aFilename = new File(this.defaultDirectory(), this.defaultVisualCatalogDirectoryName());
			this.visualCatalogDirectory_(aFilename);
		}
		return visualCatalogDirectory;
	}

	/**
	 * Set the visual catalog directory.
	 * 
	 * @param aFilename java.io.File
	 */
	public void visualCatalogDirectory_(File aFilename) {
		visualCatalogDirectory = aFilename;
		if (visualCatalogDirectory.exists()) {
			if (visualCatalogDirectory.isDirectory() == false) {
				new SmalltalkException("Can't access directory: " + visualCatalogDirectory.toString());
				visualCatalogDirectory = null;
				return;
			}
		} else {
			visualCatalogDirectory.mkdirs();
		}
		if (this.isEmpty() == false) {
			File visualClipDirectory = this.visualClipDirectory();
			if (visualClipDirectory.exists() == false) {
				visualClipDirectory.mkdir();
			}
			File thumbnailImageDirectory = this.thumbnailImageDirectory();
			if (thumbnailImageDirectory.exists() == false) {
				thumbnailImageDirectory.mkdir();
			}
		}
	}

	/**
	 * Answer the visual clip directory.
	 * 
	 * @return java.io.File
	 */
	public File visualClipDirectory() {
		return new File(this.visualCatalogDirectory(), this.defaultVisualClipDirectoryName());
	}

	/**
	 * Answer the visual clip files.
	 * 
	 * @return java.io.File[]
	 */
	public File[] visualClipFiles() {
		File[] originalVisualFiles = this.originalVisualFiles();
		File[] visualClipFiles = new File[originalVisualFiles.length];
		for (int i = 0; i < originalVisualFiles.length; i++) {
			File each = originalVisualFiles[i];
			String aString = each.getName();
			File aFilename = new File(this.visualClipDirectory(), aString);
			visualClipFiles[i] = aFilename;
		}

		return visualClipFiles;
	}

	/**
	 * Answer true if this object with file name, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean withFileName() {
		return withFileName == true;
	}

	/**
	 * Set the with file name.
	 * 
	 * @param aBoolean boolean
	 */
	public void withFileName_(boolean aBoolean) {
		withFileName = (aBoolean == true);
	}

	/**
	 * Shorten by ellipsis if too long.
	 *
	 * @return java.lang.String
	 * @param aString java.lang.String
	 * @param charCount int
	 */
	protected String _contractTo_(String aString, int charCount) {
		String shortString = new String(aString);
		if (aString.length() > charCount) {
			int half = charCount / 2;
			shortString = aString.substring(0, half - 1) + "..." + aString.substring(aString.length() - (charCount - half) + 2);
		}
		return shortString;
	}

	/**
	 * Convert a filename to url string and answer it.
	 * 
	 * @return java.lang.String
	 * @param aFilename java.io.File
	 */
	protected String convertFilenameToUrlString_(File aFilename) {
		String baseString = this.visualCatalogDirectory().toString();
		String aString = aFilename.toString();
		aString = aString.substring(baseString.length() + 1);
		if (aString.charAt(0) == File.separatorChar) {
			aString = aString.substring(1);
		}
		aString = aString.replace(File.separatorChar, '/');
		return aString;
	}

	/**
	 * Initialize this when created.
	 */
	protected void initialize() {
		originalVisualDirectory = null;
		originalVisualFiles = null;
		visualCatalogDirectory = null;
		this.keyFramePosition_(this.defaultKeyFramePosition());
		this.thumbnailImageSize_(this.defaultThumbnailImageSize());
		this.numberOfColumns_(this.defaultNumberOfColumns());
		this.withFileName_(false);
		progressValue = new StValueHolder(0.0d);
		progressMessage = new StValueHolder("");
	}

	/**
	 * Make index.html file.
	 */
	protected void makeIndexHtmlFile() {
		final File[] visualClipFiles = this.visualClipFiles();
		final File[] thumbnailImageFiles = this.thumbnailImageFiles();
		BufferedWriter writer = null;
		JunCursors cursor = new JunCursors(JunCursors.WriteCursor());
		try {
			cursor._show();

			writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(this.indexHtmlFile()), "MS932"));
			writer.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">");
			writer.newLine();
			writer.write("<HTML>");
			writer.newLine();
			writer.write("<HEAD>");
			writer.newLine();
			writer.write("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=Shift_JIS\">");
			writer.newLine();
			writer.write("<LINK REV=\"MADE\" HREF=\"mailto:jun@ktlsva.sra.co.jp\">");
			writer.newLine();
			writer.write("<LINK REL=\"INDEX\" HREF=\"./index.html\">");
			writer.newLine();
			writer.write("<TITLE>" + this.defaultTitleString() + "</TITLE>");
			writer.newLine();
			writer.write("</HEAD>");
			writer.newLine();
			writer.write("<BODY BGCOLOR=\"#FFFFFF\" TEXT=\"#000000\" LINK=\"#0000FF\" VLINK=\"#663399\" ALINK=\"#FF0000\">");
			writer.newLine();
			writer.write("<H2>" + this.defaultTitleString() + "</H2>");
			writer.newLine();
			if (this.isEmpty() == false) {
				writer.write("<TABLE BORDER=\"0\" CELLPADDING=\"5\">");
				writer.newLine();
				writer.write("<TR>");
				writer.newLine();
				final BufferedWriter bw = writer;
				final JunVisualCatalog self = this;
				this.urlsDo_(new StBlockClosure() {
					public Object valueWithArguments_(Object[] anArray) {
						String movieClipUrl = (String) anArray[0];
						String thumbnailImageUrl = (String) anArray[1];
						Dimension thumbnailImageExtent = (Dimension) anArray[2];
						int anIndex = ((Integer) anArray[3]).intValue();

						try {
							bw.write("<TD ALIGN=\"CENTER\" VALIGN=\"MIDDLE\">");
							bw.newLine();
							bw.write("<A HREF=\"" + movieClipUrl + "\">");
							bw.write("<IMG SRC=\"" + thumbnailImageUrl + "\" ");
							bw.write("WIDTH=\"" + thumbnailImageExtent.width + "\" HEIGHT=\"" + thumbnailImageExtent.height + "\" ");
							bw.write("BORDER=\"1\" ALT=\"" + thumbnailImageFiles[anIndex].getName() + "\">");
							if (self.withFileName()) {
								bw.write("<BR>");
								bw.write(thumbnailImageFiles[anIndex].getName());
							}
							bw.write("</A>");
							bw.newLine();
							bw.write("</TD>");
							bw.newLine();

							if ((anIndex + 1) % self.numberOfColumns() == 0 && anIndex < self.numberOfVisuals()) {
								bw.write("</TR>");
								bw.newLine();
								bw.write("<TR>");
								bw.newLine();
							}
						} catch (IOException e) {
							return e;
						}

						return null;
					}
				});
				writer.write("</TR>");
				writer.newLine();
				writer.write("</TABLE>");
			}

			writer.write("<HR>");
			writer.newLine();
			writer.write("This page was created by " + this._className() + " on " + JunSystem.FullName());
			writer.newLine();
			writer.write("</BODY>");
			writer.newLine();
			writer.write("</HTML>");
			writer.newLine();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			if (writer != null) {
				try {
					writer.flush();
					writer.close();
				} catch (IOException e) {
				} finally {
					writer = null;
				}
			}
			cursor._restore();
		}
	}

	/**
	 * Make thumbnail image files.
	 */
	protected void makeThumbnailImageFiles() {
		progressValue.value_(0.0f);
		int anIndex = 1;
		File[] originalVisualFiles = this.originalVisualFiles();
		File[] thumbnailImageFiles = this.thumbnailImageFiles();
		for (int i = 0; i < originalVisualFiles.length; i++) {
			String aString = "Thumbnail:" + " " + thumbnailImageFiles[i].getName();
			progressMessage.value_(this._contractTo_(aString, this.defaultContractLength()));

			StImage anImage = null;
			JunMoviePlayer aMoviePlayer = null;
			try {
				aMoviePlayer = new JunMoviePlayer(originalVisualFiles[i]);
				aMoviePlayer.goto_(this.keyFramePosition());
				anImage = new StImage(aMoviePlayer.asImage());
			} finally {
				if (aMoviePlayer != null) {
					aMoviePlayer.release();
					aMoviePlayer = null;
				}
			}

			JunImageAdjuster anAdjuster = new JunImageAdjuster();
			Dimension imageExtent = this.proportionalExtentOfImage_within_(anImage, this.thumbnailImageSize);
			anImage = anAdjuster.adjust_extent_(anImage, imageExtent);

			JunImageStream writeStream = null;
			try {
				writeStream = JunJpegImageStream.On_(new BufferedOutputStream(new FileOutputStream(thumbnailImageFiles[i])));
				writeStream.nextPutImage_(anImage);
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			} finally {
				if (writeStream != null) {
					try {
						writeStream.flush();
						writeStream.close();
					} catch (IOException e) {
					} finally {
						writeStream = null;
					}
				}
			}

			progressValue.value_(((float) anIndex) / this.numberOfVisuals());
			anIndex = anIndex + 1;
		}
	}

	/**
	 * Make visual clip files.
	 */
	protected void makeVisualClipFiles() {
		progressValue.value_(0.0f);
		int anIndex = 1;
		File[] originalVisualFiles = this.originalVisualFiles();
		File[] visualClipFiles = this.visualClipFiles();
		for (int i = 0; i < originalVisualFiles.length; i++) {
			String aString = "Visual:" + " " + visualClipFiles[i].getName();
			progressMessage.value_(this._contractTo_(aString, this.defaultContractLength()));

			if (originalVisualFiles[i].equals(visualClipFiles[i]) == false) {
				InputStream readStream = null;
				OutputStream writeStream = null;
				try {
					readStream = new BufferedInputStream(new FileInputStream(originalVisualFiles[i]));
					writeStream = new BufferedOutputStream(new FileOutputStream(visualClipFiles[i]));
					byte[] buffer = new byte[1024];
					int size;
					while ((size = readStream.read(buffer, 0, buffer.length)) != -1) {
						writeStream.write(buffer, 0, size);
					}
				} catch (IOException e) {
					System.out.println(e.getMessage());
					e.printStackTrace();
				} finally {
					if (writeStream != null) {
						try {
							writeStream.flush();
							writeStream.close();
						} catch (IOException e) {
						} finally {
							writeStream = null;
						}
					}
					if (readStream != null) {
						try {
							readStream.close();
						} catch (IOException e) {
						} finally {
							readStream = null;
						}
					}
				}
			}

			progressValue.value_(((float) anIndex) / this.numberOfVisuals());
			anIndex = anIndex + 1;
		}
	}

	/**
	 * Answer the propotional extent of image with in a point.
	 * 
	 * @return java.awt.Dimension
	 * @param anImage sra.smalltalk.StImage
	 * @param aPoint java.awt.Point
	 */
	protected Dimension proportionalExtentOfImage_within_(StImage anImage, Point aPoint) {
		Point imageExtent = anImage.extent();
		if (imageExtent.x > aPoint.x) {
			int imageClipHeight = Math.round(imageExtent.y * aPoint.x / (float) imageExtent.x);
			imageExtent = new Point(aPoint.x, imageClipHeight);
		} else {
			int imageClipWidth = Math.round(imageExtent.x * aPoint.y / (float) imageExtent.y);
			imageExtent = new Point(imageClipWidth, aPoint.y);
		}
		return new Dimension(imageExtent.x, imageExtent.y);
	}

	/**
	 * Answer the thumbnail image extents.
	 * 
	 * @return java.awt.Dimension[]
	 */
	protected Dimension[] thumbnailImageExtents() {
		File[] thumbnailImageFiles = this.thumbnailImageFiles();
		Dimension[] thumbnailImageExtents = new Dimension[thumbnailImageFiles.length];

		StImage image = null;
		JunImageStream stream = null;
		JunCursors cursor = new JunCursors(JunCursors.ReadCursor());
		for (int i = 0; i < thumbnailImageFiles.length; i++) {
			try {
				cursor._show();
				stream = JunJpegImageStream.On_(new FileInputStream(thumbnailImageFiles[i]));
				image = stream.nextImage();
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			} finally {
				if (stream != null) {
					try {
						stream.close();
					} catch (IOException e) {
					} finally {
						stream = null;
					}
				}
			}
			thumbnailImageExtents[i] = image.bounds().getSize();
		}
		return thumbnailImageExtents;
	}

	/**
	 * Answer the strings of thumbnail image URL.
	 * 
	 * @return java.lang.String[]
	 */
	protected String[] thumbnailImageUrls() {
		File[] thumbnailImageFiles = this.thumbnailImageFiles();
		String[] thumbnailImageUrls = new String[thumbnailImageFiles.length];
		for (int i = 0; i < thumbnailImageFiles.length; i++) {
			thumbnailImageUrls[i] = this.convertFilenameToUrlString_(thumbnailImageFiles[i]);
		}
		return thumbnailImageUrls;
	}

	/**
	 * Answer the strings of visual clip URL.
	 * 
	 * @return java.lang.String[]
	 */
	protected String[] visualClipUrls() {
		File[] visualClipFiles = this.visualClipFiles();
		String[] visualClipUrls = new String[visualClipFiles.length];
		for (int i = 0; i < visualClipFiles.length; i++) {
			visualClipUrls[i] = this.convertFilenameToUrlString_(visualClipFiles[i]);
		}
		return visualClipUrls;
	}
}
