package sra.jun.goodies.catalog;

import java.io.File;
import sra.jun.goodies.files.JunFileDialog;
import sra.jun.goodies.progress.JunProgress;
import sra.jun.system.support.JunTestExamples;
import sra.smalltalk.*;

/**
 * JunVisualCatalogTestExamples class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2003/05/27 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun462 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVisualCatalogTestExamples.java,v 6.0 2003/10/28 10:52:08 nisinaka Exp $
 */
public class JunVisualCatalogTestExamples extends JunTestExamples {

	/**
	 * Example 1.
	 * 
	 * @return boolean
	 */
	public static boolean Example1() {
		JunVisualCatalog visualCatalog = new JunVisualCatalog();
		visualCatalog.make();
		visualCatalog.show();

		return true;
	}

	/**
	 * Example 2.
	 * 
	 * @return boolean
	 */
	public static boolean Example2() {
		File aDirectory = JunFileDialog.RequestDirectory("");
		if (aDirectory == null) {
			return true;
		}

		JunVisualCatalog visualCatalog = new JunVisualCatalog(aDirectory);
		visualCatalog.computeWhenProgressMessageChanged_(new StBlockClosure() {
			public Object value_(Object m) {
				System.out.println((String) m);
				return null;
			}
		});
		visualCatalog.make();
		visualCatalog.show();

		return true;
	}

	/**
	 * Example 3.
	 * 
	 * @return boolean
	 */
	public static boolean Example3() {
		File aDirectory = JunFileDialog.RequestDirectory("");
		if (aDirectory == null) {
			return true;
		}

		final JunVisualCatalog visualCatalog = new JunVisualCatalog(aDirectory);
		final JunProgress aProgress = new JunProgress();
		visualCatalog.computeWhenProgressMessageChanged_(new StBlockClosure() {
			public Object value_(Object m) {
				aProgress.message_((String) m);
				return null;
			}
		});
		visualCatalog.computeWhenProgressValueChanged_(new StBlockClosure() {
			public Object value_(Object v) {
				aProgress.value_(((Number) v).floatValue());
				return null;
			}
		});
		aProgress.do_(new StBlockClosure() {
			public Object value() {
				visualCatalog.make();
				return null;
			}
		});
		visualCatalog.show();

		return true;
	}

	/**
	 * Example 4.
	 * 
	 * @return boolean
	 */
	public static boolean Example4() {
		File aDirectory = JunFileDialog.RequestDirectory("");
		if (aDirectory == null) {
			return true;
		}

		final JunVisualCatalog visualCatalog = new JunVisualCatalog(aDirectory);
		visualCatalog.thumbnailImageSize_(32);
		visualCatalog.numberOfColumns_(3);
		visualCatalog.withFileName_(true);
		final JunProgress aProgress = new JunProgress();
		visualCatalog.computeWhenProgressMessageChanged_(new StBlockClosure() {
			public Object value_(Object m) {
				aProgress.message_((String) m);
				return null;
			}
		});
		visualCatalog.computeWhenProgressValueChanged_(new StBlockClosure() {
			public Object value_(Object v) {
				aProgress.value_(((Number) v).floatValue());
				return null;
			}
		});
		aProgress.do_(new StBlockClosure() {
			public Object value() {
				visualCatalog.make();
				return null;
			}
		});
		visualCatalog.show();

		return true;
	}

	/**
	 * Example 5.
	 * 
	 * @return boolean
	 */
	public static boolean Example5() {
		File aDirectory = JunFileDialog.RequestDirectory("");
		if (aDirectory == null) {
			return true;
		}

		JunVisualCatalog visualCatalog = JunVisualCatalog.Dive_(aDirectory);
		visualCatalog.show();

		return true;
	}

	/**
	 * Example 6.
	 * 
	 * @return boolean
	 */
	public static boolean Example6() {
		File aDirectory = JunFileDialog.RequestDirectory("");
		if (aDirectory == null) {
			return true;
		}

		JunVisualCatalog visualCatalog = JunVisualCatalog.Dive_block_(aDirectory, new StBlockClosure() {
			public Object value_(Object anObject) {
				final JunVisualCatalog visualCatalog = (JunVisualCatalog) anObject;
				visualCatalog.thumbnailImageSize_(32);
				visualCatalog.numberOfColumns_(3);
				visualCatalog.withFileName_(true);
				final JunProgress aProgress = new JunProgress();
				visualCatalog.computeWhenProgressMessageChanged_(new StBlockClosure() {
					public Object value_(Object m) {
						aProgress.message_((String) m);
						return null;
					}
				});
				visualCatalog.computeWhenProgressValueChanged_(new StBlockClosure() {
					public Object value_(Object v) {
						aProgress.value_(((Number) v).floatValue());
						return null;
					}
				});
				aProgress.do_(new StBlockClosure() {
					public Object value() {
						visualCatalog.make();
						return null;
					}
				});
				return null;
			}
		});
		visualCatalog.show();

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args an array of command-line arguments
	 */
	public static void main(java.lang.String[] args) {
		new JunVisualCatalogTestExamples();
	}
}
