package sra.jun.goodies.colors;

import java.awt.*;
import sra.smalltalk.*;
import sra.jun.geometry.basic.Jun2dPoint;

/**
 * JunColorChoiceHSB class
 * 
 *  @author    Hirotsugu Kondo
 *  @created   1998/11/05 (by Hirotsugu Kondo)
 *  @updated   2003/04/02 (by Mituhiro Asada)
 *  @version   406b (with StPL6.0) based on Jun490 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunColorChoiceHSB.java,v 6.0 2003/10/28 10:52:08 nisinaka Exp $
 */
public class JunColorChoiceHSB extends JunColorChoiceModel {
	protected static StImage[] bars;
	protected static StImage[] plates;

	static {
		plates = _InitializePlates();
		bars = _InitializeBars();
	}

	/**
	 * Initialize color bars.
	 * 
	 * @return java.lang.reflect.Array of StImage
	 */
	static StImage[] _InitializeBars() {
		int gap = _Gap();
		int interval = _Intervals();
		int ySize = interval / gap;
		int zSize = interval / gap;
		int width = _barArea().width;
		StImage[] images = new StImage[zSize];
		for (int z = 0; z < zSize; z++) {
			int[] pixels = new int[interval * width];
			for (int y = 0; y < ySize; y++) {
				double hue = 1.0;
				double saturation = 0.0;
				double brightness = 1 - (y * 1.0 / ySize);
				Color color = Color.getHSBColor((float) hue, (float) saturation, (float) brightness);
				for (int dy = 0; dy < gap; dy++) {
					for (int dx = 0; dx < width; dx++) {
						pixels[((y * gap + dy) * width) + dx] = color.getRGB();
					}
				}
			}
			images[z] = new StImage(width, interval, pixels);
		}

		return images;
	}

	/**
	 * Initialize color plates.
	 * 
	 * @return java.lang.reflect.Array of StImage
	 */
	static StImage[] _InitializePlates() {
		int gap = _Gap();
		int interval = _Intervals();
		int size = interval / gap;
		StImage[] images = new StImage[size];
		for (int z = 0; z < size; z++) {
			int[] pixels = new int[interval * interval];

			for (int y = 0; y < size; y++) {
				for (int x = 0; x < size; x++) {
					double hue = (x * 1.0) / size;
					double saturation = (y * 1.0) / size;
					double brightness = 1 - (z * 1.0 / size);
					Color color = Color.getHSBColor((float) hue, (float) saturation, (float) brightness);
					for (int dy = 0; dy < gap; dy++) {
						for (int dx = 0; dx < gap; dx++) {
							pixels[((y * gap + dy) * interval) + (x * gap) + dx] = color.getRGB();
						}
					}
				}
			}
			images[z] = new StImage(interval, interval, pixels);
		}

		return images;
	}

	/**
	 * Answer the images of bar.
	 * 
	 * @return sra.smalltalk.StImage[]
	 */
	public StImage[] bars() {
		StImage[] images = this.bars;
		return images;
	}

	/**
	 * Answer the brighness value.
	 * 
	 * @return double
	 */
	public double brightness() {
		double value = 1 - this.z();

		return value;
	}

	/**
	 * Set the brighness value.
	 * 
	 * @param value double
	 */
	public void brightness_(double value) {
		this.z_(1 - value);
	}

	/**
	 * Set the color and mapping this.
	 * 
	 * @param color java.awt.Color
	 */
	public void color_(Color color) {
		this.z_(1 - (StColorValue._GetBrightness(color)));
		this.xy_(new Jun2dPoint(StColorValue._GetHue(color), (StColorValue._GetSaturation(color))));
		this.changed_($("color"));
	}

	/**
	 * Answer the hue value.
	 * 
	 * @return double
	 */
	public double hue() {
		double value = this.xy().x();
		return value;
	}

	/**
	 * Set the hue value.
	 * 
	 * @param value double
	 */
	public void hue_(double value) {
		this.xy_(new Jun2dPoint(value, this.xy().y()));
	}

	/**
	 * Answer the images of color plate.
	 * 
	 * @return sra.smalltalk.StImage[]
	 */
	public StImage[] plates() {
		StImage[] images = this.plates;
		return images;
	}

	/**
	 * Answer the saturation value.
	 * 
	 * @return double
	 */
	public double saturation() {
		double value = this.xy().y();
		return value;
	}

	/**
	 * Set the saturation value.
	 * 
	 * @param value double
	 */
	public void saturation_(double value) {
		this.xy_(new Jun2dPoint(this.xy().x(), value));
	}
}
