package sra.jun.goodies.cursors;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.Hashtable;
import sra.smalltalk.StImage;

/**
 * JunCursors class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/11/24 (by MATSUDA Ryouichi)
 *  @updated   1999/08/04 (by nisinaka)
 *  @updated   2001/11/20 (by nisinaka)
 *  @updated   2002/04/04 (by nisinaka)
 *  @updated   2002/11/27 (by nisinaka)
 *  @updated   2003/03/04 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun472 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCursors.java,v 6.0 2003/10/28 10:52:08 nisinaka Exp $
 */
public class JunCursors extends sra.smalltalk.StObject {

	public static final int WIDTH = 16;
	public static final int HEIGHT = 16;
	public static final Hashtable images = new Hashtable();
	public static final Hashtable cursors = new Hashtable();

	protected Cursor _cursor;
	protected Frame[] _frames = null;
	protected Cursor[] _savedCursors = null;

	/**
	 * Create a new instance of JunCursor.
	 *
	 * @param aCursor java.awt.Cursor
	 */
	public JunCursors(Cursor aCursor) {
		_cursor = aCursor;
	}

	/**
	 * Create an image from the array of pixels.
	 * 
	 * @param pixels int[]
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static BufferedImage _CreateImage(int[] pixels) {
		BufferedImage image = new BufferedImage(WIDTH, HEIGHT, BufferedImage.TYPE_INT_ARGB);
		image.setRGB(0, 0, WIDTH, HEIGHT, pixels, 0, WIDTH);
		return image;
	}

	/**
	 * Create an array of pixels from an image string and a mask string.
	 * 
	 * @param imageString java.lang.String
	 * @param maskString java.lang.String
	 * 
	 * @return int[]
	 */
	public static int[] _Pixels(String imageString, String maskString) {
		int[] pixels = new int[WIDTH * HEIGHT];
		for (int i = 0; i < (WIDTH * HEIGHT); i++) {
			int pixel = 0;
			if (imageString.charAt(i) != '0') {
				pixel = Color.black.getRGB();
			} else if (maskString.charAt(i) != '0') {
				pixel = Color.white.getRGB();
			}
			pixels[i] = pixel;
		}

		return pixels;
	}

	/**
	 * Resize the image to fit as a cursor.
	 * 
	 * @param anImage java.awt.image.BufferedImage
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static BufferedImage _ResizeToFit(BufferedImage anImage) {
		Dimension size = Toolkit.getDefaultToolkit().getBestCursorSize(WIDTH, HEIGHT);
		BufferedImage cursorImage = new BufferedImage(size.width, size.height, BufferedImage.TYPE_INT_ARGB);
		cursorImage.setData(anImage.getRaster());
		return cursorImage;
	}

	/**
	 * Answer an anti oblique cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor AntiObliqueCursor() {
		if (cursors.contains($("antiOblique")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(AntiObliqueCursorImage()), new Point(7, 7), "antiOblique");
			cursors.put($("antiOblique"), cursor);
		}

		return (Cursor) cursors.get($("antiOblique"));
	}

	/**
	 * Answer an image of anti oblique cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage AntiObliqueCursorImage() {
		if (images.contains($("antiOblique")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0001111000000000";
			imageString += "0001100000000000";
			imageString += "0001010000000000";
			imageString += "0001001000000000";
			imageString += "0000000100000000";
			imageString += "0000000010000000";
			imageString += "0000000001001000";
			imageString += "0000000000101000";
			imageString += "0000000000011000";
			imageString += "0000000001111000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0001111000000000";
			maskString += "0011111100000000";
			maskString += "0011111000000000";
			maskString += "0011111000000000";
			maskString += "0011111100000000";
			maskString += "0001001110000000";
			maskString += "0000000111001000";
			maskString += "0000000011111100";
			maskString += "0000000001111100";
			maskString += "0000000001111100";
			maskString += "0000000011111100";
			maskString += "0000000001111000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";

			images.put($("antiOblique"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("antiOblique"));
	}

	/**
	 * Answer an arrow cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor ArrowCursor() {
		if (cursors.contains($("arrow")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(ArrowCursorImage()), new Point(7, 7), "arrow");
			cursors.put($("arrow"), cursor);
		}

		return (Cursor) cursors.get($("arrow"));
	}

	/**
	 * Answer an image of arrow cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage ArrowCursorImage() {
		if (images.contains($("arrow")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000010000";
			imageString += "0000000000011000";
			imageString += "0011111111111100";
			imageString += "0000000000011000";
			imageString += "0000000000010000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000010000";
			maskString += "0000000000111000";
			maskString += "0011111111111100";
			maskString += "0111111111111110";
			maskString += "0011111111111100";
			maskString += "0000000000111000";
			maskString += "0000000000010000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			images.put($("arrow"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("arrow"));
	}

	/**
	 * Answer a between cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor BetweenCursor() {
		if (cursors.contains($("between")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(BetweenCursorImage()), new Point(7, 7), "between");
			cursors.put($("between"), cursor);
		}

		return (Cursor) cursors.get($("between"));
	}

	/**
	 * Answer an image of between cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage BetweenCursorImage() {
		if (images.contains($("between")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000100000010000";
			imageString += "0001100000011000";
			imageString += "0011111111111100";
			imageString += "0001100000011000";
			imageString += "0000100000010000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000100000010000";
			maskString += "0001110000111000";
			maskString += "0011111111111100";
			maskString += "0111111111111110";
			maskString += "0011111111111100";
			maskString += "0001110000111000";
			maskString += "0000100000010000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			images.put($("between"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("between"));
	}

	/**
	 * Answer a brush cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor BrushCursor() {
		if (cursors.contains($("brush")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(BrushCursorImage()), new Point(2, 14), "brush");
			cursors.put($("brush"), cursor);
		}

		return (Cursor) cursors.get($("brush"));
	}

	/**
	 * Answer an image of brush cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage BrushCursorImage() {
		if (images.contains($("brush")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000001001000";
			imageString += "0000000001001000";
			imageString += "0000000010010000";
			imageString += "0000000010010000";
			imageString += "0000000100100000";
			imageString += "0000000100100000";
			imageString += "0000001111000000";
			imageString += "0000010011000000";
			imageString += "0000100011000000";
			imageString += "0000100111000000";
			imageString += "0001001110000000";
			imageString += "0001011100000000";
			imageString += "0010111000000000";
			imageString += "0111100000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000001001000";
			maskString += "0000000011111100";
			maskString += "0000000011111100";
			maskString += "0000000111111000";
			maskString += "0000000111111000";
			maskString += "0000001111110000";
			maskString += "0000001111110000";
			maskString += "0000011111100000";
			maskString += "0000111111100000";
			maskString += "0001111111100000";
			maskString += "0001111111100000";
			maskString += "0011111111000000";
			maskString += "0011111110000000";
			maskString += "0111111100000000";
			maskString += "1111111000000000";
			maskString += "0111100000000000";
			images.put($("brush"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("brush"));
	}

	/**
	 * Answer a bull cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor BullCursor() {
		if (cursors.contains($("bull")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(BullCursorImage()), new Point(7, 7), "bull");
			cursors.put($("bull"), cursor);
		}

		return (Cursor) cursors.get($("bull"));
	}

	/**
	 * Answer an image of bull cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage BullCursorImage() {
		if (images.contains($("bull")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000011111000000";
			imageString += "0001100100110000";
			imageString += "0010000100001000";
			imageString += "0100000100000100";
			imageString += "0100000100000100";
			imageString += "1000000100000010";
			imageString += "1000000100000010";
			imageString += "1111111111111110";
			imageString += "1000000100000010";
			imageString += "1000000100000010";
			imageString += "0100000100000100";
			imageString += "0100000100000100";
			imageString += "0010000100001000";
			imageString += "0001100100110000";
			imageString += "0000011111000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000111111100000";
			maskString += "0011111111111000";
			maskString += "0111001110011100";
			maskString += "1110001110001110";
			maskString += "1110001110001110";
			maskString += "1100001110000110";
			maskString += "1111111111111110";
			maskString += "1111111111111110";
			maskString += "1111111111111110";
			maskString += "1100001110000110";
			maskString += "1110001110001110";
			maskString += "1110001110001110";
			maskString += "0111001110011100";
			maskString += "0011111111111000";
			maskString += "0000111111100000";
			images.put($("bull"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("bull"));
	}

	/**
	 * Answer a camera cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor CameraCursor() {
		if (cursors.contains($("camera")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(CameraCursorImage()), new Point(7, 7), "camera");
			cursors.put($("camera"), cursor);
		}

		return (Cursor) cursors.get($("camera"));
	}

	/**
	 * Answer an image of camera cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage CameraCursorImage() {
		if (images.contains($("camera")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0011100000000000";
			imageString += "0010100000111100";
			imageString += "0111111111111110";
			imageString += "0100000000000010";
			imageString += "0100001111000010";
			imageString += "0100011001100010";
			imageString += "0100110000110010";
			imageString += "0100100000010010";
			imageString += "0100100001010010";
			imageString += "0100110010110010";
			imageString += "0100011001100010";
			imageString += "0100001111000010";
			imageString += "0100000000000010";
			imageString += "0111111111111110";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0011100000000000";
			maskString += "0111110000111100";
			maskString += "0111111111111110";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "0111111111111110";
			images.put($("camera"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("camera"));
	}

	/**
	 * Answer a clock cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor ClockCursor() {
		if (cursors.contains($("clock")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(ClockCursorImage()), new Point(5, 8), "clock");
			cursors.put($("clock"), cursor);
		}

		return (Cursor) cursors.get($("clock"));
	}

	/**
	 * Answer an image of clock cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage ClockCursorImage() {
		if (images.contains($("clock")) == false) {
			String imageString = "";
			imageString += "0011111100000000";
			imageString += "0011111100000000";
			imageString += "0011111100000000";
			imageString += "0011111100000000";
			imageString += "0100000010000000";
			imageString += "1000010001000000";
			imageString += "1000010001000000";
			imageString += "1000010001100000";
			imageString += "1001110001100000";
			imageString += "1000000001000000";
			imageString += "1000000001000000";
			imageString += "0100000010000000";
			imageString += "0011111100000000";
			imageString += "0011111100000000";
			imageString += "0011111100000000";
			imageString += "0011111100000000";

			String maskString = "";
			maskString += "0011111100000000";
			maskString += "0011111100000000";
			maskString += "0011111100000000";
			maskString += "0011111100000000";
			maskString += "0111111110000000";
			maskString += "1111111111000000";
			maskString += "1111111111000000";
			maskString += "1111111111100000";
			maskString += "1111111111100000";
			maskString += "1111111111000000";
			maskString += "1111111111000000";
			maskString += "0111111110000000";
			maskString += "0011111100000000";
			maskString += "0011111100000000";
			maskString += "0011111100000000";
			maskString += "0011111100000000";
			images.put($("clock"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("clock"));
	}

	/**
	 * Answer a corner1 cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor Corner1Cursor() {
		if (cursors.contains($("corner1")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(Corner1CursorImage()), new Point(7, 7), "corner1");
			cursors.put($("corner1"), cursor);
		}

		return (Cursor) cursors.get($("corner1"));
	}

	/**
	 * Answer an image of corner1 cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage Corner1CursorImage() {
		if (images.contains($("corner1")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000011111100";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000100000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001111111100";
			maskString += "0000001011111110";
			maskString += "0000001111111100";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			images.put($("corner1"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("corner1"));
	}

	/**
	 * Answer a corner2 cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor Corner2Cursor() {
		if (cursors.contains($("corner2")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(Corner2CursorImage()), new Point(7, 7), "corner2");
			cursors.put($("corner2"), cursor);
		}

		return (Cursor) cursors.get($("corner2"));
	}

	/**
	 * Answer an image of corner2 cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage Corner2CursorImage() {
		if (images.contains($("corner2")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0111111000000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0111111110000000";
			maskString += "1111111010000000";
			maskString += "0111111110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000000100000000";
			maskString += "0000000000000000";
			images.put($("corner2"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("corner2"));
	}

	/**
	 * Answer a corner3 cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor Corner3Cursor() {
		if (cursors.contains($("corner3")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(Corner3CursorImage()), new Point(7, 7), "corner3");
			cursors.put($("corner3"), cursor);
		}

		return (Cursor) cursors.get($("corner3"));
	}

	/**
	 * Answer an image of corner3 cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage Corner3CursorImage() {
		if (images.contains($("corner3")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0111111000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000100000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0111111110000000";
			maskString += "1111111010000000";
			maskString += "0111111110000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			images.put($("corner3"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("corner3"));
	}

	/**
	 * Answer a corner4 cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor Corner4Cursor() {
		if (cursors.contains($("corner4")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(Corner4CursorImage()), new Point(7, 7), "corner4");
			cursors.put($("corner4"), cursor);
		}

		return (Cursor) cursors.get($("corner4"));
	}

	/**
	 * Answer an image of corner4 cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage Corner4CursorImage() {
		if (images.contains($("corner4")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000011111100";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000001111111100";
			maskString += "0000001011111110";
			maskString += "0000001111111100";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000000100000000";
			maskString += "0000000000000000";
			images.put($("corner4"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("corner4"));
	}

	/**
	 * Answer a cross cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor CrossCursor() {
		if (cursors.contains($("cross")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(CrossCursorImage()), new Point(7, 7), "cross");
			cursors.put($("cross"), cursor);
		}

		return (Cursor) cursors.get($("cross"));
	}

	/**
	 * Answer an image of cross cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage CrossCursorImage() {
		if (images.contains($("cross")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000000000000";
			imageString += "0111110001111110";
			imageString += "0000000000000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000100000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0111111011111110";
			maskString += "1111110001111111";
			maskString += "0111111011111110";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000000100000000";
			maskString += "0000000000000000";
			images.put($("cross"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("cross"));
	}

	/**
	 * Answer a dolly2 cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor Dolly2Cursor() {
		if (cursors.contains($("dolly2")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(Dolly2CursorImage()), new Point(7, 7), "dolly2");
			cursors.put($("dolly2"), cursor);
		}

		return (Cursor) cursors.get($("dolly2"));
	}

	/**
	 * Answer an image of dolly cursor 2.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage Dolly2CursorImage() {
		if (images.contains($("dolly2")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000011110000";
			imageString += "0000000000110000";
			imageString += "0000000000110000";
			imageString += "0000111100110000";
			imageString += "0011000011110000";
			imageString += "0011000000110000";
			imageString += "0011000000110000";
			imageString += "0011000000110000";
			imageString += "0000111111111100";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000011110000";
			maskString += "0000001111111100";
			maskString += "0000000011111100";
			maskString += "0000111111111100";
			maskString += "0011111111111100";
			maskString += "1111111111111100";
			maskString += "1111110011111100";
			maskString += "1111110011111100";
			maskString += "1111111111111100";
			maskString += "0011111111111111";
			maskString += "0000111111111100";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			images.put($("dolly2"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("dolly2"));
	}

	/**
	 * Answer a dolly cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor DollyCursor() {
		if (cursors.contains($("dolly")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(DollyCursorImage()), new Point(7, 7), "dolly");
			cursors.put($("dolly"), cursor);
		}

		return (Cursor) cursors.get($("dolly"));
	}

	/**
	 * Answer an image of dolly cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage DollyCursorImage() {
		if (images.contains($("dolly")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000011000000";
			imageString += "0000000001000000";
			imageString += "0000000001000000";
			imageString += "0000001101000000";
			imageString += "0000010011000000";
			imageString += "0000010001000000";
			imageString += "0000010001000000";
			imageString += "0000010001000000";
			imageString += "0000001111100000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000011000000";
			maskString += "0000000111100000";
			maskString += "0000000011100000";
			maskString += "0000001111100000";
			maskString += "0000011111100000";
			maskString += "0000111111100000";
			maskString += "0000111011100000";
			maskString += "0000111011100000";
			maskString += "0000111111100000";
			maskString += "0000011111110000";
			maskString += "0000001111100000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			images.put($("dolly"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("dolly"));
	}

	/**
	 * Answer an down arrow cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor DownArrowCursor() {
		if (cursors.contains($("downArrow")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(DownArrowCursorImage()), new Point(7, 7), "downArrow");
			cursors.put($("downArrow"), cursor);
		}

		return (Cursor) cursors.get($("downArrow"));
	}

	/**
	 * Answer an image of down arrow cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage DownArrowCursorImage() {
		if (images.contains($("downArrow")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000011111000000";
			imageString += "0000001110000000";
			imageString += "0000000100000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000010000000";
			maskString += "0000000111000000";
			maskString += "0000000111000000";
			maskString += "0000000111000000";
			maskString += "0000000111000000";
			maskString += "0000000111000000";
			maskString += "0000000111000000";
			maskString += "0000000111000000";
			maskString += "0000000111000000";
			maskString += "0000001111100000";
			maskString += "0000011111110000";
			maskString += "0000001111100000";
			maskString += "0000000111000000";
			maskString += "0000000010000000";
			maskString += "0000000000000000";

			images.put($("downArrow"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("downArrow"));
	}

	/**
	 * Answer an epilogue cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor EpilogueCursor() {
		if (cursors.contains($("epilogue")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(EpilogueCursorImage()), new Point(7, 7), "epilogue");
			cursors.put($("epilogue"), cursor);
		}

		return (Cursor) cursors.get($("epilogue"));
	}

	/**
	 * Answer an image of epilogue cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage EpilogueCursorImage() {
		if (images.contains($("epilogue")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0001100000001000";
			imageString += "0001110000001000";
			imageString += "0001111000001000";
			imageString += "0001101100001000";
			imageString += "0001100110001000";
			imageString += "0001100011001000";
			imageString += "0001100011001000";
			imageString += "0001100110001000";
			imageString += "0001101100001000";
			imageString += "0001111000001000";
			imageString += "0001110000001000";
			imageString += "0001100000001000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0001100000001000";
			maskString += "0011110000011100";
			maskString += "0011111000011100";
			maskString += "0011111100011100";
			maskString += "0011111110011100";
			maskString += "0011111111011100";
			maskString += "0011110111111100";
			maskString += "0011110111111100";
			maskString += "0011111111011100";
			maskString += "0011111110011100";
			maskString += "0011111100011100";
			maskString += "0011111000011100";
			maskString += "0011110000011100";
			maskString += "0001100000001000";
			maskString += "0000000000000000";
			images.put($("epilogue"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("epilogue"));
	}

	/**
	 * Answer an eraser cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor EraserCursor() {
		if (cursors.contains($("eraser")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(EraserCursorImage()), new Point(1, 12), "eraser");
			cursors.put($("eraser"), cursor);
		}

		return (Cursor) cursors.get($("eraser"));
	}

	/**
	 * Answer an image of eraser cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage EraserCursorImage() {
		if (images.contains($("eraser")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000111111110";
			imageString += "0000001000000110";
			imageString += "0000010000001010";
			imageString += "0000100000010010";
			imageString += "0001000000100100";
			imageString += "0010000001001000";
			imageString += "0111111110010000";
			imageString += "0100000010100000";
			imageString += "0100000011000000";
			imageString += "0111111110000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000111111110";
			maskString += "0000001111111111";
			maskString += "0000011111111111";
			maskString += "0000111111111111";
			maskString += "0001111111111111";
			maskString += "0011111111111110";
			maskString += "0111111111111100";
			maskString += "1111111111111000";
			maskString += "1111111111110000";
			maskString += "1111111111100000";
			maskString += "1111111111000000";
			maskString += "0111111110000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			images.put($("eraser"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("eraser"));
	}

	/**
	 * Answer an epilogue cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor ExecuteCursor() {
		if (cursors.contains($("execute")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(ExecuteCursorImage()), new Point(1, 1), "execute");
			cursors.put($("execute"), cursor);
		}

		return (Cursor) cursors.get($("execute"));
	}

	/**
	 * Answer an image of execute cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage ExecuteCursorImage() {
		if (images.contains($("execute")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0100000000010000";
			imageString += "0110000000010000";
			imageString += "0111000000111000";
			imageString += "0111100111111111";
			imageString += "0111110011000110";
			imageString += "0111111001000100";
			imageString += "0111111101111100";
			imageString += "0111110001101100";
			imageString += "0111110011000110";
			imageString += "0100110010000010";
			imageString += "0000011000000000";
			imageString += "0000011000000000";
			imageString += "0000001100000000";
			imageString += "0000001100000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "1100000000010000";
			maskString += "1110000000111000";
			maskString += "1111000000111000";
			maskString += "1111100111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111110";
			maskString += "1111111111111110";
			maskString += "1111111011111110";
			maskString += "1111111111101111";
			maskString += "1111111111000111";
			maskString += "1000111111000111";
			maskString += "0000111100000000";
			maskString += "0000011110000000";
			maskString += "0000011110000000";
			maskString += "0000001100000000";

			images.put($("execute"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("execute"));
	}

	/**
	 * Answer a film cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor FilmCursor() {
		if (cursors.contains($("film")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(FilmCursorImage()), new Point(7, 7), "film");
			cursors.put($("film"), cursor);
		}

		return (Cursor) cursors.get($("film"));
	}

	/**
	 * Answer an image of film cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage FilmCursorImage() {
		if (images.contains($("film")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0111000000001110";
			imageString += "0111000000001110";
			imageString += "0101111111111010";
			imageString += "0111000000001110";
			imageString += "0111000000001110";
			imageString += "0101000000001010";
			imageString += "0111000000001110";
			imageString += "0111000000001110";
			imageString += "0101000000001010";
			imageString += "0111000000001110";
			imageString += "0111000000001110";
			imageString += "0101111111111010";
			imageString += "0111000000001110";
			imageString += "0111000000001110";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0111000000001110";
			maskString += "1111100000011111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111100000011111";
			maskString += "0111000000001110";
			images.put($("film"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("film"));
	}

	/**
	 * Answer a frame cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor FrameCursor() {
		if (cursors.contains($("frame")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(FrameCursorImage()), new Point(7, 7), "frame");
			cursors.put($("frame"), cursor);
		}

		return (Cursor) cursors.get($("frame"));
	}

	/**
	 * Answer an image of frame cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage FrameCursorImage() {
		if (images.contains($("frame")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0111100111001110";
			imageString += "0100000000000010";
			imageString += "0100000000000010";
			imageString += "0000000000000010";
			imageString += "0000000000000000";
			imageString += "0100000000000000";
			imageString += "0100000000000010";
			imageString += "0100000000000010";
			imageString += "0000000000000010";
			imageString += "0000000000000000";
			imageString += "0100000000000000";
			imageString += "0100000000000010";
			imageString += "0100000000000010";
			imageString += "0111001110011110";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0111100111001110";
			maskString += "1111111111111111";
			maskString += "1111100111001111";
			maskString += "1110000000000111";
			maskString += "0100000000000111";
			maskString += "0100000000000010";
			maskString += "1110000000000010";
			maskString += "1110000000000111";
			maskString += "1110000000000111";
			maskString += "0100000000000111";
			maskString += "0100000000000010";
			maskString += "1110000000000010";
			maskString += "1110000000000111";
			maskString += "1111001110011111";
			maskString += "1111111111111111";
			maskString += "0111001110011110";
			images.put($("frame"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("frame"));
	}

	/**
	 * Answer a gauge cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor GaugeCursor() {
		if (cursors.contains($("gauge")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(GaugeCursorImage()), new Point(7, 7), "gauge");
			cursors.put($("gauge"), cursor);
		}

		return (Cursor) cursors.get($("gauge"));
	}

	/**
	 * Answer an image of gauge cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage GaugeCursorImage() {
		if (images.contains($("gauge")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0001111011111000";
			imageString += "0000000010001000";
			imageString += "0000011010001000";
			imageString += "0000000010001000";
			imageString += "0000011010001000";
			imageString += "0000000010001000";
			imageString += "0000111010001000";
			imageString += "0000000010001000";
			imageString += "0000011011111000";
			imageString += "0000000011111000";
			imageString += "0000011011111000";
			imageString += "0000000011111000";
			imageString += "0001111011111000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0001111011111000";
			maskString += "0011111111111100";
			maskString += "0001111111111100";
			maskString += "0000111111111100";
			maskString += "0000011111111100";
			maskString += "0000111111111100";
			maskString += "0000111111111100";
			maskString += "0001111111111100";
			maskString += "0000111111111100";
			maskString += "0000111111111100";
			maskString += "0000011111111100";
			maskString += "0000111111111100";
			maskString += "0001111111111100";
			maskString += "0011111111111100";
			maskString += "0001111011111000";
			images.put($("gauge"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("gauge"));
	}

	/**
	 * Answer a glass cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor GlassCursor() {
		if (cursors.contains($("glass")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(GlassCursorImage()), new Point(5, 5), "glass");
			cursors.put($("glass"), cursor);
		}

		return (Cursor) cursors.get($("glass"));
	}

	/**
	 * Answer an image of grass cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage GlassCursorImage() {
		if (images.contains($("glass")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000111100000000";
			imageString += "0011000011000000";
			imageString += "0010000001000000";
			imageString += "0100000000100000";
			imageString += "0100000000100000";
			imageString += "0100000000100000";
			imageString += "0100000000100000";
			imageString += "0010000001000000";
			imageString += "0011000011100000";
			imageString += "0000111101110000";
			imageString += "0000000000111000";
			imageString += "0000000000011100";
			imageString += "0000000000001110";
			imageString += "0000000000000110";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000111100000000";
			maskString += "0011111111000000";
			maskString += "0111111111100000";
			maskString += "0111000011100000";
			maskString += "1110000001110000";
			maskString += "1110000001110000";
			maskString += "1110000001110000";
			maskString += "1110000001110000";
			maskString += "0111000011100000";
			maskString += "0111111111110000";
			maskString += "0011111111111000";
			maskString += "0000111101111100";
			maskString += "0000000000111110";
			maskString += "0000000000011111";
			maskString += "0000000000001111";
			maskString += "0000000000000110";
			images.put($("glass"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("glass"));
	}

	/**
	 * Answer a go cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor GoCursor() {
		if (cursors.contains($("go")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(GoCursorImage()), new Point(7, 7), "go");
			cursors.put($("go"), cursor);
		}

		return (Cursor) cursors.get($("go"));
	}

	/**
	 * Answer an image of go cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage GoCursorImage() {
		if (images.contains($("go")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0001110000111000";
			imageString += "0010001001000100";
			imageString += "0010000001000100";
			imageString += "0010000001000100";
			imageString += "0010011101000100";
			imageString += "0010001001000100";
			imageString += "0010001001000100";
			imageString += "0001110000111000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0001110000111000";
			maskString += "0011111001111100";
			maskString += "0111111111111110";
			maskString += "0111001011101110";
			maskString += "0111011111101110";
			maskString += "0111111111101110";
			maskString += "0111011111101110";
			maskString += "0111111111111110";
			maskString += "0011111001111100";
			maskString += "0001110000111000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			images.put($("go"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("go"));
	}

	/**
	 * Answer a hand cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor HandCursor() {
		if (cursors.contains($("hand")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(HandCursorImage()), new Point(7, 7), "hand");
			cursors.put($("hand"), cursor);
		}

		return (Cursor) cursors.get($("hand"));
	}

	/**
	 * Answer an image of hand cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage HandCursorImage() {
		if (images.contains($("hand")) == false) {
			String imageString = "";
			imageString += "0000000110000000";
			imageString += "0001101001110000";
			imageString += "0010011001001000";
			imageString += "0010011001001010";
			imageString += "0001001001001101";
			imageString += "0001001001001001";
			imageString += "0110100000001001";
			imageString += "1001100000000001";
			imageString += "1000100000000010";
			imageString += "0100000000000010";
			imageString += "0010000000000010";
			imageString += "0010000000000100";
			imageString += "0001000000000100";
			imageString += "0000100000001000";
			imageString += "0000010000001000";
			imageString += "0000010000001000";

			String maskString = "";
			maskString += "0000000110000000";
			maskString += "0001101111110000";
			maskString += "0011111111111000";
			maskString += "0011111111111010";
			maskString += "0001111111111111";
			maskString += "0001111111111111";
			maskString += "0110111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111110";
			maskString += "0111111111111110";
			maskString += "0011111111111110";
			maskString += "0011111111111100";
			maskString += "0001111111111100";
			maskString += "0000111111111000";
			maskString += "0000011111111000";
			maskString += "0000011111111000";
			images.put($("hand"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("hand"));
	}

	/**
	 * Answer a juice cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor JuiceCursor() {
		if (cursors.contains($("juice")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(JuiceCursorImage()), new Point(7, 8), "juice");
			cursors.put($("juice"), cursor);
		}

		return (Cursor) cursors.get($("juice"));
	}

	/**
	 * Answer an image of juice cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage JuiceCursorImage() {
		if (images.contains($("juice")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000001010000000";
			imageString += "0000001010000000";
			imageString += "0000010001000000";
			imageString += "0000100000100000";
			imageString += "0000100000100000";
			imageString += "0001000000010000";
			imageString += "0001000000010000";
			imageString += "0001000000010000";
			imageString += "0001000000010000";
			imageString += "0000100000100000";
			imageString += "0000010001000000";
			imageString += "0000001110000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000100000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000011111000000";
			maskString += "0000011111000000";
			maskString += "0000111011100000";
			maskString += "0001110001110000";
			maskString += "0001110001110000";
			maskString += "0011100000111000";
			maskString += "0011100000111000";
			maskString += "0011100000111000";
			maskString += "0011100000111000";
			maskString += "0001110001110000";
			maskString += "0000111111100000";
			maskString += "0000011111000000";
			maskString += "0000001110000000";
			images.put($("juice"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("juice"));
	}

	/**
	 * Answer a lasso cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor LassoCursor() {
		if (cursors.contains($("lasso")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(LassoCursorImage()), new Point(3, 14), "lasso");
			cursors.put($("lasso"), cursor);
		}

		return (Cursor) cursors.get($("lasso"));
	}

	/**
	 * Answer an image of lasso cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage LassoCursorImage() {
		if (images.contains($("lasso")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000111110000";
			imageString += "0000111000001100";
			imageString += "0001000000000010";
			imageString += "0010000000000010";
			imageString += "0100000000000010";
			imageString += "0100000000001100";
			imageString += "0100000000110000";
			imageString += "0011100011000000";
			imageString += "0110011100000000";
			imageString += "0101010000000000";
			imageString += "0011100000000000";
			imageString += "0000100000000000";
			imageString += "0000100000000000";
			imageString += "0001000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000111110000";
			maskString += "0000111111111100";
			maskString += "0001111111111110";
			maskString += "0011111000001111";
			maskString += "0111000000000111";
			maskString += "1110000000001111";
			maskString += "1110000000111110";
			maskString += "1111100011111100";
			maskString += "0111111111110000";
			maskString += "1111111111000000";
			maskString += "1111111100000000";
			maskString += "0111110000000000";
			maskString += "0011110000000000";
			maskString += "0001110000000000";
			maskString += "0011100000000000";
			maskString += "0001000000000000";
			images.put($("lasso"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("lasso"));
	}

	/**
	 * Answer a layer cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor LayerCursor() {
		if (cursors.contains($("layer")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(LayerCursorImage()), new Point(7, 7), "layer");
			cursors.put($("layer"), cursor);
		}

		return (Cursor) cursors.get($("layer"));
	}

	/**
	 * Answer an image of layer cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage LayerCursorImage() {
		if (images.contains($("layer")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000011111111110";
			imageString += "0000010000000010";
			imageString += "0001111111111010";
			imageString += "0001000000001010";
			imageString += "0111111111101010";
			imageString += "0100000000101010";
			imageString += "0100000000101010";
			imageString += "0100000000101010";
			imageString += "0100000000101010";
			imageString += "0100000000101110";
			imageString += "0100000000101000";
			imageString += "0100000000111000";
			imageString += "0100000000100000";
			imageString += "0111111111100000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000011111111110";
			maskString += "0000111111111111";
			maskString += "0001111111111111";
			maskString += "0011111111111111";
			maskString += "0111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111110";
			maskString += "1111111111111100";
			maskString += "1111111111111000";
			maskString += "1111111111110000";
			maskString += "0111111111100000";
			images.put($("layer"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("layer"));
	}

	/**
	 * Answer a left arrow cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor LeftArrowCursor() {
		if (cursors.contains($("leftArrow")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(LeftArrowCursorImage()), new Point(7, 7), "leftArrow");
			cursors.put($("leftArrow"), cursor);
		}

		return (Cursor) cursors.get($("leftArrow"));
	}

	/**
	 * Answer an image of left arrow cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage LeftArrowCursorImage() {
		if (images.contains($("leftArrow")) == false) {
			BufferedImage leftArrow = new StImage(RightArrowCursorImage()).rotatedByQuadrants_(2).image();
			images.put($("leftArrow"), leftArrow);
		}

		return (BufferedImage) images.get($("leftArrow"));
	}

	/**
	 * Answer a next cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor NextCursor() {
		if (cursors.contains($("next")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(NextCursorImage()), new Point(7, 7), "next");
			cursors.put($("next"), cursor);
		}

		return (Cursor) cursors.get($("next"));
	}

	/**
	 * Answer an image of next cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage NextCursorImage() {
		if (images.contains($("next")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0001001100000000";
			imageString += "0001001110000000";
			imageString += "0001001111000000";
			imageString += "0001001101100000";
			imageString += "0001001100110000";
			imageString += "0001001100011000";
			imageString += "0001001100011000";
			imageString += "0001001100110000";
			imageString += "0001001101100000";
			imageString += "0001001111000000";
			imageString += "0001001110000000";
			imageString += "0001001100000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0001001100000000";
			maskString += "0011111110000000";
			maskString += "0011111111000000";
			maskString += "0011111111100000";
			maskString += "0011111111110000";
			maskString += "0011111111111000";
			maskString += "0011111110111100";
			maskString += "0011111110111100";
			maskString += "0011111111111000";
			maskString += "0011111111110000";
			maskString += "0011111111100000";
			maskString += "0011111111000000";
			maskString += "0011111110000000";
			maskString += "0001001100000000";
			maskString += "0000000000000000";
			images.put($("next"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("next"));
	}

	/**
	 * Answer a normal cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor NormalCursor() {
		if (cursors.contains($("normal")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(NormalCursorImage()), new Point(7, 7), "normal");
			cursors.put($("normal"), cursor);
		}

		return (Cursor) cursors.get($("normal"));
	}

	/**
	 * Answer an image of normal cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage NormalCursorImage() {
		if (images.contains($("normal")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000010000000000";
			imageString += "0000011000000000";
			imageString += "0000011100000000";
			imageString += "0000011110000000";
			imageString += "0000011111000000";
			imageString += "0000011111100000";
			imageString += "0000011111110000";
			imageString += "0000011111000000";
			imageString += "0000011111000000";
			imageString += "0000010011000000";
			imageString += "0000000001100000";
			imageString += "0000000001100000";
			imageString += "0000000000110000";
			imageString += "0000000000110000";
			imageString += "0000000000000000)";

			String maskString = "";
			maskString += "0000110000000000";
			maskString += "0000111000000000";
			maskString += "0000111100000000";
			maskString += "0000111110000000";
			maskString += "0000111111000000";
			maskString += "0000111111100000";
			maskString += "0000111111110000";
			maskString += "0000111111111000";
			maskString += "0000111111100000";
			maskString += "0000111111100000";
			maskString += "0000111111100000";
			maskString += "0000100011110000";
			maskString += "0000000011110000";
			maskString += "0000000001111000";
			maskString += "0000000001111000";
			maskString += "0000000000110000";
			images.put($("normal"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("normal"));
	}

	/**
	 * Answer an normal oblique cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor NormalObliqueCursor() {
		if (cursors.contains($("normalOblique")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(NormalObliqueCursorImage()), new Point(7, 7), "normalOblique");
			cursors.put($("normalOblique"), cursor);
		}

		return (Cursor) cursors.get($("normalOblique"));
	}

	/**
	 * Answer an image of normal oblique cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage NormalObliqueCursorImage() {
		if (images.contains($("normalOblique")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000001111000";
			imageString += "0000000000011000";
			imageString += "0000000000101000";
			imageString += "0000000001001000";
			imageString += "0000000010000000";
			imageString += "0000000100000000";
			imageString += "0001001000000000";
			imageString += "0001010000000000";
			imageString += "0001100000000000";
			imageString += "0001111000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000001111000";
			maskString += "0000000011111100";
			maskString += "0000000001111100";
			maskString += "0000000001111100";
			maskString += "0000000011111100";
			maskString += "0000000111001000";
			maskString += "0001001110000000";
			maskString += "0011111100000000";
			maskString += "0011111000000000";
			maskString += "0011111000000000";
			maskString += "0011111100000000";
			maskString += "0001111000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";

			images.put($("normalOblique"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("normalOblique"));
	}

	/**
	 * Answer a paint cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor PaintCursor() {
		if (cursors.contains($("paint")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(PaintCursorImage()), new Point(13, 14), "paint");
			cursors.put($("paint"), cursor);
		}

		return (Cursor) cursors.get($("paint"));
	}

	/**
	 * Answer an image of paint cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage PaintCursorImage() {
		if (images.contains($("paint")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000011100000000";
			imageString += "0000100110000000";
			imageString += "0000101011000000";
			imageString += "0000110010110000";
			imageString += "0000100010011100";
			imageString += "0001000010001110";
			imageString += "0010000101000110";
			imageString += "0100000010000110";
			imageString += "0100000000001110";
			imageString += "0010000000010110";
			imageString += "0001000000100110";
			imageString += "0000100001000110";
			imageString += "0000010010000110";
			imageString += "0000001100000100";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000011100000000";
			maskString += "0000111110000000";
			maskString += "0001111111000000";
			maskString += "0001111111110000";
			maskString += "0001111111111100";
			maskString += "0001111111111110";
			maskString += "0011111111111111";
			maskString += "0111111111111111";
			maskString += "1111111111111111";
			maskString += "1111111111111111";
			maskString += "0111111111111111";
			maskString += "0011111111111111";
			maskString += "0001111111101111";
			maskString += "0000111111001111";
			maskString += "0000011110001110";
			maskString += "0000001100000100";
			images.put($("paint"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("paint"));
	}

	/**
	 * Answer a paper cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor PaperCursor() {
		if (cursors.contains($("paper")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(PaperCursorImage()), new Point(7, 7), "paper");
			cursors.put($("paper"), cursor);
		}

		return (Cursor) cursors.get($("paper"));
	}

	/**
	 * Answer an image of paper cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage PaperCursorImage() {
		if (images.contains($("paper")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0001111111111000";
			imageString += "0001000000001000";
			imageString += "0001011111101000";
			imageString += "0001000000001000";
			imageString += "0001011111101000";
			imageString += "0001000000001000";
			imageString += "0001011111101000";
			imageString += "0001000000001000";
			imageString += "0001011111101000";
			imageString += "0001000000001000";
			imageString += "0001011101111000";
			imageString += "0001000001010000";
			imageString += "0001000001100000";
			imageString += "0001111111000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0001111111111000";
			maskString += "0011111111111100";
			maskString += "0011111111111100";
			maskString += "0011111111111100";
			maskString += "0011111111111100";
			maskString += "0011111111111100";
			maskString += "0011111111111100";
			maskString += "0011111111111100";
			maskString += "0011111111111100";
			maskString += "0011111111111100";
			maskString += "0011111111111100";
			maskString += "0011111111111100";
			maskString += "0011111111111000";
			maskString += "0011111111110000";
			maskString += "0011111111100000";
			maskString += "0001111111000000";
			images.put($("paper"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("paper"));
	}

	/**
	 * Answer a pause cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor PauseCursor() {
		if (cursors.contains($("pause")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(PauseCursorImage()), new Point(7, 7), "pause");
			cursors.put($("pause"), cursor);
		}

		return (Cursor) cursors.get($("pause"));
	}

	/**
	 * Answer an image of pause cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage PauseCursorImage() {
		if (images.contains($("pause")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000110000110000";
			imageString += "0000110000110000";
			imageString += "0000110000110000";
			imageString += "0000110000110000";
			imageString += "0000110000110000";
			imageString += "0000110000110000";
			imageString += "0000110000110000";
			imageString += "0000110000110000";
			imageString += "0000110000110000";
			imageString += "0000110000110000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000110000110000";
			maskString += "0001111001111000";
			maskString += "0001111001111000";
			maskString += "0001111001111000";
			maskString += "0001111001111000";
			maskString += "0001111001111000";
			maskString += "0001111001111000";
			maskString += "0001111001111000";
			maskString += "0001111001111000";
			maskString += "0001111001111000";
			maskString += "0001111001111000";
			maskString += "0000110000110000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			images.put($("pause"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("pause"));
	}

	/**
	 * Answer a pencil cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor PencilCursor() {
		if (cursors.contains($("pencil")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(PencilCursorImage()), new Point(3, 14), "pencil");
			cursors.put($("pencil"), cursor);
		}

		return (Cursor) cursors.get($("pencil"));
	}

	/**
	 * Answer an image of pencil cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage PencilCursorImage() {
		if (images.contains($("pencil")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000111100000";
			imageString += "0000000100010000";
			imageString += "0000001000010000";
			imageString += "0000011100100000";
			imageString += "0000010011100000";
			imageString += "0000100001000000";
			imageString += "0000100010000000";
			imageString += "0001000010000000";
			imageString += "0001000100000000";
			imageString += "0001100100000000";
			imageString += "0001111000000000";
			imageString += "0001110000000000";
			imageString += "0001100000000000";
			imageString += "0001000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000111100000";
			maskString += "0000001111110000";
			maskString += "0000001111111000";
			maskString += "0000011111111000";
			maskString += "0000111111110000";
			maskString += "0000111111110000";
			maskString += "0001111111100000";
			maskString += "0001111111000000";
			maskString += "0011111111000000";
			maskString += "0011111110000000";
			maskString += "0011111110000000";
			maskString += "0011111100000000";
			maskString += "0011111000000000";
			maskString += "0011110000000000";
			maskString += "0011100000000000";
			maskString += "0001000000000000";
			images.put($("pencil"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("pencil"));
	}

	/**
	 * Answer a play cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor PlayCursor() {
		if (cursors.contains($("play")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(PlayCursorImage()), new Point(7, 7), "play");
			cursors.put($("play"), cursor);
		}

		return (Cursor) cursors.get($("play"));
	}

	/**
	 * Answer an image of play cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage PlayCursorImage() {
		if (images.contains($("play")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000011000000000";
			imageString += "0000011100000000";
			imageString += "0000011110000000";
			imageString += "0000011011000000";
			imageString += "0000011001100000";
			imageString += "0000011000110000";
			imageString += "0000011000110000";
			imageString += "0000011001100000";
			imageString += "0000011011000000";
			imageString += "0000011110000000";
			imageString += "0000011100000000";
			imageString += "0000011000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000011000000000";
			maskString += "0000111100000000";
			maskString += "0000111110000000";
			maskString += "0000111111000000";
			maskString += "0000111111100000";
			maskString += "0000111111110000";
			maskString += "0000111101111000";
			maskString += "0000111101111000";
			maskString += "0000111111110000";
			maskString += "0000111111100000";
			maskString += "0000111111000000";
			maskString += "0000111110000000";
			maskString += "0000111100000000";
			maskString += "0000011000000000";
			maskString += "0000000000000000";
			images.put($("play"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("play"));
	}

	/**
	 * Answer a previous cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor PreviousCursor() {
		if (cursors.contains($("previous")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(PreviousCursorImage()), new Point(7, 7), "previous");
			cursors.put($("previous"), cursor);
		}

		return (Cursor) cursors.get($("previous"));
	}

	/**
	 * Answer an image of previous cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage PreviousCursorImage() {
		if (images.contains($("previous")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000011001000";
			imageString += "0000000111001000";
			imageString += "0000001111001000";
			imageString += "0000011011001000";
			imageString += "0000110011001000";
			imageString += "0001100011001000";
			imageString += "0001100011001000";
			imageString += "0000110011001000";
			imageString += "0000011011001000";
			imageString += "0000001111001000";
			imageString += "0000000111001000";
			imageString += "0000000011001000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000011001000";
			maskString += "0000000111111100";
			maskString += "0000001111111100";
			maskString += "0000011111111100";
			maskString += "0000111111111100";
			maskString += "0001111111111100";
			maskString += "0011110111111100";
			maskString += "0011110111111100";
			maskString += "0001111111111100";
			maskString += "0000111111111100";
			maskString += "0000011111111100";
			maskString += "0000001111111100";
			maskString += "0000000111111100";
			maskString += "0000000011001000";
			maskString += "0000000000000000";
			images.put($("previous"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("previous"));
	}

	/**
	 * Answer a prologue cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor PrologueCursor() {
		if (cursors.contains($("prologue")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(PrologueCursorImage()), new Point(7, 7), "prologue");
			cursors.put($("prologue"), cursor);
		}

		return (Cursor) cursors.get($("prologue"));
	}

	/**
	 * Answer an image of prologue cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage PrologueCursorImage() {
		if (images.contains($("prologue")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0001000000011000";
			imageString += "0001000000111000";
			imageString += "0001000001111000";
			imageString += "0001000011011000";
			imageString += "0001000110011000";
			imageString += "0001001100011000";
			imageString += "0001001100011000";
			imageString += "0001000110011000";
			imageString += "0001000011011000";
			imageString += "0001000001111000";
			imageString += "0001000000111000";
			imageString += "0001000000011000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0001000000011000";
			maskString += "0011100000111100";
			maskString += "0011100001111100";
			maskString += "0011100011111100";
			maskString += "0011100111111100";
			maskString += "0011101111111100";
			maskString += "0011111110111100";
			maskString += "0011111110111100";
			maskString += "0011101111111100";
			maskString += "0011100111111100";
			maskString += "0011100011111100";
			maskString += "0011100001111100";
			maskString += "0011100000111100";
			maskString += "0001000000011000";
			maskString += "0000000000000000";
			images.put($("prologue"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("prologue"));
	}

	/**
	 * Answer a quarters2 cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor Quarters2Cursor() {
		if (cursors.contains($("quarters2")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(Quarters2CursorImage()), new Point(7, 7), "quarters2");
			cursors.put($("quarters2"), cursor);
		}

		return (Cursor) cursors.get($("quarters2"));
	}

	/**
	 * Answer an image of quarters2 cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage Quarters2CursorImage() {
		if (images.contains($("quarters2")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000100000000";
			imageString += "0000001110000000";
			imageString += "0000011111000000";
			imageString += "0000000100000000";
			imageString += "0001000100010000";
			imageString += "0011000000011000";
			imageString += "0111110001111100";
			imageString += "0011000000011000";
			imageString += "0001000100010000";
			imageString += "0000000100000000";
			imageString += "0000011111000000";
			imageString += "0000001110000000";
			imageString += "0000000100000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000100000000";
			maskString += "0000001110000000";
			maskString += "0000011111000000";
			maskString += "0000111111100000";
			maskString += "0001001110010000";
			maskString += "0011001110011000";
			maskString += "0111111011111100";
			maskString += "1111110001111110";
			maskString += "0111111011111100";
			maskString += "0011001110011000";
			maskString += "0001001110010000";
			maskString += "0000111111100000";
			maskString += "0000011111000000";
			maskString += "0000001110000000";
			maskString += "0000000100000000";
			maskString += "0000000000000000";

			images.put($("quarters2"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("quarters2"));
	}

	/**
	 * Answer a quarters cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor QuartersCursor() {
		if (cursors.contains($("quarters")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(QuartersCursorImage()), new Point(7, 7), "quarters");
			cursors.put($("quarters"), cursor);
		}

		return (Cursor) cursors.get($("quarters"));
	}

	/**
	 * Answer an image of quarters cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage QuartersCursorImage() {
		if (images.contains($("quarters")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000100000000";
			imageString += "0000001110000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0010000000000100";
			imageString += "0111110001111110";
			imageString += "0010000000000100";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000001110000000";
			imageString += "0000000100000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000100000000";
			maskString += "0000001110000000";
			maskString += "0000011111000000";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0010001110000100";
			maskString += "0111111011111110";
			maskString += "1111110001111111";
			maskString += "0111111011111110";
			maskString += "0010001110000100";
			maskString += "0000001110000000";
			maskString += "0000001110000000";
			maskString += "0000011111000000";
			maskString += "0000001110000000";
			maskString += "0000000100000000";
			maskString += "0000000000000000";
			images.put($("quarters"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("quarters"));
	}

	/**
	 * Answer a quarters cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor ReadCursor() {
		if (cursors.contains($("read")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(ReadCursorImage()), new Point(5, 10), "read");
			cursors.put($("read"), cursor);
		}

		return (Cursor) cursors.get($("read"));
	}

	/**
	 * Answer an image of read cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage ReadCursorImage() {
		if (images.contains($("read")) == false) {
			String imageString = "";
			imageString += "0000110000000110";
			imageString += "0001001000001001";
			imageString += "0001001000001001";
			imageString += "0010000000010000";
			imageString += "0100000000100000";
			imageString += "1111101111100000";
			imageString += "1000010000100000";
			imageString += "1000010000100000";
			imageString += "1011010110100000";
			imageString += "0111101111000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0001000000001000";
			maskString += "0010010000010010";
			maskString += "0010010000010010";
			maskString += "0100000000100000";
			maskString += "1000000001000000";
			maskString += "0000000000000000";
			maskString += "0111101111000000";
			maskString += "0111101111000000";
			maskString += "0100101001000000";
			maskString += "0000010000000000";
			maskString += "0111101111000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			images.put($("read"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("read"));
	}

	/**
	 * Answer a repeat cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor RepeatCursor() {
		if (cursors.contains($("repeat")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(RepeatCursorImage()), new Point(7, 7), "repeat");
			cursors.put($("repeat"), cursor);
		}

		return (Cursor) cursors.get($("repeat"));
	}

	/**
	 * Answer an image of repeat cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage RepeatCursorImage() {
		if (images.contains($("repeat")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000100000000";
			imageString += "0000000110000000";
			imageString += "0001111111111000";
			imageString += "0011000110001100";
			imageString += "0010000100000100";
			imageString += "0010000000000100";
			imageString += "0011000000001100";
			imageString += "0001111111111000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000100000000";
			maskString += "0000001110000000";
			maskString += "0001111111111000";
			maskString += "0011111111111100";
			maskString += "0111111111111110";
			maskString += "0111001110001110";
			maskString += "0111000100001110";
			maskString += "0111111111111110";
			maskString += "0011111111111100";
			maskString += "0001111111111000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			images.put($("repeat"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("repeat"));
	}

	/**
	 * Answer a right arrow cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor RightArrowCursor() {
		if (cursors.contains($("rightArrow")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(RightArrowCursorImage()), new Point(7, 7), "rightArrow");
			cursors.put($("rightArrow"), cursor);
		}

		return (Cursor) cursors.get($("rightArrow"));
	}

	/**
	 * Answer an image of right arrow cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage RightArrowCursorImage() {
		if (images.contains($("rightArrow")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000100000000000";
			imageString += "0001100000000000";
			imageString += "0011111111111100";
			imageString += "0001100000000000";
			imageString += "0000100000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000100000000000";
			maskString += "0001110000000000";
			maskString += "0011111111111100";
			maskString += "0111111111111110";
			maskString += "0011111111111100";
			maskString += "0001110000000000";
			maskString += "0000100000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";

			images.put($("rightArrow"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("rightArrow"));
	}

	/**
	 * Answer a roundtrip cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor RoundtripCursor() {
		if (cursors.contains($("roundtrip")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(RoundtripCursorImage()), new Point(7, 7), "roundtrip");
			cursors.put($("roundtrip"), cursor);
		}

		return (Cursor) cursors.get($("roundtrip"));
	}

	/**
	 * Answer an image of roundtrip cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage RoundtripCursorImage() {
		if (images.contains($("roundtrip")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000010000";
			imageString += "0000000000011000";
			imageString += "0011111111111100";
			imageString += "0000000000011000";
			imageString += "0000000000010000";
			imageString += "0000100000000000";
			imageString += "0001100000000000";
			imageString += "0011111111111100";
			imageString += "0001100000000000";
			imageString += "0000100000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000000000010000";
			maskString += "0000000000111000";
			maskString += "0011111111111100";
			maskString += "0111111111111110";
			maskString += "0011111111111100";
			maskString += "0000100000111000";
			maskString += "0001110000010000";
			maskString += "0011111111111100";
			maskString += "0111111111111110";
			maskString += "0011111111111100";
			maskString += "0001110000000000";
			maskString += "0000100000000000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			images.put($("roundtrip"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("roundtrip"));
	}

	/**
	 * Answer a scissors cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor ScissorsCursor() {
		if (cursors.contains($("scissors")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(ScissorsCursorImage()), new Point(7, 8), "scissors");
			cursors.put($("scissors"), cursor);
		}

		return (Cursor) cursors.get($("scissors"));
	}

	/**
	 * Answer an image of scissors cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage ScissorsCursorImage() {
		if (images.contains($("scissors")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000001010000000";
			imageString += "0000011011000000";
			imageString += "0000011011000000";
			imageString += "0000011011000000";
			imageString += "0000011011000000";
			imageString += "0000001010000000";
			imageString += "0000001110000000";
			imageString += "0000001010000000";
			imageString += "0000001110000000";
			imageString += "0000011011000000";
			imageString += "0000101010100000";
			imageString += "0001001010010000";
			imageString += "0001001010010000";
			imageString += "0000111011100000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000001010000000";
			maskString += "0000011111000000";
			maskString += "0000111111100000";
			maskString += "0000111111100000";
			maskString += "0000111111100000";
			maskString += "0000111111100000";
			maskString += "0000011111000000";
			maskString += "0000011111000000";
			maskString += "0000011111000000";
			maskString += "0000011111000000";
			maskString += "0000111111100000";
			maskString += "0001111111110000";
			maskString += "0011111111111000";
			maskString += "0011111111111000";
			maskString += "0001111111110000";
			maskString += "0000111011100000";
			images.put($("scissors"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("scissors"));
	}

	/**
	 * Answer a speaker cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor SpeakerCursor() {
		if (cursors.contains($("speaker")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(SpeakerCursorImage()), new Point(7, 7), "speaker");
			cursors.put($("speaker"), cursor);
		}

		return (Cursor) cursors.get($("speaker"));
	}

	/**
	 * Answer an image of speaker cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage SpeakerCursorImage() {
		if (images.contains($("speaker")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000100000000";
			imageString += "0000001100010000";
			imageString += "0000010100001000";
			imageString += "0000100100000100";
			imageString += "0001000100100100";
			imageString += "0110000100100100";
			imageString += "0100000100010010";
			imageString += "0100000100010010";
			imageString += "0110000100100100";
			imageString += "0001000100100100";
			imageString += "0000100100000100";
			imageString += "0000010100001000";
			imageString += "0000001100010000";
			imageString += "0000000100000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000100000000";
			maskString += "0000001110010000";
			maskString += "0000011110111000";
			maskString += "0000111110011100";
			maskString += "0001111110101110";
			maskString += "0111111111111110";
			maskString += "1111111111111110";
			maskString += "1111111110111111";
			maskString += "1111111110111111";
			maskString += "1111111111111110";
			maskString += "0111111111111110";
			maskString += "0001111110101110";
			maskString += "0000111110011100";
			maskString += "0000011110111000";
			maskString += "0000001110010000";
			maskString += "0000000100000000";
			images.put($("speaker"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("speaker"));
	}

	/**
	 * Answer a spuit cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor SpuitCursor() {
		if (cursors.contains($("spuit")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(SpuitCursorImage()), new Point(2, 13), "spuit");
			cursors.put($("spuit"), cursor);
		}

		return (Cursor) cursors.get($("spuit"));
	}

	/**
	 * Answer an image of spuit cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage SpuitCursorImage() {
		if (images.contains($("spuit")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000011100";
			imageString += "0000000000111110";
			imageString += "0000000000111110";
			imageString += "0000000111111110";
			imageString += "0000000011111100";
			imageString += "0000000101110000";
			imageString += "0000001000110000";
			imageString += "0000010001010000";
			imageString += "0000100010000000";
			imageString += "0001000100000000";
			imageString += "0010001000000000";
			imageString += "0010010000000000";
			imageString += "0101100000000000";
			imageString += "0010000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000011100";
			maskString += "0000000000111110";
			maskString += "0000000001111111";
			maskString += "0000011111111111";
			maskString += "0000001111111111";
			maskString += "0000000111111110";
			maskString += "0000001111111100";
			maskString += "0000011111111000";
			maskString += "0000111111111000";
			maskString += "0001111111011000";
			maskString += "0011111110001000";
			maskString += "0111111100000000";
			maskString += "0111111000000000";
			maskString += "1101110000000000";
			maskString += "0111100000000000";
			maskString += "0010000000000000";
			images.put($("spuit"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("spuit"));
	}

	/**
	 * Answer a stop cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor StopCursor() {
		if (cursors.contains($("stop")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(StopCursorImage()), new Point(7, 7), "stop");
			cursors.put($("stop"), cursor);
		}

		return (Cursor) cursors.get($("stop"));
	}

	/**
	 * Answer an image of stop cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage StopCursorImage() {
		if (images.contains($("stop")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000111111110000";
			imageString += "0000111111110000";
			imageString += "0000110000110000";
			imageString += "0000110000110000";
			imageString += "0000110000110000";
			imageString += "0000110000110000";
			imageString += "0000110000110000";
			imageString += "0000110000110000";
			imageString += "0000111111110000";
			imageString += "0000111111110000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			maskString += "0000111111110000";
			maskString += "0001111111111000";
			maskString += "0001111111111000";
			maskString += "0001111111111000";
			maskString += "0001111001111000";
			maskString += "0001111001111000";
			maskString += "0001111001111000";
			maskString += "0001111001111000";
			maskString += "0001111111111000";
			maskString += "0001111111111000";
			maskString += "0001111111111000";
			maskString += "0000111111110000";
			maskString += "0000000000000000";
			maskString += "0000000000000000";
			images.put($("stop"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("stop"));
	}

	/**
	 * Answer a up arrow cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor UpArrowCursor() {
		if (cursors.contains($("upArrow")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(UpArrowCursorImage()), new Point(7, 7), "upArrow");
			cursors.put($("upArrow"), cursor);
		}

		return (Cursor) cursors.get($("upArrow"));
	}

	/**
	 * Answer an image of up arrow cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage UpArrowCursorImage() {
		if (images.contains($("upArrow")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0000000100000000";
			imageString += "0000001110000000";
			imageString += "0000011111000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000100000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "0000000000000000";
			maskString += "0000000010000000";
			maskString += "0000000111000000";
			maskString += "0000001111100000";
			maskString += "0000011111110000";
			maskString += "0000001111100000";
			maskString += "0000000111000000";
			maskString += "0000000111000000";
			maskString += "0000000111000000";
			maskString += "0000000111000000";
			maskString += "0000000111000000";
			maskString += "0000000111000000";
			maskString += "0000000111000000";
			maskString += "0000000111000000";
			maskString += "0000000010000000";
			maskString += "0000000000000000";

			images.put($("upArrow"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("upArrow"));
	}

	/**
	 * Answer an up cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor UpCursor() {
		if (cursors.contains($("up")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(UpCursorImage()), new Point(7, 7), "up");
			cursors.put($("up"), cursor);
		}

		return (Cursor) cursors.get($("up"));
	}

	/**
	 * Answer an image of up cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage UpCursorImage() {
		if (images.contains($("up")) == false) {
			String imageString = "";
			imageString += "0000000000000000";
			imageString += "0100000000000000";
			imageString += "0110000000000000";
			imageString += "0111000000000000";
			imageString += "0111100000000000";
			imageString += "0111110000000000";
			imageString += "0111111000000000";
			imageString += "0110000000000000";
			imageString += "0110000000000000";
			imageString += "0110000000000000";
			imageString += "0110000000000000";
			imageString += "0110000000000000";
			imageString += "0110000000000000";
			imageString += "0110000000000000";
			imageString += "0110000000000000";
			imageString += "0000000000000000";

			String maskString = "";
			maskString += "1100000000000000";
			maskString += "1110000000000000";
			maskString += "1111000000000000";
			maskString += "1111100000000000";
			maskString += "1111110000000000";
			maskString += "1111111000000000";
			maskString += "1111111100000000";
			maskString += "1111111100000000";
			maskString += "1111000000000000";
			maskString += "1111000000000000";
			maskString += "1111000000000000";
			maskString += "1111000000000000";
			maskString += "1111000000000000";
			maskString += "1111000000000000";
			maskString += "1111000000000000";
			maskString += "1111000000000000";
			images.put($("up"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("up"));
	}

	/**
	 * Answer an write cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public static final Cursor WriteCursor() {
		if (cursors.contains($("write")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(WriteCursorImage()), new Point(0, 0), "write");
			cursors.put($("write"), cursor);
		}

		return (Cursor) cursors.get($("write"));
	}

	/**
	 * Answer an image of write cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public static final BufferedImage WriteCursorImage() {
		if (images.contains($("write")) == false) {
			String imageString = "";
			imageString += "0000000000000110";
			imageString += "0000000000001111";
			imageString += "0000000000010110";
			imageString += "0000000000100100";
			imageString += "0000000001001000";
			imageString += "0000000010010000";
			imageString += "0000000100100000";
			imageString += "0000001001000011";
			imageString += "0000010010000010";
			imageString += "0000100100000110";
			imageString += "0001001000001000";
			imageString += "0010010000001000";
			imageString += "0111100001001000";
			imageString += "0101000010111000";
			imageString += "0110000110000000";
			imageString += "1111111100000000";

			String maskString = "";
			maskString += "0000000000001111";
			maskString += "0000000000011111";
			maskString += "0000000000111110";
			maskString += "0000000001111100";
			maskString += "0000000011111000";
			maskString += "0000000111110000";
			maskString += "0000001111100011";
			maskString += "0000011111001111";
			maskString += "0000111110011111";
			maskString += "0001111100011111";
			maskString += "0011111000011110";
			maskString += "0111110011111110";
			maskString += "1111100111111110";
			maskString += "1111001111111110";
			maskString += "1111111111111100";
			maskString += "1111111111000000";

			images.put($("write"), _CreateImage(_Pixels(imageString, maskString)));
		}

		return (BufferedImage) images.get($("write"));
	}

	/**
	 * Restore the original cursor for all frames.
	 */
	public void _restore() {
		if (_savedCursors == null) {
			return;
		}

		Frame[] frames = this._frames();
		for (int i = 0; i < frames.length; i++) {
			frames[i].setCursor(_savedCursors[i]);
		}
		_frames = null;
		_savedCursors = null;
	}

	/**
	 * Set the cursor for all frames.
	 */
	public void _show() {
		Frame[] frames = this._frames();
		_savedCursors = new Cursor[frames.length];
		for (int i = 0; i < frames.length; i++) {
			_savedCursors[i] = frames[i].getCursor();
			frames[i].setCursor(_cursor);
		}
	}

	/**
	 * Answer the current frames.
	 * 
	 * @return java.awt.Frame[]
	 */
	protected Frame[] _frames() {
		if (_frames == null) {
			_frames = Frame.getFrames();
		}
		return _frames;
	}
}
