package sra.jun.goodies.display;

import java.awt.*;
import java.io.*;
import sra.smalltalk.*;
import sra.jun.goodies.cursors.JunCursors;
import sra.jun.goodies.files.*;
import sra.jun.goodies.images.streams.*;
import sra.jun.system.framework.JunApplicationModel;

/**
 * JunDisplayModel class
 * 
 *  @author    nisinaka
 *  @created   1998/11/16 (by nisinaka)
 *  @updated   1999/11/11 (by MATSUDA Ryouichi)
 *  @updated   2002/01/09 (by nisinaka)
 *  @updated   2002/11/21 (by nisinaka)
 *  @updated   2003/03/19 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun490 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunDisplayModel.java,v 6.0 2003/10/28 10:52:08 nisinaka Exp $
 */
public class JunDisplayModel extends JunApplicationModel {

	protected StDisplayable displayObject;
	protected StMenu _popupMenu;
	protected String kindString;
	protected boolean _scrollable;

	/**
	 * Create a new instance of JunDisplayModel.
	 */
	public JunDisplayModel() {
		super();
	}

	/**
	 * Create a new instance of JunDisplayModel and set a visual object.
	 * 
	 * @param anObject java.lang.Object
	 */
	public JunDisplayModel(Object anObject) {
		this.visual_(anObject);
	}

	/**
	 * Create a new instance of JunDisplayModel, set a visual object, and open it.
	 * 
	 * @param anObject java.lang.Object
	 * @param aString java.lang.String
	 * 
	 * @return sra.jun.goodies.display.JunDisplayModel
	 */
	public static JunDisplayModel OpenVisual_(Object anObject) {
		return OpenVisual_label_(anObject, anObject.getClass().getName());
	}

	/**
	 * Create a new instance of JunDisplayModel, set a visual object and a
	 * window title, and open it.
	 * 
	 * @param anObject java.lang.Object
	 * @param aString java.lang.String
	 * 
	 * @return sra.jun.goodies.display.JunDisplayModel
	 */
	public static JunDisplayModel OpenVisual_label_(Object anObject, String aString) {
		JunDisplayModel model = new JunDisplayModel(anObject);
		Frame window = model.open();
		window.setTitle(aString);

		return model;
	}

	/**
	 * Show the object.
	 * 
	 * @return sra.jun.goodies.display.JunDisplayModel
	 * @param anObject java.lang.Object
	 * @param aString java.lang.String
	 */
	public static JunDisplayModel Show_(Object anObject) {
		return OpenVisual_(anObject);
	}

	/**
	 * Create a new instance of JunDisplayModel and set a visual object.
	 * 
	 * @param anObject java.lang.Object
	 * 
	 * @return sra.jun.goodies.display.JunDisplayModel
	 */
	public static JunDisplayModel Visual_(Object anObject) {
		return new JunDisplayModel(anObject);
	}

	/**
	 * Answer whether the view is opened with scrollbars or not.
	 *
	 * @return boolean
	 */
	public boolean _isScrollable() {
		return _scrollable;
	}

	/**
	 * Set whether the view is opened with scrollbars or not.
	 *
	 * @param aBoolean boolean
	 */
	public void _isScrollable(boolean aBoolean) {
		_scrollable = aBoolean;
	}

	/**
	 * Create a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunDisplayViewAwt(this, this.visual().bounds().getSize());
		} else {
			return new JunDisplayViewSwing(this, this.visual().bounds().getSize());
		}
	}

	/**
	 * Menu messages: Save the display object as an image file.
	 */
	public void saveAsImage() {
		StImage image = this.visual().asImage();
		if (image == null) {
			return;
		}

		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType("Image files", new String[] { ".jpg", ".JPG" })
		};
		File file = JunFileDialog.RequestNewFilename("Input a file.", new File(this.defaultBaseName() + ".jpg"), fileTypes, fileTypes[0]);
		if (file == null) {
			return;
		}

		JunImageStream stream = null;
		JunCursors cursor = new JunCursors(JunCursors.WriteCursor());
		try {
			cursor._show();

			stream = JunJpegImageStream.On_(new FileOutputStream(file));
			stream.nextPutImage_(image);
		} catch (IOException e) {
		} finally {
			try {
				if (stream != null) {
					stream.flush();
					stream.close();
				}
			} catch (IOException e) {
			}

			cursor._restore();
		}

	}

	/**
	 * Answer my visual object.
	 * 
	 * @return sra.smalltalk.StDisplayable
	 */
	public StDisplayable visual() {
		if (displayObject == null) {
			this.visual_(new String());
		}
		return displayObject;
	}

	/**
	 * Set a new visual object. 
	 * If the visual object is changed, announce it.
	 * 
	 * @param anObject java.lang.Object
	 */
	public void visual_(Object anObject) {
		kindString = anObject.getClass().getName();

		Object visual = anObject;
		if (visual != displayObject) {
			if (visual == null) {
				displayObject = new StComposedText(new String());
			} else if (visual instanceof StDisplayable) {
				displayObject = (StDisplayable) visual;
			} else if (visual instanceof Image) {
				displayObject = new StImage((Image) visual);
			} else {
				String aString = (anObject instanceof String) ? (String) anObject : anObject.toString();
				if (aString.length() == 0) {
					aString = " ";
				}
				displayObject = new StComposedText(aString);
			}
			displayObject = displayObject.asImage();
			this.visualIsChanged();
		}
	}

	/**
	 * Answer a view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView getView() {
		Object[] dependents = this.dependents();
		for (int i = 0; i < dependents.length; i++) {
			if (dependents[i] instanceof JunDisplayView && ((JunDisplayView) dependents[i]).model() == this) {
				return (JunDisplayView) dependents[i];
			}
		}

		return null;
	}

	/**
	 * Initialize the JunDisplayModel when created.
	 */
	protected void initialize() {
		super.initialize();

		displayObject = null;
		_popupMenu = null;
		kindString = null;
		_scrollable = true;
	}

	/**
	 * Create a popup menu.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu popupMenu() {
		if (_popupMenu == null) {
			StMenu popupMenu = new StMenu("");
			popupMenu.add(this.createMenuItem("Save as image...", "saveAsImage"));
			_popupMenu = popupMenu;
		}
		return _popupMenu;
	}

	/**
	 * Process the change of my visual object.
	 */
	protected void visualIsChanged() {
		this.changed_($("visual"));
		this.setWindowLabel();
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		if (kindString == null) {
			return "Display";
		}
		return kindString;
	}
}
