package sra.jun.goodies.display;

import java.awt.*;
import javax.swing.*;
import sra.smalltalk.*;

/**
 * JunDisplayViewSwing class
 * 
 *  @author    nisinaka
 *  @created   1998/11/16 (by nisinaka)
 *  @updated   1999/11/11 (by MATSUDA Ryouichi)
 *  @updated   2002/01/09 (by nisinaka)
 *  @updated   2002/11/21 (by nisinaka)
 *  @updated   2003/03/19 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun473 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunDisplayViewSwing.java,v 6.0 2003/10/28 10:52:08 nisinaka Exp $
 */
public class JunDisplayViewSwing extends StViewJPanel implements JunDisplayView {

	protected StViewJPanel canvas;
	protected JScrollPane scrollPane;

	/**
	 * Create a new instance of JunDisplayViewSwing and initialize it.
	 * 
	 * @param sra.jun.goodies.display.JunDisplayModel
	 */
	public JunDisplayViewSwing(JunDisplayModel aDisplayModel) {
		super(aDisplayModel);
	}

	/**
	 * Create a new instance of JunDisplayViewSwing and initialize it.
	 *
	 * @param aDisplayModel sra.jun.goodies.display.JunDisplayModel
	 * @param aDimension java.awt.Dimension
	 */
	public JunDisplayViewSwing(JunDisplayModel aDisplayModel, Dimension aDimension) {
		this(aDisplayModel);
		this.setPreferredSize(aDimension);
	}

	/**
	 * Do the layout on the scroll pane.
	 */
	public void addNotify() {
		super.addNotify();

		this.scrollPane().doLayout();
	}

	/**
	 * Answer my model as JunDisplayModel.
	 * 
	 * @return sra.jun.goodies.display.JunDisplayModel
	 */
	public JunDisplayModel getDisplayModel() {
		return (JunDisplayModel) this.model();
	}

	/**
	 * model property.
	 * 
	 * @return sra.jun.goodies.display.JunDisplayModel
	 * @deprecated since Jun379, use getDisplayModel() instead.
	 */
	public JunDisplayModel getModel() {
		return (JunDisplayModel) this.model();
	}

	/**
	 * Gets the preferred size of this component.
	 *
	 * @return A dimension object indicating this component's preferred size.
	 * @see #getMinimumSize
	 * @see java.awt.LayoutManager
	 */
	public Dimension getPreferredSize() {
		Point location = this.scrollPane().getLocation();
		Dimension size = this.scrollPane().getPreferredSize();
		return new Dimension(size.width + location.x * 2, size.height + location.y * 2);
	}

	/**
	 * visual property.
	 * 
	 * @return sra.smaltalk.StDisplayable
	 * @deprecated since Jun379, use visual()
	 */
	public StDisplayable getVisual() {
		return this.getModel().visual();
	}

	/**
	 * Answer my scroll pane, which already contains the canvas.
	 *
	 * @return javax.swing.JScrollPane
	 */
	public JScrollPane scrollPane() {
		if (scrollPane == null) {
			scrollPane = new JScrollPane(this.canvas()) {
				public Dimension getPreferredSize() {
					Dimension size = visual().bounds().getSize();
					Insets insets = this.getInsets();
					return new Dimension(size.width + insets.left + insets.right, size.height + insets.top + insets.bottom);
				}

				public void setBounds(int x, int y, int width, int height) {
					super.setBounds(x, y, width, height);

					if (getDisplayModel()._isScrollable() == false) {
						Insets insets = this.getInsets();
						canvas().setSize(width - insets.left - insets.right, height - insets.top - insets.bottom);
					}
				}
			};
		}

		return scrollPane;
	}

	/**
	 * model property.
	 * 
	 * @param aModel DOCUMENT ME!
	 * @deprecated since Jun379
	 */
	public void setModel(JunDisplayModel aModel) {
		this.model_(aModel);
	}

	/**
	 * visual property.
	 * 
	 * @param anObject java.lang.Object
	 * @deprecated since Jun379
	 */
	public void setVisual(Object anObject) {
		this.getModel().visual_(anObject);
	}

	/**
	 * Update the receiver according to the change notification from the model.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if (evt.getAspect() == $("redisplay")) {
			this.canvas().repaint();
		} else if (evt.getAspect() == $("visual")) {
			//Update client's preferred size because the area taken up
			//by the graphics has gotten larger or smaller (if cleared)
			this.canvas().setPreferredSize(this.canvas().getPreferredSize());
			//This lets the scroll pane know to update itself
			//and its scroll bars.
			this.canvas().revalidate();
			// Finally repaint.
			this.canvas().repaint();
		} else {
			super.update_(evt);
		}
	}

	/**
	 * Answer my visual object.
	 * 
	 * @return sra.smaltalk.StDisplayable
	 */
	public StDisplayable visual() {
		return this.getModel().visual();
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.scrollPane(), WindowSpecLayout.Position(0f, 0, 0f, 0, 1f, 0, 1f, 0));
	}

	/**
	 * Answer my canvas.
	 *
	 * @return sra.smalltalk.StViewJPanel
	 */
	public StViewJPanel canvas() {
		if (canvas == null) {
			canvas = new StViewJPanel() {
				protected JPopupMenu _popupMenu;

				public Dimension getPreferredSize() {
					if (getDisplayModel()._isScrollable()) {
						return visual().bounds().getSize();
					}
					return super.getPreferredSize();
				}

				public void paint(Graphics g) {
					super.paint(g);
					g.setColor(Color.white);
					Dimension visualSize = visual().bounds().getSize();
					int x = (this.getWidth() - visualSize.width) / 2;
					int y = (this.getHeight() - visualSize.height) / 2;
					visual().displayOn_at_(g, new Point(x, y));
				}

				protected JPopupMenu popupMenu() {
					if (_popupMenu == null) {
						StMenu aMenu = getDisplayModel().popupMenu();
						if (aMenu != null) {
							_popupMenu = this._createPopupMenu(aMenu);
							this.add(_popupMenu);
						}
					}
					return _popupMenu;
				}
			};
		}

		return canvas;
	}

	/**
	 * Answer a default controller.
	 * 
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new JunDisplayController();
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 * @deprecated since Jun379
	 */
	protected StModel defaultModel() {
		return new JunDisplayModel();
	}

	/**
	 * Initialize the receiver.
	 */
	protected void initialize() {
		super.initialize();

		canvas = null;
		scrollPane = null;
	}
}
