package sra.jun.goodies.files;

import java.awt.Dialog;
import java.io.*;
import sra.smalltalk.*;
import sra.jun.system.framework.JunDialog;

/**
 * JunFileDialog class
 * 
 *  @author    nisinaka
 *  @created   2000/12/18 (by nisinaka)
 *  @updated   2003/03/09 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun373 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFileDialog.java,v 6.0 2003/10/28 10:52:09 nisinaka Exp $
 */
public class JunFileDialog extends StSimpleDialog {

	protected static File PreviousPath = null;

	protected JunFileModel fileModel;
	protected StBlockClosure acceptBlock;
	protected StBlockClosure cancelBlock;

	/**
	 * Flush the previous path.
	 */
	public static void FlushPrevious() {
		PreviousPath = null;
	}

	/**
	 * Answer the current previous path.
	 *
	 * @return java.io.File
	 */
	public static File PreviousPath() {
		return PreviousPath;
	}

	/**
	 * Set the new previous path.
	 *
	 * @param aFile java.io.File
	 */
	public static void PreviousPath_(File aFile) {
		PreviousPath = aFile;
	}

	/**
	 * Open a file dialog and let a user to choose.
	 * 
	 * @return java.io.File
	 */
	public static File Request() {
		return Request(null);
	}

	/**
	 * Open a file dialog showing the message string and let a user to choose.
	 *
	 * @return java.io.File
	 * @param messageString java.lang.String
	 */
	public static File Request(String messageString) {
		return Request(messageString, null, null, null, null, null);
	}

	/**
	 * Open a file dialog and let a user to choose.
	 *
	 * @return java.io.File
	 * @param messageString java.lang.String
	 * @param fileTypes sra.jun.goodies.files.JunFileModel.FileType[]
	 * @param initialFileType sra.jun.goodies.files.JunFileModel.FileType
	 */
	public static File Request(String messageString, JunFileModel.FileType[] fileTypes, JunFileModel.FileType initialFileType) {
		return Request(messageString, null, fileTypes, initialFileType, null, null);
	}

	/**
	 * Open a file dialog and let a user to choose.
	 *
	 * @return java.io.File
	 * @param messageString java.lang.String
	 * @param initialFile java.io.File
	 * @param fileTypes sra.jun.goodies.files.JunFileModel.FileType[]
	 * @param initialFileType sra.jun.goodies.files.JunFileModel.FileType
	 * @param acceptBlock sra.smalltalk.StBlockClosure
	 * @param cancelBlock sra.smalltalk.StBlockClosure
	 */
	public static File Request(String messageString, File initialFile, JunFileModel.FileType[] fileTypes, JunFileModel.FileType initialFileType, StBlockClosure acceptBlock, StBlockClosure cancelBlock) {
		JunFileDialog fileDialog = new JunFileDialog();
		JunFileModel fileModel = fileDialog.fileModel();

		if (fileTypes != null) {
			fileModel.fileTypes_(fileTypes);
		}

		if (initialFileType != null) {
			fileModel.fileType().value_(initialFileType);
		} else {
			fileTypes = fileModel.fileTypes();
			if (fileTypes != null && fileTypes.length > 0) {
				fileModel.fileType().value_(fileTypes[0]);
			}
		}

		if (initialFile == null) {
			if (PreviousPath != null && PreviousPath.exists()) {
				fileModel.value_(PreviousPath);
			} else {
				FlushPrevious();
			}
		} else {
			if (initialFile.isAbsolute()) {
				fileModel.value_(initialFile);
			} else {
				if (PreviousPath != null && PreviousPath.exists()) {
					fileModel.value_(new File(PreviousPath, initialFile.getPath()));
				} else {
					FlushPrevious();
					fileModel.value_(initialFile);
				}
			}
		}

		Dialog dialog = fileDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			fileDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			fileDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);
		if (fileDialog.accept()._booleanValue()) {
			File aFile = (File) fileModel.value();
			if (aFile.exists() && aFile.isDirectory()) {
				PreviousPath = aFile;
			} else {
				PreviousPath = aFile.getParentFile();
			}
			return aFile;
		} else {
			return null;
		}
	}

	/**
	 * Open a file dialog and let a user to choose.
	 *
	 * @return java.io.File
	 * @param messageString java.lang.String
	 * @param initialDirectory java.io.File
	 * @param initialFilenameString java.lang.String
	 * @param fileTypes sra.jun.goodies.files.JunFileModel.FileType[]
	 * @param initialFileType sra.jun.goodies.files.JunFileModel.FileType
	 * @param acceptBlock sra.smalltalk.StBlockClosure
	 * @param cancelBlock sra.smalltalk.StBlockClosure
	 */
	public static File Request(String messageString, File initialDirectory, String initialFilenameString, JunFileModel.FileType[] fileTypes, JunFileModel.FileType initialFileType, StBlockClosure acceptBlock, StBlockClosure cancelBlock) {
		File initialFile = null;
		if (initialDirectory != null) {
			initialFile = initialDirectory;
		}
		if (initialFilenameString != null && initialFilenameString.length() > 0) {
			initialFile = new File(initialFile, initialFilenameString);
		}

		return Request(messageString, initialFile, fileTypes, initialFileType, acceptBlock, cancelBlock);
	}

	/**
	 * Request for a directory.
	 *
	 * @return java.io.File
	 * @param messageString java.lang.String
	 */
	public static File RequestDirectory(String messageString) {
		return RequestDirectory(messageString, null);
	}

	/**
	 * Request for a directory.
	 *
	 * @return java.io.File
	 * @param messageString java.lang.String
	 * @param initialDirectory java.io.File
	 */
	public static File RequestDirectory(String messageString, File initialDirectory) {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { JunFileModel.FileType.All("All files")};
		return Request(messageString, initialDirectory, null, fileTypes, fileTypes[0], new StBlockClosure() {
			public Object value_(Object fileModel) {
				File aFile = (File) ((JunFileModel) fileModel).value();
				if (aFile.exists()) {
					String message = "";
					if (aFile.isDirectory()) {
						return Boolean.TRUE;
					} else {
						message += aFile.getPath() + " is not a directory.\n";
						message += "(It may be a file.)\n";
						message += "Input a directory.";
						JunDialog.Warn_(message);
						return Boolean.FALSE;
					}
				} else {
					String message = aFile.getPath() + " does not exist\n";
					message += "Input a directory.";
					JunDialog.Warn_(message);
					return Boolean.FALSE;
				}
			}
		}, new StBlockClosure() {
			public Object value_(Object fileModel) {
				return Boolean.TRUE;
			}
		});
	}

	/**
	 * Request for a new directory.
	 *
	 * @return java.io.File
	 * @param messageString java.lang.String
	 */
	public static File RequestNewDirectory(String messageString) {
		return RequestNewDirectory(messageString, null);
	}

	/**
	 * Request for a new directory.
	 *
	 * @return java.io.File
	 * @param messageString java.lang.String
	 * @param initialDirectory java.io.File
	 */
	public static File RequestNewDirectory(String messageString, File initialDirectory) {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { JunFileModel.FileType.All("All files")};
		return Request(messageString, initialDirectory, null, fileTypes, fileTypes[0], new StBlockClosure() {
			public Object value_(Object fileModel) {
				File aFile = (File) ((JunFileModel) fileModel).value();
				if (aFile.exists()) {
					String message = "";
					if (aFile.isDirectory()) {
						message += aFile.getPath() + " already exits.\n";
						message += "Do you want to go a head?";
						return new Boolean(JunDialog.Confirm_(message, false));
					} else {
						message += aFile.getPath() + " already exists as file.\n";
						message += "Input a directory.";
						JunDialog.Warn_(message);
						return Boolean.FALSE;
					}
				} else {
					return Boolean.TRUE;
				}
			}
		}, new StBlockClosure() {
			public Object value_(Object fileModel) {
				return Boolean.TRUE;
			}
		});
	}

	/**
	 * Request for a new file name.
	 *
	 * @return java.io.File
	 * @param messageString java.lang.String
	 */
	public static File RequestNewFilename(String messageString) {
		return RequestNewFilename(messageString, null, null, null);
	}

	/**
	 * Request for a new file name.
	 *
	 * @return java.io.File
	 * @param messageString java.lang.String
	 * @param fileTypes sra.jun.goodies.files.JunFileModel.FileType[]
	 * @param initialFileType sra.jun.goodies.files.JunFileModel.FileType
	 */
	public static File RequestNewFilename(String messageString, JunFileModel.FileType[] fileTypes, JunFileModel.FileType initialFileType) {
		return RequestNewFilename(messageString, null, fileTypes, initialFileType);
	}

	/**
	 * Request for a new file name.
	 *
	 * @return java.io.File
	 * @param messageString java.lang.String
	 * @param initialFile java.io.File
	 */
	public static File RequestNewFilename(String messageString, File initialFile) {
		return RequestNewFilename(messageString, initialFile, null, null);
	}

	/**
	 * Request for a new file name.
	 *
	 * @return java.io.File
	 * @param messageString java.lang.String
	 * @param initialFile java.io.File
	 * @param fileTypes sra.jun.goodies.files.JunFileModel.FileType[]
	 * @param initialFileType sra.jun.goodies.files.JunFileModel.FileType
	 */
	public static File RequestNewFilename(String messageString, File initialFile, JunFileModel.FileType[] fileTypes, JunFileModel.FileType initialFileType) {
		return Request(messageString, initialFile, fileTypes, initialFileType, new StBlockClosure() {
			public Object value_(Object fileModel) {
				File aFile = (File) ((JunFileModel) fileModel).value();
				if (aFile.exists()) {
					String message = "";
					if (aFile.isDirectory()) {
						message += aFile.getPath() + " is not a file.\n";
						message += "(It may be a directory.)\n";
						message += "Input a file.";
						JunDialog.Warn_(message);
						return Boolean.FALSE;
					} else {
						message += aFile.getPath() + " already exists.\n";
						message += "Do you want to go ahead?";
						return new Boolean(JunDialog.Confirm_(message, false));
					}
				} else {
					return Boolean.TRUE;
				}
			}
		}, new StBlockClosure() {
			public Object value_(Object fileModel) {
				return Boolean.TRUE;
			}
		});
	}

	/**
	 * Called when an accept button is pressed.
	 *
	 * @return boolean
	 */
	public boolean acceptButtonPressed() {
		if (acceptBlock != null) {
			return ((Boolean) acceptBlock.value_(this.fileModel())).booleanValue();
		}
		return true;
	}

	/**
	 * Called when a cancel button is pressed.
	 *
	 * @return boolean
	 */
	public boolean cancelButtonPressed() {
		if (cancelBlock != null) {
			return ((Boolean) cancelBlock.value_(this.fileModel())).booleanValue();
		}
		return true;
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunFileDialogViewAwt(this);
		} else {
			return new JunFileDialogViewSwing(this);
		}
	}

	/**
	 * Answer my file model.
	 *
	 * @return sra.jun.goodies.files.JunFileModel
	 */
	public JunFileModel fileModel() {
		if (fileModel == null) {
			fileModel = new JunFileModel();
		}
		return fileModel;
	}

	/**
	 * Set the new accept block.
	 *
	 * @param newAcceptBlock sra.smalltalk.StBlockClosure
	 */
	protected void acceptBlock_(StBlockClosure newAcceptBlock) {
		acceptBlock = newAcceptBlock;
	}

	/**
	 * Set the new cancel block.
	 *
	 * @param newCancelBlock sra.smalltalk.StBlockClosure
	 */
	protected void cancelBlock_(StBlockClosure newCancelBlock) {
		cancelBlock = newCancelBlock;
	}

	/**
	 * Initialize the ApplicationModel when created.
	 */
	protected void initialize() {
		super.initialize();

		this.fileModel().currentDirectory();
		acceptBlock = null;
		cancelBlock = null;
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "File Dialog";
	}
}
