package sra.jun.goodies.font;

import java.awt.*;
import sra.smalltalk.*;

/**
 * JunFontDialog class
 * 
 *  @author    nisinaka
 *  @created   2003/03/03 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun373 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFontDialog.java,v 6.0 2003/10/28 10:52:09 nisinaka Exp $
 */
public class JunFontDialog extends StSimpleDialog {

	protected JunFontModel fontModel;
	protected StBlockClosure acceptBlock;
	protected StBlockClosure cancelBlock;
	protected Font font;

	/**
	 * Open a font dialog and request to choose a font.
	 *
	 * @return java.awt.Font
	 */
	public static Font Request() {
		JunFontDialog fontDialog = new JunFontDialog();
		fontDialog.openDialog();
		return fontDialog.font();
	}

	/**
	 * Open a font dialog showing a font and request to choose a font.
	 *
	 * @return java.awt.Font
	 * @param font java.awt.Font
	 */
	public static Font Request(Font font) {
		JunFontDialog fontDialog = new JunFontDialog();
		fontDialog.fontName_(font.getName());
		fontDialog.fontSize_(font.getSize());
		fontDialog.openDialog();
		return fontDialog.font();
	}

	/**
	 * Called when an accept button is pressed.
	 *
	 * @return boolean
	 */
	public boolean acceptButtonPressed() {
		if (acceptBlock != null) {
			acceptBlock.value();
		}
		return true;
	}

	/**
	 * Called when an cancel button is pressed.
	 *
	 * @return boolean
	 */
	public boolean cancelButtonPressed() {
		if (cancelBlock != null) {
			cancelBlock.value();
		}
		return true;
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunFontDialogViewAwt(this);
		} else {
			return new JunFontDialogViewSwing(this);
		}
	}

	/**
	 * Answer the current font.
	 *
	 * @return java.awt.Font
	 */
	public Font font() {
		return font;
	}

	/**
	 * Answer the current font model.
	 *
	 * @return sra.jun.goodies.font.JunFontModel
	 */
	public JunFontModel fontModel() {
		if (fontModel == null) {
			fontModel = new JunFontModel();
		}
		return fontModel;
	}

	/**
	 * Answer the current font name.
	 *
	 * @return java.lang.String
	 */
	public String fontName() {
		return this.fontModel().fontName();
	}

	/**
	 * Set the new font name.
	 *
	 * @param newFontName java.lang.String
	 */
	public void fontName_(String newFontName) {
		this.fontModel().fontName_(newFontName);
	}

	/**
	 * Answer the current font size.
	 *
	 * @return int
	 */
	public int fontSize() {
		return this.fontModel().fontSize();
	}

	/**
	 * Set the new font size.
	 *
	 * @param newFontSize int
	 */
	public void fontSize_(int newFontSize) {
		this.fontModel().fontSize_(newFontSize);
	}

	/**
	 * Set the new accept block.
	 *
	 * @param newAcceptBlock sra.smalltalk.StBlockClosure
	 */
	protected void acceptBlock_(StBlockClosure newAcceptBlock) {
		acceptBlock = newAcceptBlock;
	}

	/**
	 * Do everything short of actually opening the window.
	 *
	 * @return java.awt.Dialog
	 */
	protected Dialog allButOpenDialog() {
		Dialog aDialog = super.allButOpenDialog();
		this.fontModel().updateVisualObject();
		return aDialog;
	}

	/**
	 * Set the new cancel block.
	 *
	 * @param newCancelBlock sra.smalltalk.StBlockClosure
	 */
	protected void cancelBlock_(StBlockClosure newCancelBlock) {
		cancelBlock = newCancelBlock;
	}

	/**
	 * Initialize the JunFontDialog when created.
	 */
	protected void initialize() {
		super.initialize();

		fontModel = null;
		acceptBlock = new StBlockClosure() {
			public Object value() {
				font = fontModel().getFont();
				return null;
			}
		};
		cancelBlock = new StBlockClosure() {
			public Object value() {
				font = null;
				return null;
			}
		};
		font = null;
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "Font Dialog";
	}
}
