package sra.jun.goodies.font;

import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;
import sra.jun.goodies.display.JunDisplayViewAwt;

/**
 * JunFontViewAwt class
 * 
 *  @author    nisinaka
 *  @created   2003/02/28 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun433 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFontViewAwt.java,v 6.0 2003/10/28 10:52:09 nisinaka Exp $
 */
public class JunFontViewAwt extends StViewPanel implements JunFontView {

	protected JunDisplayViewAwt displayView;
	protected List fontNameList;
	protected List fontSizeList;

	/**
	 * Create a new instance of JunFontViewAwt and initialize it with the JunFontModel.
	 *
	 * @param newFontModel sra.jun.goodies.font.JunFontModel
	 */
	public JunFontViewAwt(JunFontModel newFontModel) {
		super(newFontModel);
	}

	/**
	 * Add WindowListener to make a selection visible.
	 * JList#ensureIndexIsVisible does not work before a window is opened.
	 */
	public void addNotify() {
		super.addNotify();

		this.topComponent().addWindowListener(new WindowAdapter() {
			public void windowOpened(WindowEvent e) {
				super.windowOpened(e);
				updateFontNameList();
				updateFontSizeList();
			}
		});
	}

	/**
	 * Answer my model as a JunFontModel.
	 *
	 * @return sra.jun.goodies.font.JunFontModel
	 */
	public JunFontModel getFontModel() {
		return (JunFontModel) this.model();
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.displayView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 0f, 79));
		this.add(this.fontNameList(), WindowSpecLayout.Position(0f, 1, 0f, 80, 0.7f, -1, 1f, -1));
		this.add(this.fontSizeList(), WindowSpecLayout.Position(0.7f, 0, 0f, 80, 1f, -1, 1f, -1));
		this.setSize(350, 250);
	}

	/**
	 * Answer my display view.
	 *
	 * @return sra.jun.goodies.display.JunDisplayViewAwt
	 */
	protected JunDisplayViewAwt displayView() {
		if (displayView == null) {
			displayView = new JunDisplayViewAwt(this.getFontModel().displayModel());
		}

		return displayView;
	}

	/**
	 * Answer my font name list.
	 *
	 * @return java.awt.List
	 */
	protected List fontNameList() {
		if (fontNameList == null) {
			fontNameList = new List();
			fontNameList.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (e.getStateChange() == ItemEvent.SELECTED) {
						getFontModel().setFontNameIndex(((Number) e.getItem()).intValue());
					}
				}
			});
		}

		return fontNameList;
	}

	/**
	 * Answer my font name list.
	 *
	 * @return java.awt.List
	 */
	protected List fontSizeList() {
		if (fontSizeList == null) {
			fontSizeList = new List();
			fontSizeList.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (e.getStateChange() == ItemEvent.SELECTED) {
						getFontModel().setFontSizeIndex(((Number) e.getItem()).intValue());
					}
				}
			});
		}

		return fontSizeList;
	}

	/**
	 * Update the font name list.
	 */
	protected void updateFontNameList() {
		fontNameList.removeAll();

		String[] fontNames = this.getFontModel().fontNames();
		for (int i = 0; i < fontNames.length; i++) {
			fontNameList.add(fontNames[i]);
		}

		int fontNameIndex = this.getFontModel().getFontNameIndex();
		if (fontNameIndex >= 0) {
			fontNameList.select(fontNameIndex);
			fontNameList.makeVisible(fontNameIndex);
		}
	}

	/**
	 * Update the font size list.
	 */
	protected void updateFontSizeList() {
		fontSizeList.removeAll();

		int[] fontSizes = this.getFontModel().fontSizes();
		for (int i = 0; i < fontSizes.length; i++) {
			fontSizeList.add(String.valueOf(fontSizes[i]));
		}

		int fontSizeIndex = this.getFontModel().getFontSizeIndex();
		if (fontSizeIndex >= 0) {
			fontSizeList.select(fontSizeIndex);
			fontSizeList.makeVisible(fontSizeIndex);
		}
	}
}
