package sra.jun.goodies.font;

import java.awt.Dimension;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import sra.smalltalk.*;
import sra.jun.goodies.display.JunDisplayViewSwing;

/**
 * JunFontViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2003/02/28 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun433 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFontViewSwing.java,v 6.0 2003/10/28 10:52:09 nisinaka Exp $
 */
public class JunFontViewSwing extends StViewJPanel implements JunFontView {

	protected JunDisplayViewSwing displayView;
	protected JList fontNameList;
	protected JScrollPane fontNameScrollPane;
	protected JList fontSizeList;
	protected JScrollPane fontSizeScrollPane;

	/**
	 * Create a new instance of JunFontViewSwing and initialize it with the JunFontModel.
	 *
	 * @param newFontModel sra.jun.goodies.font.JunFontModel
	 */
	public JunFontViewSwing(JunFontModel newFontModel) {
		super(newFontModel);
	}

	/**
	 * Add WindowListener to make a selection visible.
	 * JList#ensureIndexIsVisible does not work before a window is opened.
	 */
	public void addNotify() {
		super.addNotify();

		this.topComponent().addWindowListener(new WindowAdapter() {
			public void windowOpened(WindowEvent e) {
				super.windowOpened(e);
				updateFontNameList();
				updateFontSizeList();
			}
		});
	}

	/**
	 * Answer my model as a JunFontModel.
	 *
	 * @return sra.jun.goodies.font.JunFontModel
	 */
	public JunFontModel getFontModel() {
		return (JunFontModel) this.model();
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.displayView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 0f, 79));
		this.add(this.fontNameScrollPane(), WindowSpecLayout.Position(0f, 1, 0f, 80, 0.7f, -1, 1f, -1));
		this.add(this.fontSizeScrollPane(), WindowSpecLayout.Position(0.7f, 0, 0f, 80, 1f, -1, 1f, -1));
		this.setPreferredSize(new Dimension(350, 250));
	}

	/**
	 * Answer my display view.
	 *
	 * @return sra.jun.goodies.display.JunDisplayViewSwing
	 */
	protected JunDisplayViewSwing displayView() {
		if (displayView == null) {
			displayView = new JunDisplayViewSwing(this.getFontModel().displayModel());
		}

		return displayView;
	}

	/**
	 * Answer my font name list.
	 *
	 * @return javax.swing.JList
	 */
	protected JList fontNameList() {
		if (fontNameList == null) {
			fontNameList = new JList(new DefaultListModel());
			fontNameList.addListSelectionListener(new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e) {
					getFontModel().setFontNameIndex(fontNameList.getSelectedIndex());
				}
			});
		}

		return fontNameList;
	}

	/**
	 * Answer my font name scroll pane.
	 *
	 * @return javax.swing.JScrollPane
	 */
	protected JScrollPane fontNameScrollPane() {
		if (fontNameScrollPane == null) {
			fontNameScrollPane = new JScrollPane(this.fontNameList(), ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		}

		return fontNameScrollPane;
	}

	/**
	 * Answer my font size list.
	 *
	 * @return javax.swing.JList
	 */
	protected JList fontSizeList() {
		if (fontSizeList == null) {
			fontSizeList = new JList(new DefaultListModel());
			fontSizeList.addListSelectionListener(new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e) {
					if (e.getValueIsAdjusting()) {
						return;
					}
					getFontModel().setFontSizeIndex(fontSizeList.getSelectedIndex());
				}
			});
		}

		return fontSizeList;
	}

	/**
	 * Answer my font size scroll pane.
	 *
	 * @return javax.swing.JScrollPane
	 */
	protected JScrollPane fontSizeScrollPane() {
		if (fontSizeScrollPane == null) {
			fontSizeScrollPane = new JScrollPane(this.fontSizeList(), ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		}

		return fontSizeScrollPane;
	}

	/**
	 * Update the font name list.
	 */
	protected void updateFontNameList() {
		DefaultListModel model = (DefaultListModel) this.fontNameList().getModel();
		model.clear();

		String[] fontNames = this.getFontModel().fontNames();
		for (int i = 0; i < fontNames.length; i++) {
			model.add(i, fontNames[i]);
		}

		int fontNameIndex = this.getFontModel().getFontNameIndex();
		if (fontNameIndex >= 0) {
			this.fontNameList().setSelectedIndex(fontNameIndex);
			this.fontNameList().ensureIndexIsVisible(fontNameIndex);
		}
	}

	/**
	 * Update the font size list.
	 */
	protected void updateFontSizeList() {
		DefaultListModel model = (DefaultListModel) this.fontSizeList().getModel();
		model.clear();

		int[] fontSizes = this.getFontModel().fontSizes();
		for (int i = 0; i < fontSizes.length; i++) {
			model.add(i, String.valueOf(fontSizes[i]));
		}

		int fontSizeIndex = this.getFontModel().getFontSizeIndex();
		if (fontSizeIndex >= 0) {
			this.fontSizeList().setSelectedIndex(fontSizeIndex);
			this.fontSizeList().ensureIndexIsVisible(fontSizeIndex);
		}
	}
}
