package sra.jun.goodies.freehand;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import sra.smalltalk.*;

/**
 * JunFreehandController class
 * 
 *  @author    nisinaka
 *  @created   2002/07/08 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFreehandController.java,v 6.0 2003/10/28 10:52:09 nisinaka Exp $
 */
public class JunFreehandController extends StController implements MouseListener, MouseMotionListener {
	private static final int _DeltaTime = 20;
	protected Graphics2D _graphics;
	protected Point _currentPoint;
	protected Point _previousPoint;
	protected Vector _pointCollection;
	protected long _previousTime;
	protected Vector _timeCollection;

	/**
	 * Answer the model as a JunFreehandPage.
	 * 
	 * @return sra.jun.goodies.freehand.JunFreehandPage
	 */
	public JunFreehandPage getModel() {
		return (JunFreehandPage) this.model();
	}

	/**
	 * Invoked when a mouse button is dragged on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 */
	public void mouseDragged(MouseEvent e) {
		_currentPoint = e.getPoint();

		if ((_previousPoint == null) || _currentPoint.equals(_previousPoint)) {
			return;
		}

		long currentTime = System.currentTimeMillis();

		if ((currentTime - _previousTime) <= _DeltaTime) {
			return;
		}

		_pointCollection.add(_currentPoint);
		_previousPoint = _currentPoint;

		_timeCollection.add(new Long(currentTime));
		_previousTime = currentTime;

		Point[] points = null;
		int size = _pointCollection.size();

		if (size >= 4) {
			points = new Point[] {(Point) _pointCollection.get(size - 4), (Point) _pointCollection.get(size - 3), (Point) _pointCollection.get(size - 2), (Point) _pointCollection.get(size - 1)};
		} else if (size >= 3) {
			points = new Point[] {(Point) _pointCollection.get(size - 3), (Point) _pointCollection.get(size - 2), (Point) _pointCollection.get(size - 1)};
		} else {
			points = new Point[] {(Point) _pointCollection.get(size - 2), (Point) _pointCollection.get(size - 1)};
		}

		this.getModel()._drawPolyline(this._graphics(), points);
	}

	/**
	 * Invoked when a mouse button is pressed on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 */
	public void mousePressed(MouseEvent e) {
		this._flush();

		_currentPoint = _previousPoint = e.getPoint();
		_pointCollection = new Vector(1024);
		_pointCollection.add(_currentPoint);
		_timeCollection = new Vector(1024);
		_previousTime = System.currentTimeMillis();
		_timeCollection.add(new Long(_previousTime));

		this.getModel()._drawMark(this._graphics(), _currentPoint);
	}

	/**
	 * Invoked when a mouse button is released on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 */
	public void mouseReleased(MouseEvent e) {
		this.getModel()._drawPolyline(this._graphics(), _pointCollection);
		this.getModel()._drawMark(this._graphics(), e.getPoint());
		this.getModel().pointCollection_boundingBox_timeCollection_(_pointCollection, this._component().getBounds(), _timeCollection);

		this._flush();
	}

	/**
	 * Answer the view as a component.
	 * 
	 * @return java.awt.Component
	 */
	protected Component _component() {
		return (Component) this.view();
	}

	/**
	 * Flush the instance variables.
	 */
	protected void _flush() {
		if (_graphics != null) {
			_graphics.dispose();
			_graphics = null;
		}

		_currentPoint = null;
		_previousPoint = null;
		_pointCollection = null;
		_previousTime = 0;
		_timeCollection = null;
	}

	/**
	 * Answer the graphics of the component. The sender is responsible to
	 * dispose.
	 * 
	 * @return java.awt.Graphics2D
	 */
	protected Graphics2D _graphics() {
		if (_graphics == null) {
			_graphics = (Graphics2D) this._component().getGraphics();
			_graphics.setPaint(this.getModel().nibColor());
			_graphics.setStroke(this.getModel()._stroke(this.getModel().nibWidth()));
		}

		return _graphics;
	}

	/**
	 * Add listeners.
	 * 
	 * @param aView sra.smalltalk.StView
	 */
	protected void buildListener(StView aView) {
		Component aComponent = (Component) aView;
		aComponent.addMouseListener(this);
		aComponent.addMouseMotionListener(this);
	}
}
