package sra.jun.goodies.freehand;

import java.util.Vector;
import sra.smalltalk.*;

/**
 * JunFreehandNote class
 * 
 *  @author    nisinaka
 *  @created   2002/07/09 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFreehandNote.java,v 6.0 2003/10/28 10:52:09 nisinaka Exp $
 */
public class JunFreehandNote extends JunFreehandPage {
	protected int pageIndex;
	protected Vector pageCollection;
	protected StMenuModel _menuModel;

	/**
	 * Create a new instance of JunFreehandNote and initialize it.
	 */
	public JunFreehandNote() {
		this.initialize();
	}

	/**
	 * Flush the note.
	 */
	public void flushNote() {
		pageIndex = -1;
		pageCollection = new Vector();
		this.makePage();
	}

	/**
	 * Menu message: go to the first page.
	 */
	public void goToFirstPage() {
		this.savePage();
		pageIndex = 0;
		this.loadPage();
		this.changed();
	}

	/**
	 * Menu message: go to the last page.
	 */
	public void goToLastPage() {
		this.savePage();
		pageIndex = this.pageCollection().size() - 1;
		this.loadPage();
		this.changed();
	}

	/**
	 * Menu message: go to the next page.
	 */
	public void goToNextPage() {
		this.savePage();

		if (this.pageIndex() < (this.pageCollection().size() - 1)) {
			pageIndex++;
		} else {
			this.makePage();
		}

		this.loadPage();
		this.changed();
	}

	/**
	 * Menu message: go to the previous page.
	 */
	public void goToPreviousPage() {
		this.savePage();

		if (this.pageIndex() <= 0) {
			return;
		}

		pageIndex--;
		this.loadPage();
		this.changed();
	}

	/**
	 * Answer a menu model.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 */
	public StMenuModel menuModel() {
		if (_menuModel == null) {
			_menuModel = this.defaultMenu();
		}

		return _menuModel;
	}

	/**
	 * Menu message: create a new note.
	 */
	public void newNote() {
		(new JunFreehandNote()).open();
	}

	/**
	 * Answer the current collection of the pages.
	 * 
	 * @return java.util.Vector
	 */
	public Vector pageCollection() {
		return pageCollection;
	}

	/**
	 * Answer the current index of the pages.
	 * 
	 * @return int
	 */
	public int pageIndex() {
		return pageIndex;
	}

	/**
	 * Answer the default menu model.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 */
	protected StMenuModel defaultMenu() {
		StMenuModel rootMenu = new StMenuModel();

		//
		StMenu fileMenu = new StMenu("File");
		fileMenu.add(createMenuItem("New", "newNote"));
		fileMenu.addSeparator();
		fileMenu.add(createMenuItem("Quit", "closeRequest"));
		rootMenu.add(fileMenu);

		//
		StMenu editMenu = new StMenu("Edit");
		editMenu.add(createMenuItem("Back", "goBackward"));
		editMenu.add(createMenuItem("Forward", "goForward"));
		editMenu.addSeparator();
		editMenu.add(createMenuItem("Color", "changeNibColor"));

		StMenu nibMenu = new StMenu("Nib");
		nibMenu.add(createMenuItem("1", "changeNibWidth1"));
		nibMenu.add(createMenuItem("2", "changeNibWidth2"));
		nibMenu.add(createMenuItem("3", "changeNibWidth3"));
		nibMenu.add(createMenuItem("4", "changeNibWidth4"));
		nibMenu.add(createMenuItem("5", "changeNibWidth5"));
		editMenu.add(nibMenu);
		editMenu.addSeparator();
		editMenu.add(createMenuItem("Clear", "clearPage"));
		rootMenu.add(editMenu);

		//
		StMenu pageMenu = new StMenu("Page");
		pageMenu.add(createMenuItem("Next (New Page)", "goToNextPage"));
		pageMenu.add(createMenuItem("Previous", "goToPreviousPage"));
		pageMenu.addSeparator();
		pageMenu.add(createMenuItem("First", "goToFirstPage"));
		pageMenu.add(createMenuItem("Last", "goToLastPage"));
		rootMenu.add(pageMenu);

		//
		StMenu miscMenu = new StMenu("Misc");
		miscMenu.add(createMenuItem("Viewfinder", "openViewfinder"));
		rootMenu.add(miscMenu);

		//
		return rootMenu;
	}

	/**
	 * Initialize the instance variables.
	 */
	protected void initialize() {
		super.initialize();

		this.flushNote();
		_menuModel = null;
	}

	/**
	 * Load the current page.
	 */
	protected void loadPage() {
		JunFreehandPage aPage = (JunFreehandPage) this.pageCollection().get(this.pageIndex());
		this.nibColor_nibWidth_nibCollection_locusCollection_scalarCollection_currentIndex_(aPage.nibColor(), aPage.nibWidth(), aPage.nibCollection(), aPage.locusCollection(), aPage.scalarCollection(), aPage.currentIndex());
	}

	/**
	 * Make a page.
	 */
	protected void makePage() {
		this.pageCollection().add(new JunFreehandPage());
		pageIndex++;
	}

	/**
	 * Save the current page.
	 */
	protected void savePage() {
		JunFreehandPage aPage = new JunFreehandPage();
		aPage.nibColor_nibWidth_nibCollection_locusCollection_scalarCollection_currentIndex_(this.nibColor(), this.nibWidth(), this.nibCollection(), this.locusCollection(), this.scalarCollection(), this.currentIndex());
		this.pageCollection().set(this.pageIndex(), aPage);
	}

	/**
	 * Answer the title string.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "Freehand Note";
	}
}
