package sra.jun.goodies.freehand;

import java.awt.*;
import java.util.Vector;
import sra.jun.geometry.basic.*;
import sra.jun.goodies.colors.JunColorChoiceDialog;
import sra.jun.opengl.display.JunOpenGLDisplayModel;
import sra.jun.opengl.objects.*;
import sra.smalltalk.*;

/**
 * JunFreehandPage class
 * 
 *  @author    nisinaka
 *  @created   2002/07/05 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFreehandPage.java,v 6.0 2003/10/28 10:52:09 nisinaka Exp $
 */
public class JunFreehandPage extends StApplicationModel {
	protected Color nibColor;
	protected int nibWidth;
	protected Vector nibCollection;
	protected Vector locusCollection;
	protected Vector scalarCollection;
	protected int currentIndex;

	/**
	 * Create a new instance of JunFreehandPage and initialize it.
	 */
	public JunFreehandPage() {
		this.initialize();
	}

	/**
	 * Menu message: change the nib color.
	 */
	public void changeNibColor() {
		Color newColor = JunColorChoiceDialog.Request_color_("Select a color.", this.nibColor());
		this.nibColor_(newColor);
	}

	/**
	 * Menu message: change the nib width to 1.
	 */
	public void changeNibWidth1() {
		this.nibWidth_(1);
	}

	/**
	 * Menu message: change the nib width to 2.
	 */
	public void changeNibWidth2() {
		this.nibWidth_(2);
	}

	/**
	 * Menu message: change the nib width to 3.
	 */
	public void changeNibWidth3() {
		this.nibWidth_(3);
	}

	/**
	 * Menu message: change the nib width to 4.
	 */
	public void changeNibWidth4() {
		this.nibWidth_(4);
	}

	/**
	 * Menu message: change the nib width to 5.
	 */
	public void changeNibWidth5() {
		this.nibWidth_(5);
	}

	/**
	 * Menu message: clear the page.
	 */
	public void clearPage() {
		if (StDialog.Confirm_("Really clear all?", false)) {
			this.flushPage();
			this.changed();
		}
	}

	/**
	 * Answer the current index.
	 * 
	 * @return int
	 */
	public int currentIndex() {
		return currentIndex;
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunFreehandViewAwt(this);
		} else {
			return new JunFreehandViewSwing(this);
		}
	}

	/**
	 * Flush the page.
	 */
	public void flushPage() {
		nibCollection = new Vector();
		locusCollection = new Vector();
		scalarCollection = new Vector();
		currentIndex = -1;
	}

	/**
	 * Menu message: go backward.
	 */
	public void goBackward() {
		currentIndex = Math.max(-1, Math.min(this.currentIndex() - 1, this.locusCollection().size() - 1));
		this.changed();
	}

	/**
	 * Menu message: go forward.
	 */
	public void goForward() {
		currentIndex = Math.max(-1, Math.min(this.currentIndex() + 1, this.locusCollection().size() - 1));
		this.changed();
	}

	/**
	 * Answer the locus collection.
	 * 
	 * @return java.util.Vector
	 */
	public Vector locusCollection() {
		return locusCollection;
	}

	/**
	 * Answer the nib collection.
	 * 
	 * @return java.util.Vector
	 */
	public Vector nibCollection() {
		return nibCollection;
	}

	/**
	 * Answer the current nib color.
	 * 
	 * @return java.awt.Color
	 */
	public Color nibColor() {
		return nibColor;
	}

	/**
	 * Set the new nib color.
	 * 
	 * @param newNibColor java.awt.Color
	 */
	public void nibColor_(Color newNibColor) {
		nibColor = newNibColor;
	}

	/**
	 * Answer the current nib width.
	 * 
	 * @return int
	 */
	public int nibWidth() {
		return nibWidth;
	}

	/**
	 * Set the new nib width.
	 * 
	 * @param newNibWidth int
	 */
	public void nibWidth_(int newNibWidth) {
		nibWidth = Math.max(1, Math.min(newNibWidth, 10));
	}

	/**
	 * Menu message: open a viewfinder.
	 */
	public void openViewfinder() {
		JunOpenGL3dObject anObject = this.threeDimensionalObject();

		if (anObject == null) {
			return;
		}

		JunOpenGLDisplayModel.Show_eyePoint_sightPoint_upVector_(anObject, new Jun3dPoint(0, 0, 1000), new Jun3dPoint(0, 0, 0), new Jun3dPoint(0, 1, 0));
	}

	/**
	 * Answer the scalar collection.
	 * 
	 * @return java.util.Vector
	 */
	public Vector scalarCollection() {
		return scalarCollection;
	}

	/**
	 * Draw a mark on the graphics at the specified point.
	 * 
	 * @param aGraphics java.awt.Graphics
	 * @param aPoint java.awt.Point
	 */
	protected void _drawMark(Graphics aGraphics, Point aPoint) {
		int nibWidth = this.nibWidth();
		int x = aPoint.x - (nibWidth / 2);
		int y = aPoint.y - (nibWidth / 2);
		int side = (nibWidth / 2 * 2) + 1;
		aGraphics.fillRect(x, y, side, side);
	}

	/**
	 * Draw a polyline on the graphics.
	 * 
	 * @param aGraphics java.awt.Graphics
	 * @param points java.awt.Point[]
	 */
	protected void _drawPolyline(Graphics aGraphics, Point[] points) {
		for (int i = 1; i < points.length; i++) {
			aGraphics.drawLine(points[i - 1].x, points[i - 1].y, points[i].x, points[i].y);
		}
	}

	/**
	 * Draw a polyline on the graphics.
	 * 
	 * @param aGraphics java.awt.Graphics
	 * @param pointCollection java.util.Vector
	 */
	protected void _drawPolyline(Graphics aGraphics, Vector pointCollection) {
		Point[] points = new Point[pointCollection.size()];
		pointCollection.copyInto(points);
		this._drawPolyline(aGraphics, points);
	}

	/**
	 * Answer the stroke.
	 * 
	 * @param width float
	 * 
	 * @return java.awt.Stroke
	 */
	protected Stroke _stroke(float width) {
		return new BasicStroke(width, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
	}

	/**
	 * Initialize the instance variables.
	 */
	protected void initialize() {
		nibColor = Color.black;
		nibWidth = 1;
		this.flushPage();
	}

	/**
	 * Initialize with the specified values.
	 * 
	 * @param newNibColor java.awt.Color
	 * @param newNibWidth int
	 * @param newNibCollection java.util.Vector
	 * @param newLocusCollection java.util.Vector
	 * @param newScalarCollection java.util.Vector
	 * @param newCurrentIndex int
	 */
	protected void nibColor_nibWidth_nibCollection_locusCollection_scalarCollection_currentIndex_(Color newNibColor, int newNibWidth, Vector newNibCollection, Vector newLocusCollection, Vector newScalarCollection, int newCurrentIndex) {
		this.nibColor_(newNibColor);
		this.nibWidth_(newNibWidth);
		nibCollection = newNibCollection;
		locusCollection = newLocusCollection;
		scalarCollection = newScalarCollection;
		currentIndex = Math.min(newCurrentIndex, locusCollection.size());
	}

	/**
	 * Set the stroke information.
	 * 
	 * @param pointCollection java.util.Vector
	 * @param boundingBox java.awt.Rectangle
	 * @param timeCollection java.util.Vector
	 */
	protected void pointCollection_boundingBox_timeCollection_(Vector pointCollection, Rectangle boundingBox, Vector timeCollection) {
		if (pointCollection.isEmpty()) {
			return;
		}

		if (timeCollection.isEmpty()) {
			return;
		}

		int denomiWidth = boundingBox.width / 2;
		int denomiHeight = boundingBox.height / 2;
		Point centerPoint = new Point(denomiWidth, denomiHeight);

		int numberOfPoints = pointCollection.size();
		Jun2dPoint[] normalizedPointCollection = new Jun2dPoint[numberOfPoints];

		if ((denomiWidth == 0) || (denomiHeight == 0)) {
			for (int i = 0; i < numberOfPoints; i++) {
				normalizedPointCollection[i] = new Jun2dPoint(0, -0);
			}
		} else {
			for (int i = 0; i < numberOfPoints; i++) {
				Point p = (Point) pointCollection.get(i);
				double x = (double) (p.x - centerPoint.x) / denomiWidth;
				double y = (double) (p.y - centerPoint.y) / denomiHeight;
				normalizedPointCollection[i] = new Jun2dPoint(x, -y);
			}
		}

		long startTime = ((Number) timeCollection.firstElement()).longValue();
		long denomiTime = ((Number) timeCollection.lastElement()).longValue() - startTime;
		double[] normalizedTimeCollection = null;

		if (denomiTime == 0) {
			normalizedTimeCollection = new double[] { 0 };
		} else {
			int numberOfTimes = timeCollection.size();
			normalizedTimeCollection = new double[numberOfTimes];

			for (int i = 0; i < numberOfTimes; i++) {
				double t = ((Number) timeCollection.get(i)).doubleValue();
				normalizedTimeCollection[i] = (t - startTime) / denomiTime;
			}
		}

		Jun3dPoint[] locus = new Jun3dPoint[numberOfPoints];

		for (int i = 0; i < numberOfPoints; i++) {
			locus[i] = new Jun3dPoint(normalizedPointCollection[i], normalizedTimeCollection[i]);
		}

		this.nibCollection().setSize(currentIndex + 1);
		this.locusCollection().setSize(currentIndex + 1);
		this.scalarCollection().setSize(currentIndex + 1);

		this.nibCollection().add(new Object[] { this.nibColor(), new Integer(this.nibWidth())});
		this.locusCollection().add(locus);
		this.scalarCollection().add(new Jun3dPoint(denomiWidth, denomiHeight, denomiTime));

		currentIndex++;
	}

	/**
	 * Answer the 3D object of the current status.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected JunOpenGL3dObject threeDimensionalObject() {
		if (this.currentIndex() < 0) {
			return null;
		}

		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
		Jun3dPoint translationPoint = new Jun3dPoint(0, 0, 0);

		for (int i = 0; i <= this.currentIndex(); i++) {
			Jun3dPoint[] points = (Jun3dPoint[]) locusCollection.get(i);
			Jun3dPoint scalar = (Jun3dPoint) scalarCollection.get(i);
			Object[] nib = (Object[]) nibCollection.get(i);
			Color color = (Color) nib[0];
			int dot = ((Number) nib[1]).intValue();

			Jun3dPoint[] scaledPoints = new Jun3dPoint[points.length];

			for (int j = 0; j < scaledPoints.length; j++) {
				scaledPoints[j] = points[j].multipliedBy_(scalar);
			}

			JunOpenGL3dObject object = null;

			if (scaledPoints.length == 1) {
				JunOpenGL3dVertex vertex = new JunOpenGL3dVertex(scaledPoints[0]);
				vertex.size_(dot);
				object = vertex;
			} else {
				JunOpenGL3dPolyline polyline = new JunOpenGL3dPolyline(scaledPoints);
				polyline.lineWidth_(dot);
				object = polyline;
			}

			object.paint_(color);
			object = object.translatedBy_(translationPoint);
			compoundObject.add_(object);
			translationPoint = translationPoint.plus_(new Jun3dPoint(0, 0, scalar.z()));
		}

		Jun3dBoundingBox boundingBox = compoundObject.boundingBox();
		compoundObject = (JunOpenGL3dCompoundObject) compoundObject.scaledBy_(new Jun3dPoint(2 / boundingBox.width(), 2 / boundingBox.height(), 2 / boundingBox.depth()));

		return compoundObject;
	}

	/**
	 * Answer the title string.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "Freehand Page";
	}
}
