package sra.jun.goodies.freehand;

import java.awt.*;
import java.util.Vector;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.smalltalk.*;

/**
 * JunFreehandViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2002/07/08 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFreehandViewSwing.java,v 6.0 2003/10/28 10:52:11 nisinaka Exp $
 */
public class JunFreehandViewSwing extends StViewJPanel implements JunFreehandView {

	/**
	 * Create a new instance of JunFreehandViewSwing and initialize it.
	 * 
	 * @param aFreehandPage sra.jun.goodies.freehand.JunFreehandPage
	 */
	public JunFreehandViewSwing(JunFreehandPage aFreehandPage) {
		super(aFreehandPage);
	}

	/**
	 * Display on the graphics.
	 * 
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayOn_(Graphics aGraphics) {
		if (this.isShowing() == false) {
			return;
		}

		Vector locusCollection = this.getModel().locusCollection();

		if (locusCollection.isEmpty()) {
			return;
		}

		Vector scalarCollection = this.getModel().scalarCollection();
		Vector nibCollection = this.getModel().nibCollection();
		Rectangle bounds = this.getBounds();
		Point originPoint = new Point(bounds.x + (bounds.width / 2), bounds.y + (bounds.height / 2));

		for (int index = 0; index < this.getModel().currentIndex(); index++) {
			Jun3dPoint[] normalizedPoints = (Jun3dPoint[]) locusCollection.get(index);
			Jun3dPoint scalar = (Jun3dPoint) scalarCollection.get(index);
			Object[] nib = (Object[]) nibCollection.get(index);
			Color color = (Color) nib[0];
			float dot = ((Number) nib[1]).floatValue();
			((Graphics2D) aGraphics).setPaint(color);
			((Graphics2D) aGraphics).setStroke(this.getModel()._stroke(dot));

			Point[] points = new Point[normalizedPoints.length];

			for (int i = 0; i < points.length; i++) {
				int x = (int) Math.round(normalizedPoints[i].x() * scalar.x());
				int y = (int) Math.round(normalizedPoints[i].y() * scalar.y());
				points[i] = new Point(originPoint.x + x, originPoint.y - y);
			}

			this.getModel()._drawMark(aGraphics, points[0]);
			this.getModel()._drawPolyline(aGraphics, points);
			this.getModel()._drawMark(aGraphics, points[points.length - 1]);
		}
	}

	/**
	 * Answer the model as a JunFreehandPage.
	 * 
	 * @return sra.jun.goodies.freehand.JunFreehandPage
	 */
	public JunFreehandPage getModel() {
		return (JunFreehandPage) this.model();
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setPreferredSize(new Dimension(300, 300));
	}

	/**
	 * Answer the default controller.
	 * 
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new JunFreehandController();
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunFreehandPage();
	}
}
