package sra.jun.goodies.images.streams;

import java.awt.*;
import java.io.*;
import sra.smalltalk.*;

/**
 * JunDefaultImageStream class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2001/04/18 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunDefaultImageStream.java,v 6.0 2003/10/28 10:52:11 nisinaka Exp $
 */
public class JunDefaultImageStream extends JunImageStream {
	/**
	 * Constractor with input stream.
	 * 
	 * @return sra.jun.goodies.images.streams.JunImageStream
	 * @param stream java.io.InputStream
	 * @exception java.io.IOException
	 * @category Instance creation
	 */
	public static JunImageStream On_(InputStream stream) throws IOException {
		return On_(new JunDefaultImageStream(), stream);
	}

	/**
	 * Constractor with output stream.
	 * 
	 * @return sra.jun.goodies.images.streams.JunImageStream
	 * @param stream java.io.OutputStream
	 * @exception java.io.IOException
	 * @category Instance creation
	 */
	public static JunImageStream On_(OutputStream stream) throws IOException {
		throw new IOException("could not save image");
	}

	/**
	 * Read the image on input stream. (used Java core API - toolkil)
	 * 
	 * @return sra.smalltalk.StImage
	 * @exception java.io.IOException
	 * @category accessing
	 */
	public StImage nextImage() throws IOException {
		if (inStream != null) {
			Frame frame = new Frame();
			Toolkit toolkit = frame.getToolkit();
			Image image = toolkit.createImage(this.readInputStream());
			MediaTracker tracker = new MediaTracker(frame);
			tracker.addImage(image, 0);
			try {
				tracker.waitForAll();
			} catch (InterruptedException exception) {
				throw new IOException("image load interrupted.");
			}
			int status = tracker.statusAll(false);
			if (((status & MediaTracker.ABORTED) != 0) || ((status & MediaTracker.ERRORED) != 0)) {
				throw new IOException("image load interrupted.");
			}
			image.flush();
			imageObject = new StImage(image);
		}
		return imageObject;
	}

	/**
	 * Write the image on output stream. (used Java core API - toolkil)
	 * 
	 * @param newImage sra.smalltalk.StImage
	 * @exception java.io.IOException
	 * @category accessing
	 */
	public void nextPutImage_(StImage newImage) throws IOException {
		throw new IOException("could not save image");
	}

	/**
	 * Set the output stream.
	 * 
	 * @param stream java.io.OutputStream
	 * @exception java.io.IOException
	 * @category initialize-release
	 */
	protected void on_(OutputStream stream) throws IOException {
		throw new IOException("could not save image");
	}

	/**
	 * Read the input stream.
	 * 
	 * @return sra.smalltalk.StImage
	 * @exception java.io.IOException
	 * @ category stream access
	 */
	private byte[] readInputStream() throws IOException {
		byte[] bytes = null;
		if (inStream != null) {
			ByteArrayOutputStream byteStream = null;
			BufferedOutputStream bufferedStream = null;
			try {
				byteStream = new ByteArrayOutputStream();
				bufferedStream = new BufferedOutputStream(byteStream);
				byte[] buffer = new byte[5120];
				int size;
				while ((size = inStream.read(buffer, 0, buffer.length)) != -1) {
					bufferedStream.write(buffer, 0, size);
				}
				bufferedStream.flush();
				bytes = byteStream.toByteArray();
			} finally {
				if (bufferedStream != null) {
					bufferedStream.close();
					bufferedStream = null;
				}
				if (byteStream != null) {
					byteStream.close();
					byteStream = null;
				}
			}
		}
		return bytes;
	}
}
