package sra.jun.goodies.images.streams;

import java.io.*;
import java.util.*;
import sra.smalltalk.*;

/**
 * JunImageStream class
 * 
 *  @author    Hirotsugu Kondo
 *  @created   1999/01/20 (by Hirotsugu Kondo)
 *  @updated   2000/11/12 (by Mitsuhiro Asada)
 *  @updated   2001/04/12 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunImageStream.java,v 6.0 2003/10/28 10:52:11 nisinaka Exp $
 */
public abstract class JunImageStream extends StObject {
	protected InputStream inStream;
	protected OutputStream outStream;
	protected StImage imageObject;
	protected StValueHolder progressValue = new StValueHolder(0.0f);
	//
	private static Hashtable ImageKindTable;
	static {
		ImageKindTable = _InitializeImageKindTable();
	}

	/**
	 * Answer a Class of image stream.
	 * 
	 * @return java.lang.Class
	 * @param aFilename java.io.File
	 * @category Accessing
	 */
	public static Class ImageStreamClassForFileName_(File aFilename) {
		return ImageStreamClassForFileName_(aFilename.toString());
	}

	/**
	 * Answer a Class of image stream.
	 * 
	 * @return java.lang.Class
	 * @param aFilename java.lang.String
	 * @category Accessing
	 */
	public static Class ImageStreamClassForFileName_(String aFilename) {
		String aString = aFilename.substring(aFilename.lastIndexOf('.') + 1).toLowerCase();
		String aSymbol = (String) ImageKindTable.get(aString);
		if (aSymbol == null) {
			return null;
		}
		Class aClass = null;
		try {
			aClass = Class.forName(aSymbol);
		} catch (ClassNotFoundException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		return aClass;
	}

	/**
	 * Constractor with input stream.
	 * 
	 * @return sra.jun.goodies.images.JunImageStream
	 * @param stream java.io.InputStream
	 * @exception java.io.IOException
	 * @category Instance creation
	 */
	public static JunImageStream On_(InputStream stream) throws IOException {
		throw new IOException("subclassResponsibilitty");
	}

	/**
	 * Constractor with output stream.
	 * 
	 * @return sra.jun.goodies.images.JunImageStream
	 * @param stream java.io.OutputStream
	 * @exception java.io.IOException
	 * @category Instance creation
	 */
	public static JunImageStream On_(OutputStream stream) throws IOException {
		throw new IOException("subclassResponsibilitty");
	}

	/**
	 * Initialize image kind table.
	 * 
	 * @return java.util.Hashtable
	 * @category Class initialization
	 */
	private static Hashtable _InitializeImageKindTable() {
		if (ImageKindTable == null) {
			Hashtable aDictionary = new Hashtable();
			aDictionary.put("bmp", "sra.jun.goodies.images.streams.JunBmpImageStream");
			aDictionary.put("jpg", "sra.jun.goodies.images.streams.JunJpegImageStream");
			aDictionary.put("jpeg", "sra.jun.goodies.images.streams.JunJpegImageStream");
			aDictionary.put("png", "sra.jun.goodies.images.streams.JunPngImageStream");
			aDictionary.put("gif", "sra.jun.goodies.images.streams.JunDefaultImageStream");
			ImageKindTable = aDictionary;
		}
		return ImageKindTable;
	}

	/**
	 * Constractor with image stream and input stream.
	 * 
	 * @return sra.jun.goodies.images.streams.JunImageStream
	 * @param imageStream sra.jun.goodies.images.streams.JunImageStream
	 * @param stream java.io.InputStream
	 * @exception java.io.IOException
	 * @category Instance creation
	 */
	protected static JunImageStream On_(JunImageStream imageStream, InputStream stream) throws IOException {
		if (imageStream == null) {
			throw new IOException("could not load image");
		}
		imageStream.on_(stream);
		return imageStream;
	}

	/**
	 * Constractor with image stream and output stream.
	 * 
	 * @return sra.jun.goodies.images.streams.JunImageStream
	 * @param imageStream sra.jun.goodies.images.streams.JunImageStream
	 * @param stream java.io.OutputStream
	 * @exception java.io.IOException
	 * @category Instance creation
	 */
	protected static JunImageStream On_(JunImageStream imageStream, OutputStream stream) throws IOException {
		if (imageStream == null) {
			throw new IOException("could not save image");
		}
		imageStream.on_(stream);
		return imageStream;
	}

	/**
	 * This image stream close.
	 * 
	 * @exception java.io.IOException
	 * @category stream access
	 */
	public void close() throws IOException {
		if (inStream != null) {
			inStream.close();
			inStream = null;
		}

		if (outStream != null) {
			outStream.close();
			outStream = null;
		}
	}

	/**
	 * Do something with the progress value.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @category progress
	 */
	public void compute_(StBlockClosure aBlock) {
		this.progressValue().compute_(aBlock);
	}

	/**
	 * This image stream flush.
	 * 
	 * @exception java.io.IOException
	 * @category stream access
	 */
	public void flush() throws IOException {
		if (outStream != null) {
			outStream.flush();
		}
	}

	/**
	 * Read the image on input stream.
	 * 
	 * @return sra.smalltalk.StImage
	 * @exception java.io.IOException
	 * @category accessing
	 */
	public abstract StImage nextImage() throws IOException;

	/**
	 * Write the image on output stream.
	 * 
	 * @param newImage sra.smalltalk.StImage
	 * @exception java.io.IOException
	 * @category accessing
	 */
	public abstract void nextPutImage_(StImage newImage) throws IOException;

	/**
	 * Set the input stream.
	 * 
	 * @param stream java.io.InputStream
	 * @exception java.io.IOException
	 * @category initialize-release
	 */
	protected void on_(InputStream stream) throws IOException {
		inStream = stream;
	}

	/**
	 * Set the output stream.
	 * 
	 * @param stream java.io.OutputStream
	 * @exception java.io.IOException
	 * @category initialize-release
	 */
	protected void on_(OutputStream stream) throws IOException {
		outStream = stream;
	}

	/**
	 * Set the progress value.
	 * 
	 * @param normalizedNumber float
	 * @category progress
	 */
	protected void progress_(float normalizedNumber) {
		if ((0 <= normalizedNumber) && (normalizedNumber <= 1)) {
			float truncatedValue = Math.round(normalizedNumber / 0.005) * 0.005f;

			if (this.progressValue()._floatValue() != truncatedValue) {
				this.progressValue().value_(truncatedValue);
			}
		}
	}

	/**
	 * Answer the progress value.
	 * 
	 * @return float
	 * @category progress
	 */
	protected void progress() {
		this.progressValue()._floatValue();
	}

	/**
	 * Set the StValueHolder for the progress value.
	 * 
	 * @param aValueHolder sra.smalltalk.StValueHolder
	 * @category progress
	 */
	protected void progressValue_(StValueHolder aValueHolder) {
		progressValue = aValueHolder;
	}

	/**
	 * Answer the StValueHolder for the progress value.
	 * 
	 * @return sra.smalltalk.StValueHolder
	 * @category progress
	 */
	protected StValueHolder progressValue() {
		return progressValue;
	}
}
