package sra.jun.goodies.images.streams;

import com.sun.media.jai.codec.*;
import java.awt.image.*;
import java.awt.image.renderable.*;
import java.io.*;
import javax.media.jai.*;
import sra.smalltalk.*;

/**
 * JunJaiJpegImageStream class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2000/11/12 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunJaiJpegImageStream.java,v 6.0 2003/10/28 10:52:11 nisinaka Exp $
 */
public class JunJaiJpegImageStream extends JunJpegImageStream {
	/**
	 * Constractor with input stream.
	 * 
	 * @return sra.jun.goodies.images.streams.JunImageStream
	 * @param stream java.io.InputStream
	 * @exception java.io.IOException
	 * @category Instance creation
	 */
	public static JunImageStream On_(InputStream stream) throws IOException {
		return On_(new JunJaiJpegImageStream(), stream);
	}

	/**
	 * Constractor with output stream.
	 * 
	 * @return sra.jun.goodies.images.streams.JunImageStream
	 * @param stream java.io.OutputStream
	 * @exception java.io.IOException
	 * @category Instance creation
	 */
	public static JunImageStream On_(OutputStream stream) throws IOException {
		return On_(new JunJaiJpegImageStream(), stream);
	}

	/**
	 * Read the image on input stream. (used Java Advanced Imaging API)
	 * 
	 * @return sra.smalltalk.StImage
	 * @exception java.io.IOException
	 * @category accessing
	 */
	public StImage nextImage() throws IOException {
		if (inStream != null) {
			ImageDecoder imageDecoder = ImageCodec.createImageDecoder("JPEG", inStream, new JPEGEncodeParam());
			RenderedImage renderedImage = imageDecoder.decodeAsRenderedImage();
			WritableRaster writableRaster = renderedImage.getData().createCompatibleWritableRaster();
			BufferedImage anImage = new BufferedImage(renderedImage.getColorModel(), renderedImage.copyData(writableRaster), false, null);
			imageObject = new StImage(anImage);
		}
		return imageObject;
	}

	/**
	 * Write the image on output stream. (used Java Advanced Imaging API)
	 * 
	 * @param newImage sra.smalltalk.StImage
	 * @exception java.io.IOException
	 * @category accessing
	 */
	public void nextPutImage_(StImage newImage) throws IOException {
		if (outStream != null) {
			imageObject = newImage;
			ParameterBlock pb = new ParameterBlock();
			pb.add(newImage.image());
			RenderedOp renderedOp = JAI.create("awtImage", pb);
			ImageEncoder imageEncoder = ImageCodec.createImageEncoder("JPEG", outStream, new JPEGEncodeParam());
			imageEncoder.encode(renderedOp);
		}
	}
}
