package sra.jun.goodies.images.streams;

import com.sun.image.codec.jpeg.*;
import java.awt.image.*;
import java.io.*;
import sra.smalltalk.*;

/**
 * JunJava2JpegImageStream class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2000/11/22 (by Mitsuhiro Asada)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunJava2JpegImageStream.java,v 6.0 2003/10/28 10:52:11 nisinaka Exp $
 */
public class JunJava2JpegImageStream extends JunJpegImageStream {
	/**
	 * Constractor with input stream.
	 * 
	 * @return sra.jun.goodies.images.streams.JunImageStream
	 * @param stream java.io.InputStream
	 * @exception java.io.IOException
	 * @category Instance creation
	 */
	public static JunImageStream On_(InputStream stream) throws IOException {
		return On_(new JunJava2JpegImageStream(), stream);
	}

	/**
	 * Constractor with output stream.
	 * 
	 * @return sra.jun.goodies.images.streams.JunImageStream
	 * @param stream java.io.OutputStream
	 * @exception java.io.IOException
	 * @category Instance creation
	 */
	public static JunImageStream On_(OutputStream stream) throws IOException {
		return On_(new JunJava2JpegImageStream(), stream);
	}

	/**
	 * Read the image on input stream. (used Java2 core API)
	 * 
	 * @return sra.smalltalk.StImage
	 * @exception java.io.IOException
	 * @category accessing
	 */
	public StImage nextImage() throws IOException {
		if (inStream != null) {
			JPEGImageDecoder imageDecoder = JPEGCodec.createJPEGDecoder(inStream);
			BufferedImage anImage = imageDecoder.decodeAsBufferedImage();
			imageObject = new StImage(anImage);
		}
		return imageObject;
	}

	/**
	 * Write the image on output stream. (used Java2 core API)
	 * 
	 * @param newImage sra.smalltalk.StImage
	 * @exception java.io.IOException
	 * @category accessing
	 */
	public void nextPutImage_(StImage newImage) throws IOException {
		if (outStream != null) {
			imageObject = newImage;
			int width = imageObject.width();
			int height = imageObject.height();
			BufferedImage anImage = new BufferedImage(width, height, BufferedImage.TYPE_3BYTE_BGR);
			anImage.setRGB(0, 0, width, height, imageObject.getPixels(), 0, width);
			JPEGImageEncoder imageEncoder = JPEGCodec.createJPEGEncoder(outStream);
			imageEncoder.encode(anImage);
		}
	}
}
