package sra.jun.goodies.images.support;

import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;

/**
 * JunImageAdjuster class
 * 
 *  @author    nisinaka
 *  @created   2001/04/05 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunImageAdjuster.java,v 6.0 2003/10/28 10:52:11 nisinaka Exp $
 */
public class JunImageAdjuster extends StObject {
	protected StValueHolder progressValue = new StValueHolder(0.0f);

	/**
	 * Answer an image adjusted to the specified dimension.
	 * 
	 * @param anImage sra.smalltalk.StImage
	 * @param aDimension java.awt.Dimension
	 * 
	 * @return sra.smalltalk.StImage
	 */
	public static StImage Adjust_extent_(StImage anImage, Dimension aDimension) {
		return (new JunImageAdjuster()).adjust_extent_(anImage, aDimension);
	}

	/**
	 * Show the image.
	 * 
	 * @param anImage sra.smalltalk.StImage
	 */
	public static void Show_(StImage anImage) {
		Frame aFrame = new Frame();
		aFrame.setTitle("Image " + anImage.width() + "x" + anImage.height());
		aFrame.setLocation(100, 100);
		aFrame.setLayout(new BorderLayout());
		aFrame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				e.getWindow().dispose();
			}
		});

		final StImage _image = anImage;
		Canvas newCanvas = new Canvas() {
			public void paint(Graphics g) {
				_image.displayOn_(g);

				return;
			}
		};

		newCanvas.setBounds(0, 0, anImage.width(), anImage.height());
		aFrame.add(newCanvas);
		aFrame.pack();
		aFrame.show();
	}

	/**
	 * Answer an image adjusted to the specified dimension.
	 * 
	 * @param anImage sra.smalltalk.StImage
	 * @param aDimension java.awt.Dimension
	 * 
	 * @return sra.smalltalk.StImage
	 */
	public StImage adjust_extent_(StImage anImage, Dimension aDimension) {
		this.progress_(0.0f);

		StImage theImage = this.computeImage_magnifyExtent_(anImage, aDimension);
		this.progress_(1.0f);

		return theImage;
	}

	/**
	 * Do something with the progress value.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void compute_(StBlockClosure aBlock) {
		this.progressValue().compute_(aBlock);
	}

	/**
	 * Compute a double size image.
	 * 
	 * @param anImage sra.smalltalk.StImage
	 * 
	 * @return sra.smalltalk.StImage
	 */
	protected StImage computeDoubleSizeImage_(StImage anImage) {
		int width = anImage.width();
		int height = anImage.height();
		StImage magnifiedImage = new StImage(width * 2, height * 2);

		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				int colorValue = anImage.getPixel(x, y);
				magnifiedImage.setPixel(x * 2, y * 2, colorValue);
				magnifiedImage.setPixel((x * 2) + 1, y * 2, colorValue);
				magnifiedImage.setPixel(x * 2, (y * 2) + 1, colorValue);
				magnifiedImage.setPixel((x * 2) + 1, (y * 2) + 1, colorValue);
			}
		}

		return magnifiedImage;
	}

	/**
	 * Compute an image maginfied to the specified dimension.
	 * 
	 * @param anImage sra.smalltalk.StImage
	 * @param aDimension java.awt.Dimension
	 * 
	 * @return sra.smalltalk.StImage
	 */
	protected StImage computeImage_magnifyExtent_(StImage anImage, Dimension aDimension) {
		StImage theImage = anImage;

		while ((theImage.width() < aDimension.width) || (theImage.height() < aDimension.height)) {
			theImage = this.computeDoubleSizeImage_(theImage);
		}

		StImage magnifiedImage = this.computeImage_shrinkExtent_(theImage, aDimension);

		return magnifiedImage;
	}

	/**
	 * Compute an image shrunken to the specified dimension.
	 * 
	 * @param anImage sra.smalltalk.StImage
	 * @param aDimension java.awt.Dimension
	 * 
	 * @return sra.smalltalk.StImage
	 */
	protected StImage computeImage_shrinkExtent_(StImage anImage, Dimension aDimension) {
		int width = anImage.width();
		int height = anImage.height();
		StInterval[] xIntervals = this.computeIntervals_divisionSize_(width, aDimension.width);
		StInterval[] yIntervals = this.computeIntervals_divisionSize_(height, aDimension.height);
		StImage shrunkenImage = new StImage(aDimension.width, aDimension.height);
		int yValue = 0;

		for (int yi = 0; yi < yIntervals.length; yi++) {
			int xValue = 0;

			for (int xi = 0; xi < xIntervals.length; xi++) {
				int redValue = 0;
				int greenValue = 0;
				int blueValue = 0;
				int count = 0;

				for (int y = (int) yIntervals[yi].start(); y <= (int) yIntervals[yi].stop(); y++) {
					for (int x = (int) xIntervals[xi].start(); x <= (int) xIntervals[xi].stop(); x++) {
						Color color = anImage.valueAtPoint_(new Point(x, y));
						redValue += color.getRed();
						greenValue += color.getGreen();
						blueValue += color.getBlue();
						count++;
					}
				}

				Color color = null;

				if (count > 0) {
					int r = Math.max(0, Math.min(255, redValue / count));
					int g = Math.max(0, Math.min(255, greenValue / count));
					int b = Math.max(0, Math.min(255, blueValue / count));
					color = new Color(r, g, b);
				} else {
					color = new Color(0, 0, 0);
				}

				shrunkenImage.valueAtPoint_put_(new Point(xValue, yValue), color);
				xValue++;
			}

			yValue++;
			this.progress_((float) yValue / yIntervals.length);
		}

		return shrunkenImage;
	}

	/**
	 * Compute appropriate intervals for the division size.
	 * 
	 * @param anInteger int
	 * @param divisionSize int
	 * 
	 * @return sra.smalltalk.StInterval[]
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	protected StInterval[] computeIntervals_divisionSize_(int anInteger, int divisionSize) {
		if (anInteger < divisionSize) {
			throw new SmalltalkException(anInteger + " is smaller than " + divisionSize);
		}

		//
		float value = 0;
		float stepValue = anInteger / (float) divisionSize;
		int[] values = new int[divisionSize + 1];

		for (int i = 0; i <= divisionSize; i++, value += stepValue) {
			values[i] = Math.round(value);
		}

		StInterval[] intervals = new StInterval[divisionSize];

		for (int i = 0; i < divisionSize; i++) {
			int start = values[i];
			int stop = values[i + 1] - 1;
			intervals[i] = new StInterval(start, stop);
		}

		return intervals;
	}

	/**
	 * Set the progress value.
	 * 
	 * @param normalizedNumber float
	 */
	protected void progress_(float normalizedNumber) {
		if ((0 <= normalizedNumber) && (normalizedNumber <= 1)) {
			float truncatedValue = Math.round(normalizedNumber / 0.005) * 0.005f;

			if (this.progressValue()._floatValue() != truncatedValue) {
				this.progressValue().value_(truncatedValue);
			}
		}
	}

	/**
	 * Answer the StValueHolder for the progress value.
	 * 
	 * @return sra.smalltalk.StValueHolder
	 */
	protected StValueHolder progressValue() {
		return progressValue;
	}
}
