package sra.jun.goodies.images.support;

import java.awt.*;
import java.io.*;
import sra.jun.goodies.files.*;
import sra.jun.goodies.images.streams.*;
import sra.jun.goodies.progress.JunProgress;
import sra.jun.system.support.JunTestExamples;
import sra.smalltalk.*;

/**
 * JunImageAdjusterTestExamples class
 * 
 *  @author    nisinaka
 *  @created   2001/04/05 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunImageAdjusterTestExamples.java,v 6.0 2003/10/28 10:52:11 nisinaka Exp $
 */
public class JunImageAdjusterTestExamples extends JunTestExamples {

	/**
	 * Example: Adjust an image.
	 * 
	 * @return DOCUMENT ME!
	 */
	public static boolean Example1() {
		try {
			StImage anImage = TestImage();
			JunImageAdjuster anAdjuster = new JunImageAdjuster();
			anImage = anAdjuster.adjust_extent_(anImage, new Dimension(128, 96));
			JunImageAdjuster.Show_(anImage);

			return true;
		} catch (Exception e) {
			e.printStackTrace();

			return false;
		}
	}

	/**
	 * Example: Adjust an image while showing a progress bar.
	 * 
	 * @return DOCUMENT ME!
	 */
	public static boolean Example2() {
		try {
			final JunImageAdjuster anAdjuster = new JunImageAdjuster();
			final JunProgress aProgress = new JunProgress();
			anAdjuster.compute_(new StBlockClosure() {
				public Object value_(Object anObject) {
					aProgress.value_(((Number) anObject).floatValue());

					return null;
				}
			});

			final StValueHolder imageHolder = new StValueHolder(TestImage());
			aProgress.message_("adusting image...");
			aProgress.do_(new StBlockClosure() {
				public Object value() {
					StImage anImage = (StImage) imageHolder.value();
					anImage = anAdjuster.adjust_extent_(anImage, new Dimension(128, 96));
					imageHolder.value_(anImage);

					return null;
				}
			});
			JunImageAdjuster.Show_((StImage) imageHolder.value());

			return true;
		} catch (Exception e) {
			e.printStackTrace();

			return false;
		}
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args an array of command-line arguments
	 */
	public static void main(java.lang.String[] args) {
		new JunImageAdjusterTestExamples();
	}

	/**
	 * Test: computeIntervals_divisionSize_()
	 * 
	 * @return DOCUMENT ME!
	 */
	public static boolean Test$computeIntervals_divisionSize_$1() {
		(new JunImageAdjuster()).computeIntervals_divisionSize_(1000, 10);

		return true;
	}

	/**
	 * Test: computeIntervals_divisionSize_()
	 * 
	 * @return DOCUMENT ME!
	 */
	public static boolean Test$computeIntervals_divisionSize_$2() {
		(new JunImageAdjuster()).computeIntervals_divisionSize_(1005, 10);

		return true;
	}

	/**
	 * Example: ImageAdjuster by java.awt.Image#getScaledInstance()
	 * 
	 * @return DOCUMENT ME!
	 */
	public static boolean Test$Image$SCALE_AVERAGE() {
		try {
			StImage anImage = TestImage();
			Image scaledImage = anImage.image().getScaledInstance(128, 96, Image.SCALE_AREA_AVERAGING);
			JunImageAdjuster.Show_(new StImage(scaledImage));

			return true;
		} catch (Exception e) {
			e.printStackTrace();

			return false;
		}
	}

	/**
	 * Example: ImageAdjuster by java.awt.Image#getScaledInstance()
	 * 
	 * @return DOCUMENT ME!
	 */
	public static boolean Test$Image$SCALE_DEFAULT() {
		try {
			StImage anImage = TestImage();
			Image scaledImage = anImage.image().getScaledInstance(128, 96, Image.SCALE_DEFAULT);
			JunImageAdjuster.Show_(new StImage(scaledImage));

			return true;
		} catch (Exception e) {
			e.printStackTrace();

			return false;
		}
	}

	/**
	 * Example: ImageAdjuster by java.awt.Image#getScaledInstance()
	 * 
	 * @return DOCUMENT ME!
	 */
	public static boolean Test$Image$SCALE_FAST() {
		try {
			StImage anImage = TestImage();
			Image scaledImage = anImage.image().getScaledInstance(128, 96, Image.SCALE_FAST);
			JunImageAdjuster.Show_(new StImage(scaledImage));

			return true;
		} catch (Exception e) {
			e.printStackTrace();

			return false;
		}
	}

	/**
	 * Example: ImageAdjuster by java.awt.Image#getScaledInstance()
	 * 
	 * @return DOCUMENT ME!
	 */
	public static boolean Test$Image$SCALE_REPLICATE() {
		try {
			StImage anImage = TestImage();
			Image scaledImage = anImage.image().getScaledInstance(128, 96, Image.SCALE_REPLICATE);
			JunImageAdjuster.Show_(new StImage(scaledImage));

			return true;
		} catch (Exception e) {
			e.printStackTrace();

			return false;
		}
	}

	/**
	 * Example: ImageAdjuster by java.awt.Image#getScaledInstance()
	 * 
	 * @return DOCUMENT ME!
	 */
	public static boolean Test$Image$SCALE_SMOOTH() {
		try {
			StImage anImage = TestImage();
			Image scaledImage = anImage.image().getScaledInstance(128, 96, Image.SCALE_SMOOTH);
			JunImageAdjuster.Show_(new StImage(scaledImage));

			return true;
		} catch (Exception e) {
			e.printStackTrace();

			return false;
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.smalltalk.StImage
	 * 
	 * @exception IOException
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	private static StImage TestImage() throws IOException {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType("jpeg files (*.jpg)", new String[] { ".jpg", ".JPG", ".jpeg", ".JPEG" })
		};
		File file = JunFileDialog.Request("Please select a JPEG file", fileTypes, fileTypes[0]);
		if (file == null) {
			throw new SmalltalkException("cancel");
		}

		//
		JunImageStream stream = JunJpegImageStream.On_(new FileInputStream(file));
		StImage image = null;

		try {
			image = stream.nextImage();
		} finally {
			stream.close();
		}

		return image;
	}
}
