package sra.jun.goodies.images.support;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import sra.smalltalk.*;
import sra.jun.collections.support.JunCorrelation;
import sra.jun.goodies.files.*;
import sra.jun.goodies.images.streams.*;
import sra.jun.goodies.progress.JunProgress;

/**
 * JunImageToAscii class
 * 
 *  @author    nisinaka
 *  @created   2000/12/13 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunImageToAscii.java,v 6.0 2003/10/28 10:52:11 nisinaka Exp $
 */
public class JunImageToAscii extends StApplicationModel {
	protected Dimension divisionPoint = null;
	protected int asciiLength = 0;
	protected StValueHolder progressValue = new StValueHolder(0.0f);
	protected String baseName = null;
	//
	protected StMenuModel _menuModel;
	protected String _text;
	protected Cursor _cursor = Cursor.getDefaultCursor();

	/**
	 * Show the about dialog.
	 */
	protected static void About() {
		final int offsetX = 24;
		final int offsetY = 24;
		int maxX = 0;
		int maxY = 0;
		final StImage image = new StImage(LogoImage());
		int width = image.width();
		int height = image.height();
		maxX = Math.max(maxX, width);
		maxY += height;

		final Font font = new Font("dialog", Font.PLAIN, 16);
		final FontMetrics fontMetrics = SystemResourceSupport.getFontMetrics(font);
		final String[] text = LogoText();

		for (int i = 0; i < text.length; i++) {
			maxX = Math.max(maxX, fontMetrics.stringWidth(text[i]));
			maxY += fontMetrics.getAscent();
		}

		maxX += (offsetX * 2);
		maxY += (offsetY * 2);

		final int centerX = maxX / 2;

		//
		Frame aFrame = new Frame();
		aFrame.setTitle("Image To Ascii Picture");
		aFrame.setLayout(new BorderLayout());
		aFrame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				e.getWindow().dispose();
			}
		});

		final StImage _image = new StImage(LogoImage());
		Canvas newCanvas = new Canvas() {
			public void paint(Graphics g) {
				int x = centerX - (image.width() / 2);
				image.displayOn_at_(g, new Point(x, offsetY));
				g.setFont(font);

				int y = offsetY + image.height();

				for (int i = 0; i < text.length; i++) {
					x = centerX - (fontMetrics.stringWidth(text[i]) / 2);
					y += fontMetrics.getAscent();
					g.drawString(text[i], x, y);
				}

				return;
			}
		};

		newCanvas.setBounds(0, 0, maxX, maxY);
		newCanvas.setBackground(Color.white);
		aFrame.add(newCanvas);
		aFrame.pack();

		//
		Rectangle bounds = aFrame.getBounds();
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		int x = Math.max(0, (screenSize.width - bounds.width) / 2);
		int y = Math.max(0, (screenSize.height - bounds.height) / 2);
		aFrame.setLocation(x, y);
		aFrame.show();
	}

	/**
	 * Answer the default ascii length.
	 * 
	 * @return int
	 */
	protected static final int AsciiLength() {
		return 78;
	}

	/**
	 * Answer the default division point.
	 * 
	 * @return java.awt.Dimension
	 */
	protected static Dimension DivisionPoint() {
		Point point = SystemResourceSupport._getStringExtentFor_("A", Font());
		int width = Math.max(point.x / 2, 1);
		int height = Math.max(point.y / 2, 1);

		return new Dimension(width, height);
	}

	/**
	 * Answer the default font. This method corresponds to "textAttribues" of
	 * Jun for Smalltalk.
	 * 
	 * @return java.awt.Font
	 */
	protected static Font Font() {
		return new Font(FontName(), Font.PLAIN, FontSize());
	}

	/**
	 * Answer the default font name.
	 * 
	 * @return java.lang.String
	 */
	protected static final String FontName() {
		return "Courier";
	}

	/**
	 * Answer the default font size.
	 * 
	 * @return int
	 */
	protected static final int FontSize() {
		return 12;
	}

	/**
	 * Answer the image part of the logo.
	 * 
	 * @return java.awt.Image
	 */
	protected static final Image LogoImage() {
		return SystemResourceSupport.createImage("/sra/jun/goodies/images/support/JunImageToAsciiLogoImage.jpg");
	}

	/**
	 * Answer the text part of the logo.
	 * 
	 * @return java.lang.String[]
	 */
	protected static String[] LogoText() {
		String[] text = new String[] { "(1999.12.07)", "", "", "AOKI Atsushi", "", "aoki@sra.co.jp", "", "http://www.sra.co.jp/people/aoki/" };

		return text;
	}

	/**
	 * Answer the current cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public Cursor _getCursor() {
		return _cursor;
	}

	/**
	 * Answer the text.
	 * 
	 * @return java.lang.String
	 */
	public String _getText() {
		return _text;
	}

	/**
	 * Set the cursor.
	 * 
	 * @param newCursor java.awt.Cursor
	 */
	public void _setCursor(Cursor newCursor) {
		_cursor = newCursor;
		this.changed_($("cursor"));
	}

	/**
	 * Set the text.
	 * 
	 * @param newText java.lang.String
	 */
	public void _setText(String newText) {
		_text = newText;
		this.changed_($("text"));
	}

	/**
	 * Show the about dialog.
	 */
	public void aboutImageToAscii() {
		About();
	}

	/**
	 * Answer the current ascii length;
	 * 
	 * @return int
	 */
	public int asciiLength() {
		if (asciiLength <= 0) {
			asciiLength = AsciiLength();
		}

		return asciiLength;
	}

	/**
	 * Set the new ascii length.
	 * 
	 * @param newAsciiLength int
	 */
	public void asciiLength_(int newAsciiLength) {
		asciiLength = Math.max(1, newAsciiLength);
	}

	/**
	 * Do something with the progress value.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void compute_(StBlockClosure aBlock) {
		this.progressValue().compute_(aBlock);
	}

	/**
	 * Convert the image to ascii characters.
	 * 
	 * @param anImage sra.smalltalk.StImage
	 * 
	 * @return java.lang.String
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public String convert_(StImage anImage) {
		this.progress_(0.0f);

		int divisionWidth = this.divisionPoint().width;
		int divisionHeight = this.divisionPoint().height;
		int imageWidth = this.asciiLength() * divisionWidth;
		float aspect = (anImage.height() * imageWidth) / (float) (anImage.width() * divisionHeight);
		int imageHeight = Math.round(aspect) * divisionHeight;
		JunImageAdjuster anAdjuster = new JunImageAdjuster();
		final JunImageToAscii _this = this;
		anAdjuster.compute_(new StBlockClosure() {
			public Object value_(Object anObject) {
				_this.progress_(((Number) anObject).floatValue() / 2);

				return null;
			}
		});

		StImage theImage = anAdjuster.adjust_extent_(anImage, new Dimension(imageWidth, imageHeight));
		Object[][] characterTable = this.characterTable();
		float[] luminanceCollection = new float[characterTable.length];

		for (int i = 0; i < characterTable.length; i++) {
			luminanceCollection[i] = ((Number) characterTable[i][0]).floatValue();
		}

		this.progress_(0.5f);

		String result = null;
		BufferedWriter aWriter = null;
		StringWriter buffer = null;

		try {
			buffer = new StringWriter(1024);
			aWriter = new BufferedWriter(buffer);

			for (int y = 0; y < theImage.height(); y += divisionHeight) {
				for (int x = 0; x < theImage.width(); x += divisionWidth) {
					StImage smallImage = new StImage(divisionWidth, divisionHeight);
					smallImage.copy_from_in_rule_(smallImage.bounds(), new Point(x, y), theImage, StImage.Over);

					float imageLuminance = this.imageLuminance_(smallImage);
					Object[] anArray = characterTable[this.search_in_(imageLuminance, luminanceCollection)];
					StAssociation[] candidateCollection = (StAssociation[]) anArray[1];
					float[] imageFeature = this.imageFeature_(smallImage);
					StBlockClosure sortBlock = new StBlockClosure() {
						public Object value_value_(Object e1, Object e2) {
							return new Boolean(((Number) ((Object[]) e1)[1]).floatValue() < ((Number) ((Object[]) e2)[1]).floatValue());
						}
					};

					Character aCharacter;
					StSortedCollection aCollection = new StSortedCollection(sortBlock);

					for (int i = 0; i < candidateCollection.length; i++) {
						StAssociation anAssociation = candidateCollection[i];
						aCharacter = (Character) anAssociation.key();

						float[] feature = (float[]) anAssociation.value();
						float coefficientOfCorrelation = (float) (new JunCorrelation(imageFeature, feature)).coefficient();
						aCollection.add_(new Object[] { aCharacter, new Float(coefficientOfCorrelation)});
					}

					aCharacter = (Character) ((Object[]) aCollection.last())[0];
					aWriter.write(aCharacter.charValue());
				}

				aWriter.newLine();
				aWriter.flush();
				this.progress_(0.5f + ((float) y / (theImage.height() - 1) / 2));
			}

			aWriter.flush();
			result = buffer.toString();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		} finally {
			try {
				if (aWriter != null) {
					aWriter.close();
				}

				if (buffer != null) {
					buffer.close();
				}
			} catch (IOException e) {
			}
		}

		this.progress_(1.0f);

		return result;
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunImageToAsciiViewAwt(this);
		} else {
			return new JunImageToAsciiViewSwing(this);
		}
	}

	/**
	 * Answer the current division point.
	 * 
	 * @return java.awt.Dimension
	 */
	public Dimension divisionPoint() {
		if (divisionPoint == null) {
			divisionPoint = DivisionPoint();
		}

		return divisionPoint;
	}

	/**
	 * Set the new division point.
	 * 
	 * @param aPoint java.awt.Dimension
	 */
	public void divisionPoint_(Dimension aPoint) {
		divisionPoint = new Dimension(Math.max(aPoint.width, 3), Math.max(aPoint.height, 6));
	}

	/**
	 * Set the new division point.
	 * 
	 * @param aPoint java.awt.Dimension
	 */
	public void divisionPoint_(Point aPoint) {
		divisionPoint = new Dimension(Math.max(aPoint.x, 3), Math.max(aPoint.y, 6));
	}

	/**
	 * Set the ascii length.
	 */
	public void lengthImageToAscii() {
		String aString = String.valueOf(this.asciiLength());
		aString = StDialog.Request_("Ascii Length?", aString);
		if ((aString == null) || (aString.length() == 0)) {
			return;
		}

		try {
			int aNumber = Integer.parseInt(aString);
			this.asciiLength_(Math.max(10, Math.min(aNumber, 1000)));
		} catch (NumberFormatException e) {
			// just ignore.
		}
	}

	/**
	 * Answer a menu model.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 */
	public StMenuModel menuModel() {
		if (_menuModel == null) {
			_menuModel = this.defaultMenu();
		}

		return _menuModel;
	}

	/**
	 * Open an image file and convert it to an ascii string.
	 * 
	 * @return java.lang.String
	 */
	public String openImageToAscii() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType("jpeg files (*.jpg)", new String[] { ".jpg", ".JPG", ".jpeg", ".JPEG" })
		};
		File file = JunFileDialog.Request("Please select a JPEG file", fileTypes, fileTypes[0]);
		if (file == null) {
			return null;
		}

		// Read the image.
		Cursor previousCursor = this._getCursor();
		StImage image = null;

		try {
			final JunImageStream stream = JunJpegImageStream.On_(new FileInputStream(file));

			try {
				this._setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

				final JunProgress progress = new JunProgress();
				stream.compute_(new StBlockClosure() {
					public Object value_(Object anObject) {
						progress.value_(((Number) anObject).floatValue());

						return null;
					}
				});
				progress.message_("reading jpeg...");
				image = (StImage) progress.do_(new StBlockClosure() {
					public Object value() {
						try {
							return stream.nextImage();
						} catch (IOException e) {
							return null;
						}
					}
				});
			} finally {
				stream.close();
				this._setCursor(previousCursor);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		if (image == null) {
			System.err.println("Failed to open a JPEG file - " + file);

			return null;
		}

		// Convert the image to ascii characters.
		this.lengthImageToAscii();
		previousCursor = this._getCursor();

		String string = null;

		try {
			this._setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

			final JunProgress progress = new JunProgress();
			this.compute_(new StBlockClosure() {
				public Object value_(Object anObject) {
					progress.value_(((Number) anObject).floatValue());

					return null;
				}
			});
			progress.message_("converting jpeg...");

			final JunImageToAscii _this = this;
			final StImage _image = image;
			string = (String) progress.do_(new StBlockClosure() {
				public Object value() {
					return _this.convert_(_image);
				}
			});
		} finally {
			this._setCursor(previousCursor);
		}

		this._setText(string);

		// Change the title.
		Window[] windows = this.builder().windows();
		if (windows != null) {
			baseName = file.getName();
			int index = baseName.lastIndexOf('.');
			if (index >= 0) {
				baseName = baseName.substring(0, index);
			}

			for (int i = 0; i < windows.length; i++) {
				((Frame) windows[i]).setTitle(baseName);
			}
		}

		return string;
	}

	/**
	 * Quit the application.
	 */
	public void quitImageToAscii() {
		this.closeRequest();
	}

	/**
	 * Save the current ascii string to a file.
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public void saveImageToAscii() {
		String aString = this._getText();
		if ((aString == null) || (aString.length() == 0)) {
			return;
		}

		File file = new File((baseName == null) ? "filename.txt" : baseName + ".txt");
		file = JunFileDialog.RequestNewFilename("Please input a new file name", file);
		if (file == null) {
			return;
		}

		FileWriter aWriter = null;
		try {
			aWriter = new FileWriter(file);
			aWriter.write(aString);
			aWriter.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		} finally {
			try {
				if (aWriter != null) {
					aWriter.close();
				}
			} catch (IOException e) {
				throw new SmalltalkException(e);
			}
		}
	}

	/**
	 * Create features of all printable characters.
	 * 
	 * @return float[][]
	 */
	protected float[][] characterFeatures() {
		StImage[] characterImages = this.characterImages();
		float[][] characterFeatures = new float[characterImages.length][];

		for (int i = 0; i < characterImages.length; i++) {
			StImage anImage = characterImages[i];
			StImage shrinkImage = JunImageAdjuster.Adjust_extent_(anImage, this.divisionPoint());
			float[] imageFeature = this.imageFeature_(shrinkImage);
			characterFeatures[i] = imageFeature;
		}

		return characterFeatures;
	}

	/**
	 * Create all printable character images.
	 * 
	 * @return sra.smalltalk.StImage[]
	 */
	protected StImage[] characterImages() {
		char[] printableCharacters = this.printableCharacters();
		int size = printableCharacters.length;
		StImage[] images = new StImage[size];

		for (int i = 0; i < size; i++) {
			String str = String.valueOf(printableCharacters[i]);
			Point extent = SystemResourceSupport._getStringExtentFor_(str, Font());
			StImage image = new StImage(extent.x, extent.y);
			Graphics graphics = null;

			try {
				graphics = image.image().getGraphics();
				graphics.setColor(Color.white);
				graphics.fillRect(0, 0, extent.x, extent.y);
				graphics.setColor(Color.black);

				int y = SystemResourceSupport.getFontMetrics(Font()).getAscent();
				graphics.drawString(str, 0, y);
				image.image().flush();
			} finally {
				if (graphics != null) {
					graphics.dispose();
				}
			}

			// Convert the image to a bitmap.
			images[i] = this.convertToWhiteBlackPalette_(image);
		}

		return images;
	}

	/**
	 * Create luminances of all printable characters.
	 * 
	 * @return float[]
	 */
	protected float[] characterLuminances() {
		float maxLuminance = Float.NaN;
		float minLuminance = Float.NaN;
		StImage[] characterImages = this.characterImages();
		int size = characterImages.length;
		float[] characterLuminances = new float[size];

		for (int i = 0; i < size; i++) {
			StImage anImage = characterImages[i];
			float imageLuminance = this.imageLuminance_(anImage);

			if (Float.isNaN(maxLuminance) == true) {
				maxLuminance = imageLuminance;
			} else {
				maxLuminance = Math.max(maxLuminance, imageLuminance);
			}

			if (Float.isNaN(minLuminance) == true) {
				minLuminance = imageLuminance;
			} else {
				minLuminance = Math.min(minLuminance, imageLuminance);
			}

			characterLuminances[i] = imageLuminance;
		}

		float[] normalizedCharacterLuminances = new float[size];

		for (int i = 0; i < size; i++) {
			normalizedCharacterLuminances[i] = (characterLuminances[i] - minLuminance) / (maxLuminance - minLuminance);
		}

		return normalizedCharacterLuminances;
	}

	/**
	 * Create a character table which contains all information about
	 * characters.
	 * 
	 * @return java.lang.Object[][]
	 */
	protected Object[][] characterTable() {
		char[] printableCharacters = this.printableCharacters();
		float[] characterLuminances = this.characterLuminances();
		float[][] characterFeatures = this.characterFeatures();
		Hashtable aDictionary = new Hashtable();

		for (int i = 0; i < printableCharacters.length; i++) {
			Character aCharacter = new Character(printableCharacters[i]);
			Float luminance = new Float(characterLuminances[i]);
			float[] feature = characterFeatures[i];

			if (aDictionary.get(luminance) == null) {
				aDictionary.put(luminance, new Vector());
			}

			Vector collection = (Vector) aDictionary.get(luminance);
			collection.addElement(new StAssociation(aCharacter, feature));
		}

		StBlockClosure sortBlock = new StBlockClosure() {
			public Object value_value_(Object e1, Object e2) {
				return new Boolean(((Number) ((Object[]) e1)[0]).floatValue() < ((Number) ((Object[]) e2)[0]).floatValue());
			}
		};

		StSortedCollection sortedCollection = new StSortedCollection(sortBlock);
		Enumeration keys = aDictionary.keys();

		while (keys.hasMoreElements()) {
			Number aNumber = (Number) keys.nextElement();
			Vector aVector = (Vector) aDictionary.get(aNumber);
			StAssociation[] anArray = new StAssociation[aVector.size()];
			aVector.copyInto(anArray);
			sortedCollection.add_(new Object[] { aNumber, anArray });
		}

		Object[] anArray = sortedCollection._asArray();
		Object[][] characterTable = new Object[anArray.length][];

		for (int i = 0; i < anArray.length; i++) {
			characterTable[i] = (Object[]) anArray[i];
		}

		return characterTable;
	}

	/**
	 * Convert a image to white-black palette.
	 * 
	 * @param anImage sra.smalltalk.StImage
	 * 
	 * @return sra.smalltalk.StImage
	 */
	protected StImage convertToWhiteBlackPalette_(StImage anImage) {
		int width = anImage.width();
		int height = anImage.height();
		StImage whiteBlackImage = new StImage(width, height);

		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				Point point = new Point(x, y);
				Color color = anImage.valueAtPoint_(point);

				if (StColorValue._GetLuminance(color) > 0.5) {
					whiteBlackImage.valueAtPoint_put_(point, Color.white);
				} else {
					whiteBlackImage.valueAtPoint_put_(point, Color.black);
				}
			}
		}

		return whiteBlackImage;
	}

	/**
	 * Create the default menubar.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 */
	protected StMenuModel defaultMenu() {
		StMenu fileMenu = new StMenu("File");
		fileMenu.add(createMenuItem("Open...", "openImageToAscii"));
		fileMenu.addSeparator();
		fileMenu.add(createMenuItem("Save...", "saveImageToAscii"));
		fileMenu.addSeparator();
		fileMenu.add(createMenuItem("Quit", "quitImageToAscii"));

		//
		StMenu helpMenu = new StMenu("Help");
		helpMenu.add(createMenuItem("About...", "aboutImageToAscii"));

		//
		StMenuModel rootMenu = new StMenuModel();
		rootMenu.add(fileMenu);
		rootMenu.add(helpMenu);

		return rootMenu;
	}

	/**
	 * Answer the array of luminances of all pixels.
	 * 
	 * @param anImage sra.smalltalk.StImage
	 * 
	 * @return float[]
	 */
	protected float[] imageFeature_(StImage anImage) {
		int width = anImage.width();
		int height = anImage.height();
		float[] imageFeature = new float[width * height];

		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				Color color = anImage.valueAtPoint_(new Point(x, y));
				imageFeature[x + (y * width)] = (float) StColorValue._GetLuminance(color);
			}
		}

		return imageFeature;
	}

	/**
	 * Answer the luminance of the image.
	 * 
	 * @param anImage sra.smalltalk.StImage
	 * 
	 * @return float
	 */
	protected float imageLuminance_(StImage anImage) {
		float[] imageFeature = this.imageFeature_(anImage);
		float totalValue = 0;

		for (int i = 0; i < imageFeature.length; i++) {
			totalValue += imageFeature[i];
		}

		float imageLuminance = totalValue / imageFeature.length;

		return imageLuminance;
	}

	/**
	 * Answer the array of the printable characters.
	 * 
	 * @return char[]
	 */
	protected char[] printableCharacters() {
		boolean[] table = new boolean[256];

		for (int i = 0; i < 256; i++) {
			table[i] = false;
		}

		table[' '] = true;
		table['!'] = true;
		table['"'] = true;
		table['#'] = true;
		table['$'] = true;
		table['%'] = true;
		table['&'] = true;
		table['\''] = true;
		table['('] = true;
		table[')'] = true;
		table['*'] = true;
		table['+'] = true;
		table[','] = true;
		table['-'] = true;
		table['.'] = true;
		table['/'] = true;
		table[':'] = true;
		table[';'] = true;

		// table['<'] = true;
		table['='] = true;

		// table['>'] = true;
		table['?'] = true;
		table['@'] = true;
		table['['] = true;

		// table['\\'] = true;
		table[']'] = true;
		table['^'] = true;
		table['_'] = true;
		table['`'] = true;
		table['{'] = true;
		table['|'] = true;
		table['}'] = true;
		table['~'] = true;

		//
		int count = 0;
		char[] collection = new char[256];

		for (int i = 0; i < 256; i++) {
			if (('0' <= i && i <= '9') || ('A' <= i && i <= 'Z') || ('a' <= i && i <= 'z')) {
				table[i] = true;
			}

			if (table[i] == true) {
				collection[count] = (char) i;
				count++;
			}
		}

		char[] result = new char[count];
		System.arraycopy(collection, 0, result, 0, count);

		return result;
	}

	/**
	 * Set the progress value.
	 * 
	 * @param normalizedNumber float
	 */
	protected void progress_(float normalizedNumber) {
		if ((0 <= normalizedNumber) && (normalizedNumber <= 1)) {
			float truncatedValue = Math.round(normalizedNumber / 0.005) * 0.005f;

			if (this.progressValue()._floatValue() != truncatedValue) {
				this.progressValue().value_(truncatedValue);
			}
		}
	}

	/**
	 * Answer the StValueHolder for the progress value.
	 * 
	 * @return sra.smalltalk.StValueHolder
	 */
	protected StValueHolder progressValue() {
		return progressValue;
	}

	/**
	 * Answer an appropriate index of the specified number in the sorted
	 * numbers.
	 * 
	 * @param aNumber float
	 * @param sortedNumbers float[]
	 * 
	 * @return int
	 */
	protected int search_in_(float aNumber, float[] sortedNumbers) {
		int min = 0;
		int max = sortedNumbers.length - 1;
		int index = (min + max) / 2;
		float it = sortedNumbers[index];

		while ((aNumber != it) && (min <= max)) {
			if (aNumber < it) {
				max = index - 1;
			} else {
				min = index + 2;
			}

			index = (min + max) / 2;
			it = sortedNumbers[index];
		}

		if (aNumber != it) {
			index = Math.max(1, Math.min(Math.max(min, max), sortedNumbers.length - 1));
		}

		return index;
	}

	/**
	 * Answer the window title.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "Image to Ascii Picture";
	}
}
