package sra.jun.goodies.images.support;

import java.awt.*;
import sra.smalltalk.*;
import sra.jun.system.support.JunTestExamples;

/**
 * JunImageToAsciiTestExamples class
 * 
 *  @author    nisinaka
 *  @created   2000/12/13 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunImageToAsciiTestExamples.java,v 6.0 2003/10/28 10:52:11 nisinaka Exp $
 */
public class JunImageToAsciiTestExamples extends JunTestExamples {

	/**
	 * Example: Open the application
	 * 
	 * @return boolean
	 */
	public static boolean Example() {
		JunImageToAscii imageToAscii = new JunImageToAscii();
		imageToAscii.open();

		return true;
	}

	/**
	 * Example: Open the application
	 * 
	 * @return boolean
	 */
	public static boolean Example_Awt() {
		StApplicationModel.SetDefaultViewMode(StApplicationModel.VIEW_AWT);

		return Example();
	}

	/**
	 * Example: Open the application
	 * 
	 * @return boolean
	 */
	public static boolean Example_Swing() {
		StApplicationModel.SetDefaultViewMode(StApplicationModel.VIEW_SWING);

		return Example();
	}

	/**
	 * Example0: Show characters' luminances.
	 * 
	 * @return boolean
	 */
	public static boolean Example0() {
		JunImageToAscii imageToAscii = new JunImageToAscii();
		Point point = SystemResourceSupport._getStringExtentFor_("A", JunImageToAscii.Font());
		int imageWidth = point.x;
		int imageHeight = point.y;
		Object[][] characterTable = imageToAscii.characterTable();
		int maxElements = 0;

		for (int i = 0; i < characterTable.length; i++) {
			StAssociation[] elements = (StAssociation[]) characterTable[i][1];
			maxElements = Math.max(maxElements, elements.length);
		}

		Point extent = new Point((maxElements * imageWidth * 2) + 100, characterTable.length * imageHeight);
		Image image = new StImage(extent.x, extent.y).image();
		Graphics graphics = null;

		try {
			graphics = image.getGraphics();

			if (graphics instanceof Graphics2D) {
				((Graphics2D) graphics).setBackground(Color.white);
			}

			float brightness = 5.0f / 6.0f;
			graphics.setColor(new Color(brightness, brightness, brightness));
			graphics.fillRect(0, 0, extent.x, extent.y);

			int y = imageHeight - 2;

			for (int i = 0; i < characterTable.length; i++) {
				float luminance = ((Number) characterTable[i][0]).floatValue();
				luminance = Math.round(luminance * 1000) / 1000.0f;
				graphics.setColor(Color.black);
				graphics.drawString(String.valueOf(luminance), 4, y);

				StAssociation[] associations = (StAssociation[]) characterTable[i][1];
				int x = 100;

				for (int j = 0; j < associations.length; j++) {
					Character aCharacter = (Character) associations[j].key();
					graphics.drawString(aCharacter.toString(), x, y);
					x += imageWidth;

					float[] feature = (float[]) associations[j].value();
					int divisionWidth = imageToAscii.divisionPoint().width;
					int divisionHeight = imageToAscii.divisionPoint().height;
					StImage featureImage = new StImage(divisionWidth, divisionHeight);

					for (int yy = 0; yy < divisionHeight; yy++) {
						for (int xx = 0; xx < divisionWidth; xx++) {
							int index = (divisionWidth * yy) + xx;
							brightness = feature[index];

							Color color = new Color(brightness, brightness, brightness);
							featureImage.valueAtPoint_put_(new Point(xx, yy), color);
						}
					}

					int ix = x + ((imageWidth - divisionWidth) / 2);
					int iy = y - divisionHeight;
					featureImage.displayOn_at_(graphics, new Point(ix, iy));
					x += imageWidth;
				}

				y += imageHeight;
			}

			image.flush();
		} finally {
			if (graphics != null) {
				graphics.dispose();
			}
		}

		JunImageAdjuster.Show_(new StImage(image));

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args an array of command-line arguments
	 */
	public static void main(java.lang.String[] args) {
		new JunImageToAsciiTestExamples();
	}
}
