package sra.jun.goodies.lisp;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import sra.smalltalk.*;

/**
 * JunLispInterpreterViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2003/05/07 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun490 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunLispInterpreterViewSwing.java,v 6.0 2003/10/28 10:52:11 nisinaka Exp $
 */
public class JunLispInterpreterViewSwing extends StViewJPanel implements JunLispInterpreterView {

	protected JTextArea textValueView;
	protected JScrollPane textValueViewScrollPane;
	protected JTextArea textCollectorView;
	protected JScrollPane textCollectorViewScrollPane;

	/**
	 * Create a new instance of JunLispInterpreterViewSwing and initialize it.
	 *
	 * @param aLispInterpreter sra.jun.goodies.lisp.JunLispInterpreter
	 */
	public JunLispInterpreterViewSwing(JunLispInterpreter aLispInterpreter) {
		super(aLispInterpreter);
	}

	/**
	 * Answer my model as JunLispInterpreter.
	 *
	 * @return sra.jun.goodies.lisp.JunLispInterpreter
	 */
	public JunLispInterpreter getLispInterpreter() {
		return (JunLispInterpreter) this.model();
	}

	/**
	 * Answer the selected text to 'do it'.
	 *
	 * @return java.lang.String
	 */
	public String getSelectedText() {
		return this.textValueView().getSelectedText();
	}

	/**
	 * Set the bounds of myself and my components.
	 *
	 * @param x int
	 * @param y int
	 * @param width int
	 * @param height int
	 */
	public void setBounds(int x, int y, int width, int height) {
		super.setBounds(x, y, width, height);

		this.textValueViewScrollPane().setBounds(1, 1, width - 2, (int) (height * 0.3) - 2);
		this.textCollectorViewScrollPane().setBounds(1, (int) (height * 0.3), width - 2, (int) (height * 0.7) - 1);
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(null);
		this.add(this.textValueViewScrollPane());
		this.add(this.textCollectorViewScrollPane());
		this.setPreferredSize(new Dimension(350, 400));
	}

	/**
	 * Answer the view of my text collector.
	 *
	 * @return javax.swing.JTextArea
	 */
	protected JTextArea textCollectorView() {
		if (textCollectorView == null) {
			textCollectorView = new JTextArea("", 0, 0);
			textCollectorView.setBackground(Color.lightGray);

			ByteArrayOutputStream aByteArrayOutputStream = new ByteArrayOutputStream() {
				public synchronized void write(byte b[], int off, int len) {
					super.write(b, off, len);
					textCollectorView.append(new String(b, off, len));
				}
			};
			this.getLispInterpreter().textCollector_(new PrintStream(aByteArrayOutputStream));
		}
		return textCollectorView;
	}

	/**
	 * Answer the scroll pane for the text collector view.
	 *
	 * @return javax.swing.JScrollPane
	 */
	protected JScrollPane textCollectorViewScrollPane() {
		if (textCollectorViewScrollPane == null) {
			textCollectorViewScrollPane = new JScrollPane(this.textCollectorView());
			textCollectorViewScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			textCollectorViewScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		}
		return textCollectorViewScrollPane;
	}

	/**
	 * Answer the view of my text value.
	 *
	 * @return javax.swing.JTextArea
	 */
	protected JTextArea textValueView() {
		if (textValueView == null) {
			textValueView = new JTextArea((String) this.getLispInterpreter().textValue().value(), 0, 0);

			StMenu questionMenu = this.getLispInterpreter().questionMenu();
			StMenuItem doIt = questionMenu.getItem("do it");
			doIt.setActionArgument(this);

			final JPopupMenu aPopupMneu = this._createPopupMenu(questionMenu);
			textValueView.add(aPopupMneu);
			textValueView.addMouseListener(new MouseAdapter() {
				public void mousePressed(MouseEvent e) {
					if (e.isMetaDown()) {
						aPopupMneu.show(textValueView, e.getX(), e.getY());
					}
				}
			});

			this.getLispInterpreter().textValue().compute_(new StBlockClosure() {
				public Object value_(Object newValue) {
					textValueView.setText((String) newValue);
					return null;
				}
			});
		}
		return textValueView;
	}

	/**
	 * Answer the scroll pane for the text value view.
	 *
	 * @return javax.swing.JScrollPane
	 */
	protected JScrollPane textValueViewScrollPane() {
		if (textValueViewScrollPane == null) {
			textValueViewScrollPane = new JScrollPane(this.textValueView());
			textValueViewScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			textValueViewScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		}
		return textValueViewScrollPane;
	}
}
