package sra.jun.goodies.lisp;

import java.util.*;
import sra.smalltalk.*;

/**
 * JunLispTable class
 * 
 *  @author    ASTI Beijing
 *  @created   1998/12/09 (by ASTI Beijing)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunLispTable.java,v 6.0 2003/10/28 10:52:11 nisinaka Exp $
 */
public class JunLispTable extends StObject {
	/** The hashtable which holds the properties. */
	protected Hashtable properties = null;

	/**
	 * Create a new instance of JunLispTable.
	 */
	public JunLispTable() {
		super();
		this.initialize();
	}

	/**
	 * Create a new property which is specified with the symbol.
	 * 
	 * @param symbol java.lang.Object
	 */
	public void add_(Object symbol) {
		this.intern_(symbol);
	}

	/**
	 * Answer the property which corresponds to the object.
	 * 
	 * @param symbol sra.smalltalk.StSymbol
	 * 
	 * @return java.lang.Object
	 */
	public Object at_(Object symbol) {
		return this.getprop_key_(symbol, $("apval"));
	}

	/**
	 * Set the value as a property which is specified with the symbol.
	 * 
	 * @param symbol java.lang.Object
	 * @param value java.lang.Object
	 * 
	 * @return java.lang.Object
	 */
	public Object at_put_(Object symbol, Object value) {
		this.intern_(symbol);
		return this.putprop_key_value_(symbol, $("apval"), value);
	}

	/**
	 * Answer the property which is specified with the identifier and the key.
	 * 
	 * @param identifier java.lang.Object
	 * @param key java.lang.Object
	 * 
	 * @return java.lang.Object
	 */
	public Object getprop_key_(Object identifier, Object key) {
		Hashtable property = (Hashtable) properties.get(identifier);
		if (property == null) {
			this.errorSymbolNotFound();
			return null;
		}

		return property.get(key);
	}

	/**
	 * Answer the array of all identifiers.
	 * 
	 * @return java.lang.Object[]
	 */
	public Object[] identifiers() {
		StBlockClosure sortBlock = new StBlockClosure() {
			public Object value_value_(Object e1, Object e2) {
				return new Boolean(e1.toString().compareTo(e2.toString()) < 0);
			}
		};

		StSortedCollection collection = new StSortedCollection(sortBlock);
		Enumeration keys_enum = properties.keys();
		while (keys_enum.hasMoreElements()) {
			collection.add_(keys_enum.nextElement());
		}

		return collection._asArray();
	}

	/**
	 * Initialize the receiver.
	 */
	public void initialize() {
		properties = new Hashtable();
	}

	/**
	 * Set the property which is specified with the identifier and the key.
	 * 
	 * @param identifier java.lang.Object
	 * @param key java.lang.Object
	 * @param value DOCUMENT ME!
	 * 
	 * @return java.lang.Object
	 */
	public Object putprop_key_value_(Object identifier, Object key, Object value) {
		Hashtable property = (Hashtable) properties.get(identifier);
		if (property == null) {
			this.errorSymbolNotFound();
			return null;
		}

		return property.put(key, value);
	}

	/**
	 * Remove the property which is specified with the symbol.
	 * 
	 * @param symbol java.lang.Object
	 * 
	 * @return java.lang.Object
	 */
	public Object remove_(Object symbol) {
		return properties.remove(symbol);
	}

	/**
	 * Remove the property which is specified with the identifier and the key.
	 * 
	 * @param identifier java.lang.Object
	 * @param key java.lang.Object
	 * 
	 * @return java.lang.Object
	 */
	public Object remprop_key_(Object identifier, Object key) {
		Hashtable property = (Hashtable) properties.get(identifier);
		if (property == null) {
			this.errorSymbolNotFound();
			return null;
		}

		return property.remove(key);
	}

	/**
	 * Throw a runtime exception to nofity an error.
	 */
	protected void errorSymbolNotFound() {
		throw SmalltalkException.Error("symbol not found");
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param symbol java.lang.Object
	 * 
	 * @return java.lang.Object
	 */
	protected Object intern_(Object symbol) {
		if (properties.containsKey(symbol) == false) {
			properties.put(symbol, new Hashtable());
		}

		return symbol;
	}
}
