package sra.jun.goodies.milliseconds;

import java.awt.Point;
import java.awt.event.MouseEvent;
import sra.smalltalk.*;
import sra.jun.system.framework.JunDialog;

/**
 * JunMillisecondController class
 * 
 *  @author    nisinaka
 *  @created   2003/05/28 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun473 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMillisecondController.java,v 6.0 2003/10/28 10:52:11 nisinaka Exp $
 */
public class JunMillisecondController extends StController {

	protected boolean _mousePressed;

	/**
	 * Answer my model as JunMillisecondModel.
	 *
	 * @return sra.jun.goodies.milliseconds.JunMillisecondModel
	 */
	public JunMillisecondModel getMillisecondModel() {
		return (JunMillisecondModel) this.model();
	}

	/**
	 * Answer my view as JunMillisecondView.
	 *
	 * @return sra.jun.goodies.milliseconds.JunMillisecondView
	 */
	public JunMillisecondView getMillisecondView() {
		return (JunMillisecondView) this.view();
	}

	/**
	 * Invoked when a mouse button has been pressed on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 */
	public void mousePressed(MouseEvent e) {
		if (e.isMetaDown()) {
			// yellow button activity
			this.view()._showPopupMenu(e.getX(), e.getY());
		} else {
			// red button activity
			this.frameActivity();
		}
	}

	/**
	 * Invoked when a mouse button has been released on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 */
	public void mouseReleased(MouseEvent e) {
		_mousePressed = false;
		synchronized (this) {
			this.notify(); // to stop the thread for auto repeating.
		}
	}

	/**
	 * Activity for a button.
	 *
	 * @param anInfo sra.jun.goodies.milliseconds.JunMillisecondView.FrameInfo
	 */
	protected void buttonActivity_(JunMillisecondView.FrameInfo aFrameInfo) {
		StSymbol subkey = aFrameInfo.subkey();
		if (subkey == $("hourDown")) {
			this.getMillisecondModel().hourDown();
		} else if (subkey == $("hourUp")) {
			this.getMillisecondModel().hourUp();
		} else if (subkey == $("minuteDown")) {
			this.getMillisecondModel().minuteDown();
		} else if (subkey == $("minuteUp")) {
			this.getMillisecondModel().minuteUp();
		} else if (subkey == $("secondDown")) {
			this.getMillisecondModel().secondDown();
		} else if (subkey == $("secondUp")) {
			this.getMillisecondModel().secondUp();
		} else if (subkey == $("msecDown")) {
			this.getMillisecondModel().msecDown();
		} else if (subkey == $("msecUp")) {
			this.getMillisecondModel().msecUp();
		}
	}

	/**
	 * Activity for the frame.
	 */
	protected void frameActivity() {
		Point point = this.cursorPoint();
		JunMillisecondView.FrameInfo[] info = this.getMillisecondView().frames();
		for (int i = 0; i < info.length; i++) {
			StRectangle box = info[i].box();
			if (box.containsPoint_(point)) {
				StSymbol key = info[i].key();
				if (key == $("hours")) {
					this.hoursActivity_(info[i]);
				} else if (key == $("minutes")) {
					this.minutesActivity_(info[i]);
				} else if (key == $("seconds")) {
					this.secondsActivity_(info[i]);
				} else if (key == $("msecs")) {
					this.msecsActivity_(info[i]);
				} else if (key == $("label")) {
					this.labelActivity_(info[i]);
				} else if (key == $("button")) {
					this.buttonActivity_(info[i]);

					_mousePressed = true;
					final JunMillisecondController this_ = this;
					final JunMillisecondView.FrameInfo info_ = info[i];
					(new Thread() {
						public void run() {
							try {
								synchronized (this_) {
									this_.wait(1000);
								}
								while (_mousePressed) {
									this_.buttonActivity_(info_);
									Thread.sleep(50);
								}
							} catch (InterruptedException e) {
							}
						}
					}).start();
				}
			}
		}
	}

	/**
	 * Activity for the hours.
	 *
	 * @param anInfo sra.jun.goodies.milliseconds.JunMillisecondView.FrameInfo
	 */
	protected void hoursActivity_(JunMillisecondView.FrameInfo aFrameInfo) {
		String aString = JunDialog.Request_("Hours?", String.valueOf(this.getMillisecondModel().hours()));
		if (aString.length() == 0) {
			return;
		}

		StringBuffer digits = new StringBuffer();
		for (int i = 0; i < aString.length(); i++) {
			char c = aString.charAt(i);
			if (Character.isDigit(c)) {
				digits.append(c);
			}
		}
		aString = digits.toString();
		if (aString.length() == 0) {
			return;
		}

		this.getMillisecondModel().hours_(Integer.parseInt(aString));
	}

	/**
	 * Activity for a label.
	 *
	 * @param anInfo sra.jun.goodies.milliseconds.JunMillisecondView.FrameInfo
	 */
	protected void labelActivity_(JunMillisecondView.FrameInfo aFrameInfo) {
		// Nothing to do.
	}

	/**
	 * Activity for the minutes.
	 *
	 * @param anInfo sra.jun.goodies.milliseconds.JunMillisecondView.FrameInfo
	 */
	protected void minutesActivity_(JunMillisecondView.FrameInfo aFrameInfo) {
		String aString = JunDialog.Request_("Minutes?", String.valueOf(this.getMillisecondModel().minutes()));
		if (aString.length() == 0) {
			return;
		}

		StringBuffer digits = new StringBuffer();
		for (int i = 0; i < aString.length(); i++) {
			char c = aString.charAt(i);
			if (Character.isDigit(c)) {
				digits.append(c);
			}
		}
		aString = digits.toString();
		if (aString.length() == 0) {
			return;
		}

		this.getMillisecondModel().minutes_(Integer.parseInt(aString));
	}

	/**
	 * Activity for the msecs.
	 *
	 * @param anInfo sra.jun.goodies.milliseconds.JunMillisecondView.FrameInfo
	 */
	protected void msecsActivity_(JunMillisecondView.FrameInfo aFrameInfo) {
		String aString = JunDialog.Request_("Milliseconds?", String.valueOf(this.getMillisecondModel().msecs()));
		if (aString.length() == 0) {
			return;
		}

		StringBuffer digits = new StringBuffer();
		for (int i = 0; i < aString.length(); i++) {
			char c = aString.charAt(i);
			if (Character.isDigit(c)) {
				digits.append(c);
			}
		}
		aString = digits.toString();
		if (aString.length() == 0) {
			return;
		}

		this.getMillisecondModel().msecs_(Integer.parseInt(aString));
	}

	/**
	 * Activity for the seconds.
	 *
	 * @param anInfo sra.jun.goodies.milliseconds.JunMillisecondView.FrameInfo
	 */
	protected void secondsActivity_(JunMillisecondView.FrameInfo aFrameInfo) {
		String aString = JunDialog.Request_("Seconds?", String.valueOf(this.getMillisecondModel().seconds()));
		if (aString.length() == 0) {
			return;
		}

		StringBuffer digits = new StringBuffer();
		for (int i = 0; i < aString.length(); i++) {
			char c = aString.charAt(i);
			if (Character.isDigit(c)) {
				digits.append(c);
			}
		}
		aString = digits.toString();
		if (aString.length() == 0) {
			return;
		}

		this.getMillisecondModel().seconds_(Integer.parseInt(aString));
	}
}
