package sra.jun.goodies.milliseconds;

import sra.smalltalk.StBlockClosure;

/**
 * JunMillisecondDialogTestExamples class
 * 
 *  @author    nisinaka
 *  @created   2003/05/28 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun473 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMillisecondDialogTestExamples.java,v 6.0 2003/10/28 10:52:11 nisinaka Exp $
 */
public class JunMillisecondDialogTestExamples extends sra.jun.system.support.JunTestExamples {

	/**
	 * Example1: Use JunMillisecondDialog to request for a millisecond value.
	 * 
	 * @return boolean
	 */
	public static boolean Example1() {
		long milliseconds = JunMillisecondDialog.Request();
		if (milliseconds < 0) {
			return false;
		}

		String string = JunMillisecondModel.ConvertMillisecondsToString_(milliseconds);
		System.out.println(string);

		return true;
	}

	/**
	 * Example2: Use JunMillisecondDialog to request for a millisecond value.
	 * 
	 * @return boolean
	 */
	public static boolean Example2() {
		long milliseconds = JunMillisecondDialog.Request("Input time.");
		if (milliseconds < 0) {
			return false;
		}

		String string = JunMillisecondModel.ConvertMillisecondsToString_(milliseconds);
		System.out.println(string);

		return true;
	}

	/**
	 * Example3: Use JunMillisecondDialog to request for a millisecond value.
	 * 
	 * @return boolean
	 */
	public static boolean Example3() {
		long milliseconds = JunMillisecondModel.ConvertStringToMilliseconds_("100:00:00.000");
		milliseconds = JunMillisecondDialog.Request("Input time.", milliseconds);
		if (milliseconds < 0) {
			return false;
		}

		String string = JunMillisecondModel.ConvertMillisecondsToString_(milliseconds);
		System.out.println(string);

		return true;
	}

	/**
	 * Example4: Use JunMillisecondDialog to request for a millisecond value.
	 * 
	 * @return boolean
	 */
	public static boolean Example4() {
		long milliseconds = JunMillisecondModel.ConvertStringToMilliseconds_("100:00:00.000");
		milliseconds = JunMillisecondDialog.Request("Input time.", milliseconds, new StBlockClosure() {
			public Object value_(Object model) {
				System.out.println("Accept button was pressed.");
				System.out.println(model);
				return Boolean.TRUE;
			}
		}, new StBlockClosure() {
			public Object value_(Object model) {
				System.out.println("Cancel button was pressed.");
				System.out.println(model);
				return Boolean.TRUE;
			}
		});
		if (milliseconds < 0) {
			return false;
		}

		String string = JunMillisecondModel.ConvertMillisecondsToString_(milliseconds);
		System.out.println(string);

		return true;
	}

	/**
	 * Example5: Use JunMillisecondDialog to request for a millisecond value.
	 * 
	 * @return boolean
	 */
	public static boolean Example5() {
		final long maxValue = (new JunMillisecondModel(100, 0, 0, 10)).milliseconds();
		final long minValue = (new JunMillisecondModel(99, 59, 59, 990)).milliseconds();
		long milliseconds = JunMillisecondModel.ConvertStringToMilliseconds_("100:00:00.000");
		milliseconds = JunMillisecondDialog.Request("Input time.", milliseconds, new StBlockClosure() {
			public Object value_value_(Object oldValue, Object newValue) {
				long value = ((Number) newValue).longValue();
				if (minValue <= value && value <= maxValue) {
					return newValue;
				} else {
					return oldValue;
				}
			}
		});
		if (milliseconds < 0) {
			return false;
		}

		String string = JunMillisecondModel.ConvertMillisecondsToString_(milliseconds);
		System.out.println(string);

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 */
	public static void main(String[] args) {
		new JunMillisecondDialogTestExamples();
	}
}
