package sra.jun.goodies.movie;

import java.awt.*;
import sra.qt4jun.JunQTInterface;
import sra.smalltalk.*;

/**
 * JunImagesToMovie class
 * 
 *  @author    Hoshi Takanori
 *  @created   2002/10/24 (by Hoshi Takanori)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunImagesToMovie.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunImagesToMovie {
    protected static final Color DEFAULT_BACKGROUND = Color.white;
    protected static final int TIME_SCALE = JunQTInterface.kVideoTimeScale;

    protected String filename;
    protected Dimension extent;
    protected Color background;
    protected int movie;
    protected int track;
    protected int media;

    /**
     * Constructor.
     *
     * @param aFilename java.lang.String
     * @param anExtent java.awt.Dimension
     */
    public JunImagesToMovie(String aFilename, Dimension anExtent) {
        filename = aFilename;
        extent = anExtent;
        background = DEFAULT_BACKGROUND;
    }

    /**
     * Add an image.
     *
     * @param image sra.smalltalk.StImage
     */
    public void add_(StImage image) {
        this.add_milliseconds_(image, 33);
    }

    /**
     * Add an image with duration.
     *
     * @param image sra.smalltalk.StImage
     * @param duration int
     */
    public void add_duration_(StImage image, int duration) {
        this.addPixels_width_height_duration_(
                image.getPixels(), image.width(), image.height(), duration);
    }

    /**
     * Add an image with milliseconds.
     *
     * @param image sra.smalltalk.StImage
     * @param milliseconds int
     */
    public void add_milliseconds_(StImage image, int milliseconds) {
        this.add_duration_(image, milliseconds * TIME_SCALE / 1000);
    }

    /**
     * Add pixels as an image with duration.
     *
     * @param pixels int[]
     * @param width int
     * @param height int
     * @param duration int
     */
    public void addPixels_width_height_duration_(
            int[] pixels, int width, int height, int duration) {
        JunQTInterface interface_ = JunQTInterface.Interface();
        interface_.qtAddMediaSampleImage(media, pixels, width, height,
                extent.width, extent.height, interface_.codecNormalQuality,
                interface_.kAnimationCodecType, duration);
    }

    /**
     * Add pixels as an image with milliseconds.
     *
     * @param pixels int[]
     * @param width int
     * @param height int
     * @param milliseconds int
     */
    public void addPixels_width_height_milliseconds_(
            int[] pixels, int width, int height, int milliseconds) {
        this.addPixels_width_height_duration_(
                pixels, width, height, milliseconds * TIME_SCALE / 1000);
    }

    /**
     * Answer an adjusted image.
     *
     * @param image sra.smalltalk.StImage
     * @return sra.smalltalk.StImage
     */
    public StImage adjustedImage(StImage image) {
        return image;
    }

    /**
     * Do the job.
     *
     * @param doBlock sra.smalltalk.StBlockClosure
     */
    public void do_(StBlockClosure doBlock) {
        JunQTInterface interface_ = JunQTInterface.Interface();
        int err;

        int[] array = interface_.qtCreateMovieFile(filename,
                interface_.sigMoviePlayer, interface_.smSystemScript,
                interface_.createMovieFileDeleteCurFile
                        + interface_.createMovieFileDontCreateResFile);

        if (array != null && array[0] == interface_.noErr) {
            int resRefNum = array[1];
            movie = array[2];

            track = interface_.qtNewMovieTrack(movie,
                    interface_.qtFixRatio(extent.width, 1),
                    interface_.qtFixRatio(extent.height, 1),
                    interface_.kNoVolume);

            if (interface_.qtGetMoviesError() == interface_.noErr) {
                media = interface_.qtNewTrackMedia(
                        track, interface_.VideoMediaType, TIME_SCALE, 0, 0);

                if (interface_.qtGetMoviesError() == interface_.noErr) {
                    err = interface_.qtBeginMediaEdits(media);

                    if (err == interface_.noErr) {
                        if (doBlock.numArgs() == 0) {
                            doBlock.value();
                        } else {
                            doBlock.value_(this);
                        }

                        err = interface_.qtEndMediaEdits(media);

                        if (err == interface_.noErr) {
                            int duration = interface_.qtGetMediaDuration(media);
                            err = interface_.qtInsertMediaIntoTrack(
                                    track, 0, 0, duration, interface_.fixed1);
                        }
                    }
                }
            }

            err = interface_.qtAddMovieResource(
                    movie, resRefNum, interface_.movieInDataForkResID, null);
            err = interface_.qtCloseMovieFile(resRefNum);

            interface_.qtDisposeMovie(movie);
        }
    }
}
