package sra.jun.goodies.movie;

import java.awt.*;
import java.io.*;
import sra.qt4jun.*;
import sra.smalltalk.*;

/**
 * JunMovieHandle class
 * 
 *  @author    NISHIHARA Satoshi
 *  @created   2001/01/12 (by NISHIHARA Satoshi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMovieHandle.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunMovieHandle extends StModel {
	protected File filename;
	protected int movie;
	protected Dimension originalExtent;
	protected boolean dontRecover;

	/**
	 * ConvertToRelative_
	 * 
	 * @param aFilename java.lang.String
	 * 
	 * @return java.lang.String
	 */
	public static String ConvertToRelative_(String aFilename) {
		return aFilename;
	}

	/**
	 * get aJunMovieHandle with file.
	 * 
	 * @param aFilename java.io.File
	 * 
	 * @return sra.jun.goodies.movie.JunMovieHandle
	 */
	public static JunMovieHandle Filename_(File aFilename) {
		if ((aFilename == null) || !aFilename.exists()) {
			return null;
		}

		JunMovieHandle aMovieHandle = new JunMovieHandle();
		aMovieHandle.setFilename_(aFilename);
		aMovieHandle.setMovie_(aMovieHandle._interface().openMovie(aFilename));

		return aMovieHandle;
	}

	/**
	 * get aJunMovieHandle with file.
	 * 
	 * @param aFilename java.lang.String
	 * 
	 * @return sra.jun.goodies.movie.JunMovieHandle
	 */
	public static JunMovieHandle Filename_(String aFilename) {
		return JunMovieHandle.Filename_(new File(aFilename));
	}

	/**
	 * open the movie and return the movie handle.
	 * 
	 * @return sra.jun.goodies.movie.JunMovieHandle
	 */
	public static JunMovieHandle OpenMovie() {
		JunMovieHandle movieHandle = new JunMovieHandle();
		movieHandle.openMovie();

		return movieHandle;
	}

	/**
	 * private.
	 * 
	 * @return sra.jun.goodies.movie.JunMovieInterface
	 */
	public JunQTInterface _interface() {
		return JunQTInterface.Interface();
	}

	/**
	 * set the movie to active/deactive.
	 * 
	 * @param aBoolean boolean
	 */
	public void active_(boolean aBoolean) {
		if (this.hasMovie() == true) {
			this._interface().qtSetMovieActive(movie, aBoolean);
		}
	}

	/**
	 * get the image from the movie.
	 * 
	 * @return java.awt.Image
	 */
	public Image asImage() {
		if (this.hasMovie() == true) {
			return this._interface().getMovieImageAt(movie, this.timeValue(), this.originalExtent());
		}

		return null;
	}

	/**
	 * get the image from the movie at the given time value.
	 * 
	 * @param timeValue int
	 * 
	 * @return java.awt.Image
	 */
	public Image asImageAt_(int timeValue) {
		if (this.hasMovie() == true) {
			return this._interface().getMovieImageAt(movie, timeValue, this.originalExtent());
		}

		return null;
	}

	/**
	 * get the sound data from the movie.
	 * 
	 * @param start int
	 * @param duration int
	 * 
	 * @return byte[]
	 */
	public byte[] asSoundDataFrom_duration_(int start, int duration) {
		if (this.hasMovie() == true) {
			return this._interface().convertMovieToSoundData(movie, start, duration);
		}

		return null;
	}

	/**
	 * copy the selected portions of the movie.
	 * 
	 * @return int
	 */
	public int copyMovie() {
		if (this.hasMovie() == true) {
			return this._interface().qtCopyMovieSelection(movie);
		}

		return 0;
	}

	/**
	 * do movie task.
	 */
	public void doMovieTask() {
		if (this.hasMovie() == true) {
			this._interface().qtMoviesTask(movie, 0);
		}
	}

	/**
	 * Is the movie done or not?
	 * 
	 * @return boolean
	 */
	public boolean done() {
		if (this.hasMovie() == true) {
			return this._interface().isMovieDone(movie);
		}

		return true;
	}

	/**
	 * get the duration.
	 * 
	 * @return int
	 */
	public int duration() {
		if (this.hasMovie() == true) {
			int duration = this._interface().qtGetMovieDuration(movie);
			if (duration > 0) {
				return duration;
			}
		}

		return 1;
	}

	/**
	 * get the duration in seconds.
	 * 
	 * @return double
	 */
	public double durationInSeconds() {
		return (double) this.duration() / this.timeScale();
	}

	/**
	 * get the filename.
	 * 
	 * @return java.io.File
	 */
	public File filename() {
		return filename;
	}

	/**
	 * go to start of the movie.
	 */
	public void goToBeginning() {
		if (this.hasMovie() == true) {
			this._interface().qtGoToBeginningOfMovie(movie);
		}
	}

	/**
	 * go to end of the movie.
	 */
	public void goToEnd() {
		if (this.hasMovie() == true) {
			this._interface().qtGoToEndOfMovie(movie);
		}
	}

	/**
	 * Has movie or not?
	 * 
	 * @return boolean
	 */
	public boolean hasMovie() {
		if (movie == 0) {
			this.recoverMovie();
		}

		return movie != 0;
	}

	/**
	 * is movie a sound?
	 * 
	 * @return boolean
	 */
	public boolean isSound() {
		if (this.hasMovie() == true) {
			Rectangle box = this._interface().getMovieBox(movie);
			if ((box.width == 0) && (box.height == 0)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * get the box of the movie.
	 * 
	 * @return java.awt.Rectangle
	 */
	public Rectangle movieBox() {
		if (this.hasMovie() == true) {
			Rectangle box = this._interface().getMovieBox(movie);
			if ((box.width == 0) && (box.height == 0)) {
				box.setSize(this.minimumSize());
			}
			return box;
		}

		return new Rectangle(0, 0, 1, 1);
	}

	/**
	 * set the box of the movie.
	 * 
	 * @param aRectangle java.awt.Rectangle
	 */
	public void movieBox_(Rectangle aRectangle) {
		if (this.hasMovie() == true) {
			this._interface().setMovieBox(movie, aRectangle);
		}
	}

	/**
	 * get the next interesting time of the movie.
	 * 
	 * @return int
	 */
	public int nextInterestingTime() {
		if (this.hasMovie() == true) {
			int time = this.timeValue();
			int nextTime = this._interface().nextInterestingTimeOfMovie(movie, time, 1);
			if (nextTime < time) {
				return this.duration();
			} else {
				return nextTime;
			}
		} else {
			return 0;
		}
	}

	/**
	 * get the original extent of the movie.
	 * 
	 * @return java.awt.Dimension
	 */
	public Dimension originalExtent() {
		return originalExtent;
	}

	/**
	 * get the previous interesting time of the movie.
	 * 
	 * @return int
	 */
	public int previousInterestingTime() {
		if (this.hasMovie() == true) {
			int time = this.timeValue();
			int nextTime = this._interface().nextInterestingTimeOfMovie(movie, time, -1);
			if (nextTime >= time) {
				return 0;
			} else {
				return nextTime;
			}
		} else {
			return 0;
		}
	}

	/**
	 * releasing.
	 */
	public void release() {
		this.releaseMovie();
		super.release();
	}

	/**
	 * release the movie.
	 */
	public void releaseMovie() {
		if (movie != 0) {
			this.changed_($("releasing"));
			this._interface().qtDisposeMovie(movie);
			movie = 0;
		}

		dontRecover = true;
	}

	/**
	 * select portions of the movie.
	 * 
	 * @param selectionTime int
	 * @param selectionDuration int
	 */
	public void selection_duration_(int selectionTime, int selectionDuration) {
		if (this.hasMovie() == true) {
			this._interface().qtSetMovieSelection(movie, selectionTime, selectionDuration);
		}
	}

	/**
	 * start the movie.
	 */
	public void start() {
		if (this.hasMovie() == true) {
			this._interface().qtStartMovie(movie);
		}
	}

	/**
	 * stop the movie.
	 */
	public void stop() {
		if (this.hasMovie() == true) {
			this._interface().qtStopMovie(movie);
		}
	}

	/**
	 * get the time scale.
	 * 
	 * @return int
	 */
	public int timeScale() {
		if (this.hasMovie() == true) {
			return this._interface().qtGetMovieTimeScale(movie);
		}

		return 600;
	}

	/**
	 * get the time value.
	 * 
	 * @return int
	 */
	public int timeValue() {
		if (this.hasMovie() == true) {
			return this._interface().qtGetMovieTimeValue(movie);
		}

		return 0;
	}

	/**
	 * set the time value.
	 * 
	 * @param aTimeValue int
	 */
	public void timeValue_(int aTimeValue) {
		if (this.hasMovie() == true) {
			this._interface().qtSetMovieTimeValue(movie, aTimeValue);
		}
	}

	/**
	 * get the time value in seconds.
	 * 
	 * @return double
	 */
	public double timeValueInSeconds() {
		return (double) this.timeValue() / this.timeScale();
	}

	/**
	 * set the time value in seconds.
	 * 
	 * @param aTimeValue double
	 */
	public void timeValueInSeconds_(double aTimeValue) {
		this.timeValue_((int) (aTimeValue * this.timeScale()));
	}

	/**
	 * Print my string representation on the Writer.
	 * 
	 * @return a string representation of this <code>JunMoviePlayer</code>
	 *         object,  including the values of its member fields.
	 */
	public String toString() {
		String aString = new String();
		aString += this.getClass().getName();
		aString += "\n";
		aString += "\t";
		aString += "(";
		aString += "\n";
		aString += "\t";
		aString += ("_interface: " + this._interface());
		aString += "\n";
		aString += "\t";
		aString += ("filename: " + this.filename);
		aString += "\n\t";
		aString += ("movie: " + this.movie);
		aString += "\n\t";
		aString += ("originalExtent: " + this.originalExtent);
		aString += "\n\t";
		aString += ("dontRecover: " + this.dontRecover);
		aString += "\n";
		aString += ")";

		return aString;
	}

	/**
	 * update the movie.
	 */
	public void updateMovie() {
		if (this.hasMovie() == true) {
			this._interface().qtUpdateMovie(movie);
		}
	}

	/**
	 * get the volume.
	 * 
	 * @return double
	 */
	public double volume() {
		if (this.hasMovie() == true) {
			int volume = this._interface().qtGetMovieVolume(movie);
			return (double) volume / this._interface().kFullVolume;
		}

		return 0;
	}

	/**
	 * set the volume.
	 * 
	 * @param newVolume double
	 */
	public void volume_(double newVolume) {
		if (this.hasMovie() == true) {
			int volume = (int) (newVolume * this._interface().kFullVolume);
			this._interface().qtSetMovieVolume(movie, volume);
		}
	}

	/**
	 * private.     set the filename.
	 * 
	 * @param aFilename java.io.File
	 */
	protected void setFilename_(File aFilename) {
		filename = aFilename;
	}

	/**
	 * private.     set the movie.
	 * 
	 * @param aMovie int
	 */
	protected void setMovie_(int aMovie) {
		movie = aMovie;

		if (movie != 0) {
			originalExtent = this.movieBox().getSize();
		}
	}

	/**
	 * enableRecover.
	 */
	protected void enableRecover() {
		dontRecover = false;
	}

	/**
	 * finalize.
	 */
	protected void finalize() {
		this.release();
	}

	/**
	 * minimumSize.
	 * 
	 * @return java.awt.Dimension
	 */
	protected Dimension minimumSize() {
		return new Dimension(160, 12);
	}

	/**
	 * set the GWorld of movie with grafPtr.
	 * 
	 * @param grafPtr int
	 */
	protected void movieGrafPtr_(int grafPtr) {
		if (this.hasMovie() == true) {
			this._interface().qtSetMovieGWorld(movie, grafPtr, 0);
		}
	}

	/**
	 * private.     ask movie file and open it.
	 */
	protected void openMovie() {
		File aFilename = null;
		String osName = System.getProperty("os.name");

		if ((osName != null) && osName.equals("Mac OS X")) {
			aFilename = StDialog.OpenFileDialog();
		} else {
			String theFilename = this._interface().getMovieFile();

			if (theFilename != null && theFilename.length() > 0) {
				aFilename = new File(theFilename);
			}
		}

		if (aFilename != null) {
			filename = aFilename;
			this.setMovie_(this._interface().openMovie(filename.getPath()));
		}
	}

	/**
	 * recover the movie.
	 */
	protected void recoverMovie() {
		if ((filename == null) || (dontRecover == true)) {
			return;
		}

		if (filename.exists()) {
			this.setMovie_(this._interface().openMovie(filename.getPath()));
		}

		if (movie != 0) {
			this.changed_($("recovered"));
		} else {
			dontRecover = true;
		}
	}
}
