package sra.jun.goodies.movie;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import sra.smalltalk.*;
import sra.qt4jun.JunQTInterface;
import sra.jun.goodies.button.JunButtonModel;
import sra.jun.goodies.cursors.JunCursors;
import sra.jun.goodies.display.JunDisplayModel;
import sra.jun.goodies.files.*;
import sra.jun.goodies.gauge.JunSimpleGaugeModel;
import sra.jun.goodies.sound.JunSoundMeterModel;
import sra.jun.goodies.track.*;
import sra.jun.system.framework.JunApplicationModel;

/**
 * JunMoviePlayer class
 * 
 *  @author    NISHIHARA Satoshi
 *  @created   2001/01/23 (by NISHIHARA Satoshi)
 *  @updated   2002/04/25 (by hoshi)
 *  @updated   2002/11/14 (by nisinaka)
 *  @updated   2003/03/24 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun490 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMoviePlayer.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunMoviePlayer extends JunApplicationModel {

	protected final static int DefaultSoundMeterTimeStep = 20;
	protected final static int MinimumSoundLevelTimeStep = 50;
	protected final static int CalculateSoundWaveTimeStep = 600;

	protected JunMovieHandle movieHandle;
	protected int grafPtr;
	protected JunTrackerModel trackerModel;
	protected Thread trackingProcess;
	protected boolean trackingFlag;
	protected boolean playingFlag;
	protected double startTime;
	protected JunTrackSliderModel trackerModel2;
	protected int soundMeterTimeStep = DefaultSoundMeterTimeStep;
	protected StValueHolder soundValueHolder;
	protected JunSoundMeterModel soundSpectrum;
	protected JunSimpleGaugeModel volumeGauge;
	protected boolean keepAspect;

	protected StMenu _popupMenu;

	/**
	 * Create a new instance of JunMoviePlayer.
	 */
	public JunMoviePlayer() {
		super();
	}

	/**
	 * Create a new instance of JunMoviePlayer with the specified File.
	 *
	 * @param aFile java.io.File
	 */
	public JunMoviePlayer(File aFile) {
		this(JunMovieHandle.Filename_(aFile));
	}

	/**
	 * Create a new instance of JunMoviePlayer and initialize it with a movie handle.
	 *
	 * @param aMovieHandle sra.jun.goodies.movie.JunMovieHandle
	 */
	public JunMoviePlayer(JunMovieHandle aMovieHandle) {
		super();
		if (aMovieHandle != null) {
			this.movieHandle_(aMovieHandle);
		}
	}

	/**
	 * Answer the aligned rectangle.
	 *
	 * @return java.awt.Rectangle
	 * @param aRectangle1 java.awt.Rectangle
	 * @param aRectangle2 java.awt.Rectangle
	 */
	public static Rectangle AlignRectangle_withRectangle_(Rectangle aRectangle1, Rectangle aRectangle2) {
		int width = aRectangle2.width;
		int height = aRectangle1.height * aRectangle2.width / aRectangle1.width;
		Point center2 = (new StRectangle(aRectangle2)).center();
		StRectangle box = new StRectangle(0, 0, width, height);
		box = box.align_with_(box.center(), center2);
		if (box.height() <= aRectangle2.height) {
			return box.toRectangle();
		}
		width = aRectangle1.width * aRectangle2.height / aRectangle1.height;
		height = aRectangle2.height;
		box = new StRectangle(0, 0, width, height);
		box = box.align_with_(box.center(), center2);
		return box.toRectangle();
	}

	/**
	 * Create a new instance of JunMoviePlayer and open it with the specified movie file.
	 * 
	 * @param filename java.lang.String
	 * 
	 * @return sra.jun.goodies.movie.JunMoviePlayer
	 */
	public static JunMoviePlayer Filename_(String filename) {
		JunMovieHandle aMovieHandle = JunMovieHandle.Filename_(filename);
		if (aMovieHandle == null) {
			return null;
		}
		JunMoviePlayer aMoviePlayer = JunMoviePlayer.MovieHandle_(aMovieHandle);
		return aMoviePlayer;
	}

	/**
	 * Create a new instance of JunMoviePlayer and initialize it with a movie handle.
	 * 
	 * @param aMovieHandle sra.jun.goodies.movie.JunMovieHandle
	 * 
	 * @return sra.jun.goodies.movie.JunMoviePlayer
	 */
	public static JunMoviePlayer MovieHandle_(JunMovieHandle aMovieHandle) {
		return new JunMoviePlayer(aMovieHandle);
	}

	/**
	 * Create a new instance of JunMoviePlayer, choose a movie file, and open it.
	 * 
	 * @return sra.jun.goodies.movie.JunMoviePlayer
	 */
	public static JunMoviePlayer OpenMovie() {
		JunMovieHandle aMovieHandle = JunMovieHandle.OpenMovie();
		if (aMovieHandle == null) {
			return null;
		}

		JunMoviePlayer aMoviePlayer = new JunMoviePlayer(aMovieHandle);
		aMoviePlayer.open();
		return aMoviePlayer;
	}

	/**
	 * Open a file dialog to choose a movie file and create a new instance of JunMoviePlayer for the movie.
	 * 
	 * @return sra.jun.goodies.movie.JunMoviePlayer
	 */
	public static JunMoviePlayer Request() {
		return Request_("Please select a file.");
	}

	/**
	 * Open a file dialog to choose a movie file and create a new instance of JunMoviePlayer for the movie.
	 * 
	 * @param messageString java.lang.String
	 * 
	 * @return sra.jun.goodies.movie.JunMoviePlayer
	 */
	public static JunMoviePlayer Request_(String messageString) {
		FileDialog fileDialog = new FileDialog(new Frame(), messageString);
		fileDialog.show();
		String directory = fileDialog.getDirectory();
		String filename = fileDialog.getFile();
		if ((directory == null) || (filename == null)) {
			return null;
		}
		return JunMoviePlayer.Filename_(directory + filename);
	}

	/**
	 * Open a file dialog to choose a movie file, create a new instance of JunMoviePlayer for the movie, and open it.
	 * 
	 * @return sra.jun.goodies.movie.JunMoviePlayer
	 */
	public static JunMoviePlayer RequestOpen() {
		JunMoviePlayer aMoviePlayer = Request();
		if (aMoviePlayer == null) {
			return null;
		}
		aMoviePlayer.open();
		return aMoviePlayer;
	}

	/**
	 * Open a file dialog to choose a movie file, create a new instance of JunMoviePlayer for the movie, open it, and play it.
	 * 
	 * @return sra.jun.goodies.movie.JunMoviePlayer
	 */
	public static JunMoviePlayer RequestOpenAndPlay() {
		JunMoviePlayer aMoviePlayer = Request();
		if (aMoviePlayer == null) {
			return null;
		}
		aMoviePlayer.openAndPlay();
		return aMoviePlayer;
	}

	/**
	 * Answer true if the markers of the tracker are active, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean areMarkersActive() {
		return this.trackerModel().areMarkersActive();
	}

	/**
	 * converting.
	 * 
	 * @return java.awt.Image
	 */
	public Image asImage() {
		Image anImage = null;

		if (this.hasMovie()) {
			anImage = this.movieHandle().asImage();
		}

		return anImage;
	}

	/**
	 * Answer true if the player is at the end position, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean atEnd() {
		if (this.isIntervalNotEmpty() && (startTime <= this.trackerModel().lastMarker()) && (this.trackerModel().lastMarker() <= this.now())) {
			return true;
		}
		return (this.now() >= 1);
	}

	/**
	 * Answer true if the player is at the start position, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean atStart() {
		return (this.now() <= 0);
	}

	/**
	 * Calculate sound wave.
	 */
	public void calculateSoundWave() {
		this.soundMeter();

		Thread aThread = new Thread() {
			public void run() {
				int duration = movieHandle().duration();
				int start = 0, end = duration;
				if (isIntervalNotEmpty()) {
					start = (int) Math.round(trackerModel().firstMarker() * duration / soundMeterTimeStep) * soundMeterTimeStep;
					end = (int) (trackerModel().lastMarker() * duration);
				}
				int cnt = (CalculateSoundWaveTimeStep + soundMeterTimeStep - 1) / soundMeterTimeStep;
				JunSoundMeterModel soundMeter = soundSpectrum;
				for (int time = start; time < end; time += soundMeterTimeStep * cnt) {
					if (soundSpectrum != null) {
						double[] level = soundLevelFrom_to_length_((double) time / duration, (double) (time + soundMeterTimeStep * cnt) / duration, cnt);
						if (level != null) {
							for (int i = 0; i < cnt; ++i) {
								double t = (double) (time + soundMeterTimeStep * i) / duration;
								if (t < 1) {
									soundMeter.soundLevelData().put(new Double(t), new double[] { level[i * 2], level[i * 2 + 1] });
								}
							}
						}
						soundMeter.changed();
					}
				}
			}
		};
		aThread.start();
	}

	/**
	 * Create a clone of the JunMoviePlayer.
	 *
	 * @return sra.smalltalk.StObject
	 */
	public StObject copy() {
		JunMoviePlayer aMoviePlayer = new JunMoviePlayer(this.movieHandle().filename());
		aMoviePlayer.goto_(this.now());
		aMoviePlayer.trackerModel().lastMarker_(this.trackerModel().lastMarker());
		aMoviePlayer.trackerModel().fixLastMarker_(this.trackerModel().fixLastMarker());
		aMoviePlayer.trackerModel().firstMarker_(this.trackerModel().firstMarker());
		aMoviePlayer.trackerModel().fixFirstMarker_(this.trackerModel().fixFirstMarker());
		aMoviePlayer.trackerModel().loopCondition_(this.trackerModel().loopCondition());
		// aMoviePlayer.popupMenu_(this.popupMenu());
		if (this.trackerModel().areMarkersActive()) {
			aMoviePlayer.trackerModel().enableMarkers();
		} else {
			aMoviePlayer.trackerModel().disableMarkers();
		}
		aMoviePlayer.keepAspect_(keepAspect);
		return aMoviePlayer;
	}

	/**
	 * Answer my default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		return this.defaultViewWithTracker();
	}

	/**
	 * Answer my default view with two trackers.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultViewWithDoubleTracker() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return JunMovieViewAwt.WithDoubleTracker(this);
		} else {
			return JunMovieViewSwing.WithDoubleTracker(this);
		}
	}

	/**
	 * Answer my default view without a tracker.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultViewWithoutTracker() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return JunMovieViewAwt.WithoutTracker(this);
		} else {
			return JunMovieViewSwing.WithoutTracker(this);
		}
	}

	/**
	 * Answer my default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultViewWithTracker() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return JunMovieViewAwt.WithTracker(this);
		} else {
			return JunMovieViewSwing.WithTracker(this);
		}
	}

	/**
	 * Menu message: make the view size double.
	 */
	public void doubleSize() {
		if (this.hasMovie()) {
			Dimension oldSize = movieHandle.originalExtent();
			Dimension newSize = new Dimension(oldSize.width * 2, oldSize.height * 2);
			this.setSize_(newSize);
		}
	}

	/**
	 * Go to the epilogue.
	 */
	public void epilogue() {
		this.goto_(1);
	}

	/**
	 * Go forward by the specified amount.
	 *
	 * @param incrementNumber double
	 */
	public void go_(double incrementNumber) {
		this.goto_(this.now() + incrementNumber);
	}

	/**
	 * Go forward by the specified milliseconds.
	 *
	 * @param incrementMilliseconds double
	 */
	public void goInMilliseconds_(double incrementMilliseconds) {
		this.gotoInMilliseconds_(this.nowInMilliseconds() + incrementMilliseconds);
	}

	/**
	 * Go forward by the specified seconds.
	 *
	 * @param incrementSeconds double
	 */
	public void goInSeconds_(double incrementSeconds) {
		this.gotoInSeconds_(this.nowInSeconds() + incrementSeconds);
	}

	/**
	 * Go to the specified place.
	 *
	 * @param normalizedNumber double
	 */
	public void goto_(double normalizedNumber) {
		this.trackerModel().value_(Math.max(0, Math.min(normalizedNumber, 1)));
	}

	/**
	 * Go to the specified milliseconds.
	 *
	 * @param numberInMilliseconds double
	 */
	public void gotoInMilliseconds_(double numberInMilliseconds) {
		double totalMilliseconds = this.howLongInMilliseconds();
		double whereMilliseconds = Math.max(0, Math.min(numberInMilliseconds, totalMilliseconds));
		this.goto_(whereMilliseconds / totalMilliseconds);
	}

	/**
	 * Go to the specified seconds.
	 *
	 * @param numberInSeconds double
	 */
	public void gotoInSeconds_(double numberInSeconds) {
		double totalSeconds = this.howLongInSeconds();
		double whereSeconds = Math.max(0, Math.min(numberInSeconds, totalSeconds));
		this.goto_(whereSeconds / totalSeconds);
	}

	/**
	 * Go to the next interesting time.
	 */
	public void gotoNextInterestingTime() {
		if (this.hasMovie()) {
			this.trackerModel().value_(movieHandle.nextInterestingTime() / (double) movieHandle.duration());
		}
	}

	/**
	 * Go to the previous interesting time.
	 */
	public void gotoPreviousInterestingTime() {
		if (this.hasMovie()) {
			this.trackerModel().value_(movieHandle.previousInterestingTime() / (double) movieHandle.duration());
		}
	}

	/**
	 * Answer my current graf pointer.
	 * 
	 * @return int
	 */
	public int grafPtr() {
		if (grafPtr == 0) {
			grafPtr = JunQTInterface.Interface().createPortAssociation((Component) this.getMovieView());
		}
		return grafPtr;
	}

	/**
	 * Menu message: make the view size half.
	 */
	public void halfSize() {
		if (this.hasMovie()) {
			Dimension oldSize = movieHandle.originalExtent();
			Dimension newSize = new Dimension(oldSize.width / 2, oldSize.height / 2);
			this.setSize_(newSize);
		}
	}

	/**
	 * Answer true if the player has a movie, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasMovie() {
		return ((movieHandle() != null) && (movieHandle().hasMovie()));
	}

	/**
	 * Hide the tracker.
	 */
	public void hideTracker() {
		JunMovieView aView = this.getMovieView();
		if (aView == null) {
			return;
		}
		Window aWindow = aView.topComponent();
		if (aWindow == null) {
			return;
		}
		if (aView.hasTracker() == false) {
			return;
		}

		int deltaHeight = aView.hasTracker2() ? -38 : -19;
		Rectangle displayRectangle = aWindow.getBounds();
		displayRectangle.height += deltaHeight;

		boolean isPlay = this.isPlay();
		if (isPlay) {
			this.stop();
		}
		JunMoviePlayer aMoviePlayer = (JunMoviePlayer) this.copy();
		aMoviePlayer.openViewIn_(aMoviePlayer.defaultViewWithoutTracker(), displayRectangle);
		this.closeRequest();
		if (isPlay) {
			aMoviePlayer.start();
		}
	}

	/**
	 * Answer the total time of the movie in milliseconds.
	 *
	 * @return double
	 */
	public double howLongInMilliseconds() {
		return this.howLongInSeconds() * 1000;
	}

	/**
	 * Answer the total time of the movie in seconds.
	 *
	 * @return double
	 */
	public double howLongInSeconds() {
		return this.movieHandle().durationInSeconds();
	}

	/**
	 * Answer true if the imag model is embedded, otherwise false.
	 *
	 * @return boolean
	 */
	public boolean isEmbedded() {
		return false;
	}

	/**
	 * Answer true if the tracker has an interval which is not empty, otherwise
	 * false.
	 * 
	 * @return boolean
	 */
	public boolean isIntervalNotEmpty() {
		return this.trackerModel().isIntervalNotEmpty();
	}

	/**
	 * Answer true if the player is now playing a movie, otherwise false.
	 *
	 * @return boolean
	 */
	public boolean isPlay() {
		return this.trackerModel().playButton().value();
	}

	/**
	 * Answer true if the player is now playing a sound, otherwise false.
	 *
	 * @return boolean
	 */
	public boolean isSound() {
		if (this.movieHandle() == null) {
			return false;
		}
		return this.movieHandle().isSound();
	}

	/**
	 * Answer whether to keep aspect or not.
	 *
	 * @return boolean
	 */
	public boolean keepAspect() {
		return keepAspect;
	}

	/**
	 * Set whether to keep aspect or not.
	 *
	 * @param newKeepAspect boolean
	 */
	public void keepAspect_(boolean newKeepAspect) {
		boolean oldBoolean = keepAspect;
		keepAspect = newKeepAspect;
		if (oldBoolean != keepAspect) {
			JunMovieView aView = this.getMovieView();
			if (aView != null) {
				aView.setMovieView();
				aView.toComponent().repaint();
			}
		}
	}

	/**
	 * Called when the loop button is pressed.
	 *
	 * @param model sra.jun.goodies.button.JunButtonModel
	 */
	public void loopAction_(JunButtonModel model) {
		if (this.trackerModel().loopCondition() == $("oneWay")) {
			this.trackerModel().loop();
		} else {
			this.trackerModel().oneWay();
		}
	}

	/**
	 * accessing.
	 * 
	 * @return sra.jun.goodies.movie.JunMovieHandle
	 */
	public JunMovieHandle movieHandle() {
		return movieHandle;
	}

	/**
	 * accessing.
	 * 
	 * @param aMovieHandle sra.jun.goodies.movie.JunMovieHandle
	 */
	public void movieHandle_(JunMovieHandle aMovieHandle) {
		if (movieHandle != null) {
			movieHandle.removeDependent_(this);
		}

		movieHandle = aMovieHandle;

		if (movieHandle != null) {
			movieHandle.addDependent_(this);
		}

		if (this.hasMovie()) {
			this.trackerModel().step_(1 / (double) movieHandle.duration());
			if (volumeGauge != null) {
				movieHandle.volume_(volumeGauge.doubleValue());
			}
		}
	}

	/**
	 * Play to the next.
	 */
	public synchronized void next() {
		if (this.hasMovie()) {
			startTime = this.now();
			movieHandle.doMovieTask();
			this.setTrackerValue_(movieHandle.timeValue() / (double) movieHandle.duration());

			if (this.atEnd()) {
				if (movieHandle.done()) {
					if (this.trackerModel().loopCondition() == $("loop")) {
						if (this.isIntervalNotEmpty()) {
							this.setTrackerValue_(this.trackerModel().lastMarker());
						}
					} else {
						this.setTrackerValue_(1);
					}
				}

				// It is better not to stop and start again because of the thread timing.
				if (this.trackerModel().loopCondition() == $("loop")) {
					movieHandle.goToBeginning();
					if (this.isIntervalNotEmpty()) {
						this.goto_(this.trackerModel().firstMarker());
					}
				} else {
					this.stop();
				}
			}
		} else {
			this.stop();
		}
	}

	/**
	 * Called when the next button is pressed.
	 *
	 * @param model sra.jun.goodies.button.JunButtonModel
	 */
	public void nextAction_(JunButtonModel model) {
		if (model._isPressedWithShiftDown()) {
			this.trackerModel().last();
		} else {
			if (this.isSound()) {
				this.go_(0.005);
			} else {
				this.gotoNextInterestingTime();
			}
		}
	}

	/**
	 * Menu message: make the view size normal.
	 */
	public void normalSize() {
		if (this.hasMovie()) {
			this.setSize_(movieHandle.originalExtent());
		}
	}

	/**
	 * Answer the current position.
	 * 
	 * @return double
	 */
	public double now() {
		return this.trackerModel().doubleValue();
	}

	/**
	 * Answer the current position in milliseconds.
	 *
	 * @return double
	 */
	public double nowInMilliseconds() {
		return this.now() * this.howLongInMilliseconds();
	}

	/**
	 * Answer the current position in seconds.
	 *
	 * @return double
	 */
	public double nowInSeconds() {
		return this.now() * this.howLongInSeconds();
	}

	/**
	 * Create a frame of the default view and open it.
	 * 
	 * @return java.awt.Frame
	 */
	public Frame open() {
		Frame aFrame = super.open();
		this.normalSize();
		return aFrame;
	}

	/**
	 * Open the player and play the movie.
	 */
	public void openAndPlay() {
		this.open();
		this.start();
	}

	/**
	 * Open the player, play the movie, and close the player.
	 */
	public void openAndPlayAndClose() {
		this.open();
		this.start();

		Thread aThread = new Thread() {
			public void run() {
				while (hasMovie() && atEnd() == false) {
					try {
						Thread.sleep(100);
					} catch (InterruptedException e) {
					}
				}
				closeRequest();
			}
		};
		aThread.start();
	}

	/**
	 * Open the player without a tracker, play the movie, and close the player.
	 */
	public void openAndPlayAndCloseWithoutTracker() {
		this.openView_(this.defaultViewWithoutTracker());
		this.start();

		Thread aThread = new Thread() {
			public void run() {
				while (hasMovie() && atEnd() == false) {
					try {
						Thread.sleep(100);
					} catch (InterruptedException e) {
					}
				}
				closeRequest();
			}
		};
		aThread.start();
	}

	/**
	 * menu messages.
	 */
	public void openMovie() {
		JunMovieHandle aMovieHandle = JunMovieHandle.OpenMovie();

		if (aMovieHandle.hasMovie()) {
			if (movieHandle != null) {
				JunMoviePlayer aPlayer = JunMoviePlayer.MovieHandle_(aMovieHandle);
				aPlayer.open();
			} else {
				this.movieHandle_(aMovieHandle);
				this.normalSize();
			}
		}
	}

	/**
	 * Called when the play button is pressed.
	 *
	 * @param model sra.jun.goodies.button.JunButtonModel
	 */
	public void playAction_(JunButtonModel model) {
		if (model.value()) {
			this.stop();
		} else {
			if (model._isPressedWithShiftDown()) {
				this.trackerModel().first();
			}
			this.start();
		}
	}

	/**
	 * Called when the previous button is pressed.
	 *
	 * @param model sra.jun.goodies.button.JunButtonModel
	 */
	public void previousAction_(JunButtonModel model) {
		if (model._isPressedWithShiftDown()) {
			this.trackerModel().first();
		} else {
			if (this.isSound()) {
				this.go_(-0.005);
			} else {
				this.gotoPreviousInterestingTime();
			}
		}
	}

	/**
	 * Go to the prologue.
	 */
	public void prologue() {
		this.goto_(0);
	}

	/**
	 * Release the resources.
	 */
	public void release() {
		this.trackerModel().end();

		this.stop(); // The tracking process is terminated in the stop method.

		if (movieHandle != null) {
			movieHandle.release();
			movieHandle = null;
		}

		this.releaseGrafPtr();

		super.release();
	}

	/**
	 * Save the selected portion as a QuickTime movie file.
	 */
	public void saveSelection() {
		if (this.hasMovie() == false) {
			return;
		}

		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType("Movie files", new String[] { ".mov", ".MOV" })
		};
		File aFile = JunFileDialog.RequestNewFilename("Input a movie file.", new File(this.defaultBaseName() + ".mov"), fileTypes, fileTypes[0]);
		if (aFile == null) {
			return;
		}

		this.saveSelectionTo_(aFile);
	}

	/**
	 * Save the selected portion as a QuickTime movie file.
	 *
	 * @param aFile java.io.File
	 */
	public void saveSelectionTo_(File aFile) {
		int start, duration;
		if (this.isIntervalNotEmpty()) {
			start = (int) Math.round(this.trackerModel().firstMarker() * this.movieHandle().duration());
			duration = (int) Math.round((this.trackerModel().lastMarker() - this.trackerModel().firstMarker()) * this.movieHandle().duration());
		} else {
			start = 0;
			duration = this.movieHandle().duration();
		}

		JunCursors executeCursor = new JunCursors(JunCursors.ExecuteCursor());
		int newMovie = 0;
		try {
			executeCursor._show();
			this.movieHandle().selection_duration_(start, duration);
			newMovie = this.movieHandle().copyMovie();
		} finally {
			executeCursor._restore();
		}

		if (newMovie == 0) {
			return;
		}

		JunCursors writeCursor = new JunCursors(JunCursors.WriteCursor());
		try {
			writeCursor._show();
			this.movieHandle()._interface().flattenMovie_filename_(newMovie, aFile);
		} finally {
			writeCursor._restore();
		}
	}

	/**
	 * Show the tracker.
	 */
	public void showTracker() {
		JunMovieView aView = this.getMovieView();
		if (aView == null) {
			return;
		}
		Window aWindow = aView.topComponent();
		if (aWindow == null) {
			return;
		}

		int deltaHeight;
		if (aView.hasTracker()) {
			if (aView.hasTracker2()) {
				if (this.areMarkersActive()) {
					return;
				}
				deltaHeight = -19;
			} else {
				if (this.areMarkersActive() == false) {
					return;
				}
				deltaHeight = 19;
			}
		} else {
			if (this.areMarkersActive()) {
				deltaHeight = 38;
			} else {
				deltaHeight = 19;
			}
		}
		Rectangle displayRectangle = aWindow.getBounds();
		displayRectangle.height += deltaHeight;

		boolean isPlay = this.isPlay();
		if (isPlay) {
			this.stop();
		}
		JunMoviePlayer aMoviePlayer = (JunMoviePlayer) this.copy();
		if (this.areMarkersActive()) {
			aMoviePlayer.openViewIn_(aMoviePlayer.defaultViewWithDoubleTracker(), displayRectangle);
		} else {
			aMoviePlayer.openViewIn_(aMoviePlayer.defaultViewWithTracker(), displayRectangle);
		}
		this.closeRequest();
		if (isPlay) {
			aMoviePlayer.start();
		}
	}

	/**
	 * Return the sound level.
	 * 
	 * @param startNormalizedValue double
	 * @param endNormalizedValue double
	 * @param length int
	 * @return double[]
	 */
	public double[] soundLevelFrom_to_length_(double startNormalizedValue, double endNormalizedValue, int length) {
		int duration = this.movieHandle().duration();
		int start = Math.max(0, Math.min((int) (startNormalizedValue * duration), duration));
		int end = Math.min(Math.max(start + Math.max(soundMeterTimeStep, MinimumSoundLevelTimeStep), (int) (endNormalizedValue * duration)), duration);
		byte[] data = this.movieHandle().asSoundDataFrom_duration_(start, end - start);
		if (data == null) {
			return null;
		}
		int cnt = data.length;
		if (end - start > soundMeterTimeStep) {
			cnt = cnt * soundMeterTimeStep / (end - start);
		}
		double[] array = new double[length * 2];
		for (int i = 0; i < length; ++i) {
			int low = 128, high = 128;
			int max = Math.min((i + 1) * cnt, data.length);
			for (int j = i * cnt; j < max; ++j) {
				int value = ((int) data[j]) & 0xff;
				if (value < low) {
					low = value;
				} else if (value > high) {
					high = value;
				}
			}
			array[i * 2] = low / 255.0;
			array[i * 2 + 1] = high / 255.0;
		}
		return array;
	}

	/**
	 * Menu messages: open the sound meter.
	 */
	public void soundMeter() {
		JunSoundMeterModel model = this.soundSpectrum();
		if (model.builder().windows().length > 0) {
			model._windowExpandAndRaise();
		} else {
			StView view = this.getView();
			if (view == null) {
				model.open();
			} else {
				StRectangle box = new StRectangle(view.topComponent().getBounds());
				StRectangle area = new StRectangle(0, 0, 190, 45);
				area = area.expandedBy_(new StRectangle(0, 0, 8, 27)); // window border
				area = area.align_with_(area.bottomLeft(), new Point(box.left(), box.top() - 5));
				model.openIn_(area.toRectangle());
			}
			Frame[] frames = model.builder().frames();
			for (int i = 0; i < frames.length; ++i) {
				frames[i].setTitle("Meter");
			}
		}
	}

	/**
	 * Answer my sound time step.
	 * 
	 * @return int
	 */
	public int soundMeterTimeStep() {
		return soundMeterTimeStep;
	}

	/**
	 * Set my sound time step to aNumber.
	 * 
	 * @param aNumber int
	 */
	public void soundMeterTimeStep_(int aNumber) {
		soundMeterTimeStep = aNumber;
	}

	/**
	 * Answer my sound spectrum.
	 * 
	 * @return sra.jun.goodies.sound.JunSoundMeterModel
	 */
	public JunSoundMeterModel soundSpectrum() {
		if (soundSpectrum == null) {
			soundSpectrum = new JunSoundMeterModel();
			soundSpectrum.soundValueHolder_(this.soundValueHolder());
		}

		return soundSpectrum;
	}

	/**
	 * Answer my sound value holder.
	 * 
	 * @return sra.smalltalk.StValueHolder
	 */
	public StValueHolder soundValueHolder() {
		if (soundValueHolder == null) {
			soundValueHolder = new StValueHolder();
		}

		return soundValueHolder;
	}

	/**
	 * Menu messages: open the volume gauge.
	 */
	public void soundVolume() {
		JunSimpleGaugeModel model = this.volumeGauge();
		if (model.builder().windows().length > 0) {
			model._windowExpandAndRaise();
		} else {
			StView view = this.getView();
			if (view == null) {
				model.open();
			} else {
				StRectangle box = new StRectangle(view.topComponent().getBounds());
				StRectangle area = new StRectangle(0, 0, 124, 45);
				area = area.expandedBy_(new StRectangle(0, 0, 8, 27)); // window border
				area = area.align_with_(area.bottomRight(), new Point(box.right(), box.top() - 5));
				model.openIn_(area.toRectangle());
			}
			Frame[] frames = model.builder().frames();
			for (int i = 0; i < frames.length; ++i) {
				frames[i].setTitle("Volume");
			}
		}
	}

	/**
	 * Menu messages: spawn the current frame as an image.
	 */
	public void spawnImage() {
		Image anImage = this.asImage();
		if (anImage == null) {
			return;
		}

		JunDisplayModel aDisplayModel = new JunDisplayModel(anImage);
		aDisplayModel._isScrollable(false);
		aDisplayModel.open();
	}

	/**
	 * Menu messages: spawn the movie player.
	 */
	public void spawnMovie() {
		if (this.hasMovie() == false) {
			return;
		}

		JunMoviePlayer aMoviePlayer = new JunMoviePlayer(this.movieHandle().filename());
		aMoviePlayer.goto_(this.now());
		aMoviePlayer.open();
	}

	/**
	 * Start playing.
	 */
	public void start() {
		if (this.hasMovie()) {
			trackingProcess = new Thread() {
				public void run() {
					Thread.yield();
					JunTrackerModel tracker = trackerModel();
					tracker.playButtonVisual_(true);
					try {
						if (movieHandle().done()) {
							movieHandle().goToBeginning();
						}
						if (isIntervalNotEmpty() && (now() >= tracker.lastMarker())) {
							goto_(tracker.firstMarker());
						}
						startTime = now();
						movieHandle().start();
						while (isPlay()) {
							Thread.yield();
							next();
						}
					} finally {
						tracker.playButtonVisual_(false);
						// trackingProcess = null;
					}
				}
			};

			trackingProcess.setPriority(Thread.NORM_PRIORITY);
			trackingProcess.start();
		}
	}

	/**
	 * Stop playing.
	 */
	public void stop() {
		if (this.hasMovie()) {
			movieHandle.stop();
		}
		this.trackerModel().playButtonVisual_(false);
		startTime = -1;

		// Make sure to stop the tracking process.
		if (trackingProcess != null) {
			try {
				trackingProcess.join(1000);
			} catch (InterruptedException ie) {
			}
			trackingProcess = null;
		}
	}

	/**
	 * Menu message: toggle the keep aspect flag.
	 */
	public void toggleKeepAspect() {
		this.keepAspect_(!this.keepAspect());
	}

	/**
	 * Answer my current tracker model.
	 * 
	 * @return sra.jun.goodies.track.JunTrackerModel
	 */
	public JunTrackerModel trackerModel() {
		if (trackerModel == null) {
			trackerModel = this._newTrackerModel();
			trackerModel.addDependent_(this);
			trackerModel.playButton().action_(new StBlockClosure() {
				public Object value_(Object o) {
					playAction_((JunButtonModel) o);
					return null;
				}
			});
			trackerModel.loopButton().action_(new StBlockClosure() {
				public Object value_(Object o) {
					loopAction_((JunButtonModel) o);
					return null;
				}
			});
			trackerModel.nextButton().action_(new StBlockClosure() {
				public Object value_(Object o) {
					nextAction_((JunButtonModel) o);
					return null;
				}
			});
			trackerModel.previousButton().action_(new StBlockClosure() {
				public Object value_(Object o) {
					previousAction_((JunButtonModel) o);
					return null;
				}
			});
			trackerModel.compute_(new StBlockClosure() {
				public Object value_(Object o) {
					setTime_(((Double) o).doubleValue());
					return null;
				}
			});
		}

		return trackerModel;
	}

	/**
	 * Answer my second tracker model.
	 * 
	 * @return sra.jun.goodies.track.JunTrackSliderModel
	 */
	public JunTrackSliderModel trackerModel2() {
		if (trackerModel2 == null) {
			trackerModel2 = this._newTrackerModel2();
			trackerModel2.parentTracker_(this.trackerModel());
			trackerModel2.compute_(new StBlockClosure() {
				public Object value_(Object o) {
					setTime2_(((Double) o).doubleValue());
					return null;
				}
			});
		}

		return trackerModel2;
	}

	/**
	 * Called to update the model and the dependents of the receiver.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		StSymbol anAspect = evt.getAspect();
		if (anAspect == $("releasing")) {
			playingFlag = this.isPlay();
		} else if (anAspect == $("recovered")) {
			if (this.hasMovie()) {
				this.trackerModel().step_(1 / (double) movieHandle.duration());
				movieHandle.timeValue_((int) Math.round(this.trackerModel().doubleValue() * movieHandle.duration()));
				JunMovieView aView = this.getMovieView();
				if (aView != null) {
					aView.setMovieView();
				}
				if (playingFlag == true) {
					this.start();
				}
			}
		} else if (anAspect == $("useMarkers")) {
			this.showTracker();
		} else if (anAspect == $("originalExtent")) {
			this.normalSize();
		}
	}

	/**
	 * Answer true if the view has a tracker.
	 *
	 * @return boolean
	 */
	public boolean viewHasTracker() {
		JunMovieView aView = this.getMovieView();
		if (aView == null) {
			return false;
		}

		return aView.hasTracker();
	}

	/**
	 * Answer true if the view has a tracker2.
	 *
	 * @return boolean
	 */
	public boolean viewHasTracker2() {
		JunMovieView aView = this.getMovieView();
		if (aView == null) {
			return false;
		}

		return aView.hasTracker2();
	}

	/**
	 * Answer my volume gauge model.
	 * 
	 * @return sra.jun.goodies.gauge.JunSimpleGaugeModel
	 */
	public JunSimpleGaugeModel volumeGauge() {
		if (volumeGauge == null) {
			final JunMoviePlayer this_ = this;
			volumeGauge = new JunSimpleGaugeModel(this.hasMovie() ? movieHandle.volume() : 1);
			volumeGauge.compute_(new StBlockClosure() {
				public Object value_(Object volume) {
					if (this_.hasMovie()) {
						movieHandle.volume_(((Double) volume).doubleValue());
					}
					return null;
				}
			});
			volumeGauge.valueStringBlock_(new StBlockClosure() {
				public Object value_(Object volume) {
					return Long.toString(Math.round(((Double) volume).doubleValue() * 100));
				}
			});
		}

		return volumeGauge;
	}

	/**
	 * Invoked when a window is in the process of being closed.
	 * 
	 * @param e java.awt.event.WindowEvent
	 * @see sra.smalltalk.StApplicationModel#noticeOfWindowClose()
	 * @category interface closing
	 */
	public void noticeOfWindowClose(WindowEvent e) {
		if (soundSpectrum != null) {
			soundSpectrum.closeRequest();
			soundSpectrum = null;
		}
		if (volumeGauge != null) {
			volumeGauge.closeRequest();
			volumeGauge = null;
		}

		this.release();

		super.noticeOfWindowClose(e);
	}

	/**
	 * Process the key event.
	 *
	 * @param ev java.awt.event.KeyEvent
	 */
	protected void _keyboardEvent(KeyEvent ev) {
		if (this.hasMovie() == false) {
			return;
		}

		char ch = ev.getKeyChar();
		switch (ch) {
			case KeyEvent.VK_SPACE :
			case KeyEvent.VK_ENTER :
				if (this.isPlay()) {
					this.stop();
				} else {
					this.start();
				}
				break;
			case KeyEvent.VK_LEFT :
				this.previousAction_(this.trackerModel().previousButton());
				break;
			case KeyEvent.VK_RIGHT :
				this.nextAction_(this.trackerModel().nextButton());
				break;
			case KeyEvent.VK_TAB :
				if (this.trackerModel().loopCondition() == $("oneWay")) {
					this.trackerModel().loopCondition_($("loop"));
				} else {
					this.trackerModel().loopCondition_($("oneWay"));
				}
				break;
			case '[' :
				this.trackerModel().firstMarker_(this.now());
				break;
			case ']' :
				this.trackerModel().lastMarker_(this.now());
				break;
			case 'p' :
				this.prologue();
				break;
			case 'e' :
				this.epilogue();
				break;
			case 'f' :
				this.trackerModel().first();
				break;
			case 'l' :
				this.trackerModel().last();
				break;
		}
	}

	/**
	 * Answer my KeyListener which handles my keyboard events.
	 *
	 * @return java.awt.event.KeyListener
	 */
	protected KeyListener _keyListener() {
		return new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				_keyboardEvent(e);
			}
		};
	}

	/**
	 * Create a new instance of the tracker model.
	 *
	 * @return sra.jun.goodies.track.JunTrackerModel
	 */
	protected JunTrackerModel _newTrackerModel() {
		return new JunTrackerModel();
	}

	/**
	 * Create a new instance of the tracker model 2.
	 *
	 * @return sra.jun.goodies.track.JunTrackerModel
	 */
	protected JunTrackSliderModel _newTrackerModel2() {
		return new JunTrackSliderModel();
	}

	/**
	 * initialize-release.
	 */
	protected void finalize() {
		this.release();
	}

	/**
	 * Answer one of the movie views.
	 * 
	 * @return sra.jun.goodies.movie.JunMovieView
	 */
	protected JunMovieView getMovieView() {
		Object[] dependents = this.dependents();
		for (int i = 0; i < dependents.length; i++) {
			if (dependents[i] instanceof JunMovieView) {
				return (JunMovieView) dependents[i];
			}
		}

		return null;
	}

	/**
	 * Initialize the receiver.
	 */
	protected void initialize() {
		super.initialize();

		movieHandle = null;
		grafPtr = 0;
		trackerModel = null;
		trackingProcess = null;
		trackingFlag = false;
		playingFlag = false;
		startTime = -1;
		trackerModel2 = null;
		soundValueHolder = null;
		soundSpectrum = null;
		volumeGauge = null;
		keepAspect = true;

		_popupMenu = null;
	}

	/**
	 * Answer the margins of the marker.
	 *
	 * @return double[]
	 */
	protected double[] markerMargins() {
		return new double[] { 0.1, 0.1 };
	}

	/**
	 * Answer the StMenu for the popup menu.
	 * 
	 * @return StMenu
	 */
	protected StMenu popupMenu() {
		if (_popupMenu == null) {
			StMenu popupMenu = new StMenu("");
			popupMenu.add(createMenuItem("Open...", "openMovie"));
			popupMenu.add(createMenuItem("Open url...", "openMovieFromURL"));
			popupMenu.addSeparator();
			popupMenu.add(createMenuItem("Start", "start"));
			popupMenu.add(createMenuItem("Stop", "stop"));
			popupMenu.addSeparator();
			popupMenu.add(createMenuItem("Half size", "halfSize"));
			popupMenu.add(createMenuItem("Normal size", "normalSize"));
			popupMenu.add(createMenuItem("Double size", "doubleSize"));
			popupMenu.add(createCheckBoxMenuItem("Keep aspect", "toggleKeepAspect", this.keepAspect()));
			popupMenu.addSeparator();
			popupMenu.add(createMenuItem("Show tracker", "showTracker"));
			popupMenu.add(createMenuItem("Hide tracker", "hideTracker"));
			popupMenu.addSeparator();
			popupMenu.add(createMenuItem("Sound volume", "soundVolume"));
			popupMenu.add(createMenuItem("Sound meter", "soundMeter"));
			popupMenu.add(createMenuItem("Calculate sound wave", "calculateSoundWave"));
			popupMenu.addSeparator();
			popupMenu.add(createMenuItem("Spawn", "spawnMovie"));
			popupMenu.add(createMenuItem("Spawn image", "spawnImage"));
			popupMenu.addSeparator();
			popupMenu.add(createMenuItem("Save selection...", "saveSelection"));

			popupMenu.getItemWithValue("openMovieFromURL").setEnabled(false); // not implemented.

			 (new StMenuModel()).add(popupMenu);
			_popupMenu = popupMenu;
		}

		StMenuItem menuItem;
		menuItem = _popupMenu.getItemWithValue("start");
		if (menuItem != null) {
			if (this.hasMovie()) {
				menuItem.setEnabled(!this.isPlay());
			} else {
				menuItem.setEnabled(false);
			}
		}
		menuItem = _popupMenu.getItemWithValue("stop");
		if (menuItem != null) {
			if (this.hasMovie()) {
				menuItem.setEnabled(this.isPlay());
			} else {
				menuItem.setEnabled(false);
			}
		}
		menuItem = _popupMenu.getItemWithValue("halfSize");
		if (menuItem != null) {
			if (this.hasMovie()) {
				menuItem.setEnabled(!this.isEmbedded());
			} else {
				menuItem.setEnabled(false);
			}
		}
		menuItem = _popupMenu.getItemWithValue("normalSize");
		if (menuItem != null) {
			if (this.hasMovie()) {
				menuItem.setEnabled(!this.isEmbedded());
			} else {
				menuItem.setEnabled(false);
			}
		}
		menuItem = _popupMenu.getItemWithValue("doubleSize");
		if (menuItem != null) {
			if (this.hasMovie()) {
				menuItem.setEnabled(!this.isEmbedded());
			} else {
				menuItem.setEnabled(false);
			}
		}
		menuItem = _popupMenu.getItemWithValue("toggleKeepAspect");
		if (menuItem != null) {
			if (this.hasMovie()) {
				menuItem.setEnabled(true);
				((StCheckBoxMenuItem) menuItem).setSelected(this.keepAspect());
			} else {
				menuItem.setEnabled(false);
			}
		}
		menuItem = _popupMenu.getItemWithValue("showTracker");
		if (menuItem != null) {
			if (this.viewHasTracker()) {
				menuItem.setEnabled(false);
			} else {
				menuItem.setEnabled(!this.isEmbedded());
			}
		}
		menuItem = _popupMenu.getItemWithValue("hideTracker");
		if (menuItem != null) {
			if (this.viewHasTracker()) {
				menuItem.setEnabled(!this.isEmbedded());
			} else {
				menuItem.setEnabled(false);
			}
		}
		menuItem = _popupMenu.getItemWithValue("soundVolume");
		if (menuItem != null) {
			if (this.hasMovie()) {
				menuItem.setEnabled(true);
			} else {
				menuItem.setEnabled(false);
			}
		}
		menuItem = _popupMenu.getItemWithValue("soundMeter");
		if (menuItem != null) {
			if (this.hasMovie()) {
				menuItem.setEnabled(true);
			} else {
				menuItem.setEnabled(false);
			}
		}
		menuItem = _popupMenu.getItemWithValue("calculateSoundWave");
		if (menuItem != null) {
			if (this.hasMovie()) {
				menuItem.setEnabled(true);
			} else {
				menuItem.setEnabled(false);
			}
		}
		menuItem = _popupMenu.getItemWithValue("spawnImage");
		if (menuItem != null) {
			if (this.hasMovie()) {
				menuItem.setEnabled(!this.isSound());
			} else {
				menuItem.setEnabled(false);
			}
		}
		menuItem = _popupMenu.getItemWithValue("saveSelection");
		if (menuItem != null) {
			if (this.hasMovie()) {
				menuItem.setEnabled(true);
				if (this.isIntervalNotEmpty()) {
					menuItem.setLabel("Save selection...");
				} else {
					menuItem.setLabel("Save whole...");
				}
			} else {
				menuItem.setEnabled(false);
			}
		}

		return _popupMenu;
	}

	/**
	 * Release the graf pointer.
	 */
	protected void releaseGrafPtr() {
		if (grafPtr != 0) {
			JunQTInterface.Interface().destroyPortAssociation(grafPtr);
			grafPtr = 0;
		}
	}

	/**
	 * Set the size for the view.
	 * 
	 * @param extent java.awt.Dimension
	 */
	protected void setSize_(Dimension extent) {
		if (this.hasMovie() == false) {
			return;
		}

		{ // instead of super.setSize_(extent);
			JunMovieView aView = this.getMovieView();
			if (aView == null) {
				return;
			}

			Window aWindow = aView.topComponent();
			if (aWindow == null) {
				return;
			}

			Dimension oldExtent = aView.toComponent().getSize();
			Dimension newExtent = extent;
			int width = aWindow.getWidth() - oldExtent.width + newExtent.width;
			int height = aWindow.getHeight() - oldExtent.height + newExtent.height;
			aWindow.setSize(width, height);
			aWindow.validate();
		}

		JunMovieView aView = this.getMovieView();
		if (aView == null) {
			return;
		}

		aView.setMovieView();
	}

	/**
	 * Set the sound value.
	 * 
	 * @param time double
	 */
	protected void setSoundValue_(double time) {
		if (soundSpectrum != null) {
			int duration = this.movieHandle().duration();
			int timeValue = (int) Math.round(time * duration / soundMeterTimeStep) * soundMeterTimeStep;
			time = (double) timeValue / duration;
			Double key = new Double(time);
			double[] level = soundSpectrum.soundLevelDataAt_(key);
			if (level == null) {
				level = this.soundLevelFrom_to_length_(time, time, 1);
			}
			if (level != null) {
				this.soundValueHolder().value_(new Object[] { key, level });
			}
		}
	}

	/**
	 * Set the current time.
	 * 
	 * @param time double
	 */
	protected synchronized void setTime_(double time) {
		if (trackingFlag != true && this.hasMovie()) {
			movieHandle.timeValue_((int) Math.round(time * movieHandle.duration()));
			movieHandle.doMovieTask();
		}
		this.setTime1_(time);
	}

	/**
	 * Set the current time for the first tracker.
	 * 
	 * @param time double
	 */
	protected void setTime1_(double time) {
		boolean aBoolean = this.trackerModel2().areMarkersActive();
		if (this.trackerModel().isIntervalNotEmpty()) {
			this.trackerModel2().enableMarkers();
		} else {
			this.trackerModel2().disableMarkers();
		}
		if (aBoolean != this.trackerModel2().areMarkersActive()) {
			DependentListener[] dependents = this.trackerModel2().dependents();
			for (int i = 0; i < dependents.length; i++) {
				if (dependents[i] instanceof JunTrackSliderView) {
					dependents[i].update_(new DependentEvent(dependents[i], $("interval"), null));
				}
			}
		}

		double value;
		double[] markerMargins = this.markerMargins();
		if (this.trackerModel().isIntervalNotEmpty()) {
			value = (this.now() - this.trackerModel().firstMarker()) / (this.trackerModel().lastMarker() - this.trackerModel().firstMarker());
			value = value * (1 - markerMargins[0] - markerMargins[1]) + markerMargins[0];
			value = Math.max(0, Math.min(value, 1));
		} else {
			value = this.now();
		}
		this.trackerModel2().intervalHolder().value_(new double[] { markerMargins[0], 1 - markerMargins[1] });
		this.trackerModel2().valueHolder().value_(value);

		DependentListener[] dependents = this.trackerModel2().dependents();
		for (int i = 0; i < dependents.length; i++) {
			if (dependents[i] instanceof JunTrackSliderView) {
				dependents[i].update_(new DependentEvent(dependents[i], $("value"), null));
			}
		}
	}

	/**
	 * Set the current time for the second tracker.
	 * 
	 * @param time double
	 */
	protected synchronized void setTime2_(double time) {
		if (trackingFlag != true && this.hasMovie()) {
			if (this.trackerModel().isIntervalNotEmpty()) {
				double[] markerMargins = this.markerMargins();
				double value = (time - markerMargins[0]) / (1 - markerMargins[0] - markerMargins[1]);
				value = (this.trackerModel().lastMarker() - this.trackerModel().firstMarker()) * value + this.trackerModel().firstMarker();
				this.goto_(value);
			} else {
				this.goto_(time);
			}
		}
	}

	/**
	 * Set the tracker value.
	 * 
	 * @param value double
	 */
	protected void setTrackerValue_(double value) {
		trackingFlag = true;
		try {
			this.trackerModel().value_(value);
		} finally {
			trackingFlag = false;
		}
		this.setSoundValue_(value);
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "Movie Player";
	}

}
