package sra.jun.goodies.movie;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import sra.jun.goodies.files.*;
import sra.jun.system.support.*;

/**
 * JunMoviePlayerTestExamples class
 * 
 *  @author    NISHIHARA Satoshi
 *  @created   2001/01/24 (by NISHIHARA Satoshi)
 *  @updated   2002/11/14 (by nisinaka)
 *  @updated   2003/03/24 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun490 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMoviePlayerTestExamples.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunMoviePlayerTestExamples extends JunTestExamples {

	/**
	 * Create a new instance of JunMoviePlayer and just open it.
	 * 
	 * @return boolean
	 */
	public static boolean Example() {
		(new JunMoviePlayer()).open();
		return true;
	}

	/**
	 * Example1: Create a new instance of JunMoviePlayer, ask to choose a movie file, and open it.
	 * 
	 * @return boolean
	 */
	public static boolean Example1() {
		JunMoviePlayer.OpenMovie();
		return true;
	}

	/**
	 * Example2: Create a new instance of JunMoviePlayer, ask to choose a movie file, and open it.
	 * 
	 * @return boolean
	 */
	public static boolean Example2() {
		JunFileModel.FileType[] fileTypes =
			new JunFileModel.FileType[] {
				new JunFileModel.FileType("Movie files", JunSystem.DefaultMovieExtensions()),
				new JunFileModel.FileType("Sound files", JunSystem.DefaultSoundExtensions()),
				new JunFileModel.FileType("Image files", JunSystem.DefaultImageExtensions()),
				JunFileModel.FileType.All("All files")};
		File aFile = JunFileDialog.Request("Select a file.", fileTypes, fileTypes[0]);
		if (aFile == null) {
			return false;
		}

		JunMoviePlayer aMoviePlayer = new JunMoviePlayer(aFile);
		if (aMoviePlayer == null) {
			return false;
		}

		aMoviePlayer.open();
		return true;
	}

	/**
	 * Example4: Open a movie, go to the specified frame, and spawn the frame as an image.
	 * 
	 * @return boolean
	 */
	public static boolean Example4() {
		JunMoviePlayer aMoviePlayer = JunMoviePlayer.Request();
		if (aMoviePlayer == null) {
			return false;
		}

		aMoviePlayer.open();
		for (double i = 0; i <= 1.0; i += 0.25) {
			aMoviePlayer.goto_(i);
			sra.jun.goodies.display.JunDisplayModel.OpenVisual_label_(aMoviePlayer.asImage(), "Display");
		}
		aMoviePlayer.prologue();

		return true;
	}

	/**
	 * Example5: Open a movie and play it.
	 * 
	 * @return boolean
	 */
	public static boolean Example5() {
		JunMoviePlayer aMoviePlayer = JunMoviePlayer.Request();
		if (aMoviePlayer == null) {
			return false;
		}

		aMoviePlayer.openAndPlay();

		return true;
	}

	/**
	 * Example6: Open a movie, play it until the end, and close the player.
	 * 
	 * @return boolean
	 */
	public static boolean Example6() {
		JunMoviePlayer aMoviePlayer = JunMoviePlayer.Request();
		if (aMoviePlayer == null) {
			return false;
		}

		aMoviePlayer.goto_(0.9);
		aMoviePlayer.openAndPlayAndClose();

		return true;
	}

	/**
	 * Example7: Open a movie without a tracker, play it until the end, and close the player.
	 * 
	 * @return boolean
	 */
	public static boolean Example7() {
		JunMoviePlayer aMoviePlayer = JunMoviePlayer.Request();
		if (aMoviePlayer == null) {
			return false;
		}

		aMoviePlayer.goto_(0.9);
		aMoviePlayer.openAndPlayAndCloseWithoutTracker();

		return true;
	}

	/**
	 * Example8: Open a movie and play it by every 0.5 seconds.
	 * 
	 * @return boolean
	 */
	public static boolean Example8() {
		JunMoviePlayer aMoviePlayer = JunMoviePlayer.Request();
		if (aMoviePlayer == null) {
			return false;
		}

		aMoviePlayer.open();
		aMoviePlayer.prologue();
		while (aMoviePlayer.atEnd() == false) {
			aMoviePlayer.goInSeconds_(0.5);
		}

		return true;
	}

	/**
	 * Example9: Open a movie player with a tracker specified.
	 * 
	 * @return boolean
	 */
	public static boolean Example9() {
		JunMoviePlayer aMoviePlayer = JunMoviePlayer.Request();
		if (aMoviePlayer == null) {
			return false;
		}

		aMoviePlayer.trackerModel().interval_(new double[] { 0.2, 0.8 });
		aMoviePlayer.trackerModel().fixFirstMarker_(true);
		// aMoviePlayer.trackerModel().fixLastMarker_(true);
		aMoviePlayer.open();

		return true;
	}

	/**
	 * ExampleA: Open three types of the movie player.
	 * 
	 * @return boolean
	 */
	public static boolean ExampleA() {
		JunMoviePlayer aMoviePlayer = JunMoviePlayer.Request();
		if (aMoviePlayer == null) {
			return false;
		}

		JunMoviePlayer aMoviePlayerWithoutTracker = (JunMoviePlayer) aMoviePlayer.copy();
		aMoviePlayerWithoutTracker.openViewAt_(aMoviePlayerWithoutTracker.defaultViewWithoutTracker(), new Point(100, 200));
		aMoviePlayerWithoutTracker.normalSize();
		aMoviePlayerWithoutTracker.start();

		JunMoviePlayer aMoviePlayerWithTracker = (JunMoviePlayer) aMoviePlayer.copy();
		aMoviePlayerWithTracker.openViewAt_(aMoviePlayerWithTracker.defaultViewWithTracker(), new Point(250, 200));
		aMoviePlayerWithTracker.normalSize();
		aMoviePlayerWithTracker.start();

		JunMoviePlayer aMoviePlayerWithDoubleTracker = (JunMoviePlayer) aMoviePlayer.copy();
		aMoviePlayerWithDoubleTracker.openViewAt_(aMoviePlayerWithDoubleTracker.defaultViewWithDoubleTracker(), new Point(400, 200));
		aMoviePlayerWithDoubleTracker.normalSize();
		aMoviePlayerWithDoubleTracker.start();

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 */
	public static void main(String[] args) {
		System.setErr(System.out);
		new JunMoviePlayerTestExamples();
	}

	/**
	 * Example3: 
	 * 
	 * @return boolean
	 */
	private static boolean Example3() {
		FileDialog fileDialog = new FileDialog(new Frame(), "Please select a file.");
		fileDialog.show();
		String directory = fileDialog.getDirectory();
		String filename = fileDialog.getFile();
		if ((directory == null) || (filename == null)) {
			return false;
		}

		final JunMoviePlayer aMoviePlayer = JunMoviePlayer.Filename_(directory + filename);
		if (aMoviePlayer == null) {
			return false;
		}

		class ExampleThread extends Thread {
			volatile boolean isLoop = true;
			volatile double nth = 0;
			final double step = 0.1;
			volatile int sign = 1;

			public void interrupt() {
				isLoop = false;
				super.interrupt();
			}

			public void run() {
				while (isLoop) {
					try {
						sleep(100);
						Thread.yield();
					} catch (InterruptedException e) {
						isLoop = false;
					}
					synchronized (aMoviePlayer) {
						aMoviePlayer.goto_(nth);
					}

					if (nth >= 1) {
						sign = -1;
					}

					if (nth <= 0) {
						sign = 1;
					}

					nth = nth + (step * sign);
				}

				return;
			}
		}

		final ExampleThread aThread = new ExampleThread();
		final Frame frame = aMoviePlayer.open();
		final Cursor cursorType = frame.getCursor();
		frame.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				synchronized (aThread) {
					aThread.interrupt();

					try {
						aThread.join();
					} catch (InterruptedException ie) {
						ie.printStackTrace();
					}
				}

				frame.setCursor(cursorType);
				e.getWindow().dispose();
			}
		});
		aThread.setPriority((Thread.MIN_PRIORITY + Thread.NORM_PRIORITY) / 2);
		aThread.start();

		return true;
	}
}
