package sra.jun.goodies.movie;

import java.awt.*;
import sra.smalltalk.*;
import sra.jun.goodies.button.JunButtonViewAwt;
import sra.jun.goodies.track.JunTrackSliderViewAwt;

/**
 * JunMovieViewAwt class
 * 
 *  @author    NISHIHARA Satoshi
 *  @created   2001/01/24 (by NISHIHARA Satoshi)
 *  @updated   2002/11/11 (by nisinaka)
 *  @updated   2003/03/24 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun490 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMovieViewAwt.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunMovieViewAwt extends StViewCanvas implements JunMovieView {

	protected JunTrackSliderViewAwt sliderView;
	protected JunTrackSliderViewAwt sliderView2;
	protected JunButtonViewAwt firstButtonView;
	protected JunButtonViewAwt lastButtonView;
	protected JunButtonViewAwt previousButtonView;
	protected JunButtonViewAwt nextButtonView;
	protected JunButtonViewAwt loopButtonView;
	protected JunButtonViewAwt playButtonView;
	protected JunButtonViewAwt firstMarkerButtonView;
	protected JunButtonViewAwt lastMarkerButtonView;
	protected PopupMenu _popupMenu;

	/**
	 * Create a new instance of JunMovieViewAwt and initialize it.
	 *
	 * @param aMoviePlayer sra.jun.goodies.movie.JunMoviePlayer
	 */
	public JunMovieViewAwt(JunMoviePlayer aMoviePlayer) {
		super(aMoviePlayer);
	}

	/**
	 * Create a new view for the JunMoviePlayer.
	 * The view consists of a JunMovieView and two trackers.
	 *
	 * @return sra.smalltalk.StView
	 * @param aMoviePlayer sra.jun.goodies.movie.JunMoviePlayer
	 */
	public static StView WithDoubleTracker(JunMoviePlayer aMoviePlayer) {
		return new StViewPanel(aMoviePlayer) {

			protected JunMovieViewAwt movieView;

			protected JunMovieViewAwt movieView() {
				if (movieView == null) {
					movieView = new JunMovieViewAwt((JunMoviePlayer) this.model());
				}
				return movieView;
			}

			protected void buildComponent() {
				this.setLayout(new WindowSpecLayout());
				this.add(this.movieView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 1f, -39));
				this.add(this.movieView().sliderView(), WindowSpecLayout.Position(0f, 77, 1f, -19, 1f, -77, 1f, -1));
				this.add(this.movieView().sliderView2(), WindowSpecLayout.Position(0f, 1, 1f, -38, 1f, -1, 1f, -20));
				this.add(this.movieView().firstButtonView(), WindowSpecLayout.Position(0f, 1, 1f, -19, 0f, 19, 1f, -1));
				this.add(this.movieView().lastButtonView(), WindowSpecLayout.Position(0f, 20, 1f, -19, 0f, 38, 1f, -1));
				this.add(this.movieView().previousButtonView(), WindowSpecLayout.Position(0f, 39, 1f, -19, 0f, 57, 1f, -1));
				this.add(this.movieView().nextButtonView(), WindowSpecLayout.Position(0f, 58, 1f, -19, 0f, 76, 1f, -1));
				this.add(this.movieView().loopButtonView(), WindowSpecLayout.Position(1f, -76, 1f, -19, 1f, -58, 1f, -1));
				this.add(this.movieView().playButtonView(), WindowSpecLayout.Position(1f, -57, 1f, -19, 1f, -39, 1f, -1));
				this.add(this.movieView().firstMarkerButtonView(), WindowSpecLayout.Position(1f, -38, 1f, -19, 1f, -20, 1f, -1));
				this.add(this.movieView().lastMarkerButtonView(), WindowSpecLayout.Position(1f, -19, 1f, -19, 1f, -1, 1f, -1));
				this.setSize(260, 180);
			}
		};
	}

	/**
	 * Create a new view for the JunMoviePlayer.
	 * The view consists of a JunMovieView and no tracker.
	 *
	 * @return sra.smalltalk.StView
	 * @param aMoviePlayer sra.jun.goodies.movie.JunMoviePlayer
	 */
	public static StView WithoutTracker(JunMoviePlayer aMoviePlayer) {
		return new StViewPanel(aMoviePlayer) {

			protected JunMovieViewAwt movieView;

			protected JunMovieViewAwt movieView() {
				if (movieView == null) {
					movieView = new JunMovieViewAwt((JunMoviePlayer) this.model());
				}
				return movieView;
			}

			protected void buildComponent() {
				this.setLayout(new WindowSpecLayout());
				this.add(this.movieView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 1f, -1));
				this.setSize(260, 180);
			}
		};
	}

	/**
	 * Create a new view for the JunMoviePlayer.
	 * The view consists of a JunMovieView and a tracker.
	 *
	 * @return sra.smalltalk.StView
	 * @param aMoviePlayer sra.jun.goodies.movie.JunMoviePlayer
	 */
	public static StView WithTracker(JunMoviePlayer aMoviePlayer) {
		return new StViewPanel(aMoviePlayer) {

			protected JunMovieViewAwt movieView;

			protected JunMovieViewAwt movieView() {
				if (movieView == null) {
					movieView = new JunMovieViewAwt((JunMoviePlayer) this.model());
				}
				return movieView;
			}

			protected void buildComponent() {
				this.setLayout(new WindowSpecLayout());
				this.add(this.movieView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 1f, -20));
				this.add(this.movieView().sliderView(), WindowSpecLayout.Position(0f, 39, 1f, -19, 1f, -39, 1f, -1));
				this.add(this.movieView().previousButtonView(), WindowSpecLayout.Position(0f, 1, 1f, -19, 0f, 19, 1f, -1));
				this.add(this.movieView().nextButtonView(), WindowSpecLayout.Position(0f, 20, 1f, -19, 0f, 38, 1f, -1));
				this.add(this.movieView().loopButtonView(), WindowSpecLayout.Position(1f, -38, 1f, -19, 1f, -20, 1f, -1));
				this.add(this.movieView().playButtonView(), WindowSpecLayout.Position(1f, -19, 1f, -19, 1f, -1, 1f, -1));
				this.setSize(260, 180);
			}
		};
	}

	/**
	 * Display the movie.
	 * 
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayOn_(Graphics aGraphics) {
		if (this.isShowing() && this.hasMovie()) {
			this.movieHandle().updateMovie();
			this.movieHandle().doMovieTask();
		}
	}

	/**
	 * Answer my first button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt firstButtonView() {
		if (firstButtonView == null) {
			firstButtonView = new JunButtonViewAwt(this.getMoviePlayer().trackerModel().firstButton());
		}
		return firstButtonView;
	}

	/**
	 * Answer my first marker button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt firstMarkerButtonView() {
		if (firstMarkerButtonView == null) {
			firstMarkerButtonView = new JunButtonViewAwt(this.getMoviePlayer().trackerModel().firstMarkerButton());
		}
		return firstMarkerButtonView;
	}

	/**
	 * Answer my model as a JunMoviePlayer.
	 * 
	 * @return sra.jun.goodies.movie.JunMoviePlayer
	 */
	public JunMoviePlayer getMoviePlayer() {
		return (JunMoviePlayer) this.model();
	}

	/**
	 * Answer true if the view has a movie, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasMovie() {
		return this.getMoviePlayer().hasMovie();
	}

	/**
	 * Ansewr true if the view has a tracker, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasTracker() {
		Object[] dependents = this.getMoviePlayer().trackerModel().dependents();
		for (int i = 0; i < dependents.length; i++) {
			if (dependents[i] instanceof JunTrackSliderViewAwt) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Ansewr true if the view has a second tracker, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasTracker2() {
		Object[] dependents = this.getMoviePlayer().trackerModel2().dependents();
		for (int i = 0; i < dependents.length; i++) {
			if (dependents[i] instanceof JunTrackSliderViewAwt) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Answer my last button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt lastButtonView() {
		if (lastButtonView == null) {
			lastButtonView = new JunButtonViewAwt(this.getMoviePlayer().trackerModel().lastButton());
		}
		return lastButtonView;
	}

	/**
	 * Answer my last marker button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt lastMarkerButtonView() {
		if (lastMarkerButtonView == null) {
			lastMarkerButtonView = new JunButtonViewAwt(this.getMoviePlayer().trackerModel().lastMarkerButton());
		}
		return lastMarkerButtonView;
	}

	/**
	 * Create a loop button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt loopButtonView() {
		if (loopButtonView == null) {
			loopButtonView = new JunButtonViewAwt(this.getMoviePlayer().trackerModel().loopButton());
		}
		return loopButtonView;
	}

	/**
	 * Answer the current movie handle.
	 * 
	 * @return sra.jun.goodies.JunMovieHandle
	 */
	public JunMovieHandle movieHandle() {
		return this.getMoviePlayer().movieHandle();
	}

	/**
	 * Create a next button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt nextButtonView() {
		if (nextButtonView == null) {
			nextButtonView = new JunButtonViewAwt(this.getMoviePlayer().trackerModel().nextButton());
		}
		return nextButtonView;
	}

	/**
	 * Answer my play button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt playButtonView() {
		if (playButtonView == null) {
			playButtonView = new JunButtonViewAwt(this.getMoviePlayer().trackerModel().playButton());
		}
		return playButtonView;
	}

	/**
	 * Answer my previous button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt previousButtonView() {
		if (previousButtonView == null) {
			previousButtonView = new JunButtonViewAwt(this.getMoviePlayer().trackerModel().previousButton());
		}
		return previousButtonView;
	}

	/**
	 * Set the bounds of the view.
	 * Also needs to set the movie for the new bounds.
	 *
	 * @param x The new <i>x</i>-coordinate of this component.
	 * @param y The new <i>y</i>-coordinate of this component.
	 * @param width The new <code>width</code> of this component.
	 * @param height The new <code>height</code> of this component.
	 * @see java.awt.Component#setBounds(int, int, int, int)
	 */
	public void setBounds(int x, int y, int width, int height) {
		super.setBounds(x, y, width, height);
		this.setMovieView();
	}

	/**
	 * Set the movie view.
	 */
	public void setMovieView() {
		if (this.topComponent() == null) {
			return;
		}

		if (this.hasMovie()) {
			this.movieHandle().movieGrafPtr_(this.getMoviePlayer().grafPtr());
			this.movieHandle().movieBox_(this.movieBox());
			this.movieHandle().active_(true);
		}
	}

	/**
	 * Answer my first slider view.
	 *
	 * @return sra.jun.goodies.track.JunTrackSliderViewAwt
	 */
	public JunTrackSliderViewAwt sliderView() {
		if (sliderView == null) {
			sliderView = new JunTrackSliderViewAwt(this.getMoviePlayer().trackerModel());
		}
		return sliderView;
	}

	/**
	 * Answer my second slider view.
	 *
	 * @return sra.jun.goodies.track.JunTrackSliderViewAwt
	 */
	public JunTrackSliderViewAwt sliderView2() {
		if (sliderView2 == null) {
			sliderView2 = new JunTrackSliderViewAwt(this.getMoviePlayer().trackerModel2());
		}
		return sliderView2;
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		super.setBounds(0, 0, 240, 180);
	}

	/**
	 * Answer my default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunMoviePlayer();
	}

	/**
	 * Answer the current area of the movie.
	 *
	 * @return java.awt.Rectangle
	 */
	protected Rectangle movieBox() {
		if (this.getMoviePlayer().keepAspect() && this.hasMovie()) {
			Rectangle movieOriginalBox = new Rectangle(new Point(0, 0), this.movieHandle().originalExtent());
			Rectangle movieViewBox = new Rectangle(0, 0, this.getWidth(), this.getHeight());
			return this.getMoviePlayer().AlignRectangle_withRectangle_(movieOriginalBox, movieViewBox);
		}

		return new Rectangle(0, 0, this.getWidth(), this.getHeight());
	}

	/**
	 * Answer the popup menu.
	 *
	 * @return java.awt.PopupMenu
	 */
	protected PopupMenu popupMenu() {
		if (_popupMenu != null) {
			this.remove(_popupMenu);
		}

		_popupMenu = this._createPopupMenu(this.getMoviePlayer().popupMenu());
		this.add(_popupMenu);
		return _popupMenu;
	}
}
