package sra.jun.goodies.movie;

import java.awt.*;
import java.io.*;
import sra.jun.goodies.images.streams.*;
import sra.jun.goodies.progress.JunProgress;
import sra.smalltalk.*;

/**
 * JunScreenRecorder class
 * 
 *  @author    Hoshi Takanori
 *  @created   2002/10/18 (by Hoshi Takanori)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunScreenRecorder.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunScreenRecorder extends StApplicationModel implements Runnable {
    protected File fileName;
    protected Rectangle movieArea;
    protected Dimension movieExtent;
    protected int watchTick = 1000;
    protected boolean watchState = false;
    protected volatile Thread watchThread = null;
    protected File imagesFile;
    protected int imagesCount;
    protected volatile boolean watchDone;
    protected StMenuModel menuModel;

    /**
     * Answer movie area.
     * 
     * @return java.awt.Rectangle
     */
    public Rectangle getMovieArea() {
        if (movieArea == null) {
            movieArea = new Rectangle(
                    Toolkit.getDefaultToolkit().getScreenSize());
        }
        return movieArea;
    }

    /**
     * Set movie area.
     * 
     * @param aRectangle java.awt.Rectangle
     */
    public void setMovieArea(Rectangle aRectangle) {
        movieArea = aRectangle;
        this.changed_($("movieArea"));

        if (movieExtent != null) {
            this.setMovieExtent(new Dimension(movieExtent.width,
                    movieArea.height * movieExtent.width / movieArea.width));
        }
    }

    /**
     * Answer movie extent.
     * 
     * @return java.awt.Dimension
     */
    public Dimension getMovieExtent() {
        if (movieExtent == null) {
            movieExtent = new Dimension(320, 240);
        }
        return movieExtent;
    }

    /**
     * Set movie extent.
     * 
     * @param aDimension java.awt.Dimension
     */
    public void setMovieExtent(Dimension aDimension) {
        movieExtent = aDimension;
        this.changed_($("movieExtent"));
    }

    /**
     * Answer watch tick.
     * 
     * @return int
     */
    public int getWatchTick() {
        return watchTick;
    }

    /**
     * Set watch tick.
     * 
     * @param milliseconds int
     */
    public void setWatchTick(int milliseconds) {
        watchTick = milliseconds;
        this.changed_($("watchTick"));
    }

    /**
     * Answer watch State.
     * 
     * @return boolean
     */
    public boolean getWatchState() {
        return watchState;
    }

    /**
     * Answer a default view.
     * 
     * @return sra.smalltalk.StView
     */
    public StView defaultView() {
        if (GetDefaultViewMode() == VIEW_AWT) {
            return new JunScreenRecorderViewAwt(this);
        } else {
            return new JunScreenRecorderViewSwing(this);
        }
    }

    /**
     * Answer the menu model.
     * 
     * @return sra.smalltalk.StMenuModel
     */
    public StMenuModel getMenuModel() {
        if (menuModel == null) {
            menuModel = new StMenuModel();

            StMenu areaMenu = new StMenu("Area");
            areaMenu.add(createMenuItem("whole screen", "areaWholeScreen"));
            areaMenu.addSeparator();
            areaMenu.add(createMenuItem("from you ...", "areaFromYou"));
            menuModel.add(areaMenu);

            StMenu sizeMenu = new StMenu("Size");
            sizeMenu.add(createMenuItem("320@240 (half)", "sizeHalf"));
            sizeMenu.add(createMenuItem("160@120 (quart)", "sizeQuart"));
            sizeMenu.add(createMenuItem("640@480 (full)", "sizeFull"));
            sizeMenu.addSeparator();
            sizeMenu.add(createMenuItem("area (as is)", "sizeAsIs"));
            sizeMenu.add(createMenuItem("area / 2", "sizeDiv2"));
            sizeMenu.add(createMenuItem("area / 4", "sizeDiv4"));
            sizeMenu.add(createMenuItem("area / 8", "sizeDiv8"));
            menuModel.add(sizeMenu);

            StMenu speedMenu = new StMenu("Speed");
            speedMenu.add(createMenuItem("100 msec. (10 frames per sec.)", "speed100msec"));
            speedMenu.add(createMenuItem("200 msec. (5 frames per sec.)", "speed200msec"));
            speedMenu.add(createMenuItem("500 msec. (2 frames per sec.)", "speed500msec"));
            speedMenu.add(createMenuItem("1 sec. (1 frame per sec.)", "speed1sec"));
            speedMenu.add(createMenuItem("2 sec. (30 frames per min.)", "speed2sec"));
            speedMenu.add(createMenuItem("5 sec. (12 frames per min.)", "speed5sec"));
            speedMenu.add(createMenuItem("10 sec. (6 frames per min.)", "speed10sec"));
            speedMenu.add(createMenuItem("15 sec. (4 frames per min.)", "speed15sec"));
            speedMenu.add(createMenuItem("20 sec. (3 frames per min.)", "speed20sec"));
            speedMenu.add(createMenuItem("30 sec. (2 frames per min.)", "speed30sec"));
            speedMenu.add(createMenuItem("1 min. (1 frame per min.)", "speed1min"));
            menuModel.add(speedMenu);
        }

        return menuModel;
    }

    /**
     * Return record/stop button label.
     * 
     * @return java.lang.String
     */
    public String getRecordButtonLabel() {
        return watchState == false ? "Record" : "Stop";
    }

    /**
     * Return area button label.
     * 
     * @return java.lang.String
     */
    public String getAreaButtonLabel() {
        Rectangle rect = this.getMovieArea();
        return "area: (" + rect.x + "@" + rect.y + " corner: "
                + (rect.x + rect.width) + "@" + (rect.y + rect.height) + ")";
    }

    /**
     * Return size button label.
     * 
     * @return java.lang.String
     */
    public String getSizeButtonLabel() {
        Dimension size = this.getMovieExtent();
        return "size: (" + size.width + "@" + size.height + ")";
    }

    /**
     * Return speed button label.
     * 
     * @return java.lang.String
     */
    public String getSpeedButtonLabel() {
        if (watchTick < 1000) {
            return "speed: " + watchTick + " msec.";
        } else if (watchTick < 60000) {
            return "speed: " + (watchTick / 1000) + " sec.";
        } else {
            return "speed: " + (watchTick / 60000) + " min.";
        }
    }

    /**
     * Set area to the whole screen.
     */
    public void areaWholeScreen() {
        this.setMovieArea(
                new Rectangle(Toolkit.getDefaultToolkit().getScreenSize()));
    }

    /**
     * Set area from user.
     */
    public void areaFromYou() {
        Rectangle rect = SystemInterface._RectangleFromUser();

        if (rect != null) {
            this.setMovieArea(rect);
        }
    }

    /**
     * Set size to "half" (320 x 240).
     */
    public void sizeHalf() {
        this.setMovieExtent(new Dimension(320, 240));
    }

    /**
     * Set size to "quart" (160 x 120).
     */
    public void sizeQuart() {
        this.setMovieExtent(new Dimension(160, 120));
    }

    /**
     * Set size to "full" (640 x 480).
     */
    public void sizeFull() {
        this.setMovieExtent(new Dimension(640, 480));
    }

    /**
     * Set size to area.
     */
    public void sizeAsIs() {
        this.setMovieExtent(getMovieArea().getSize());
    }

    /**
     * Set size to area / 2.
     */
    public void sizeDiv2() {
        Dimension size = getMovieArea().getSize();
        this.setMovieExtent(new Dimension(size.width / 2, size.height / 2));
    }

    /**
     * Set size to area / 4.
     */
    public void sizeDiv4() {
        Dimension size = getMovieArea().getSize();
        this.setMovieExtent(new Dimension(size.width / 4, size.height / 4));
    }

    /**
     * Set size to area / 8.
     */
    public void sizeDiv8() {
        Dimension size = getMovieArea().getSize();
        this.setMovieExtent(new Dimension(size.width / 8, size.height / 8));
    }

    /**
     * Set speed to 100 msec.
     */
    public void speed100msec() {
        this.setWatchTick(100);
    }

    /**
     * Set speed to 200 msec.
     */
    public void speed200msec() {
        this.setWatchTick(200);
    }

    /**
     * Set speed to 500 msec.
     */
    public void speed500msec() {
        this.setWatchTick(500);
    }

    /**
     * Set speed to 1 sec.
     */
    public void speed1sec() {
        this.setWatchTick(1000);
    }

    /**
     * Set speed to 2 sec.
     */
    public void speed2sec() {
        this.setWatchTick(2000);
    }

    /**
     * Set speed to 5 sec.
     */
    public void speed5sec() {
        this.setWatchTick(5000);
    }

    /**
     * Set speed to 10 sec.
     */
    public void speed10sec() {
        this.setWatchTick(10000);
    }

    /**
     * Set speed to 15 sec.
     */
    public void speed15sec() {
        this.setWatchTick(15000);
    }

    /**
     * Set speed to 20 sec.
     */
    public void speed20sec() {
        this.setWatchTick(20000);
    }

    /**
     * Set speed to 30 sec.
     */
    public void speed30sec() {
        this.setWatchTick(30000);
    }

    /**
     * Set speed to 1 min.
     */
    public void speed1min() {
        this.setWatchTick(60000);
    }

    /**
     * Runner.
     */
    public void run() {
        Thread thisThread = Thread.currentThread();
        ObjectOutputStream stream;

        try {
            imagesFile = File.createTempFile("images", ".tmp");
            stream = new ObjectOutputStream(new FileOutputStream(imagesFile));
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

        watchDone = false;
        imagesCount = 0;

        while (watchThread == thisThread) {
            int[] pixels;

            SystemInterface StPLInterface = SystemInterface.Current();
            synchronized (StPLInterface) {
                Rectangle rect = this.getMovieArea();
                pixels = StPLInterface.utilImageOfArea(
                        rect.x, rect.y, rect.width, rect.height);
            }

            try {
                stream.writeObject(pixels);
                stream.reset();
            } catch (IOException e) {
                e.printStackTrace();
                watchDone = true;
                return;
            }

            ++imagesCount;

            try {
                Thread.sleep(watchTick);
            } catch (InterruptedException e) {
            }
        }

        try {
            stream.writeObject(null);
            stream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }

        watchDone = true;
    }

    /**
     * Make movie file.
     */
    public void makeMovieFile() throws Exception {
        final ObjectInputStream stream =
                new ObjectInputStream(new FileInputStream(imagesFile));

        final JunImagesToMovie imagesToMovie =
                new JunImagesToMovie(fileName.getPath(), this.getMovieExtent());

        final JunProgress progress = new JunProgress();
        progress.message_("make movie...");

        imagesToMovie.do_(new StBlockClosure() {
            public Object value() {
                progress.do_(new StBlockClosure() {
                    public Object value() {
                        int[] pixels;
                        int i = 0;
                        try {
                            while ((pixels = (int[]) stream.readObject()) != null) {
                                Dimension size = getMovieArea().getSize();
                                imagesToMovie.addPixels_width_height_milliseconds_(
                                        pixels, size.width, size.height, watchTick);
                                progress.value_((float) ++i / imagesCount);
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                        return null;
                    }
                });
                return null;
            }
        });

        stream.close();

        imagesFile.delete();
        imagesFile = null;
    }

    /**
     * Process record/stop button.
     */
    public void recordButtonPressed() {
        if (watchState == false && watchThread == null) {
            watchState = true;
            this.changed_($("watchState"));

            fileName = StDialog.SaveFileDialog();
            if (fileName != null) {
                watchThread = new Thread(this);
                watchThread.start();
            } else {
                watchState = false;
                this.changed_($("watchState"));
            }
        } else if (watchState && watchThread != null) {
            Thread thread = watchThread;
            watchThread = null;
            thread.interrupt();

            while (! watchDone) {
                try {
                    Thread.sleep(100);
                } catch (InterruptedException e) {
                }
            }

            try {
                makeMovieFile();
            } catch (Exception e) {
                e.printStackTrace();
            }

            watchState = false;
            this.changed_($("watchState"));
        }
    }

    /**
     * Process snap button.
     */
    public void snapButtonPressed() {
        StImage image = StImage._OfArea(this.getMovieArea());

        File file = StDialog.SaveFileDialog();
        if (file == null) {
            return;
        }

        try {
            OutputStream stream = new FileOutputStream(file);
            JunImageStream imageStream = JunJpegImageStream.On_(stream);
            imageStream.nextPutImage_(image);
            imageStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
