package sra.jun.goodies.movie;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import sra.smalltalk.*;

/**
 * JunScreenRecorderViewSwing class
 * 
 *  @author    Hoshi Takanori
 *  @created   2002/10/18 (by Hoshi Takanori)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunScreenRecorderViewSwing.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunScreenRecorderViewSwing extends StViewJPanel implements JunScreenRecorderView, ActionListener {

    protected JButton recordButton;
    protected JButton areaButton;
    protected JButton sizeButton;
    protected JButton speedButton;
    protected JButton snapButton;
    protected JPopupMenu areaPopupMenu;
    protected JPopupMenu sizePopupMenu;
    protected JPopupMenu speedPopupMenu;

    /**
     * Create a new instance of JunScreenRecorderViewSwing and initialize it.
     *
     * @param aScreenRecorder sra.jun.goodies.movie.JunScreenRecorder
     */
    public JunScreenRecorderViewSwing(JunScreenRecorder aScreenRecorder) {
        super(aScreenRecorder);
    }

    /**
     * Actions for the event.
     * 
     * @param e java.awt.event.ActionEvent
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == recordButton) {
            getScreenRecorder().recordButtonPressed();
        } else if (e.getSource() == areaButton) {
            if (areaPopupMenu == null) {
                areaPopupMenu = this._createPopupMenu((StMenu) getScreenRecorder().getMenuModel().getItem("Area"));
                this.add(areaPopupMenu);
            }
            areaPopupMenu.show(areaButton, 10, 10);
        } else if (e.getSource() == sizeButton) {
            if (sizePopupMenu == null) {
                sizePopupMenu = this._createPopupMenu((StMenu) getScreenRecorder().getMenuModel().getItem("Size"));
                this.add(sizePopupMenu);
            }
            sizePopupMenu.show(sizeButton, 10, 10);
        } else if (e.getSource() == speedButton) {
            if (speedPopupMenu == null) {
                speedPopupMenu = this._createPopupMenu((StMenu) getScreenRecorder().getMenuModel().getItem("Speed"));
                this.add(speedPopupMenu);
            }
            speedPopupMenu.show(speedButton, 10, 10);
        } else if (e.getSource() == snapButton) {
            getScreenRecorder().snapButtonPressed();
        }
    }

    /**
     * Answer insets.
     * 
     * @return java.awt.Insets
     */
    public Insets getInsets() {
        return new Insets(6, 6, 6, 6);
    }

    /**
     * Answer the preferred size of this component.
     * 
     * @return java.awt.Dimension
     */
    public Dimension getPreferredSize() {
        return new Dimension(250, 150);
    }

    /**
     * Answer my model as JunScreenRecorder.
     *
     * @return sra.jun.goodies.movie.JunScreenRecorder
     */
    public JunScreenRecorder getScreenRecorder() {
        return (JunScreenRecorder) this.model();
    }

    /**
     * DOCUMENT ME!
     * 
     * @param evt sra.smalltalk.DependentEvent
     */
    public void update_(DependentEvent evt) {
        StSymbol anAspect = evt.getAspect();
    
        if (anAspect == $("watchState")) {
            this.updateRecordButtonLabel();
        } else if (anAspect == $("movieArea")) {
            this.updateAreaButtonLabel();
        } else if (anAspect == $("movieExtent")) {
            this.updateSizeButtonLabel();
        } else if (anAspect == $("watchTick")) {
            this.updateSpeedButtonLabel();
        } else {
            super.update_(evt);
        }
    }

    /**
     * Update area button label.
     */
    public void updateAreaButtonLabel() {
        areaButton.setText(getScreenRecorder().getAreaButtonLabel());
    }

    /**
     * Update record/stop button label.
     */
    public void updateRecordButtonLabel() {
        recordButton.setText(getScreenRecorder().getRecordButtonLabel());
    
        areaButton.setEnabled(!getScreenRecorder().getWatchState());
        sizeButton.setEnabled(!getScreenRecorder().getWatchState());
        speedButton.setEnabled(!getScreenRecorder().getWatchState());
        snapButton.setEnabled(!getScreenRecorder().getWatchState());
    }

    /**
     * Update size button label.
     */
    public void updateSizeButtonLabel() {
        sizeButton.setText(getScreenRecorder().getSizeButtonLabel());
    }

    /**
     * Update speed button label.
     */
    public void updateSpeedButtonLabel() {
        speedButton.setText(getScreenRecorder().getSpeedButtonLabel());
    }

    /**
     * Build this component.
     */
    protected void buildComponent() {
        this.setBackground(new Color(0xff, 0xff, 0xbf));
        this.setLayout(new GridLayout(0, 1, 2, 2));
    
        recordButton = new JButton("Record");
        recordButton.setBackground(new Color(0xff, 0xbf, 0xbf));
        recordButton.addActionListener(this);
        this.add(recordButton);
    
        areaButton = new JButton("Area");
        areaButton.setBackground(new Color(0xbf, 0xff, 0xbf));
        areaButton.addActionListener(this);
        this.add(areaButton);
    
        sizeButton = new JButton("Size");
        sizeButton.setBackground(new Color(0xbf, 0xff, 0xbf));
        sizeButton.addActionListener(this);
        this.add(sizeButton);
    
        speedButton = new JButton("Speed");
        speedButton.setBackground(new Color(0xbf, 0xff, 0xbf));
        speedButton.addActionListener(this);
        this.add(speedButton);
    
        snapButton = new JButton("Snap");
        snapButton.setBackground(new Color(0xbf, 0xbf, 0xff));
        snapButton.addActionListener(this);
        this.add(snapButton);
    
        this.updateRecordButtonLabel();
        this.updateAreaButtonLabel();
        this.updateSizeButtonLabel();
        this.updateSpeedButtonLabel();
    }
}
