package sra.jun.goodies.multimedia;

import java.awt.*;
import java.io.File;
import sra.smalltalk.*;
import sra.jun.goodies.display.JunDisplayModel;
import sra.jun.goodies.movie.JunMoviePlayer;

/**
 * JunMultiMediaImageModel class
 * 
 *  @author    nisinaka
 *  @created   2003/03/18 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun490 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMultiMediaImageModel.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunMultiMediaImageModel extends JunDisplayModel implements JunMultiMediaViewable {

	/**
	 * Create a new instance of JunMultiMediaImageModel.
	 */
	public JunMultiMediaImageModel() {
		super();
	}

	/**
	 * Create a new instance of JunMultiMediaImageModel and initialize it.
	 *
	 * @param aFile java.io.File
	 */
	public JunMultiMediaImageModel(File aFile) {
		Object aVisual;
		if (aFile.exists()) {
			JunMoviePlayer aMoviePlayer = new JunMoviePlayer(aFile);
			try {
				if (aMoviePlayer.hasMovie()) {
					aVisual = aMoviePlayer.asImage();
				} else {
					aVisual = new StComposedText("Unexpected image file");
				}
			} finally {
				aMoviePlayer.release();
			}
		} else {
			aVisual = new StComposedText("Unexpected image file");
		}

		this.visual_(aVisual);
	}

	/**
	 * Make be invisible.
	 */
	public void beInvisible() {
		Object[] dependents = this.dependents();
		for (int i = 0; i < dependents.length; i++) {
			if (dependents[i] instanceof JunMultiMediaImageView) {
				((JunMultiMediaImageView) dependents[i]).toComponent().setVisible(false);
			}
		}
	}

	/**
	 * Make be visible.
	 */
	public void beVisible() {
		Object[] dependents = this.dependents();
		for (int i = 0; i < dependents.length; i++) {
			if (dependents[i] instanceof JunMultiMediaImageView) {
				((JunMultiMediaImageView) dependents[i]).toComponent().setVisible(true);
			}
		}
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public sra.smalltalk.StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunMultiMediaImageViewAwt(this);
		} else {
			return new JunMultiMediaImageViewSwing(this);
		}
	}

	/**
	 * Answer true if the imag model is embedded, otherwise false.
	 *
	 * @return boolean
	 */
	public boolean isEmbedded() {
		StView aView = this.getView();
		if (aView != null && aView.toComponent().getParent() instanceof Window) {
			return false;
		}

		return true;
	}

	/**
	 * Menu messages: Spawn the image.
	 */
	public void spawnImage() {
		try {
			JunMultiMediaImageModel imageModel = (JunMultiMediaImageModel) this.getClass().newInstance();
			imageModel.visual_(this.visual());
			StView aView = this.getView();
			if (aView == null) {
				imageModel.open();
			} else {
				Point location = aView.toComponent().getLocationOnScreen();
				Point offsetPoint = this.defaultOffsetPoint();
				location.translate(offsetPoint.x, offsetPoint.y);
				imageModel.openViewAt_(imageModel.defaultView(), location);
			}
		} catch (InstantiationException e) {
		} catch (IllegalAccessException e) {
		}
	}

	/**
	 * Create a popup menu.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu popupMenu() {
		if (_popupMenu == null) {
			StMenuModel rootMenu = new StMenuModel();
			StMenu popupMenu = new StMenu("");
			popupMenu.add(createMenuItem("Spawn", "spawnImage"));
			rootMenu.add(popupMenu);
			_popupMenu = popupMenu;
		}
		return _popupMenu;
	}

	/**
	 * Set the size for the view.
	 * 
	 * @param extent java.awt.Dimension
	 */
	protected void setSize_(Dimension extent) {
		if (this.isEmbedded()) {
			return;
		}

		super.setSize_(extent);
	}

	/**
	 * Process the change of my visual object.
	 */
	protected void visualIsChanged() {
		this.changed_($("visual"));
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "Image";
	}
}
