package sra.jun.goodies.multimedia;

import java.awt.*;
import java.io.File;
import sra.smalltalk.StView;
import sra.jun.goodies.movie.*;

/**
 * JunMultiMediaMovieModel class
 * 
 *  @author    nisinaka
 *  @created   2003/03/18 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun490 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMultiMediaMovieModel.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunMultiMediaMovieModel extends JunMoviePlayer implements JunMultiMediaViewable {

	/**
	 * Create a new instance of JunMultiMediaMovieModel.
	 */
	public JunMultiMediaMovieModel() {
		super();
	}

	/**
	 * Create a new instance of JunMultiMediaMovieModel and initialize it.
	 *
	 * @param aFile java.io.File
	 */
	public JunMultiMediaMovieModel(File aFile) {
		super(aFile);
	}

	/**
	 * Make be invisible.
	 */
	public void beInvisible() {
		if (this.isPlay()) {
			this.stop();
		}
		if (this.hasMovie()) {
			this.stop();
			this.movieHandle().release();
		}

		Object[] dependents = this.dependents();
		for (int i = 0; i < dependents.length; i++) {
			if (dependents[i] instanceof JunMultiMediaMovieView) {
				((JunMultiMediaMovieView) dependents[i]).toComponent().setVisible(false);
			}
		}
	}

	/**
	 * Make be visible.
	 */
	public void beVisible() {
		Object[] dependents = this.dependents();
		for (int i = 0; i < dependents.length; i++) {
			if (dependents[i] instanceof JunMultiMediaMovieView) {
				JunMultiMediaMovieView aView = (JunMultiMediaMovieView) dependents[i];
				aView.setMovieView();
				aView.toComponent().setVisible(true);
			}
		}
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunMultiMediaMovieViewAwt(this);
		} else {
			return new JunMultiMediaMovieViewSwing(this);
		}
	}

	/**
	 * Hide the tracker.
	 */
	public void hideTracker() {
		if (this.isEmbedded()) {
			return;
		}

		super.hideTracker();
	}

	/**
	 * Answer true if a view of the image model is embedded, otherwise false.
	 *
	 * @return boolean
	 */
	public boolean isEmbedded() {
		return this.builder().windows().length == 0;
	}

	/**
	 * Show the tracker.
	 */
	public void showTracker() {
		if (this.isEmbedded()) {
			return;
		}

		super.showTracker();
	}

	/**
	 * Answer one of the movie views.
	 * Override the method of JunMoviePlayer since JunMultiMediaMovieView is a Panel that contains the real movie view.
	 * 
	 * @return sra.jun.goodies.movie.JunMovieView
	 */
	protected JunMovieView getMovieView() {
		JunMultiMediaMovieView aView = this.getMultiMediaMovieView();
		return (aView == null) ? null : aView.getMovieView();
	}

	/**
	 * Answer one of the multi media movie views.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaMovieView
	 */
	protected JunMultiMediaMovieView getMultiMediaMovieView() {
		Object[] dependents = this.dependents();
		for (int i = 0; i < dependents.length; i++) {
			if (dependents[i] instanceof JunMultiMediaMovieView) {
				return (JunMultiMediaMovieView) dependents[i];
			}
		}

		return null;
	}

	/**
	 * Initialize the receiver.
	 */
	protected void initialize() {
		super.initialize();
		this.trackerModel().enableMarkers();
	}

	/**
	 * Set the size for the view.
	 * 
	 * @param extent java.awt.Dimension
	 */
	protected void setSize_(Dimension extent) {
		if (this.isEmbedded()) {
			if (this.hasMovie() == false) {
				return;
			}
			JunMovieView aView = this.getMovieView();
			if (aView == null) {
				return;
			}
			aView.setMovieView();
			return;
		}

		super.setSize_(extent);
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		if (this.hasMovie()) {
			if (this.isSound()) {
				return "Sound";
			}
		}
		return "Movie";
	}
}
