package sra.jun.goodies.multimedia;

import sra.smalltalk.StViewPanel;
import sra.jun.goodies.movie.*;

/**
 * JunMultiMediaMovieViewAwt class
 * 
 *  @author    nisinaka
 *  @created   2003/03/24 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun490 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMultiMediaMovieViewAwt.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunMultiMediaMovieViewAwt extends StViewPanel implements JunMultiMediaMovieView {

	protected JunMovieViewAwt movieView;

	/**
	 * Create a new instance of JunMultiMediaMovieViewAwt and initialize it.
	 *
	 * @param aMovieModel sra.jun.goodies.multimedia.JunMultiMediaMovieModel
	 */
	public JunMultiMediaMovieViewAwt(JunMultiMediaMovieModel aMovieModel) {
		super(aMovieModel);
	}

	/**
	 * Answer my model as a JunMoviePlayer.
	 * 
	 * @return sra.jun.goodies.movie.JunMoviePlayer
	 */
	public JunMoviePlayer getMoviePlayer() {
		return (JunMoviePlayer) this.model();
	}

	/**
	 * Answer my movie view as JunMovieView.
	 *
	 * @return sra.jun.goodies.movie.JunMovieView
	 */
	public JunMovieView getMovieView() {
		return (JunMovieView) this.movieView();
	}

	/**
	 * Ansewr true if the view has a movie, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasMovie() {
		return this.movieView().hasMovie();
	}

	/**
	 * Ansewr true if the view has a tracker, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasTracker() {
		return this.movieView().hasTracker();
	}

	/**
	 * Ansewr true if the view has a second tracker, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasTracker2() {
		return this.movieView().hasTracker2();
	}

	/**
	 * Answer my movie handle.
	 * 
	 * @return sra.jun.goodies.JunMovieHandle
	 */
	public JunMovieHandle movieHandle() {
		return this.movieView().movieHandle();
	}

	/**
	 * Answer my movie view.
	 *
	 * @return sra.jun.goodies.movie.JunMovieViewAwt
	 */
	public JunMovieViewAwt movieView() {
		if (movieView == null) {
			movieView = new JunMovieViewAwt(this.getMoviePlayer());
		}

		return movieView;
	}

	/**
	 * Set the bounds of myself and my components.
	 *
	 * @param x int
	 * @param y int
	 * @param width int
	 * @param height int
	 */
	public void setBounds(int x, int y, int width, int height) {
		super.setBounds(x, y, width, height);

		this.movieView().setBounds(1, 1, width - 1 - 1, height - 39 - 1);
		this.movieView().sliderView().setBounds(77, height - 19, width - 77 - 77, 18);
		this.movieView().sliderView2().setBounds(1, height - 38, width - 1 - 1, 18);
		this.movieView().firstButtonView().setBounds(1, height - 19, 18, 18);
		this.movieView().lastButtonView().setBounds(20, height - 19, 18, 18);
		this.movieView().previousButtonView().setBounds(39, height - 19, 18, 18);
		this.movieView().nextButtonView().setBounds(58, height - 19, 18, 18);
		this.movieView().loopButtonView().setBounds(width - 76, height - 19, 18, 18);
		this.movieView().playButtonView().setBounds(width - 57, height - 19, 18, 18);
		this.movieView().firstMarkerButtonView().setBounds(width - 38, height - 19, 18, 18);
		this.movieView().lastMarkerButtonView().setBounds(width - 19, height - 19, 18, 18);
	}

	/**
	 * Set the movie view.
	 */
	public void setMovieView() {
		this.movieView().setMovieView();
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(null);
		this.add(this.movieView());
		this.add(this.movieView().sliderView());
		this.add(this.movieView().sliderView2());
		this.add(this.movieView().firstButtonView());
		this.add(this.movieView().lastButtonView());
		this.add(this.movieView().previousButtonView());
		this.add(this.movieView().nextButtonView());
		this.add(this.movieView().loopButtonView());
		this.add(this.movieView().playButtonView());
		this.add(this.movieView().firstMarkerButtonView());
		this.add(this.movieView().lastMarkerButtonView());
		this.setSize(260, 180);
	}
}
