package sra.jun.goodies.multimedia;

import java.awt.Frame;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.*;
import sra.smalltalk.*;
import sra.jun.goodies.files.JunFileModel;
import sra.jun.goodies.movie.JunMovieHandle;
import sra.jun.system.framework.*;
import sra.jun.system.support.JunSystem;

/**
 * JunMultiMediaViewer class
 * 
 *  @author    nisinaka
 *  @created   2003/03/18 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun486 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMultiMediaViewer.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunMultiMediaViewer extends JunApplicationModel {

	protected JunFileModel fileModel;
	protected JunMultiMediaImageModel imageModel;
	protected JunMultiMediaMovieModel movieModel;
	protected JunMultiMediaTextModel textModel;
	protected boolean isMoviePlay;

	/**
	 * Release the resources of the JunMultiMediaViewer.
	 * 
	 * @see sra.smalltalk.StObject#release()
	 * @category initialize-release
	 */
	public void release() {
		this.fileModel().release();
		this.movieModel().release();
		this.imageModel().release();
		this.textModel().release();

		super.release();
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public sra.smalltalk.StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunMultiMediaViewerViewAwt(this);
		} else {
			return new JunMultiMediaViewerViewSwing(this);
		}
	}

	/**
	 * Answer my file model.
	 *
	 * @return sra.jun.goodies.files.JunFileModel
	 */
	public JunFileModel fileModel() {
		if (fileModel == null) {
			Set aSet = new HashSet();
			String[] patterns = JunSystem.DefaultImageExtensionPatterns();
			for (int i = 0; i < patterns.length; i++) {
				aSet.add(patterns[i]);
			}
			patterns = JunSystem.DefaultMovieExtensionPatterns();
			for (int i = 0; i < patterns.length; i++) {
				aSet.add(patterns[i]);
			}
			patterns = JunSystem.DefaultSoundExtensionPatterns();
			for (int i = 0; i < patterns.length; i++) {
				aSet.add(patterns[i]);
			}
			patterns = JunSystem.DefaultTextExtensionPatterns();
			for (int i = 0; i < patterns.length; i++) {
				aSet.add(patterns[i]);
			}
			String[] extensionPatterns = new String[aSet.size()];
			aSet.toArray(extensionPatterns);
			JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType("Multi-media files", extensionPatterns), JunFileModel.FileType.All("All files")};
			fileModel = new JunFileModel();
			fileModel.fileTypes_(fileTypes);
			fileModel.compute_(new StBlockClosure() {
				public Object value_(Object value) {
					checkFile_((File) value);
					return null;
				}
			});
		}
		return fileModel;
	}

	/**
	 * Answer my image model.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaImageModel
	 */
	public JunMultiMediaImageModel imageModel() {
		if (imageModel == null) {
			imageModel = new JunMultiMediaImageModel();
		}
		return imageModel;
	}

	/**
	 * Answer my movie model.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaMovieModel
	 */
	public JunMultiMediaMovieModel movieModel() {
		if (movieModel == null) {
			movieModel = new JunMultiMediaMovieModel();
			movieModel.trackerModel().loop();
		}
		return movieModel;
	}

	/**
	 * Answer my text model.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaTextModel
	 */
	public JunMultiMediaTextModel textModel() {
		if (textModel == null) {
			textModel = new JunMultiMediaTextModel();
		}
		return textModel;
	}

	/**
	 * Invoked when a window is in the process of being closed.
	 * 
	 * @param e java.awt.event.WindowEvent
	 * @see sra.smalltalk.StApplicationModel#noticeOfWindowClose()
	 * @category interface closing
	 */
	public void noticeOfWindowClose(WindowEvent e) {
		this.fileModel().closeRequest();
		this.movieModel().closeRequest();
		this.imageModel().closeRequest();
		this.textModel().closeRequest();
		this.release();
		super.noticeOfWindowClose(e);
	}

	/**
	 * Create a Frame for the application model.
	 * Do everything but open the frame.
	 *
	 * @return java.awt.Frame
	 * @param aView sra.smalltalk.StView
	 */
	protected Frame allButOpenView_(StView aView) {
		this.movieModel()._setKeyListener(this.movieModel().getMultiMediaMovieView());
		return super.allButOpenView_(aView);
	}

	/**
	 * Check the file.
	 *
	 * @param aFile java.io.File
	 */
	protected void checkFile_(File aFile) {
		this.imageModel().beInvisible();
		this.movieModel().beInvisible();
		this.textModel().beInvisible();

		if (aFile == null) {
			return;
		}
		if (aFile.exists() == false) {
			return;
		}
		if (aFile.isDirectory()) {
			return;
		}

		String filename = aFile.getName();
		String extensionString = filename.substring(filename.lastIndexOf('.') + 1).toLowerCase();

		if (Arrays.asList(JunSystem.DefaultMovieExtensions()).contains(extensionString) || Arrays.asList(JunSystem.DefaultSoundExtensions()).contains(extensionString)) {
			// Movie and Sound
			JunMovieHandle aMovieHandle = JunMovieHandle.Filename_(aFile);
			if (aMovieHandle == null) {
				return;
			}

			this.movieModel().movieHandle_(aMovieHandle);
			this.movieModel().normalSize();
			this.movieModel().beVisible();
			this.movieModel().goto_(this.movieModel().now());
			if (this.movieModel().isIntervalNotEmpty()) {
				this.movieModel().trackerModel().firstMarker_(this.movieModel().trackerModel().firstMarker());
				this.movieModel().trackerModel().lastMarker_(this.movieModel().trackerModel().lastMarker());
			}
			if (isMoviePlay) {
				this.movieModel().start();
			}
		} else if (Arrays.asList(JunSystem.DefaultImageExtensions()).contains(extensionString)) {
			// Image
			this.imageModel().visual_((new JunMultiMediaImageModel(aFile)).visual());
			this.imageModel().beVisible();
		} else if (Arrays.asList(JunSystem.DefaultTextExtensions()).contains(extensionString)) {
			// Text
			this.textModel().textHolder().value_((new JunMultiMediaTextModel(aFile)).textHolder().value());
			this.textModel().beVisible();
		} else {
			// Unknown
			Object aString;
			int limitSize = 1024 * 1024 * 4;
			long fileSize = aFile.length();
			if (fileSize > limitSize && JunDialog.Confirm_("The file is large: " + fileSize + "bytes.\nWould you like to dump on the head?")) {
				aString = (new JunMultiMediaTextModel(aFile, limitSize)).textHolder().value();
			} else {
				aString = (new JunMultiMediaTextModel(aFile)).textHolder().value();
			}
			this.textModel().textHolder().value_(aString);
			this.textModel().beVisible();
		}
	}

	/**
	 * Initialize the JunMultiMediaViewer when created.
	 */
	protected void initialize() {
		super.initialize();

		fileModel = null;
		imageModel = null;
		movieModel = null;
		textModel = null;
		isMoviePlay = false;
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "Multi-Media Viewer ";
	}

}
