package sra.jun.goodies.multimedia;

import sra.smalltalk.StViewPanel;
import sra.jun.goodies.files.JunFileViewAwt;

/**
 * JunMultiMediaViewerViewAwt class
 * 
 *  @author    nisinaka
 *  @created   2003/03/18 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun486 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMultiMediaViewerViewAwt.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunMultiMediaViewerViewAwt extends StViewPanel implements JunMultiMediaViewerView {

	protected JunFileViewAwt fileView;
	protected JunMultiMediaImageViewAwt imageView;
	protected JunMultiMediaMovieViewAwt movieView;
	protected JunMultiMediaTextViewAwt textView;

	/**
	 * Create a new instance of JunMultiMediaViewerViewAwt and initialize it.
	 *
	 * @param aMultiMediaViewer sra.jun.goodies.multimedia.JunMultiMediaViewer
	 */
	public JunMultiMediaViewerViewAwt(JunMultiMediaViewer aMultiMediaViewer) {
		super(aMultiMediaViewer);
	}

	/**
	 * Answer my file view.
	 *
	 * @return sra.jun.goodies.files.JunFileViewAwt
	 */
	public JunFileViewAwt fileView() {
		if (fileView == null) {
			fileView = new JunFileViewAwt(this.getMultiMediaViewer().fileModel());
		}
		return fileView;
	}

	/**
	 * Answer my model as JunMultiMediaViewer.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaViewer
	 */
	public JunMultiMediaViewer getMultiMediaViewer() {
		return (JunMultiMediaViewer) this.model();
	}

	/**
	 * Answer my image view.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaImageViewAwt
	 */
	public JunMultiMediaImageViewAwt imageView() {
		if (imageView == null) {
			imageView = new JunMultiMediaImageViewAwt(this.getMultiMediaViewer().imageModel());
			imageView.setVisible(false);
		}
		return imageView;
	}

	/**
	 * Answer my movie view.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaMovieViewAwt
	 */
	public JunMultiMediaMovieViewAwt movieView() {
		if (movieView == null) {
			movieView = new JunMultiMediaMovieViewAwt(this.getMultiMediaViewer().movieModel());
			movieView.setVisible(false);
		}
		return movieView;
	}

	/**
	 * Set the bounds of myself and my components.
	 *
	 * @param x int
	 * @param y int
	 * @param width int
	 * @param height int
	 */
	public void setBounds(int x, int y, int width, int height) {
		super.setBounds(x, y, width, height);

		this.fileView().setBounds(1, 1, width - 2, 290 - 1);
		this.imageView().setBounds(8, 300, width - 8 - 8, height - 8 - 300);
		this.movieView().setBounds(8, 300, width - 8 - 8, height - 8 - 300);
		this.textView().setBounds(8, 300, width - 8 - 8, height - 8 - 300);
	}

	/**
	 * Answer my text view.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaTextViewAwt
	 */
	public JunMultiMediaTextViewAwt textView() {
		if (textView == null) {
			textView = new JunMultiMediaTextViewAwt(this.getMultiMediaViewer().textModel());
			textView.setVisible(false);
		}
		return textView;
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(null);
		this.add(this.fileView());
		this.add(this.imageView());
		this.add(this.movieView());
		this.add(this.textView());
		this.setSize(480, 580);

		// this.getMultiMediaViewer().builder().componentAt_put_($(""), null);
	}
}
