package sra.jun.goodies.multimedia;

import java.awt.Dimension;
import sra.smalltalk.StViewJPanel;
import sra.jun.goodies.files.JunFileViewSwing;

/**
 * JunMultiMediaViewerViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2003/03/20 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun486 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunMultiMediaViewerViewSwing.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunMultiMediaViewerViewSwing extends StViewJPanel implements JunMultiMediaViewerView {

	protected JunFileViewSwing fileView;
	protected JunMultiMediaImageViewSwing imageView;
	protected JunMultiMediaMovieViewSwing movieView;
	protected JunMultiMediaTextViewSwing textView;

	/**
	 * Create a new instance of JunMultiMediaViewerViewSwing and initialize it.
	 *
	 * @param aMultiMediaViewer sra.jun.goodies.multimedia.JunMultiMediaViewer
	 */
	public JunMultiMediaViewerViewSwing(JunMultiMediaViewer aMultiMediaViewer) {
		super(aMultiMediaViewer);
	}

	/**
	 * Answer my file view.
	 *
	 * @return sra.jun.goodies.files.JunFileViewSwing
	 */
	public JunFileViewSwing fileView() {
		if (fileView == null) {
			fileView = new JunFileViewSwing(this.getMultiMediaViewer().fileModel());
		}
		return fileView;
	}

	/**
	 * Answer my model as JunMultiMediaViewer.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaViewer
	 */
	public JunMultiMediaViewer getMultiMediaViewer() {
		return (JunMultiMediaViewer) this.model();
	}

	/**
	 * Answer my image view.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaImageViewSwing
	 */
	public JunMultiMediaImageViewSwing imageView() {
		if (imageView == null) {
			imageView = new JunMultiMediaImageViewSwing(this.getMultiMediaViewer().imageModel());
			imageView.setVisible(false);
		}
		return imageView;
	}

	/**
	 * Answer my movie view.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaMovieViewSwing
	 */
	public JunMultiMediaMovieViewSwing movieView() {
		if (movieView == null) {
			movieView = new JunMultiMediaMovieViewSwing(this.getMultiMediaViewer().movieModel());
			movieView.setVisible(false);
		}
		return movieView;
	}

	/**
	 * Set the bounds of myself and my components.
	 *
	 * @param x int
	 * @param y int
	 * @param width int
	 * @param height int
	 */
	public void setBounds(int x, int y, int width, int height) {
		super.setBounds(x, y, width, height);

		this.fileView().setBounds(1, 1, width - 2, 290 - 1);
		this.imageView().setBounds(8, 300, width - 8 - 8, height - 8 - 300);
		this.movieView().setBounds(8, 300, width - 8 - 8, height - 8 - 300);
		this.textView().setBounds(8, 300, width - 8 - 8, height - 8 - 300);
	}

	/**
	 * Answer my text view.
	 *
	 * @return sra.jun.goodies.multimedia.JunMultiMediaTextViewSwing
	 */
	public JunMultiMediaTextViewSwing textView() {
		if (textView == null) {
			textView = new JunMultiMediaTextViewSwing(this.getMultiMediaViewer().textModel());
			textView.setVisible(false);
		}
		return textView;
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(null);
		this.add(this.fileView());
		this.add(this.imageView());
		this.add(this.movieView());
		this.add(this.textView());
		this.setPreferredSize(new Dimension(480, 580));

		// this.getMultiMediaViewer().builder().componentAt_put_($(""), null);
	}
}
