package sra.jun.goodies.parameter;

import sra.smalltalk.*;
import sra.jun.system.framework.JunApplicationModel;

/**
 * JunParameterModel class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/12/09 (by MATSUDA Ryouichi)
 *  @updated   2003/01/16 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunParameterModel.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunParameterModel extends JunApplicationModel {

	protected JunParameter parameter;
	protected String parameterName;

	/**
	 * Create a new instance of JunParameterModel.
	 */
	public JunParameterModel() {
		super();
	}

	/**
	 * Create a new instance of JunParameterModel and initialize it.
	 *s
	 * @param aParameter sra.jun.goodies.parameter.JunParameter
	 */
	public JunParameterModel(JunParameter aParameter) {
		this();
		this.parameter_(aParameter);
	}

	/**
	 * Create a new instance of JunParameterModel and set a Parameter.
	 * 
	 * @return sra.jun.goodies.parameter.JunParameterModel
	 * @param aParameter sra.jun.goodies.parameter.JunParameter
	 * @deprecated since Jun366, use the constructor.
	 */
	public static JunParameterModel On_(JunParameter aParameter) {
		return new JunParameterModel(aParameter);
	}

	/**
	 * Create a new instance of JunParameterModel and set a Parameter and open it.
	 * 
	 * @return sra.jun.goodies.parameter.JunParameterModel
	 * @param aParameter sra.jun.goodies.parameter.JunParameter
	 */
	public static JunParameterModel OpenOnParameter_(JunParameter aParameter) {
		JunParameterModel aParameterModel = new JunParameterModel(aParameter);
		aParameterModel.open();
		return aParameterModel;
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunParameterViewAwt(this);
		} else {
			return new JunParameterViewSwing(this);
		}
	}

	/**
	 * Answer a label string.
	 * 
	 * @return java.lang.String
	 */
	public String labelString() {
		return this.parameterName() + " : ";
	}

	/**
	 * Answer the parameter.
	 * 
	 * @return sra.jun.goodies.parameter.JunParameter
	 */
	public JunParameter parameter() {
		if (parameter == null) {
			parameter = JunParameter.Name_("parameter");
			parameter.addDependent_(this);
		}

		return parameter;
	}

	/**
	 * Set the parameter.
	 * 
	 * @param aParameter sra.jun.goodies.parameter.JunParameter
	 */
	public void parameter_(JunParameter aParameter) {
		if (parameter != null) {
			parameter.removeDependent_(this);
		}
		parameter = aParameter;
		if (parameter != null) {
			parameter.addDependent_(this);
		}
		this.updateParameter();
	}

	/**
	 * Answer the parameter name.
	 * 
	 * @return java.lang.String
	 */
	public String parameterName() {
		if (parameterName != null) {
			return parameterName;
		} else {
			return this.parameter().name();
		}
	}

	/**
	 * Set the parameter name.
	 * 
	 * @param aString java.lang.String
	 */
	public void parameterName_(String aString) {
		parameterName = aString;
	}

	/**
	 * Answer my preferred height.
	 *
	 * @return int
	 */
	public int preferredHeight() {
		return 40;
	}

	/**
	 * Action for the update notification.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if (evt.getAspect() == $("value")) {
			this.changed_($("value"));
		}
	}

	/**
	 * Initialize this object.
	 */
	protected void initialize() {
		super.initialize();

		parameter = null;
		parameterName = null;
	}

	/**
	 * Notify that my parameter is changed.
	 */
	protected void updateParameter() {
	}
}
