package sra.jun.goodies.parameter;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import sra.smalltalk.*;

/**
 * JunParameterViewSwing class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/06/02 (by MATSUDA Ryouichi)
 *  @updated   2003/01/16 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunParameterViewSwing.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunParameterViewSwing extends StViewJPanel implements JunParameterView {

	protected JLabel label;
	protected JTextField textField;
	protected JScrollBar scrollbar;

	/**
	 * Create a new instance of JunParameterViewSwing and initialize it.
	 *
	 * @param aModel sra.jun.goodies.parameter.JunParameterModel
	 */
	public JunParameterViewSwing(JunParameterModel aModel) {
		super(aModel);

		this.updateParameter();
	}

	/**
	 * Invoked when an action occurs.
	 * 
	 * @param e java.event.ActionEvent
	 */
	public void actionPerformed(ActionEvent e) {
		try {
			double value = new Double(e.getActionCommand()).doubleValue();
			this.getParameterModel().parameter().value_(value);
		} catch (NumberFormatException e2) {
		}
	}

	/**
	 * Invoked when the value of the adjustable has changed.
	 * 
	 * @param e java.event.ActionEvent
	 */
	public void adjustmentValueChanged(AdjustmentEvent e) {
		this.getParameterModel().parameter()._setRangeValue(e.getValue());
	}

	/**
	 * Answer my model as JunParameterModel.
	 * 
	 * @return sra.jun.goodies.parameter.JunParameterModel
	 */
	public JunParameterModel getParameterModel() {
		return (JunParameterModel) this.model();
	}

	/**
	 * Set the bounds of myself and my components.
	 *
	 * @param x int
	 * @param y int
	 * @param width int
	 * @param height int
	 */
	public void setBounds(int x, int y, int width, int height) {
		super.setBounds(x, y, width, height);
		this._layoutComponents(width, height);
	}

	/**
	 * Update the view to reflect the changed value.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if (evt.getAspect() == $("value")) {
			this.updateParameter();
		} else {
			super.update_(evt);
		}
	}

	/**
	 * Update parameter.
	 */
	public void updateParameter() {
		scrollbar.removeAdjustmentListener(this);
		textField.removeActionListener(this);
		try {
			JunParameter parameter = this.getParameterModel().parameter();
			textField.setText(String.valueOf(parameter.value()));
			scrollbar.setValues(parameter._rangeValue(), 0, parameter._rangeStartValue(), parameter._rangeStopValue());
		} finally {
			textField.addActionListener(this);
			scrollbar.addAdjustmentListener(this);
		}
	}

	/**
	 * Layout my components.
	 *
	 * @param width int
	 * @param height int
	 */
	protected void _layoutComponents(int width, int height) {
		label.setBounds(1, height / 2 - 6, (int) (width * 0.4) - 4, 12);
		textField.setBounds((int) (width * 0.4) + 1, 1, (int) (width * 0.6) - 2, 23);
		scrollbar.setBounds((int) (width * 0.4) + 1, 26, (int) (width * 0.6) - 2, height - 27);
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		label = new JLabel(this.getParameterModel().labelString());
		label.setFont(new Font("dialog", Font.BOLD, 12));
		textField = new JTextField();
		scrollbar = new JScrollBar(Scrollbar.HORIZONTAL);

		this.setLayout(null);
		this.add(label);
		this.add(textField);
		this.add(scrollbar);
		this.setPreferredSize(new Dimension(200, 40));
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunParameterModel(JunParameter.Name_value_("parameter", 10));
	}
}
