package sra.jun.goodies.parameter;

import sra.smalltalk.*;
import sra.jun.system.framework.JunApplicationModel;

/**
 * JunParametersModel class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/12/11 (by MATSUDA Ryouichi)
 *  @updated   2003/01/16 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunParametersModel.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunParametersModel extends JunApplicationModel {

	protected JunParameter[] parameters;

	/**
	 * Create a new instance of JunParametersModel.
	 */
	public JunParametersModel() {
		super();
	}

	/**
	 * Create a new instance of JunParametersModel and initialize it.
	 *
	 * @param anArrayOfParameters sra.jun.goodies.parameter.JunParameter[]
	 */
	public JunParametersModel(JunParameter[] anArrayOfParameters) {
		this();
		this.parameters_(anArrayOfParameters);
	}

	/**
	 * Create a new instance of JunParametersModel and set Parameters.
	 * 
	 * @return sra.jun.goodies.parameter.JunParameterModel
	 * @param anArrayOfParameters sra.jun.goodies.parameter.JunParameter[]
	 * @deprecated since Jun379, use the constructor.
	 */
	public static JunParametersModel On_(JunParameter[] anArrayOfParameters) {
		return new JunParametersModel(anArrayOfParameters);
	}

	/**
	 * Create a new instance of JunParametersModel and set Parameters and open it.
	 * 
	 * @return sra.jun.goodies.parameter.JunParameterModel
	 * @param anArrayOfParameters sra.jun.goodies.parameter.JunParameter[]
	 */
	public static JunParametersModel OpenOnParameters_(JunParameter[] anArrayOfParameters) {
		JunParametersModel aParametersModel = new JunParametersModel(anArrayOfParameters);
		aParametersModel.open();
		return aParametersModel;
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunParametersViewAwt(this);
		} else {
			return new JunParametersViewSwing(this);
		}
	}

	/**
	 * Answer my current parameters.
	 *
	 * @return sra.jun.goodies.parameter.JunParameter[]
	 */
	public JunParameter[] parameters() {
		if (parameters == null) {
			parameters = new JunParameter[0];
		}
		return parameters;
	}

	/**
	 * Set my new parameters.
	 *
	 * @param newParameters sra.jun.goodies.parameter.JunParameter[]
	 */
	public void parameters_(JunParameter[] newParameters) {
		parameters = newParameters;
		this.buildComposite();
	}

	/**
	 * Build the composite.
	 */
	protected void buildComposite() {
		this.changed_($("build"));
	}

	/**
	 * Initialize this object.
	 */
	protected void initialize() {
		super.initialize();

		parameters = null;
	}
}
