package sra.jun.goodies.parameter;

import java.awt.*;
import sra.smalltalk.*;

/**
 * JunParametersViewAwt class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/06/02 (by MATSUDA Ryouichi)
 *  @updated   2003/01/16 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunParametersViewAwt.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunParametersViewAwt extends StViewPanel implements JunParametersView {

	protected JunParameterViewAwt[] _parameterViews;
	protected Panel _panel;

	/**
	 * Create a new instance of JunParametersViewAwt and initialize it.
	 *
	 * @param aModel sra.jun.goodies.parameter.JunParametersModel
	 */
	public JunParametersViewAwt(JunParametersModel aModel) {
		super(aModel);
	}

	/**
	 * Answer my model as a JunParametersModel.
	 * 
	 * @return sra.jun.goodies.parameter.JunParametersModel
	 */
	public JunParametersModel getParametersModel() {
		return (JunParametersModel) this.model();
	}

	/**
	 * Update the components of the parameters if changed.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if (evt.getAspect() == $("build")) {
			this.buildComposite();
			this._layoutComponents(_panel.getWidth(), _panel.getHeight());
		} else {
			super.update_(evt);
		}
	}

	/**
	 * Layout my components.
	 *
	 * @return int
	 * @param width int
	 * @param height int
	 */
	protected int _layoutComponents(int width, int height) {
		if (_parameterViews == null) {
			return -1;
		}

		int y = 4;
		for (int i = 0; i < _parameterViews.length; i++) {
			int viewHeight = _parameterViews[i].getParameterModel().preferredHeight();
			_parameterViews[i].setBounds(4, y, width - 4, viewHeight);
			y += viewHeight + 4;
		}
		return y;
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		_panel = new Panel() {
			public void setBounds(int x, int y, int width, int height) {
				int bottom = _layoutComponents(width, height);
				if (bottom < 0) {
					bottom = height;
				}
				super.setBounds(x, y, width, bottom);
			}
		};
		_panel.setLayout(null);

		ScrollPane aScrollPane = new ScrollPane() {
			public void setBounds(int x, int y, int width, int height) {
				super.setBounds(x, y, width, height);
				_panel.setBounds(0, 0, width - this.getVScrollbarWidth() - 4, height);
			}
		};
		aScrollPane.add(_panel);
		aScrollPane.setSize(200, 200);

		this.setLayout(new BorderLayout());
		this.add(aScrollPane, BorderLayout.CENTER);

		this.buildComposite();
	}

	/**
	 * Build the composite.
	 */
	protected void buildComposite() {
		_panel.removeAll();

		JunParameter[] parameters = this.getParametersModel().parameters();
		_parameterViews = new JunParameterViewAwt[parameters.length];
		for (int i = 0; i < parameters.length; i++) {
			_parameterViews[i] = new JunParameterViewAwt(new JunParameterModel(parameters[i]));
			_panel.add(_parameterViews[i]);
		}
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunParametersModel();
	}
}
