package sra.jun.goodies.pen;

import java.awt.Color;
import java.util.Vector;
import sra.smalltalk.*;

/**
 * JunPen class
 * 
 *  @author    Hirotsugu Kondo
 *  @created   1999/1/5 (by Hirotsugu Kondo)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPen.java,v 6.0 2003/10/28 10:52:12 nisinaka Exp $
 */
public class JunPen extends StObject {
	private JunPenLocation location;
	private JunPenDirection direction;
	private boolean down;
	private int nib;
	private Color color;
	private Vector sequence;
	private StValueHolder drawing;

	/**
	 * Creates a new JunPen object.
	 */
	public JunPen() {
		super();
		this.location = JunPenLocation.Zero();
		this.direction = JunPenDirection.Longitude_latitude_(0.0d, 0.0d);
		this.down = true;
		this.nib = 1;
		this.color = Color.black;
		this.drawing = new StValueHolder(new Vector());
	}

	/**
	 * DOCUMENT ME!
	 */
	public void begin() {
		Vector vector = new Vector();
		vector.addElement(this.location());
		this.sequence = vector;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public Color color() {
		return this.color;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param colorValue DOCUMENT ME!
	 */
	public void color_(Color colorValue) {
		this.color = colorValue;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBlock DOCUMENT ME!
	 */
	public void compute_(StBlockClosure aBlock) {
		this.drawing.compute_(aBlock);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public JunPenDirection direction() {
		return this.direction;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aDirection DOCUMENT ME!
	 */
	public void direction_(double[] aDirection) {
		this.direction = JunPenDirection.Coerce_(aDirection);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aDirection DOCUMENT ME!
	 */
	public void direction_(double aDirection) {
		this.direction = JunPenDirection.Coerce_(aDirection);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aDirection may be JunPenDirection,Double and Vector of Double(see
	 *        JunPenDirection.Coerce_(Object))
	 */
	public void direction_(Object aDirection) {
		this.direction = JunPenDirection.Coerce_(aDirection);
	}

	/**
	 * DOCUMENT ME!
	 */
	public void down() {
		this.down = true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param orderNumbers DOCUMENT ME!
	 * @param distance DOCUMENT ME!
	 */
	public void dragon_distance_(int orderNumbers, int distance) {
		if (orderNumbers == 0) {
			this.go_(distance);
		} else {
			if (orderNumbers > 0) {
				this.dragon_distance_(orderNumbers - 1, distance);
				this.turn_(90);
				this.dragon_distance_(1 - orderNumbers, distance);
			} else {
				this.dragon_distance_(-1 - orderNumbers, distance);
				this.turn_(-90);
				this.dragon_distance_(1 + orderNumbers, distance);
			}
		}
	}

	/**
	 * DOCUMENT ME!
	 */
	public void fill() {
		this.fillPolygon_(this.sequence);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBlock DOCUMENT ME!
	 */
	public void fill_(StBlockClosure aBlock) {
		boolean oldDown = this.isDown();
		final StValueHolder savedState = new StValueHolder(this.state());
		final JunPen this_ = this;
		final StBlockClosure aBlock_ = aBlock;
		StBlockClosure assertBlock = new StBlockClosure() {
			public Object value() {
				savedState.value_(this_.state());

				return null;
			}
		};

		StBlockClosure doBlock = new StBlockClosure() {
			public Object value() {
				boolean oldState = this_.isDown();
				this_.down();
				this_.begin();
				aBlock_.value();
				this_.fill();

				return null;
			}
		};

		StBlockClosure ensureBlock = new StBlockClosure() {
			public Object value() {
				this_.state_((StAssociation[]) savedState.value());

				return null;
			}
		};

		this.assert_do_ensure_(assertBlock, doBlock, ensureBlock);

		if (oldDown) {
			this.down();
		} else {
			this.up();
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param distance DOCUMENT ME!
	 */
	public void go_(double distance) {
		JunPenLocation unitVector = JunPenLocation.X_y_z_(1.0d, 0.0d, 0.0d);
		JunPenTransformation aTransformation = JunPenTransformation.Unity();
		JunPenAngle latitude = this.direction().latitude();
		JunPenAngle longitude = this.direction().longitude();

		if (longitude.abs().isGreaterThan_(new Double(longitude.accuracy()))) {
			JunPenTransformation aT = JunPenTransformation.RotateZ_(longitude);
			aTransformation = aTransformation.product_(aT);
		}

		if (latitude.abs().isGreaterThan_(new Double(latitude.accuracy()))) {
			JunPenLocation aroundVector = JunPenLocation.X_y_z_(0.0d, -1.0d, 0.0d);
			aroundVector = aroundVector.transform_(aTransformation);

			JunPenLocation[] aLine = new JunPenLocation[2];
			aLine[0] = JunPenLocation.Zero();
			aLine[1] = aroundVector;

			JunPenTransformation aT = JunPenTransformation.Rotate_around_(latitude, aLine);
			aTransformation = aTransformation.product_(aT);
		}

		aTransformation = aTransformation.product_(JunPenTransformation.Scale_(new Double(distance)));
		aTransformation = aTransformation.product_(JunPenTransformation.Translate_(location));

		JunPenLocation newLocation = unitVector.transform_(aTransformation);
		this.goto_(newLocation);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param newPoint DOCUMENT ME!
	 */
	public void goto_(double[] newPoint) {
		JunPenLocation oldPoint = this.location();
		this.location_(newPoint);
		this.drawLineFrom_to_(oldPoint, this.location());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param newPoint DOCUMENT ME!
	 */
	public void goto_(double newPoint) {
		JunPenLocation oldPoint = this.location();
		this.location_(newPoint);
		this.drawLineFrom_to_(oldPoint, this.location());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param newPoint DOCUMENT ME!
	 */
	public void goto_(Object newPoint) {
		JunPenLocation oldPoint = this.location();
		this.location_(newPoint);
		this.drawLineFrom_to_(oldPoint, this.location());
	}

	/**
	 * DOCUMENT ME!
	 */
	public void home() {
		this.location_(JunPenLocation.Zero());
		this.direction_(JunPenDirection.Zero());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public boolean isDown() {
		return down;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public boolean isUp() {
		return !down;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public JunPenLocation location() {
		return this.location;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aLocation DOCUMENT ME!
	 */
	public void location_(double[] aLocation) {
		this.location = JunPenLocation.Coerce_(aLocation);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aLocation DOCUMENT ME!
	 */
	public void location_(double aLocation) {
		this.location = JunPenLocation.Coerce_(aLocation);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aLocation may be JunPenLocation,JunPoint,Double and Vector(see
	 *        JunPenLocation.Coerce_(Object))
	 */
	public void location_(Object aLocation) {
		this.location = JunPenLocation.Coerce_(aLocation);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param npoints DOCUMENT ME!
	 * @param d DOCUMENT ME!
	 */
	public void mandala_diameter_(int npoints, int d) {
		int l = (int) Math.round((Math.PI * d) / npoints);
		this.home();
		this.up();
		this.turn_(-90);
		this.go_(d / 2);
		this.turn_(90);
		this.go_((0 - l) / 2);

		JunPenLocation[] points = new JunPenLocation[npoints];

		for (int i = 0; i < npoints; i++) {
			points[i] = this.location().rounded();
			this.go_(l);
			this.turn_(360 / npoints);
		}

		this.down();

		for (int i = npoints / 2; i > 0; i--) {
			for (int j = 0; j < npoints; j++) {
				this.location_(points[j]);
				this.goto_(points[(j + i) % npoints]);
			}
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public int nib() {
		return this.nib;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anInteger DOCUMENT ME!
	 */
	public void nib_(int anInteger) {
		this.nib = Math.max(anInteger, 1);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param n DOCUMENT ME!
	 * @param a DOCUMENT ME!
	 */
	public void spiral_angle_(int n, int a) {
		for (int i = 1; i < (n + 1); i++) {
			this.go_(i);
			this.turn_(a);
		}
	}

	/**
	 * Answer the current state information.
	 * 
	 * @return StAssociation[]
	 */
	public StAssociation[] state() {
		StAssociation[] state = new StAssociation[6];
		state[0] = new StAssociation($("location"), this.location());
		state[1] = new StAssociation($("direction"), this.direction());
		state[2] = new StAssociation($("down"), new Boolean(this.down));
		state[3] = new StAssociation($("nib"), new Integer(this.nib()));
		state[4] = new StAssociation($("color"), this.color());
		state[5] = new StAssociation($("sequence"), this.sequence);

		return state;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anArray
	 */
	public void state_(StAssociation[] anArray) {
		for (int index = 0; index < anArray.length; index++) {
			StAssociation association = anArray[index];
			StSymbol key = (StSymbol) association.key();
			Object value = association.value();

			if (key == $("location")) {
				this.location_((JunPenLocation) value);
			}

			if (key == $("direction")) {
				this.direction_((JunPenDirection) value);
			}

			if (key == $("down")) {
				this.down = ((Boolean) value).booleanValue();
			}

			if (key == $("nib")) {
				this.nib_(((Integer) value).intValue());
			}

			if (key == $("color")) {
				this.color_((Color) value);
			}

			if (key == $("sequence")) {
				this.sequence = (Vector) value;
			}
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param degrees DOCUMENT ME!
	 */
	public void tilt_(double degrees) {
		double longitude = this.direction().longitude().deg();
		double latitude = (this.direction().latitude().deg() + degrees) % 360.0d;
		this.direction_(JunPenDirection.Longitude_latitude_(longitude, latitude));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param degrees DOCUMENT ME!
	 */
	public void turn_(double degrees) {
		double longitude = (this.direction().longitude().deg() + degrees) % 360.0d;
		double latitude = this.direction().latitude().deg();
		this.direction_(JunPenDirection.Longitude_latitude_(longitude, latitude));
	}

	/**
	 * DOCUMENT ME!
	 */
	public void up() {
		this.down = false;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param assertBlock DOCUMENT ME!
	 * @param doBlock DOCUMENT ME!
	 * @param ensureBlock DOCUMENT ME!
	 */
	private void assert_do_ensure_(StBlockClosure assertBlock, StBlockClosure doBlock, StBlockClosure ensureBlock) {
		assertBlock.value();

		try {
			doBlock.value();
		} catch (Exception e) {
		} finally {
			ensureBlock.value();
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param startPoint DOCUMENT ME!
	 * @param endPoint DOCUMENT ME!
	 */
	private void drawLineFrom_to_(JunPenLocation startPoint, JunPenLocation endPoint) {
		Vector anArray = new Vector();

		if (this.isDown()) {
			anArray.addElement($("stroke"));
			anArray.addElement(new Integer(this.nib()));
			anArray.addElement(this.color());
			anArray.addElement(startPoint);
			anArray.addElement(endPoint);
			this.drawing.value_(anArray);
		}

		if (this.sequence != null) {
			sequence.addElement(endPoint);
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param collectionOfLocation DOCUMENT ME!
	 */
	private void fillPolygon_(Vector collectionOfLocation) {
		if (collectionOfLocation == null) {
			return;
		}

		if (collectionOfLocation.size() < 3) {
			return;
		}

		Vector anArray = new Vector();

		if (this.isDown()) {
			anArray.addElement($("fill"));
			anArray.addElement(this.color());
			anArray.addElement(collectionOfLocation);
			this.drawing.value_(anArray);
		}

		this.sequence = null;
	}
}
