package sra.jun.goodies.progress;

import java.awt.*;
import java.awt.image.BufferedImage;
import sra.smalltalk.*;

/**
 * JunCursorAnimator class
 * 
 *  @author    nisinaka
 *  @created   2001/11/20 (by nisinaka)
 *  @updated   2002/04/04 (by nisinaka)
 *  @updated   2003/05/14 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun455 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCursorAnimator.java,v 6.0 2003/10/28 10:52:13 nisinaka Exp $
 */
public class JunCursorAnimator extends StObject {

	protected static final int WIDTH = 16;
	protected static final int HEIGHT = 16;
	protected static Cursor[] ClockCursors = null;
	protected static Cursor[] HandCursors = null;

	protected Cursor[] cursors = new Cursor[0];
	protected int tick = 1000;
	protected Frame[] _frames = null;
	protected Cursor[] _savedCursors;
	protected Thread _process;
	protected boolean _finished;

	/**
	 * JunCorsorAnimator constructor comment.
	 */
	public JunCursorAnimator() {
		super();
	}

	/**
	 * Create a new instance of JunCursorAnimator and initialize it.
	 * 
	 * @param cursors java.awt.Cursor[]
	 */
	public JunCursorAnimator(Cursor[] cursors) {
		this.cursors_(cursors);
	}

	/**
	 * Create the clock cursors.
	 * 
	 * @return sra.jun.goodies.progress.JunCursorAnimator
	 */
	public static JunCursorAnimator ClockCursors() {
		if (ClockCursors == null) {
			ClockCursors = new Cursor[] { ClockCursor0(), ClockCursor1(), ClockCursor2(), ClockCursor3(), ClockCursor4(), ClockCursor5(), ClockCursor6(), ClockCursor7()};
		}

		return new JunCursorAnimator(ClockCursors);
	}

	/**
	 * Ansewr the hand cursors.
	 * 
	 * @return sra.jun.goodies.progress.JunCursorAnimator
	 */
	public static JunCursorAnimator HandCursors() {
		if (HandCursors == null) {
			HandCursors = new Cursor[] { HandCursor0(), HandCursor1(), HandCursor2(), HandCursor3(), HandCursor4(), HandCursor5(), HandCursor4(), HandCursor3(), HandCursor2(), HandCursor1()};
		}

		return new JunCursorAnimator(HandCursors);
	}

	/**
	 * Create an image from the array of pixels.
	 * 
	 * @param pixels int[]
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	private static BufferedImage _CreateImage(int[] pixels) {
		BufferedImage image = new BufferedImage(WIDTH, HEIGHT, BufferedImage.TYPE_INT_ARGB);
		image.setRGB(0, 0, WIDTH, HEIGHT, pixels, 0, WIDTH);

		return image;
	}

	/**
	 * Create an array of pixels from an image string and a mask string.
	 * 
	 * @param imageString java.lang.String
	 * @param maskString java.lang.String
	 * 
	 * @return int[]
	 */
	private static int[] _Pixels(String imageString, String maskString) {
		int[] pixels = new int[WIDTH * HEIGHT];

		for (int i = 0; i < (WIDTH * HEIGHT); i++) {
			int pixel = 0;

			if (imageString.charAt(i) != '0') {
				pixel = Color.black.getRGB();
			} else if (maskString.charAt(i) != '0') {
				pixel = Color.white.getRGB();
			}

			pixels[i] = pixel;
		}

		return pixels;
	}

	/**
	 * Resize the image to fit as a cursor.
	 * 
	 * @param anImage java.awt.image.BufferedImage
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	private static BufferedImage _ResizeToFit(BufferedImage anImage) {
		Dimension size = Toolkit.getDefaultToolkit().getBestCursorSize(WIDTH, HEIGHT);
		BufferedImage cursorImage = new BufferedImage(size.width, size.height, BufferedImage.TYPE_INT_ARGB);
		cursorImage.setData(anImage.getRaster());

		return cursorImage;
	}

	/**
	 * Answer the clock cursor #0.
	 * 
	 * @return java.awt.Cursor
	 */
	protected static Cursor ClockCursor0() {
		String image =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0100000010000000"
				+ "1000010001000000"
				+ "1000010001000000"
				+ "1000010001100000"
				+ "1001110001100000"
				+ "1000000001000000"
				+ "1000000001000000"
				+ "0100000010000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";
		String mask =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0111111110000000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "1111111111100000"
				+ "1111111111100000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "0111111110000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";

		return Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(_CreateImage(_Pixels(image, mask))), new Point(5, 7), "clock0");
	}

	/**
	 * Answer the clock cursor #1.
	 * 
	 * @return java.awt.Cursor
	 */
	protected static Cursor ClockCursor1() {
		String image =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0100000010000000"
				+ "1000000001000000"
				+ "1000000101000000"
				+ "1000001001100000"
				+ "1001110001100000"
				+ "1000000001000000"
				+ "1000000001000000"
				+ "0100000010000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";
		String mask =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0111111110000000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "1111111111100000"
				+ "1111111111100000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "0111111110000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";

		return Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(_CreateImage(_Pixels(image, mask))), new Point(5, 7), "clock1");
	}

	/**
	 * Answer the clock cursor #2.
	 * 
	 * @return java.awt.Cursor
	 */
	protected static Cursor ClockCursor2() {
		String image =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0100000010000000"
				+ "1000000001000000"
				+ "1000000001000000"
				+ "1000000001100000"
				+ "1001111101100000"
				+ "1000000001000000"
				+ "1000000001000000"
				+ "0100000010000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";
		String mask =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0111111110000000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "1111111111100000"
				+ "1111111111100000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "0111111110000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";

		return Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(_CreateImage(_Pixels(image, mask))), new Point(5, 7), "clock2");
	}

	/**
	 * Answer the clock cursor #3.
	 * 
	 * @return java.awt.Cursor
	 */
	protected static Cursor ClockCursor3() {
		String image =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0100000010000000"
				+ "1000000001000000"
				+ "1000000001000000"
				+ "1000000001100000"
				+ "1001110001100000"
				+ "1000001001000000"
				+ "1000000001000000"
				+ "0100000010000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";
		String mask =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0111111110000000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "1111111111100000"
				+ "1111111111100000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "0111111110000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";

		return Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(_CreateImage(_Pixels(image, mask))), new Point(5, 7), "clock3");
	}

	/**
	 * Answer the clock cursor #4.
	 * 
	 * @return java.awt.Cursor
	 */
	protected static Cursor ClockCursor4() {
		String image =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0100000010000000"
				+ "1000000001000000"
				+ "1000000001000000"
				+ "1000000001100000"
				+ "1001110001100000"
				+ "1000010001000000"
				+ "1000010001000000"
				+ "0100000010000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";
		String mask =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0111111110000000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "1111111111100000"
				+ "1111111111100000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "0111111110000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";

		return Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(_CreateImage(_Pixels(image, mask))), new Point(5, 7), "clock4");
	}

	/**
	 * Answer the clock cursor #5.
	 * 
	 * @return java.awt.Cursor
	 */
	protected static Cursor ClockCursor5() {
		String image =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0100000010000000"
				+ "1000000001000000"
				+ "1000000001000000"
				+ "1000000001100000"
				+ "1001110001100000"
				+ "1000100001000000"
				+ "1001000001000000"
				+ "0100000010000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";
		String mask =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0111111110000000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "1111111111100000"
				+ "1111111111100000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "0111111110000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";

		return Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(_CreateImage(_Pixels(image, mask))), new Point(5, 7), "clock5");
	}

	/**
	 * Answer the clock cursor #6.
	 * 
	 * @return java.awt.Cursor
	 */
	protected static Cursor ClockCursor6() {
		String image =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0100000010000000"
				+ "1000000001000000"
				+ "1000000001000000"
				+ "1000000001100000"
				+ "1011110001100000"
				+ "1000000001000000"
				+ "1000000001000000"
				+ "0100000010000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";
		String mask =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0111111110000000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "1111111111100000"
				+ "1111111111100000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "0111111110000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";

		return Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(_CreateImage(_Pixels(image, mask))), new Point(5, 7), "clock6");
	}

	/**
	 * Answer the clock cursor #7.
	 * 
	 * @return java.awt.Cursor
	 */
	protected static Cursor ClockCursor7() {
		String image =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0100000010000000"
				+ "1000000001000000"
				+ "1001000001000000"
				+ "1000100001100000"
				+ "1001110001100000"
				+ "1000000001000000"
				+ "1000000001000000"
				+ "0100000010000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";
		String mask =
			"0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0111111110000000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "1111111111100000"
				+ "1111111111100000"
				+ "1111111111000000"
				+ "1111111111000000"
				+ "0111111110000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000"
				+ "0011111100000000";

		return Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(_CreateImage(_Pixels(image, mask))), new Point(5, 7), "clock7");
	}

	/**
	 * Answer the hand cursor #0.
	 * 
	 * @return java.awt.Cursor
	 */
	protected static Cursor HandCursor0() {
		String image =
			"0000000110000000"
				+ "0001101001110000"
				+ "0010011001001000"
				+ "0010011001001010"
				+ "0001001001001101"
				+ "0001001001001001"
				+ "0110100000001001"
				+ "1001100000000001"
				+ "1000100000000010"
				+ "0100000000000010"
				+ "0010000000000010"
				+ "0010000000000100"
				+ "0001000000000100"
				+ "0000100000001000"
				+ "0000010000001000"
				+ "0000010000001000";
		String mask =
			"0000000110000000"
				+ "0001101111110000"
				+ "0011111111111000"
				+ "0011111111111010"
				+ "0001111111111111"
				+ "0001111111111111"
				+ "0110111111111111"
				+ "1111111111111111"
				+ "1111111111111110"
				+ "0111111111111110"
				+ "0011111111111110"
				+ "0011111111111100"
				+ "0001111111111100"
				+ "0000111111111000"
				+ "0000011111111000"
				+ "0000011111111000";

		return Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(_CreateImage(_Pixels(image, mask))), new Point(7, 7), "hand0");
	}

	/**
	 * Answer the hand cursor #1.
	 * 
	 * @return java.awt.Cursor
	 */
	protected static Cursor HandCursor1() {
		String image =
			"0000000110000000"
				+ "0001101001110000"
				+ "0010011001001000"
				+ "0010011001001010"
				+ "0001001001001101"
				+ "0001001001001001"
				+ "0000100000001001"
				+ "0000100000000001"
				+ "0001100000000010"
				+ "0010000000000010"
				+ "0010000000000010"
				+ "0010000000000100"
				+ "0001000000000100"
				+ "0000100000001000"
				+ "0000010000001000"
				+ "0000010000001000";
		String mask =
			"0000000110000000"
				+ "0001101111110000"
				+ "0011111111111000"
				+ "0011111111111010"
				+ "0001111111111111"
				+ "0001111111111111"
				+ "0000111111111111"
				+ "0000111111111111"
				+ "0001111111111110"
				+ "0011111111111110"
				+ "0011111111111110"
				+ "0011111111111100"
				+ "0001111111111100"
				+ "0000111111111000"
				+ "0000011111111000"
				+ "0000011111111000";

		return Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(_CreateImage(_Pixels(image, mask))), new Point(7, 7), "hand1");
	}

	/**
	 * Answer the hand cursor #2.
	 * 
	 * @return java.awt.Cursor
	 */
	protected static Cursor HandCursor2() {
		String image =
			"0000000110000000"
				+ "0000001001110000"
				+ "0000001001001000"
				+ "0000001001001010"
				+ "0000111001001101"
				+ "0001001001001001"
				+ "0001000000001001"
				+ "0000100000000001"
				+ "0001100000000010"
				+ "0010000000000010"
				+ "0010000000000010"
				+ "0010000000000100"
				+ "0001000000000100"
				+ "0000100000001000"
				+ "0000010000001000"
				+ "0000010000001000";
		String mask =
			"0000000110000000"
				+ "0000001111110000"
				+ "0000001111111000"
				+ "0000001111111010"
				+ "0000111111111111"
				+ "0001111111111111"
				+ "0001111111111111"
				+ "0000111111111111"
				+ "0001111111111110"
				+ "0011111111111110"
				+ "0011111111111110"
				+ "0011111111111100"
				+ "0001111111111100"
				+ "0000111111111000"
				+ "0000011111111000"
				+ "0000011111111000";

		return Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(_CreateImage(_Pixels(image, mask))), new Point(7, 7), "hand2");
	}

	/**
	 * Answer the hand cursor #3.
	 * 
	 * @return java.awt.Cursor
	 */
	protected static Cursor HandCursor3() {
		String image =
			"0000000000000000"
				+ "0000000000110000"
				+ "0000000001001000"
				+ "0000000001001010"
				+ "0000110111001101"
				+ "0001001001001001"
				+ "0001000000001001"
				+ "0000100000000001"
				+ "0001100000000010"
				+ "0010000000000010"
				+ "0010000000000010"
				+ "0010000000000100"
				+ "0001000000000100"
				+ "0000100000001000"
				+ "0000010000001000"
				+ "0000010000001000";
		String mask =
			"0000000000000000"
				+ "0000000000110000"
				+ "0000000001111000"
				+ "0000000001111010"
				+ "0000110111111111"
				+ "0001111111111111"
				+ "0001111111111111"
				+ "0000111111111111"
				+ "0001111111111110"
				+ "0011111111111110"
				+ "0011111111111110"
				+ "0011111111111100"
				+ "0001111111111100"
				+ "0000111111111000"
				+ "0000011111111000"
				+ "0000011111111000";

		return Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(_CreateImage(_Pixels(image, mask))), new Point(7, 7), "hand3");
	}

	/**
	 * Answer the hand cursor #4.
	 * 
	 * @return java.awt.Cursor
	 */
	protected static Cursor HandCursor4() {
		String image =
			"0000000000000000"
				+ "0000000000000000"
				+ "0000000000000000"
				+ "0000000000000010"
				+ "0000110110110101"
				+ "0001001001001001"
				+ "0001000000001001"
				+ "0000100000000001"
				+ "0001100000000010"
				+ "0010000000000010"
				+ "0010000000000010"
				+ "0010000000000100"
				+ "0001000000000100"
				+ "0000100000001000"
				+ "0000010000001000"
				+ "0000010000001000";
		String mask =
			"0000000000000000"
				+ "0000000000000000"
				+ "0000000000000000"
				+ "0000000000000010"
				+ "0000110110110111"
				+ "0001111111111111"
				+ "0001111111111111"
				+ "0000111111111111"
				+ "0001111111111110"
				+ "0011111111111110"
				+ "0011111111111110"
				+ "0011111111111100"
				+ "0001111111111100"
				+ "0000111111111000"
				+ "0000011111111000"
				+ "0000011111111000";

		return Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(_CreateImage(_Pixels(image, mask))), new Point(7, 7), "hand4");
	}

	/**
	 * Answer the hand cursor #5.
	 * 
	 * @return java.awt.Cursor
	 */
	protected static Cursor HandCursor5() {
		String image =
			"0000000000000000"
				+ "0000000000000000"
				+ "0000000000000000"
				+ "0000000000000000"
				+ "0000110110110000"
				+ "0001001001001100"
				+ "0001000000001010"
				+ "0000100000000010"
				+ "0001100000000010"
				+ "0010000000000010"
				+ "0010000000000010"
				+ "0010000000000100"
				+ "0001000000000100"
				+ "0000100000001000"
				+ "0000010000001000"
				+ "0000010000001000";
		String mask =
			"0000000000000000"
				+ "0000000000000000"
				+ "0000000000000000"
				+ "0000000000000000"
				+ "0000110110110000"
				+ "0001111111111100"
				+ "0001111111111110"
				+ "0000111111111110"
				+ "0001111111111110"
				+ "0011111111111110"
				+ "0011111111111110"
				+ "0011111111111100"
				+ "0001111111111100"
				+ "0000111111111000"
				+ "0000011111111000"
				+ "0000011111111000";

		return Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(_CreateImage(_Pixels(image, mask))), new Point(7, 7), "hand5");
	}

	/**
	 * Set the new frames.
	 * 
	 * @param newFrames java.awt.Frame[]
	 */
	public void _frames(Frame[] newFrames) {
		_frames = newFrames;
	}

	/**
	 * Restore the original cursor for all frames.
	 */
	public void _restore() {
		// Stop the thread for the animation.
		_finished = true;
		try {
			_process.join();
		} catch (InterruptedException e) {
		} finally {
			_process = null;
		}

		// Restore the original cursors.
		if (_savedCursors == null) {
			return;
		}
		Frame[] frames = this._frames();
		for (int i = 0; i < frames.length; i++) {
			frames[i].setCursor(_savedCursors[i]);
		}
		_frames = null;
		_savedCursors = null;
	}

	/**
	 * Set the cursor for all frames.
	 */
	public void _show() {
		this._show(null);
	}

	/**
	 * Show the cursor animation.
	 *
	 * @param interimBlock sra.smalltalk.StBlockClosure
	 */
	public void _show(final StBlockClosure interimBlock) {
		// Save the original cursors.
		Frame[] frames = this._frames();
		_savedCursors = new Cursor[frames.length];
		for (int i = 0; i < frames.length; i++) {
			_savedCursors[i] = frames[i].getCursor();
		}

		// Start the thread for the animation.
		_process = new Thread() {
			public void run() {
				int index = 0;
				Cursor[] cursors = cursors();
				while (_finished == false) {
					Thread.yield();

					try {
						_process.sleep(tick());
					} catch (InterruptedException e) {
					}

					if (interimBlock != null) {
						interimBlock.value();
					}

					if (index < cursors.length) {
						Frame[] frames = _frames();
						for (int i = 0; i < frames.length; i++) {
							frames[i].setCursor(cursors[index]);
						}
						index++;
						if (index >= cursors.length) {
							index = 0;
						}
					}
				}
			}
		};
		_process.setPriority(Thread.NORM_PRIORITY);
		_process.start();
	}

	/**
	 * Answer the current cursors.
	 * 
	 * @return java.awt.Cursor[]
	 */
	public Cursor[] cursors() {
		return cursors;
	}

	/**
	 * Set the new cursors.
	 * 
	 * @param newCursors java.awt.Cursor[]
	 */
	public void cursors_(Cursor[] newCursors) {
		cursors = new Cursor[newCursors.length];
		System.arraycopy(newCursors, 0, cursors, 0, newCursors.length);
	}

	/**
	 * Show the animation cursor while executing a block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object showWhile_(StBlockClosure aBlock) {
		return this.showWhile_interim_(aBlock, null);
	}

	/**
	 * Show the animation cursor while executing a block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @param interimBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object showWhile_interim_(StBlockClosure aBlock, StBlockClosure interimBlock) {
		try {
			this._show(interimBlock);
			return aBlock.value();
		} finally {
			this._restore();
		}
	}

	/**
	 * Answer the current tick in milliseconds.
	 * 
	 * @return int
	 */
	public int tick() {
		return tick;
	}

	/**
	 * Set the new tick in milliseconds.
	 * 
	 * @param newTick int
	 */
	public void tick_(int newTick) {
		tick = Math.max(10, Math.min(newTick, 1000));
	}

	/**
	 * Answer the current frames.
	 * 
	 * @return java.awt.Frame[]
	 */
	protected Frame[] _frames() {
		if (_frames == null) {
			_frames = Frame.getFrames();
		}
		return _frames;
	}
}
