package sra.jun.goodies.progress;

import java.awt.*;
import javax.swing.*;
import sra.smalltalk.*;
import sra.jun.goodies.button.JunButtonViewSwing;

/**
 * JunProgressViewSwing class
 * 
 *  @author    Mitsuhiro Asada
 *  @created   2000/03/16 (by Mitsuhiro Asada)
 *  @updated   2003/01/07 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunProgressViewSwing.java,v 6.0 2003/10/28 10:52:13 nisinaka Exp $
 */
public class JunProgressViewSwing extends StViewJPanel implements JunProgressView {

	protected JunButtonViewSwing _stopButtonView;

	/**
	 * Create a new instance of JunProgressViewAwt and initialize it.
	 *
	 * @param newModel sra.jun.goodies.progress.JunProgress
	 */
	public JunProgressViewSwing(JunProgress newModel) {
		super(newModel);
	}

	/**
	 * Create a new view for the JunProgress.
	 * The view consists of a JunProgressViewSwing and no button.
	 *
	 * @return sra.smalltalk.StView
	 * @param aProgress sra.jun.goodies.progress.JunProgress
	 */
	public static StView WithNoButton_(JunProgress aProgress) {
		return new StViewJPanel(aProgress) {
			protected void buildComponent() {
				JunProgress theProgress = (JunProgress) this.model();
				JunProgressViewSwing progressView = new JunProgressViewSwing(theProgress);

				JPanel borderedPanel = StApplicationModel._JPanelWithBorder();
				borderedPanel.setLayout(new BorderLayout());
				borderedPanel.add(progressView, BorderLayout.CENTER);

				this.setLayout(new BorderLayout());
				this.add(borderedPanel, BorderLayout.CENTER);
			}
		};
	}

	/**
	 * Create a new view for the JunProgress.
	 * The view consists of a JunProgressViewSwing and a stop button.
	 *
	 * @return sra.smalltalk.StView
	 * @param aProgress sra.jun.goodies.progress.JunProgress
	 */
	public static StView WithStopButton_(JunProgress aProgress) {
		return new StViewJPanel(aProgress) {
			protected void buildComponent() {
				JunProgress theProgress = (JunProgress) this.model();
				JunProgressViewSwing progressView = new JunProgressViewSwing(theProgress);

				JPanel borderedPanel = StApplicationModel._JPanelWithBorder();
				borderedPanel.setLayout(new BorderLayout());
				borderedPanel.add(progressView, BorderLayout.CENTER);

				progressView._stopButtonView = new JunButtonViewSwing(theProgress.stopButton(), new Dimension(60, 24));

				JPanel buttonPanel = new JPanel() {
					public Insets getInsets() {
						return new Insets(10, 10, 10, 10);
					}
				};
				buttonPanel.setLayout(new BorderLayout());
				buttonPanel.add(progressView._stopButtonView, BorderLayout.SOUTH);

				this.setLayout(new BorderLayout());
				this.add(borderedPanel, BorderLayout.CENTER);
				this.add(buttonPanel, BorderLayout.EAST);
			}
		};
	}

	/**
	 * Display the message using the Graphics.
	 * 
	 * @param graphics java.awt.Graphics
	 */
	public void displayMessageOn_(Graphics graphics) {
		String messageString = this.getProgress().message();
		if (messageString == null) {
			messageString = new String();
		}
		StComposedText composedText = new StComposedText(messageString, MessageFont);
		StRectangle displayRectangle = new StRectangle(composedText.bounds());
		StRectangle scaleRectangle = new StRectangle(0, 0, 200 + 1, displayRectangle.height());
		Dimension size = this.getSize();
		StRectangle clippingRectangle = new StRectangle(0, size.height / 3 * 0, size.width, size.height / 3 * 1);
		scaleRectangle = scaleRectangle.align_with_(scaleRectangle.bottomCenter(), clippingRectangle.bottomCenter());
		displayRectangle = displayRectangle.align_with_(displayRectangle.bottomLeft(), scaleRectangle.bottomLeft());
		graphics.setClip(clippingRectangle.originX(), clippingRectangle.originY(), clippingRectangle.width(), clippingRectangle.height());
		graphics.clearRect(clippingRectangle.originX(), clippingRectangle.originY(), clippingRectangle.width(), clippingRectangle.height());
		graphics.setColor(Color.black);
		composedText.displayOn_at_(graphics, displayRectangle.origin());
	}

	/**
	 * Display the receiver using the Graphics.
	 * 
	 * @param graphics java.awt.Graphics
	 */
	public void displayOn_(Graphics graphics) {
		Dimension size = this.getSize();
		Image offScreenImage = this.createImage(size.width, size.height);
		Graphics offScreenGraphics = offScreenImage.getGraphics();

		try {
			this.displayMessageOn_(offScreenGraphics);
			this.displayScaleOn_(offScreenGraphics);
			this.displayValueOn_(offScreenGraphics);
		} finally {
			graphics.drawImage(offScreenImage, 0, 0, this);
			offScreenGraphics.dispose();
			offScreenImage.flush();
		}
	}

	/**
	 * Display the scale using the Graphics.
	 * 
	 * @param graphics java.awt.Graphics
	 */
	public void displayScaleOn_(Graphics graphics) {
		StRectangle displayRectangle = new StRectangle(0, 0, 200 + 1, 10);
		Dimension bounds = this.getSize();
		StRectangle clippingRectangle = new StRectangle(0, bounds.height / 3 * 2, bounds.width, bounds.height);
		displayRectangle = displayRectangle.align_with_(displayRectangle.topCenter(), clippingRectangle.topCenter());
		graphics.setClip(clippingRectangle.x(), clippingRectangle.y(), clippingRectangle.width(), clippingRectangle.height());
		graphics.clearRect(clippingRectangle.x(), clippingRectangle.y(), clippingRectangle.width(), clippingRectangle.height());
		graphics.setColor(Color.black);
		int x = displayRectangle.left();
		int y;

		for (int n = 0; n <= 100; n++) {
			if ((n % 5) == 0) {
				if ((n % 10) == 0) {
					if ((n % 50) == 0) {
						y = displayRectangle.y();
						graphics.drawLine(x, y, x, y + 10);
					} else {
						y = displayRectangle.y() + 2;
						graphics.drawLine(x, y, x, y + 6);
					}
				} else {
					if ((n % 25) == 0) {
						y = displayRectangle.y() + 4;
						graphics.drawLine(x, y, x, y + 6);
					} else {
						y = displayRectangle.y() + 4;
						graphics.drawLine(x, y, x, y + 4);
					}
				}
			} else {
				y = displayRectangle.y() + 6;
				graphics.drawLine(x, y, x, y + 2);
			}

			x += 2;
		}
	}

	/**
	 * Display the value using the Graphics.
	 * 
	 * @param graphics java.awt.Graphics
	 */
	public void displayValueOn_(Graphics graphics) {
		float progressValue = this.getProgress().value();
		StRectangle gaugeRectangle = new StRectangle(0, 0, 6, 12);
		StRectangle displayRectangle = new StRectangle(0, 0, 200 + 1, 10);
		Dimension bounds = this.getSize();
		StRectangle clippingRectangle = new StRectangle(0, bounds.height / 3, bounds.width, bounds.height / 3 * 2);
		displayRectangle = displayRectangle.align_with_(displayRectangle.bottomCenter(), clippingRectangle.bottomCenter());
		gaugeRectangle = gaugeRectangle.align_with_(gaugeRectangle.bottomCenter(), displayRectangle.bottomLeft());
		gaugeRectangle = gaugeRectangle.translatedBy_((int) (progressValue * 200), -1);

		Polygon polygon = new Polygon();
		Point aPoint = gaugeRectangle.topCenter();
		polygon.addPoint(aPoint.x, aPoint.y);
		aPoint = gaugeRectangle.rightCenter();
		polygon.addPoint(aPoint.x, aPoint.y);
		aPoint = gaugeRectangle.bottomCenter();
		polygon.addPoint(aPoint.x, aPoint.y);
		aPoint = gaugeRectangle.leftCenter();
		polygon.addPoint(aPoint.x, aPoint.y);
		aPoint = gaugeRectangle.topCenter();
		polygon.addPoint(aPoint.x, aPoint.y);

		graphics.setClip(clippingRectangle.x(), clippingRectangle.y(), clippingRectangle.width(), clippingRectangle.height());
		graphics.clearRect(clippingRectangle.x(), clippingRectangle.y(), clippingRectangle.width(), clippingRectangle.height());
		graphics.setColor(Color.gray);
		graphics.fillPolygon(polygon);
		graphics.setColor(Color.black);
		graphics.drawPolyline(polygon.xpoints, polygon.ypoints, polygon.npoints);
	}

	/**
	 * Answer my model as JunProgress.
	 * 
	 * @return sra.jun.goodies.progress.JunProgress
	 */
	public JunProgress getProgress() {
		return (JunProgress) this.model();
	}

	/**
	 * Update the receiver according to the change notification from the model.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public synchronized void update_(DependentEvent evt) {
		if (this.topComponent() != null) {
			if (this.getProgress().raisingFlag()) {
				this.topComponent().show();
			}
		}

		if (this.isShowing()) {
			if (evt.getAspect() == $("value") || evt.getAspect() == $("message")) {
				Graphics graphics = this.getGraphics();
				try {
					this.displayOn_(graphics);
				} finally {
					graphics.dispose();
				}
			}
		}
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setPreferredSize(new Dimension(226 - 4, 60 - 4));
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunProgress();
	}
}
