package sra.jun.goodies.prolog;

import java.io.*;

/**
 * JunPrologTerms class
 * 
 *  @author    kondo
 *  @created   1999/09/09 (by kondo)
 *  @updated   2003/04/30 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun301 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPrologTerms.java,v 6.0 2003/10/28 10:52:14 nisinaka Exp $
 */
public class JunPrologTerms extends JunPrologList {

	/**
	 * Create a new instance of JunPrologTerms and initialize it with the car part and the cdr part.
	 *
	 * @param anObject sra.jun.goodies.prolog.JunPrologEntity
	 * @param link sra.jun.goodies.prolog.JunPrologEntity
	 */
	public JunPrologTerms(JunPrologEntity anObject, JunPrologEntity link) {
		this.term_next_(anObject, link);
	}

	/**
	 * Create a new instance of JunPrologTerms from the list.
	 *
	 * @return sra.jun.goodies.prolog.JunPrologTerms
	 * @param aJunPrologDotPair sra.jun.goodies.prolog.JunPrologList
	 */
	public static JunPrologTerms FromList_(JunPrologList aJunPrologDotPair) {
		return FromReverseList_(aJunPrologDotPair.reverse());
	}

	/**
	 * Create a new instance of JunPrologTerms from the reverse list.
	 *
	 * @return sra.jun.goodies.prolog.JunPrologTerms
	 * @param aJunPrologDotPair sra.jun.goodies.prolog.JunPrologList
	 */
	public static JunPrologTerms FromReverseList_(JunPrologList aJunPrologDotPair) {
		JunPrologEntity list = aJunPrologDotPair;
		JunPrologList mark = null;
		JunPrologTerms terms = null;
		while (list != null && list.consp()) {
			if (terms == null) {
				mark = terms = new JunPrologTerms(list.car(), terms);
			} else {
				terms = new JunPrologTerms(list.car(), terms);
			}
			list = list.cdr();
		}
		mark.cdr_(list);
		return terms;
	}

	/**
	 * Print my string representation on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws IOException if failed.
	 */
	public void printJunPrologOn_(Writer aWriter) throws IOException {
		aWriter.write('(');
		JunPrologEntity list = this;
		while (list.cdr() != null && list.cdr().consp()) {
			if (list != this) {
				aWriter.write(',');
			}
			list._carPrintJunPrologOn_(aWriter);
			list = list.cdr();
		}
		if (list != this) {
			aWriter.write(',');
		}

		list._carPrintJunPrologOn_(aWriter);

		if (list.cdr() == null) {
			aWriter.write(')');
		} else {
			aWriter.write('|');
			list.cdr().printJunPrologOn_(aWriter);
			aWriter.write(')');
		}
	}

	/**
	 * Print my string representation on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @param anInteger int
	 * @throws IOException if failed.
	 */
	public void printJunPrologOn_level_(Writer aWriter, int anInteger) throws IOException {
		this.printJunPrologOn_(aWriter);
	}

	/**
	 * Create a new instance with the specified arguments.
	 * This method have to be overrided to create a subclass specified instance.
	 *
	 * @return sra.jun.goodies.prolog.JunPrologList
	 * @param carEntity sra.jun.goodies.prolog.JunPrologEntity
	 * @param cdrEntity sra.jun.goodies.prolog.JunPrologEntity
	 */
	protected JunPrologList _newInstanceWith(JunPrologEntity carEntity, JunPrologEntity cdrEntity) {
		return new JunPrologTerms(carEntity, cdrEntity);
	}

	/**
	 * Set the car part and the cdr part.
	 *
	 * @param anObject sra.jun.goodies.prolog.JunPrologEntity
	 * @param link sra.jun.goodies.prolog.JunPrologEntity
	 */
	protected void term_next_(JunPrologEntity anObject, JunPrologEntity link) {
		carPart = anObject;
		cdrPart = link;
	}
}
