package sra.jun.goodies.sound;

import java.awt.*;
import sra.smalltalk.*;

/**
 * JunSoundLevelMeterViewAwt class
 * 
 *  @author    Hoshi Takanori
 *  @created   2003/08/29 (by Hoshi Takanori)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun472 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunSoundLevelMeterViewAwt.java,v 6.0 2003/10/28 10:52:14 nisinaka Exp $
 */
public class JunSoundLevelMeterViewAwt extends StViewCanvas implements JunSoundLevelMeterView {

	/**
	 * Create a new instance of JunSoundLevelMeterViewAwt and initialize it.
	 *
	 * @param aSoundMeterModel sra.jun.goodies.sound.JunSoundMeterModel
	 */
	public JunSoundLevelMeterViewAwt(JunSoundMeterModel aSoundMeterModel) {
		super(aSoundMeterModel);
	}

	/**
	 * Display on a graphics context.
	 * 
	 * @param graphicsContext java.awt.Graphics
	 */
	public void displayOn_(Graphics graphicsContext) {
		if (!this.isShowing()) {
			return;
		}

		Dimension size = this.getSize();
		graphicsContext.setColor(Color.white);
		graphicsContext.fillRect(0, 0, size.width, size.height);
		graphicsContext.setColor(Color.gray);
		graphicsContext.drawLine(size.width / 2, 0, size.width / 2, size.height);
	}

	/**
	 * Display the level value on the graphics context.
	 *
	 * @param graphicsContext java.awt.Graphics
	 * @param levelValue double[]
	 */
	public void displayOn_levelValue_(Graphics graphicsContext, double[] levelValue) {
		Dimension size = this.getSize();
		int left = (int) Math.round(levelValue[0] * size.width);
		int right = (int) Math.round(levelValue[1] * size.width);
		graphicsContext.setColor(Color.white);
		graphicsContext.fillRect(0, 0, left, size.height);
		graphicsContext.fillRect(right, 0, size.width - right, size.height);
		float darkness = 0.75f - (float) (levelValue[1] - levelValue[0]) * 0.5f;
		graphicsContext.setColor(new Color(darkness, darkness, darkness));
		graphicsContext.fillRect(left, 0, right - left, size.height);
		graphicsContext.setColor(Color.gray);
		graphicsContext.drawLine(size.width / 2, 0, size.width / 2, size.height);
	}

	/**
	 * Action for the update notification.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		StSymbol aSymbol = evt.getAspect();
		if (aSymbol == $("level")) {
			Object parameter = evt.getParameter();
			if (parameter instanceof double[]) {
				Graphics graphics = this.getGraphics();
				if (graphics != null) {
					this.displayOn_levelValue_(graphics, (double[]) parameter);
				}
			}
		} else if (aSymbol == $("wave")) {
			return;
		} else {
			super.update_(evt);
		}
	}
}
