package sra.jun.goodies.track;

import java.io.*;
import sra.smalltalk.*;

/**
 * JunSimpleSliderModel class
 * 
 *  @author    Hoshi Takanori
 *  @created   2003/08/27 (by Hoshi Takanori)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun475 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunSimpleSliderModel.java,v 6.0 2003/10/28 10:52:14 nisinaka Exp $
 */
public class JunSimpleSliderModel extends StApplicationModel implements StBlockValued {

	protected StValueHolder valueHolder;

	/**
	 * Create a new instance of JunSimpleSliderModel.
	 */
	public JunSimpleSliderModel() {
		this.initialize();
	}

	/**
	 * Answer a BlockValue.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return sra.smalltalk.StBlockValue
	 */
	public StBlockValue compute_(StBlockClosure aBlock) {
		return new StBlockValue(aBlock, this);
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunSimpleSliderViewAwt(this);
		} else {
			return new JunSimpleSliderViewSwing(this);
		}
	}

	/**
	 * Answer my value as double.
	 * 
	 * @return java.lang.Object
	 */
	public double doubleValue() {
		return ((Number) this.value()).doubleValue();
	}

	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws IOException
	 */
	public void printOn_(Writer aWriter) throws IOException {
		super.printOn_(aWriter);
		aWriter.write(" (");
		aWriter.write(String.valueOf(this.value()));
		aWriter.write(')');
	}

	/**
	 * Get the value of valueHolder.
	 * 
	 * @return java.lang.Object
	 */
	public Object value() {
		return valueHolder.value();
	}

	/**
	 * Set the value of valueHolder.
	 * 
	 * @param normalizedNumber double
	 */
	public void value_(double normalizedNumber) {
		valueHolder.value_(Math.max(0, Math.min(normalizedNumber, 1)));
		this.changed_($("value"));
	}

	/**
	 * Get the valueHolder with double value.
	 * 
	 * @return sra.smalltalk.StValueHolder
	 */
	public StValueHolder valueHolder() {
		if (valueHolder == null) {
			valueHolder = new StValueHolder(0.0d);
		}
		return valueHolder;
	}

	/**
	 * Initialize the JunSimpleSliderModel.
	 */
	protected void initialize() {
		super.initialize();

		valueHolder = null;
		this.valueHolder();
	}
}
