package sra.jun.goodies.track;

import sra.smalltalk.*;
import sra.jun.goodies.button.*;

/**
 * JunTrackerViewAwt class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/11/24 (by MATSUDA Ryouichi)
 *  @updated   1999/12/10 (by MATSUDA Ryouichi)
 *  @updated   2002/10/30 (by MATSUDA Ryouichi)
 *  @updated   2003/03/24 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun472 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTrackerViewAwt.java,v 6.0 2003/10/28 10:52:14 nisinaka Exp $
 */
public class JunTrackerViewAwt extends StViewPanel implements JunTrackerView {

	protected JunTrackSliderViewAwt sliderView;
	protected JunButtonViewAwt playButtonView;
	protected JunButtonViewAwt loopButtonView;
	protected JunButtonViewAwt previousButtonView;
	protected JunButtonViewAwt nextButtonView;
	protected JunButtonViewAwt firstMarkerButtonView;
	protected JunButtonViewAwt lastMarkerButtonView;
	protected JunButtonViewAwt firstButtonView;
	protected JunButtonViewAwt lastButtonView;

	/**
	 * Create a new instance of JunTrackerViewAwt and initialize it.
	 * 
	 * @param aTrackerModel
	 */
	public JunTrackerViewAwt(JunTrackerModel aTrackerModel) {
		super(aTrackerModel);
	}

	/**
	 * model property accessing
	 * 
	 * @return sra.jun.goodies.track.JunTrackerModel
	 * @deprecated since Jun379, use getTrackerModel()
	 */
	public JunTrackerModel getModel() {
		return (JunTrackerModel) this.model();
	}

	/**
	 * Answer my model as JunTrackerModel.
	 * 
	 * @return sra.jun.goodies.track.JunTrackerModel
	 */
	public JunTrackerModel getTrackerModel() {
		return (JunTrackerModel) this.model();
	}

	/**
	 * Stop thread before component is removed.
	 */
	public void removeNotify() {
		this.getTrackerModel().end();
		super.removeNotify();
	}

	/**
	 * model property accessing
	 * 
	 * @param aModel sra.jun.goodies.track.JunTrackerModel
	 * @deprecated since Jun379
	 */
	public void setModel(JunTrackerModel aModel) {
		model_(aModel);
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.sliderView(), WindowSpecLayout.Position(0f, 39, 1f, -19, 1f, -39, 1f, -1));
		this.add(this.playButtonView(), WindowSpecLayout.Position(0f, 1, 1f, -19, 0f, 19, 1f, -1));
		this.add(this.loopButtonView(), WindowSpecLayout.Position(0f, 20, 1f, -19, 0f, 38, 1f, -1));
		this.add(this.previousButtonView(), WindowSpecLayout.Position(1f, -38, 1f, -19, 1f, -20, 1f, -1));
		this.add(this.nextButtonView(), WindowSpecLayout.Position(1f, -19, 1f, -19, 1f, -1, 1f, -1));
		this.setSize(200, 100);
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunTrackerModel();
	}

	/**
	 * Answer my first button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt firstButtonView() {
		if (firstButtonView == null) {
			firstButtonView = new JunButtonViewAwt(this.getTrackerModel().firstButton());
		}
		return firstButtonView;
	}

	/**
	 * Answer my first marker button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt firstMarkerButtonView() {
		if (firstMarkerButtonView == null) {
			firstMarkerButtonView = new JunButtonViewAwt(this.getTrackerModel().firstMarkerButton());
		}
		return firstMarkerButtonView;
	}

	/**
	 * Answer my last button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt lastButtonView() {
		if (lastButtonView == null) {
			lastButtonView = new JunButtonViewAwt(this.getTrackerModel().lastButton());
		}
		return lastButtonView;
	}

	/**
	 * Answer my last marker button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt lastMarkerButtonView() {
		if (lastMarkerButtonView == null) {
			lastMarkerButtonView = new JunButtonViewAwt(this.getTrackerModel().lastMarkerButton());
		}
		return lastMarkerButtonView;
	}

	/**
	 * Answer my loop button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt loopButtonView() {
		if (loopButtonView == null) {
			loopButtonView = new JunButtonViewAwt(this.getTrackerModel().loopButton());
		}
		return loopButtonView;
	}

	/**
	 * Answer my next button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt nextButtonView() {
		if (nextButtonView == null) {
			nextButtonView = new JunButtonViewAwt(this.getTrackerModel().nextButton());
		}
		return nextButtonView;
	}

	/**
	 * Answer my play button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt playButtonView() {
		if (playButtonView == null) {
			playButtonView = new JunButtonViewAwt(this.getTrackerModel().playButton());
		}
		return playButtonView;
	}

	/**
	 * Answer my previous button view.
	 *
	 * @return sra.jun.goodies.button.JunButtonViewAwt
	 */
	public JunButtonViewAwt previousButtonView() {
		if (previousButtonView == null) {
			previousButtonView = new JunButtonViewAwt(this.getTrackerModel().previousButton());
		}
		return previousButtonView;
	}

	/**
	 * Answer my slider view.
	 *
	 * @return sra.jun.goodies.track.JunTrackSliderViewAwt
	 */
	public JunTrackSliderViewAwt sliderView() {
		if (sliderView == null) {
			sliderView = new JunTrackSliderViewAwt(this.getTrackerModel());
		}
		return sliderView;
	}
}
