package sra.jun.goodies.wheels;

import java.io.*;
import sra.smalltalk.*;

/**
 * JunThumbWheel class
 * 
 *  @author    He Weijie
 *  @created   1998/08/13 (by He Weijie)
 *  @updated   1999/12/17 (by MATSUDA Ryouichi)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunThumbWheel.java,v 6.0 2003/10/28 10:52:14 nisinaka Exp $
 */
public class JunThumbWheel extends StApplicationModel {
	/** A boolean value as a button model. */
	protected StValueHolder angularValue;

	/**
	 * Construsctor
	 */
	public JunThumbWheel() {
		final JunThumbWheel self = this;
		angularValue = new StValueHolder(0.0);
		angularValue.compute_(new StBlockClosure() {
			public Object value() {
				self.changed_with_($("value"), new Double(self.angularValue()));

				return null;
			}
		});
	}

	/**
	 * Just return the current angularValue.
	 * 
	 * @return double
	 */
	public double angularValue() {
		return ((Number) angularValue.value()).doubleValue();
	}

	/**
	 * Set the new angularValue.
	 * 
	 * @param aValue double
	 */
	public void angularValue_(double aValue) {
		angularValue.value_(new Double(aValue));
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return sra.smalltalk.StBlockValue
	 */
	public StBlockValue compute_(StBlockClosure aBlock) {
		return new StBlockValue(aBlock, angularValue);
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunThumbWheelViewAwt(this);
		} else {
			return new JunThumbWheelViewSwing(this);
		}
	}

	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * 
	 * @throws IOException DOCUMENT ME!
	 */
	public void printOn_(Writer aWriter) throws IOException {
		super.printOn_(aWriter);
		aWriter.write(" on: ");
		aWriter.write(String.valueOf(this.angularValue()));
	}
}
