package sra.jun.goodies.wheels;

import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;

/**
 * JunThumbWheelController class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/12/17 (by MATSUDA Ryouichi)
 *  @updated   2000/11/16 (by Mitsuhiro Asada)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunThumbWheelController.java,v 6.0 2003/10/28 10:52:14 nisinaka Exp $
 */
public class JunThumbWheelController extends StController implements MouseListener, MouseMotionListener {
	protected Point previousPoint;

	/**
	 * general dragging action.
	 * 
	 * @param event java.awt.event.MouseEvent
	 */
	public void mouseDragged(MouseEvent event) {
		if (previousPoint != null) {
			Point currentPoint = event.getPoint();

			if (currentPoint.equals(previousPoint) == false) {
				this.getView().rotateDelta_(new Point(currentPoint.x - previousPoint.x, currentPoint.y - previousPoint.y));
				previousPoint = currentPoint;
			}
		}
	}

	/**
	 * Invoked when a mouse button has been pressed.
	 * 
	 * @param event java.awt.event.MouseEvent
	 */
	public void mousePressed(MouseEvent event) {
		if (event.isMetaDown()) {
			//
		} else if (event.isAltDown()) {
			//
		} else {
			((Component) view).setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
			previousPoint = event.getPoint();
		}
	}

	/**
	 * Invoked when a mouse button has been released. cursor reset to default.
	 * 
	 * @param event java.awt.event.MouseEvent
	 */
	public void mouseReleased(MouseEvent event) {
		if (previousPoint != null) {
			((Component) view).setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			previousPoint = null;
		}
	}

	/**
	 * add listeners.
	 * 
	 * @param newView sra.mvctest.StView
	 */
	protected void buildListener(StView newView) {
		final Component aView = (Component) newView;
		aView.addMouseListener(this);
		aView.addMouseMotionListener(this);
	}

	/**
	 * Answer this view.
	 * 
	 * @return sra.jun.goodies.wheels.JunThumbWheelView interface
	 */
	protected JunThumbWheelView getView() {
		return (JunThumbWheelView) this.view();
	}
}
