package sra.jun.metaball.abstracts;

import sra.jun.geometry.basic.*;
import sra.smalltalk.*;

/**
 * JunDistanceMetaball class
 * 
 *  @author    nishihara
 *  @created   1999/11/05 (by nishihara)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunDistanceMetaball.java,v 6.0 2003/10/28 10:52:15 nisinaka Exp $
 */
public abstract class JunDistanceMetaball extends JunMetaball {
	public static final double ACCURACY = 1.0e-12d;

	/** number of weight. */
	protected double weight;

	/** number of order. */
	protected int order;

	/**
	 * Create a new instance of JunDistanceMetaball and initialize it.
	 * 
	 * @return sra.jun.metaball.abstracts.JunDistanceMetaball
	 */
	public JunDistanceMetaball() {
		super();
		this.initialize();
	}

	/**
	 * Create a new instance of JunDistanceMetaball and initialize it.
	 * 
	 * @param number1 int
	 * @param number2 double
	 */
	public JunDistanceMetaball(int number1, double number2) {
		this();
		this.order_(number1);
		this.weight_(number2);
	}

	/**
	 * Answer the accuracy.
	 * 
	 * @return double
	 */
	public static final double Accuracy() {
		return ACCURACY;
	}

	/**
	 * Answer the default order.
	 * 
	 * @return int
	 */
	public static final int defaultOrder() {
		return 2;
	}

	/**
	 * Answer the default weight.
	 * 
	 * @return double
	 */
	public static final double defaultWeight() {
		return 1.0d;
	}

	/**
	 * All subclasses must implement distance_(Jun3dPoint aJun3dPoint).
	 * 
	 * @param aJun3dPoint DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public double distance_(Jun3dPoint aJun3dPoint) {
		throw SmalltalkException.SubclassResponsibility();
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @return DOCUMENT ME!
	 */
	public JunDistanceMetaball initialize() {
		weight = this.defaultWeight();
		order = this.defaultOrder();

		return this;
	}

	/**
	 * Answer the order.
	 * 
	 * @return int
	 */
	public int order() {
		return this.order;
	}

	/**
	 * Set the order.
	 * 
	 * @param aNumber
	 */
	public void order_(int aNumber) {
		this.order = aNumber;
	}

	/**
	 * Answer a value at the specified point.
	 * 
	 * @param aJun3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return double
	 */
	public double valueAt_(Jun3dPoint aJun3dPoint) {
		double distance = this.distance_(aJun3dPoint);
		final int sign = ((this.weight == 0) ? 0 : ((this.weight > 0) ? 1 : -1));

		if (distance < ACCURACY) {
			distance = ACCURACY;
		}

		return Math.pow((Math.abs(this.weight) / distance), this.order) * sign;
	}

	/**
	 * Answer the weight.
	 * 
	 * @return double
	 */
	public double weight() {
		return this.weight;
	}

	/**
	 * Set the weight.
	 * 
	 * @param aNumber
	 */
	public void weight_(double aNumber) {
		this.weight = aNumber;
	}
}
