package sra.jun.metaball.atoms;

import sra.jun.geometry.basic.*;
import sra.jun.metaball.abstracts.*;
import sra.jun.solid.abstracts.*;

/**
 * JunSolidMetaball class
 * 
 *  @author    nishihara
 *  @created   1999/11/05 (by nishihara)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunSolidMetaball.java,v 6.0 2003/10/28 10:52:16 nisinaka Exp $
 */
public class JunSolidMetaball extends JunMetaball {
	public static final double defaultInsideValue = 1.0d;
	public static final double defaultOutsideValue = 0.0d;

	/** solid. */
	protected JunSolid solid;

	/** insideValue. */
	protected double insideValue;

	/** outsideValue. */
	protected double outsideValue;

	/**
	 * Create a new instance of JunSolidMetaball and set it with solid.
	 * 
	 * @param aJunSolid
	 * 
	 * @return sra.jun.metaball.atoms.JunSolidMetaball
	 */
	public static JunSolidMetaball On_(JunSolid aJunSolid) {
		JunSolidMetaball metaball = new JunSolidMetaball();
		metaball.initialize();
		metaball.solid_(aJunSolid);

		return metaball;
	}

	/**
	 * Create a new instance of JunSolidMetaball and set it with solid,
	 * insideValue and outsideValue.
	 * 
	 * @param aJunSolid
	 * @param aNumber1
	 * @param aNumber2
	 * 
	 * @return sra.jun.metaball.atoms.JunSolidMetaball
	 */
	public static JunSolidMetaball On_insideValue_outsideValue_(JunSolid aJunSolid, double aNumber1, double aNumber2) {
		JunSolidMetaball metaball = JunSolidMetaball.On_(aJunSolid);
		metaball.insideValue_(aNumber1);
		metaball.outsideValue_(aNumber2);

		return metaball;
	}

	/**
	 * Initialize the receiver.
	 * 
	 * @return DOCUMENT ME!
	 */
	public JunSolidMetaball initialize() {
		this.insideValue = this.defaultInsideValue;
		this.outsideValue = this.defaultInsideValue;

		return this;
	}

	/**
	 * anser the insideValue.
	 * 
	 * @return double
	 */
	public double insideValue() {
		return this.insideValue;
	}

	/**
	 * set the insideValue.
	 * 
	 * @param aNumber
	 */
	public void insideValue_(double aNumber) {
		this.insideValue = aNumber;
	}

	/**
	 * anser the outsideValue.
	 * 
	 * @return double
	 */
	public double outsideValue() {
		return this.outsideValue;
	}

	/**
	 * set the outsideValue.
	 * 
	 * @param aNumber
	 */
	public void outsideValue_(double aNumber) {
		this.outsideValue = aNumber;
	}

	/**
	 * answer the solid.
	 * 
	 * @return sra.jun.solid.abstracts.aJunSolid
	 */
	public JunSolid solid() {
		return this.solid;
	}

	/**
	 * set the solid.
	 * 
	 * @param aJunSolid
	 */
	public void solid_(JunSolid aJunSolid) {
		this.solid = aJunSolid;
	}

	/**
	 * Answer a value at the specified point.
	 * 
	 * @param aJun3dPoint
	 * 
	 * @return double
	 */
	public double valueAt_(Jun3dPoint aJun3dPoint) {
		if (this.solid.containsPoint_(aJun3dPoint)) {
			return this.insideValue;
		} else {
			return this.outsideValue;
		}
	}
}
