package sra.jun.octree.basic;

import sra.jun.geometry.basic.*;
import sra.jun.octree.nodes.*;
import sra.smalltalk.StBlockClosure;

/**
 * Jun27Tree class
 * 
 *  @author    nisinaka
 *  @created   1999/12/28 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: Jun27Tree.java,v 6.0 2003/10/28 10:52:16 nisinaka Exp $
 */
public class Jun27Tree extends JunOctree {

	/**
	 * Create an instance of Jun27Tree and initialize it.
	 * 
	 * @param aJun3dBoundingBox sra.jun.geometry.basic.Jun3dBoundingBox
	 * 
	 * @return sra.jun.octree.basic.JunOctree
	 */
	public static JunOctree Bounds_(Jun3dBoundingBox aJun3dBoundingBox) {
		JunOctree octree = new Jun27Tree();
		octree.setBounds_(aJun3dBoundingBox);

		return octree;
	}

	/**
	 * Create an instance of Jun27Tree and initialize it.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @param anInteger int
	 * @param aJun3dBoundingBox sra.jun.geometry.basic.Jun3dBoundingBox
	 * 
	 * @return sra.jun.octree.basic.JunOctree
	 */
	public static JunOctree FromBlock_depth_bounds_(StBlockClosure aBlock, int anInteger, Jun3dBoundingBox aJun3dBoundingBox) {
		JunOctree octree = Jun27Tree.Bounds_(aJun3dBoundingBox);
		octree.block_depth_(aBlock, anInteger);

		return octree;
	}

	/**
	 * Create a unit JunOctree.
	 * 
	 * @return sra.jun.octree.basic.JunOctree
	 */
	public static JunOctree Unity() {
		return Jun27Tree.Bounds_(Jun3dBoundingBox.Origin_corner_(Jun3dPoint.Zero(), Jun3dPoint.Unity()));
	}

	/**
	 * Create a new node.
	 * 
	 * @return sra.jun.octree.nodes.JunOctreeNode
	 */
	protected JunOctreeNode _newNode() {
		return Jun27TreeNode.Filled();
	}
}
