package sra.jun.octree.editor;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.util.*;
import sra.smalltalk.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.curves.Jun3dLine;
import sra.jun.geometry.surfaces.*;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.goodies.button.JunButtonModel;
import sra.jun.goodies.cursors.JunCursors;
import sra.jun.metaball.abstracts.JunMetaball;
import sra.jun.metaball.atoms.JunMetaSphere;
import sra.jun.metaball.solid.JunMetaballSolid;
import sra.jun.octree.basic.JunOctree;
import sra.jun.octree.nodes.JunOctreeNode;
import sra.jun.opengl.display.*;
import sra.jun.opengl.lights.JunOpenGLLight;
import sra.jun.opengl.objects.*;
import sra.jun.opengl.picking.JunOpenGLPickingRenderingContext;
import sra.jun.opengl.projection.*;
import sra.jun.opengl.support.JunOpenGLRenderingContext;
import sra.jun.system.framework.JunDialog;

/**
 * JunOctreeEditModel class
 * 
 *  @author    nisinaka
 *  @created   1999/12/28 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun476 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOctreeEditModel.java,v 6.0 2003/10/28 10:52:16 nisinaka Exp $
 */
public class JunOctreeEditModel extends JunOpenGLDisplayModel {

	public static final double ThetaAccuracy = 0.001;

	/** The octrees to be edited. */
	protected JunOctree[] octrees = null;

	/** Selected octree nodes. */
	protected Vector selection = null;
	//
	protected JunOpenGLDisplayLight[] displayLights = null;
	protected JunOpenGLProjector displayProjector = null;
	protected Jun3dPoint defaultEyePoint = null;
	protected Jun3dPoint defaultSightPoint = null;
	protected Jun3dPoint defaultUpVector = null;
	protected Double defaultViewFactor = null;
	//
	protected StMenuModel menuModel;
	//
	protected JunButtonModel pickButton = null;
	protected JunButtonModel grabButton = null;
	protected JunButtonModel dragButton = null;
	protected JunButtonModel dollyButton = null;
	protected JunButtonModel focusButton = null;
	//
	private int pickTipProjection_index;

	protected StMenu _manipulateMenu;

	/**
	 * Constructor Method
	 */
	public JunOctreeEditModel() {
		super();
		initialize();
	}

	/**
	 * Create a new instance of JunOctreeEditModel with the specified
	 * JunOctree.
	 * 
	 * @param aJunOctree sra.jun.octree.basic.JunOctree
	 */
	public JunOctreeEditModel(JunOctree aJunOctree) {
		this();

		Vector octrees = new Vector(1);
		octrees.addElement(aJunOctree);
		this.setOctrees_(octrees);
	}

	/**
	 * Create a new instance of JunOctreeEditModel with the specified JunOctree
	 * and open it.
	 * 
	 * @param aJunOctree sra.jun.octree.basic.JunOctree
	 * 
	 * @return sra.jun.octree.editor.JunOctreeEditModel
	 */
	public static JunOctreeEditModel Edit_(JunOctree aJunOctree) {
		JunOctreeEditModel editModel = new JunOctreeEditModel(aJunOctree);
		editModel.open();

		return editModel;
	}

	/**
	 * Create a new instance of JunOctreeEditModel with the specified
	 * JunOctree.
	 * 
	 * @param aJunOctree sra.jun.octree.basic.JunOctree
	 * 
	 * @return sra.jun.octree.editor.JunOctreeEditModel
	 */
	public static JunOctreeEditModel On_(JunOctree aJunOctree) {
		return new JunOctreeEditModel(aJunOctree);
	}

	/**
	 * Add a selected node.
	 * 
	 * @param aJunOctreeNode sra.jun.octree.nodes.JunOctreeNode
	 */
	public void addSelection_(JunOctreeNode aJunOctreeNode) {
		if (selection.contains(aJunOctreeNode) == false) {
			selection.addElement(aJunOctreeNode);
		}
	}

	/**
	 * Called when a node is picked with the alt key down.
	 * 
	 * @param pickedNode sra.jun.octree.nodes.JunOctreeNode
	 */
	public void altClick_(JunOctreeNode pickedNode) {
		// do nothing.
	}

	/**
	 * Answer the bounding box of the octrees.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dBoundingBox
	 */
	public Jun3dBoundingBox boundingBox() {
		if (this.hasOctree() == false) {
			return Jun3dBoundingBox.Origin_corner_(new Jun3dPoint(0, 0, 0), new Jun3dPoint(0, 0, 0));
		} else {
			Jun3dBoundingBox bounds = octrees[0].bounds();

			for (int i = 1; i < octrees.length; i++) {
				bounds = (Jun3dBoundingBox) bounds.merge_(octrees[i].bounds());
			}

			return bounds;
		}
	}

	/**
	 * Answer the current button status.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public StSymbol buttonState() {
		if (pickButton.value() == true) {
			return $("pick");
		} else if (grabButton.value() == true) {
			return $("grab");
		} else if (dragButton.value() == true) {
			return $("drag");
		} else if (focusButton.value() == true) {
			return $("focus");
		}

		return $("none");
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	public void changeEyePoint() {
		Jun3dPoint point = this.displayProjector().eyePoint();
		float x = (float) point.x();
		float y = (float) point.y();
		float z = (float) point.z();
		String textValue = StDialog.Request_("Input a new eye point (x, y, z).", "(" + x + "," + y + "," + z + ")");

		if (textValue != null) {
			try {
				StringTokenizer st = new StringTokenizer(textValue, "(), ");
				x = Float.valueOf(st.nextToken()).floatValue();
				y = Float.valueOf(st.nextToken()).floatValue();
				z = Float.valueOf(st.nextToken()).floatValue();
				this.eyePoint_(new Jun3dPoint(x, y, z));
			} catch (Exception e) {
				StDialog.Warn_("invalid value");
			}
		}
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	public void changeSightPoint() {
		Jun3dPoint point = this.displayProjector().sightPoint();
		float x = (float) point.x();
		float y = (float) point.y();
		float z = (float) point.z();
		String textValue = StDialog.Request_("Input a new sight point (x, y, z).", "(" + x + "," + y + "," + z + ")");

		if (textValue != null) {
			try {
				StringTokenizer st = new StringTokenizer(textValue, "(), ");
				x = Float.valueOf(st.nextToken()).floatValue();
				y = Float.valueOf(st.nextToken()).floatValue();
				z = Float.valueOf(st.nextToken()).floatValue();
				this.sightPoint_(new Jun3dPoint(x, y, z));
			} catch (Exception e) {
				StDialog.Warn_("invalid value");
			}
		}
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	public void changeUpVector() {
		Jun3dPoint point = this.displayProjector().upVector();
		float x = (float) point.x();
		float y = (float) point.y();
		float z = (float) point.z();
		String textValue = StDialog.Request_("Input a new up vector (x, y, z).", "(" + x + "," + y + "," + z + ")");

		if (textValue != null) {
			try {
				StringTokenizer st = new StringTokenizer(textValue, "(), ");
				x = Float.valueOf(st.nextToken()).floatValue();
				y = Float.valueOf(st.nextToken()).floatValue();
				z = Float.valueOf(st.nextToken()).floatValue();
				this.upVector_(new Jun3dPoint(x, y, z));
			} catch (Exception e) {
				StDialog.Warn_("invalid value");
			}
		}
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	public void changeViewFactor() {
		double value = this.displayProjector().viewFactor();
		String textValue = StDialog.Request_("Input a new view factor.", (new Double(value)).toString());

		if (textValue != null) {
			try {
				value = Double.valueOf(textValue).doubleValue();
				this.viewFactor_(value);
			} catch (Exception e) {
				StDialog.Warn_("invalid value");
			}
		}
	}

	/**
	 * Clear the selected nodes.
	 */
	public void clearSelection() {
		selection = new Vector();
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	public void closeUp() {
		JunOpenGLProjection aProjection = this.displayProjection();
		double aRadius = aProjection.regularHeight() / 2;
		double aDistance = aProjection.sightPoint().distance_(aProjection.eyePoint());
		Jun3dLine aLine = new Jun3dLine(aProjection.sightPoint(), aProjection.eyePoint());
		Jun3dPoint aPoint = (Jun3dPoint) aLine.atT_(aRadius / aDistance);
		aProjection.eyePoint_(aPoint);
		aProjection.zoomHeight_(this.computeZoomHeight());
		this.displayProjection_(aProjection);
		this.changed_($("projection"));
	}

	/**
	 * Called when a node is picked with the control key down.
	 * 
	 * @param pickedNode sra.jun.octree.nodes.JunOctreeNode
	 */
	public void ctrlClick_(JunOctreeNode pickedNode) {
		// do nothing.
	}

	/**
	 * Answer the default eye point.
	 * 
	 * @return sra.jun.opengl.objects.Jun3dPoint
	 */
	public Jun3dPoint defaultEyePoint() {
		if (this.hasOctree() == false) {
			return new Jun3dPoint(10000, 10000, 10000);
		}

		if (defaultEyePoint == null) {
			Jun3dBoundingBox box = this.boundingBox();
			double distance = box.origin().distance_(box.corner());
			distance = distance * 2;

			return new Jun3dPoint(distance, distance, distance);
		}

		return defaultEyePoint;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param eyePoint sra.jun.opengl.objects.Jun3dPoint
	 */
	public void defaultEyePoint_(Jun3dPoint eyePoint) {
		defaultEyePoint = eyePoint;
	}

	/**
	 * Answer the default light color.
	 * 
	 * @return java.awt.Color
	 */
	public Color defaultLightColor() {
		return Color.gray;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return sra.jun.opengl.objects.Jun3dPoint
	 */
	public Jun3dPoint defaultLightPoint() {
		return new Jun3dPoint(0, 0, 1);
	}

	/**
	 * Answer the default sight point.
	 * 
	 * @return sra.jun.opengl.objects.Jun3dPoint
	 */
	public Jun3dPoint defaultSightPoint() {
		if (this.hasOctree() == false) {
			return new Jun3dPoint(0, 0, 0);
		}

		if (defaultSightPoint == null) {
			return (Jun3dPoint) this.boundingBox().center();
		}

		return defaultSightPoint;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param sightPoint sra.jun.opengl.objects.Jun3dPoint
	 */
	public void defaultSightPoint_(Jun3dPoint sightPoint) {
		defaultSightPoint = sightPoint;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return sra.jun.opengl.objects.Jun3dPoint
	 */
	public Jun3dPoint defaultUpVector() {
		if (defaultUpVector == null) {
			return new Jun3dPoint(-1, -1, 1);
		}

		return defaultUpVector;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param upVector sra.jun.opengl.objects.Jun3dPoint
	 */
	public void defaultUpVector_(Jun3dPoint upVector) {
		defaultUpVector = upVector;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return sra.jun.opengl.objects.Jun3dPoint
	 */
	public double defaultViewFactor() {
		if (defaultViewFactor == null) {
			return 10.0;
		}

		return defaultViewFactor.doubleValue();
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param viewFactor sra.jun.opengl.objects.Jun3dPoint
	 */
	public void defaultViewFactor_(double viewFactor) {
		defaultViewFactor = new Double(viewFactor);
	}

	/**
	 * Answer the first display light.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayLight
	 */
	public JunOpenGLDisplayLight displayLight1() {
		return this.displayLights()[0];
	}

	/**
	 * Answer the second display light.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayLight
	 */
	public JunOpenGLDisplayLight displayLight2() {
		return this.displayLights()[1];
	}

	/**
	 * Answer the third display light.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayLight
	 */
	public JunOpenGLDisplayLight displayLight3() {
		return this.displayLights()[2];
	}

	/**
	 * Answer the fourth display light.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayLight
	 */
	public JunOpenGLDisplayLight displayLight4() {
		return this.displayLights()[3];
	}

	/**
	 * Answer the fifth display light.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayLight
	 */
	public JunOpenGLDisplayLight displayLight5() {
		return this.displayLights()[4];
	}

	/**
	 * Answer the array of lights which are on.
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLLight[]
	 */
	public JunOpenGLLight[] displayLightCollection() {
		Vector lightCollection = new Vector();
		Jun3dTransformation t = (Jun3dTransformation) this.displayProjection().asEyeTransformation().inverse();

		for (int i = 0; i < this.displayLights().length; i++) {
			JunOpenGLDisplayLight light = this.displayLights()[i];

			if (light.isOn()) {
				lightCollection.addElement(light.light().transform_(t));
			}
		}

		int size = lightCollection.size();
		JunOpenGLLight[] lights = new JunOpenGLLight[size];
		lightCollection.copyInto(lights);

		return lights;
	}

	/**
	 * Answer the array of all display lights.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayLight[]
	 */
	public JunOpenGLDisplayLight[] displayLights() {
		if (displayLights == null) {
			displayLights = new JunOpenGLDisplayLight[5];
			displayLights[0] = JunOpenGLDisplayLight.ParallelLight_color_position_(true, this.defaultLightColor(), this.defaultLightPoint());
			displayLights[1] = new JunOpenGLDisplayLight();
			displayLights[2] = new JunOpenGLDisplayLight();
			displayLights[3] = new JunOpenGLDisplayLight();
			displayLights[4] = JunOpenGLDisplayLight.AmbientLight_color_(true, this.defaultLightColor());

			final JunOctreeEditModel self = this;

			for (int i = 0; i < displayLights.length; i++) {
				displayLights[i].compute_(new StBlockClosure() {
					public Object value() {
						self.updateLightMenuIndication();
						self.changed_($("light"));

						return null;
					}
				});
			}
		}

		return displayLights;
	}

	/**
	 * Answer the current projection.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjection
	 */
	public JunOpenGLProjection displayProjection() {
		return this.displayProjector().projection();
	}

	/**
	 * Set the projection.
	 * 
	 * @param aProjection sra.jun.opengl.projection.JunOpenGLProjection
	 */
	public void displayProjection_(JunOpenGLProjection aProjection) {
		this.displayProjector().projection_(aProjection);
	}

	/**
	 * Answer the current projector.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjector
	 */
	public JunOpenGLProjector displayProjector() {
		if (displayProjector == null) {
			displayProjector = new JunOpenGLProjector();
			displayProjector.eyePoint_(this.defaultEyePoint());
			displayProjector.sightPoint_(this.defaultSightPoint());
			displayProjector.upVector_(this.defaultUpVector());
			displayProjector.viewFactor_(this.defaultViewFactor());
		}

		return displayProjector;
	}

	/**
	 * Divide the selected nodes.
	 */
	public void divideSelection() {
		if (selection.isEmpty() == false) {
			JunOctreeNode[] nodes = new JunOctreeNode[selection.size()];
			selection.copyInto(nodes);

			Vector newSelection = new Vector(nodes.length * 8);

			for (int i = 0; i < nodes.length; i++) {
				nodes[i].beDivided();

				JunOctreeNode[] moreNodes = (JunOctreeNode[]) nodes[i].contents();

				for (int j = 0; j < moreNodes.length; j++) {
					if (newSelection.contains(moreNodes[j]) == false) {
						newSelection.addElement(moreNodes[j]);
					}
				}
			}

			selection = newSelection;
			this.changed_($("object"));
		}
	}

	/**
	 * Change the dolly and update the projector.
	 * 
	 * @param factor double
	 */
	public void dolly_(double factor) {
		double dollyFactor;
		Jun3dLine aLine;
		Jun3dPoint eyePoint;
		Jun3dPoint sightPoint;
		JunOpenGLProjector thisDisplayProjector = this.displayProjector();

		//if (StController._ShiftDown()) {
		//	dollyFactor = factor / 20;
		//} else {
		dollyFactor = factor;

		//}
		aLine = new Jun3dLine(thisDisplayProjector.eyePoint(), thisDisplayProjector.sightPoint());
		eyePoint = (Jun3dPoint) aLine.atT_(0.0 + dollyFactor);
		sightPoint = (Jun3dPoint) aLine.atT_(1.0 + dollyFactor);
		thisDisplayProjector.eyePoint_(eyePoint);
		thisDisplayProjector.sightPoint_(sightPoint);
		this.changed_($("projection"));
	}

	/**
	 * Initialize the dolly button.
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 */
	public JunButtonModel dollyButton() {
		if (dollyButton == null) {
			JunButtonModel button = new JunButtonModel();
			button.value_(false);
			button.visual_(JunCursors.DollyCursor());
			button.action_(new StBlockClosure() {
				public Object value_(Object o) {
					JunButtonModel model = (JunButtonModel) o;
					model.value_(!model.value());

					return model;
				}
			});
			dollyButton = button;
		}

		return dollyButton;
	}

	/**
	 * Answer my drag button.
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 */
	public JunButtonModel dragButton() {
		if (dragButton == null) {
			final JunOctreeEditModel self = this;
			JunButtonModel button = new JunButtonModel();
			button.value_(false);
			button.visual_(JunCursors.QuartersCursor());
			button.action_(new StBlockClosure() {
				public Object value_(Object o) {
					JunButtonModel model = (JunButtonModel) o;
					model.value_(!model.value());

					if (model.value()) {
						self.focusButton().value_(false);
						self.grabButton().value_(false);
						self.pickButton().value_(false);
					}

					self.changed_($("state"));

					return model;
				}
			});
			dragButton = button;
		}

		return dragButton;
	}

	/**
	 * Erase the selected nodes.
	 */
	public void eraseSelection() {
		if (selection.isEmpty() == false) {
			JunOctreeNode[] nodes = new JunOctreeNode[selection.size()];
			selection.copyInto(nodes);

			for (int i = 0; i < nodes.length; i++) {
				nodes[i].beEmpty();
			}

			this.changed_($("object"));
		}
	}

	/**
	 * Change the eye point and update the projector.
	 * 
	 * @param a3dPoint sra.jun.grometry.basic.Jun3dPoint
	 */
	public void eyePoint_(Jun3dPoint a3dPoint) {
		this.displayProjector().eyePoint_(a3dPoint);
		this.changed_($("projection"));
	}

	/**
	 * Fill the selected nodes.
	 */
	public void fillSelection() {
		if (selection.isEmpty() == false) {
			JunOctreeNode[] nodes = new JunOctreeNode[selection.size()];
			selection.copyInto(nodes);

			for (int i = 0; i < nodes.length; i++) {
				nodes[i].beFilled();
			}

			this.changed_($("object"));
		}
	}

	/**
	 * Fit the projector to display the entire objects.
	 */
	public void fit() {
		if (this.hasOctree()) {
			this.fitSilently();
			this.changed_($("projection"));
		}
	}

	/**
	 * Fit the sight point to the current view.
	 */
	public void fitSight() {
		if (this.hasOctree()) {
			Jun3dPoint sightPoint = this.computeSightPoint();
			this.sightPoint_(sightPoint);
		}
	}

	/**
	 * Fit the 3d object to the current view.
	 */
	public void fitSilently() {
		if (this.hasOctree()) {
			Jun3dPoint sightPoint = this.computeSightPoint();
			this.displayProjector().sightPoint_(sightPoint);

			double zoomHeight = this.computeZoomHeight();
			this.displayProjector().zoomHeight_(zoomHeight);
		}
	}

	/**
	 * Fit the zoom height.
	 */
	public void fitZoom() {
		if (this.hasOctree()) {
			double zoomHeight = this.computeZoomHeight();
			this.zoomHeight_(zoomHeight);
		}
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	public void flushProjector() {
		displayProjector = null;
	}

	/**
	 * Change the focus area.
	 * 
	 * @param originalRectangle sra.jun.geometry.basic.Jun2dBoundingBox
	 * @param newRectangle sra.jun.geometry.basic.Jun2dBoundingBox
	 */
	public void focus_to_(Jun2dBoundingBox originalRectangle, Jun2dBoundingBox newRectangle) {
		JunOpenGLProjection aProjection = this.displayProjection();
		Jun3dPoint originalCenter = aProjection.translateTo3dPointFromPoint_((Jun2dPoint) originalRectangle.origin().plus_(originalRectangle.corner()).dividedBy_(2));
		Jun3dPoint newCenter = aProjection.translateTo3dPointFromPoint_((Jun2dPoint) newRectangle.origin().plus_(newRectangle.corner()).dividedBy_(2));
		aProjection.sightPoint_((Jun3dPoint) aProjection.sightPoint().plus_(newCenter.minus_(originalCenter)));

		Jun3dPoint originalTop = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(0, originalRectangle.origin().y()));
		Jun3dPoint originalBottom = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(0, originalRectangle.corner().y()));
		Jun3dPoint originalLeft = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(originalRectangle.origin().x(), 0));
		Jun3dPoint originalRight = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(originalRectangle.corner().x(), 0));
		double originalHeight = originalTop.minus_(originalBottom).length();
		double originalWidth = originalLeft.minus_(originalRight).length();
		Jun3dPoint newTop = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(0, newRectangle.origin().y()));
		Jun3dPoint newBottom = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(0, newRectangle.corner().y()));
		Jun3dPoint newLeft = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(newRectangle.origin().x(), 0));
		Jun3dPoint newRight = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(newRectangle.corner().x(), 0));
		double newHeight = newTop.minus_(newBottom).length();
		double newWidth = newLeft.minus_(newRight).length();
		double newViewHeight = aProjection.regularHeight() * newHeight / originalHeight;
		double newViewWidth = newViewHeight * newWidth / newHeight;
		double newZoomHeight = newViewHeight;
		if (originalWidth / originalHeight * newViewHeight < newViewWidth) {
			double tmpViewHeight = originalWidth * newViewHeight / (originalHeight * newViewWidth);
			newZoomHeight = newViewHeight * tmpViewHeight;
		}

		if (((originalHeight * 0.01d) < newHeight) && ((newHeight * 0.01d) < originalHeight)) {
			aProjection.zoomHeight_(newZoomHeight);
		}

		this.displayProjection_(aProjection);
		this.changed_($("projection"));
	}

	/**
	 * Answer my focus button.
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 */
	public JunButtonModel focusButton() {
		if (focusButton == null) {
			final JunOctreeEditModel self = this;
			JunButtonModel button = new JunButtonModel();
			button.value_(false);
			button.visual_(JunCursors.GlassCursor());
			button.action_(new StBlockClosure() {
				public Object value_(Object o) {
					JunButtonModel model = (JunButtonModel) o;
					model.value_(!model.value());

					if (model.value()) {
						self.grabButton().value_(false);
						self.dragButton().value_(false);
						self.pickButton().value_(false);
					}

					self.changed_($("state"));

					return model;
				}
			});
			focusButton = button;
		}

		return focusButton;
	}

	/**
	 * Grab and rotate the 3d object.
	 * 
	 * @param deltaPoint sra.jun.grometry.basic.Jun2dPoint
	 */
	public void grab_(Jun2dPoint deltaPoint) {
		this.grab_(deltaPoint, false);
	}

	/**
	 * Grab and rotate the 3d object.
	 * 
	 * @param deltaPoint sra.jun.grometry.basic.Jun2dPoint
	 * @param shiftDown boolean
	 */
	public void grab_(Jun2dPoint deltaPoint, boolean shiftDown) {
		if (this.hasOctree()) {
			Jun2dPoint fromPoint = new Jun2dPoint(0, 0);
			Jun2dPoint toPoint;

			if (shiftDown) {
				toPoint = new Jun2dPoint(deltaPoint.x() / 10, deltaPoint.y() / 10);
			} else {
				toPoint = new Jun2dPoint(deltaPoint.x(), deltaPoint.y());
			}

			this.grab_xy_(fromPoint, toPoint);
		}
	}

	/**
	 * Grab and rotate the 3d object.
	 * 
	 * @param from2dPoint sra.jun.grometry.basic.Jun2dPoint
	 * @param to2dPoint sra.jun.grometry.basic.Jun2dPoint
	 */
	public void grab_xy_(Jun2dPoint from2dPoint, Jun2dPoint to2dPoint) {
		if (this.hasOctree()) {
			JunOpenGLProjection aProjection = this.displayProjection();
			JunSphereSurface grabSphere = this.grabSphere();
			Jun3dPoint fromPoint = this.grab3dPoint_(new Jun2dPoint(from2dPoint.x(), from2dPoint.y()));
			Jun3dPoint toPoint = this.grab3dPoint_(new Jun2dPoint(to2dPoint.x(), to2dPoint.y()));
			JunAngle rotationAngle = (new Jun3dLine(grabSphere.center(), toPoint)).angleWithLine_(new Jun3dLine(grabSphere.center(), fromPoint));

			if (Math.abs(rotationAngle.rad()) > ThetaAccuracy) {
				Jun3dLine rotationAxis = new Jun3dLine(grabSphere.center(), (Jun3dPoint) grabSphere.center().minus_(((Jun3dPoint) fromPoint.minus_(grabSphere.center())).product_((Jun3dPoint) toPoint.minus_(grabSphere.center()))));
				Jun3dTransformation transformation = Jun3dTransformation.Rotate_around_(rotationAngle, rotationAxis);
				Jun3dTransformation transformationInv = Jun3dTransformation.Rotate_around_(rotationAngle.mul_(-1), rotationAxis);
				Jun3dPoint upPoint = (Jun3dPoint) aProjection.sightPoint().plus_(aProjection.unitUpVector());
				aProjection.eyePoint_(transformationInv.applyTo_(aProjection.eyePoint()));
				aProjection.upVector_((Jun3dPoint) transformation.applyTo_(upPoint).minus_(aProjection.sightPoint()));
				this.displayProjection_(aProjection);
				this.changed_($("projection"));
			}
		}
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param aPoint
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint grab3dPoint_(Jun2dPoint aPoint) {
		JunOpenGLProjection projection = this.displayProjection();
		JunSphereSurface handleSphere = this.grabSphere();
		JunOpenGLParallelProjection prj = (JunOpenGLParallelProjection) projection.asParallelProjection();
		Jun3dLine line = new Jun3dLine(prj.translateTo3dPointInNearBoundaryPlaneFromPoint_(aPoint), prj.translateTo3dPointInFarBoundaryPlaneFromPoint_(aPoint));
		Jun3dPoint[] candidates = handleSphere.crossPointsWithLine_(line);

		if (candidates.length == 0) {
			return prj.translateTo3dPointFromPoint_(aPoint);
		} else {
			return candidates[0];
		}
	}

	/**
	 * Answer my grab button model.
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 */
	public JunButtonModel grabButton() {
		if (grabButton == null) {
			final JunOctreeEditModel self = this;
			JunButtonModel button = new JunButtonModel();
			button.value_(false);
			button.visual_(JunCursors.HandCursor());
			button.action_(new StBlockClosure() {
				public Object value_(Object o) {
					JunButtonModel model = (JunButtonModel) o;
					model.value_(!model.value());

					if (model.value()) {
						self.focusButton().value_(false);
						self.dragButton().value_(false);
						self.pickButton().value_(false);
					}

					self.changed_($("state"));

					return model;
				}
			});
			grabButton = button;
		}

		return grabButton;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return sra.jun.geometry.surfaces.JunSphereSurface
	 */
	public JunSphereSurface grabSphere() {
		JunSphereSurface grabSphere;
		JunOpenGLProjection projection = this.displayProjection();
		grabSphere = JunSphereSurface.Center_radius_(projection.sightPoint(), projection.regularHeight() / 2);

		return grabSphere;
	}

	/**
	 * Answer true if the receiver has octrees, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasOctree() {
		return (octrees != null && octrees.length > 0);
	}

	/**
	 * Initialize the receiver.
	 */
	public void initialize() {
		octrees = new JunOctree[0];
		displayLights = null;
		displayProjector = null;
		selection = new Vector();
		dollyButton = null;
		pickButton = null;
		grabButton = null;
		dragButton = null;
	}

	/**
	 * Focus on the specified area.
	 * 
	 * @param from2dPoint sra.jun.grometry.basic.Jun2dPoint
	 * @param to2dPoint sra.jun.grometry.basic.Jun2dPoint
	 */
	public void look_xy_(Jun2dPoint from2dPoint, Jun2dPoint to2dPoint) {
		if (this.hasOctree()) {
			JunOpenGLProjection aProjection = this.displayProjection();
			Jun3dPoint fromPoint = this.grab3dPoint_(new Jun2dPoint(from2dPoint.x(), from2dPoint.y()));
			Jun3dPoint toPoint = this.grab3dPoint_(new Jun2dPoint(to2dPoint.x(), to2dPoint.y()));
			aProjection.eyePoint_((Jun3dPoint) aProjection.eyePoint().minus_((Jun3dPoint) toPoint.minus_(fromPoint)));
			this.displayProjection_(aProjection);
			this.changed_($("projection"));
		}
	}

	/**
	 * Specify an accuracy and compute a melted octree.
	 */
	public void meltWithAccuracy() {
		if (this.hasOctree()) {
			String accuracyString = JunDialog.Request_("accuracy", "1.0d");
			if (accuracyString == null) {
				return;
			}

			double accuracy = Double.valueOf(accuracyString).doubleValue();
			if (accuracy < (1.0d - 8)) {
				return;
			}

			this.meltWithAccuracy_(accuracy);
			this.changed_($("object"));
		}
	}

	/**
	 * Specify a depth and compute a melted octree.
	 */
	public void meltWithDepth() {
		if (this.hasOctree()) {
			String depthString = JunDialog.Request_("depth", "1");
			if (depthString == null) {
				return;
			}

			int depth = Math.round(Float.valueOf(depthString).floatValue());
			if (depth < 1) {
				return;
			}

			this.meltWithDepth_(depth);
			this.changed_($("object"));
		}
	}

	/**
	 * Answer a menu model.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 */
	public StMenuModel menuModel() {
		if (menuModel == null) {
			menuModel = defaultMenu();
		}

		return menuModel;
	}

	/**
	 * Reset the octrees.
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public void newModel() {
		try {
			octrees = new JunOctree[] {(JunOctree) _PerformWith(octrees[0].getClass(), "Bounds_", Jun3dBoundingBox.Origin_extent_(Jun3dPoint.Zero(), Jun3dPoint.Unity()))};
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}

		this.changed_($("object"));
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return DOCUMENT ME!
	 */
	public Frame open() {
		this.flushProjector();
		this.fitSilently();
		this.updateMenuIndication();

		return super.open();
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	public void openLight1() {
		JunOpenGLDisplayLight displayLight = this.displayLight1();

		Window[] windows = displayLight.builder().windows();
		for (int i = 0; i < windows.length; i++) {
			if (windows[i].isShowing()) {
				windows[i].toFront();
			} else {
				windows[i].show();
			}
		}

		if (windows.length == 0) {
			displayLight.openPositionLight();
		}
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	public void openLight2() {
		JunOpenGLDisplayLight displayLight = this.displayLight2();

		Window[] windows = displayLight.builder().windows();
		for (int i = 0; i < windows.length; i++) {
			if (windows[i].isShowing()) {
				windows[i].toFront();
			} else {
				windows[i].show();
			}
		}

		if (windows.length == 0) {
			displayLight.openPositionLight();
		}
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	public void openLight3() {
		JunOpenGLDisplayLight displayLight = this.displayLight3();

		Window[] windows = displayLight.builder().windows();
		for (int i = 0; i < windows.length; i++) {
			if (windows[i].isShowing()) {
				windows[i].toFront();
			} else {
				windows[i].show();
			}
		}

		if (windows.length == 0) {
			displayLight.openPositionLight();
		}
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	public void openLight4() {
		JunOpenGLDisplayLight displayLight = this.displayLight4();

		Window[] windows = displayLight.builder().windows();
		for (int i = 0; i < windows.length; i++) {
			if (windows[i].isShowing()) {
				windows[i].toFront();
			} else {
				windows[i].show();
			}
		}

		if (windows.length == 0) {
			displayLight.openPositionLight();
		}
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	public void openLight5() {
		JunOpenGLDisplayLight displayLight = this.displayLight5();

		Window[] windows = displayLight.builder().windows();
		for (int i = 0; i < windows.length; i++) {
			if (windows[i].isShowing()) {
				windows[i].toFront();
			} else {
				windows[i].show();
			}
		}

		if (windows.length == 0) {
			displayLight.openAmbientLight();
		}
	}

	/**
	 * Pan the camera.
	 */
	public void pan() {
		this.zoom_(0.5);
	}

	/**
	 * Pick a JunOctreeNode at the specified point on the view.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint,MouseEvent
	 * @param event DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject pick_with_(Jun2dPoint aPoint, MouseEvent event) {
		if (this.hasOctree()) {
			JunOctreeNode pickedTip = this.pickTipAt_(aPoint);

			if (pickedTip == null) {
				if (selection.isEmpty() == false) {
					this.clearSelection();
					this.changed_($("selection"));
				}
			} else {
				if (event.isShiftDown()) {
					if (selection.contains(pickedTip)) {
						this.removeSelection_(pickedTip);
						this.changed_($("selection"));
					} else {
						this.addSelection_(pickedTip);
						this.changed_($("selection"));
					}
				} else {
					if (event.isControlDown()) {
						this.ctrlClick_(pickedTip);
					} else if (event.isAltDown()) {
						this.altClick_(pickedTip);
					} else {
						this.clearSelection();
						this.addSelection_(pickedTip);
						this.changed_($("selection"));
					}
				}
			}
		}

		return null;
	}

	/**
	 * Answer my pick button.
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 */
	public JunButtonModel pickButton() {
		if (pickButton == null) {
			final JunOctreeEditModel self = this;
			JunButtonModel button = new JunButtonModel();
			button.value_(false);
			button.visual_(JunCursors.NormalCursor());
			button.action_(new StBlockClosure() {
				public Object value_(Object o) {
					JunButtonModel model = (JunButtonModel) o;
					model.value_(!model.value());

					if (model.value()) {
						self.focusButton().value_(false);
						self.dragButton().value_(false);
						self.grabButton().value_(false);
					}

					self.changed_($("state"));

					return model;
				}
			});
			pickButton = button;
		}

		return pickButton;
	}

	/**
	 * Called when quiting.
	 */
	public void quitDoing() {
		Window[] windows = this.builder().windows();
		for (int i = 0; i < windows.length; i++) {
			windows[i].dispose();
		}
	}

	/**
	 * Close the display model.
	 */
	public void release() {
		if (this.displayLights() != null) {
			for (int i = 0; i < this.displayLights().length; i++) {
				JunOpenGLDisplayLight light = this.displayLights()[i];
				//if (light.builder() != null) {
				//	light.builder().window().setVisible(false);
				//	super._close(light.builder().window());
				//}
			}
		}
	}

	/**
	 * Remove the node from the currently selected nodes.
	 * 
	 * @param aJunOctreeNode sra.jun.octree.nodes.JunOctreeNode
	 */
	public void removeSelection_(JunOctreeNode aJunOctreeNode) {
		selection.removeElement(aJunOctreeNode);
	}

	/**
	 * Render the octrees on the redering context.
	 * 
	 * @param renderingContext sra.jun.opengl.support.JunOpenGLRenderingContext
	 */
	public void renderOn_(final JunOpenGLRenderingContext renderingContext) {
		renderingContext.clear();
		renderingContext.cullFaceBack();

		JunOpenGLLight[] lights = this.displayLightCollection();

		if (this.hasOctree() == false) {
			this.displayProjector().projectOn_(renderingContext);
		} else {
			this.displayProjector().project_withLights_on_(null, lights, renderingContext);
			renderingContext.paint_(Color.black);
			renderingContext.lineWidth_(4);

			for (int i = 0; i < octrees.length; i++) {
				octrees[i].originExtentNodesDo_(new StBlockClosure() {
					public Object value_value_value_(Object o1, Object o2, Object o3) {
						Jun3dPoint origin = (Jun3dPoint) o1;
						Jun3dPoint extent = (Jun3dPoint) o2;
						final JunOctreeNode node = (JunOctreeNode) o3;

						if (selection.contains(node)) {
							renderingContext.productTransformation_while_((Jun3dTransformation) Jun3dTransformation.Scale_(extent).product_(Jun3dTransformation.Translate_(origin)), new StBlockClosure() {
								public Object value() {
									node.renderWireframeOn_(renderingContext);

									return null;
								}
							});
						}

						return null;
					}
				});
			}

			renderingContext.pointSize_(5);

			for (int i = 0; i < octrees.length; i++) {
				octrees[i].originExtentTipsDo_(new StBlockClosure() {
					public Object value_value_value_(Object o1, Object o2, Object o3) {
						Jun3dPoint origin = (Jun3dPoint) o1;
						Jun3dPoint extent = (Jun3dPoint) o2;
						final JunOctreeNode tip = (JunOctreeNode) o3;
						renderingContext.productTransformation_while_((Jun3dTransformation) Jun3dTransformation.Scale_(extent).product_(Jun3dTransformation.Translate_(origin)), new StBlockClosure() {
							public Object value() {
								if (tip.isEmpty() == false) {
									renderingContext.paint_(Color.white);
									tip.renderPolygonOn_(renderingContext);
								} else {
									renderingContext.paint_(Color.black);
									tip.renderVertexOn_(renderingContext);
								}

								return null;
							}
						});

						return null;
					}
				});
			}

			renderingContext.flush();
		}
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	public void resetView() {
		this.flushProjector();
		this.fit();
		this.updateMenuIndication();
	}

	/**
	 * Select all nodes.
	 */
	public void selectAll() {
		final JunOctreeEditModel self = this;
		this.clearSelection();

		for (int i = 0; i < octrees.length; i++) {
			octrees[i].originExtentTipsDo_(new StBlockClosure() {
				public Object value_value_value_(Object origin, Object extent, Object tip) {
					self.addSelection_((JunOctreeNode) tip);

					return null;
				}
			});
		}

		this.changed_($("selection"));
		this.updateMenuIndication();
	}

	/**
	 * Change the sight point and update the projector.
	 * 
	 * @param a3dPoint sra.jun.grometry.basic.Jun3dPoint
	 */
	public void sightPoint_(Jun3dPoint a3dPoint) {
		this.displayProjector().sightPoint_(a3dPoint);
		this.changed_($("projection"));
	}

	/**
	 * Simplify the entire octree.
	 */
	public void simplify() {
		if (this.hasOctree()) {
			for (int i = 0; i < octrees.length; i++) {
				octrees[i].beCompact();
			}
		}

		this.clearSelection();
		this.changed_($("object"));
	}

	/**
	 * Slide the projection.
	 * 
	 * @param from2dPoint sra.jun.grometry.basic.Jun2dPoint
	 * @param to2dPoint sra.jun.grometry.basic.Jun2dPoint
	 */
	public void slide_xy_(Jun2dPoint from2dPoint, Jun2dPoint to2dPoint) {
		if (this.hasOctree()) {
			JunOpenGLProjection aProjection = this.displayProjection();
			Jun3dPoint fromPoint = this.grab3dPoint_(new Jun2dPoint(from2dPoint.x(), from2dPoint.y()));
			Jun3dPoint toPoint = this.grab3dPoint_(new Jun2dPoint(to2dPoint.x(), to2dPoint.y()));
			aProjection.sightPoint_((Jun3dPoint) aProjection.sightPoint().minus_((Jun3dPoint) toPoint.minus_(fromPoint)));
			aProjection.eyePoint_((Jun3dPoint) aProjection.eyePoint().minus_((Jun3dPoint) toPoint.minus_(fromPoint)));
			this.displayProjection_(aProjection);
			this.changed_($("projection"));
		}
	}

	/**
	 * Spawning the JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject spawningObject() {
		if (this.hasOctree()) {
			Vector objects = new Vector();

			for (int i = 0; i < octrees.length; i++) {
				objects.addElement(octrees[i].asJunOpenGL3dObject());
			}

			return JunOpenGL3dCompoundObject.Components_(objects);
		} else {
			return null;
		}
	}

	/**
	 * Spawn as an OpenGL object.
	 */
	public void spawnOpenGLObject() {
		JunOpenGLDisplayModel displayModel = this.spawningObject().show();
		displayModel.displayProjection_((JunOpenGLProjection) this.displayProjection().copy());
		displayModel.changed_($("object"));
	}

	/**
	 * Spawn the selected nodes.
	 */
	public void spawnSelection() {
		if (selection.isEmpty() == false) {
			final Vector spawningOctrees = new Vector(selection.size());

			for (int i = 0; i < octrees.length; i++) {
				octrees[i].originExtentNodesDo_(new StBlockClosure() {
					public Object value_value_value_(Object origin, Object extent, Object node) {
						if (selection.contains(node)) {
							Jun3dBoundingBox bbox = Jun3dBoundingBox.Origin_extent_((Jun3dPoint) origin, (Jun3dPoint) extent);
							JunOctree newOctree = JunOctree.Bounds_(bbox);
							spawningOctrees.addElement(newOctree);
						}

						return null;
					}
				});
			}

			JunOctreeEditModel editModel = new JunOctreeEditModel();
			editModel.open();
			editModel.displayProjection_((JunOpenGLProjection) this.displayProjection().copy());
			editModel.setOctrees_(spawningOctrees);
			editModel.changed_($("object"));
		}
	}

	/**
	 * Unselect all nodes.
	 */
	public void unselectAll() {
		this.clearSelection();
		this.changed_($("selection"));
	}

	/**
	 * Change the up vector and update the projector.
	 * 
	 * @param a3dPoint sra.jun.grometry.basic.Jun3dPoint
	 */
	public void upVector_(Jun3dPoint a3dPoint) {
		this.displayProjector().upVector_(a3dPoint);
		this.changed_($("projection"));
	}

	/**
	 * Change the view factor and update the projector.
	 * 
	 * @param factor double
	 */
	public void viewFactor_(double factor) {
		this.displayProjector().viewFactor_(factor);
		this.changed_($("projection"));
	}

	/**
	 * Look closer.
	 */
	public void zoom() {
		this.zoom_(2.0);
	}

	/**
	 * Change the zoom and update the projector.
	 * 
	 * @param factor double
	 */
	public void zoom_(double factor) {
		double zoomFactor;

		//if (StController._ShiftDown()) {
		//	if (factor >= 1.0) {
		//		zoomFactor = 1.0 + (factor - 1.0) * 0.1;
		//	} else {
		//		zoomFactor = 1.0 - (1.0 - factor) * 0.1;
		//	}
		//} else {
		zoomFactor = factor;

		//}
		this.displayProjector().zoom_(zoomFactor);
		this.changed_($("projection"));
	}

	/**
	 * Change the zoom height and update the projector.
	 * 
	 * @param aNumber double
	 */
	public void zoomHeight_(double aNumber) {
		this.displayProjector().zoomHeight_(aNumber);
		this.changed_($("projection"));
	}

	/**
	 * Answer the default edit menu for JunOctreeEditModel.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _defaultEditMenu() {
		StMenu defaultEditMenu = new StMenu("Edit");

		defaultEditMenu.add(createCheckBoxMenuItem("Select all", "selectAll"));
		defaultEditMenu.add(createCheckBoxMenuItem("Unselect all", "unselectAll"));

		return defaultEditMenu;
	}

	/**
	 * Answer the default file menu for JunOctreeEditModel.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _defaultFileMenu() {
		StMenu defaultFileMenu = new StMenu("File");

		// New
		defaultFileMenu.add(createMenuItem("New", "newModel"));

		//
		defaultFileMenu.addSeparator();

		// Quit
		defaultFileMenu.add(createMenuItem("Quit", "quitDoing"));

		return defaultFileMenu;
	}

	/**
	 * Answer the default manipulate menu for JunOctreeEditModel.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _defaultManipulateMenu() {
		StMenu defaultManipulateMenu = new StMenu("Manipulate");

		// Divide
		defaultManipulateMenu.add(createMenuItem("Divide", "divideSelection"));

		// Erase
		defaultManipulateMenu.add(createMenuItem("Erase", "eraseSelection"));

		// Fill
		defaultManipulateMenu.add(createMenuItem("Fill", "fillSelection"));

		// Melt
		StMenu meltMenu = new StMenu("Melt");
		meltMenu.add(createMenuItem("Accuracy...", "meltWithAccuracy"));
		meltMenu.add(createMenuItem("Depth...", "meltWithDepth"));
		defaultManipulateMenu.add(meltMenu);

		// Simplify
		defaultManipulateMenu.add(createMenuItem("Simplify", "simplify"));

		//
		defaultManipulateMenu.addSeparator();

		// Spawn
		defaultManipulateMenu.add(createMenuItem("Spawn", "spawnSelection"));

		return defaultManipulateMenu;
	}

	/**
	 * Answer the default misc menu for JunOctreeEditModel.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _defaultMiscMenu() {
		StMenu defaultMiscMenu = new StMenu("Misc");

		defaultMiscMenu.add(createMenuItem("View", "spawnOpenGLObject"));

		return defaultMiscMenu;
	}

	/**
	 * Answer my manipulate menu.
	 *
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _manipulateMenu() {
		if (_manipulateMenu == null) {
			_manipulateMenu = this._defaultManipulateMenu();
		}
		return _manipulateMenu;
	}

	/**
	 * Compute the current sight point.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	protected Jun3dPoint computeSightPoint() {
		return (Jun3dPoint) this.boundingBox().center();
	}

	/**
	 * Compute the current zoom height.
	 * 
	 * @return double
	 */
	protected double computeZoomHeight() {
		double maxHeight;
		double maxOffsetX = 0;
		double maxOffsetY = 0;
		double depth = this.displayProjection().distance();
		Jun3dPoint up = this.displayProjection().translateTo3dPointFromPoint_depth_(new Jun2dPoint(0, -1), depth);
		Jun3dPoint down = this.displayProjection().translateTo3dPointFromPoint_depth_(new Jun2dPoint(0, 1), depth);
		Jun3dPoint right = this.displayProjection().translateTo3dPointFromPoint_depth_(new Jun2dPoint(1, 0), depth);
		Jun3dPoint left = this.displayProjection().translateTo3dPointFromPoint_depth_(new Jun2dPoint(-1, 0), depth);
		final JunPlane horizontal = new JunPlane(this.displayProjection().eyePoint(), left, right);
		final JunPlane vertical = new JunPlane(this.displayProjection().eyePoint(), up, down);

		if (this.hasOctree()) {
			for (int i = 0; i < octrees.length; i++) {
				JunOctree octree = octrees[i];
				Jun3dPoint origin = octree.origin();
				Jun3dPoint corner = octree.corner();
				Jun3dPoint[] points = new Jun3dPoint[8];
				points[0] = new Jun3dPoint(origin.x(), origin.y(), origin.z());
				points[1] = new Jun3dPoint(corner.x(), origin.y(), origin.z());
				points[2] = new Jun3dPoint(origin.x(), corner.y(), origin.z());
				points[3] = new Jun3dPoint(corner.x(), corner.y(), origin.z());
				points[4] = new Jun3dPoint(origin.x(), origin.y(), corner.z());
				points[5] = new Jun3dPoint(corner.x(), origin.y(), corner.z());
				points[6] = new Jun3dPoint(origin.x(), corner.y(), corner.z());
				points[7] = new Jun3dPoint(corner.x(), corner.y(), corner.z());

				for (int j = 0; j < 8; j++) {
					double offsetX = vertical.distanceFromPoint_(points[j]);
					double offsetY = horizontal.distanceFromPoint_(points[j]);

					if (offsetX > maxOffsetX) {
						maxOffsetX = offsetX;
					}

					if (offsetY > maxOffsetY) {
						maxOffsetY = offsetY;
					}
				}
			}
		}

		double factor = 2.5;

		if (maxOffsetX > maxOffsetY) {
			maxHeight = maxOffsetX * factor;
		} else {
			maxHeight = maxOffsetY * factor;
		}

		return maxHeight;
	}

	/**
	 * Create the menubar for JunOpenGLDisplay.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 */
	protected StMenuModel defaultMenu() {
		StMenuModel rootMenu = new StMenuModel();

		rootMenu.add(this._fileMenu());
		rootMenu.add(this._editMenu());
		rootMenu.add(this._manipulateMenu());
		rootMenu.add(this._lightMenu());
		rootMenu.add(this._miscMenu());

		return rootMenu;
	}

	/**
	 * Specify an accuracy and compute a melted octree.
	 * 
	 * @param accuracy double
	 */
	protected void meltWithAccuracy_(final double accuracy) {
		final Vector meltNodes = (Vector) selection.clone();
		final JunMetaballSolid metaballSolid = this.metaballSolidOrder_withoutAll_(4, meltNodes);

		for (int i = 0; i < octrees.length; i++) {
			octrees[i].originExtentNodesDo_(new StBlockClosure() {
				public Object value_value_value_(Object origin, Object extent, Object node) {
					if (meltNodes.contains(node)) {
						((JunOctreeNode) node).origin_extent_accuracy_block_((Jun3dPoint) origin, (Jun3dPoint) extent, accuracy, new StBlockClosure() {
							public Object value_(Object point) {
								metaballSolid.valueAt_((Jun3dPoint) point);

								return null;
							}
						});
					}

					return null;
				}
			});
		}
	}

	/**
	 * Specify a depth and compute a melted octree.
	 * 
	 * @param depth double
	 */
	protected void meltWithDepth_(final int depth) {
		final Vector meltNodes = (Vector) selection.clone();
		final JunMetaballSolid metaballSolid = this.metaballSolidOrder_withoutAll_(3, meltNodes);

		for (int i = 0; i < octrees.length; i++) {
			octrees[i].originExtentNodesDo_(new StBlockClosure() {
				public Object value_value_value_(Object origin, Object extent, Object node) {
					if (meltNodes.contains(node)) {
						((JunOctreeNode) node).origin_extent_depth_block_((Jun3dPoint) origin, (Jun3dPoint) extent, depth + 1, new StBlockClosure() {
							public Object value_(Object point) {
								metaballSolid.valueAt_((Jun3dPoint) point);

								return null;
							}
						});
					}

					return null;
				}
			});
		}
	}

	/**
	 * Create a new JunMetaballSolid.
	 * 
	 * @param order int
	 * @param aCollectionOfJunOctreeNode java.util.Vector
	 * 
	 * @return sra.jun.metaball.solid.JunMetaballSolid
	 */
	protected JunMetaballSolid metaballSolidOrder_withoutAll_(final int order, Vector aCollectionOfJunOctreeNode) {
		final JunMetaballSolid metaballSolid = new JunMetaballSolid();

		for (int i = 0; i < octrees.length; i++) {
			octrees[i].originExtentNodesWithoutAll_do_(aCollectionOfJunOctreeNode, new StBlockClosure() {
				public Object value_value_value_(Object o1, Object o2, Object o3) {
					Jun3dPoint origin = (Jun3dPoint) o1;
					Jun3dPoint extent = (Jun3dPoint) o2;
					JunOctreeNode node = (JunOctreeNode) o3;

					if (node.isTip()) {
						double r = (extent.x() * extent.y() * extent.z() * 3.0d) / 4.0d / Math.PI;
						r = Math.pow(r, 1.0d / 3.0d);

						double sign = node.isFilled() ? 1.0d : -1.0d;
						double weight = r * sign;
						JunMetaball metaball = JunMetaSphere.Center_order_weight_(extent, order, weight);
						metaballSolid.add_(metaball);
					}

					return null;
				}
			});
		}

		return metaballSolid;
	}

	/**
	 * Pick a JunOctreeNode at the specified point.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * 
	 * @return sra.jun.octree.nodes.JunOctreeNode
	 */
	protected JunOctreeNode pickTipAt_(Jun2dPoint aPoint) {
		Jun3dPoint sightPoint = this.displayProjection().translateTo3dPointFromPoint_(aPoint);
		double zoomHeight = sightPoint.minus_(this.displayProjection().translateTo3dPointFromPoint_((Jun2dPoint) aPoint.plus_(new Jun2dPoint(0, -0.01d)))).length();
		JunOpenGLProjection projection = (JunOpenGLProjection) this.displayProjection().copy();
		projection.sightPoint_(sightPoint);

		if (this.displayProjection().isParallel()) {
			projection.eyePoint_((Jun3dPoint) projection.eyePoint().plus_(sightPoint).minus_(this.displayProjection().sightPoint()));
		}

		projection.zoomHeight_(zoomHeight * 5);

		return this.pickTipProjection_(projection);
	}

	/**
	 * Pick a tip with the projection.
	 * 
	 * @param projection sra.jun.opengl.projection.JunOpenGLProjection
	 * 
	 * @return sra.jun.octree.nodes.JunOctreeNode
	 */
	protected JunOctreeNode pickTipProjection_(JunOpenGLProjection projection) {
		final JunOpenGLPickingRenderingContext rc = new JunOpenGLPickingRenderingContext();
		synchronized (rc.OpenGLInterface()) {
			rc.gljInitialize();
			rc.gljCurrent();
			rc.projection_(projection);

			final Vector tips = new Vector();
			pickTipProjection_index = -1;

			int pickedIndex = rc.selectNearestObjectWhile_(new StBlockClosure() {
				public Object value() {
					for (int i = 0; i < octrees.length; i++) {
						octrees[i].originExtentTipsDo_(new StBlockClosure() {
							public Object value_value_value_(Object o1, Object o2, Object o3) {
								Jun3dPoint origin = (Jun3dPoint) o1;
								Jun3dPoint extent = (Jun3dPoint) o2;
								final JunOctreeNode tip = (JunOctreeNode) o3;
								pickTipProjection_index++;
								tips.addElement(tip);
								rc.setId_(pickTipProjection_index);
								rc.productTransformation_while_((Jun3dTransformation) Jun3dTransformation.Scale_(extent).product_(Jun3dTransformation.Translate_(origin)), new StBlockClosure() {
									public Object value() {
										if (tip.isFilled()) {
											tip.renderPolygonOn_(rc);
										} else {
											tip.renderVertexOn_(rc);
										}

										return null;
									}
								});

								return null;
							}
						});
					}

					return null;
				}
			});
			rc.gljFlush();
			rc.gljRelease();

			//
			if (pickedIndex >= 0) {
				return (JunOctreeNode) tips.elementAt(pickedIndex);
			} else {
				return null;
			}
		}
	}

	/**
	 * Set the octrees.
	 * 
	 * @param aCollectionOfJunOctree java.util.Vector
	 */
	protected void setOctrees_(Vector aCollectionOfJunOctree) {
		if ((aCollectionOfJunOctree == null) || aCollectionOfJunOctree.isEmpty()) {
			octrees = new JunOctree[0];
		} else {
			octrees = new JunOctree[aCollectionOfJunOctree.size()];
			aCollectionOfJunOctree.copyInto(octrees);
		}

		this.clearSelection();
	}

	/**
	 * Answer the title string.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "OpenGL Viewfinder";
	}

}
