package sra.jun.opengl.chart;

import java.awt.event.MouseEvent;
import java.util.*;
import sra.jun.geometry.basic.*;
import sra.jun.opengl.display.*;
import sra.jun.opengl.objects.*;
import sra.smalltalk.StBlockClosure;

/**
 * JunChartCombination class
 * 
 *  @author    nisinaka
 *  @created   1999/01/14 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunChartCombination.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public class JunChartCombination extends JunOpenGLDisplayModel {
	/** The interval between charts. */
	protected static double DefaultIntervalBetweenCharts = 0.3;

	/** The collection of all charts. */
	protected Vector charts = null;

	/** The hashtable which keeps all visual objects. */
	protected Hashtable visualObjectDictionary = null;
	protected double intervalBetweenCharts = DefaultIntervalBetweenCharts;

	/**
	 * The default constructor.
	 */
	public JunChartCombination() {
		super();
	}

	/**
	 * Create a new instance of JunChartCombination and initialize it with the
	 * collection of charts.
	 * 
	 * @param aCollection java.util.Vector
	 * 
	 * @return sra.jun.opengl.chart.JunChartCombination
	 */
	public static final JunChartCombination Charts_(Vector aCollection) {
		JunChartCombination aChart = new JunChartCombination();
		aChart.charts_(aCollection);

		return aChart;
	}

	/**
	 * Create a new instance of JunChartCombination with the chart.
	 * 
	 * @param aChart sra.jun.opengl.chart.JunChartAbstract
	 * 
	 * @return sra.jun.opengl.chart.JunChartCombination
	 */
	public static final JunChartCombination With_(JunChartAbstract aChart) {
		JunChartCombination aChartCombination = new JunChartCombination();
		aChartCombination.add_(aChart);

		return aChartCombination;
	}

	/**
	 * Create a new instance of JunChartCombination with the chart.
	 * 
	 * @param aChart1 sra.jun.opengl.chart.JunChartAbstract
	 * @param aChart2 sra.jun.opengl.chart.JunChartAbstract
	 * 
	 * @return sra.jun.opengl.chart.JunChartCombination
	 */
	public static final JunChartCombination With_with_(JunChartAbstract aChart1, JunChartAbstract aChart2) {
		JunChartCombination aChartCombination = new JunChartCombination();
		aChartCombination.add_(aChart1);
		aChartCombination.add_(aChart2);

		return aChartCombination;
	}

	/**
	 * Create a new instance of JunChartCombination with the chart.
	 * 
	 * @param aChart1 sra.jun.opengl.chart.JunChartAbstract
	 * @param aChart2 sra.jun.opengl.chart.JunChartAbstract
	 * @param aChart3 sra.jun.opengl.chart.JunChartAbstract
	 * 
	 * @return sra.jun.opengl.chart.JunChartCombination
	 */
	public static final JunChartCombination With_with_with_(JunChartAbstract aChart1, JunChartAbstract aChart2, JunChartAbstract aChart3) {
		JunChartCombination aChartCombination = new JunChartCombination();
		aChartCombination.add_(aChart1);
		aChartCombination.add_(aChart2);
		aChartCombination.add_(aChart3);

		return aChartCombination;
	}

	/**
	 * Create a new instance of JunChartCombination with the chart.
	 * 
	 * @param aChart1 sra.jun.opengl.chart.JunChartAbstract
	 * @param aChart2 sra.jun.opengl.chart.JunChartAbstract
	 * @param aChart3 sra.jun.opengl.chart.JunChartAbstract
	 * @param aChart4 sra.jun.opengl.chart.JunChartAbstract
	 * 
	 * @return sra.jun.opengl.chart.JunChartCombination
	 */
	public static final JunChartCombination With_with_with_with_(JunChartAbstract aChart1, JunChartAbstract aChart2, JunChartAbstract aChart3, JunChartAbstract aChart4) {
		JunChartCombination aChartCombination = new JunChartCombination();
		aChartCombination.add_(aChart1);
		aChartCombination.add_(aChart2);
		aChartCombination.add_(aChart3);
		aChartCombination.add_(aChart4);

		return aChartCombination;
	}

	/**
	 * Create a new instance of JunChartCombination and initialize it with the
	 * collection of charts.
	 * 
	 * @param aCollection java.util.Vector
	 * 
	 * @return sra.jun.opengl.chart.JunChartCombination
	 */
	public static final JunChartCombination WithAll_(Vector aCollection) {
		return JunChartCombination.Charts_(aCollection);
	}

	/**
	 * Answer the title string.
	 * 
	 * @return java.lang.String
	 */
	public String _title() {
		return "OpenGL Chart";
	}

	/**
	 * Add the chart to the receiver.
	 * 
	 * @param aChart sra.jun.opengl.chart.JunChartAbstract
	 */
	public void add_(JunChartAbstract aChart) {
		this.charts().addElement(aChart);
		this.flushDisplayObject();
	}

	/**
	 * Convert the receiver as a JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject as3dObject() {
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
		int size = this.charts().size();
		JunChartAbstract[] arrayOfCharts = new JunChartAbstract[size];
		this.charts().copyInto(arrayOfCharts);

		for (int i = 0; i < size; i++) {
			JunOpenGL3dObject[] components = ((JunOpenGL3dCompoundObject) arrayOfCharts[i].displayObject().translatedBy_(new Jun3dPoint(0, 0, this.intervalBetweenCharts() * i))).components();
			compoundObject.addAll_(components);

			Hashtable dictionary = arrayOfCharts[i].visualObjectDictionary();
			Enumeration keys = dictionary.keys();

			while (keys.hasMoreElements()) {
				Object key = keys.nextElement();
				Object value = dictionary.get(key);
				this.visualObjectDictionary().put(key, value);
			}
		}

		return compoundObject;
	}

	/**
	 * Answer the collection of charts.
	 * 
	 * @return java.util.Vector
	 */
	public Vector charts() {
		if (charts == null) {
			charts = new Vector();
		}

		return charts;
	}

	/**
	 * Set the collection of charts.
	 * 
	 * @param aCollection java.util.Vector
	 */
	public void charts_(Vector aCollection) {
		charts = aCollection;
		this.flushDisplayObject();
	}

	/**
	 * Answer the JunOpenGL3dObject to show.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject displayObject() {
		if (super.displayObject() == null) {
			super.displayObject_(this.as3dObject());
		}

		return super.displayObject();
	}

	/**
	 * Initialize the receiver.
	 */
	public void initialize() {
		super.initialize();
		this.initializeLights();
		this.initializeProjector();

		//
		charts = null;
		visualObjectDictionary = null;
		intervalBetweenCharts = DefaultIntervalBetweenCharts;
	}

	/**
	 * Initialize the lights.
	 */
	public void initializeLights() {
		JunOpenGLDisplayLight[] lights = this.displayLights();

		for (int i = 0; i < lights.length; i++) {
			lights[i].beOff();
		}
	}

	/**
	 * Initialize the default projector.
	 */
	public void initializeProjector() {
		this.defaultEyePoint_(new Jun3dPoint(0.5, 0.5, 100.0));
		this.defaultSightPoint_(new Jun3dPoint(0.5, 0.5, 0.0));
		this.defaultUpVector_(new Jun3dPoint(0, 1, 0));
	}

	/**
	 * Answer the interval between charts of the receiver.
	 * 
	 * @return double
	 */
	public double intervalBetweenCharts() {
		return intervalBetweenCharts;
	}

	/**
	 * Set the inteval between charts of the receiver.
	 * 
	 * @param aNumber double
	 */
	public void intervalBetweenCharts_(double aNumber) {
		intervalBetweenCharts = aNumber;
	}

	/**
	 * Answer the JunOpenGL3dObject at the specified point on the view.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * @param anEvent java.awt.event.MouseEvent
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject pick_with_(Jun2dPoint aPoint, MouseEvent anEvent) {
		JunOpenGL3dObject pickedObject = super.pick_with_(aPoint, anEvent);

		if (pickedObject != null) {
			this.pickedObject_(pickedObject);
		}

		return pickedObject;
	}

	/**
	 * Answer the Object which corresponds to the JunOpenGL3dObject.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @return java.lang.Object
	 */
	public Object valueForVisualObject_(JunOpenGL3dObject a3dObject) {
		if (a3dObject == null) {
			return null;
		}

		return this.visualObjectDictionary.get(a3dObject);
	}

	/**
	 * Answer the hastable for all visual objects.
	 * 
	 * @return java.util.Hashtable
	 */
	public Hashtable visualObjectDictionary() {
		if (visualObjectDictionary == null) {
			visualObjectDictionary = new Hashtable();
		}

		return visualObjectDictionary;
	}

	/**
	 * Answer the block closure to evaluate which an object is picked.
	 * 
	 * @return StBlockClosure
	 */
	protected StBlockClosure defaultActionForPickedObject() {
		return new StBlockClosure();
	}

	/**
	 * Flush the cache for display objects.
	 */
	protected void flushDisplayObject() {
		super.flushObject();
		visualObjectDictionary = null;
	}

	/**
	 * The JunOpenGL3dObject is picked. Do something according to the
	 * definition.
	 * 
	 * @param anObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected void pickedObject_(JunOpenGL3dObject anObject) {
		this.defaultActionForPickedObject().value_(anObject);
	}
}
