package sra.jun.opengl.chart;

import java.awt.*;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.opengl.objects.*;

/**
 * JunChartLine class
 * 
 *  @author    nisinaka
 *  @created   1999/01/14 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunChartLine.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public class JunChartLine extends JunChartPlot {
	/** The line width. */
	protected double lineWidth = 1.0;

	/**
	 * The default constructor.
	 */
	public JunChartLine() {
		super();
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 */
	public JunChartLine(java.util.Vector aCollectionOfSample) {
		super(aCollectionOfSample);
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample and a number of keys.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 * @param numberOfKeys int
	 */
	public JunChartLine(java.util.Vector aCollectionOfSample, int numberOfKeys) {
		super(aCollectionOfSample, numberOfKeys);
	}

	/**
	 * Create a new instance of a Chart and initialize it with the
	 * JunChartData.
	 * 
	 * @param aChartData sra.jun.opengl.chart.JunChartData
	 */
	public JunChartLine(JunChartData aChartData) {
		super(aChartData);
	}

	/**
	 * Answer the line width of the receiver.
	 * 
	 * @return double
	 */
	public double lineWidth() {
		return lineWidth;
	}

	/**
	 * Set the line width of the receiver.
	 * 
	 * @param aNumber double
	 */
	public void lineWidth_(double aNumber) {
		lineWidth = aNumber;
	}

	/**
	 * Initialize the receiver with the attributes of the chart.
	 * 
	 * @param aChart sra.jun.opengl.chart.JunChartLine
	 */
	protected void _copyAttributes(JunChartLine aChart) {
		super._copyAttributes(aChart);

		//
		aChart.lineWidth_(lineWidth);
	}

	/**
	 * Create a JunOpenGL3dObject which represents the chart.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 */
	protected JunOpenGL3dCompoundObject createChart3dObject() {
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
		JunChartData myData = this.data();
		JunChartDataSheet[] sheets = myData.sheets();

		if (sheets == null) {
			return compoundObject;
		}

		for (int n = 0; n < sheets.length; n++) {
			JunChartDataSheet sheet = sheets[n];
			int rowSize = sheet.rowSize();
			double z = this.intervalBetweenCharts() * n;
			double width = (1 - this.leftMargin() - this.rightMargin() - this.interval() * (rowSize - 1)) / rowSize;

			for (int j = 0; j < sheet.valueSize(); j++) {
				Jun3dPoint[] points = new Jun3dPoint[rowSize];
				Color color = this.nextColor();

				for (int i = 0; i < rowSize; i++) {
					Number value = (Number) sheet.valueAtPoint_(new Point(i, j));
					double x = this.leftMargin() + ((width + this.interval) * i) + (width / 2);
					Jun3dPoint point = new Jun3dPoint(x, myData.normalizeValue_(value.doubleValue()), z);
					points[i] = point;

					JunOpenGL3dObject vertex = this.createSmallCubeAt_(point);
					vertex.paint_(color);
					compoundObject.add_(vertex);
					this.registerVisualObject_withValue_(vertex, new Object[] { sheet.keysAtRow_(i), value });
				}

				JunOpenGL3dPolyline line = (JunOpenGL3dPolyline) JunOpenGL3dPolyline.Vertexes_(points);
				line.paint_(color);
				line.lineWidth_((float) this.lineWidth());
				compoundObject.add_(line);
			}
		}

		return compoundObject;
	}
}
