package sra.jun.opengl.chart;

import java.awt.Color;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.opengl.objects.*;

/**
 * JunChartLineWithNumericalKeys class
 * 
 *  @author    nisinaka
 *  @created   1999/01/14 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunChartLineWithNumericalKeys.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public class JunChartLineWithNumericalKeys extends JunChartPlotWithNumericalKeys {
	/** The line width. */
	protected double lineWidth = 1.0;

	/**
	 * The default constructor.
	 */
	public JunChartLineWithNumericalKeys() {
		super();
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 */
	public JunChartLineWithNumericalKeys(java.util.Vector aCollectionOfSample) {
		super(aCollectionOfSample);
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample and a number of keys.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 * @param numberOfKeys int
	 */
	public JunChartLineWithNumericalKeys(java.util.Vector aCollectionOfSample, int numberOfKeys) {
		super(aCollectionOfSample, numberOfKeys);
	}

	/**
	 * Create a new instance of a Chart and initialize it with the
	 * JunChartData.
	 * 
	 * @param aChartData sra.jun.opengl.chart.JunChartData
	 */
	public JunChartLineWithNumericalKeys(JunChartData aChartData) {
		super(aChartData);
	}

	/**
	 * Answer the line width of the receiver.
	 * 
	 * @return double
	 */
	public double lineWidth() {
		return lineWidth;
	}

	/**
	 * Set the line width of the receiver.
	 * 
	 * @param aNumber double
	 */
	public void lineWidth_(double aNumber) {
		lineWidth = aNumber;
	}

	/**
	 * Create a JunOpenGL3dObject which represents the chart.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 */
	protected JunOpenGL3dCompoundObject createChart3dObject() {
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
		JunChartDataWithNumericalKeys myData = (JunChartDataWithNumericalKeys) this.data();
		JunChartDataSheet[] sheets = myData.sheets();

		if (sheets == null) {
			return compoundObject;
		}

		for (int n = 0; n < sheets.length; n++) {
			JunChartDataSheet sheet = sheets[n];
			int rowSize = sheet.rowSize();
			double z = this.intervalBetweenCharts() * n;
			Object[] keys = sheet.keysAtColumn_(0);

			for (int j = 0; j < sheet.valueSize(); j++) {
				Object[] values = sheet.valuesAtColumn_(j);
				Jun3dPoint[] points = new Jun3dPoint[rowSize];
				Color color = this.nextColor();

				for (int i = 0; i < keys.length; i++) {
					Number key = (Number) keys[i];
					Number value = (Number) values[i];
					Jun3dPoint point = new Jun3dPoint(myData.normalizeKey_(key.doubleValue()), myData.normalizeValue_(value.doubleValue()), z);
					points[i] = point;

					JunOpenGL3dObject vertex = this.createSmallCubeAt_(point);
					vertex.paint_(color);
					compoundObject.add_(vertex);
					this.registerVisualObject_withValue_(vertex, new Object[] { sheet.keysAtRow_(i), value });
				}

				JunOpenGL3dPolyline line = (JunOpenGL3dPolyline) JunOpenGL3dPolyline.Vertexes_(points);
				line.paint_(color);
				line.lineWidth_((float) this.lineWidth());
				compoundObject.add_(line);
			}
		}

		return compoundObject;
	}
}
