package sra.jun.opengl.chart;

import java.io.*;
import sra.jun.geometry.basic.*;
import sra.jun.opengl.objects.*;
import sra.jun.opengl.texture.JunOpenGLTexture;

/**
 * JunChartPie class
 * 
 *  @author    nisinaka
 *  @created   1999/01/14 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunChartPie.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public class JunChartPie extends JunChartWithRoundShape {
	/** The flag whether the receiver has labels or not. */
	protected boolean hasLabels = true;

	/**
	 * The default constructor.
	 */
	public JunChartPie() {
		super();
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 */
	public JunChartPie(java.util.Vector aCollectionOfSample) {
		super(aCollectionOfSample);
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample and a number of keys.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 * @param numberOfKeys int
	 */
	public JunChartPie(java.util.Vector aCollectionOfSample, int numberOfKeys) {
		super(aCollectionOfSample, numberOfKeys);
	}

	/**
	 * Create a new instance of a Chart and initialize it with the
	 * JunChartData.
	 * 
	 * @param aChartData sra.jun.opengl.chart.JunChartData
	 */
	public JunChartPie(JunChartData aChartData) {
		super(aChartData);
	}

	/**
	 * Answer true if the receiver has labels, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasLabels() {
		return hasLabels;
	}

	/**
	 * Hide the labels.
	 */
	public void hideLabels() {
		if (this.hasLabels() == true) {
			hasLabels = false;
			this.flushDisplayObject();
			this.changed_($("object"));
		}
	}

	/**
	 * Show the labels.
	 */
	public void showLabels() {
		if (this.hasLabels() == false) {
			hasLabels = true;
			this.flushDisplayObject();
			this.changed_($("object"));
		}
	}

	/**
	 * Initialize the receiver with the attributes of the chart.
	 * 
	 * @param aChart sra.jun.opengl.chart.JunChartPie
	 */
	protected void _copyAttributes(JunChartPie aChart) {
		super._copyAttributes(aChart);

		//
		if (this.hasLabels()) {
			aChart.showLabels();
		} else {
			aChart.hideLabels();
		}
	}

	/**
	 * Create a JunOpenGL3dObject which represents the JunChartBar.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 */
	protected JunOpenGL3dCompoundObject createChart3dObject() {
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
		JunChartDataSheet[] sheets = this.data().sheets();

		if (sheets == null) {
			return compoundObject;
		}

		for (int n = 0; n < sheets.length; n++) {
			JunChartDataSheet sheet = sheets[n];
			double z = this.intervalBetweenCharts() * n;
			Object[] values = sheet.valuesAtColumn_(0);
			double totalValue = 0;

			for (int i = 0; i < values.length; i++) {
				totalValue += ((Number) values[i]).doubleValue();
			}

			double degree = 90;

			for (int i = 0; i < values.length; i++) {
				double value = ((Number) values[i]).doubleValue();
				double theta = (360 * value) / totalValue;
				JunOpenGL3dObject pie = JunOpenGL3dObject.PieFrom_to_by_radius_thickness_(degree, degree - theta, -10, this.radius(), this.thickness());
				pie = pie.translatedBy_(new Jun3dPoint(0, 0, z));
				pie.paint_(this.nextColor());
				compoundObject.add_(pie);
				this.registerVisualObject_withValue_(pie, new Object[] { sheet.keysAtRow_(0), new Double(value)});

				if (this.hasLabels()) {
					String label = this.labelStringFor_within_(value, totalValue);
					Jun3dPoint point = new Jun3dPoint(Jun2dPoint.Rho_theta_(this.radius(), JunAngle._DegreesToRadians(degree - (theta / 2))), z);
					compoundObject.add_(this.label_at_(label, point));
				}

				degree -= theta;
			}
		}

		return compoundObject;
	}

	/**
	 * Answer the JunOpenGL3dObject as for a label at the specified point.
	 * 
	 * @param aString java.lang.String
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected JunOpenGL3dObject label_at_(String aString, Jun3dPoint aPoint) {
		JunOpenGLTexture texture = JunOpenGLTexture.TextureForString_foreColor_backColor_(aString, this.labelForegroundColor(), this.labelBackgroundColor());
		char[] characters = aString.toCharArray();
		int numberOfLines = 0;

		for (int i = 0; i < characters.length; i++) {
			if (characters[i] == '\n') {
				numberOfLines++;
			}
		}

		double height = this.defaultLabelHeight() * (numberOfLines + 1);
		double width = (height * texture.width()) / texture.height();
		Jun2dPoint a2dPoint = new Jun2dPoint(aPoint.x(), aPoint.y());
		a2dPoint = Jun2dPoint.Rho_theta_(a2dPoint.rho() + (Math.sqrt((width * width) + (height * height)) / 2), a2dPoint.theta());

		Jun2dPoint origin = new Jun2dPoint(a2dPoint.x() - (width / 2), a2dPoint.y() - (height / 2));
		Jun2dPoint corner = new Jun2dPoint(a2dPoint.x() + (width / 2), a2dPoint.y() + (height / 2));
		Jun3dPoint[] points = { new Jun3dPoint(origin.x(), origin.y(), aPoint.z()), new Jun3dPoint(corner.x(), origin.y(), aPoint.z()), new Jun3dPoint(corner.x(), corner.y(), aPoint.z()), new Jun3dPoint(origin.x(), corner.y(), aPoint.z())};
		JunOpenGL3dPolygon label = JunOpenGL3dPolygon.Vertexes_(points);
		label.texture_(texture);

		return label;
	}

	/**
	 * Answer the label string for the value.
	 * 
	 * @param value double
	 * @param totalValue double
	 * 
	 * @return java.lang.String
	 */
	protected String labelStringFor_within_(double value, double totalValue) {
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		pw.println(value);
		pw.print('(');
		pw.print(Math.floor((value * 100) / totalValue));
		pw.print("%)");
		pw.flush();

		return sw.toString();
	}
}
