package sra.jun.opengl.chart;

import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.opengl.objects.*;

/**
 * JunChartPlot class
 * 
 *  @author    nisinaka
 *  @created   1999/01/14 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunChartPlot.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public class JunChartPlot extends JunChartWithSymbolicKeys {
	/** The point size. */
	protected double pointSize = 0.01;

	/**
	 * The default constructor.
	 */
	public JunChartPlot() {
		super();
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 */
	public JunChartPlot(java.util.Vector aCollectionOfSample) {
		super(aCollectionOfSample);
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample and a number of keys.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 * @param numberOfKeys int
	 */
	public JunChartPlot(java.util.Vector aCollectionOfSample, int numberOfKeys) {
		super(aCollectionOfSample, numberOfKeys);
	}

	/**
	 * Create a new instance of a Chart and initialize it with the
	 * JunChartData.
	 * 
	 * @param aChartData sra.jun.opengl.chart.JunChartData
	 */
	public JunChartPlot(JunChartData aChartData) {
		super(aChartData);
	}

	/**
	 * Answer the point size of the receiver.
	 * 
	 * @return double
	 */
	public double pointSize() {
		return pointSize;
	}

	/**
	 * Set the point size of the receiver.
	 * 
	 * @param aNumber double
	 */
	public void pointSize_(double aNumber) {
		pointSize = aNumber;
	}

	/**
	 * Initialize the receiver with the attributes of the chart.
	 * 
	 * @param aChart sra.jun.opengl.chart.JunChartPlot
	 */
	protected void _copyAttributes(JunChartPlot aChart) {
		super._copyAttributes(aChart);

		//
		aChart.pointSize_(pointSize);
	}

	/**
	 * Create a JunOpenGL3dObject which represents the chart.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 */
	protected JunOpenGL3dCompoundObject createChart3dObject() {
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
		JunChartData myData = this.data();
		JunChartDataSheet[] sheets = myData.sheets();

		if (sheets == null) {
			return compoundObject;
		}

		for (int n = 0; n < sheets.length; n++) {
			JunChartDataSheet sheet = sheets[n];
			double z = this.intervalBetweenCharts() * n;
			int rowSize = sheet.rowSize();

			for (int i = 0; i < rowSize; i++) {
				Object[] keys = sheet.valuesAtRow_(i);
				Object[] values = sheet.valuesAtRow_(i);
				double width = (1 - this.leftMargin() - this.rightMargin() - this.interval() * (rowSize - 1)) / rowSize;
				double x = this.leftMargin() + ((width + this.interval) * i) + (width / 2);

				for (int j = 0; j < values.length; j++) {
					JunOpenGL3dObject vertex = this.createSmallCubeAt_(new Jun3dPoint(x, myData.normalizeValue_(((Number) values[j]).doubleValue()), z));
					vertex.paint_(this.nextColor());
					compoundObject.add_(vertex);
					this.registerVisualObject_withValue_(vertex, new Object[] { keys, values[j] });
				}
			}
		}

		return compoundObject;
	}

	/**
	 * Create a small cube to represents the point.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected JunOpenGL3dObject createSmallCubeAt_(Jun3dPoint a3dPoint) {
		double size = this.pointSize();
		JunOpenGL3dObject vertex = JunOpenGL3dObject.Box_(new Jun3dPoint(size, size, size));
		vertex = vertex.translatedBy_((Jun3dPoint) a3dPoint.minus_(size / 2));

		return vertex;
	}
}
