package sra.jun.opengl.chart;

import java.awt.*;
import sra.jun.geometry.basic.*;
import sra.jun.opengl.objects.*;

/**
 * JunChartRadar class
 * 
 *  @author    nisinaka
 *  @created   1999/01/14 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunChartRadar.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public class JunChartRadar extends JunChartWithRoundShape {
	/** The point size. */
	protected double pointSize = 0.01;

	/** The line width. */
	protected double lineWidth = 1.0;

	/**
	 * The default constructor.
	 */
	public JunChartRadar() {
		super();
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 */
	public JunChartRadar(java.util.Vector aCollectionOfSample) {
		super(aCollectionOfSample);
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample and a number of keys.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 * @param numberOfKeys int
	 */
	public JunChartRadar(java.util.Vector aCollectionOfSample, int numberOfKeys) {
		super(aCollectionOfSample, numberOfKeys);
	}

	/**
	 * Create a new instance of a Chart and initialize it with the
	 * JunChartData.
	 * 
	 * @param aChartData sra.jun.opengl.chart.JunChartData
	 */
	public JunChartRadar(JunChartData aChartData) {
		super(aChartData);
	}

	/**
	 * Answer the line width of the receiver.
	 * 
	 * @return double
	 */
	public double lineWidth() {
		return lineWidth;
	}

	/**
	 * Set the line width of the receiver.
	 * 
	 * @param aNumber double
	 */
	public void lineWidth_(double aNumber) {
		lineWidth = aNumber;
	}

	/**
	 * Answer the point size of the receiver.
	 * 
	 * @return double
	 */
	public double pointSize() {
		return pointSize;
	}

	/**
	 * Set the point size of the receiver.
	 * 
	 * @param aNumber double
	 */
	public void pointSize_(double aNumber) {
		pointSize = aNumber;
	}

	/**
	 * Initialize the receiver with the attributes of the chart.
	 * 
	 * @param aChart sra.jun.opengl.chart.JunChartRadar
	 */
	protected void _copyAttributes(JunChartRadar aChart) {
		super._copyAttributes(aChart);

		//
		aChart.pointSize_(pointSize);
		aChart.lineWidth_(lineWidth);
	}

	/**
	 * Create a JunOpenGL3dObject which represents the chart.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 */
	protected JunOpenGL3dCompoundObject createChart3dObject() {
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
		JunChartData myData = this.data();
		JunChartDataSheet[] sheets = myData.sheets();

		if (sheets == null) {
			return compoundObject;
		}

		for (int n = 0; n < sheets.length; n++) {
			JunChartDataSheet sheet = sheets[n];
			int rowSize = sheet.rowSize();
			double z = this.intervalBetweenCharts() * n;

			for (int j = 0; j < sheet.valueSize(); j++) {
				Jun3dPoint[] points = new Jun3dPoint[rowSize];
				Color color = this.nextColor();

				for (int i = 0; i < rowSize; i++) {
					Number value = (Number) sheet.valueAtPoint_(new Point(i, j));
					Jun3dPoint point = new Jun3dPoint(0, myData.normalizeValue_(value.doubleValue()), z);
					point = (Jun3dPoint) point.rotatedBy_(JunAngle.FromRad_((2 * Math.PI * i) / rowSize));
					points[i] = point;

					JunOpenGL3dObject vertex = this.createSmallCubeAt_(point);
					vertex.paint_(color);
					compoundObject.add_(vertex);
					this.registerVisualObject_withValue_(vertex, new Object[] { sheet.keysAtRow_(i), value });
				}

				JunOpenGL3dPolyline line = (JunOpenGL3dPolyline) JunOpenGL3dPolyline.Vertexes_(points);
				line.paint_(color);
				line.lineWidth_((float) this.lineWidth());
				compoundObject.add_(line);
			}
		}

		return compoundObject;
	}

	/**
	 * Create a small cube to represents the point.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected JunOpenGL3dObject createSmallCubeAt_(Jun3dPoint a3dPoint) {
		double size = this.pointSize();
		JunOpenGL3dObject vertex = JunOpenGL3dObject.Box_(new Jun3dPoint(size, size, size));
		vertex = vertex.translatedBy_((Jun3dPoint) a3dPoint.minus_(size / 2));

		return vertex;
	}
}
