package sra.jun.opengl.chart;

import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.opengl.objects.*;

/**
 * JunChartWithNumericalKeys class
 * 
 *  @author    nisinaka
 *  @created   1999/01/13 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunChartWithNumericalKeys.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public abstract class JunChartWithNumericalKeys extends JunChartAbstract {

	/**
	 * The default constructor.
	 */
	protected JunChartWithNumericalKeys() {
		super();
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 */
	protected JunChartWithNumericalKeys(java.util.Vector aCollectionOfSample) {
		this();
		this.data_(new JunChartDataWithNumericalKeys(aCollectionOfSample));
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample and a number of keys.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 * @param numberOfKeys int
	 */
	protected JunChartWithNumericalKeys(java.util.Vector aCollectionOfSample, int numberOfKeys) {
		this();
		this.data_(new JunChartDataWithNumericalKeys(aCollectionOfSample, numberOfKeys));
	}

	/**
	 * Create a new instance of a Chart and initialize it with the
	 * JunChartData.
	 * 
	 * @param aChartData sra.jun.opengl.chart.JunChartData
	 */
	protected JunChartWithNumericalKeys(JunChartData aChartData) {
		super(aChartData);
	}

	/**
	 * Answer the class for the chart data of this chart.
	 * 
	 * @return java.lang.Class
	 */
	public static Class ChartDataClass() {
		return JunChartDataWithNumericalKeys.class;
	}

	/**
	 * Set the information about the key range.
	 * 
	 * @param minimumNumber double
	 * @param maximumNumber double
	 */
	public void keyRangeFrom_to_(double minimumNumber, double maximumNumber) {
		((JunChartDataWithNumericalKeys) this.data()).keyRangeFrom_to_(minimumNumber, maximumNumber);
	}

	/**
	 * Set the information about the key range.
	 * 
	 * @param minimumNumber double
	 * @param maximumNumber double
	 * @param interval double
	 */
	public void keyRangeFrom_to_(double minimumNumber, double maximumNumber, double interval) {
		((JunChartDataWithNumericalKeys) this.data()).keyRangeFrom_to_by_(minimumNumber, maximumNumber, interval);
	}

	/**
	 * Action for a menu : Spawn a line chart.
	 */
	public void spawnLineChart() {
		JunChartLineWithNumericalKeys aChart = new JunChartLineWithNumericalKeys();
		this._copyAttributes(aChart);
		aChart.open();
	}

	/**
	 * Action for a menu : Spawn a plot chart.
	 */
	public void spawnPlotChart() {
		JunChartPlotWithNumericalKeys aChart = new JunChartPlotWithNumericalKeys();
		this._copyAttributes(aChart);
		aChart.open();
	}

	/**
	 * Create a JunOpenGL3dObject of x-axis labels.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected JunOpenGL3dObject createXAxisLabels() {
		JunChartDataWithNumericalKeys data = (JunChartDataWithNumericalKeys) this.data();
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
		double minimumNumber = data.minimumOfKeyRange();
		double maximumNumber = data.maximumOfKeyRange();
		double interval = data.intervalOfKeyRange();

		for (double aNumber = (Math.floor(minimumNumber / interval) + 1) * interval; aNumber <= (Math.floor(maximumNumber / interval) * interval); aNumber += interval) {
			if ((aNumber != minimumNumber) && (aNumber != maximumNumber)) {
				double x = data.normalizeKey_(aNumber);
				Jun3dPoint[] points = { new Jun3dPoint(x, 0, 0), new Jun3dPoint(x, -0.05, 0)};
				compoundObject.add_(JunOpenGL3dPolyline.Vertexes_(points));
				compoundObject.add_(this.label_topCenterAt_(String.valueOf(aNumber), new Jun3dPoint(x, -0.05, 0)));
			}
		}

		return compoundObject;
	}
}
