package sra.jun.opengl.chart;

import java.awt.Color;
import sra.jun.geometry.basic.*;
import sra.jun.opengl.objects.*;
import sra.smalltalk.*;

/**
 * JunChartWithRoundShape class
 * 
 *  @author    nisinaka
 *  @created   1999/01/14 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunChartWithRoundShape.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public abstract class JunChartWithRoundShape extends JunChartAbstract {
	/** The number as of the radius. */
	protected double radius = 0.5;

	/** The JunOpenGL3dObject as of the axes. */
	protected JunOpenGL3dObject axes = null;

	/**
	 * The default constructor.
	 */
	protected JunChartWithRoundShape() {
		super();
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 */
	protected JunChartWithRoundShape(java.util.Vector aCollectionOfSample) {
		super(aCollectionOfSample);
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample and a number of keys.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 * @param numberOfKeys int
	 */
	protected JunChartWithRoundShape(java.util.Vector aCollectionOfSample, int numberOfKeys) {
		super(aCollectionOfSample, numberOfKeys);
	}

	/**
	 * Create a new instance of a Chart and initialize it with the
	 * JunChartData.
	 * 
	 * @param aChartData sra.jun.opengl.chart.JunChartData
	 */
	protected JunChartWithRoundShape(JunChartData aChartData) {
		super(aChartData);
	}

	/**
	 * Conver the receiver as a JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject as3dObject() {
		JunOpenGL3dCompoundObject chart3dObject = this.createChart3dObject();

		if (this.hasZAxis()) {
			chart3dObject.add_(this.zAxis());
		}

		if (this.hasAxes()) {
			chart3dObject.add_(this.axes());
		}

		if (this.extra3dObjects() != null) {
			chart3dObject.addAll_(this.extra3dObjects());
		}

		return chart3dObject;
	}

	/**
	 * Answer the axes of the receiver.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject axes() {
		return axes;
	}

	/**
	 * Answer true if the axes of the receiver exists, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasAxes() {
		return (this.axes() != null);
	}

	/**
	 * Hide the axes.
	 */
	public void hideAxes() {
		axes = null;
		this.flushDisplayObject();
		this.changed_($("object"));
	}

	/**
	 * Answer the radius of the receiver.
	 * 
	 * @return double
	 */
	public double radius() {
		return radius;
	}

	/**
	 * Set the radius of the receiver.
	 * 
	 * @param aNumber double
	 */
	public void radius_(double aNumber) {
		radius = aNumber;
	}

	/**
	 * Show the axes.
	 */
	public void showAxes() {
		axes = this.createAxes();
		this.flushDisplayObject();
		this.changed_($("object"));
	}

	/**
	 * Show the axes with labels.
	 */
	public void showAxesWithLabels() {
		axes = JunOpenGL3dCompoundObject.With_with_(this.createAxes(), this.createAxesLabels());
		this.flushDisplayObject();
		this.changed_($("object"));
	}

	/**
	 * Show the x-axis.
	 */
	public void showXAxis() {
		// Do nothing.
	}

	/**
	 * Show the x-axis with labels.
	 */
	public void showXAxisWithLabels() {
		// Do nothing.
	}

	/**
	 * Show the y-axis.
	 */
	public void showYAxis() {
		// Do nothing.
	}

	/**
	 * Show the y-axis with labels.
	 */
	public void showYAxisWithLabels() {
		// Do nothing.
	}

	/**
	 * Show the z-axis.
	 */
	public void showZAxis() {
		// Do nothing.
	}

	/**
	 * Initialize the receiver with the attributes of the chart.
	 * 
	 * @param aChart sra.jun.opengl.chart.JunChartWithRoundShape
	 */
	protected void _copyAttributes(JunChartWithRoundShape aChart) {
		super._copyAttributes(aChart);

		//
		aChart.radius_(radius);

		if (this.hasAxes()) {
			aChart.showAxes();
		} else {
			aChart.hideAxes();
		}
	}

	/**
	 * Create a JunOpenGL3dObject of axes.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected JunOpenGL3dObject createAxes() {
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
		compoundObject.name_("round axes");

		int numberOfAxes = this.data().sheet().rowSize();
		Jun3dPoint[] points = { new Jun3dPoint(0, 0, 0), new Jun3dPoint(0, 1, 0)};
		JunOpenGL3dCompoundObject axis = ((JunOpenGL3dPrimitiveObject) JunOpenGL3dPolyline.Vertexes_(points)).asCompoundObject();

		for (int i = 0; i < numberOfAxes; i++) {
			compoundObject.add_(axis.rotatedBy_(JunAngle.FromRad_((2 * Math.PI * i) / numberOfAxes)));
		}

		compoundObject.paint_(this.axisColor());

		return compoundObject;
	}

	/**
	 * Create a JunOpenGL3dObject of axes labels.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected JunOpenGL3dObject createAxesLabels() {
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
		JunOpenGL3dObject standardObjects = super.createYAxisLabels();
		int numberOfAxes = this.data().sheet().rowSize();
		Jun3dPoint[] points = { new Jun3dPoint(0, 0, 0), new Jun3dPoint(0, 1, 0)};
		JunOpenGL3dCompoundObject axis = ((JunOpenGL3dPrimitiveObject) JunOpenGL3dPolyline.Vertexes_(points)).asCompoundObject();

		for (int i = 0; i < numberOfAxes; i++) {
			compoundObject.add_(standardObjects.rotatedBy_(JunAngle.FromRad_((2 * Math.PI * i) / numberOfAxes)));
		}

		compoundObject.paint_(this.axisColor());

		return compoundObject;
	}

	/**
	 * Create a JunOpenGL3dObject of x-axis.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	protected JunOpenGL3dObject createXAxis() {
		throw new SmalltalkException("should not implement");
	}

	/**
	 * Create a JunOpenGL3dObject of x-axis labels.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	protected JunOpenGL3dObject createXAxisLabels() {
		throw new SmalltalkException("should not implement");
	}

	/**
	 * Create a JunOpenGL3dObject of y-axis.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	protected JunOpenGL3dObject createYAxis() {
		throw new SmalltalkException("should not implement");
	}

	/**
	 * Create a JunOpenGL3dObject of y-axis labels.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	protected JunOpenGL3dObject createYAxisLabels() {
		throw new SmalltalkException("should not implement");
	}

	/**
	 * Answer the default colors.
	 * 
	 * @return java.awt.Color[]
	 */
	protected Color[] defaultColors() {
		if (this.samples() == null) {
			return super.defaultColors();
		}

		return this.defaultColors_(this.data().sheet().rowSize());
	}

	/**
	 * Flush the cached JunOpenGL3dObjects for axes.
	 */
	protected void flushAxes() {
		super.flushAxes();
		axes = null;
	}
}
