package sra.jun.opengl.chart;

import java.util.Vector;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.opengl.objects.*;

/**
 * JunChartWithSymbolicKeys class
 * 
 *  @author    nisinaka
 *  @created   1998/11/19 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunChartWithSymbolicKeys.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public abstract class JunChartWithSymbolicKeys extends JunChartAbstract {
	protected double leftMargin = 0.05;
	protected double rightMargin = 0.05;
	protected double interval = 0.05;

	/**
	 * The default constructor.
	 */
	protected JunChartWithSymbolicKeys() {
		super();
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 */
	protected JunChartWithSymbolicKeys(Vector aCollectionOfSample) {
		super(aCollectionOfSample);
	}

	/**
	 * Create a new instance of a Chart and initialize it with the collection
	 * of sample and a number of keys.
	 * 
	 * @param aCollectionOfSample java.util.Vector
	 * @param numberOfKeys int
	 */
	protected JunChartWithSymbolicKeys(Vector aCollectionOfSample, int numberOfKeys) {
		super(aCollectionOfSample, numberOfKeys);
	}

	/**
	 * Create a new instance of a Chart and initialize it with the
	 * JunChartData.
	 * 
	 * @param aChartData sra.jun.opengl.chart.JunChartData
	 */
	protected JunChartWithSymbolicKeys(JunChartData aChartData) {
		super(aChartData);
	}

	/**
	 * Answer the interval between the chart elements of the receiver.
	 * 
	 * @return double
	 */
	public double interval() {
		return interval;
	}

	/**
	 * Set the interval between the chart elements of the receiver.
	 * 
	 * @param aNumber double
	 */
	public void interval_(double aNumber) {
		interval = aNumber;
	}

	/**
	 * Answer the left margin of the receiver.
	 * 
	 * @return double
	 */
	public double leftMargin() {
		return leftMargin;
	}

	/**
	 * Set the left margin of the receiver.
	 * 
	 * @param aNumber double
	 */
	public void leftMargin_(double aNumber) {
		leftMargin = aNumber;
	}

	/**
	 * Answer the right margin of the receiver.
	 * 
	 * @return double
	 */
	public double rightMargin() {
		return rightMargin;
	}

	/**
	 * Set the right margin of the receiver.
	 * 
	 * @param aNumber double
	 */
	public void rightMargin_(double aNumber) {
		rightMargin = aNumber;
	}

	/**
	 * Initialize the receiver with the attributes of the chart.
	 * 
	 * @param aChart sra.jun.opengl.chart.JunChartWithSymbolicKeys
	 */
	protected void _copyAttributes(JunChartWithSymbolicKeys aChart) {
		super._copyAttributes(aChart);

		//
		aChart.leftMargin_(leftMargin);
		aChart.rightMargin_(rightMargin);
		aChart.interval_(interval);
	}

	/**
	 * Create a JunOpenGL3dObject of x-axis labels.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected JunOpenGL3dObject createXAxisLabels() {
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
		int size = this.data().sheet().rowSize();
		double width = (1 - this.leftMargin() - this.rightMargin() - this.interval() * (size - 1)) / size;
		double x = this.leftMargin() + (width / 2);

		for (int i = 0; i < size; i++) {
			Jun3dPoint[] points = { new Jun3dPoint(x, 0, 0), new Jun3dPoint(x, -0.05, 0)};
			compoundObject.add_(JunOpenGL3dPolyline.Vertexes_(points));
			compoundObject.add_(this.label_topCenterAt_(this.xAxisLabels()[i], new Jun3dPoint(x, -0.05, 0)));
			x += (width + this.interval());
		}

		return compoundObject;
	}
}
