package sra.jun.opengl.display;

import java.awt.*;
import javax.swing.*;
import sra.smalltalk.*;
import sra.gl4jun.*;
import sra.jun.opengl.support.*;
import sra.jun.system.support.*;

/**
 * JunOpenGL3dViewSwing class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   2000/01/06 (by MATSUDA Ryouichi)
 *  @updated   2002/04/12 (by nisinaka)
 *  @updated   2003/05/15 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun475 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dViewSwing.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public class JunOpenGL3dViewSwing extends GLjJCanvas implements JunOpenGL3dView {

	protected StModel model;
	protected StController controller;
	protected JPopupMenu _popupMenu;

	/**
	 * Create a new instance of JunOpenGL3dViewSwing and initialize it.
	 */
	public JunOpenGL3dViewSwing() {
		this(null);
	}

	/**
	 * Create a new instance of JunOpenGL3dViewSwing and initialize it.
	 * 
	 * @param newModel
	 */
	public JunOpenGL3dViewSwing(StModel newModel) {
		super();
		this.initialize();
		model = (newModel == null) ? this.defaultModel() : newModel;
		model.addDependent_(this);
		this.buildComponent();
	}

	/**
	 * Replacement for the # notation of Smalltalk. :-)
	 * 
	 * @return sra.smalltalk.StSymbol
	 * @param aString java.lang.String
	 */
	public static final StSymbol $(String aString) {
		return StSymbol.Intern_(aString);
	}

	/**
	 * Remove the receiver as a dependent of its model.
	 */
	public void release() {
		this.model().removeDependent_(this);
	}

	/**
	 * Create a Frame for the view.
	 *
	 * @return java.awt.Frame
	 * @param anApplicationModel sra.smalltalk.StApplicationModel
	 */
	public Frame _createFrame(StApplicationModel anApplicationModel) {
		return StMenuViewForSwing.CreateJFrame(anApplicationModel, this);
	}

	/**
	 * Create a JPopupMenu for the view.
	 *
	 * @return javax.swing.JPopupMenu
	 * @param aMenu sra.smalltalk.StMenu
	 */
	public JPopupMenu _createPopupMenu(StMenu aMenu) {
		return StMenuViewForSwing.CreateJPopupMenu(aMenu);
	}

	/**
	 * Answer the bordered panel which contains the receiver.
	 *
	 * @return javax.swing.JPanel
	 */
	public JPanel _onBorderedPanel() {
		JPanel aPanel = StApplicationModel._JPanelWithBorder();
		aPanel.setLayout(new BorderLayout());
		aPanel.add(this, BorderLayout.CENTER);
		return aPanel;
	}

	/**
	 * Show the popup menu at the specified point on the view.
	 *
	 * @param x int
	 * @param y int
	 */
	public void _showPopupMenu(int x, int y) {
		JPopupMenu popupMenu = this.popupMenu();
		if (popupMenu != null) {
			popupMenu.show(this, x, y);
		}
	}

	/**
	 * Make sure to have both a model and a controller before showing.
	 */
	public void addNotify() {
		if (model == null) {
			this.model_(this.defaultModel());
		}

		if (controller == null) {
			this.controller_(this.defaultController());
		}

		super.addNotify();
	}

	/**
	 * Answer my current controller.
	 * 
	 * @return sra.smalltalk.StController
	 */
	public StController controller() {
		if (controller == null) {
			this.controller_(this.defaultController());
		}
		return controller;
	}

	/**
	 * Set my new controller.
	 * 
	 * @param newController sra.smalltalk.StController
	 */
	public void controller_(StController newController) {
		if (newController != null) {
			newController.view_(this);
		}
		controller = newController;
	}

	/**
	 * Answer my model as JunOpenGL3dModel.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLShowModel
	 * @deprecated since Jun379, use getOpenGL3dModel().
	 */
	public JunOpenGL3dModel getModel() {
		return (JunOpenGL3dModel) this.model();
	}

	/**
	 * Answer my model as JunOpenGL3dModel.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLShowModel
	 */
	public JunOpenGL3dModel getOpenGL3dModel() {
		return (JunOpenGL3dModel) this.model();
	}

	/**
	 * Answer my current model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	public StModel model() {
		if (model == null) {
			this.model_(this.defaultModel());
		}
		return model;
	}

	/**
	 * Set my new model.
	 * 
	 * @param newModel sra.smalltalk.StModel
	 */
	public void model_(StModel newModel) {
		if (model != newModel) {
			if (model != null) {
				model.removeDependent_(this);
			}
			model = newModel;
			if (model != null) {
				model.addDependent_(this);
			}
		}
	}

	/**
	 * Set my new model.
	 * 
	 * @param newModel sra.jun.opengl.display.JunOpenGL3dModel
	 * @deprecated
	 */
	public void setModel(JunOpenGL3dModel newModel) {
		this.model_(newModel);
	}

	/**
	 * Set my new popup menu.
	 * 
	 * @param newPopupMenu javax.swing.JPopupMenu
	 * @deprecated
	 */
	public void setPopupMenu(JPopupMenu newPopupMenu) {
		if (_popupMenu != null) {
			this.remove(_popupMenu);
		}

		_popupMenu = newPopupMenu;

		if (_popupMenu != null) {
			this.add(_popupMenu);
		}
	}

	/**
	 * Create a new popup menu from StMenu and set it.
	 * 
	 * @param aMenu sra.smalltalk.StMenu
	 * @deprecated
	 */
	public void setPopupMenu(StMenu aMenu) {
		if (aMenu == null) {
			this.setPopupMenu((JPopupMenu) null);
		} else {
			this.setPopupMenu(StMenuViewForSwing.CreateJPopupMenu(aMenu));
		}
	}

	/**
	 * Answer the view as a java component.
	 *
	 * @return java.awt.Component
	 */
	public Component toComponent() {
		return (Component) this;
	}

	/**
	 * Answer the top component of the view.
	 *
	 * @return java.awt.Window
	 */
	public Window topComponent() {
		Component aComponent = this.toComponent();
		do {
			if (aComponent instanceof Window) {
				return (Window) aComponent;
			}
			aComponent = aComponent.getParent();
		} while (aComponent != null);

		return null;
	}

	/**
	 * Receive a change notice from an object of whom the receiver is a
	 * dependent.  The argument anAspectSymbol is typically a Symbol that
	 * indicates what change has occurred.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if (this.isShowing()) {
			this.getOpenGL3dModel().updateMenuIndication();
			this.repaint();
		}
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setPreferredSize(new Dimension(256, 256));
	}

	/**
	 * Answer my default controller. Subclasses may override this.
	 * 
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new StController();
	}

	/**
	 * Answer a default model. Subclasses may override this.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunOpenGL3dModel();
	}

	/**
	 * Answer my default rendering context.
	 * 
	 * @return sra.gl4jun.GLjRenderingContext
	 */
	protected GLjRenderingContext defaultRenderingContext() {
		this.setRenderingMode(JunSystem.getDefaultRenderingMode());
		return new JunOpenGLRenderingContext(this, (JFrame) this.topComponent());
	}

	/**
	 * Initialize the receiver.
	 */
	protected void initialize() {
		model = null;
		controller = null;
		_popupMenu = null;
	}

	/**
	 * Answer the popup menu if exist.
	 *
	 * @return javax.swing.JPopupMenu
	 */
	protected JPopupMenu popupMenu() {
		if (_popupMenu != null) {
			this.remove(_popupMenu);
		}

		StMenu popupMenu = this.getOpenGL3dModel().popupMenu();
		if (popupMenu == null) {
			_popupMenu = null;
		} else {
			_popupMenu = this._createPopupMenu(popupMenu);
			this.add(_popupMenu);
		}

		return _popupMenu;
	}

	/**
	 * Render myself on the graphics.
	 * 
	 * @param g sra.gl4jun.GLjRenderingContext
	 */
	protected void render(Graphics g) {
		if (this.getOpenGL3dModel().openGL3dObject() != null) {
			super.render(g);
		}
	}

	/**
	 * Render myself on the rendering context.
	 * 
	 * @param aRenderingContext java.awt.Graphics
	 */
	protected void render(GLjRenderingContext aRenderingContext) {
		this.getOpenGL3dModel().renderOn_((JunOpenGLRenderingContext) aRenderingContext);
	}
}
