package sra.jun.opengl.display;

import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;
import sra.jun.goodies.colors.*;

/**
 * JunOpenGLDisplayLightViewAwt class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   UNKNOWN
 *  @updated   2000/01/25 (by MATSUDA Ryouichi)
 *  @updated   2003/05/09 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLDisplayLightViewAwt.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public class JunOpenGLDisplayLightViewAwt extends StViewPanel implements JunOpenGLDisplayLightView {

	protected JunColorChoiceViewAwt colorChoiceView;
	protected CheckboxGroup lightState;
	protected Checkbox lightStateOn;
	protected Checkbox lightStateOff;
	protected CheckboxGroup lightKind;
	protected Checkbox lightKindAmbient;
	protected Checkbox lightKindParallel;
	protected Checkbox lightKindPoint;
	protected Checkbox lightKindSpot;
	protected TextField positionTextValue;
	protected TextField directionTextValue;
	protected TextField cutoffAngleTextValue;

	/**
	 * Create a new instance of JunOpenGLDisplayLightViewAwt and initialize it.
	 *
	 * @param anOpenGLDisplayLight sra.jun.opengl.display.JunOpenGLDisplayLight
	 */
	public JunOpenGLDisplayLightViewAwt(JunOpenGLDisplayLight anOpenGLDisplayLight) {
		super(anOpenGLDisplayLight);
	}

	/**
	 * Create a new view for an ambient light.
	 *
	 * @return sra.jun.opengl.display.JunOpenGLDisplayLightViewAwt
	 * @param anOpenGLDisplayLight sra.jun.opengl.display.JunOpenGLDisplayLight
	 */
	public static JunOpenGLDisplayLightViewAwt ForAmbientLight(JunOpenGLDisplayLight anOpenGLDisplayLight) {
		return new JunOpenGLDisplayLightViewAwt(anOpenGLDisplayLight) {
			protected void buildComponent() {
				this.setLayout(new WindowSpecLayout());
				this.add(this.colorChoiceView()._onBorderedPanel(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 0f, 127));
				this.add(new Label("Switch:"), WindowSpecLayout.Position(0f, 1, 0f, 128));
				this.add(this.lightStateOn(), WindowSpecLayout.Position(0f, 10, 0f, 151));
				this.add(this.lightStateOff(), WindowSpecLayout.Position(0f, 10, 0f, 174));
				this.setSize(234, 198);
			}
		};
	}

	/**
	 * Create a new view for a position light.
	 *
	 * @return sra.jun.opengl.display.JunOpenGLDisplayLightViewAwt
	 * @param anOpenGLDisplayLight sra.jun.opengl.display.JunOpenGLDisplayLight
	 */
	public static JunOpenGLDisplayLightViewAwt ForPositionLight(JunOpenGLDisplayLight anOpenGLDisplayLight) {
		return new JunOpenGLDisplayLightViewAwt(anOpenGLDisplayLight) {
			protected void buildComponent() {
				this.setLayout(new WindowSpecLayout());
				this.add(this.colorChoiceView()._onBorderedPanel(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 0f, 127));
				this.add(new Label("Switch:"), WindowSpecLayout.Position(0f, 1, 0f, 128));
				this.add(this.lightStateOn(), WindowSpecLayout.Position(0f, 10, 0f, 151));
				this.add(this.lightStateOff(), WindowSpecLayout.Position(0f, 10, 0f, 174));
				this.add(new Label("Kind:"), WindowSpecLayout.Position(0.5f, 1, 0f, 128));
				this.add(this.lightKindParallel(), WindowSpecLayout.Position(0.5f, 10, 0f, 151));
				this.add(this.lightKindPoint(), WindowSpecLayout.Position(0.5f, 10, 0f, 174));
				this.add(this.lightKindSpot(), WindowSpecLayout.Position(0.5f, 10, 0f, 197));
				this.add(new Label("Position: (x , y , z)"), WindowSpecLayout.Position(0f, 1, 0f, 220));
				this.add(this.positionTextValue(), WindowSpecLayout.Position(0f, 1, 0f, 243, 1f, -1, 0f, 265));
				this.add(new Label("Direction: (x , y , z)"), WindowSpecLayout.Position(0f, 1, 0f, 266));
				this.add(this.directionTextValue(), WindowSpecLayout.Position(0f, 1, 0f, 289, 1f, -1, 0f, 311));
				this.add(new Label("Cutoff angle: (degree)"), WindowSpecLayout.Position(0f, 1, 0f, 312));
				this.add(this.cutoffAngleTextValue(), WindowSpecLayout.Position(0f, 1, 0f, 335, 1f, -1, 0f, 357));
				this.setSize(234, 358);
			}
		};
	}

	/**
	 * Answer my color choice view.
	 *
	 * @return sra.jun.goodies.colors.JunColorChoiceViewAwt
	 */
	public JunColorChoiceViewAwt colorChoiceView() {
		if (colorChoiceView == null) {
			colorChoiceView = new JunColorChoiceViewAwt(this.getOpenGLDisplayLight().lightColor());
		}
		return colorChoiceView;
	}

	/**
	 * Answer the text field of the cutoff angle text value.
	 *
	 * @return java.awt.TextField
	 */
	public TextField cutoffAngleTextValue() {
		if (cutoffAngleTextValue == null) {
			cutoffAngleTextValue = new TextField((String) this.getOpenGLDisplayLight().cutoffAngleTextValue().value());
			cutoffAngleTextValue.setVisible(this.getOpenGLDisplayLight().lightKind().value() == $("spot"));
		}
		return cutoffAngleTextValue;
	}

	/**
	 * Answer the text field of the direction text value.
	 *
	 * @return java.awt.TextField
	 */
	public TextField directionTextValue() {
		if (directionTextValue == null) {
			directionTextValue = new TextField((String) this.getOpenGLDisplayLight().directionTextValue().value());
			directionTextValue.setVisible(this.getOpenGLDisplayLight().lightKind().value() == $("parallel") || this.getOpenGLDisplayLight().lightKind().value() == $("spot"));
		}
		return directionTextValue;
	}

	/**
	 * Answer my model as JunOpenGLDisplayLight.
	 *
	 * @return sra.jun.opengl.display.JunOpenGLDisplayLight
	 */
	public JunOpenGLDisplayLight getOpenGLDisplayLight() {
		return (JunOpenGLDisplayLight) this.model();
	}

	/**
	 * Answer the checkbox group of the light kinds.
	 *
	 * @return java.awt.CheckboxGroup
	 */
	public CheckboxGroup lightKind() {
		if (lightKind == null) {
			lightKind = new CheckboxGroup();
		}
		return lightKind;
	}

	/**
	 * Answer the checkbox for the light kind AMBIENT.
	 *
	 * @return java.awt.Checkbox
	 */
	public Checkbox lightKindAmbient() {
		if (lightKindAmbient == null) {
			lightKindAmbient = new Checkbox("Ambient", this.lightKind(), this.getOpenGLDisplayLight().lightKind().value() == $("ambient"));
			lightKindAmbient.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					positionTextValue().setVisible(false);
					directionTextValue().setVisible(false);
					cutoffAngleTextValue().setVisible(false);
					getOpenGLDisplayLight().lightKind().value_($("ambient"));
				}
			});
		}
		return lightKindAmbient;
	}

	/**
	 * Answer the checkbox for the light kind PARALLEL.
	 *
	 * @return java.awt.Checkbox
	 */
	public Checkbox lightKindParallel() {
		if (lightKindParallel == null) {
			lightKindParallel = new Checkbox("Parallel", this.lightKind(), this.getOpenGLDisplayLight().lightKind().value() == $("parallel"));
			lightKindParallel.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					positionTextValue().setVisible(false);
					directionTextValue().setVisible(true);
					cutoffAngleTextValue().setVisible(false);
					getOpenGLDisplayLight().lightKind().value_($("parallel"));
				}
			});
		}
		return lightKindParallel;
	}

	/**
	 * Answer the checkbox for the light kind POINT.
	 *
	 * @return java.awt.Checkbox
	 */
	public Checkbox lightKindPoint() {
		if (lightKindPoint == null) {
			lightKindPoint = new Checkbox("Point", this.lightKind(), this.getOpenGLDisplayLight().lightKind().value() == $("point"));
			lightKindPoint.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					positionTextValue().setVisible(true);
					directionTextValue().setVisible(false);
					cutoffAngleTextValue().setVisible(false);
					getOpenGLDisplayLight().lightKind().value_($("point"));
				}
			});
		}
		return lightKindPoint;
	}

	/**
	 * Answer the checkbox for the light kind SPOT.
	 *
	 * @return java.awt.Checkbox
	 */
	public Checkbox lightKindSpot() {
		if (lightKindSpot == null) {
			lightKindSpot = new Checkbox("Spot", this.lightKind(), this.getOpenGLDisplayLight().lightKind().value() == $("spot"));
			lightKindSpot.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					positionTextValue().setVisible(true);
					directionTextValue().setVisible(true);
					cutoffAngleTextValue().setVisible(true);
					getOpenGLDisplayLight().lightKind().value_($("spot"));
				}
			});
		}
		return lightKindSpot;
	}

	/**
	 * Answer the checkbox group of the light states.
	 *
	 * @return java.awt.CheckboxGroup
	 */
	public CheckboxGroup lightState() {
		if (lightState == null) {
			lightState = new CheckboxGroup();
		}
		return lightState;
	}

	/**
	 * Answer the checkbox for the light state ON.
	 *
	 * @return java.awt.Checkbox
	 */
	public Checkbox lightStateOff() {
		if (lightStateOff == null) {
			lightStateOff = new Checkbox("Off", this.lightState(), this.getOpenGLDisplayLight().lightState()._booleanValue() == false);
			lightStateOff.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					getOpenGLDisplayLight().beOff();
				}
			});
		}
		return lightStateOff;
	}

	/**
	 * Answer the checkbox for the light state ON.
	 *
	 * @return java.awt.Checkbox
	 */
	public Checkbox lightStateOn() {
		if (lightStateOn == null) {
			lightStateOn = new Checkbox("On", this.lightState(), this.getOpenGLDisplayLight().lightState()._booleanValue() == true);
			lightStateOn.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					getOpenGLDisplayLight().beOn();
				}
			});
		}
		return lightStateOn;
	}

	/**
	 * Answer the TextField of the position text value.
	 *
	 * @return java.awt.TextField
	 */
	public TextField positionTextValue() {
		if (positionTextValue == null) {
			positionTextValue = new TextField((String) this.getOpenGLDisplayLight().textValue().value());
			positionTextValue.setVisible(this.getOpenGLDisplayLight().lightKind().value() == $("point") || this.getOpenGLDisplayLight().lightKind().value() == $("spot"));
			positionTextValue.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					getOpenGLDisplayLight().textValueChanged_(positionTextValue.getText());
				}
			});
		}
		return positionTextValue;
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.colorChoiceView()._onBorderedPanel(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -1, 0f, 127));
		this.add(new Label("Switch:"), WindowSpecLayout.Position(0f, 1, 0f, 128));
		this.add(this.lightStateOn(), WindowSpecLayout.Position(0f, 10, 0f, 151));
		this.add(this.lightStateOff(), WindowSpecLayout.Position(0f, 10, 0f, 174));
		this.add(new Label("Kind:"), WindowSpecLayout.Position(0.5f, 1, 0f, 128));
		this.add(this.lightKindAmbient(), WindowSpecLayout.Position(0.5f, 10, 0f, 151));
		this.add(this.lightKindParallel(), WindowSpecLayout.Position(0.5f, 10, 0f, 174));
		this.add(this.lightKindPoint(), WindowSpecLayout.Position(0.5f, 10, 0f, 197));
		this.add(this.lightKindSpot(), WindowSpecLayout.Position(0.5f, 10, 0f, 220));
		this.add(new Label("Position: (x , y , z)"), WindowSpecLayout.Position(0f, 1, 0f, 242));
		this.add(this.positionTextValue(), WindowSpecLayout.Position(0f, 1, 0f, 266, 1f, -1, 0f, 288));
		this.add(new Label("Direction: (x , y , z)"), WindowSpecLayout.Position(0f, 1, 0f, 289));
		this.add(this.directionTextValue(), WindowSpecLayout.Position(0f, 1, 0f, 312, 1f, -1, 0f, 334));
		this.add(new Label("Cutoff angle: (degree)"), WindowSpecLayout.Position(0f, 1, 0f, 335));
		this.add(this.cutoffAngleTextValue(), WindowSpecLayout.Position(0f, 1, 0f, 358, 1f, -1, 0f, 380));
		this.setSize(234, 381);
	}
}
