package sra.jun.opengl.display;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.io.*;
import java.text.DateFormat;
import java.util.Date;
import sra.smalltalk.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.curves.Jun3dLine;
import sra.jun.geometry.surfaces.JunPlane;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.goodies.button.JunButtonModel;
import sra.jun.goodies.cursors.JunCursors;
import sra.jun.goodies.files.*;
import sra.jun.goodies.images.streams.*;
import sra.jun.goodies.images.support.JunImageProcessor;
import sra.jun.goodies.progress.*;
import sra.jun.goodies.wheels.JunThumbWheel;
import sra.jun.opengl.lights.JunOpenGLLight;
import sra.jun.opengl.objects.*;
import sra.jun.opengl.picking.*;
import sra.jun.opengl.projection.*;
import sra.jun.opengl.support.JunOpenGLRenderingContext;
import sra.jun.opengl.texture.JunOpenGLTexture;
import sra.jun.system.framework.JunDialog;
import sra.jun.system.support.JunSystem;
import sra.jun.vrml.support.*;

/**
 * JunOpenGLDisplayModel class
 * 
 *  @author    He Weijie
 *  @created   1998/10/08 (by MATSUDA Ryouichi)
 *  @updated   1998/10/27 (by He Weijie)
 *  @updated   1999/06/25 (by nisinaka)
 *  @updated   1999/08/04 (by nisinaka)
 *  @updated   2000/01/06 (by MATSUDA Ryouichi)
 *  @updated   2003/05/13 (by nisinaka)
 *  @updated   2003/07/01 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun493 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLDisplayModel.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public class JunOpenGLDisplayModel extends JunOpenGL3dModel implements ClipboardOwner {

	protected JunButtonModel pickButton;
	protected JunButtonModel grabButton;
	protected JunButtonModel dragButton;
	protected JunButtonModel dollyButton;
	protected JunButtonModel focusButton;
	protected StMenu _fileMenu;
	protected StMenu _editMenu;
	protected StMenu _miscMenu;

	/**
	 * Constructor Method
	 */
	public JunOpenGLDisplayModel() {
		super();
	}

	/**
	 * Create a new instance of JunOpenGLDisplayModel and set the JunOpenGL3dObject.
	 * This is the replacement of the class method "JunOpenGLDisplayModel class >> displayObject:" of the Smalltalk version.
	 *
	 * @param anOpenGL3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGLDisplayModel(JunOpenGL3dObject anOpenGL3dObject) {
		this();
		this.displayObject_(anOpenGL3dObject);
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param aClass sra.java.Class
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public static JunOpenGLDisplayModel DisplayObject_(Class aClass, JunOpenGL3dObject a3dObject) {
		JunOpenGLDisplayModel displayModel = (JunOpenGLDisplayModel) _New(aClass);
		displayModel.displayObject_(a3dObject);

		return displayModel;
	}

	/**
	 * Create a new instance of JunOpenGLDisplayModel and set the JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @deprecated since Jun379. Use the constructor.
	 */
	public static JunOpenGLDisplayModel DisplayObject_(JunOpenGL3dObject a3dObject) {
		JunOpenGLDisplayModel displayModel = new JunOpenGLDisplayModel();
		displayModel.displayObject_(a3dObject);
		return displayModel;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public static JunOpenGLDisplayModel Show_(JunOpenGL3dObject a3dObject) {
		return Show_eyePoint_(a3dObject, null);
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param eyePoint sra.jun.grometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public static JunOpenGLDisplayModel Show_eyePoint_(JunOpenGL3dObject a3dObject, Jun3dPoint eyePoint) {
		return Show_eyePoint_sightPoint_(a3dObject, eyePoint, null);
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param eyePoint sra.jun.grometry.basic.Jun3dPoint
	 * @param sightPoint sra.jun.grometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public static JunOpenGLDisplayModel Show_eyePoint_sightPoint_(JunOpenGL3dObject a3dObject, Jun3dPoint eyePoint, Jun3dPoint sightPoint) {
		return Show_eyePoint_sightPoint_upVector_(a3dObject, eyePoint, sightPoint, null);
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param eyePoint sra.jun.grometry.basic.Jun3dPoint
	 * @param sightPoint sra.jun.grometry.basic.Jun3dPoint
	 * @param upVector sra.jun.grometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public static JunOpenGLDisplayModel Show_eyePoint_sightPoint_upVector_(JunOpenGL3dObject a3dObject, Jun3dPoint eyePoint, Jun3dPoint sightPoint, Jun3dPoint upVector) {
		return Show_eyePoint_sightPoint_upVector_viewFactor_(a3dObject, eyePoint, sightPoint, upVector, null);
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param eyePoint sra.jun.grometry.basic.Jun3dPoint
	 * @param sightPoint sra.jun.grometry.basic.Jun3dPoint
	 * @param upVector sra.jun.grometry.basic.Jun3dPoint
	 * @param viewFactor java.lang.Double
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public static JunOpenGLDisplayModel Show_eyePoint_sightPoint_upVector_viewFactor_(JunOpenGL3dObject a3dObject, Jun3dPoint eyePoint, Jun3dPoint sightPoint, Jun3dPoint upVector, Double viewFactor) {
		JunOpenGLDisplayModel displayModel = new JunOpenGLDisplayModel(a3dObject);

		if (eyePoint != null) {
			displayModel.defaultEyePoint_(eyePoint);
		}

		if (sightPoint != null) {
			displayModel.defaultSightPoint_(sightPoint);
		}

		if (upVector != null) {
			displayModel.defaultUpVector_(upVector);
		}

		if (viewFactor != null) {
			displayModel.defaultViewFactor_(viewFactor.doubleValue());
		}

		displayModel.open();

		return displayModel;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject boundingObjectFor_(JunOpenGL3dObject a3dObject) {
		Jun3dBoundingBox aBox = a3dObject.boundingBox();
		Jun3dPoint originPoint = aBox.origin();
		Jun3dPoint cornerPoint = aBox.corner();
		Jun3dPoint[] fourPoints = new Jun3dPoint[4];
		fourPoints[0] = new Jun3dPoint(originPoint.x(), originPoint.y(), cornerPoint.z());
		fourPoints[1] = new Jun3dPoint(originPoint.x(), cornerPoint.y(), cornerPoint.z());
		fourPoints[2] = new Jun3dPoint(originPoint.x(), cornerPoint.y(), originPoint.z());
		fourPoints[3] = new Jun3dPoint(originPoint.x(), originPoint.y(), originPoint.z());

		JunOpenGL3dPolylineLoop polylineLoop1 = (JunOpenGL3dPolylineLoop) JunOpenGL3dPolylineLoop.Vertexes_paint_(fourPoints, Color.gray);
		polylineLoop1.alpha_(0.5f);
		fourPoints[3] = new Jun3dPoint(cornerPoint.x(), originPoint.y(), cornerPoint.z());
		fourPoints[2] = new Jun3dPoint(cornerPoint.x(), cornerPoint.y(), cornerPoint.z());
		fourPoints[1] = new Jun3dPoint(cornerPoint.x(), cornerPoint.y(), originPoint.z());
		fourPoints[0] = new Jun3dPoint(cornerPoint.x(), originPoint.y(), originPoint.z());

		JunOpenGL3dPolylineLoop polylineLoop2 = (JunOpenGL3dPolylineLoop) JunOpenGL3dPolylineLoop.Vertexes_paint_(fourPoints, Color.gray);
		polylineLoop2.alpha_(0.5f);
		fourPoints[0] = new Jun3dPoint(originPoint.x(), cornerPoint.y(), cornerPoint.z());
		fourPoints[1] = new Jun3dPoint(cornerPoint.x(), cornerPoint.y(), cornerPoint.z());
		fourPoints[2] = new Jun3dPoint(cornerPoint.x(), cornerPoint.y(), originPoint.z());
		fourPoints[3] = new Jun3dPoint(originPoint.x(), cornerPoint.y(), originPoint.z());

		JunOpenGL3dPolylineLoop polylineLoop3 = (JunOpenGL3dPolylineLoop) JunOpenGL3dPolylineLoop.Vertexes_paint_(fourPoints, Color.gray);
		polylineLoop3.alpha_(0.5f);
		fourPoints[3] = new Jun3dPoint(originPoint.x(), cornerPoint.y(), cornerPoint.z());
		fourPoints[2] = new Jun3dPoint(cornerPoint.x(), cornerPoint.y(), cornerPoint.z());
		fourPoints[1] = new Jun3dPoint(cornerPoint.x(), cornerPoint.y(), originPoint.z());
		fourPoints[0] = new Jun3dPoint(originPoint.x(), cornerPoint.y(), originPoint.z());

		JunOpenGL3dPolylineLoop polylineLoop4 = (JunOpenGL3dPolylineLoop) JunOpenGL3dPolylineLoop.Vertexes_paint_(fourPoints, Color.gray);
		polylineLoop4.alpha_(0.5f);
		fourPoints[0] = new Jun3dPoint(originPoint.x(), originPoint.y(), cornerPoint.z());
		fourPoints[1] = new Jun3dPoint(cornerPoint.x(), originPoint.y(), cornerPoint.z());
		fourPoints[2] = new Jun3dPoint(cornerPoint.x(), cornerPoint.y(), cornerPoint.z());
		fourPoints[3] = new Jun3dPoint(originPoint.x(), cornerPoint.y(), cornerPoint.z());

		JunOpenGL3dPolylineLoop polylineLoop5 = (JunOpenGL3dPolylineLoop) JunOpenGL3dPolylineLoop.Vertexes_paint_(fourPoints, Color.gray);
		polylineLoop5.alpha_(0.5f);
		fourPoints[0] = new Jun3dPoint(originPoint.x(), originPoint.y(), cornerPoint.z());
		fourPoints[1] = new Jun3dPoint(cornerPoint.x(), originPoint.y(), cornerPoint.z());
		fourPoints[2] = new Jun3dPoint(cornerPoint.x(), cornerPoint.y(), cornerPoint.z());
		fourPoints[3] = new Jun3dPoint(originPoint.x(), cornerPoint.y(), cornerPoint.z());

		JunOpenGL3dPolylineLoop polylineLoop6 = (JunOpenGL3dPolylineLoop) JunOpenGL3dPolylineLoop.Vertexes_paint_(fourPoints, Color.gray);
		polylineLoop6.alpha_(0.5f);

		JunOpenGL3dObject anAxes = JunOpenGL3dObject.Axes2();
		anAxes = anAxes.transform_(Jun3dTransformation.Scale_(cornerPoint.minus_(originPoint)));
		anAxes = anAxes.transform_(Jun3dTransformation.Translate_(originPoint));

		JunOpenGL3dCompoundObject boundingObject = new JunOpenGL3dCompoundObject();
		boundingObject.add_(polylineLoop1);
		boundingObject.add_(polylineLoop2);
		boundingObject.add_(polylineLoop3);
		boundingObject.add_(polylineLoop4);
		boundingObject.add_(polylineLoop5);
		boundingObject.add_(polylineLoop6);
		boundingObject.add_(anAxes);

		return boundingObject;
	}

	/**
	 * Answer my current button status.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public StSymbol buttonState() {
		if (pickButton().value() == true) {
			return $("pick");
		} else if (grabButton().value() == true) {
			return $("grab");
		} else if (dragButton().value() == true) {
			return $("drag");
		} else if (focusButton().value() == true) {
			return $("focus");
		}

		return $("none");
	}

	/**
	 * Menu message: Clear the display object.
	 */
	public void clearObject() {
		if (this.selectedObjects().isEmpty()) {
			return;
		}

		if (this.displayObject().isCompound() == false) {
			return;
		}

		JunCursorAnimator cursorAnimator = JunCursorAnimator.ClockCursors();
		try {
			cursorAnimator._show();

			Object[] objects = this.selectedObjects().toArray();
			for (int i = 0; i < objects.length; i++) {
				((JunOpenGL3dCompoundObject) this.displayObject()).remove_((JunOpenGL3dObject) objects[i]);
			}
			this.clearSelectedObjects();
			this.changed_($("object"));
		} finally {
			cursorAnimator._restore();
		}

		this.updateMenuIndication();
	}

	/**
	 * Menu message: Copy the object.
	 */
	public void copyObject() {
		if (this.openGL3dObject() == null) {
			return;
		}

		JunCursorAnimator cursorAnimator = JunCursorAnimator.ClockCursors();
		try {
			cursorAnimator._show();

			JunOpenGL3dObject object = this.spawningObject();
			String string = this.defaultStampForLST10() + object.toLispList().saveString();
			Toolkit.getDefaultToolkit().getSystemClipboard().setContents(new StringSelection(string), this);
		} finally {
			cursorAnimator._restore();
		}
	}

	/**
	 * Menu message: Cut the selected objects.
	 */
	public void cutObject() {
		if (this.selectedObjects().isEmpty()) {
			return;
		}

		if (this.displayObject().isCompound() == false) {
			return;
		}

		JunCursorAnimator cursorAnimator = JunCursorAnimator.ClockCursors();
		try {
			cursorAnimator._show();

			this.copyObject();
			Object[] objects = this.selectedObjects().toArray();
			for (int i = 0; i < objects.length; i++) {
				((JunOpenGL3dCompoundObject) this.displayObject()).remove_((JunOpenGL3dObject) objects[i]);
			}
			this.clearSelectedObjects();
			this.changed_($("object"));
		} finally {
			cursorAnimator._restore();
		}

		this.updateMenuIndication();
	}

	/**
	 * Answer the stamp string for the LST 1.0 file format.
	 * 
	 * @return java.lang.String
	 */
	public String defaultStampForLST10() {
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		pw.println("%LST V1.0 List Shape Transmission (Lisp S Expression)");
		pw.println("% This file was created by " + JunSystem.System() + JunSystem.Version());
		pw.println("% " + DateFormat.getInstance().format(new Date()));
		pw.println();
		pw.flush();

		return sw.toString();
	}

	/**
	 * Answer the stamp string for the VRML 1.0 file format.
	 * 
	 * @return java.lang.String
	 */
	public String defaultStampForWRL10() {
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		pw.println("#VRML V1.0 ascii");
		pw.println("# This file was created by " + JunSystem.System() + JunSystem.Version());
		pw.println("# " + DateFormat.getInstance().format(new Date()));
		pw.println();
		pw.flush();

		return sw.toString();
	}

	/**
	 * Answer the stamp string for the VRML 2.0 file format.
	 * 
	 * @return java.lang.String
	 */
	public String defaultStampForWRL20() {
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		pw.println("#VRML V2.0 utf8");
		pw.println();
		pw.println("WorldInfo {");
		pw.print("\t");
		pw.println("info [");
		pw.print("\t\t");
		pw.println("\"This file was created by " + JunSystem.System() + JunSystem.Version() + "\"");
		pw.print("\t\t");
		pw.println("\"" + DateFormat.getInstance().format(new Date()) + "\"");
		pw.print("\t");
		pw.println("]");
		pw.print("\t");
		pw.print("title \"");

		if (this.displayObject() != null) {
			pw.print(this.displayObject().name());
		}

		pw.print(": Sample of ");
		pw.print(this.getClass().getName());
		pw.println("\"");
		pw.println("}");
		pw.println();
		pw.flush();

		return sw.toString();
	}

	/**
	 * Answer the stamp string for the VRML 97 file format.
	 * 
	 * @return java.lang.String
	 */
	public String defaultStampForWRL97() {
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		pw.println("#VRML V2.0 utf8");
		pw.println("# This file was created by " + JunSystem.System() + JunSystem.Version() + " for Java");
		pw.println("# " + DateFormat.getInstance().format(new Date()));
		pw.println();
		pw.flush();

		return sw.toString();
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunOpenGLDisplayViewAwt(this);
		} else {
			return new JunOpenGLDisplayViewSwing(this);
		}
	}

	/**
	 * Answer my current display object.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject displayObject() {
		return super.openGL3dObject();
	}

	/**
	 * Set my new display object.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public void displayObject_(JunOpenGL3dObject a3dObject) {
		super.openGL3dObject_(a3dObject);

		if (this.showModel() != null) {
			this.showModel().flushObject();
		}
	}

	/**
	 * Answer my current projection.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjection
	 */
	public JunOpenGLProjection displayProjection() {
		return super.openGLProjection();
	}

	/**
	 * Set my new projection.
	 * 
	 * @param aProjection sra.jun.opengl.projection.JunOpenGLProjection
	 */
	public void displayProjection_(JunOpenGLProjection aProjection) {
		super.openGLProjection_(aProjection);
	}

	/**
	 * Answer my current projector.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjector
	 */
	public JunOpenGLProjector displayProjector() {
		return super.openGLProjector();
	}

	/**
	 * Answer the button model for the dolly button.
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 */
	public JunButtonModel dollyButton() {
		if (dollyButton == null) {
			JunButtonModel button = new JunButtonModel();
			button.value_(false);
			button.visual_(JunCursors.DollyCursorImage());
			button.action_(new StBlockClosure() {
				public Object value_(Object o) {
					JunButtonModel model = (JunButtonModel) o;
					model.value_(!model.value());
					return model;
				}
			});
			dollyButton = button;
		}

		return dollyButton;
	}

	/**
	 * Answer my drag button.
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 */
	public JunButtonModel dragButton() {
		if (dragButton == null) {
			final JunOpenGLDisplayModel self = this;
			JunButtonModel button = new JunButtonModel();
			button.value_(false);
			button.visual_(JunCursors.QuartersCursorImage());
			button.action_(new StBlockClosure() {
				public Object value_(Object o) {
					JunButtonModel model = (JunButtonModel) o;
					model.value_(!model.value());
					if (model.value()) {
						self.focusButton().value_(false);
						self.grabButton().value_(false);
						self.pickButton().value_(false);
					}
					self.changed_($("state"));
					return model;
				}
			});
			dragButton = button;
		}

		return dragButton;
	}

	/**
	 * Change the focus area.
	 * 
	 * @param originalRectangle sra.jun.geometry.basic.Jun2dBoundingBox
	 * @param newRectangle sra.jun.geometry.basic.Jun2dBoundingBox
	 */
	public void focus_to_(Jun2dBoundingBox originalRectangle, Jun2dBoundingBox newRectangle) {
		if (this.displayObject() == null) {
			return;
		}

		JunOpenGLProjection aProjection = this.displayProjection();
		Jun3dPoint originalCenter = aProjection.translateTo3dPointFromPoint_((Jun2dPoint) originalRectangle.origin().plus_(originalRectangle.corner()).dividedBy_(2));
		Jun3dPoint newCenter = aProjection.translateTo3dPointFromPoint_((Jun2dPoint) newRectangle.origin().plus_(newRectangle.corner()).dividedBy_(2));
		aProjection.sightPoint_((Jun3dPoint) aProjection.sightPoint().plus_(newCenter.minus_(originalCenter)));

		Jun3dPoint originalTop = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(0, originalRectangle.origin().y()));
		Jun3dPoint originalBottom = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(0, originalRectangle.corner().y()));
		Jun3dPoint originalLeft = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(originalRectangle.origin().x(), 0));
		Jun3dPoint originalRight = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(originalRectangle.corner().x(), 0));
		double originalHeight = originalTop.minus_(originalBottom).length();
		double originalWidth = originalLeft.minus_(originalRight).length();
		Jun3dPoint newTop = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(0, newRectangle.origin().y()));
		Jun3dPoint newBottom = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(0, newRectangle.corner().y()));
		Jun3dPoint newLeft = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(newRectangle.origin().x(), 0));
		Jun3dPoint newRight = aProjection.translateTo3dPointFromPoint_(new Jun2dPoint(newRectangle.corner().x(), 0));
		double newHeight = newTop.minus_(newBottom).length();
		double newWidth = newLeft.minus_(newRight).length();
		double newViewHeight = aProjection.regularHeight() * newHeight / originalHeight;
		double newViewWidth = newViewHeight * newWidth / newHeight;
		double newZoomHeight = newViewHeight;
		if (originalWidth / originalHeight * newViewHeight < newViewWidth) {
			double tmpViewHeight = originalWidth * newViewHeight / (originalHeight * newViewWidth);
			newZoomHeight = newViewHeight * tmpViewHeight;
		}

		if (((originalHeight * 0.01d) < newHeight) && ((newHeight * 0.01d) < originalHeight)) {
			aProjection.zoomHeight_(newZoomHeight);
		}

		this.displayProjection_(aProjection);
		this.changed_($("projection"));
	}

	/**
	 * Answer my focus button.
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 */
	public JunButtonModel focusButton() {
		if (focusButton == null) {
			final JunOpenGLDisplayModel self = this;
			JunButtonModel button = new JunButtonModel();
			button.value_(false);
			button.visual_(JunCursors.GlassCursorImage());
			button.action_(new StBlockClosure() {
				public Object value_(Object o) {
					JunButtonModel model = (JunButtonModel) o;
					model.value_(!model.value());
					if (model.value()) {
						self.grabButton().value_(false);
						self.dragButton().value_(false);
						self.pickButton().value_(false);
					}
					self.changed_($("state"));
					return model;
				}
			});
			focusButton = button;
		}

		return focusButton;
	}

	/**
	 * Answer my grab button model.
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 */
	public JunButtonModel grabButton() {
		if (grabButton == null) {
			final JunOpenGLDisplayModel self = this;
			JunButtonModel button = new JunButtonModel();
			button.value_(false);
			button.visual_(JunCursors.HandCursorImage());
			button.action_(new StBlockClosure() {
				public Object value_(Object o) {
					JunButtonModel model = (JunButtonModel) o;
					model.value_(!model.value());
					if (model.value()) {
						self.focusButton().value_(false);
						self.dragButton().value_(false);
						self.pickButton().value_(false);
					}
					self.changed_($("state"));
					return model;
				}
			});
			grabButton = button;
		}

		return grabButton;
	}

	/**
	 * Load an LST1.0 file and create a JunOpenGL3dObject.
	 * 
	 * @param aStream sra.smalltalk.StReadStream
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject loadFromLST10_(StReadStream aStream) {
		JunCursors cursor = new JunCursors(JunCursors.ReadCursor());
		try {
			cursor._show();

			return JunOpenGL3dObject.LoadFrom_(aStream);
		} catch (SmalltalkException e) {
			JunDialog.Warn_(e.getMessage());
			return null;
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Load a VRML1.0 file and create a JunOpenGL3dObject.
	 * 
	 * @param aStream sra.smalltalk.StReadStream
	 */
	public void loadFromWRL10_(StReadStream aStream) {
		JunCursors cursor = new JunCursors(JunCursors.ReadCursor());
		try {
			cursor._show();

			JunVrmlCompiler.Source_parser_generator_displayModel_(aStream, new JunVrmlParser10(), new JunVrmlGenerator10(), this);
		} catch (SmalltalkException e) {
			JunDialog.Warn_(e.getMessage());
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Load a VRML2.0 file and create a JunOpenGL3dObject.
	 * 
	 * @param aStream sra.smalltalk.StReadStream
	 */
	public void loadFromWRL20_(StReadStream aStream) {
		JunCursors cursor = new JunCursors(JunCursors.ReadCursor());
		try {
			cursor._show();

			JunVrmlCompiler.Source_parser_generator_displayModel_(aStream, new JunVrmlParser97(), new JunVrmlGenerator97(), this);
		} catch (SmalltalkException e) {
			JunDialog.Warn_(e.getMessage());
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Load a VRML97 file and create a JunOpenGL3dObject.
	 * 
	 * @param aStream sra.smalltalk.StReadStream
	 */
	public void loadFromWRL97_(StReadStream aStream) {
		JunCursors cursor = new JunCursors(JunCursors.ReadCursor());
		try {
			cursor._show();

			JunVrmlCompiler.Source_parser_generator_displayModel_(aStream, new JunVrmlParser97(), new JunVrmlGenerator97(), this);
		} catch (SmalltalkException e) {
			JunDialog.Warn_(e.getMessage());
		} finally {
			cursor._restore();
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param clipboard DOCUMENT ME!
	 * @param contents DOCUMENT ME!
	 */
	public void lostOwnership(Clipboard clipboard, Transferable contents) {
		//		System.out.println("Clipboard contents replaced");
	}

	/**
	 * Menu message: Open a new display model.
	 */
	public void newModel() {
		try {
			JunOpenGLDisplayModel displayModel = (JunOpenGLDisplayModel) this.getClass().newInstance();
			displayModel.open();
		} catch (InstantiationException e) {
		} catch (IllegalAccessException e) {
		}
	}

	/**
	 * Answer my current display object.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public final JunOpenGL3dObject openGL3dObject() {
		return this.displayObject();
	}

	/**
	 * Set my new display object.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public final void openGL3dObject_(JunOpenGL3dObject a3dObject) {
		this.displayObject_(a3dObject);
	}

	/**
	 * Answer my current projection.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjection
	 */
	public final JunOpenGLProjection openGLProjection() {
		return this.displayProjection();
	}

	/**
	 * Set my new projection.
	 * 
	 * @param aProjection sra.jun.opengl.projection.JunOpenGLProjection
	 */
	public final void openGLProjection_(JunOpenGLProjection aProjection) {
		this.displayProjection_(aProjection);
	}

	/**
	 * Answer my current projector.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjector
	 */
	public final JunOpenGLProjector openGLProjector() {
		return this.displayProjector();
	}

	/**
	 * Menu message: Open an LST file.
	 */
	public void openLST() {
		this.openLST10();
	}

	/**
	 * Menu message: Open an LST1.0 file.
	 * 
	 * @throws sra.smalltalk.SmalltalkException
	 */
	public void openLST10() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType("LST files", new String[] { ".lst", ".LST" })
		};
		File file = JunFileDialog.Request("Select an LST file.", fileTypes, fileTypes[0]);
		if (file == null) {
			return;
		}

		JunOpenGL3dObject object = null;
		try {
			object = this.readFromLST10_(file);
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
		if (object == null) {
			return;
		}

		this.displayObject_(object);

		this.resetView();
		if (this.showModel() != null) {
			this.showModel().resetView();
		}
	}

	/**
	 * Menu message: Open a VRML1.0 file.
	 * 
	 * @throws sra.smalltalk.SmalltalkException
	 */
	public void openWRL10() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType("VRML files", new String[] { ".wrl", ".WRL" })
		};
		File file = JunFileDialog.Request("Select a VRML file.", fileTypes, fileTypes[0]);
		if (file == null) {
			return;
		}

		try {
			this.readFromWRL10_(file);
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}

		this.resetView();
		if (this.showModel() != null) {
			this.showModel().resetView();
		}
	}

	/**
	 * Menu message: Open a VRML2.0 file.
	 * 
	 * @throws sra.smalltalk.SmalltalkException
	 */
	public void openWRL20() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType("VRML files", new String[] { ".wrl", ".WRL" })
		};
		File file = JunFileDialog.Request("Select a VRML file.", fileTypes, fileTypes[0]);
		if (file == null) {
			return;
		}

		try {
			this.readFromWRL20_(file);
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}

		this.resetView();
		if (this.showModel() != null) {
			this.showModel().resetView();
		}
	}

	/**
	 * Menu message: Open a VRML97 file.
	 * 
	 * @throws sra.smalltalk.SmalltalkException
	 */
	public void openWRL97() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType("VRML files", new String[] { ".wrl", ".WRL" })
		};
		File file = JunFileDialog.Request("Select a VRML file.", fileTypes, fileTypes[0]);
		if (file == null) {
			return;
		}

		try {
			this.readFromWRL97_(file);
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Menu message: Paste the object from the clipboard.
	 * 
	 * @throws sra.smalltalk.SmalltalkException
	 */
	public void pasteObject() {
		Transferable content = Toolkit.getDefaultToolkit().getSystemClipboard().getContents(this);
		String string = null;
		try {
			string = (String) content.getTransferData(DataFlavor.stringFlavor);
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}
		if (string == null) {
			return;
		}

		if (string.startsWith(this.defaultStampForLST10().substring(0, 9)) == false) {
			JunDialog.Warn_("The contents of copy buffer is not LST.");
			return;
		}

		JunCursorAnimator cursorAnimator = JunCursorAnimator.ClockCursors();
		try {
			cursorAnimator._show();

			JunOpenGL3dObject object = JunOpenGL3dObject.LoadFrom_(string);
			if (this.displayObject() == null) {
				this.displayObject_(object);
				this.clearSelectedObjects();
				this.addSelectedObject_(object);
				this.resetView();
			} else {
				if (this.displayObject().isCompound()) {
					((JunOpenGL3dCompoundObject) this.displayObject()).add_(object);
					this.clearSelectedObjects();
					this.addSelectedObject_(object);
					this.changed_($("object"));
				} else {
					object = JunOpenGL3dCompoundObject.With_with_(this.displayObject(), object);
					this.displayObject_(object);
					this.clearSelectedObjects();
					this.addSelectedObject_(object);
					this.changed_($("object"));
				}
			}
		} finally {
			cursorAnimator._restore();
		}

		this.updateMenuIndication();
	}

	/**
	 * Answer the JunOpenGL3dObject at the specified point on the view.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint,MouseEvent
	 * @param event DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject pick_with_(Jun2dPoint aPoint, MouseEvent event) {
		if (this.displayObject() == null) {
			return null;
		}

		if (this.displayObject().isPrimitive()) {
			return null;
		}

		JunOpenGL3dObject pickedObject = JunOpenGLObjectPicker.PickObjectAt_fromCompound_projection_(aPoint, (JunOpenGLPickingObjects) this.displayObject(), this.displayProjection());
		if (pickedObject == null) {
			if (this.selectedObjects().isEmpty() == false) {
				this.clearSelectedObjects();
				this.changed_($("selection"));
			}
			return null;
		} else {
			if (event.isShiftDown()) {
				if (this.selectedObjects().contains(pickedObject)) {
					this.removeSelectedObject_(pickedObject);
					this.changed_($("selection"));
				} else {
					this.addSelectedObject_(pickedObject);
					this.changed_($("selection"));
				}
			} else {
				if (event.isControlDown()) {
					this.ctrlClick_(pickedObject);
				} else if (event.isAltDown()) {
					this.altClick_(pickedObject);
				} else {
					this.clearSelectedObjects();
					this.addSelectedObject_(pickedObject);
					this.changed_($("selection"));
				}
			}

			return pickedObject;
		}
	}

	/**
	 * Answer my pick button.
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 */
	public JunButtonModel pickButton() {
		if (pickButton == null) {
			final JunOpenGLDisplayModel self = this;
			JunButtonModel button = new JunButtonModel();
			button.value_(false);
			button.visual_(JunCursors.NormalCursorImage());
			button.action_(new StBlockClosure() {
				public Object value_(Object o) {
					JunButtonModel model = (JunButtonModel) o;
					model.value_(!model.value());
					if (model.value()) {
						self.focusButton().value_(false);
						self.dragButton().value_(false);
						self.grabButton().value_(false);
					}
					self.changed_($("state"));
					return model;
				}
			});
			pickButton = button;
		}

		return pickButton;
	}

	/**
	 * Menu message: Close all windows to quit.
	 */
	public void quitDoing() {
		this.closeRequest();
	}

	/**
	 * Load an LST1.0 file and create a JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param aFile java.io.File
	 * @exception java.io.IOException
	 */
	public JunOpenGL3dObject readFromLST10_(File aFile) throws IOException {
		if (aFile == null) {
			return null;
		}

		JunCursors cursor = new JunCursors(JunCursors.ReadCursor());
		try {
			cursor._show();

			final long size = aFile.length();
			final StReadStream stream = new StReadStream(aFile);
			try {
				if (size <= 100 * 1000) {
					return this.loadFromLST10_(stream);
				} else {
					// Show the progress.
					final StValueHolder result = new StValueHolder();
					final JunProgress progress = new JunProgress();
					progress.message_("reading LST...");
					progress.do_(new StBlockClosure() {
						public Object value() {
							progress.value_(0);

							Thread thread = new Thread() {
								public void run() {
									while (progress.value() < 1) {
										float value = Math.round((float) stream.position() / size / 0.005) * 0.005f;
										if (progress.value() != value) {
											progress.value_(value);
										}
										try {
											Thread.sleep(1000);
										} catch (InterruptedException e) {
										}
									}
								}
							};
							thread.setPriority(Thread.NORM_PRIORITY + 1);
							thread.start();

							try {
								result.value_(JunOpenGLDisplayModel.this.loadFromLST10_(stream));
							} finally {
								progress.value_(1);
							}

							return null;
						}
					});
					return (JunOpenGL3dObject) result.value();
				}
			} finally {
				stream.close();
			}
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Load a VRML1.0 file and create a JunOpenGL3dObject.
	 * 
	 * @param aFile java.io.File
	 * @exception java.io.IOException
	 */
	public void readFromWRL10_(File aFile) throws IOException {
		if (aFile == null) {
			return;
		}

		JunCursors cursor = new JunCursors(JunCursors.ReadCursor());
		try {
			cursor._show();

			final long size = aFile.length();
			final StReadStream stream = new StReadStream(aFile);
			try {
				if (size <= 100 * 1000) {
					this.loadFromWRL10_(stream);
				} else {
					// Show the progress.
					final JunProgress progress = new JunProgress();
					progress.message_("reading WRL...");
					progress.do_(new StBlockClosure() {
						public Object value() {
							progress.value_(0);

							Thread thread = new Thread() {
								public void run() {
									while (progress.value() < 1) {
										float value = Math.round((float) stream.position() / size / 0.005) * 0.005f;
										if (progress.value() != value) {
											progress.value_(value);
										}
										try {
											Thread.sleep(1000);
										} catch (InterruptedException e) {
										}
									}
								}
							};
							thread.setPriority(Thread.NORM_PRIORITY + 1);
							thread.start();

							try {
								JunOpenGLDisplayModel.this.loadFromWRL10_(stream);
							} finally {
								progress.value_(1);
							}

							return null;
						}
					});
				}
			} finally {
				stream.close();
			}
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Load a VRML2.0 file and create a JunOpenGL3dObject.
	 * 
	 * @param aFile java.io.File
	 * @exception java.io.IOException
	 */
	public void readFromWRL20_(File aFile) throws IOException {
		if (aFile == null) {
			return;
		}

		JunCursors cursor = new JunCursors(JunCursors.ReadCursor());
		try {
			cursor._show();

			final long size = aFile.length();
			final StReadStream stream = new StReadStream(aFile);
			try {
				if (size <= 100 * 1000) {
					this.loadFromWRL20_(stream);
				} else {
					// Show the progress.
					final JunProgress progress = new JunProgress();
					progress.message_("reading WRL...");
					progress.do_(new StBlockClosure() {
						public Object value() {
							progress.value_(0);

							Thread thread = new Thread() {
								public void run() {
									while (progress.value() < 1) {
										float value = Math.round((float) stream.position() / size / 0.005) * 0.005f;
										if (progress.value() != value) {
											progress.value_(value);
										}
										try {
											Thread.sleep(1000);
										} catch (InterruptedException e) {
										}
									}
								}
							};
							thread.setPriority(Thread.NORM_PRIORITY + 1);
							thread.start();

							try {
								JunOpenGLDisplayModel.this.loadFromWRL20_(stream);
							} finally {
								progress.value_(1);
							}

							return null;
						}
					});
				}
			} finally {
				stream.close();
			}
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Load a VRML97 file and create a JunOpenGL3dObject.
	 * 
	 * @param aFile java.io.File
	 * @throws java.io.IOException
	 */
	public void readFromWRL97_(File aFile) throws IOException {
		if (aFile == null) {
			return;
		}

		JunCursors cursor = new JunCursors(JunCursors.ReadCursor());
		try {
			cursor._show();

			final long size = aFile.length();
			final StReadStream stream = new StReadStream(aFile);
			try {
				if (size <= 100 * 1000) {
					this.loadFromWRL97_(stream);
				} else {
					// Show the progress.
					final JunProgress progress = new JunProgress();
					progress.message_("reading WRL...");
					progress.do_(new StBlockClosure() {
						public Object value() {
							progress.value_(0);

							Thread thread = new Thread() {
								public void run() {
									while (progress.value() < 1) {
										float value = Math.round((float) stream.position() / size / 0.005) * 0.005f;
										if (progress.value() != value) {
											progress.value_(value);
										}
										try {
											Thread.sleep(1000);
										} catch (InterruptedException e) {
										}
									}
								}
							};
							thread.setPriority(Thread.NORM_PRIORITY + 1);
							thread.start();

							try {
								JunOpenGLDisplayModel.this.loadFromWRL97_(stream);
							} finally {
								progress.value_(1);
							}

							return null;
						}
					});
				}
			} finally {
				stream.close();
			}
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Render the display objects on the redering context.
	 * 
	 * @param renderingContext sra.jun.opengl.objects.JunOpenGLRenderingContext
	 */
	public void renderOn_(JunOpenGLRenderingContext renderingContext) {
		renderingContext.clear();

		JunOpenGL3dObject renderingObject = this.displayObject();

		if (renderingObject == null) {
			this.displayProjector().projectOn_(renderingContext);
		} else {
			JunOpenGLLight[] lights = this.displayLightCollection();

			if (this.selectedObjects().isEmpty()) {
				displayProjector().project_withLights_on_(renderingObject, lights, renderingContext);
			} else {
				JunOpenGLProjector aProjector = (JunOpenGLProjector) this.displayProjector().copy();

				if (this.displayProjector().presentation() == $("solidPresentation")) {
					aProjector.wireframePresentation();
				}

				if (this.displayProjector().presentation() == $("wireframePresentation")) {
					aProjector.wireframePresentation();
				}

				if (this.displayProjector().presentation() == $("hiddenlinePresentation")) {
					aProjector.hiddenlinePresentation();
				}

				aProjector.project_withLights_on_(renderingObject, lights, renderingContext);

				if (this.displayProjector().presentation() == $("solidPresentation")) {
					aProjector.solidPresentation();
				}

				if (this.displayProjector().presentation() == $("wireframePresentation")) {
					aProjector.solidPresentation();
				}

				if (this.displayProjector().presentation() == $("hiddenlinePresentation")) {
					aProjector.solidPresentation();
				}

				for (int i = 0; i < selectedObjects.size(); i++) {
					JunOpenGL3dObject each = (JunOpenGL3dObject) selectedObjects.elementAt(i);
					aProjector.project_on_(each, renderingContext);
				}
			}
		}

		renderingContext.flush();
	}

	/**
	 * Menu message: Save the view as an image.
	 */
	public void saveAsImage() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType("Image files", new String[] { ".jpg", ".JPG" })
		};
		File file = JunFileDialog.RequestNewFilename("Input a JPG file.", new File(this.defaultBaseName() + ".jpg"), fileTypes, fileTypes[0]);
		if (file == null) {
			return;
		}

		JunOpenGLDisplayView aView = (JunOpenGLDisplayView) this.getView();
		Dimension extent = (aView == null) ? this.defaultImageExtent() : aView.getOpenGLView().toComponent().getSize();
		Image image = this.asImage(extent);

		JunImageStream stream = null;
		JunCursors cursor = new JunCursors(JunCursors.WriteCursor());
		try {
			cursor._show();

			stream = JunJpegImageStream.On_(new FileOutputStream(file));
			stream.nextPutImage_(new StImage(image));
		} catch (IOException e) {
		} finally {
			try {
				if (stream != null) {
					stream.flush();
					stream.close();
				}
			} catch (IOException e) {
			}

			cursor._restore();
		}
	}

	/**
	 * Menu message: Save the display object to an LST file.
	 */
	public void saveLST() {
		this.saveLST10();
	}

	/**
	 * Menu message: Save the display object to an LST1.0 file.
	 * 
	 * @throws sra.smalltalk.SmalltalkException
	 */
	public void saveLST10() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType("LST files", new String[] { ".lst", ".LST" })
		};
		File file = JunFileDialog.RequestNewFilename("Input an LST file.", new File(this.displayObject().name() + ".lst"), fileTypes, fileTypes[0]);
		if (file == null) {
			return;
		}

		try {
			this.writeToLST10_object_(file, this.displayObject());
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Write the JunOpenGL3dObject to the writer as LST1.0.
	 * 
	 * @param aWriter java.io.BufferedWriter
	 * @param anObject sra.jun.opengl.flux.JunOpenGL3dObject
	 * 
	 * @exception IOException
	 */
	public void saveToLST10_object_(BufferedWriter aWriter, JunOpenGL3dObject anObject) throws IOException {
		JunCursors cursor = new JunCursors(JunCursors.WriteCursor());
		try {
			cursor._show();

			aWriter.write(this.defaultStampForLST10());
			anObject.saveOn_(aWriter);
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Write the JunOpenGL3dObject to the writer as VRML1.0.
	 * 
	 * @param aWriter java.io.BufferedWriter
	 * @param anObject sra.jun.opengl.flux.JunOpenGL3dObject
	 * 
	 * @exception IOException
	 */
	public void saveToWRL10_object_(BufferedWriter aWriter, JunOpenGL3dObject anObject) throws IOException {
		JunCursors cursor = new JunCursors(JunCursors.WriteCursor());
		try {
			cursor._show();

			aWriter.write(this.defaultStampForWRL10());
			anObject.vrml10On_(aWriter);
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Write the JunOpenGL3dObject to the writer as VRML2.0.
	 * 
	 * @param aWriter java.io.BufferedWriter
	 * @param anObject sra.jun.opengl.flux.JunOpenGL3dObject
	 * 
	 * @exception IOException
	 */
	public void saveToWRL20_object_(BufferedWriter aWriter, JunOpenGL3dObject anObject) throws IOException {
		JunCursors cursor = new JunCursors(JunCursors.WriteCursor());
		try {
			cursor._show();

			double size = anObject.boundingBox().width();
			size = Math.max(size, anObject.boundingBox().height());
			size = Math.max(size, anObject.boundingBox().depth());
			aWriter.write(this.defaultStampForWRL20());
			aWriter.write("Viewpoint {");
			aWriter.newLine();
			aWriter.write("\t");
			aWriter.write("position 0.0 0.0 ");
			aWriter.write(String.valueOf((float) size * 2));
			aWriter.newLine();
			aWriter.write("}");
			aWriter.newLine();
			aWriter.newLine();
			aWriter.write("NavigationInfo {");
			aWriter.newLine();
			aWriter.write("\t");
			aWriter.write("speed ");
			aWriter.write(String.valueOf((float) size / 10));
			aWriter.newLine();
			aWriter.write("}");
			aWriter.newLine();
			aWriter.newLine();
			anObject.vrml20On_(aWriter);
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Write the display object to the writer as VRML97.
	 * 
	 * @param aWriter java.io.BufferedWriter
	 * @exception IOException
	 */
	public void saveToWRL97_(BufferedWriter aWriter) throws IOException {
		JunCursors cursor = new JunCursors(JunCursors.WriteCursor());
		try {
			cursor._show();
			this.vrml97On_(aWriter);
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Mene message: Save the display object to the VRML1.0 file.
	 * 
	 * @throws sra.smalltalk.SmalltalkException
	 */
	public void saveWRL10() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType("VRML files", new String[] { ".wrl", ".WRL" })
		};
		File file = JunFileDialog.RequestNewFilename("Input a VRML file.", new File(this.displayObject().name() + ".wrl"), fileTypes, fileTypes[0]);
		if (file == null) {
			return;
		}

		try {
			this.writeToWRL10_object_(file, this.displayObject());
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Save the display object to the VRML2.0 file.
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public void saveWRL20() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType("VRML files", new String[] { ".wrl", ".WRL" })
		};
		File file = JunFileDialog.RequestNewFilename("Input a VRML file.", new File(this.displayObject().name() + ".wrl"), fileTypes, fileTypes[0]);
		if (file == null) {
			return;
		}

		try {
			this.writeToWRL20_object_(file, this.displayObject());
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Save the display object to the VRML97 file.
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public void saveWRL97() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType("VRML files", new String[] { ".wrl", ".WRL" })
		};
		File file = JunFileDialog.RequestNewFilename("Input a VRML file.", new File(this.displayObject().name() + ".wrl"), fileTypes, fileTypes[0]);
		if (file == null) {
			return;
		}

		try {
			this.writeToWRL97_(file);
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Select all objects.
	 */
	public void selectAll() {
		if (this.openGL3dObject() == null) {
			return;
		}

		if (this.openGL3dObject().isPrimitive()) {
			return;
		}

		this.clearSelectedObjects();

		if (this.openGL3dObject().isCompound()) {
			JunOpenGL3dObject[] objects = ((JunOpenGL3dCompoundObject) this.openGL3dObject()).components();
			for (int i = 0; i < objects.length; i++) {
				this.selectedObjects.add(objects[i]);
			}
		} else {
			this.selectedObjects().add(this.openGL3dObject());
		}

		this.changed_($("selection"));
		this.updateMenuIndication();
	}

	/**
	 * Menu message: Show the bounding box.
	 */
	public void showBounds() {
		if (this.displayObject() == null) {
			return;
		}

		JunOpenGL3dObject spawningObject = this.spawningObject();
		JunOpenGL3dObject boundingObject = this.boundingObjectFor_(spawningObject);
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject();
		compoundObject.add_(boundingObject);
		compoundObject.add_(spawningObject);

		JunOpenGLDisplayModel displayModel = new JunOpenGLDisplayModel();
		displayModel.displayObject_(compoundObject);
		displayModel.displayProjection_((JunOpenGLProjection) this.displayProjection().copy());
		displayModel.open();
		displayModel.changed_($("object"));
	}

	/**
	 * Answer my show model.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLShowModel
	 */
	public JunOpenGLShowModel showModel() {
		Object[] dependents = this.dependents();
		for (int i = 0; i < dependents.length; i++) {
			if (dependents[i] instanceof JunOpenGLShowModel) {
				return (JunOpenGLShowModel) dependents[i];
			}
		}

		return null;
	}

	/**
	 * Answer the spawning object.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject spawningObject() {
		JunOpenGL3dObject spawningObject;

		if ((this.selectedObjects() == null) || (this.selectedObjects().size() == 0)) {
			if (this.displayObject() != null) {
				spawningObject = (JunOpenGL3dObject) this.displayObject().copy();
			} else {
				spawningObject = null;
			}
		} else {
			Object[] objects = this.selectedObjects().toArray();
			if (objects.length > 1) {
				spawningObject = new JunOpenGL3dCompoundObject();
				for (int i = 0; i < objects.length; i++) {
					((JunOpenGL3dCompoundObject) spawningObject).add_((JunOpenGL3dObject) objects[i]);
				}
			} else {
				spawningObject = (JunOpenGL3dObject) ((JunOpenGL3dObject) objects[0]).copy();
			}
		}

		return spawningObject;
	}

	/**
	 * Menu message: Spawn the display object.
	 */
	public void spawnObject() {
		try {
			JunOpenGLDisplayModel displayModel = (JunOpenGLDisplayModel) this.getClass().newInstance();
			displayModel.displayObject_(this.spawningObject());
			displayModel.displayProjection_((JunOpenGLProjection) this.displayProjection().copy());
			displayModel.open();
			displayModel.changed_($("object"));
		} catch (InstantiationException e) {
		} catch (IllegalAccessException e) {
		}
	}

	/**
	 * Menu message: Spawn the viewport.
	 */
	public void spawnViewport() {
		JunOpenGLShowModel model = this.showModel();
		if (model == null) {
			model = JunOpenGLShowModel.DisplayModel_(this);
		}

		if (model.builder().windows().length > 0) {
			model._windowExpandAndRaise();
		} else {
			StView view = this.getView();
			if (view == null) {
				model.open();
			} else {
				StRectangle box = new StRectangle(view.topComponent().getBounds());
				StRectangle area = new StRectangle(0, 0, box.width(), box.height());
				area = area.align_with_(area.topLeft(), new Point(box.right() + 5, box.top()));
				model.openIn_(area.toRectangle());
			}
		}
	}

	/**
	 * Menu message: Toggle a texture clamp effect on a projector.
	 */
	public void textureClamp() {
		JunOpenGL3dObject displayObject = this.displayObject();
		if (displayObject == null) {
			return;
		}

		final Boolean aBoolean = new Boolean(!this.clamp());
		JunOpenGLTexture aTexture = displayObject.texture();
		if (aTexture != null) {
			aTexture.clamp_(aBoolean.booleanValue());
			displayObject.texture_(aTexture);
		}

		if (displayObject.isCompound()) {
			displayObject.primitivesDo_(new StBlockClosure() {
				public Object value_(Object anObject) {
					JunOpenGL3dObject each = (JunOpenGL3dObject) anObject;
					JunOpenGLTexture texture = each.texture();
					if (texture != null) {
						texture.clamp_(aBoolean.booleanValue());
						each.texture_(texture);
					}
					return null;
				}
			});
		}

		this.updateMiscMenuIndication();
		this.changed_($("texture"));
	}

	/**
	 * Get an image from a file and set it as a texture.
	 */
	public void textureFromFile() {
		if (this.displayObject() == null) {
			return;
		}

		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType("Image files", new String[] { ".jpg", ".JPG", ".gif", ".GIF" }), JunFileModel.FileType.All("All files")
		};
		File file = JunFileDialog.Request("Select an Image file.", fileTypes, fileTypes[0]);
		if (file == null) {
			return;
		}

		StImage anImage = JunImageProcessor.ImageFromFile_(file.getPath());
		if (anImage == null) {
			StDialog.Warn_(file + " is not a kind of image file");
			return;
		}

		JunOpenGLTexture aTexture = JunOpenGLTexture.Image_(anImage);
		if (this.hasTextureMapping()) {
			aTexture.linear_(this.linear());
			aTexture.clamp_(this.clamp());
			aTexture.modulate_(this.modulate());
			aTexture.mipmap_(this.mipmap());
		}

		if (this.selectedObjects().isEmpty()) {
			this.displayObject().texture_(aTexture);
		} else {
			Object[] objects = this.selectedObjects().toArray();
			for (int i = 0; i < objects.length; i++) {
				((JunOpenGL3dObject) objects[i]).texture_(aTexture);
			}
		}

		this.updateMiscMenuIndication();
		this.changed_($("texture"));
	}

	/**
	 * Menu message: Get an image from a user and set it as a texture.
	 */
	public void textureFromUser() {
		if (this.displayObject() == null) {
			return;
		}

		StImage anImage = StImage._FromUser();
		if (anImage == null) {
			return;
		}

		JunOpenGLTexture aTexture = JunOpenGLTexture.Image_(anImage);
		if (this.hasTextureMapping()) {
			aTexture.linear_(this.linear());
			aTexture.clamp_(this.clamp());
			aTexture.modulate_(this.modulate());
			aTexture.mipmap_(this.mipmap());
		}

		if (this.selectedObjects().isEmpty()) {
			this.displayObject().texture_(aTexture);
		} else {
			Object[] objects = this.selectedObjects().toArray();
			for (int i = 0; i < objects.length; i++) {
				((JunOpenGL3dObject) objects[i]).texture_(aTexture);
			}
		}

		this.updateMiscMenuIndication();
		this.changed_($("texture"));
	}

	/**
	 * Menu message: Toggle a texture linear effect on a projector.
	 */
	public void textureLinear() {
		JunOpenGL3dObject displayObject = this.displayObject();
		if (displayObject == null) {
			return;
		}

		final Boolean aBoolean = new Boolean(!this.linear());

		JunOpenGLTexture aTexture = displayObject.texture();
		if (aTexture != null) {
			aTexture.linear_(aBoolean.booleanValue());
			displayObject.texture_(aTexture);
		}

		if (displayObject.isCompound()) {
			displayObject.primitivesDo_(new StBlockClosure() {
				public Object value_(Object anObject) {
					JunOpenGL3dObject each = (JunOpenGL3dObject) anObject;
					JunOpenGLTexture texture = each.texture();
					if (texture != null) {
						texture.linear_(aBoolean.booleanValue());
						each.texture_(texture);
					}
					return null;
				}
			});
		}

		this.updateMiscMenuIndication();
		this.changed_($("texture"));
	}

	/**
	 * Menu message: Toggle a texture mipmap effect on a projector.
	 */
	public void textureMipmap() {
		JunOpenGL3dObject displayObject = this.displayObject();
		if (displayObject == null) {
			return;
		}

		final Boolean aBoolean = new Boolean(!this.mipmap());

		JunOpenGLTexture aTexture = displayObject.texture();
		if (aTexture != null) {
			aTexture.mipmap_(aBoolean.booleanValue());
			displayObject.texture_(aTexture);
		}

		if (displayObject.isCompound()) {
			displayObject.primitivesDo_(new StBlockClosure() {
				public Object value_(Object anObject) {
					JunOpenGL3dObject each = (JunOpenGL3dObject) anObject;
					JunOpenGLTexture texture = each.texture();
					if (texture != null) {
						texture.mipmap_(aBoolean.booleanValue());
						each.texture_(texture);
					}
					return null;
				}
			});
		}

		this.updateMiscMenuIndication();
		this.changed_($("texture"));
	}

	/**
	 * Menu message: Toggle a texture modulate effect on a projector.
	 */
	public void textureModulate() {
		JunOpenGL3dObject displayObject = this.displayObject();
		if (displayObject == null) {
			return;
		}

		final Boolean aBoolean = new Boolean(!this.modulate());

		JunOpenGLTexture aTexture = displayObject.texture();
		if (aTexture != null) {
			aTexture.modulate_(aBoolean.booleanValue());
			displayObject.texture_(aTexture);
		}

		if (displayObject.isCompound()) {
			displayObject.primitivesDo_(new StBlockClosure() {
				public Object value_(Object anObject) {
					JunOpenGL3dObject each = (JunOpenGL3dObject) anObject;
					JunOpenGLTexture texture = each.texture();
					if (texture != null) {
						texture.modulate_(aBoolean.booleanValue());
						each.texture_(texture);
					}
					return null;
				}
			});
		}

		this.updateMiscMenuIndication();
		this.changed_($("texture"));
	}

	/**
	 * Menu message: Take a texture away if exists.
	 */
	public void textureTakeAway() {
		if (this.displayObject() == null) {
			return;
		}

		if (this.selectedObjects().isEmpty()) {
			this.displayObject().texture_(null);
		} else {
			Object[] objects = this.selectedObjects().toArray();
			for (int i = 0; i < objects.length; i++) {
				((JunOpenGL3dObject) objects[i]).texture_(null);
			}
		}

		this.updateMiscMenuIndication();
		this.changed_($("texture"));
	}

	/**
	 * Update the edit menu indication.
	 */
	public void updateEditMenuIndication() {
		StMenu editMenu = this._editMenu();
		if (editMenu == null) {
			return;
		}

		StMenuItem menuItem;
		boolean displayObjectIsNotEmpty = !this.isEmpty();

		menuItem = editMenu.getItemWithValue("copyObject");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}

		menuItem = editMenu.getItemWithValue("cutObject");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}

		menuItem = editMenu.getItemWithValue("clearObject");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}

		menuItem = editMenu.getItemWithValue("selectAll");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}

	}

	/**
	 * Update the file menu indication.
	 */
	public void updateFileMenuIndication() {
		StMenu fileMenu = this._fileMenu();
		if (fileMenu == null) {
			return;
		}

		StMenuItem menuItem;
		boolean displayObjectIsNotEmpty = !this.isEmpty();

		// Save...
		menuItem = fileMenu.getItemWithValue("saveLST");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}

		// Save as...
		menuItem = fileMenu.getItem("Save as...");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}

		// Save as image
		menuItem = fileMenu.getItemWithValue("saveAsImage");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}
	}

	/**
	 * Update the menu indications.
	 */
	public void updateMenuIndication() {
		super.updateMenuIndication();

		this.updateFileMenuIndication();
		this.updateEditMenuIndication();
		this.updateMiscMenuIndication();
	}

	/**
	 * Update the misc menu indication.
	 */
	public void updateMiscMenuIndication() {
		StMenu miscMenu = this._miscMenu();
		if (miscMenu == null) {
			return;
		}

		StMenuItem menuItem;
		boolean displayObjectIsNotEmpty = !this.isEmpty();

		// Texture
		StMenu textureMenu = (StMenu) miscMenu.getItem("Texture");
		if (textureMenu != null) {
			textureMenu.setEnabled(displayObjectIsNotEmpty);
			if (displayObjectIsNotEmpty) {
				StCheckBoxMenuItem checkBoxMenuItem;

				// Mipmap
				checkBoxMenuItem = (StCheckBoxMenuItem) textureMenu.getItemWithValue("textureMipmap");
				if (checkBoxMenuItem != null) {
					checkBoxMenuItem.setSelected(this.mipmap());
				}

				// Modulate
				checkBoxMenuItem = (StCheckBoxMenuItem) textureMenu.getItemWithValue("textureModulate");
				if (checkBoxMenuItem != null) {
					checkBoxMenuItem.setSelected(this.modulate());
				}

				// Clamp
				checkBoxMenuItem = (StCheckBoxMenuItem) textureMenu.getItemWithValue("textureClamp");
				if (checkBoxMenuItem != null) {
					checkBoxMenuItem.setSelected(this.clamp());
				}

				// Linear
				checkBoxMenuItem = (StCheckBoxMenuItem) textureMenu.getItemWithValue("textureLinear");
				if (checkBoxMenuItem != null) {
					checkBoxMenuItem.setSelected(this.linear());
				}
			}
		}

		// Spawn
		menuItem = miscMenu.getItemWithValue("spawnObject");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}

		// Viewport
		menuItem = miscMenu.getItemWithValue("spawnViewport");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}

		// Bounds
		menuItem = miscMenu.getItemWithValue("showBounds");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}
	}

	/**
	 * Invoked when a window is in the process of being closed.
	 * 
	 * @param e java.awt.event.WindowEvent
	 * @see sra.smalltalk.StApplicationModel#noticeOfWindowClose()
	 * @category interface closing
	 */
	public void noticeOfWindowClose(WindowEvent e) {
		JunOpenGLDisplayLight[] lights = this.displayLights();
		for (int i = 0; i < lights.length; i++) {
			lights[i].closeRequest();
		}

		if (this.showModel() != null) {
			this.showModel().closeRequest();
		}

		super.noticeOfWindowClose(e);
	}

	/**
	 * Write the JunOpenGL3dObject to the LST1.0 file.
	 * 
	 * @param aFile java.io.File
	 * @param anObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @exception java.io.IOException
	 */
	public void writeToLST10_object_(File aFile, JunOpenGL3dObject anObject) throws IOException {
		if (aFile == null) {
			return;
		}

		BufferedWriter aWriter = new BufferedWriter(new FileWriter(aFile));
		try {
			this.saveToLST10_object_(aWriter, anObject);
		} finally {
			aWriter.flush();
			aWriter.close();
		}
	}

	/**
	 * Write the JunOpenGL3dObject to the VRML1.0 file.
	 * 
	 * @param aFile java.io.File
	 * @param anObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @exception java.io.IOException
	 */
	public void writeToWRL10_object_(File aFile, JunOpenGL3dObject anObject) throws IOException {
		if (aFile == null) {
			return;
		}

		BufferedWriter aWriter = new BufferedWriter(new FileWriter(aFile));
		try {
			this.saveToWRL10_object_(aWriter, anObject);
		} finally {
			aWriter.flush();
			aWriter.close();
		}
	}

	/**
	 * Write the JunOpenGL3dObject to the VRML2.0 file.
	 * 
	 * @param aFile java.io.File
	 * @param anObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @exception java.io.IOException
	 */
	public void writeToWRL20_object_(File aFile, JunOpenGL3dObject anObject) throws IOException {
		if (aFile == null) {
			return;
		}

		BufferedWriter aWriter = new BufferedWriter(new FileWriter(aFile));
		try {
			this.saveToWRL20_object_(aWriter, anObject);
		} finally {
			aWriter.flush();
			aWriter.close();
		}
	}

	/**
	 * Write the display object as VRML97 to the file.
	 * 
	 * @param aFile java.io.File
	 * @exception java.io.IOException
	 */
	public void writeToWRL97_(File aFile) throws IOException {
		if (aFile == null) {
			return;
		}

		BufferedWriter aWriter = new BufferedWriter(new FileWriter(aFile));
		try {
			this.saveToWRL97_(aWriter);
		} finally {
			aWriter.flush();
			aWriter.close();
		}
	}

	/**
	 * Answer a xThumbWheel.
	 * 
	 * @return sra.jun.goodies.wheels.JunThumbWheel
	 */
	public JunThumbWheel xThumbWheel() {
		JunThumbWheel thumbWheel = new JunThumbWheel();
		thumbWheel.compute_(new StBlockClosure() {
			public Object value_(Object value) {
				Jun2dPoint factor = new Jun2dPoint(0, - ((Number) value).doubleValue() / 50);
				grab_(factor);
				return null;
			}
		});

		return thumbWheel;
	}

	/**
	 * Answer a yThumbWheel.
	 * 
	 * @return sra.jun.goodies.wheels.JunThumbWheel
	 */
	public JunThumbWheel yThumbWheel() {
		JunThumbWheel thumbWheel = new JunThumbWheel();
		thumbWheel.compute_(new StBlockClosure() {
			public Object value_(Object value) {
				Jun2dPoint factor = new Jun2dPoint(((Number) value).doubleValue() / 50, 0);
				grab_(factor);
				return null;
			}
		});

		return thumbWheel;
	}

	/**
	 * Answer a zThumbWheel.
	 * 
	 * @return sra.jun.goodies.wheels.JunThumbWheel
	 */
	public JunThumbWheel zThumbWheel() {
		JunThumbWheel thumbWheel = new JunThumbWheel();
		thumbWheel.compute_(new StBlockClosure() {
			public Object value_(Object value) {
				if (dollyButton().value() == true) {
					double factor = ((Number) value).doubleValue() / 400;
					dolly_(factor);
				} else {
					double factor = 1 + (((Number) value).doubleValue() / 100);
					zoom_(factor);
				}
				return null;
			}
		});

		return thumbWheel;
	}

	/**
	 * Answer the default edit menu for JunOpenGLDisplayModel.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _defaultEditMenu() {
		StMenu defaultEditMenu = new StMenu("Edit");

		// Copy
		defaultEditMenu.add(createMenuItem("Copy", "copyObject"));

		// Cut
		defaultEditMenu.add(createMenuItem("Cut", "cutObject"));

		// Paste
		defaultEditMenu.add(createMenuItem("Paste", "pasteObject"));

		// Clear
		defaultEditMenu.add(createMenuItem("Clear", "clearObject"));

		//
		defaultEditMenu.addSeparator();

		// Select all
		defaultEditMenu.add(createMenuItem("Select all", "selectAll"));

		return defaultEditMenu;
	}

	/**
	 * Answer the default file menu for JunOpenGLDisplayModel.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _defaultFileMenu() {
		StMenu defaultFileMenu = new StMenu("File");

		// New
		defaultFileMenu.add(createMenuItem("New", "newModel"));

		// Open...
		defaultFileMenu.add(createMenuItem("Open...", "openLST"));

		// Open as...
		StMenu openAsMenu = new StMenu("Open as...");
		openAsMenu.add(createMenuItem("VRML1.0...", "openWRL10"));
		openAsMenu.add(createMenuItem("VRML2.0...", "openWRL20"));
		openAsMenu.add(createMenuItem("VRML97...", "openWRL97"));
		defaultFileMenu.add(openAsMenu);

		//
		defaultFileMenu.addSeparator();

		// Save...
		defaultFileMenu.add(createMenuItem("Save...", "saveLST"));

		// Save as...
		StMenu saveAsMenu = new StMenu("Save as...");
		saveAsMenu.add(createMenuItem("VRML1.0...", "saveWRL10"));
		saveAsMenu.add(createMenuItem("VRML2.0...", "saveWRL20"));
		saveAsMenu.add(createMenuItem("VRML97...", "saveWRL97"));
		defaultFileMenu.add(saveAsMenu);

		// Save as image...
		defaultFileMenu.add(createMenuItem("Save as image...", "saveAsImage"));

		//
		defaultFileMenu.addSeparator();

		// Quit
		defaultFileMenu.add(createMenuItem("Quit", "quitDoing"));

		return defaultFileMenu;
	}

	/**
	 * Answer the default misc menu for JunOpenGLDisplayModel.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _defaultMiscMenu() {
		StMenu defaultMiscMenu = new StMenu("Misc");

		// Texture
		StMenu textureMenu = new StMenu("Texture");
		textureMenu.add(createMenuItem("From user...", "textureFromUser"));
		textureMenu.add(createMenuItem("From file...", "textureFromFile"));
		textureMenu.add(createMenuItem("Take away", "textureTakeAway"));
		textureMenu.addSeparator();
		textureMenu.add(createCheckBoxMenuItem("Linear approx.", "textureLinear"));
		textureMenu.add(createCheckBoxMenuItem("Clamp", "textureClamp"));
		textureMenu.add(createCheckBoxMenuItem("Modulate", "textureModulate"));
		textureMenu.add(createCheckBoxMenuItem("Mipmap", "textureMipmap"));
		defaultMiscMenu.add(textureMenu);

		// Spawn
		defaultMiscMenu.add(createMenuItem("Spawn", "spawnObject"));

		// Viewport
		defaultMiscMenu.add(createMenuItem("Viewport", "spawnViewport"));

		// Bounds
		defaultMiscMenu.add(createMenuItem("Bounds", "showBounds"));

		return defaultMiscMenu;
	}

	/**
	 * Answer my edit menu.
	 *
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _editMenu() {
		if (_editMenu == null) {
			_editMenu = this._defaultEditMenu();
		}
		return _editMenu;
	}

	/**
	 * Answer my file menu.
	 *
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _fileMenu() {
		if (_fileMenu == null) {
			_fileMenu = this._defaultFileMenu();
		}
		return _fileMenu;
	}

	/**
	 * Answer my misc menu.
	 *
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _miscMenu() {
		if (_miscMenu == null) {
			_miscMenu = this._defaultMiscMenu();
		}
		return _miscMenu;
	}

	/**
	 * Process the JunOpenGL3dObject picked with the alt key down.
	 *
	 * @param pickedObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected void altClick_(JunOpenGL3dObject pickedObject) {
		// Nothing to do as a default.
	}

	/**
	 * Compute the current sight point.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	protected Jun3dPoint computeSightPoint() {
		if (this.selectedObjects().isEmpty()) {
			return super.computeSightPoint();
		} else {
			int size = this.selectedObjects().size();
			JunOpenGL3dObject[] selectedObjects = new JunOpenGL3dObject[size];
			this.selectedObjects().copyInto(selectedObjects);

			double x = 0;
			double y = 0;
			double z = 0;

			for (int i = 0; i < size; i++) {
				Jun3dPoint center = (Jun3dPoint) selectedObjects[i].boundingBox().center();
				x += center.x();
				y += center.y();
				z += center.z();
			}

			return new Jun3dPoint(x / size, y / size, z / size);
		}
	}

	/**
	 * Compute the current zoom height.
	 * 
	 * @param vertical DOCUMENT ME!
	 * @param horizontal DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	protected Jun2dPoint computeZoomHeightMaxOffset(final JunPlane vertical, final JunPlane horizontal) {
		if (this.selectedObjects().isEmpty()) {
			return super.computeZoomHeightMaxOffset(vertical, horizontal);
		} else {
			final StValueHolder maxOffset_ = new StValueHolder(new Jun2dPoint(0, 0));
			int size = this.selectedObjects().size();
			JunOpenGL3dObject[] selectedObjects = new JunOpenGL3dObject[size];
			this.selectedObjects().copyInto(selectedObjects);

			for (int i = 0; i < size; i++) {
				selectedObjects[i].pointsDo_(new StBlockClosure() {
					public Object value_(Object value) {
						Jun3dPoint p = (Jun3dPoint) value;
						Jun2dPoint offset = new Jun2dPoint(vertical.distanceFromPoint_(p), horizontal.distanceFromPoint_(p));
						maxOffset_.value_(offset.max_((Jun2dPoint) maxOffset_.value()));

						return null;
					}
				});
			}

			return (Jun2dPoint) maxOffset_.value();
		}
	}

	/**
	 * Process the JunOpenGL3dObject picked with the control key down.
	 *
	 * @param pickedObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected void ctrlClick_(JunOpenGL3dObject pickedObject) {
		// Nothing to do as a default.
	}

	/**
	 * Create the menubar for JunOpenGLDisplay.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 */
	protected StMenuModel defaultMenu() {
		StMenuModel rootMenu = new StMenuModel();

		rootMenu.add(this._fileMenu());
		rootMenu.add(this._editMenu());
		rootMenu.add(this._viewMenu());
		rootMenu.add(this._lightMenu());
		rootMenu.add(this._miscMenu());

		return rootMenu;
	}

	/**
	 * Answer the array of the Jun3dPoint, which is a collection between 'fromValue' and 'toValue' divided into 'numberOfValues'.
	 *
	 * @return sra.jun.geometry.basic.Jun3dPoint[]
	 * @param numberOfValues int
	 * @param centerValue sra.jun.geometry.basic.Jun3dPoint
	 * @param fromValue sra.jun.geometry.basic.Jun3dPoint
	 * @param toValue sra.jun.geometry.basic.Jun3dPoint
	 */
	protected Jun3dPoint[] getCollection_centerValue_fromValue_toValue_(int numberOfValues, Jun3dPoint centerValue, Jun3dPoint fromValue, Jun3dPoint toValue) {
		int howMany = numberOfValues - 1;
		double aDistance = centerValue.distance_(fromValue);
		Jun3dLine aLine = new Jun3dLine(centerValue, toValue);
		aLine = aLine.normalizedLine();
		Jun3dPoint stepValue = aLine.atT_(aDistance).minus_(fromValue).dividedBy_(howMany);
		Jun3dPoint[] values = new Jun3dPoint[numberOfValues];
		Jun3dPoint aValue = fromValue;
		values[0] = aValue;
		for (int i = 1; i < howMany; i++) {
			aValue = aValue.plus_(stepValue);
			aLine = new Jun3dLine(centerValue, aValue);
			aLine = aLine.normalizedLine();
			values[i] = aLine.atT_(aDistance);
		}
		values[howMany] = toValue;
		return values;
	}

	/**
	 * Answer the array of the double, which is a collection between 'fromValue' and 'toValue' divided into 'numberOfValues'.
	 *
	 * @return double[]
	 * @param numberOfValues int
	 * @param fromValue sra.jun.geometry.basic.Jun3dPoint
	 * @param toValue sra.jun.geometry.basic.Jun3dPoint
	 */
	protected double[] getCollection_fromValue_toValue_(int numberOfValues, double fromValue, double toValue) {
		int howMany = numberOfValues - 1;
		double stepValue = (toValue - fromValue) / howMany;
		double[] values = new double[numberOfValues];
		double aValue = fromValue;
		values[0] = aValue;
		for (int i = 1; i < howMany; i++) {
			aValue = aValue + stepValue;
			values[i] = aValue;
		}
		values[howMany] = toValue;
		return values;
	}

	/**
	 * Answer the array of the Jun3dPoint, which is a collection between 'fromValue' and 'toValue' divided into 'numberOfValues'.
	 *
	 * @return sra.jun.geometry.basic.Jun3dPoint[]
	 * @param numberOfValues int
	 * @param fromValue sra.jun.geometry.basic.Jun3dPoint
	 * @param toValue sra.jun.geometry.basic.Jun3dPoint
	 */
	protected Jun3dPoint[] getCollection_fromValue_toValue_(int numberOfValues, Jun3dPoint fromValue, Jun3dPoint toValue) {
		int howMany = numberOfValues - 1;
		Jun3dPoint stepValue = toValue.minus_(fromValue).dividedBy_(howMany);
		Jun3dPoint[] values = new Jun3dPoint[numberOfValues];
		Jun3dPoint aValue = fromValue;
		values[0] = aValue;
		for (int i = 1; i < howMany; i++) {
			aValue = aValue.plus_(stepValue);
			values[i] = aValue;
		}
		values[howMany] = toValue;
		return values;
	}

	/**
	 * Initialize the JunOpenGLDisplayModel.
	 */
	protected void initialize() {
		super.initialize();

		pickButton = null;
		grabButton = null;
		dragButton = null;
		dollyButton = null;
		focusButton = null;
		_editMenu = null;
	}

	/**
	 * Answer the transformation for the VRML97 Viewpoint node.
	 *
	 * @return sra.jun.geometry.transformations.Jun3dTransformation
	 */
	protected Jun3dTransformation transformationForVrml97() {
		Jun3dPoint vz = this.eyePoint().minus_(this.sightPoint()).unitVector();
		Jun3dPoint vy = this.upVector();
		Jun3dPoint vx = (new Jun3dPoint(vy.y() * vz.z() - vy.z() * vz.y(), vy.z() * vz.x() - vy.x() * vz.z(), vy.x() * vz.y() - vy.y() * vz.x())).unitVector();
		double[] collection = new double[16];
		collection[0] = vx.x();
		collection[1] = vy.x();
		collection[2] = vz.x();
		collection[3] = 0;
		collection[4] = vx.y();
		collection[5] = vy.y();
		collection[6] = vz.y();
		collection[7] = 0;
		collection[8] = vx.z();
		collection[9] = vy.z();
		collection[10] = vz.z();
		collection[11] = 0;
		collection[12] = 0;
		collection[13] = 0;
		collection[14] = 0;
		collection[15] = 1;
		return Jun3dTransformation.Translate_((new Jun3dPoint(0, 0, 0)).minus_(this.sightPoint())).product_(Jun3dTransformation.FromArray_(collection));
	}

	/**
	 * Write the VRML97 NavigationInfo node to the writer.
	 *
	 * @param aWriter java.io.BufferedWriter
	 * @exception java.io.IOException
	 */
	protected void vrml97NavigationInfoOn_(BufferedWriter aWriter) throws IOException {
		aWriter.write("NavigationInfo {");
		aWriter.newLine();
		aWriter.write('\t');
		aWriter.write("type [\"EXAMINE\", \"ANY\"]");
		aWriter.newLine();
		aWriter.write('\t');
		aWriter.write("avatarSize [0.25, 1.6, 0.75");
		aWriter.write(", " + (float) this.sightPoint().x());
		aWriter.write(", " + (float) this.sightPoint().y());
		aWriter.write(", " + (float) this.sightPoint().z());
		aWriter.write(']');
		aWriter.newLine();
		aWriter.write('\t');
		aWriter.write("visibilityLimit " + (float) this.displayProjector().far());
		aWriter.newLine();
		aWriter.write('}');
		aWriter.newLine();
	}

	/**
	 * Write my VRML97 representation on the writer.
	 *
	 * @param aWriter java.io.BufferedWriter
	 * @exception java.io.IOException
	 */
	protected void vrml97On_(BufferedWriter aWriter) throws IOException {
		aWriter.write(this.defaultStampForWRL97());
		aWriter.newLine();
		this.vrml97WorldInfoOn_(aWriter);
		aWriter.newLine();
		this.vrml97ViewpointOn_(aWriter);
		aWriter.newLine();
		this.vrml97NavigationInfoOn_(aWriter);
		aWriter.newLine();
		this.displayObject().vrml20On_(aWriter);
	}

	/**
	 * Write the VRML97 Viewpoint node to the writer.
	 *
	 * @param aWriter java.io.BufferedWriter
	 * @exception java.io.IOException
	 */
	protected void vrml97ViewpointOn_(BufferedWriter aWriter) throws IOException {
		Jun3dPoint eyePoint = this.eyePoint();
		Jun3dTransformation transformation = this.transformationForVrml97();
		Jun3dPoint rotationVector = transformation.rotationVector();
		JunAngle rotationAngle = transformation.rotationAngle();
		aWriter.write("Viewpoint {");
		aWriter.newLine();
		aWriter.write('\t');
		aWriter.write("fieldOfView " + (float) Math.atan(this.zoomHeight() / this.eyePoint().distance_(this.sightPoint()) / 2) * 2);
		aWriter.newLine();
		aWriter.write('\t');
		aWriter.write("position");
		aWriter.write(" " + (float) eyePoint.x());
		aWriter.write(" " + (float) eyePoint.y());
		aWriter.write(" " + (float) eyePoint.z());
		aWriter.newLine();
		aWriter.write('\t');
		aWriter.write("orientation");
		aWriter.write(" " + (float) rotationVector.x());
		aWriter.write(" " + (float) rotationVector.y());
		aWriter.write(" " + (float) rotationVector.z());
		aWriter.write(" " + (float) rotationAngle.rad());
		aWriter.newLine();
		aWriter.write('}');
		aWriter.newLine();
	}

	/**
	 * Write the VRML97 WorldInfo node to the writer.
	 *
	 * @param aWriter java.io.BufferedWriter
	 * @exception java.io.IOException
	 */
	protected void vrml97WorldInfoOn_(BufferedWriter aWriter) throws IOException {
		aWriter.write("WorldInfo {");
		aWriter.newLine();
		aWriter.write('\t');
		aWriter.write("info [");
		aWriter.newLine();
		aWriter.write('\t');
		aWriter.write('\t');
		aWriter.write("\"This file was created by " + JunSystem.System() + JunSystem.Version() + " for Java\"");
		aWriter.newLine();
		aWriter.write('\t');
		aWriter.write('\t');
		aWriter.write("\"" + DateFormat.getInstance().format(new Date()) + "\"");
		aWriter.newLine();
		aWriter.write('\t');
		aWriter.write(']');
		aWriter.newLine();
		aWriter.write('\t');
		aWriter.write("title \"");
		if (this.displayObject() != null) {
			aWriter.write(this.displayObject().name() + ": ");
		}
		aWriter.write("Sample of " + this._className() + "\"");
		aWriter.newLine();
		aWriter.write('}');
		aWriter.newLine();
	}

	/**
	 * Answer this window title.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "Viewfinder";
	}

}
