package sra.jun.opengl.display;

import java.awt.*;
import java.util.Vector;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.*;
import sra.jun.opengl.objects.*;
import sra.jun.opengl.texture.*;
import sra.smalltalk.StBlockClosure;

/**
 * JunOpenGLDisplayModelTestExamples class
 * 
 *  @author    he
 *  @created   1998/11/09 (by he)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on Jun493 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLDisplayModelTestExamples.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public class JunOpenGLDisplayModelTestExamples extends sra.jun.system.support.JunTestExamples {

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example1() {
		JunOpenGLDisplayModel model = new JunOpenGLDisplayModel();
		JunOpenGL3dObject a3dObject = JunOpenGL3dObject.Axes();
		a3dObject = a3dObject.transform_(Jun3dTransformation.Scale_(2.0));
		model.displayObject_(a3dObject);
		model.open();

		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example2() {
		JunOpenGLDisplayModel model = new JunOpenGLDisplayModel();
		JunOpenGL3dObject a3dObject = JunOpenGL3dObject.Cube();
		a3dObject.paint_(new Color(255, 235, 215));
		model.displayObject_(a3dObject);
		model.open();

		//		model.open();
		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example3() {
		JunOpenGLDisplayModel model = new JunOpenGLDisplayModel();
		JunOpenGL3dObject a3dObject = JunOpenGL3dObject.Sphere();
		a3dObject.paint_(Color.blue);
		model.displayObject_(a3dObject);
		model.open();

		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example8() {
		JunOpenGL3dObject aC = JunOpenGL3dObject.Globe_(30);
		aC.paint_(new Color(127, 127, 255));

		JunOpenGL3dObject aH = aC.transform_(Jun3dTransformation.Scale_(0.6));
		aH = aH.transform_(Jun3dTransformation.Translate_(new Jun3dPoint(0.75, 0, 0)));
		aH.paint_(new Color(127, 255, 127));

		JunOpenGL3dObject aCH = JunOpenGL3dCompoundObject.With_with_(aH, aC);
		aCH = aCH.transform_(Jun3dTransformation.Translate_(new Jun3dPoint(0.75, 0, 0)));

		JunOpenGL3dCompoundObject displayObject = new JunOpenGL3dCompoundObject();

		for (int each = 60; each <= 360; each += 60) {
			Jun3dTransformation aT = Jun3dTransformation.Rotate_(JunAngle.FromDeg_(each));
			displayObject.add_(aCH.transform_(aT));
		}

		displayObject.name_("benzene");

		JunOpenGLDisplayModel displayModel = new JunOpenGLDisplayModel();
		displayModel.displayObject_(displayObject);
		displayModel.viewFactor_(1.2);
		displayModel.open();

		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean ExampleD() {
		StBlockClosure aBlock = new StBlockClosure() {
			public Object value_(Object anImage) {
				Object[] xyPointsAndSphere = JunOpenGL3dObject.XyPointsAndSphere_radius_longitude_latitude_(30, 1, 360, 180);
				Vector points = (Vector) xyPointsAndSphere[0];
				Jun2dPoint[] xyPoints = new Jun2dPoint[points.size()];

				for (int index = 0; index < points.size(); index++) {
					Jun2dPoint aPoint = (Jun2dPoint) points.elementAt(index);
					xyPoints[index] = new Jun2dPoint(aPoint.x(), 1 - aPoint.y());
				}

				JunOpenGL3dCompoundObject anObject = (JunOpenGL3dCompoundObject) xyPointsAndSphere[1];
				JunOpenGLTexture aTexture = JunOpenGLTexture.Image_((Image) anImage);
				aTexture.coordinates_(xyPoints);
				anObject.texture_(aTexture);
				anObject.name_("earth");
				anObject.show();

				return null;
			}
		};

		aBlock.value_(JunOpenGLTextureTestExamples.ImageEarth());
		aBlock.value_(JunOpenGLTextureTestExamples.ImageEarth1());
		aBlock.value_(JunOpenGLTextureTestExamples.ImageEarth2());
		aBlock.value_(JunOpenGLTextureTestExamples.ImageEarth3());

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 */
	public static void main(String[] args) {
		new JunOpenGLDisplayModelTestExamples();
	}
}
