package sra.jun.opengl.display;

import sra.smalltalk.*;
import sra.jun.goodies.button.JunButtonViewAwt;
import sra.jun.goodies.wheels.JunThumbWheelViewAwt;

/**
 * JunOpenGLDisplayViewAwt class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/06/02 (by MATSUDA Ryouichi)
 *  @updated   2003/01/09 (by nisinaka)
 *  @updated   2003/05/14 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun475 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLDisplayViewAwt.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public class JunOpenGLDisplayViewAwt extends StViewPanel implements JunOpenGLDisplayView {

	protected JunOpenGL3dViewAwt openGLView;

	/**
	 * Create a new instance of JunOpenGLDisplayViewAwt and initialize it.
	 * 
	 * @param junOpenGLDisplayModel
	 */
	public JunOpenGLDisplayViewAwt(JunOpenGLDisplayModel junOpenGLDisplayModel) {
		super(junOpenGLDisplayModel);
	}

	/**
	 * Answer my model as JunOpenGLDisplayModel.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 * @deprecated since Jun379, use getOpenGLDisplayModel().
	 */
	public JunOpenGLDisplayModel getModel() {
		return (JunOpenGLDisplayModel) this.model();
	}

	/**
	 * Answer my model as JunOpenGLDisplayModel.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public JunOpenGLDisplayModel getOpenGLDisplayModel() {
		return (JunOpenGLDisplayModel) this.model();
	}

	/**
	 * Answer my current OpenGL view.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayView
	 */
	public JunOpenGL3dView getOpenGLView() {
		return this.openGLView();
	}

	/**
	 * Answer my current OpenGL view.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGL3dViewAwt
	 */
	public JunOpenGL3dViewAwt openGLView() {
		if (openGLView == null) {
			openGLView = new JunOpenGL3dViewAwt(this.getOpenGLDisplayModel());
		}

		return openGLView;
	}

	/**
	 * Make sure to stop the rotation thread.
	 */
	public void removeNotify() {
		((JunOpenGLDisplayController) this.controller()).flushMovementVector();
		super.removeNotify();
	}

	/**
	 * model property accessing.
	 * 
	 * @param newModel sra.jun.opengl.display.JunOpenGLDisplayModel
	 * @deprecated
	 */
	public void setModel(JunOpenGLDisplayModel newModel) {
		this.model_(newModel);
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.openGLView()._onBorderedPanel(), WindowSpecLayout.Position(0f, 20, 0f, 1, 1f, -20, 1f, -20));
		this.add(new JunButtonViewAwt(this.getOpenGLDisplayModel().pickButton()), WindowSpecLayout.Position(0f, 1, 0f, 1, 0f, 19, 0f, 19));
		this.add(new JunButtonViewAwt(this.getOpenGLDisplayModel().grabButton()), WindowSpecLayout.Position(0f, 1, 0f, 20, 0f, 19, 0f, 38));
		this.add(new JunButtonViewAwt(this.getOpenGLDisplayModel().dragButton()), WindowSpecLayout.Position(0f, 1, 0f, 39, 0f, 19, 0f, 57));
		this.add(new JunThumbWheelViewAwt(this.getOpenGLDisplayModel().xThumbWheel(), true), WindowSpecLayout.Position(0f, 1, 0.5f, -61, 0f, 19, 0.5f, 61));
		this.add(new JunThumbWheelViewAwt(this.getOpenGLDisplayModel().yThumbWheel(), false), WindowSpecLayout.Position(0.5f, -61, 1f, -19, 0.5f, 61, 1f, -1));
		this.add(new JunThumbWheelViewAwt(this.getOpenGLDisplayModel().zThumbWheel(), true), WindowSpecLayout.Position(1f, -19, 0.5f, -61, 1f, -1, 0.5f, 61));
		this.add(new JunButtonViewAwt(this.getOpenGLDisplayModel().focusButton()), WindowSpecLayout.Position(1f, -19, 0.5f, -80, 1f, -1, 0.5f, -62));
		this.add(new JunButtonViewAwt(this.getOpenGLDisplayModel().dollyButton()), WindowSpecLayout.Position(1f, -19, 0.5f, 62, 1f, -1, 0.5f, 80));
		this.setSize(300, 280);
	}

	/**
	 * Answer a default controller.
	 * 
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new JunOpenGLDisplayController();
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunOpenGLDisplayModel();
	}
}
