package sra.jun.opengl.display;

import sra.smalltalk.*;
import sra.jun.opengl.objects.JunOpenGL3dObject;
import sra.jun.opengl.projection.JunOpenGLProjection;

/**
 * JunOpenGLEmbeddedDisplayModel class
 * 
 *  @author    nisinaka
 *  @created   1998/12/17 (by nisinaka)
 *  @updated   1999/06/25 (by nisinaka)
 *  @updated   2003/01/08 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLEmbeddedDisplayModel.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public class JunOpenGLEmbeddedDisplayModel extends JunOpenGLDisplayModel {
	/**
	 * The flag to determine whether the receiver is needed to be fit when a
	 * window is resized.
	 */
	protected transient boolean needsToFit = true;

	/**
	 * Create a new instance of JunOpenGLEmbeddedDisplayModel and initialize it.
	 */
	public JunOpenGLEmbeddedDisplayModel() {
		super();
	}

	/**
	 * Create a new instance of JunOpenGLDisplayModel and set the JunOpenGL3dObject.
	 *
	 * @param anOpenGL3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGLEmbeddedDisplayModel(JunOpenGL3dObject anOpenGL3dObject) {
		super(anOpenGL3dObject);
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunOpenGLEmbeddedDisplayViewAwt(this);
		} else {
			return new JunOpenGLEmbeddedDisplayViewSwing(this);
		}
	}

	/**
	 * Set the JunOpenGL3dObject to be shown on the receiver.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public void displayObject_(JunOpenGL3dObject a3dObject) {
		this.needsToFit_(true);
		this.setDisplayObject_(a3dObject);
	}

	/**
	 * Answer true if the receiver is needed to be fit when a window is
	 * resized.
	 * 
	 * @return boolean
	 */
	public boolean needsToFit() {
		return needsToFit;
	}

	/**
	 * Set the flag to determine whether the receiver is needed to be fit when
	 * a window is resized.
	 * 
	 * @param aBoolean boolean
	 */
	public void needsToFit_(boolean aBoolean) {
		needsToFit = aBoolean;
	}

	/**
	 * Set the JunOpenGL3dObject to be shown on the receiver.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public void setDisplayObject_(JunOpenGL3dObject a3dObject) {
		if (this.displayObject() == null) {
			this.flushProjector();
		}

		super.displayObject_(a3dObject);

		if (this.needsToFit()) {
			this.fitSilently();
			this.needsToFit_(false);
		}

		this.changed_($("object"));
	}

	/**
	 * Spawn the display model and show the object.
	 */
	public void spawnObject() {
		JunOpenGLDisplayModel displayModel;
		displayModel = DisplayObject_(this.getClass().getSuperclass(), this.spawningObject());
		displayModel.displayProjection_((JunOpenGLProjection) this.displayProjection().copy());
		displayModel.open();
		displayModel.changed_($("object"));
	}

	/**
	 * Answer the default file menu for JunOpenGLEmbeddedDisplayModel.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _defaultFileMenu() {
		StMenu defaultFileMenu = new StMenu("File");

		// Save...
		defaultFileMenu.add(createMenuItem("Save...", "saveLST"));

		// Save as...
		StMenu saveAsMenu = new StMenu("Save as...");
		saveAsMenu.add(createMenuItem("VRML1.0...", "saveWRL10"));
		saveAsMenu.add(createMenuItem("VRML2.0...", "saveWRL97"));
		saveAsMenu.add(createMenuItem("VRML97...", "saveWRL97"));
		defaultFileMenu.add(saveAsMenu);

		// Save as image...
		defaultFileMenu.add(createMenuItem("Save as image...", "saveAsImage"));

		return defaultFileMenu;
	}

	/**
	 * Create the popup menu for JunOpenGLEmbeddedDisplayModel.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 */
	protected StMenuModel defaultMenu() {
		StMenuModel rootMenu = new StMenuModel();

		rootMenu.add(this._fileMenu());
		rootMenu.add(this._editMenu());
		rootMenu.add(this._viewMenu());
		rootMenu.add(this._lightMenu());
		rootMenu.add(this._miscMenu());

		return rootMenu;
	}

	/**
	 * Answer the popup menu if exist.
	 *
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu popupMenu() {
		return this.defaultMenu();
	}
}
