package sra.jun.opengl.display;

import java.awt.*;
import java.awt.event.*;
import sra.jun.geometry.basic.*;
import sra.smalltalk.*;

/**
 * JunOpenGLShowController class
 * 
 *  @author    He Weijie
 *  @created   1998/11/26 (by He Weijie)
 *  @updated   2000/01/13 (by MATSUDA Ryouichi)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLShowController.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public class JunOpenGLShowController extends StController implements MouseListener, MouseMotionListener {
	protected transient JunOpenGLShowView view;
	protected transient Point startPoint;

	/**
	 * general dragging action.
	 * 
	 * @param event java.awt.event.MouseEvent
	 */
	public void mouseDragged(MouseEvent event) {
		if (event.isMetaDown()) {
			return;
		}

		Point currentPoint = event.getPoint();
		Point previousPoint = startPoint;

		if (!currentPoint.equals(previousPoint)) {
			this.mouseDraggedFrom_to_with_(previousPoint, currentPoint, event);
			startPoint = currentPoint;
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param fromPoint
	 * @param toPoint DOCUMENT ME!
	 * @param event DOCUMENT ME!
	 */
	public void mouseDraggedFrom_to_with_(Point fromPoint, Point toPoint, MouseEvent event) {
		StSymbol state = getModel().buttonState();

		if (state == $("bull")) {
			this.bullActivityFrom_to_(fromPoint, toPoint);

			return;
		}

		if (state == $("grab")) {
			this.grabActivityFrom_to_(fromPoint, toPoint);

			return;
		}

		if (state == $("drag")) {
			this.dragActivityFrom_to_with_(fromPoint, toPoint, event);
		}
	}

	/**
	 * Invoked when a mouse button has been pressed.
	 * 
	 * @param event java.awt.event.MouseEvent
	 */
	public void mousePressed(MouseEvent event) {
		startPoint = event.getPoint();
	}

	/**
	 * Invoked when a mouse button has been released. cursor reset to default.
	 * 
	 * @param event java.awt.event.MouseEvent
	 */
	public void mouseReleased(MouseEvent event) {
		((Component) getOpenGLView()).setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}

	/**
	 * add listeners.
	 * 
	 * @param newView sra.mvctest.StView
	 */
	protected void buildListener(StView newView) {
		Component glView = (Component) this.getOpenGLView();
		glView.addMouseListener(this);
		glView.addMouseMotionListener(this);
	}

	/**
	 * Bull action definition.
	 * 
	 * @param fromPoint
	 * @param toPoint DOCUMENT ME!
	 */
	protected void bullActivityFrom_to_(Point fromPoint, Point toPoint) {
		((Component) getOpenGLView()).setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));

		Jun2dPoint currentPoint = this.regularizePoint_(toPoint);
		Jun2dPoint previousPoint = this.regularizePoint_(fromPoint);

		if (!currentPoint.equals(previousPoint)) {
			getModel().bull_xy_(currentPoint, previousPoint);
		}

		((Component) getOpenGLView()).setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		getModel().changed_($("object"));
	}

	/**
	 * dragging action definition.
	 * 
	 * @param fromPoint
	 * @param toPoint DOCUMENT ME!
	 * @param event DOCUMENT ME!
	 */
	protected void dragActivityFrom_to_with_(Point fromPoint, Point toPoint, MouseEvent event) {
		((Component) getOpenGLView()).setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));

		Jun2dPoint currentPoint = this.regularizePoint_(toPoint);
		Jun2dPoint startPoint = this.regularizePoint_(fromPoint);

		if (currentPoint.equals(startPoint)) {
			return;
		}

		if (event.isShiftDown()) {
			getModel().look_xy_(startPoint, currentPoint);
		} else {
			getModel().slide_xy_(startPoint, currentPoint);
		}
	}

	/**
	 * model accessing.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLShowModel
	 */
	protected JunOpenGLShowModel getModel() {
		return (JunOpenGLShowModel) this.model();
	}

	/**
	 * view accessing.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGL3dView
	 */
	protected JunOpenGL3dView getOpenGLView() {
		StView view = this.view();

		if (view instanceof JunOpenGL3dView) {
			return (JunOpenGL3dView) view;
		} else if (view instanceof JunOpenGLShowView) {
			return ((JunOpenGLShowView) view).getOpenGLView();
		} else {
			return null;
		}
	}

	/**
	 * grabbing action definition.
	 * 
	 * @param fromPoint
	 * @param toPoint DOCUMENT ME!
	 */
	protected void grabActivityFrom_to_(Point fromPoint, Point toPoint) {
		((Component) getOpenGLView()).setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));

		Jun2dPoint currentPoint = regularizePoint_(toPoint);
		Jun2dPoint previousPoint = regularizePoint_(fromPoint);

		if (!currentPoint.equals(previousPoint)) {
			getModel().grab_xy_(previousPoint, currentPoint);
		}
	}

	/**
	 * Regularize the point on the current view.
	 * 
	 * @param aPoint
	 * 
	 * @return sra.jun.geometry.basic.Jun2dPoint
	 */
	protected Jun2dPoint regularizePoint_(Point aPoint) {
		Dimension bounds = ((Component) getOpenGLView()).getSize();
		double x = (aPoint.x - bounds.width / 2.0) / (bounds.height / 2.0);
		double y = - (aPoint.y - bounds.height / 2.0) / (bounds.height / 2.0);

		return new Jun2dPoint(x, y);
	}
}
