package sra.jun.opengl.display;

import java.awt.*;
import java.util.*;
import sra.smalltalk.*;
import sra.jun.geometry.abstracts.JunGeometry;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.curves.Jun3dLine;
import sra.jun.geometry.surfaces.JunSphereSurface;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.goodies.button.JunButtonModel;
import sra.jun.goodies.cursors.JunCursors;
import sra.jun.goodies.wheels.JunThumbWheel;
import sra.jun.opengl.lights.JunOpenGLLight;
import sra.jun.opengl.objects.*;
import sra.jun.opengl.projection.*;
import sra.jun.opengl.support.JunOpenGLRenderingContext;
import sra.jun.opengl.texture.JunOpenGLStipple;

/**
 * JunOpenGLShowModel class
 * 
 *  @author    He Weijie
 *  @created   1998/11/26 (by He Weijie)
 *  @updated   1999/06/25 (by nisinaka)
 *  @updated   1999/08/04 (by nisinaka)
 *  @updated   2000/01/06 (by MATSUDA Ryouichi)
 *  @updated   2003/05/08 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun490 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLShowModel.java,v 6.0 2003/10/28 10:52:17 nisinaka Exp $
 */
public class JunOpenGLShowModel extends JunOpenGL3dModel {

	protected JunOpenGLDisplayModel displayModel;
	protected Jun3dPoint viewportSight;
	protected double viewportHeight;
	protected boolean useTransparency;
	protected Hashtable pushButtons;

	/**
	 * Create a new instance of JunOpenGLShowModel.
	 */
	public JunOpenGLShowModel() {
		super();
	}

	/**
	 * Create a new instance of JunOpenGLShowModel with the specified display model.
	 *
	 * @param anOpenGLDisplayModel sra.jun.opengl.display.JunOpenGLDisplayModel
	 * @deprecated since Jun392, use the constructor instead.
	 */
	public JunOpenGLShowModel(JunOpenGLDisplayModel anOpenGLDisplayModel) {
		this.displayModel_(anOpenGLDisplayModel);
	}

	/**
	 * Create a new instance of JunOpenGLShowModel with the specified 3D object.
	 *
	 * @param anOpenGL3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGLShowModel(JunOpenGL3dObject anOpenGL3dObject) {
		this.displayObject_(anOpenGL3dObject);
	}

	/**
	 * Create a new instance of JunOpenGLShowModel with the specified display model.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLShowModel
	 * @param anOpenGLDisplayModel sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public static JunOpenGLShowModel DisplayModel_(JunOpenGLDisplayModel anOpenGLDisplayModel) {
		return new JunOpenGLShowModel(anOpenGLDisplayModel);
	}

	/**
	 * Create a new instance of JunOpenGLShowModel with the specified 3D object.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLShowModel
	 * @param anOpenGL3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @deprecated since Jun392, use the constructor instead.
	 */
	public static JunOpenGLShowModel DisplayObject_(JunOpenGL3dObject anOpenGL3dObject) {
		return new JunOpenGLShowModel(anOpenGL3dObject);
	}

	/**
	 * Create a new instance of JunOpenGLShowModel with the specified 3D object and open it.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLShowModel
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGLShowModel Show_(JunOpenGL3dObject a3dObject) {
		return Show_eyePoint_(a3dObject, null);
	}

	/**
	 * Create a new instance of JunOpenGLShowModel with the specified 3D object and open it.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLShowModel
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param eyePoint sra.jun.grometry.basic.Jun3dPoint
	 */
	public static JunOpenGLShowModel Show_eyePoint_(JunOpenGL3dObject a3dObject, Jun3dPoint eyePoint) {
		return Show_eyePoint_sightPoint_(a3dObject, eyePoint, null);
	}

	/**
	 * Create a new instance of JunOpenGLShowModel with the specified 3D object and open it.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLShowModel
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param eyePoint sra.jun.grometry.basic.Jun3dPoint
	 * @param sightPoint sra.jun.grometry.basic.Jun3dPoint
	 */
	public static JunOpenGLShowModel Show_eyePoint_sightPoint_(JunOpenGL3dObject a3dObject, Jun3dPoint eyePoint, Jun3dPoint sightPoint) {
		return Show_eyePoint_sightPoint_upVector_(a3dObject, eyePoint, sightPoint, null);
	}

	/**
	 * Create a new instance of JunOpenGLShowModel with the specified 3D object and open it.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLShowModel
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param eyePoint sra.jun.grometry.basic.Jun3dPoint
	 * @param sightPoint sra.jun.grometry.basic.Jun3dPoint
	 * @param upVector sra.jun.grometry.basic.Jun3dPoint
	 */
	public static JunOpenGLShowModel Show_eyePoint_sightPoint_upVector_(JunOpenGL3dObject a3dObject, Jun3dPoint eyePoint, Jun3dPoint sightPoint, Jun3dPoint upVector) {
		return Show_eyePoint_sightPoint_upVector_viewFactor_(a3dObject, eyePoint, sightPoint, upVector, null);
	}

	/**
	 * Create a new instance of JunOpenGLShowModel with the specified 3D object and open it.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLShowModel
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param eyePoint sra.jun.grometry.basic.Jun3dPoint
	 * @param sightPoint sra.jun.grometry.basic.Jun3dPoint
	 * @param upVector sra.jun.grometry.basic.Jun3dPoint
	 * @param viewFactor java.lang.Double
	 */
	public static JunOpenGLShowModel Show_eyePoint_sightPoint_upVector_viewFactor_(JunOpenGL3dObject a3dObject, Jun3dPoint eyePoint, Jun3dPoint sightPoint, Jun3dPoint upVector, Double viewFactor) {
		JunOpenGLShowModel displayModel = new JunOpenGLShowModel(a3dObject);

		if (eyePoint != null) {
			displayModel.defaultEyePoint_(eyePoint);
		}

		if (sightPoint != null) {
			displayModel.defaultSightPoint_(sightPoint);
		}

		if (upVector != null) {
			displayModel.defaultUpVector_(upVector);
		}

		if (viewFactor != null) {
			displayModel.defaultViewFactor_(viewFactor.doubleValue());
		}

		displayModel.open();

		return displayModel;
	}

	/**
	 * Bull the show object.
	 * Called from the controller.
	 * 
	 * @param from2dPoint sra.jun.geometry.basic.Jun2dPoint
	 * @param to2dPoint DOCUMENT ME!
	 */
	public void bull_xy_(Jun2dPoint from2dPoint, Jun2dPoint to2dPoint) {
		if (this.displayObject() == null) {
			return;
		}

		JunOpenGLDisplayModel aModel = this.displayModel();
		JunOpenGLProjection aProjection = aModel.displayProjection();
		JunSphereSurface grabSphere = aModel.grabSphere();
		Jun3dPoint fromPoint = this.grab3dPoint_(new Jun2dPoint(from2dPoint.x(), from2dPoint.y()));
		Jun3dPoint toPoint = this.grab3dPoint_(new Jun2dPoint(to2dPoint.x(), to2dPoint.y()));
		JunAngle rotationAngle = (new Jun3dLine(grabSphere.center(), toPoint)).angleWithLine_(new Jun3dLine(grabSphere.center(), fromPoint));

		if (Math.abs(rotationAngle.rad()) > ThetaAccuracy) {
			Jun3dLine rotationAxis = new Jun3dLine(grabSphere.center(), (Jun3dPoint) grabSphere.center().minus_(((Jun3dPoint) fromPoint.minus_(grabSphere.center())).product_((Jun3dPoint) toPoint.minus_(grabSphere.center()))));
			Jun3dTransformation transformation = Jun3dTransformation.Rotate_around_(rotationAngle, rotationAxis);
			Jun3dTransformation transformationInv = Jun3dTransformation.Rotate_around_(rotationAngle.mul_(-1), rotationAxis);
			Jun3dPoint upPoint = (Jun3dPoint) aProjection.sightPoint().plus_(aProjection.unitUpVector());
			aProjection.eyePoint_(transformationInv.applyTo_(aProjection.eyePoint()));
			aProjection.upVector_((Jun3dPoint) transformation.applyTo_(upPoint).minus_(aProjection.sightPoint()));
			aModel.displayProjection_(aProjection);
			aModel.changed_($("projection"));
		}
	}

	/**
	 * Answer the JunButtonModel for the bull button.
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 */
	public JunButtonModel bullButton() {
		if (pushButtons == null) {
			pushButtons = new Hashtable(3);
		}

		if (pushButtons.containsKey($("bull")) == false) {
			final JunOpenGLShowModel self = this;
			JunButtonModel button = new JunButtonModel();
			button.value_(false);
			button.visual_(JunCursors.BullCursorImage());
			button.action_(new StBlockClosure() {
				public Object value_(Object o) {
					JunButtonModel model = (JunButtonModel) o;
					model.value_(!model.value());
					if (model.value()) {
						self.dragButton().value_(false);
						self.grabButton().value_(false);
					}
					self.changed_($("state"));
					return model;
				}
			});

			pushButtons.put($("bull"), button);
		}

		return (JunButtonModel) pushButtons.get($("bull"));
	}

	/**
	 * Answer the current button state.
	 * 
	 * @return java.lang.String
	 */
	public StSymbol buttonState() {
		if (pushButtons == null) {
			this.bullButton();
			this.dragButton();
			this.grabButton();
		}

		Enumeration keys = pushButtons.keys();
		while (keys.hasMoreElements()) {
			StSymbol key = (StSymbol) keys.nextElement();
			JunButtonModel buttonModel = (JunButtonModel) pushButtons.get(key);
			if (buttonModel.value() == true) {
				return key;
			}
		}

		return $("none");
	}

	/**
	 * Answer the default eye point.
	 * 
	 * @return sra.jun.opengl.objects.Jun3dPoint
	 */
	public Jun3dPoint defaultEyePoint() {
		return this.displayModel().defaultEyePoint();
	}

	/**
	 * Set the default eye point.
	 * 
	 * @param eyePoint sra.jun.opengl.objects.Jun3dPoint
	 */
	public void defaultEyePoint_(Jun3dPoint eyePoint) {
		this.displayModel().defaultEyePoint_(eyePoint);
	}

	/**
	 * Answer the default sight point.
	 * 
	 * @return sra.jun.opengl.objects.Jun3dPoint
	 */
	public Jun3dPoint defaultSightPoint() {
		return this.displayModel().defaultSightPoint();
	}

	/**
	 * Set the default sight point.
	 * 
	 * @param sightPoint sra.jun.opengl.objects.Jun3dPoint
	 */
	public void defaultSightPoint_(Jun3dPoint sightPoint) {
		this.displayModel().defaultSightPoint_(sightPoint);
	}

	/**
	 * Answer the default up vector.
	 * 
	 * @return sra.jun.opengl.objects.Jun3dPoint
	 */
	public Jun3dPoint defaultUpVector() {
		return this.displayModel().defaultUpVector();
	}

	/**
	 * Set the default up vector.
	 * 
	 * @param upVector sra.jun.opengl.objects.Jun3dPoint
	 */
	public void defaultUpVector_(Jun3dPoint upVector) {
		this.displayModel().defaultUpVector_(upVector);
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunOpenGLShowViewAwt(this);
		} else {
			return new JunOpenGLShowViewSwing(this);
		}
	}

	/**
	 * Answer the default view factor.
	 * 
	 * @return sra.jun.opengl.objects.Jun3dPoint
	 */
	public double defaultViewFactor() {
		return this.displayModel().defaultViewFactor();
	}

	/**
	 * Set the default view factor.
	 * 
	 * @param viewFactor sra.jun.opengl.objects.Jun3dPoint
	 */
	public void defaultViewFactor_(double viewFactor) {
		this.displayModel().defaultViewFactor_(viewFactor);
	}

	/**
	 * Answer my current display model.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public JunOpenGLDisplayModel displayModel() {
		if (displayModel == null) {
			this.displayModel_(new JunOpenGLDisplayModel());
		}

		return displayModel;
	}

	/**
	 * Set my new display model.
	 * 
	 * @param aDisplayModel sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public void displayModel_(JunOpenGLDisplayModel aDisplayModel) {
		if (displayModel != aDisplayModel) {
			displayModel = aDisplayModel;
			if (displayModel.myDependents() == null || displayModel.myDependents().contains(this) == false) {
				displayModel.addDependent_(this);
			}
		}
	}

	/**
	 * Answer my current display object.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject displayObject() {
		return this.displayModel().displayObject();
	}

	/**
	 * Set my new display object.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public void displayObject_(JunOpenGL3dObject a3dObject) {
		this.displayModel().displayObject_(a3dObject);
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param from2dPoint sra.jun.grometry.basic.Jun2dPoint
	 * @param to2dPoint sra.jun.grometry.basic.Jun2dPoint
	 */
	public void drag_xy_(Jun2dPoint from2dPoint, Jun2dPoint to2dPoint) {
		if (this.showObject() == null) {
			return;
		}

		//if (StController._ShiftDown()) {
		//	this.look_xy_(from2dPoint, to2dPoint);
		//} else {
		this.slide_xy_(from2dPoint, to2dPoint);
		//}
	}

	/**
	 * Answer the JunButtonModel for the drag button.
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 */
	public JunButtonModel dragButton() {
		if (pushButtons == null) {
			pushButtons = new Hashtable(3);
		}

		if (pushButtons.containsKey($("drag")) == false) {
			final JunOpenGLShowModel self = this;
			JunButtonModel button = new JunButtonModel();
			button.value_(false);
			button.visual_(JunCursors.QuartersCursorImage());
			button.action_(new StBlockClosure() {
				public Object value_(Object o) {
					JunButtonModel model = (JunButtonModel) o;
					model.value_(!model.value());
					if (model.value()) {
						self.grabButton().value_(false);
						self.bullButton().value_(false);
					}
					self.changed_($("state"));
					return model;
				}
			});

			pushButtons.put($("drag"), button);
		}

		return (JunButtonModel) pushButtons.get($("drag"));
	}

	/**
	 * Answer the JunButtonModel for the grab button.
	 * 
	 * @return sra.jun.goodies.button.JunButtonModel
	 */
	public JunButtonModel grabButton() {
		if (pushButtons == null) {
			pushButtons = new Hashtable(3);
		}

		if (pushButtons.containsKey($("grab")) == false) {
			final JunOpenGLShowModel self = this;
			JunButtonModel button = new JunButtonModel();
			button.value_(false);
			button.visual_(JunCursors.HandCursorImage());
			button.action_(new StBlockClosure() {
				public Object value_(Object o) {
					JunButtonModel model = (JunButtonModel) o;
					model.value_(!model.value());
					if (model.value()) {
						self.dragButton().value_(false);
						self.bullButton().value_(false);
					}
					self.changed_($("state"));
					return model;
				}
			});

			pushButtons.put($("grab"), button);
		}

		return (JunButtonModel) pushButtons.get($("grab"));
	}

	/**
	 * Answer my current display object.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public final JunOpenGL3dObject openGL3dObject() {
		return this.showObject();
	}

	/**
	 * Answer my current projector.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjector
	 */
	public final JunOpenGLProjector openGLProjector() {
		if (openGLProjector == null) {
			openGLProjector = new JunOpenGLProjector();
			openGLProjector.eyePoint_(this.defaultEyePoint());
			openGLProjector.sightPoint_(this.defaultSightPoint());
			openGLProjector.upVector_(this.defaultUpVector());
			openGLProjector.viewFactor_(this.defaultViewFactor());
		}

		return openGLProjector;
	}

	/**
	 * Render myself on the rendering context.
	 * 
	 * @param renderingContext sra.jun.opengl.objects.JunOpenGLRenderingContext
	 */
	public void renderOn_(final JunOpenGLRenderingContext renderingContext) {
		renderingContext.clear();

		if (this.displayObject() == null) {
			return;
		}

		JunOpenGLLight[] lightCollection = this.displayModel().displayLightCollection();
		this.showProjector().project_withLights_on_(this.showObject(), lightCollection, renderingContext);
		this.showProjector().project_on_(this.viewportBeam(), renderingContext);
		renderingContext.flush();
	}

	/**
	 * Show the object.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject showObject() {
		JunOpenGLProjection aProjection = this.displayModel().displayProjection();
		double currentHeight = this.computeViewportHeight();
		Jun3dPoint currentSight = aProjection.sightPoint();
		double nearHeight = 2 * aProjection.sightPoint().distance_(aProjection.nearCenter());

		if (currentHeight > nearHeight) {
			currentHeight = nearHeight;
		}

		if ((openGL3dObject == null) || Double.isNaN(viewportHeight) || (Math.abs(currentHeight - viewportHeight) > JunGeometry.ACCURACY) || (viewportSight == null) || (currentSight.distance_(viewportSight) > JunGeometry.ACCURACY)) {
			viewportHeight = currentHeight;
			viewportSight = currentSight;

			//			JunScavenger aScavenger = new JunScavenger();
			//			if (showObject == null) {
			//				aScavenger.run();
			//			} else {
			//				JunOpenGL3dObject aSphere;
			//				if (this.displayObject() == null) {
			//					aSphere = showObject.components().first();
			//				} else {
			//					aSphere = showObject.components().second();
			//				}
			//				aSphere do :[: each | each stipple notNil ifTrue :[each stipple flushPointerOfPixels]].aScavenger.run();
			//			}
			JunOpenGL3dObject aSphere = JunOpenGL3dObject.Globe_radius_center_(15, viewportHeight / 2, viewportSight);
			aSphere.paint_(Color.cyan);

			if (this.useTransparency()) {
				aSphere.alpha_(0.25f);
			} else {
				JunOpenGLStipple aStipple = JunOpenGLStipple.Halftone_(0.25);

				if (aSphere instanceof JunOpenGL3dCompoundObject) {
					JunOpenGL3dObject[] components = ((JunOpenGL3dCompoundObject) aSphere).components();

					for (int i = 0; i < components.length; i++) {
						components[i].stipple_(aStipple);
					}
				} else {
					aSphere.stipple_(aStipple);
				}
			}

			JunOpenGL3dObject xyzAxes = JunOpenGL3dObject.Axes2();
			xyzAxes = xyzAxes.scaledBy_(viewportHeight / 2);

			JunOpenGL3dCompoundObject aBody = new JunOpenGL3dCompoundObject();

			if (this.displayObject() != null) {
				aBody.add_(this.displayObject());
			}

			aBody.add_(aSphere);
			aBody.add_(xyzAxes);
			openGL3dObject = aBody;
		}

		return JunOpenGL3dCompoundObject.With_with_(openGL3dObject, this.viewportObject());
	}

	/**
	 * Answer the current projection of the receiver.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjection
	 */
	public JunOpenGLProjection showProjection() {
		return this.showProjector().projection();
	}

	/**
	 * Set the new projection for the receiver.
	 * 
	 * @param aProjection sra.jun.opengl.projection.JunOpenGLProjection
	 */
	public void showProjection_(JunOpenGLProjection aProjection) {
		this.showProjector().projection_(aProjection);
	}

	/**
	 * Answer the projector for the receiver.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjector
	 */
	public JunOpenGLProjector showProjector() {
		return this.openGLProjector();
	}

	/**
	 * Spawn a show object and open it with JunOpenGLDisplayModel.
	 */
	public void spawnObject() {
		JunOpenGLDisplayModel aDiaplayModel = new JunOpenGLDisplayModel(this.showObject());
		aDiaplayModel.displayProjection_((JunOpenGLProjection) this.showProjection().copy());
		aDiaplayModel.open();
		aDiaplayModel.changed_($("object"));
	}

	/**
	 * Spawn a Viewfinder.
	 */
	public void spawnViewfinder() {
		JunOpenGLDisplayModel model = this.displayModel();
		if (model.builder().windows().length > 0) {
			model._windowExpandAndRaise();
		} else {
			model.open();
		}
	}

	/**
	 * Toggle between stipple and transparency.
	 */
	public void switchStippleAndTransparency() {
		this.useTransparency_(!this.useTransparency());
	}

	/**
	 * Called when a changed method is invoked on a dependence.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if (evt.getSource() == this.displayModel()) {
			this.changed_(evt.getAspect());
		}
	}

	/**
	 * Update the menu indications.
	 */
	public void updateMenuIndication() {
		this.updateViewMenuIndication();
	}

	/**
	 * Answer true if I use the transparency.
	 * 
	 * @return boolean
	 */
	public boolean useTransparency() {
		return useTransparency;
	}

	/**
	 * Set a boolean whether I use the transparency or not.
	 * 
	 * @param aBoolean boolean
	 */
	public void useTransparency_(boolean aBoolean) {
		boolean previousBoolean = this.useTransparency();
		useTransparency = aBoolean;

		if (previousBoolean != this.useTransparency()) {
			this.flushObject();
			this.changed_($("object"));
		}
	}

	/**
	 * Answer the viewport beam.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject viewportBeam() {
		JunOpenGLProjection aProjection = this.displayModel().displayProjection();
		Jun3dPoint far = aProjection.farCenter();
		Jun3dPoint sight = aProjection.sightPoint();
		Jun3dPoint near = aProjection.nearCenter();
		Jun3dPoint eye = aProjection.eyePoint();
		JunOpenGL3dCompoundObject eyeBeam = new JunOpenGL3dCompoundObject();
		Jun3dPoint[] twoPoints = new Jun3dPoint[2];
		JunOpenGL3dPolyline aPolyline;

		if (far.distance_(near) > sight.distance_(near)) {
			twoPoints[0] = far;
			twoPoints[1] = sight;
			aPolyline = (JunOpenGL3dPolyline) JunOpenGL3dPolyline.Vertexes_(twoPoints);
			aPolyline.paint_(Color.magenta);
			eyeBeam.add_(aPolyline);
		}

		twoPoints[0] = sight;
		twoPoints[1] = near;
		aPolyline = (JunOpenGL3dPolyline) JunOpenGL3dPolyline.Vertexes_(twoPoints);
		aPolyline.paint_(Color.blue);
		eyeBeam.add_(aPolyline);

		if (eye.distance_(sight) > near.distance_(sight)) {
			twoPoints[0] = near;
			twoPoints[1] = eye;
			aPolyline = (JunOpenGL3dPolyline) JunOpenGL3dPolyline.Vertexes_(twoPoints);
			aPolyline.paint_(Color.yellow);
			eyeBeam.add_(aPolyline);
		}

		eyeBeam.name_("viewport eye beam");

		return eyeBeam;
	}

	/**
	 * Answer the viewport object.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject viewportObject() {
		JunOpenGL3dCompoundObject aPlate = this.viewportPlate();
		JunOpenGLProjection aProjection = this.displayModel().displayProjection();

		if (viewportHeight == Double.NaN) {
			viewportHeight = this.computeViewportHeight();
		}

		double aRadius = viewportHeight / 2;
		double aDistance = aProjection.sightPoint().distance_(aProjection.eyePoint());
		Jun3dLine aLine = new Jun3dLine(aProjection.sightPoint(), aProjection.eyePoint());
		Jun3dPoint aPoint = (Jun3dPoint) aLine.atT_(aRadius / aDistance);
		aPlate = (JunOpenGL3dCompoundObject) aPlate.translatedBy_(aPoint);

		JunOpenGL3dObject[] components = aPlate.components();
		JunOpenGL3dPolylineLoop aPolyline = (JunOpenGL3dPolylineLoop) JunOpenGL3dPolylineLoop.Vertexes_(((JunOpenGL3dVertexesObject) components[components.length - 1]).vertexes());
		Jun3dPoint[] vertexes = aPolyline.vertexes();
		Jun3dPoint p1 = vertexes[0];
		Jun3dPoint p2 = vertexes[1];
		Jun3dPoint p3 = vertexes[2];
		Jun3dPoint p4 = vertexes[3];
		Jun3dLine xLine = new Jun3dLine((Jun3dPoint) p1.plus_(((Jun3dPoint) p4.minus_(p1)).dividedBy_(2)), (Jun3dPoint) p2.plus_(((Jun3dPoint) p3.minus_(p2)).dividedBy_(2)));
		Jun3dLine yLine = new Jun3dLine((Jun3dPoint) p1.plus_(((Jun3dPoint) p2.minus_(p1)).dividedBy_(2)), (Jun3dPoint) p4.plus_(((Jun3dPoint) p3.minus_(p4)).dividedBy_(2)));
		JunOpenGL3dCompoundObject xAxis = new JunOpenGL3dCompoundObject();
		Jun3dPoint[] twoPoints = new Jun3dPoint[2];
		Jun3dPoint[] fourPoints = new Jun3dPoint[4];
		twoPoints[0] = (Jun3dPoint) xLine.atT_(0);
		twoPoints[1] = (Jun3dPoint) xLine.atT_(1);
		xAxis.add_(JunOpenGL3dPolyline.Vertexes_(twoPoints));
		fourPoints[0] = (Jun3dPoint) xLine.atT_(1);
		fourPoints[1] = (Jun3dPoint) xLine.atT_(0.9).minus_(((Jun3dPoint) ((Jun3dPoint) yLine.atT_(1)).minus_(yLine.atT_(0.9))).dividedBy_(4));
		fourPoints[2] = (Jun3dPoint) xLine.atT_(0.9).plus_(((Jun3dPoint) ((Jun3dPoint) yLine.atT_(1)).minus_(yLine.atT_(0.9))).dividedBy_(4));
		fourPoints[3] = (Jun3dPoint) xLine.atT_(1);
		xAxis.add_(JunOpenGL3dPolyline.Vertexes_(fourPoints));
		xAxis.paint_(Color.red);
		xAxis.name_(aPlate.name() + " x axis");

		JunOpenGL3dCompoundObject yAxis = new JunOpenGL3dCompoundObject();
		twoPoints[0] = (Jun3dPoint) yLine.atT_(0);
		twoPoints[1] = (Jun3dPoint) yLine.atT_(1);
		yAxis.add_(JunOpenGL3dPolyline.Vertexes_(twoPoints));
		fourPoints[0] = (Jun3dPoint) yLine.atT_(1);
		fourPoints[1] = (Jun3dPoint) yLine.atT_(0.9).minus_(((Jun3dPoint) ((Jun3dPoint) xLine.atT_(1)).minus_(xLine.atT_(0.9))).dividedBy_(4));
		fourPoints[2] = (Jun3dPoint) yLine.atT_(0.9).plus_(((Jun3dPoint) ((Jun3dPoint) xLine.atT_(1)).minus_(xLine.atT_(0.9))).dividedBy_(4));
		fourPoints[3] = (Jun3dPoint) yLine.atT_(1);
		yAxis.add_(JunOpenGL3dPolyline.Vertexes_(fourPoints));
		yAxis.paint_(Color.green);
		yAxis.name_(aPlate.name() + " y axis");

		JunOpenGL3dCompoundObject aBody = new JunOpenGL3dCompoundObject();
		aBody.add_(aPlate);
		aBody.add_(aPolyline);
		aBody.add_(xAxis);
		aBody.add_(yAxis);
		aBody.name_("viewport");

		return aBody;
	}

	/**
	 * Answer the viewport plate.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 */
	public JunOpenGL3dCompoundObject viewportPlate() {
		JunOpenGLProjection aProjection = this.displayModel().displayProjection();

		if (Double.isNaN(viewportHeight)) {
			viewportHeight = this.computeViewportHeight();
		}

		double height = viewportHeight;
		double width;

		if (this.displayModel().builder().windows().length > 0) {
			Window aWindow = (Window) this.displayModel().builder().windows()[0];
			width = (height * aWindow.getBounds().width) / aWindow.getBounds().height;
		} else {
			width = height;
		}

		Jun3dPoint p1 = new Jun3dPoint(0, 0, 0);
		Jun3dPoint p2 = (Jun3dPoint) aProjection.unitRightVector().scaledBy_(new Jun3dPoint(width, width, width));
		Jun3dPoint p4 = (Jun3dPoint) aProjection.unitUpVector().scaledBy_(new Jun3dPoint(height, height, height));
		Jun3dPoint p3 = (Jun3dPoint) p2.plus_(((Jun3dPoint) p4.minus_(p2)).dividedBy_(2));
		p3 = (Jun3dPoint) p3.plus_(p3);

		Jun3dPoint[] anArray = new Jun3dPoint[] { p1, p2, p3, p4 };
		JunOpenGL3dCompoundObject aPlate = new JunOpenGL3dCompoundObject();

		if (this.useTransparency()) {
			Jun3dPoint[] reversedArray = new Jun3dPoint[] { p4, p3, p2, p1 };
			aPlate.add_(JunOpenGL3dPolygon.Vertexes_(reversedArray));
		}

		aPlate.add_(JunOpenGL3dPolygon.Vertexes_(anArray));

		Jun3dPoint p5 = (Jun3dPoint) p2.plus_(((Jun3dPoint) p4.minus_(p2)).dividedBy_(2));
		Jun3dPoint p6 = (Jun3dPoint) (new Jun3dPoint(0, 0, 0)).minus_(p5);
		aPlate = (JunOpenGL3dCompoundObject) aPlate.translatedBy_(p6);
		aPlate.paint_(Color.yellow);

		if (this.useTransparency()) {
			aPlate.alpha_(0.2f);
		} else {
			aPlate.components()[0].stipple_(JunOpenGLStipple.Halftone_(0.125));
		}

		aPlate.name_("viewport plate");

		return aPlate;
	}

	/**
	 * Create a Frame for the application model.
	 * Do everything but open the frame.
	 *
	 * @return java.awt.Frame
	 * @param aView sra.smalltalk.StView
	 */
	protected Frame allButOpenView_(StView aView) {
		Frame aFrame = super.allButOpenView_(aView);
		this.spawnViewfinder();
		return aFrame;
	}

	/**
	 * Compute the current sight point.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	protected Jun3dPoint computeSightPoint() {
		return this.displayModel().displayProjection().sightPoint();
	}

	/**
	 * Compute the current viewport height.
	 * 
	 * @return double
	 */
	protected double computeViewportHeight() {
		JunOpenGLProjection aProjection = this.displayModel().displayProjection();
		double currentHeight = aProjection.regularHeight();
		double nearHeight = 2 * aProjection.sightPoint().distance_(aProjection.nearCenter());

		if (currentHeight > nearHeight) {
			currentHeight = nearHeight;
		}

		return currentHeight;
	}

	/**
	 * Create the menubar for JunOpenGLShowModel.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 */
	protected StMenuModel defaultMenu() {
		StMenuModel rootMenu = new StMenuModel();

		// View
		rootMenu.add(this._viewMenu());

		// Misc
		StMenu miscMenu = new StMenu("Misc");
		miscMenu.add(createMenuItem("Spawn", "spawnObject"));
		miscMenu.add(createMenuItem("Viewfinder", "spawnViewfinder"));
		miscMenu.addSeparator();
		miscMenu.add(createMenuItem("Stipple or Transparency", "switchStippleAndTransparency"));
		rootMenu.add(miscMenu);

		return rootMenu;
	}

	/**
	 * Initialize the JunOpenGLShowModel.
	 */
	protected void initialize() {
		super.initialize();

		displayModel = null;
		viewportSight = null;
		viewportHeight = Double.NaN;
		useTransparency = false;
		pushButtons = null;
	}

	/**
	 * Answer this window title.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "Viewport";
	}

	/**
	 * Answer a xThumbWheel.
	 * 
	 * @return sra.jun.goodies.wheels.JunThumbWheel
	 */
	protected JunThumbWheel xThumbWheel() {
		final JunOpenGLShowModel self = this;
		JunThumbWheel thumbWheel = new JunThumbWheel();
		thumbWheel.compute_(new StBlockClosure() {
			public Object value_(Object value) {
				Jun2dPoint factor = new Jun2dPoint(0, - ((Number) value).doubleValue() / 50);
				self.grab_(factor);
				return null;
			}
		});

		return thumbWheel;
	}

	/**
	 * Answer a yThumbWheel.
	 * 
	 * @return sra.jun.goodies.wheels.JunThumbWheel
	 */
	protected JunThumbWheel yThumbWheel() {
		final JunOpenGLShowModel self = this;
		JunThumbWheel thumbWheel = new JunThumbWheel();
		thumbWheel.compute_(new StBlockClosure() {
			public Object value_(Object value) {
				Jun2dPoint factor = new Jun2dPoint(((Number) value).doubleValue() / 50, 0);
				self.grab_(factor);
				return null;
			}
		});

		return thumbWheel;
	}

	/**
	 * Answer a zThumbWheel
	 * 
	 * @return sra.jun.goodies.wheels.JunThumbWheel
	 */
	protected JunThumbWheel zThumbWheel() {
		final JunOpenGLShowModel self = this;
		JunThumbWheel thumbWheel = new JunThumbWheel();
		thumbWheel.compute_(new StBlockClosure() {
			public Object value_(Object value) {
				double factor = 1 + (((Number) value).doubleValue() / 100);
				self.zoom_(factor);
				return null;
			}
		});

		return thumbWheel;
	}
}
