package sra.jun.opengl.display;

import sra.smalltalk.*;
import sra.jun.goodies.button.JunButtonViewAwt;
import sra.jun.goodies.wheels.JunThumbWheelViewAwt;

/**
 * JunOpenGLShowViewAwt class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/06/02 (by MATSUDA Ryouichi)
 *  @updated   2000/01/09 (by MATSUDA Ryouichi)
 *  @updated   2003/05/08 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLShowViewAwt.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGLShowViewAwt extends StViewPanel implements JunOpenGLShowView {

	protected JunOpenGL3dViewAwt showView;

	/**
	 * Create a new instance of JunOpenGLShowViewAwt and initialize it.
	 *
	 * @param anOpenGLShowModel sra.jun.opengl.display.JunOpenGLShowModel
	 */
	public JunOpenGLShowViewAwt(JunOpenGLShowModel anOpenGLShowModel) {
		super(anOpenGLShowModel);
	}

	/**
	 * Answer my model as JunOpenGLDisplayModel.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public JunOpenGLDisplayModel getOpenGLDisplayModel() {
		return (JunOpenGLDisplayModel) this.model();
	}

	/**
	 * Answer my model as JunOpenGLShowModel.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLShowModel
	 */
	public JunOpenGLShowModel getOpenGLShowModel() {
		return (JunOpenGLShowModel) this.model();
	}

	/**
	 * Answer my current OpenGL view.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayView
	 */
	public JunOpenGL3dView getOpenGLView() {
		return this.showView();
	}

	/**
	 * Answer my show view.
	 *
	 * @return sra.jun.opengl.display.JunOpenGL3dViewAwt
	 */
	public JunOpenGL3dViewAwt showView() {
		if (showView == null) {
			showView = new JunOpenGL3dViewAwt(this.getOpenGLShowModel()) {
				public void update_(DependentEvent evt) {
					if (evt.getAspect() != $("state")) {
						super.update_(evt);
					}
				}
			};
		}
		return showView;
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.showView()._onBorderedPanel(), WindowSpecLayout.Position(0f, 20, 0f, 1, 1f, -20, 1f, -20));
		this.add(new JunButtonViewAwt(this.getOpenGLShowModel().bullButton()), WindowSpecLayout.Position(0f, 1, 0f, 1, 0f, 19, 0f, 19));
		this.add(new JunButtonViewAwt(this.getOpenGLShowModel().grabButton()), WindowSpecLayout.Position(0f, 1, 0f, 20, 0f, 19, 0f, 38));
		this.add(new JunButtonViewAwt(this.getOpenGLShowModel().dragButton()), WindowSpecLayout.Position(0f, 1, 0f, 39, 0f, 19, 0f, 57));
		this.add(new JunThumbWheelViewAwt(this.getOpenGLShowModel().xThumbWheel(), true), WindowSpecLayout.Position(0f, 1, 0.5f, -61, 0f, 19, 0.5f, 61));
		this.add(new JunThumbWheelViewAwt(this.getOpenGLShowModel().yThumbWheel(), false), WindowSpecLayout.Position(0.5f, -61, 1f, -19, 0.5f, 61, 1f, -1));
		this.add(new JunThumbWheelViewAwt(this.getOpenGLShowModel().zThumbWheel(), true), WindowSpecLayout.Position(1f, -19, 0.5f, -61, 1f, -1, 0.5f, 61));
		this.setSize(300, 300);
	}

	/**
	 * Answer a default controller.
	 * 
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new JunOpenGLShowController();
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunOpenGLShowModel();
	}
}
