package sra.jun.opengl.flux;

import sra.jun.geometry.abstracts.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.*;

/**
 * JunCoordinateAngles class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/12/02 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCoordinateAngles.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunCoordinateAngles extends JunGeometry {
	JunAngle omegaAngle;
	JunAngle phiAngle;
	JunAngle thetaAngle;

	/**
	 * Create a new instance of JunCoordinateAngles.
	 */
	public JunCoordinateAngles() {
		super();
		this.initialize();
	}

	/**
	 * Create a new instance of JunCoordinateAngles with omega, phi and theta.
	 * 
	 * @param omegaAngle sra.jun.geometry.basic.JunAngle
	 * @param phiAngle sra.jun.geometry.basic.JunAngle
	 * @param thetaAngle sra.jun.geometry.basic.JunAngle
	 * 
	 * @return sra.jun.opengl.flux.JunCoordinateAngles
	 */
	public static JunCoordinateAngles Omega_phi_theta_(JunAngle omegaAngle, JunAngle phiAngle, JunAngle thetaAngle) {
		JunCoordinateAngles eulerAngles;

		eulerAngles = new JunCoordinateAngles();
		eulerAngles.omega_(omegaAngle);
		eulerAngles.phi_(phiAngle);
		eulerAngles.theta_(thetaAngle);

		return eulerAngles;
	}

	/**
	 * Create a new instance of JunCoordinateAngles with three points.
	 * 
	 * @param pointA sra.jun.geometry.basic.Jun3dPoint
	 * @param pointB sra.jun.geometry.basic.Jun3dPoint
	 * @param pointC sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.flux.JunCoordinateAngles
	 */
	public static JunCoordinateAngles P1_p2_p3_(Jun3dPoint pointA, Jun3dPoint pointB, Jun3dPoint pointC) {
		JunCoordinateAngles instance;

		instance = new JunCoordinateAngles();
		instance.p1_p2_p3_(pointA, pointB, pointC);

		return instance;
	}

	/**
	 * Create a new instance of JunCoordinateAngles with four points.
	 * 
	 * @param pointA sra.jun.geometry.basic.Jun3dPoint
	 * @param pointB sra.jun.geometry.basic.Jun3dPoint
	 * @param pointC sra.jun.geometry.basic.Jun3dPoint
	 * @param pointD sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.flux.JunCoordinateAngles
	 */
	public static JunCoordinateAngles P1_p2_p3_p4_(Jun3dPoint pointA, Jun3dPoint pointB, Jun3dPoint pointC, Jun3dPoint pointD) {
		JunCoordinateAngles instance;

		instance = new JunCoordinateAngles();
		instance.p1_p2_p3_p4_(pointA, pointB, pointC, pointD);

		return instance;
	}

	/**
	 * Create a new instance of JunCoordinateAngles with two vectors.
	 * 
	 * @param vectorA sra.jun.geometry.basic.Jun3dPoint
	 * @param vectorB sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.flux.JunCoordinateAngles
	 */
	public static JunCoordinateAngles V1_v2_(Jun3dPoint vectorA, Jun3dPoint vectorB) {
		JunCoordinateAngles instance;

		instance = new JunCoordinateAngles();
		instance.v1_v2_(vectorA, vectorB);

		return instance;
	}

	/**
	 * Answer inverse transformation.
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public Jun3dTransformation asInverseTransformation() {
		Jun3dTransformation aTransformationAroundX;
		Jun3dTransformation aTransformationAroundY;
		Jun3dTransformation aTransformationAroundZ;
		Jun3dTransformation aTransformation;

		aTransformationAroundX = Jun3dTransformation.RotateX_(this.omega().negated());
		aTransformationAroundY = Jun3dTransformation.RotateY_(this.phi().negated());
		aTransformationAroundZ = Jun3dTransformation.RotateZ_(this.theta().negated());
		aTransformation = Jun3dTransformation.Unity();
		aTransformation = (Jun3dTransformation) aTransformation.product_(aTransformationAroundZ);
		aTransformation = (Jun3dTransformation) aTransformation.product_(aTransformationAroundY);
		aTransformation = (Jun3dTransformation) aTransformation.product_(aTransformationAroundX);

		return aTransformation;
	}

	/**
	 * Answer transformation.
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public Jun3dTransformation asTransformation() {
		Jun3dTransformation aTransformationAroundX;
		Jun3dTransformation aTransformationAroundY;
		Jun3dTransformation aTransformationAroundZ;
		Jun3dTransformation aTransformation;

		aTransformationAroundX = Jun3dTransformation.RotateX_(this.omega());
		aTransformationAroundY = Jun3dTransformation.RotateY_(this.phi());
		aTransformationAroundZ = Jun3dTransformation.RotateZ_(this.theta());
		aTransformation = Jun3dTransformation.Unity();
		aTransformation = (Jun3dTransformation) aTransformation.product_(aTransformationAroundX);
		aTransformation = (Jun3dTransformation) aTransformation.product_(aTransformationAroundY);
		aTransformation = (Jun3dTransformation) aTransformation.product_(aTransformationAroundZ);

		return aTransformation;
	}

	/**
	 * Initialize.
	 */
	public void initialize() {
		this.omega_(0);
		this.phi_(0);
		this.theta_(0);
	}

	/**
	 * Answer omega angle.
	 * 
	 * @return sra.jun.geometry.basic.JunAngle
	 */
	public final JunAngle omega() {
		return omegaAngle;
	}

	/**
	 * Set omega angle.
	 * 
	 * @param anAngle double
	 */
	public void omega_(double anAngle) {
		omegaAngle = JunAngle.FromRad_(anAngle);
	}

	/**
	 * Set omega angle.
	 * 
	 * @param anAngle sra.jun.geometry.basic.JunAngle
	 */
	public final void omega_(JunAngle anAngle) {
		omegaAngle = anAngle;
	}

	/**
	 * Compute angles from three points.
	 * 
	 * @param pointA sra.jun.geometry.basic.Jun3dPoint
	 * @param pointB sra.jun.geometry.basic.Jun3dPoint
	 * @param pointC sra.jun.geometry.basic.Jun3dPoint
	 */
	public void p1_p2_p3_(Jun3dPoint pointA, Jun3dPoint pointB, Jun3dPoint pointC) {
		Jun3dPoint aVector;
		Jun3dPoint normalVector;

		aVector = (Jun3dPoint) pointB.minus_(pointA);
		normalVector = this.outerProduct_with_(aVector, (Jun3dPoint) pointC.minus_(pointA));
		normalVector = this.outerProduct_with_(normalVector, aVector);
		this.computeAnglesFrom_and_(aVector, normalVector);
	}

	/**
	 * Compute angles from four points.
	 * 
	 * @param pointA sra.jun.geometry.basic.Jun3dPoint
	 * @param pointB sra.jun.geometry.basic.Jun3dPoint
	 * @param pointC sra.jun.geometry.basic.Jun3dPoint
	 * @param pointD sra.jun.geometry.basic.Jun3dPoint
	 */
	public void p1_p2_p3_p4_(Jun3dPoint pointA, Jun3dPoint pointB, Jun3dPoint pointC, Jun3dPoint pointD) {
		Jun3dPoint aVector;
		Jun3dPoint normalVector;

		aVector = (Jun3dPoint) pointB.minus_(pointA);
		normalVector = this.outerProduct_with_(aVector, (Jun3dPoint) pointD.minus_(pointC));
		this.computeAnglesFrom_and_(aVector, normalVector);
	}

	/**
	 * Answer phi angle.
	 * 
	 * @return sra.jun.geometry.basic.JunAngle
	 */
	public final JunAngle phi() {
		return phiAngle;
	}

	/**
	 * Set phi angle.
	 * 
	 * @param anAngle double
	 */
	public void phi_(double anAngle) {
		phiAngle = JunAngle.FromRad_(anAngle);
	}

	/**
	 * Set phi angle.
	 * 
	 * @param anAngle sra.jun.geometry.basic.JunAngle
	 */
	public final void phi_(JunAngle anAngle) {
		phiAngle = anAngle;
	}

	/**
	 * Answer theta angle.
	 * 
	 * @return sra.jun.geometry.basic.JunAngle
	 */
	public final JunAngle theta() {
		return thetaAngle;
	}

	/**
	 * Set theta angle.
	 * 
	 * @param anAngle double
	 */
	public void theta_(double anAngle) {
		thetaAngle = JunAngle.FromRad_(anAngle);
	}

	/**
	 * Set theta angle.
	 * 
	 * @param anAngle sra.jun.geometry.basic.JunAngle
	 */
	public final void theta_(JunAngle anAngle) {
		thetaAngle = anAngle;
	}

	/**
	 * Compute angles from two vectors.
	 * 
	 * @param vectorA sra.jun.geometry.basic.Jun3dPoint
	 * @param vectorB sra.jun.geometry.basic.Jun3dPoint
	 */
	public void v1_v2_(Jun3dPoint vectorA, Jun3dPoint vectorB) {
		this.p1_p2_p3_(new Jun3dPoint(0, 0, 0), vectorA, vectorB);
	}

	/**
	 * Compute angles from vector and normalVector.
	 * 
	 * @param aVector sra.jun.geometry.basic.Jun3dPoint
	 * @param normalVector sra.jun.geometry.basic.Jun3dPoint
	 */
	protected void computeAnglesFrom_and_(Jun3dPoint aVector, Jun3dPoint normalVector) {
		Jun3dPoint vectorB;
		Jun3dPoint vectorN;
		double sinTheta;
		double thetaRadians;
		double sinPhi;
		double phiRadians;
		double sinOmega;
		double omegaRadians;
		Jun3dTransformation aTransformation;

		vectorB = aVector;
		vectorN = normalVector;

		double aLength = this.vectorLength_(new Jun3dPoint(vectorB.x(), vectorB.y(), 0));

		if (aLength < Accuracy()) {
			thetaRadians = 0;
		} else {
			sinTheta = vectorB.y() / aLength;
			thetaRadians = Math.acos(vectorB.x() / aLength);

			if (sinTheta > 0) {
				thetaRadians = thetaRadians * -1;
			}
		}

		aTransformation = Jun3dTransformation.RotateZ_(JunAngle.FromRad_(thetaRadians));
		vectorB = (Jun3dPoint) vectorB.transform_(aTransformation);
		vectorN = (Jun3dPoint) vectorN.transform_(aTransformation);
		aLength = this.vectorLength_(vectorB);
		sinPhi = vectorB.z() / aLength;
		phiRadians = Math.acos(this.vectorLength_(new Jun3dPoint(vectorB.x(), vectorB.y(), 0)) / aLength);

		if (sinPhi < 0) {
			phiRadians = phiRadians * -1;
		}

		aTransformation = Jun3dTransformation.RotateY_(JunAngle.FromRad_(phiRadians));
		vectorB = (Jun3dPoint) vectorB.transform_(aTransformation);
		vectorN = (Jun3dPoint) vectorN.transform_(aTransformation);
		aLength = this.vectorLength_(new Jun3dPoint(0, vectorN.y(), vectorN.z()));
		sinOmega = vectorN.y() / aLength;
		omegaRadians = Math.acos(vectorN.z() / aLength);

		if (sinOmega > 0) {
			omegaRadians = omegaRadians * -1;
		}

		thetaAngle = JunAngle.FromRad_(thetaRadians * -1);
		phiAngle = JunAngle.FromRad_(phiRadians * -1);
		omegaAngle = JunAngle.FromRad_(omegaRadians * -1);
	}

	/**
	 * Answer outer product vector with vector.
	 * 
	 * @param aVector1 sra.jun.geometry.basic.Jun3dPoint
	 * @param aVector2 sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	protected Jun3dPoint outerProduct_with_(Jun3dPoint aVector1, Jun3dPoint aVector2) {
		return new Jun3dPoint((aVector1.y() * aVector2.z()) - (aVector1.z() * aVector2.y()), (aVector1.z() * aVector2.x()) - (aVector1.x() * aVector2.z()), (aVector1.x() * aVector2.y()) - (aVector1.y() * aVector2.x()));
	}

	/**
	 * Answer vector length.
	 * 
	 * @param aVector sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return double
	 */
	protected double vectorLength_(Jun3dPoint aVector) {
		return Math.sqrt((aVector.x() * aVector.x()) + (aVector.y() * aVector.y()) + (aVector.z() * aVector.z()));
	}
}
