package sra.jun.opengl.flux;

import java.io.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.*;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.objects.*;
import sra.smalltalk.*;

/**
 * JunOpenGLFluxAbstract class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/11/25 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLFluxAbstract.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public abstract class JunOpenGLFluxAbstract extends StObject {

	/**
	 * Load a JunOpenGLFluxObject from the object.
	 * 
	 * @param anObject java.lang.Object
	 * 
	 * @return sra.jun.opengl.flux.JunOpenGLFluxObject
	 */
	public static JunOpenGLFluxObject LoadFrom_(Object anObject) {
		JunLispList aList = (JunLispList) JunLispParser.Parse_(anObject);

		if ((aList != null) && aList instanceof JunLispCons) {
			return JunOpenGLFluxObject.FromLispList_((JunLispCons) aList);
		} else {
			return null;
		}
	}

	/**
	 * Answer point array.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint[]
	 */
	public Jun3dPoint[] asPointArray() {
		return null;
	}

	/**
	 * Answer the bounding box of the receiver.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dBoundingBox
	 */
	public Jun3dBoundingBox boundingBox() {
		return this.preferredBoundingBox();
	}

	/**
	 * Flush my bounds.
	 */
	public void flushBounds() {
	}

	/**
	 * Answer the StSymbol which represents the kind of the receiver.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public abstract StSymbol kindName();

	/**
	 * Create an instance of JunLispCons.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons lispCons() {
		return (JunLispCons) JunLispCons.Cell();
	}

	/**
	 * Create an instance of JunLispNil.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispNil
	 */
	public JunLispNil lispNil() {
		return (JunLispNil) JunLispNil.NullList();
	}

	/**
	 * Enumerate every points and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public abstract void pointsDo_(StBlockClosure aBlock);

	/**
	 * Answer the preferred bounding box of the receiver.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dBoundingBox
	 */
	public abstract Jun3dBoundingBox preferredBoundingBox();

	/**
	 * Write the receiver's list representation on the Writer.
	 * 
	 * @param aWriter java.io.Writer
	 * 
	 * @throws IOException DOCUMENT ME!
	 */
	public void saveOn_(Writer aWriter) throws IOException {
		JunLispList aList = this.toLispList();
		aList.saveOn_(aWriter);
	}

	/**
	 * Popup a window which displays the Lisp List of the receiver.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList showLispList() {
		return this.showLispList_(this.toLispList());
	}

	/**
	 * Popup a window which displays the Lisp List.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public JunLispList showLispList_(JunLispList aList) {
		try {
			return (JunLispList) _PerformWith(this.getClass(), "ShowLispList_", aList);
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Convert the receiver as a lisp list.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public abstract JunLispList toLispList();

	/**
	 * Convert the transformation as a lisp list.
	 * 
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList transformationToLispList_(Jun3dTransformation aTransformation) {
		return JunOpenGL3dObject.TransformationToLispList_(aTransformation);
	}
}
