package sra.jun.opengl.flux;

import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.*;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.objects.*;
import sra.smalltalk.*;

/**
 * JunOpenGLFluxImmutableObject class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/11/25 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLFluxImmutableObject.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGLFluxImmutableObject extends JunOpenGLFluxAbstract {
	protected JunOpenGL3dObject originalObject;

	/**
	 * Create a JunOpenGLFluxImmutableObject from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * 
	 * @return sra.jun.opengl.flux.JunOpenGLFluxImmutableObject
	 */
	public static final JunOpenGLFluxImmutableObject FromLispList_(JunLispCons aList) {
		JunOpenGLFluxImmutableObject object = new JunOpenGLFluxImmutableObject();
		object = ObjectFromLispList_for_(aList, object);

		return object;
	}

	/**
	 * Create a new instance of JunOpenGLFluxImmutableObject with 3d object.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @return sra.jun.opengl.flux.JunOpenGLFluxImmutableObject
	 */
	public static JunOpenGLFluxImmutableObject Object_(JunOpenGL3dObject a3dObject) {
		JunOpenGLFluxImmutableObject mutableObject;

		mutableObject = new JunOpenGLFluxImmutableObject();
		mutableObject.originalObject_(a3dObject);

		return mutableObject;
	}

	/**
	 * Create a JunOpenGLFluxImmutableObject from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param anObject sra.jun.opengl.flux.JunOpenGLFluxImmutableObject
	 * 
	 * @return sra.jun.opengl.flux.JunOpenGLFluxImmutableObject
	 */
	public static JunOpenGLFluxImmutableObject ObjectFromLispList_for_(JunLispCons aList, final JunOpenGLFluxImmutableObject anObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object car) {
				return new Boolean(car instanceof JunLispCons && (((JunLispCons) car).head() == $("object")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return anObject;
		}

		anObject.originalObject_(JunOpenGL3dCompoundObject.ObjectFromLispList_((JunLispCons) list.tail()));

		return anObject;
	}

	/**
	 * Add a Jun3dTransformation.
	 * 
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public void add_(Jun3dTransformation aTransformation) {
	}

	/**
	 * Answer receiver at normalized number.
	 * 
	 * @param normalizedNumber double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject at_(double normalizedNumber) {
		if (this.originalObject() == null) {
			return null;
		}

		return this.originalObject();
	}

	/**
	 * Answer compound object.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 */
	public JunOpenGL3dCompoundObject compoundObject() {
		JunOpenGL3dCompoundObject aCompoundObject;

		aCompoundObject = new JunOpenGL3dCompoundObject();
		aCompoundObject.add_(this.originalObject());

		return aCompoundObject;
	}

	/**
	 * Answer true if the receiver is empty, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean isEmpty() {
		throw SmalltalkException.Error("Please use \"isEmptyTransformation\" instead of \"isEmpty\"");
	}

	/**
	 * Answer true if the receiver is empty transformation, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean isEmptyTransformation() {
		return true;
	}

	/**
	 * Answer the StSymbol which represents the kind of the receiver.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public StSymbol kindName() {
		return $("JunOpenGLFluxImmutableObject");
	}

	/**
	 * Convert the object as a lisp list.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons objectToLispList() {
		JunLispCons list;

		list = this.lispCons();
		list.head_($("object"));

		if (this.originalObject() != null) {
			list.tail_(this.originalObject().toLispList());
		}

		return list;
	}

	/**
	 * Answer original object.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject originalObject() {
		return originalObject;
	}

	/**
	 * Set original object.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public void originalObject_(JunOpenGL3dObject a3dObject) {
		originalObject = a3dObject;
		this.flushBounds();
	}

	/**
	 * Enumerate the points and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void pointsDo_(StBlockClosure aBlock) {
		if (this.originalObject() == null) {
			return;
		}

		this.originalObject().pointsDo_(aBlock);
	}

	/**
	 * Answer the preferred bounding box of the receiver.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dBoundingBox
	 */
	public Jun3dBoundingBox preferredBoundingBox() {
		return this.compoundObject().boundingBox();
	}

	/**
	 * Display receiver.
	 */
	public void show() {
		JunOpenGLFluxObject fluxObject;

		fluxObject = new JunOpenGLFluxObject();
		fluxObject.addImmutable_(this);
		fluxObject.show();
	}

	/**
	 * Answer the number of all elements.
	 * 
	 * @return int
	 */
	public int size() {
		return 0;
	}

	/**
	 * Convert the receiver as a lisp list.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList toLispList() {
		JunLispCons list;

		list = this.lispCons();
		list.head_(this.kindName());
		list.add_(this.objectToLispList());

		return list;
	}

	/**
	 * Answer transformation sequence.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject[]
	 */
	public JunOpenGL3dObject[] transformationSequence() {
		return new JunOpenGL3dObject[] {
		};
	}
}
