package sra.jun.opengl.flux;

import java.util.*;
import sra.jun.geometry.abstracts.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.curves.*;
import sra.jun.geometry.surfaces.*;
import sra.jun.geometry.transformations.*;
import sra.smalltalk.*;

/**
 * JunOpenGLFluxMaker class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/12/02 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLFluxMaker.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGLFluxMaker extends StObject {
	Vector lineSegments; // (Jun3dPoint[2])[]
	JunCoordinateAngles originalCoordinateAngles;
	JunCoordinateAngles[] coordinateAnglesCollection;
	Jun3dTransformation[] rotationTransformations;
	Jun3dTransformation[] revisionTransformations;
	Jun3dTransformation[] scalingTransformations;
	Jun3dTransformation[] translationTransformations;
	Jun3dTransformation[] segmentTransformations;

	/**
	 * Create a new instance of JunOpenGLFluxMaker.
	 */
	public JunOpenGLFluxMaker() {
		super();
		this.initialize();
	}

	/**
	 * Create a new instance of JunOpenGLFluxMaker with line segments.
	 * 
	 * @param lineSegments java.util.Vector
	 * 
	 * @return sra.jun.opengl.flux.JunOpenGLFluxMaker
	 */
	public static JunOpenGLFluxMaker LineSegments_(Vector lineSegments) {
		JunOpenGLFluxMaker instance;

		instance = new JunOpenGLFluxMaker();
		instance.lineSegments_(lineSegments);

		return instance;
	}

	/**
	 * Add a segment.
	 * 
	 * @param aSegment sra.jun.geometry.basic.Jun3dPoint[]
	 */
	public void add_(Jun3dPoint[] aSegment) {
		this.lineSegments().addElement(aSegment);
		this.flushTransformations();
	}

	/**
	 * Answer coordinateAnglesCollection.
	 * 
	 * @return sra.jun.opengl.flux.JunCoordinateAngles[]
	 */
	public JunCoordinateAngles[] coordinateAnglesCollection() {
		if (coordinateAnglesCollection == null) {
			this.computeTransformations();
		}

		return coordinateAnglesCollection;
	}

	/**
	 * Evaluate a Block for debug.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void debug_(StBlockClosure aBlock) {
		//aBlock.value();
	}

	/**
	 * Answer directionVector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint directionVector() {
		return new Jun3dPoint(1, 0, 0);
	}

	/**
	 * Initialize.
	 */
	public void initialize() {
		lineSegments = new Vector();
		this.flushTransformations();
	}

	/**
	 * Answer lineSegments.
	 * 
	 * @return java.util.Vector
	 */
	public final Vector lineSegments() {
		return lineSegments;
	}

	/**
	 * Set lineSegments.
	 * 
	 * @param segmentCollection java.util.Vector
	 */
	public void lineSegments_(Vector segmentCollection) {
		lineSegments = new Vector();

		for (int i = 0; i < segmentCollection.size(); i++) {
			Jun3dPoint[] each = (Jun3dPoint[]) segmentCollection.elementAt(i);
			this.add_(each);
		}
	}

	/**
	 * Answer originalCoordinateAngles.
	 * 
	 * @return sra.jun.opengl.flux.JunCoordinateAngles
	 */
	public JunCoordinateAngles originalCoordinateAngles() {
		if (originalCoordinateAngles == null) {
			this.computeTransformations();
		}

		return originalCoordinateAngles;
	}

	/**
	 * Answer revisionTransformations.
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation[]
	 */
	public Jun3dTransformation[] revisionTransformations() {
		if (revisionTransformations == null) {
			this.computeTransformations();
		}

		return revisionTransformations;
	}

	/**
	 * Answer rotationTransformations.
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation[]
	 */
	public Jun3dTransformation[] rotationTransformations() {
		if (rotationTransformations == null) {
			this.computeTransformations();
		}

		return rotationTransformations;
	}

	/**
	 * Answer scalingTransformations.
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation[]
	 */
	public Jun3dTransformation[] scalingTransformations() {
		if (scalingTransformations == null) {
			this.computeTransformations();
		}

		return scalingTransformations;
	}

	/**
	 * Answer segmentTransformations.
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation[]
	 */
	public Jun3dTransformation[] segmentTransformations() {
		if (segmentTransformations == null) {
			this.computeTransformations();
		}

		return segmentTransformations;
	}

	/**
	 * Answer translationTransformations.
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation[]
	 */
	public Jun3dTransformation[] translationTransformations() {
		if (translationTransformations == null) {
			this.computeTransformations();
		}

		return translationTransformations;
	}

	/**
	 * Answer unit vector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint unitVector() {
		return new Jun3dPoint(0, 0, 1);
	}

	/**
	 * Compute transformations.
	 */
	protected void computeTransformations() {
		Jun3dPoint[] lineSegment;
		Jun3dPoint aPoint0;
		Jun3dPoint aPoint1;
		Jun3dPoint aVector;
		JunCoordinateAngles coordinateAngles;
		Jun3dTransformation rotationT;
		Jun3dTransformation scalingT;
		Jun3dTransformation translationT;
		Jun3dTransformation aTransformation;
		Jun3dTransformation revisionT;
		double aLength;
		JunPlane aPlane;
		Jun3dLine rotationLine;

		originalCoordinateAngles = JunCoordinateAngles.V1_v2_(this.unitVector(), this.directionVector());
		coordinateAnglesCollection = new JunCoordinateAngles[this.lineSegments().size()];
		rotationTransformations = new Jun3dTransformation[this.lineSegments().size()];
		revisionTransformations = new Jun3dTransformation[this.lineSegments().size()];
		scalingTransformations = new Jun3dTransformation[this.lineSegments().size()];
		translationTransformations = new Jun3dTransformation[this.lineSegments().size()];
		segmentTransformations = new Jun3dTransformation[this.lineSegments().size()];

		for (int index = 0; index < this.lineSegments().size(); index++) {
			lineSegment = (Jun3dPoint[]) lineSegments.elementAt(index);
			aPoint0 = lineSegment[0];
			aPoint1 = lineSegment[1];
			aLength = aPoint0.distance_(aPoint1);
			coordinateAngles = JunCoordinateAngles.V1_v2_((Jun3dPoint) aPoint1.minus_(aPoint0), this.directionVector());
			coordinateAnglesCollection[index] = coordinateAngles;
			rotationT = Jun3dTransformation.Unity();
			rotationT = (Jun3dTransformation) rotationT.product_(originalCoordinateAngles.asInverseTransformation());
			rotationT = (Jun3dTransformation) rotationT.product_(coordinateAngles.asTransformation());
			rotationTransformations[index] = rotationT;
			aVector = (Jun3dPoint) this.directionVector().transform_(rotationT);
			aPlane = new JunPlane(new Jun3dPoint(0, 0, 0), this.unitVector(), this.directionVector());
			rotationLine = new Jun3dLine(new Jun3dPoint(0, 0, 0), (Jun3dPoint) this.unitVector().transform_(rotationT));
			revisionT = this.transformationToBeParallel_and_byRotationAround_(aPlane, aVector, rotationLine);
			lineSegment = (Jun3dPoint[]) lineSegments.elementAt(index);
			aPoint0 = lineSegment[0];
			aPoint1 = lineSegment[1];

			if (aPoint0.z() < aPoint1.z()) {
				Jun3dTransformation verticalT;
				verticalT = Jun3dTransformation.Rotate_around_(JunAngle.FromDeg_(180), rotationLine);
				revisionT = (Jun3dTransformation) revisionT.product_(verticalT);
			}

			revisionTransformations[index] = revisionT;
			scalingT = Jun3dTransformation.Scale_(new Jun3dPoint(aLength, aLength, aLength));
			scalingTransformations[index] = scalingT;
			translationT = Jun3dTransformation.Translate_(aPoint0);
			translationTransformations[index] = translationT;
			aTransformation = Jun3dTransformation.Unity();
			aTransformation = (Jun3dTransformation) aTransformation.product_(scalingT);
			aTransformation = (Jun3dTransformation) aTransformation.product_(rotationT);
			aTransformation = (Jun3dTransformation) aTransformation.product_(revisionT);
			aTransformation = (Jun3dTransformation) aTransformation.product_(translationT);
			segmentTransformations[index] = aTransformation;
		}

		return;
	}

	/**
	 * Flush transformations.
	 */
	protected void flushTransformations() {
		originalCoordinateAngles = null;
		coordinateAnglesCollection = null;
		rotationTransformations = null;
		revisionTransformations = null;
		scalingTransformations = null;
		translationTransformations = null;
		segmentTransformations = null;
	}

	/**
	 * Transformation to be parallel JunPlane and vector by rotation around
	 * rotationLine.
	 * 
	 * @param aPlane sra.jun.geometry.surfaces.JunPlane
	 * @param aVector sra.jun.geometry.basic.Jun3dPoint
	 * @param rotationLine sra.jun.geometry.curves.Jun3dLine
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation
	 */
	protected Jun3dTransformation transformationToBeParallel_and_byRotationAround_(final JunPlane aPlane, final Jun3dPoint aVector, final Jun3dLine rotationLine) {
		final StValueHolder revisionT = new StValueHolder(); // Jun3dTransformation
		final StValueHolder anAngle = new StValueHolder(); // JunAngle
		final StValueHolder aPoint = new StValueHolder(); // Jun3dPoint
		final StValueHolder aLine = new StValueHolder(); // Jun3dLine
		StBlockClosure convergeBlock;
		int accuracyPlace;

		revisionT.setValue_(Jun3dTransformation.Unity());
		anAngle.setValue_(JunAngle.FromDeg_(0));
		aPoint.setValue_(aVector);
		aLine.setValue_(new Jun3dLine(new Jun3dPoint(0, 0, 0), (Jun3dPoint) aPoint.value()));
		convergeBlock = new StBlockClosure() {
			public Object value_(Object o) {
				double stepDegrees = ((Number) o).doubleValue();

				//this.debug_();
				while (Math.abs(aPlane.angleWithLine_((Jun3dLine) aLine.value()).deg()) > stepDegrees) {
					anAngle.setValue_(JunAngle.FromDeg_(((JunAngle) anAngle.value()).deg() + stepDegrees));
					revisionT.setValue_(Jun3dTransformation.Rotate_around_((JunAngle) anAngle.value(), rotationLine));
					aPoint.setValue_(aVector.transform_((Jun3dTransformation) revisionT.value()));
					aLine.setValue_(new Jun3dLine(new Jun3dPoint(0, 0, 0), (Jun3dPoint) aPoint.value()));
				}

				return anAngle;
			}
		};

		//this.debug_();
		final double[] _steps = { 90.0d, 45.0d, 10.0d, 5.0d, 1.0d, 0.5d };

		for (int i = 0; i < _steps.length; i++) {
			double step = _steps[i];

			//this.debug_();
			convergeBlock.value_(new Double(step));
		}

		accuracyPlace = (int) Math.round(Math.abs(Math.log(JunGeometry.Accuracy()) / Math.log(10))) - 1;

		for (int it = 1; it <= accuracyPlace; it++) {
			double denominator;
			denominator = Math.pow(10.0d, it);

			final double[] _eachs = { 1.0d, 0.5d };

			for (int i = 0; i < _eachs.length; i++) {
				double each = _eachs[i];
				double step = each / denominator;

				//this.debug_();
				convergeBlock.value_(new Double(step));
			}
		}

		if (((Jun3dPoint) aPoint.value()).x() < 0) {
			anAngle.setValue_(JunAngle.FromDeg_(180).minus_((JunAngle) anAngle.value()).negated());
			revisionT.setValue_(Jun3dTransformation.Rotate_around_((JunAngle) anAngle.value(), rotationLine));
		}

		return (Jun3dTransformation) revisionT.value();
	}
}
