package sra.jun.opengl.flux;

import java.util.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.*;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.objects.*;
import sra.smalltalk.*;

/**
 * JunOpenGLFluxMutableObject class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/11/26 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLFluxMutableObject.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGLFluxMutableObject extends JunOpenGLFluxAbstract {
	protected JunOpenGL3dObject originalObject;
	protected Vector transformationSequence; // Jun3dTransformation[]
	protected Jun3dBoundingBox boundingBox;

	/**
	 * Create a JunOpenGLFluxMutableObject from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * 
	 * @return sra.jun.opengl.flux.JunOpenGLFluxMutableObject
	 */
	public static final JunOpenGLFluxMutableObject FromLispList_(JunLispCons aList) {
		JunOpenGLFluxMutableObject object = new JunOpenGLFluxMutableObject();
		object = ObjectFromLispList_for_(aList, object);
		object = TransformationsFromLispList_for_(aList, object);

		return object;
	}

	/**
	 * Create a new JunOpenGLFluxMutableObject with 3d object.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @return sra.jun.opengl.flux.JunOpenGLFluxMutableObject
	 */
	public static JunOpenGLFluxMutableObject Object_(JunOpenGL3dObject a3dObject) {
		JunOpenGLFluxMutableObject mutableObject;

		mutableObject = new JunOpenGLFluxMutableObject();
		mutableObject.originalObject_(a3dObject);

		return mutableObject;
	}

	/**
	 * Create a JunOpenGLFluxMutableObject from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param anObject sra.jun.opengl.flux.JunOpenGLFluxMutableObject
	 * 
	 * @return sra.jun.opengl.flux.JunOpenGLFluxMutableObject
	 */
	public static JunOpenGLFluxMutableObject ObjectFromLispList_for_(JunLispCons aList, final JunOpenGLFluxMutableObject anObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object car) {
				return new Boolean(car instanceof JunLispCons && (((JunLispCons) car).head() == $("object")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return anObject;
		}

		anObject.originalObject_((JunOpenGL3dCompoundObject) JunOpenGL3dCompoundObject.ObjectFromLispList_((JunLispCons) list.tail()));

		return anObject;
	}

	/**
	 * Create a JunOpenGLFluxMutableObject from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param anObject sra.jun.opengl.flux.JunOpenGLFluxMutableObject
	 * 
	 * @return sra.jun.opengl.flux.JunOpenGLFluxMutableObject
	 */
	public static JunOpenGLFluxMutableObject TransformationsFromLispList_for_(JunLispCons aList, final JunOpenGLFluxMutableObject anObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object car) {
				return new Boolean(car instanceof JunLispCons && (((JunLispCons) car).head() == $("transformations")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return anObject;
		}

		((JunLispList) list.tail()).do_(new StBlockClosure() {
			public Object value_(Object object) {
				JunLispCons each = (JunLispCons) object;
				JunLispCons cons = JunLispCons.Cell();
				cons.head_(each);

				Jun3dTransformation transformation = JunOpenGL3dObject.TransformationFromLispList_(cons);
				anObject.add_(transformation);

				return null;
			}
		});

		return anObject;
	}

	/**
	 * Add a transformation.
	 * 
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public void add_(Jun3dTransformation aTransformation) {
		this.transformationSequence().addElement(aTransformation);
	}

	/**
	 * Answer receiver at normalized number.
	 * 
	 * @param normalizedNumber double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject at_(double normalizedNumber) {
		int index;
		Jun3dTransformation transformation;

		if (this.originalObject() == null) {
			return null;
		}

		index = (int) Math.round(normalizedNumber * (this.size() - 1)) + 1;

		if (this.isEmptyTransformation()) {
			if (this.originalObject() instanceof JunOpenGL3dCompoundObject) {
				return ((JunOpenGL3dCompoundObject) this.originalObject()).components()[index - 1];
			} else {
				return this.originalObject();
			}
		} else {
			if ((0 < index) && (index <= this.size())) {
				transformation = (Jun3dTransformation) this.transformationSequence().elementAt(index - 1);

				return JunOpenGL3dTransformedObject.Object_transformation_(this.originalObject(), transformation);
			} else {
				return this.originalObject();
			}
		}
	}

	/**
	 * Answer the bounding box of the receiver.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dBoundingBox
	 */
	public Jun3dBoundingBox boundingBox() {
		if (boundingBox == null) {
			boundingBox = this.preferredBoundingBox();
		}

		return boundingBox;
	}

	/**
	 * Answer compound object.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dCompoundObject compoundObject() {
		JunOpenGL3dCompoundObject compoundObject;

		if (this.isEmptyTransformation()) {
			if (this.originalObject() instanceof JunOpenGL3dCompoundObject) {
				compoundObject = (JunOpenGL3dCompoundObject) this.originalObject();
			} else {
				compoundObject = new JunOpenGL3dCompoundObject();
				compoundObject.add_(this.originalObject());
			}
		} else {
			compoundObject = new JunOpenGL3dCompoundObject();

			for (int i = 0; i < this.transformationSequence().size(); i++) {
				Jun3dTransformation transformation = (Jun3dTransformation) this.transformationSequence().elementAt(i);
				JunOpenGL3dObject body = this.originalObject().transform_(transformation);
				compoundObject.add_(body);
			}
		}

		return compoundObject;
	}

	/**
	 * Flush my bounds.
	 */
	public void flushBounds() {
		boundingBox = null;
	}

	/**
	 * Answer true if the receiver is empty, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean isEmpty() {
		throw SmalltalkException.Error("Please use \"isEmptyTransformation\" instead of \"isEmpty\"");
	}

	/**
	 * Answer true if the receiver is empty transformation, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean isEmptyTransformation() {
		return (this.transformationSequence().size() < 1);
	}

	/**
	 * Answer the StSymbol which represents the kind of the receiver.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public StSymbol kindName() {
		return $("JunOpenGLFluxMutableObject");
	}

	/**
	 * Convert the object as a lisp list.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons objectToLispList() {
		JunLispCons list;

		list = this.lispCons();
		list.head_($("object"));

		if (this.originalObject() != null) {
			list.tail_(this.originalObject().toLispList());
		}

		return list;
	}

	/**
	 * Answer original object.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject originalObject() {
		return originalObject;
	}

	/**
	 * Set original object.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public void originalObject_(JunOpenGL3dObject a3dObject) {
		originalObject = a3dObject;
		this.flushBounds();
	}

	/**
	 * Enumerate the points and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void pointsDo_(final StBlockClosure aBlock) {
		if (this.originalObject() == null) {
			return;
		}

		if (this.isEmptyTransformation()) {
			this.originalObject().pointsDo_(aBlock);
		} else {
			for (int i = 0; i < this.transformationSequence().size(); i++) {
				final Jun3dTransformation transformation = (Jun3dTransformation) this.transformationSequence().elementAt(i);
				this.originalObject().pointsDo_(new StBlockClosure() {
					public Object value_(Object o) {
						Jun3dPoint point = (Jun3dPoint) o;
						aBlock.value_(point.transform_(transformation));

						return null;
					}
				});
			}
		}
	}

	/**
	 * Answer the preferred bounding box of the receiver.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dBoundingBox
	 */
	public Jun3dBoundingBox preferredBoundingBox() {
		Jun3dBoundingBox aBox;

		if (this.isEmptyTransformation()) {
			aBox = this.originalObject().boundingBox();
		} else {
			final StValueHolder minX = new StValueHolder(Double.POSITIVE_INFINITY);
			final StValueHolder minY = new StValueHolder(Double.POSITIVE_INFINITY);
			final StValueHolder minZ = new StValueHolder(Double.POSITIVE_INFINITY);
			final StValueHolder maxX = new StValueHolder(Double.NEGATIVE_INFINITY);
			final StValueHolder maxY = new StValueHolder(Double.NEGATIVE_INFINITY);
			final StValueHolder maxZ = new StValueHolder(Double.NEGATIVE_INFINITY);
			this.pointsDo_(new StBlockClosure() {
				public Object value_(Object anObject) {
					Jun3dPoint aPoint = (Jun3dPoint) anObject;
					minX.value_(Math.min(minX._doubleValue(), aPoint.x()));
					minY.value_(Math.min(minY._doubleValue(), aPoint.y()));
					minZ.value_(Math.min(minZ._doubleValue(), aPoint.z()));
					maxX.value_(Math.max(maxX._doubleValue(), aPoint.x()));
					maxY.value_(Math.max(maxY._doubleValue(), aPoint.y()));
					maxZ.value_(Math.max(maxZ._doubleValue(), aPoint.z()));

					return null;
				}
			});

			Jun3dPoint originPoint = new Jun3dPoint(minX._doubleValue(), minY._doubleValue(), minZ._doubleValue());
			Jun3dPoint cornerPoint = new Jun3dPoint(maxX._doubleValue(), maxY._doubleValue(), maxZ._doubleValue());
			aBox = (Jun3dBoundingBox) Jun3dBoundingBox.Origin_corner_(originPoint, cornerPoint);
		}

		return aBox;
	}

	/**
	 * Display receiver.
	 */
	public void show() {
		JunOpenGLFluxObject fluxObject;

		fluxObject = new JunOpenGLFluxObject();
		fluxObject.addMutable_(this);
		fluxObject.show();
	}

	/**
	 * Answer the number of elements of the receiver.
	 * 
	 * @return int
	 */
	public int size() {
		if (this.originalObject() == null) {
			return 0;
		}

		if (this.isEmptyTransformation()) {
			if (this.originalObject() instanceof JunOpenGL3dCompoundObject) {
				return ((JunOpenGL3dCompoundObject) this.originalObject()).components().length;
			} else {
				return 1;
			}
		} else {
			return this.transformationSequence().size();
		}
	}

	/**
	 * Convert the receiver as a lisp list.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList toLispList() {
		JunLispCons list;

		list = this.lispCons();
		list.head_(this.kindName());
		list.add_(this.objectToLispList());
		list.add_(this.transformationsToLispList());

		return list;
	}

	/**
	 * Answer transformation sequence.
	 * 
	 * @return java.util.Vector
	 */
	public Vector transformationSequence() {
		if (transformationSequence == null) {
			transformationSequence = new Vector();
		}

		return transformationSequence;
	}

	/**
	 * Answer the transformations to LispList
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList transformationsToLispList() {
		JunLispCons list;

		list = this.lispCons();
		list.head_($("transformations"));

		for (int i = 0; i < this.transformationSequence().size(); i++) {
			Jun3dTransformation transformation = (Jun3dTransformation) this.transformationSequence().elementAt(i);
			list.add_(this.transformationToLispList_(transformation));
		}

		return list;
	}
}
