package sra.jun.opengl.flux;

import java.util.*;
import sra.jun.geometry.basic.*;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.objects.*;
import sra.smalltalk.*;

/**
 * JunOpenGLFluxObject class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/11/27 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLFluxObject.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGLFluxObject extends JunOpenGLFluxAbstract {
	protected Vector immutableObjects;
	protected Vector mutableObjects;
	protected Jun3dBoundingBox boundingBox;

	/**
	 * Create a JunOpenGLFluxObject from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * 
	 * @return sra.jun.opengl.flux.JunOpenGLFluxObject
	 */
	public static JunOpenGLFluxObject FromLispList_(JunLispCons aList) {
		JunOpenGLFluxObject object = new JunOpenGLFluxObject();
		object = JunOpenGLFluxObject.ImmutableObjectsFromLispList_for_(aList, object);
		object = JunOpenGLFluxObject.MutableObjectsFromLispList_for_(aList, object);

		return object;
	}

	/**
	 * Create a JunOpenGLFluxObject from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param anObject sra.jun.opengl.flux.JunOpenGLFluxObject
	 * 
	 * @return sra.jun.opengl.flux.JunOpenGLFluxObject
	 */
	public static JunOpenGLFluxObject ImmutableObjectsFromLispList_for_(JunLispCons aList, final JunOpenGLFluxObject anObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object car) {
				return new Boolean(car instanceof JunLispCons && (((JunLispCons) car).head() == $("immutables")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return anObject;
		}

		((JunLispList) list.tail()).do_(new StBlockClosure() {
			public Object value_(Object object) {
				JunLispCons each = (JunLispCons) object;
				anObject.addImmutable_(JunOpenGLFluxImmutableObject.FromLispList_(each));

				return null;
			}
		});

		return anObject;
	}

	/**
	 * Create a JunOpenGLFluxObject from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param anObject sra.jun.opengl.flux.JunOpenGLFluxObject
	 * 
	 * @return sra.jun.opengl.flux.JunOpenGLFluxObject
	 */
	public static JunOpenGLFluxObject MutableObjectsFromLispList_for_(JunLispCons aList, final JunOpenGLFluxObject anObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object car) {
				return new Boolean(car instanceof JunLispCons && (((JunLispCons) car).head() == $("mutables")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return anObject;
		}

		((JunLispCons) list.tail()).do_(new StBlockClosure() {
			public Object value_(Object object) {
				JunLispCons each = (JunLispCons) object;
				anObject.addMutable_(JunOpenGLFluxMutableObject.FromLispList_(each));

				return null;
			}
		});

		return anObject;
	}

	/**
	 * Add the immutable object.
	 * 
	 * @param immutableObject sra.jun.opengl.flux.JunOpenGLFluxImmutableObject
	 */
	public void addImmutable_(JunOpenGLFluxImmutableObject immutableObject) {
		if (this.immutableObjects().contains(immutableObject) == false) {
			this.immutableObjects().addElement(immutableObject);
			this.flushBounds();
		}
	}

	/**
	 * Add the mutable object.
	 * 
	 * @param mutableObject sra.jun.opengl.flux.JunOpenGLFluxMutableObject
	 */
	public void addMutable_(JunOpenGLFluxMutableObject mutableObject) {
		if (this.mutableObjects().contains(mutableObject) == false) {
			this.mutableObjects().addElement(mutableObject);
			this.flushBounds();
		}
	}

	/**
	 * Answer reciver at normalized number.
	 * 
	 * @param normalizedNumber double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 */
	public JunOpenGL3dCompoundObject at_(double normalizedNumber) {
		JunOpenGL3dCompoundObject compoundObject;

		compoundObject = new JunOpenGL3dCompoundObject();

		for (int i = 0; i < this.immutableObjects().size(); i++) {
			JunOpenGLFluxImmutableObject each = (JunOpenGLFluxImmutableObject) this.immutableObjects().elementAt(i);
			JunOpenGL3dObject object = each.at_(normalizedNumber);

			if (object != null) {
				compoundObject.add_(object);
			}
		}

		for (int i = 0; i < this.mutableObjects().size(); i++) {
			JunOpenGLFluxMutableObject each = (JunOpenGLFluxMutableObject) this.mutableObjects().elementAt(i);
			JunOpenGL3dObject object = each.at_(normalizedNumber);

			if (object != null) {
				compoundObject.add_(object);
			}
		}

		return compoundObject;
	}

	/**
	 * Answer the bounding box of the receiver.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dBoundingBox
	 */
	public Jun3dBoundingBox boundingBox() {
		if (boundingBox == null) {
			boundingBox = this.preferredBoundingBox();
		}

		return boundingBox;
	}

	/**
	 * Anseer the compound object.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 */
	public JunOpenGL3dCompoundObject compoundObject() {
		JunOpenGL3dCompoundObject compoundObject;

		compoundObject = new JunOpenGL3dCompoundObject();

		for (int i = 0; i < this.immutableObjects().size(); i++) {
			JunOpenGLFluxImmutableObject each = (JunOpenGLFluxImmutableObject) this.immutableObjects().elementAt(i);
			compoundObject.add_(each.compoundObject());
		}

		for (int i = 0; i < this.mutableObjects().size(); i++) {
			JunOpenGLFluxMutableObject each = (JunOpenGLFluxMutableObject) this.mutableObjects().elementAt(i);
			compoundObject.add_(each.compoundObject());
		}

		return compoundObject;
	}

	/**
	 * Flush my bounds.
	 */
	public void flushBounds() {
		boundingBox = null;
	}

	/**
	 * Answer the immutable objects.
	 * 
	 * @return java.util.Vector
	 */
	public Vector immutableObjects() {
		if (immutableObjects == null) {
			immutableObjects = new Vector();
		}

		return immutableObjects;
	}

	/**
	 * Answer the immutable objects to LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList immutableObjectsToLispList() {
		JunLispCons list;

		list = this.lispCons();
		list.head_($("immutables"));

		for (int i = 0; i < this.immutableObjects().size(); i++) {
			JunOpenGLFluxImmutableObject each = (JunOpenGLFluxImmutableObject) this.immutableObjects().elementAt(i);
			list.add_(each.toLispList());
		}

		return list;
	}

	/**
	 * Answer the StSymbol which represents the kind of the receiver.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public StSymbol kindName() {
		return $("JunOpenGLFluxObject");
	}

	/**
	 * Answer the mutable objects.
	 * 
	 * @return java.util.Vector
	 */
	public Vector mutableObjects() {
		if (mutableObjects == null) {
			mutableObjects = new Vector();
		}

		return mutableObjects;
	}

	/**
	 * Answer the mutable objects to LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList mutableObjectsToLispList() {
		JunLispCons list;

		list = this.lispCons();
		list.head_($("mutables"));

		for (int i = 0; i < this.mutableObjects().size(); i++) {
			JunOpenGLFluxMutableObject each = (JunOpenGLFluxMutableObject) this.mutableObjects().elementAt(i);
			list.add_(each.toLispList());
		}

		return list;
	}

	/**
	 * Answer the name.
	 * 
	 * @return java.lang.String
	 */
	public String name() {
		return JunOpenGL3dObject.UniqueId();
	}

	/**
	 * Enumerate the points and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void pointsDo_(StBlockClosure aBlock) {
		for (int i = 0; i < this.immutableObjects().size(); i++) {
			JunOpenGLFluxImmutableObject each = (JunOpenGLFluxImmutableObject) this.immutableObjects().elementAt(i);
			each.pointsDo_(aBlock);
		}

		for (int i = 0; i < this.mutableObjects().size(); i++) {
			JunOpenGLFluxMutableObject each = (JunOpenGLFluxMutableObject) this.mutableObjects().elementAt(i);
			each.pointsDo_(aBlock);
		}
	}

	/**
	 * Answer the preferred bounding box of the receiver.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dBoundingBox
	 */
	public Jun3dBoundingBox preferredBoundingBox() {
		Jun3dBoundingBox aBox;
		aBox = null;

		for (int i = 0; i < this.immutableObjects().size(); i++) {
			JunOpenGLFluxImmutableObject each = (JunOpenGLFluxImmutableObject) this.immutableObjects().elementAt(i);

			if (aBox == null) {
				aBox = each.boundingBox();
			} else {
				aBox = (Jun3dBoundingBox) aBox.merge_(each.boundingBox());
			}
		}

		for (int i = 0; i < this.mutableObjects().size(); i++) {
			JunOpenGLFluxMutableObject each = (JunOpenGLFluxMutableObject) this.mutableObjects().elementAt(i);

			if (aBox == null) {
				aBox = each.boundingBox();
			} else {
				aBox = (Jun3dBoundingBox) aBox.merge_(each.boundingBox());
			}
		}

		if (aBox == null) {
			aBox = Jun3dBoundingBox.Origin_corner_(new Jun3dPoint(0, 0, 0), new Jun3dPoint(0, 0, 0));
		}

		return aBox;
	}

	/**
	 * Display receiver.
	 */
	public void show() {
		JunOpenGLFluxModel.FluxObject_(this).open();
	}

	/**
	 * Answer the number of elements of the receiver.
	 * 
	 * @return int
	 */
	public int size() {
		Integer size;

		size = null;

		for (int i = 0; i < this.mutableObjects().size(); i++) {
			JunOpenGLFluxMutableObject each = (JunOpenGLFluxMutableObject) this.mutableObjects().elementAt(i);

			if (size == null) {
				size = new Integer(each.size());
			} else {
				size = new Integer(Math.min(each.size(), size.intValue()));
			}
		}

		if (size == null) {
			size = new Integer(0);
		}

		return size.intValue();
	}

	/**
	 * Convert the receiver as a lisp list.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList toLispList() {
		JunLispCons list;

		list = this.lispCons();
		list.head_(this.kindName());
		list.add_(this.immutableObjectsToLispList());
		list.add_(this.mutableObjectsToLispList());

		return list;
	}
}
