package sra.jun.opengl.flux;

import java.awt.*;
import javax.swing.JPanel;
import sra.smalltalk.*;
import sra.jun.goodies.button.JunButtonViewSwing;
import sra.jun.goodies.track.JunTrackSliderViewSwing;
import sra.jun.opengl.display.JunOpenGLDisplayViewSwing;

/**
 * JunOpenGLFluxViewSwing class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   2000/01/20 (by MATSUDA Ryouichi)
 *  @updated   2003/03/24 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLFluxViewSwing.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGLFluxViewSwing extends StViewJPanel implements JunOpenGLFluxView {

	protected JunOpenGLDisplayViewSwing displayView;
	protected JunButtonViewSwing playButtonView;
	protected JunButtonViewSwing loopButtonView;
	protected JunTrackSliderViewSwing sliderView;
	protected JunButtonViewSwing previousButtonView;
	protected JunButtonViewSwing nextButtonView;

	/**
	 * Create a new instance of JunOpenGLFluxViewSwing and initialize it.
	 *
	 * @param aFluxModel sra.jun.opengl.flux.JunOpenGLFluxModel
	 */
	public JunOpenGLFluxViewSwing(JunOpenGLFluxModel aFluxModel) {
		super(aFluxModel);
	}

	/**
	 * model property accessing.
	 * 
	 * @return sra.jun.opengl.flux.JunOpenGLFluxModel
	 * @deprecated since Jun379, use getOpenGLFluxModel()
	 */
	public JunOpenGLFluxModel getModel() {
		return (JunOpenGLFluxModel) this.model();
	}

	/**
	 * Answer my model as JunOpenGLFluxModel.
	 * 
	 * @return sra.jun.opengl.flux.JunOpenGLFluxModel
	 */
	public JunOpenGLFluxModel getOpenGLFluxModel() {
		return (JunOpenGLFluxModel) this.model();
	}

	/**
	 * Call noticeOfComponentRemove() before removeNotify().
	 */
	public void removeNotify() {
		getModel().end();
		super.removeNotify();
	}

	/**
	 * model property accessing.
	 * 
	 * @param aModel sra.jun.opengl.flux.JunOpenGLFluxModel
	 * @deprecated since Jun379
	 */
	public void setModel(JunOpenGLFluxModel aModel) {
		this.model_(aModel);
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(new BorderLayout());

		final Dimension buttonSize = new Dimension(18, 18);

		// center
		displayView = new JunOpenGLDisplayViewSwing(this.getOpenGLFluxModel());
		this.add(displayView, "Center");

		// buttom
		JPanel trackSliderPanel = new JPanel(new BorderLayout());

		//
		JPanel westPanel = new JPanel(new GridLayout(1, 2));
		playButtonView = new JunButtonViewSwing(this.getOpenGLFluxModel().trackerModel().playButton());
		playButtonView.setPreferredSize(buttonSize);
		westPanel.add(playButtonView);
		loopButtonView = new JunButtonViewSwing(this.getOpenGLFluxModel().trackerModel().loopButton());
		loopButtonView.setPreferredSize(buttonSize);
		westPanel.add(loopButtonView);
		trackSliderPanel.add("West", westPanel);

		//
		JPanel centerPanel = new JPanel(new BorderLayout());
		sliderView = new JunTrackSliderViewSwing(this.getOpenGLFluxModel().trackerModel());
		centerPanel.add(sliderView);
		trackSliderPanel.add("Center", centerPanel);

		//
		JPanel eastPanel = new JPanel(new GridLayout(1, 2));
		previousButtonView = new JunButtonViewSwing(this.getOpenGLFluxModel().trackerModel().previousButton());
		previousButtonView.setPreferredSize(buttonSize);
		eastPanel.add(previousButtonView);
		nextButtonView = new JunButtonViewSwing(this.getOpenGLFluxModel().trackerModel().nextButton());
		nextButtonView.setPreferredSize(buttonSize);
		eastPanel.add(nextButtonView);
		trackSliderPanel.add("East", eastPanel);

		//
		this.add(trackSliderPanel, "South");
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunOpenGLFluxModel();
	}
}
