package sra.jun.opengl.grapher;

import java.awt.Color;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.*;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.objects.*;
import sra.smalltalk.*;

/**
 * JunOpenGL3dArc class
 * 
 *  @author    Hirotsugu Kondo
 *  @created   1998/12/07 (by Hirotsugu Kondo)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dArc.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGL3dArc extends JunOpenGL3dGraphAbstract {
	private boolean crankArc;
	private JunOpenGL3dNode fromNode;
	private JunOpenGL3dNode toNode;

	/**
	 * Constainer.
	 */
	public JunOpenGL3dArc() {
		this.crankArc = false;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param fromNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * @param toNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dArc
	 */
	public static JunOpenGL3dArc From_to_(JunOpenGL3dNode fromNode, JunOpenGL3dNode toNode) {
		return From_to_color_(fromNode, toNode, Color.gray);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param fromNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * @param toNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * @param colorValue java.awt.Color
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dArc
	 */
	public static JunOpenGL3dArc From_to_color_(JunOpenGL3dNode fromNode, JunOpenGL3dNode toNode, Color colorValue) {
		JunOpenGL3dArc arc = new JunOpenGL3dArc();
		arc.from_to_(fromNode, toNode);
		arc.color_(colorValue);

		return arc;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dGraphAbstract
	 */
	public static JunOpenGL3dGraphAbstract FromLispList_(JunLispList aList) {
		JunOpenGL3dArc arc = new JunOpenGL3dArc();
		arc = (JunOpenGL3dArc) ColorFromLispList_for_(aList, arc);
		arc = CrankFromLispList_for_(aList, arc);
		arc = FromNodeFromLispList_for_(aList, arc);
		arc = ToNodeFromLispList_for_(aList, arc);

		return arc;
	}

	/**
	 * Load a JunOpenGL3dArc from the object.
	 * 
	 * @param anObject java.lang.Object
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject LoadFrom_(Object anObject) {
		JunLispList aList = (JunLispList) JunLispParser.Parse_(anObject);

		return JunOpenGL3dArc.FromLispList_(aList);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @param anArc sra.jun.opengl.grapher.JunOpenGL3dArc
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dArc
	 */
	private static JunOpenGL3dArc CrankFromLispList_for_(JunLispList aList, JunOpenGL3dArc anArc) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("crank")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return anArc;
		}

		anArc.crank_(((Boolean) list.tail()).booleanValue());

		return anArc;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @param anArc sra.jun.opengl.grapher.JunOpenGL3dArc
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dArc
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	private static JunOpenGL3dArc FromNodeFromLispList_for_(JunLispList aList, JunOpenGL3dArc anArc) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("from")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return anArc;
		}

		try {
			anArc.from_((JunOpenGL3dNode) StObject._PerformWith(DefaultNodeClass(), "FromLispList_", list.tail()));
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}

		return anArc;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @param anArc sra.jun.opengl.grapher.JunOpenGL3dArc
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dArc
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	private static JunOpenGL3dArc ToNodeFromLispList_for_(JunLispList aList, JunOpenGL3dArc anArc) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("to")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return anArc;
		}

		try {
			anArc.to_((JunOpenGL3dNode) StObject._PerformWith(DefaultNodeClass(), "FromLispList_", list.tail()));
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}

		return anArc;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean crank() {
		return this.crankArc;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBoolean boolean
	 */
	public void crank_(boolean aBoolean) {
		this.crankArc = aBoolean;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons crankToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("crank"));
		list.tail_(new Boolean(this.crank()));

		return list;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	public JunOpenGL3dNode from() {
		return this.fromNode;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	public void from_(JunOpenGL3dNode aNode) {
		this.fromNode = aNode;
		this.flushDisplayObject();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNode1 sra.jun.opengl.grapher.JunOpenGL3dNode
	 * @param aNode2 sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	public void from_to_(JunOpenGL3dNode aNode1, JunOpenGL3dNode aNode2) {
		this.fromNode = aNode1;
		this.toNode = aNode2;
		this.flushDisplayObject();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons fromNodeToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("from"));
		list.tail_(this.from().toLispList());

		return list;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean isArc() {
		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean isPrimitive() {
		return true;
	}

	/**
	 * Answer a String which represents the receiver's kind.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public StSymbol kindName() {
		return $("Arc");
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	public JunOpenGL3dNode to() {
		return this.toNode;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	public void to_(JunOpenGL3dNode aNode) {
		this.toNode = aNode;
		this.flushDisplayObject();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList toLispList() {
		JunLispCons list = this.lispCons();
		list.head_(this.kindName());

		if (this.hasColor()) {
			list.add_(this.colorToLispList());
		}

		list.add_(this.crankToLispList());
		list.add_(this.fromNodeToLispList());
		list.add_(this.toNodeToLispList());

		return list;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons toLispListWithoutFromNodeAndToNode() {
		JunLispCons list = this.lispCons();
		list.head_(this.kindName());

		if (this.hasColor()) {
			list.add_(this.colorToLispList());
		}

		list.add_(this.crankToLispList());

		return list;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons toNodeToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("to"));
		list.tail_(this.to().toLispList());

		return list;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aTransformation
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject transform_(Jun3dTransformation aTransformation) {
		this.flushDisplayObject();

		return this;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected JunOpenGL3dObject createDisplayObject() {
		if (this.crank()) {
			Jun3dPoint p1 = (Jun3dPoint) this.from().location().plus_(new Jun3dPoint(this.from().width(), 0, 0));
			Jun3dPoint p4 = this.to().location();
			Jun3dPoint p2 = new Jun3dPoint(p1.x() + ((p4.x() - (p1.x())) / 2), p1.y(), p1.z());
			Jun3dPoint p3 = new Jun3dPoint(p1.x() + ((p4.x() - (p1.x())) / 2), p4.y(), p4.z());
			Jun3dPoint[] vertexes = new Jun3dPoint[4];
			vertexes[0] = p1;
			vertexes[1] = p2;
			vertexes[2] = p3;
			vertexes[3] = p4;

			JunOpenGL3dPolyline object = (JunOpenGL3dPolyline) JunOpenGL3dPolyline.Vertexes_(vertexes);

			if (this.hasColor()) {
				object.paint_(this.color());
			}

			return (JunOpenGL3dObject) object;
		} else {
			Jun3dPoint p1 = (Jun3dPoint) this.from().location().plus_(new Jun3dPoint(this.from().width(), 0, 0));
			Jun3dPoint p4 = this.to().location();
			Jun3dPoint[] vertexes = new Jun3dPoint[2];
			vertexes[0] = p1;
			vertexes[1] = p4;

			JunOpenGL3dPolyline object = (JunOpenGL3dPolyline) JunOpenGL3dPolyline.Vertexes_(vertexes);

			if (this.hasColor()) {
				object.paint_(this.color());
			}

			return (JunOpenGL3dObject) object;
		}
	}
}
