package sra.jun.opengl.grapher;

import java.awt.*;
import java.io.*;
import java.util.*;
import sra.smalltalk.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.*;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.display.*;
import sra.jun.opengl.objects.*;
import sra.jun.opengl.picking.JunOpenGLPickingObjects;

/**
 * JunOpenGL3dGraph class
 * 
 *  @author    Hirotsugu Kondo
 *  @created   1998/12/07 (by Hirotsugu Kondo)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dGraph.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGL3dGraph extends JunOpenGL3dGraphAbstract implements JunOpenGLPickingObjects {
	private Vector graphNodes;
	private Vector graphArcs;
	private Dictionary arrangeFormat;

	/**
	 * Constainer.
	 */
	public JunOpenGL3dGraph() {
		super();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dGraphAbstract
	 */
	public static JunOpenGL3dGraphAbstract FromLispList_(JunLispList aList) {
		JunOpenGL3dGraph graph = new JunOpenGL3dGraph();
		graph = (JunOpenGL3dGraph) ColorFromLispList_for_(aList, graph);
		graph = NodesFromLispList_for_(aList, graph);

		Dictionary[] tables = new Dictionary[2];
		tables[0] = NodeTableFromLispList_for_(aList, graph);
		graph = ArcsFromLispList_for_(aList, graph);
		tables[1] = ArcTableFromLispList_for_(aList, graph);
		graph = ConnectionsFromLispList_for_tables(aList, graph, tables);

		return graph;
	}

	/**
	 * Load a JunOpenGL3dGraph from the object.
	 * 
	 * @param anObject java.lang.Object
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject LoadFrom_(Object anObject) {
		JunLispList aList = (JunLispList) JunLispParser.Parse_(anObject);

		return JunOpenGL3dGraph.FromLispList_(aList);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @param aGraph sra.jun.opengl.grapher.JunOpenGL3dGraph
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dGraph
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	private static JunOpenGL3dGraph ArcsFromLispList_for_(JunLispList aList, JunOpenGL3dGraph aGraph) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("arcs")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return aGraph;
		}

		final JunOpenGL3dGraph aGraph_ = aGraph;
		((JunLispCons) list.tail()).do_(new StBlockClosure() {
			public Object value_(Object anObject) {
				JunLispCons each = (JunLispCons) anObject;

				try {
					JunOpenGL3dArc arc = (JunOpenGL3dArc) StObject._PerformWith(DefaultArcClass(), "FromLispList_", each);
					aGraph_.addArc_(arc);
				} catch (Exception e) {
					throw new SmalltalkException(e);
				}

				return null;
			}
		});

		return aGraph;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @param aGraph sra.jun.opengl.grapher.JunOpenGL3dGraph
	 * 
	 * @return java.util.Dictionary
	 */
	private static Dictionary ArcTableFromLispList_for_(JunLispList aList, JunOpenGL3dGraph aGraph) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("arcs")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return null;
		}

		final JunLispCons list_ = (JunLispCons) list.tail();
		Hashtable table = new Hashtable(list_.size());
		final Hashtable table_ = table;
		final StValueHolder index = StValueHolder.With_(new Integer(1));
		aGraph.arcsDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				JunOpenGL3dArc arc = (JunOpenGL3dArc) anObject;
				String id = (String) ((JunLispCons) list_.nth_(((Integer) index.value()).intValue())).head();
				table_.put(id, arc);
				index.value_(((Integer) index.value()).intValue() + 1);

				return null;
			}
		});

		return table;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @param aGraph sra.jun.opengl.grapher.JunOpenGL3dGraph
	 * @param anArray java.util.Dictionary[]
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dGraph
	 */
	private static JunOpenGL3dGraph ConnectionsFromLispList_for_tables(JunLispList aList, JunOpenGL3dGraph aGraph, Dictionary[] anArray) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("connections")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return aGraph;
		}

		final Dictionary table1 = anArray[0];
		final Dictionary table2 = anArray[1];
		((JunLispCons) list.tail()).do_(new StBlockClosure() {
			public Object value_(Object anObject) {
				JunLispCons each = (JunLispCons) anObject;

				/*JunOpenGL3dNode from = (JunOpenGL3dNode) table1.get((String) each.head());
				JunOpenGL3dNode to = (JunOpenGL3dNode) table1.get((String) ((JunLispCons) each.tail()).head());
				JunOpenGL3dArc arc = (JunOpenGL3dArc) table2.get((String) ((JunLispCons) each.tail()).tail());*/
				JunOpenGL3dNode from = (JunOpenGL3dNode) table1.get(each.head());
				JunOpenGL3dNode to = (JunOpenGL3dNode) table1.get(((JunLispCons) each.tail()).head());
				JunOpenGL3dArc arc = (JunOpenGL3dArc) table2.get(((JunLispCons) each.tail()).tail());

				if (arc != null) {
					arc.from_to_(from, to);
				}

				return null;
			}
		});

		return aGraph;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @param aGraph sra.jun.opengl.grapher.JunOpenGL3dGraph
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dGraph
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	private static JunOpenGL3dGraph NodesFromLispList_for_(JunLispList aList, JunOpenGL3dGraph aGraph) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("nodes")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return aGraph;
		}

		final JunOpenGL3dGraph aGraph_ = aGraph;
		((JunLispCons) list.tail()).do_(new StBlockClosure() {
			public Object value_(Object anObject) {
				JunLispCons each = (JunLispCons) anObject;

				try {
					JunOpenGL3dNode node = (JunOpenGL3dNode) StObject._PerformWith(DefaultNodeClass(), "FromLispList_", each);
					aGraph_.addNode_(node);
				} catch (Exception e) {
					throw new SmalltalkException(e);
				}

				return null;
			}
		});

		return aGraph;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @param aGraph sra.jun.opengl.grapher.JunOpenGL3dGraph
	 * 
	 * @return java.util.Dictionary
	 */
	private static Dictionary NodeTableFromLispList_for_(JunLispList aList, JunOpenGL3dGraph aGraph) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("nodes")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return null;
		}

		final JunLispCons list_ = (JunLispCons) list.tail();
		Hashtable table = new Hashtable(list_.size());
		final Hashtable table_ = table;
		final StValueHolder index = StValueHolder.With_(new Integer(1));
		aGraph.nodesDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				JunOpenGL3dNode node = (JunOpenGL3dNode) anObject;
				String id = (String) ((JunLispCons) list_.nth_(((Integer) index.value()).intValue())).head();
				table_.put(id, node);
				index.value_(((Integer) index.value()).intValue() + 1);

				return null;
			}
		});

		return table;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param table java.util.Hashtable
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @param nodeBlock sra.smalltalk.StBlockClosure
	 * @param arcBlock sra.smalltalk.StBlockClosure
	 */
	public void _generateTreeFromLispList_nodeBlock_arcBlock_(Hashtable table, JunLispList aList, StBlockClosure nodeBlock, StBlockClosure arcBlock) {
		if (StSymbol.class.isInstance(aList)) {
			return;
		}

		JunOpenGL3dNode fromNode = null;
		JunOpenGL3dArc anArc;

		if (table.containsKey(((JunLispCons) aList).head())) {
			fromNode = (JunOpenGL3dNode) table.get(((JunLispCons) aList).head());
		} else {
			fromNode = (JunOpenGL3dNode) nodeBlock.value_(((JunLispCons) aList).head());
			this.addNode_(fromNode);
			table.put(((JunLispCons) aList).head(), fromNode);
		}

		if (JunLispNil.class.isInstance(((JunLispCons) aList).tail())) {
			return;
		}

		final JunLispCons tails = (JunLispCons) ((JunLispCons) aList).tail();
		final Hashtable table_ = table;
		final JunOpenGL3dNode fromNode_ = fromNode;
		final StBlockClosure nodeBlock_ = nodeBlock;
		final StBlockClosure arcBlock_ = arcBlock;
		final JunOpenGL3dGraph this_ = this;
		tails.do_(new StBlockClosure() {
			public Object value_(Object object) {
				JunOpenGL3dNode toNode;
				JunLispCons each = (JunLispCons) object;

				if (table_.containsKey(each.head())) {
					toNode = (JunOpenGL3dNode) table_.get(each.head());
				} else {
					toNode = (JunOpenGL3dNode) nodeBlock_.value_(each.head());
					this_.addNode_((JunOpenGL3dNode) toNode);
					table_.put(each.head(), toNode);
				}

				JunOpenGL3dArc anArc_ = (JunOpenGL3dArc) arcBlock_.value_value_(fromNode_, toNode);
				this_.addArc_(anArc_);

				return null;
			}
		});

		final JunLispList aList_ = aList;
		final JunLispCons tails2 = (JunLispCons) ((JunLispCons) aList).tail();
		tails2.do_(new StBlockClosure() {
			public Object value_(Object object) {
				JunLispList each = (JunLispList) object;
				this_._generateTreeFromLispList_nodeBlock_arcBlock_(table_, each, nodeBlock_, arcBlock_);

				return null;
			}
		});
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anArc sra.jun.opengl.grapher.JunOpenGL3dArc
	 */
	public void addArc_(JunOpenGL3dArc anArc) {
		JunOpenGL3dArc[] oldArcs = this.arcs();
		Vector newArcs = new Vector();
		int size = oldArcs.length;

		for (int index = 0; index < size; index++) {
			newArcs.addElement(oldArcs[index]);
		}

		newArcs.addElement(anArc);
		this.arcs_(newArcs);
		this.flushDisplayObject();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	public void addNode_(JunOpenGL3dNode aNode) {
		JunOpenGL3dNode[] oldNodes = this.nodes();
		Vector newNoodes = new Vector();
		int size = oldNodes.length;

		for (int index = 0; index < size; index++) {
			newNoodes.addElement(oldNodes[index]);
		}

		newNoodes.addElement(aNode);
		this.nodes_(newNoodes);
		this.flushDisplayObject();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dArc[]
	 */
	public JunOpenGL3dArc[] arcs() {
		Vector arcs = this.graphArcs;

		if (arcs == null) {
			arcs = new Vector();
			this.graphArcs = arcs;
		}

		int size = arcs.size();
		JunOpenGL3dArc[] result = new JunOpenGL3dArc[size];

		for (int index = 0; index < size; index++) {
			result[index] = (JunOpenGL3dArc) arcs.elementAt(index);
		}

		return result;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void arcsDo_(StBlockClosure aBlock) {
		JunOpenGL3dArc[] arcs = this.arcs();
		int size = arcs.length;

		for (int i = 0; i < size; i++) {
			aBlock.value_(arcs[i]);
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons arcsToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("arcs"));

		final JunOpenGL3dGraph this_ = this;
		final JunLispCons list_ = list;
		this.arcsDo_(new StBlockClosure() {
			public Object value_(Object each) {
				JunOpenGL3dArc arc = (JunOpenGL3dArc) each;
				JunLispCons pair = this_.lispCons();
				pair.head_(this_.generateIdString());
				pair.tail_(arc.toLispListWithoutFromNodeAndToNode());
				list_.add_(pair);

				return null;
			}
		});

		return list;
	}

	/**
	 * DOCUMENT ME!
	 */
	public void arrange() {
		this.arrange_(null);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void arrange_(StBlockClosure aBlock) {
		this.arrange1_(aBlock);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aDictionary java.util.Dictionary
	 * @param aBlock sra.smalltalk.StBlock
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint arrange_xyz_visited_interrim_(JunOpenGL3dNode aNode, Jun3dPoint aPoint, Dictionary aDictionary, StBlockClosure aBlock) {
		aNode.location_(aPoint);

		if (aBlock != null) {
			int numArgs = aBlock.numArgs();

			if (numArgs == 0) {
				aBlock.value();
			}

			if (numArgs == 1) {
				aBlock.value_(aNode);
			}
		}

		aDictionary.put(aNode, $("visited"));

		JunOpenGL3dNode[] connectedNodes = this.connectedNodes_(aNode);
		int size = connectedNodes.length;

		if (size == 0) {
			double width = aPoint.x() + aNode.width();
			double height = aPoint.y() + aNode.height();

			return new Jun3dPoint(width, height, aPoint.z());
		}

		double width = aPoint.x() + aNode.width();
		double height = aPoint.y();
		double x = width + this.arrangeIntervalX();
		double y = height;
		double min = height;

		for (int index = 0; index < size; index++) {
			JunOpenGL3dNode node = connectedNodes[index];
			Object value = aDictionary.get(node);

			if (value == null) {
				value = $("unvisited");
			}

			if (value == $("unvisited")) {
				Jun3dPoint extent = this.arrange_xyz_visited_interrim_(node, new Jun3dPoint(x, y, aPoint.z()), aDictionary, aBlock);
				y = Math.max(extent.y(), y + aNode.height());
				width = Math.max(extent.x(), width);
				height = Math.max(y, height);
				y = y + this.arrangeIntervalY();
			}

			if (this.arrangeBalance()) {
				this.arrangeBalance_xyz_minHeight_interrim_(aNode, new Jun3dPoint(x, y, aPoint.z()), min, aBlock);
				height = Math.max(height, aNode.location().y() + aNode.height());
			}
		}

		return new Jun3dPoint(width, height, aPoint.z());
	}

	/**
	 * DOCUMENT ME!
	 */
	public void arrange1() {
		this.arrange1_(null);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void arrange1_(StBlockClosure aBlock) {
		Jun3dPoint origin = new Jun3dPoint(0, 0, 0);
		Vector roots = this.arrangeRoots();

		for (int index = 0; index < roots.size(); index++) {
			JunOpenGL3dNode root = (JunOpenGL3dNode) roots.elementAt(index);
			this.arrange_xyz_visited_interrim_(root, origin, new Hashtable(), aBlock);

			if (this.arrangeBalance()) {
				Jun3dPoint delta = new Jun3dPoint(0, -1 * (root.location().y()), 0);
				final Jun3dTransformation transformation = Jun3dTransformation.Translate_(delta);
				final StBlockClosure block = aBlock;
				this.breadthFirstFrom_nodesDo_(root, new StBlockClosure() {
					public Object value_(Object each) {
						JunOpenGL3dNode eachNode = (JunOpenGL3dNode) each;
						eachNode.transform_(transformation);

						if (block != null) {
							int numArgs = block.numArgs();

							if (numArgs == 0) {
								block.value();
							}

							if (numArgs == 1) {
								block.value_(eachNode);
							}
						}

						return null;
					}
				});
			}

			origin = (Jun3dPoint) origin.plus_(new Jun3dPoint(0, 0, this.arrangeIntervalZ()));
		}

		this.flushDisplayObject();
	}

	/**
	 * DOCUMENT ME!
	 */
	public void arrange2() {
		this.arrange2_(null);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void arrange2_(StBlockClosure aBlock) {
		Jun3dPoint origin = new Jun3dPoint(0, 0, 0);
		Jun3dPoint corner = origin;
		Vector roots = this.arrangeRoots();

		for (int index = 0; index < roots.size(); index++) {
			JunOpenGL3dNode root = (JunOpenGL3dNode) roots.elementAt(index);
			Jun3dPoint point = new Jun3dPoint(0, 0, origin.z());
			JunOpenGL3dNode[] connectedNodes = this.connectedNodes_(root);

			for (int i = 0; i < connectedNodes.length; i++) {
				JunOpenGL3dNode node = connectedNodes[i];
				Jun3dPoint xyz = this.arrange_xyz_visited_interrim_(node, origin, new Hashtable(), aBlock);

				if (this.arrangeBalance()) {
					Jun3dPoint delta = new Jun3dPoint(0, -1 * (node.location().y()), 0);
					final Jun3dTransformation transformation = Jun3dTransformation.Translate_(delta);
					final StBlockClosure block = aBlock;
					this.breadthFirstFrom_nodesDo_(node, new StBlockClosure() {
						public Object value_(Object each) {
							JunOpenGL3dNode eachNode = (JunOpenGL3dNode) each;
							eachNode.transform_(transformation);

							if (block != null) {
								int numArgs = block.numArgs();

								if (numArgs == 0) {
									block.value();
								}

								if (numArgs == 1) {
									block.value_(eachNode);
								}
							}

							return null;
						}
					});
				}

				corner = new Jun3dPoint(Math.max(corner.x(), xyz.x()), Math.max(corner.y(), xyz.y()), Math.max(corner.z(), xyz.z()));
				origin = (Jun3dPoint) origin.plus_(new Jun3dPoint(0, 0, this.arrangeIntervalZ()));
			}

			point = (Jun3dPoint) point.plus_((new Jun3dPoint(0, 0, corner.z()).minus_(point)).dividedBy_(2));
			root.location_((Jun3dPoint) point.minus_(new Jun3dPoint((this.arrangeIntervalX() + root.width()), 0, 0)));

			if (aBlock != null) {
				int numArgs = aBlock.numArgs();

				if (numArgs == 0) {
					aBlock.value();
				}

				if (numArgs == 1) {
					aBlock.value_(root);
				}

				Jun3dPoint delta = new Jun3dPoint(-1 * root.location().x(), 0, 0);
				final Jun3dTransformation transformation = Jun3dTransformation.Translate_(delta);
				final StBlockClosure block = aBlock;
				this.depthFirstFrom_nodeDo_(root, new StBlockClosure() {
					public Object value_(Object each) {
						JunOpenGL3dNode eachNode = (JunOpenGL3dNode) each;
						eachNode.transform_(transformation);

						if (block != null) {
							int numArgs2 = block.numArgs();

							if (numArgs2 == 0) {
								block.value();
							}

							if (numArgs2 == 1) {
								block.value_(eachNode);
							}
						}

						return null;
					}
				});
			}
		}

		this.flushDisplayObject();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean arrangeBalance() {
		return ((Boolean) this.arrangeFormat().get($("balance"))).booleanValue();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBoolean boolean
	 */
	public void arrangeBalance_(boolean aBoolean) {
		this.arrangeFormat().put($("balance"), new Boolean(aBoolean));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param min double
	 * @param aBlock sra.smalltalk.StBlocClosure
	 */
	public void arrangeBalance_xyz_minHeight_interrim_(JunOpenGL3dNode aNode, Jun3dPoint aPoint, double min, StBlockClosure aBlock) {
		double y = aPoint.y() - this.arrangeIntervalY();

		if (y > (aNode.location().y() + aNode.height())) {
			y = Math.max(min, min + ((y - min - aNode.height()) / 2));
		} else {
			y = min;
		}

		aNode.location_(new Jun3dPoint(aNode.location().x(), y, aPoint.z()));

		if (aBlock != null) {
			int numArgs = aBlock.numArgs();

			if (numArgs == 0) {
				aBlock.value();
			}

			if (numArgs == 1) {
				aBlock.value_(aNode);
			}
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.util.Dictionary
	 */
	public Dictionary arrangeFormat() {
		Dictionary format = this.arrangeFormat;

		if (format == null) {
			format = new Hashtable();
			format.put($("interval"), new Jun3dPoint(10, 2, 4));
			format.put($("balance"), new Boolean(true));
			this.arrangeFormat = format;
		}

		return format;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint arrangeInterval() {
		return (Jun3dPoint) this.arrangeFormat().get($("interval"));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aPoint sra.jun.geometry.Jun3dPoint
	 */
	public void arrangeInterval_(Jun3dPoint aPoint) {
		this.arrangeFormat().put($("interval"), aPoint);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return double
	 */
	public double arrangeIntervalX() {
		return ((Jun3dPoint) this.arrangeFormat().get($("interval"))).x();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return double
	 */
	public double arrangeIntervalY() {
		return ((Jun3dPoint) this.arrangeFormat().get($("interval"))).y();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return double
	 */
	public double arrangeIntervalZ() {
		return ((Jun3dPoint) this.arrangeFormat().get($("interval"))).z();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.util.Vector
	 */
	public Vector arrangeRoots() {
		if (this.nodes().length == 0) {
			return new Vector();
		}

		final Vector exceptions = new Vector();
		this.arcsDo_(new StBlockClosure() {
			public Object value_(Object arc) {
				exceptions.addElement(((JunOpenGL3dArc) arc).to());

				return null;
			}
		});

		final Vector roots = new Vector();
		this.nodesDo_(new StBlockClosure() {
			public Object value_(Object node) {
				JunOpenGL3dNode aNode = (JunOpenGL3dNode) node;

				if (!exceptions.contains(aNode)) {
					roots.addElement(aNode);
				}

				return null;
			}
		});

		if (roots.isEmpty()) {
			roots.addElement(this.atNode_(1));
		}

		return roots;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anInteger int
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	public JunOpenGL3dNode atNode_(int anInteger) {
		return this.nodes()[anInteger - 1];
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * @param nodeBlock sra.smalltalk.StBlockClosure
	 */
	public void breadthFirstFrom_nodesDo_(JunOpenGL3dNode aNode, StBlockClosure nodeBlock) {
		this.breadthFirstFrom_nodesDo_arcDo_(aNode, nodeBlock, new StBlockClosure());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * @param nodeBlock sra.smalltalk.StBlockClosure
	 * @param arcBlock sra.smalltalk.StBlockClosure
	 */
	public void breadthFirstFrom_nodesDo_arcDo_(JunOpenGL3dNode aNode, StBlockClosure nodeBlock, StBlockClosure arcBlock) {
		this.breadthFirstFrom_nodesDo_arcDo_visited_queue_(aNode, nodeBlock, arcBlock, new Hashtable(), new Vector());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * @param nodeBlock sra.smalltalk.StBlockClosure
	 * @param arcBlock sra.smalltalk.StBlockClosure
	 * @param aDictionary java.util.Dictionary
	 * @param aQueue java.util.Vector
	 */
	public void breadthFirstFrom_nodesDo_arcDo_visited_queue_(JunOpenGL3dNode aNode, StBlockClosure nodeBlock, StBlockClosure arcBlock, Dictionary aDictionary, Vector aQueue) {
		aDictionary.put(aNode, $("entered"));
		aQueue.addElement(aNode);

		while (!aQueue.isEmpty()) {
			JunOpenGL3dNode node = (JunOpenGL3dNode) aQueue.elementAt(0);
			aQueue.removeElementAt(0);

			Object value = aDictionary.get(node);

			if (value == null) {
				value = $("unvisited");
			}

			if (value != $("visited")) {
				nodeBlock.value_(node);
			}

			aDictionary.put(node, $("visited"));

			JunOpenGL3dArc[] arcs = this.connectedArcs_(node);

			for (int index = 0; index < arcs.length; index++) {
				JunOpenGL3dArc arc = arcs[index];
				JunOpenGL3dNode toNode = arc.to();
				arcBlock.value_(arc);
				value = aDictionary.get(toNode);

				if (value == null) {
					value = $("unvisited");

					if (value == $("unvisited")) {
						aQueue.addElement(toNode);
						aDictionary.put(toNode, $("entered"));
					}
				}
			}
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject[]
	 */
	public JunOpenGL3dObject[] components() {
		return ((JunOpenGL3dCompoundObject) this.displayObject()).components();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dArc[]
	 */
	public JunOpenGL3dArc[] connectedArcs_(JunOpenGL3dNode aNode) {
		JunOpenGL3dArc[] arcs = this.arcs();
		Vector connectedArcs = new Vector();
		int size = arcs.length;

		for (int index = 0; index < size; index++) {
			JunOpenGL3dArc arc = (JunOpenGL3dArc) arcs[index];

			if (arc.from() == aNode) {
				if (connectedArcs.indexOf(arc) == -1) {
					connectedArcs.addElement(arc);
				}
			}
		}

		JunOpenGL3dArc[] result = new JunOpenGL3dArc[connectedArcs.size()];
		connectedArcs.copyInto(result);

		return result;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode[]
	 */
	public JunOpenGL3dNode[] connectedNodes_(JunOpenGL3dNode aNode) {
		JunOpenGL3dArc[] arcs = this.connectedArcs_(aNode);
		int size = arcs.length;
		JunOpenGL3dNode[] result = new JunOpenGL3dNode[size];

		for (int index = 0; index < size; index++) {
			result[index] = arcs[index].to();
		}

		return result;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anArray sra.jun.goodies.lisp.JunLispCons[]
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons connectionsToLispList_(JunLispCons[] anArray) {
		JunLispCons list = this.lispCons();
		list.head_($("connections"));

		final JunOpenGL3dGraph this_ = this;
		final JunLispCons list_ = list;
		final JunLispCons list1 = anArray[0];
		final Hashtable table1 = new Hashtable();
		final StValueHolder index1 = new StValueHolder(1);
		this.nodesDo_(new StBlockClosure() {
			public Object value_(Object each) {
				JunOpenGL3dNode node = (JunOpenGL3dNode) each;
				int indexValue = ((Integer) index1.value()).intValue();
				String id = (String) ((JunLispCons) list1.nth_(indexValue)).head();
				table1.put(node, id);
				index1.value_(indexValue + 1);

				return null;
			}
		});

		final JunLispCons list2 = anArray[1];
		final Hashtable table2 = new Hashtable();
		final StValueHolder index2 = new StValueHolder(1);
		this.arcsDo_(new StBlockClosure() {
			public Object value_(Object each) {
				JunOpenGL3dArc arc = (JunOpenGL3dArc) each;
				int indexValue = ((Integer) index2.value()).intValue();
				String id = (String) ((JunLispCons) list2.nth_(indexValue)).head();
				table2.put(arc, id);
				index2.value_(indexValue + 1);

				return null;
			}
		});
		this.arcsDo_(new StBlockClosure() {
			public Object value_(Object each) {
				JunOpenGL3dArc arc = (JunOpenGL3dArc) each;
				JunOpenGL3dNode from = arc.from();
				JunOpenGL3dNode to = arc.to();
				Object car = table1.get(from);
				Object cdar = table1.get(to);
				Object cddr = table2.get(arc);
				JunLispList connection = (JunLispList) JunLispCons.Head_tail_(cdar, cddr);
				connection = JunLispCons.Head_tail_(car, connection);
				list_.add_(connection);

				return null;
			}
		});

		return list;
	}

	/**
	 * DOCUMENT ME!
	 */
	public void debug() {
		System.out.print("TEST");

		return;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * @param nodeBlock sra.smalltalk.StBlockClosure
	 */
	public void depthFirstFrom_nodeDo_(JunOpenGL3dNode aNode, StBlockClosure nodeBlock) {
		this.depthFirstFrom_nodeDo_arcDo_(aNode, nodeBlock, new StBlockClosure());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * @param nodeBlock sra.smalltalk.StBlockClosure
	 * @param arcBlock sra.smalltalk.StBlockClosure
	 */
	public void depthFirstFrom_nodeDo_arcDo_(JunOpenGL3dNode aNode, StBlockClosure nodeBlock, StBlockClosure arcBlock) {
		this.depthFirstFrom_nodeDo_arcDo_visited_(aNode, nodeBlock, arcBlock, new Hashtable());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * @param nodeBlock sra.smalltalk.StBlockClosure
	 * @param arcBlock sra.smalltalk.StBlockClosure
	 * @param aDictionary java.util.Dictionary
	 */
	public void depthFirstFrom_nodeDo_arcDo_visited_(JunOpenGL3dNode aNode, StBlockClosure nodeBlock, StBlockClosure arcBlock, Dictionary aDictionary) {
		Object value = aDictionary.get(aNode);

		if (value == null) {
			value = $("unvisited");
		}

		if (value != $("visited")) {
			nodeBlock.value_(aNode);
		}

		aDictionary.put(aNode, $("visited"));

		JunOpenGL3dArc[] arcs = this.connectedArcs_(aNode);

		for (int index = 0; index < arcs.length; index++) {
			JunOpenGL3dArc arc = arcs[index];
			arcBlock.value_(arc);

			JunOpenGL3dNode toNode = arc.to();
			value = aDictionary.get(toNode);

			if (value == null) {
				value = $("unvisited");

				if (value == $("unvisited")) {
					this.depthFirstFrom_nodeDo_arcDo_visited_(aNode, nodeBlock, arcBlock, aDictionary);
				}
			}
		}
	}

	/**
	 * Generate a tree from a lisp list.
	 * 
	 * @param aList
	 */
	public void generateTreeFromLispList_(JunLispList aList) {
		this.generateTreeFromLispList_nodeBlock_arcBlock_(aList, new StBlockClosure() {
			public Object value_(Object each) {
				String nodeName = each.toString();
				Point extent = SystemResourceSupport._getStringExtentFor_(nodeName, null);
				Rectangle aRectangle = new Rectangle(0, 0, extent.x, extent.y);
				double aWidth;

				if (aRectangle.height <= 0) {
					aWidth = 1;
				} else {
					aWidth = aRectangle.width / aRectangle.height;
				}

				JunOpenGL3dNode aNode = JunOpenGL3dNode.Extent_color_entity_(new Jun3dPoint(aWidth, 1, 1), JunOpenGL3dObject.SampleColor(), nodeName);

				return aNode;
			}
		}, new StBlockClosure() {
			public Object value_value_(Object each1, Object each2) {
				JunOpenGL3dArc anArc = JunOpenGL3dArc.From_to_((JunOpenGL3dNode) each1, (JunOpenGL3dNode) each2);

				return anArc;
			}
		});
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @param nodeBlock sra.smalltalk.StBlockclosure
	 * @param arcBlock sra.smalltalk.StBlockclosure
	 */
	public void generateTreeFromLispList_nodeBlock_arcBlock_(JunLispList aList, StBlockClosure nodeBlock, StBlockClosure arcBlock) {
		this._generateTreeFromLispList_nodeBlock_arcBlock_(new Hashtable(), aList, nodeBlock, arcBlock);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anArc sra.jun.opengl.grapher.JunOpenGL3dArc
	 * 
	 * @return boolean
	 */
	public boolean includesArc_(JunOpenGL3dArc anArc) {
		JunOpenGL3dArc[] arcs = this.arcs();
		int size = arcs.length;

		for (int index = 0; index < size; index++) {
			if (arcs[index] == anArc) {
				return true;
			}
		}

		return false;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * 
	 * @return boolean
	 */
	public boolean includesNode_(JunOpenGL3dNode aNode) {
		JunOpenGL3dNode[] nodes = this.nodes();
		int size = nodes.length;

		for (int index = 0; index < size; index++) {
			if (nodes[index] == aNode) {
				return true;
			}
		}

		return false;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean isCompound() {
		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean isGraph() {
		return true;
	}

	/**
	 * Answer a String which represents the receiver's kind.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public StSymbol kindName() {
		return $("Graph");
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.lang.String
	 */
	public String name() {
		return JunOpenGL3dObject.UniqueId();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode[]
	 */
	public JunOpenGL3dNode[] nodes() {
		Vector nodes = this.graphNodes;

		if (nodes == null) {
			nodes = new Vector();
			this.graphNodes = nodes;
		}

		int size = nodes.size();
		JunOpenGL3dNode[] result = new JunOpenGL3dNode[size];

		for (int index = 0; index < size; index++) {
			result[index] = (JunOpenGL3dNode) nodes.elementAt(index);
		}

		return result;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void nodesDo_(StBlockClosure aBlock) {
		JunOpenGL3dNode[] nodes = this.nodes();
		int size = nodes.length;

		for (int i = 0; i < size; i++) {
			aBlock.value_(nodes[i]);
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons nodesToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("nodes"));

		final JunOpenGL3dGraph this_ = this;
		final JunLispCons list_ = list;
		this.nodesDo_(new StBlockClosure() {
			public Object value_(Object each) {
				JunOpenGL3dNode node = (JunOpenGL3dNode) each;
				JunLispCons pair = this_.lispCons();
				pair.head_(this_.generateIdString());
				pair.tail_(node.toLispList());
				list_.add_(pair);

				return null;
			}
		});

		return list;
	}

	/**
	 * Enumerate the primitives and evaluate the block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object primitivesDo_(StBlockClosure aBlock) {
		if (this.displayObject() == null) {
			return null;
		}

		return this.displayObject().primitivesDo_(aBlock);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public JunOpenGLDisplayModel show() {
		return JunOpenGL3dGrapher.Show_(this);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList toLispList() {
		JunLispCons list = this.lispCons();
		list.head_(this.kindName());

		if (this.hasColor()) {
			list.add_(this.colorToLispList());
		}

		JunLispCons list1 = this.nodesToLispList();
		JunLispCons list2 = this.arcsToLispList();
		list.add_(list1);
		list.add_(list2);

		JunLispCons[] array = new JunLispCons[2];
		array[0] = (JunLispCons) list1.tail();
		array[1] = (JunLispCons) list2.tail();
		list.add_(this.connectionsToLispList_(array));

		return list;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.opengl.oejects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject transform_(Jun3dTransformation aTransformation) {
		final Jun3dTransformation transformation = aTransformation;
		this.nodesDo_(new StBlockClosure() {
			public Object value_(Object each) {
				JunOpenGL3dNode node = (JunOpenGL3dNode) each;
				node.transform_(transformation);

				return null;
			}
		});
		this.arcsDo_(new StBlockClosure() {
			public Object value_(Object each) {
				JunOpenGL3dArc arc = (JunOpenGL3dArc) each;
				arc.transform_(transformation);

				return null;
			}
		});
		this.flushDisplayObject();

		return this;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param writer java.io.Writer
	 */
	public void vrmlOn_(Writer writer) {
		JunOpenGL3dObject displayObject = this.displayObject();

		if (displayObject == null) {
			return;
		}

		displayObject.vrmlOn_(writer);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param newVector java.util.Vector
	 */
	private void arcs_(Vector newVector) {
		this.graphArcs = newVector;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected JunOpenGL3dObject createDisplayObject() {
		final JunOpenGL3dCompoundObject object = new JunOpenGL3dCompoundObject();
		this.nodesDo_(new StBlockClosure() {
			public Object value_(Object each) {
				JunOpenGL3dNode node = (JunOpenGL3dNode) each;
				node.flushDisplayObject();
				object.add_(node.displayObject());

				return null;
			}
		});
		this.arcsDo_(new StBlockClosure() {
			public Object value_(Object each) {
				JunOpenGL3dArc arc = (JunOpenGL3dArc) each;
				arc.flushDisplayObject();
				object.add_(arc.displayObject());

				return null;
			}
		});

		return (JunOpenGL3dObject) object;
	}

	/**
	 * DOCUMENT ME!
	 */
	protected void flushDisplayObject() {
		this.displayObject = null;
		this.nodesDo_(new StBlockClosure() {
			public Object value_(Object each) {
				JunOpenGL3dNode node = (JunOpenGL3dNode) each;
				node.flushDisplayObject();

				return null;
			}
		});
		this.arcsDo_(new StBlockClosure() {
			public Object value_(Object each) {
				JunOpenGL3dArc arc = (JunOpenGL3dArc) each;
				arc.flushDisplayObject();

				return null;
			}
		});
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param newVector java.util.Vector
	 */
	private void nodes_(Vector newVector) {
		this.graphNodes = newVector;
	}
}
