package sra.jun.opengl.grapher;

import java.awt.Color;
import java.io.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.*;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.objects.*;
import sra.jun.opengl.support.*;
import sra.smalltalk.*;

/**
 * JunOpenGL3dGraphAbstract class
 * 
 *  @author    Hirotsugu Kondo
 *  @created   1998/12/07 (by Hirotsugu Kondo)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dGraphAbstract.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public abstract class JunOpenGL3dGraphAbstract extends JunOpenGL3dObject {
	private static long GenerateIDTime = System.currentTimeMillis();
	protected JunOpenGL3dObject displayObject;
	private Color displayColor;

	/**
	 * Get a color from the lisp list and set it as the name of the
	 * JunOpenGL3dObject.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @param anAbstract sra.jun.opengl.objects.JunOpenGL3dGraphAbstract
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dGraphAbstract
	 */
	public static JunOpenGL3dGraphAbstract ColorFromLispList_for_(JunLispList aList, JunOpenGL3dGraphAbstract anAbstract) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("color")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return anAbstract;
		}

		list = (JunLispCons) list.tail();

		double red = ((Number) list.nth_(1)).doubleValue();
		double green = ((Number) list.nth_(2)).doubleValue();
		double blue = ((Number) list.nth_(3)).doubleValue();
		Number alpha = null;

		if (list.length() >= 4) {
			alpha = (Number) list.nth_(4);

			if (alpha instanceof Integer) {
				alpha = new Float(Math.max(0, Math.min(alpha.floatValue() / 255, 1)));
			}
		}

		anAbstract.color_(new Color((float) red, (float) green, (float) blue));

		return anAbstract;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.lang.Class
	 */
	public static Class DefaultArcClass() {
		return JunOpenGL3dArc.class;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.lang.Class
	 */
	public static Class DefaultGraphClass() {
		return JunOpenGL3dGraph.class;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.lang.Class
	 */
	public static Class DefaultNodeClass() {
		return JunOpenGL3dNode.class;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dGraphAbstract
	 */
	public static JunOpenGL3dGraphAbstract FromLispList_(JunLispList aList) {
		System.out.print("self subclassResponsibility");

		return null;
	}

	/**
	 * Load a graph element of the specified class from the read stream.
	 * 
	 * @param aClass java.lang.Class
	 * @param aStream sra.smalltalk.StReadStream
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dGraphAbstract
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public static JunOpenGL3dGraphAbstract LoadFrom_(Class aClass, StReadStream aStream) {
		JunLispList list = (JunLispList) JunLispParser.Parse_(aStream);

		try {
			JunOpenGL3dGraphAbstract graph = (JunOpenGL3dGraphAbstract) _PerformWith(aClass, "FromLispList_", list);

			return graph;
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.geometry.basic.JunPoint[]
	 */
	public JunPoint[] asPointArray() {
		return this.displayObject.asPointArray();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.geometry.basic.Jun3dBoundingBox
	 */
	public Jun3dBoundingBox boundingBox() {
		return this.displayObject().boundingBox();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.awt.Color
	 */
	public Color color() {
		return this.displayColor;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param colorValue java.awt.Color
	 */
	public void color_(Color colorValue) {
		this.displayColor = colorValue;
		this.flushDisplayObject();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList colorToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("color"));

		Color color = this.color();
		list.add_(new Float(color.getRed() / 255f));
		list.add_(new Float(color.getGreen() / 255f));
		list.add_(new Float(color.getBlue() / 255f));

		return list;
	}

	/**
	 * Make a copy of the receiver.
	 * 
	 * @return sra.smalltalk.StObject
	 */
	public StObject copy() {
		// return (JunOpenGL3dGraphAbstract) _PerformWith(this.getClass(), "FromLispList_", this.toLispList());
		// can not copy because of the image.
		return this;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject displayObject() {
		JunOpenGL3dObject displayObject = this.displayObject;

		if (displayObject == null) {
			displayObject = this.createDisplayObject();
			this.displayObject = displayObject;
		}

		return displayObject;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.lang.String
	 */
	public String generateIdString() {
		return JunOpenGL3dObject.UniqueId();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean hasColor() {
		return this.color() != null;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean isArc() {
		return false;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean isCompound() {
		return false;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean isGraph() {
		return false;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean isNode() {
		return false;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean isPrimitive() {
		return false;
	}

	/**
	 * Answer a String which represents the receiver's kind.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public StSymbol kindName() {
		return this._className();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons lispCons() {
		return (JunLispCons) JunLispCons.Cell();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispNil
	 */
	public JunLispNil lispNil() {
		return (JunLispNil) JunLispNil.NullList();
	}

	/**
	 * Answer the number of polygons.
	 * 
	 * @return int
	 */
	public int numberOfPolygons() {
		return this.displayObject().numberOfPolygons();
	}

	/**
	 * Answer the number of primitives.
	 * 
	 * @return int
	 */
	public int numberOfPrimitives() {
		return this.displayObject().numberOfPrimitives();
	}

	/**
	 * Enumerate every points and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void pointsDo_(StBlockClosure aBlock) {
		JunOpenGL3dObject displayObject = this.displayObject();

		if (displayObject != null) {
			displayObject.pointsDo_(aBlock);
		}
	}

	/**
	 * Render the OpenGL object on a rendering context.
	 * 
	 * @param aRenderingContext
	 *        sra.jun.opengl.support.JunOpenGLRenderingContext
	 */
	public void renderOn_(JunOpenGLRenderingContext aRenderingContext) {
		this.displayObject().renderOn_(aRenderingContext);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anAngle sra.jun.geometry.basic.JunAngle
	 */
	public void rotateBy_(JunAngle anAngle) {
		this.transform_(Jun3dTransformation.Rotate_(anAngle));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param writer java.io.Writer
	 */
	public void saveOn_(Writer writer) {
		try {
			this.toLispList().saveOn_(writer);
		} catch (Exception e) {
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject scaledBy_(Jun3dPoint a3dPoint) {
		return this.transform_(Jun3dTransformation.Scale_(a3dPoint));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param a3dObject java.lang.Object
	 */
	public void setValue_(Object a3dObject) {
		this.displayObject = (JunOpenGL3dObject) a3dObject;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public abstract JunLispList toLispList();

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public abstract JunOpenGL3dObject transform_(Jun3dTransformation aTransformation);

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public Jun3dTransformation transformation() {
		return this.displayObject().transformation();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public void translateBy_(Jun3dPoint a3dPoint) {
		this.transform_(Jun3dTransformation.Translate_(a3dPoint));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.lang.Object
	 */
	public Object value() {
		return (Object) this.displayObject();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param writer
	 */
	public void vrml10On_(Writer writer) {
		JunOpenGL3dObject displayObject = this.displayObject();

		if (displayObject == null) {
			return;
		}

		displayObject.vrml10On_(writer);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param writer
	 */
	public void vrml20On_(Writer writer) {
		JunOpenGL3dObject displayObject = this.displayObject();

		if (displayObject == null) {
			return;
		}

		displayObject.vrml20On_(writer);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param writer
	 */
	public void vrmlOn_(Writer writer) {
		JunOpenGL3dObject displayObject = this.displayObject();

		if (displayObject == null) {
			return;
		}

		displayObject.vrmlOn_(writer);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected abstract JunOpenGL3dObject createDisplayObject();

	/**
	 * DOCUMENT ME!
	 */
	protected void flushDisplayObject() {
		this.displayObject = null;
	}
}
