package sra.jun.opengl.grapher;

import java.io.*;
import java.text.DateFormat;
import java.util.*;
import sra.smalltalk.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.*;
import sra.jun.opengl.display.*;
import sra.jun.opengl.objects.*;
import sra.jun.system.support.*;

/**
 * JunOpenGL3dGrapher class
 * 
 *  @author    Hirotsugu Kondo
 *  @created   1998/12/07 (by Hirotsugu Kondo)
 *  @updated   1999/06/25 (by nisinaka)
 *  @updated   2003/05/15 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun475 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dGrapher.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGL3dGrapher extends JunOpenGLDisplayModel {

	protected StMenu _graphMenu;

	/**
	 * Constructor.
	 */
	public JunOpenGL3dGrapher() {
		super();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public void addSelectedObject_(JunOpenGL3dObject a3dObject) {
		JunOpenGL3dObject displayObject = this.displayObject();

		if (displayObject == null) {
			return;
		}

		if (displayObject.isPrimitive()) {
			return;
		}

		JunOpenGL3dNode[] nodes = ((JunOpenGL3dGraph) displayObject).nodes();
		int size = nodes.length;
		JunOpenGL3dNode node = null;

		for (int index = 0; index < size; index++) {
			JunOpenGL3dNode each = nodes[index];

			if (each.displayObject() == a3dObject) {
				node = each;
			}
		}

		if (node == null) {
			return;
		}

		super.addSelectedObject_(a3dObject);
	}

	/**
	 * DOCUMENT ME!
	 */
	public void arrangeGraph() {
		JunOpenGL3dGraph graph = (JunOpenGL3dGraph) this.displayObject();

		if (graph == null) {
			return;
		}

		this.clearSelectedObjects();
		graph.arrange();
		this.resetView();
	}

	/**
	 * DOCUMENT ME!
	 */
	public void arrangeGraph2() {
		JunOpenGL3dGraph graph = (JunOpenGL3dGraph) this.displayObject();

		if (graph == null) {
			return;
		}

		this.clearSelectedObjects();
		graph.arrange2();
		this.resetView();
	}

	/**
	 * DOCUMENT ME!
	 */
	public void arrangeGraphBalance() {
		JunOpenGL3dGraph graph = (JunOpenGL3dGraph) this.displayObject();

		if (graph == null) {
			return;
		}

		graph.arrangeBalance_(!graph.arrangeBalance());
		graph.arrange();
		this.clearSelectedObjects();
		this.updateGraphMenuIndication();
		this.changed_($("object"));
	}

	/**
	 * DOCUMENT ME!
	 */
	public void arrangeGraphCrankArcs() {
		JunOpenGL3dGraph graph = (JunOpenGL3dGraph) this.displayObject();

		if (graph == null) {
			return;
		}

		boolean bool = !this.isCrankArcs();
		JunOpenGL3dArc[] arcs = graph.arcs();
		int arcSize = arcs.length;

		for (int index = 0; index < arcSize; index++) {
			arcs[index].crank_(bool);
		}

		graph.flushDisplayObject();
		this.updateGraphMenuIndication();
		this.changed_($("object"));
	}

	/**
	 * DOCUMENT ME!
	 */
	public void arrangeGraphInterval() {
		JunOpenGL3dGraph graph = (JunOpenGL3dGraph) this.displayObject();

		if (graph == null) {
			return;
		}

		Jun3dPoint interval = graph.arrangeInterval();
		String string = new Double(interval.x()).toString() + "," + new Double(interval.y()).toString() + "," + new Double(interval.z()).toString();
		string = StDialog.Request_("Arrange Interval (x , y , z)", string);

		if (string == null) {
			return;
		}

		if (string.length() == 0) {
			return;
		}

		float x;
		float y;
		float z;

		try {
			StringTokenizer st = new StringTokenizer(string, "(), ");
			x = Float.valueOf(st.nextToken()).floatValue();
			y = Float.valueOf(st.nextToken()).floatValue();
			z = Float.valueOf(st.nextToken()).floatValue();
		} catch (Exception e) {
			return;
		}

		Jun3dPoint point = new Jun3dPoint(x, y, z);
		graph.arrangeInterval_(point);
		graph.arrange();
		this.clearSelectedObjects();
		this.changed_($("object"));
	}

	/**
	 * DOCUMENT ME!
	 */
	public void arrangeGraphWithInterim() {
		final JunOpenGL3dGraph graph = (JunOpenGL3dGraph) this.displayObject();

		if (graph == null) {
			return;
		}

		this.clearSelectedObjects();

		final JunOpenGL3dGrapher this_ = this;
		graph.arrange_(new StBlockClosure() {
			public Object value_(Object arc) {
				graph.flushDisplayObject();
				this_.resetView();

				return null;
			}
		});
		this.resetView();
	}

	/**
	 * DOCUMENT ME!
	 */
	public void arrangeProperties() {
		JunOpenGL3dGraph graph = (JunOpenGL3dGraph) this.displayObject();

		if (graph == null) {
			return;
		}

		System.out.print(graph.arrangeFormat());
	}

	/**
	 * This is stub method in original jun.
	 */
	public void clearObject() {
		System.out.print("do nothing");
	}

	/**
	 * This is stub method in original jun.
	 */
	public void copyObject() {
		System.out.print("do nothing");
	}

	/**
	 * This is stub method in original jun.
	 */
	public void cutObject() {
		System.out.print("do nothing");
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean isArrangeBalance() {
		JunOpenGL3dGraph graph = (JunOpenGL3dGraph) this.displayObject();

		if (graph == null) {
			return new JunOpenGL3dGraph().arrangeBalance();
		}

		return graph.arrangeBalance();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean isCrankArcs() {
		JunOpenGL3dGraph graph = (JunOpenGL3dGraph) this.displayObject();

		if (graph == null) {
			return new JunOpenGL3dArc().crank();
		}

		JunOpenGL3dArc[] arcs = graph.arcs();
		int size = arcs.length;

		for (int index = 0; index < size; index++) {
			boolean crank = arcs[index].crank();

			if (crank) {
				return crank;
			}
		}

		return false;
	}

	/**
	 * Answer whether the display object is empty or not.
	 *
	 * @return boolean
	 */
	public boolean isEmpty() {
		if (this.displayObject() == null) {
			return true;
		}

		if (this.displayObject().isCompound() == false) {
			return false;
		}

		return ((JunOpenGL3dGraph) this.displayObject()).components().length == 0;
	}

	/**
	 * Load a JunOpenGL3dGraph from the reader.
	 * 
	 * @param aReader java.io.BufferedReader
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dGraph
	 * 
	 * @exception IOException
	 */
	public JunOpenGL3dGraph loadFromLGT10_(BufferedReader aReader) throws IOException {
		StringWriter aWriter = new StringWriter();
		JunOpenGL3dGraph anObject = null;

		try {
			int ch;

			while ((ch = aReader.read()) > 0) {
				aWriter.write(ch);
			}

			aWriter.flush();
			anObject = (JunOpenGL3dGraph) JunOpenGL3dGraph.LoadFrom_(aWriter.toString());
		} finally {
			aWriter.close();
		}

		return anObject;
	}

	/**
	 * DOCUMENT ME!
	 */
	public void openLGT() {
		this.openLGT10();
	}

	/**
	 * DOCUMENT ME!
	 */
	public void openLGT10() {
		File file = StDialog.OpenFileDialog_("file dialog");

		if (file == null) {
			return;
		}

		try {
			JunOpenGL3dGraph object = this.readFromLGT10_(file);

			if (object == null) {
				return;
			}

			this.displayObject_(object);
			this.resetView();

			JunOpenGLShowModel model = this.showModel();

			if (model != null) {
				model.resetView();
			}
		} catch (Exception e) {
		}
	}

	/**
	 * This is stub method in original jun.
	 */
	public void pasteObject() {
		System.out.print("do nothing");
	}

	/**
	 * Read a LGT1.0 file and create a graph object.
	 * 
	 * @param aFile java.io.File
	 * 
	 * @return sra.jun.opengl.flux.JunOpenGLFluxObject
	 * 
	 * @exception IOException
	 */
	public JunOpenGL3dGraph readFromLGT10_(File aFile) throws IOException {
		FileReader aFileReader;

		try {
			aFileReader = new FileReader(aFile);
		} catch (FileNotFoundException e) {
			return null;
		}

		BufferedReader aReader = new BufferedReader(aFileReader);
		JunOpenGL3dGraph anObject = null;

		try {
			anObject = this.loadFromLGT10_(aReader);
		} finally {
			aReader.close();
		}

		return anObject;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public void removeSelectedObject_(JunOpenGL3dObject a3dObject) {
		JunOpenGL3dObject displayObject = this.displayObject();

		if (displayObject == null) {
			return;
		}

		if (displayObject.isPrimitive()) {
			return;
		}

		JunOpenGL3dNode[] nodes = ((JunOpenGL3dGraph) displayObject).nodes();
		int size = nodes.length;
		JunOpenGL3dNode node = null;

		for (int index = 0; index < size; index++) {
			JunOpenGL3dNode each = nodes[index];

			if (each.displayObject() == a3dObject) {
				node = each;
			}
		}

		if (node == null) {
			return;
		}

		if (this.selectedObjects().contains(a3dObject)) {
			this.selectedObjects().removeElement(a3dObject);
		}
	}

	/**
	 * Save the graph object to a file.
	 */
	public void saveLGT() {
		this.saveLGT10();
	}

	/**
	 * Save the graph object to a LGT1.0 file.
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public void saveLGT10() {
		JunOpenGL3dGraphAbstract graph = (JunOpenGL3dGraphAbstract) this.displayObject();

		if (graph == null) {
			return;
		}

		File file = StDialog.SaveFileDialog_default_("", graph.name() + ".lgt");

		if (file == null) {
			return;
		}

		try {
			this.writeToLGT10_object_(file, graph);
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Write the JunOpenGL3dObject to the writer.
	 * 
	 * @param aWriter java.io.BufferedWriter
	 * @param aGraph sra.jun.opengl.grapher.JunOpenGL3dGraphAbstract
	 * 
	 * @exception IOException
	 */
	public void saveToLGT10_object_(BufferedWriter aWriter, JunOpenGL3dGraphAbstract aGraph) throws IOException {
		aWriter.write(this.defaultStampForLGT10());
		aGraph.saveOn_(aWriter);
	}

	/**
	 * Save the display object to the VRML1.0 file.
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public void saveWRL10() {
		JunOpenGL3dGraphAbstract graph = (JunOpenGL3dGraphAbstract) this.displayObject();

		if (graph == null) {
			return;
		}

		File file = StDialog.SaveFileDialog_default_("", graph.name() + ".wrl");

		if (file == null) {
			return;
		}

		try {
			this.writeToWRL20_object_(file, graph.displayObject());
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Save the display object to the VRML2.0 file.
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public void saveWRL20() {
		JunOpenGL3dGraphAbstract graph = (JunOpenGL3dGraphAbstract) this.displayObject();

		if (graph == null) {
			return;
		}

		File file = StDialog.SaveFileDialog_default_("", graph.name() + ".wrl");

		if (file == null) {
			return;
		}

		Jun3dBoundingBox box = graph.boundingBox();
		Jun3dPoint center = box.center();
		Jun3dTransformation transformation = Jun3dTransformation.Translate_(new Jun3dPoint(0 - center.x(), 0 - center.y(), 0 - center.z()));
		JunOpenGL3dObject object = graph.displayObject().transform_(transformation);

		try {
			this.writeToWRL20_object_(file, object);
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * DOCUMENT ME!
	 */
	public void selectAll() {
		JunOpenGL3dObject displayObject = this.displayObject();

		if (displayObject == null) {
			return;
		}

		if (displayObject.isPrimitive()) {
			return;
		}

		Vector selectedObjects = this.selectedObjects();
		this.clearSelectedObjects();

		JunOpenGL3dNode[] nodes = ((JunOpenGL3dGraph) displayObject).nodes();
		int size = nodes.length;

		for (int index = 0; index < size; index++) {
			selectedObjects.addElement(nodes[index]);
		}

		this.changed_($("selection"));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.util.Vector
	 */
	public Vector selectedNodes() {
		Vector selectedNodes = new Vector();
		Vector selectedObjects = this.selectedObjects();

		for (int index = 0; index < selectedObjects.size(); index++) {
			JunOpenGL3dGraph each = (JunOpenGL3dGraph) selectedObjects.elementAt(index);
			JunOpenGL3dNode[] nodes = ((JunOpenGL3dGraph) each.displayObject()).nodes();
			int size = nodes.length;

			for (int i = 0; i < size; i++) {
				JunOpenGL3dNode node = nodes[index];

				if (node.displayObject() == each) {
					selectedNodes.addElement(node);
				}
			}
		}

		return selectedNodes;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject spawningObject() {
		JunOpenGL3dGraph spawningObject;
		Vector selectedObjects = this.selectedObjects();

		if (selectedObjects.isEmpty()) {
			spawningObject = (JunOpenGL3dGraph) this.displayObject().copy();
		} else {
			spawningObject = new JunOpenGL3dGraph();

			Vector selectedNodes = this.selectedNodes();

			for (int index = 0; index < selectedNodes.size(); index++) {
				spawningObject.addNode_((JunOpenGL3dNode) selectedNodes.elementAt(index));
			}
		}

		return spawningObject;
	}

	/**
	 * Update the graph menu indication.
	 */
	public void updateGraphMenuIndication() {
		StMenu graphMenu = (StMenu) this.menuModel().getItem("Graph");
		if (graphMenu == null) {
			return;
		}

		StMenuItem menuItem;
		boolean displayObjectIsNotEmpty = !this.isEmpty();

		// Arrange
		menuItem = graphMenu.getItemWithValue("arrangeGraph");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}

		// Arrange2
		menuItem = graphMenu.getItemWithValue("arrangeGraph2");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}

		// Interval
		menuItem = graphMenu.getItemWithValue("arrangeGraphInterval");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
		}

		// Balance
		menuItem = graphMenu.getItemWithValue("arrangeGraphBalance");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
			if (displayObjectIsNotEmpty) {
				((StCheckBoxMenuItem) menuItem).setSelected(this.isArrangeBalance());
			}
		}

		// Crank arcs
		menuItem = graphMenu.getItemWithValue("arrangeGraphCrankArcs");
		if (menuItem != null) {
			menuItem.setEnabled(displayObjectIsNotEmpty);
			if (displayObjectIsNotEmpty) {
				((StCheckBoxMenuItem) menuItem).setSelected(this.isCrankArcs());
			}
		}
	}

	/**
	 * Update the menu indications.
	 */
	public void updateMenuIndication() {
		super.updateMenuIndication();
		this.updateGraphMenuIndication();
	}

	/**
	 * Write the JunOpenGL3dObject to the LGT1.0 file.
	 * 
	 * @param aFile java.io.File
	 * @param aGraph sra.jun.opengl.grapher.JunOpenGL3dGraphAbstract
	 * 
	 * @exception IOException
	 */
	public void writeToLGT10_object_(File aFile, JunOpenGL3dGraphAbstract aGraph) throws IOException {
		if (aFile == null) {
			return;
		}

		BufferedWriter aWriter = new BufferedWriter(new FileWriter(aFile));

		try {
			this.saveToLGT10_object_(aWriter, aGraph);
		} finally {
			aWriter.flush();
			aWriter.close();
		}
	}

	/**
	 * Answer the default file menu for JunOpenGL3dGrapher.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _defaultFileMenu() {
		StMenu defaultFileMenu = new StMenu("File");

		// New
		defaultFileMenu.add(createMenuItem("New", "newModel"));

		// Open...
		defaultFileMenu.add(createMenuItem("Open...", "openLST"));

		//
		defaultFileMenu.addSeparator();

		// Save...
		defaultFileMenu.add(createMenuItem("Save...", "saveLST"));

		// Save as...
		StMenu saveAsMenu = new StMenu("Save as...");
		saveAsMenu.add(createMenuItem("VRML1.0...", "saveWRL10"));
		saveAsMenu.add(createMenuItem("VRML2.0...", "saveWRL20"));
		saveAsMenu.add(createMenuItem("VRML97...", "saveWRL97"));
		defaultFileMenu.add(saveAsMenu);

		// Save as image...
		defaultFileMenu.add(createMenuItem("Save as image...", "saveAsImage"));

		//
		defaultFileMenu.addSeparator();

		// Quit
		defaultFileMenu.add(createMenuItem("Quit", "quitDoing"));

		return defaultFileMenu;
	}

	/**
	 * Answer the default graph menu for JunOpenGL3dGrapher.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _defaultGraphMenu() {
		StMenu defaultGraphMenu = new StMenu("Graph");

		defaultGraphMenu.add(createMenuItem("Arrange", "arrangeGraph"));
		defaultGraphMenu.add(createMenuItem("Arrange2", "arrangeGraph2"));
		defaultGraphMenu.addSeparator();
		defaultGraphMenu.add(createMenuItem("Interval", "arrangeGraphInterval"));
		defaultGraphMenu.add(createCheckBoxMenuItem("Balance", "arrangeGraphBalance"));
		defaultGraphMenu.addSeparator();
		defaultGraphMenu.add(createCheckBoxMenuItem("Crank arcs", "arrangeGraphCrankArcs"));

		return defaultGraphMenu;
	}

	/**
	 * Answer the default misc menu for JunOpenGL3dGrapher.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _defaultMiscMenu() {
		StMenu defaultMiscMenu = new StMenu("Misc");

		defaultMiscMenu.add(createMenuItem("Spawn", "spawnObject"));
		defaultMiscMenu.add(createMenuItem("Bounds", "showBounds"));

		return defaultMiscMenu;
	}

	/**
	 * Answer my graph menu.
	 *
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu _graphMenu() {
		if (_graphMenu == null) {
			_graphMenu = this._defaultGraphMenu();
		}
		return _graphMenu;
	}

	/**
	 * Create the menubar for JunOpenGLDisplay.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 */
	protected StMenuModel defaultMenu() {
		StMenuModel rootMenu = new StMenuModel();

		rootMenu.add(this._fileMenu());
		rootMenu.add(this._editMenu());
		rootMenu.add(this._viewMenu());
		rootMenu.add(this._lightMenu());
		rootMenu.add(this._graphMenu());
		rootMenu.add(this._miscMenu());

		return rootMenu;
	}

	/**
	 * Answer the default stamp string for LGT1.0.
	 * 
	 * @return java.lang.String
	 */
	private String defaultStampForLGT10() {
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		pw.println("%LGT V1.0 List Grapher Transmission (Lisp S Expression)");
		pw.println("% This file was created by " + JunSystem.System() + JunSystem.Version());
		pw.println("% " + DateFormat.getInstance().format(new Date()));
		pw.println();
		pw.flush();

		return sw.toString();
	}

	/**
	 * Answer the window title.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "OpenGL 3D Grapher";
	}
}
