package sra.jun.opengl.grapher;

import java.awt.Color;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.*;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.objects.*;
import sra.jun.opengl.texture.*;
import sra.smalltalk.*;

/**
 * JunOpenGL3dNode class
 * 
 *  @author    Hirotsugu Kondo
 *  @created   1998/12/07 (by Hirotsugu Kondo)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dNode.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGL3dNode extends JunOpenGL3dGraphAbstract {
	private Jun3dPoint nodeLocation;
	private Jun3dPoint nodeExtent;
	private Object nodeEntity;
	private JunOpenGLTexture nodeTexture;

	/**
	 * Constructor.
	 */
	public JunOpenGL3dNode() {
		super();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param extentPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	public static JunOpenGL3dNode Extent_(Jun3dPoint extentPoint) {
		return Extent_color_(extentPoint, Color.gray);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param extentPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param colorValue java.awt.Color
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	public static JunOpenGL3dNode Extent_color_(Jun3dPoint extentPoint, Color colorValue) {
		return Location_extent_color_(new Jun3dPoint(0, 0, 0), extentPoint, colorValue);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param extentPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param colorValue java.awt.Color
	 * @param anObject java.lang.Object
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	public static JunOpenGL3dNode Extent_color_entity_(Jun3dPoint extentPoint, Color colorValue, Object anObject) {
		return Location_extent_color_entity_(new Jun3dPoint(0, 0, 0), extentPoint, colorValue, anObject);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param extentPoint sra.jun.opengl.basic.Jun3dPoint
	 * @param anObject java.lang.Object
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	public static JunOpenGL3dNode Extent_entity_(Jun3dPoint extentPoint, Object anObject) {
		return Location_extent_color_entity_(new Jun3dPoint(0, 0, 0), extentPoint, null, anObject);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dGraphAbstract
	 */
	public static JunOpenGL3dGraphAbstract FromLispList_(JunLispList aList) {
		JunOpenGL3dNode node = new JunOpenGL3dNode();
		node = (JunOpenGL3dNode) ColorFromLispList_for_(aList, node);
		node = LocationFromLispList_for_(aList, node);
		node = ExtentFromLispList_for_(aList, node);
		node = EntityFromLispList_for_(aList, node);
		node = TextureFromLispList_for_(aList, node);

		return node;
	}

	/**
	 * Load a JunOpenGL3dNode from the object.
	 * 
	 * @param anObject java.lang.Object
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject LoadFrom_(Object anObject) {
		JunLispList aList = (JunLispList) JunLispParser.Parse_(anObject);

		return JunOpenGL3dNode.FromLispList_(aList);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param locationPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param extentPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	public static JunOpenGL3dNode Location_extent_(Jun3dPoint locationPoint, Jun3dPoint extentPoint) {
		return Location_extent_color_entity_(locationPoint, extentPoint, null, null);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param locationPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param extentPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param colorValue java.awt.Color
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	public static JunOpenGL3dNode Location_extent_color_(Jun3dPoint locationPoint, Jun3dPoint extentPoint, Color colorValue) {
		return Location_extent_color_entity_(locationPoint, extentPoint, colorValue, null);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param locationPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param extentPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param colorValue java.awt.Color
	 * @param anObject java.lang.Object
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	public static JunOpenGL3dNode Location_extent_color_entity_(Jun3dPoint locationPoint, Jun3dPoint extentPoint, Color colorValue, Object anObject) {
		JunOpenGL3dNode node = new JunOpenGL3dNode();
		node.location_(locationPoint);
		node.extent_(extentPoint);
		node.color_(colorValue);
		node.entity_(anObject);

		return node;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param locationPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param extentPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param anObject java.lang.Object
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	public static JunOpenGL3dNode Location_extent_entity_(Jun3dPoint locationPoint, Jun3dPoint extentPoint, Object anObject) {
		return Location_extent_color_entity_(locationPoint, extentPoint, null, anObject);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	private static JunOpenGL3dNode EntityFromLispList_for_(JunLispList aList, JunOpenGL3dNode aNode) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("entity")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return aNode;
		}

		aNode.entity_(list.tail());

		return aNode;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	private static JunOpenGL3dNode ExtentFromLispList_for_(JunLispList aList, JunOpenGL3dNode aNode) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("extent")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return aNode;
		}

		aNode.extent_((Jun3dPoint) list.tail());

		return aNode;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	private static JunOpenGL3dNode LocationFromLispList_for_(JunLispList aList, JunOpenGL3dNode aNode) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("location")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return aNode;
		}

		aNode.location_((Jun3dPoint) list.tail());

		return aNode;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @param aNode sra.jun.opengl.grapher.JunOpenGL3dNode
	 * 
	 * @return sra.jun.opengl.grapher.JunOpenGL3dNode
	 */
	private static JunOpenGL3dNode TextureFromLispList_for_(JunLispList aList, JunOpenGL3dNode aNode) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("texture")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return aNode;
		}

		aNode.texture_(JunOpenGLTexture.FromLispList_((JunLispList) list.tail()));

		return aNode;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint corner() {
		Jun3dPoint location = this.location();

		return new Jun3dPoint(location.x() + this.width(), location.y() + (this.height() / 2), location.z() + (this.depth() / 2));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return double
	 */
	public double depth() {
		return this.extent().z();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.lang.Object
	 */
	public Object entity() {
		return this.nodeEntity;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anObject java.lang.Object
	 */
	public void entity_(Object anObject) {
		this.nodeEntity = anObject;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons entityToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("entity"));
		list.tail_(this.entity());

		return list;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint extent() {
		Jun3dPoint extent = this.nodeExtent;

		if (extent == null) {
			extent = new Jun3dPoint(1, 1, 1);
			this.nodeExtent = extent;
		}

		return extent;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public void extent_(Jun3dPoint a3dPoint) {
		this.nodeExtent = a3dPoint;
		this.flushDisplayObject();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons extentToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("extent"));
		list.tail_(this.extent());

		return list;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean hasEntity() {
		return this.entity() != null;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean hasTexture() {
		return this.texture() != null;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return double
	 */
	public double height() {
		return this.extent().y();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean isNode() {
		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean isPrimitive() {
		return true;
	}

	/**
	 * Answer a String which represents the receiver's kind.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public StSymbol kindName() {
		return $("Node");
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint location() {
		Jun3dPoint location = this.nodeLocation;

		if (location == null) {
			location = new Jun3dPoint(0, 0, 0);
			this.nodeLocation = location;
		}

		return location;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public void location_(Jun3dPoint a3dPoint) {
		this.nodeLocation = a3dPoint;
		this.flushDisplayObject();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons locationToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("location"));
		list.tail_(this.location());

		return list;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint origin() {
		Jun3dPoint location = this.location();

		return new Jun3dPoint(location.x(), location.y() - (this.height() / 2), location.z() - (this.depth() / 2));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.texture.JunOpenGLTexture
	 */
	public JunOpenGLTexture texture() {
		return this.nodeTexture;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aTexture sra.jun.opengl.texture.JunOpenGLTexture
	 */
	public void texture_(JunOpenGLTexture aTexture) {
		this.nodeTexture = aTexture;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList textureToLispList() {
		if (!this.hasTexture()) {
			return this.lispNil();
		} else {
			JunLispCons list = this.lispCons();
			list.head_($("texture"));
			list.tail_(this.texture().toLispList());

			return list;
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList toLispList() {
		JunLispCons list = this.lispCons();
		list.head_(this.kindName());

		if (this.hasColor()) {
			list.add_(this.colorToLispList());
		}

		list.add_(this.locationToLispList());
		list.add_(this.extentToLispList());
		list.head_(this.kindName());

		if (this.hasEntity()) {
			list.add_(this.entityToLispList());
		}

		list.head_(this.kindName());

		if (this.hasTexture()) {
			list.add_(this.textureToLispList());
		}

		return list;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject transform_(Jun3dTransformation aTransformation) {
		this.location_((Jun3dPoint) this.location().transform_(aTransformation));

		return this;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return double
	 */
	public double width() {
		return this.extent().x();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected JunOpenGL3dObject createDisplayObject() {
		JunOpenGL3dObject object = JunOpenGL3dObject.Origin_corner_(this.origin(), this.corner());

		if (this.hasColor()) {
			object.paint_(this.color());
		}

		if (this.hasTexture()) {
			object.texture_(this.texture());

			if (this.texture().coordinates() == null) {
				this.texture().zPositiveParameters();
			}
		}

		return object;
	}
}
