package sra.jun.opengl.lights;

import java.awt.Color;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.opengl.support.JunOpenGLRenderingContext;

/**
 * JunOpenGLAmbientLight class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/10/23 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLAmbientLight.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGLAmbientLight extends JunOpenGLLight {

	/**
	 * Create a new instance of JunOpenGLAmbientLight.
	 */
	public JunOpenGLAmbientLight() {
		super();
	}

	/**
	 * Create a new JunOpenGLAmbientLight and set color and answer it.
	 * 
	 * @param aColorValue java.awt.Color
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLAmbientLight
	 */
	public static JunOpenGLAmbientLight Color_(Color aColorValue) {
		return Color_alpha_(aColorValue, DefaultAlpha());
	}

	/**
	 * Create a new JunOpenGLAmbientLight and set color with alpha and answer
	 * it.
	 * 
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLAmbientLight
	 */
	public static JunOpenGLAmbientLight Color_alpha_(Color aColorValue, float aNumber) {
		JunOpenGLAmbientLight aLight;

		aLight = new JunOpenGLAmbientLight();
		aLight.setColor_alpha_(aColorValue, aNumber);

		return aLight;
	}

	/**
	 * Lighting on a RenderingContext.
	 * 
	 * @param aRenderingContext
	 *        sra.jun.opengl.support.JunOpenGLRenderingContext
	 */
	public void lightingOn_(JunOpenGLRenderingContext aRenderingContext) {
		aRenderingContext.lightAmbientColor_alpha_(this.color, this.alpha);
		aRenderingContext.enableLighting();
	}

	/**
	 * Transformation on the light.
	 * 
	 * @param aJun3dTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLLight
	 */
	public JunOpenGLLight transform_(Jun3dTransformation aJun3dTransformation) {
		return (JunOpenGLAmbientLight) this.copy();
	}

	/**
	 * Set color with alpha to receiver.
	 * 
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 */
	protected void setColor_alpha_(Color aColorValue, float aNumber) {
		color = aColorValue;
		alpha = aNumber;
	}
}
