package sra.jun.opengl.lights;

import java.awt.Color;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.opengl.support.JunOpenGLRenderingContext;
import sra.smalltalk.*;

/**
 * JunOpenGLLight class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/10/23 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLLight.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public abstract class JunOpenGLLight extends StObject {
	/** The color as a Color. */
	protected Color color = Color.white;

	/** The alpha as a float. */
	protected float alpha = 1.0f;

	/**
	 * Create a new instance of JunOpenGLLight.
	 */
	public JunOpenGLLight() {
		super();
	}

	/**
	 * Answer a DefaultAlpha.
	 * 
	 * @return float
	 */
	public static final float DefaultAlpha() {
		return 1.0f;
	}

	/**
	 * Answer a DefaultColor.
	 * 
	 * @return java.awt.Color
	 */
	public static final Color DefaultColor() {
		return Color.white;
	}

	/**
	 * Answer the alpha.
	 * 
	 * @return float
	 */
	public final float alpha() {
		return alpha;
	}

	/**
	 * Set a number to the alpha.
	 * 
	 * @param aNumber float
	 */
	public final void alpha_(float aNumber) {
		alpha = aNumber;
	}

	/**
	 * Answer the color.
	 * 
	 * @return java.awt.Color
	 */
	public final Color color() {
		return color;
	}

	/**
	 * Set a Color to the color.
	 * 
	 * @param aColorValue java.awt.Color
	 */
	public final void color_(Color aColorValue) {
		color = aColorValue;
	}

	/**
	 * Answer true if the receiver is a kind of JunOpenGLLight, otherwise
	 * false.
	 * 
	 * @return boolean
	 */
	public boolean isLight() {
		return true;
	}

	/**
	 * Lighting on a RenderingContext.
	 * 
	 * @param aRenderingContext float
	 */
	public abstract void lightingOn_(JunOpenGLRenderingContext aRenderingContext);

	/**
	 * Transformation on the light.
	 * 
	 * @param aJun3dTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLLight
	 */
	public abstract JunOpenGLLight transform_(Jun3dTransformation aJun3dTransformation);

	/**
	 * Answer a value as receiver.
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLLight
	 */
	public JunOpenGLLight value() {
		return this;
	}
}
