package sra.jun.opengl.lights;

import java.awt.Color;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.opengl.support.JunOpenGLRenderingContext;

/**
 * JunOpenGLParallelLight class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/10/23 (by MATSUDA Ryouichi)
 *  @updated   1999/08/04 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLParallelLight.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGLParallelLight extends JunOpenGLLight {
	/** The direction as a Jun3dPoint. */
	protected Jun3dPoint direction;

	/**
	 * Create a new instance of JunOpenGLParallelLight.
	 */
	public JunOpenGLParallelLight() {
		super();
	}

	/**
	 * Create a new JunOpenGLParallelLight and set point and answer it.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLParallelLight
	 */
	public static JunOpenGLParallelLight At3dPoint_(Jun3dPoint a3dPoint) {
		return At3dPoint_color_(a3dPoint, DefaultColor());
	}

	/**
	 * Create a new JunOpenGLParallelLight and set point with color and answer
	 * it.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aColorValue java.awt.Color
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLParallelLight
	 */
	public static JunOpenGLParallelLight At3dPoint_color_(Jun3dPoint a3dPoint, Color aColorValue) {
		return At3dPoint_color_alpha_(a3dPoint, aColorValue, DefaultAlpha());
	}

	/**
	 * Create a new JunOpenGLParallelLight and set point with color with alpha
	 * and answer it.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLParallelLight
	 */
	public static JunOpenGLParallelLight At3dPoint_color_alpha_(Jun3dPoint a3dPoint, Color aColorValue, float aNumber) {
		JunOpenGLParallelLight aLight = new JunOpenGLParallelLight();
		aLight.position_(a3dPoint);
		aLight.color_(aColorValue);
		aLight.alpha_(aNumber);

		return aLight;
	}

	/**
	 * Create a new JunOpenGLParallelLight and set point with color with alpha
	 * and answer it.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aColorValue java.awt.Color
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLParallelLight
	 */
	public static JunOpenGLParallelLight Direction_color_(Jun3dPoint a3dPoint, Color aColorValue) {
		return Direction_color_alpha_(a3dPoint, aColorValue, DefaultAlpha());
	}

	/**
	 * Create a new JunOpenGLParallelLight and set point with color with alpha
	 * and answer it.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLParallelLight
	 */
	public static JunOpenGLParallelLight Direction_color_alpha_(Jun3dPoint a3dPoint, Color aColorValue, float aNumber) {
		JunOpenGLParallelLight aLight = new JunOpenGLParallelLight();
		aLight.direction_(a3dPoint);
		aLight.color_(aColorValue);
		aLight.alpha_(aNumber);

		return aLight;
	}

	/**
	 * Answer the direction.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public final Jun3dPoint direction() {
		return direction;
	}

	/**
	 * Set the direction.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public final void direction_(Jun3dPoint a3dPoint) {
		direction = a3dPoint;
	}

	/**
	 * Lighting on a RenderingContext.
	 * 
	 * @param aRenderingContext
	 *        sra.jun.opengl.support.JunOpenGLRenderingContext
	 */
	public void lightingOn_(JunOpenGLRenderingContext aRenderingContext) {
		int index = aRenderingContext.getLightIndex();
		aRenderingContext.lightIndex_position_direction_cutoffAngle_color_alpha_(index, null, this.direction(), null, this.color(), this.alpha());
		aRenderingContext.enableLightIndex_(index);
		aRenderingContext.enableLighting();
	}

	/**
	 * Answer the position.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint position() {
		return (Jun3dPoint) this.direction().negated();
	}

	/**
	 * Set the position.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public final void position_(Jun3dPoint a3dPoint) {
		this.direction_((Jun3dPoint) a3dPoint.negated());
	}

	/**
	 * Transformation on the light.
	 * 
	 * @param aJun3dTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLLight
	 */
	public JunOpenGLLight transform_(Jun3dTransformation aJun3dTransformation) {
		JunOpenGLParallelLight transformed = (JunOpenGLParallelLight) this.copy();
		transformed.direction_((Jun3dPoint) ((Jun3dPoint) this.direction().transform_(aJun3dTransformation)).minus_((Jun3dPoint) Jun3dPoint.Zero().transform_(aJun3dTransformation)));

		return transformed;
	}
}
