package sra.jun.opengl.lights;

import java.awt.Color;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.opengl.support.JunOpenGLRenderingContext;

/**
 * JunOpenGLPointLight class
 * 
 *  @author    Hirotsugu Kondo
 *  @created   1999/5/10 (by Hirotsugu Kondo)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLPointLight.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGLPointLight extends JunOpenGLLight {
	/** The position as a Jun3dPoint. */
	protected Jun3dPoint position;

	/**
	 * Create a new JunOpenGLPointLight and set point and answer it.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLPointLight
	 */
	public static JunOpenGLPointLight At3dPoint_(Jun3dPoint a3dPoint) {
		return At3dPoint_color_(a3dPoint, DefaultColor());
	}

	/**
	 * Create a new JunOpenGLPointLight and set point with color and answer it.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aColorValue java.awt.Color
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLPointLight
	 */
	public static JunOpenGLPointLight At3dPoint_color_(Jun3dPoint a3dPoint, Color aColorValue) {
		return At3dPoint_color_alpha_(a3dPoint, aColorValue, DefaultAlpha());
	}

	/**
	 * Create a new JunOpenGLPointLight and set point with color with alpha and
	 * answer it.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLPointLight
	 */
	public static JunOpenGLPointLight At3dPoint_color_alpha_(Jun3dPoint a3dPoint, Color aColorValue, float aNumber) {
		JunOpenGLPointLight aLight;

		aLight = new JunOpenGLPointLight();
		aLight.setPosition_color_alpha_(a3dPoint, aColorValue, aNumber);

		return aLight;
	}

	/**
	 * Lighting on a RenderingContext.
	 * 
	 * @param aRenderingContext
	 *        sra.jun.opengl.support.JunOpenGLRenderingContext
	 */
	public void lightingOn_(JunOpenGLRenderingContext aRenderingContext) {
		int index;
		index = aRenderingContext.getLightIndex();
		aRenderingContext.lightIndex_position_direction_cutoffAngle_color_alpha_(index, this.position(), null, null, this.color(), this.alpha());
		aRenderingContext.enableLightIndex_(index);
		aRenderingContext.enableLighting();
	}

	/**
	 * Answer the position.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public final Jun3dPoint position() {
		return position;
	}

	/**
	 * Set the position.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public final void position_(Jun3dPoint a3dPoint) {
		position = a3dPoint;
	}

	/**
	 * Transformation on the light.
	 * 
	 * @param aJun3dTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLLight
	 */
	public JunOpenGLLight transform_(Jun3dTransformation aJun3dTransformation) {
		JunOpenGLPointLight transformed = (JunOpenGLPointLight) this.copy();
		transformed.position_((Jun3dPoint) this.position().transform_(aJun3dTransformation));

		return transformed;
	}

	/**
	 * Set the position, color and alpha.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 */
	protected void setPosition_color_alpha_(Jun3dPoint a3dPoint, Color aColorValue, float aNumber) {
		position = a3dPoint;
		color = aColorValue;
		alpha = aNumber;
	}
}
