package sra.jun.opengl.lights;

import java.awt.Color;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.geometry.basic.JunAngle;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.opengl.support.JunOpenGLRenderingContext;

/**
 * JunOpenGLSpotLight class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/10/23 (by MATSUDA Ryouichi)
 *  @updated   1999/05/10 (by kondo)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLSpotLight.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGLSpotLight extends JunOpenGLLight {
	/** Position of the spot light. */
	protected Jun3dPoint position;

	/** Direction of the spot light. */
	protected Jun3dPoint direction;

	/** Cutoff angle of the spot light. */
	protected JunAngle cutoffAngle;

	/**
	 * Create a new instance of JunOpenGLSpotLight.
	 */
	public JunOpenGLSpotLight() {
		super();
	}

	/**
	 * Create a new JunOpenGLSpotLight and set point and answer it.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLSpotLight
	 */
	public static JunOpenGLSpotLight At3dPoint_(Jun3dPoint a3dPoint) {
		return At3dPoint_color_(a3dPoint, DefaultColor());
	}

	/**
	 * Create a new JunOpenGLParallelLight and set point with color and answer
	 * it.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aColorValue java.awt.Color
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLSpotLight
	 */
	public static JunOpenGLSpotLight At3dPoint_color_(Jun3dPoint a3dPoint, Color aColorValue) {
		return At3dPoint_color_alpha_(a3dPoint, aColorValue, DefaultAlpha());
	}

	/**
	 * Create a new JunOpenGLSpotLight and set point with color with alpha and
	 * answer it.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLSpotLight
	 */
	public static JunOpenGLSpotLight At3dPoint_color_alpha_(Jun3dPoint a3dPoint, Color aColorValue, float aNumber) {
		return At3dPoint_direction_cutoffAngle_color_alpha_(a3dPoint, null, null, aColorValue, aNumber);
	}

	/**
	 * Create a new JunOpenGLSpotLight and set point with color with alpha and
	 * answer it.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint2 DOCUMENT ME!
	 * @param aJunAngle DOCUMENT ME!
	 * @param aColorValue java.awt.Color
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLSpotLight
	 */
	public static JunOpenGLSpotLight At3dPoint_direction_cutoffAngle_color_(Jun3dPoint a3dPoint, Jun3dPoint a3dPoint2, JunAngle aJunAngle, Color aColorValue) {
		return At3dPoint_direction_cutoffAngle_color_alpha_(a3dPoint, a3dPoint2, aJunAngle, aColorValue, DefaultAlpha());
	}

	/**
	 * Create a new JunOpenGLSpotLight and set point with color with alpha and
	 * answer it.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint2 DOCUMENT ME!
	 * @param aJunAngle DOCUMENT ME!
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLSpotLight
	 */
	public static JunOpenGLSpotLight At3dPoint_direction_cutoffAngle_color_alpha_(Jun3dPoint a3dPoint, Jun3dPoint a3dPoint2, JunAngle aJunAngle, Color aColorValue, float aNumber) {
		JunOpenGLSpotLight aLight = new JunOpenGLSpotLight();
		aLight.setPosition_direction_cutoffAngle_color_alpha_(a3dPoint, a3dPoint2, aJunAngle, aColorValue, aNumber);

		return aLight;
	}

	/**
	 * Answer the cutoffAngle.
	 * 
	 * @return sra.jun.geometry.basic.JunAngle
	 */
	public final JunAngle cutoffAngle() {
		return cutoffAngle;
	}

	/**
	 * Set the cutoff angle.
	 * 
	 * @param aJunAngle
	 */
	public void cutoffAngle_(JunAngle aJunAngle) {
		cutoffAngle = aJunAngle;
	}

	/**
	 * Answer the direction.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public final Jun3dPoint direction() {
		return direction;
	}

	/**
	 * Set the direction.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public final void direction_(Jun3dPoint a3dPoint) {
		direction = a3dPoint;
	}

	/**
	 * Lighting on a RenderingContext.
	 * 
	 * @param aRenderingContext
	 *        sra.jun.opengl.support.JunOpenGLRenderingContext
	 */
	public void lightingOn_(JunOpenGLRenderingContext aRenderingContext) {
		Jun3dPoint lightingDirection = this.direction();

		if (lightingDirection == null) {
			lightingDirection = (Jun3dPoint) aRenderingContext.projection().sightPoint().minus_(this.position());
		}

		JunAngle lightingAngle = this.cutoffAngle();

		if (lightingAngle == null) {
			lightingAngle = JunAngle.FromDeg_(45);
		}

		int index = aRenderingContext.getLightIndex();
		aRenderingContext.lightIndex_position_direction_cutoffAngle_color_alpha_(index, this.position(), lightingDirection, lightingAngle, this.color(), this.alpha());
		aRenderingContext.enableLightIndex_(index);
		aRenderingContext.enableLighting();
	}

	/**
	 * Answer the position.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public final Jun3dPoint position() {
		return position;
	}

	/**
	 * Set the position.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public final void position_(Jun3dPoint a3dPoint) {
		position = a3dPoint;
	}

	/**
	 * Transformation on the light.
	 * 
	 * @param aJun3dTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.opengl.lights.JunOpenGLLight
	 */
	public JunOpenGLLight transform_(Jun3dTransformation aJun3dTransformation) {
		JunOpenGLSpotLight transformed = (JunOpenGLSpotLight) this.copy();
		transformed.position_((Jun3dPoint) this.position().transform_(aJun3dTransformation));

		if (direction != null) {
			Jun3dPoint p1 = (Jun3dPoint) this.direction().transform_(aJun3dTransformation);
			Jun3dPoint p2 = (Jun3dPoint) Jun3dPoint.Zero().transform_(aJun3dTransformation);
			transformed.direction_((Jun3dPoint) p1.minus_(p2));
		}

		return transformed;
	}

	/**
	 * Set the position, color and alpha.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 */
	protected void setPosition_color_alpha_(Jun3dPoint a3dPoint, Color aColorValue, float aNumber) {
		position = a3dPoint;
		color = aColorValue;
		alpha = aNumber;
	}

	/**
	 * Set the position, color and alpha.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunAngle sra.jun.geometry.basic.JunAngle
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 */
	protected void setPosition_direction_cutoffAngle_color_alpha_(Jun3dPoint a3dPoint, Jun3dPoint a3dPoint2, JunAngle aJunAngle, Color aColorValue, float aNumber) {
		position = a3dPoint;
		direction = a3dPoint2;
		cutoffAngle = aJunAngle;
		color = aColorValue;
		alpha = aNumber;
	}
}
