package sra.jun.opengl.microcosm;

import java.awt.*;
import sra.smalltalk.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.curves.Jun3dLine;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.opengl.objects.*;
import sra.jun.opengl.texture.JunOpenGLTexture;

/**
 * JunOpenGLMicrocosmModelTestExamples class
 * 
 *  @author    nisinaka
 *  @created   1999/08/18 (by nisinaka)
 *  @updated   2002/01/23 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLMicrocosmModelTestExamples.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGLMicrocosmModelTestExamples extends sra.jun.system.support.JunTestExamples {

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example1() {
		JunOpenGLMicrocosmModel displayModel = new JunOpenGLMicrocosmModel();
		displayModel.displayObject_(Room1());
		displayModel.open();

		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param args java.lang.String[]
	 */
	public static void main(String[] args) {
		new JunOpenGLMicrocosmModelTestExamples();
	}

	/**
	 * Create a back side image1.
	 * 
	 * @return java.awt.Image
	 */
	protected static Image BackSideImage1() {
		return SystemResourceSupport.createImage("/sra/jun/opengl/microcosm/BackSideImage1.jpg");
	}

	/**
	 * Create a back side wall 1.
	 * 
	 * @return java.awt.Image
	 */
	protected static JunOpenGL3dObject BackSideWall1() {
		Jun3dPoint[] vertexes = new Jun3dPoint[] { new Jun3dPoint(1, -1, 0), new Jun3dPoint(1, 1, 0), new Jun3dPoint(-1, 1, 0), new Jun3dPoint(-1, -1, 0)};
		JunOpenGL3dPolygon aPolygon = new JunOpenGL3dPolygon(vertexes);
		aPolygon.paint_(Color.getHSBColor(0f, 0f, 0.9f));

		JunAngle anAngle = JunAngle.FromDeg_(0);
		Jun3dTransformation aT = Jun3dTransformation.Translate_(new Jun3dPoint(0, 0, -1));
		Jun3dLine aLine = new Jun3dLine(new Jun3dPoint(0, 0, 0), new Jun3dPoint(0, 1, 0));
		Jun3dTransformation aTransformation = Jun3dTransformation.Unity();
		aTransformation = (Jun3dTransformation) aTransformation.product_(Jun3dTransformation.Rotate_around_(anAngle, aLine));
		aTransformation = (Jun3dTransformation) aTransformation.product_(aT);

		JunOpenGL3dObject aWall = aPolygon.transform_(aTransformation);
		Image anImage = BackSideImage1();
		JunOpenGLTexture aTexture = JunOpenGLTexture.Image_(anImage);
		aTexture.coordinates_(new Jun2dPoint[] { new Jun2dPoint(1.0, 0.0), new Jun2dPoint(1.0, 1.0), new Jun2dPoint(0.0, 1.0), new Jun2dPoint(0.0, 0.0)});
		aTexture.linear_(true);
		aTexture.clamp_(true);
		aTexture.modulate_(true);
		aTexture.mipmap_(true);
		aWall.texture_(aTexture);

		return aWall;
	}

	/**
	 * Create a down side image1.
	 * 
	 * @return java.awt.Image
	 */
	protected static Image DownSideImage1() {
		return SystemResourceSupport.createImage("/sra/jun/opengl/microcosm/DownSideImage1.jpg");
	}

	/**
	 * Create a down side wall 1.
	 * 
	 * @return java.awt.Image
	 */
	protected static JunOpenGL3dObject DownSideWall1() {
		Jun3dPoint[] vertexes = new Jun3dPoint[] { new Jun3dPoint(1, -1, 0), new Jun3dPoint(1, 1, 0), new Jun3dPoint(-1, 1, 0), new Jun3dPoint(-1, -1, 0)};
		JunOpenGL3dPolygon aPolygon = new JunOpenGL3dPolygon(vertexes);
		aPolygon.paint_(Color.getHSBColor(0f, 0f, 0.9f));

		JunAngle anAngle = JunAngle.FromDeg_(-90);
		Jun3dTransformation aT = Jun3dTransformation.Translate_(new Jun3dPoint(0, -1, 0));
		Jun3dLine aLine = new Jun3dLine(new Jun3dPoint(0, 0, 0), new Jun3dPoint(1, 0, 0));
		Jun3dTransformation aTransformation = Jun3dTransformation.Unity();
		aTransformation = (Jun3dTransformation) aTransformation.product_(Jun3dTransformation.Rotate_around_(anAngle, aLine));
		aTransformation = (Jun3dTransformation) aTransformation.product_(aT);

		JunOpenGL3dObject aWall = aPolygon.transform_(aTransformation);
		Image anImage = DownSideImage1();
		JunOpenGLTexture aTexture = JunOpenGLTexture.Image_(anImage);
		aTexture.coordinates_(new Jun2dPoint[] { new Jun2dPoint(1.0, 0.0), new Jun2dPoint(1.0, 1.0), new Jun2dPoint(0.0, 1.0), new Jun2dPoint(0.0, 0.0)});
		aTexture.linear_(true);
		aTexture.clamp_(true);
		aTexture.modulate_(true);
		aTexture.mipmap_(true);
		aWall.texture_(aTexture);

		return aWall;
	}

	/**
	 * Create a left side image1.
	 * 
	 * @return java.awt.Image
	 */
	protected static Image LeftSideImage1() {
		return SystemResourceSupport.createImage("/sra/jun/opengl/microcosm/LeftSideImage1.jpg");
	}

	/**
	 * Create a left side wall 1.
	 * 
	 * @return java.awt.Image
	 */
	protected static JunOpenGL3dObject LeftSideWall1() {
		Jun3dPoint[] vertexes = new Jun3dPoint[] { new Jun3dPoint(1, -1, 0), new Jun3dPoint(1, 1, 0), new Jun3dPoint(-1, 1, 0), new Jun3dPoint(-1, -1, 0)};
		JunOpenGL3dPolygon aPolygon = new JunOpenGL3dPolygon(vertexes);
		aPolygon.paint_(Color.getHSBColor(0f, 0f, 0.9f));

		JunAngle anAngle = JunAngle.FromDeg_(90);
		Jun3dTransformation aT = Jun3dTransformation.Translate_(new Jun3dPoint(-1, 0, 0));
		Jun3dLine aLine = new Jun3dLine(new Jun3dPoint(0, 0, 0), new Jun3dPoint(0, 1, 0));
		Jun3dTransformation aTransformation = Jun3dTransformation.Unity();
		aTransformation = (Jun3dTransformation) aTransformation.product_(Jun3dTransformation.Rotate_around_(anAngle, aLine));
		aTransformation = (Jun3dTransformation) aTransformation.product_(aT);

		JunOpenGL3dObject aWall = aPolygon.transform_(aTransformation);
		Image anImage = LeftSideImage1();
		JunOpenGLTexture aTexture = JunOpenGLTexture.Image_(anImage);
		aTexture.coordinates_(new Jun2dPoint[] { new Jun2dPoint(1.0, 0.0), new Jun2dPoint(1.0, 1.0), new Jun2dPoint(0.0, 1.0), new Jun2dPoint(0.0, 0.0)});
		aTexture.linear_(true);
		aTexture.clamp_(true);
		aTexture.modulate_(true);
		aTexture.mipmap_(true);
		aWall.texture_(aTexture);

		return aWall;
	}

	/**
	 * Create a right side image1.
	 * 
	 * @return java.awt.Image
	 */
	protected static Image RightSideImage1() {
		return SystemResourceSupport.createImage("/sra/jun/opengl/microcosm/RightSideImage1.jpg");
	}

	/**
	 * Create a right side wall 1.
	 * 
	 * @return java.awt.Image
	 */
	protected static JunOpenGL3dObject RightSideWall1() {
		Jun3dPoint[] vertexes = new Jun3dPoint[] { new Jun3dPoint(1, -1, 0), new Jun3dPoint(1, 1, 0), new Jun3dPoint(-1, 1, 0), new Jun3dPoint(-1, -1, 0)};
		JunOpenGL3dPolygon aPolygon = new JunOpenGL3dPolygon(vertexes);
		aPolygon.paint_(Color.getHSBColor(0f, 0f, 0.9f));

		JunAngle anAngle = JunAngle.FromDeg_(-90);
		Jun3dTransformation aT = Jun3dTransformation.Translate_(new Jun3dPoint(1, 0, 0));
		Jun3dLine aLine = new Jun3dLine(new Jun3dPoint(0, 0, 0), new Jun3dPoint(0, 1, 0));
		Jun3dTransformation aTransformation = Jun3dTransformation.Unity();
		aTransformation = (Jun3dTransformation) aTransformation.product_(Jun3dTransformation.Rotate_around_(anAngle, aLine));
		aTransformation = (Jun3dTransformation) aTransformation.product_(aT);

		JunOpenGL3dObject aWall = aPolygon.transform_(aTransformation);
		Image anImage = RightSideImage1();
		JunOpenGLTexture aTexture = JunOpenGLTexture.Image_(anImage);
		aTexture.coordinates_(new Jun2dPoint[] { new Jun2dPoint(1.0, 0.0), new Jun2dPoint(1.0, 1.0), new Jun2dPoint(0.0, 1.0), new Jun2dPoint(0.0, 0.0)});
		aTexture.linear_(true);
		aTexture.clamp_(true);
		aTexture.modulate_(true);
		aTexture.mipmap_(true);
		aWall.texture_(aTexture);

		return aWall;
	}

	/**
	 * Create a room1.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected static JunOpenGL3dObject Room1() {
		JunOpenGL3dCompoundObject room = new JunOpenGL3dCompoundObject();
		room.add_(BackSideWall1());
		room.add_(LeftSideWall1());
		room.add_(RightSideWall1());
		room.add_(DownSideWall1());
		room.add_(UpSideWall1());

		return room;
	}

	/**
	 * Create a up side image1.
	 * 
	 * @return java.awt.Image
	 */
	protected static Image UpSideImage1() {
		return SystemResourceSupport.createImage("/sra/jun/opengl/microcosm/UpSideImage1.jpg");
	}

	/**
	 * Create a up side wall 1.
	 * 
	 * @return java.awt.Image
	 */
	protected static JunOpenGL3dObject UpSideWall1() {
		Jun3dPoint[] vertexes = new Jun3dPoint[] { new Jun3dPoint(1, -1, 0), new Jun3dPoint(1, 1, 0), new Jun3dPoint(-1, 1, 0), new Jun3dPoint(-1, -1, 0)};
		JunOpenGL3dPolygon aPolygon = new JunOpenGL3dPolygon(vertexes);
		aPolygon.paint_(Color.getHSBColor(0f, 0f, 0.9f));

		JunAngle anAngle = JunAngle.FromDeg_(90);
		Jun3dTransformation aT = Jun3dTransformation.Translate_(new Jun3dPoint(0, 1, 0));
		Jun3dLine aLine = new Jun3dLine(new Jun3dPoint(0, 0, 0), new Jun3dPoint(1, 0, 0));
		Jun3dTransformation aTransformation = Jun3dTransformation.Unity();
		aTransformation = (Jun3dTransformation) aTransformation.product_(Jun3dTransformation.Rotate_around_(anAngle, aLine));
		aTransformation = (Jun3dTransformation) aTransformation.product_(aT);

		JunOpenGL3dObject aWall = aPolygon.transform_(aTransformation);
		Image anImage = UpSideImage1();
		JunOpenGLTexture aTexture = JunOpenGLTexture.Image_(anImage);
		aTexture.coordinates_(new Jun2dPoint[] { new Jun2dPoint(1.0, 0.0), new Jun2dPoint(1.0, 1.0), new Jun2dPoint(0.0, 1.0), new Jun2dPoint(0.0, 0.0)});
		aTexture.linear_(true);
		aTexture.clamp_(true);
		aTexture.modulate_(true);
		aTexture.mipmap_(true);
		aWall.texture_(aTexture);

		return aWall;
	}
}
