package sra.jun.opengl.microcosm;

import java.awt.*;
import sra.jun.goodies.button.*;
import sra.jun.goodies.wheels.*;
import sra.jun.opengl.display.*;
import sra.smalltalk.*;

/**
 * JunOpenGLMicrocosmViewAwt class
 * 
 *  @author    nisinaka
 *  @created   1999/08/17 (by nisinaka)
 *  @updated   2002/11/14 (by nisinaka)
 *  @updated   2003/05/15 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLMicrocosmViewAwt.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGLMicrocosmViewAwt extends StViewPanel implements JunOpenGLMicrocosmView {

	protected JunOpenGL3dViewAwt openGLView;
	protected JunButtonViewAwt pickButtonView;
	protected JunButtonViewAwt grabButtonView;
	protected JunButtonViewAwt dragButtonView;
	protected JunButtonViewAwt dollyButtonView;
	protected JunThumbWheelViewAwt xThumbWheelView;
	protected JunThumbWheelViewAwt yThumbWheelView;
	protected JunThumbWheelViewAwt zThumbWheelView;

	/**
	 * Create a new instance of JunOpenGLMicrocosmViewAwt and initialize it.
	 *
	 * @param newModel sra.jun.opengl.microcosm.JunOpenGLMicrocosmModel
	 */
	public JunOpenGLMicrocosmViewAwt(JunOpenGLMicrocosmModel newModel) {
		super(newModel);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public JunOpenGLMicrocosmModel getModel() {
		return (JunOpenGLMicrocosmModel) this.model();
	}

	/**
	 * Answer my model as JunOpenGLDisplayModel.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public JunOpenGLDisplayModel getOpenGLDisplayModel() {
		return (JunOpenGLDisplayModel) this.model();
	}

	/**
	 * Answer my current OpenGL view.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayView
	 */
	public JunOpenGL3dView getOpenGLView() {
		if (openGLView == null) {
			openGLView = this.defaultOpenGLView();
		}

		return openGLView;
	}

	/**
	 * Call noticeOfComponentRemove() before removeNotify().
	 */
	public void removeNotify() {
		((JunOpenGLMicrocosmController) this.controller()).flushMovementVector();
		super.removeNotify();
	}

	/**
	 * model property accessing.
	 * 
	 * @param newModel sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public void setModel(JunOpenGLMicrocosmModel newModel) {
		this.model_(newModel);
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(new BorderLayout());

		final Dimension buttonSize = new Dimension(18, 18);

		// center
		Panel displayPanel = StApplicationModel._PanelWithBorder();
		displayPanel.setLayout(new BorderLayout());
		displayPanel.add((Component) this.getOpenGLView(), "Center");
		this.add(displayPanel, "Center");

		// left
		Panel leftPanel = new Panel();
		leftPanel.setLayout(new BorderLayout());

		//
		Panel leftTopPanel = new Panel();
		leftTopPanel.setLayout(new GridLayout(3, 1));
		pickButtonView = new JunButtonViewAwt(this.getModel().pickButton());
		pickButtonView.setSize(buttonSize);
		leftTopPanel.add(pickButtonView);
		grabButtonView = new JunButtonViewAwt(this.getModel().grabButton());
		grabButtonView.setSize(buttonSize);
		leftTopPanel.add(grabButtonView);
		dragButtonView = new JunButtonViewAwt(this.getModel().dragButton());
		dragButtonView.setSize(buttonSize);
		leftTopPanel.add(dragButtonView);
		leftPanel.add(leftTopPanel, "North");

		//
		xThumbWheelView = new JunThumbWheelViewAwt(this.getModel().xThumbWheel());
		xThumbWheelView.setVertical(true);
		leftPanel.add(xThumbWheelView, "Center");

		//
		Canvas dummy0 = new Canvas();
		dummy0.setSize(buttonSize.width, buttonSize.height * 3);
		leftPanel.add(dummy0, "South");

		//
		this.add(leftPanel, "West");

		// bottom
		Panel bottomPanel = new Panel();
		bottomPanel.setLayout(new BorderLayout());
		yThumbWheelView = new JunThumbWheelViewAwt(this.getModel().yThumbWheel());
		bottomPanel.add(yThumbWheelView, "Center");
		this.add(bottomPanel, "South");

		// right
		Panel rightPanel = new Panel();
		rightPanel.setLayout(new BorderLayout());

		Panel rightCenterPanel = new Panel();
		GridBagLayout gbl = new GridBagLayout();
		rightCenterPanel.setLayout(gbl);

		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;

		//

		/*
		focusButtonView = new JunButtonView();
		c.gridy = 0;
		gbl.setConstraints(focusButtonView, c);
		rightCenterPanel.add(focusButtonView);
		*/

		//
		zThumbWheelView = new JunThumbWheelViewAwt(this.getModel().zThumbWheel());
		zThumbWheelView.setVertical(true);
		c.gridy = 1;
		gbl.setConstraints(zThumbWheelView, c);
		rightCenterPanel.add(zThumbWheelView);

		//
		dollyButtonView = new JunButtonViewAwt(this.getModel().dollyButton());
		dollyButtonView.setSize(buttonSize);
		c.gridy = 2;
		gbl.setConstraints(dollyButtonView, c);
		rightCenterPanel.add(dollyButtonView);

		//
		rightPanel.add(rightCenterPanel, "Center");

		Canvas dummy2 = new Canvas();
		dummy2.setSize(buttonSize.width, 1);
		rightPanel.add(dummy2, "North");

		Canvas dummy3 = new Canvas();
		dummy3.setSize(buttonSize.width, 1);
		rightPanel.add(dummy3, "South");
		this.add(rightPanel, "East");
	}

	/**
	 * Answer a default controller.
	 * 
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new JunOpenGLMicrocosmController();
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunOpenGLMicrocosmModel();
	}

	/**
	 * Answer my default OpenGL view.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayView
	 */
	protected JunOpenGL3dViewAwt defaultOpenGLView() {
		return new JunOpenGL3dViewAwt(this.getModel());
	}
}
