package sra.jun.opengl.microcosm;

import java.awt.*;
import javax.swing.*;
import sra.jun.goodies.button.*;
import sra.jun.goodies.wheels.*;
import sra.jun.opengl.display.*;
import sra.smalltalk.*;

/**
 * JunOpenGLMicrocosmViewSwing class
 * 
 *  @author    nisinaka
 *  @created   1999/08/17 (by nisinaka)
 *  @updated   2002/11/14 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLMicrocosmViewSwing.java,v 6.0 2003/10/28 10:52:18 nisinaka Exp $
 */
public class JunOpenGLMicrocosmViewSwing extends StViewJPanel implements JunOpenGLMicrocosmView {

	protected JunOpenGL3dView openGLView;
	protected JunButtonViewSwing pickButtonView;
	protected JunButtonViewSwing grabButtonView;
	protected JunButtonViewSwing dragButtonView;
	protected JunButtonViewSwing dollyButtonView;
	protected JunThumbWheelViewSwing xThumbWheelView;
	protected JunThumbWheelViewSwing yThumbWheelView;
	protected JunThumbWheelViewSwing zThumbWheelView;

	/**
	 * Create a new instance of JunOpenGLMicrocosmViewAwt and initialize it.
	 *
	 * @param newModel sra.jun.opengl.microcosm.JunOpenGLMicrocosmModel
	 */
	public JunOpenGLMicrocosmViewSwing(JunOpenGLMicrocosmModel newModel) {
		super(newModel);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public JunOpenGLMicrocosmModel getModel() {
		return (JunOpenGLMicrocosmModel) this.model();
	}

	/**
	 * Answer my model as JunOpenGLDisplayModel.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public JunOpenGLDisplayModel getOpenGLDisplayModel() {
		return (JunOpenGLDisplayModel) this.model();
	}

	/**
	 * Answer my current OpenGL view.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayView
	 */
	public JunOpenGL3dView getOpenGLView() {
		if (openGLView == null) {
			openGLView = this.defaultOpenGLView();
		}

		return openGLView;
	}

	/**
	 * Call noticeOfComponentRemove() before removeNotify().
	 */
	public void removeNotify() {
		((JunOpenGLMicrocosmController) this.controller()).flushMovementVector();
		super.removeNotify();
	}

	/**
	 * model property accessing.
	 * 
	 * @param newModel sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public void setModel(JunOpenGLMicrocosmModel newModel) {
		this.model_(newModel);
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(new BorderLayout());

		final Dimension buttonSize = new Dimension(18, 18);

		// center
		JPanel displayPanel = StApplicationModel._JPanelWithBorder();
		displayPanel.setLayout(new BorderLayout());
		displayPanel.add((Component) this.getOpenGLView(), "Center");
		this.add(displayPanel, "Center");

		// left
		JPanel leftPanel = new JPanel();
		leftPanel.setLayout(new BorderLayout());

		//
		JPanel leftTopPanel = new JPanel();
		leftTopPanel.setLayout(new GridLayout(3, 1));
		pickButtonView = new JunButtonViewSwing(this.getModel().pickButton());
		pickButtonView.setPreferredSize(buttonSize);
		leftTopPanel.add(pickButtonView);
		grabButtonView = new JunButtonViewSwing(this.getModel().grabButton());
		grabButtonView.setPreferredSize(buttonSize);
		leftTopPanel.add(grabButtonView);
		dragButtonView = new JunButtonViewSwing(this.getModel().dragButton());
		dragButtonView.setPreferredSize(buttonSize);
		leftTopPanel.add(dragButtonView);
		leftPanel.add(leftTopPanel, "North");

		//
		xThumbWheelView = new JunThumbWheelViewSwing(this.getModel().xThumbWheel());
		xThumbWheelView.setVertical(true);
		leftPanel.add(xThumbWheelView, "Center");

		//
		JPanel dummy0 = new JPanel();
		dummy0.setPreferredSize(new Dimension(buttonSize.width, buttonSize.height * 3));
		leftPanel.add(dummy0, "South");

		//
		this.add(leftPanel, "West");

		// bottom
		JPanel bottomPanel = new JPanel();
		bottomPanel.setLayout(new BorderLayout());
		yThumbWheelView = new JunThumbWheelViewSwing(this.getModel().yThumbWheel());
		bottomPanel.add(yThumbWheelView, "Center");
		this.add(bottomPanel, "South");

		// right
		JPanel rightPanel = new JPanel();
		rightPanel.setLayout(new BorderLayout());

		JPanel rightCenterPanel = new JPanel();
		GridBagLayout gbl = new GridBagLayout();
		rightCenterPanel.setLayout(gbl);

		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;

		//

		/*
		focusButtonView = new JunButtonView();
		c.gridy = 0;
		gbl.setConstraints(focusButtonView, c);
		rightCenterPanel.add(focusButtonView);
		*/

		//
		zThumbWheelView = new JunThumbWheelViewSwing(this.getModel().zThumbWheel());
		zThumbWheelView.setVertical(true);
		c.gridy = 1;
		gbl.setConstraints(zThumbWheelView, c);
		rightCenterPanel.add(zThumbWheelView);

		//
		dollyButtonView = new JunButtonViewSwing(this.getModel().dollyButton());
		dollyButtonView.setPreferredSize(buttonSize);
		c.gridy = 2;
		gbl.setConstraints(dollyButtonView, c);
		rightCenterPanel.add(dollyButtonView);

		//
		rightPanel.add(rightCenterPanel, "Center");

		JPanel dummy2 = new JPanel();
		dummy2.setPreferredSize(new Dimension(buttonSize.width, 1));
		rightPanel.add(dummy2, "North");

		JPanel dummy3 = new JPanel();
		dummy3.setPreferredSize(new Dimension(buttonSize.width, 1));
		rightPanel.add(dummy3, "South");
		this.add(rightPanel, "East");
	}

	/**
	 * Answer my default controller.
	 * 
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new JunOpenGLMicrocosmController();
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunOpenGLMicrocosmModel();
	}

	/**
	 * Answer my default OpenGL view.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGL3dView
	 */
	protected JunOpenGL3dView defaultOpenGLView() {
		if (StApplicationModel.GetDefaultViewMode() != StApplicationModel.VIEW_SWING_ONLY) {
			return new JunOpenGL3dViewAwt(this.getModel());
		} else {
			return new JunOpenGL3dViewSwing(this.getModel());
		}
	}
}
