package sra.jun.opengl.objects;

import java.awt.Cursor;
import java.io.*;
import java.util.Vector;
import sra.smalltalk.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.goodies.cursors.JunCursors;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.picking.JunOpenGLPickingObjects;
import sra.jun.opengl.support.JunOpenGLRenderingContext;
import sra.jun.opengl.texture.JunOpenGLTexture;

/**
 * JunOpenGL3dCompoundObject class
 * 
 *  @author    nisinaka
 *  @created   1998/10/13 (by nisinaka)
 *  @updated   1999/08/04 (by nisinaka)
 *  @updated   2000/01/06 (by nisinaka)
 *  @updated   2001/11/20 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dCompoundObject.java,v 6.0 2003/10/28 10:52:19 nisinaka Exp $
 */
public class JunOpenGL3dCompoundObject extends JunOpenGL3dObject implements JunOpenGLPickingObjects {

	/** Vector of components. */
	protected Vector components = null;

	/** Texture */
	protected JunOpenGLTexture texture = null;

	/** Array of components as a cached value. */
	private JunOpenGL3dObject[] _components = null;

	/**
	 * Create a new instance of JunOpenGL3dCompoundObject.
	 * 
	 * @category Instance creation
	 */
	public JunOpenGL3dCompoundObject() {
		this(0);
	}

	/**
	 * Create an instance of JunOpenGL3dCompoundObject and initialize its initial capacity.
	 * 
	 * @param size int
	 * @category Instance creation
	 */
	public JunOpenGL3dCompoundObject(int size) {
		super();
		this.initialize_(size);
	}

	/**
	 * Create a new instance of JunOpenGL3dCompoundObject and initialize it with a collection of JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 * @param aCollection java.util.Vector
	 * @category Instance creation
	 */
	public static JunOpenGL3dCompoundObject Components_(Vector aCollection) {
		JunOpenGL3dCompoundObject aCompoundObject = new JunOpenGL3dCompoundObject();
		aCompoundObject.components_(aCollection);
		return aCompoundObject;
	}

	/**
	 * Create a new JunOpenGL3dCompoundObject and initialize it with a JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Instance creation 
	 */
	public static final JunOpenGL3dCompoundObject With_(JunOpenGL3dObject a3dObject) {
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject(1);
		compoundObject.add_(a3dObject);
		return compoundObject;
	}

	/**
	 * Create a new JunOpenGL3dCompoundObject and initialize it with two JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 * @param a3dObject1 sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param a3dObject2 sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Instance creation 
	 */
	public static final JunOpenGL3dCompoundObject With_with_(JunOpenGL3dObject a3dObject1, JunOpenGL3dObject a3dObject2) {
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject(2);
		compoundObject.add_(a3dObject1);
		compoundObject.add_(a3dObject2);
		return compoundObject;
	}

	/**
	 * Create a new JunOpenGL3dCompoundObject and initialize it with three JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 * @param a3dObject1 sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param a3dObject2 sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param a3dObject3 sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Instance creation 
	 */
	public static final JunOpenGL3dCompoundObject With_with_with_(JunOpenGL3dObject a3dObject1, JunOpenGL3dObject a3dObject2, JunOpenGL3dObject a3dObject3) {
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject(3);
		compoundObject.add_(a3dObject1);
		compoundObject.add_(a3dObject2);
		compoundObject.add_(a3dObject3);
		return compoundObject;
	}

	/**
	 * Create a new JunOpenGL3dCompoundObject and initialize it with four JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 * @param a3dObject1 sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param a3dObject2 sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param a3dObject3 sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param a3dObject4 sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Instance creation 
	 */
	public static final JunOpenGL3dCompoundObject With_with_with_with_(JunOpenGL3dObject a3dObject1, JunOpenGL3dObject a3dObject2, JunOpenGL3dObject a3dObject3, JunOpenGL3dObject a3dObject4) {
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject(4);
		compoundObject.add_(a3dObject1);
		compoundObject.add_(a3dObject2);
		compoundObject.add_(a3dObject3);
		compoundObject.add_(a3dObject4);
		return compoundObject;
	}

	/**
	 * Create a new JunOpenGL3dCompoundObject and initialize it with an array of JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 * @param anArray sra.jun.opengl.objects.JunOpenGL3dObject[]
	 * @category Instance creation 
	 */
	public static final JunOpenGL3dCompoundObject WithAll_(JunOpenGL3dObject[] anArray) {
		JunOpenGL3dCompoundObject compoundObject = new JunOpenGL3dCompoundObject(anArray.length);
		compoundObject.addAll_(anArray);
		return compoundObject;
	}

	/**
	 * Create a new JunOpenGL3dCompoundObject and initialize it with an array of JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 * @param aClass java.lang.Class
	 * @param anArray sra.jun.opengl.objects.JunOpenGL3dObject[]
	 * @category Instance creation 
	 */
	protected static JunOpenGL3dCompoundObject WithAll_(Class aClass, JunOpenGL3dObject[] anArray) {
		JunOpenGL3dCompoundObject compoundObject = (JunOpenGL3dCompoundObject) _New(aClass, new Integer(anArray.length));
		compoundObject.addAll_(anArray);
		return compoundObject;
	}

	/**
	 * Get components from the lisp list and set it as the components of the JunOpenGL3dCompoundObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 * @category Instance creation 
	 */
	public static JunOpenGL3dCompoundObject ComponentsFromLispList_for_(JunLispCons aList, JunOpenGL3dCompoundObject a3dObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("bodies")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return a3dObject;
		}

		Object[] objects = ((JunLispList) list.tail()).asArray();
		for (int i = 0; i < objects.length; i++) {
			a3dObject.add_(JunOpenGL3dCompoundObject.ObjectFromLispList_((JunLispCons) objects[i]));
		}

		return a3dObject;
	}

	/**
	 * Create a JunOpenGL3dCompoundObject from the lisp list.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @category Lisp support 
	 */
	public static JunOpenGL3dObject FromLispList_(JunLispCons aList) {
		JunOpenGL3dObject object = new JunOpenGL3dCompoundObject();
		object = NameFromLispList_for_(aList, object);
		object = ColorFromLispList_for_(aList, object);
		object = ComponentsFromLispList_for_(aList, (JunOpenGL3dCompoundObject) object);
		object = TextureFromLispList_for_(aList, object);
		object = TransformationFromLispList_for_(aList, object);
		return object;
	}

	/**
	 * Create a JunOpenGL3dObject from the lisp list.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @category Lisp support 
	 */
	public static JunOpenGL3dObject ObjectFromLispList_(JunLispCons aList) {
		StSymbol kindName = (StSymbol) aList.head();

		if (kindName == $("CompoundBody")) {
			return JunOpenGL3dCompoundObject.FromLispList_(aList);
		}

		if (kindName == $("PatchedBody")) {
			return JunOpenGL3dCompoundObject.PatchedBodyFromLispList_(aList);
		}

		if (kindName == $("SkinningBody")) {
			return JunOpenGL3dCompoundObject.SkinningBodyFromLispList_(aList);
		}

		if (kindName == $("Polygon")) {
			return JunOpenGL3dPolygon.FromLispList_(aList);
		}

		if (kindName == $("Polyline")) {
			return JunOpenGL3dPolyline.FromLispList_(aList);
		}

		if (kindName == $("PolylineLoop")) {
			return JunOpenGL3dPolylineLoop.FromLispList_(aList);
		}

		if (kindName == $("Spot")) {
			return JunOpenGL3dVertex.FromLispList_(aList);
		}

		if (kindName == $("NurbsCurve")) {
			//			return JunOpenGL3dNurbsCurve.FromLispList_(aList);
		}

		if (kindName == $("NurbsSurface")) {
			//			return JunOpenGL3dNurbsSurface.FromLispList_(aList);
		}

		throw SmalltalkException.Error("unexpected error");
	}

	/**
	 * Create a patched body from the lisp list.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @category Lisp support 
	 */
	public static JunOpenGL3dCompoundObject PatchedBodyFromLispList_(JunLispCons aList) {
		JunOpenGL3dCompoundObject object = new JunOpenGL3dCompoundObject();
		object = (JunOpenGL3dCompoundObject) NameFromLispList_for_(aList, object);
		object = (JunOpenGL3dCompoundObject) ColorFromLispList_for_(aList, object);

		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("points")));
			}
		}, new StBlockClosure());
		Object[] points = (list == null) ? null : ((JunLispCons) list.tail()).asArray();

		list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("patches")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return object;
		}

		if (points == null) {
			Object[] objects = ((JunLispCons) list.tail()).asArray();
			for (int i = 0; i < objects.length; i++) {
				object.add_(JunOpenGL3dPolygon.FromLispList_((JunLispCons) objects[i]));
			}
		} else {
			Object[] objects = ((JunLispCons) list.tail()).asArray();
			Object[] patches = new Object[objects.length];
			for (int i = 0; i < objects.length; i++) {
				patches[i] = ((JunLispList) objects[i]).asArray();
			}
			for (int i = 0; i < patches.length; i++) {
				Object[] indexes = (Object[]) patches[i];
				int size = indexes.length;
				Jun3dPoint[] vertexes = new Jun3dPoint[size];
				for (int j = 0; j < size; j++) {
					vertexes[j] = (Jun3dPoint) points[((Number) indexes[j]).intValue() - 1];
				}
				object.add_(JunOpenGL3dPolygon.Vertexes_(vertexes));
			}
		}

		object = (JunOpenGL3dCompoundObject) TextureFromLispList_for_(aList, object);
		object = (JunOpenGL3dCompoundObject) TransformationFromLispList_for_(aList, object);

		return object;
	}

	/**
	 * Create a skinning body from the lisp list.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @category Lisp support 
	 */
	public static JunOpenGL3dCompoundObject SkinningBodyFromLispList_(JunLispCons aList) {
		JunOpenGL3dCompoundObject object = new JunOpenGL3dCompoundObject();
		object = (JunOpenGL3dCompoundObject) NameFromLispList_for_(aList, object);
		object = (JunOpenGL3dCompoundObject) ColorFromLispList_for_(aList, object);

		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("polygons")));
			}
		}, new StBlockClosure());
		if (list == null) {
			return object;
		}

		Object[] objects = ((JunLispList) list.tail()).asArray();
		for (int i = 0; i < objects.length; i++) {
			object.add_(JunOpenGL3dPolygon.FromLispList_((JunLispCons) objects[i]));
		}
		object = (JunOpenGL3dCompoundObject) TextureFromLispList_for_(aList, object);
		object = (JunOpenGL3dCompoundObject) TransformationFromLispList_for_(aList, object);

		return object;
	}

	/**
	 * Get a texture from the lisp list and set it as the texture of the JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Lisp support 
	 */
	public static JunOpenGL3dObject TextureFromLispList_for_(JunLispCons aList, JunOpenGL3dObject a3dObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("texture")));
			}
		}, new StBlockClosure());

		if ((list != null) && list.tail() instanceof JunLispCons) {
			a3dObject.texture_(JunOpenGLTexture.FromLispList_((JunLispCons) list.tail()));
		}

		return a3dObject;
	}

	/**
	 * Answer the array of components.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject[]
	 * @category accessing
	 */
	public JunOpenGL3dObject[] _components() {
		if ((_components == null) && (components != null)) {
			int size = components.size();
			_components = new JunOpenGL3dObject[size];
			for (int i = 0; i < size; i++) {
				_components[i] = (JunOpenGL3dObject) components.elementAt(i);
			}
		}

		return _components;
	}

	/**
	 * Flush the cached value of components.
	 * 
	 * @category flushing
	 */
	public void _flushComponents() {
		_components = null;
	}

	/**
	 * Add a JunOpenGL3dObject to the JunOpenGL3dCompoundObject.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category adding
	 */
	public void add_(JunOpenGL3dObject a3dObject) {
		components.addElement(a3dObject);
		this.flushBounds();
		this._flushComponents();
	}

	/**
	 * Add all of the array elements.
	 * 
	 * @param anArray sra.jun.opengl.objects.JunOpenGL3dObject[]
	 * @category adding
	 */
	public void addAll_(JunOpenGL3dObject[] anArray) {
		for (int i = 0; i < anArray.length; i++) {
			this.add_(anArray[i]);
		}
	}

	/**
	 * Add all of the vector elements.
	 * 
	 * @param aVector java.util.Vector
	 * @category adding
	 */
	public void addAll_(Vector aVector) {
		int size = aVector.size();
		for (int i = 0; i < size; i++) {
			this.add_((JunOpenGL3dObject) aVector.elementAt(i));
		}
	}

	/**
	 * Convert the receiver to a JunOpenGL3dCompoundObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dCompoundObject
	 * @category converting
	 */
	public JunOpenGL3dCompoundObject asCompoundObject() {
		return this;
	}

	/**
	 * Answer the array of components.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject[]
	 * @category accessing
	 */
	public JunOpenGL3dObject[] components() {
		return this._components();
	}

	/**
	 * Set an array as components.
	 * 
	 * @param anArray sra.jun.opengl.objects.JunOpenGL3dObject[]
	 * @category accessing
	 */
	public void components_(JunOpenGL3dObject[] anArray) {
		_components = anArray;
		components = new Vector(anArray.length);
		this.addAll_(anArray);
	}

	/**
	 * Set a vector as components.
	 * 
	 * @param aVector java.util.Vector
	 * @category accessing
	 */
	public void components_(Vector aVector) {
		components = aVector;
		this._flushComponents();
	}

	/**
	 * Convert the receiver's texture as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList componentsToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("bodies"));
		JunOpenGL3dObject[] components = this.components();
		for (int i = 0; i < components.length; i++) {
			list.add_(components[i].toLispList());
		}
		return list;
	}

	/**
	 * Enumerate each geometries and evaluate the block.
	 * 
	 * @return java.lang.Object
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @category enumerating 
	 */
	public Object geometriesDo_(StBlockClosure aBlock) {
		JunOpenGL3dObject[] anArrayOfComponents = this._components();
		for (int i = 0; i < anArrayOfComponents.length; i++) {
			Object result = anArrayOfComponents[i].geometriesDo_(aBlock);
			if (result != null) {
				return result;
			}
		}
		return null;
	}

	/**
	 * Answer true if the texture is specified, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean hasTexture() {
		return (texture != null);
	}

	/**
	 * Initialize the receiver. Create a component with its initial capacity 'size'.
	 * 
	 * @param size int
	 * @category initialize-release
	 */
	public void initialize_(int size) {
		components = new Vector(size);
	}

	/**
	 * Answer true if the receiver is a compound object, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isCompound() {
		return true;
	}

	/**
	 * Answer true if the receiver is a compound body, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isCompoundBody() {
		if (components == null) {
			return true;
		}
		if (components.isEmpty()) {
			return true;
		}

		JunOpenGL3dObject[] anArray = this.components();
		int size = anArray.length;
		for (int i = 0; i < size; i++) {
			JunOpenGL3dObject each = anArray[i];
			if (each.hasColor()) {
				return true;
			}
			if (each.hasStipple()) {
				return true;
			}
			if (!(each instanceof JunOpenGL3dPolygon)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Answer true if the receiver is a patched body, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isPatchedBody() {
		if (this.isCompoundBody()) {
			return false;
		}

		int total = 0;
		Vector points = new Vector(1024);
		int min = Integer.MAX_VALUE;
		int max = Integer.MIN_VALUE;
		JunOpenGL3dObject[] array = this.components();
		for (int i = 0; i < array.length; i++) {
			JunOpenGL3dPolygon polygon = (JunOpenGL3dPolygon) array[i];
			if (polygon.hasColors() || polygon.hasNormalVectors()) {
				return false;
			}

			Jun3dPoint[] vertexes = polygon.vertexes();
			int size = vertexes.length;
			total += size;
			for (int j = 0; j < size; j++) {
				Jun3dPoint p = vertexes[j];
				int index = -1;
				for (int k = 0; k < points.size(); k++) {
					if (p.equals((Jun3dPoint) points.elementAt(k))) {
						index = k;
						break;
					}
				}
				if (index < 0) {
					points.addElement(p);
				}
			}
			min = Math.min(min, size);
			max = Math.max(max, size);
		}

		if ((min == max) && ((points.size() * 2) > total)) {
			return false;
		}
		return true;
	}

	/**
	 * Answer true if the receiver is a skinning body, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isSkinningBody() {
		if (this.isCompoundBody()) {
			return false;
		}

		return (!this.isPatchedBody());
	}

	/**
	 * Answer the StSymbol which represents the kind of the receiver.
	 * 
	 * @return sra.smalltalk.StSymbol
	 * @category lisp support
	 */
	public StSymbol kindName() {
		return $("CompoundBody");
	}

	/**
	 * Answer the number of polygons.
	 * 
	 * @return int
	 * @category utilities
	 */
	public int numberOfPolygons() {
		int number = 0;
		JunOpenGL3dObject[] myComponents = this.components();
		for (int i = 0; i < myComponents.length; i++) {
			number += myComponents[i].numberOfPolygons();
		}
		return number;
	}

	/**
	 * Answer the number of primitives.
	 * 
	 * @return int
	 * @category utilities
	 */
	public int numberOfPrimitives() {
		int number = 0;
		JunOpenGL3dObject[] myComponents = this.components();
		for (int i = 0; i < myComponents.length; i++) {
			number += myComponents[i].numberOfPrimitives();
		}
		return number;
	}

	/**
	 * Enumerate each objects and evaluate the block.
	 * 
	 * @return java.lang.Object
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @category enumerating 
	 */
	public Object objectsDo_(StBlockClosure aBlock) {
		JunOpenGL3dObject[] anArrayOfComponents = this._components();
		for (int i = 0; i < anArrayOfComponents.length; i++) {
			Object result = anArrayOfComponents[i].objectsDo_(aBlock);
			if (result != null) {
				return result;
			}
		}

		return null;
	}

	/**
	 * Convert the receiver (which is a patched body) as a JunLispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList patchedBodyFromLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("PatchedBody"));

		if (this.hasName()) {
			list.add_(this.nameToLispList());
		}
		if (this.hasColor()) {
			list.add_(this.colorToLispList());
		}
		if (this.hasTexture()) {
			list.add_(this.textureToLispList());
		}

		Vector points;
		Object[] patches;

		JunCursors _cursor = new JunCursors(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		try {
			_cursor._show();

			JunOpenGL3dObject[] polygons = this.components();
			points = new Vector(1024);
			for (int i = 0; i < polygons.length; i++) {
				this.pointCollection_for_(points, polygons[i]);
			}

			Vector patcheCollection = new Vector(1024);
			for (int i = 0; i < polygons.length; i++) {
				this.patcheCollection_from_for_(patcheCollection, points, polygons[i]);
			}

			patches = patcheCollection.toArray();
			for (int i = 0; i < patches.length; i++) {
				patches[i] = JunLispCons.List_((Vector) patches[i]);
			}
		} finally {
			_cursor._restore();
		}

		JunLispList pointList = JunLispCons.List_(points);
		JunLispList patcheList = JunLispCons.List_(patches);
		pointList = JunLispCons.Head_tail_($("points"), pointList);
		patcheList = JunLispCons.Head_tail_($("patches"), patcheList);
		list.add_(pointList);
		list.add_(patcheList);

		return list;
	}

	/**
	 * Convert the receiver as an appropriate LispList. The receiver could be
	 * either a patched body or a skinning body.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList patchedOrSkinningBodyFromLispList() {
		if (this.isSkinningBody()) {
			return this.skinningBodyFromLispList();
		} else {
			return this.patchedBodyFromLispList();
		}
	}

	/**
	 * Enumerate every points and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @category enumerating
	 */
	public void pointsDo_(StBlockClosure aBlock) {
		JunOpenGL3dObject[] anArrayOfComponents = this._components();
		for (int i = 0; i < anArrayOfComponents.length; i++) {
			anArrayOfComponents[i].pointsDo_(aBlock);
		}
	}

	/**
	 * Do an extra copy of the receiver.
	 * 
	 * @return sra.smalltalk.StObject
	 * @category copying
	 */
	public StObject postCopy() {
		super.postCopy();

		if (components != null) {
			components = (Vector) components.clone();
		}

		return this;
	}

	/**
	 * Enumerate all of the primitive objects and evaluate the Block.
	 * 
	 * @return java.lang.Object
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @category enumerating 
	 */
	public Object primitivesDo_(StBlockClosure aBlock) {
		JunOpenGL3dObject[] components = this._components();
		for (int i = 0; i < components.length; i++) {
			if (components[i].isCompound()) {
				components[i].primitivesDo_(aBlock);
			} else {
				aBlock.value_(components[i]);
			}
		}

		return null;
	}

	/**
	 * Remove a JunOpenGL3dObject from the JunOpenGL3dCompoundObject.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category removing
	 */
	public void remove_(JunOpenGL3dObject a3dObject) {
		components.removeElement(a3dObject);
		this.flushBounds();
		this._flushComponents();
	}

	/**
	 * Remove all of the array elements from the JunOpenGL3dCompoundObject.
	 * 
	 * @param anArray sra.jun.opengl.objects.JunOpenGL3dObject[]
	 * @category removing
	 */
	public void removeAll_(JunOpenGL3dObject[] anArray) {
		for (int i = 0; i < anArray.length; i++) {
			this.remove_(anArray[i]);
		}
	}

	/**
	 * Remove all of the vector elements from the JunOpenGL3dCompoundObject.
	 * 
	 * @param aVector java.util.Vector
	 * @category removing
	 */
	public void removeAll_(Vector aVector) {
		int size = aVector.size();
		for (int i = 0; i < size; i++) {
			this.remove_((JunOpenGL3dObject) aVector.elementAt(i));
		}
	}

	/**
	 * Render the OpenGL object on a rendering context.
	 * 
	 * @param aRenderingContext sra.jun.opengl.support.JunOpenGLRenderingContext
	 * @category rendering
	 */
	public void renderOn_(JunOpenGLRenderingContext aRenderingContext) {
		int oldTextureCoordinateIndex = Integer.MIN_VALUE;

		if (this.hasTexture()) {
			oldTextureCoordinateIndex = aRenderingContext.textureCoordinateIndex();
			aRenderingContext.textureCoordinateIndex_(0);
			this.texture().enableTextureOn_(aRenderingContext);
		}

		if (this.hasTransparency()) {
			aRenderingContext.enableCullFace();
			aRenderingContext.disableDepthMask();
			aRenderingContext.enableBlending();
		}

		this.renderOpaqueComponentsOn_(aRenderingContext);
		this.renderTransparentComponentsOn_(aRenderingContext);

		if (this.hasTransparency()) {
			aRenderingContext.disableCullFace();
			aRenderingContext.enableDepthMask();
			aRenderingContext.disableBlending();
		}

		if (this.hasTexture()) {
			this.texture().disableTextureOn_(aRenderingContext);
			aRenderingContext.textureCoordinateIndex_(oldTextureCoordinateIndex);
		}
	}

	/**
	 * Render the opaque components on the rendering context.
	 * 
	 * @param aRenderingContext sra.jun.opengl.support.JunOpenGLRenderingContext
	 * @category rendering
	 */
	protected void renderOpaqueComponentsOn_(JunOpenGLRenderingContext aRenderingContext) {
		JunOpenGL3dObject[] myComponents = this.components();
		for (int i = 0; i < myComponents.length; i++) {
			if (!myComponents[i].hasTransparency()) {
				if (this.hasPaint()) {
					aRenderingContext.paint_(this.paint());
				}
				if (this.hasAlpha()) {
					aRenderingContext.alpha_(this.alpha());
				}
				if (this.hasTexture()) {
					this.texture().enableTextureOn_(aRenderingContext);
				}
				myComponents[i].renderOn_(aRenderingContext);
			}
		}
	}

	/**
	 * Render the transparent components on the rendering context.
	 * 
	 * @param aRenderingContext sra.jun.opengl.support.JunOpenGLRenderingContext
	 * @category rendering
	 */
	protected void renderTransparentComponentsOn_(JunOpenGLRenderingContext aRenderingContext) {
		JunOpenGL3dObject[] myComponents = this.components();
		for (int i = 0; i < myComponents.length; i++) {
			if (myComponents[i].hasTransparency()) {
				if (this.hasPaint()) {
					aRenderingContext.paint_(this.paint());
				}
				if (this.hasAlpha()) {
					aRenderingContext.alpha_(this.alpha());
				}
				if (this.hasTexture()) {
					this.texture().enableTextureOn_(aRenderingContext);
				}
				myComponents[i].renderOn_(aRenderingContext);
			}
		}
	}

	/**
	 * Anser the reversed JunOpenGL3dObject of the receiver.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category converting
	 */
	public JunOpenGL3dObject reversed() {
		JunOpenGL3dObject[] array = this.components();
		JunOpenGL3dCompoundObject newObject = (JunOpenGL3dCompoundObject) _New(this.getClass(), new Integer(array.length));
		for (int i = 0; i < array.length; i++) {
			newObject.add_(array[i].reversed());
		}
		return newObject;
	}

	/**
	 * Convert the receiver (which is a skinning body) as a JunLispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList skinningBodyFromLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("SkinningBody"));

		if (this.hasName()) {
			list.add_(this.nameToLispList());
		}
		if (this.hasColor()) {
			list.add_(this.colorToLispList());
		}
		if (this.hasTexture()) {
			list.add_(this.textureToLispList());
		}

		JunLispList[] patches = new JunLispList[components.size()];
		for (int i = 0; i < patches.length; i++) {
			patches[i] = ((JunOpenGL3dObject) components.elementAt(i)).toLispList();
		}
		JunLispList patcheList = JunLispCons.List_(patches);
		patcheList = JunLispCons.Head_tail_($("polygons"), patcheList);
		list.add_(patcheList);

		return list;
	}

	/**
	 * Answer the texture.
	 * 
	 * @return sra.jun.opengl.texture.JunOpenGLTexture
	 * @category accessing
	 */
	public JunOpenGLTexture texture() {
		return texture;
	}

	/**
	 * Set a texture.
	 * 
	 * @param aTexture sra.jun.opengl.texture.JunOpenGLTexture
	 * @category accessing
	 */
	public void texture_(JunOpenGLTexture aTexture) {
		texture = aTexture;

		if (aTexture != null) {
			aTexture.object_(this);
		}
	}

	/**
	 * Convert the receiver's texture as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList textureToLispList() {
		if (!this.hasTexture()) {
			return this.lispNil();
		}

		JunLispCons list = this.lispCons();
		list.head_($("texture"));
		list.tail_(this.texture().toLispList());

		return list;
	}

	/**
	 * Convert the receiver as a JunLispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList toLispList() {
		if (!this.isCompoundBody()) {
			return this.patchedOrSkinningBodyFromLispList();
		}

		JunLispCons list = this.lispCons();
		list.head_(this.kindName());

		if (this.hasName()) {
			list.add_(this.nameToLispList());
		}
		if (this.hasColor()) {
			list.add_(this.colorToLispList());
		}
		if (this.hasTexture()) {
			list.add_(this.textureToLispList());
		}
		list.add_(this.componentsToLispList());

		return list;
	}

	/**
	 * Answer the new JunOpenGL3dCompoundObject transformed with aTransformation.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param aTransformation sra.jun.geometry.transformations.Jun3dTransformation
	 * @category transforming 
	 */
	public JunOpenGL3dObject transform_(Jun3dTransformation aTransformation) {
		JunOpenGL3dCompoundObject newObject = (JunOpenGL3dCompoundObject) this.copy();

		if (components != null) {
			JunOpenGL3dObject[] array = this._components();
			JunOpenGL3dObject[] newArray = new JunOpenGL3dObject[array.length];
			for (int i = 0; i < array.length; i++) {
				newArray[i] = array[i].transform_(aTransformation);
			}
			newObject.components_(newArray);
		}

		return newObject;
	}

	/**
	 * Write my VRML1.0 string on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws SmalltalkException 
	 * @category vrml support
	 */
	public void vrml10On_(Writer aWriter) {
		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
			bw.write("DEF ");
			bw.write(this.legalName());
			bw.newLine();
			bw.write("Separator {");
			bw.newLine();
			this.vrml10ColorOn_(bw);
			this.vrml10ComponentsOn_(bw);
			bw.write("} #Separator");
			bw.newLine();
			bw.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Write the VRML2.0 string of the receiver on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws SmalltalkException 
	 * @category vrml support
	 */
	public void vrml20On_(Writer aWriter) {
		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
			bw.write("Transform {");
			bw.newLine();
			bw.write("\t");
			bw.write("children [");
			bw.newLine();
			this.vrml20ComponentsOn_(bw);
			bw.write("\t");
			bw.write("] #children");
			bw.newLine();
			bw.write("\t");
			bw.write("bboxCenter ");

			Jun3dBoundingBox box = this.boundingBox();
			Jun3dPoint center = (Jun3dPoint) box.center();
			bw.write(String.valueOf((float) center.x()));
			bw.write(" ");
			bw.write(String.valueOf((float) center.y()));
			bw.write(" ");
			bw.write(String.valueOf((float) center.z()));
			bw.newLine();
			bw.write("\t");
			bw.write("bboxSize ");
			bw.write(String.valueOf((float) box.width()));
			bw.write(" ");
			bw.write(String.valueOf((float) box.height()));
			bw.write(" ");
			bw.write(String.valueOf((float) box.depth()));
			bw.newLine();
			bw.write("} #Transform");
			bw.newLine();
			bw.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Write my components as VRML1.0 on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @category vrml support
	 */
	private void vrml10ComponentsOn_(Writer aWriter) {
		if (!this.isCompoundBody()) {
			this.vrml10IndexedFaceSetOn_(aWriter);
			return;
		}

		JunOpenGL3dObject[] components = this.components();
		for (int index = 0; index < components.length; index++) {
			components[index].vrml10On_(aWriter);
		}
	}

	/**
	 * Write my IndexedFaceSet as VRML1.0 on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws SmalltalkException 
	 * @category vrml support
	 */
	private void vrml10IndexedFaceSetOn_(Writer aWriter) {
		JunOpenGL3dObject[] polygons = this.components();
		Vector points = new Vector();
		for (int index = 0; index < polygons.length; index++) {
			this.pointCollection_for_(points, polygons[index]);
		}
		Vector patches = new Vector();
		for (int index = 0; index < polygons.length; index++) {
			this.patcheCollection_from_for_(patches, points, polygons[index]);
		}

		if (points.isEmpty()) {
			return;
		}

		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
			bw.write("\t");
			bw.write("Coordinate3 {");
			bw.newLine();
			bw.write("\t\t");
			bw.write("point [");
			bw.newLine();

			for (int index = 0; index < points.size(); index++) {
				Jun3dPoint point = (Jun3dPoint) points.elementAt(index);
				bw.write("\t\t\t");
				bw.write(String.valueOf((float) point.x()));
				bw.write(" ");
				bw.write(String.valueOf((float) point.y()));
				bw.write(" ");
				bw.write(String.valueOf((float) point.z()));
				if (index != (points.size() - 1)) {
					bw.write(",");
				}
				bw.write(" #");
				bw.write(String.valueOf(index));
				bw.newLine();
			}

			bw.write("\t\t");
			bw.write("] #point");
			bw.newLine();
			bw.write("\t");
			bw.write("} #Coordinate3");
			bw.newLine();

			if (this.hasTexture()) {
				bw.write("\t");
				bw.write("Texture2 {");
				bw.newLine();
				bw.write("\t\t");
				bw.write("image ");
				this.texture.vrmlSFImageOn_(bw);
				bw.newLine();
				bw.write("\t");
				bw.write("} #Texture2");
				bw.newLine();
			}

			if (this.hasTexture() && (this.texture().hasCoordinates())) {
				bw.write("\t");
				bw.write("TextureCoordinate2 {");
				bw.newLine();
				bw.write("\t\t");
				bw.write("point [");
				bw.newLine();

				Jun2dPoint[] coordinates = this.texture().coordinates();
				for (int index = 0; index < coordinates.length; index++) {
					Jun2dPoint point = coordinates[index];
					bw.write("\t\t\t");
					bw.write(String.valueOf((float) point.x()));
					bw.write(" ");
					bw.write(String.valueOf((float) point.y()));
					if (index != (coordinates.length - 1)) {
						bw.write(",");
					}
					bw.write(" #");
					bw.write(String.valueOf(index));
					bw.newLine();
				}

				bw.write("\t\t");
				bw.write("]");
				bw.newLine();
				bw.write("\t");
				bw.write("} #TextureCoordinate2");
				bw.newLine();
			}

			bw.write("\t");
			bw.write(this.vrml10IndexedSetName());
			bw.write(" {");
			bw.newLine();
			bw.write("\t\t");
			bw.write("coordIndex [");
			bw.newLine();
			for (int nth = 0; nth < patches.size(); nth++) {
				Vector indexes = (Vector) patches.elementAt(nth);
				bw.write("\t\t\t");
				for (int i = 0; i < indexes.size(); i++) {
					int index = ((Integer) indexes.elementAt(i)).intValue();
					bw.write(String.valueOf(index - 1));
					bw.write(", ");
				}
				bw.write("-1");
				if (nth != (patches.size() - 1)) {
					bw.write(",");
				}
				bw.newLine();
			}

			bw.write("\t\t");
			bw.write("] #coordIndex");
			bw.newLine();

			if (this.hasTexture() && (this.texture().hasCoordinates())) {
				bw.write("\t\t");
				bw.write("textureCoordIndex [");
				bw.newLine();

				int i = 0;
				for (int nth = 1; nth < (patches.size() + 1); nth++) {
					Vector indexes = (Vector) patches.elementAt(nth - 1);
					bw.write("\t\t\t");
					for (int index = i; index < (i + indexes.size() - 1); index++) {
						bw.write(String.valueOf(index));
						bw.write(", ");
					}
					bw.write("-1");
					i = i + indexes.size();
					if (nth != patches.size()) {
						bw.write(",");
					}
					bw.newLine();
				}

				bw.write("\t\t");
				bw.write("] #textureCoordIndex");
				bw.newLine();
			}

			bw.write("\t");
			bw.write("} #");
			bw.write(this.vrml10IndexedSetName());
			bw.newLine();
			bw.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Answer the name of my indexed set node.
	 * 
	 * @return String
	 * @category vrml support
	 */
	protected String vrml10IndexedSetName() {
		return "IndexedFaceSet";
	}

	/**
	 * Write my components as VRML2.0 on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws SmalltalkException 
	 * @category vrml support
	 */
	private void vrml20ComponentsOn_(Writer aWriter) {
		if (!this.isCompoundBody()) {
			this.vrml20IndexedFaceSetOn_(aWriter);
			return;
		}

		BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
		JunOpenGL3dObject[] components = this.components();
		try {
			for (int nth = 0; nth < components.length; nth++) {
				JunOpenGL3dObject each = components[nth];
				each.vrml20On_(bw);
				if (nth != (components.length - 1)) {
					bw.write("\t\t");
					bw.write(",");
					bw.newLine();
				}
			}
			bw.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Write my IndexedFaceSet node as VRML2.0 on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @category vrml support
	 */
	private void vrml20IndexedFaceSetOn_(Writer aWriter) {
		JunOpenGL3dObject[] polygons = this.components();
		Vector points = new Vector();

		for (int index = 0; index < polygons.length; index++) {
			this.pointCollection_for_(points, polygons[index]);
		}

		Vector patches = new Vector();
		for (int index = 0; index < polygons.length; index++) {
			this.patcheCollection_from_for_(patches, points, polygons[index]);
		}

		if (points.isEmpty()) {
			return;
		}

		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
			if (this.name() != this.defaultName()) {
				bw.write("#");
				bw.write(this.name());
				bw.newLine();
			}

			bw.write("Shape {");
			bw.newLine();
			this.vrml20AppearanceOn_(bw);
			bw.write("\t");
			bw.write("geometry ");
			bw.write(this.vrml20IndexedSetName());
			bw.write(" {");
			bw.newLine();
			bw.write("\t\t");
			bw.write("coord Coordinate {");
			bw.newLine();
			bw.write("\t\t\t");
			bw.write("point [");
			bw.newLine();

			for (int index = 0; index < points.size(); index++) {
				Jun3dPoint point = (Jun3dPoint) points.elementAt(index);
				bw.write("\t\t\t\t");
				bw.write(String.valueOf((float) point.x()));
				bw.write(" ");
				bw.write(String.valueOf((float) point.y()));
				bw.write(" ");
				bw.write(String.valueOf((float) point.z()));
				if (index != (points.size() - 1)) {
					bw.write(",");
				}
				bw.write(" #");
				bw.write(String.valueOf(index));
				bw.newLine();
			}

			bw.write("\t\t\t");
			bw.write("] #point");
			bw.newLine();
			bw.write("\t\t");
			bw.write("} #Coordinate");
			bw.newLine();
			bw.write("\t\t");
			bw.write("coordIndex [");
			bw.write(" #patches: ");
			bw.write(String.valueOf(patches.size()));
			bw.newLine();

			for (int nth = 0; nth < patches.size(); nth++) {
				Vector indexes = (Vector) patches.elementAt(nth);
				bw.write("\t\t\t");
				for (int i = 0; i < indexes.size(); i++) {
					int index = ((Integer) indexes.elementAt(i)).intValue();
					bw.write(String.valueOf(index - 1));
					bw.write(", ");
				}
				bw.write("-1");
				if (nth != (patches.size() - 1)) {
					bw.write(",");
				}
				bw.newLine();
			}

			bw.write("\t\t");
			bw.write("] #coordIndex");
			bw.newLine();

			if (this.hasTexture() && (this.texture().hasCoordinates())) {
				bw.write("\t\t");
				bw.write("texCoord TextureCoordinate {");
				bw.newLine();
				bw.write("\t\t\t");
				bw.write("point [");
				bw.newLine();

				Jun2dPoint[] coordinates = this.texture().coordinates();
				for (int index = 0; index < coordinates.length; index++) {
					Jun2dPoint point = coordinates[index];
					bw.write("\t\t\t\t");
					bw.write(String.valueOf((float) point.x()));
					bw.write(" ");
					bw.write(String.valueOf((float) point.y()));
					if (index != (coordinates.length - 1)) {
						bw.write(",");
					}
					bw.write(" #");
					bw.write(String.valueOf(index));
					bw.newLine();
				}

				bw.write("\t\t\t");
				bw.write("]");
				bw.newLine();
				bw.write("\t\t");
				bw.write("} #texCoord");
				bw.newLine();
				bw.write("\t\t");
				bw.write("texCoordIndex [");
				bw.newLine();

				int i = 0;
				for (int nth = 1; nth < (patches.size() + 1); nth++) {
					Vector indexes = (Vector) patches.elementAt(nth - 1);
					bw.write("\t\t\t");

					for (int index = i; index < (i + indexes.size() - 1); index++) {
						bw.write(String.valueOf(index));
						bw.write(", ");
					}
					bw.write("-1");
					i = i + indexes.size();
					if (nth != patches.size()) {
						bw.write(",");
					}
					bw.newLine();
				}

				bw.write("\t\t");
				bw.write("] #texCoordIndex");
				bw.newLine();
			}

			bw.write("\t\t");
			bw.write("solid FALSE");
			bw.newLine();
			bw.write("\t");
			bw.write("} #");
			bw.write(this.vrml20IndexedSetName());
			bw.newLine();
			bw.write("} #Shape");
			bw.newLine();
			bw.flush();
		} catch (Exception e) {
		}
	}

	/**
	 * Answer the name of my indexed set node.
	 * 
	 * @return String
	 * @category vrml support
	 */
	protected String vrml20IndexedSetName() {
		return "IndexedFaceSet";
	}

	/**
	 * Collect all the patches.
	 * 
	 * @param patches java.util.Vector
	 * @param points java.util.Vector
	 * @param aPolygon sra.smalltalk.opengl.objects.JunOpenGL3dObject
	 * @category private
	 */
	protected void patcheCollection_from_for_(Vector patches, Vector points, JunOpenGL3dObject aPolygon) {
		if (aPolygon instanceof JunOpenGL3dPolygon) {
			Jun3dPoint[] vertexes = ((JunOpenGL3dPolygon) aPolygon).vertexes();
			Vector collection = new Vector(vertexes.length);
			for (int i = 0; i < vertexes.length; i++) {
				int index = points.indexOf(vertexes[i]);
				if (index >= 0) {
					collection.addElement(new Integer(index + 1));
				}
			}
			patches.addElement(collection);
		}
	}

	/**
	 * Collect all the points of the Polygon. The Vector "points" could already
	 * contains some points and avoid adding the same point to the vector.
	 * 
	 * @param points java.util.Vector
	 * @param aPolygon sra.smalltalk.opengl.objects.JunOpenGL3dObject
	 * @category private
	 */
	protected void pointCollection_for_(Vector points, JunOpenGL3dObject aPolygon) {
		if (aPolygon instanceof JunOpenGL3dPolygon) {
			Jun3dPoint[] vertexes = ((JunOpenGL3dPolygon) aPolygon).vertexes();
			for (int i = 0; i < vertexes.length; i++) {
				if (points.indexOf(vertexes[i]) < 0) {
					points.addElement(vertexes[i]);
				}
			}
		}
	}

}
