package sra.jun.opengl.objects;

import java.awt.Color;
import sra.jun.geometry.basic.Jun3dPoint;

/**
 * JunOpenGL3dNurbsCurveTestExamples class
 * 
 *  @author    nisinaka
 *  @created   1999/01/20 (by nisinaka)
 *  @updated   2001/11/20 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dNurbsCurveTestExamples.java,v 6.0 2003/10/28 10:52:19 nisinaka Exp $
 */
public class JunOpenGL3dNurbsCurveTestExamples extends sra.jun.system.support.JunTestExamples {

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example4() {
		JunOpenGL3dNurbsCurve nurbs = ExampleCircle();
		nurbs.name_("example");
		nurbs.paint_(Color.red);
		nurbs.show();

		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example5() {
		JunOpenGL3dNurbsCurve nurbs = ExampleBezierCurve();
		nurbs.name_("example");
		nurbs.paint_(Color.blue);
		nurbs.show();

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 */
	public static void main(String[] args) {
		new JunOpenGL3dNurbsCurveTestExamples();
	}

	/**
	 * Create an instance of JunOpenGL3dNurbsCurve which represents a Bezier
	 * curve.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dNurbsCurve
	 */
	protected static JunOpenGL3dNurbsCurve ExampleBezierCurve() {
		Jun3dPoint[] controlPoints = new Jun3dPoint[5];
		controlPoints[0] = new Jun3dPoint(1, -1, 0);
		controlPoints[1] = new Jun3dPoint(-1, -1, 0);
		controlPoints[2] = new Jun3dPoint(-1, 1, 0);
		controlPoints[3] = new Jun3dPoint(1, 1, 0);
		controlPoints[4] = new Jun3dPoint(1, -1, 0);

		return JunOpenGL3dNurbsCurve.BezierControlPoints_(controlPoints);
	}

	/**
	 * Create an instance of JunOpenGL3dNurbsCurve which represents a circle.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dNurbsCurve
	 */
	protected static JunOpenGL3dNurbsCurve ExampleCircle() {
		Jun3dPoint[] controlPoints = new Jun3dPoint[9];
		controlPoints[0] = new Jun3dPoint(0, -1, 0);
		controlPoints[1] = new Jun3dPoint(1, -1, 0);
		controlPoints[2] = new Jun3dPoint(1, 0, 0);
		controlPoints[3] = new Jun3dPoint(1, 1, 0);
		controlPoints[4] = new Jun3dPoint(0, 1, 0);
		controlPoints[5] = new Jun3dPoint(-1, 1, 0);
		controlPoints[6] = new Jun3dPoint(-1, 0, 0);
		controlPoints[7] = new Jun3dPoint(-1, -1, 0);
		controlPoints[8] = new Jun3dPoint(0, -1, 0);

		double[] weights = new double[9];
		weights[0] = 1.0;
		weights[1] = Math.sqrt(0.5);
		weights[2] = 1.0;
		weights[3] = Math.sqrt(0.5);
		weights[4] = 1.0;
		weights[5] = Math.sqrt(0.5);
		weights[6] = 1.0;
		weights[7] = Math.sqrt(0.5);
		weights[8] = 1.0;

		double[] knots = { 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 4 };

		return JunOpenGL3dNurbsCurve.ControlPoints_weights_knotVector_(controlPoints, weights, knots);
	}
}
