package sra.jun.opengl.objects;

import java.io.Writer;
import java.util.Vector;
import sra.jun.collections.sequences.*;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.geometry.surfaces.JunNurbsSurface;
import sra.jun.geometry.transformations.*;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.support.JunOpenGLRenderingContext;
import sra.jun.opengl.texture.*;
import sra.smalltalk.*;

/**
 * JunOpenGL3dNurbsSurface class
 * 
 *  @author    nisinaka
 *  @created   1998/10/29 (by nisinaka)
 *  @updated   1999/08/04 (by nisinaka)
 *  @updated   2001/11/20 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dNurbsSurface.java,v 6.0 2003/10/28 10:52:19 nisinaka Exp $
 */
public class JunOpenGL3dNurbsSurface extends JunOpenGL3dPrimitiveObject {
	/** The stipple of the receiver. */
	protected JunOpenGLStipple stipple = null;

	/** The texture of the receiver. */
	protected JunOpenGLTexture texture = null;

	/** The matrix of the control points. */
	protected JunMatrix controlPoints = null;

	/** The matrix of the weight number. */
	protected JunDoubleMatrix weights = null;

	/** The knot vector of u-parameter. */
	protected double[] uKnotVector = null;

	/** The knot vector of v-parameter. */
	protected double[] vKnotVector = null;

	/**
	 * Create a new instance of JunOpenGL3dNurbsSurface and initialize it with
	 * the bezier control points.
	 * 
	 * @param aMatrix sra.jun.collections.sequences.JunMatrix
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dNurbsSurface
	 */
	public static final JunOpenGL3dNurbsSurface BezierControlPoints_(JunMatrix aMatrix) {
		int columnSize;
		int rowSize;
		double[] uKnotVector;
		double[] vKnotVector;

		columnSize = aMatrix.columnSize();
		rowSize = aMatrix.rowSize();
		uKnotVector = new double[rowSize * 2];

		for (int index = 0; index < rowSize; index++) {
			uKnotVector[index] = 0.0;
			uKnotVector[index + rowSize] = 1.0;
		}

		vKnotVector = new double[columnSize * 2];

		for (int index = 0; index < columnSize; index++) {
			vKnotVector[index] = 0.0;
			vKnotVector[index + columnSize] = 1.0;
		}

		return BSplineControlPoints_uKnotVector_vKnotVector_(aMatrix, uKnotVector, vKnotVector);
	}

	/**
	 * Create an instance of JunOpenGL3dNurbsSurface and initialize it with the
	 * arguments.
	 * 
	 * @param aMatrix sra.jun.collections.sequences.JunMatrix
	 * @param anArray1 double[]
	 * @param anArray2 double[]
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dNurbsSurface
	 */
	public static final JunOpenGL3dNurbsSurface BSplineControlPoints_uKnotVector_vKnotVector_(JunMatrix aMatrix, double[] anArray1, double[] anArray2) {
		int rowSize = aMatrix.rowSize();
		int columnSize = aMatrix.columnSize();
		double[] weights = new double[rowSize * columnSize];

		for (int i = 0; i < weights.length; i++) {
			weights[i] = 1;
		}

		return ControlPoints_weights_uKnotVector_vKnotVector_(aMatrix, new JunDoubleMatrix(rowSize, columnSize, weights), anArray1, anArray2);
	}

	/**
	 * Create an instance of JunOpenGL3dNurbsSurface and initialize it with the
	 * arguments.
	 * 
	 * @param aMatrix1 sra.jun.collections.sequences.JunMatrix
	 * @param aMatrix2 sra.jun.collections.sequences.JunDoubleMatrix
	 * @param anArray1 double[]
	 * @param anArray2 double[]
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dNurbsSurface
	 */
	public static final JunOpenGL3dNurbsSurface ControlPoints_weights_uKnotVector_vKnotVector_(JunMatrix aMatrix1, JunDoubleMatrix aMatrix2, double[] anArray1, double[] anArray2) {
		JunOpenGL3dNurbsSurface nurbsSurface = new JunOpenGL3dNurbsSurface();
		nurbsSurface.controlPoints_(aMatrix1);
		nurbsSurface.weights_(aMatrix2);
		nurbsSurface.uKnotVector_(anArray1);
		nurbsSurface.vKnotVector_(anArray2);

		return nurbsSurface;
	}

	/**
	 * Convert the receiver as a JunNurbsSurface.
	 * 
	 * @return sra.jun.geometry.surfaces.JunNurbsSurface
	 */
	public JunNurbsSurface _asJunNurbsSurface() {
		return JunNurbsSurface.ControlPoints_weights_uKnotVector_vKnotVector_(controlPoints, weights, uKnotVector, vKnotVector);
	}

	/**
	 * Answer the matrix of control points.
	 * 
	 * @return sra.jun.collections.sequences.JunMatrix
	 */
	public JunMatrix controlPoints() {
		return controlPoints;
	}

	/**
	 * Set the matrix of control points.
	 * 
	 * @param aJunMatrix sra.jun.collections.sequences.JunMatrix
	 */
	public void controlPoints_(JunMatrix aJunMatrix) {
		controlPoints = aJunMatrix;
	}

	/**
	 * Convert the receiver's control points as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList controlsToLispList() {
		final Vector collection = new Vector(controlPoints.size() + 3);
		collection.addElement($("controls"));
		collection.addElement(new Integer(controlPoints.rowSize()));
		collection.addElement(new Integer(controlPoints.columnSize()));
		controlPoints.do_(new StBlockClosure() {
			public Object value_(Object anObject) {
				collection.addElement(anObject);

				return null;
			}
		});

		return JunLispCons.List_(collection);
	}

	/**
	 * Enumerate every geometries and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object geometriesDo_(StBlockClosure aBlock) {
		return aBlock.value_(JunNurbsSurface.ControlPoints_weights_uKnotVector_vKnotVector_(controlPoints, weights, uKnotVector, vKnotVector));
	}

	/**
	 * Answer true if this JunOpenGL3dObject has a texture, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasTexture() {
		return texture != null;
	}

	/**
	 * Answer the StSymbol which represents the kind of the receiver.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public StSymbol kindName() {
		return $("NurbsSurface");
	}

	/**
	 * Answer the number of polygons.
	 * 
	 * @return int
	 */
	public int numberOfPolygons() {
		return 1;
	}

	/**
	 * Enumerate every points and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void pointsDo_(StBlockClosure aBlock) {
		controlPoints.do_(aBlock);
	}

	/**
	 * Do an extra copy of the receiver.
	 * 
	 * @return sra.smalltalk.StObject
	 */
	public StObject postCopy() {
		super.postCopy();
		controlPoints = (JunMatrix) controlPoints.copy();
		weights = (JunDoubleMatrix) weights.copy();

		double[] newUKnotVector = new double[uKnotVector.length];
		System.arraycopy(uKnotVector, 0, newUKnotVector, 0, uKnotVector.length);
		uKnotVector = newUKnotVector;

		double[] newVKnotVector = new double[vKnotVector.length];
		System.arraycopy(vKnotVector, 0, newVKnotVector, 0, vKnotVector.length);
		vKnotVector = newVKnotVector;

		return this;
	}

	/**
	 * Render the OpenGL object on a rendering context.
	 * 
	 * @param aRenderingContext
	 *        sra.jun.opengl.support.JunOpenGLRenderingContext
	 */
	public void renderOn_(JunOpenGLRenderingContext aRenderingContext) {
		int oldTextureCoordinateIndex = Integer.MIN_VALUE;

		if (this.hasPaint()) {
			aRenderingContext.paint_(this.paint());
		}

		if (this.hasAlpha()) {
			aRenderingContext.alpha_(this.alpha());
		}

		if (this.hasStipple()) {
			this.stipple().enableStippleOn_(aRenderingContext);
		}

		if (this.hasTexture()) {
			oldTextureCoordinateIndex = aRenderingContext.textureCoordinateIndex();
			aRenderingContext.textureCoordinateIndex_(0);
			this.texture().enableTextureOn_(aRenderingContext);
		}

		if (this.hasTransparency()) {
			aRenderingContext.enableCullFace();
			aRenderingContext.disableDepthMask();
			aRenderingContext.enableBlending();
		}

		aRenderingContext.displayNurbsSurface_(this._asJunNurbsSurface());

		if (this.hasTransparency()) {
			aRenderingContext.disableCullFace();
			aRenderingContext.enableDepthMask();
			aRenderingContext.disableBlending();
		}

		if (this.hasTexture()) {
			this.texture().disableTextureOn_(aRenderingContext);
			aRenderingContext.textureCoordinateIndex_(oldTextureCoordinateIndex);
		}

		if (this.hasStipple()) {
			this.stipple().disableStippleOn_(aRenderingContext);
		}
	}

	/**
	 * Anser the reversed JunOpenGL3dObject of the receiver.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public JunOpenGL3dObject reversed() {
		throw new SmalltalkException("not implemented yet");
	}

	/**
	 * Convert the receiver's stipple as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList stippleToLispList() {
		if (!this.hasStipple()) {
			return this.lispNil();
		}

		JunLispCons list = this.lispCons();
		list.head_($("stipple"));
		list.tail_(this.stipple().image());

		return list;
	}

	/**
	 * Answer the texture of the receiver.
	 * 
	 * @return sra.jun.opengl.texture.JunOpenGLTexture
	 */
	public JunOpenGLTexture texture() {
		return texture;
	}

	/**
	 * Convert the receiver's texture as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList textureToLispList() {
		if (!this.hasTexture()) {
			return this.lispNil();
		}

		JunLispCons list = this.lispCons();
		list.head_($("texture"));
		list.tail_(this.texture().toLispList());

		return list;
	}

	/**
	 * Convert the receiver as a JunLispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList toLispList() {
		JunLispCons list = this.lispCons();
		list.head_(this.kindName());

		if (this.hasName()) {
			list.add_(this.nameToLispList());
		}

		if (this.hasColor()) {
			list.add_(this.colorToLispList());
		}

		if (this.hasStipple()) {
			list.add_(this.stippleToLispList());
		}

		if (this.hasTexture()) {
			list.add_(this.textureToLispList());
		}

		list.add_(this.controlsToLispList());
		list.add_(this.weightsToLispList());
		list.add_(this.uKnotsToLispList());
		list.add_(this.vKnotsToLispList());

		return list;
	}

	/**
	 * Answer the new JunOpen3dObject transformed with aTransformation.
	 * 
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject transform_(final Jun3dTransformation aTransformation) {
		JunOpenGL3dNurbsSurface theNurbsSurface = (JunOpenGL3dNurbsSurface) this.copy();
		final JunMatrix cps = theNurbsSurface.controlPoints();
		cps.doIJ_(new StBlockClosure() {
			public Object value_value_value_(Object o1, Object o2, Object o3) {
				Jun3dPoint point = ((Jun3dPoint) o1).transform_(aTransformation);
				int i = ((Number) o2).intValue();
				int j = ((Number) o3).intValue();
				cps.row_column_put_(i, j, point);

				return null;
			}
		});

		return theNurbsSurface;
	}

	/**
	 * Convert the receiver's knot vector of u-parameter as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList uKnotsToLispList() {
		Object[] collection = new Object[uKnotVector.length + 1];
		collection[0] = $("vKnots");

		for (int i = 0; i < uKnotVector.length; i++) {
			collection[i + 1] = new Double(uKnotVector[i]);
		}

		return JunLispCons.List_(collection);
	}

	/**
	 * Answer the knot vector of u-parameter.
	 * 
	 * @return double[]
	 */
	public double[] uKnotVector() {
		return uKnotVector;
	}

	/**
	 * Set the knot vector of u-parameter.
	 * 
	 * @param anArray double[]
	 */
	public void uKnotVector_(double[] anArray) {
		uKnotVector = anArray;
	}

	/**
	 * Convert the receiver's knot vector of v-parameter as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList vKnotsToLispList() {
		Object[] collection = new Object[vKnotVector.length + 1];
		collection[0] = $("vKnots");

		for (int i = 0; i < vKnotVector.length; i++) {
			collection[i + 1] = new Double(vKnotVector[i]);
		}

		return JunLispCons.List_(collection);
	}

	/**
	 * Answer the V knot vector.
	 * 
	 * @return double[]
	 */
	public double[] vKnotVector() {
		return vKnotVector;
	}

	/**
	 * Set the knot vector of v-parameter.
	 * 
	 * @param anArray double[]
	 */
	public void vKnotVector_(double[] anArray) {
		vKnotVector = anArray;
	}

	/**
	 * not supported in original smalltalk jun
	 * 
	 * @param writer java.io.Writer
	 */
	public void vrml10On_(Writer writer) {
	}

	/**
	 * not supported in original smalltalk jun
	 * 
	 * @param writer java.io.Writer
	 */
	public void vrml20On_(Writer writer) {
	}

	/**
	 * Answer the matrix of the weight numbers.
	 * 
	 * @return sra.jun.collections.sequences.JunDoubleMatrix
	 */
	public JunDoubleMatrix weights() {
		return weights;
	}

	/**
	 * Set the matrix of the weight numbers.
	 * 
	 * @param aMatrix sra.jun.collections.sequences.JunDoubleMatrix
	 */
	public void weights_(JunDoubleMatrix aMatrix) {
		weights = aMatrix;
	}

	/**
	 * Convert the receiver's weight numbers as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList weightsToLispList() {
		final Vector collection = new Vector(weights.size() + 3);
		collection.addElement($("weights"));
		collection.addElement(new Integer(weights.rowSize()));
		collection.addElement(new Integer(weights.columnSize()));
		controlPoints.do_(new StBlockClosure() {
			public Object value_(Object anObject) {
				collection.addElement(anObject);

				return null;
			}
		});

		return JunLispCons.List_(collection);
	}
}
