package sra.jun.opengl.objects;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import sra.smalltalk.*;
import sra.jun.geometry.abstracts.JunGeometry;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.boundaries.*;
import sra.jun.geometry.surfaces.*;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.display.*;
import sra.jun.opengl.support.JunOpenGLRenderingContext;
import sra.jun.opengl.texture.*;
import sra.jun.vrml.support.JunVrmlRenderingContext;

/**
 * JunOpenGL3dObject class
 * 
 *  @author    nisinaka
 *  @created   1998/10/13 (by nisinaka)
 *  @updated   1999/08/04 (by nisinaka)
 *  @updated   2000/01/06 (by nisinaka)
 *  @updated   2001/11/20 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dObject.java,v 6.0 2003/10/28 10:52:19 nisinaka Exp $
 */
public abstract class JunOpenGL3dObject extends StObject {

	/** unique clock value for id */
	private static long IdClock = System.currentTimeMillis();
	private static long IdNumber = 0;

	/** The name of the object. */
	protected String name = null;

	/** The color of the object. */
	protected Color paint = null;

	/** The alpha of the object. */
	protected float alpha = Float.NaN;

	/** The alpha of the object. */
	protected Jun3dBoundingBox bounds = null;

	/**
	 * Create a new instance of JunOpenGL3dObject.
	 */
	public JunOpenGL3dObject() {
		super();
		this.initialize();
	}

	/**
	 * Typical objects - axes
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Axes() {
		Jun3dPoint[] twoPoints = new Jun3dPoint[2];
		Jun3dPoint[] fourPoints = new Jun3dPoint[4];
		JunOpenGL3dCompoundObject xAxis = new JunOpenGL3dCompoundObject();
		xAxis.name_("x axis");
		twoPoints[0] = new Jun3dPoint(-1, 0, 0);
		twoPoints[1] = new Jun3dPoint(1, 0, 0);
		xAxis.add_(JunOpenGL3dPolyline.Vertexes_paint_(twoPoints, Color.red));
		fourPoints[0] = new Jun3dPoint(1, 0, 0);
		fourPoints[1] = new Jun3dPoint(0.9, 0.05, 0);
		fourPoints[2] = new Jun3dPoint(0.9, 0, 0);
		fourPoints[3] = new Jun3dPoint(1, 0, 0);
		xAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.red));
		fourPoints[0] = new Jun3dPoint(1, 0, 0);
		fourPoints[1] = new Jun3dPoint(0.9, 0, 0);
		fourPoints[2] = new Jun3dPoint(0.9, -0.05, 0);
		fourPoints[3] = new Jun3dPoint(1, 0, 0);
		xAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.red));
		fourPoints[0] = new Jun3dPoint(1, 0, 0);
		fourPoints[1] = new Jun3dPoint(0.9, 0, 0.05);
		fourPoints[2] = new Jun3dPoint(0.9, 0, 0);
		fourPoints[3] = new Jun3dPoint(1, 0, 0);
		xAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.red));
		fourPoints[0] = new Jun3dPoint(1, 0, 0);
		fourPoints[1] = new Jun3dPoint(0.9, 0, 0);
		fourPoints[2] = new Jun3dPoint(0.9, 0, -0.05);
		fourPoints[3] = new Jun3dPoint(1, 0, 0);
		xAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.red));

		JunOpenGL3dCompoundObject yAxis = new JunOpenGL3dCompoundObject();
		yAxis.name_("y axis");
		twoPoints[0] = new Jun3dPoint(0, -1, 0);
		twoPoints[1] = new Jun3dPoint(0, 1, 0);
		yAxis.add_(JunOpenGL3dPolyline.Vertexes_paint_(twoPoints, Color.green));
		fourPoints[0] = new Jun3dPoint(0, 1, 0);
		fourPoints[1] = new Jun3dPoint(0.05, 0.9, 0);
		fourPoints[2] = new Jun3dPoint(0, 0.9, 0);
		fourPoints[3] = new Jun3dPoint(0, 1, 0);
		yAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.green));
		fourPoints[0] = new Jun3dPoint(0, 1, 0);
		fourPoints[1] = new Jun3dPoint(0, 0.9, 0);
		fourPoints[2] = new Jun3dPoint(-0.05, 0.9, 0);
		fourPoints[3] = new Jun3dPoint(0, 1, 0);
		yAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.green));
		fourPoints[0] = new Jun3dPoint(0, 1, 0);
		fourPoints[1] = new Jun3dPoint(0, 0.9, 0.05);
		fourPoints[2] = new Jun3dPoint(0, 0.9, 0);
		fourPoints[3] = new Jun3dPoint(0, 1, 0);
		yAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.green));
		fourPoints[0] = new Jun3dPoint(0, 1, 0);
		fourPoints[1] = new Jun3dPoint(0, 0.9, 0);
		fourPoints[2] = new Jun3dPoint(0, 0.9, -0.05);
		fourPoints[3] = new Jun3dPoint(0, 1, 0);
		yAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.green));

		JunOpenGL3dCompoundObject zAxis = new JunOpenGL3dCompoundObject();
		zAxis.name_("z axis");
		twoPoints[0] = new Jun3dPoint(0, 0, -1);
		twoPoints[1] = new Jun3dPoint(0, 0, 1);
		zAxis.add_(JunOpenGL3dPolyline.Vertexes_paint_(twoPoints, Color.blue));
		fourPoints[0] = new Jun3dPoint(0, 0, 1);
		fourPoints[1] = new Jun3dPoint(0.05, 0, 0.9);
		fourPoints[2] = new Jun3dPoint(0, 0, 0.9);
		fourPoints[3] = new Jun3dPoint(0, 0, 1);
		zAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.blue));
		fourPoints[0] = new Jun3dPoint(0, 0, 1);
		fourPoints[1] = new Jun3dPoint(0, 0, 0.9);
		fourPoints[2] = new Jun3dPoint(-0.05, 0, 0.9);
		fourPoints[3] = new Jun3dPoint(0, 0, 1);
		zAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.blue));
		fourPoints[0] = new Jun3dPoint(0, 0, 1);
		fourPoints[1] = new Jun3dPoint(0, 0.05, 0.9);
		fourPoints[2] = new Jun3dPoint(0, 0, 0.9);
		fourPoints[3] = new Jun3dPoint(0, 0, 1);
		zAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.blue));
		fourPoints[0] = new Jun3dPoint(0, 0, 1);
		fourPoints[1] = new Jun3dPoint(0, 0, 0.9);
		fourPoints[2] = new Jun3dPoint(0, -0.05, 0.9);
		fourPoints[3] = new Jun3dPoint(0, 0, 1);
		zAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.blue));

		JunOpenGL3dCompoundObject axes = JunOpenGL3dCompoundObject.With_with_with_(xAxis, yAxis, zAxis);
		axes.name_("axes");

		return axes;
	}

	/**
	 * Typical objects - axes
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Axes2() {
		Jun3dPoint[] twoPoints = new Jun3dPoint[2];
		Jun3dPoint[] fourPoints = new Jun3dPoint[4];
		JunOpenGL3dCompoundObject xAxis = new JunOpenGL3dCompoundObject();
		xAxis.name_("x axis");
		twoPoints[0] = new Jun3dPoint(0, 0, 0);
		twoPoints[1] = new Jun3dPoint(1, 0, 0);
		xAxis.add_(JunOpenGL3dPolyline.Vertexes_paint_(twoPoints, Color.red));
		fourPoints[0] = new Jun3dPoint(1, 0, 0);
		fourPoints[1] = new Jun3dPoint(0.9, 0.05, 0);
		fourPoints[2] = new Jun3dPoint(0.9, 0, 0);
		fourPoints[3] = new Jun3dPoint(1, 0, 0);
		xAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.red));
		fourPoints[0] = new Jun3dPoint(1, 0, 0);
		fourPoints[1] = new Jun3dPoint(0.9, 0, 0.05);
		fourPoints[2] = new Jun3dPoint(0.9, 0, 0);
		fourPoints[3] = new Jun3dPoint(1, 0, 0);
		xAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.red));

		JunOpenGL3dCompoundObject yAxis = new JunOpenGL3dCompoundObject();
		yAxis.name_("y axis");
		twoPoints[0] = new Jun3dPoint(0, 0, 0);
		twoPoints[1] = new Jun3dPoint(0, 1, 0);
		yAxis.add_(JunOpenGL3dPolyline.Vertexes_paint_(twoPoints, Color.green));
		fourPoints[0] = new Jun3dPoint(0, 1, 0);
		fourPoints[1] = new Jun3dPoint(0.05, 0.9, 0);
		fourPoints[2] = new Jun3dPoint(0, 0.9, 0);
		fourPoints[3] = new Jun3dPoint(0, 1, 0);
		yAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.green));
		fourPoints[0] = new Jun3dPoint(0, 1, 0);
		fourPoints[1] = new Jun3dPoint(0, 0.9, 0.05);
		fourPoints[2] = new Jun3dPoint(0, 0.9, 0);
		fourPoints[3] = new Jun3dPoint(0, 1, 0);
		yAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.green));

		JunOpenGL3dCompoundObject zAxis = new JunOpenGL3dCompoundObject();
		zAxis.name_("z axis");
		twoPoints[0] = new Jun3dPoint(0, 0, 0);
		twoPoints[1] = new Jun3dPoint(0, 0, 1);
		zAxis.add_(JunOpenGL3dPolyline.Vertexes_paint_(twoPoints, Color.blue));
		fourPoints[0] = new Jun3dPoint(0, 0, 1);
		fourPoints[1] = new Jun3dPoint(0.05, 0, 0.9);
		fourPoints[2] = new Jun3dPoint(0, 0, 0.9);
		fourPoints[3] = new Jun3dPoint(0, 0, 1);
		zAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.blue));
		fourPoints[0] = new Jun3dPoint(0, 0, 1);
		fourPoints[1] = new Jun3dPoint(0, 0.05, 0.9);
		fourPoints[2] = new Jun3dPoint(0, 0, 0.9);
		fourPoints[3] = new Jun3dPoint(0, 0, 1);
		zAxis.add_(JunOpenGL3dPolygon.Vertexes_paint_(fourPoints, Color.blue));

		JunOpenGL3dCompoundObject axes = JunOpenGL3dCompoundObject.With_with_with_(xAxis, yAxis, zAxis);
		axes.name_("axes");

		return axes;
	}

	/**
	 * Typical objects - box
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Box() {
		return JunOpenGL3dObject.Box_(new Jun3dPoint(1, 1, 1));
	}

	/**
	 * Typical objects - box
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Box_(Jun3dPoint aPoint) {
		Jun3dPoint p0 = new Jun3dPoint(0, 0, 0);
		Jun3dPoint p1 = new Jun3dPoint(0, 0, 1);
		Jun3dPoint p2 = new Jun3dPoint(0, 1, 0);
		Jun3dPoint p3 = new Jun3dPoint(0, 1, 1);
		Jun3dPoint p4 = new Jun3dPoint(1, 0, 0);
		Jun3dPoint p5 = new Jun3dPoint(1, 0, 1);
		Jun3dPoint p6 = new Jun3dPoint(1, 1, 0);
		Jun3dPoint p7 = new Jun3dPoint(1, 1, 1);
		Jun3dPoint[] floorPoints = { p1.multipliedBy_(aPoint), p5.multipliedBy_(aPoint), p7.multipliedBy_(aPoint), p3.multipliedBy_(aPoint)};
		Jun3dPoint[] ceilPoints = { p2.multipliedBy_(aPoint), p6.multipliedBy_(aPoint), p4.multipliedBy_(aPoint), p0.multipliedBy_(aPoint)};
		Jun3dPoint[] leftPoints = { p4.multipliedBy_(aPoint), p6.multipliedBy_(aPoint), p7.multipliedBy_(aPoint), p5.multipliedBy_(aPoint)};
		Jun3dPoint[] rightPoints = { p1.multipliedBy_(aPoint), p3.multipliedBy_(aPoint), p2.multipliedBy_(aPoint), p0.multipliedBy_(aPoint)};
		Jun3dPoint[] nearPoints = { p3.multipliedBy_(aPoint), p7.multipliedBy_(aPoint), p6.multipliedBy_(aPoint), p2.multipliedBy_(aPoint)};
		Jun3dPoint[] farPoints = { p0.multipliedBy_(aPoint), p4.multipliedBy_(aPoint), p5.multipliedBy_(aPoint), p1.multipliedBy_(aPoint)};
		JunOpenGL3dCompoundObject box = new JunOpenGL3dCompoundObject();
		box.name_("box");
		box.add_(JunOpenGL3dPolygon.Vertexes_(floorPoints));
		box.add_(JunOpenGL3dPolygon.Vertexes_(ceilPoints));
		box.add_(JunOpenGL3dPolygon.Vertexes_(leftPoints));
		box.add_(JunOpenGL3dPolygon.Vertexes_(rightPoints));
		box.add_(JunOpenGL3dPolygon.Vertexes_(nearPoints));
		box.add_(JunOpenGL3dPolygon.Vertexes_(farPoints));
		box.paint_(DefaultPaint());

		return box;
	}

	/**
	 * Typical objects misc - characterN
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject CharacterN() {
		JunOpenGL3dObject aCube;
		JunOpenGL3dCompoundObject characterN;

		characterN = new JunOpenGL3dCompoundObject();
		aCube = Cube().scaledBy_(new Jun3dPoint(0.5, 0.25, 1)).translatedBy_(new Jun3dPoint(0, -0.75, 0));
		characterN.add_(aCube);
		aCube = Cube().scaledBy_(new Jun3dPoint(0.5, 0.25, 1)).translatedBy_(new Jun3dPoint(0, 0.75, 0));
		characterN.add_(aCube);
		aCube = Cube().scaledBy_(new Jun3dPoint(0.5, 0.25, 1)).transform_(Jun3dTransformation.RotateX_(JunAngle.FromDeg_(45)));
		characterN.add_(aCube);
		characterN.name_("character N");

		// characterN do: [:each | each paint: nil].
		for (int i = 0; i < characterN.components().length; i++) {
			characterN.components()[i].paint_((Color) null);
		}

		characterN.paint_(DefaultPaint());

		return characterN;
	}

	/**
	 * Get a color from the lisp list and set it as the name of the
	 * JunOpenGL3dObject.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject ColorFromLispList_for_(JunLispCons aList, JunOpenGL3dObject a3dObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("color")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return a3dObject;
		}

		list = (JunLispCons) list.tail();

		double red = ((Number) list.nth_(1)).doubleValue();
		double green = ((Number) list.nth_(2)).doubleValue();
		double blue = ((Number) list.nth_(3)).doubleValue();
		Number alpha = null;

		if (list.length() >= 4) {
			alpha = (Number) list.nth_(4);

			if (alpha instanceof Integer) {
				alpha = new Float(Math.max(0, Math.min(alpha.floatValue() / 255, 1)));
			}
		}

		a3dObject.paint_(new Color((float) red, (float) green, (float) blue));

		if (alpha != null) {
			a3dObject.alpha_(alpha.floatValue());
		}

		return a3dObject;
	}

	/**
	 * Typical objects - cone
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Cone() {
		return Cone_(10);
	}

	/**
	 * Typical objects - cone
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param degrees double
	 */
	public static final JunOpenGL3dObject Cone_(double degrees) {
		return Cone_radius_height_(degrees, 1, 2);
	}

	/**
	 * Typical objects - cone
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param degrees double
	 * @param radius double
	 * @param height double
	 */
	public static final JunOpenGL3dObject Cone_radius_height_(double degrees, double radius, double height) {
		JunOpenGL3dCompoundObject cone = new JunOpenGL3dCompoundObject();
		StBlockClosure rotationBlock = new StBlockClosure() {
			public Object value_value_(Object o1, Object o2) {
				Jun3dPoint p = (Jun3dPoint) o1;
				double t = ((Number) o2).doubleValue();
				double x = (p.x() * Math.cos(t)) - (p.y() * Math.sin(t));
				double y = (p.x() * Math.sin(t)) + (p.y() * Math.cos(t));
				if (Math.abs(x) < JunGeometry.ACCURACY) {
					x = 0.0d;
				}
				if (Math.abs(y) < JunGeometry.ACCURACY) {
					y = 0.0d;
				}
				return new Jun3dPoint(x, y, 0.0d);
			}
		};

		Jun3dPoint apex = new Jun3dPoint(0, 0, height);
		Jun3dPoint base = new Jun3dPoint(radius, 0, 0);
		Jun3dPoint base1 = base;
		for (double d = degrees; d <= 360; d += degrees) {
			Jun3dPoint base2 = (Jun3dPoint) rotationBlock.value_value_(base, new Double(JunAngle._DegreesToRadians(d)));
			cone.add_(new JunOpenGL3dPolygon(new Jun3dPoint[] { apex, base1, base2 }));
			cone.add_(new JunOpenGL3dPolygon(new Jun3dPoint[] { Jun3dPoint.Zero(), base2, base1 }));
			base1 = base2;
		}
		cone.objectsDo_(new StBlockClosure() {
			public Object value_(Object each) {
				((JunOpenGL3dObject) each).paint_(null);
				return null;
			}
		});
		cone.paint_(DefaultPaint());

		return cone;
	}

	/**
	 * Typical objects - cube
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Cube() {
		return Box().scaledBy_(2).translatedBy_(-1);
	}

	/**
	 * Answer the default color to paint JunOpenGL3dObject.
	 * 
	 * @return java.awt.Color
	 */
	public static Color DefaultPaint() {
		return Color.white;
	}

	/**
	 * typical objects misc
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject Eddy() {
		Vector collection = new Vector();
		Jun3dPoint[] triangle = new Jun3dPoint[3];
		Jun3dPoint[] triangle1 = new Jun3dPoint[3];
		triangle[0] = new Jun3dPoint(1.0, 1.0, 0.0);
		triangle[1] = new Jun3dPoint(-1.0, 1.0, 0.0);
		triangle[2] = new Jun3dPoint(-1.0, -1.0, 0.0);

		Jun3dTransformation translationT1 = Jun3dTransformation.Translate_(new Jun3dPoint(0.5, 0.1, 0.0));
		Jun3dTransformation yRotationT1 = Jun3dTransformation.RotateY_(JunAngle.FromDeg_(10.0));
		triangle1 = triangle;

		for (int i = 0; i < 100; i++) {
			Color color = JunOpenGL3dObject.SampleColor();
			StAssociation assoc = new StAssociation(color, triangle1.clone());
			collection.addElement(assoc);

			for (int j = 0; j < triangle1.length; j++) {
				Jun3dPoint point = triangle1[j];
				point = (point.transform_(translationT1)).transform_(yRotationT1);
				triangle1[j] = point;
			}
		}

		JunOpenGL3dCompoundObject eddy = new JunOpenGL3dCompoundObject();
		eddy.name_("eddy");

		for (Enumeration e = collection.elements(); e.hasMoreElements();) {
			JunOpenGL3dCompoundObject body = new JunOpenGL3dCompoundObject();
			StAssociation assoc = (StAssociation) e.nextElement();
			Color color = (Color) assoc.key();
			Jun3dPoint[] points = (Jun3dPoint[]) assoc.value();
			Jun3dPoint[] points2 = new Jun3dPoint[points.length + 1];

			for (int i = 0; i < points.length; i++) {
				points2[i] = points[i];
			}

			points2[points.length] = points[0];
			body.add_(JunOpenGL3dPolygon.Vertexes_paint_(points, color));
			body.add_(JunOpenGL3dPolyline.Vertexes_paint_(points2, Color.black));
			eddy.add_(body);
		}

		return eddy;
	}

	/**
	 * Typical objects - globe
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Globe() {
		JunOpenGL3dObject globe = Sphere();
		globe.name_("globe");

		return globe;
	}

	/**
	 * Typical objects - globe
	 * 
	 * @param degrees double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Globe_(double degrees) {
		JunOpenGL3dObject globe = Sphere_(degrees);
		globe.name_("globe");

		return globe;
	}

	/**
	 * Typical objects - globe
	 * 
	 * @param degrees double
	 * @param radius double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Globe_radius_(double degrees, double radius) {
		JunOpenGL3dObject globe = Sphere_radius_(degrees, radius);
		globe.name_("globe");

		return globe;
	}

	/**
	 * Typical objects - globe
	 * 
	 * @param degrees double
	 * @param radius double
	 * @param center sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Globe_radius_center_(double degrees, double radius, Jun3dPoint center) {
		JunOpenGL3dObject globe = Sphere_radius_center_(degrees, radius, center);
		globe.name_("globe");

		return globe;
	}

	/**
	 * Typical objects - sphere
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Hemisphere() {
		JunOpenGL3dObject hemisphere = Sphere_radius_longitude_latitude_(10, 1, 180, 180);
		hemisphere.name_("hemisphere");

		return hemisphere;
	}

	/**
	 * Typical objects - sphere
	 * 
	 * @param degrees double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Hemisphere_(double degrees) {
		JunOpenGL3dObject hemisphere = Sphere_radius_longitude_latitude_(degrees, 1, 180, 180);
		hemisphere.name_("hemisphere");

		return hemisphere;
	}

	/**
	 * Typical objects - sphere
	 * 
	 * @param degrees double
	 * @param radius double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Hemisphere_redius_(double degrees, double radius) {
		JunOpenGL3dObject hemisphere = Sphere_radius_longitude_latitude_(degrees, radius, 180, 180);
		hemisphere.name_("hemisphere");

		return hemisphere;
	}

	/**
	 * Typical objects - sphere
	 * 
	 * @param degrees double
	 * @param radius double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Hemisphere_redius_center_(double degrees, double radius) {
		JunOpenGL3dObject hemisphere = Sphere_radius_longitude_latitude_(degrees, radius, 180, 180);
		hemisphere.name_("hemisphere");

		return hemisphere;
	}

	/**
	 * Typical objects - sphere
	 * 
	 * @param degrees double
	 * @param radius double
	 * @param center sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Hemisphere_redius_center_(double degrees, double radius, Jun3dPoint center) {
		JunOpenGL3dObject hemisphere = Sphere_radius_longitude_latitude_(degrees, radius, 180, 180);
		hemisphere.transform_(Jun3dTransformation.Align_with_(new Jun3dPoint(0, 0, 0), center));
		hemisphere.name_("hemisphere");

		return hemisphere;
	}

	/**
	 * Load a JunOpenGL3dObject from the object.
	 * 
	 * @param anObject java.lang.Object
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject LoadFrom_(Object anObject) {
		JunLispList aList = (JunLispList) JunLispParser.Parse_(anObject);

		if ((aList != null) && aList instanceof JunLispCons) {
			return JunOpenGL3dCompoundObject.ObjectFromLispList_((JunLispCons) aList);
		} else {
			return null;
		}
	}

	/**
	 * Get a name from the lisp list and set it as the name of the
	 * JunOpenGL3dObject.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject NameFromLispList_for_(JunLispCons aList, JunOpenGL3dObject a3dObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("name")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return a3dObject;
		}

		a3dObject.name_((String) ((JunLispCons) list).tail());

		return a3dObject;
	}

	/**
	 * Typical objects - box
	 * 
	 * @param originPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param cornerPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Origin_corner_(Jun3dPoint originPoint, Jun3dPoint cornerPoint) {
		double originX = Math.min(originPoint.x(), cornerPoint.x());
		double originY = Math.min(originPoint.y(), cornerPoint.y());
		double originZ = Math.min(originPoint.z(), cornerPoint.z());
		Jun3dPoint extent = (Jun3dPoint) cornerPoint.minus_(originPoint).abs();

		return Origin_extent_(new Jun3dPoint(originX, originY, originZ), extent);
	}

	/**
	 * Typical objects - box
	 * 
	 * @param originPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param extentPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Origin_extent_(Jun3dPoint originPoint, Jun3dPoint extentPoint) {
		JunOpenGL3dObject box = Box_(extentPoint);
		Jun3dTransformation transformation = Jun3dTransformation.Align_with_(Jun3dPoint.Zero(), originPoint);

		return box.transform_(transformation);
	}

	/**
	 * Typical object - pie.
	 * 
	 * @param fromDegree double
	 * @param toDegree double
	 * @param byDegree double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject PieFrom_to_by_(double fromDegree, double toDegree, double byDegree) {
		return PieFrom_to_by_radius_(fromDegree, toDegree, byDegree, 1.0d);
	}

	/**
	 * Typical object - pie.
	 * 
	 * @param fromDegree double
	 * @param toDegree double
	 * @param byDegree double
	 * @param radius double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject PieFrom_to_by_radius_(double fromDegree, double toDegree, double byDegree, double radius) {
		if (Math.abs(fromDegree - toDegree) > 180) {
			return SlicedPieFrom_to_by_radius_(fromDegree, toDegree, byDegree, radius);
		} else {
			return ChunkOfPieFrom_to_by_radius_(fromDegree, toDegree, byDegree, radius);
		}
	}

	/**
	 * Typical object - pie.
	 * 
	 * @param fromDegree double
	 * @param toDegree double
	 * @param byDegree double
	 * @param radius double
	 * @param thickness double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject PieFrom_to_by_radius_thickness_(double fromDegree, double toDegree, double byDegree, double radius, double thickness) {
		if (Math.abs(fromDegree - toDegree) > 180) {
			return SlicedPieFrom_to_by_radius_thickness_(fromDegree, toDegree, byDegree, radius, thickness);
		} else {
			return ChunkOfPieFrom_to_by_radius_thickness_(fromDegree, toDegree, byDegree, radius, thickness);
		}
	}

	/**
	 * Answer a Color.
	 * 
	 * @return java.awt.Color
	 */
	public static Color SampleColor() {
		Vector symbols = StColorValue.ConstantNames();
		StSymbol symbol = (StSymbol) symbols.elementAt((int) Math.round(Math.random() * (symbols.size() - 1)));

		return StColorValue._GetColorByName_(symbol.toString());
	}

	/**
	 * Popup a window which displays the Lisp List.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public static JunLispList ShowLispList_(JunLispList aList) {
		TextArea textArea = new TextArea(5, 40);
		textArea.setEditable(false);
		textArea.append(aList.saveString());

		//
		Frame aFrame = new Frame();
		aFrame.add("Center", textArea);
		aFrame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				e.getWindow().dispose();
			}
		});
		aFrame.setTitle("Lisp List");
		aFrame.pack();
		aFrame.setVisible(true);

		//
		return aList;
	}

	/**
	 * Typical objects - sphere
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Sphere() {
		return (JunOpenGL3dObject) XyPointsAndSphere_radius_longitude_latitude_(10, 1, 360, 180)[1];
	}

	/**
	 * Typical objects - sphere
	 * 
	 * @param degrees double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Sphere_(double degrees) {
		return (JunOpenGL3dObject) XyPointsAndSphere_radius_longitude_latitude_(degrees, 1, 360, 180)[1];
	}

	/**
	 * Typical objects - sphere
	 * 
	 * @param degrees double
	 * @param radius double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Sphere_radius_(double degrees, double radius) {
		return (JunOpenGL3dObject) XyPointsAndSphere_radius_longitude_latitude_(degrees, radius, 360, 180)[1];
	}

	/**
	 * Typical objects - sphere
	 * 
	 * @param degrees double
	 * @param radius double
	 * @param center sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Sphere_radius_center_(double degrees, double radius, Jun3dPoint center) {
		return Jun3dTransformation.Align_with_(Jun3dPoint.Zero(), center).applyTo_(Sphere_radius_(degrees, radius));
	}

	/**
	 * Typical objects - sphere
	 * 
	 * @param degrees double
	 * @param radius double
	 * @param longitude double
	 * @param latitude double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Sphere_radius_longitude_latitude_(double degrees, double radius, double longitude, double latitude) {
		return (JunOpenGL3dObject) XyPointsAndSphere_radius_longitude_latitude_(degrees, radius, longitude, latitude)[1];
	}

	/**
	 * typical objects misc
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject Stack() {
		JunOpenGL3dCompoundObject stack = new JunOpenGL3dCompoundObject();
		stack.name_("stack");

		Jun3dPoint[] polyline = { new Jun3dPoint(-1, -1, 0), new Jun3dPoint(-1, 1, 0), new Jun3dPoint(1, 1, 0), new Jun3dPoint(1, -1, 0)};

		for (float i = 0; i < 5; i += 0.5) {
			Color color = JunOpenGL3dObject.SampleColor();
			Jun3dPoint[] points = new Jun3dPoint[polyline.length];

			for (int j = 0; j < polyline.length; j++) {
				points[j] = new Jun3dPoint(polyline[j].x(), polyline[j].y(), i * -1);
			}

			Jun3dPoint[] points2 = new Jun3dPoint[points.length + 1];

			for (int j = 0; j < points.length; j++) {
				points2[j] = points[j];
			}

			points2[points.length] = points[0];

			JunOpenGL3dCompoundObject body = new JunOpenGL3dCompoundObject();
			body.add_(JunOpenGL3dPolygon.Vertexes_paint_(points, color));
			body.add_(JunOpenGL3dPolyline.Vertexes_paint_(points2, Color.black));
			stack.add_(body);
		}

		for (float i = 0; i < 5; i += 0.5) {
			Color color = JunOpenGL3dObject.SampleColor();
			Jun3dPoint[] points = new Jun3dPoint[polyline.length];

			for (int j = 0; j < polyline.length; j++) {
				points[j] = new Jun3dPoint(polyline[j].x(), polyline[j].y(), i);
			}

			Jun3dPoint[] points2 = new Jun3dPoint[points.length + 1];

			for (int j = 0; j < points.length; j++) {
				points2[j] = points[j];
			}

			points2[points.length] = points[0];

			JunOpenGL3dCompoundObject body = new JunOpenGL3dCompoundObject();
			body.add_(JunOpenGL3dPolygon.Vertexes_paint_(points, color));
			body.add_(JunOpenGL3dPolyline.Vertexes_paint_(points2, Color.black));
			stack.add_(body);
		}

		return stack;
	}

	/**
	 * Typical objects - 2D Text
	 * 
	 * @param aString java.lang.String
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject Text2d_(String aString) {
		return Text2d_weight_(aString, 0.5);
	}

	/**
	 * Typical objects - 2D Text
	 * 
	 * @param aString java.lang.String
	 * @param aNumber double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject Text2d_weight_(String aString, double aNumber) {
		return Text2d_weight_font_(aString, aNumber, null);
	}

	/**
	 * Typical objects - 2D Text
	 * 
	 * @param aString java.lang.String
	 * @param aNumber double
	 * @param aFont java.awt.Font
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject Text2d_weight_font_(String aString, double aNumber, Font aFont) {
		double weight = Math.max(0.1, Math.min(aNumber, 0.9));
		Font font = (aFont != null) ? aFont : new Font("Courier", Font.PLAIN, 12);
		Point extent = SystemResourceSupport._getStringExtentFor_(aString, font);
		final StImage image = new StImage(extent.x, extent.y);
		Graphics graphics = null;

		try {
			graphics = image.image().getGraphics();
			graphics.setColor(Color.black);
			graphics.fillRect(0, 0, extent.x, extent.y);
			graphics.setColor(Color.white);
			graphics.setFont(font);

			int y = SystemResourceSupport.getFontMetrics(font).getAscent();
			graphics.drawString(aString, 1, y);
			image.image().flush();
		} finally {
			if (graphics != null) {
				graphics.dispose();
				graphics = null;
			}
		}

		StBlockClosure evaluationBlock = new StBlockClosure() {
			public Object value_value_(Object x, Object y) {
				Color color = image.valueAtPoint_(new Point(((Number) x).intValue(), (image.height() - ((Number) y).intValue())));
				float[] hsb = Color.RGBtoHSB(color.getRed(), color.getGreen(), color.getBlue(), null);

				return new Float(hsb[2]);
			}
		};

		Jun2dPolygonalBoundary polygonalBoundary = JunBorderGenerator.PolygonalBoundaryWith_min_xInterval_yInterval_interim_(evaluationBlock, 1 - weight, new StInterval(0, image.width() - 1), new StInterval(1, image.height() - 1), new StBlockClosure());

		JunOpenGL3dObject object = polygonalBoundary.asJunOpenGL3dObject();
		object.objectsDo_(new StBlockClosure() {
			public Object value_(Object each) {
				((JunOpenGL3dObject) each).paint_(null);

				return null;
			}
		});
		object.paint_(DefaultPaint());

		return object;
	}

	/**
	 * Typical objects - 3D Text
	 * 
	 * @param aString java.lang.String
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject Text3d_(String aString) {
		return Text3d_weight_(aString, 0.5);
	}

	/**
	 * Typical objects - 3D Text
	 * 
	 * @param aString java.lang.String
	 * @param aNumber double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject Text3d_weight_(String aString, double aNumber) {
		return Text3d_weight_font_(aString, aNumber, null);
	}

	/**
	 * Typical objects - 3D Text
	 * 
	 * @param aString java.lang.String
	 * @param aNumber double
	 * @param aFont java.awt.Font
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject Text3d_weight_font_(String aString, double aNumber, Font aFont) {
		double weight = Math.max(0.1, Math.min(aNumber, 0.9));
		Font font = (aFont != null) ? aFont : new Font("Courier", Font.PLAIN, 12);
		Point extent = SystemResourceSupport._getStringExtentFor_(aString, font);
		final StImage image = new StImage(extent.x, extent.y);
		Graphics graphics = null;

		try {
			graphics = image.image().getGraphics();
			graphics.setColor(Color.black);
			graphics.fillRect(0, 0, extent.x, extent.y);
			graphics.setColor(Color.white);
			graphics.setFont(font);

			int y = SystemResourceSupport.getFontMetrics(font).getAscent();
			graphics.drawString(aString, 1, y);
			image.image().flush();
		} finally {
			if (graphics != null) {
				graphics.dispose();
				graphics = null;
			}
		}

		StBlockClosure evaluationBlock = new StBlockClosure() {
			public Object value_value_value_(Object objX, Object objY, Object objZ) {
				int x = ((Number) objX).intValue();
				int y = ((Number) objY).intValue();
				int z = ((Number) objZ).intValue();
				Color color = image.valueAtPoint_(new Point(x, image.height() - y));
				float[] hsb = Color.RGBtoHSB(color.getRed(), color.getGreen(), color.getBlue(), null);
				double result = 1 - hsb[2];
				result = Math.sqrt((result * result) + (z * z));

				return new Double(result);
			}
		};

		JunPolygon[] polygons =
			JunBorderGenerator.ContourPolygonsFrom_at_sign_xInterval_yInterval_zInterval_interim_(evaluationBlock, weight, -1, new StInterval(0, image.width() - 1), new StInterval(1, image.height()), new StInterval(-1, 1), new StBlockClosure());

		JunOpenGL3dCompoundObject object = new JunOpenGL3dCompoundObject();

		for (int i = 0; i < polygons.length; i++) {
			JunOpenGL3dObject each = polygons[i].asJunOpenGL3dObject();
			each.paint_(null);
			object.add_(each);
		}

		object.paint_(DefaultPaint());

		return object;
	}

	/**
	 * Get a transformation from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public static Jun3dTransformation TransformationFromLispList_(JunLispCons aList) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("transformation")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return null;
		}

		list = (JunLispCons) list.tail();

		double[] array = new double[16];

		for (int i = 0; i < 16; i++) {
			array[i] = ((Number) list.nth_(i + 1)).doubleValue();
		}

		return Jun3dTransformation.FromArray_(array);
	}

	/**
	 * Get a transformation from the lisp list and set it as the transformation
	 * of the JunOpenGL3dObject.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject TransformationFromLispList_for_(JunLispCons aList, JunOpenGL3dObject a3dObject) {
		Jun3dTransformation transformation = TransformationFromLispList_(aList);

		if (transformation != null) {
			a3dObject.transform_(transformation);
		}

		return a3dObject;
	}

	/**
	 * Create a lisp list from the Jun3dTransformation.
	 * 
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public static JunLispList TransformationToLispList_(Jun3dTransformation aTransformation) {
		JunLispCons list = JunLispCons.Cell();
		list.head_($("transformation"));
		list.add_(new Double(aTransformation.a));
		list.add_(new Double(aTransformation.b));
		list.add_(new Double(aTransformation.c));
		list.add_(new Double(aTransformation.p));
		list.add_(new Double(aTransformation.d));
		list.add_(new Double(aTransformation.e));
		list.add_(new Double(aTransformation.f));
		list.add_(new Double(aTransformation.q));
		list.add_(new Double(aTransformation.g));
		list.add_(new Double(aTransformation.h));
		list.add_(new Double(aTransformation.i));
		list.add_(new Double(aTransformation.r));
		list.add_(new Double(aTransformation.l));
		list.add_(new Double(aTransformation.m));
		list.add_(new Double(aTransformation.n));
		list.add_(new Double(aTransformation.s));

		return list;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static String UniqueId() {
		long currentClock = System.currentTimeMillis();

		if (currentClock == IdClock) {
			IdNumber = IdNumber + 1;
		} else {
			IdClock = currentClock;
			IdNumber = 0;
		}

		return "id" + new Long(IdClock).toString() + new Long(IdNumber).toString();
	}

	/**
	 * Answer the array which includes a collection of points and a sphere.
	 * 
	 * @param degrees double
	 * @param radius double
	 * @param longitude double
	 * @param latitude double
	 * 
	 * @return java.lang.Object[]
	 */
	public static final Object[] XyPointsAndSphere_radius_longitude_latitude_(double degrees, double radius, double longitude, double latitude) {
		final double theDegrees = Math.max(1, Math.min(degrees, 90));
		final double theRadius = Math.max(0, radius);
		final double theLongitude = Math.max(1, Math.min(longitude, 360));
		final double theLatitude = Math.max(1, Math.min(latitude, 180));
		StBlockClosure xyzBlock = new StBlockClosure() {
			public Object value_value_value_(Object r, Object pd, Object td) {
				double phi = JunAngle._DegreesToRadians(((Double) pd).doubleValue());
				double theta = JunAngle._DegreesToRadians(((Double) td).doubleValue());
				double rd = ((Double) r).doubleValue();
				double x = rd * Math.sin(theta) * Math.cos(phi);
				double y = rd * Math.sin(theta) * Math.sin(phi);
				double z = rd * Math.cos(theta);

				if (Math.abs(x - 0.0d) < JunGeometry.ACCURACY) {
					x = 0.0d;
				}

				if (Math.abs(y - 0.0d) < JunGeometry.ACCURACY) {
					y = 0.0d;
				}

				if (Math.abs(z - 0.0d) < JunGeometry.ACCURACY) {
					z = 0.0d;
				}

				return new Jun3dPoint(x, y, z);
			}
		};

		StBlockClosure xyBlock = new StBlockClosure() {
			public Object value_value_(Object phi, Object theta) {
				return new Jun2dPoint(((Double) phi).doubleValue() / theLongitude, ((Double) theta).doubleValue() / theLatitude);
			}
		};

		int numberOfPolygons = (int) Math.round(theLongitude / theDegrees * (theLatitude / theDegrees));
		Vector[] pointCollections = new Vector[numberOfPolygons];
		int indexOfPointCollections = 0;
		Vector xyCollection = new Vector(numberOfPolygons * 4);

		for (double phi = 0; phi <= (theLongitude - theDegrees); phi += theDegrees) {
			for (double theta = 0; theta <= (theLatitude - theDegrees); theta += theDegrees) {
				Vector pointCollection = new Vector(4);
				Jun3dPoint currentPoint = (Jun3dPoint) xyzBlock.value_value_value_(new Double(theRadius), new Double(phi), new Double(theta));
				pointCollection.addElement(currentPoint);
				xyCollection.addElement(xyBlock.value_value_(new Double(phi), new Double(theta)));

				Jun3dPoint firstPoint = currentPoint;
				Jun3dPoint previousPoint = currentPoint;
				currentPoint = (Jun3dPoint) xyzBlock.value_value_value_(new Double(theRadius), new Double(phi), new Double(theta + theDegrees));

				if (!currentPoint.equals(previousPoint)) {
					pointCollection.addElement(currentPoint);
					xyCollection.addElement(xyBlock.value_value_(new Double(phi), new Double(theta + theDegrees)));
				}

				previousPoint = currentPoint;
				currentPoint = (Jun3dPoint) xyzBlock.value_value_value_(new Double(theRadius), new Double(phi + theDegrees), new Double(theta + theDegrees));

				if (!currentPoint.equals(previousPoint) && !currentPoint.equals(firstPoint)) {
					pointCollection.addElement(currentPoint);
					xyCollection.addElement(xyBlock.value_value_(new Double(phi + theDegrees), new Double(theta + theDegrees)));
				}

				previousPoint = currentPoint;
				currentPoint = (Jun3dPoint) xyzBlock.value_value_value_(new Double(theRadius), new Double(phi + theDegrees), new Double(theta));

				if (!currentPoint.equals(previousPoint) && !currentPoint.equals(firstPoint)) {
					pointCollection.addElement(currentPoint);
					xyCollection.addElement(xyBlock.value_value_(new Double(phi + theDegrees), new Double(theta)));
				}

				previousPoint = currentPoint;
				pointCollections[indexOfPointCollections++] = pointCollection;
			}
		}

		JunOpenGL3dCompoundObject aSphere = new JunOpenGL3dCompoundObject();
		aSphere.name_("sphere");

		for (int i = 0; i < indexOfPointCollections; i++) {
			aSphere.add_(JunOpenGL3dPolygon.Vertexes_(pointCollections[i]));
		}

		aSphere.paint_(DefaultPaint());

		Object[] result = { xyCollection, aSphere };

		return result;
	}

	/**
	 * Typical object - chunk of pie.
	 * 
	 * @param fromDegree double
	 * @param toDegree double
	 * @param byDegree double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected static final JunOpenGL3dObject ChunkOfPieFrom_to_by_(double fromDegree, double toDegree, double byDegree) {
		return ChunkOfPieFrom_to_by_radius_(fromDegree, toDegree, byDegree, 1.0d);
	}

	/**
	 * Typical object - chunk of pie.
	 * 
	 * @param fromDegree double
	 * @param toDegree double
	 * @param byDegree double
	 * @param radius double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected static final JunOpenGL3dObject ChunkOfPieFrom_to_by_radius_(double fromDegree, double toDegree, double byDegree, double radius) {
		StBlockClosure rotationBlock = new StBlockClosure() {
			public Object value_value_(Object anObject1, Object anObject2) {
				Jun3dPoint p = (Jun3dPoint) anObject1;
				double t = ((Number) anObject2).doubleValue();
				double x = (p.x() * Math.cos(t)) - (p.y() * Math.sin(t));
				double y = (p.x() * Math.sin(t)) + (p.y() * Math.cos(t));

				if (Math.abs(x) < JunGeometry.ACCURACY) {
					x = 0.0d;
				}

				if (Math.abs(y) < JunGeometry.ACCURACY) {
					y = 0.0d;
				}

				return new Jun3dPoint(x, y, 0.0d);
			}
		};

		Jun3dPoint standardPoint = new Jun3dPoint(radius, 0.0d, 0.0d);
		Vector points = new Vector();
		boolean extraFlag = false;

		if (byDegree > 0) {
			for (double theta = fromDegree; theta < toDegree; theta += byDegree) {
				points.addElement(rotationBlock.value_value_(standardPoint, new Double(JunAngle._DegreesToRadians(theta))));
				extraFlag = (theta == toDegree);
			}
		} else {
			for (double theta = fromDegree; theta > toDegree; theta += byDegree) {
				points.addElement(rotationBlock.value_value_(standardPoint, new Double(JunAngle._DegreesToRadians(theta))));
				extraFlag = (theta == toDegree);
			}
		}

		if (extraFlag == false) {
			points.addElement(rotationBlock.value_value_(standardPoint, new Double(JunAngle._DegreesToRadians(toDegree))));
		}

		points.addElement(new Jun3dPoint(0.0d, 0.0d, 0.0d));

		return JunOpenGL3dPolygon.Vertexes_(points);
	}

	/**
	 * Typical object - chunk of pie.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param fromDegree double
	 * @param toDegree double
	 * @param byDegree double
	 * @param radius double
	 * @param thickness double
	 */
	protected static final JunOpenGL3dObject ChunkOfPieFrom_to_by_radius_thickness_(double fromDegree, double toDegree, double byDegree, double radius, double thickness) {
		JunOpenGL3dPolygon pie1 = (JunOpenGL3dPolygon) ChunkOfPieFrom_to_by_radius_(fromDegree, toDegree, byDegree, radius);
		JunOpenGL3dPolygon pie2 = (JunOpenGL3dPolygon) Jun3dTransformation.Align_with_(new Jun3dPoint(0, 0, 0), new Jun3dPoint(0, 0, thickness)).applyTo_(pie1);
		JunOpenGL3dCompoundObject thickPie = JunOpenGL3dCompoundObject.With_(pie2);
		Jun3dPoint[] firstPoints = null;
		Jun3dPoint[] points = null;
		Jun3dPoint[] vertexes1 = pie1.vertexes();
		Jun3dPoint[] vertexes2 = pie2.vertexes();
		int size = vertexes1.length;
		for (int i = 0; i < size; i++) {
			if (firstPoints == null) {
				firstPoints = points = new Jun3dPoint[] { vertexes1[i], vertexes2[i] };
			} else {
				thickPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { vertexes1[i], vertexes2[i], points[1], points[0] }));
				points = new Jun3dPoint[] { vertexes1[i], vertexes2[i] };
			}
		}
		thickPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { firstPoints[0], firstPoints[1], points[1], points[0] }));
		thickPie.add_(pie1.reversed());
		thickPie.objectsDo_(new StBlockClosure() {
			public Object value_(Object each) {
				((JunOpenGL3dObject) each).paint_(null);
				return null;
			}
		});
		thickPie.paint_(DefaultPaint());

		return thickPie;
	}

	/**
	 * Typical object - sliced pie.
	 * 
	 * @param fromDegree double
	 * @param toDegree double
	 * @param byDegree double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected static final JunOpenGL3dObject SlicedPieFrom_to_by_(double fromDegree, double toDegree, double byDegree) {
		return SlicedPieFrom_to_by_radius_(fromDegree, toDegree, byDegree, 1.0d);
	}

	/**
	 * Typical object - sliced pie.
	 * 
	 * @param fromDegree double
	 * @param toDegree double
	 * @param byDegree double
	 * @param radius double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	protected static final JunOpenGL3dObject SlicedPieFrom_to_by_radius_(double fromDegree, double toDegree, double byDegree, double radius) {
		StBlockClosure rotationBlock = new StBlockClosure() {
			public Object value_value_(Object anObject1, Object anObject2) {
				Jun3dPoint p = (Jun3dPoint) anObject1;
				double t = ((Number) anObject2).doubleValue();
				double x = (p.x() * Math.cos(t)) - (p.y() * Math.sin(t));
				double y = (p.x() * Math.sin(t)) + (p.y() * Math.cos(t));

				if (Math.abs(x) < JunGeometry.ACCURACY) {
					x = 0.0d;
				}

				if (Math.abs(y) < JunGeometry.ACCURACY) {
					y = 0.0d;
				}

				return new Jun3dPoint(x, y, 0.0d);
			}
		};

		JunOpenGL3dCompoundObject aPie = new JunOpenGL3dCompoundObject();
		Jun3dPoint standardPoint = new Jun3dPoint(radius, 0.0d, 0.0d);
		Jun3dPoint lastPoint = (Jun3dPoint) rotationBlock.value_value_(standardPoint, new Double(JunAngle._DegreesToRadians(fromDegree)));
		Jun3dPoint point = null;

		if (byDegree > 0) {
			for (double theta = fromDegree + byDegree; theta < toDegree; theta += byDegree) {
				point = (Jun3dPoint) rotationBlock.value_value_(standardPoint, new Double(JunAngle._DegreesToRadians(theta)));
				aPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { lastPoint, point, new Jun3dPoint(0, 0, 0)}));
				lastPoint = (theta == toDegree) ? null : point;
			}
		} else {
			for (double theta = fromDegree + byDegree; theta > toDegree; theta += byDegree) {
				point = (Jun3dPoint) rotationBlock.value_value_(standardPoint, new Double(JunAngle._DegreesToRadians(theta)));
				aPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { lastPoint, point, new Jun3dPoint(0, 0, 0)}));
				lastPoint = (theta == toDegree) ? null : point;
			}
		}

		if (lastPoint == null) {
			point = (Jun3dPoint) rotationBlock.value_value_(standardPoint, new Double(JunAngle._DegreesToRadians(toDegree)));
			aPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { lastPoint, point, new Jun3dPoint(0, 0, 0)}));
		}

		return aPie;
	}

	/**
	 * Typical object - sliced pie.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param fromDegree double
	 * @param toDegree double
	 * @param byDegree double
	 * @param radius double
	 * @param thickness double
	 */
	protected static final JunOpenGL3dObject SlicedPieFrom_to_by_radius_thickness_(double fromDegree, double toDegree, double byDegree, double radius, double thickness) {
		StBlockClosure rotationBlock = new StBlockClosure() {
			public Object value_value_(Object anObject1, Object anObject2) {
				Jun3dPoint p = (Jun3dPoint) anObject1;
				double t = ((Number) anObject2).doubleValue();
				double x = (p.x() * Math.cos(t)) - (p.y() * Math.sin(t));
				double y = (p.x() * Math.sin(t)) + (p.y() * Math.cos(t));
				if (Math.abs(x) < JunGeometry.ACCURACY) {
					x = 0.0d;
				}
				if (Math.abs(y) < JunGeometry.ACCURACY) {
					y = 0.0d;
				}
				return new Jun2dPoint(x, y);
			}
		};

		JunOpenGL3dCompoundObject aPie = new JunOpenGL3dCompoundObject();
		Jun3dPoint standardPoint = new Jun3dPoint(radius, 0.0d, 0.0d);
		Jun2dPoint firstPoint = (Jun2dPoint) rotationBlock.value_value_(standardPoint, new Double(JunAngle._DegreesToRadians(fromDegree)));
		Jun2dPoint lastPoint = firstPoint;
		Jun2dPoint point = null;
		boolean extraDegreeExists = false;
		if (byDegree > 0) {
			for (double theta = fromDegree + byDegree; theta < toDegree; theta += byDegree) {
				point = (Jun2dPoint) rotationBlock.value_value_(standardPoint, new Double(JunAngle._DegreesToRadians(theta)));
				aPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { new Jun3dPoint(0, 0, 0), new Jun3dPoint(point, 0), new Jun3dPoint(lastPoint, 0)}));
				aPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { new Jun3dPoint(lastPoint, thickness), new Jun3dPoint(point, thickness), new Jun3dPoint(0, 0, thickness)}));
				aPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { new Jun3dPoint(lastPoint, 0), new Jun3dPoint(point, 0), new Jun3dPoint(point, thickness), new Jun3dPoint(lastPoint, thickness)}));
				lastPoint = point;
				extraDegreeExists = (theta == toDegree);
			}
		} else {
			for (double theta = fromDegree + byDegree; theta > toDegree; theta += byDegree) {
				point = (Jun2dPoint) rotationBlock.value_value_(standardPoint, new Double(JunAngle._DegreesToRadians(theta)));
				aPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { new Jun3dPoint(0, 0, 0), new Jun3dPoint(point, 0), new Jun3dPoint(lastPoint, 0)}));
				aPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { new Jun3dPoint(lastPoint, thickness), new Jun3dPoint(point, thickness), new Jun3dPoint(0, 0, thickness)}));
				aPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { new Jun3dPoint(lastPoint, 0), new Jun3dPoint(point, 0), new Jun3dPoint(point, thickness), new Jun3dPoint(lastPoint, thickness)}));
				lastPoint = point;
				extraDegreeExists = (theta == toDegree);
			}
		}

		if (extraDegreeExists == false) {
			point = (Jun2dPoint) rotationBlock.value_value_(standardPoint, new Double(JunAngle._DegreesToRadians(toDegree)));
			aPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { new Jun3dPoint(0, 0, 0), new Jun3dPoint(point, 0), new Jun3dPoint(lastPoint, 0)}));
			aPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { new Jun3dPoint(lastPoint, thickness), new Jun3dPoint(point, thickness), new Jun3dPoint(0, 0, thickness)}));
			aPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { new Jun3dPoint(lastPoint, 0), new Jun3dPoint(point, 0), new Jun3dPoint(point, thickness), new Jun3dPoint(lastPoint, thickness)}));
			lastPoint = point;
		}

		aPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { new Jun3dPoint(firstPoint, 0), new Jun3dPoint(firstPoint, thickness), new Jun3dPoint(0, 0, thickness), new Jun3dPoint(0, 0, 0)}));
		aPie.add_(JunOpenGL3dPolygon.Vertexes_(new Jun3dPoint[] { new Jun3dPoint(0, 0, 0), new Jun3dPoint(0, 0, thickness), new Jun3dPoint(lastPoint, thickness), new Jun3dPoint(lastPoint, 0)}));

		aPie.objectsDo_(new StBlockClosure() {
			public Object value_(Object each) {
				((JunOpenGL3dObject) each).paint_(null);
				return null;
			}
		});
		aPie.paint_(DefaultPaint());

		return aPie;
	}

	/**
	 * Answer the alpha number of this JunOpenGL3dObject.
	 * 
	 * @return float
	 */
	public float alpha() {
		if (!this.hasAlpha()) {
			return this.defaultAlpha();
		}

		return alpha;
	}

	/**
	 * Set the alpha number of this JunOpenGL3dObject.
	 * 
	 * @param aNumber float
	 */
	public void alpha_(float aNumber) {
		alpha = aNumber;
	}

	/**
	 * Convert the receiver to an image.
	 * 
	 * @return java.awt.Image
	 * @category converting
	 */
	public Image asImage() {
		return this.asImageExtent_(JunOpenGL3dModel.DefaultImageExtent);
	}

	/**
	 * Convert the receiver to an image with the specified extent.
	 * 
	 * @param imageExtent java.awt.Dimension
	 * @return java.awt.Image
	 * @category converting
	 */
	public Image asImageExtent_(Dimension imageExtent) {
		JunOpenGL3dModel displayModel = new JunOpenGL3dModel(this);
		return this.asImageExtent_eyePoint_sightPoint_upVector_(imageExtent, displayModel.defaultEyePoint(), displayModel.defaultSightPoint(), displayModel.defaultUpVector());
	}

	/**
	 * Convert the receiver to an image with the specified extent and the projection.
	 *  
	 * @param imageExtent java.awt.Dimension
	 * @param eyePoint sra.jun.geometry.basic.Jun3dPoint
	 * @param sightPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param upVector sra.jun.geometry.basic.Jun3dPoint
	 * @return java.awt.Image
	 * @category converting
	 */
	public Image asImageExtent_eyePoint_sightPoint_upVector_(Dimension imageExtent, Jun3dPoint eyePoint, Jun3dPoint sightPoint, Jun3dPoint upVector) {
		JunOpenGL3dModel displayModel = new JunOpenGL3dModel(this);
		displayModel.defaultEyePoint_(eyePoint);
		displayModel.defaultSightPoint_(sightPoint);
		displayModel.defaultUpVector_(upVector);
		displayModel.resetView();
		return displayModel.asImage(imageExtent);
	}

	/**
	 * Convert the receiver as a JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		return this;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return JunPoint[]
	 */
	public JunPoint[] asPointArray() {
		final Vector points = new Vector();
		this.pointsDo_(new StBlockClosure() {
			public Object value_(Object each) {
				points.addElement((JunPoint) each);

				return null;
			}
		});

		int size = points.size();
		JunPoint[] result = new JunPoint[size];

		for (int index = 0; index < size; index++) {
			result[index] = (JunPoint) points.elementAt(index);
		}

		return result;
	}

	/**
	 * Answer the bounding box of the receiver.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dBoundingBox
	 */
	public Jun3dBoundingBox boundingBox() {
		if (bounds == null) {
			bounds = this.preferredBoundingBox();
		}

		return bounds;
	}

	/**
	 * Convert the receiver's color as a JunLispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList colorToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("color"));

		Color colorValue = this.paint();
		list.add_(new Float(colorValue.getRed() / 255f));
		list.add_(new Float(colorValue.getGreen() / 255f));
		list.add_(new Float(colorValue.getBlue() / 255f));

		if (this.hasAlpha()) {
			list.add_(new Float(this.alpha()));
		}

		return list;
	}

	/**
	 * Answer the default alpha number.
	 * 
	 * @return float
	 */
	public float defaultAlpha() {
		return 1.0f;
	}

	/**
	 * Answer the default line width.
	 * 
	 * @return float
	 */
	public float defaultLineWidth() {
		return 1.0f;
	}

	/**
	 * Answer the default paint.
	 * 
	 * @return java.awt.Color
	 */
	public Color defaultPaint() {
		return Color.white;
	}

	/**
	 * Answer the default point.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint defaultPoint() {
		return new Jun3dPoint(0, 0, 0);
	}

	/**
	 * Answer the default size.
	 * 
	 * @return float
	 */
	public float defaultSize() {
		return 1.0f;
	}

	/**
	 * Flush my bounds.
	 */
	public void flushBounds() {
		bounds = null;
	}

	/**
	 * Enumerate every geometries and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return DOCUMENT ME!
	 */
	public Object geometriesDo_(StBlockClosure aBlock) {
		throw SmalltalkException.SubclassResponsibility();
	}

	/**
	 * Answer true if the alpha is specified, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasAlpha() {
		return !Float.isNaN(alpha);
	}

	/**
	 * Answer true if the color is specified, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasColor() {
		return (this.hasPaint() || this.hasAlpha());
	}

	/**
	 * Answer true if the receiver has a name, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasName() {
		return (name != null);
	}

	/**
	 * Answer true if the paint is specified, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasPaint() {
		return (paint != null);
	}

	/**
	 * Answer true if this JunOpenGL3dObject has a stipple, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasStipple() {
		return false;
	}

	/**
	 * Answer true if this JunOpenGL3dObject has a texture, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasTexture() {
		return false;
	}

	/**
	 * Answer true if I have a transparency effect, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasTransparency() {
		return this.hasAlpha() && (this.alpha() < 1.0);
	}

	/**
	 * Answer my height.
	 *
	 * @return double
	 */
	public double height() {
		return this.boundingBox().height();
	}

	/**
	 * Initialize the receiver.
	 */
	public void initialize() {
		name = null;
		paint = null;
		alpha = Float.NaN;
		bounds = null;
	}

	/**
	 * Answer true if the receiver is a compound object, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean isCompound() {
		return false;
	}

	/**
	 * Answer true if the receiver is a kind of JunOpenGL3dPrimitiveObject,
	 * otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean isPrimitive() {
		return false;
	}

	/**
	 * Answer true if the receiver is a kind of JunOpenGL3dVertexesObject, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean isVertexesObject() {
		return false;
	}

	/**
	 * Answer the StSymbol which represents the kind of the receiver.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public abstract StSymbol kindName();

	/**
	 * DOCUMENT ME!
	 * 
	 * @return String
	 */
	public String legalName() {
		StringWriter writer = new StringWriter();
		String name = this.name();

		if (Character.isLetter(name.charAt(0)) == false) {
			writer.write("id");
		}

		for (int index = 0; index < name.length(); index++) {
			char character = name.charAt(index);

			if (Character.isLetterOrDigit(character)) {
				writer.write(character);
			} else {
				writer.write("_");
			}
		}

		return writer.toString();
	}

	/**
	 * Create an instance of JunLispCons.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons lispCons() {
		return (JunLispCons) JunLispList.Cell();
	}

	/**
	 * Create an instance of JunLispNil.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispNil
	 */
	public JunLispNil lispNil() {
		return (JunLispNil) JunLispList.NullList();
	}

	/**
	 * Answer the name of this JunOpenGL3dObject.
	 * 
	 * @return java.lang.String
	 */
	public String name() {
		if (name == null) {
			return this.defaultName();
		}

		return name;
	}

	/**
	 * Set name of this JunOpenGL3dObject.
	 * 
	 * @param aString java.lang.String
	 */
	public void name_(String aString) {
		name = aString;
	}

	/**
	 * Convert the receiver's name as a JunLispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList nameToLispList() {
		if (!this.hasName()) {
			return this.lispNil();
		}

		JunLispCons list = this.lispCons();
		list.head_($("name"));
		list.tail_(this.name());

		return list;
	}

	/**
	 * Insert the method's description here. Creation date: (01/07/03 13:02:43)
	 * 
	 * @param param sra.jun.geometry.basic.Jun3dPoint
	 */
	public void newMethod(Jun3dPoint param) {
	}

	/**
	 * Answer the number of polygons.
	 * 
	 * @return int
	 */
	public abstract int numberOfPolygons();

	/**
	 * Answer the number of primitives.
	 * 
	 * @return int
	 */
	public abstract int numberOfPrimitives();

	/**
	 * Enumerate every objects and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object objectsDo_(StBlockClosure aBlock) {
		throw SmalltalkException.SubclassResponsibility();
	}

	/**
	 * Answer the color of this JunOpenGL3dObject.
	 * 
	 * @return java.awt.Color
	 */
	public Color paint() {
		if (paint == null) {
			return this.defaultPaint();
		}

		return paint;
	}

	/**
	 * Set the color of this JunOpenGL3dObject.
	 * 
	 * @param aColor java.awt.Color
	 */
	public void paint_(Color aColor) {
		paint = aColor;
	}

	/**
	 * Enumerate every points and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public abstract void pointsDo_(StBlockClosure aBlock);

	/**
	 * Enumerate all of the primitive objects and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public Object primitivesDo_(StBlockClosure aBlock) {
		throw new SmalltalkException("Overwrite this method if the isCompound() returns true.");
	}

	/**
	 * Render the OpenGL object on a rendering context.
	 * 
	 * @param aRenderingContext sra.jun.opengl.support.JunRenderingContext
	 */
	public void renderOn_(JunOpenGLRenderingContext aRenderingContext) {
		throw SmalltalkException.SubclassResponsibility();
	}

	/**
	 * Render the OpenGL object on a rendering context.
	 * 
	 * @param aRenderingContext sra.jun.vrml.support.JunVrmlRenderingContext
	 */
	public void renderOn_(JunVrmlRenderingContext aRenderingContext) {
		aRenderingContext.displayObject_(this);
	}

	/**
	 * Anser the reversed JunOpenGL3dObject of the receiver.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject reversed() {
		throw SmalltalkException.SubclassResponsibility();
	}

	/**
	 * Rotate the receiver by the specified angle.
	 * 
	 * @param anAngle sra.jun.geometry.basic.JunAngle
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject rotatedBy_(JunAngle anAngle) {
		return this.transform_(Jun3dTransformation.Rotate_(anAngle));
	}

	/**
	 * Write the receiver's list representation on the Writer.
	 * 
	 * @param aWriter java.io.Writer
	 * 
	 * @throws IOException DOCUMENT ME!
	 */
	public void saveOn_(Writer aWriter) throws IOException {
		JunLispList aList = this.toLispList();
		aList.saveOn_(aWriter);
	}

	/**
	 * Answer the copy of the receiver scaled by a specified size.
	 * 
	 * @param aNumber double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject scaledBy_(double aNumber) {
		return this.transform_(Jun3dTransformation.Scale_(aNumber));
	}

	/**
	 * Answer the copy of the receiver scaled by a specified size.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject scaledBy_(Jun3dPoint aPoint) {
		return this.transform_(Jun3dTransformation.Scale_(aPoint));
	}

	/**
	 * Utility to show the receiver using JunOpenGLDisplayModel.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public JunOpenGLDisplayModel show() {
		return JunOpenGLDisplayModel.Show_(this);
	}

	/**
	 * Popup a window which displays the Lisp List of the receiver.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList showLispList() {
		return this.showLispList_(this.toLispList());
	}

	/**
	 * Popup a window which displays the Lisp List.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public JunLispList showLispList_(JunLispList aList) {
		try {
			return (JunLispList) _PerformWith(this.getClass(), "ShowLispList_", aList);
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Answer the stipple of the receiver. Does not have a stipple as a
	 * default.
	 * 
	 * @return sra.jun.opengl.texture.JunOpenGLStipple
	 */
	public JunOpenGLStipple stipple() {
		return null;
	}

	/**
	 * Set the stipple of the receiver. Do nothing as a default.
	 * 
	 * @param aStipple sra.jun.opengl.texture.JunOpenGLStipple
	 */
	public void stipple_(JunOpenGLStipple aStipple) {
	}

	/**
	 * Answer the texture of the receiver. Does not have a texture as a
	 * default.
	 * 
	 * @return sra.jun.opengl.texture.JunOpenGLTexture
	 */
	public JunOpenGLTexture texture() {
		return null;
	}

	/**
	 * Set the texture of the receiver. Do nothing as a default.
	 * 
	 * @param aTexture sra.jun.opengl.texture.JunOpenGLTexture
	 */
	public void texture_(JunOpenGLTexture aTexture) {
	}

	/**
	 * DOCUMENT ME!
	 */
	public void thisIsUnderconstruction() {
		System.out.print("This is under construction");
	}

	/**
	 * Convert the receiver as a lisp list.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public abstract JunLispList toLispList();

	/**
	 * Answer the new JunOpen3dObject transformed with aTransformation.
	 * 
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject transform_(Jun3dTransformation aTransformation) {
		throw SmalltalkException.SubclassResponsibility();
	}

	/**
	 * Answer the transformation of the receiver.
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public Jun3dTransformation transformation() {
		return Jun3dTransformation.Unity();
	}

	/**
	 * Answer the copy of the receiver translated by a specified size.
	 * 
	 * @param aNumber double
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject translatedBy_(double aNumber) {
		return this.transform_(Jun3dTransformation.Translate_(aNumber));
	}

	/**
	 * Answer the copy of the receiver translated by a specified size.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject translatedBy_(Jun3dPoint aPoint) {
		return this.transform_(Jun3dTransformation.Translate_(aPoint));
	}

	/**
	 * Write the VRML1.0 string of the receiver on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 */
	public abstract void vrml10On_(Writer aWriter);

	/**
	 * Answer the VRML1.0 string of the receiver.
	 * 
	 * @return String
	 */
	public String vrml10String() {
		StringWriter aWriter = new StringWriter();

		try {
			this.vrml10On_(aWriter);
			aWriter.close();
		} catch (Exception e) {
			new SmalltalkException(e);
		}

		return aWriter.toString();
	}

	/**
	 * Write the VRML2.0 string of the receiver on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 */
	public abstract void vrml20On_(Writer aWriter);

	/**
	 * Answer the VRML2.0 string of the receiver.
	 * 
	 * @return String
	 */
	public String vrml20String() {
		StringWriter aWriter = new StringWriter();

		try {
			this.vrml20On_(aWriter);
			aWriter.close();
		} catch (Exception e) {
			new SmalltalkException(e);
		}

		return aWriter.toString();
	}

	/**
	 * Write the VRML string of the receiver on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 */
	public void vrmlOn_(Writer aWriter) {
		this.vrml20On_(aWriter);
	}

	/**
	 * Answer the VRML string of the receiver.
	 * 
	 * @return String
	 */
	public String vrmlString() {
		return this.vrml20String();
	}

	/**
	 * Answer my width.
	 *
	 * @return double
	 */
	public double width() {
		return this.boundingBox().width();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return String
	 */
	protected String defaultName() {
		return UniqueId();
	}

	/**
	 * Answer the preferred bounding box of the receiver.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dBoundingBox
	 */
	protected Jun3dBoundingBox preferredBoundingBox() {
		final StValueHolder minX = new StValueHolder(Double.POSITIVE_INFINITY);
		final StValueHolder minY = new StValueHolder(Double.POSITIVE_INFINITY);
		final StValueHolder minZ = new StValueHolder(Double.POSITIVE_INFINITY);
		final StValueHolder maxX = new StValueHolder(Double.NEGATIVE_INFINITY);
		final StValueHolder maxY = new StValueHolder(Double.NEGATIVE_INFINITY);
		final StValueHolder maxZ = new StValueHolder(Double.NEGATIVE_INFINITY);
		this.pointsDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				Jun3dPoint aPoint = (Jun3dPoint) anObject;
				minX.value_(Math.min(minX._doubleValue(), aPoint.x()));
				minY.value_(Math.min(minY._doubleValue(), aPoint.y()));
				minZ.value_(Math.min(minZ._doubleValue(), aPoint.z()));
				maxX.value_(Math.max(maxX._doubleValue(), aPoint.x()));
				maxY.value_(Math.max(maxY._doubleValue(), aPoint.y()));
				maxZ.value_(Math.max(maxZ._doubleValue(), aPoint.z()));

				return null;
			}
		});

		if (minX._doubleValue() == Double.POSITIVE_INFINITY) {
			minX.value_(0);
		}

		if (minY._doubleValue() == Double.POSITIVE_INFINITY) {
			minY.value_(0);
		}

		if (minZ._doubleValue() == Double.POSITIVE_INFINITY) {
			minZ.value_(0);
		}

		if (maxX._doubleValue() == Double.NEGATIVE_INFINITY) {
			maxX.value_(0);
		}

		if (maxY._doubleValue() == Double.NEGATIVE_INFINITY) {
			maxY.value_(0);
		}

		if (maxZ._doubleValue() == Double.NEGATIVE_INFINITY) {
			maxZ.value_(0);
		}

		Jun3dPoint originPoint = new Jun3dPoint(minX._doubleValue(), minY._doubleValue(), minZ._doubleValue());
		Jun3dPoint cornerPoint = new Jun3dPoint(maxX._doubleValue(), maxY._doubleValue(), maxZ._doubleValue());

		return Jun3dBoundingBox.Origin_corner_(originPoint, cornerPoint);
	}

	/**
	 * Write my appearance as VRML1.0 on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	protected void vrml10ColorOn_(Writer aWriter) {
		if (!this.hasColor()) {
			return;
		}

		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
			bw.write("\t");
			bw.write("Material {");
			bw.newLine();
			bw.write("\t\t");
			bw.write("diffuseColor ");

			Color color = this.paint();
			bw.write(String.valueOf(color.getRed() / 255f));
			bw.write(" ");
			bw.write(String.valueOf(color.getGreen() / 255f));
			bw.write(" ");
			bw.write(String.valueOf(color.getBlue() / 255f));
			bw.newLine();
			bw.write("\t");
			bw.write("} #Material");
			bw.newLine();
			bw.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Write my appearance as VRML2.0 on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	protected void vrml20AppearanceOn_(Writer aWriter) {
		if ((this.hasColor() == false) && (this.hasTexture() == false)) {
			return;
		}

		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
			bw.write("\t");
			bw.write("appearance Appearance {");
			bw.newLine();

			if (this.hasColor()) {
				bw.write("\t\t");
				bw.write("material Material {");
				bw.newLine();
				bw.write("\t\t\t");
				bw.write("diffuseColor ");

				Color color = this.paint();
				bw.write(String.valueOf(color.getRed() / 255f));
				bw.write(" ");
				bw.write(String.valueOf(color.getGreen() / 255f));
				bw.write(" ");
				bw.write(String.valueOf(color.getBlue() / 255f));
				bw.newLine();
				bw.write("\t\t");
				bw.write("} #Material");
				bw.newLine();
			}

			if (this.hasTexture()) {
				bw.write("\t\t");
				bw.write("texture PixelTexture {");
				bw.newLine();
				bw.write("\t\t\t");
				bw.write("image ");
				this.texture().vrmlSFImageOn_(bw);
				bw.newLine();
				bw.write("\t\t");
				bw.write("} #Texture");
				bw.newLine();
			}

			bw.write("\t");
			bw.write("} #Appearance");
			bw.newLine();
			bw.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Write my appearance as VRML2.0 on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	protected void vrml20ColorOn_(Writer aWriter) {
		if (!this.hasColor()) {
			return;
		}

		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
			bw.write("\t");
			bw.write("appearance Appearance {");
			bw.newLine();
			bw.write("\t\t");
			bw.write("material Material {");
			bw.newLine();
			bw.write("\t\t\t");
			bw.write("diffuseColor ");

			Color color = this.paint();
			bw.write(String.valueOf(color.getRed() / 255f));
			bw.write(" ");
			bw.write(String.valueOf(color.getGreen() / 255f));
			bw.write(" ");
			bw.write(String.valueOf(color.getBlue() / 255f));
			bw.newLine();
			bw.write("\t\t");
			bw.write("} #Material");
			bw.newLine();
			bw.write("\t");
			bw.write("} #Appearance");
			bw.newLine();
			bw.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

}
