package sra.jun.opengl.objects;

import java.awt.Color;
import java.util.Vector;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.geometry.surfaces.JunPolygon;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.support.JunOpenGLRenderingContext;
import sra.jun.opengl.texture.*;
import sra.smalltalk.*;

/**
 * JunOpenGL3dPolygon class
 * 
 *  @author    nisinaka
 *  @created   1998/10/13 (by nisinaka)
 *  @updated   1999/08/04 (by nisinaka)
 *  @updated   2001/11/20 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun517 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dPolygon.java,v 6.0 2003/10/28 10:52:19 nisinaka Exp $
 */
public class JunOpenGL3dPolygon extends JunOpenGL3dVertexesObject {

	/** A stipple of a JunOpenGL3dPolygon. */
	protected JunOpenGLStipple stipple = null;

	/** A texture of a JunOpenGL3dPolygon. */
	protected JunOpenGLTexture texture = null;

	/**
	 * Create a new JunOpenGL3dPolygon and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunOpenGL3dPolygon() {
		super();
	}

	/**
	 * Create a new JunOpenGL3dPolygon and initialize it with an array of 3d points.
	 * 
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @category Instance creation
	 */
	public JunOpenGL3dPolygon(Jun3dPoint[] anArrayOf3dPoints) {
		super(anArrayOf3dPoints);
	}

	/**
	 * Create a new JunOpenGL3dPolygon and initialize it with an array of 3d points and a Color.
	 * 
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @param aColor java.awt.Color
	 * @category Instance creation
	 */
	public JunOpenGL3dPolygon(Jun3dPoint[] anArrayOf3dPoints, Color aColor) {
		super(anArrayOf3dPoints, aColor);
	}

	/**
	 * Create a new JunOpenGL3dPolygon and initialize it with an array of 3d points.
	 * 
	 * @param aCollectionOf3dPoints java.util.Vector
	 * @category Instance creation
	 */
	public JunOpenGL3dPolygon(Vector aCollectionOf3dPoints) {
		super(aCollectionOf3dPoints);
	}

	/**
	 * Create a new JunOpenGL3dPolygon and initialize it with an array of 3d points.
	 * 
	 * @param aCollectionOf3dPoints java.util.Vector
	 * @param aColor java.awt.Color
	 * @category Instance creation
	 */
	public JunOpenGL3dPolygon(Vector aCollectionOf3dPoints, Color aColor) {
		super(aCollectionOf3dPoints, aColor);
	}

	/**
	 * Create a new instance of JunOpenGL3dPolygon and initialize it with an array of 3d points.
	 * 
	 * @param anArrayOf3dPoints java.util.Vector
	 * @return sra.jun.opengl.objects.JunOpenGL3dPolygon
	 * @category Instance creation
	 */
	public static final JunOpenGL3dPolygon Vertexes_(Jun3dPoint[] anArrayOf3dPoints) {
		//return (JunOpenGL3dPolygon) Vertexes_(JunOpenGL3dPolygon.class, anArrayOf3dPoints);
		return new JunOpenGL3dPolygon(anArrayOf3dPoints);
	}

	/**
	 * Create a new instance of JunOpenGL3dPolygon and initialize it with a collection of 3d points.
	 * 
	 * @param aCollectionOf3dPoints java.util.Vector
	 * @return sra.jun.opengl.objects.JunOpenGL3dPolygon
	 * @category Instance creation
	 */
	public static final JunOpenGL3dPolygon Vertexes_(Vector aCollectionOf3dPoints) {
		//return (JunOpenGL3dPolygon) Vertexes_(JunOpenGL3dPolygon.class, aCollectionOf3dPoints);
		return new JunOpenGL3dPolygon(aCollectionOf3dPoints);
	}

	/**
	 * Create a new JunOpenGL3dPolygon and initialize it with an array of 3d points and a Color.
	 * 
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @param aColor java.awt.Color
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Instance creation
	 */
	public static final JunOpenGL3dObject Vertexes_paint_(Jun3dPoint[] anArrayOf3dPoints, Color aColor) {
		//return JunOpenGL3dVertexesObject.Vertexes_paint_(JunOpenGL3dPolygon.class, anArrayOf3dPoints, aColor);
		return new JunOpenGL3dPolygon(anArrayOf3dPoints, aColor);
	}

	/**
	 * Create a JunOpenGL3dPolygon from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Lisp support
	 */
	public static JunOpenGL3dObject FromLispList_(JunLispCons aList) {
		JunOpenGL3dObject object = new JunOpenGL3dPolygon();
		object = NameFromLispList_for_(aList, object);
		object = ColorFromLispList_for_(aList, object);
		object = StippleFromLispList_for_(aList, object);
		object = TextureFromLispList_for_(aList, object);
		object = TransformationFromLispList_for_(aList, object);
		object = VertexesFromLispList_for_(aList, (JunOpenGL3dVertexesObject) object);
		object = ColorsFromLispList_for_(aList, (JunOpenGL3dVertexesObject) object);
		object = NormalVectorsFromLispList_for_(aList, (JunOpenGL3dVertexesObject) object);
		return object;
	}

	/**
	 * Get a stipple from the lisp list and set it as the stipple of the JunOpenGL3dObject.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Lisp support
	 */
	public static JunOpenGL3dObject StippleFromLispList_for_(JunLispCons aList, JunOpenGL3dObject a3dObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("stipple")));
			}
		}, new StBlockClosure());
		if (list == null) {
			return a3dObject;
		}

		a3dObject.stipple_(JunOpenGLStipple.Image_((StImage) list.tail()));
		return a3dObject;
	}

	/**
	 * Get a texture from the lisp list and set it as the texture of the JunOpenGL3dPolygon.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Lisp support
	 */
	public static JunOpenGL3dObject TextureFromLispList_for_(JunLispCons aList, JunOpenGL3dObject a3dObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("texture")));
			}
		}, new StBlockClosure());
		if (list == null) {
			return a3dObject;
		}

		a3dObject.texture_(JunOpenGLTexture.FromLispList_((JunLispList) list.tail()));
		return a3dObject;
	}

	/**
	 * Initialize the JunOpenGL3dPolygon.
	 * 
	 * @category initialize-release
	 */
	public void initialize() {
		super.initialize();
		stipple = null;
		texture = null;
	}

	/**
	 * Set a stipple with the specified scale.
	 * 
	 * @param halftoneScale float
	 * @category accessing
	 */
	public void halftone_(float halftoneScale) {
		this.stipple_(JunOpenGLStipple.Halftone_(halftoneScale));
	}

	/**
	 * Answer the stipple of the receiver..
	 * 
	 * @return sra.jun.opengl.texture.JunOpenGLStipple
	 * @category accessing
	 */
	public JunOpenGLStipple stipple() {
		return stipple;
	}

	/**
	 * Set a stipple of the receiver..
	 * 
	 * @param aStipple sra.jun.opengl.texture.JunOpenGLStipple
	 * @category accessing
	 */
	public void stipple_(JunOpenGLStipple aStipple) {
		stipple = aStipple;
	}

	/**
	 * Answer the texture of the receiver.
	 * 
	 * @return sra.jun.opengl.texture.JunOpenGLTexture
	 * @category accessing
	 */
	public JunOpenGLTexture texture() {
		return texture;
	}

	/**
	 * Set a texture of the receiver.
	 * 
	 * @param aTexture sra.jun.opengl.texture.JunOpenGLTexture
	 * @category accessing
	 */
	public void texture_(JunOpenGLTexture aTexture) {
		texture = aTexture;

		if (aTexture != null) {
			aTexture.object_(this);
		}
	}

	/**
	 * Set the vertexes with an array of 3d points.
	 * 
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @category accessing
	 */
	public void vertexes_(Jun3dPoint[] anArrayOf3dPoints) {
		if (anArrayOf3dPoints.length < 3) {
			throw SmalltalkException.Error("few vertexes");
		}

		super.vertexes_(anArrayOf3dPoints);
	}

	/**
	 * Answer true if this JunOpenGL3dObject has a stipple, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean hasStipple() {
		return stipple != null;
	}

	/**
	 * Answer true if this JunOpenGL3dObject has a texture, otherwise false.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean hasTexture() {
		return texture != null;
	}

	/**
	 * Render the OpenGL object on a rendering context.
	 * 
	 * @param aRenderingContext sra.jun.opengl.support.JunOpenGLRenderingContext
	 * @category rendering
	 */
	public void renderOn_(JunOpenGLRenderingContext aRenderingContext) {
		int oldTextureCoordinateIndex = Integer.MIN_VALUE;
		if (this.hasPaint()) {
			aRenderingContext.paint_(this.paint());
		}
		if (this.hasAlpha()) {
			aRenderingContext.alpha_(this.alpha());
		}
		if (this.hasStipple()) {
			this.stipple().enableStippleOn_(aRenderingContext);
		}
		if (this.hasTexture()) {
			oldTextureCoordinateIndex = aRenderingContext.textureCoordinateIndex();
			aRenderingContext.textureCoordinateIndex_(0);
			this.texture().enableTextureOn_(aRenderingContext);
		}
		if (this.hasTransparency()) {
			aRenderingContext.enableCullFace();
			aRenderingContext.disableDepthMask();
			aRenderingContext.enableBlending();
		}

		if (this.hasColors()) {
			aRenderingContext.displayPolygon_colors_normalVectors_(this.vertexes(), this.colors(), this.normalVectors());
		} else {
			aRenderingContext.displayPolygon_normalVectors_(this.vertexes(), this.normalVectors());
		}

		if (this.hasTransparency()) {
			aRenderingContext.disableCullFace();
			aRenderingContext.enableDepthMask();
			aRenderingContext.disableBlending();
		}
		if (this.hasTexture()) {
			this.texture().disableTextureOn_(aRenderingContext);
			aRenderingContext.textureCoordinateIndex_(oldTextureCoordinateIndex);
		}
		if (this.hasStipple()) {
			this.stipple().disableStippleOn_(aRenderingContext);
		}
	}

	/**
	 * Enumerate every geometries and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public Object geometriesDo_(StBlockClosure aBlock) {
		return aBlock.value_(JunPolygon.Points_(vertexes));
	}

	/**
	 * Enumerate every polygons and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category enumerating
	 */
	public Object polygonsDo_(StBlockClosure aBlock) {
		return aBlock.value_(this);
	}

	/**
	 * Answer the StSymbol which represents the kind of the receiver.
	 * 
	 * @return sra.smalltalk.StSymbol
	 * @category lisp support
	 */
	public StSymbol kindName() {
		return $("Polygon");
	}

	/**
	 * Convert the receiver's stipple as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList stippleToLispList() {
		if (!this.hasStipple()) {
			return this.lispNil();
		}

		JunLispCons list = this.lispCons();
		list.head_($("stipple"));
		list.tail_(this.stipple().image());
		return list;
	}

	/**
	 * Convert the receiver's texture as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList textureToLispList() {
		if (!this.hasTexture()) {
			return this.lispNil();
		}

		JunLispCons list = this.lispCons();
		list.head_($("texture"));
		list.tail_(this.texture().toLispList());
		return list;
	}

	/**
	 * Answer the name of my indexed set node.
	 * 
	 * @return java.lang.String
	 * @category vrml support
	 */
	protected String vrml10IndexedSetName() {
		return "IndexedFaceSet";
	}

	/**
	 * Answer the name of my indexed set node.
	 * 
	 * @return java.lang.String
	 * @category vrml support
	 */
	protected String vrml20IndexedSetName() {
		return "IndexedFaceSet";
	}

	/**
	 * Answer the number of polygons.
	 * 
	 * @return int
	 * @category utilities
	 */
	public int numberOfPolygons() {
		return 1;
	}

}
