package sra.jun.opengl.objects;

import java.awt.Color;
import java.util.Vector;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.geometry.curves.Jun3dLine;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.support.JunOpenGLRenderingContext;
import sra.smalltalk.*;

/**
 * JunOpenGL3dPolyline class
 * 
 *  @author    nisinaka
 *  @created   1998/10/21 (by nisinaka)
 *  @updated   1999/08/04 (by nisinaka)
 *  @updated   2001/11/20 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dPolyline.java,v 6.0 2003/10/28 10:52:19 nisinaka Exp $
 */
public class JunOpenGL3dPolyline extends JunOpenGL3dVertexesObject {
	/** The line width. */
	protected float lineWidth;

	/** The stipple factor. */
	protected int stippleFactor = 1;

	/** The stipple pattern. */
	protected short stipplePattern;

	/**
	 * Create a new JunOpenGL3dPolyline and initialize it.
	 */
	public JunOpenGL3dPolyline() {
		super();
	}

	/**
	 * Create a new JunOpenGL3dPolyline and initialize it with an array of 3d
	 * points.
	 * 
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 */
	public JunOpenGL3dPolyline(Jun3dPoint[] anArrayOf3dPoints) {
		super(anArrayOf3dPoints);
	}

	/**
	 * Create a new JunOpenGL3dPolyline and initialize it with an array of 3d
	 * points and a Color.
	 * 
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @param aColor java.awt.Color
	 */
	public JunOpenGL3dPolyline(Jun3dPoint[] anArrayOf3dPoints, Color aColor) {
		super(anArrayOf3dPoints, aColor);
	}

	/**
	 * Create a new JunOpenGL3dPolyline and initialize it with an array of 3d
	 * points.
	 * 
	 * @param aCollectionOf3dPoints java.util.Vector
	 */
	public JunOpenGL3dPolyline(Vector aCollectionOf3dPoints) {
		super(aCollectionOf3dPoints);
	}

	/**
	 * Create a new JunOpenGL3dPolyline and initialize it with an array of 3d
	 * points.
	 * 
	 * @param aCollectionOf3dPoints java.util.Vector
	 * @param aColor java.awt.Color
	 */
	public JunOpenGL3dPolyline(Vector aCollectionOf3dPoints, Color aColor) {
		super(aCollectionOf3dPoints, aColor);
	}

	/**
	 * Create a JunOpenGL3dPolyline from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject FromLispList_(JunLispCons aList) {
		JunOpenGL3dObject object = new JunOpenGL3dPolyline();
		object = NameFromLispList_for_(aList, object);
		object = ColorFromLispList_for_(aList, object);
		object = LineWidthFromLispList_for_(aList, object);
		object = StippleFromLispList_for_(aList, object);
		object = VertexesFromLispList_for_(aList, (JunOpenGL3dVertexesObject) object);
		object = TransformationFromLispList_for_(aList, object);

		return object;
	}

	/**
	 * Get a line width from the lisp list and set it as the line width of the
	 * JunOpenGL3dObject.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject LineWidthFromLispList_for_(JunLispCons aList, JunOpenGL3dObject a3dObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("width")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return a3dObject;
		}

		((JunOpenGL3dPolyline) a3dObject).lineWidth_(((Number) list.tail()).intValue());

		return a3dObject;
	}

	/**
	 * Get a stipple from the lisp list and set it as the stipple of the
	 * JunOpenGL3dObject.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject StippleFromLispList_for_(JunLispCons aList, JunOpenGL3dObject a3dObject) {
		JunOpenGL3dPolyline the3dObject = (JunOpenGL3dPolyline) a3dObject;
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("stipple")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return a3dObject;
		}

		int factor = ((Number) ((JunLispCons) list.tail()).nth_(1)).intValue();
		short pattern = ((Number) ((JunLispCons) list.tail()).nth_(2)).shortValue();
		the3dObject.stippleFactor_(factor);
		the3dObject.stipplePattern_(pattern);

		return a3dObject;
	}

	/**
	 * Create a new JunOpenGL3dPolyline and initialize it with an array of 3d
	 * points.
	 * 
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject Vertexes_(Jun3dPoint[] anArrayOf3dPoints) {
		//return JunOpenGL3dVertexesObject.Vertexes_(JunOpenGL3dPolyline.class, anArrayOf3dPoints);
		return new JunOpenGL3dPolyline(anArrayOf3dPoints);
	}

	/**
	 * Create a new JunOpenGL3dPolyline and initialize it with an array of 3d
	 * points.
	 * 
	 * @param aCollectionOf3dPoints java.util.Vector
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject Vertexes_(Vector aCollectionOf3dPoints) {
		//return JunOpenGL3dVertexesObject.Vertexes_(JunOpenGL3dPolyline.class, aCollectionOf3dPoints);
		return new JunOpenGL3dPolyline(aCollectionOf3dPoints);
	}

	/**
	 * Create a new JunOpenGL3dPolyline and initialize it with an array of 3d
	 * points and a Color.
	 * 
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @param aColor java.awt.Color
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject Vertexes_paint_(Jun3dPoint[] anArrayOf3dPoints, Color aColor) {
		//return JunOpenGL3dVertexesObject.Vertexes_paint_(JunOpenGL3dPolyline.class, anArrayOf3dPoints, aColor);
		return new JunOpenGL3dPolyline(anArrayOf3dPoints, aColor);
	}

	/**
	 * Enumerate every geometries and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object geometriesDo_(StBlockClosure aBlock) {
		for (int i = 0; i < (vertexes.length - 1); i++) {
			Object result = aBlock.value_(new Jun3dLine(vertexes[i], vertexes[i + 1]));

			if (result != null) {
				return result;
			}
		}

		return null;
	}

	/**
	 * Answer true if this JunOpenGL3dObject has a stipple, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasStipple() {
		return stipplePattern != Short.MIN_VALUE;
	}

	/**
	 * Initialize the receiver.
	 */
	public void initialize() {
		super.initialize();
		lineWidth = this.defaultLineWidth();
		stippleFactor = 1;
		stipplePattern = Short.MIN_VALUE;
	}

	/**
	 * Answer true if the receiver is looped line segments.
	 * 
	 * @return boolean
	 */
	public boolean isLooped() {
		return false;
	}

	/**
	 * Answer the StSymbol which represents the kind of the receiver.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public StSymbol kindName() {
		return $("Polyline");
	}

	/**
	 * Answer the line width.
	 * 
	 * @return float
	 */
	public float lineWidth() {
		return lineWidth;
	}

	/**
	 * Set a number as the line width.
	 * 
	 * @param aNumber float
	 */
	public void lineWidth_(float aNumber) {
		lineWidth = aNumber;
	}

	/**
	 * Convert the receiver's stipple as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList lineWidthToLispList() {
		if (this.lineWidth <= 1) {
			return this.lispNil();
		}

		JunLispCons list = this.lispCons();
		list.head_($("width"));
		list.tail_(new Float(this.lineWidth()));

		return list;
	}

	/**
	 * Answer the number of polygons.
	 * 
	 * @return int
	 */
	public int numberOfPolygons() {
		return 0;
	}

	/**
	 * Render the OpenGL object on a rendering context.
	 * 
	 * @param aRenderingContext
	 *        sra.jun.opengl.support.JunOpenGLRenderingContext
	 */
	public void renderOn_(JunOpenGLRenderingContext aRenderingContext) {
		if (this.hasPaint()) {
			aRenderingContext.paint_(this.paint());
		}

		if (this.hasAlpha()) {
			aRenderingContext.alpha_(this.alpha());
		}

		aRenderingContext.lineWidth_(this.lineWidth());

		if (this.hasStipple()) {
			aRenderingContext.lineStippleFactor_pattern_(this.stippleFactor(), this.stipplePattern());
			aRenderingContext.enableLineStipple();
		}

		aRenderingContext.displayLines_(this.vertexes());

		if (this.hasStipple()) {
			aRenderingContext.disableLineStipple();
		}
	}

	/**
	 * Answer the stipple factor.
	 * 
	 * @return int
	 */
	public int stippleFactor() {
		return stippleFactor;
	}

	/**
	 * Set a number as the stipple factor.
	 * 
	 * @param aNumber int
	 */
	public void stippleFactor_(int aNumber) {
		stippleFactor = aNumber;
	}

	/**
	 * Answer the stipple pattern
	 * 
	 * @return short
	 */
	public short stipplePattern() {
		return stipplePattern;
	}

	/**
	 * Set a number as the stipple pattern.
	 * 
	 * @param aNumber short
	 */
	public void stipplePattern_(short aNumber) {
		stipplePattern = aNumber;
	}

	/**
	 * Convert the receiver's stipple as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList stippleToLispList() {
		if (!this.hasStipple()) {
			return this.lispNil();
		}

		JunLispCons list = this.lispCons();
		list.head_($("stipple"));
		list.add_(new Integer(this.stippleFactor()));
		list.add_(new Short(this.stipplePattern()));

		return list;
	}

	/**
	 * Convert the receiver's texture as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList textureToLispList() {
		throw SmalltalkException.ShouldNotImplement();
	}

	/**
	 * Convert the receiver as a JunLispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList toLispList() {
		JunLispCons list = this.lispCons();
		list.head_(this.kindName());

		if (!this.hasName() && !this.hasColor() && !this.hasStipple() && (this.lineWidth <= 1)) {
			for (int i = 0; i < vertexes.length; i++) {
				list.add_(vertexes[i]);
			}

			return list;
		}

		if (this.hasName()) {
			list.add_(this.nameToLispList());
		}

		if (this.hasColor()) {
			list.add_(this.colorToLispList());
		}

		if (this.lineWidth() > 1) {
			list.add_(this.lineWidthToLispList());
		}

		if (this.hasStipple()) {
			list.add_(this.stippleToLispList());
		}

		if (this.hasTexture()) {
			list.add_(this.textureToLispList());
		}

		list.add_(this.vertexesToLispList());

		return list;
	}

	/**
	 * Answer the name of my indexed set node.
	 * 
	 * @return String
	 */
	protected String vrml10IndexedSetName() {
		return "IndexedLineSet";
	}

	/**
	 * Answer the name of my indexed set node.
	 * 
	 * @return String
	 */
	protected String vrml20IndexedSetName() {
		return "IndexedLineSet";
	}
}
