package sra.jun.opengl.objects;

import java.awt.Color;
import java.util.Vector;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.geometry.curves.Jun3dLine;
import sra.jun.goodies.lisp.JunLispCons;
import sra.jun.opengl.support.JunOpenGLRenderingContext;
import sra.smalltalk.*;

/**
 * JunOpenGL3dPolylineLoop class
 * 
 *  @author    nisinaka
 *  @created   1998/10/21 (by nisinaka)
 *  @updated   1999/08/04 (by nisinaka)
 *  @updated   2001/11/20 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dPolylineLoop.java,v 6.0 2003/10/28 10:52:19 nisinaka Exp $
 */
public class JunOpenGL3dPolylineLoop extends JunOpenGL3dPolyline {

	/**
	 * Create a new JunOpenGL3dPolylineLoop and initialize it.
	 */
	public JunOpenGL3dPolylineLoop() {
		super();
	}

	/**
	 * Create a new JunOpenGL3dPolylineLoop and initialize it with an array of
	 * 3d points.
	 * 
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 */
	public JunOpenGL3dPolylineLoop(Jun3dPoint[] anArrayOf3dPoints) {
		super(anArrayOf3dPoints);
	}

	/**
	 * Create a new JunOpenGL3dPolylineLoop and initialize it with an array of
	 * 3d points and a Color.
	 * 
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @param aColor java.awt.Color
	 */
	public JunOpenGL3dPolylineLoop(Jun3dPoint[] anArrayOf3dPoints, Color aColor) {
		super(anArrayOf3dPoints, aColor);
	}

	/**
	 * Create a new JunOpenGL3dPolylineLoop and initialize it with an array of
	 * 3d points.
	 * 
	 * @param aCollectionOf3dPoints java.util.Vector
	 */
	public JunOpenGL3dPolylineLoop(Vector aCollectionOf3dPoints) {
		super(aCollectionOf3dPoints);
	}

	/**
	 * Create a new JunOpenGL3dPolylineLoop and initialize it with an array of
	 * 3d points.
	 * 
	 * @param aCollectionOf3dPoints java.util.Vector
	 * @param aColor java.awt.Color
	 */
	public JunOpenGL3dPolylineLoop(Vector aCollectionOf3dPoints, Color aColor) {
		super(aCollectionOf3dPoints, aColor);
	}

	/**
	 * Create a JunOpenGL3dPolyline from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject FromLispList_(JunLispCons aList) {
		JunOpenGL3dObject object = new JunOpenGL3dPolylineLoop();
		object = NameFromLispList_for_(aList, object);
		object = ColorFromLispList_for_(aList, object);
		object = LineWidthFromLispList_for_(aList, object);
		object = StippleFromLispList_for_(aList, object);
		object = VertexesFromLispList_for_(aList, (JunOpenGL3dVertexesObject) object);
		object = TransformationFromLispList_for_(aList, object);

		return object;
	}

	/**
	 * Create a new JunOpenGL3dPolylineLoop and initialize it with an array of
	 * 3d points.
	 * 
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Vertexes_(Jun3dPoint[] anArrayOf3dPoints) {
		//return JunOpenGL3dVertexesObject.Vertexes_(JunOpenGL3dPolylineLoop.class, anArrayOf3dPoints);
		return new JunOpenGL3dPolylineLoop(anArrayOf3dPoints);
	}

	/**
	 * Create a new JunOpenGL3dPolylineLoop and initialize it with an array of
	 * 3d points.
	 * 
	 * @param aCollectionOf3dPoints java.util.Vector
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Vertexes_(Vector aCollectionOf3dPoints) {
		//return JunOpenGL3dVertexesObject.Vertexes_(JunOpenGL3dPolylineLoop.class, aCollectionOf3dPoints);
		return new JunOpenGL3dPolylineLoop(aCollectionOf3dPoints);
	}

	/**
	 * Create a new JunOpenGL3dPolylineLoop and initialize it with an array of
	 * 3d points and a Color.
	 * 
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @param aColor java.awt.Color
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static final JunOpenGL3dObject Vertexes_paint_(Jun3dPoint[] anArrayOf3dPoints, Color aColor) {
		//return JunOpenGL3dVertexesObject.Vertexes_paint_(JunOpenGL3dPolylineLoop.class, anArrayOf3dPoints, aColor);
		return new JunOpenGL3dPolylineLoop(anArrayOf3dPoints, aColor);
	}

	/**
	 * Enumerate every geometries and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object geometriesDo_(StBlockClosure aBlock) {
		for (int i = 0; i < (vertexes.length - 1); i++) {
			Object result = aBlock.value_(new Jun3dLine(vertexes[i], vertexes[i + 1]));

			if (result != null) {
				return result;
			}
		}

		return aBlock.value_(new Jun3dLine(vertexes[vertexes.length - 1], vertexes[0]));
	}

	/**
	 * Answer true if the receiver is looped line segments.
	 * 
	 * @return boolean
	 */
	public boolean isLooped() {
		return true;
	}

	/**
	 * Answer the StSymbol which represents the kind of the receiver.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public StSymbol kindName() {
		return $("PolylineLoop");
	}

	/**
	 * Render the OpenGL object on a rendering context.
	 * 
	 * @param aRenderingContext
	 *        sra.jun.opengl.support.JunOpenGLRenderingContext
	 */
	public void renderOn_(JunOpenGLRenderingContext aRenderingContext) {
		if (this.hasPaint()) {
			aRenderingContext.paint_(this.paint());
		}

		if (this.hasAlpha()) {
			aRenderingContext.alpha_(this.alpha());
		}

		aRenderingContext.lineWidth_(this.lineWidth());

		if (this.hasStipple()) {
			aRenderingContext.lineStippleFactor_pattern_(this.stippleFactor(), this.stipplePattern());
			aRenderingContext.enableLineStipple();
		}

		aRenderingContext.displayLineLoop_(this.vertexes());

		if (this.hasStipple()) {
			aRenderingContext.disableLineStipple();
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return Jun3dPoint[]
	 */
	protected Jun3dPoint[] vrmlVertexes() {
		Jun3dPoint[] points = this.vertexes();
		int size = points.length;

		if (points[0] == points[size - 1]) {
			return points;
		}

		Jun3dPoint[] newPoints = new Jun3dPoint[size + 1];

		for (int index = 0; index < size; index++) {
			newPoints[index] = points[index];
		}

		newPoints[size] = points[0];

		return newPoints;
	}
}
