package sra.jun.opengl.objects;

import java.awt.*;
import java.io.Writer;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.*;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.support.*;
import sra.smalltalk.*;

/**
 * JunOpenGL3dTransformedObject class
 * 
 *  @author    r-matuda
 *  @created   1998/11/18 (by r-matuda)
 *  @updated   2001/11/20 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dTransformedObject.java,v 6.0 2003/10/28 10:52:19 nisinaka Exp $
 */
public class JunOpenGL3dTransformedObject extends JunOpenGL3dPrimitiveObject {
	protected JunOpenGL3dObject object;
	protected Jun3dTransformation transformation;

	/**
	 * Create a new JunOpenGL3dTransformedObject and initialize it.
	 */
	public JunOpenGL3dTransformedObject() {
		super();
	}

	/**
	 * Create a new JunOpenGL3dTransformedObject and initialize it.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public JunOpenGL3dTransformedObject(JunOpenGL3dObject a3dObject, Jun3dTransformation aTransformation) {
		this();
		this.object_(a3dObject);
		this.transformation_(aTransformation);
	}

	/**
	 * Create a new JunOpenGL3dTransformedObject and initialize it.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dTransformedObject
	 */
	public static final JunOpenGL3dTransformedObject Object_transformation_(JunOpenGL3dObject a3dObject, Jun3dTransformation aTransformation) {
		return new JunOpenGL3dTransformedObject(a3dObject, aTransformation);
	}

	/**
	 * Answer the alpha number.
	 * 
	 * @return float
	 */
	public float alpha() {
		if (this.object() == null) {
			return super.alpha();
		}

		return this.object().alpha();
	}

	/**
	 * Set the alpha number.
	 * 
	 * @param aNumber float
	 */
	public void alpha_(float aNumber) {
		super.alpha_(aNumber);
		this.object().alpha_(aNumber);
	}

	/**
	 * Enumerate each geometries and evaluate the block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object geometriesDo_(StBlockClosure aBlock) {
		return this.transformedObject().geometriesDo_(aBlock);
	}

	/**
	 * Answer true if the alpha is specified, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasAlpha() {
		if (this.object() == null) {
			return super.hasAlpha();
		}

		return this.object().hasAlpha();
	}

	/**
	 * Answer true if the receiver has a name, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasName() {
		if (this.object() == null) {
			return super.hasName();
		}

		return this.object().hasName();
	}

	/**
	 * Answer true if the paint is specified, otherwise false.
	 * 
	 * @return boolean
	 */
	public boolean hasPaint() {
		if (this.object() == null) {
			return super.hasPaint();
		}

		return this.object().hasPaint();
	}

	/**
	 * Initialize the receiver.
	 */
	public void initialize() {
		super.initialize();
		object = null;
		transformation = null;
	}

	/**
	 * Answer the StSymbol which represents the kind of the receiver.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public StSymbol kindName() {
		return this.transformedObject().kindName();
	}

	/**
	 * Answer the name.
	 * 
	 * @return java.lang.String
	 */
	public String name() {
		if (this.object() == null) {
			return super.name();
		}

		return this.object().name();
	}

	/**
	 * Set name.
	 * 
	 * @param aString java.lang.String
	 */
	public void name_(String aString) {
		super.name_(aString);
		this.object().name_(aString);
	}

	/**
	 * Answer the number of polygons.
	 * 
	 * @return int
	 */
	public int numberOfPolygons() {
		return this.object().numberOfPolygons();
	}

	/**
	 * Answer the object.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject object() {
		return object;
	}

	/**
	 * Set the object.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public void object_(JunOpenGL3dObject a3dObject) {
		if (a3dObject instanceof JunOpenGL3dTransformedObject) {
			object = ((JunOpenGL3dTransformedObject) a3dObject).transformedObject();
		} else {
			object = a3dObject;
		}

		this.flushBounds();
	}

	/**
	 * Enumerate each objects and evaluate the block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object objectsDo_(StBlockClosure aBlock) {
		return this.transformedObject().objectsDo_(aBlock);
	}

	/**
	 * Answer the color.
	 * 
	 * @return java.awt.Color
	 */
	public Color paint() {
		if (this.object() == null) {
			return super.paint();
		}

		return this.object().paint();
	}

	/**
	 * Set the color.
	 * 
	 * @param colorValue java.awt.Color
	 */
	public void paint_(Color colorValue) {
		super.paint_(colorValue);
		this.object().paint_(colorValue);
	}

	/**
	 * Enumerate every points and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void pointsDo_(final StBlockClosure aBlock) {
		if (this.object() == null) {
			return;
		}

		if (this.transformation() == null) {
			this.object().pointsDo_(aBlock);
		} else {
			this.object().pointsDo_(new StBlockClosure() {
				public Object value_(Object o) {
					Jun3dPoint point = (Jun3dPoint) o;
					aBlock.value_(point.transform_(transformation()));

					return null;
				}
			});
		}
	}

	/**
	 * Render the OpenGL object on a rendering context.
	 * 
	 * @param aRenderingContext
	 *        sra.jun.opengl.support.JunOpenGLRenderingContext
	 */
	public void renderOn_(final JunOpenGLRenderingContext aRenderingContext) {
		final JunOpenGL3dTransformedObject this_ = this;

		if (this.object() == null) {
			return;
		}

		if (this.hasPaint()) {
			aRenderingContext.paint_(this.paint());
		}

		if (this.hasAlpha()) {
			aRenderingContext.alpha_(this.alpha());
		}

		if (this.transformation() == null) {
			this.object().renderOn_(aRenderingContext);
		} else {
			aRenderingContext.productTransformation_while_(this.transformation(), new StBlockClosure() {
				public Object value() {
					this_.object().renderOn_(aRenderingContext);

					return null;
				}
			});
		}
	}

	/**
	 * Anser the reversed JunOpenGL3dObject of the receiver.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject reversed() {
		JunOpenGL3dTransformedObject temp = (JunOpenGL3dTransformedObject) this.copy();
		temp.object_(this.object().reversed());

		return temp;
	}

	/**
	 * Convert the receiver as a lisp list.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList toLispList() {
		return this.transformedObject().toLispList();
	}

	/**
	 * Answer the new JunOpenGL3dTransformedObject transformed with
	 * aTransformation.
	 * 
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.JunTransformation
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject transform_(JunTransformation aTransformation) {
		JunOpenGL3dTransformedObject newObject = (JunOpenGL3dTransformedObject) this.copy();

		if (this.transformation() == null) {
			newObject.transformation_((Jun3dTransformation) aTransformation);
		} else {
			newObject.transformation_((Jun3dTransformation) this.transformation().transform_(aTransformation));
		}

		return newObject;
	}

	/**
	 * Answer the transformation.
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public Jun3dTransformation transformation() {
		return transformation;
	}

	/**
	 * Set the transforamtion.
	 * 
	 * @param aTransforamtion
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public void transformation_(Jun3dTransformation aTransforamtion) {
		transformation = aTransforamtion;
		this.flushBounds();
	}

	/**
	 * Answer the transformedObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject transformedObject() {
		if (this.object() == null) {
			return new JunOpenGL3dCompoundObject();
		}

		if (this.transformation() == null) {
			return this.object();
		}

		return this.object().transform_(this.transformation());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param writer java.io.Writer
	 */
	public void vrml10On_(Writer writer) {
		this.transformedObject().vrml10On_(writer);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param writer java.io.Writer
	 */
	public void vrml20On_(Writer writer) {
		this.transformedObject().vrml20On_(writer);
	}
}
