package sra.jun.opengl.objects;

import java.awt.Color;
import java.io.*;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.support.JunOpenGLRenderingContext;
import sra.smalltalk.*;

/**
 * JunOpenGL3dVertex class
 * 
 *  @author    nisinaka
 *  @created   1998/10/21 (by nisinaka)
 *  @updated   2001/11/20 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dVertex.java,v 6.0 2003/10/28 10:52:19 nisinaka Exp $
 */
public class JunOpenGL3dVertex extends JunOpenGL3dPrimitiveObject {
	/** The point which corresponds to the vertex. */
	protected Jun3dPoint point = null;

	/** The vertex size. */
	protected float size = Float.NaN;

	/**
	 * Create a new instance of JunOpenGL3dVertex and initialize it.
	 */
	public JunOpenGL3dVertex() {
		super();
	}

	/**
	 * Create a new instance of JunOpenGL3dVertex and initialize it with the
	 * Jun3dPoint.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public JunOpenGL3dVertex(Jun3dPoint aPoint) {
		this();
		this.point_(aPoint);
	}

	/**
	 * Create a new instance of JunOpenGL3dVertex and initialize it with the
	 * Jun3dPoint and the Color.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aColor java.awt.Color
	 */
	public JunOpenGL3dVertex(Jun3dPoint aPoint, Color aColor) {
		this();
		this.point_(aPoint);
		this.paint_(aColor);
	}

	/**
	 * Create a JunOpenGL3dVertex from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject FromLispList_(JunLispCons aList) {
		JunOpenGL3dObject object = new JunOpenGL3dVertex();
		object = NameFromLispList_for_(aList, object);
		object = ColorFromLispList_for_(aList, object);
		object = PointFromLispList_for_(aList, object);
		object = SizeFromLispList_for_(aList, object);
		object = TransformationFromLispList_for_(aList, object);

		return object;
	}

	/**
	 * Create a new instance of JunOpenGL3dVertex and initialize it with the
	 * Jun3dPoint.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dVertex
	 */
	public static final JunOpenGL3dVertex Point_(Jun3dPoint aPoint) {
		//JunOpenGL3dVertex vertex = new JunOpenGL3dVertex();
		//vertex.point_(aPoint);
		//return vertex;
		return new JunOpenGL3dVertex(aPoint);
	}

	/**
	 * Create a new instance of JunOpenGL3dVertex and initialize it with the
	 * Jun3dPoint and the Color.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aColor java.awt.Color
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dVertex
	 */
	public static final JunOpenGL3dVertex Point_paint_(Jun3dPoint aPoint, Color aColor) {
		//JunOpenGL3dVertex vertex = new JunOpenGL3dVertex();
		//vertex.point_(aPoint);
		//vertex.paint_(aColor);
		//return vertex;
		return new JunOpenGL3dVertex(aPoint, aColor);
	}

	/**
	 * Get a point from the lisp list and set it as the point of the
	 * JunOpenGL3dVertex.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject PointFromLispList_for_(JunLispCons aList, JunOpenGL3dObject a3dObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("point")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return a3dObject;
		}

		((JunOpenGL3dVertex) a3dObject).point_((Jun3dPoint) list.tail());

		return a3dObject;
	}

	/**
	 * Get a size from the lisp list and set it as the size of the
	 * JunOpenGL3dVertex.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject SizeFromLispList_for_(JunLispCons aList, JunOpenGL3dObject a3dObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("size")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return a3dObject;
		}

		((JunOpenGL3dVertex) a3dObject).size_(((Number) list.tail()).floatValue());

		return a3dObject;
	}

	/**
	 * Enumerate each geometries and evaluate the block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return java.lang.Object
	 */
	public Object geometriesDo_(StBlockClosure aBlock) {
		return aBlock.value_(point);
	}

	/**
	 * Initialize the receiver.
	 */
	public void initialize() {
		super.initialize();
		point = this.defaultPoint();
		size = this.defaultSize();
	}

	/**
	 * Answer the StSymbol which represents the kind of the receiver.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public StSymbol kindName() {
		return $("Spot");
	}

	/**
	 * Answer the number of polygons.
	 * 
	 * @return int
	 */
	public int numberOfPolygons() {
		return 0;
	}

	/**
	 * Answer the point.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint point() {
		return point;
	}

	/**
	 * Set a point.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public void point_(Jun3dPoint aPoint) {
		point = aPoint;
		this.flushBounds();
	}

	/**
	 * Enumerate every points and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void pointsDo_(StBlockClosure aBlock) {
		aBlock.value_(this.point());
	}

	/**
	 * Convert the receiver's point as a JunLispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList pointToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("point"));
		list.tail_(this.point());

		return list;
	}

	/**
	 * Render the OpenGL object on a rendering context.
	 * 
	 * @param aRenderingContext
	 *        sra.jun.opengl.support.JunOpenGLRenderingContext
	 */
	public void renderOn_(JunOpenGLRenderingContext aRenderingContext) {
		if (this.hasPaint()) {
			aRenderingContext.paint_(this.paint());
		}

		if (this.hasAlpha()) {
			aRenderingContext.alpha_(this.alpha());
		}

		aRenderingContext.pointSize_(this.size());
		aRenderingContext.displayPoint_(this.point());
	}

	/**
	 * Anser the reversed JunOpenGL3dObject of the receiver.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject reversed() {
		return (JunOpenGL3dVertex) this.copy();
	}

	/**
	 * Answer the size.
	 * 
	 * @return float
	 */
	public float size() {
		return size;
	}

	/**
	 * Set a size.
	 * 
	 * @param aNumber float
	 */
	public void size_(float aNumber) {
		size = aNumber;
	}

	/**
	 * Convert the receiver's size as a JunLispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList sizeToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("size"));
		list.tail_(new Float(this.size()));

		return list;
	}

	/**
	 * Convert the receiver as a JunLispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList toLispList() {
		JunLispCons list = this.lispCons();
		list.head_(this.kindName());

		if (this.hasName()) {
			list.add_(this.nameToLispList());
		}

		if (this.hasColor()) {
			list.add_(this.colorToLispList());
		}

		list.add_(this.pointToLispList());
		list.add_(this.sizeToLispList());

		return list;
	}

	/**
	 * Answer the new JunOpen3dObject transformed with aTransformation.
	 * 
	 * @param aTransformation
	 *        sra.jun.geometry.transformations.Jun3dTransformation
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject transform_(Jun3dTransformation aTransformation) {
		JunOpenGL3dVertex transformedObject = (JunOpenGL3dVertex) this.copy();
		transformedObject.point_((Jun3dPoint) point.transform_(aTransformation));
		return transformedObject;
	}

	/**
	 * Write my VRML1.0 string on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public void vrml10On_(Writer aWriter) {
		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
			bw.write("DEF ");
			bw.write(this.legalName());
			bw.newLine();
			bw.write("Separator {");
			bw.newLine();
			this.vrml10ColorOn_(bw);
			this.vrml10PointOn_(bw);
			bw.write("} #Separator");
			bw.newLine();
			bw.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Write my point as VRML1.0 on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public void vrml10PointOn_(Writer aWriter) {
		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
			bw.write("\t");
			bw.write("Coordinate3 {");
			bw.newLine();
			bw.write("\t\t");
			bw.write("point [");
			bw.newLine();

			Jun3dPoint point = this.point;
			bw.write("\t\t\t");
			bw.write(String.valueOf((float) point.x()));
			bw.write(" ");
			bw.write(String.valueOf((float) point.y()));
			bw.write(" ");
			bw.write(String.valueOf((float) point.z()));
			bw.write(" #1");
			bw.newLine();
			bw.write("\t\t");
			bw.write("] #point");
			bw.newLine();
			bw.write("\t");
			bw.write("} #Coordinate3");
			bw.newLine();
			bw.write("\t");
			bw.write(this.vrml10IndexedSetName());
			bw.write(" {");
			bw.newLine();
			bw.write("\t\t");
			bw.write("coordIndex [");
			bw.newLine();
			bw.write("\t\t\t");
			bw.write("1, -1");
			bw.newLine();
			bw.write("\t\t");
			bw.write("] #coordIndex");
			bw.newLine();
			bw.write("\t");
			bw.write("} #");
			bw.write(this.vrml10IndexedSetName());
			bw.newLine();
			bw.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Write my VRML2.0 string on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public void vrml20On_(Writer aWriter) {
		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);

			if (this.name() != this.defaultName()) {
				bw.write(" #");
				bw.write(this.name());
				bw.newLine();
			}

			bw.write("Shape {");
			bw.newLine();
			this.vrml20ColorOn_(bw);
			this.vrml20PointOn_(bw);
			bw.write("} #Shape");
			bw.newLine();
			bw.flush();
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Write my point as VRML1.0 on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public void vrml20PointOn_(Writer aWriter) {
		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
			bw.write("\t");
			bw.write("geometry ");
			bw.write(this.vrml20IndexedSetName());
			bw.write(" {");
			bw.newLine();
			bw.write("\t\t");
			bw.write("coord Coordinate {");
			bw.newLine();
			bw.write("\t\t\t");
			bw.write("point [");
			bw.newLine();

			Jun3dPoint point = this.point;
			bw.write("\t\t\t\t");
			bw.write(String.valueOf((float) point.x()));
			bw.write(" ");
			bw.write(String.valueOf((float) point.y()));
			bw.write(" ");
			bw.write(String.valueOf((float) point.z()));
			bw.write(" #1");
			bw.newLine();
			bw.write("\t\t\t");
			bw.write("] #point");
			bw.newLine();
			bw.write("\t\t");
			bw.write("} #Coordinate");
			bw.newLine();
			bw.write("\t");
			bw.write("} #");
			bw.write(this.vrml20IndexedSetName());
			bw.newLine();
			bw.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Answer the name of my indexed set node.
	 * 
	 * @return String
	 */
	protected String vrml10IndexedSetName() {
		return "IndexedLineSet";
	}

	/**
	 * Answer the name of my indexed set node.
	 * 
	 * @return String
	 */
	protected String vrml20IndexedSetName() {
		return "PointSet";
	}
}
