package sra.jun.opengl.objects;

import java.awt.Color;
import java.io.*;
import java.util.Vector;
import sra.smalltalk.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.goodies.lisp.*;

/**
 * JunOpenGL3dVertexesObject class
 * 
 *  @author    nisinaka
 *  @created   1998/10/13 (by nisinaka)
 *  @updated   2000/01/06 (by nisinaka)
 *  @updated   2001/11/20 (by nisinaka)
 *  @updated   2003/07/01 (by nisinaka)
 *  @updated   2003/09/09 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun517 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGL3dVertexesObject.java,v 6.0 2003/10/28 10:52:19 nisinaka Exp $
 */
public abstract class JunOpenGL3dVertexesObject extends JunOpenGL3dPrimitiveObject {

	/** An array of Jun3dPoint which compose a JunOpenGL3dVertexesObject. */
	protected Jun3dPoint[] vertexes;

	/** An array of Color which is for Gouraud Shading. */
	protected Color[] colors;

	/** An array of Jun3dPoint which holds normal vectors of each vertexes. */
	protected Jun3dPoint[] normalVectors;

	/**
	 * Create a new JunOpenGL3dVertexesObject and initialize it.
	 * 
	 * @category Instance creation
	 */
	public JunOpenGL3dVertexesObject() {
		super();
	}

	/**
	 * Create a new JunOpenGL3dVertexesObject and initialize it with an array of 3d points.
	 * 
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @category Instance creation
	 */
	public JunOpenGL3dVertexesObject(Jun3dPoint[] anArrayOf3dPoints) {
		super();
		this.vertexes_(anArrayOf3dPoints);
	}

	/**
	 * Create a new JunOpenGL3dVertexesObject and initialize it with an array of 3d points and a Color.
	 * 
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @param aColor java.awt.Color
	 * @category Instance creation
	 */
	public JunOpenGL3dVertexesObject(Jun3dPoint[] anArrayOf3dPoints, Color aColor) {
		super();
		this.vertexes_(anArrayOf3dPoints);
		this.paint_(aColor);
	}

	/**
	 * Create a new JunOpenGL3dVertexesObject and initialize it with an array of 3d points.
	 * 
	 * @param aCollectionOf3dPoints java.util.Vector
	 * @category Instance creation
	 */
	public JunOpenGL3dVertexesObject(Vector aCollectionOf3dPoints) {
		super();
		this.vertexes_(aCollectionOf3dPoints);
	}

	/**
	 * Create a new JunOpenGL3dVertexesObject and initialize it with an array of 3d points.
	 * 
	 * @param aCollectionOf3dPoints java.util.Vector
	 * @param aColor java.awt.Color
	 * @category Instance creation
	 */
	public JunOpenGL3dVertexesObject(Vector aCollectionOf3dPoints, Color aColor) {
		super();
		this.vertexes_(aCollectionOf3dPoints);
		this.paint_(aColor);
	}

	/**
	 * Create a new JunOpenGL3dObject and initialize it with an array of 3d points.
	 * 
	 * @param aClass java.lang.Class
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Instance creation
	 */
	protected static JunOpenGL3dObject Vertexes_(Class aClass, Jun3dPoint[] anArrayOf3dPoints) {
		JunOpenGL3dVertexesObject a3dObject = (JunOpenGL3dVertexesObject) _New(aClass);
		a3dObject.vertexes_(anArrayOf3dPoints);
		return a3dObject;
	}

	/**
	 * Create a new JunOpenGL3dObject and initialize it with an array of 3d
	 * points.
	 * 
	 * @param aClass java.lang.Class
	 * @param aCollectionOf3dPoints java.util.Vector
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Instance creation
	 */
	protected static JunOpenGL3dObject Vertexes_(Class aClass, Vector aCollectionOf3dPoints) {
		JunOpenGL3dVertexesObject a3dObject = (JunOpenGL3dVertexesObject) _New(aClass);
		a3dObject.vertexes_(aCollectionOf3dPoints);
		return a3dObject;
	}

	/**
	 * Create a new JunOpenGL3dObject and initialize it with an array of 3d
	 * points and a Color.
	 * 
	 * @param aClass java.lang.Class
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @param aColor java.awt.Color
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Instance creation
	 */
	protected static JunOpenGL3dObject Vertexes_paint_(Class aClass, Jun3dPoint[] anArrayOf3dPoints, Color aColor) {
		JunOpenGL3dObject anObject = Vertexes_(aClass, anArrayOf3dPoints);
		anObject.paint_(aColor);
		return anObject;
	}

	/**
	 * Create a JunOpenGL3dVertexes from the lisp list.
	 * 
	 * @param aClass DOCUMENT ME!
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Lisp support
	 */
	public static JunOpenGL3dObject FromLispList_(Class aClass, JunLispCons aList) {
		JunOpenGL3dObject object = (JunOpenGL3dVertexesObject) _New(aClass);
		object = NameFromLispList_for_(aList, object);
		object = ColorFromLispList_for_(aList, object);
		object = StippleFromLispList_for_(aList, object);
		object = TextureFromLispList_for_(aList, object);
		object = TransformationFromLispList_for_(aList, object);
		object = VertexesFromLispList_for_(aList, (JunOpenGL3dVertexesObject) object);
		object = ColorsFromLispList_for_(aList, (JunOpenGL3dVertexesObject) object);
		object = NormalVectorsFromLispList_for_(aList, (JunOpenGL3dVertexesObject) object);
		return object;
	}

	/**
	 * Get a stipple from the lisp list and set it as the stipple of the JunOpenGL3dObject.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Lisp support
	 */
	public static JunOpenGL3dObject StippleFromLispList_for_(JunLispCons aList, JunOpenGL3dObject a3dObject) {
		throw SmalltalkException.SubclassResponsibility();
	}

	/**
	 * Get a texture from the lisp list and set it as the texture of the JunOpenGL3dObject.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category Lisp support
	 */
	public static JunOpenGL3dObject TextureFromLispList_for_(JunLispCons aList, JunOpenGL3dObject a3dObject) {
		throw SmalltalkException.SubclassResponsibility();
	}

	/**
	 * Get vertexes from the lisp list and set it as the vertexes of the JunOpenGL3dVertexesObject.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dVertexesObject
	 * @return sra.jun.opengl.objects.JunOpenGL3dVertexesObject
	 * @category Lisp support
	 */
	public static JunOpenGL3dObject VertexesFromLispList_for_(JunLispCons aList, JunOpenGL3dVertexesObject a3dObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("points")));
			}
		}, new StBlockClosure());

		if (list == null) {
			a3dObject.vertexes_(((JunLispList) aList.tail()).toVector());
		} else {
			a3dObject.vertexes_(((JunLispList) list.tail()).toVector());
		}

		return a3dObject;
	}

	/**
	 * Get colors from the lisp list and set it as the colors of the JunOpenGL3dVertexesObject.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dVertexesObject
	 * @return sra.jun.opengl.objects.JunOpenGL3dVertexesObject
	 * @category Lisp support
	 */
	public static JunOpenGL3dObject ColorsFromLispList_for_(JunLispCons aList, JunOpenGL3dVertexesObject a3dObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("colors")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return a3dObject;
		}

		Object[] anArray = ((JunLispCons) list.tail()).asArray();
		Color[] colors = new Color[anArray.length];
		for (int i = 0; i < colors.length; i++) {
			JunLispCons rgbList = (JunLispCons) anArray[i];
			float red = ((Number) rgbList.nth_(1)).floatValue();
			float green = ((Number) rgbList.nth_(2)).floatValue();
			float blue = ((Number) rgbList.nth_(3)).floatValue();
			colors[i] = new Color(red, green, blue);
		}
		a3dObject.colors_(colors);

		return a3dObject;
	}

	/**
	 * Get normal vectors from the lisp list and set it as the normal vectors of the JunOpenGL3dVertexesObject.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispCons
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dVertexesObject
	 * @return sra.jun.opengl.objects.JunOpenGL3dVertexesObject
	 * @category Lisp support
	 */
	public static JunOpenGL3dObject NormalVectorsFromLispList_for_(JunLispCons aList, JunOpenGL3dVertexesObject a3dObject) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("normalVectors")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return a3dObject;
		}

		Object[] anArray = ((JunLispCons) list.tail()).asArray();
		Jun3dPoint[] normalVectors = new Jun3dPoint[anArray.length];
		System.arraycopy(anArray, 0, normalVectors, 0, normalVectors.length);
		a3dObject.normalVectors_(normalVectors);

		return a3dObject;
	}

	/**
	 * Initialize the JunOpenGL3dVertexesObject.
	 * 
	 * @category initialize-release
	 */
	public void initialize() {
		super.initialize();
		vertexes = null;
		colors = null;
		normalVectors = null;
	}

	/**
	 * Answer my vertexes.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint[]
	 * @category accessing
	 */
	public Jun3dPoint[] vertexes() {
		return vertexes;
	}

	/**
	 * Set the vertexes with an array of 3d points.
	 * 
	 * @param anArrayOf3dPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @category accessing
	 */
	public void vertexes_(Jun3dPoint[] anArrayOf3dPoints) {
		int size = anArrayOf3dPoints.length;
		vertexes = new Jun3dPoint[size];
		System.arraycopy(anArrayOf3dPoints, 0, vertexes, 0, size);
		this.flushBounds();
		this.flushNormalVectors();
	}

	/**
	 * Set the vertexes with an array of 3d points.
	 * 
	 * @param aCollectionOf3dPoints java.util.Vector
	 * @category accessing
	 */
	public void vertexes_(Vector aCollectionOf3dPoints) {
		vertexes = new Jun3dPoint[aCollectionOf3dPoints.size()];
		aCollectionOf3dPoints.copyInto(vertexes);
		this.flushBounds();
		this.flushNormalVectors();
	}

	/**
	 * Answer my colors for Gouraud Shading.
	 * 
	 * @return java.awt.Color[]
	 * @category accessing
	 */
	public Color[] colors() {
		return colors;
	}

	/**
	 * Set my new colors for Gouraud Shading.
	 * 
	 * @param anArrayOfColors java.awt.Color[]
	 * @category accessing
	 */
	public void colors_(Color[] anArrayOfColors) {
		colors = anArrayOfColors;
	}

	/**
	 * Answer true if the receiver is a vertexes object.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isVertexesObject() {
		return true;
	}

	/**
	 * Answer true if the receiver has colors.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean hasColors() {
		return (colors != null);
	}

	/**
	 * Answer true if the receiver has normal vectors.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean hasNormalVectors() {
		return (normalVectors != null);
	}

	/**
	 * Flush my normal vector.
	 * 
	 * @category vector accessing
	 */
	public void flushNormalVectors() {
		normalVectors = null;
	}

	/**
	 * Answer the current normal vectors.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint[]
	 * @category vector accessing
	 */
	public Jun3dPoint[] normalVectors() {
		if (normalVectors == null) {
			normalVectors = new Jun3dPoint[] { this.preferredNormalVector()};
		}
		return normalVectors;
	}

	/**
	 * Set the new normal vectors.
	 * 
	 * @param newNormalVectors
	 * @category vector accessing
	 */
	public void normalVectors_(Jun3dPoint[] newNormalVectors) {
		normalVectors = newNormalVectors;
	}

	/**
	 * Answer the current normal vector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category vector accessing
	 */
	public Jun3dPoint normalVector() {
		return this.normalVectors()[0];
	}

	/**
	 * Set the new normal vector.
	 * 
	 * @param newNormalVector
	 * @category vector accessing
	 */
	public void normalVector_(Jun3dPoint newNormalVector) {
		this.normalVectors_(new Jun3dPoint[] { newNormalVector });
	}

	/**
	 * Do an extra copy of the receiver.
	 * 
	 * @return sra.smalltalk.StObject
	 * @category copying
	 */
	public StObject postCopy() {
		super.postCopy();

		if (vertexes != null) {
			Jun3dPoint[] newVertexes = new Jun3dPoint[vertexes.length];
			System.arraycopy(vertexes, 0, newVertexes, 0, vertexes.length);
			vertexes = newVertexes;
		}

		if (colors != null) {
			Color[] newColors = new Color[colors.length];
			System.arraycopy(colors, 0, newColors, 0, colors.length);
			colors = newColors;
		}

		if (normalVectors != null) {
			Jun3dPoint[] newNormalVectors = new Jun3dPoint[normalVectors.length];
			System.arraycopy(normalVectors, 0, newNormalVectors, 0, normalVectors.length);
			normalVectors = newNormalVectors;
		}

		return this;
	}

	/**
	 * Enumerate every points and evaluate the Block.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @category enumerating
	 */
	public void pointsDo_(StBlockClosure aBlock) {
		for (int i = 0; i < vertexes.length; i++) {
			aBlock.value_(vertexes[i]);
		}
	}

	/**
	 * Answer the new JunOpen3dObject transformed with aTransformation.
	 * 
	 * @param aTransformation sra.jun.geometry.transformations.Jun3dTransformation
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category transforming
	 */
	public JunOpenGL3dObject transform_(Jun3dTransformation aTransformation) {
		JunOpenGL3dVertexesObject transformedObject = (JunOpenGL3dVertexesObject) this.copy();

		transformedObject.vertexes = new Jun3dPoint[vertexes.length];
		for (int i = 0; i < vertexes.length; i++) {
			transformedObject.vertexes[i] = (Jun3dPoint) vertexes[i].transform_(aTransformation);
		}

		if (normalVectors != null) {
			transformedObject.normalVectors = new Jun3dPoint[normalVectors.length];
			for (int i = 0; i < normalVectors.length; i++) {
				transformedObject.normalVectors[i] = (Jun3dPoint) normalVectors[i].transform_(aTransformation);
			}
		}

		return transformedObject;
	}

	/**
	 * Anser the reversed JunOpenGL3dObject of the receiver.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category converting
	 */
	public JunOpenGL3dObject reversed() {
		JunOpenGL3dVertexesObject reversedObject = (JunOpenGL3dVertexesObject) this.copy();

		if (vertexes != null) {
			reversedObject.vertexes = new Jun3dPoint[vertexes.length];
			for (int i = 0; i < vertexes.length; i++) {
				reversedObject.vertexes[i] = vertexes[vertexes.length - i - 1];
			}
		}

		if (colors != null) {
			reversedObject.colors = new Color[colors.length];
			for (int i = 0; i < colors.length; i++) {
				reversedObject.colors[i] = colors[colors.length - i - 1];
			}
		}

		if (normalVectors != null) {
			reversedObject.normalVectors = new Jun3dPoint[normalVectors.length];
			for (int i = 0; i < normalVectors.length; i++) {
				reversedObject.normalVectors[i] = normalVectors[normalVectors.length - i - 1];
			}
		}

		return reversedObject;
	}

	/**
	 * Convert the receiver's stipple as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public abstract JunLispList stippleToLispList();

	/**
	 * Convert the receiver's texture as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public abstract JunLispList textureToLispList();

	/**
	 * Convert the receiver as a JunLispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList toLispList() {
		JunLispCons list = this.lispCons();
		list.head_(this.kindName());

		if (!this.hasName() && !this.hasColor() && !this.hasStipple() && !this.hasTexture() && !this.hasColors() && !this.hasNormalVectors()) {
			for (int i = 0; i < vertexes.length; i++) {
				list.add_(vertexes[i]);
			}
			return list;
		}

		if (this.hasName()) {
			list.add_(this.nameToLispList());
		}
		if (this.hasColor()) {
			list.add_(this.colorToLispList());
		}
		if (this.hasStipple()) {
			list.add_(this.stippleToLispList());
		}
		if (this.hasTexture()) {
			list.add_(this.textureToLispList());
		}
		list.add_(this.vertexesToLispList());
		if (this.hasColors()) {
			list.add_(this.colorsToLispList());
		}
		if (this.hasNormalVectors()) {
			list.add_(this.normalVectorsToLispList());
		}

		return list;
	}

	/**
	 * Convert the receiver's vertexes as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList vertexesToLispList() {
		Object[] collection = new Object[vertexes.length + 1];
		collection[0] = $("points");
		for (int i = 0; i < vertexes.length; i++) {
			collection[i + 1] = vertexes[i];
		}
		return JunLispCons.List_(collection);
	}

	/**
	 * Convert the receiver's colors as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList colorsToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("colors"));
		if (colors != null) {
			for (int i = 0; i < colors.length; i++) {
				list.add_(JunLispCons.List_(new float[] { colors[i].getRed() / 255f, colors[i].getGreen() / 255f, colors[i].getBlue() / 255f }));
			}
		}
		return list;
	}

	/**
	 * Convert the receiver's normal vectors as a LispList.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	public JunLispList normalVectorsToLispList() {
		Object[] collection = new Object[normalVectors.length + 1];
		collection[0] = $("normalVectors");
		if (normalVectors != null) {
			for (int i = 0; i < normalVectors.length; i++) {
				collection[i + 1] = normalVectors[i];
			}
		}
		return JunLispCons.List_(collection);
	}

	/**
	 * Write my VRML1.0 string on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws SmalltalkException DOCUMENT ME!
	 * @category vrml support
	 */
	public void vrml10On_(Writer aWriter) {
		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
			bw.write("DEF ");
			bw.write(this.legalName());
			bw.newLine();
			bw.write("Separator {");
			bw.newLine();
			this.vrml10ColorOn_(bw);
			this.vrml10VertexesOn_(bw);
			bw.write("} #Separator");
			bw.newLine();
			bw.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Write my vertexes as VRML1.0 on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws SmalltalkException DOCUMENT ME!
	 * @category vrml support
	 */
	public void vrml10VertexesOn_(Writer aWriter) {
		Jun3dPoint[] points = this.vrmlVertexes();
		if (points.length == 0) {
			return;
		}

		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
			if (this.hasTexture()) {
				bw.write("\t");
				bw.write("Texture2");
				bw.newLine();
				bw.write("\t\t");
				bw.write("image ");
				this.texture().vrmlSFImageOn_(bw);
				bw.newLine();
				bw.write("\t");
				bw.write("} #Texture2");
				bw.newLine();
			}
			bw.write("\t");
			bw.write("Coordinate3 {");
			bw.newLine();
			bw.write("\t\t");
			bw.write("point [");
			bw.newLine();
			for (int index = 0; index < points.length; index++) {
				Jun3dPoint point = points[index];
				bw.write("\t\t\t");
				bw.write(String.valueOf((float) point.x()));
				bw.write(" ");
				bw.write(String.valueOf((float) point.y()));
				bw.write(" ");
				bw.write(String.valueOf((float) point.z()));
				if (index != (points.length - 1)) {
					bw.write(",");
				}
				bw.write(" #");
				bw.write(String.valueOf(index));
				bw.newLine();
			}
			bw.write("\t\t");
			bw.write("] #point");
			bw.newLine();
			bw.write("\t");
			bw.write("} #Coordinate3");
			bw.newLine();
			if (this.hasTexture() && (this.texture().hasCoordinates())) {
				bw.write("\t");
				bw.write("TextureCoordinate2 {");
				bw.newLine();
				bw.write("\t\t");
				bw.write("point [");
				bw.newLine();
				Jun2dPoint[] coordinates = this.texture().coordinates();
				for (int index = 0; index < coordinates.length; index++) {
					Jun2dPoint point = coordinates[index];
					bw.write("\t\t\t");
					bw.write(String.valueOf((float) point.x()));
					bw.write(" ");
					bw.write(String.valueOf((float) point.y()));
					if (index != (coordinates.length - 1)) {
						bw.write(",");
					}
					bw.write(" #");
					bw.write(String.valueOf(index));
					bw.newLine();
				}
				bw.write("\t\t");
				bw.write("]");
				bw.newLine();
				bw.write("\t");
				bw.write("} #TextureCoordinate2");
				bw.newLine();
			}
			bw.write("\t");
			bw.write(this.vrml10IndexedSetName());
			bw.write(" {");
			bw.newLine();
			bw.write("\t\t");
			bw.write("coordIndex [");
			bw.newLine();
			bw.write("\t\t\t");
			for (int index = 0; index < points.length; index++) {
				bw.write(String.valueOf(index));
				bw.write(", ");
			}
			bw.write("-1");
			bw.newLine();
			bw.write("\t\t");
			bw.write("] #coordIndex");
			bw.newLine();
			if (this.hasTexture()) {
				bw.write("\t");
				bw.write("textureCoordIndex [");
				bw.newLine();
				bw.write("\t\t");
				for (int index = 0; index < points.length; index++) {
					bw.write(String.valueOf(index));
					bw.write(", ");
				}
				bw.write("-1");
				bw.newLine();
				bw.write("\t");
				bw.write("] #textureCoordIndex");
			}
			bw.write("\t");
			bw.write("} #");
			bw.write(this.vrml10IndexedSetName());
			bw.newLine();
			bw.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Write my VRML2.0 string on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws SmalltalkException DOCUMENT ME!
	 * @category vrml support
	 */
	public void vrml20On_(Writer aWriter) {
		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
			if (this.name() != this.defaultName()) {
				bw.write(" #");
				bw.write(this.name());
				bw.newLine();
			}
			bw.write("Shape {");
			bw.newLine();
			this.vrml20AppearanceOn_(bw);
			this.vrml20VertexesOn_(bw);
			bw.write("} #Shape");
			bw.newLine();
			bw.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Write my vertexes as VRML2.0 on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws SmalltalkException DOCUMENT ME!
	 * @category vrml support
	 */
	public void vrml20VertexesOn_(Writer aWriter) {
		Jun3dPoint[] points = this.vrmlVertexes();
		if (points.length == 0) {
			return;
		}

		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
			bw.write("\t");
			bw.write("geometry ");
			bw.write(this.vrml20IndexedSetName());
			bw.write(" {");
			bw.newLine();
			bw.write("\t\t");
			bw.write("coord Coordinate {");
			bw.newLine();
			bw.write("\t\t\t");
			bw.write("point [");
			bw.newLine();
			for (int index = 0; index < points.length; index++) {
				Jun3dPoint point = points[index];
				bw.write("\t\t\t\t");
				bw.write(String.valueOf((float) point.x()));
				bw.write(" ");
				bw.write(String.valueOf((float) point.y()));
				bw.write(" ");
				bw.write(String.valueOf((float) point.z()));
				if (index != (points.length - 1)) {
					bw.write(",");
				}
				bw.write(" #");
				bw.write(String.valueOf(index));
				bw.newLine();
			}
			bw.write("\t\t\t");
			bw.write("] #point");
			bw.newLine();
			bw.write("\t\t");
			bw.write("} #Coordinate");
			bw.newLine();
			bw.write("\t\t");
			bw.write("coordIndex [");
			bw.newLine();
			bw.write("\t\t\t");
			for (int index = 0; index < points.length; index++) {
				bw.write(String.valueOf(index));
				bw.write(", ");
			}
			bw.write("-1 ");
			bw.newLine();
			bw.write("\t\t");
			bw.write("] #coordIndex");
			if (this.hasTexture() && (this.texture().hasCoordinates())) {
				bw.newLine();
				bw.write("\t\t");
				bw.write("texCoord TextureCoordinate {");
				bw.newLine();
				bw.write("\t\t\t");
				bw.write("point[");
				bw.newLine();
				Jun2dPoint[] coordinates = this.texture().coordinates();
				for (int index = 0; index < coordinates.length; index++) {
					Jun2dPoint point = coordinates[index];
					bw.write("\t\t\t\t");
					bw.write(String.valueOf((float) point.x()));
					bw.write(" ");
					bw.write(String.valueOf((float) point.y()));
					if (index != (coordinates.length - 1)) {
						bw.write(", ");
					}
					bw.write("#");
					bw.write(String.valueOf(index));
					bw.newLine();
				}
				bw.write("\t\t\t");
				bw.write("]");
				bw.newLine();
				bw.write("\t\t");
				bw.write("} #texCoord");
				bw.newLine();
				bw.write("\t\t");
				bw.write("texCoordIndex [");
				bw.newLine();
				bw.write("\t\t\t");
				for (int index = 0; index < points.length; index++) {
					bw.write(String.valueOf(index));
					bw.write(", ");
				}
				bw.write("-1");
				bw.newLine();
				bw.write("\t\t");
				bw.write("] #texCoordIndex");
			}
			bw.newLine();
			bw.write("\t");
			bw.write("} #");
			bw.write(this.vrml20IndexedSetName());
			bw.newLine();
			bw.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Answer the indexed set name for VRML1.0.
	 * 
	 * @return String java.lang.String
	 * @category vrml support
	 */
	protected abstract String vrml10IndexedSetName();

	/**
	 * Write my appearance as VRML2.0 on the writer.
	 * 
	 * @param aWriter java.io.Writer
	 * @throws SmalltalkException DOCUMENT ME!
	 * @category vrml support
	 */
	protected void vrml20AppearanceOn_(Writer aWriter) {
		if ((this.hasColor() == false) && (this.hasTexture() == false)) {
			return;
		}

		try {
			BufferedWriter bw = (aWriter instanceof BufferedWriter) ? (BufferedWriter) aWriter : new BufferedWriter(aWriter);
			bw.write("\t");
			bw.write("appearance Appearance {");
			bw.newLine();
			if (this.hasColor()) {
				bw.write("\t\t");
				bw.write("material Material {");
				bw.newLine();
				bw.write("\t\t\t");
				bw.write("emissiveColor ");
				Color color = this.paint();
				bw.write(String.valueOf(color.getRed() / 255f));
				bw.write(" ");
				bw.write(String.valueOf(color.getGreen() / 255f));
				bw.write(" ");
				bw.write(String.valueOf(color.getBlue() / 255f));
				bw.newLine();
				bw.write("\t\t");
				bw.write("} #Material");
				bw.newLine();
			}
			if (this.hasTexture()) {
				bw.write("\t\t");
				bw.write("texture PixelTexture {");
				bw.newLine();
				bw.write("\t\t\t");
				bw.write("image ");
				this.texture().vrmlSFImageOn_(bw);
				bw.newLine();
				bw.write("\t\t");
				bw.write("} #Texture");
				bw.newLine();
			}
			bw.write("\t");
			bw.write("} #Appearance");
			bw.newLine();
			bw.flush();
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Answer the indexed set name for VRML2.0.
	 * 
	 * @return String java.lang.String
	 * @category vrml support
	 */
	protected abstract String vrml20IndexedSetName();

	/**
	 * Answer the vertexes for VRML.
	 * 
	 * @return Jun3dPoint[] sra.jun.geometry.basic.Jun3dPoint[]
	 * @category vrml support
	 */
	protected Jun3dPoint[] vrmlVertexes() {
		return this.vertexes();
	}

	/**
	 * Calculate the preferred normal vector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category private
	 */
	protected Jun3dPoint preferredNormalVector() {
		StBlockClosure prevPointBlock = new StBlockClosure() {
			public Object value_(Object argument) {
				int startIndex = ((Integer) argument).intValue();
				Jun3dPoint startPoint = vertexes[startIndex];
				int index = startIndex - 1;
				int length = vertexes.length;
				while ((startIndex + index + length) > 0) {
					int i = (index % length + length) % length;
					if (startPoint.distance_(vertexes[i]) >= Jun3dPoint.ACCURACY) {
						return vertexes[i];
					}
					index--;
				}
				return null;
			}
		};

		StBlockClosure nextPointBlock = new StBlockClosure() {
			public Object value_(Object argument) {
				int startIndex = ((Integer) argument).intValue();
				Jun3dPoint startPoint = vertexes[startIndex];
				int index = startIndex + 1;
				int length = vertexes.length;
				while ((startIndex - index + length) > 0) {
					int i = (index % length + length) % length;
					if (startPoint.distance_(vertexes[i]) >= Jun3dPoint.ACCURACY) {
						return vertexes[i];
					}
					index++;
				}
				return null;
			}
		};

		Jun3dPoint vector = Jun3dPoint.Zero();
		for (int i = 0; i < vertexes.length; i++) {
			Jun3dPoint prevPoint = (Jun3dPoint) prevPointBlock.value_(new Integer(i));
			Jun3dPoint nextPoint = (Jun3dPoint) nextPointBlock.value_(new Integer(i));
			if ((prevPoint != null) && (nextPoint != null)) {
				Jun3dPoint thePoint = vertexes[i];
				vector = (Jun3dPoint) vector.plus_(((Jun3dPoint) thePoint.minus_(prevPoint)).vectorProduct_((Jun3dPoint) nextPoint.minus_(thePoint)));
			}
		}

		return (Jun3dPoint) vector.unitVector();
	}

}
