package sra.jun.opengl.parameterizedbody;

import java.awt.*;
import java.awt.event.*;
import sra.jun.goodies.parameter.*;
import sra.jun.opengl.display.*;
import sra.smalltalk.*;

/**
 * JunParameterizedBodyViewAwt class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   2000/01/19 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunParameterizedBodyViewAwt.java,v 6.0 2003/10/28 10:52:19 nisinaka Exp $
 */
public class JunParameterizedBodyViewAwt extends StViewPanel implements JunParameterizedBodyView {

	protected JunOpenGLDisplayViewAwt displayView;
	protected JunParametersViewAwt parametersView;
	protected Checkbox interlockCheckbox;
	protected Button createBodyButton;

	/**
	 * Create a new instance of JunParameterizedBodyViewAwt and initialize it.
	 *
	 * @param aModel sra.jun.opengl.parameterizedbody.JunParameterizedBodyModel
	 */
	public JunParameterizedBodyViewAwt(JunParameterizedBodyModel aModel) {
		super(aModel);

		displayView.getOpenGLView().setPopupMenu(aModel.displayModel().menuModel());
	}

	/**
	 * Answer insets.
	 * 
	 * @return java.awt.Insets
	 */
	public Insets getInsets() {
		return new Insets(4, 4, 4, 4);
	}

	/**
	 * model property accessing
	 * 
	 * @return sra.jun.opengl.parameterizedbody.JunParameterizedBodyModel
	 */
	public JunParameterizedBodyModel getModel() {
		return (JunParameterizedBodyModel) this.model();
	}

	/**
	 * model property accessing
	 * 
	 * @param aModel sra.jun.opengl.parameterizedbody.JunParameterizedBodyModel
	 */
	public void setModel(JunParameterizedBodyModel aModel) {
		this.model_(aModel);
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		JunParameterizedBodyModel theModel = this.getModel();

		// 
		Label label = new Label("Parameters :");
		label.setFont(new Font("dialog", Font.BOLD, 12));

		Checkbox interlockCheckbox = new Checkbox("Interlock", null, true);
		interlockCheckbox.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					getModel().interlockHolder().value_(Boolean.TRUE);
				}
				if (e.getStateChange() == ItemEvent.DESELECTED) {
					getModel().interlockHolder().value_(Boolean.FALSE);
				}
			}
		});

		Panel northPanel = new Panel();
		northPanel.setLayout(new BorderLayout());
		northPanel.add(label, BorderLayout.WEST);
		northPanel.add(interlockCheckbox, BorderLayout.EAST);

		parametersView = new JunParametersViewAwt(theModel.parametersModel());

		Button createBodyButton = new Button("Create Body");
		createBodyButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				getModel().createBody();
			}
		});

		Panel centerPanel = new Panel();
		centerPanel.setLayout(new BorderLayout());
		centerPanel.add(northPanel, BorderLayout.NORTH);
		centerPanel.add(parametersView, BorderLayout.CENTER);
		centerPanel.add(createBodyButton, BorderLayout.SOUTH);

		//
		displayView = new JunOpenGLDisplayViewAwt(theModel.displayModel());

		// Layout components.
		this.setLayout(new GridLayout(1, 2));
		((GridLayout) this.getLayout()).setHgap(8);
		this.add(displayView);
		this.add(centerPanel);
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunParameterizedBodyModel();
	}
}
