package sra.jun.opengl.picking;

import sra.smalltalk.*;
import sra.jun.geometry.basic.*;
import sra.jun.opengl.objects.*;
import sra.jun.opengl.projection.*;

/**
 * JunOpenGLObjectPicker class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/11/12 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLObjectPicker.java,v 6.0 2003/10/28 10:52:19 nisinaka Exp $
 */
public class JunOpenGLObjectPicker extends sra.smalltalk.StObject {

	/**
	 * Select the 3dObject which 2dPoint shows through Projection among a set
	 * of 3dObject.
	 * 
	 * @param a2dPoint sra.jun.geometry.basic.Jun2dPoint
	 * @param aCollectionOfJunOpenGLDisplayObject
	 *        sra.jun.opengl.objects.JunOpenGL3dObject[]
	 * @param aJunOpenGLProjection
	 *        sra.jun.opengl.projection.JunOpenGLProjection
	 * 
	 * @return ra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject PickObjectAt_from_projection_(Jun2dPoint a2dPoint, JunOpenGL3dObject[] aCollectionOfJunOpenGLDisplayObject, JunOpenGLProjection aJunOpenGLProjection) {
		return new JunOpenGLObjectPicker().pickObjectAt_from_projection_(a2dPoint, aCollectionOfJunOpenGLDisplayObject, aJunOpenGLProjection);
	}

	/**
	 * Select the 3dObject which 2dPoint shows through Projection among a set
	 * of 3dObject.
	 * 
	 * @param a2dPoint sra.jun.geometry.basic.Jun2dPoint
	 * @param aJunOpenGL3dCompoundDisplayObject
	 *        sra.jun.opengl.objects.JunOpenGLPickingObjects
	 * @param aJunOpenGLProjection
	 *        sra.jun.opengl.projection.JunOpenGLProjection
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public static JunOpenGL3dObject PickObjectAt_fromCompound_projection_(Jun2dPoint a2dPoint, JunOpenGLPickingObjects aJunOpenGL3dCompoundDisplayObject, JunOpenGLProjection aJunOpenGLProjection) {
		return new JunOpenGLObjectPicker().pickObjectAt_fromCompound_projection_(a2dPoint, aJunOpenGL3dCompoundDisplayObject, aJunOpenGLProjection);
	}

	/**
	 * Answer the picking object from 3dObjects on Projection.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * @param aCollectionOfJunOpenGLDisplayObject
	 *        sra.jun.opengl.objects.JunOpenGL3dObject[]
	 * @param aJunOpenGLProjection
	 *        sra.jun.opengl.projection.JunOpenGLProjection
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject pickObjectAt_from_projection_(Jun2dPoint aPoint, JunOpenGL3dObject[] aCollectionOfJunOpenGLDisplayObject, JunOpenGLProjection aJunOpenGLProjection) {
		return this.pickObjectFrom_projection_(aCollectionOfJunOpenGLDisplayObject, aJunOpenGLProjection.asPickingProjectionAt_(aPoint));
	}

	/**
	 * Answer the picking object from 3dCompoundObject on Projection.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * @param aJunOpenGL3dCompoundDisplayObject
	 *        sra.jun.opengl.picking.JunOpenGLPickingObjects
	 * @param aJunOpenGLProjection
	 *        sra.jun.opengl.projection.JunOpenGLProjection
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject pickObjectAt_fromCompound_projection_(Jun2dPoint aPoint, JunOpenGLPickingObjects aJunOpenGL3dCompoundDisplayObject, JunOpenGLProjection aJunOpenGLProjection) {
		return this.pickObjectFromCompound_projection_(aJunOpenGL3dCompoundDisplayObject, aJunOpenGLProjection.asPickingProjectionAt_(aPoint));
	}

	/**
	 * Answer the picking object.
	 * 
	 * @param aCollectionOfJunOpenGL3dObject
	 *        sra.jun.opengl.objects.JunOpenGL3dObject[]
	 * @param aJunOpenGLProjection
	 *        sra.jun.opengl.projection.JunOpenGLProjection
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject pickObjectFrom_projection_(JunOpenGL3dObject[] aCollectionOfJunOpenGL3dObject, JunOpenGLProjection aJunOpenGLProjection) {
		int pickedIndex;
		final JunOpenGL3dObject[] objects = aCollectionOfJunOpenGL3dObject;
		final JunOpenGLPickingRenderingContext pickingRenderingContext = new JunOpenGLPickingRenderingContext();
		synchronized (pickingRenderingContext.OpenGLInterface()) {
			pickingRenderingContext.gljInitialize();
			pickingRenderingContext.gljCurrent();
			pickingRenderingContext.selectionBufferSize_(objects.length * 4);
			pickingRenderingContext.projection_(aJunOpenGLProjection);
			pickedIndex = pickingRenderingContext.selectNearestObjectWhile_(new StBlockClosure() {
				public Object value() {
					for (int index = 0; index < objects.length; index++) {
						pickingRenderingContext.setId_(index);
						objects[index].renderOn_(pickingRenderingContext);
					}

					return null;
				}
			});
			pickingRenderingContext.gljFlush();
			pickingRenderingContext.gljRelease();
		}

		return (0 <= pickedIndex) ? objects[pickedIndex] : null;
	}

	/**
	 * Answer the picking object.
	 * 
	 * @param aJunOpenGL3dCompoundObject
	 *        sra.jun.opengl.picking.JunOpenGLPickingObjects
	 * @param aJunOpenGLProjection
	 *        sra.jun.opengl.projection.JunOpenGLProjection
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject pickObjectFromCompound_projection_(JunOpenGLPickingObjects aJunOpenGL3dCompoundObject, JunOpenGLProjection aJunOpenGLProjection) {
		final JunOpenGL3dObject[] objects = aJunOpenGL3dCompoundObject.components();
		final StValueHolder pickedIndex_ = new StValueHolder();
		final JunOpenGLPickingRenderingContext pickingRenderingContext = new JunOpenGLPickingRenderingContext();
		synchronized (pickingRenderingContext.OpenGLInterface()) {
			pickingRenderingContext.gljInitialize();
			pickingRenderingContext.gljCurrent();
			pickingRenderingContext.selectionBufferSize_(objects.length * 4);
			pickingRenderingContext.projection_(aJunOpenGLProjection);
			pickingRenderingContext.productTransformation_while_(aJunOpenGL3dCompoundObject.transformation(), new StBlockClosure() {
				public Object value() {
					int pickedIndex = pickingRenderingContext.selectNearestObjectWhile_(new StBlockClosure() {
						public Object value() {
							for (int index = 0; index < objects.length; index++) {
								pickingRenderingContext.setId_(index);
								objects[index].renderOn_(pickingRenderingContext);
							}

							return null;
						}
					});
					pickedIndex_.value_(new Integer(pickedIndex));

					return null;
				}
			});
			pickingRenderingContext.gljFlush();
			pickingRenderingContext.gljRelease();
		}

		int pickedIndex = ((Number) pickedIndex_.value()).intValue();

		return (0 <= pickedIndex) ? objects[pickedIndex] : null;
	}
}
