package sra.jun.opengl.projection;

import java.lang.Math;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.opengl.support.JunOpenGLRenderingContext;

/**
 * JunOpenGLParallelProjection class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/10/29 (by MATSUDA Ryouichi)
 *  @updated   1999/08/04 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLParallelProjection.java,v 6.0 2003/10/28 10:52:21 nisinaka Exp $
 */
public class JunOpenGLParallelProjection extends JunOpenGLProjection {
	/** The height is a double. */
	protected double height;

	/**
	 * Constainer.
	 */
	public JunOpenGLParallelProjection() {
		super();
	}

	/**
	 * Create a new default projection and answer it.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjection
	 */
	public static JunOpenGLProjection Default() {
		return EyePoint_sightPoint_upVector_height_near_far_(DefaultEyePoint(), DefaultSightPoint(), DefaultUpVector(), DefaultHeight(), DefaultNear(), DefaultFar());
	}

	/**
	 * Answer a default height.
	 * 
	 * @return double
	 */
	public static final double DefaultHeight() {
		return 13.681728885883d;
	}

	/**
	 * Create a new projection and set parameters and answer it.
	 * 
	 * @param a3dPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint3 sra.jun.geometry.basic.Jun3dPoint
	 * @param aNumber1 double
	 * @param aNumber2 double
	 * @param aNumber3 double
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLParallelProjection
	 */
	public static JunOpenGLParallelProjection EyePoint_sightPoint_upVector_height_near_far_(Jun3dPoint a3dPoint1, Jun3dPoint a3dPoint2, Jun3dPoint a3dPoint3, double aNumber1, double aNumber2, double aNumber3) {
		JunOpenGLParallelProjection projection;

		projection = new JunOpenGLParallelProjection();
		projection.setEyePoint_sightPoint_upVector_height_near_far_(a3dPoint1, a3dPoint2, a3dPoint3, aNumber1, aNumber2, aNumber3);

		return projection;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return Jun3dTransformation
	 */
	public Jun3dTransformation asEyeTransformation() {
		Jun3dPoint transformedSightPoint;
		Jun3dPoint transformedEyePoint;
		Jun3dPoint transformedUpVector;
		Jun3dPoint transformedRightVector;
		double scale;
		Jun3dPoint[][] alignPoints;

		transformedSightPoint = new Jun3dPoint(0, 0, 0);
		transformedEyePoint = new Jun3dPoint(0, 0, 1);
		transformedUpVector = new Jun3dPoint(0, 1, 0);
		transformedRightVector = new Jun3dPoint(1, 0, 0);
		scale = Math.abs(this.height()) / 2;
		alignPoints = new Jun3dPoint[][] { { this.sightPoint(), transformedSightPoint }, {
				(Jun3dPoint) this.sightPoint().minus_(this.unitSightVector().multipliedBy_(scale)), transformedEyePoint }, {
				(Jun3dPoint) this.sightPoint().plus_(this.unitUpVector().multipliedBy_(scale)), transformedUpVector }, {
				(Jun3dPoint) this.sightPoint().plus_(this.unitRightVector().multipliedBy_(scale)), transformedRightVector }
		};

		Jun3dTransformation transformation = Jun3dTransformation.AlignPoints_(alignPoints);

		return transformation;
	}

	/**
	 * Answer a projection as ParallelProjection.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLParallelProjection
	 */
	public JunOpenGLParallelProjection asParallelProjection() {
		return (JunOpenGLParallelProjection) this.copy();
	}

	/**
	 * Answer a projection as Perspective.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLPerspective
	 */
	public JunOpenGLPerspective asPerspective() {
		JunAngle fovy;

		fovy = JunAngle.FromRad_(Math.atan(this.height() / 2 / this.distance()) * 2);

		return PerspectiveEyePoint_sightPoint_upVector_fovy_near_far_(this.eyePoint(), this.sightPoint(), this.upVector(), fovy, this.near(), this.far());
	}

	/**
	 * Answer a transformation of receiver.
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public Jun3dTransformation asTransformation() {
		Jun3dPoint transformedSightPoint;
		Jun3dPoint transformedEyePoint;
		Jun3dPoint transformedUpVector;
		Jun3dPoint transformedRightVector;
		double scale;
		Jun3dPoint[][] alignPoints;

		transformedSightPoint = new Jun3dPoint(0, 0, 1);
		transformedEyePoint = new Jun3dPoint(0, 0, 0);
		transformedUpVector = new Jun3dPoint(0, 1, 0);
		transformedRightVector = new Jun3dPoint(1, 0, 0);
		scale = Math.abs(this.height()) / 2;
		alignPoints = new Jun3dPoint[][] { { this.sightPoint(), transformedSightPoint }, {
				this.eyePoint(), transformedEyePoint }, {
				(Jun3dPoint) this.sightPoint().plus_(this.unitUpVector().multipliedBy_(scale)), transformedUpVector }, {
				(Jun3dPoint) this.sightPoint().plus_(this.unitRightVector().multipliedBy_(scale)), transformedRightVector }
		};

		Jun3dTransformation transformation = Jun3dTransformation.AlignPoints_(alignPoints);

		return transformation;
	}

	/**
	 * Answer the height.
	 * 
	 * @return double
	 */
	public final double height() {
		return height;
	}

	/**
	 * Set number to the height.
	 * 
	 * @param aNumber double
	 */
	public final void height_(double aNumber) {
		height = aNumber;
	}

	/**
	 * Answer true if I am a parallel projection.
	 * 
	 * @return boolean
	 */
	public boolean isParallel() {
		return true;
	}

	/**
	 * Answer a 3dPoint, translate from 3dPoint.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint org_translateToPointFrom3dPoint_(Jun3dPoint a3dPoint) {
		Jun3dPoint transformedSightPoint;
		Jun3dPoint transformedEyePoint;
		Jun3dPoint transformedUpVector;
		Jun3dPoint transformedRightVector;
		double transformedNear;
		double transformedFar;
		double scale;
		Jun3dPoint[][] alignPoints;
		Jun3dTransformation transformation;
		Jun3dPoint transformed3dPoint;

		transformedSightPoint = new Jun3dPoint(0, 0, 1);
		transformedEyePoint = new Jun3dPoint(0, 0, 0);
		transformedUpVector = new Jun3dPoint(0, 1, 0);
		transformedRightVector = new Jun3dPoint(1, 0, 0);
		transformedNear = this.near() / this.distance();
		transformedFar = this.far() / this.distance();
		scale = Math.abs(this.height()) / 2;
		alignPoints = new Jun3dPoint[][] { { this.sightPoint(), transformedSightPoint }, {
				this.eyePoint(), transformedEyePoint }, {
				(Jun3dPoint) this.unitUpVector().multipliedBy_(scale), transformedUpVector }, {
				(Jun3dPoint) this.unitRightVector().multipliedBy_(scale), transformedRightVector }
		};
		transformation = Jun3dTransformation.AlignPoints_(alignPoints);
		transformed3dPoint = (Jun3dPoint) a3dPoint.transform_(transformation);

		if ((transformed3dPoint.z() < Math.abs(transformedNear)) || (transformed3dPoint.z() > Math.abs(transformedFar))) {
			return null;
		}

		return transformed3dPoint;
	}

	/**
	 * Set pan factor.
	 * 
	 * @param factor double
	 */
	public void pan_(double factor) {
		height = height * factor;
	}

	/**
	 * Project on a RenderingContext.
	 * 
	 * @param aRenderingContext
	 *        sra.jun.opengl.support.JunOpenGLRenderingContext
	 */
	public void projectOn_(JunOpenGLRenderingContext aRenderingContext) {
		aRenderingContext.parallelProjection_(this);
	}

	/**
	 * Answer a regular height.
	 * 
	 * @return double
	 */
	public double regularHeight() {
		return this.height();
	}

	/**
	 * Answer a 3dPoint, translate from 2dPoint with depth.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * @param aNumber double
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint translateTo3dPointFromPoint_depth_(Jun2dPoint aPoint, double aNumber) {
		double scale;
		Jun3dPoint forward;
		Jun3dPoint right;
		Jun3dPoint up;

		scale = Math.abs(this.height()) / 2;
		forward = (Jun3dPoint) this.unitSightVector().multipliedBy_(aNumber);
		right = (Jun3dPoint) this.unitRightVector().multipliedBy_(aPoint.x()).multipliedBy_(scale);
		up = (Jun3dPoint) this.unitUpVector().multipliedBy_(aPoint.y()).multipliedBy_(scale);

		return (Jun3dPoint) this.eyePoint().plus_(forward).plus_(right).plus_(up);
	}

	/**
	 * Set zoom factor.
	 * 
	 * @param factor double
	 */
	public void zoom_(double factor) {
		height = height / factor;
	}

	/**
	 * Answer my zoom height.
	 *
	 * @return double
	 */
	public double zoomHeight() {
		return this.height();
	}

	/**
	 * Set zoom height.
	 * 
	 * @param aNumber double
	 */
	public void zoomHeight_(double aNumber) {
		this.height_(aNumber);
	}

	/**
	 * Set eye point with parameters.
	 * 
	 * @param a3dPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint3 sra.jun.geometry.basic.Jun3dPoint
	 * @param aNumber1 double
	 * @param aNumber2 double
	 * @param aNumber3 double
	 */
	protected void setEyePoint_sightPoint_upVector_height_near_far_(Jun3dPoint a3dPoint1, Jun3dPoint a3dPoint2, Jun3dPoint a3dPoint3, double aNumber1, double aNumber2, double aNumber3) {
		eyePoint = a3dPoint1;
		sightPoint = a3dPoint2;
		upVector = a3dPoint3;
		height = aNumber1;
		near = aNumber2;
		far = aNumber3;
		this.normalizeUpVector();
	}
}
