package sra.jun.opengl.projection;

import java.lang.Math;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.opengl.support.JunOpenGLRenderingContext;

/**
 * JunOpenGLPerspective class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/10/29 (by MATSUDA Ryouichi)
 *  @updated   1999/08/04 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLPerspective.java,v 6.0 2003/10/28 10:52:21 nisinaka Exp $
 */
public class JunOpenGLPerspective extends JunOpenGLProjection {
	/** The fovy is a JunAngle. */
	protected JunAngle fovy;

	/** My zoom height */
	protected double zoomHeight;

	/**
	 * Constainer.
	 */
	public JunOpenGLPerspective() {
		super();

		zoomHeight = Double.NaN;
	}

	/**
	 * Create a new default projection and answer it.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjection
	 */
	public static JunOpenGLProjection Default() {
		return EyePoint_sightPoint_upVector_fovy_near_far_(DefaultEyePoint(), DefaultSightPoint(), DefaultUpVector(), DefaultFovy(), DefaultNear(), DefaultFar());
	}

	/**
	 * Answer a default fovy.
	 * 
	 * @return sra.jun.geometry.basic.JunAngle
	 */
	public static JunAngle DefaultFovy() {
		return JunAngle.FromDeg_(15);
	}

	/**
	 * Create a new Perspective and set parameters and answer it.
	 * 
	 * @param a3dPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint3 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunAngle double
	 * @param aNumber1 double
	 * @param aNumber2 double
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLPerspective
	 */
	public static JunOpenGLPerspective EyePoint_sightPoint_upVector_fovy_near_far_(Jun3dPoint a3dPoint1, Jun3dPoint a3dPoint2, Jun3dPoint a3dPoint3, double aJunAngle, double aNumber1, double aNumber2) {
		JunOpenGLPerspective projection;

		projection = new JunOpenGLPerspective();
		projection.setEyePoint_sightPoint_upVector_fovy_near_far_(a3dPoint1, a3dPoint2, a3dPoint3, aJunAngle, aNumber1, aNumber2);

		return projection;
	}

	/**
	 * Create a new Perspective and set parameters and answer it.
	 * 
	 * @param a3dPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint3 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunAngle sra.jun.geometry.basic.JunAngle
	 * @param aNumber1 double
	 * @param aNumber2 double
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLPerspective
	 */
	public static JunOpenGLPerspective EyePoint_sightPoint_upVector_fovy_near_far_(Jun3dPoint a3dPoint1, Jun3dPoint a3dPoint2, Jun3dPoint a3dPoint3, JunAngle aJunAngle, double aNumber1, double aNumber2) {
		JunOpenGLPerspective projection;

		projection = new JunOpenGLPerspective();
		projection.setEyePoint_sightPoint_upVector_fovy_near_far_(a3dPoint1, a3dPoint2, a3dPoint3, aJunAngle, aNumber1, aNumber2);

		return projection;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return Jun3dTransformation
	 */
	public Jun3dTransformation asEyeTransformation() {
		return this.asParallelProjection().asEyeTransformation();
	}

	/**
	 * Answer a projection as ParallelProjection.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLParallelProjection
	 */
	public JunOpenGLParallelProjection asParallelProjection() {
		double height;

		height = this.fovy().div_(2).tan() * this.distance() * 2;

		return ParallelProjectionEyePoint_sightPoint_upVector_height_near_far_(this.eyePoint(), this.sightPoint(), this.upVector(), height, this.near(), this.far());
	}

	/**
	 * Answer a projection as Perspective.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLPerspective
	 */
	public JunOpenGLPerspective asPerspective() {
		return (JunOpenGLPerspective) this.copy();
	}

	/**
	 * Answer a transformation of receiver.
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public Jun3dTransformation asTransformation() {
		Jun3dPoint transformedSightPoint;
		Jun3dPoint transformedEyePoint;
		Jun3dPoint transformedUpVector;
		Jun3dPoint transformedRightVector;
		double scale;
		Jun3dPoint[][] alignPoints;
		Jun3dTransformation transformation;

		transformedSightPoint = new Jun3dPoint(0, 0, 0);
		transformedEyePoint = new Jun3dPoint(0, 0, -1);
		transformedUpVector = new Jun3dPoint(0, 1, 0);
		transformedRightVector = new Jun3dPoint(1, 0, 0);
		scale = this.distance() * Math.abs(this.fovy().div_(2).tan());
		alignPoints = new Jun3dPoint[][] { { this.sightPoint(), transformedSightPoint }, {
				this.eyePoint(), transformedEyePoint }, {
				(Jun3dPoint) this.sightPoint().plus_(this.unitUpVector().multipliedBy_(scale)), transformedUpVector }, {
				(Jun3dPoint) this.sightPoint().plus_(this.unitRightVector().multipliedBy_(scale)), transformedRightVector }
		};
		transformation = Jun3dTransformation.AlignPoints_(alignPoints);

		return (Jun3dTransformation) transformation.product_(Jun3dTransformation.Perspective_(transformedEyePoint));
	}

	/**
	 * Answer the fovy.
	 * 
	 * @return sra.jun.geometry.basic.JunAngle
	 */
	public final JunAngle fovy() {
		return fovy;
	}

	/**
	 * Answer true if I am a perspective projection.
	 * 
	 * @return boolean
	 */
	public boolean isPerspective() {
		return true;
	}

	/**
	 * Set pan factor.
	 * 
	 * @param factor double
	 */
	public void pan_(double factor) {
		this.zoom_(1 / factor);
	}

	/**
	 * Project on a RenderingContext.
	 * 
	 * @param aRenderingContext
	 *        sra.jun.opengl.support.JunOpenGLRenderingContext
	 */
	public void projectOn_(JunOpenGLRenderingContext aRenderingContext) {
		aRenderingContext.perspective_(this);
	}

	/**
	 * Answer a regular height.
	 * 
	 * @return double
	 */
	public double regularHeight() {
		return this.fovy().div_(2).tan() * this.distance() * 2;
	}

	/**
	 * Answer a 3dPoint, translate from 2dPoint with depth.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * @param aNumber double
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint translateTo3dPointFromPoint_depth_(Jun2dPoint aPoint, double aNumber) {
		double scale;
		Jun3dPoint forward;
		Jun3dPoint right;
		Jun3dPoint up;

		scale = aNumber * Math.abs(this.fovy().div_(2).tan());
		forward = (Jun3dPoint) this.unitSightVector().multipliedBy_(aNumber);
		right = (Jun3dPoint) this.unitRightVector().multipliedBy_(aPoint.x() * scale);
		up = (Jun3dPoint) this.unitUpVector().multipliedBy_(aPoint.y() * scale);

		return (Jun3dPoint) eyePoint.plus_(forward).plus_(right).plus_(up);
	}

	/**
	 * Set zoom factor.
	 * 
	 * @param factor double
	 */
	public void zoom_(double factor) {
		fovy = fovy.div_(factor);
		double min = 0.001;
		double max = 180 - min;
		if (fovy.deg() > max) {
			fovy = JunAngle.FromDeg_(max);
		}
		if (fovy.deg() < min) {
			fovy = JunAngle.FromDeg_(min);
		}
		zoomHeight = fovy.dividedBy_(2).tan() * this.distance() * 2;
	}

	/**
	 * Answer my zoom height.
	 *
	 * @return double
	 */
	public double zoomHeight() {
		if (Double.isNaN(zoomHeight)) {
			this.zoom_(1);
		}
		return zoomHeight;
	}

	/**
	 * Set zoom height.
	 * 
	 * @param aNumber double
	 */
	public void zoomHeight_(double aNumber) {
		zoomHeight = Math.max(aNumber, 0);
		fovy = JunAngle.FromRad_(Math.atan(aNumber / 2 / this.distance()) * 2);
		double min = 0.001;
		double max = 180 - min;
		if (fovy.deg() > max) {
			fovy = JunAngle.FromDeg_(max);
		}
		if (fovy.deg() < min) {
			fovy = JunAngle.FromDeg_(min);
		}
	}

	/**
	 * Set eye point with parameters.
	 * 
	 * @param a3dPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint3 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunAngle double
	 * @param aNumber1 double
	 * @param aNumber2 double
	 */
	protected void setEyePoint_sightPoint_upVector_fovy_near_far_(Jun3dPoint a3dPoint1, Jun3dPoint a3dPoint2, Jun3dPoint a3dPoint3, double aJunAngle, double aNumber1, double aNumber2) {
		eyePoint = a3dPoint1;
		sightPoint = a3dPoint2;
		upVector = a3dPoint3;
		fovy = JunAngle.FromDeg_(aJunAngle);
		near = aNumber1;
		far = aNumber2;
		this.normalizeUpVector();
	}

	/**
	 * Set eye point with parameters.
	 * 
	 * @param a3dPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint3 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunAngle sra.jun.geometry.basic.JunAngle
	 * @param aNumber1 double
	 * @param aNumber2 double
	 */
	protected void setEyePoint_sightPoint_upVector_fovy_near_far_(Jun3dPoint a3dPoint1, Jun3dPoint a3dPoint2, Jun3dPoint a3dPoint3, JunAngle aJunAngle, double aNumber1, double aNumber2) {
		eyePoint = a3dPoint1;
		sightPoint = a3dPoint2;
		upVector = a3dPoint3;
		fovy = aJunAngle;
		near = aNumber1;
		far = aNumber2;
		this.normalizeUpVector();
	}
}
