package sra.jun.opengl.projection;

import java.awt.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.curves.Jun3dLine;
import sra.jun.geometry.surfaces.JunPlane;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.opengl.support.JunOpenGLRenderingContext;
import sra.smalltalk.*;

/**
 * JunOpenGLProjection class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/10/27 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLProjection.java,v 6.0 2003/10/28 10:52:21 nisinaka Exp $
 */
public abstract class JunOpenGLProjection extends StObject {
	/** The eyePoint is a Jun3dPoint. */
	protected Jun3dPoint eyePoint;

	/** The sightPoint is a Jun3dPoint. */
	protected Jun3dPoint sightPoint;

	/** The upVector is a Jun3dPoint. */
	protected Jun3dPoint upVector;

	/** The near is a double. */
	protected double near;

	/** The far is a double. */
	protected double far;

	/** The viewFactor is a double. */
	protected double viewFactor = 50;

	/**
	 * Constainer.
	 */
	public JunOpenGLProjection() {
		super();
	}

	/**
	 * Create a new default projection and answer it.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjection
	 */
	public static JunOpenGLProjection Default() {
		return DefaultPerspective();
	}

	/**
	 * Answer a default eye point.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public static Jun3dPoint DefaultEyePoint() {
		return new Jun3dPoint(30, 30, 30);
	}

	/**
	 * Answer a default far.
	 * 
	 * @return double
	 */
	public static final double DefaultFar() {
		return 1000;
	}

	/**
	 * Answer a default near.
	 * 
	 * @return double
	 */
	public static final double DefaultNear() {
		return 0.001;
	}

	/**
	 * Answer a default ParallelProjection class.
	 * 
	 * @return java.lang.Class
	 */
	public static Class DefaultParallelProjectionClass() {
		return JunOpenGLParallelProjection.class;
	}

	/**
	 * Answer a default peojection of Perspective.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjection
	 */
	public static JunOpenGLProjection DefaultPerspective() {
		//return (JunOpenGLProjection) _Perform(DefaultPerspectiveClass(), "Default");
		return JunOpenGLPerspective.Default();
	}

	/**
	 * Answer a default Perspective class.
	 * 
	 * @return java.lang.Class
	 */
	public static Class DefaultPerspectiveClass() {
		return JunOpenGLPerspective.class;
	}

	/**
	 * Answer a default sight point.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public static Jun3dPoint DefaultSightPoint() {
		return Jun3dPoint.Zero();
	}

	/**
	 * Answer a default up vector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public static Jun3dPoint DefaultUpVector() {
		return new Jun3dPoint(0, 0, 1);
	}

	/**
	 * Create a new ParallelProjection and set parameters and answer it.
	 * 
	 * @param a3dPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint3 sra.jun.geometry.basic.Jun3dPoint
	 * @param aNumber1 double
	 * @param aNumber2 double
	 * @param aNumber3 double
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunOpenGLParallelProjection ParallelProjectionEyePoint_sightPoint_upVector_height_near_far_(Jun3dPoint a3dPoint1, Jun3dPoint a3dPoint2, Jun3dPoint a3dPoint3, double aNumber1, double aNumber2, double aNumber3) {
		//JunOpenGLParallelProjection projection = (JunOpenGLParallelProjection) _New(DefaultParallelProjectionClass());
		//projection.setEyePoint_sightPoint_upVector_height_near_far_(a3dPoint1, a3dPoint2, a3dPoint3, aNumber1, aNumber2, aNumber3);
		//return projection;
		return JunOpenGLParallelProjection.EyePoint_sightPoint_upVector_height_near_far_(a3dPoint1, a3dPoint2, a3dPoint3, aNumber1, aNumber2, aNumber3);
	}

	/**
	 * Create a new Perspective and set parameters and answer it.
	 * 
	 * @param a3dPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint3 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunAngle sra.jun.geometry.basic.JunAngle
	 * @param aNumber1 double
	 * @param aNumber2 double
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunOpenGLPerspective PerspectiveEyePoint_sightPoint_upVector_fovy_near_far_(Jun3dPoint a3dPoint1, Jun3dPoint a3dPoint2, Jun3dPoint a3dPoint3, JunAngle aJunAngle, double aNumber1, double aNumber2) {
		//JunOpenGLPerspective projection = (JunOpenGLPerspective) _New(DefaultPerspectiveClass());
		//projection.setEyePoint_sightPoint_upVector_fovy_near_far_(a3dPoint1, a3dPoint2, a3dPoint3, aJunAngle, aNumber1, aNumber2);
		//return projection;
		return JunOpenGLPerspective.EyePoint_sightPoint_upVector_fovy_near_far_(a3dPoint1, a3dPoint2, a3dPoint3, aJunAngle, aNumber1, aNumber2);
	}

	/**
	 * Apply to a 3dPoint.
	 * 
	 * @param anObject sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint applyTo_(Jun3dPoint anObject) {
		return (Jun3dPoint) anObject.transform_(this.asTransformation());
	}

	/**
	 * Apply to a 3dPoint.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint applyToPoint_(Jun3dPoint a3dPoint) {
		return this.translateToPointFrom3dPoint_(a3dPoint);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return Jun3dTransformation
	 */
	public abstract Jun3dTransformation asEyeTransformation();

	/**
	 * Answer a projection as ParallelProjection.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLParallelProjection
	 */
	public abstract JunOpenGLParallelProjection asParallelProjection();

	/**
	 * Answer a projection as Perspective.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLPerspective
	 */
	public abstract JunOpenGLPerspective asPerspective();

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjection
	 */
	public JunOpenGLProjection asPickingProjectionAt_(Jun2dPoint aPoint) {
		Jun3dPoint pickingCenter = this.translateTo3dPointFromPoint_(aPoint);
		double pickingZoomHeight = pickingCenter.minus_(this.translateTo3dPointFromPoint_((Jun2dPoint) aPoint.plus_(new Jun2dPoint(0, -0.01d)))).length();
		JunOpenGLProjection projection = (JunOpenGLProjection) this.copy();
		projection.sightPoint_(pickingCenter);

		if (this.isParallel()) {
			projection.eyePoint_((Jun3dPoint) this.eyePoint().plus_(pickingCenter).minus_(this.sightPoint()));
		}

		projection.zoomHeight_(pickingZoomHeight);

		return projection;
	}

	/**
	 * Answer a transformation of receiver.
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation
	 */
	public abstract Jun3dTransformation asTransformation();

	/**
	 * Answer a Block of boundary condition for a Rectangle.
	 * 
	 * @param aRectangle java.awt.Rectangle
	 * 
	 * @return sra.smalltalk.StBlockClosure
	 */
	public StBlockClosure boundaryConditionForRectangle_(Rectangle aRectangle) {
		final Jun3dPoint topLeft = this.translateTo3dPointInFarBoundaryPlaneFromPoint_(new Jun2dPoint(aRectangle.x, aRectangle.y));
		final Jun3dPoint topRight = this.translateTo3dPointInFarBoundaryPlaneFromPoint_(new Jun2dPoint(aRectangle.x + aRectangle.width, aRectangle.y));
		final Jun3dPoint bottomLeft = this.translateTo3dPointInFarBoundaryPlaneFromPoint_(new Jun2dPoint(aRectangle.x, aRectangle.y + aRectangle.height));
		final Jun3dPoint bottomRight = this.translateTo3dPointInFarBoundaryPlaneFromPoint_(new Jun2dPoint(aRectangle.x + aRectangle.width, aRectangle.y + aRectangle.height));
		final Jun3dPoint anotherTopLeft = this.translateTo3dPointInNearBoundaryPlaneFromPoint_(new Jun2dPoint(aRectangle.x, aRectangle.y));
		final Jun3dPoint anotherBottomRight = this.translateTo3dPointInNearBoundaryPlaneFromPoint_(new Jun2dPoint(aRectangle.x + aRectangle.width, aRectangle.y + aRectangle.height));
		final StBlockClosure leftBlock = this.boundaryConditionFrom3dPoint_and_and_includes_(topLeft, bottomLeft, anotherTopLeft, topRight);
		final StBlockClosure rightBlock = this.boundaryConditionFrom3dPoint_and_and_includes_(topRight, bottomRight, anotherBottomRight, topLeft);
		final StBlockClosure topBlock = this.boundaryConditionFrom3dPoint_and_and_includes_(topLeft, topRight, anotherTopLeft, bottomLeft);
		final StBlockClosure bottomBlock = this.boundaryConditionFrom3dPoint_and_and_includes_(bottomLeft, bottomRight, anotherBottomRight, topLeft);

		return new StBlockClosure() {
			public Object value_(Object o) {
				if (((Boolean) leftBlock.value_(o)).booleanValue()) {
					if (((Boolean) rightBlock.value_(o)).booleanValue()) {
						if (((Boolean) topBlock.value_(o)).booleanValue()) {
							if (((Boolean) bottomBlock.value_(o)).booleanValue()) {
								return Boolean.TRUE;
							}
						}
					}
				}

				return Boolean.FALSE;
			}
		};
	}

	/**
	 * Answer a Block of boundary condition from parameters.
	 * 
	 * @param a3dPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint3 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint4 sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.smalltalk.StBlockClosure
	 */
	public StBlockClosure boundaryConditionFrom3dPoint_and_and_includes_(Jun3dPoint a3dPoint1, Jun3dPoint a3dPoint2, Jun3dPoint a3dPoint3, Jun3dPoint a3dPoint4) {
		JunPlane plane;

		plane = new JunPlane(a3dPoint1, a3dPoint2, a3dPoint3);

		if (plane == null) {
			return new StBlockClosure() {
				public Object value_(Object o) {
					return Boolean.FALSE;
				}
			};
		}

		final double a = plane.a();
		final double b = plane.b();
		final double c = plane.c();
		final double d = plane.d();
		final double leftHand = (a * a3dPoint4.x()) + (b * a3dPoint4.y()) + (c * a3dPoint4.z()) + d;

		if (leftHand < 0) {
			return new StBlockClosure() {
				public Object value_(Object o) {
					Jun3dPoint p = (Jun3dPoint) o;

					return new Boolean(((a * p.x()) + (b * p.y()) + (c * p.z()) + d) < 0);
				}
			};
		} else if (leftHand > 0) {
			return new StBlockClosure() {
				public Object value_(Object o) {
					Jun3dPoint p = (Jun3dPoint) o;

					return new Boolean(((a * p.x()) + (b * p.y()) + (c * p.z()) + d) > 0);
				}
			};
		} else {
			return new StBlockClosure() {
				public Object value_(Object o) {
					Jun3dPoint p = (Jun3dPoint) o;

					return new Boolean(((a * p.x()) + (b * p.y()) + (c * p.z()) + d) == 0);
				}
			};
		}
	}

	/**
	 * Answer a distance.
	 * 
	 * @return double
	 */
	public double distance() {
		return sightPoint.minus_(eyePoint).abs().length();
	}

	/**
	 * Answer the eyePoint.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public final Jun3dPoint eyePoint() {
		return eyePoint;
	}

	/**
	 * Set a 3dPoint to the eyePoint.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public void eyePoint_(Jun3dPoint a3dPoint) {
		eyePoint = a3dPoint;
		this.fitDistance();
		this.normalizeUpVector();
	}

	/**
	 * Answer the far.
	 * 
	 * @return double
	 */
	public final double far() {
		return far;
	}

	/**
	 * Set a number to the far.
	 * 
	 * @param aNumber double
	 */
	public void far_(double aNumber) {
		far = aNumber;
	}

	/**
	 * Answer a far center.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint farCenter() {
		return (Jun3dPoint) eyePoint.plus_(this.unitSightVector().multipliedBy_(this.far()));
	}

	/**
	 * Answer true if I am a parallel projection.
	 * 
	 * @return boolean
	 */
	public boolean isParallel() {
		return false;
	}

	/**
	 * Answer true if I am a perspective projection.
	 * 
	 * @return boolean
	 */
	public boolean isPerspective() {
		return false;
	}

	/**
	 * Move the eyePoint to aNumber backward from the center
	 * 
	 * @param aNumber double
	 */
	public void moveBackward_(double aNumber) {
		this.eyePoint_((Jun3dPoint) eyePoint.minus_(this.unitSightVector().multipliedBy_(aNumber)));
	}

	/**
	 * Move the eyePoint to aNumber forward to the center
	 * 
	 * @param aNumber double
	 */
	public void moveForward_(double aNumber) {
		this.eyePoint_((Jun3dPoint) eyePoint.plus_(this.unitSightVector().multipliedBy_(aNumber)));
	}

	/**
	 * Move the eyePoint to aNumber backward from the center
	 * 
	 * @param aNumber double
	 */
	public void moveLeft_(double aNumber) {
		this.eyePoint_((Jun3dPoint) eyePoint.minus_(this.unitRightVector().multipliedBy_(aNumber)));
	}

	/**
	 * Move the eyePoint to aNumber backward from the center
	 * 
	 * @param aNumber double
	 */
	public void moveRight_(double aNumber) {
		this.eyePoint_((Jun3dPoint) eyePoint.plus_(this.unitRightVector().multipliedBy_(aNumber)));
	}

	/**
	 * Answer the near.
	 * 
	 * @return double
	 */
	public final double near() {
		return near;
	}

	/**
	 * Set a number to the near.
	 * 
	 * @param aNumber double
	 */
	public final void near_(double aNumber) {
		near = aNumber;
	}

	/**
	 * Answer a 3dPoint of near center.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint nearCenter() {
		return (Jun3dPoint) eyePoint.plus_(this.unitSightVector().multipliedBy_(this.near()));
	}

	/**
	 * Pan.
	 */
	public void pan() {
		this.pan_(2);
	}

	/**
	 * Pan with factor.
	 * 
	 * @param factor double
	 */
	public abstract void pan_(double factor);

	/**
	 * Project on a RenderingContext.
	 * 
	 * @param aRenderingContext
	 *        sra.jun.opengl.support.JunOpenGLRenderingContext
	 */
	public abstract void projectOn_(JunOpenGLRenderingContext aRenderingContext);

	/**
	 * Answer a regular height.
	 * 
	 * @return double
	 */
	public abstract double regularHeight();

	/**
	 * Answer a right vector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint rightVector() {
		return this.sightVector().product_(this.upVector());
	}

	/**
	 * Set a 3dPoint as right vector.
	 * 
	 * @param aJun3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public void rightVector_(Jun3dPoint aJun3dPoint) {
		this.upVector_((Jun3dPoint) aJun3dPoint.product_(this.sightVector()).unitVector());
	}

	/**
	 * Answer the sightPoint.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public final Jun3dPoint sightPoint() {
		return sightPoint;
	}

	/**
	 * Set a 3dPoint to sightPoint.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public void sightPoint_(Jun3dPoint a3dPoint) {
		sightPoint = a3dPoint;
		this.fitDistance();
		this.normalizeUpVector();
	}

	/**
	 * Answer a sight vector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint sightVector() {
		return (Jun3dPoint) sightPoint.minus_(eyePoint);
	}

	/**
	 * Answer a 3dLine of translate from a 2dPoint.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * 
	 * @return sra.jun.geometry.curves.Jun3dLine
	 */
	public Jun3dLine translateTo3dLineFromPoint_(Jun2dPoint aPoint) {
		Jun3dPoint from;
		Jun3dPoint to;

		from = this.translateTo3dPointFromPoint_depth_(aPoint, this.near());
		to = this.translateTo3dPointFromPoint_depth_(aPoint, this.far());

		return new Jun3dLine(from, to);
	}

	/**
	 * Answer a 3dPoint of translate from 2dPoint.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint translateTo3dPointFromPoint_(Jun2dPoint aPoint) {
		return this.translateTo3dPointFromPoint_depth_(aPoint, this.distance());
	}

	/**
	 * Answer a 3dPoint, translate from 2dPoint with depth.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * @param aNumber double
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public abstract Jun3dPoint translateTo3dPointFromPoint_depth_(Jun2dPoint aPoint, double aNumber);

	/**
	 * Answer a 3dPoint of translate in far boundary plane from a 2dPoint.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint translateTo3dPointInFarBoundaryPlaneFromPoint_(Jun2dPoint aPoint) {
		return this.translateTo3dPointFromPoint_depth_(aPoint, this.far());
	}

	/**
	 * Answer a 3dPoint of translate in near boundary plane from a 2dPoint.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun2dPoint
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint translateTo3dPointInNearBoundaryPlaneFromPoint_(Jun2dPoint aPoint) {
		return this.translateTo3dPointFromPoint_depth_(aPoint, this.near());
	}

	/**
	 * Answer a 3dPoint of translate from a 3dPoint.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint translateToPointFrom3dPoint_(Jun3dPoint a3dPoint) {
		return (Jun3dPoint) a3dPoint.transform_(this.asTransformation());
	}

	/**
	 * Answer a 3dPoint as unit rightVector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint unitRightVector() {
		Jun3dPoint rightVector;

		rightVector = this.rightVector();

		return (Jun3dPoint) rightVector.dividedBy_(rightVector.length());
	}

	/**
	 * Answer a 3dPoint as unit sightVector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint unitSightVector() {
		Jun3dPoint sightVector;

		sightVector = this.sightVector();

		return (Jun3dPoint) sightVector.dividedBy_(sightVector.length());
	}

	/**
	 * Answer a 3dPoint as unit upVector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint unitUpVector() {
		return (Jun3dPoint) this.upVector().dividedBy_(this.upVector().length());
	}

	/**
	 * Answer the upVector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public final Jun3dPoint upVector() {
		return upVector;
	}

	/**
	 * Set a 3dPoint to the upVector.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public void upVector_(Jun3dPoint a3dPoint) {
		upVector = a3dPoint;
		this.normalizeUpVector();
	}

	/**
	 * Answer the viewFactor.
	 * 
	 * @return double
	 */
	public final double viewFactor() {
		return viewFactor;
	}

	/**
	 * Set a double value to the viewFactor.
	 * 
	 * @param factor double
	 */
	public void viewFactor_(double factor) {
		viewFactor = factor;
		this.fitDistance();
		this.normalizeUpVector();
	}

	/**
	 * Zoom.
	 */
	public void zoom() {
		this.zoom_(2);
	}

	/**
	 * Zoom with factor.
	 * 
	 * @param factor double
	 */
	public abstract void zoom_(double factor);

	/**
	 * Answer my zoom height.
	 *
	 * @return double
	 */
	public abstract double zoomHeight();

	/**
	 * Zoom height.
	 * 
	 * @param aNumber double
	 */
	public abstract void zoomHeight_(double aNumber);

	/**
	 * Fit distance.
	 */
	protected void fitDistance() {
		double factor;

		factor = this.viewFactor();
		far = this.distance() * factor;
		near = this.distance() / factor;
	}

	/**
	 * Set normalize to the upVector.
	 */
	protected void normalizeUpVector() {
		upVector = (Jun3dPoint) this.rightVector().product_(this.sightVector()).unitVector();
	}
}
