package sra.jun.opengl.rotation;

import java.awt.*;
import sra.smalltalk.*;
import sra.jun.goodies.cursors.JunCursors;
import sra.jun.opengl.display.JunOpenGLEmbeddedDisplayModel;
import sra.jun.opengl.objects.JunOpenGL3dObject;
import sra.jun.topology.elements.JunBody;

/**
 * JunOpenGLRotationApplicationModel class
 * 
 *  @author    nisinaka
 *  @created   1998/12/09 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLRotationApplicationModel.java,v 6.0 2003/10/28 10:52:21 nisinaka Exp $
 */
public class JunOpenGLRotationApplicationModel extends StApplicationModel {
	/** The instance of JunOpenGLRotationModel. */
	protected JunOpenGLRotationModel rotationModel = null;

	/** The instance of JunOpenGLEmbeddedDisplayModel. */
	protected JunOpenGLEmbeddedDisplayModel displayModel = new JunOpenGLEmbeddedDisplayModel();

	/** The number of divisions. */
	protected int divisions = 10;

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunOpenGLRotationApplicationViewAwt(this);
		} else {
			return new JunOpenGLRotationApplicationViewSwing(this);
		}
	}

	/**
	 * Answer the instance of JunOpenGLEmbeddedDisplayModel.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLEmbeddedDisplayModel
	 */
	public final JunOpenGLEmbeddedDisplayModel displayModel() {
		return displayModel;
	}

	/**
	 * Answer the integer value as a division number.
	 * 
	 * @return int
	 */
	public int divisions() {
		return divisions;
	}

	/**
	 * Set the number of divisions.
	 * 
	 * @param anInteger int
	 */
	public void divisions_(int anInteger) {
		divisions = anInteger;
		this.changed_($("divisions"));
	}

	/**
	 * Answer the instance of JunOpenGLRotationModel.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLRotationModel
	 */
	public JunOpenGLRotationModel rotationModel() {
		if (rotationModel == null) {
			rotationModel = new JunOpenGLRotationModel();
			rotationModel.openBlock_(new StBlockClosure() {
				public Object value_(Object anObject) {
					String aFilename = (String) anObject;
					JunOpenGLRotationApplicationModel anApplicationModel = new JunOpenGLRotationApplicationModel();

					//					anApplicationModel.rotationModel().readFromLRT10(aFilename);
					if (anApplicationModel.rotationModel().numVertices() > 0) {
						anApplicationModel.open();
					}

					return null;
				}
			});
			rotationModel.addDependent_(this);
		}

		return rotationModel;
	}

	/**
	 * Update the receiver according to the change notification from the model.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if (evt.getAspect() == $("polyline")) {
			this.divisions_(this.rotationModel().lastDivisions());
			this.displayModel().displayObject_(null);
		}
	}

	/**
	 * Rotate the contents of the rotation model, create a body, and show it on
	 * a display model.
	 */
	protected void doRotate() {
		JunCursors cursor = new JunCursors(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		try {
			cursor._show();

			JunBody aBody = this.rotationModel.rotatedBody_(this.divisions());
			if (aBody != null) {
				JunOpenGL3dObject displayObject = aBody.asJunOpenGL3dObject();
				displayObject.paint_(Color.white);
				this.displayModel().displayObject_(displayObject);
			}
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Create the windows for the receiver. .
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "OpenGL Rotation Application";
	}
}
