package sra.jun.opengl.rotation;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import sra.jun.opengl.display.*;
import sra.smalltalk.*;

/**
 * JunOpenGLRotationApplicationViewSwing class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/06/02 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLRotationApplicationViewSwing.java,v 6.0 2003/10/28 10:52:21 nisinaka Exp $
 */
public class JunOpenGLRotationApplicationViewSwing extends StViewJPanel implements JunOpenGLRotationApplicationView, ActionListener {
	protected JunOpenGLRotationViewSwing rotationView;
	protected JTextField divisionsField;
	protected JButton rotateButton;
	protected JunOpenGLDisplayViewSwing displayView;

	/**
	 * Create a new instance of JunOpenGLRotationApplicationViewSwing and initialize it.
	 *
	 * @param aModel sra.jun.opengl.rotation.JunOpenGLRotationApplicationModel
	 */
	public JunOpenGLRotationApplicationViewSwing(JunOpenGLRotationApplicationModel aModel) {
		super(aModel);

		displayView.getOpenGLView().setPopupMenu(aModel.displayModel().menuModel());
		divisionsField.setText(String.valueOf(aModel.divisions()));
	}

	/**
	 * Actions for the event.
	 * 
	 * @param e java.awt.event.ActionEvent
	 */
	public void actionPerformed(ActionEvent e) {
		try {
			int divisions = this.divisions();
			this.getModel().divisions_(divisions);
			this.getModel().doRotate();
		} catch (NumberFormatException ex) {
			this.getModel().divisions_(this.getModel().rotationModel().lastDivisions());
		}
	}

	/**
	 * Answer insets.
	 * 
	 * @return java.awt.Insets
	 */
	public Insets getInsets() {
		return new Insets(4, 4, 4, 4);
	}

	/**
	 * model property accessing
	 * 
	 * @return sra.jun.opengl.rotation.JunOpenGLRotationApplicationModel
	 */
	public JunOpenGLRotationApplicationModel getModel() {
		return (JunOpenGLRotationApplicationModel) this.model();
	}

	/**
	 * model property accessing
	 * 
	 * @param aModel sra.jun.opengl.rotation.JunOpenGLRotationApplicationModel
	 */
	public void setModel(JunOpenGLRotationApplicationModel aModel) {
		this.model_(aModel);
	}

	/**
	 * Update the receiver according to the change notification from the model.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if (evt.getAspect() == $("divisions")) {
			this.divisions_(this.getModel().divisions());
		}
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		JunOpenGLRotationApplicationModel aModel = this.getModel();

		// Left side pane.
		rotationView = new JunOpenGLRotationViewSwing(aModel.rotationModel());
		JPanel borderPanel = StApplicationModel._JPanelWithBorder();
		borderPanel.setLayout(new BorderLayout());
		borderPanel.add(rotationView, BorderLayout.CENTER);

		JPanel subSubPanel = new JPanel();
		subSubPanel.setLayout(new FlowLayout());
		subSubPanel.add(new JLabel("Divisions:"));
		divisionsField = new JTextField("", 5);
		subSubPanel.add(divisionsField);
		rotateButton = new JButton("Rotate");
		rotateButton.addActionListener(this);
		subSubPanel.add(rotateButton);

		JPanel subPanel = new JPanel(new BorderLayout());
		subPanel.add(borderPanel, BorderLayout.CENTER);
		subPanel.add(subSubPanel, BorderLayout.SOUTH);

		// Right side pane.
		displayView = new JunOpenGLDisplayViewSwing(aModel.displayModel());

		// Layout components.
		this.setLayout(new GridLayout(1, 2));
		((GridLayout) this.getLayout()).setHgap(8);
		this.add(subPanel);
		this.add(displayView);
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunOpenGLRotationApplicationModel();
	}

	/**
	 * Get the number entered in the text field.
	 * 
	 * @return int
	 */
	protected int divisions() {
		return Integer.valueOf(divisionsField.getText()).intValue();
	}

	/**
	 * Set the number entered in the text field.
	 * 
	 * @param anInteger int
	 */
	protected void divisions_(int anInteger) {
		divisionsField.setText(String.valueOf(anInteger));
	}
}
