package sra.jun.opengl.rotation;

import java.awt.*;
import javax.swing.*;
import sra.jun.geometry.basic.Jun2dPoint;
import sra.jun.geometry.curves.Jun2dLine;
import sra.smalltalk.*;

/**
 * JunOpenGLRotationViewSwing class
 * 
 *  @author    nisinaka
 *  @created   1998/12/09 (by nisinaka)
 *  @updated   2000/01/20 (by MATSUDA Ryouichi)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLRotationViewSwing.java,v 6.0 2003/10/28 10:52:21 nisinaka Exp $
 */
public class JunOpenGLRotationViewSwing extends StViewJPanel implements JunOpenGLRotationView {
	//
	protected static final Color _DarkGreenColor = new Color(0, 127, 0);
	protected static final Color _RoyalBlueColor = new Color(127, 127, 255);
	protected JPopupMenu _popupMenu;

	/**
	 * Create a new instance of JunOpenGLRotationViewSwing and initialize it.
	 *
	 * @param aModel sra.jun.opengl.rotation.JunOpenGLRotationModel
	 */
	public JunOpenGLRotationViewSwing(JunOpenGLRotationModel aModel) {
		super(aModel);
	}

	/**
	 * Answer the de-scaled vertex at the specified Point.
	 * 
	 * @param aPoint java.awt.Point
	 * 
	 * @return sra.jun.geometry.basic.Jun2dPoint
	 */
	public Jun2dPoint descaledPoint_(Point aPoint) {
		Point centerPoint = this.centerPoint();

		return this.getModel().descaledPoint_(new Point(aPoint.x - centerPoint.x, aPoint.y - centerPoint.y));
	}

	/**
	 * Display the receiver on the Graphics.
	 * 
	 * @param graphics java.awt.Graphics
	 */
	public void displayOn_(final Graphics graphics) {
		final JunOpenGLRotationViewSwing this_ = this;

		if (!this.isShowing()) {
			return;
		}

		this.displayAxesOn_(graphics);
		this.getModel().edgesDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				Jun2dLine edge = (Jun2dLine) anObject;
				graphics.setColor(this_.edgeColorFor_(edge));

				Point fromPoint = this_.scaledPoint_((Jun2dPoint) edge.from());
				Point toPoint = this_.scaledPoint_((Jun2dPoint) edge.to());
				graphics.drawLine(fromPoint.x, fromPoint.y, toPoint.x, toPoint.y);

				return null;
			}
		});
		this.getModel().verticesDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				Jun2dPoint vertex = (Jun2dPoint) anObject;
				graphics.setColor(this_.vertexColorFor_(vertex));

				Point atPoint = this_.scaledPoint_(vertex);
				graphics.fillRect(atPoint.x - 1, atPoint.y - 1, 3, 3);

				return null;
			}
		});
	}

	/**
	 * model property accessing.
	 * 
	 * @return sra.jun.opengl.rotation.JunOpenGLRotationModel
	 */
	public JunOpenGLRotationModel getModel() {
		return (JunOpenGLRotationModel) this.model();
	}

	/**
	 * Answer the scaled point of the Jun2dPoint.
	 * 
	 * @param aJun2dPoint sra.jun.geometry.basic.Jun2dPoint
	 * 
	 * @return java.awt.Point
	 */
	public Point scaledPoint_(Jun2dPoint aJun2dPoint) {
		Point centerPoint = this.centerPoint();
		Point scaledPoint = this.getModel().scaledPoint_(aJun2dPoint);
		scaledPoint.x += centerPoint.x;
		scaledPoint.y += centerPoint.y;

		return scaledPoint;
	}

	/**
	 * Answer the scaled point of the vertex with the specified index.
	 * 
	 * @param anIndex int
	 * 
	 * @return java.awt.Point
	 */
	public Point scaledVertexAt_(int anIndex) {
		return this.scaledPoint_(this.getModel().vertexAt_(anIndex));
	}

	/**
	 * Answer the index of the vertex at the specified point. If there is no
	 * such vertex, return -1.
	 * 
	 * @param aPoint java.awt.Point
	 * 
	 * @return int
	 */
	public int scaledVertexIndex_(Point aPoint) {
		int size = this.getModel().numVertices();

		for (int i = 0; i < size; i++) {
			Point anotherPoint = this.scaledVertexAt_(i);

			if ((Math.abs(aPoint.x - anotherPoint.x) < 4) && (Math.abs(aPoint.y - anotherPoint.y) < 4)) {
				return i;
			}
		}

		return -1;
	}

	/**
	 * model property accessing.
	 * 
	 * @param aModel sra.jun.opengl.rotation.JunOpenGLRotationModel
	 */
	public void setModel(JunOpenGLRotationModel aModel) {
		this.model_(aModel);
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setPreferredSize(new Dimension(192, 192));
	}

	/**
	 * Answer the center point of the receiver's view.
	 * 
	 * @return java.awt.Point
	 */
	protected Point centerPoint() {
		Rectangle box = new Rectangle(0, 0, this.getSize().width, this.getSize().height);

		if (this.getModel().swapXY()) {
			return new Point(box.x + (box.width / 4), box.y + (box.height / 2));
		} else {
			return new Point(box.x + (box.width / 2), box.y + ((box.height * 3) / 4));
		}
	}

	/**
	 * Answer my default controller.
	 * 
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new JunOpenGLRotationController();
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunOpenGLRotationModel();
	}

	/**
	 * Show axes on the receiver's view.
	 * 
	 * @param graphics java.awt.Graphics
	 */
	protected void displayAxesOn_(Graphics graphics) {
		JunOpenGLRotationModel model = this.getModel();
		Rectangle box = new Rectangle(0, 0, this.getSize().width, this.getSize().height);
		graphics.clipRect(box.x, box.y, box.width, box.height);
		graphics.setColor(Color.white);
		graphics.fillRect(box.x, box.y, box.width, box.height);

		Point center = this.centerPoint();
		int boxLeft = box.x;
		int boxRight = box.x + box.width;
		int boxTop = box.y;
		int boxBottom = box.y + box.height;

		if (Double.isNaN(model.grid()) == false) {
			graphics.setColor(this.gridColor());

			double scaledGrid = model.grid() * model.scale();
			int x;
			int y;

			for (int i = 1;; i++) {
				boolean flag = false;
				int delta = (int) (i * scaledGrid);

				if ((x = center.x - delta) >= boxLeft) {
					graphics.drawLine(x, boxTop, x, boxBottom);
					flag = true;
				}

				if ((x = center.x + delta) <= boxRight) {
					graphics.drawLine(x, boxTop, x, boxBottom);
					flag = true;
				}

				if ((y = center.y - delta) >= boxTop) {
					graphics.drawLine(boxLeft, y, boxRight, y);
					flag = true;
				}

				if ((y = center.y + delta) <= boxBottom) {
					graphics.drawLine(boxLeft, y, boxRight, y);
					flag = true;
				}

				if (flag == false) {
					break;
				}
			}
		}

		if (this.getModel().showXAxis() == true) {
			graphics.setColor(this.xAxisColor());
			graphics.drawLine(boxLeft, center.y, boxRight, center.y);
		}

		if (this.getModel().showYAxis() == true) {
			graphics.setColor(this.yAxisColor());
			graphics.drawLine(center.x, boxTop, center.x, boxBottom);
		}
	}

	/**
	 * Answer the color for an edge.
	 * 
	 * @return java.awt.Color
	 */
	protected Color edgeColor() {
		return Color.black;
	}

	/**
	 * Answer the color for the edge.
	 * 
	 * @param edge sra.jun.geometry.curves.Jun2dLine
	 * 
	 * @return java.awt.Color
	 */
	protected Color edgeColorFor_(Jun2dLine edge) {
		if (edge.to().equals(this.getModel().firstVertex())) {
			return this.loopEdgeColor();
		}

		if (edge.from().equals(this.getModel().lastVertex())) {
			return this.loopEdgeColor();
		}

		return this.edgeColor();
	}

	/**
	 * Answer the color for a first vertex.
	 * 
	 * @return java.awt.Color
	 */
	protected Color firstVertexColor() {
		return _RoyalBlueColor;
	}

	/**
	 * Answer the color for the grid.
	 * 
	 * @return java.awt.Color
	 */
	protected Color gridColor() {
		return Color.lightGray;
	}

	/**
	 * Answer the color for a last vertex.
	 * 
	 * @return java.awt.Color
	 */
	protected Color lastVertexColor() {
		return _DarkGreenColor;
	}

	/**
	 * Answer the color for a loop edge.
	 * 
	 * @return java.awt.Color
	 */
	protected Color loopEdgeColor() {
		return Color.gray;
	}

	/**
	 * Answer the popup menu if exist.
	 *
	 * @return javax.swing.JPopupMenu
	 */
	protected JPopupMenu popupMenu() {
		if (_popupMenu == null) {
			StMenuModel menuModel = this.getModel().menuModel();
			if (menuModel != null) {
				StMenu editMenu = (StMenu) menuModel.getItem("Edit");
				_popupMenu = StMenuViewForSwing.CreateJPopupMenu(editMenu);
				this.add(_popupMenu);
			}
		}
		return _popupMenu;
	}

	/**
	 * Answer the color for a selected vertex.
	 * 
	 * @return java.awt.Color
	 */
	protected Color selectedVertexColor() {
		return Color.red;
	}

	/**
	 * Answer the color for a vertex.
	 * 
	 * @return java.awt.Color
	 */
	protected Color vertexColor() {
		return Color.black;
	}

	/**
	 * Answer the color for the edge.
	 * 
	 * @param vertex sra.jun.geometry.basic.Jun2dPoint
	 * 
	 * @return java.awt.Color
	 */
	protected Color vertexColorFor_(Jun2dPoint vertex) {
		JunOpenGLRotationModel model = this.getModel();

		if (model.vertexSelected_(vertex)) {
			return this.selectedVertexColor();
		}

		if (vertex.equals(model.firstVertex())) {
			return this.firstVertexColor();
		}

		if (vertex.equals(model.lastVertex())) {
			return this.lastVertexColor();
		}

		return this.vertexColor();
	}

	/**
	 * Answer the color for X-axis.
	 * 
	 * @return java.awt.Color
	 */
	protected Color xAxisColor() {
		if (this.getModel().swapXY()) {
			return _DarkGreenColor;
		} else {
			return Color.blue;
		}
	}

	/**
	 * Answer the color for Y-axis.
	 * 
	 * @return java.awt.Color
	 */
	protected Color yAxisColor() {
		if (this.getModel().swapXY()) {
			return Color.blue;
		} else {
			return _DarkGreenColor;
		}
	}
}
