package sra.jun.opengl.support;

import java.awt.*;
import java.util.*;
import sra.smalltalk.*;
import sra.gl4jun.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.curves.*;
import sra.jun.geometry.surfaces.JunNurbsSurface;
import sra.jun.geometry.transformations.*;
import sra.jun.opengl.projection.*;
import sra.jun.opengl.texture.*;

/**
 * JunOpenGLRenderingContext class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1998/10/09 (by MATSUDA Ryouichi)
 *  @updated   1999/06/25 (by nisinaka)
 *  @updated   2000/01/06 (by nisinaka)
 *  @updated   2003/09/09 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun517 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLRenderingContext.java,v 6.0 2003/10/28 10:52:21 nisinaka Exp $
 */
public class JunOpenGLRenderingContext extends GLjRenderingContext {

	protected Color paint;
	protected float alpha = 1.0f;
	protected float lineWidth;
	protected float pointSize;
	protected JunOpenGLProjection projection;
	protected StSymbol shading = null;
	protected int lightIndex;
	protected Hashtable lightStates;
	protected float nurbsSamplingTolerance;
	protected StSymbol presentation = null;
	protected Jun3dTransformation transformation;
	protected JunOpenGLTexture texture;
	protected StBlockClosure textureCoordinateBlock;
	protected int textureCoordinateIndex = 0;

	/**
	 * Constainer Method
	 * 
	 * @param aCanvas DOCUMENT ME!
	 * @category Instance creation
	 */
	public JunOpenGLRenderingContext(Canvas aCanvas) {
		super(aCanvas);
		lightStates = new Hashtable();
		transformation = Jun3dTransformation.Unity();
	}

	/**
	 * Constainer Method
	 * 
	 * @param aGLjJCanvas DOCUMENT ME!
	 * @param aJFrame DOCUMENT ME!
	 * @category Instance creation
	 */
	public JunOpenGLRenderingContext(Container aGLjJCanvas, Frame aJFrame) {
		super(aGLjJCanvas, aJFrame);
		lightStates = new Hashtable();
		transformation = Jun3dTransformation.Unity();
	}

	/**
	 * Constainer Method
	 * 
	 * @param aImageSource DOCUMENT ME!
	 * @category Instance creation
	 */
	public JunOpenGLRenderingContext(GLjImageSource aImageSource) {
		super(aImageSource);
		lightStates = new Hashtable();
		transformation = Jun3dTransformation.Unity();
	}

	/**
	 * Replacement for the # notation of Smalltalk. :-)
	 * 
	 * @param aString java.lang.String
	 * @return sra.smalltalk.StSymbol
	 * @category Utilities
	 */
	public static final StSymbol $(String aString) {
		return StSymbol.Intern_(aString);
	}

	/**
	 * Answer the NormalUnitVector of 3d-points.
	 * 
	 * @param anArrayOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category Utilities
	 */
	public static Jun3dPoint NormalUnitVectorOfPoints_(Jun3dPoint[] anArrayOfPoints) {
		Jun3dPoint normalVector = NormalVectorOfPoints_(anArrayOfPoints);
		if (normalVector == null) {
			return null;
		}

		return normalVector.unitVector();
	}

	/**
	 * Answer the NormalVector of 3d-points.
	 * 
	 * @param anArrayOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category Utilities
	 */
	public static Jun3dPoint NormalVectorOfPoints_(Jun3dPoint[] anArrayOfPoints) {
		int size = anArrayOfPoints.length;
		if (size < 3) {
			return null;
		}

		int oneThirds = size / 3;
		Jun3dPoint point1 = (Jun3dPoint) anArrayOfPoints[0];
		Jun3dPoint point2 = (Jun3dPoint) ((Jun3dPoint) anArrayOfPoints[1]).plus_(oneThirds);
		Jun3dPoint point3 = (Jun3dPoint) ((Jun3dPoint) anArrayOfPoints[2]).plus_(oneThirds * 2);
		Jun3dPoint vector1 = new Jun3dPoint(point2.x() - point1.x(), point2.y() - point1.y(), point2.z() - point1.z());
		Jun3dPoint vector2 = new Jun3dPoint(point3.x() - point2.x(), point3.y() - point2.y(), point3.z() - point2.z());
		return vector1.product_(vector2);
	}

	/**
	 * Answer the alpha.
	 * 
	 * @return float
	 * @category accessing
	 */
	public float alpha() {
		return alpha;
	}

	/**
	 * Set the aplha.
	 * 
	 * @param aNumber
	 * @category accessing
	 */
	public void alpha_(float aNumber) {
		this.paint_alpha_(paint, aNumber);
	}

	/**
	 * Answer the current background color.
	 * 
	 * @return java.awt.Color
	 * @category accessing
	 */
	public Color background() {
		Color background = this.getBackground();
		if (background == null) {
			background = Color.white;
		}
		return background;
	}

	/**
	 * Call the glCallList() of OpenGL.
	 * 
	 * @param anInteger int
	 * @category display list
	 */
	public void callDisplayList_(final int anInteger) {
		OpenGLInterface.glCallList(anInteger);
	}

	/**
	 * Clear the DepthBuffer and ColorBuffer.
	 * 
	 * @category displaying
	 */
	public void clear() {
		// Smalltalk: initializeContext
		this.projection_(this.defaultProjection());
		this.enableDepthTest();
		this.enableColorMaterial();
		this.paint_alpha_(this.defaultPaint(), this.defaultAlpha());
		this.resetLightIndex();
		this.disableLighting();
		this.disableCullFace();
		this.enableNormalize();
		this.disableBlending();
		this.disableTexture2d();
		this.nurbsSamplingTolerance_(this.defaultNurbsSamplingTolerance());

		// Smalltalk: clear
		this.clearDepthBuffer();
		this.clearColorBuffer();

		// Java only: initialize color
		this.lightAmbientColor_(Color.BLACK);
		for (int index = 0; index <= 7; index++) {
			this.lightIndex_position_color_(index, Jun3dPoint.Zero(), Color.BLACK);
			this.disableLightIndex_(index);
		}
		this.disableLighting();
	}

	/**
	 * Clear the ColorBuffer of OpenGL with background color.
	 * 
	 * @category displaying
	 */
	public void clearColorBuffer() {
		Color background = this.background();
		OpenGLInterface.glClearColor((float) background.getRed() / 255.0f, (float) background.getGreen() / 255.0f, (float) background.getBlue() / 255.0f, 1.0f);
		OpenGLInterface.glClear(OpenGLInterface.GL_COLOR_BUFFER_BIT);
	}

	/**
	 * Clear the DepthBuffer of OpenGL.
	 * 
	 * @category displaying
	 */
	public void clearDepthBuffer() {
		OpenGLInterface.glClear(OpenGLInterface.GL_DEPTH_BUFFER_BIT);
	}

	/**
	 * Clear the Lights of OpenGL.
	 * 
	 * @category lighting
	 */
	public void clearLights() {
		for (int index = 1; index <= lightIndex; index++) {
			this.disableLightIndex_(index);
		}
		this.resetLightIndex();
	}

	/**
	 * Create the DisplayList of OpenGL and answer the number.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @param aBoolean boolean
	 * @return int
	 * @category display list
	 */
	public int createDisplayList_displayImmediately_(final StBlockClosure aBlock, final boolean aBoolean) {
		int listId = OpenGLInterface.glGenLists(1);
		OpenGLInterface.glNewList(listId, aBoolean ? OpenGLInterface.GL_COMPILE_AND_EXECUTE : OpenGLInterface.GL_COMPILE);
		try {
			aBlock.value();
		} finally {
			OpenGLInterface.glEndList();
		}
		return listId;
	}

	/**
	 * Call the glCullFace() of OpenGL.
	 * 
	 * @category culling
	 */
	public void cullFaceBack() {
		OpenGLInterface.glCullFace(OpenGLInterface.GL_BACK);
	}

	/**
	 * Call the glCullFace() of OpenGL.
	 * 
	 * @category culling
	 */
	public void cullFaceFront() {
		OpenGLInterface.glCullFace(OpenGLInterface.GL_FRONT);
	}

	/**
	 * Answer the defaultAlpha.
	 * 
	 * @return float
	 * @category defaults
	 */
	public float defaultAlpha() {
		return 1.0f;
	}

	/**
	 * Answer a 3d-point as defaultCenter.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category defaults
	 */
	public Jun3dPoint defaultCenter() {
		return Jun3dPoint.Zero();
	}

	/**
	 * Answer a 3d-point as defaultEyePoint.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category defaults
	 */
	public Jun3dPoint defaultEyePoint() {
		return new Jun3dPoint(0.0d, 0.0d, -4.0d);
	}

	/**
	 * Answer the defaultFar.
	 * 
	 * @return double
	 * @category defaults
	 */
	public double defaultFar() {
		return 100.0d;
	}

	/**
	 * Answer the defaultNear.
	 * 
	 * @return double
	 * @category defaults
	 */
	public double defaultNear() {
		return 0.1d;
	}

	/**
	 * Answer the defaultNurbsSamplingTolerance.
	 * 
	 * @return float
	 * @category defaults
	 */
	public float defaultNurbsSamplingTolerance() {
		return 10.0f;
	}

	/**
	 * Answer a Color as defaultPaint.
	 * 
	 * @return java.awt.Color
	 * @category defaults
	 */
	public Color defaultPaint() {
		return Color.black;
	}

	/**
	 * Answer a class of JunOpenGLParallelProjection.
	 * 
	 * @return java.lang.Class
	 * @category defaults
	 */
	public Class defaultParallelProjectionClass() {
		return JunOpenGLParallelProjection.class;
	}

	/**
	 * Answer a class of JunOpenGLPerspective.
	 * 
	 * @return java.lang.Class
	 * @category defaults
	 */
	public Class defaultPerspectiveClass() {
		return JunOpenGLPerspective.class;
	}

	/**
	 * Answer a defaultProjection.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjection
	 * @category defaults
	 */
	public JunOpenGLProjection defaultProjection() {
		return JunOpenGLProjection.Default();
	}

	/**
	 * Answer a 3d-point as defaultUpVector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category defaults
	 */
	public Jun3dPoint defaultUpVector() {
		return new Jun3dPoint(0.0d, 1.0d, 0.0d);
	}

	/**
	 * Disable the Blending of OpenGL.
	 * 
	 * @category blending
	 */
	public void disableBlending() {
		OpenGLInterface.glDisable(OpenGLInterface.GL_BLEND);
	}

	/**
	 * Disable the ColorMaterial of OpenGL.
	 * 
	 * @category material
	 */
	public void disableColorMaterial() {
		OpenGLInterface.glDisable(OpenGLInterface.GL_COLOR_MATERIAL);
	}

	/**
	 * Disable the CullFace of OpenGL.
	 * 
	 * @category culling
	 */
	public void disableCullFace() {
		OpenGLInterface.glDisable(OpenGLInterface.GL_CULL_FACE);
	}

	/**
	 * Disable the DepthMask of OpenGL.
	 * 
	 * @category z testing
	 */
	public void disableDepthMask() {
		OpenGLInterface.glDepthMask(false);
	}

	/**
	 * Disable the DepthTest of OpenGL.
	 * 
	 * @category z testing
	 */
	public void disableDepthTest() {
		OpenGLInterface.glDisable(OpenGLInterface.GL_DEPTH_TEST);
	}

	/**
	 * Disable the Dithering of OpenGL.
	 * 
	 * @category dithering
	 */
	public void disableDithering() {
		OpenGLInterface.glDisable(OpenGLInterface.GL_DITHER);
	}

	/**
	 * Disable the Fog of OpenGL.
	 * 
	 * @category foggin
	 */
	public void disableFog() {
		OpenGLInterface.glDisable(OpenGLInterface.GL_FOG);
	}

	/**
	 * Disable the Light at index of OpenGL.
	 * 
	 * @param index0to7 int
	 * @category lighting
	 */
	public void disableLightIndex_(final int index0to7) {
		int index = OpenGLInterface.GL_LIGHT0 + index0to7;
		lightStates.put(new Integer(index0to7), Boolean.FALSE);
		OpenGLInterface.glDisable(index);
	}

	/**
	 * Disable the Lighting of OpenGL.
	 * 
	 * @category lighting
	 */
	public void disableLighting() {
		OpenGLInterface.glDisable(OpenGLInterface.GL_LIGHTING);
	}

	/**
	 * Disable the LineSmooth of OpenGL.
	 * 
	 * @category smoothing
	 */
	public void disableLineSmooth() {
		OpenGLInterface.glDisable(OpenGLInterface.GL_LINE_SMOOTH);
	}

	/**
	 * Disable the LineStipple of OpenGL.
	 * 
	 * @category stippling
	 */
	public void disableLineStipple() {
		OpenGLInterface.glDisable(OpenGLInterface.GL_LINE_STIPPLE);
	}

	/**
	 * Disable the Normalize of OpenGL.
	 * 
	 * @category normalizing
	 */
	public void disableNormalize() {
		OpenGLInterface.glDisable(OpenGLInterface.GL_NORMALIZE);
	}

	/**
	 * Disable the PolygonSmooth of OpenGL.
	 * 
	 * @category smoothing
	 */
	public void disablePolygonSmooth() {
		OpenGLInterface.glDisable(OpenGLInterface.GL_POLYGON_SMOOTH);
	}

	/**
	 * Disable the PolygonStipple of OpenGL.
	 * 
	 * @category stippling
	 */
	public void disablePolygonStipple() {
		OpenGLInterface.glDisable(OpenGLInterface.GL_POLYGON_STIPPLE);
	}

	/**
	 * Disable the Texture-2d of OpenGL.
	 * 
	 * @category texture mapping
	 */
	public void disableTexture2d() {
		OpenGLInterface.glDisable(OpenGLInterface.GL_TEXTURE_2D);
	}

	/**
	 * Display 2d-point to OpenGL.
	 * 
	 * @param a2dPoint sra.jun.geometry.basic.Jun2dPoint
	 * @category displaying
	 */
	public void display2dPoint_(Jun2dPoint a2dPoint) {
		this.display2dPoints_(new Jun2dPoint[] { a2dPoint });
	}

	/**
	 * Display 2d-points to OpenGL.
	 * 
	 * @param aCollectionOfPoints sra.jun.geometry.basic.Jun2dPoint[]
	 * @category displaying
	 */
	public void display2dPoints_(final Jun2dPoint[] aCollectionOfPoints) {
		OpenGLInterface.glBegin(OpenGLInterface.GL_POINTS);
		for (int i = 0; i < aCollectionOfPoints.length; i++) {
			Jun2dPoint a2dPoint = aCollectionOfPoints[i];
			OpenGLInterface.glVertex2d(a2dPoint.x(), a2dPoint.y());
		}
		OpenGLInterface.glEnd();
	}

	/**
	 * Display hidden-lines to OpenGL.
	 * 
	 * @param aCollectionOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @category displaying
	 */
	public void displayHiddenLineRendering_(Jun3dPoint[] aCollectionOfPoints) {
		OpenGLInterface.glPolygonMode(OpenGLInterface.GL_FRONT_AND_BACK, OpenGLInterface.GL_LINE);
		OpenGLInterface.glBegin(OpenGLInterface.GL_LINE_LOOP);
		for (int i = 0; i < aCollectionOfPoints.length; i++) {
			Jun3dPoint a3dPoint = aCollectionOfPoints[i];
			OpenGLInterface.glVertex3d(a3dPoint.x(), a3dPoint.y(), a3dPoint.z());
		}
		OpenGLInterface.glEnd();

		this.disableLighting();
		Color savedPaint = this.paint();
		this.paint_(this.background());

		OpenGLInterface.glPolygonMode(OpenGLInterface.GL_FRONT_AND_BACK, OpenGLInterface.GL_FILL);
		OpenGLInterface.glEnable(OpenGLInterface.GL_POLYGON_OFFSET_FILL);
		OpenGLInterface.glPolygonOffset(1.0f, 1.0f);
		OpenGLInterface.glBegin(OpenGLInterface.GL_POLYGON);
		for (int i = 0; i < aCollectionOfPoints.length; i++) {
			Jun3dPoint a3dPoint = aCollectionOfPoints[i];
			OpenGLInterface.glVertex3d(a3dPoint.x(), a3dPoint.y(), a3dPoint.z());
		}
		OpenGLInterface.glEnd();
		OpenGLInterface.glDisable(OpenGLInterface.GL_POLYGON_OFFSET_FILL);

		this.paint_(savedPaint);
		this.enableLighting();
	}

	/**
	 * Display Line to OpenGL.
	 * 
	 * @param aCollectionOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @category displaying
	 */
	public void displayLine_(Jun3dPoint[] aCollectionOfPoints) {
		if (aCollectionOfPoints.length <= 2) {
			this.displayLines_(aCollectionOfPoints);
		} else {
			Jun3dPoint[] firstLast = { aCollectionOfPoints[0], aCollectionOfPoints[aCollectionOfPoints.length] };
			this.displayLines_(firstLast);
		}
	}

	/**
	 * Display LineLoop to OpenGL.
	 * 
	 * @param aCollectionOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @category displaying
	 */
	public void displayLineLoop_(final Jun3dPoint[] aCollectionOfPoints) {
		OpenGLInterface.glBegin(OpenGLInterface.GL_LINE_LOOP);
		for (int i = 0; i < aCollectionOfPoints.length; i++) {
			Jun3dPoint a3dPoint = aCollectionOfPoints[i];
			OpenGLInterface.glVertex3d(a3dPoint.x(), a3dPoint.y(), a3dPoint.z());
		}
		OpenGLInterface.glEnd();
	}

	/**
	 * Display Lines to OpenGL.
	 * 
	 * @param aCollectionOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @category displaying
	 */
	public void displayLines_(final Jun3dPoint[] aCollectionOfPoints) {
		OpenGLInterface.glBegin(OpenGLInterface.GL_LINE_STRIP);
		for (int i = 0; i < aCollectionOfPoints.length; i++) {
			Jun3dPoint a3dPoint = aCollectionOfPoints[i];
			OpenGLInterface.glVertex3d(a3dPoint.x(), a3dPoint.y(), a3dPoint.z());
		}
		OpenGLInterface.glEnd();
	}

	/**
	 * Display LineStrip to OpenGL.
	 * 
	 * @param aCollectionOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @category displaying
	 */
	public void displayLineStrip_(final Jun3dPoint[] aCollectionOfPoints) {
		Jun3dPoint normalVector = this.normalUnitVectorOfPoints_(aCollectionOfPoints);
		if (normalVector == null) {
			return;
		}

		OpenGLInterface.glBegin(OpenGLInterface.GL_LINE_STRIP);
		OpenGLInterface.glNormal3d(normalVector.x(), normalVector.y(), normalVector.z());
		for (int i = 0; i < aCollectionOfPoints.length; i++) {
			Jun3dPoint a3dPoint = aCollectionOfPoints[i];
			OpenGLInterface.glVertex3d(a3dPoint.x(), a3dPoint.y(), a3dPoint.z());
		}
		OpenGLInterface.glEnd();
	}

	/**
	 * Display NurbsCurve to OpenGL.
	 * 
	 * @param aNurbsCurve sra.jun.geometry.curves.JunNurbsCurve
	 * @category displaying
	 */
	public void displayNurbsCurve_(final JunNurbsCurve aNurbsCurve) {
		float[] knotVector = new float[aNurbsCurve.knotVector().length];
		for (int i = 0; i < aNurbsCurve.knotVector().length; i++) {
			knotVector[i] = (float) aNurbsCurve.knotVector()[i];
		}
		Jun3dPoint[] controlPoints = aNurbsCurve.controlPoints();
		double[] weights = aNurbsCurve.weights();
		float[] floatArray = new float[aNurbsCurve.controlPointSize() * 4];
		for (int i = 0; i < aNurbsCurve.controlPointSize(); i++) {
			Jun3dPoint point = controlPoints[i];
			double weight = weights[i];
			floatArray[(i * 4) + 0] = (float) (point.x() * weight);
			floatArray[(i * 4) + 1] = (float) (point.y() * weight);
			floatArray[(i * 4) + 2] = (float) (point.z() * weight);
			floatArray[(i * 4) + 3] = (float) weight;
		}
		OpenGLInterface.glEnable(OpenGLInterface.GL_AUTO_NORMAL);
		int aNurbs = OpenGLInterface.gluNewNurbsRenderer();
		OpenGLInterface.gluNurbsProperty(aNurbs, OpenGLInterface.GLU_SAMPLING_TOLERANCE, nurbsSamplingTolerance);
		OpenGLInterface.glPushMatrix();
		OpenGLInterface.gluBeginCurve(aNurbs);
		OpenGLInterface.gluNurbsCurve(aNurbs, knotVector.length, knotVector, 4, floatArray, aNurbsCurve.order(), OpenGLInterface.GL_MAP1_VERTEX_4);
		OpenGLInterface.gluEndCurve(aNurbs);
		OpenGLInterface.glPopMatrix();
		OpenGLInterface.gluDeleteNurbsRenderer(aNurbs);
	}

	/**
	 * Display NurbsSurface to OpenGL.
	 * 
	 * @param aNurbsSurface sra.jun.geometry.surfaces.JunNurbsSurface
	 * @category displaying
	 */
	public void displayNurbsSurface_(JunNurbsSurface aNurbsSurface) {
		if (this.presentation() == $("wireframe")) {
			this.displayNurbsSurface_displayMode_(aNurbsSurface, OpenGLInterface.GLU_OUTLINE_POLYGON);
			return;
		}

		if (this.presentation() == $("hiddenline")) {
			this.disableLighting();
			Color savedPaint = null;
			if (this.paint().equals(Color.white)) {
				savedPaint = this.paint();
				this.paint_(StColorValue.Brightness_(0.95));
			}
			OpenGLInterface.glEnable(OpenGLInterface.GL_DEPTH_TEST);
			OpenGLInterface.glPolygonMode(OpenGLInterface.GL_FRONT_AND_BACK, OpenGLInterface.GL_LINE);
			this.displayNurbsSurface_displayMode_(aNurbsSurface, OpenGLInterface.GLU_OUTLINE_POLYGON);
			if (savedPaint != null) {
				this.paint_(savedPaint);
			}
			savedPaint = this.paint();
			this.paint_(Color.white);
			OpenGLInterface.glPolygonMode(OpenGLInterface.GL_FRONT_AND_BACK, OpenGLInterface.GL_FILL);
			OpenGLInterface.glEnable(OpenGLInterface.GL_POLYGON_OFFSET_FILL);
			OpenGLInterface.glPolygonOffset(1.0f, 1.0f);
			this.displayNurbsSurface_displayMode_(aNurbsSurface, OpenGLInterface.GLU_FILL);
			OpenGLInterface.glDisable(OpenGLInterface.GL_POLYGON_OFFSET_FILL);
			this.paint_(savedPaint);
			this.enableLighting();
			return;
		}

		this.displayNurbsSurface_displayMode_(aNurbsSurface, OpenGLInterface.GLU_FILL);
	}

	/**
	 * Display NurbsSurfaceMesh to OpenGL.
	 * 
	 * @param aNurbsSurface sra.jun.geometry.surfaces.JunNurbsSurface
	 * @category displaying
	 */
	public void displayNurbsSurfaceMesh_(JunNurbsSurface aNurbsSurface) {
		this.displayNurbsSurface_displayMode_(aNurbsSurface, OpenGLInterface.GLU_OUTLINE_POLYGON);
	}

	/**
	 * Display Point to OpenGL.
	 * 
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @category displaying
	 */
	public void displayPoint_(Jun3dPoint a3dPoint) {
		this.displayPoints_(new Jun3dPoint[] { a3dPoint });
	}

	/**
	 * Display Points to OpenGL.
	 * 
	 * @param aCollectionOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @category displaying
	 */
	public void displayPoints_(final Jun3dPoint[] aCollectionOfPoints) {
		OpenGLInterface.glBegin(OpenGLInterface.GL_POINTS);
		for (int i = 0; i < aCollectionOfPoints.length; i++) {
			Jun3dPoint a3dPoint = aCollectionOfPoints[i];
			OpenGLInterface.glVertex3d(a3dPoint.x(), a3dPoint.y(), a3dPoint.z());
		}
		OpenGLInterface.glEnd();
	}

	/**
	 * Display Polygons to OpenGL.
	 * 
	 * @param aCollectionOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @category displaying
	 */
	public void displayPolygon_(final Jun3dPoint[] aCollectionOfPoints) {
		if (this.presentation() == $("wireframe")) {
			this.displaySeeThroughWireframe_(aCollectionOfPoints);
			return;
		}
		if (this.presentation() == $("hiddenline")) {
			this.displayHiddenLineRendering_(aCollectionOfPoints);
			return;
		}

		Jun3dPoint normalVector = this.normalVectorOfPoints_(aCollectionOfPoints);
		if (normalVector == null) {
			return;
		}

		this.displayPolygon_normalVector_(aCollectionOfPoints, normalVector);
	}

	/**
	 * Display Polygon with normal vector to OpenGL.
	 * 
	 * @param aCollectionOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @param normalVector sra.jun.geometry.basic.Jun3dPoint[]
	 * @category displaying
	 */
	public void displayPolygon_normalVector_(Jun3dPoint[] aCollectionOfPoints, Jun3dPoint normalVector) {
		this.displayPolygon_normalVectors_(aCollectionOfPoints, new Jun3dPoint[] { normalVector });
	}

	/**
	 * Display Polygon with normal vectors to OpenGL.
	 * 
	 * @param aCollectionOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @param normalVectors sra.jun.geometry.basic.Jun3dPoint[]
	 * @category displaying
	 */
	public void displayPolygon_normalVectors_(Jun3dPoint[] aCollectionOfPoints, Jun3dPoint[] normalVectors) {
		if (this.presentation() == $("wireframe")) {
			this.displaySeeThroughWireframe_(aCollectionOfPoints);
			return;
		}
		if (this.presentation() == $("hiddenline")) {
			this.displayHiddenLineRendering_(aCollectionOfPoints);
			return;
		}

		OpenGLInterface.glBegin(OpenGLInterface.GL_POLYGON);
		for (int i = 0; i < aCollectionOfPoints.length; i++) {
			Jun3dPoint a3dPoint = aCollectionOfPoints[i];
			if (textureCoordinateBlock != null) {
				Jun2dPoint a2dPoint = (Jun2dPoint) textureCoordinateBlock.value_(a3dPoint);
				if (a2dPoint != null) {
					OpenGLInterface.glTexCoord2d(a2dPoint.x(), a2dPoint.y());
				}
			}
			if (normalVectors != null && normalVectors.length > i) {
				Jun3dPoint normalVector = normalVectors[i];
				OpenGLInterface.glNormal3d(normalVector.x(), normalVector.y(), normalVector.z());
			}
			OpenGLInterface.glVertex3d(a3dPoint.x(), a3dPoint.y(), a3dPoint.z());
		}
		OpenGLInterface.glEnd();
	}

	/**
	 * Display Polygon with colors and normal vectors to OpenGL.
	 * 
	 * @param aCollectionOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @param normalVectors sra.jun.geometry.basic.Jun3dPoint
	 * @category displaying
	 */
	public void displayPolygon_colors_normalVectors_(Jun3dPoint[] aCollectionOfPoints, Color[] anArrayOfColors, Jun3dPoint[] normalVectors) {
		if (this.presentation() == $("wireframe")) {
			this.displaySeeThroughWireframe_(aCollectionOfPoints);
			return;
		}
		if (this.presentation() == $("hiddenline")) {
			this.displayHiddenLineRendering_(aCollectionOfPoints);
			return;
		}

		OpenGLInterface.glBegin(OpenGLInterface.GL_POLYGON);
		for (int i = 0; i < aCollectionOfPoints.length; i++) {
			Jun3dPoint a3dPoint = aCollectionOfPoints[i];
			if (textureCoordinateBlock != null) {
				Jun2dPoint a2dPoint = (Jun2dPoint) textureCoordinateBlock.value_(a3dPoint);
				if (a2dPoint != null) {
					OpenGLInterface.glTexCoord2d(a2dPoint.x(), a2dPoint.y());
				}
			}
			if (anArrayOfColors != null) {
				Color color = anArrayOfColors[i];
				OpenGLInterface.glColor4f((float) color.getRed() / 255.0f, (float) color.getGreen() / 255.0f, (float) color.getBlue() / 255.0f, alpha);
			}
			if (normalVectors != null && normalVectors.length > i) {
				Jun3dPoint normalVector = normalVectors[i];
				OpenGLInterface.glNormal3d(normalVector.x(), normalVector.y(), normalVector.z());
			}
			OpenGLInterface.glVertex3d(a3dPoint.x(), a3dPoint.y(), a3dPoint.z());
		}
		OpenGLInterface.glEnd();
	}

	/**
	 * Display Polyline to OpenGL.
	 * 
	 * @param aCollectionOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @category displaying
	 */
	public void displayPolyline_(Jun3dPoint[] aCollectionOfPoints) {
		this.displayLineStrip_(aCollectionOfPoints);
	}

	/**
	 * Display a see-through wireframe to OpenGL.
	 * 
	 * @param aCollectionOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @category displaying
	 */
	public void displaySeeThroughWireframe_(Jun3dPoint[] aCollectionOfPoints) {
		this.displayLineLoop_(aCollectionOfPoints);
	}

	/**
	 * Display Sphere with slices to OpenGL.
	 * 
	 * @param aNumber float
	 * @param anInteger int
	 * @category displaying
	 */
	public void displaySphere_slices_(final float aNumber, final int anInteger) {
		int sphere = OpenGLInterface.gluNewQuadric();
		OpenGLInterface.gluSphere(sphere, aNumber, anInteger, anInteger);
		OpenGLInterface.gluDeleteQuadric(sphere);
	}

	/**
	 * Enable the Blending of OpenGL.
	 * 
	 * @category blending
	 */
	public void enableBlending() {
		OpenGLInterface.glBlendFunc(OpenGLInterface.GL_SRC_ALPHA, OpenGLInterface.GL_ONE_MINUS_SRC_ALPHA);
		OpenGLInterface.glEnable(OpenGLInterface.GL_BLEND);
	}

	/**
	 * Enable the ColorMaterial of OpenGL.
	 * 
	 * @category material
	 */
	public void enableColorMaterial() {
		OpenGLInterface.glColorMaterial(OpenGLInterface.GL_FRONT_AND_BACK, OpenGLInterface.GL_AMBIENT_AND_DIFFUSE);
		OpenGLInterface.glEnable(OpenGLInterface.GL_COLOR_MATERIAL);
	}

	/**
	 * Enable the CullFace of OpenGL.
	 * 
	 * @category culling
	 */
	public void enableCullFace() {
		OpenGLInterface.glEnable(OpenGLInterface.GL_CULL_FACE);
	}

	/**
	 * Enable the DepthMask of OpenGL.
	 * 
	 * @category z testing
	 */
	public void enableDepthMask() {
		OpenGLInterface.glDepthMask(true);
	}

	/**
	 * Enable the DepthTest of OpenGL.
	 * 
	 * @category z testing
	 */
	public void enableDepthTest() {
		OpenGLInterface.glEnable(OpenGLInterface.GL_DEPTH_TEST);
	}

	/**
	 * Enable the Dithering of OpenGL.
	 * 
	 * @category dithering
	 */
	public void enableDithering() {
		OpenGLInterface.glEnable(OpenGLInterface.GL_DITHER);
	}

	/**
	 * Enable the Fog of OpenGL.
	 * 
	 * @category fogging
	 */
	public void enableFog() {
		OpenGLInterface.glEnable(OpenGLInterface.GL_FOG);
	}

	/**
	 * Enable the Light with index of OpenGL.
	 * 
	 * @param index0to7 int
	 * @category lighting
	 */
	public void enableLightIndex_(final int index0to7) {
		int index = OpenGLInterface.GL_LIGHT0 + index0to7;
		lightStates.put(new Integer(index0to7), Boolean.TRUE);
		OpenGLInterface.glEnable(index);
	}

	/**
	 * Enable the Lighting of OpenGL.
	 * 
	 * @category lighting
	 */
	public void enableLighting() {
		OpenGLInterface.glEnable(OpenGLInterface.GL_LIGHTING);
	}

	/**
	 * Enable the LineSmooth of OpenGL.
	 * 
	 * @category smoothing
	 */
	public void enableLineSmooth() {
		OpenGLInterface.glEnable(OpenGLInterface.GL_LINE_SMOOTH);
		OpenGLInterface.glHint(OpenGLInterface.GL_LINE_SMOOTH_HINT, OpenGLInterface.GL_NICEST);
	}

	/**
	 * Enable the LineStipple of OpenGL.
	 * 
	 * @category stippling
	 */
	public void enableLineStipple() {
		OpenGLInterface.glEnable(OpenGLInterface.GL_LINE_STIPPLE);
	}

	/**
	 * Enable the Normalize of OpenGL.
	 * 
	 * @category normalizing
	 */
	public void enableNormalize() {
		OpenGLInterface.glEnable(OpenGLInterface.GL_NORMALIZE);
	}

	/**
	 * Enable the PolygonSmooth of OpenGL.
	 * 
	 * @category smoothing
	 */
	public void enablePolygonSmooth() {
		OpenGLInterface.glEnable(OpenGLInterface.GL_POLYGON_SMOOTH);
		OpenGLInterface.glHint(OpenGLInterface.GL_POLYGON_SMOOTH_HINT, OpenGLInterface.GL_NICEST);
	}

	/**
	 * Enable the PolygonStipple of OpenGL.
	 * 
	 * @category stippling
	 */
	public void enablePolygonStipple() {
		OpenGLInterface.glEnable(OpenGLInterface.GL_POLYGON_STIPPLE);
	}

	/**
	 * Enable the Texture-2d of OpenGL.
	 * 
	 * @category texture mapping
	 */
	public void enableTexture2d() {
		OpenGLInterface.glEnable(OpenGLInterface.GL_TEXTURE_2D);
	}

	/**
	 * Change shading of OpenGL to flat.
	 * 
	 * @category shading
	 */
	public void flatShading() {
		shading = $("flat");
		OpenGLInterface.glShadeModel(OpenGLInterface.GL_FLAT);
	}

	/**
	 * Call the glFlush() of OpenGL.
	 * 
	 * @category user interaction
	 */
	public void flush() {
		OpenGLInterface.glFlush();
	}

	/**
	 * Call the glFogfv() of OpenGL with a Color.
	 * 
	 * @param aColorValue java.awt.Color
	 * @category fogging
	 */
	public void fogColor_(Color aColorValue) {
		float[] colorArray = {(float) aColorValue.getRed() / 255.0f, (float) aColorValue.getGreen() / 255.0f, (float) aColorValue.getBlue() / 255.0f, 1.0f };
		OpenGLInterface.glFogfv(OpenGLInterface.GL_FOG_COLOR, colorArray);
	}

	/**
	 * Call the glFog of OpenGL with EXP2.
	 * 
	 * @param aNumber float
	 * @category fogging
	 */
	public void fogExp2Density_(final float aNumber) {
		OpenGLInterface.glFogi(OpenGLInterface.GL_FOG_MODE, OpenGLInterface.GL_EXP2);
		OpenGLInterface.glFogf(OpenGLInterface.GL_FOG_DENSITY, aNumber);
	}

	/**
	 * Call the glFog() of OpenGL with EXP.
	 * 
	 * @param aNumber float
	 * @category fogging
	 */
	public void fogExpDensity_(final float aNumber) {
		OpenGLInterface.glFogi(OpenGLInterface.GL_FOG_MODE, OpenGLInterface.GL_EXP);
		OpenGLInterface.glFogf(OpenGLInterface.GL_FOG_DENSITY, aNumber);
	}

	/**
	 * Call the glFog() of OpenGL with StartDensity and EndDensity.
	 * 
	 * @param aNumber1 float
	 * @param aNumber2 float
	 * @category fogging
	 */
	public void fogLinearStartDensity_endDensity_(final float aNumber1, final float aNumber2) {
		OpenGLInterface.glFogi(OpenGLInterface.GL_FOG_MODE, OpenGLInterface.GL_LINEAR);
		OpenGLInterface.glFogf(OpenGLInterface.GL_FOG_START, aNumber1);
		OpenGLInterface.glFogf(OpenGLInterface.GL_FOG_END, aNumber2);
	}

	/**
	 * Answer index of Light.
	 * 
	 * @return int
	 * @category lighting
	 */
	public int getLightIndex() {
		lightIndex = (lightIndex % 7) + 1;
		return lightIndex;
	}

	/**
	 * Set the presentation mode as hidden-line.
	 * 
	 * @category presentation
	 */
	public void hiddenlinePresentation() {
		this.presentation_($("hiddenline"));
	}

	/**
	 * Set a color as LightAmbient.
	 * 
	 * @param aColorValue java.awt.Color
	 * @category lighting
	 */
	public void lightAmbientColor_(Color aColorValue) {
		this.lightAmbientColor_alpha_twoSide_(aColorValue, 1.0f, false);
	}

	/**
	 * Set a color as LightAmbient and a number as alpha.
	 * 
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * @category lighting
	 */
	public void lightAmbientColor_alpha_(Color aColorValue, float aNumber) {
		this.lightAmbientColor_alpha_twoSide_(aColorValue, aNumber, false);
	}

	/**
	 * Call the glLightModel() of OpenGL with ambientColor and alpha and boolean of two side.
	 * 
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * @param aBoolean boolean
	 * @category lighting
	 */
	public void lightAmbientColor_alpha_twoSide_(Color aColorValue, float aNumber, final boolean aBoolean) {
		float[] colorArray = {(float) aColorValue.getRed() / 255.0f, (float) aColorValue.getGreen() / 255.0f, (float) aColorValue.getBlue() / 255.0f, aNumber };
		OpenGLInterface.glLightModelfv(OpenGLInterface.GL_LIGHT_MODEL_AMBIENT, colorArray);
		OpenGLInterface.glLightModelf(OpenGLInterface.GL_LIGHT_MODEL_LOCAL_VIEWER, 0.0f);
		OpenGLInterface.glLightModelf(OpenGLInterface.GL_LIGHT_MODEL_TWO_SIDE, aBoolean ? 1.0f : 0.0f);
	}

	/**
	 * Set index of Light with 3d-position.
	 * 
	 * @param index0to7 int
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @category lighting
	 */
	public void lightIndex_position_(int index0to7, Jun3dPoint a3dPoint) {
		this.lightIndex_position_direction_cutoffAngle_color_alpha_(index0to7, a3dPoint, null, null, Color.white, 1.0f);
	}

	/**
	 * Set index of Light with 3d-position with color.
	 * 
	 * @param index0to7 int
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aColorValue java.awt.Color
	 * @category lighting
	 */
	public void lightIndex_position_color_(int index0to7, Jun3dPoint a3dPoint, Color aColorValue) {
		this.lightIndex_position_direction_cutoffAngle_color_alpha_(index0to7, a3dPoint, null, null, aColorValue, 1.0f);
	}

	/**
	 * Set index of Light with 3d-position with boolean as parallel with color with alpha.
	 * 
	 * @param index0to7 int
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunAngle sra.jun.geometry.basic.JunAngle
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * @category lighting
	 */
	public void lightIndex_position_direction_cutoffAngle_color_alpha_(int index0to7, Jun3dPoint a3dPoint, Jun3dPoint a3dPoint2, JunAngle aJunAngle, Color aColorValue, float aNumber) {
		this.lightIndex_position_direction_cutoffAngle_color_alpha_linearAttenuation_quadraticAttenuation_constantAttenuation_(index0to7, a3dPoint, a3dPoint2, aJunAngle, aColorValue, aNumber, 0.0f, 0.0f, 1.0f);
	}

	/**
	 * Set index of Light with 3d-position with boolean as parallel with color with alpha.
	 * 
	 * @param index0to7 int
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunAngle sra.jun.geometry.basic.JunAngle
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * @param aNumber2 float
	 * @param aNumber3 float
	 * @param aNumber4 float
	 * @category lighting
	 */
	public void lightIndex_position_direction_cutoffAngle_color_alpha_linearAttenuation_quadraticAttenuation_constantAttenuation_(int index0to7, Jun3dPoint a3dPoint, Jun3dPoint a3dPoint2, JunAngle aJunAngle, Color aColorValue, float aNumber, float aNumber2, float aNumber3, float aNumber4) {
		float[] positionArray;
		float[] directionArray;
		Float cutoff;

		final StBlockClosure copyPointToArray3Block = new StBlockClosure() {
			public Object value_value_(Object o1, Object o2) {
				float[] array = (float[]) o1;
				Jun3dPoint point = (Jun3dPoint) o2;
				array[0] = (float) point.x();
				array[1] = (float) point.y();
				array[2] = (float) point.z();
				return this;
			}
		};
		final StBlockClosure copyPointToArray4Block = new StBlockClosure() {
			public Object value_value_value_(Object o1, Object o2, Object o3) {
				float[] array = (float[]) o1;
				Jun3dPoint point = (Jun3dPoint) o2;
				Number w = (Number) o3;
				array[0] = (float) point.x();
				array[1] = (float) point.y();
				array[2] = (float) point.z();
				array[3] = w.floatValue();
				return this;
			}
		};
		final StBlockClosure copyColorToArray4Block = new StBlockClosure() {
			public Object value_value_value_(Object o1, Object o2, Object o3) {
				float[] array = (float[]) o1;
				Color colorValue = (Color) o2;
				Number a = (Number) o3;
				array[0] = (float) colorValue.getRed() / 255.0f;
				array[1] = (float) colorValue.getGreen() / 255.0f;
				array[2] = (float) colorValue.getBlue() / 255.0f;
				array[3] = a.floatValue();
				return this;
			}
		};
		Jun3dPoint position = a3dPoint;
		Jun3dPoint direction = a3dPoint2;
		if (position == null) {
			if (direction == null) {
				return;
			} else {
				positionArray = new float[4];
				copyPointToArray4Block.value_value_value_(positionArray, direction.negated(), new Float(0.0f));
				directionArray = new float[3];
				copyPointToArray3Block.value_value_(directionArray, direction);
				cutoff = new Float(90.0f);
			}
		} else {
			if (direction == null) {
				positionArray = new float[4];
				copyPointToArray4Block.value_value_value_(positionArray, position, new Float(1.0f));
				directionArray = null;
				cutoff = null;
			} else {
				positionArray = new float[4];
				copyPointToArray4Block.value_value_value_(positionArray, position, new Float(1.0f));
				directionArray = new float[3];
				copyPointToArray3Block.value_value_(directionArray, direction);
				if (aJunAngle == null) {
					cutoff = new Float(1.0f);
				} else if (aJunAngle.respondsTo_("deg")) {
					cutoff = new Float(aJunAngle.deg());
				} else {
					//TODO: JunAngle.asFloat() is not implement
					//cutoff = new Float(aJunAngle.asFloat());
					cutoff = null;
				}
			}
		}
		float[] colorArray = new float[4];
		// TODO: aNumber is not null
		copyColorToArray4Block.value_value_value_(colorArray, (aColorValue == null) ? Color.WHITE : aColorValue, new Float(aNumber));
		Object[] lightInfo = new Object[7];
		lightInfo[0] = positionArray;
		lightInfo[1] = directionArray;
		lightInfo[2] = cutoff;
		lightInfo[3] = colorArray;
		lightInfo[4] = new Float(aNumber2);
		lightInfo[5] = new Float(aNumber3);
		lightInfo[6] = new Float(aNumber4);
		this.lightIndex_put_(index0to7, lightInfo);
	}

	/**
	 * Set index of Light with 3d-position with boolean as parallel with color.
	 * 
	 * @param index0to7 int
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aBoolean boolean
	 * @param aColorValue java.awt.Color
	 * @category lighting
	 */
	public void lightIndex_position_parallel_color_(int index0to7, Jun3dPoint a3dPoint, boolean aBoolean, Color aColorValue) {
		this.lightIndex_position_direction_cutoffAngle_color_alpha_(index0to7, null, (Jun3dPoint) a3dPoint.negated(), null, aColorValue, 1.0f);
	}

	/**
	 * Set index of Light with 3d-position with boolean as parallel with color with alpha.
	 * 
	 * @param index0to7 int
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aBoolean boolean
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * @category lighting
	 */
	public void lightIndex_position_parallel_color_alpha_(int index0to7, Jun3dPoint a3dPoint, boolean aBoolean, Color aColorValue, float aNumber) {
		this.lightIndex_position_direction_cutoffAngle_color_alpha_(index0to7, null, (Jun3dPoint) a3dPoint.negated(), null, aColorValue, aNumber);
	}

	/**
	 *  Set index of Light.
	 * 
	 * @param index0to7
	 * @param anArray
	 * @category lighting
	 */
	public void lightIndex_put_(int index0to7, Object[] anArray) {
		float[] positionArray = (float[]) anArray[0];
		float[] directionArray = (float[]) anArray[1];
		Float cutoff = (Float) anArray[2];
		float[] colorArray = (float[]) anArray[3];
		Float linear = (Float) anArray[4];
		Float quadratic = (Float) anArray[5];
		Float constant = (Float) anArray[6];
		this.enableLightIndex_(index0to7);
		
		// TODO: lights not imprement
		// lights.put(new Integer(index0to7 + 1), anArray)
		int index = OpenGLInterface.GL_LIGHT0 + index0to7;
		if (positionArray != null) {
			OpenGLInterface.glLightfv(index, OpenGLInterface.GL_POSITION, positionArray);
		} else {
			OpenGLInterface.glLightfv(index, OpenGLInterface.GL_POSITION, new float[] {0.0f, 0.0f, 1.0f, 0.0f});
		}
		if (directionArray != null) {
			OpenGLInterface.glLightfv(index, OpenGLInterface.GL_SPOT_DIRECTION, directionArray);
		} else {
			OpenGLInterface.glLightfv(index, OpenGLInterface.GL_SPOT_DIRECTION, new float[] {0.0f, 0.0f, -1.0f});
		}
		if (cutoff != null) {
			OpenGLInterface.glLightf(index, OpenGLInterface.GL_SPOT_CUTOFF, cutoff.floatValue());
		} else {
			OpenGLInterface.glLightf(index, OpenGLInterface.GL_SPOT_CUTOFF, 180.0f);
		}
		if (colorArray != null) {
			OpenGLInterface.glLightfv(index, OpenGLInterface.GL_DIFFUSE, colorArray);
			OpenGLInterface.glLightfv(index, OpenGLInterface.GL_SPECULAR, colorArray);
			OpenGLInterface.glLightf(index, OpenGLInterface.GL_LINEAR_ATTENUATION, linear.floatValue());
			OpenGLInterface.glLightf(index, OpenGLInterface.GL_QUADRATIC_ATTENUATION, quadratic.floatValue());
			OpenGLInterface.glLightf(index, OpenGLInterface.GL_CONSTANT_ATTENUATION, constant.floatValue());
		} else {
			OpenGLInterface.glLightfv(index, OpenGLInterface.GL_DIFFUSE, new float[] {1.0f, 1.0f, 1.0f, 1.0f});
			OpenGLInterface.glLightfv(index, OpenGLInterface.GL_SPECULAR, new float[] {1.0f, 1.0f, 1.0f, 1.0f});
			OpenGLInterface.glLightf(index, OpenGLInterface.GL_LINEAR_ATTENUATION, 0.0f);
			OpenGLInterface.glLightf(index, OpenGLInterface.GL_QUADRATIC_ATTENUATION, 0.0f);
			OpenGLInterface.glLightf(index, OpenGLInterface.GL_CONSTANT_ATTENUATION, 0.0f);
		}
	}

	/**
	 * Answer the lightStates.
	 * 
	 * @return java.util.Hashtable
	 * @category lighting
	 */
	public Hashtable lightStates() {
		return (Hashtable) lightStates.clone();
	}

	/**
	 * Set the lightStates.
	 * 
	 * @param anIdentityDictionary java.util.Hashtable
	 * @category lighting
	 */
	public void lightStates_(Hashtable anIdentityDictionary) {
		Enumeration enum = anIdentityDictionary.keys();
		while (enum.hasMoreElements()) {
			Object key = enum.nextElement();
			if (key.getClass() == Integer.class) {
				if (anIdentityDictionary.get(key) == Boolean.TRUE) {
					this.enableLightIndex_(((Integer) key).intValue());
				} else {
					this.disableLightIndex_(((Integer) key).intValue());
				}
			}
		}

		Integer index = (Integer) anIdentityDictionary.get("#lightIndex");
		if (index != null) {
			lightIndex = index.intValue();
		}
	}

	/**
	 * Call the glHint() of OpenGL with lineSmoothDontCare.
	 * 
	 * @category smoothing
	 */
	public void lineSmoothDontCare() {
		OpenGLInterface.glHint(OpenGLInterface.GL_LINE_SMOOTH_HINT, OpenGLInterface.GL_DONT_CARE);
	}

	/**
	 * Call the glHint() of OpenGL with lineSmoothFastest.
	 * 
	 * @category smoothing
	 */
	public void lineSmoothFastest() {
		OpenGLInterface.glHint(OpenGLInterface.GL_LINE_SMOOTH_HINT, OpenGLInterface.GL_FASTEST);
	}

	/**
	 * Call the glHint() of OpenGL with lineSmoothNicest.
	 * 
	 * @category smoothing
	 */
	public void lineSmoothNicest() {
		OpenGLInterface.glHint(OpenGLInterface.GL_LINE_SMOOTH_HINT, OpenGLInterface.GL_NICEST);
	}

	/**
	 * Call the glHint() of OpenGL with lineStippleFactor with pattern.
	 * 
	 * @param factor int
	 * @param pattern short
	 * @category stippling
	 */
	public void lineStippleFactor_pattern_(final int factor, final short pattern) {
		OpenGLInterface.glLineStipple(factor, pattern);
	}

	/**
	 * Answer the lineWidth.
	 * 
	 * @return float
	 * @category accessing
	 */
	public float lineWidth() {
		return lineWidth;
	}

	/**
	 * Set the lineWidth.
	 * 
	 * @param width float
	 * @category accessing
	 */
	public void lineWidth_(final float width) {
		lineWidth = width;
		OpenGLInterface.glLineWidth(lineWidth);
	}

	/**
	 * Set color as materialAmbient.
	 * 
	 * @param aColorValue java.awt.Color
	 * @category material
	 */
	public void materialAmbient_(Color aColorValue) {
		this.materialAmbient_alpha_(aColorValue, 1.0f);
	}

	/**
	 * Call the glMaterial() with color of materialAmbient with alpha.
	 * 
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * @category material
	 */
	public void materialAmbient_alpha_(Color aColorValue, float aNumber) {
		float[] colorArray = {(float) aColorValue.getRed() / 255.0f, (float) aColorValue.getGreen() / 255.0f, (float) aColorValue.getBlue() / 255.0f, aNumber };
		OpenGLInterface.glMaterialfv(OpenGLInterface.GL_FRONT_AND_BACK, OpenGLInterface.GL_AMBIENT, colorArray);
	}

	/**
	 * Set color of materialDiffuse.
	 * 
	 * @param aColorValue java.awt.Color
	 * @category material
	 */
	public void materialDiffuse_(Color aColorValue) {
		this.materialDiffuse_alpha_(aColorValue, 1.0f);
	}

	/**
	 * Set color of materialDiffuse with alpha.
	 * 
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * @category material
	 */
	public void materialDiffuse_alpha_(Color aColorValue, float aNumber) {
		float[] colorArray = {(float) aColorValue.getRed() / 255.0f, (float) aColorValue.getGreen() / 255.0f, (float) aColorValue.getBlue() / 255.0f, aNumber };
		OpenGLInterface.glMaterialfv(OpenGLInterface.GL_FRONT_AND_BACK, OpenGLInterface.GL_DIFFUSE, colorArray);
	}

	/**
	 * Set color of materialEmission.
	 * 
	 * @param aColorValue java.awt.Color
	 * @category material
	 */
	public void materialEmission_(Color aColorValue) {
		this.materialEmission_alpha_(aColorValue, 1.0f);
	}

	/**
	 * Call glMaterial() of OpenGL with color of materialEmission with alpha.
	 * 
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * @category material
	 */
	public void materialEmission_alpha_(Color aColorValue, float aNumber) {
		float[] colorArray = {(float) aColorValue.getRed() / 255.0f, (float) aColorValue.getGreen() / 255.0f, (float) aColorValue.getBlue() / 255.0f, aNumber };
		OpenGLInterface.glMaterialfv(OpenGLInterface.GL_FRONT_AND_BACK, OpenGLInterface.GL_EMISSION, colorArray);
	}

	/**
	 * Call glMaterial() of OpenGL with number of materialShininess.
	 * 
	 * @param aNumber float
	 * @category material
	 */
	public void materialShininess_(final float aNumber) {
		OpenGLInterface.glMaterialf(OpenGLInterface.GL_FRONT_AND_BACK, OpenGLInterface.GL_SHININESS, aNumber);
	}

	/**
	 * Set color of materialSpecular.
	 * 
	 * @param aColorValue java.awt.Color
	 * @category material
	 */
	public void materialSpecular_(Color aColorValue) {
		this.materialSpecular_alpha_(aColorValue, 1.0f);
	}

	/**
	 * Call glMaterial() of OpenGL with color of materialSpecular with alpha.
	 * 
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * @category material
	 */
	public void materialSpecular_alpha_(Color aColorValue, float aNumber) {
		float[] colorArray = {(float) aColorValue.getRed() / 255.0f, (float) aColorValue.getGreen() / 255.0f, (float) aColorValue.getBlue() / 255.0f, aNumber };
		OpenGLInterface.glMaterialfv(OpenGLInterface.GL_FRONT_AND_BACK, OpenGLInterface.GL_SPECULAR, colorArray);
	}

	/**
	 * Answer the number of maxHandleAllocationRetries.
	 * 
	 * @return int
	 * @category constants access
	 */
	public int maxHandleAllocationRetries() {
		return 3;
	}

	/**
	 * Answer 3d-point as normalUnitVector of 3d-points.
	 * 
	 * @param anArrayOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category utilities
	 */
	public Jun3dPoint normalUnitVectorOfPoints_(Jun3dPoint[] anArrayOfPoints) {
		return NormalUnitVectorOfPoints_(anArrayOfPoints);
	}

	/**
	 * Answer 3d-point as normalVector of 3d-points.
	 * 
	 * @param anArrayOfPoints sra.jun.geometry.basic.Jun3dPoint[]
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 * @category utilities
	 */
	public Jun3dPoint normalVectorOfPoints_(Jun3dPoint[] anArrayOfPoints) {
		return NormalVectorOfPoints_(anArrayOfPoints);
	}

	/**
	 * Answer the nurbsSamplingTolerance.
	 * 
	 * @return float
	 * @category accessing
	 */
	public float nurbsSamplingTolerance() {
		return nurbsSamplingTolerance;
	}

	/**
	 * Set the nurbsSamplingTolerance.
	 * 
	 * @param aNumber float
	 * @category accessing
	 */
	public void nurbsSamplingTolerance_(float aNumber) {
		nurbsSamplingTolerance = aNumber;
	}

	/**
	 * Answer the paint.
	 * 
	 * @return java.awt.Color
	 * @category accessing
	 */
	public Color paint() {
		return paint;
	}

	/**
	 * Set the paint.
	 * 
	 * @param aPaint java.awt.Color
	 * @category accessing
	 */
	public void paint_(Color aPaint) {
		this.paint_alpha_(aPaint, alpha);
	}

	/**
	 * Call glColor4() of OpenGL with color of paint with number of alpha.
	 * 
	 * @param aPaint java.awt.Color
	 * @param aNumber
	 * @category accessing
	 */
	public void paint_alpha_(Color aPaint, float aNumber) {
		Color thePaint = (aPaint != null) ? aPaint : Color.white;
		float theNumber = aNumber;
		if ((thePaint != paint) || (theNumber != alpha)) {
			paint = thePaint;
			alpha = theNumber;
			OpenGLInterface.glColor4f((float) paint.getRed() / 255.0f, (float) paint.getGreen() / 255.0f, (float) paint.getBlue() / 255.0f, alpha);
		}
	}

	/**
	 * Set the projection with instance of JunOpenGLParallelProjection.
	 * 
	 * @param aJunOpenGLParallelProjection sra.jun.opengl.projection.JunOpenGLParallelProjection
	 * @category projecting
	 */
	public void parallelProjection_(JunOpenGLParallelProjection aJunOpenGLParallelProjection) {
		this.setProjection_((JunOpenGLParallelProjection) aJunOpenGLParallelProjection.copy());
		double aspect = (double) this.getSize().width / (double) this.getSize().height;
		double halfHeight = ((JunOpenGLParallelProjection) projection).height() / 2;
		double halfWidth = halfHeight * aspect;
		OpenGLInterface.glMatrixMode(OpenGLInterface.GL_PROJECTION);
		OpenGLInterface.glLoadIdentity();
		OpenGLInterface.glOrtho(-halfWidth, halfWidth, -halfHeight, halfHeight, projection.near(), projection.far());
		OpenGLInterface.glMatrixMode(OpenGLInterface.GL_MODELVIEW);
		OpenGLInterface.glLoadIdentity();
		OpenGLInterface.gluLookAt(
			projection.eyePoint().x(),
			projection.eyePoint().y(),
			projection.eyePoint().z(),
			projection.sightPoint().x(),
			projection.sightPoint().y(),
			projection.sightPoint().z(),
			projection.upVector().x(),
			projection.upVector().y(),
			projection.upVector().z());
	}

	/**
	 * Set the projection with instance of JunOpenGLPerspective.
	 * 
	 * @param aJunOpenGLPerspective sra.jun.opengl.projection.JunOpenGLPerspective
	 * @category projecting
	 */
	public void perspective_(final JunOpenGLPerspective aJunOpenGLPerspective) {
		setProjection_((JunOpenGLPerspective) aJunOpenGLPerspective.copy());
		OpenGLInterface.glMatrixMode(OpenGLInterface.GL_PROJECTION);
		OpenGLInterface.glLoadIdentity();
		OpenGLInterface.gluPerspective(((JunOpenGLPerspective) projection).fovy().deg(), (double) this.getSize().width / (double) this.getSize().height, projection.near(), projection.far());
		OpenGLInterface.glMatrixMode(OpenGLInterface.GL_MODELVIEW);
		OpenGLInterface.glLoadIdentity();
		OpenGLInterface.gluLookAt(
			projection.eyePoint().x(),
			projection.eyePoint().y(),
			projection.eyePoint().z(),
			projection.sightPoint().x(),
			projection.sightPoint().y(),
			projection.sightPoint().z(),
			projection.upVector().x(),
			projection.upVector().y(),
			projection.upVector().z());
	}

	/**
	 * Set perspectiveProjection with eyePoint with sightPoint with upVector with fovy with near with far.
	 * 
	 * @param a3dPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param a3dPoint3 sra.jun.geometry.basic.Jun3dPoint
	 * @param aNumber1 double
	 * @param aNumber2 double
	 * @param aNumber3 double
	 * @category projecting
	 */
	public void perspectiveEyePoint_sightPoint_upVector_fovy_near_far_(Jun3dPoint a3dPoint1, Jun3dPoint a3dPoint2, Jun3dPoint a3dPoint3, double aNumber1, double aNumber2, double aNumber3) {
		this.projection_(JunOpenGLPerspective.EyePoint_sightPoint_upVector_fovy_near_far_(a3dPoint1, a3dPoint2, a3dPoint3, aNumber1, aNumber2, aNumber3));
	}

	/**
	 * Answer the pointSize.
	 * 
	 * @return float
	 * @category accessing
	 */
	public float pointSize() {
		return pointSize;
	}

	/**
	 * Set the pointSize.
	 * 
	 * @param size float
	 * @category accessing
	 */
	public void pointSize_(final float size) {
		pointSize = size;
		OpenGLInterface.glPointSize(pointSize);
	}

	/**
	 * Call the glHint() of OpenGL with polygonSmoothDontCare.
	 * 
	 * @category smoothing
	 */
	public void polygonSmoothDontCare() {
		OpenGLInterface.glHint(OpenGLInterface.GL_POLYGON_SMOOTH_HINT, OpenGLInterface.GL_DONT_CARE);
	}

	/**
	 * Call the glHint() of OpenGL with polygonSmoothFastest.
	 * 
	 * @category smoothing
	 */
	public void polygonSmoothFastest() {
		OpenGLInterface.glHint(OpenGLInterface.GL_POLYGON_SMOOTH_HINT, OpenGLInterface.GL_FASTEST);
	}

	/**
	 * Call the glHint() of OpenGL with polygonSmoothNicest.
	 * 
	 * @category smoothing
	 */
	public void polygonSmoothNicest() {
		OpenGLInterface.glHint(OpenGLInterface.GL_POLYGON_SMOOTH_HINT, OpenGLInterface.GL_NICEST);
	}

	/**
	 * Call the glPolygonStipple() of OpenGL with a JunOpenGLStipple.
	 * 
	 * @param aJunOpenGLStipple sra.jun.opengl.texture.JunOpenGLStipple
	 * @category stippling
	 */
	public void polygonStipple_(final JunOpenGLStipple aJunOpenGLStipple) {
		OpenGLInterface.glPolygonStipple(aJunOpenGLStipple.pointerOfPixels());
	}

	/**
	 * Answer the current presentation mode. The value of presentation should
	 * be one of the followings:     #solid     #wireframe     #hiddenline
	 * 
	 * @return sra.smalltalk.StSymbol
	 * @category presentation
	 */
	public StSymbol presentation() {
		if (presentation == null) {
			presentation = $("solid");
		}
		return presentation;
	}

	/**
	 * Set the presentation mode. The value of aSymbol should be one of the
	 * followings:     #solid     #wireframe     #hiddenline
	 * 
	 * @param aSymbol sra.smalltalk.StSymbol
	 * @category presentation
	 */
	public void presentation_(StSymbol aSymbol) {
		if ((aSymbol == $("solid")) || (aSymbol == $("wireframe")) || (aSymbol == $("hiddenline"))) {
			presentation = aSymbol;
		}
	}

	/**
	 * Evaluate a Block while productTransformation.
	 * 
	 * @param aJun3dTransformation sra.jun.geometry.transformations.Jun3dTransformation
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @category transformation
	 */
	public void productTransformation_while_(Jun3dTransformation aJun3dTransformation, StBlockClosure aBlock) {
		Jun3dTransformation origTransformation;
		origTransformation = transformation;
		transformation = (Jun3dTransformation) aJun3dTransformation.product_(transformation);
		double[] matrixPointer_ = aJun3dTransformation.asArray();
		OpenGLInterface.glPushMatrix();
		OpenGLInterface.glMultMatrixd(matrixPointer_);
		try {
			aBlock.value();
		} finally {
			OpenGLInterface.glPopMatrix();
			transformation = origTransformation;
		}
	}

	/**
	 * Answer the projection.
	 * 
	 * @return sra.jun.opengl.projection.JunOpenGLProjection
	 * @category projecting
	 */
	public JunOpenGLProjection projection() {
		return (JunOpenGLProjection) projection.copy();
	}

	/**
	 * Set the projection.
	 * 
	 * @param aJunOpenGLProjection sra.jun.opengl.projection.JunOpenGLProjection
	 * @category projecting
	 */
	public void projection_(JunOpenGLProjection aJunOpenGLProjection) {
		aJunOpenGLProjection.projectOn_(this);
	}

	/**
	 * Remove display list of OpenGL at index.
	 * 
	 * @param anInteger int
	 * @category display list
	 */
	public void removeDisplayList_(final int anInteger) {
		OpenGLInterface.glDeleteLists(anInteger, 1);
	}

	/**
	 * Reset LightIndex.
	 * 
	 * @category lighting
	 */
	public void resetLightIndex() {
		lightIndex = 0;
		lightStates.put("#lightIndex", new Integer(0));
	}

	/**
	 * Set the projection.
	 * 
	 * @param aJunOpenGLProjection sra.jun.opengl.projection.JunOpenGLProjection
	 * @category private
	 */
	protected final void setProjection_(JunOpenGLProjection aJunOpenGLProjection) {
		projection = aJunOpenGLProjection;
	}

	/**
	 * Answer the shading.
	 * 
	 * @return sra.smalltalk.StSymbol
	 * @category shading
	 */
	public StSymbol shading() {
		return shading;
	}

	/**
	 * Call the glShadeModel() of OpenGL with smooth.
	 * 
	 * @category shading
	 */
	public void smoothShading() {
		shading = $("smooth");
		OpenGLInterface.glShadeModel(OpenGLInterface.GL_SMOOTH);
	}

	/**
	 * Set the presentation mode as solid
	 * 
	 * @category presentation
	 */
	public void solidPresentation() {
		this.presentation_($("solid"));
	}

	/**
	 * Answer the texture.
	 * 
	 * @return sra.jun.opengl.texture.JunOpenGLTexture
	 * @category texture mapping
	 */
	public JunOpenGLTexture texture() {
		return texture;
	}

	/**
	 * Set the texture.
	 * 
	 * @param aJunOpenGLTexture sra.jun.opengl.texture.JunOpenGLTexture
	 * @category texture mapping
	 */
	public void texture_(final JunOpenGLTexture aJunOpenGLTexture) {
		final JunOpenGLRenderingContext this_ = this;
		if (aJunOpenGLTexture != null) {
			OpenGLInterface.glPixelStorei(OpenGLInterface.GL_UNPACK_ALIGNMENT, 1);
			if (aJunOpenGLTexture.mipmap()) {
				OpenGLInterface.gluBuild2DMipmaps(OpenGLInterface.GL_TEXTURE_2D, 3, aJunOpenGLTexture.width(), aJunOpenGLTexture.height(), OpenGLInterface.GL_RGB, OpenGLInterface.GL_UNSIGNED_BYTE, aJunOpenGLTexture.pointerOfPixels());
			} else {
				OpenGLInterface.glTexImage2D(OpenGLInterface.GL_TEXTURE_2D, 0, 3, aJunOpenGLTexture.width(), aJunOpenGLTexture.height(), 0, OpenGLInterface.GL_RGB, OpenGLInterface.GL_UNSIGNED_BYTE, aJunOpenGLTexture.pointerOfPixels());
			}
			OpenGLInterface.glTexParameteri(OpenGLInterface.GL_TEXTURE_2D, OpenGLInterface.GL_TEXTURE_WRAP_S, (aJunOpenGLTexture.clamp()) ? OpenGLInterface.GL_CLAMP : OpenGLInterface.GL_REPEAT);
			OpenGLInterface.glTexParameteri(OpenGLInterface.GL_TEXTURE_2D, OpenGLInterface.GL_TEXTURE_WRAP_T, (aJunOpenGLTexture.clamp()) ? OpenGLInterface.GL_CLAMP : OpenGLInterface.GL_REPEAT);
			OpenGLInterface.glTexParameteri(OpenGLInterface.GL_TEXTURE_2D, OpenGLInterface.GL_TEXTURE_MAG_FILTER, (aJunOpenGLTexture.nearest()) ? OpenGLInterface.GL_NEAREST : OpenGLInterface.GL_LINEAR);
			if (aJunOpenGLTexture.mipmap()) {
				OpenGLInterface.glTexParameteri(OpenGLInterface.GL_TEXTURE_2D, OpenGLInterface.GL_TEXTURE_MIN_FILTER, (aJunOpenGLTexture.nearest()) ? OpenGLInterface.GL_NEAREST_MIPMAP_NEAREST : OpenGLInterface.GL_LINEAR_MIPMAP_LINEAR);
			} else {
				OpenGLInterface.glTexParameteri(OpenGLInterface.GL_TEXTURE_2D, OpenGLInterface.GL_TEXTURE_MIN_FILTER, (aJunOpenGLTexture.nearest()) ? OpenGLInterface.GL_NEAREST : OpenGLInterface.GL_LINEAR);
			}
			if (aJunOpenGLTexture.modulate()) {
				OpenGLInterface.glTexEnvi(OpenGLInterface.GL_TEXTURE_ENV, OpenGLInterface.GL_TEXTURE_ENV_MODE, OpenGLInterface.GL_MODULATE);
			} else {
				OpenGLInterface.glTexEnvi(OpenGLInterface.GL_TEXTURE_ENV, OpenGLInterface.GL_TEXTURE_ENV_MODE, OpenGLInterface.GL_DECAL);
			}
			if (aJunOpenGLTexture.hasCoordinates()) {
				textureCoordinateBlock = new StBlockClosure() {
					public Object value_(Object o) {
						Jun3dPoint aPoint = (Jun3dPoint) o;
						if (this_.textureCoordinateIndex() > (texture.coordinates().length - 1)) {
							this_.textureCoordinateIndex_(0);
						}
						Jun2dPoint point = (Jun2dPoint) texture.coordinates()[this_.textureCoordinateIndex()];
						this_.textureCoordinateIndex_(this_.textureCoordinateIndex + 1);
						return point;
					}
				};
			} else {
				OpenGLInterface.glTexGeni(OpenGLInterface.GL_S, OpenGLInterface.GL_TEXTURE_GEN_MODE, OpenGLInterface.GL_OBJECT_LINEAR);
				OpenGLInterface.glTexGenfv(OpenGLInterface.GL_S, OpenGLInterface.GL_OBJECT_PLANE, aJunOpenGLTexture.pointerOfParametersForS());
				OpenGLInterface.glTexGeni(OpenGLInterface.GL_T, OpenGLInterface.GL_TEXTURE_GEN_MODE, OpenGLInterface.GL_OBJECT_LINEAR);
				OpenGLInterface.glTexGenfv(OpenGLInterface.GL_T, OpenGLInterface.GL_OBJECT_PLANE, aJunOpenGLTexture.pointerOfParametersForT());
				OpenGLInterface.glEnable(OpenGLInterface.GL_TEXTURE_GEN_S);
				OpenGLInterface.glEnable(OpenGLInterface.GL_TEXTURE_GEN_T);
				OpenGLInterface.glEnable(OpenGLInterface.GL_TEXTURE_2D);
			}
		} else {
			textureCoordinateBlock = null;
		}

		texture = aJunOpenGLTexture;
	}

	/**
	 * Answer the textureCoordinateIndex.
	 * 
	 * @return int
	 * @category texture mapping
	 */
	public int textureCoordinateIndex() {
		return textureCoordinateIndex;
	}

	/**
	 * Set the textureCoordinateIndex.
	 * 
	 * @param anInteger int
	 * @category texture mapping
	 */
	public void textureCoordinateIndex_(int anInteger) {
		textureCoordinateIndex = anInteger;
	}

	/**
	 * Answer the transformation.
	 * 
	 * @return sra.jun.geometry.transformations.Jun3dTransformation
	 * @category transformation
	 */
	public Jun3dTransformation transformation() {
		return transformation;
	}

	/**
	 * Set the presentation mode as wireframe.
	 * 
	 * @category presentation
	 */
	public void wireframePresentation() {
		this.presentation_($("wireframe"));
	}

	/**
	 * Set index of Light with 3d-position with boolean as parallel with color with alpha.
	 * 
	 * @param index0to7 int
	 * @param a3dPoint sra.jun.geometry.basic.Jun3dPoint
	 * @param aBoolean boolean
	 * @param aColorValue java.awt.Color
	 * @param aNumber float
	 * @category lighting
	 */
	public void zlightIndex_position_parallel_color_alpha_(int index0to7, Jun3dPoint a3dPoint, boolean aBoolean, Color aColorValue, float aNumber) {
		float[] pointArray = {(float) a3dPoint.x(), (float) a3dPoint.y(), (float) a3dPoint.z(), aBoolean ? 0.0f : 1.0f };
		float[] colorArray = {(float) aColorValue.getRed() / 255.0f, (float) aColorValue.getGreen() / 255.0f, (float) aColorValue.getBlue() / 255.0f, aNumber };
		int index = OpenGLInterface.GL_LIGHT0 + index0to7;
		OpenGLInterface.glLightfv(index, OpenGLInterface.GL_POSITION, pointArray);
		OpenGLInterface.glLightfv(index, OpenGLInterface.GL_DIFFUSE, colorArray);
		OpenGLInterface.glEnable(index);
		OpenGLInterface.glLightfv(index, OpenGLInterface.GL_SPECULAR, colorArray);
		OpenGLInterface.glLightf(index, OpenGLInterface.GL_LINEAR_ATTENUATION, 0.0f);
		OpenGLInterface.glLightf(index, OpenGLInterface.GL_QUADRATIC_ATTENUATION, 0.0f);
		OpenGLInterface.glLightf(index, OpenGLInterface.GL_CONSTANT_ATTENUATION, 1.0f);
	}

	/**
	 * Display NurbsSurface with displayMode.
	 * 
	 * @param aNurbsSurface sra.jun.geometry.surfaces.JunNurbsSurface
	 * @param mode int
	 * @category private
	 */
	protected void displayNurbsSurface_displayMode_(final JunNurbsSurface aNurbsSurface, final int mode) {
		float[] uKnotVector = new float[aNurbsSurface.uKnotVector().length];
		for (int i = 0; i < aNurbsSurface.uKnotVector().length; i++) {
			uKnotVector[i] = (float) aNurbsSurface.uKnotVector()[i];
		}
		float[] vKnotVector = new float[aNurbsSurface.vKnotVector().length];
		for (int i = 0; i < aNurbsSurface.vKnotVector().length; i++) {
			vKnotVector[i] = (float) aNurbsSurface.vKnotVector()[i];
		}
		int uSize = aNurbsSurface.uSize();
		int vSize = aNurbsSurface.vSize();
		float[] floatArray = new float[uSize * vSize * 4];
		int _arrayIndex = 0;
		for (int uIndex = 0; uIndex < uSize; uIndex++) {
			for (int vIndex = 0; vIndex < vSize; vIndex++) {
				Jun3dPoint point;
				double weight;
				point = aNurbsSurface.controlPointAtU_v_(uIndex, vIndex);
				weight = aNurbsSurface.weightAtU_v_(uIndex, vIndex);
				floatArray[_arrayIndex++] = (float) (point.x() * weight);
				floatArray[_arrayIndex++] = (float) (point.y() * weight);
				floatArray[_arrayIndex++] = (float) (point.z() * weight);
				floatArray[_arrayIndex++] = (float) weight;
			}
		}

		OpenGLInterface.glEnable(OpenGLInterface.GL_AUTO_NORMAL);
		int aNurbs = OpenGLInterface.gluNewNurbsRenderer();
		OpenGLInterface.gluNurbsProperty(aNurbs, OpenGLInterface.GLU_SAMPLING_TOLERANCE, nurbsSamplingTolerance);
		OpenGLInterface.gluNurbsProperty(aNurbs, OpenGLInterface.GLU_DISPLAY_MODE, mode);
		OpenGLInterface.glPushMatrix();
		OpenGLInterface.gluBeginSurface(aNurbs);
		OpenGLInterface.gluNurbsSurface(aNurbs, uKnotVector.length, uKnotVector, vKnotVector.length, vKnotVector, 4 * vSize, 4, floatArray, aNurbsSurface.uOrder(), aNurbsSurface.vOrder(), OpenGLInterface.GL_MAP2_VERTEX_4);
		/*
		aNurbsSurface.trimCurvesDo_(new StBlockClosure() {
		public Object value_(Object o) {
		JunOpenGL2dTrimCurve trimCurve = (JunOpenGL2dTrimCurve) o;
		trimCurve.renderOn_with_for_(this, aNurbsSurface, aNurbs);
		return null;
		}
		})
		*/
		OpenGLInterface.gluEndSurface(aNurbs);
		OpenGLInterface.glPopMatrix();
		OpenGLInterface.gluDeleteNurbsRenderer(aNurbs);
	}

	/**
	 * do nothing
	 * 
	 * @param aRectangle java.awt.Rectangle
	 * @category accessing
	 */
	private void intersectClip_(Rectangle aRectangle) {
	}

}
