package sra.jun.opengl.texture;

import java.awt.*;
import java.util.*;
import sra.jun.goodies.images.support.JunImageProcessor;
import sra.jun.opengl.support.*;
import sra.smalltalk.*;

/**
 * JunOpenGLStipple class
 * 
 *  @author    nisinaka
 *  @created   1998/10/13 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLStipple.java,v 6.0 2003/10/28 10:52:22 nisinaka Exp $
 */
public class JunOpenGLStipple extends StObject {
	private static Hashtable HalftoneStipples;

	static {
		HalftoneStipples = _InitializeHalftoneStipples();
	}

	private StImage stippleImage;
	private byte[] cachedPixels;

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anImage anImage
	 * 
	 * @return sra.jun.opengl.texture.JunOpenGLStipple
	 */
	public JunOpenGLStipple(Image anImage) {
		this.image_(new StImage(anImage));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anImage anImage
	 * 
	 * @return sra.jun.opengl.texture.JunOpenGLStipple
	 */
	public JunOpenGLStipple(StImage anImage) {
		this.image_(anImage);
	}

	/**
	 * Create a new JunOpenGLStipple with default halftone scale.
	 * 
	 * @return sra.jun.opengl.texture.JunOpenGLStipple
	 */
	public static JunOpenGLStipple Halftone() {
		return Halftone_(0.5);
	}

	/**
	 * Create a new JunOpenGLStipple with the specified halftone scale.
	 * 
	 * @param halftoneScale double
	 * 
	 * @return sra.jun.opengl.texture.JunOpenGLStipple
	 */
	public static JunOpenGLStipple Halftone_(double halftoneScale) {
		double halftone = Math.max(0, (Math.min(halftoneScale, 1)));
		Double key = new Double(halftone);

		if (HalftoneStipples.containsKey(key)) {
			return (JunOpenGLStipple) HalftoneStipples.get(new Double(halftone));
		} else {
			return Image_(JunImageProcessor.Halftone_(halftone));
		}
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param anImage DOCUMENT ME!
	 * 
	 * @return JunOpenGLStipple
	 */
	public static JunOpenGLStipple Image_(Image anImage) {
		return new JunOpenGLStipple(anImage);
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param anImage DOCUMENT ME!
	 * 
	 * @return JunOpenGLStipple
	 */
	public static JunOpenGLStipple Image_(StImage anImage) {
		return new JunOpenGLStipple(anImage);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.util.Hashtable
	 */
	private static Hashtable _InitializeHalftoneStipples() {
		Hashtable dictionary = new Hashtable();

		for (int index = 0; index < 9; index++) {
			double key = index / 8.0;
			JunOpenGLStipple value = JunOpenGLStipple.Image_(JunImageProcessor.Halftone_(key));
			dictionary.put(new Double(key), value);
		}

		return dictionary;
	}

	/**
	 * hexagonnal bits. In StImage, all images has 24bit bits in natural (even
	 * if B/W). But in some situation, some methods expect 8 bit bits. This
	 * method returns it.
	 * 
	 * @return int[]
	 */
	public int[] _hexedBits() {
		StImage anImage = this.image();
		int rowByteSize = 4;
		int width = anImage.width();
		int height = anImage.height();
		int[] pixels = anImage.getPixels();
		int[] hexedBits = new int[rowByteSize * height];

		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				int pixel = pixels[(y * width) + x];

				if (pixel != -1) {
					int dX = x / 8;
					hexedBits[(y * rowByteSize) + dX] = hexedBits[(y * rowByteSize) + dX] + (int) Math.pow(2, (7 - (x % 8)));
				}
			}
		}

		return hexedBits;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param renderingContext
	 */
	public void disableStippleOn_(JunOpenGLRenderingContext renderingContext) {
		renderingContext.disablePolygonStipple();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param renderingContext
	 */
	public void enableStippleOn_(JunOpenGLRenderingContext renderingContext) {
		renderingContext.polygonStipple_(this);
		renderingContext.enablePolygonStipple();
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return StImage
	 */
	public StImage image() {
		if (this.stippleImage == null) {
			this.stippleImage = JunImageProcessor.Halftone_(0.5);
		}

		return this.stippleImage;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param anImage
	 */
	public void image_(StImage anImage) {
		if ((anImage.width() == 0) || (anImage.height() == 0)) {
			return;
		}

		this.stippleImage = this.convertToStippleImage_(anImage);
		this.flushPointerOfPixels();
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return DOCUMENT ME!
	 */
	public byte[] pointerOfPixels() {
		if (this.cachedPixels == null) {
			int[] bits = this._hexedBits();
			byte[] bytes = new byte[bits.length];

			for (int index = 0; index < bits.length; index++) {
				bytes[index] = (byte) bits[index];
			}

			this.cachedPixels = bytes;
		}

		return this.cachedPixels;
	}

	/**
	 * In JAVA, StImage does not require palette matching (in Bit Operations).
	 * So method "convertToStippleImage_()" does not use this method. This is
	 * why this method is empty stub method. Please refer to the corresponding
	 * method in Smalltalk.
	 * 
	 * @param anImage
	 * 
	 * @return StImage
	 */
	private StImage convertToBitmap_(StImage anImage) {
		return anImage;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param anImage
	 * 
	 * @return StImage
	 */
	private StImage convertToStippleImage_(StImage anImage) {
		StImage maskImage = new StImage(32, 32);
		maskImage.tile_from_in_rule_(new Rectangle(0, 0, 32, 32), new Point(0, 0), anImage, StImage.Over);

		return maskImage;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	private void flushPointerOfPixels() {
		this.cachedPixels = null;
	}
}
