package sra.jun.opengl.texture;

import java.awt.*;
import java.io.*;
import java.util.Vector;
import sra.jun.geometry.basic.*;
import sra.jun.goodies.images.support.JunImageAdjuster;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.objects.JunOpenGL3dObject;
import sra.jun.opengl.support.JunOpenGLRenderingContext;
import sra.smalltalk.*;

/**
 * JunOpenGLTexture class
 * 
 *  @author    nisinaka
 *  @created   1998/10/13 (by nisinaka)
 *  @updated   1999/06/25 (by nisinaka)
 *  @updated   2000/03/17 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLTexture.java,v 6.0 2003/10/28 10:52:22 nisinaka Exp $
 */
public class JunOpenGLTexture extends StObject {
	/** The 3D object on which the texture is mapped. */
	protected JunOpenGL3dObject textureObject = null;
	private StImage textureImage;
	private byte[] cachedPixels;
	private Jun2dPoint[] textureCoordinates;
	private double[] sParameters;
	private double[] tParameters;
	private boolean clampBoolean = true;
	private boolean nearestBoolean = true;
	private boolean modulateBoolean = true;
	private boolean mipmapBoolean = false;
	private boolean _sParametersSet = false;
	private boolean _tParametersSet = false;

	/**
	 * Create a JunOpenGLTexture from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * 
	 * @return sra.jun.opengl.texture.JunOpenGLTexture
	 */
	public static JunOpenGLTexture FromLispList_(JunLispList aList) {
		JunOpenGLTexture texture = new JunOpenGLTexture();
		texture = ImageFromLispList_for_(aList, texture);
		texture = CoordinatesFromLispList_for_(aList, texture);
		texture = ClampFromLispList_for_(aList, texture);
		texture = NearestFromLispList_for_(aList, texture);
		texture = MipmapFromLispList_for_(aList, texture);
		texture = ModulateFromLispList_for_(aList, texture);
		texture = SParametersFromLispList_for_(aList, texture);
		texture = TParametersFromLispList_for_(aList, texture);

		return texture;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anImage DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunOpenGLTexture Image_(Image anImage) {
		JunOpenGLTexture result = new JunOpenGLTexture();
		result.image_(new StImage(anImage));

		return result;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anImage DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunOpenGLTexture Image_(StImage anImage) {
		JunOpenGLTexture result = new JunOpenGLTexture();
		result.image_(anImage);

		return result;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anExtent DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static Point NthPowerExtent_(Point anExtent) {
		int width = anExtent.x;
		int height = anExtent.y;
		int size = 11;
		int[] collection = new int[size];

		for (int n = 0; n < size; n++) {
			collection[n] = (int) Math.pow(2, n);
		}

		for (int index = 1; index < size; index++) {
			int start = collection[index - 1];
			int end = collection[index];

			if ((width > start) & (width <= end)) {
				width = end;
			}

			if ((height > start) & (height <= end)) {
				height = end;
			}
		}

		return new Point(width, height);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aString DOCUMENT ME!
	 * @param forColor DOCUMENT ME!
	 * @param backColor DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunOpenGLTexture Texture1ForString_foreColor_backColor_(String aString, Color forColor, Color backColor) {
		Point extent = SystemResourceSupport._getStringExtentFor_(aString, null);
		Point anExtent = NthPowerExtent_(new Point(extent.x + 2, extent.y + 2));
		Image image = new StImage(anExtent.x, anExtent.y).image();
		Graphics graphics = image.getGraphics();
		graphics.setColor(backColor);

		Rectangle _newBounds = new Rectangle(0, 0, anExtent.x, anExtent.y);
		graphics.fillRect(_newBounds.x, _newBounds.y, _newBounds.width, _newBounds.height);
		graphics.setColor(forColor);

		Point origin = new Point((_newBounds.width - extent.x) / 2, (_newBounds.height - extent.y) / 2);
		graphics.drawString(aString, origin.x, origin.y + graphics.getFontMetrics(graphics.getFont()).getAscent());

		JunOpenGLTexture aTexture = Image_(image);
		Jun2dPoint[] aRunArray = new Jun2dPoint[24];
		Jun2dPoint[] points = new Jun2dPoint[4];
		points[0] = new Jun2dPoint(0.0, 0.0);
		points[1] = new Jun2dPoint(1.0, 0.0);
		points[2] = new Jun2dPoint(1.0, 1.0);
		points[3] = new Jun2dPoint(0.0, 1.0);

		int arrayIndex = 0;

		for (int index = 0; index < 4; index++) {
			aRunArray[arrayIndex++] = points[index];
		}

		for (int index = 3; index > -1; index--) {
			aRunArray[arrayIndex++] = points[index];
		}

		for (int index = 0; index < 16; index++) {
			aRunArray[arrayIndex++] = new Jun2dPoint(0.0, 0.0);
		}

		aTexture.coordinates_(aRunArray);
		aTexture.pointerOfPixels();

		return aTexture;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aString DOCUMENT ME!
	 * @param forColor DOCUMENT ME!
	 * @param backColor DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunOpenGLTexture Texture2ForString_foreColor_backColor_(String aString, Color forColor, Color backColor) {
		Point extent = SystemResourceSupport._getStringExtentFor_(aString, null);
		Point anExtent = NthPowerExtent_(new Point(extent.x + 2, extent.y + 2));
		Image image = new StImage(anExtent.x, anExtent.y).image();
		Graphics graphics = image.getGraphics();
		graphics.setColor(backColor);

		Rectangle _newBounds = new Rectangle(0, 0, anExtent.x, anExtent.y);
		graphics.fillRect(_newBounds.x, _newBounds.y, _newBounds.width, _newBounds.height);
		graphics.setColor(forColor);

		Point origin = new Point((_newBounds.width - extent.x) / 2, (_newBounds.height - extent.y) / 2);
		graphics.drawString(aString, origin.x, origin.y + graphics.getFontMetrics(graphics.getFont()).getAscent());

		JunOpenGLTexture aTexture = Image_(image);
		Jun2dPoint[] aRunArray = new Jun2dPoint[24];
		Jun2dPoint[] points = new Jun2dPoint[4];
		points[0] = new Jun2dPoint(0.0, 0.0);
		points[1] = new Jun2dPoint(1.0, 0.0);
		points[2] = new Jun2dPoint(1.0, 1.0);
		points[3] = new Jun2dPoint(0.0, 1.0);

		int arrayIndex = 0;

		for (int index = 0; index < 4; index++) {
			aRunArray[arrayIndex++] = points[index];
		}

		for (int index = 3; index > -1; index--) {
			aRunArray[arrayIndex++] = points[index];
		}

		for (int index = 0; index < 16; index++) {
			aRunArray[arrayIndex++] = new Jun2dPoint(0.0, 0.0);
		}

		aTexture.coordinates_(aRunArray);
		aTexture.pointerOfPixels();

		return aTexture;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aString DOCUMENT ME!
	 * @param forColor DOCUMENT ME!
	 * @param backColor DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunOpenGLTexture TextureForString_foreColor_backColor_(String aString, Color forColor, Color backColor) {
		return Texture1ForString_foreColor_backColor_(aString, forColor, backColor);
	}

	/**
	 * Read a lisp list and set the clamp of the texture.
	 * 
	 * @param aList DOCUMENT ME!
	 * @param aTexture DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private static JunOpenGLTexture ClampFromLispList_for_(JunLispList aList, JunOpenGLTexture aTexture) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("clamp")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return aTexture;
		}

		aTexture.clamp_(((Boolean) list.tail()).booleanValue());

		return aTexture;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList DOCUMENT ME!
	 * @param aTexture DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private static JunOpenGLTexture CoordinatesFromLispList_for_(JunLispList aList, JunOpenGLTexture aTexture) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("coordinates")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return aTexture;
		}

		Object[] anArray = ((JunLispCons) list.tail()).asArray();
		Jun2dPoint[] coordinates = new Jun2dPoint[anArray.length];
		System.arraycopy(anArray, 0, coordinates, 0, anArray.length);
		aTexture.coordinates_(coordinates);

		return aTexture;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList DOCUMENT ME!
	 * @param aTexture DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private static JunOpenGLTexture ImageFromLispList_for_(JunLispList aList, JunOpenGLTexture aTexture) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("image")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return aTexture;
		}

		aTexture.image_((StImage) list.tail());

		return aTexture;
	}

	/**
	 * Read a lisp list and set the mipmap of the texture.
	 * 
	 * @param aList DOCUMENT ME!
	 * @param aTexture DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private static JunOpenGLTexture MipmapFromLispList_for_(JunLispList aList, JunOpenGLTexture aTexture) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("mipmap")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return aTexture;
		}

		aTexture.mipmap_(((Boolean) list.tail()).booleanValue());

		return aTexture;
	}

	/**
	 * Read a lisp list and set the modulate of the texture.
	 * 
	 * @param aList DOCUMENT ME!
	 * @param aTexture DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private static JunOpenGLTexture ModulateFromLispList_for_(JunLispList aList, JunOpenGLTexture aTexture) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("modulate")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return aTexture;
		}

		aTexture.modulate_(((Boolean) list.tail()).booleanValue());

		return aTexture;
	}

	/**
	 * Read a lisp list and set the clamp of the texture.
	 * 
	 * @param aList DOCUMENT ME!
	 * @param aTexture DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private static JunOpenGLTexture NearestFromLispList_for_(JunLispList aList, JunOpenGLTexture aTexture) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("nearest")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return aTexture;
		}

		aTexture.nearest_(((Boolean) list.tail()).booleanValue());

		return aTexture;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList DOCUMENT ME!
	 * @param aTexture DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private static JunOpenGLTexture SParametersFromLispList_for_(JunLispList aList, JunOpenGLTexture aTexture) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("sParameters")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return aTexture;
		}

		Object[] anArray = ((JunLispCons) list.tail()).asArray();
		double[] parameters = new double[anArray.length];

		for (int i = 0; i < parameters.length; i++) {
			parameters[i] = ((Number) anArray[i]).doubleValue();
		}

		aTexture.parametersForS_(parameters);

		return aTexture;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aList DOCUMENT ME!
	 * @param aTexture DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private static JunOpenGLTexture TParametersFromLispList_for_(JunLispList aList, JunOpenGLTexture aTexture) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object anObject) {
				return new Boolean(anObject instanceof JunLispCons && (((JunLispCons) anObject).head() == $("tParameters")));
			}
		}, new StBlockClosure());

		if (list == null) {
			return aTexture;
		}

		Object[] anArray = ((JunLispCons) list.tail()).asArray();
		double[] parameters = new double[anArray.length];

		for (int i = 0; i < parameters.length; i++) {
			parameters[i] = ((Number) anArray[i]).doubleValue();
		}

		aTexture.parametersForT_(parameters);

		return aTexture;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean clamp() {
		Boolean value = new Boolean(this.clampBoolean);

		if (value.equals(null)) {
			this.clampBoolean = true;
		}

		return this.clampBoolean;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBoolean boolean
	 */
	public void clamp_(boolean aBoolean) {
		this.clampBoolean = aBoolean;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public JunLispCons clampToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("clamp"));
		list.tail_(new Boolean(this.clamp()));

		return list;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public Jun2dPoint[] coordinates() {
		return this.textureCoordinates;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param collectionOfPoint DOCUMENT ME!
	 */
	public void coordinates_(Jun2dPoint[] collectionOfPoint) {
		if (collectionOfPoint == null) {
			this.textureCoordinates = null;
		} else {
			Jun2dPoint[] newCoordinates = new Jun2dPoint[collectionOfPoint.length];

			for (int index = 0; index < collectionOfPoint.length; index++) {
				newCoordinates[index] = (collectionOfPoint[index]);
			}

			this.textureCoordinates = newCoordinates;
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public JunLispList coordinatesToLispList() {
		Jun2dPoint[] coordinates = this.coordinates();
		Object[] collection = new Object[coordinates.length + 1];
		collection[0] = ($("coordinates"));

		for (int index = 0; index < coordinates.length; index++) {
			collection[index + 1] = coordinates[index];
		}

		return JunLispCons.List_(collection);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aRenderingContext DOCUMENT ME!
	 */
	public void disableTextureOn_(JunOpenGLRenderingContext aRenderingContext) {
		aRenderingContext.texture_(null);
		aRenderingContext.disableTexture2d();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aRenderingContext DOCUMENT ME!
	 */
	public void enableTextureOn_(JunOpenGLRenderingContext aRenderingContext) {
		aRenderingContext.texture_(this);
		aRenderingContext.enableTexture2d();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public boolean hasCoordinates() {
		Jun2dPoint[] coordinates = this.textureCoordinates;

		if (coordinates == null) {
			return false;
		} else {
			return coordinates.length != 0;
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public boolean hasParametersS() {
		double[] sParameters = this.sParameters;

		if (sParameters == null) {
			return false;
		} else {
			return sParameters.length != 0;
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public boolean hasParametersT() {
		double[] tParameters = this.tParameters;

		if (tParameters == null) {
			return false;
		} else {
			return tParameters.length != 0;
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public int height() {
		return this.image().height();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public StImage image() {
		if (this.textureImage == null) {
			StImage newImage = new StImage(1, 1);
			newImage.valueAtPoint_put_(new Point(0, 0), Color.black);
			this.textureImage = newImage;
		}

		return this.textureImage;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anImage DOCUMENT ME!
	 */
	public void image_(Image anImage) {
		this.image_(new StImage(anImage));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anImage DOCUMENT ME!
	 */
	public void image_(StImage anImage) {
		if ((anImage.width() == 0) || (anImage.height() == 0)) {
			return;
		}

		this.textureImage = this.convertToTextureImage_(anImage);
		this.flushPointerOfPixels();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public JunLispCons imageToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("image"));
		list.tail_(this.image());

		return list;
	}

	/**
	 * Answer a String which represents the receiver's kind.
	 * 
	 * @return DOCUMENT ME!
	 */
	public StSymbol kindName() {
		return this._className();
	}

	/**
	 * Answer whether linear is set or not.
	 * 
	 * @return DOCUMENT ME!
	 */
	public boolean linear() {
		return !this.nearest();
	}

	/**
	 * Whether linear is set or not.
	 * 
	 * @param aBoolean DOCUMENT ME!
	 */
	public void linear_(boolean aBoolean) {
		this.nearest_(!aBoolean);
	}

	/**
	 * Answer whether mipmap is set or not.
	 * 
	 * @return boolean
	 */
	public boolean mipmap() {
		return mipmapBoolean;
	}

	/**
	 * Whether mipmap is set or not.
	 * 
	 * @param aBoolean boolean
	 */
	public void mipmap_(boolean aBoolean) {
		mipmapBoolean = aBoolean;
	}

	/**
	 * Answer the lisp representation of the mipmap setting.
	 * 
	 * @return DOCUMENT ME!
	 */
	public JunLispCons mipmapToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("mipmap"));
		list.tail_(new Boolean(this.mipmap()));

		return list;
	}

	/**
	 * Answer whether modulate is set or not.
	 * 
	 * @return boolean
	 */
	public boolean modulate() {
		return modulateBoolean;
	}

	/**
	 * Whether modulate is set or not.
	 * 
	 * @param aBoolean boolean
	 */
	public void modulate_(boolean aBoolean) {
		modulateBoolean = aBoolean;
	}

	/**
	 * Answer the lisp representation of the modulate setting.
	 * 
	 * @return DOCUMENT ME!
	 */
	public JunLispCons modulateToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("modulate"));
		list.tail_(new Boolean(this.modulate()));

		return list;
	}

	/**
	 * Answer whether nearest is set or not.
	 * 
	 * @return DOCUMENT ME!
	 */
	public boolean nearest() {
		return nearestBoolean;
	}

	/**
	 * Whether nearest is set or not.
	 * 
	 * @param aBoolean DOCUMENT ME!
	 */
	public void nearest_(boolean aBoolean) {
		nearestBoolean = aBoolean;
	}

	/**
	 * Answer the lisp representation of the nearest setting.
	 * 
	 * @return DOCUMENT ME!
	 */
	public JunLispCons nearestToLispList() {
		JunLispCons list = this.lispCons();
		list.head_($("nearest"));
		list.tail_(new Boolean(this.nearest()));

		return list;
	}

	/**
	 * Answer the 3D object on which the texture is mapped.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject object() {
		return textureObject;
	}

	/**
	 * Set a 3D object on which the texture is mapped.
	 * 
	 * @param a3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public void object_(JunOpenGL3dObject a3dObject) {
		textureObject = a3dObject;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public float[] pointerOfParametersForS() {
		double[] floatArray = this.parametersForS();
		int arraySize = floatArray.length;
		float[] floatArrayPointer = new float[arraySize];

		for (int index = 0; index < arraySize; index++) {
			floatArrayPointer[index] = (float) floatArray[index];
		}

		return floatArrayPointer;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public float[] pointerOfParametersForT() {
		double[] floatArray = this.parametersForT();
		int arraySize = floatArray.length;
		float[] floatArrayPointer = new float[arraySize];

		for (int index = 0; index < arraySize; index++) {
			floatArrayPointer[index] = (float) floatArray[index];
		}

		return floatArrayPointer;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public byte[] pointerOfPixels() {
		byte[] byteArray = this.cachedPixels;

		if (byteArray == null) {
			StImage image = this.image();
			int imageWidth = image.width();
			int imageHeight = image.height();
			byteArray = new byte[imageWidth * imageHeight * 3];

			int index = 0;

			for (int y = imageHeight - 1; y > -1; y = y - 1) {
				for (int x = 0; x < imageWidth; x++) {
					Color color = image.valueAtPoint_(new Point(x, y));
					int red = Math.min(255, Math.max(0, color.getRed()));
					int green = Math.min(255, Math.max(0, color.getGreen()));
					int blue = Math.min(255, Math.max(0, color.getBlue()));
					byteArray[index++] = (byte) red;
					byteArray[index++] = (byte) green;
					byteArray[index++] = (byte) blue;
				}
			}

			this.cachedPixels = byteArray;
		}

		return byteArray;
	}

	/**
	 * Answer whether repeat is set or not.
	 * 
	 * @return boolean
	 */
	public boolean repeat() {
		return !this.clamp();
	}

	/**
	 * Whether repeat is set or not.
	 * 
	 * @param aBoolean boolean
	 */
	public void repeat_(boolean aBoolean) {
		this.clamp_(!aBoolean);
	}

	/**
	 * Convert my s-parametes to a lisp list.
	 * 
	 * @return DOCUMENT ME!
	 */
	public JunLispList sParametersToLispList() {
		double[] parametersForS = this.parametersForS();
		Vector elements = new Vector();
		elements.addElement($("sParameters"));

		for (int index = 0; index < parametersForS.length; index++) {
			elements.addElement(new Double(parametersForS[index]));
		}

		return JunLispCons.List_(elements);
	}

	/**
	 * Answer the lisp representation of the texture.
	 * 
	 * @return DOCUMENT ME!
	 */
	public JunLispCons toLispList() {
		JunLispCons list = this.lispCons();
		list.head_(this.kindName());
		list.add_(this.imageToLispList());

		if (this.hasCoordinates()) {
			list.add_(this.coordinatesToLispList());
		}

		list.add_(this.clampToLispList());
		list.add_(this.nearestToLispList());
		list.add_(this.modulateToLispList());
		list.add_(this.mipmapToLispList());

		if (this.hasParametersS()) {
			list.add_(this.sParametersToLispList());
		}

		if (this.hasParametersT()) {
			list.add_(this.tParametersToLispList());
		}

		return list;
	}

	/**
	 * Convert my t-parametes to a lisp list.
	 * 
	 * @return DOCUMENT ME!
	 */
	public JunLispList tParametersToLispList() {
		double[] parametersForT = this.parametersForT();
		Vector elements = new Vector();
		elements.addElement($("tParameters"));

		for (int index = 0; index < parametersForT.length; index++) {
			elements.addElement(new Double(parametersForT[index]));
		}

		return JunLispCons.List_(elements);
	}

	/**
	 * Write the texture as SFImage of VRML on the writer.
	 * 
	 * @param writer
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public void vrmlSFImageOn_(Writer writer) {
		try {
			StImage image = this.image();
			int width = image.width();
			int height = image.height();
			writer.write(String.valueOf(width));
			writer.write(" ");
			writer.write(String.valueOf(height));
			writer.write(" 3");

			for (int y = height - 1; y >= 0; y--) {
				for (int x = 0; x < width; x++) {
					writer.write(" 0x");
					writer.write(Integer.toHexString(image.getPixel(x, y)).toUpperCase());
				}
			}
		} catch (IOException e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public int width() {
		return this.image().width();
	}

	/**
	 * DOCUMENT ME!
	 */
	public void xNegativeParameters() {
		Jun3dBoundingBox bounds = this.object().boundingBox();
		double height = bounds.height();
		double depth = bounds.depth();

		if ((height == 0.0) || (depth == 0.0)) {
			this.defaultParameters();
		} else {
			double[] defaultS = new double[4];
			defaultS[0] = 0.0;
			defaultS[1] = 1.0 / height * -1;
			defaultS[2] = 0.0;
			defaultS[3] = 0.5;

			double[] defaultT = new double[4];
			defaultT[0] = 0.0;
			defaultT[1] = 0.0;
			defaultT[2] = 1.0 / depth;
			defaultT[3] = 0.5;
			this.parametersForS_(defaultS);
			this.parametersForT_(defaultT);
		}
	}

	/**
	 * DOCUMENT ME!
	 */
	public void xPositiveParameters() {
		Jun3dBoundingBox bounds = this.object().boundingBox();
		double height = bounds.height();
		double depth = bounds.depth();

		if ((height == 0.0) || (depth == 0.0)) {
			this.defaultParameters();
		} else {
			double[] defaultS = new double[4];
			defaultS[0] = 0.0;
			defaultS[1] = 1.0 / height;
			defaultS[2] = 0.0;
			defaultS[3] = 0.5;

			double[] defaultT = new double[4];
			defaultT[0] = 0.0;
			defaultT[1] = 0.0;
			defaultT[2] = 1.0 / depth;
			defaultT[3] = 0.5;
			this.parametersForS_(defaultS);
			this.parametersForT_(defaultT);
		}
	}

	/**
	 * DOCUMENT ME!
	 */
	public void yNegativeParameters() {
		Jun3dBoundingBox bounds = this.object().boundingBox();
		double width = bounds.width();
		double depth = bounds.depth();

		if ((width == 0.0) || (depth == 0.0)) {
			this.defaultParameters();
		} else {
			double[] defaultS = new double[4];
			defaultS[0] = 1.0 / width;
			defaultS[1] = 0.0;
			defaultS[2] = 0.0;
			defaultS[3] = 0.5;

			double[] defaultT = new double[4];
			defaultT[0] = 0.0;
			defaultT[1] = 0.0;
			defaultT[2] = 1.0 / depth;
			defaultT[3] = 0.5;
			this.parametersForS_(defaultS);
			this.parametersForT_(defaultT);
		}
	}

	/**
	 * DOCUMENT ME!
	 */
	public void yPositiveParameters() {
		Jun3dBoundingBox bounds = this.object().boundingBox();
		double width = bounds.width();
		double depth = bounds.depth();

		if ((width == 0.0) || (depth == 0.0)) {
			this.defaultParameters();
		} else {
			double[] defaultS = new double[4];
			defaultS[0] = 1.0 / width * -1;
			defaultS[1] = 0.0;
			defaultS[2] = 0.0;
			defaultS[3] = 0.5;

			double[] defaultT = new double[4];
			defaultT[0] = 0.0;
			defaultT[1] = 0.0;
			defaultT[2] = 1.0 / depth;
			defaultT[3] = 0.5;
			this.parametersForS_(defaultS);
			this.parametersForT_(defaultT);
		}
	}

	/**
	 * DOCUMENT ME!
	 */
	public void zNegativeParameters() {
		Jun3dBoundingBox bounds = this.object().boundingBox();
		double width = bounds.width();
		double height = bounds.height();

		if ((width == 0.0) || (height == 0.0)) {
			this.defaultParameters();
		} else {
			double[] defaultS = new double[4];
			defaultS[0] = 1.0 / width * -1;
			defaultS[1] = 0.0;
			defaultS[2] = 0.0;
			defaultS[3] = 0.5;

			double[] defaultT = new double[4];
			defaultT[0] = 0.0;
			defaultT[1] = 1.0 / height;
			defaultT[2] = 0.0;
			defaultT[3] = 0.5;
			this.parametersForS_(defaultS);
			this.parametersForT_(defaultT);
		}
	}

	/**
	 * DOCUMENT ME!
	 */
	public void zPositiveParameters() {
		Jun3dBoundingBox bounds = this.object().boundingBox();
		double width = bounds.width();
		double height = bounds.height();

		if ((width == 0.0) || (height == 0.0)) {
			this.defaultParameters();
		} else {
			double[] defaultS = new double[4];
			defaultS[0] = 1.0 / width;
			defaultS[1] = 0.0;
			defaultS[2] = 0.0;
			defaultS[3] = 0.5;

			double[] defaultT = new double[4];
			defaultT[0] = 0.0;
			defaultT[1] = 1.0 / height;
			defaultT[2] = 0.0;
			defaultT[3] = 0.5;
			this.parametersForS_(defaultS);
			this.parametersForT_(defaultT);
		}
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param anImage
	 * 
	 * @return StImage
	 */
	private StImage convertToNthPowerExtentImage_(StImage anImage) {
		StImage image = anImage;

		if ((image.width() == 0) || (image.height() == 0)) {
			return image;
		}

		Point extent = NthPowerExtent_(anImage.extent());
		image = JunImageAdjuster.Adjust_extent_(image, new Dimension(extent.x, extent.y));

		return image;
	}

	/**
	 * This is only stubbing method. an StImage always use standard RGB
	 * palette. So, this palette converting method is not needed.
	 * 
	 * @param anImage
	 * 
	 * @return StImage
	 */
	private StImage convertToPaletteImage_(StImage anImage) {
		return anImage;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param anImage
	 * 
	 * @return StImage
	 */
	private StImage convertToTextureImage_(StImage anImage) {
		StImage image = this.convertToNthPowerExtentImage_(anImage);
		image = this.convertToPaletteImage_(image);

		return image;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private Class defaultLispConsClass() {
		return JunLispCons.class;
	}

	/**
	 * DOCUMENT ME!
	 */
	private void defaultParameters() {
		double[] defaultS = new double[4];
		defaultS[0] = 0.0;
		defaultS[1] = 1.0;
		defaultS[2] = 0.0;
		defaultS[3] = 0.0;

		double[] defaultT = new double[4];
		defaultT[0] = 0.0;
		defaultT[1] = 0.0;
		defaultT[2] = 1.0;
		defaultT[3] = 0.0;
		this.parametersForS_(defaultS);
		this.parametersForT_(defaultT);
	}

	/**
	 * DOCUMENT ME!
	 */
	private void flushPointerOfPixels() {
		this.cachedPixels = null;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private JunLispCons lispCons() {
		return (JunLispCons) JunLispCons.Cell();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private double[] parametersForS() {
		if (this._sParametersSet == false) {
			if (this.object().equals(null)) {
				this.defaultParameters();
			} else {
				this.xPositiveParameters();
			}

			this._sParametersSet = true;
		}

		return this.sParameters;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anArray DOCUMENT ME!
	 */
	private void parametersForS_(double[] anArray) {
		this.sParameters = anArray;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	private double[] parametersForT() {
		if (this._tParametersSet == false) {
			if (this.object().equals(null)) {
				this.defaultParameters();
			} else {
				this.xPositiveParameters();
			}

			this._tParametersSet = true;
		}

		return this.tParameters;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param anArray DOCUMENT ME!
	 */
	private void parametersForT_(double[] anArray) {
		this.tParameters = anArray;
	}
}
