package sra.jun.system.support;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import sra.smalltalk.*;
import sra.gl4jun.GLjRenderingMode;
import sra.jun.goodies.cursors.JunCursors;
import sra.jun.goodies.display.*;
import sra.jun.goodies.lisp.JunLispInterpreter;
import sra.jun.goodies.prolog.JunPrologInterpreter;
import sra.jun.opengl.objects.JunOpenGL3dObject;
import sra.jun.opengl.display.JunOpenGLDisplayModel;
import sra.jun.system.framework.JunDialog;

/**
 * JunLauncher class
 * 
 *  @author    nisinaka
 *  @created   1998/11/16 (by nisinaka)
 *  @updated   2000/02/17 (by MATSUDA Ryouichi)
 *  @updated   2002/11/21 (by nisinaka)
 *  @updated   2003/03/03 (by Mitsuhiro Asada)
 *  @updated   2003/03/20 (by Mitsuhiro Asada)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunLauncher.java,v 6.0 2003/10/28 10:52:22 nisinaka Exp $
 */
public class JunLauncher extends JunDisplayModel {
	protected StMenuModel menuModel;

	public final static int _DefaultWidth = 187;
	protected static boolean _closeToExit = true;

	/**
	 * Print this usage.
	 * 
	 * @category Usage
	 */
	public static void _PrintUsage() {
		System.out.println("Usage: java sra.jun.system.support.JunLauncher {options}");
		System.out.println();
		System.out.println("  Options are:");
		System.out.println("    -help (or help)    Show this usage report.");

		System.out.println("    -view <type>       View mode. the <type> is selected from a menu of ");
		System.out.println("                       'Default View Mode' in JunLauncher :");
		System.out.println("                           Awt : Awt mode");
		UIManager.LookAndFeelInfo[] infos = UIManager.getInstalledLookAndFeels();
		if (infos.length > 0) {
			System.out.print("                           ");
			for (int i = 0; i < infos.length; i++) {
				System.out.print(infos[i].getName());
				if (i < infos.length - 1) {
					System.out.print(", ");
				}
			}
			System.out.println(" : Swing mode");
		}
		System.out.println("                           Swing only : Swing only mode.");

		System.out.println("    -rendering <type>  Rendering Mode :");
		System.out.println("                         NOTHING : nothing");
		System.out.println("                         SINGLE_BUFFER : single buffer");
		System.out.println("                         IMAGE : image");
		System.out.println("                         DIRECT : direct");
		System.out.println("                         DOUBLE_BUFFER : double buffer");
		System.out.println();

		System.out.println("By defalut, view mode is 'Awt' and rendering mode is 'double buffer'.");
	}

	/**
	 * Process arguments - set object properties from the list of args.
	 *
	 * @return boolean
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static boolean _ProcessArgs_(String[] args) {
		if (args.length == 0) {
			// StApplicationModel.SetDefaultViewMode(StApplicationModel.VIEW_AWT);
			// JunSystem.setDefaultRenderingMode(GLjRenderingMode.DOUBLE_BUFFER);
			return true;
		}

		try {
			for (int i = 0; i < args.length; i++) {
				String arg = args[i].toLowerCase();

				if (arg.equals("-help") || arg.equals("help")) {
					_PrintUsage();
					return false;
				} else if (arg.startsWith("-view")) {
					i++;
					if (i >= args.length) {
						throw new Exception("-view type argument is missing.");
					}
					String viewMode = args[i].toLowerCase();
					boolean setFlag = false;
					if (viewMode.equals("Awt".toLowerCase())) {
						StApplicationModel.SetDefaultViewMode(StApplicationModel.VIEW_AWT);
						setFlag = true;
					} else if (viewMode.equals("Swing only".toLowerCase())) {
						StApplicationModel.SetDefaultViewMode(StApplicationModel.VIEW_SWING_ONLY);
						setFlag = true;
					} else {
						UIManager.LookAndFeelInfo[] infos = UIManager.getInstalledLookAndFeels();
						for (int j = 0; j < infos.length; j++) {
							if (viewMode.equals(infos[j].getName().toLowerCase())) {
								StApplicationModel.SetDefaultViewMode(StApplicationModel.VIEW_SWING);
								UIManager.setLookAndFeel(infos[j].getClassName());
								setFlag = true;
								break;
							}
						}
						if (setFlag == false) {
							throw new Exception("\"" + args[i] + "\" argument is missing.");
						}
					}
				} else if (arg.startsWith("-rendering")) {
					i++;
					if (i >= args.length) {
						throw new Exception("-rendering type argument is missing.");
					}
					String renderingMode = args[i].toUpperCase();
					if (renderingMode.equals("NOTHING")) {
						JunSystem.setDefaultRenderingMode(GLjRenderingMode.NOTHING);
					} else if (renderingMode.equals("SINGLE_BUFFER")) {
						JunSystem.setDefaultRenderingMode(GLjRenderingMode.SINGLE_BUFFER);
					} else if (renderingMode.equals("IMAGE")) {
						JunSystem.setDefaultRenderingMode(GLjRenderingMode.IMAGE);
					} else if (renderingMode.equals("DIRECT")) {
						JunSystem.setDefaultRenderingMode(GLjRenderingMode.DIRECT);
					} else if (renderingMode.equals("DOUBLE_BUFFER")) {
						JunSystem.setDefaultRenderingMode(GLjRenderingMode.DOUBLE_BUFFER);
					} else {
						throw new Exception("\"" + args[i] + "\" argument is missing.");
					}
				} else {
					throw new Exception("\"" + args[i] + "\" argument is missing.");
				}
			}
		} catch (Exception e) {
			System.out.println(e.getMessage());
			_PrintUsage();
			return false;
		}

		return true;
	}

	/**
	 * Open JunLauncher.
	 * 
	 * @param args java.lang.String[]
	 * @category Main
	 */
	public static void main(String[] args) {
		if (JunLauncher._ProcessArgs_(args) == false) {
			System.exit(0);
		}

		JunLauncher aLauncher = new JunLauncher();
		aLauncher.open();
	}

	/**
	 * Answer the array of all TestExamples classes.
	 * 
	 * @return java.lang.Class[]
	 * @category Resources
	 */
	public static Class[] TestExamplesClasses() {
		final Class[] testExamplesClasses =
			{
				sra.jun.collections.sequences.JunDoubleMatrixTestExamples.class,
				sra.jun.collections.sequences.JunLinearEquationsTestExamples.class,
				sra.jun.collections.support.JunCorrelationTestExamples.class,
				sra.jun.csg.abstracts.JunCSGTestExamples.class,
				sra.jun.dxf.JunCADModelTestExamples.class,
				sra.jun.dxf.JunDXFParserTestExamples.class,
				sra.jun.geometry.basic.Jun2dBoundingBoxTestExamples.class,
				sra.jun.geometry.basic.Jun2dPointTestExamples.class,
				sra.jun.geometry.basic.Jun3dBoundingBoxTestExamples.class,
				sra.jun.geometry.basic.Jun3dPointTestExamples.class,
				sra.jun.geometry.basic.JunAngleTestExamples.class,
				sra.jun.geometry.boundaries.Jun3dBoundingBallTestExamples.class,
				sra.jun.geometry.boundaries.JunBorderGeneratorModelTestExamples.class,
				sra.jun.geometry.boundaries.JunBorderGeneratorTestExamples.class,
				sra.jun.geometry.coordinate.Jun3dCoordinateSystemTestExamples.class,
				sra.jun.geometry.curves.Jun2dLineTestExamples.class,
				sra.jun.geometry.curves.Jun3dLineTestExamples.class,
				sra.jun.geometry.surfaces.JunPlaneTestExamples.class,
				sra.jun.geometry.surfaces.JunPolygonTestExamples.class,
				sra.jun.geometry.transformations.Jun3dTransformationTestExamples.class,
				sra.jun.goodies.bulletinboard.JunBulletinBoardTestExamples.class,
				sra.jun.goodies.button.JunButtonModelTestExamples.class,
				sra.jun.goodies.calendar.JunCalendarClockDateAndTimeTestExamples.class,
				sra.jun.goodies.calendar.JunCalendarClockDateTestExamples.class,
				sra.jun.goodies.calendar.JunCalendarClockTimeTestExamples.class,
				sra.jun.goodies.calendar.JunCalendarDateAndTimeDialogTestExamples.class,
				sra.jun.goodies.calendar.JunCalendarDateAndTimeIntervalDialogTestExamples.class,
				sra.jun.goodies.calendar.JunCalendarDateAndTimeIntervalModelTestExamples.class,
				sra.jun.goodies.calendar.JunCalendarDateAndTimeModelTestExamples.class,
				sra.jun.goodies.calendar.JunCalendarDateDialogTestExamples.class,
				sra.jun.goodies.calendar.JunCalendarDateIntervalDialogTestExamples.class,
				sra.jun.goodies.calendar.JunCalendarDateIntervalModelTestExamples.class,
				sra.jun.goodies.calendar.JunCalendarDateModelTestExamples.class,
				sra.jun.goodies.calendar.JunCalendarTimeDialogTestExamples.class,
				sra.jun.goodies.calendar.JunCalendarTimeIntervalDialogTestExamples.class,
				sra.jun.goodies.calendar.JunCalendarTimeIntervalModelTestExamples.class,
				sra.jun.goodies.calendar.JunCalendarTimeModelTestExamples.class,
				sra.jun.goodies.catalog.JunImageCatalogTestExamples.class,
				sra.jun.goodies.catalog.JunMovieCatalogTestExamples.class,
				sra.jun.goodies.catalog.JunVisualCatalogTestExamples.class,
				sra.jun.goodies.colors.JunColorChoiceDialogTestExamples.class,
				sra.jun.goodies.colors.JunColorChoiceTestExamples.class,
				sra.jun.goodies.colors.JunColorSpuitTestExamples.class,
				sra.jun.goodies.display.JunDisplayModelTestExamples.class,
				sra.jun.goodies.files.JunFileDialogTestExamples.class,
				sra.jun.goodies.files.JunFileModelTestExamples.class,
				sra.jun.goodies.font.JunFontDialogTestExamples.class,
				sra.jun.goodies.font.JunFontModelTestExamples.class,
				sra.jun.goodies.freehand.JunFreehandNoteTestExamples.class,
				sra.jun.goodies.freehand.JunFreehandPageTestExamples.class,
				sra.jun.goodies.gauge.JunSimpleGaugeModelTestExamples.class,
				sra.jun.goodies.images.support.JunImageAdjusterTestExamples.class,
				sra.jun.goodies.images.support.JunImageProcessorTestExamples.class,
				sra.jun.goodies.images.support.JunImageToAsciiTestExamples.class,
				sra.jun.goodies.lisp.JunLispConsTestExamples.class,
				sra.jun.goodies.lisp.JunLispInterpreterTestExamples.class,
				sra.jun.goodies.lisp.JunLispParserTestExamples.class,
				sra.jun.goodies.milliseconds.JunMillisecondDialogTestExamples.class,
				sra.jun.goodies.milliseconds.JunMillisecondModelTestExamples.class,
				sra.jun.goodies.movie.JunMoviePlayerTestExamples.class,
				sra.jun.goodies.movie.JunScreenRecorderTestExamples.class,
				sra.jun.goodies.multimedia.JunMultiMediaImageModelTestExamples.class,
				sra.jun.goodies.multimedia.JunMultiMediaMovieModelTestExamples.class,
				sra.jun.goodies.multimedia.JunMultiMediaTextModelTestExamples.class,
				sra.jun.goodies.multimedia.JunMultiMediaViewerTestExamples.class,
				sra.jun.goodies.parameter.JunParameterModelTestExamples.class,
				sra.jun.goodies.parameter.JunParametersModelTestExamples.class,
				sra.jun.goodies.progress.JunCursorAnimatorTestExamples.class,
				sra.jun.goodies.progress.JunProgressTestExamples.class,
				sra.jun.goodies.prolog.JunPrologInterpreterTestExamples.class,
				sra.jun.goodies.track.JunSimpleSliderModelTestExamples.class,
				sra.jun.goodies.track.JunTrackSliderModelTestExamples.class,
				sra.jun.goodies.track.JunTrackerModelTestExamples.class,
				sra.jun.goodies.url.JunURLTestExamples.class,
				sra.jun.goodies.wheels.JunThumbWheelTestExamples.class,
				sra.jun.metaball.solid.JunMetaballSolidTestExamples.class,
				sra.jun.octree.basic.Jun27TreeTestExamples.class,
				sra.jun.octree.basic.JunOctreeTestExamples.class,
				sra.jun.opengl.chart.JunChartBandTestExamples.class,
				sra.jun.opengl.chart.JunChartBarTestExamples.class,
				sra.jun.opengl.chart.JunChartCombinationTestExamples.class,
				sra.jun.opengl.chart.JunChartDataSheetTestExamples.class,
				sra.jun.opengl.chart.JunChartDataTestExamples.class,
				sra.jun.opengl.chart.JunChartDataWithNumericalKeysTestExamples.class,
				sra.jun.opengl.chart.JunChartLineTestExamples.class,
				sra.jun.opengl.chart.JunChartLineWithNumericalKeysTestExamples.class,
				sra.jun.opengl.chart.JunChartPieTestExamples.class,
				sra.jun.opengl.chart.JunChartPlotTestExamples.class,
				sra.jun.opengl.chart.JunChartPlotWithNumericalKeysTestExamples.class,
				sra.jun.opengl.chart.JunChartRadarTestExamples.class,
				sra.jun.opengl.display.JunOpenGL3dModelTestExamples.class,
				sra.jun.opengl.display.JunOpenGLDisplayLightTestExamples.class,
				sra.jun.opengl.display.JunOpenGLDisplayModelTestExamples.class,
				sra.jun.opengl.display.JunOpenGLShowModelTestExamples.class,
				sra.jun.opengl.flux.JunOpenGLFluxMakerTestExamples.class,
				sra.jun.opengl.flux.JunOpenGLFluxModelTestExamples.class,
				sra.jun.opengl.grapher.JunOpenGL3dArcTestExamples.class,
				sra.jun.opengl.grapher.JunOpenGL3dGraphTestExamples.class,
				sra.jun.opengl.grapher.JunOpenGL3dGrapherTestExamples.class,
				sra.jun.opengl.grapher.JunOpenGL3dNodeTestExamples.class,
				sra.jun.opengl.objects.JunOpenGL3dNurbsCurveTestExamples.class,
				sra.jun.opengl.objects.JunOpenGL3dNurbsSurfaceTestExamples.class,
				sra.jun.opengl.objects.JunOpenGL3dObjectTestExamples.class,
				sra.jun.opengl.objects.JunOpenGL3dVertexesTestExamples.class,
				sra.jun.opengl.parameterizedbody.JunParameterizedBodyTestExamples.class,
				sra.jun.opengl.parameterizedbody.JunParameterizedBodyModelTestExamples.class,
				sra.jun.opengl.plotter.JunOpenGLPlotterTestExamples.class,
				sra.jun.opengl.rotation.JunOpenGLRotationApplicationModelTestExamples.class,
				sra.jun.opengl.rotation.JunOpenGLRotationModelTestExamples.class,
				sra.jun.opengl.texture.JunOpenGLStippleTestExamples.class,
				sra.jun.opengl.texture.JunOpenGLTextureTestExamples.class,
				sra.jun.system.framework.JunDialogTestExamples.class,
				sra.jun.system.support.JunSmallCompilerTestExamples.class,
				sra.jun.system.support.JunTestExamples.class,
				sra.jun.terrain.editor.JunTerrainEditModelTestExamples.class,
				sra.jun.topology.elements.JunBodyTestExamples.class,
				sra.jun.topology.euleroperators.JunKCLMPLTestExamples.class,
				sra.jun.topology.euleroperators.JunKDEVTestExamples.class,
				sra.jun.topology.euleroperators.JunKELTestExamples.class,
				sra.jun.topology.euleroperators.JunKEVTestExamples.class,
				sra.jun.topology.euleroperators.JunKEVVLTestExamples.class,
				sra.jun.topology.euleroperators.JunKVETestExamples.class,
				sra.jun.topology.euleroperators.JunMDEVTestExamples.class,
				sra.jun.topology.euleroperators.JunMEKLTestExamples.class,
				sra.jun.topology.euleroperators.JunMELTestExamples.class,
				sra.jun.topology.euleroperators.JunMEVTestExamples.class,
				sra.jun.topology.euleroperators.JunMEVVLTestExamples.class,
				sra.jun.topology.euleroperators.JunMVETestExamples.class,
				sra.jun.topology.geometricoperators.JunKCVTestExamples.class,
				sra.jun.topology.geometricoperators.JunKSFTestExamples.class,
				sra.jun.topology.geometricoperators.JunMCVTestExamples.class,
				sra.jun.topology.geometricoperators.JunMSFTestExamples.class,
				sra.jun.topology.geometricoperators.JunTCVTestExamples.class,
				sra.jun.topology.geometricoperators.JunTSFTestExamples.class,
				sra.jun.topology.geometricoperators.JunTVTestExamples.class,
				sra.jun.topology.globaloperators.JunADDTestExamples.class,
				sra.jun.topology.globaloperators.JunLineSCALETestExamples.class,
				sra.jun.topology.globaloperators.JunMIRRORTestExamples.class,
				sra.jun.topology.globaloperators.JunNEGATETestExamples.class,
				sra.jun.topology.globaloperators.JunPlaneSCALETestExamples.class,
				sra.jun.topology.globaloperators.JunPointSCALETestExamples.class,
				sra.jun.topology.globaloperators.JunSEPARATETestExamples.class,
				sra.jun.topology.setoperators.JunDIFFTestExamples.class,
				sra.jun.topology.setoperators.JunINTERTestExamples.class,
				sra.jun.topology.setoperators.JunUNIONTestExamples.class,
				sra.jun.voronoi.twoD.diagram.JunVoronoi2dProcessorTestExamples.class,
				sra.jun.voronoi.twoD.diagram.JunVoronoi2dDiagramTestExamples.class,
				sra.jun.voronoi.twoD.diagram.JunDelaunay2dDiagramTestExamples.class,
				sra.jun.voronoi.twoD.diagram.JunVoronoi2dDotTestExamples.class,
				sra.jun.voronoi.twoD.diagram.JunVoronoi2dLineTestExamples.class,
				sra.jun.voronoi.twoD.diagram.JunVoronoi2dPointTestExamples.class,
				sra.jun.voronoi.twoD.diagram.JunVoronoi2dSideTestExamples.class,
				sra.jun.voronoi.twoD.diagram.JunVoronoi2dTriangleTestExamples.class,
				sra.jun.vrml.support.JunVrmlCompilerTestExamples.class,
				sra.jun.vrml.support.JunVrmlGenerator10TestExamples.class,
				sra.jun.vrml.support.JunVrmlGenerator20TestExamples.class,
				sra.jun.vrml.support.JunVrmlGenerator97TestExamples.class,
				sra.jun.vrml.support.JunVrmlParser10TestExamples.class,
				sra.jun.vrml.support.JunVrmlParser20TestExamples.class,
				sra.jun.vrml.support.JunVrmlParser97TestExamples.class,
				sra.jun.vrml.support.JunVrmlRenderingContextTestExamples.class,
				sra.qt4jun.JunQTInterfaceTestExamples.class,
				sra.smalltalk.StImageTestExamples.class,
				};

		return testExamplesClasses;
	}

	/**
	 * Answer an image of about dialog.
	 * 
	 * @return sra.smalltalk.StImage
	 * @category about
	 */
	public StImage _aboutImage() {
		Font font = new Font("Dialog", Font.BOLD, 12);
		int fontHeight = SystemResourceSupport.getFontMetrics(font).getHeight();
		int widthMargin = 20;
		int heightMargin = 6;

		StImage logoImage = new StImage(JunSystem.LogoImage());
		StImage aboutImage = new StImage(logoImage.width() + widthMargin * 2, logoImage.height() + fontHeight * 2 + heightMargin * 2);
		Graphics gc = null;
		try {
			gc = aboutImage.image().getGraphics();
			gc.setColor(Color.white);
			gc.fillRect(0, 0, aboutImage.width(), aboutImage.height());

			// Logo image
			gc.drawImage(logoImage.image(), widthMargin, fontHeight + heightMargin, Color.white, null);

			// System and Version
			String top = JunSystem.FullName();
			gc.setColor(Color.black);
			gc.setFont(font);
			FontMetrics metrics = gc.getFontMetrics();
			gc.drawString(top, widthMargin + 5, metrics.getAscent() + heightMargin / 2);

			// Date
			String bottom = JunSystem.Date();
			gc.setFont(new Font("Dialog", Font.PLAIN, 12));
			metrics = gc.getFontMetrics();
			gc.drawString(bottom, aboutImage.width() - metrics.stringWidth(bottom) - widthMargin - 5, aboutImage.height() - metrics.getDescent() - (heightMargin / 2));

			aboutImage.image().flush();
		} finally {
			if (gc != null) {
				gc.dispose();
				gc = null;
			}
		}

		return aboutImage;
	}

	/**
	 * Invoked when a menu item has been selected.
	 * 
	 * @param ev java.awt.event.ActionEvent
	 * @throws SmalltalkException DOCUMENT ME!
	 * @category events
	 */
	public void actionPerformed(ActionEvent ev) {
		String command = ev.getActionCommand();

		if (command.equals("All")) {
			Class[] classes = TestExamplesClasses();
			for (int i = 0; i < classes.length; i++) {
				JunTestExamples.ExecuteMethods(classes[i]);
			}
		} else {
			int index = command.indexOf("#");
			if (index < 0) {
				this.executeMethod(this, command);
			} else {
				String className = command.substring(0, index);
				String methodName = command.substring(index + 1, command.length());
				try {
					Class aClass = Class.forName(className);
					if (methodName.equals("All")) {
						JunTestExamples.ExecuteMethods(aClass);
					} else if (methodName.equals("Spawn")) {
						JunTestExamples aTestExamples = (JunTestExamples) aClass.newInstance();
						aTestExamples.closeToExit(false);
					} else {
						this.executeMethod(aClass, methodName);
					}
				} catch (Exception e) {
					throw new SmalltalkException(e);
				}
			}
		}
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 * @category defaults
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunLauncherViewAwt(this);
		} else {
			return new JunLauncherViewSwing(this);
		}
	}

	/**
	 * Execute all methods specified with the method name.
	 * 
	 * @return java.lang.Object
	 * @param aClass java.lang.Class
	 * @param methodName java.lang.String
	 * @category events
	 */
	public Object executeMethod(Class aClass, String methodName) {
		JunCursors cursor = new JunCursors(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		try {
			cursor._show();
			return StObject._Perform(aClass, methodName);
		} catch (Exception e) {
			throw new SmalltalkException(e);
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Execute all methods specified with the method name.
	 * 
	 * @return java.lang.Object
	 * @param anObject java.lang.Object
	 * @param methodName java.lang.String
	 * @category events
	 */
	public Object executeMethod(StObject anObject, String methodName) {
		JunCursors cursor = new JunCursors(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		try {
			cursor._show();
			return anObject.perform_(methodName);
		} catch (Exception e) {
			throw new SmalltalkException(e);
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Answer a menu model.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 * @category menu accessing
	 */
	public StMenuModel menuModel() {
		if (menuModel == null) {
			menuModel = defaultMenu();
		}

		return menuModel;
	}

	/**
	 * Open an about dialog.
	 * 
	 * @category menu messages
	 */
	public void openAbout() {
		JunLauncherView aView = (JunLauncherView) this.getView();
		if (aView != null) {
			aView._openDialog_("About", this._aboutImage(), null);
		}
	}

	/**
	 * Open a band chart.
	 * 
	 * @category menu messages
	 */
	public void openBandChart() {
		sra.jun.opengl.chart.JunChartBandTestExamples.Example6();
	}

	/**
	 * Open a bar chart.
	 * 
	 * @category menu messages
	 */
	public void openBarChart() {
		sra.jun.opengl.chart.JunChartBarTestExamples.Example6();
	}

	/**
	 * Open a viewfinder with a benzene.
	 * 
	 * @category menu messages
	 */
	public void openBenzene() {
		sra.jun.opengl.display.JunOpenGLDisplayModelTestExamples.Example8();
	}

	/**
	 * Open a border generator.
	 * 
	 * @category menu messages
	 */
	public void openBorderGenerator() {
		sra.jun.geometry.boundaries.JunBorderGeneratorModelTestExamples.Example2();
	}

	/**
	 * Open a JunParameterizedBodyModel with a chair.
	 * 
	 * @category menu messages
	 */
	public void openChair() {
		sra.jun.opengl.parameterizedbody.JunParameterizedBodyModelTestExamples.Example1();
	}

	/**
	 * Open a JunOpenGL3dGrapher with a class tree.
	 * 
	 * @category menu messages
	 */
	public void openClassTree() {
		sra.jun.opengl.grapher.JunOpenGL3dGrapherTestExamples.Example1();
	}

	/**
	 * Open a combination chart.
	 * 
	 * @category menu messages
	 */
	public void openCombinationChart() {
		sra.jun.opengl.chart.JunChartCombinationTestExamples.Example1();
	}

	/**
	 * Open a copyright dialog.
	 * 
	 * @category menu messages
	 */
	public void openCopyright() {
		JunLauncherView aView = (JunLauncherView) this.getView();
		if (aView != null) {
			aView._openTextViewer_("Copyright", JunSystem.Copyright(), new Dimension(500, 160));
		}
	}

	/**
	 * Open a viewfinder with a sphere.
	 * 
	 * @category menu messages
	 */
	public void openCube() {
		sra.jun.opengl.display.JunOpenGLDisplayModelTestExamples.Example2();
	}

	/**
	 * Open a JunOpenGLFluxModel.
	 * 
	 * @category menu messages
	 */
	public void openFlux() {
		(new sra.jun.opengl.flux.JunOpenGLFluxModel()).open();
	}

	/**
	 * Open a JunOpenGL3dGrapher with a class tree.
	 * 
	 * @category menu messages
	 */
	public void openGlass() {
		sra.jun.opengl.rotation.JunOpenGLRotationApplicationModelTestExamples.Example7();
	}

	/**
	 * Open a JunOpenGLGrapher.
	 * 
	 * @category menu messages
	 */
	public void openGrapher() {
		(new sra.jun.opengl.grapher.JunOpenGL3dGrapher()).open();
	}

	/**
	 * Open an information dialog.
	 * 
	 * @category menu messages
	 */
	public void openInformation() {
		JunLauncherView aView = (JunLauncherView) this.getView();
		if (aView != null) {
			aView._openTextViewer_("Information", JunSystem.Information(), new Dimension(600, 360));
		}
	}

	/**
	 * Open a license dialog.
	 * 
	 * @category menu messages
	 */
	public void openLicense() {
		JunLauncherView aView = (JunLauncherView) this.getView();
		if (aView != null) {
			aView._openTextViewer_("The modified BSD license", JunSystem.License(), new Dimension(600, 360));
		}
	}

	/**
	 * Open a line chart.
	 * 
	 * @category menu messages
	 */
	public void openLineChart() {
		sra.jun.opengl.chart.JunChartLineTestExamples.Example6();
	}

	/**
	 * Open a JunLispInterpreter.
	 * 
	 * @category menu messages
	 */
	public void openLispInterpreter() {
		JunLispInterpreter prologInterpreter = new JunLispInterpreter();
		prologInterpreter.open();
		prologInterpreter.textValue().value_("% Select the following expression.\n% Then do it from the popup menu.\n\n(append '(1 2) '(3 4))");
	}

	/**
	 * Open a Movie or Sound.
	 * 
	 * @category menu messages
	 */
	public void openMovieOrSound() {
		(new sra.jun.goodies.movie.JunMoviePlayer()).open();
	}

	/**
	 * Open a Multi-Media Viewer.
	 * 
	 * @category menu messages
	 */
	public void openMultiMediaViewer() {
		(new sra.jun.goodies.multimedia.JunMultiMediaViewer()).open();
	}

	/**
	 * Open a pie chart.
	 * 
	 * @category menu messages
	 */
	public void openPieChart() {
		sra.jun.opengl.chart.JunChartPieTestExamples.Example3();
	}

	/**
	 * Open a JunOpenGLFluxModel with a planet
	 * 
	 * @category menu messages
	 */
	public void openPlanet() {
		sra.jun.opengl.flux.JunOpenGLFluxModelTestExamples.Example4();
	}

	/**
	 * Open a plot chart.
	 * 
	 * @category menu messages
	 */
	public void openPlotChart() {
		sra.jun.opengl.chart.JunChartPlotTestExamples.Example6();
	}

	/**
	 * Open a JunOpenGLGrapher.
	 * 
	 * @category menu messages
	 */
	public void openPlotter() {
		(new sra.jun.opengl.plotter.JunOpenGLPlotter()).open();
	}

	/**
	 * Open a JunPrologInterpreter.
	 * 
	 * @category menu messages
	 */
	public void openPrologInterpreter() {
		JunPrologInterpreter prologInterpreter = new JunPrologInterpreter();
		prologInterpreter.open();
		prologInterpreter.textValue().value_("% Select the following expression.\n% Then do it from the popup menu.\n\n?-append(X,Y,[1,2,3,4]).");
	}

	/**
	 * Open a radar chart.
	 * 
	 * @category menu messages
	 */
	public void openRadarChart() {
		sra.jun.opengl.chart.JunChartRadarTestExamples.Example5();
	}

	/**
	 * Open a JunParameterizedBodyModel with a robot arm.
	 * 
	 * @category menu messages
	 */
	public void openRobotArm() {
		sra.jun.opengl.parameterizedbody.JunParameterizedBodyModelTestExamples.Example2();
	}

	/**
	 * Open a JunOpenGLMicrocosmModel.
	 * 
	 * @category menu messages
	 */
	public void openRoom() {
		sra.jun.opengl.microcosm.JunOpenGLMicrocosmModelTestExamples.Example1();
	}

	/**
	 * Open a JunOpenGLRotationApplicationModel.
	 * 
	 * @category menu messages
	 */
	public void openRotaionalBody() {
		(new sra.jun.opengl.rotation.JunOpenGLRotationApplicationModel()).open();
	}

	/**
	 * Open a JunScreenRecorder.
	 * 
	 * @category menu messages
	 */
	public void openScreenRecorder() {
		(new sra.jun.goodies.movie.JunScreenRecorder()).open();
	}

	/**
	 * Open a viewfinder with a sphere.
	 * 
	 * @category menu messages
	 */
	public void openSphere() {
		JunOpenGL3dObject aSphere = JunOpenGL3dObject.Sphere();
		aSphere.paint_(Color.blue);

		JunOpenGLDisplayModel displayModel = new JunOpenGLDisplayModel();
		displayModel.displayObject_(aSphere);
		displayModel.open();
	}

	/**
	 * Open a JunOpenGLFluxModel.
	 * 
	 * @category menu messages
	 */
	public void openTerrain() {
		(new sra.jun.terrain.editor.JunTerrainEditModel()).open();
	}

	/**
	 * Open a new viewfinder.
	 * 
	 * @category menu messages
	 */
	public void openViewfinder() {
		(new JunOpenGLDisplayModel()).open();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @category rendering
	 */
	public void renderDirect() {
		JunSystem.setDefaultRenderingMode(GLjRenderingMode.DIRECT);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @category rendering
	 */
	public void renderDoubleBuffer() {
		JunSystem.setDefaultRenderingMode(GLjRenderingMode.DOUBLE_BUFFER);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @category rendering
	 */
	public void renderImage() {
		JunSystem.setDefaultRenderingMode(GLjRenderingMode.IMAGE);
	}

	/**
	 * Open a viewfinder with a benzene.
	 * 
	 * @category rendering
	 */
	public void renderNothing() {
		JunSystem.setDefaultRenderingMode(GLjRenderingMode.NOTHING);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @category rendering
	 */
	public void renderSingleBuffer() {
		JunSystem.setDefaultRenderingMode(GLjRenderingMode.SINGLE_BUFFER);
	}

	/**
	 * The view mode is changed to 'AWT'.
	 * 
	 * @category viewing
	 */
	public void viewAwt() {
		StApplicationModel.SetDefaultViewMode(StApplicationModel.VIEW_AWT);
	}

	/**
	 * The view mode is changed to 'Swing'.
	 * 
	 * @category viewing
	 */
	public void viewSwing() {
		if (!StApplicationModel.SetDefaultViewMode(StApplicationModel.VIEW_SWING)) {
			StDialog.Warn_("Can't find a swing package, the view mode is changed to AWT.");
		}
	}

	/**
	 * The view mode is changed to 'Swing only'.
	 * 
	 * @category viewing
	 */
	public void viewSwingOnly() {
		StApplicationModel.SetDefaultViewMode(StApplicationModel.VIEW_SWING_ONLY);

		if (JunSystem.getDefaultRenderingMode() != GLjRenderingMode.IMAGE) {
			JunSystem.setDefaultRenderingMode(GLjRenderingMode.IMAGE);

			StMenu miscMenu = (StMenu) menuModel().getItem("Misc");
			StMenu aMenu = (StMenu) miscMenu.getItem("Default Rendering Mode");
			((StRadioButtonMenuItem) aMenu.getItem("Image")).setSelected(true);
			StDialog.Warn_("Default Rendering Mode is changed to Image.");
		}
	}

	/**
	 * The view mode is changed to 'Swing' with LookAndFeel.
	 * 
	 * @category viewing
	 */
	public void viewSwingWithLookAndFeel() {
		this.viewSwing();

		StMenu swingMenu = (StMenu) ((StMenu) ((StMenu) this.menuModel().getItem("Misc")).getItem("Default View Mode")).getItem("Swing");
		Vector children = swingMenu.getItems();
		for (int i = 0; i < children.size(); i++) {
			StRadioButtonMenuItem child = (StRadioButtonMenuItem) children.get(i);
			if (child.isSelected()) {
				try {
					UIManager.LookAndFeelInfo info = UIManager.getInstalledLookAndFeels()[i];
					UIManager.setLookAndFeel(info.getClassName());
				} catch (Exception e) {
				}
			}
		}
	}

	/**
	 * Answer my visual object.
	 * 
	 * @return sra.smalltalk.StDisplayable
	 * @category accessing
	 */
	public StDisplayable visual() {
		if (displayObject == null) {
			StImage logoImage = new StImage(JunSystem.LogoImage());
			int width = logoImage.width();
			int height = logoImage.height();
			height = (_DefaultWidth - 20) * height / width;
			width = (_DefaultWidth - 20);
			StImage image = new StImage(_DefaultWidth, height);
			Graphics aGraphics = null;
			try {
				aGraphics = image.image().getGraphics();
				aGraphics.drawImage(logoImage.image(), 10, 0, width, height, null);
			} finally {
				if (aGraphics != null) {
					aGraphics.dispose();
					aGraphics = null;
				}
			}
			displayObject = image;
		}
		return displayObject;
	}

	/**
	 * Invoked when a window is in the process of being closed.
	 * 
	 * @param e java.awt.event.WindowEvent
	 * @category events
	 */
	public void noticeOfWindowClose(WindowEvent e) {
		if (_closeToExit) {
			if (JunDialog.Confirm_("Really quit?", false)) {
				System.exit(0);
			}
		} else {
			super.noticeOfWindowClose(e);
		}
	}

	/**
	 * Create a menu for Test.
	 * 
	 * @return sra.smalltalk.StMenu
	 * @category resources
	 */
	protected StMenu _allTestMenu() {
		Hashtable testPackageMenus = new Hashtable();
		StMenu testMenu = new StMenu("Test");
		Class[] classes = TestExamplesClasses();

		for (int i = 0; i < classes.length; i++) {
			String className = classes[i].getName();
			String packageName = className.substring(0, className.lastIndexOf('.'));
			StMenu packageMenu = (StMenu) testPackageMenus.get(packageName);

			if (packageMenu == null) {
				packageMenu = new StMenu(packageName);
				testMenu.add(packageMenu);
				testPackageMenus.put(packageName, packageMenu);
			}

			packageMenu.add(this._testMenuFor(classes[i]));
		}

		return testMenu;
	}

	/**
	 * Create a submenu for the specified class.
	 * 
	 * @param aClass java.lang.Class
	 * 
	 * @return sra.smalltalk.StMenu
	 * @category resources
	 */
	protected StMenu _testMenuFor(Class aClass) {
		String[] methodNames = JunTestExamples.GetStaticMethodNamesFor(aClass);
		String className = aClass.getName();
		StMenu aMenu = new StMenu(className.substring(className.lastIndexOf('.') + 1));
		StMenuItem anItem = new StMenuItem("Spawn");
		anItem.setActionCommand(className + "#Spawn");
		anItem.addActionListener(this);
		aMenu.add(anItem);
		aMenu.addSeparator();

		for (int i = 0; i < methodNames.length; i++) {
			String methodName = (String) methodNames[i];
			anItem = new StMenuItem(methodName);
			anItem.setActionCommand(className + "#" + methodName);
			anItem.addActionListener(this);
			aMenu.add(anItem);
		}

		return aMenu;
	}

	/**
	 * Create a Frame for the application model.
	 * Do everything but open the frame.
	 *
	 * @return java.awt.Frame
	 * @param aView sra.smalltalk.StView
	 * @category interface opening
	 */
	protected Frame allButOpenView_(StView aView) {
		Frame aFrame = super.allButOpenView_(aView);
		aFrame.setSize(203, 167);
		return aFrame;
	}

	/**
	 * Create a default menu bar of the JunLauncher.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 * @category resources
	 */
	protected StMenuModel defaultMenu() {
		StMenuModel rootMenu = new StMenuModel();

		// Tools
		StMenu toolsMenu = new StMenu("Tools");
		toolsMenu.add(this.createMenuItem("Viewfinder", "openViewfinder"));
		toolsMenu.add(this.createMenuItem("Flux", "openFlux"));
		toolsMenu.add(this.createMenuItem("Grapher", "openGrapher"));
		toolsMenu.add(this.createMenuItem("Plotter", "openPlotter"));
		toolsMenu.add(this.createMenuItem("Rotational Body", "openRotaionalBody"));
		toolsMenu.add(this.createMenuItem("Terrain", "openTerrain"));
		toolsMenu.addSeparator();
		toolsMenu.add(this.createMenuItem("Movie or Sound", "openMovieOrSound"));
		toolsMenu.add(this.createMenuItem("Multi-Media Viewer", "openMultiMediaViewer"));
		toolsMenu.add(this.createMenuItem("Screen Recorder", "openScreenRecorder"));
		toolsMenu.addSeparator();
		toolsMenu.add(this.createMenuItem("Lisp", "openLispInterpreter"));
		toolsMenu.add(this.createMenuItem("Prolog", "openPrologInterpreter"));
		rootMenu.add(toolsMenu);

		// Examples
		StMenu examplesMenu = new StMenu("Examples");
		examplesMenu.add(this.createMenuItem("Cube", "openCube"));
		examplesMenu.add(this.createMenuItem("Sphere", "openSphere"));
		examplesMenu.add(this.createMenuItem("Benzene", "openBenzene"));
		examplesMenu.add(this.createMenuItem("Class Tree", "openClassTree")); // Grapher
		examplesMenu.add(this.createMenuItem("Planet", "openPlanet"));

		StMenu chartMenu = new StMenu("Charts...");
		chartMenu.add(this.createMenuItem("Bar", "openBarChart"));
		chartMenu.add(this.createMenuItem("Band", "openBandChart"));
		chartMenu.add(this.createMenuItem("Line", "openLineChart"));
		chartMenu.add(this.createMenuItem("Plot", "openPlotChart"));
		chartMenu.add(this.createMenuItem("Pie", "openPieChart"));
		chartMenu.add(this.createMenuItem("Radar", "openRadarChart"));
		chartMenu.add(this.createMenuItem("Combination", "openCombinationChart"));
		examplesMenu.add(chartMenu);
		examplesMenu.add(this.createMenuItem("Glass", "openGlass"));
		examplesMenu.add(this.createMenuItem("Chair", "openChair"));
		examplesMenu.add(this.createMenuItem("Robot Arm", "openRobotArm"));
		examplesMenu.add(this.createMenuItem("Border Generator", "openBorderGenerator"));
		examplesMenu.add(this.createMenuItem("Room", "openRoom"));
		examplesMenu.addSeparator();
		examplesMenu.add(this._allTestMenu());
		rootMenu.add(examplesMenu);

		// Misc
		rootMenu.add(this.defaultMiscMenu());

		return rootMenu;
	}

	/**
	 * Create a default menu bar of the JunLauncher.
	 * 
	 * @return sra.smalltalk.StMenu
	 * @category resources
	 */
	protected StMenu defaultMiscMenu() {
		StMenu miscMenu = new StMenu("Misc");
		miscMenu.add(this.createMenuItem("About", "openAbout"));
		miscMenu.add(this.createMenuItem("Copyright", "openCopyright"));
		miscMenu.add(this.createMenuItem("License", "openLicense"));
		miscMenu.add(this.createMenuItem("Information", "openInformation"));
		miscMenu.addSeparator();

		// View Mode
		StMenu viewMenu = new StMenu("Default View Mode");
		StButtonGroup viewGroup = new StButtonGroup();

		// View Mode - Awt
		StRadioButtonMenuItem awt = this.createRadioButtonMenuItem("Awt", "viewAwt", viewGroup);
		if (StApplicationModel.GetDefaultViewMode() == StApplicationModel.VIEW_AWT) {
			awt.setSelected(true);
		}

		// View Mode- Swing Look And Feel
		StMenu swing = new StMenu("Swing");
		UIManager.LookAndFeelInfo[] infos = UIManager.getInstalledLookAndFeels();
		LookAndFeel lookAndFeel = UIManager.getLookAndFeel();
		for (int i = 0; i < infos.length; i++) {
			StRadioButtonMenuItem lookAndFeelItem = this.createRadioButtonMenuItem(infos[i].getName(), "viewSwingWithLookAndFeel", viewGroup);
			swing.add(lookAndFeelItem);
			if (StApplicationModel.GetDefaultViewMode() == StApplicationModel.VIEW_SWING && lookAndFeelItem.getLabel().toUpperCase().equals(lookAndFeel.getName().toUpperCase())) {
				lookAndFeelItem.setSelected(true);
			}
		}

		// View Mode - SwingOnly
		StRadioButtonMenuItem swingOnly = this.createRadioButtonMenuItem("Swing only", "viewSwingOnly", viewGroup);
		if (StApplicationModel.GetDefaultViewMode() == StApplicationModel.VIEW_SWING_ONLY) {
			swingOnly.setSelected(true);
		}

		viewMenu.add(awt);
		viewMenu.add(swing);
		viewMenu.add(swingOnly);
		miscMenu.add(viewMenu);

		// Rendering Mode
		StMenu renderingMenu = new StMenu("Default Rendering Mode");
		StButtonGroup renderingGroup = new StButtonGroup();
		renderingMenu.add(this.createRadioButtonMenuItem("Nothing", "renderNothing", renderingGroup));
		renderingMenu.add(this.createRadioButtonMenuItem("Single Buffer", "renderSingleBuffer", renderingGroup));
		renderingMenu.add(this.createRadioButtonMenuItem("Image", "renderImage", renderingGroup));
		renderingMenu.add(this.createRadioButtonMenuItem("Direct", "renderDirect", renderingGroup));
		renderingMenu.add(this.createRadioButtonMenuItem("Double Buffer", "renderDoubleBuffer", renderingGroup));
		for (int i = GLjRenderingMode.NOTHING; i <= GLjRenderingMode.DOUBLE_BUFFER; i++) {
			if (JunSystem.getDefaultRenderingMode() == i) {
				StRadioButtonMenuItem menuItem = (StRadioButtonMenuItem) renderingMenu.getItem(i);
				menuItem.setSelected(true);
			}
		}
		miscMenu.add(renderingMenu);

		return miscMenu;
	}

	/**
	 * Initialize the JunLauncher when created.
	 * 
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		_scrollable = false;
	}

	/**
	 * Create a popup menu.
	 * 
	 * @return sra.smalltalk.StMenu
	 * @category menu accessing
	 */
	protected StMenu popupMenu() {
		if (_popupMenu == null) {
			_popupMenu = this.defaultMenu();
		}
		return _popupMenu;
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @category viewing
	 */
	protected String windowTitle() {
		return "Jun Launcher";
	}
}
