package sra.jun.system.support;

import java.awt.*;
import java.io.*;
import java.util.*;
import java.text.SimpleDateFormat;
import sra.gl4jun.*;
import sra.qt4jun.JunQTInterface;
import sra.smalltalk.*;

/**
 * JunSystem class
 * 
 *  @author    nisinaka
 *  @created   1998/11/16 (by nisinaka)
 *  @updated   2002/11/21 (by nisinaka)
 *  @version   406b (with StPL6.0) based on Jun457 for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunSystem.java,v 6.0 2003/10/28 10:52:22 nisinaka Exp $
 */
public class JunSystem extends StModel {

	public final static String _System = "Jun";
	public final static String _Version = "406b";
	public final static String _Date = "2003/10/28";

	protected static int DefaultRenderingMode = GLjRenderingMode.DOUBLE_BUFFER;

	/**
	 * Answer the string of the copyright.
	 * 
	 * @return java.lang.String
	 */
	public static final String Copyright() {
		StringWriter sw = new StringWriter();
		PrintWriter pw = null;

		try {
			pw = new PrintWriter(sw);
			pw.println("\"Jun for Java\" package is being published as open source software under the terms of \"The modified BSD license\".");
			pw.println();
			pw.println(CopyrightSRA());
			pw.println(CopyrightIPA());
			pw.println(CopyrightKTL());
			pw.println();
			pw.println("All rights reserved.");
			pw.println();
			pw.println(JavaTM());
		} finally {
			if (pw != null) {
				pw.flush();
				pw.close();
			}
		}

		return sw.toString();
	}

	/**
	 * Answer the string of the IPA copyright.
	 * 
	 * @return java.lang.String
	 */
	public static final String CopyrightIPA() {
		int thisYear = Calendar.getInstance().get(Calendar.YEAR);

		return "Copyright (C) 1999-" + thisYear + " Information-technology Promotion Agency, Japan (IPA)";
	}

	/**
	 * Answer the string of the KTL copyright.
	 * 
	 * @return java.lang.String
	 */
	public static final String CopyrightKTL() {
		int thisYear = Calendar.getInstance().get(Calendar.YEAR);

		return "Copyright (C) 2001-" + thisYear + " SRA/KTL (SRA Key Technology Laboratory, Inc.)";
	}

	/**
	 * Answer the string of the SRA copyright.
	 * 
	 * @return java.lang.String
	 */
	public static final String CopyrightSRA() {
		int thisYear = Calendar.getInstance().get(Calendar.YEAR);

		return "Copyright (C) 1999-" + thisYear + " SRA (Software Research Associates, Inc.)";
	}

	/**
	 * Answer the current Date of the system.
	 *
	 * @return java.lang.String
	 */
	public final static String Date() {
		return _Date;
	}

	/**
	 * Answer the string of default base name.
	 *
	 * @return java.lang.String
	 */
	public static String DefaultBaseName() {
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
		return dateFormat.format(new Date());
	}

	/**
	 * Answer the default extension for an image file.
	 *
	 * @return java.lang.String
	 */
	public static String DefaultImageExtension() {
		return "jpg";
	}

	/**
	 * Answer the default extension patterns for image files.
	 *
	 * @return java.lang.String[]
	 */
	public static String[] DefaultImageExtensionPatterns() {
		String[] extensions = DefaultImageExtensions();
		int size = extensions.length;
		String[] patterns = new String[size * 2];
		for (int i = 0; i < size; i++) {
			patterns[i] = "." + extensions[i].toLowerCase();
			patterns[i + size] = "." + extensions[i].toUpperCase();
		}
		return patterns;
	}

	/**
	 * Answer the default extensions for image files.
	 *
	 * @return java.lang.String[]
	 */
	public static String[] DefaultImageExtensions() {
		return new String[] { "jpg", "jpeg", "png", "bmp", "gif" };
	}

	/**
	 * Answer the default extension for a movie file.
	 *
	 * @return java.lang.String
	 */
	public static String DefaultMovieExtension() {
		return "mov";
	}

	/**
	 * Answer the default extension patterns for movie files.
	 *
	 * @return java.lang.String[]
	 */
	public static String[] DefaultMovieExtensionPatterns() {
		String[] extensions = DefaultMovieExtensions();
		int size = extensions.length;
		String[] patterns = new String[size * 2];
		for (int i = 0; i < size; i++) {
			patterns[i] = "." + extensions[i].toLowerCase();
			patterns[i + size] = "." + extensions[i].toUpperCase();
		}
		return patterns;
	}

	/**
	 * Answer the default extensions for movie files.
	 *
	 * @return java.lang.String[]
	 */
	public static String[] DefaultMovieExtensions() {
		return new String[] { "mov", "avi", "mpg", "mpeg" };
	}

	/**
	 * Answer the default extension for a sound file.
	 *
	 * @return java.lang.String
	 */
	public static String DefaultSoundExtension() {
		return "mov";
	}

	/**
	 * Answer the default extension patterns for sound files.
	 *
	 * @return java.lang.String[]
	 */
	public static String[] DefaultSoundExtensionPatterns() {
		String[] extensions = DefaultSoundExtensions();
		int size = extensions.length;
		String[] patterns = new String[size * 2];
		for (int i = 0; i < size; i++) {
			patterns[i] = "." + extensions[i].toLowerCase();
			patterns[i + size] = "." + extensions[i].toUpperCase();
		}
		return patterns;
	}

	/**
	 * Answer the default extensions for sound files.
	 *
	 * @return java.lang.String[]
	 */
	public static String[] DefaultSoundExtensions() {
		return new String[] { "mp3", "wav" };
	}

	/**
	 * Answer the default extension for a text file.
	 *
	 * @return java.lang.String
	 */
	public static String DefaultTextExtension() {
		return "txt";
	}

	/**
	 * Answer the default extension patterns for text files.
	 *
	 * @return java.lang.String[]
	 */
	public static String[] DefaultTextExtensionPatterns() {
		String[] extensions = DefaultTextExtensions();
		int size = extensions.length;
		String[] patterns = new String[size * 2];
		for (int i = 0; i < size; i++) {
			patterns[i] = "." + extensions[i].toLowerCase();
			patterns[i + size] = "." + extensions[i].toUpperCase();
		}
		return patterns;
	}

	/**
	 * Answer the default extensions for text files.
	 *
	 * @return java.lang.String[]
	 */
	public static String[] DefaultTextExtensions() {
		return new String[] { "txt", "java", "html", "htm", "bat" };
	}

	/**
	 * Answer the fullname of the current JunSystem.
	 *
	 * @return java.lang.String
	 */
	public static String FullName() {
		return System() + " for Java (" + JunSystem.Version() + " with " + StObject.System() + StObject.Version() + ")";
	}

	/**
	 * return default rendering mode;
	 * 
	 * @return int
	 */
	public static final int getDefaultRenderingMode() {
		return DefaultRenderingMode;
	}

	/**
	 * return Jun jni version string
	 * 
	 * @return string
	 */
	public static final String getJniVersion() {
		return GLjInterface.getVersion();
	}

	/**
	 * Answer the system information.
	 *
	 * @return java.lang.String
	 */
	public static final String Information() {
		String indent = "    ";
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);

		try {
			pw.println("Versions");
			pw.println(indent + JunSystem.System() + " : " + JunSystem.Version() + " (" + JunSystem.Date() + ")");
			pw.println(indent + StObject.System() + " : " + StObject.Version() + " (" + StObject.Date() + ")");
			pw.println();

			pw.println("JNI library versions");
			try {
				GLjInterface.Current();
				pw.println(indent + GLjInterface.libraryName + " : " + JunSystem.getJniVersion());
			} catch (Throwable e) {
				pw.println(indent + GLjInterface.libraryName + " : can not find");
			}
			try {
				JunQTInterface.Interface();
				pw.println(indent + JunQTInterface.libraryName + " : " + JunQTInterface.GetVersion());
			} catch (Throwable e) {
				pw.println(indent + JunQTInterface.libraryName + " : can not find");
			}
			try {
				SystemInterface.Current();
				pw.println(indent + SystemInterface.libraryName + " : " + SystemInterface.getVersion());
			} catch (Throwable e) {
				pw.println(indent + SystemInterface.libraryName + " : can not find");
			}
			pw.println();

			pw.println("Java Properties");
			TreeMap map = new TreeMap();
			map.putAll(System.getProperties());
			String lineSeparator = (String) map.get("line.separator");
			if (lineSeparator != null && lineSeparator.length() > 0) {
				String newLineSeparator = "";
				for (int i = 0; i < lineSeparator.length(); i++) {
					char c = lineSeparator.charAt(i);
					switch (c) {
						case '\n' :
							newLineSeparator += "\\n";
							break;
						case '\r' :
							newLineSeparator += "\\r";
							break;
						case '\f' :
							newLineSeparator += "\\f";
							break;
						default :
							newLineSeparator += String.valueOf(c);
					}
				}
				map.put("line.separator", newLineSeparator);
			}
			Object[] keys = map.keySet().toArray();
			for (int i = 0; i < keys.length; i++) {
				pw.println(indent + keys[i].toString() + " : " + map.get(keys[i]).toString());
			}
		} finally {
			pw.flush();
			pw.close();
		}

		return sw.toString();
	}

	/**
	 * Answer the string of the Java trademark legend.
	 * 
	 * @return java.lang.String
	 */
	public static final String JavaTM() {
		return "Java and all Java-based trademarks and logos are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. or other countries, and are used under license. ";
	}

	/**
	 * Answer the string of the license term.
	 * 
	 * @return java.lang.String
	 */
	public static final String License() {
		StringWriter sw = new StringWriter();
		PrintWriter pw = null;

		try {
			pw = new PrintWriter(sw);
			pw.println(CopyrightSRA());
			pw.println(CopyrightIPA());
			pw.println(CopyrightKTL());
			pw.println("All rights reserved.");
			pw.println();
			pw.println("Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:");
			pw.println();
			pw.println("1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. ");
			pw.println();
			pw.println("2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. ");
			pw.println();
			pw.println("3. The name of the author may not be used to endorse or promote products derived from this software without specific prior written permission. ");
			pw.println();
			pw.println(
				"THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.");
			pw.println();
			pw.println(JavaTM());
		} finally {
			if (pw != null) {
				pw.flush();
				pw.close();
			}
		}

		return sw.toString();
	}

	/**
	 * Create a Jun logo image.
	 * 
	 * @return java.awt.Image
	 */
	public static final Image LogoImage() {
		return SystemResourceSupport.createImage("/sra/jun/system/support/LogoImage.jpg");
	}

	/**
	 * return default rendering mode;
	 * 
	 * @param mode DOCUMENT ME!
	 */
	public static final void setDefaultRenderingMode(int mode) {
		DefaultRenderingMode = mode;
	}

	/**
	 * Answer the current Name of the system.
	 *
	 * @return java.lang.String
	 */
	public static final String System() {
		return _System;
	}

	/**
	 * Answer the current Version of the system.
	 *
	 * @return java.lang.String
	 */
	public static final String Version() {
		return _Version;
	}
}
