package sra.jun.system.support;

import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.util.*;
import javax.swing.*;
import sra.smalltalk.*;
import sra.gl4jun.GLjRenderingMode;
import sra.jun.goodies.cursors.JunCursors;
import sra.jun.goodies.display.*;
import sra.jun.system.framework.JunDialog;

/**
 * JunTestExamples class
 * 
 *  @author    nisinaka
 *  @created   1998/09/28 (by nisinaka)
 *  @updated   1999/06/25 (by MATSUDA Ryouichi)
 *  @updated   2000/01/14 (by MATSUDA Ryouichi)
 *  @updated   2001/04/10 (by nisinaka)
 *  @updated   2002/11/21 (by nisinaka)
 *  @updated   2003/03/03 (by Mitsuhiro Asada)
 *  @updated   2003/03/19 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTestExamples.java,v 6.0 2003/10/28 10:52:22 nisinaka Exp $
 */
public abstract class JunTestExamples extends JunDisplayModel {

	protected StMenuModel menuModel;
	protected StCheckBoxMenuItem elapsing;
	protected boolean closeToExit = true;

	/**
	 * Creates a new JunTestExamples object.
	 */
	public JunTestExamples() {
		super();
		this.open();
	}

	/**
	 * Execute all public methods.
	 * 
	 * @param aClass DOCUMENT ME!
	 * 
	 * @throws SmalltalkException DOCUMENT ME!
	 */
	public static final void ExecuteMethods(Class aClass) {
		try {
			String[] methodNames = GetStaticMethodNamesFor(aClass);

			for (int i = 0; i < methodNames.length; i++) {
				System.out.println(methodNames[i]);

				Boolean result = (Boolean) StObject._Perform(aClass, methodNames[i]);
				System.out.println("-> " + result);
			}
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Answer method names.
	 * 
	 * @param aClass java.lang.Class
	 * 
	 * @return java.lang.String[]
	 */
	public static final String[] GetStaticMethodNamesFor(Class aClass) {
		StBlockClosure sortBlock = new StBlockClosure() {
			public Object value_value_(Object e1, Object e2) {
				return new Boolean(e1.toString().compareTo(e2.toString()) < 0);
			}
		};

		StSortedCollection collection = new StSortedCollection(sortBlock);
		Method[] methods = aClass.getDeclaredMethods();

		for (int i = 0; i < methods.length; i++) {
			Method aMethod = methods[i];

			if (aMethod.getParameterTypes().length == 0) {
				int modifiers = aMethod.getModifiers();

				if (Modifier.isStatic(modifiers) && Modifier.isPublic(modifiers)) {
					collection.add_(aMethod.getName());
				}
			}
		}

		Object[] array = collection._asArray();
		String[] methodNames = new String[array.length];

		for (int i = 0; i < array.length; i++) {
			methodNames[i] = (String) array[i];
		}

		return methodNames;
	}

	/**
	 * Invoked when a menu item has been selected.
	 * 
	 * @param ev java.awt.event.ActionEvent
	 */
	public void actionPerformed(ActionEvent ev) {
		String command = ev.getActionCommand();
		int index = command.indexOf("#");
		if (index < 0) {
			this.executeMethod(this, command);
			return;
		}

		String className = command.substring(0, index);
		String methodName = command.substring(index + 1, command.length());
		try {
			Class aClass = Class.forName(className);
			if (methodName.equals("All")) {
				String[] methodNames = GetStaticMethodNamesFor(aClass);
				for (int i = 0; i < methodNames.length; i++) {
					this.executeMethodAndShowResult(aClass, methodNames[i]);
				}
			} else {
				this.executeMethodAndShowResult(aClass, methodName);
			}
		} catch (ClassNotFoundException e) {
			new SmalltalkException(e);
		}
	}

	/**
	 * Set the status whether be elapsing or not.
	 * 
	 * @param aBoolean boolean
	 */
	public void beElapsing(boolean aBoolean) {
		if (elapsing != null) {
			elapsing.setSelected(aBoolean);
		}
	}

	/**
	 * Set whether exit or not when closing a window.
	 *
	 * @param aBoolean boolean
	 */
	public void closeToExit(boolean aBoolean) {
		closeToExit = aBoolean;
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		return new StViewPanel(this) {
			protected void buildComponent() {
				this.setLayout(new BorderLayout());
				this.add(new JunDisplayViewAwt((JunTestExamples) this.model()), BorderLayout.CENTER);
			}
		};
	}

	/**
	 * Execute all methods specified with the method name.
	 * 
	 * @return java.lang.Object
	 * @param aClass java.lang.Class
	 * @param methodName java.lang.String
	 */
	public Object executeMethod(Class aClass, String methodName) {
		JunCursors cursor = new JunCursors(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		try {
			cursor._show();
			return StObject._Perform(aClass, methodName);
		} catch (Exception e) {
			throw new SmalltalkException(e);
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Execute all methods specified with the method name.
	 * 
	 * @return java.lang.Object
	 * @param anObject sra.smalltalk.StObject
	 * @param methodName java.lang.String
	 */
	public Object executeMethod(StObject anObject, String methodName) {
		JunCursors cursor = new JunCursors(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		try {
			cursor._show();
			return anObject.perform_(methodName);
		} catch (Exception e) {
			throw new SmalltalkException(e);
		} finally {
			cursor._restore();
		}
	}

	/**
	 * Execute all methods specified with the method name.
	 * 
	 * @param aClass java.lang.Class
	 * @param methodName java.lang.String
	 */
	public void executeMethodAndShowResult(Class aClass, String methodName) {
		System.out.println(methodName);

		long start = System.currentTimeMillis();
		Boolean result = (Boolean) this.executeMethod(aClass, methodName);
		long end = System.currentTimeMillis();
		System.out.println("-> " + result);

		if (elapsing.isSelected()) {
			System.out.println("-> elapsed " + (end - start) + " msec");
		}
	}

	/**
	 * Answer a menu model.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 */
	public StMenuModel menuModel() {
		if (menuModel == null) {
			menuModel = defaultMenu();
		}

		return menuModel;
	}

	/**
	 * DOCUMENT ME!
	 */
	public void renderDirect() {
		JunSystem.setDefaultRenderingMode(GLjRenderingMode.DIRECT);
	}

	/**
	 * DOCUMENT ME!
	 */
	public void renderDoubleBuffer() {
		JunSystem.setDefaultRenderingMode(GLjRenderingMode.DOUBLE_BUFFER);
	}

	/**
	 * DOCUMENT ME!
	 */
	public void renderImage() {
		JunSystem.setDefaultRenderingMode(GLjRenderingMode.IMAGE);
	}

	/**
	 * Open a viewfinder with a benzene.
	 */
	public void renderNothing() {
		JunSystem.setDefaultRenderingMode(GLjRenderingMode.NOTHING);
	}

	/**
	 * DOCUMENT ME!
	 */
	public void renderSingleBuffer() {
		JunSystem.setDefaultRenderingMode(GLjRenderingMode.SINGLE_BUFFER);
	}

	/**
	 * The view mode is changed to 'AWT'.
	 */
	public void viewAwt() {
		StApplicationModel.SetDefaultViewMode(StApplicationModel.VIEW_AWT);
	}

	/**
	 * The view mode is changed to 'Swing'.
	 */
	public void viewSwing() {
		StApplicationModel.SetDefaultViewMode(StApplicationModel.VIEW_SWING);
	}

	/**
	 * The view mode is changed to 'Swing only'.
	 */
	public void viewSwingOnly() {
		StApplicationModel.SetDefaultViewMode(StApplicationModel.VIEW_SWING_ONLY);

		if (JunSystem.getDefaultRenderingMode() != GLjRenderingMode.IMAGE) {
			JunSystem.setDefaultRenderingMode(GLjRenderingMode.IMAGE);

			StMenu helpMenu = (StMenu) menuModel().getItem("Help");
			StMenu aMenu = (StMenu) helpMenu.getItem("Default Rendering Mode");
			((StRadioButtonMenuItem) aMenu.getItem("Image")).setSelected(true);
			StDialog.Warn_("Default Rendering Mode is changed to Image.");
		}
	}

	/**
	 * The view mode is changed to 'Swing' with LookAndFeel.
	 */
	public void viewSwingWithLookAndFeel() {
		this.viewSwing();

		StMenu swingMenu = (StMenu) ((StMenu) ((StMenu) this.menuModel().getItem("Help")).getItem("Default View Mode")).getItem("Swing");
		Vector children = swingMenu.getItems();
		for (int i = 0; i < children.size(); i++) {
			StRadioButtonMenuItem child = (StRadioButtonMenuItem) children.get(i);
			if (child.isSelected()) {
				try {
					UIManager.LookAndFeelInfo info = UIManager.getInstalledLookAndFeels()[i];
					UIManager.setLookAndFeel(info.getClassName());
				} catch (Exception e) {
				}
			}
		}
	}

	/**
	 * Answer my visual object.
	 * 
	 * @return sra.smalltalk.StDisplayable
	 */
	public StDisplayable visual() {
		if (displayObject == null) {
			StImage logoImage = new StImage(JunSystem.LogoImage());
			int width = logoImage.width();
			int height = logoImage.height();
			height = (JunLauncher._DefaultWidth - 20) * height / width;
			width = (JunLauncher._DefaultWidth - 20);
			StImage image = new StImage(JunLauncher._DefaultWidth, height);
			Graphics aGraphics = null;
			try {
				aGraphics = image.image().getGraphics();
				aGraphics.drawImage(logoImage.image(), 10, 0, width, height, null);
			} finally {
				if (aGraphics != null) {
					aGraphics.dispose();
					aGraphics = null;
				}
			}
			displayObject = image;
		}
		return displayObject;
	}

	/**
	 * Invoked when a window is in the process of being closed.
	 * 
	 * @param e java.awt.event.WindowEvent
	 */
	public void noticeOfWindowClose(WindowEvent e) {
		if (closeToExit) {
			if (JunDialog.Confirm_("Really quit?", false)) {
				System.exit(0);
			} else {
				e.getWindow().setVisible(true);
			}
		} else {
			super.noticeOfWindowClose(e);
		}
	}

	/**
	 * Create a default menu bar of the JunLauncher.
	 * 
	 * @return java.awt.MenuBar
	 */
	protected StMenuModel defaultMenu() {
		// Examples Menu
		String[] methodNames = GetStaticMethodNamesFor(this.getClass());
		String className = this.getClass().getName();
		StMenuModel menuModel = new StMenuModel();
		StMenu aMenu = new StMenu("Examples");
		menuModel.add(aMenu);

		StMenuItem anItem = new StMenuItem("All");
		anItem.setActionCommand(className + "#All");
		anItem.addActionListener(this);
		aMenu.add(anItem);
		aMenu.addSeparator();

		for (int i = 0; i < methodNames.length; i++) {
			String methodName = (String) methodNames[i];
			anItem = new StMenuItem(methodName);
			anItem.setActionCommand(className + "#" + methodName);
			anItem.addActionListener(this);
			aMenu.add(anItem);
		}

		// Help Menu
		StMenu helpMenu = new StMenu("Help");
		elapsing = new StCheckBoxMenuItem("Measure elapsing", false);
		helpMenu.add(elapsing);

		// Swing Look And Feel
		StButtonGroup viewGroup = new StButtonGroup();
		StMenu swingMenu = new StMenu("Swing");
		UIManager.LookAndFeelInfo[] infos = UIManager.getInstalledLookAndFeels();
		for (int i = 0; i < infos.length; i++) {
			swingMenu.add(this.createRadioButtonMenuItem(infos[i].getName(), "viewSwingWithLookAndFeel", viewGroup));
		}

		// View Mode
		StMenu viewMenu = new StMenu("Default View Mode");
		viewMenu.add(this.createRadioButtonMenuItem("Awt", "viewAwt", viewGroup));
		viewMenu.add(swingMenu);
		viewMenu.add(this.createRadioButtonMenuItem("Swing only", "viewSwingOnly", viewGroup));
		((StRadioButtonMenuItem) viewMenu.getItem("Awt")).setSelected(true);

		for (int i = StApplicationModel.VIEW_AWT; i <= StApplicationModel.VIEW_SWING_ONLY; i++) {
			if (StApplicationModel.GetDefaultViewMode() == i) {
				if (i == StApplicationModel.VIEW_SWING) {
					StRadioButtonMenuItem menuItem = (StRadioButtonMenuItem) ((StMenu) viewMenu.getItem(i)).getItem("Metal");
					menuItem.setSelected(true);
				} else {
					StRadioButtonMenuItem menuItem = (StRadioButtonMenuItem) viewMenu.getItem(i);
					menuItem.setSelected(true);
				}
			}
		}

		helpMenu.add(viewMenu);

		// Rendering Mode
		StMenu renderingMenu = new StMenu("Default Rendering Mode");
		StButtonGroup renderingGroup = new StButtonGroup();
		renderingMenu.add(createRadioButtonMenuItem("Nothing", "renderNothing", renderingGroup));
		renderingMenu.add(createRadioButtonMenuItem("Single Buffer", "renderSingleBuffer", renderingGroup));
		renderingMenu.add(createRadioButtonMenuItem("Image", "renderImage", renderingGroup));
		renderingMenu.add(createRadioButtonMenuItem("Direct", "renderDirect", renderingGroup));
		renderingMenu.add(createRadioButtonMenuItem("Double Buffer", "renderDoubleBuffer", renderingGroup));

		for (int i = GLjRenderingMode.NOTHING; i <= GLjRenderingMode.DOUBLE_BUFFER; i++) {
			if (JunSystem.getDefaultRenderingMode() == i) {
				StRadioButtonMenuItem menuItem = (StRadioButtonMenuItem) renderingMenu.getItem(i);
				menuItem.setSelected(true);
			}
		}

		helpMenu.add(renderingMenu);
		menuModel.add(helpMenu);

		return menuModel;
	}

	/**
	 * Initialize the JunTestExamples when created.
	 */
	protected void initialize() {
		super.initialize();

		_scrollable = false;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		String className = this.getClass().getName();

		return className.substring(className.lastIndexOf('.') + 1);
	}
}
