package sra.jun.terrain.editor;

import java.awt.*;
import java.awt.event.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.curves.Jun2dLine;
import sra.jun.goodies.cursors.JunCursors;
import sra.smalltalk.*;

/**
 * JunTerrainEditController class
 * 
 *  @author    nisinaka
 *  @created   2001/11/19 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTerrainEditController.java,v 6.0 2003/10/28 10:52:23 nisinaka Exp $
 */
public class JunTerrainEditController extends StController implements MouseListener, MouseMotionListener {
	protected Jun2dPoint _currentPoint;
	protected Jun2dPoint _previousPoint;
	protected Cursor _savedCursor;
	protected Point _startPoint;
	protected Point _mousePointOnScreen;
	protected Point[] _lineSegment;

	/**
	 * Invoked when a mouse button is dragged on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 */
	public void mouseDragged(MouseEvent e) {
		JunTerrainEditModel theModel = (JunTerrainEditModel) this.model();
		StSymbol state = theModel.buttonState();

		if (state == $("drag")) {
			_currentPoint = this.regularizePoint_(e.getPoint());

			if (_currentPoint.equals(_previousPoint) == false) {
				theModel.drag_xy_(_previousPoint, _currentPoint);
				_previousPoint = _currentPoint;
			}
		} else if (state == $("show")) {
			if (_startPoint != null) {
				this._drawLineSegment();
				_lineSegment = new Point[] { _startPoint, e.getPoint()};
				this._drawLineSegment();
			}
		}
	}

	/**
	 * Invoked when a mouse is entered on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 */
	public void mouseEntered(MouseEvent e) {
		JunTerrainEditModel theModel = (JunTerrainEditModel) this.model();
		StSymbol state = theModel.buttonState();

		if (state == null) {
			return;
		}

		Component mainView = this._mainView();
		_savedCursor = mainView.getCursor();

		if (state == $("peak")) {
			mainView.setCursor(JunCursors.CrossCursor());
		} else if (state == $("drag")) {
			mainView.setCursor(JunCursors.QuartersCursor());
		} else if (state == $("show")) {
			mainView.setCursor(JunCursors.ReadCursor());
		}
	}

	/**
	 * Invoked when a mouse is exited on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 */
	public void mouseExited(MouseEvent e) {
		this._mainView().setCursor(_savedCursor);
	}

	/**
	 * Invoked when a mouse button is pressed on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 */
	public void mousePressed(MouseEvent e) {
		_startPoint = e.getPoint();
		_mousePointOnScreen = e.getComponent().getLocationOnScreen();
		_mousePointOnScreen.translate(_startPoint.x, _startPoint.y);
		_currentPoint = this.regularizePoint_(_startPoint);
		_previousPoint = _currentPoint;
	}

	/**
	 * Invoked when a mouse button is released on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 */
	public void mouseReleased(MouseEvent e) {
		JunTerrainEditModel theModel = (JunTerrainEditModel) this.model();
		StSymbol state = theModel.buttonState();

		if (state == $("peak")) {
			theModel.plotPeak_(this);
		} else if (state == $("show")) {
			this._drawLineSegment();
			theModel.showTerrain_(this);
			_lineSegment = null;
		}

		_startPoint = null;
		_mousePointOnScreen = null;
	}

	/**
	 * Draw the line segment if exist.
	 */
	protected void _drawLineSegment() {
		if (_lineSegment == null) {
			return;
		}

		Graphics aGraphics = this._mainView().getGraphics();

		try {
			aGraphics.setColor(Color.white);
			aGraphics.setXORMode(Color.black);
			aGraphics.drawLine(_lineSegment[0].x, _lineSegment[0].y, _lineSegment[1].x, _lineSegment[1].y);
		} finally {
			aGraphics.dispose();
		}
	}

	/**
	 * Answer the main view.
	 * 
	 * @return java.awt.Component
	 */
	protected Component _mainView() {
		return (Component) ((JunTerrainEditView) this.view()).getOpenGLView();
	}

	/**
	 * Add listeners.
	 * 
	 * @param newView sra.smalltalk.StView
	 */
	protected void buildListener(StView newView) {
		Component mainView = this._mainView();
		mainView.addMouseListener(this);
		mainView.addMouseMotionListener(this);
	}

	/**
	 * Answer the specified line segment as Jun2dLine.
	 * 
	 * @return sra.jun.geometry.curves.Jun2dLine
	 */
	protected sra.jun.geometry.curves.Jun2dLine mouse2dLineSegment() {
		if (_lineSegment == null) {
			return null;
		}

		return new Jun2dLine(this.regularizePoint_(_lineSegment[0]), this.regularizePoint_(_lineSegment[1]));
	}

	/**
	 * Answer the normalized mouse point.
	 * 
	 * @return sra.jun.geometry.basic.Jun2dPoint
	 */
	protected Jun2dPoint mouse2dPoint() {
		return _currentPoint;
	}

	/**
	 * Normalize the mouse point.
	 * 
	 * @param aPoint java.awt.Point
	 * 
	 * @return sra.jun.geometry.basic.Jun2dPoint
	 */
	protected Jun2dPoint regularizePoint_(Point aPoint) {
		Rectangle bounds = ((JunTerrainEditView) this.view()).boundsOfCanvas();
		double x = (aPoint.x - bounds.width / 2.0d) / (bounds.height / 2.0d);
		double y = (bounds.height / 2.0d - aPoint.y) / (bounds.height / 2.0d);

		return new Jun2dPoint(x, y);
	}
}
