package sra.jun.terrain.editor;

import java.awt.*;
import javax.swing.JPanel;
import sra.jun.goodies.button.JunButtonViewSwing;
import sra.jun.goodies.wheels.JunThumbWheelViewSwing;
import sra.jun.opengl.display.*;
import sra.smalltalk.*;

/**
 * JunTerrainEditViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2002/04/05 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTerrainEditViewSwing.java,v 6.0 2003/10/28 10:52:23 nisinaka Exp $
 */
public class JunTerrainEditViewSwing extends StViewJPanel implements JunTerrainEditView {
	protected JunOpenGL3dViewSwing openGLView;
	protected JunButtonViewSwing peakButtonView;
	protected JunButtonViewSwing dragButtonView;
	protected JunButtonViewSwing showButtonView;
	protected JunThumbWheelViewSwing zThumbWheelView;

	/**
	 * Create a new instance of JunTerrainEditViewSwing and initialize it.
	 *
	 * @param aTerrainEditModel sra.jun.terrain.editor.JunTerrainEditModel
	 */
	public JunTerrainEditViewSwing(JunTerrainEditModel aTerrainEditModel) {
		super(aTerrainEditModel);
	}

	/**
	 * Answer the bounds of the canvas area.
	 * 
	 * @return java.awt.Rectangle
	 */
	public Rectangle boundsOfCanvas() {
		return ((JunOpenGL3dViewSwing) this.getOpenGLView()).getBounds();
	}

	/**
	 * Answer my model as a JunTerrainEditModel.
	 * 
	 * @return JunTerrainEditModel
	 */
	public JunTerrainEditModel getModel() {
		return (JunTerrainEditModel) this.model;
	}

	/**
	 * Answer the OpenGL view.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGL3dView
	 */
	public JunOpenGL3dView getOpenGLView() {
		if (openGLView == null) {
			openGLView = new JunOpenGL3dViewSwing(this.getModel());
		}
		return openGLView;
	}

	/**
	 * Receive a change notice from an object of whom the receiver is a
	 * dependent.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if ((this.isShowing() == true) && (evt.getAspect() == $("state"))) {
			return;
		}

		//
		super.update_(evt);
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(new BorderLayout());

		final Dimension buttonSize = new Dimension(18, 18);

		//
		// center
		//
		JPanel displayPanel = StApplicationModel._JPanelWithBorder();
		displayPanel.setLayout(new BorderLayout());
		displayPanel.add((JunOpenGL3dViewSwing) this.getOpenGLView(), BorderLayout.CENTER);

		//
		this.add(displayPanel, BorderLayout.CENTER);

		//
		// left
		//
		JPanel leftPanel = new JPanel();
		leftPanel.setLayout(new BorderLayout());

		//
		JPanel leftTopPanel = new JPanel();
		leftTopPanel.setLayout(new GridLayout(3, 1));

		// peak button
		peakButtonView = new JunButtonViewSwing(this.getModel().peakButton());
		peakButtonView.setPreferredSize(buttonSize);
		leftTopPanel.add(peakButtonView);

		// drag button
		dragButtonView = new JunButtonViewSwing(this.getModel().dragButton());
		dragButtonView.setPreferredSize(buttonSize);
		leftTopPanel.add(dragButtonView);

		// show button
		showButtonView = new JunButtonViewSwing(this.getModel().showButton());
		showButtonView.setPreferredSize(buttonSize);
		leftTopPanel.add(showButtonView);
		leftPanel.add(leftTopPanel, BorderLayout.NORTH);

		//
		this.add(leftPanel, BorderLayout.WEST);

		//
		// bottom
		//
		JPanel dummy = new JPanel();
		dummy.setPreferredSize(new Dimension(buttonSize.width, buttonSize.height));
		this.add(dummy, BorderLayout.SOUTH);

		//
		// right
		//
		JPanel rightPanel = new JPanel();
		rightPanel.setLayout(new BorderLayout());

		JPanel rightCenterPanel = new JPanel();
		GridBagLayout gbl = new GridBagLayout();
		rightCenterPanel.setLayout(gbl);

		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;

		//
		zThumbWheelView = new JunThumbWheelViewSwing(this.getModel().zThumbWheel());
		zThumbWheelView.setVertical(true);
		c.gridy = 1;
		gbl.setConstraints(zThumbWheelView, c);
		rightCenterPanel.add(zThumbWheelView);
		rightPanel.add(rightCenterPanel);

		//
		this.add(rightPanel, BorderLayout.EAST);
	}

	/**
	 * Answer a default controller.
	 * 
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new JunTerrainEditController();
	}

	/**
	 * Answer a default model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new JunTerrainEditModel();
	}

	/**
	 * Rebuild this component.
	 * 
	 * @param oldModel sra.smalltalk.StModel
	 * @param newModel sra.smalltalk.StModel
	 */
	protected void rebuildComponent(StModel oldModel, StModel newModel) {
		JunTerrainEditModel model = (JunTerrainEditModel) newModel;
		peakButtonView.model_(model.peakButton());
		dragButtonView.model_(model.dragButton());
		showButtonView.model_(model.showButton());
		zThumbWheelView.model_(model.zThumbWheel());
		openGLView.model_(model);
	}
}
