package sra.jun.terrain.support;

import sra.jun.geometry.basic.*;
import sra.jun.opengl.objects.*;
import sra.smalltalk.StObject;

/**
 * JunTerrainField class
 * 
 *  @author    nisinaka
 *  @created   2002/01/22 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTerrainField.java,v 6.0 2003/10/28 10:52:23 nisinaka Exp $
 */
public class JunTerrainField extends StObject {
	protected JunTerrainTriangle[] triangles;

	/**
	 * Create a new instance of JunTerrainField and initialize it with
	 * triangles.
	 * 
	 * @param anArrayOfJunTerrainTriangle
	 *        sra.jun.terrain.support.JunTerrainTriangle[]
	 */
	public JunTerrainField(JunTerrainTriangle[] anArrayOfJunTerrainTriangle) {
		this.setTriangles_(anArrayOfJunTerrainTriangle);
	}

	/**
	 * Convert to a JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		JunOpenGL3dCompoundObject anObject = new JunOpenGL3dCompoundObject();

		for (int i = 0; i < triangles.length; i++) {
			anObject.add_(triangles[i].asJunOpenGL3dObject());
		}

		return anObject;
	}

	/**
	 * Answer the normal vector at the specified point.
	 * 
	 * @param aJun2dPoint sra.jun.geometry.basic.Jun2dPoint
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint normalVectorAt2dPoint_(Jun2dPoint aJun2dPoint) {
		for (int i = 0; i < triangles.length; i++) {
			if (triangles[i].contains2dPoint_(aJun2dPoint)) {
				return triangles[i].normalVector();
			}
		}

		return null;
	}

	/**
	 * Answer the z-coordinate at the specified point.
	 * 
	 * @param aJun2dPoint sra.jun.geometry.basic.Jun2dPoint
	 * 
	 * @return double
	 */
	public double zAt2dPoint_(Jun2dPoint aJun2dPoint) {
		for (int i = 0; i < triangles.length; i++) {
			if (triangles[i].contains2dPoint_(aJun2dPoint)) {
				return triangles[i].zAt2dPoint_(aJun2dPoint);
			}
		}

		return Double.NaN;
	}

	/**
	 * Set my triangles.
	 * 
	 * @param anArrayOfJunTerrainTriangle
	 *        sra.jun.terrain.support.JunTerrainTriangle[]
	 */
	protected void setTriangles_(JunTerrainTriangle[] anArrayOfJunTerrainTriangle) {
		triangles = anArrayOfJunTerrainTriangle;
	}
}
