package sra.jun.topology.abstracts;

import sra.jun.topology.elements.JunBody;

/*
 * @(#)JunAbstractOperator.java        1.0 98/9/17
 */
import sra.smalltalk.SmalltalkException;

/**
 * JunAbstractOperator class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunAbstractOperator.java,v 6.0 2003/10/28 10:52:24 nisinaka Exp $
 */
public abstract class JunAbstractOperator extends JunTopology {
	/** A JunBody on which the receiver operate. */
	protected JunBody body = null;

	/**
	 * Default constructor.
	 */
	public JunAbstractOperator() {
		body = null;
	}

	/**
	 * Create a new instance of JunAbstractOperator and initialize it with a
	 * JunBody.
	 * 
	 * @param aBody sra.jun.topology.elements.JunBody
	 */
	public JunAbstractOperator(JunBody aBody) {
		this();
		this.body_(aBody);
	}

	/**
	 * Answer the body.
	 * 
	 * @return sra.jun.topology.elements.JunBody
	 */
	public final JunBody body() {
		return body;
	}

	/**
	 * Set a body.
	 * 
	 * @param aBody sra.jun.topology.elements.JunBody
	 */
	public final void body_(JunBody aBody) {
		body = aBody;
	}

	/**
	 * Do the operation.     - In Smalltalk, the method name is "do".
	 */
	public void doOperation() {
		preExecute();
		execute();
		postExecute();
	}

	/**
	 * Execute the operation.
	 */
	public abstract void execute();

	/**
	 * Answer the topological element of the receiver.
	 * 
	 * @param aTopologicalElementOrProxy
	 *        sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * 
	 * @return sra.jun.topology.abstracts.JunTopologicalElement
	 */
	public JunTopologicalElement getTopologicalElement_(JunTopologicalElementOrProxy aTopologicalElementOrProxy) {
		if (aTopologicalElementOrProxy == null) {
			return null;
		}

		if (aTopologicalElementOrProxy.isTopologicalProxy()) {
			return ((JunTopologicalElementProxy) aTopologicalElementOrProxy).value_(this.body);
		} else {
			return (JunTopologicalElement) aTopologicalElementOrProxy;
		}
	}

	/**
	 * Answer the topological element proxy of the receiver.
	 * 
	 * @param aTopologicalElementOrProxy
	 *        sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * 
	 * @return sra.jun.topology.abstracts.JunTopologicalElementProxy
	 */
	public JunTopologicalElementProxy getTopologicalElementProxy_(JunTopologicalElementOrProxy aTopologicalElementOrProxy) {
		return this.getTopologicalElementProxy_advise_(aTopologicalElementOrProxy, null);
	}

	/**
	 * Answer the topological element proxy of the receiver with advise.
	 * 
	 * @param aTopologicalElementOrProxy
	 *        sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * @param aTopologicalElementProxyOrNull
	 *        sra.jun.topology.abstracts.JunTopologicalElementProxy
	 * 
	 * @return sra.jun.topology.abstracts.JunTopologicalElementProxy
	 */
	public JunTopologicalElementProxy getTopologicalElementProxy_advise_(JunTopologicalElementOrProxy aTopologicalElementOrProxy, JunTopologicalElementProxy aTopologicalElementProxyOrNull) {
		if (aTopologicalElementOrProxy == null) {
			return null;
		}

		return aTopologicalElementOrProxy.asProxyIn_advise_(this.body(), aTopologicalElementProxyOrNull);
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return sra.jun.topology.abstracts.JunAbstractOperator
	 */
	public abstract JunAbstractOperator inverse();

	/**
	 * Answer true if the receiver is a kind of Euler Operator.
	 * 
	 * @return boolean
	 */
	public boolean isEulerOperator() {
		return false;
	}

	/**
	 * Answer true if the receiver is a kind of Geometric Operator.
	 * 
	 * @return boolean
	 */
	public boolean isGeometricOperator() {
		return false;
	}

	/**
	 * Answer true if the receiver is a kind of Global Operator.
	 * 
	 * @return boolean
	 */
	public boolean isGlobalOperator() {
		return false;
	}

	/**
	 * Answer true if the receiver can be inverted.
	 * 
	 * @return boolean
	 */
	public abstract boolean isInversable();

	/**
	 * Answer true if the post-condition is valid.
	 * 
	 * @return boolean
	 */
	public boolean postcondition() {
		return true;
	}

	/**
	 * Do something after the execution.
	 */
	public void postExecute() {
		if (!postcondition()) {
			throw SmalltalkException.Error("Invalid operation");
		}
	}

	/**
	 * Answer true if the pre-condition is valid.
	 * 
	 * @return boolean
	 */
	public boolean precondition() {
		return true;
	}

	/**
	 * Do something before the execution.
	 */
	public void preExecute() {
		if (!precondition()) {
			throw SmalltalkException.Error("Invalid operation");
		}
	}

	/**
	 * Undo the operation.     - In Smalltalk, the method name is "undo".
	 */
	public void undoOperation() {
		this.inverse().doOperation();
	}

	/**
	 * Answer the value of the operator.
	 * 
	 * @return sra.jun.topology.elements.JunBody
	 */
	public JunBody value() {
		this.doOperation();

		return this.body();
	}
}
