package sra.jun.topology.abstracts;

import sra.smalltalk.*;
import sra.jun.goodies.lisp.*;
import sra.jun.opengl.display.JunOpenGLDisplayModel;
import sra.jun.opengl.objects.JunOpenGL3dObject;
import sra.jun.topology.elements.*;

/**
 * JunTopologicalElement class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   1999/08/05 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunTopologicalElement.java,v 6.0 2003/10/28 10:52:24 nisinaka Exp $
 */
public abstract class JunTopologicalElement extends JunTopology implements JunTopologicalElementOrProxy {

	/**
	 * Create a new instance of JunTopologicalElement.
	 */
	public JunTopologicalElement() {
		super();
		PermitInstanceCreation();
	}

	/**
	 * Answer the default class for LispCons.
	 * 
	 * @return java.lang.Class
	 */
	public static Class DefaultLispConsClass() {
		return JunLispCons.class;
	}

	/**
	 * Answer the default class for LispNil.
	 * 
	 * @return java.lang.Class
	 */
	public static Class DefaultLispNilClass() {
		return JunLispNil.class;
	}

	/**
	 * Answer a cell of the LispCons.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public static JunLispCons LispCons() {
		return JunLispCons.Cell();
	}

	/**
	 * Answer a cell of the LispNil.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispNil
	 */
	public static JunLispNil LispNil() {
		return JunLispNil.NullList();
	}

	/**
	 * DOCUMENT ME!
	 */
	public static void PermitInstanceCreation() {
		/*    
		"| context |
		context := thisContext.ext or: [context receiver isKindOf: JunTopologicalElement class]]]
		whileTrue: [context := context sender].
		[context notNil and: [context isBlockCont
		(context isNil or: [context receiver isKindOf: JunEulerOperator])
		ifFalse: [self eulerOperatorViolationSignal raise]"! !
		*/
	}

	/**
	 * Convert the receiver as a JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		throw SmalltalkException.SubclassResponsibility();
	}

	/**
	 * Convert the receiver as a wire-framed JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject asWireframedOpenGL3dObject() {
		throw SmalltalkException.SubclassResponsibility();
	}

	/**
	 * Answer the StSymbol which represents the receiver's kind.
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public StSymbol kindName() {
		return this._className();
	}

	/**
	 * Answer a lisp cons.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 */
	public JunLispCons lispCons() {
		return LispCons();
	}

	/**
	 * Answer a lisp nil.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispNil
	 */
	public JunLispNil lispNil() {
		return LispNil();
	}

	/**
	 * Convert the receiver as a JunOpenGL3dObject and show it with
	 * JunOpenGLDisplayModel.
	 * 
	 * @return sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	public JunOpenGLDisplayModel show() {
		JunOpenGL3dObject displayObject;
		displayObject = asJunOpenGL3dObject();

		return JunOpenGLDisplayModel.Show_(displayObject);
	}

	/**
	 * Convert the receiver as a lisp list.
	 * 
	 * @param loopArray sra.jun.topology.elements.JunLoop[]
	 * @param edgeArray sra.jun.topology.elements.JunEdge[]
	 * @param vertexArray sra.jun.topology.elements.JunVertex[]
	 * 
	 * @return sra.jun.goodies.lisp.JunLispList
	 */
	public JunLispList toLispListForLoops_edges_vertexes_(JunLoop[] loopArray, JunEdge[] edgeArray, JunVertex[] vertexArray) {
		throw SmalltalkException.SubclassResponsibility();
	}
}
