package sra.jun.topology.geometricoperators;

/*
 * @(#)JunKCV.java        1.0 98/9/17
 */
import sra.jun.geometry.abstracts.JunCurve;
import sra.jun.topology.abstracts.*;
import sra.jun.topology.elements.*;

/**
 * JunKCV class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunKCV.java,v 6.0 2003/10/28 10:52:26 nisinaka Exp $
 */
public class JunKCV extends JunGeometricOperator {
	/** The topology on which the receiver operates. */
	protected JunEdgeProxy edgeProxy;

	/** The geometry which corresponds to the receiver's topology. */
	protected JunCurve curve;

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param aBody sra.jun.topology.elements.JunBody
	 * @param anEdgeOrProxy
	 *        sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 * 
	 * @return sra.jun.topology.geometricoperators.JunKCV
	 */
	public static final JunKCV Body_edge_(JunBody aBody, JunTopologicalElementOrProxy anEdgeOrProxy) {
		JunKCV anOperator = new JunKCV();
		anOperator.body_(aBody);
		anOperator.edge_(anEdgeOrProxy);

		return anOperator;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return sra.jun.geometry.abstracts.JunCurve
	 */
	public JunCurve curve() {
		return curve;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param aCurve sra.jun.geometry.abstracts.JunCurve
	 */
	public void curve_(JunCurve aCurve) {
		curve = aCurve;
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return sra.jun.topology.elements.JunEdge
	 */
	public JunEdge edge() {
		return (JunEdge) this.getTopologicalElement_(edgeProxy);
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @param anEdgeOrProxy
	 *        sra.jun.topology.abstracts.JunTopologicalElementOrProxy
	 */
	public void edge_(JunTopologicalElementOrProxy anEdgeOrProxy) {
		edgeProxy = (JunEdgeProxy) this.getTopologicalElementProxy_advise_(anEdgeOrProxy, edgeProxy);
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	public void execute() {
		this.curve_(this.edge().basicCurve());
		this.edge().curve_(null);
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return sra.jun.topology.abstracts.JunAbstractOperator
	 */
	public JunAbstractOperator inverse() {
		return (JunAbstractOperator) (JunMCV.Body_edge_curve_(this.body(), edgeProxy, this.curve()));
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return boolean
	 */
	public boolean precondition() {
		JunEdge edge = this.edge();

		if (edge == null) {
			return false;
		}

		if (edge.basicCurve() == null) {
			return false;
		}

		return super.precondition();
	}
}
