package sra.jun.topology.globaloperators;

import sra.jun.geometry.abstracts.*;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.geometry.curves.Jun3dLine;
import sra.jun.geometry.surfaces.JunPolygon;
import sra.jun.topology.abstracts.*;
import sra.jun.topology.elements.*;
import sra.jun.topology.support.JunGeometryTransformer;
import sra.smalltalk.StBlockClosure;

/**
 * JunLineSCALE class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunLineSCALE.java,v 6.0 2003/10/28 10:52:26 nisinaka Exp $
 */
public class JunLineSCALE extends JunGlobalOperator {
	protected Jun3dPoint point;
	protected Jun3dPoint vector;
	protected double factor;

	/**
	 * Create a new instance of JunPlaneSCALE and initialize it.
	 * 
	 * @param aJunBody sra.jun.topology.elements.JunBody
	 * @param aJun3dPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param aNumber double
	 * 
	 * @return sra.jun.topology.globaloperators.JunPlaneSCALE
	 */
	public static final JunLineSCALE Body_point_vector_factor_(JunBody aJunBody, Jun3dPoint aJun3dPoint1, Jun3dPoint aJun3dPoint2, double aNumber) {
		JunLineSCALE anOperator = new JunLineSCALE();
		anOperator.body_(aJunBody);
		anOperator.point_(aJun3dPoint1);
		anOperator.vector_(aJun3dPoint2);
		anOperator.factor_(aNumber);

		return anOperator;
	}

	/**
	 * Execute the receiver.
	 */
	public void execute() {
		final Jun3dLine scaleAxis = this.scaleAxis();
		StBlockClosure scaleBlock = new StBlockClosure() {
			public Object value_(Object anObject) {
				Jun3dPoint p = (Jun3dPoint) anObject;
				Jun3dPoint referencePoint = scaleAxis.nearestPointFromPoint_(p);
				Jun3dPoint relativePosition = (Jun3dPoint) p.minus_(referencePoint);

				return relativePosition.multipliedBy_(factor).plus_(referencePoint);
			}
		};

		final JunGeometryTransformer transformer = JunGeometryTransformer.Block_(scaleBlock);
		this.body().vertexesDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				JunVertex vertex = (JunVertex) anObject;

				if (vertex.point() != null) {
					vertex.point_((Jun3dPoint) vertex.point().transform_(transformer));
				}

				return null;
			}
		});
		this.body().edgesDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				JunEdge edge = (JunEdge) anObject;
				Jun3dLine curve = (Jun3dLine) (edge.basicCurve());

				if (curve != null) {
					edge.setCurve_((JunCurve) (curve.transform_(transformer)));
				}

				return null;
			}
		});
		this.body().loopsDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				JunLoop loop = (JunLoop) anObject;
				JunPolygon surface = (JunPolygon) loop.basicSurface();

				if (surface != null) {
					loop.setSurface_((JunSurface) surface.transform_(transformer));
				}

				return null;
			}
		});
	}

	/**
	 * Answer the factor.
	 * 
	 * @return double
	 */
	public final double factor() {
		return factor;
	}

	/**
	 * Set the factor.
	 * 
	 * @param aNumber double
	 */
	public final void factor_(double aNumber) {
		factor = aNumber;
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return sra.jun.topology.abstracts.JunAbstractOperator
	 */
	public JunAbstractOperator inverse() {
		return JunLineSCALE.Body_point_vector_factor_(this.body(), this.point(), this.vector(), 1.0d / this.factor());
	}

	/**
	 * Answer the point.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public final Jun3dPoint point() {
		return point;
	}

	/**
	 * Set the point.
	 * 
	 * @param aJun3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public final void point_(Jun3dPoint aJun3dPoint) {
		point = aJun3dPoint;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public Jun3dLine scaleAxis() {
		return new Jun3dLine(this.point(), (Jun3dPoint) this.point().plus_(this.unitVector()));
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public Jun3dPoint unitVector() {
		return (Jun3dPoint) this.vector().unitVector();
	}

	/**
	 * Answer the vector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public final Jun3dPoint vector() {
		return vector;
	}

	/**
	 * Set the vector.
	 * 
	 * @param aJun3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public final void vector_(Jun3dPoint aJun3dPoint) {
		vector = aJun3dPoint;
	}
}
