package sra.jun.topology.globaloperators;

import java.util.Vector;
import sra.jun.geometry.curves.Jun3dLine;
import sra.jun.geometry.surfaces.JunPolygon;
import sra.jun.topology.abstracts.*;
import sra.jun.topology.elements.*;
import sra.smalltalk.StBlockClosure;

/**
 * JunNEGATE class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunNEGATE.java,v 6.0 2003/10/28 10:52:26 nisinaka Exp $
 */
public class JunNEGATE extends JunGlobalOperator {

	/**
	 * Create a new instance of JunNEGATE and initialize it with a body.
	 * 
	 * @param aBody sra.jun.topology.elements.JunBody
	 */
	public JunNEGATE(JunBody aBody) {
		super(aBody);
	}

	/**
	 * Create a new instance of JunNEGATE and initialize it with a body.
	 * 
	 * @param aBody sra.jun.topology.elements.JunBody
	 * 
	 * @return DOCUMENT ME!
	 */
	public static final JunNEGATE Body_(JunBody aBody) {
		return new JunNEGATE(aBody);
	}

	/**
	 * Execute the operation.
	 */
	public void execute() {
		final Vector connections = new Vector(body.numberOfEdges());
		body.edgesDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				JunEdge edge = (JunEdge) anObject;
				JunEdge rightPrev = edge.prevEdgeWithVertex_(edge.startVertex());
				JunEdge leftPrev = edge.prevEdgeWithVertex_(edge.endVertex());
				JunEdge[] anArray = { edge, rightPrev, leftPrev };
				connections.addElement(anArray);

				return null;
			}
		});

		int size = connections.size();

		for (int i = 0; i < size; i++) {
			JunEdge[] edgeRightLeft = (JunEdge[]) connections.elementAt(i);
			JunEdge edge = edgeRightLeft[0];
			JunEdge startRightEdge = edgeRightLeft[1];
			JunEdge endLeftEdge = edgeRightLeft[2];
			edge.startRightEdge_(startRightEdge);
			edge.endLeftEdge_(endLeftEdge);

			JunLoop leftLoop = edge.leftLoop();
			JunLoop rightLoop = edge.rightLoop();
			edge.leftLoop_(rightLoop);
			edge.rightLoop_(leftLoop);
		}

		body.edgesDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				JunEdge edge = (JunEdge) anObject;
				Jun3dLine curve = (Jun3dLine) edge.basicCurve();

				if (curve != null) {
					edge.setCurve_(curve.reversed());
				}

				return null;
			}
		});
		body.loopsDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				JunLoop loop = (JunLoop) anObject;
				JunPolygon surface = (JunPolygon) loop.basicSurface();

				if (surface != null) {
					loop.setSurface_(surface.reversed());
				}

				return null;
			}
		});
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return sra.jun.topology.abstracts.JunAbstractOperator
	 */
	public JunAbstractOperator inverse() {
		return new JunNEGATE(this.body());
	}
}
