package sra.jun.topology.globaloperators;

import sra.jun.geometry.abstracts.*;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.curves.Jun3dLine;
import sra.jun.geometry.surfaces.JunPolygon;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.topology.abstracts.*;
import sra.jun.topology.elements.*;
import sra.smalltalk.StBlockClosure;

/**
 * JunROTATE class
 * 
 *  @author    ASTI Shanghai
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunROTATE.java,v 6.0 2003/10/28 10:52:26 nisinaka Exp $
 */
public class JunROTATE extends JunGlobalOperator {
	protected Jun3dPoint point;
	protected Jun3dPoint vector;
	protected JunAngle angle;

	/**
	 * Create a new instance of JunROTATE and initialize it.
	 * 
	 * @param aJunBody sra.jun.topology.elements.JunBody
	 * @param aJun3dPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param aNumber double
	 * 
	 * @return sra.jun.topology.globaloperators.JunROTATE
	 */
	public static final JunROTATE Body_point_vector_angle_(JunBody aJunBody, Jun3dPoint aJun3dPoint1, Jun3dPoint aJun3dPoint2, double aNumber) {
		JunROTATE anOperator = new JunROTATE();
		anOperator.body_(aJunBody);
		anOperator.point_(aJun3dPoint1);
		anOperator.vector_(aJun3dPoint2);
		anOperator.angle_(aNumber);

		return anOperator;
	}

	/**
	 * Create a new instance of JunROTATE and initialize it.
	 * 
	 * @param aJunBody sra.jun.topology.elements.JunBody
	 * @param aJun3dPoint1 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJun3dPoint2 sra.jun.geometry.basic.Jun3dPoint
	 * @param aJunAngle sra.jun.geometry.basic.JunAngle
	 * 
	 * @return sra.jun.topology.globaloperators.JunROTATE
	 */
	public static final JunROTATE Body_point_vector_angle_(JunBody aJunBody, Jun3dPoint aJun3dPoint1, Jun3dPoint aJun3dPoint2, JunAngle aJunAngle) {
		JunROTATE anOperator = new JunROTATE();
		anOperator.body_(aJunBody);
		anOperator.point_(aJun3dPoint1);
		anOperator.vector_(aJun3dPoint2);
		anOperator.angle_(aJunAngle);

		return anOperator;
	}

	/**
	 * Answer the angle.
	 * 
	 * @return double
	 */
	public final JunAngle angle() {
		return angle;
	}

	/**
	 * Set the angle.
	 * 
	 * @param aNumber double
	 */
	public void angle_(double aNumber) {
		angle = JunAngle.FromRad_(aNumber);
	}

	/**
	 * Set the angle.
	 * 
	 * @param anAngle sra.jun.geometry.basic.JunAngle
	 */
	public final void angle_(JunAngle anAngle) {
		angle = anAngle;
	}

	/**
	 * Execute the receiver.
	 */
	public void execute() {
		final Jun3dTransformation rotation = Jun3dTransformation.Rotate_around_(this.angle(), new Jun3dLine(this.point(), (Jun3dPoint) this.point().plus_(this.vector())));
		this.body().vertexesDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				JunVertex vertex = (JunVertex) anObject;

				if (vertex.point() != null) {
					vertex.point_((Jun3dPoint) vertex.point().transform_(rotation));
				}

				return null;
			}
		});
		this.body().edgesDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				JunEdge edge = (JunEdge) anObject;
				Jun3dLine curve = (Jun3dLine) edge.basicCurve();

				if (curve != null) {
					edge.setCurve_((JunCurve) curve.transform_(rotation));
				}

				return null;
			}
		});
		this.body().loopsDo_(new StBlockClosure() {
			public Object value_(Object anObject) {
				JunLoop loop = (JunLoop) anObject;
				JunPolygon surface = (JunPolygon) loop.basicSurface();

				if (surface != null) {
					loop.setSurface_((JunSurface) surface.transform_(rotation));
				}

				return null;
			}
		});
	}

	/**
	 * Answer the inverse operation of the receiver.
	 * 
	 * @return sra.jun.topology.abstracts.JunAbstractOperator
	 */
	public JunAbstractOperator inverse() {
		return JunROTATE.Body_point_vector_angle_(this.body(), this.point(), this.vector(), this.angle().negated());
	}

	/**
	 * Answer the point.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public final Jun3dPoint point() {
		return point;
	}

	/**
	 * Set the point.
	 * 
	 * @param aJun3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public final void point_(Jun3dPoint aJun3dPoint) {
		point = aJun3dPoint;
	}

	/**
	 * Answer the vector.
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public final Jun3dPoint vector() {
		return vector;
	}

	/**
	 * Set the vector.
	 * 
	 * @param aJun3dPoint sra.jun.geometry.basic.Jun3dPoint
	 */
	public final void vector_(Jun3dPoint aJun3dPoint) {
		vector = aJun3dPoint;
	}
}
