package sra.jun.topology.support;

import sra.jun.collections.sequences.JunDoubleMatrix;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.smalltalk.*;

/**
 * JunGeometryTransformer class
 * 
 *  @author    nisinaka
 *  @created   1998/12/02 (by nisinaka)
 *  @updated   2000/01/06 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunGeometryTransformer.java,v 6.0 2003/10/28 10:52:27 nisinaka Exp $
 */
public class JunGeometryTransformer extends Jun3dTransformation {
	/** The block closure to be used for a transformation. */
	protected StBlockClosure block = null;

	/**
	 * Create a new instance of JunGeometryTransformer and initialize it with
	 * the StBlockClosure.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public JunGeometryTransformer(StBlockClosure aBlock) {
		super();
		this.block_(aBlock);
	}

	/**
	 * Can not create an instance without specifying a StBlockClosure.
	 */
	private JunGeometryTransformer() {
	}

	/**
	 * Create a new instance of JunGeometryTransformer and initialize it with
	 * the StBlockClosure.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return sra.jun.topology.support.JunGeometryTransformer
	 */
	public static JunGeometryTransformer Block_(StBlockClosure aBlock) {
		return new JunGeometryTransformer(aBlock);
	}

	/**
	 * Apply this transformation to aPoint and return the result.
	 * 
	 * @param aPoint sra.jun.geometry.basic.Jun3dPoint
	 * 
	 * @return sra.jun.geometry.basic.Jun3dPoint
	 */
	public Jun3dPoint applyToPoint_(Jun3dPoint aPoint) {
		return (Jun3dPoint) block.value_(aPoint);
	}

	/**
	 * Convert this transformation as an array.
	 * 
	 * @return double[]
	 */
	public double[] asArray() {
		throw SmalltalkException.ShouldNotImplement();
	}

	/**
	 * Convert this transformation as a matrix.
	 * 
	 * @return sra.jun.collections.sequences.JunDoubleMatrix
	 */
	public JunDoubleMatrix asMatrix() {
		throw SmalltalkException.ShouldNotImplement();
	}

	/**
	 * Answer true if aTransformation is equal to this point, otherwise false.
	 * 
	 * @param anObject java.lang.Object
	 * 
	 * @return boolean
	 */
	public boolean equals(Object anObject) {
		throw SmalltalkException.ShouldNotImplement();
	}

	/**
	 * Set the instance variable "block".
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	private void block_(StBlockClosure aBlock) {
		block = aBlock;
	}
}
