package sra.jun.voronoi.twoD.diagram;

import java.awt.*;
import sra.jun.geometry.basic.Jun2dPoint;
import sra.smalltalk.StImage;

/**
 * JunDelaunay2dDiagram class
 * 
 *  @author    NISHIHARA Satoshi
 *  @created   2000/02/07 (by NISHIHARA Satoshi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunDelaunay2dDiagram.java,v 6.0 2003/10/28 10:52:27 nisinaka Exp $
 */
public class JunDelaunay2dDiagram extends JunVoronoi2dDiagram {
	private final Color triangleColor = Color.magenta;
	private final Color borderColor = Color.cyan;

	/**
	 * Create a new instance of JunDelaunay2dDiagram.
	 * 
	 * @param d
	 */
	public JunDelaunay2dDiagram(Jun2dPoint d) {
		super(d);
	}

	/**
	 * converting
	 * 
	 * @return JunDelaunay2dDiagram
	 */
	public JunDelaunay2dDiagram asDelaunayDiagram() {
		return this;
	}

	/**
	 * converting
	 * 
	 * @return JunVoronoi2dDiagram
	 */
	public JunVoronoi2dDiagram asVoronoiDiagram() {
		JunVoronoi2dDiagram aJunVoronoi2dDiagram = new JunVoronoi2dDiagram(this.extent());
		aJunVoronoi2dDiagram.addAll_(this.points());

		return aJunVoronoi2dDiagram;
	}

	/**
	 * utilities
	 * 
	 * @return String
	 */
	public String labelString() {
		return "Delaunay Diagram".intern();
	}

	/**
	 * private
	 * 
	 * @return Image
	 */
	protected Image makeImage() {
		try {
			voronoiProcessor = JunVoronoi2dProcessor.Dots_(this.points());
			processingCondition = voronoiProcessor.compute();

			Dimension d = new Dimension((int) Math.floor(this.extent().x()), (int) Math.floor(this.extent().y()));
			Image pixmap = new StImage(d.width, d.height).image();
			Graphics gc = pixmap.getGraphics();

			if (gc instanceof Graphics2D) {
				((Graphics2D) gc).setBackground(Color.white);
			}

			Rectangle bounds = new Rectangle(0, 0, d.width, d.height);
			gc.setColor(Color.white);
			gc.fillRect(bounds.x, bounds.y, bounds.width + 1, bounds.height + 1);

			Jun2dPoint factor = new Jun2dPoint(this.extent().x() / (this.extent().x() + 1), this.extent().y() / (this.extent().y() + 1));
			gc.setColor(this.borderColor);

			//gc lineWidth: 1.
			gc.drawRect(0, 0, bounds.width - 1, bounds.height - 1);
			gc.setColor(this.triangleColor);

			Jun2dPoint[][] triangles = voronoiProcessor.triangles();

			for (int i = 0; i < triangles.length; i++) {
				Jun2dPoint point1 = triangles[i][0].scaledBy_(factor);
				Jun2dPoint point2 = triangles[i][1].scaledBy_(factor);
				Jun2dPoint point3 = triangles[i][2].scaledBy_(factor);
				Point[] points = new Point[3];
				points[0] = new Point((int) Math.round(point1.x()), (int) Math.round(point1.y()));
				points[1] = new Point((int) Math.round(point2.x()), (int) Math.round(point2.y()));
				points[2] = new Point((int) Math.round(point3.x()), (int) Math.round(point3.y()));
				voronoiProcessor.displayLoopOn_points_clip_(gc, points, bounds);
			}

			return pixmap;
		} catch (Exception e) {
			e.printStackTrace();

			return null;
		}
	}
}
