package sra.jun.voronoi.twoD.diagram;

import java.awt.*;
import java.util.*;
import sra.jun.geometry.basic.Jun2dPoint;
import sra.smalltalk.*;

/**
 * JunVoronoi2dDiagram class
 * 
 *  @author    NISHIHARA Satoshi
 *  @created   2000/02/07 (by NISHIHARA Satoshi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVoronoi2dDiagram.java,v 6.0 2003/10/28 10:52:27 nisinaka Exp $
 */
public class JunVoronoi2dDiagram extends Object {
	protected Jun2dPoint diagramExtent = new Jun2dPoint(100, 100);
	protected Vector diagramPoints;
	protected Image diagramImage;
	protected JunVoronoi2dProcessor voronoiProcessor;
	protected boolean processingCondition;
	protected boolean isTrace = false;
	private final Color sideColor = Color.red;
	private final Color dotColor = Color.blue;
	private final Color borderColor = Color.green;
	private boolean isDisplayCircle = false;

	/**
	 * Create a new instance of JunVoronoi2dDiagram.
	 * 
	 * @param d
	 */
	public JunVoronoi2dDiagram(Jun2dPoint d) {
		super();
		this.initialize();
		this.extent_(d);
	}

	/**
	 * adding
	 * 
	 * @param aPoint
	 */
	public void add_(Jun2dPoint aPoint) {
		if (this.condition()) {
			final double extentX = this.extent().x();
			final double extentY = this.extent().y();
			Jun2dPoint factor = new Jun2dPoint(extentX / (extentX + 1), extentY / (extentY + 1));
			Jun2dPoint point = new Jun2dPoint(aPoint.x() * factor.x(), aPoint.y() * factor.y());

			if ((this.containsPoint_(point)) && (!diagramPoints.contains(point))) {
				diagramPoints.addElement(point);
				this.flushImage();
			}
		}
	}

	/**
	 * removing
	 * 
	 * @param points
	 */
	public void addAll_(Vector points) {
		for (Enumeration e = points.elements(); e.hasMoreElements();) {
			Jun2dPoint point = (Jun2dPoint) e.nextElement();
			this.add_(point);
		}
	}

	/**
	 * accessing
	 * 
	 * @return double
	 */
	public double area() {
		JunVoronoi2dProcessor processor = this.processor();

		if (!this.condition()) {
			return 0.0d;
		}

		return processor.area();
	}

	/**
	 * converting
	 * 
	 * @return JunDelaunay2dDiagram
	 */
	public JunDelaunay2dDiagram asDelaunayDiagram() {
		JunDelaunay2dDiagram aJunDelaunay2dDiagram = new JunDelaunay2dDiagram(this.extent());
		aJunDelaunay2dDiagram.addAll_(this.points());

		return aJunDelaunay2dDiagram;
	}

	/**
	 * converting
	 * 
	 * @return Image
	 */
	public Image asImage() {
		return this.diagram();
	}

	/**
	 * converting
	 * 
	 * @return JunVoronoi2dDiagram
	 */
	public JunVoronoi2dDiagram asVoronoiDiagram() {
		return this;
	}

	/**
	 * bounds accessing
	 * 
	 * @return Rectangle
	 */
	public Rectangle bounds() {
		return this.preferredBounds();
	}

	/**
	 * accessing
	 * 
	 * @return boolean
	 */
	public boolean condition() {
		return processingCondition;
	}

	/**
	 * accessing
	 * 
	 * @return Image
	 */
	public Image diagram() {
		if (diagramImage == null) {
			diagramImage = this.makeImage();
		}

		return diagramImage;
	}

	/**
	 * override
	 * 
	 * @param aGraphics
	 */
	public void displayOn_(Graphics aGraphics) {
		this.paint(aGraphics);
	}

	/**
	 * accessing
	 * 
	 * @return Jun2dPoint
	 */
	public Jun2dPoint extent() {
		return diagramExtent;
	}

	/**
	 * accessing
	 * 
	 * @param extentPoint
	 */
	public void extent_(Jun2dPoint extentPoint) {
		diagramExtent = extentPoint.max_(new Jun2dPoint(1, 1));
		this.flushImage();
	}

	/**
	 * accessing
	 * 
	 * @return Dimension
	 */
	public Dimension extentByDimension() {
		return new Dimension(this.extentByPoint().x, extentByPoint().y);
	}

	/**
	 * accessing
	 * 
	 * @return Point
	 */
	public Point extentByPoint() {
		return new Point((int) Math.floor(this.extent().x()), (int) Math.floor(this.extent().y()));
	}

	/**
	 * initialize-release
	 */
	public void initialize() {
		diagramPoints = new Vector();
		diagramImage = null;
		voronoiProcessor = null;
		processingCondition = true;
	}

	/**
	 * accessing
	 * 
	 * @return boolean
	 */
	public boolean isDisplayCircle() {
		return isDisplayCircle;
	}

	/**
	 * accessing
	 * 
	 * @param which
	 */
	public void isDisplayCircle_(boolean which) {
		isDisplayCircle = which == true;
	}

	/**
	 * utilities
	 * 
	 * @return String
	 */
	public String labelString() {
		return "Voronoi Diagram".intern();
	}

	/**
	 * override
	 * 
	 * @param g
	 */
	public void paint(Graphics g) {
		g.drawImage(this.asImage(), 0, 0, null);
	}

	/**
	 * accessing
	 * 
	 * @return Vector
	 */
	public Vector points() {
		final double extentX = this.extent().x();
		final double extentY = this.extent().y();
		final Vector newPoints = new Vector(diagramPoints.size());
		Jun2dPoint factor = new Jun2dPoint((extentX + 1) / extentX, (extentY + 1) / extentY);

		for (Enumeration e = this.diagramPoints.elements(); e.hasMoreElements();) {
			Jun2dPoint pointFrom = (Jun2dPoint) e.nextElement();
			Jun2dPoint pointTo = new Jun2dPoint(pointFrom.x() * factor.x(), pointFrom.y() * factor.y());
			newPoints.addElement(pointTo);
		}

		return newPoints;
	}

	/**
	 * accessing
	 * 
	 * @return String
	 */
	public String precision() {
		return this.processor().precision().intern();
	}

	/**
	 * bounds accessing
	 * 
	 * @return Rectangle
	 */
	public Rectangle preferredBounds() {
		return new Rectangle(0, 0, (int) Math.floor(this.extent().x()), (int) Math.floor(this.extent().y()));
	}

	/**
	 * bounds accessing
	 * 
	 * @return Dimension
	 */
	public Dimension preferredSize() {
		return new Dimension(this.preferredBounds().width, this.preferredBounds().height);
	}

	/**
	 * accessing
	 * 
	 * @return JunVoronoi2dProcessor
	 */
	public JunVoronoi2dProcessor processor() {
		this.diagram();

		return voronoiProcessor;
	}

	/**
	 * removing
	 * 
	 * @param aPoint
	 * 
	 * @return Jun2dPoint
	 */
	public Jun2dPoint remove_(Jun2dPoint aPoint) {
		Jun2dPoint point = null;

		if (diagramPoints.contains(point)) {
			point = (Jun2dPoint) diagramPoints.elementAt(diagramPoints.indexOf(aPoint));
			diagramPoints.removeElement(aPoint);
			this.flushImage();
		}

		return point;
	}

	/**
	 * tracing on/off
	 * 
	 * @return boolean
	 */
	public boolean trace() {
		return isTrace;
	}

	/**
	 * tracing on/off
	 * 
	 * @param bool
	 */
	public void trace_(boolean bool) {
		isTrace = bool == true;
	}

	/**
	 * private
	 * 
	 * @param gc
	 * @param factor
	 * @param bounds
	 */
	private void circle_scale_clip_(Graphics gc, Jun2dPoint factor, Rectangle bounds) {
		//if (true) {
		if (!isDisplayCircle()) {
			//^'do nothing'
			return;
		}

		gc.setColor(Color.yellow);

		JunVoronoi2dProcessor processor = this.voronoiProcessor;
		StInterval interval = StInterval.From_to_(processor.startIndex(), processor.voronoiPoints().size());
		final int startIndex = processor.startIndex();
		final int size = processor.voronoiPoints().size();

		for (int index = startIndex; index < size; index++) {
			JunVoronoi2dPoint point = (JunVoronoi2dPoint) processor.voronoiPoints().elementAt(index);
			JunVoronoi2dDot[] dots = processor.dotsAtPoint_(point);
			JunVoronoi2dDot dot1 = dots[0];
			JunVoronoi2dDot dot2 = dots[1];
			JunVoronoi2dDot dot3 = dots[2];
			JunVoronoi2dLine line1 = dot1.bisector_(dot2);
			JunVoronoi2dLine line2 = dot2.bisector_(dot3);
			JunVoronoi2dPoint center0 = line1.intersectingPointWithLine_(line2);
			Jun2dPoint center = processor.restorePoint_(center0).scaledBy_(factor);
			final double radius = center.distance_(processor.restoreDot_(dot1).scaledBy_(factor));

			if (bounds.contains((int) Math.round(center.x()), (int) Math.round(center.y()))) {
				int iRadius = (int) Math.round(radius);
				Rectangle box = new Rectangle((int) Math.round(center.x()) - iRadius, (int) Math.round(center.y()) - iRadius, iRadius * 2, iRadius * 2);
				gc.drawArc(box.x, box.y, box.width, box.height, 0, 360);
			}
		}
	}

	/**
	 * testing
	 * 
	 * @param aPoint
	 * 
	 * @return boolean
	 */
	protected boolean containsPoint_(Jun2dPoint aPoint) {
		Rectangle aRectangle = this.bounds();

		if ((aRectangle.x <= aPoint.x()) && (aRectangle.y <= aPoint.y()) && (aRectangle.x + aRectangle.width >= aPoint.x()) && (aRectangle.y + aRectangle.height >= aPoint.y())) {
			return true;
		}

		return false;
	}

	/**
	 * removing
	 */
	private void flushImage() {
		diagramImage = null;
	}

	/**
	 * private
	 * 
	 * @return Image
	 */
	protected Image makeImage() {
		try {
			voronoiProcessor = JunVoronoi2dProcessor.Dots_(this.points());
			voronoiProcessor.trace_(this.trace());
			processingCondition = voronoiProcessor.compute();

			Dimension d = new Dimension((int) Math.floor(this.extent().x()), (int) Math.floor(this.extent().y()));
			Image pixmap = new StImage(d.width, d.height).image();
			Graphics gc = pixmap.getGraphics();

			if (gc instanceof Graphics2D) {
				((Graphics2D) gc).setBackground(Color.white);
			}

			Rectangle bounds = new Rectangle(0, 0, d.width, d.height);
			gc.setColor(Color.white);
			gc.fillRect(bounds.x, bounds.y, bounds.width + 1, bounds.height + 1);

			Jun2dPoint factor = new Jun2dPoint(this.extent().x() / (this.extent().x() + 1), this.extent().y() / (this.extent().y() + 1));
			gc.setColor(this.borderColor);

			//gc lineWidth: 1.
			gc.drawRect(0, 0, bounds.width - 1, bounds.height - 1);
			this.circle_scale_clip_(gc, factor, bounds);
			gc.setColor(this.dotColor);

			Jun2dPoint[] dots = voronoiProcessor.dots();

			for (int i = 0; i < dots.length; i++) {
				Jun2dPoint dot = dots[i];
				Rectangle box = new Rectangle(0, 0, 1, 1);
				box.x = (int) Math.round(dot.x() * factor.x());
				box.y = (int) Math.round(dot.y() * factor.y());
				box.x--;
				box.y--;
				box.width++;
				box.height++;
				voronoiProcessor.displayBoxOn_box_clip_(gc, box, bounds);
			}

			//gc lineWidth: 1.
			gc.setColor(this.sideColor);

			Jun2dPoint[][] sides = voronoiProcessor.sides();

			for (int i = 0; i < sides.length; i++) {
				Jun2dPoint[] side = sides[i];
				int x;
				int y;
				x = (int) Math.round(side[0].x() * factor.x());
				y = (int) Math.round(side[0].y() * factor.y());

				Point point1 = new Point(x, y);
				x = (int) Math.round(side[1].x() * factor.x());
				y = (int) Math.round(side[1].y() * factor.y());

				Point point2 = new Point(x, y);
				voronoiProcessor.displayLineOn_from_to_clip_(gc, point1, point2, bounds);
			}

			return pixmap;
		} catch (Exception e) {
			e.printStackTrace();

			return null;
		}
	}
}
