package sra.jun.voronoi.twoD.diagram;

import java.awt.Canvas;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Enumeration;
import java.util.Random;
import java.util.Vector;
import sra.jun.geometry.basic.Jun2dPoint;
import sra.jun.geometry.basic.JunAngle;
import sra.jun.goodies.display.JunDisplayModel;
import sra.jun.system.support.JunTestExamples;
import sra.smalltalk.StBlockClosure;
import sra.smalltalk.StInterval;

/**
 * JunVoronoi2dDiagramTestExamples class
 * 
 *  @author    NISHIHARA Satoshi
 *  @created   2000/02/07 (by NISHIHARA Satoshi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVoronoi2dDiagramTestExamples.java,v 6.0 2003/10/28 10:52:27 nisinaka Exp $
 */
public class JunVoronoi2dDiagramTestExamples extends JunTestExamples {

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example1() {
		final JunVoronoi2dDiagram diagram = new JunVoronoi2dDiagram(new Jun2dPoint(200, 200));
		final Random random = new Random();
		Canvas canvas = new Canvas() {
			public void paint(Graphics g) {
				g.drawImage(diagram.asImage(), 0, 0, null);

				return;
			}

			public void displayOn_(Graphics g) {
				this.paint(g);

				return;
			}
		};

		canvas.setSize(diagram.extentByDimension());

		Frame frame = new Frame(diagram.labelString());
		frame.add(canvas);
		frame.pack();
		frame.show();
		frame.setResizable(false);
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				e.getWindow().dispose();
			}
		});

		for (int i = 0; i < 50; i++) {
			diagram.add_(new Jun2dPoint(random.nextDouble(), random.nextDouble()).scaledBy_(diagram.extent()));
			diagram.displayOn_(canvas.getGraphics());
		}

		System.out.println("area: " + diagram.area());

		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example1b() {
		final JunVoronoi2dDiagram diagram = new JunVoronoi2dDiagram(new Jun2dPoint(200, 200));
		diagram.isDisplayCircle_(true);

		final Random random = new Random();
		Canvas canvas = new Canvas() {
			public void paint(Graphics g) {
				g.drawImage(diagram.asImage(), 0, 0, null);

				return;
			}

			public void displayOn_(Graphics g) {
				this.paint(g);

				return;
			}
		};

		canvas.setSize(diagram.extentByDimension());

		Frame frame = new Frame(diagram.labelString());
		frame.add(canvas);
		frame.pack();
		frame.show();
		frame.setResizable(false);
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				e.getWindow().dispose();
			}
		});

		for (int i = 0; i < 50; i++) {
			diagram.add_(new Jun2dPoint(random.nextDouble(), random.nextDouble()).scaledBy_(diagram.extent()));
			diagram.displayOn_(canvas.getGraphics());
		}

		System.out.println("area: " + diagram.area());

		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example2() {
		final JunVoronoi2dDiagram diagram = new JunVoronoi2dDiagram(new Jun2dPoint(200, 200));
		Random random = new Random();

		for (int i = 0; i < 50; i++) {
			diagram.add_(new Jun2dPoint(random.nextDouble(), random.nextDouble()).scaledBy_(diagram.extent()));
		}

		Frame window1 = JunDisplayModel.Visual_(diagram.asImage()).open();
		window1.setTitle(diagram.labelString());
		window1.setResizable(false);

		Frame window2 = JunDisplayModel.Visual_(diagram.asDelaunayDiagram().asImage()).open();
		window2.setTitle(diagram.asDelaunayDiagram().labelString());
		window2.setResizable(false);
		System.out.println("area: " + diagram.area());

		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example3() {
		final JunVoronoi2dDiagram diagram = new JunVoronoi2dDiagram(new Jun2dPoint(200, 200));
		final Random random = new Random();
		final Canvas canvas = new Canvas() {
			public void paint(Graphics g) {
				g.drawImage(diagram.asImage(), 0, 0, null);

				return;
			}

			public void displayOn_(Graphics g) {
				this.paint(g);

				return;
			}
		};

		canvas.setSize(diagram.extentByDimension());

		Frame frame = new Frame(diagram.labelString());
		frame.add(canvas);
		frame.pack();
		frame.show();
		frame.setResizable(false);
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				e.getWindow().dispose();
			}
		});
		class T extends Thread {
			public void run() {
				while (true) {
					if (canvas.getGraphics() == null) {
						break;
					}

					try {
						diagram.add_(new Jun2dPoint(random.nextDouble(), random.nextDouble()).scaledBy_(diagram.extent()));
						diagram.displayOn_(canvas.getGraphics());

						try {
							sleep(100);
						} catch (InterruptedException e) {
						}
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		}

		T t = new T();
		t.start();

		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example4() {
		System.out.println("This is a low precision example.");

		final JunVoronoi2dDiagram diagram = new JunVoronoi2dDiagram(new Jun2dPoint(200, 200));
		Vector dots = new Vector();
		dots.addElement(new Jun2dPoint(0.1d, 0.1d));
		dots.addElement(new Jun2dPoint(0.1d, 0.1d));
		dots.addElement(new Jun2dPoint(0.1d, 0.9d));
		dots.addElement(new Jun2dPoint(0.9d, 0.1d));
		dots.addElement(new Jun2dPoint(0.9d, 0.9d));
		dots.addElement(new Jun2dPoint(0.5d, 0.5d));
		dots.addElement(new Jun2dPoint(0.1d, 0.5d));
		dots.addElement(new Jun2dPoint(0.9d, 0.5d));
		dots.addElement(new Jun2dPoint(0.5d, 0.1d));
		dots.addElement(new Jun2dPoint(0.5d, 0.9d));

		for (Enumeration e = dots.elements(); e.hasMoreElements();) {
			Jun2dPoint dot = (Jun2dPoint) e.nextElement();
			diagram.add_(new Jun2dPoint(dot.x(), dot.y()).scaledBy_(diagram.extent()));
		}

		Frame window = JunDisplayModel.Visual_(diagram.asImage()).open();
		window.setTitle(diagram.labelString());
		window.setResizable(false);
		System.out.println("area: " + diagram.area());

		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example5() {
		System.out.println("This is a low precision example.");

		final JunVoronoi2dDiagram diagram = new JunVoronoi2dDiagram(new Jun2dPoint(200, 200));
		final int roundTimes = 10;
		int count = 0;

		for (int i = 0; i <= roundTimes; i++) {
			for (int j = 0; j <= roundTimes; j++) {
				if ((count % 2) == 0) {
					diagram.add_((new Jun2dPoint((double) j / 10, (double) i / 10)).scaledBy_(diagram.extent()));
				}

				count++;
			}
		}

		Frame window = JunDisplayModel.Visual_(diagram.asImage()).open();
		window.setTitle(diagram.labelString());
		window.setResizable(false);
		System.out.println("area: " + diagram.area());

		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example6() {
		System.out.println("This is a low precision example.");

		final JunVoronoi2dDiagram diagram = new JunVoronoi2dDiagram(new Jun2dPoint(200, 200));
		final Vector dots = new Vector();
		final int step = 10;
		dots.addElement(new Jun2dPoint(0.0d, 0.0d));

		StInterval interval = StInterval.From_to_by_(0, 360, step);
		interval.do_(new StBlockClosure() {
			public Object value_(Object valueObject) {
				double degrees = ((Double) valueObject).doubleValue();
				double x = Math.cos(JunAngle._DegreesToRadians(degrees));
				double y = Math.sin(JunAngle._DegreesToRadians(degrees));
				dots.addElement(new Jun2dPoint(x, y));
				x = Math.cos(JunAngle._DegreesToRadians(degrees + (step / 2)));
				y = Math.sin(JunAngle._DegreesToRadians(degrees + (step / 2)));
				dots.addElement(new Jun2dPoint(x * 0.5, y * 0.5));

				return null;
			}
		});

		for (Enumeration e = dots.elements(); e.hasMoreElements();) {
			Jun2dPoint dot = (Jun2dPoint) e.nextElement();
			diagram.add_(dot.plus_(1).scaledBy_(diagram.extent().dividedBy_(2)));
		}

		Frame window = JunDisplayModel.Visual_(diagram.asImage()).open();
		window.setTitle(diagram.labelString());
		window.setResizable(false);
		System.out.println("area: " + diagram.area());

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * 
	 * @throws Exception DOCUMENT ME!
	 */
	public static void main(String[] args) throws Exception {
		new JunVoronoi2dDiagramTestExamples();
	}
}
