package sra.jun.voronoi.twoD.diagram;

import java.io.IOException;
import java.io.Writer;
import sra.smalltalk.SmalltalkException;

/**
 * JunVoronoi2dLine class
 * 
 *  @author    NISHIHARA Satoshi
 *  @created   2000/01/25 (by NISHIHARA Satoshi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVoronoi2dLine.java,v 6.0 2003/10/28 10:52:27 nisinaka Exp $
 */
public class JunVoronoi2dLine extends JunVoronoi2dObject {
	protected double a;
	protected double b;
	protected double c;

	/**
	 * Create a new instance of JunVoronoi2dLine.
	 * 
	 * @param a
	 * @param b
	 * @param c
	 */
	public JunVoronoi2dLine(double a, double b, double c) {
		super();
		this.a_(a);
		this.b_(b);
		this.c_(c);
	}

	/**
	 * Create a new instance of JunVoronoi2dLine.
	 * 
	 * @param voronoi2dDot1
	 * @param voronoi2dDot2
	 */
	public JunVoronoi2dLine(JunVoronoi2dDot voronoi2dDot1, JunVoronoi2dDot voronoi2dDot2) {
		super();

		final double x1 = voronoi2dDot1.x();
		final double y1 = voronoi2dDot1.y();
		final double x2 = voronoi2dDot2.x();
		final double y2 = voronoi2dDot2.y();
		final double a = y1 - y2;
		final double b = x2 - x1;
		final double c = (x1 * y2) - (x2 * y1);

		if ((Math.abs(a) < ACCURACY) && (Math.abs(b) < ACCURACY)) {
			throw SmalltalkException.Error("two points are the same.");
		}

		this.a_(a);
		this.b_(b);
		this.c_(c);
	}

	/**
	 * Create a new instance of JunVoronoi2dLine.
	 * 
	 * @param voronoi2dDot1
	 * @param voronoi2dDot2
	 * 
	 * @return JunVoronoi2dLine
	 */
	public static JunVoronoi2dLine From_to_(JunVoronoi2dDot voronoi2dDot1, JunVoronoi2dDot voronoi2dDot2) {
		return new JunVoronoi2dLine(voronoi2dDot1, voronoi2dDot2);
	}

	/**
	 * accessor a.
	 * 
	 * @return double
	 */
	public double a() {
		return this.a;
	}

	/**
	 * accessor a:.
	 * 
	 * @param aNumber
	 */
	public void a_(double aNumber) {
		this.a = aNumber;
	}

	/**
	 * accessor b.
	 * 
	 * @return double
	 */
	public double b() {
		return this.b;
	}

	/**
	 * accessor b:.
	 * 
	 * @param aNumber
	 */
	public void b_(double aNumber) {
		this.b = aNumber;
	}

	/**
	 * accessor c.
	 * 
	 * @return double
	 */
	public double c() {
		return this.c;
	}

	/**
	 * accessor c:.
	 * 
	 * @param aNumber
	 */
	public void c_(double aNumber) {
		this.c = aNumber;
	}

	/**
	 * functions:.
	 * 
	 * @param voronoi2dLine DOCUMENT ME!
	 * 
	 * @return JunVoronoi2dLine
	 */
	public JunVoronoi2dPoint intersectingPointWithLine_(JunVoronoi2dLine voronoi2dLine) {
		final double a1 = this.a();
		final double b1 = this.b();
		final double c1 = this.c();
		final double a2 = voronoi2dLine.a();
		final double b2 = voronoi2dLine.b();
		final double c2 = voronoi2dLine.c();
		final double det = (a1 * b2) - (a2 * b1);

		if (Math.abs(det) < ACCURACY) {
			throw SmalltalkException.Error("two lines are parallel.");
		}

		final double dinv = 1.0d / det;
		final double x = (b1 * c2 - b2 * c1) * dinv;
		final double y = (a2 * c1 - a1 * c2) * dinv;

		return new JunVoronoi2dPoint(x, y);
	}

	/**
	 * testing.
	 * 
	 * @return boolean
	 */
	public boolean isHorizontal() {
		return (Math.abs(a) < ACCURACY);
	}

	/**
	 * testing.
	 * 
	 * @return boolean
	 */
	public boolean isVertical() {
		return (Math.abs(b) < ACCURACY);
	}

	/**
	 * make JunVoronoi2dPoint from parameter by JunVoronoi2dLine.
	 * 
	 * @param x
	 * 
	 * @return JunVoronoi2dPoint
	 */
	public JunVoronoi2dPoint pointAtX_(double x) {
		final double y = (-c - a * x) / b;

		return new JunVoronoi2dPoint(x, y);
	}

	/**
	 * make JunVoronoi2dPoint from parameter by JunVoronoi2dLine.
	 * 
	 * @param y
	 * 
	 * @return JunVoronoi2dPoint
	 */
	public JunVoronoi2dPoint pointAtY_(double y) {
		final double x = (-c - b * y) / a;

		return new JunVoronoi2dPoint(x, y);
	}

	/**
	 * Print my string representation on aStream.
	 * 
	 * @param aWriter java.io.OutputStream
	 * 
	 * @throws IOException DOCUMENT ME!
	 */
	public void printOn_(Writer aWriter) throws IOException {
		aWriter.write("line ( ");
		aWriter.write(String.valueOf(this.a));
		aWriter.write(" , ");
		aWriter.write(String.valueOf(this.b));
		aWriter.write(" , ");
		aWriter.write(String.valueOf(this.c));
		aWriter.write(" )");
	}
}
