package sra.jun.voronoi.twoD.diagram;

import java.io.IOException;
import java.io.Writer;

/**
 * JunVoronoi2dPoint class
 * 
 *  @author    NISHIHARA Satoshi
 *  @created   2000/01/25 (by NISHIHARA Satoshi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVoronoi2dPoint.java,v 6.0 2003/10/28 10:52:27 nisinaka Exp $
 */
public class JunVoronoi2dPoint extends JunVoronoi2dDot {
	protected JunVoronoi2dSide side1 = null;
	protected JunVoronoi2dSide side2 = null;
	protected JunVoronoi2dSide side3 = null;

	/**
	 * Create a new instance of JunVoronoi2dPoint.
	 * 
	 * @param x double
	 * @param y double
	 */
	public JunVoronoi2dPoint(double x, double y) {
		super(x, y);
	}

	/**
	 * testing:.
	 * 
	 * @return boolean
	 */
	public boolean isInvalid() {
		return !this.isValid();
	}

	/**
	 * testing:.
	 * 
	 * @return boolean
	 */
	public boolean isValid() {
		boolean bool = true;

		if (x == Double.NaN) {
			bool = false;
		}

		if (y == Double.NaN) {
			bool = false;
		}

		if (side1 == null) {
			bool = false;
		} else {
			if (!side1.isValid2()) {
				bool = false;
			}
		}

		if (side2 == null) {
			bool = false;
		} else {
			if (!side2.isValid2()) {
				bool = false;
			}
		}

		if (side3 == null) {
			bool = false;
		} else {
			if (!side3.isValid2()) {
				bool = false;
			}
		}

		return bool;
	}

	/**
	 * Print my string representation on aStream.
	 * 
	 * @param aWriter java.io.OutputStream
	 * 
	 * @throws IOException DOCUMENT ME!
	 */
	public void printOn_(Writer aWriter) throws IOException {
		aWriter.write("point ( ");
		aWriter.write(String.valueOf(this.x()));
		aWriter.write(" , ");
		aWriter.write(String.valueOf(this.y()));

		if (VoronoiDebugFlag) {
			if (side1 == null) {
				aWriter.write(" , invalid side1");
			} else {
				if (!side1.isValid2()) {
					aWriter.write(" , invalid side1");
				}
			}

			if (side2 == null) {
				aWriter.write(" , invalid side2");
			} else {
				if (!side2.isValid2()) {
					aWriter.write(" , invalid side2");
				}
			}

			if (side3 == null) {
				aWriter.write(" , invalid side3");
			} else {
				if (!side3.isValid2()) {
					aWriter.write(" , invalid side3");
				}
			}
		}

		aWriter.write(" )");
	}

	/**
	 * accessor side1.
	 * 
	 * @return JunVoronoi2dSide
	 */
	public JunVoronoi2dSide side1() {
		return this.side1;
	}

	/**
	 * accessor side1:.
	 * 
	 * @param voronoi2dSide
	 */
	public void side1_(JunVoronoi2dSide voronoi2dSide) {
		this.side1 = voronoi2dSide;
	}

	/**
	 * accessor side2.
	 * 
	 * @return JunVoronoi2dSide
	 */
	public JunVoronoi2dSide side2() {
		return this.side2;
	}

	/**
	 * accessor side2:.
	 * 
	 * @param voronoi2dSide
	 */
	public void side2_(JunVoronoi2dSide voronoi2dSide) {
		this.side2 = voronoi2dSide;
	}

	/**
	 * accessor side3.
	 * 
	 * @return JunVoronoi2dSide
	 */
	public JunVoronoi2dSide side3() {
		return this.side3;
	}

	/**
	 * accessor side3:.
	 * 
	 * @param voronoi2dSide
	 */
	public void side3_(JunVoronoi2dSide voronoi2dSide) {
		this.side3 = voronoi2dSide;
	}
}
