package sra.jun.voronoi.twoD.diagram;

import java.awt.*;
import java.io.*;
import java.util.*;
import sra.jun.geometry.basic.Jun2dPoint;
import sra.smalltalk.*;

/**
 * JunVoronoi2dProcessor class
 * 
 *  @author    NISHIHARA Satoshi
 *  @created   2000/01/25 (by NISHIHARA Satoshi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVoronoi2dProcessor.java,v 6.0 2003/10/28 10:52:27 nisinaka Exp $
 */
public class JunVoronoi2dProcessor extends JunVoronoi2dObject {
	public static final Dimension defaultDimension = new Dimension(400, 400);
	private JunVoronoi2dDot[] originalDots = null;
	private JunVoronoi2dDot[] normalizedDots = null;
	private JunVoronoi2dDot translationDelta = null;
	private JunVoronoi2dDot scalingFactor = null;
	private Vector voronoiDots = null;
	private Vector voronoiPoints = null;
	private Vector voronoiSides = null;
	private double processingPrecision = ACCURACY;
	private boolean traceBoolean = false;
	private final double bigFactor = 100.0d;
	private final double bigSize = 100.0d;
	private final double bigNumber = bigSize * bigFactor;
	private final int loopMax = 5000;
	private final int startIndex = 4 - 1;
	private Graphics graphicsContext = null;
	private StImage graphicsContextStImage = null;
	private Image graphicsContextImage = null;
	private Canvas gcMediumCanvas = null;

	/**
	 * Create a new instance of JunVoronoi2dProcessor.
	 * 
	 * @param dotCollection
	 */
	public JunVoronoi2dProcessor(Vector dotCollection) {
		super();
		this.dots_(dotCollection);
	}

	/**
	 * Create a new instance of JunVoronoi2dProcessor.
	 * 
	 * @param dotCollection
	 * 
	 * @return DOCUMENT ME!
	 */
	public static JunVoronoi2dProcessor Dots_(Vector dotCollection) {
		return new JunVoronoi2dProcessor(dotCollection);
	}

	/**
	 * accessor triangles.
	 * 
	 * @return Jun2dPoint[][]
	 */
	public Jun2dPoint[][] _triangles() {
		if (normalizedDots == null) {
			return new Jun2dPoint[0][2];
		}

		Jun2dPoint[][] streamContents = new Jun2dPoint[voronoiPoints.size()][2];

		for (int i = this.startIndex; i < voronoiPoints.size(); i++) {
			JunVoronoi2dPoint point = (JunVoronoi2dPoint) voronoiPoints.elementAt(i);
			JunVoronoi2dTriangle triangle = this.triangleAtPoint_(point);

			if (triangle != null) {
				Jun2dPoint[] dots = new Jun2dPoint[3];
				dots[0] = this.restoreDot_(triangle.dot1());
				dots[1] = this.restoreDot_(triangle.dot2());
				dots[2] = this.restoreDot_(triangle.dot3());

				for (int j = 0; i < dots.length; j++) {
					streamContents[i][j] = dots[j];
					streamContents[i][j] = dots[j];
					streamContents[i][j] = dots[j];
				}
			}
		}

		return streamContents;
	}

	/**
	 * accessor area.
	 * 
	 * @return double
	 */
	public double area() {
		double area = 0.0d;
		Jun2dPoint[][] triangles = this.triangles();

		for (int i = 0; i < triangles.length; i++) {
			Jun2dPoint[] triangle = triangles[i];

			if (triangle[0] != null) {
				JunVoronoi2dDot[] dot = new JunVoronoi2dDot[triangle.length];

				for (int j = 0; j < triangle.length; j++) {
					dot[j] = new JunVoronoi2dDot(triangle[j].x(), triangle[j].y());
				}

				area += new JunVoronoi2dTriangle(dot[0], dot[1], dot[2]).area();
			}
		}

		return area;
	}

	/**
	 * inserting.
	 * 
	 * @param voronoi2dDot1 DOCUMENT ME!
	 * @param voronoi2dDot2 DOCUMENT ME!
	 * 
	 * @return JunVoronoi2dSide
	 */
	public JunVoronoi2dSide bisectorSideBetween_and_(JunVoronoi2dDot voronoi2dDot1, JunVoronoi2dDot voronoi2dDot2) {
		JunVoronoi2dLine line = voronoi2dDot1.bisector_(voronoi2dDot2);
		JunVoronoi2dPoint point1;
		JunVoronoi2dPoint point2;

		if (line.isVertical()) {
			point1 = line.pointAtY_(this.bigNumber * -1);
			point2 = line.pointAtY_(this.bigNumber);
		} else {
			point1 = line.pointAtX_(this.bigNumber * -1);
			point2 = line.pointAtX_(this.bigNumber);
		}

		return new JunVoronoi2dSide(point1, point2);
	}

	/**
	 * accessor triangles.
	 * 
	 * @return Canvas
	 */
	public Canvas canvas() {
		if (!this.trace()) {
			//return null;
			return new Canvas();
		}

		this.traceDisplayInitialize();

		return this.gcMediumCanvas;
	}

	/**
	 * computing.
	 * 
	 * @return boolean
	 */
	public boolean compute() {
		boolean condition = this.computeFirst();

		if (!condition) {
			condition = this.computeSecond();
		}

		return condition;
	}

	/**
	 * computing.
	 * 
	 * @return boolean
	 */
	public boolean computeFirst() {
		int i = 0;

		if (normalizedDots == null) {
			return true;
		}

		this.initializeVoronoiDiagram();
		this.debugInitialize();
		this.debug_((new Date()).toString());

		if (this.trace()) {
			this.traceDisplayOn_(this.traceDisplayInitialize());
		}

		int index = 0;
		boolean condition = true;

		try {
			final JunVoronoi2dDot[] dot4Block = new JunVoronoi2dDot[1];

			while (condition && (index < normalizedDots.length)) {
				JunVoronoi2dDot dot = normalizedDots[index];
				dot4Block[0] = dot;
				this.debug_("dots[" + (voronoiDots.size() + 1) + "]" + dot);

				if (this.trace()) {
					this.traceDisplayOn_(this.traceDisplayInitialize());
				}

				condition = this.insertDot_(dot);

				if (this.trace()) {
					this.traceDisplayOn_(this.traceDisplayInitialize());
				}

				index++;
			}
		} finally {
			if (this.trace()) {
			}
		}

		this.finalizeVoronoiDiagram();

		return condition;
	}

	/**
	 * computing.
	 * 
	 * @return boolean
	 */
	public boolean computeSecond() {
		processingPrecision = Math.pow(10.0d, Math.round(Math.log(ACCURACY) / Math.log(10) / 2.0d));

		Random random = new Random();

		for (int i = 0; i < normalizedDots.length; i++) {
			final JunVoronoi2dDot dot = normalizedDots[i];
			final double trembler = random.nextDouble() - 0.5d;
			final double x = Math.max(0.0d, (Math.min(dot.x() + (trembler * processingPrecision), 1.0d)));
			final double y = Math.max(0.0d, (Math.min(dot.y() + (trembler * processingPrecision), 1.0d)));
			dot.x_(x);
			dot.y_(y);
		}

		boolean condition = this.computeFirst();

		if (!condition) {
			this.insertError();
		}

		return condition;
	}

	/**
	 * displaying.
	 * 
	 * @param gc
	 * @param aRectangle
	 * @param clippingBox
	 */
	public void displayBoxOn_box_clip_(Graphics gc, Rectangle aRectangle, Rectangle clippingBox) {
		if (aRectangle.intersects(clippingBox)) {
			gc.fillRect(aRectangle.x, aRectangle.y, aRectangle.width + 1, aRectangle.height + 1);
		}
	}

	/**
	 * displaying.
	 * 
	 * @param gc
	 * @param fromPoint
	 * @param toPoint
	 * @param clippingBox
	 */
	public void displayLineOn_from_to_clip_(Graphics gc, Point fromPoint, Point toPoint, Rectangle clippingBox) {
		double a = Math.abs(toPoint.y - fromPoint.y);
		double slope = Math.abs(toPoint.y - fromPoint.y) / Math.max(Math.abs(toPoint.x - fromPoint.x), ACCURACY);
		Rectangle bounds = clippingBox;
		bounds.x -= 0;
		bounds.y -= 0;
		bounds.width += 1;
		bounds.height += 1;

		boolean boolean1 = bounds.contains(fromPoint);
		boolean boolean2 = bounds.contains(toPoint);
		bounds.x += 0;
		bounds.y += 0;
		bounds.width -= 1;
		bounds.height -= 1;

		if (boolean1 && boolean2) {
			gc.drawLine(fromPoint.x, fromPoint.y, toPoint.x, toPoint.y);
		} else {
			JunVoronoi2dPoint topLeft = new JunVoronoi2dPoint(bounds.x, bounds.y);
			JunVoronoi2dPoint bottomLeft = new JunVoronoi2dPoint(bounds.x, bounds.y + bounds.height);
			JunVoronoi2dPoint topRight = new JunVoronoi2dPoint(bounds.x + bounds.width, bounds.y);
			JunVoronoi2dPoint bottomRight = new JunVoronoi2dPoint(bounds.x + bounds.width, bounds.y + bounds.height);

			if (!boolean1 && !boolean2) {
				boolean boolean3 = false;
				JunVoronoi2dSide side = new JunVoronoi2dSide(new JunVoronoi2dPoint(fromPoint.x, fromPoint.y), new JunVoronoi2dPoint(toPoint.x, toPoint.y));

				if ((new JunVoronoi2dSide(topLeft, bottomLeft)).intersectingPointWithSide_(side) != null) {
					boolean3 = true;
				}

				if ((new JunVoronoi2dSide(topRight, bottomRight)).intersectingPointWithSide_(side) != null) {
					boolean3 = true;
				}

				if ((new JunVoronoi2dSide(topLeft, topRight)).intersectingPointWithSide_(side) != null) {
					boolean3 = true;
				}

				if ((new JunVoronoi2dSide(bottomLeft, bottomRight)).intersectingPointWithSide_(side) != null) {
					boolean3 = true;
				}

				if (boolean3) {
					JunVoronoi2dPoint from = new JunVoronoi2dPoint(fromPoint.x, fromPoint.y);
					JunVoronoi2dPoint to = new JunVoronoi2dPoint(toPoint.x, toPoint.y);
					JunVoronoi2dLine line = new JunVoronoi2dLine(from, to);
					JunVoronoi2dPoint point1;
					JunVoronoi2dPoint point2;

					if (slope > 1.0d) {
						final double top = bounds.y;
						final double bottom = bounds.y + bounds.height;
						point1 = line.pointAtY_(top);
						point2 = line.pointAtY_(bottom);
					} else {
						final double left = bounds.x;
						final double right = bounds.x + bounds.width;
						point1 = line.pointAtX_(left);
						point2 = line.pointAtX_(right);
					}

					Point point1J = new Point((int) Math.round(point1.x()), (int) Math.round(point1.y()));
					Point point2J = new Point((int) Math.round(point2.x()), (int) Math.round(point2.y()));
					gc.drawLine(point1J.x, point1J.y, point2J.x, point2J.y);
				}
			} else {
				JunVoronoi2dPoint point1;
				JunVoronoi2dPoint point2;

				if (boolean1) {
					point1 = new JunVoronoi2dPoint(fromPoint.x, fromPoint.y);
					point2 = new JunVoronoi2dPoint(toPoint.x, toPoint.y);

					JunVoronoi2dSide side = new JunVoronoi2dSide(point1, point2);
					JunVoronoi2dPoint point;
					point = (new JunVoronoi2dSide(topLeft, bottomLeft)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}

					point = (new JunVoronoi2dSide(topRight, bottomRight)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}

					point = (new JunVoronoi2dSide(topLeft, topRight)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}

					point = (new JunVoronoi2dSide(bottomLeft, bottomRight)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}
				} else {
					point1 = new JunVoronoi2dPoint(toPoint.x, toPoint.y);
					point2 = new JunVoronoi2dPoint(fromPoint.x, fromPoint.y);

					JunVoronoi2dSide side = new JunVoronoi2dSide(point1, point2);
					JunVoronoi2dPoint point;
					point = (new JunVoronoi2dSide(topLeft, bottomLeft)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}

					point = (new JunVoronoi2dSide(topRight, bottomRight)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}

					point = (new JunVoronoi2dSide(topLeft, topRight)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}

					point = (new JunVoronoi2dSide(bottomLeft, bottomRight)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}
				}

				Point point1J = new Point((int) Math.round(point1.x()), (int) Math.round(point1.y()));
				Point point2J = new Point((int) Math.round(point2.x()), (int) Math.round(point2.y()));
				gc.drawLine(point1J.x, point1J.y, point2J.x, point2J.y);
			}
		}
	}

	/**
	 * displaying.
	 * 
	 * @param gc
	 * @param fromPoint
	 * @param toPoint
	 * @param clippingBox
	 * @param lineWidth
	 */
	public void displayLineOn_from_to_clip_width_(Graphics gc, Point fromPoint, Point toPoint, Rectangle clippingBox, int lineWidth) {
		double a = Math.abs(toPoint.y - fromPoint.y);
		double slope = Math.abs(toPoint.y - fromPoint.y) / Math.max(Math.abs(toPoint.x - fromPoint.x), ACCURACY);
		Rectangle bounds = clippingBox;
		bounds.x -= 0;
		bounds.y -= 0;
		bounds.width += 1;
		bounds.height += 1;

		boolean boolean1 = bounds.contains(fromPoint);
		boolean boolean2 = bounds.contains(toPoint);
		bounds.x += 0;
		bounds.y += 0;
		bounds.width -= 1;
		bounds.height -= 1;

		if (boolean1 && boolean2) {
			if (lineWidth > 1) {
				Polygon polygon = this.polygonByWidth(fromPoint, toPoint, lineWidth);
				gc.fillPolygon(polygon);
			} else {
				gc.drawLine(fromPoint.x, fromPoint.y, toPoint.x, toPoint.y);
			}
		} else {
			JunVoronoi2dPoint topLeft = new JunVoronoi2dPoint(bounds.x, bounds.y);
			JunVoronoi2dPoint bottomLeft = new JunVoronoi2dPoint(bounds.x, bounds.y + bounds.height);
			JunVoronoi2dPoint topRight = new JunVoronoi2dPoint(bounds.x + bounds.width, bounds.y);
			JunVoronoi2dPoint bottomRight = new JunVoronoi2dPoint(bounds.x + bounds.width, bounds.y + bounds.height);

			if (!boolean1 && !boolean2) {
				boolean boolean3 = false;
				JunVoronoi2dSide side = new JunVoronoi2dSide(new JunVoronoi2dPoint(fromPoint.x, fromPoint.y), new JunVoronoi2dPoint(toPoint.x, toPoint.y));

				if ((new JunVoronoi2dSide(topLeft, bottomLeft)).intersectingPointWithSide_(side) != null) {
					boolean3 = true;
				}

				if ((new JunVoronoi2dSide(topRight, bottomRight)).intersectingPointWithSide_(side) != null) {
					boolean3 = true;
				}

				if ((new JunVoronoi2dSide(topLeft, topRight)).intersectingPointWithSide_(side) != null) {
					boolean3 = true;
				}

				if ((new JunVoronoi2dSide(bottomLeft, bottomRight)).intersectingPointWithSide_(side) != null) {
					boolean3 = true;
				}

				if (boolean3) {
					JunVoronoi2dPoint from = new JunVoronoi2dPoint(fromPoint.x, fromPoint.y);
					JunVoronoi2dPoint to = new JunVoronoi2dPoint(toPoint.x, toPoint.y);
					JunVoronoi2dLine line = new JunVoronoi2dLine(from, to);
					JunVoronoi2dPoint point1;
					JunVoronoi2dPoint point2;

					if (slope > 1.0d) {
						final double top = bounds.y;
						final double bottom = bounds.y + bounds.height;
						point1 = line.pointAtY_(top);
						point2 = line.pointAtY_(bottom);
					} else {
						final double left = bounds.x;
						final double right = bounds.x + bounds.width;
						point1 = line.pointAtX_(left);
						point2 = line.pointAtX_(right);
					}

					Point point1J = new Point((int) Math.round(point1.x()), (int) Math.round(point1.y()));
					Point point2J = new Point((int) Math.round(point2.x()), (int) Math.round(point2.y()));

					if (lineWidth > 1) {
						gc.fillPolygon(this.polygonByWidth(point1J, point2J, lineWidth));
					} else {
						gc.drawLine(point1J.x, point1J.y, point2J.x, point2J.y);
					}
				}
			} else {
				JunVoronoi2dPoint point1;
				JunVoronoi2dPoint point2;

				if (boolean1) {
					point1 = new JunVoronoi2dPoint(fromPoint.x, fromPoint.y);
					point2 = new JunVoronoi2dPoint(toPoint.x, toPoint.y);

					JunVoronoi2dSide side = new JunVoronoi2dSide(point1, point2);
					JunVoronoi2dPoint point;
					point = (new JunVoronoi2dSide(topLeft, bottomLeft)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}

					point = (new JunVoronoi2dSide(topRight, bottomRight)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}

					point = (new JunVoronoi2dSide(topLeft, topRight)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}

					point = (new JunVoronoi2dSide(bottomLeft, bottomRight)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}
				} else {
					point1 = new JunVoronoi2dPoint(toPoint.x, toPoint.y);
					point2 = new JunVoronoi2dPoint(fromPoint.x, fromPoint.y);

					JunVoronoi2dSide side = new JunVoronoi2dSide(point1, point2);
					JunVoronoi2dPoint point;
					point = (new JunVoronoi2dSide(topLeft, bottomLeft)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}

					point = (new JunVoronoi2dSide(topRight, bottomRight)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}

					point = (new JunVoronoi2dSide(topLeft, topRight)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}

					point = (new JunVoronoi2dSide(bottomLeft, bottomRight)).intersectingPointWithSide_(side);

					if (point != null) {
						point2 = point;
					}
				}

				Point point1J = new Point((int) Math.round(point1.x()), (int) Math.round(point1.y()));
				Point point2J = new Point((int) Math.round(point2.x()), (int) Math.round(point2.y()));

				if (lineWidth > 1) {
					gc.fillPolygon(this.polygonByWidth(point1J, point2J, lineWidth));
				} else {
					gc.drawLine(point1J.x, point1J.y, point2J.x, point2J.y);
				}
			}
		}
	}

	/**
	 * displaying.
	 * 
	 * @param gc
	 * @param pointCollection
	 * @param clippingBox
	 */
	public void displayLoopOn_points_clip_(Graphics gc, Point[] pointCollection, Rectangle clippingBox) {
		if (pointCollection.length < 3) {
			return;
		}

		for (int index = 0; index < (pointCollection.length - 1); index++) {
			Point fromPoint = pointCollection[index];
			Point toPoint = pointCollection[index + 1];
			this.displayLineOn_from_to_clip_(gc, fromPoint, toPoint, clippingBox);
		}

		if (!pointCollection[0].equals(pointCollection[pointCollection.length - 1])) {
			Point fromPoint = pointCollection[0];
			Point toPoint = pointCollection[pointCollection.length - 1];
			this.displayLineOn_from_to_clip_(gc, fromPoint, toPoint, clippingBox);
		}
	}

	/**
	 * accessor dots.
	 * 
	 * @return Jun2dPoint[]
	 */
	public Jun2dPoint[] dots() {
		if (normalizedDots == null) {
			return new Jun2dPoint[0];
		}

		Vector streamContents = new Vector();

		for (int i = 0; i < normalizedDots.length; i++) {
			Jun2dPoint point = this.restoreDot_(normalizedDots[i]);

			if (point != null) {
				streamContents.addElement(point);
			}
		}

		Jun2dPoint[] anArrayOfJun2dPoint = new Jun2dPoint[streamContents.size()];
		streamContents.copyInto(anArrayOfJun2dPoint);

		return anArrayOfJun2dPoint;
	}

	/**
	 * accessor dots:.
	 * 
	 * @param dotCollection
	 */
	public void dots_(Vector dotCollection) {
		Vector dots = null;

		if ((dotCollection != null) && (!dotCollection.isEmpty())) {
			Object obj = dotCollection.firstElement();

			if (obj instanceof Point) {
				dots = dotsFromPoints(dotCollection);
			} else if (obj instanceof Jun2dPoint) {
				dots = dotsFromJun2dPoints(dotCollection);
			}
		}

		if ((dots == null) || (dots.isEmpty())) {
			return;
		}

		originalDots = this.asArrayFromVectorToJunVoronoi2dDot(dots);
		normalizedDots = this.normalizedDots_(dots);
	}

	/**
	 * basic accessing.
	 * 
	 * @param voronoi2dPoint
	 * 
	 * @return JunVoronoi2dTriangle
	 */
	public JunVoronoi2dDot[] dotsAtPoint_(JunVoronoi2dPoint voronoi2dPoint) {
		JunVoronoi2dSide side1 = voronoi2dPoint.side1();
		JunVoronoi2dSide side2 = voronoi2dPoint.side2();
		JunVoronoi2dSide side3 = voronoi2dPoint.side3();
		StSortedCollection indexesCollection = new StSortedCollection(new sra.smalltalk.StBlockClosure() {
			public Object value_value_(Object e1, Object e2) {
				return new Boolean(((Number) e1).intValue() <= ((Number) e2).intValue());
			}
		});
		indexesCollection.add_(new Integer(voronoiDots.indexOf(side1.dot1())));
		indexesCollection.add_(new Integer(voronoiDots.indexOf(side1.dot2())));
		indexesCollection.add_(new Integer(voronoiDots.indexOf(side2.dot1())));
		indexesCollection.add_(new Integer(voronoiDots.indexOf(side2.dot2())));
		indexesCollection.add_(new Integer(voronoiDots.indexOf(side3.dot1())));
		indexesCollection.add_(new Integer(voronoiDots.indexOf(side3.dot2())));

		Object[] indexes = indexesCollection._asArray();
		final int index1 = ((Number) indexes[1 - 1]).intValue();
		final int index2 = ((Number) indexes[2 - 1]).intValue();
		final int index3 = ((Number) indexes[3 - 1]).intValue();
		final int index4 = ((Number) indexes[4 - 1]).intValue();
		final int index5 = ((Number) indexes[5 - 1]).intValue();
		final int index6 = ((Number) indexes[6 - 1]).intValue();

		if ((index1 != index2) || (index3 != index4) || (index5 != index6)) {
			throw SmalltalkException.Error("unexpected error.");
		}

		JunVoronoi2dDot[] dots = new JunVoronoi2dDot[3];
		dots[0] = (JunVoronoi2dDot) voronoiDots.elementAt(index1);
		dots[1] = (JunVoronoi2dDot) voronoiDots.elementAt(index3);
		dots[2] = (JunVoronoi2dDot) voronoiDots.elementAt(index5);

		return dots;
	}

	/**
	 * initialize-release
	 */
	public void finalizeVoronoiDiagram() {
		Vector obstacles = new Vector();
		int index = 0;

		for (Enumeration e = voronoiPoints.elements(); e.hasMoreElements();) {
			JunVoronoi2dPoint point = (JunVoronoi2dPoint) e.nextElement();

			if (point.isInvalid() && (this.startIndex <= index)) {
				obstacles.addElement(point);
			}
		}

		for (Enumeration e = obstacles.elements(); e.hasMoreElements();) {
			voronoiPoints.removeElement((JunVoronoi2dPoint) e.nextElement());
		}

		obstacles = new Vector();
		index = 0;

		for (Enumeration e = voronoiSides.elements(); e.hasMoreElements();) {
			JunVoronoi2dSide side = (JunVoronoi2dSide) e.nextElement();

			if (side.isInvalid() && (this.startIndex <= index)) {
				obstacles.addElement(side);
			}
		}

		for (Enumeration e = obstacles.elements(); e.hasMoreElements();) {
			voronoiSides.removeElement((JunVoronoi2dSide) e.nextElement());
		}
	}

	/**
	 * initialize-release
	 */
	public void initializeVoronoiDiagram() {
		JunVoronoi2dDot dot1 = new JunVoronoi2dDot(this.bigSize * -1, this.bigSize * -1);
		JunVoronoi2dDot dot2 = new JunVoronoi2dDot(10.0d, this.bigSize);
		JunVoronoi2dDot dot3 = new JunVoronoi2dDot(this.bigSize, this.bigSize * -1);
		JunVoronoi2dPoint point1 = new JunVoronoi2dPoint((dot1.x() + dot2.x()) * 0.5d, (dot1.y() + dot2.y()) * 0.5d);
		JunVoronoi2dPoint point2 = new JunVoronoi2dPoint((dot2.x() + dot3.x()) * 0.5d, (dot2.y() + dot3.y()) * 0.5d);
		JunVoronoi2dPoint point3 = new JunVoronoi2dPoint((dot3.x() + dot1.x()) * 0.5d, (dot3.y() + dot1.y()) * 0.5d);
		JunVoronoi2dLine line1 = dot1.bisector_(dot2);
		JunVoronoi2dLine line2 = dot2.bisector_(dot3);
		JunVoronoi2dPoint point4 = line1.intersectingPointWithLine_(line2);
		point1 = (JunVoronoi2dLine.From_to_(point1, point4)).pointAtX_(this.bigNumber * -1);
		point2 = (JunVoronoi2dLine.From_to_(point2, point4)).pointAtX_(this.bigNumber);
		point3 = (JunVoronoi2dLine.From_to_(point3, point4)).pointAtY_(this.bigNumber * -1);

		JunVoronoi2dSide side1 = JunVoronoi2dSide.From_to_(point1, point4);
		side1.dot1_(dot1);
		side1.dot2_(dot2);

		JunVoronoi2dSide side2 = JunVoronoi2dSide.From_to_(point2, point4);
		side2.dot1_(dot2);
		side2.dot2_(dot3);

		JunVoronoi2dSide side3 = JunVoronoi2dSide.From_to_(point3, point4);
		side3.dot1_(dot3);
		side3.dot2_(dot1);
		point1.side1_(side1);
		point2.side1_(side2);
		point3.side1_(side3);
		point4.side1_(side1);
		point4.side2_(side2);
		point4.side3_(side3);
		voronoiDots = new Vector();
		voronoiPoints = new Vector();
		voronoiSides = new Vector();
		voronoiDots.addElement(dot1);
		voronoiDots.addElement(dot2);
		voronoiDots.addElement(dot3);
		voronoiPoints.addElement(point1);
		voronoiPoints.addElement(point2);
		voronoiPoints.addElement(point3);
		voronoiPoints.addElement(point4);
		voronoiSides.addElement(side1);
		voronoiSides.addElement(side2);
		voronoiSides.addElement(side3);
	}

	/**
	 * inserting.
	 * 
	 * @param voronoi2dDot
	 * 
	 * @return boolean
	 */
	public boolean insertDot_(JunVoronoi2dDot voronoi2dDot) {
		Vector newPoints = new Vector();
		Vector newSides = new Vector();
		Hashtable obsoletePoints = new Hashtable();
		Hashtable examinedSides = new Hashtable();
		Vector rememberdFromEvaluationsPoints = new Vector();
		Vector rememberdFromEvaluationsSides = new Vector();
		Vector rememberdToEvaluationsPoints = new Vector();
		Vector rememberdToEvaluationsSides = new Vector();
		final JunVoronoi2dDot nearestDot = this.nearestDot_(voronoi2dDot);
		this.debug_("\t" + "nearest point: " + nearestDot);

		JunVoronoi2dSide bisectorSide1 = this.bisectorSideBetween_and_(nearestDot, voronoi2dDot);
		Vector searchSides = this.searchSides_(nearestDot);

		for (Enumeration e = searchSides.elements(); e.hasMoreElements();) {
			JunVoronoi2dSide side1 = (JunVoronoi2dSide) e.nextElement();

			if (!examinedSides.contains(side1)) {
				JunVoronoi2dPoint fromPoint = null;
				examinedSides.put(side1, side1);
				this.debug_("\t" + "intersecting side: " + side1);

				JunVoronoi2dPoint newPoint1 = side1.intersectingPointWithSide_(bisectorSide1);
				this.debug_("\t" + "intersecting point: " + newPoint1);

				if (newPoint1 != null) {
					JunVoronoi2dPoint toPoint = newPoint1;
					JunVoronoi2dPoint oldPoint1 = this.removingPoint_with_with_with_(newPoint1, bisectorSide1.from(), voronoi2dDot, side1);
					this.debug_("\t" + "old point: " + oldPoint1);
					obsoletePoints.put(oldPoint1, oldPoint1);

					JunVoronoi2dDot nearerDot = this.nearerDot_except_in_(oldPoint1, nearestDot, side1);
					this.debug_("\t" + "nearer point: " + nearerDot);

					JunVoronoi2dSide newSide1 = JunVoronoi2dSide.From_to_(fromPoint, toPoint);
					newSide1.dot1_(voronoi2dDot);
					newSide1.dot2_(nearestDot);

					JunVoronoi2dDot previousDot = nearerDot;
					int loopCounter = 1;

					while (!nearestDot.equals(nearerDot)) {
						JunVoronoi2dSide bisectorSide2 = this.bisectorSideBetween_and_(nearerDot, voronoi2dDot);
						Vector searchSides_ = this.searchSides_(nearerDot);

						for (Enumeration e2 = searchSides_.elements(); e2.hasMoreElements();) {
							JunVoronoi2dSide side2 = (JunVoronoi2dSide) e2.nextElement();

							if (!examinedSides.contains(side2)) {
								fromPoint = toPoint;
								examinedSides.put(side2, side2);
								this.debug_("\t" + "intersecting side: " + side2);

								JunVoronoi2dPoint newPoint2 = side2.intersectingPointWithSide_(bisectorSide2);
								this.debug_("\t" + "intersecting point: " + newPoint2);

								if (newPoint2 != null) {
									toPoint = newPoint2;

									JunVoronoi2dPoint oldPoint2 = this.removingPoint_with_with_with_(newPoint2, bisectorSide2.from(), voronoi2dDot, side2);
									this.debug_("\t" + "old point: " + oldPoint2);
									obsoletePoints.put(oldPoint2, oldPoint2);
									nearerDot = this.nearerDot_except_in_(oldPoint2, nearerDot, side2);
									this.debug_("\t" + "nearer point: " + nearerDot);

									JunVoronoi2dSide newSide2 = JunVoronoi2dSide.From_to_(fromPoint, toPoint);
									newSide2.dot1_(voronoi2dDot);
									newSide2.dot2_(previousDot);

									if (side2.from().equals(oldPoint2)) {
										rememberdFromEvaluationsSides.addElement(side2);
										rememberdFromEvaluationsPoints.addElement(newPoint2);
									} else {
										rememberdToEvaluationsSides.addElement(side2);
										rememberdToEvaluationsPoints.addElement(newPoint2);
									}

									newPoint2.side1_(side2);
									newPoint2.side2_(newSide2);
									previousDot = nearerDot;
									newPoints.addElement(newPoint2);
									newSides.addElement(newSide2);
								}
							}
						}

						loopCounter++;

						if (loopCounter > this.loopMax) {
							return false;
						}
					}

					if (side1.from().equals(oldPoint1)) {
						rememberdFromEvaluationsSides.addElement(side1);
						rememberdFromEvaluationsPoints.addElement(newPoint1);
					} else {
						rememberdToEvaluationsSides.addElement(side1);
						rememberdToEvaluationsPoints.addElement(newPoint1);
					}

					newSide1.from_(toPoint);
					newPoint1.side1_(side1);
					newPoint1.side2_(newSide1);
					newPoints.addElement(newPoint1);
					newSides.addElement(newSide1);
				}
			}
		}

		for (int i = 0; i < rememberdFromEvaluationsSides.size(); i++) {
			JunVoronoi2dSide receiver = (JunVoronoi2dSide) rememberdFromEvaluationsSides.elementAt(i);
			JunVoronoi2dPoint arguments = (JunVoronoi2dPoint) rememberdFromEvaluationsPoints.elementAt(i);
			receiver.from_(arguments);
		}

		for (int i = 0; i < rememberdToEvaluationsSides.size(); i++) {
			JunVoronoi2dSide receiver = (JunVoronoi2dSide) rememberdToEvaluationsSides.elementAt(i);
			JunVoronoi2dPoint arguments = (JunVoronoi2dPoint) rememberdToEvaluationsPoints.elementAt(i);
			receiver.to_(arguments);
		}

		if ((newPoints.size() > 0) && (newSides.size() > 0)) {
			int sideIndex = 1;
			int newSidesSize = newSides.size();

			for (int pointIndex = 0; pointIndex < newPoints.size(); pointIndex++) {
				if (sideIndex >= newSidesSize) {
					sideIndex = 0;
				}

				((JunVoronoi2dPoint) newPoints.elementAt(pointIndex)).side3_((JunVoronoi2dSide) newSides.elementAt(sideIndex));
				sideIndex++;
			}
		}

		Hashtable[] anArray = this.removePoints_andSides_inside_(obsoletePoints, examinedSides, newSides);
		Hashtable removedPoints = anArray[0];
		Hashtable removedSides = anArray[1];
		voronoiDots.addElement(voronoi2dDot);

		for (Enumeration e = newPoints.elements(); e.hasMoreElements();) {
			JunVoronoi2dPoint point = (JunVoronoi2dPoint) e.nextElement();

			if (!removedPoints.contains(point)) {
				voronoiPoints.addElement(point);
			}
		}

		for (Enumeration e = newSides.elements(); e.hasMoreElements();) {
			JunVoronoi2dSide side = (JunVoronoi2dSide) e.nextElement();

			if (!removedSides.contains(side)) {
				voronoiSides.addElement(side);
			}
		}

		return true;
	}

	/**
	 * inserting.
	 * 
	 * @return boolean
	 */
	public boolean insertError() {
		String errorString = this.getClass().getName() + " Error: infinite loop, " + "because all points on a lattice, " + "or no intersecting point, " + "or too many intersecting points.";
		this.debug_(errorString);

		return false;
	}

	/**
	 * inserting.
	 * 
	 * @param voronoi2dDot
	 * @param nearestDot
	 * @param voronoi2dSide
	 * 
	 * @return boolean
	 */
	public JunVoronoi2dDot nearerDot_except_in_(JunVoronoi2dPoint voronoi2dDot, JunVoronoi2dDot nearestDot, JunVoronoi2dSide voronoi2dSide) {
		JunVoronoi2dDot[] dotsAtPoint1 = this.dotsAtPoint_(voronoi2dDot);
		JunVoronoi2dDot nearerDot = null;

		for (int i = 0; i < dotsAtPoint1.length; i++) {
			JunVoronoi2dDot dot = dotsAtPoint1[i];

			if ((!dot.equals(nearestDot)) && ((voronoi2dSide.dot1().equals(dot)) || (voronoi2dSide.dot2().equals(dot)))) {
				nearerDot = dotsAtPoint1[i];

				break;
			}
		}

		return nearerDot;
	}

	/**
	 * inserting.
	 * 
	 * @param voronoi2dDot
	 * 
	 * @return boolean
	 */
	public JunVoronoi2dDot nearestDot_(JunVoronoi2dDot voronoi2dDot) {
		double distance = this.bigNumber;
		JunVoronoi2dDot near = null;

		for (Enumeration e = voronoiDots.elements(); e.hasMoreElements();) {
			JunVoronoi2dDot dot = (JunVoronoi2dDot) e.nextElement();
			double d = voronoi2dDot.distance_(dot);

			if (distance > d) {
				distance = d;
				near = dot;
			}
		}

		return near;
	}

	/**
	 * basic accessing.
	 * 
	 * @return JunVoronoi2dDot[]
	 */
	public JunVoronoi2dDot[] originalDots() {
		if (originalDots == null) {
			return new JunVoronoi2dDot[0];
		}

		return this.originalDots;
	}

	/**
	 * accessor dots.
	 * 
	 * @return Jun2dPoint[]
	 */
	public Jun2dPoint[] points() {
		if (voronoiPoints == null) {
			return new Jun2dPoint[0];
		}

		Vector streamContents = new Vector();

		for (int i = this.startIndex; i < voronoiPoints.size(); i++) {
			Jun2dPoint point = this.restorePoint_((JunVoronoi2dPoint) voronoiPoints.elementAt(i));

			if (point != null) {
				streamContents.addElement(point);
			}
		}

		Jun2dPoint[] anArrayOfJun2dPoint = new Jun2dPoint[streamContents.size()];
		streamContents.copyInto(anArrayOfJun2dPoint);

		return anArrayOfJun2dPoint;
	}

	/**
	 * accessor precision.
	 * 
	 * @return Jun2dPoint[]
	 */
	public String precision() {
		if (processingPrecision > (ACCURACY * 10.0)) {
			return "lowPrecision".intern();
		}

		return "highPrecision".intern();
	}

	/**
	 * Print my string representation on aStream.
	 * 
	 * @param aWriter java.io.OutputStream
	 * 
	 * @throws IOException DOCUMENT ME!
	 */
	public void printOn_(Writer aWriter) throws IOException {
		aWriter.write("diagram (\n");

		Jun2dPoint[] dots = this.dots();

		for (int i = 0; i < dots.length; i++) {
			aWriter.write("\t");
			dots[i].printOn_(aWriter);
			aWriter.write("\n");
		}

		aWriter.write(")\n");
	}

	/**
	 * basic accessing.
	 * 
	 * @return double
	 */
	public double processingPrecision() {
		return this.processingPrecision;
	}

	/**
	 * inserting.
	 * 
	 * @param obsoletePoints
	 * @param examinedSides
	 * @param newSides
	 * 
	 * @return Hashtable[]
	 */
	public Hashtable[] removePoints_andSides_inside_(Hashtable obsoletePoints, Hashtable examinedSides, Vector newSides) {
		Hashtable removedPoints = new Hashtable();
		Hashtable removedSides = new Hashtable();
		JunVoronoi2dSide side = null;

		for (Enumeration e = obsoletePoints.keys(); e.hasMoreElements();) {
			JunVoronoi2dPoint obsoletePoint = (JunVoronoi2dPoint) e.nextElement();
			obsoletePoint.side1_(null);
			obsoletePoint.side2_(null);
			obsoletePoint.side3_(null);

			if (!voronoiPoints.removeElement(obsoletePoint)) {
				removedPoints.put(obsoletePoint, obsoletePoint);
			}
		}

		for (Enumeration e = examinedSides.keys(); e.hasMoreElements();) {
			JunVoronoi2dSide examinedSide = (JunVoronoi2dSide) e.nextElement();

			if ((obsoletePoints.containsKey(examinedSide.from())) || (obsoletePoints.containsKey(examinedSide.to()))) {
				JunVoronoi2dSide targetSide = null;

				for (Enumeration e2 = newSides.elements(); e2.hasMoreElements();) {
					JunVoronoi2dSide newSide = (JunVoronoi2dSide) e2.nextElement();

					if (examinedSide.intersectingPointWithSide_(newSide) != null) {
						targetSide = newSide;

						break;
					}
				}

				if (targetSide == null) {
					JunVoronoi2dPoint point;
					point = voronoiPoints.removeElement(examinedSide.from()) ? examinedSide.from() : null;

					if (point != null) {
						removedPoints.put(point, point);
					}

					point = voronoiPoints.removeElement(examinedSide.to()) ? examinedSide.to() : null;

					if (point != null) {
						removedPoints.put(point, point);
					}

					examinedSide.from_(null);
					examinedSide.to_(null);
					examinedSide.dot1_(null);
					examinedSide.dot2_(null);
					side = null;

					if (voronoiSides.removeElement(examinedSide)) {
						side = examinedSide;
					}

					if (side != null) {
						removedSides.put(examinedSide, examinedSide);
					}
				} else {
					if (examinedSide.isInvalid() && (this.startIndex <= (voronoiSides.indexOf(examinedSide)))) {
						side = null;

						if (voronoiSides.removeElement(examinedSide)) {
							side = examinedSide;
						}

						if (side != null) {
							removedSides.put(examinedSide, examinedSide);
						}
					}
				}
			} else {
				if (examinedSide.isInvalid() && (this.startIndex <= (voronoiSides.indexOf(examinedSide)))) {
					side = null;

					if (voronoiSides.removeElement(examinedSide)) {
						side = examinedSide;
					}

					if (side != null) {
						removedSides.put(examinedSide, examinedSide);
					}
				}
			}
		}

		Hashtable[] returnObject = new Hashtable[2];
		returnObject[0] = removedPoints;
		returnObject[1] = removedSides;

		return returnObject;
	}

	/**
	 * inserting.
	 * 
	 * @param intersectingPoint
	 * @param voronoiDot1
	 * @param voronoiDot2
	 * @param voronoiSide
	 * 
	 * @return JunVoronoi2dPoint
	 */
	public JunVoronoi2dPoint removingPoint_with_with_with_(JunVoronoi2dPoint intersectingPoint, JunVoronoi2dPoint voronoiDot1, JunVoronoi2dDot voronoiDot2, JunVoronoi2dSide voronoiSide) {
		JunVoronoi2dTriangle triangle1 = new JunVoronoi2dTriangle(intersectingPoint, voronoiDot1, voronoiDot2);
		JunVoronoi2dTriangle triangle2 = new JunVoronoi2dTriangle(intersectingPoint, voronoiDot1, voronoiSide.from());
		JunVoronoi2dPoint point = (triangle1.areaWithSign() * triangle2.areaWithSign() > 0) ? voronoiSide.from() : voronoiSide.to();

		return point;
	}

	/**
	 * basic accessing.
	 * 
	 * @param voronoi2dDot
	 * 
	 * @return Jun2dPoint
	 */
	public Jun2dPoint restoreDot_(JunVoronoi2dDot voronoi2dDot) {
		return new Jun2dPoint((voronoi2dDot.x() * scalingFactor.x()) + translationDelta.x(), (voronoi2dDot.y() * scalingFactor.y()) + translationDelta.y());
	}

	/**
	 * basic accessing.
	 * 
	 * @param voronoi2dPoint
	 * 
	 * @return Jun2dPoint
	 */
	public Jun2dPoint restorePoint_(JunVoronoi2dPoint voronoi2dPoint) {
		return new Jun2dPoint((voronoi2dPoint.x() * scalingFactor.x()) + translationDelta.x(), (voronoi2dPoint.y() * scalingFactor.y()) + translationDelta.y());
	}

	/**
	 * basic accessing.
	 * 
	 * @param voronoi2dSide
	 * 
	 * @return Jun2dPoint[]
	 */
	public Jun2dPoint[] restoreSide_(JunVoronoi2dSide voronoi2dSide) {
		Jun2dPoint from = this.restorePoint_(voronoi2dSide.from());
		Jun2dPoint to = this.restorePoint_(voronoi2dSide.to());
		Jun2dPoint[] side = new Jun2dPoint[2];
		side[0] = from;
		side[1] = to;

		return side;
	}

	/**
	 * inserting.
	 * 
	 * @param voronoi2dDot
	 * 
	 * @return Vector
	 */
	public Vector searchSides_(JunVoronoi2dDot voronoi2dDot) {
		Vector sides = new Vector();
		Vector obstacles = new Vector();
		JunVoronoi2dSide side = null;
		int index = 0;

		for (int i = 0; i < voronoiSides.size(); i++) {
			side = (JunVoronoi2dSide) voronoiSides.elementAt(i);

			if ((side.dot1().equals(voronoi2dDot)) || (side.dot2().equals(voronoi2dDot))) {
				if (side.isInvalid() && (this.startIndex <= index)) {
					obstacles.addElement(side);
				} else {
					sides.addElement(side);
				}
			}
		}

		for (Enumeration e = obstacles.elements(); e.hasMoreElements();) {
			voronoiSides.removeElement((JunVoronoi2dSide) e.nextElement());
		}

		return sides;
	}

	/**
	 * accessor dots.
	 * 
	 * @return Jun2dPoint[]
	 */
	public Jun2dPoint[][] sides() {
		if (voronoiSides == null) {
			return new Jun2dPoint[0][2];
		}

		Vector streamContents = new Vector();

		for (int i = this.startIndex; i < voronoiSides.size(); i++) {
			Jun2dPoint[] restoreSide = this.restoreSide_((JunVoronoi2dSide) voronoiSides.elementAt(i));

			if ((restoreSide != null) && (restoreSide[0] != null) && (restoreSide[1] != null)) {
				streamContents.addElement(restoreSide);
			}
		}

		Jun2dPoint[][] anArrayOfJun2dPoint = new Jun2dPoint[streamContents.size()][2];
		streamContents.copyInto(anArrayOfJun2dPoint);

		return anArrayOfJun2dPoint;
	}

	/**
	 * basic accessing.
	 * 
	 * @return int
	 */
	public final int startIndex() {
		return this.startIndex;
	}

	/**
	 * tracing.
	 * 
	 * @return boolean
	 */
	public boolean trace() {
		return traceBoolean == true;
	}

	/**
	 * tracing.
	 * 
	 * @param aBoolean
	 */
	public void trace_(boolean aBoolean) {
		traceBoolean = aBoolean == true;
	}

	/**
	 * tracing.
	 * 
	 * @param gc
	 */
	public void traceDisplayOn_(Graphics gc) {
		try {
			traceDisplayInitialize();

			final Rectangle bounds = new Rectangle(0, 0, graphicsContextImage.getWidth(null), graphicsContextImage.getHeight(null));
			Point origin = new Point(bounds.width / 2, bounds.height / 2);
			final Point factor = new Point(200, 200);
			origin.x -= (0.5 * factor.x);
			origin.y -= (0.5 * factor.y);
			gc.setColor(Color.white);
			gc.fillRect(bounds.x, bounds.y, bounds.width + 1, bounds.height + 1);
			gc.setColor(Color.red);

			//graphicsContext.lineWidth(1);
			for (Enumeration e = voronoiDots.elements(); e.hasMoreElements();) {
				JunVoronoi2dDot dot = (JunVoronoi2dDot) e.nextElement();
				Rectangle box = new Rectangle(0, 0, 1, 1);
				box.x = (int) Math.round((dot.x() * factor.x) + origin.x);
				box.y = (int) Math.round((dot.y() * factor.y) + origin.y);
				box.x -= 2;
				box.y -= 2;
				box.width += (2 + 1);
				box.height += (2 + 1);
				this.displayBoxOn_box_clip_(gc, box, bounds);
			}

			for (Enumeration e = voronoiPoints.elements(); e.hasMoreElements();) {
				JunVoronoi2dPoint point = (JunVoronoi2dPoint) e.nextElement();
				Rectangle box = new Rectangle(0, 0, 1, 1);
				box.x = (int) Math.round((point.x() * factor.x) + origin.x);
				box.y = (int) Math.round((point.y() * factor.y) + origin.y);

				//box := box expandedBy: 3.
				gc.setColor(Color.green);
				box.x -= 3;
				box.y -= 3;
				box.width += (3 + 1);
				box.height += (3 + 1);
				this.displayBoxOn_box_clip_(gc, box, bounds);

				Vector sides = new Vector();

				if (point.side1() != null) {
					sides.addElement(point.side1());
				}

				if (point.side2() != null) {
					sides.addElement(point.side2());
				}

				if (point.side3() != null) {
					sides.addElement(point.side3());
				}

				gc.setColor(Color.blue);

				for (Enumeration e2 = sides.elements(); e2.hasMoreElements();) {
					JunVoronoi2dSide side = (JunVoronoi2dSide) e2.nextElement();

					if ((side.from() == null) || (side.to() == null)) {
						//continue;
					}

					int x = (int) Math.round((side.from().x() * factor.x) + origin.x);
					int y = (int) Math.round((side.from().y() * factor.y) + origin.y);
					Point point1 = new Point(x, y);
					x = (int) Math.round((side.to().x() * factor.x) + origin.x);
					y = (int) Math.round((side.to().y() * factor.y) + origin.y);

					Point point2 = new Point(x, y);
					this.displayLineOn_from_to_clip_width_(gc, point1, point2, bounds, 3);
				}
			}

			gc.setColor(Color.cyan);

			//gc lineWidth: 1.
			for (Enumeration e = voronoiSides.elements(); e.hasMoreElements();) {
				JunVoronoi2dSide side = (JunVoronoi2dSide) e.nextElement();
				int x = (int) Math.round((side.from().x() * factor.x) + origin.x);
				int y = (int) Math.round((side.from().y() * factor.y) + origin.y);
				Point point1 = new Point(x, y);
				x = (int) Math.round((side.to().x() * factor.x) + origin.x);
				y = (int) Math.round((side.to().y() * factor.y) + origin.y);

				Point point2 = new Point(x, y);
				this.displayLineOn_from_to_clip_(gc, point1, point2, bounds);
			}

			gc.setColor(Color.black);

			//gc lineWidth: 1.
			Rectangle rectangle = new Rectangle(0, 0, 1, 1);
			rectangle.x *= factor.x;
			rectangle.y *= factor.y;
			rectangle.width *= factor.x;
			rectangle.height *= factor.y;
			rectangle.x += origin.x;
			rectangle.y += origin.y;
			gc.drawRect(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
			gcMediumCanvas.getGraphics().drawImage(graphicsContextImage, 0, 0, null);
		} catch (Exception e) {
			this.debug_("error in trace display");
		}
	}

	/**
	 * basic accessing.
	 * 
	 * @param voronoi2dPoint
	 * 
	 * @return JunVoronoi2dTriangle
	 */
	public JunVoronoi2dTriangle triangleAtPoint_(JunVoronoi2dPoint voronoi2dPoint) {
		JunVoronoi2dDot[] array = this.dotsAtPoint_(voronoi2dPoint);
		int index1 = voronoiDots.indexOf(array[1 - 1]);
		int index3 = voronoiDots.indexOf(array[2 - 1]);
		int index5 = voronoiDots.indexOf(array[3 - 1]);

		if ((index1 < this.startIndex) || (index3 < this.startIndex) || (index5 < this.startIndex)) {
			return null;
		}

		JunVoronoi2dTriangle triangle = new JunVoronoi2dTriangle((JunVoronoi2dDot) voronoiDots.elementAt(index1), (JunVoronoi2dDot) voronoiDots.elementAt(index3), (JunVoronoi2dDot) voronoiDots.elementAt(index5));

		return triangle;
	}

	/**
	 * basic accessing.
	 * 
	 * @param voronoi2dPoint
	 * 
	 * @return int[]
	 */
	public int[] triangleIndexesAtPoint_(JunVoronoi2dPoint voronoi2dPoint) {
		JunVoronoi2dDot[] array = this.dotsAtPoint_(voronoi2dPoint);
		int index1 = voronoiDots.indexOf(array[1 - 1]);
		int index3 = voronoiDots.indexOf(array[2 - 1]);
		int index5 = voronoiDots.indexOf(array[3 - 1]);

		if ((index1 < this.startIndex) || (index3 < this.startIndex) || (index5 < this.startIndex)) {
			return null;
		}

		int[] indexes = new int[3];
		indexes[0] = (index1 - this.startIndex + 1) - 1;
		indexes[1] = (index3 - this.startIndex + 1) - 1;
		indexes[2] = (index5 - this.startIndex + 1) - 1;

		return indexes;
	}

	/**
	 * accessor triangles.
	 * 
	 * @return Jun2dPoint[][]
	 */
	public Jun2dPoint[][] triangles() {
		if (normalizedDots == null) {
			return new Jun2dPoint[0][3];
		}

		Vector streamContents = new Vector();

		for (int i = this.startIndex; i < voronoiPoints.size(); i++) {
			JunVoronoi2dPoint point = (JunVoronoi2dPoint) voronoiPoints.elementAt(i);
			int[] indexes = this.triangleIndexesAtPoint_(point);

			if (indexes != null) {
				JunVoronoi2dDot p1 = this.originalDots[indexes[1 - 1]];
				JunVoronoi2dDot p2 = this.originalDots[indexes[2 - 1]];
				JunVoronoi2dDot p3 = this.originalDots[indexes[3 - 1]];

				if ((p1 != null) && (p2 != null) && (p3 != null)) {
					Jun2dPoint[] contents = new Jun2dPoint[3];
					contents[0] = new Jun2dPoint(p1.x(), p1.y());
					contents[1] = new Jun2dPoint(p2.x(), p2.y());
					contents[2] = new Jun2dPoint(p3.x(), p3.y());
					streamContents.addElement(contents);
				}
			}
		}

		Jun2dPoint[][] anArrayOfJun2dPoint = new Jun2dPoint[streamContents.size()][3];
		streamContents.copyInto(anArrayOfJun2dPoint);

		return anArrayOfJun2dPoint;
	}

	/**
	 * basic accessing.
	 * 
	 * @return Vector
	 */
	public Vector voronoiDots() {
		return this.voronoiDots;
	}

	/**
	 * basic accessing.
	 * 
	 * @return Vector
	 */
	public Vector voronoiPoints() {
		return this.voronoiPoints;
	}

	/**
	 * basic accessing.
	 * 
	 * @return Vector
	 */
	public Vector voronoiSides() {
		return this.voronoiSides;
	}

	/**
	 * convert vector to array
	 * 
	 * @param vector DOCUMENT ME!
	 * 
	 * @return JunVoronoi2dDot[]
	 */
	private JunVoronoi2dDot[] asArrayFromVectorToJunVoronoi2dDot(Vector vector) {
		JunVoronoi2dDot[] anArrayOfJunVoronoi2dDot = new JunVoronoi2dDot[vector.size()];
		vector.copyInto(anArrayOfJunVoronoi2dDot);

		return anArrayOfJunVoronoi2dDot;
	}

	/**
	 * for Vector of sra.jun.geometry.basic.Jun2dPoint.
	 * 
	 * @param vector
	 * 
	 * @return Vector
	 */
	private Vector dotsFromJun2dPoints(Vector vector) {
		Vector dots = new Vector(vector.size());

		for (Enumeration enum = vector.elements(); enum.hasMoreElements();) {
			Jun2dPoint dot = (Jun2dPoint) enum.nextElement();
			Jun2dPoint it = null;

			for (Enumeration enumInside = dots.elements(); enumInside.hasMoreElements();) {
				JunVoronoi2dDot each = (JunVoronoi2dDot) enumInside.nextElement();

				if ((Math.abs(each.x() - dot.x()) < ACCURACY) && (Math.abs(each.y() - dot.y()) < ACCURACY)) {
					it = dot;

					break;
				}
			}

			if (it == null) {
				dots.addElement(new JunVoronoi2dDot(dot.x(), dot.y()));
			}
		}

		return dots;
	}

	/**
	 * for Vector of java.awt.Point.
	 * 
	 * @param vector
	 * 
	 * @return Vector
	 */
	private Vector dotsFromPoints(Vector vector) {
		Vector dots = new Vector(vector.size());

		for (Enumeration enum = vector.elements(); enum.hasMoreElements();) {
			Point dot = (Point) enum.nextElement();
			Point it = null;

			for (Enumeration enumInside = dots.elements(); enumInside.hasMoreElements();) {
				JunVoronoi2dDot each = (JunVoronoi2dDot) enumInside.nextElement();

				if ((Math.abs(each.x() - dot.x) < ACCURACY) && (Math.abs(each.y() - dot.y) < ACCURACY)) {
					it = dot;

					break;
				}
			}

			if (it == null) {
				dots.addElement(new JunVoronoi2dDot(dot.x, dot.y));
			}
		}

		return dots;
	}

	/**
	 * privates.
	 * 
	 * @param dotCollection
	 * 
	 * @return JunVoronoi2dDot[]
	 */
	private JunVoronoi2dDot[] normalizedDots_(Vector dotCollection) {
		JunVoronoi2dDot[] dots = this.asArrayFromVectorToJunVoronoi2dDot(dotCollection);
		double minX = dots[0].x();
		double minY = dots[0].y();
		double maxX = minX;
		double maxY = minY;

		for (int i = 0; i < dots.length; i++) {
			JunVoronoi2dDot dot = dots[i];
			minX = Math.min(dot.x(), minX);
			minY = Math.min(dot.y(), minY);
			maxX = Math.max(dot.x(), maxX);
			maxY = Math.max(dot.y(), maxY);
		}

		if ((0.0d <= minX) && (maxX <= 1.0d) && (0.0d <= minY) && (maxY <= 1.0d)) {
			translationDelta = new JunVoronoi2dDot(0.0d, 0.0d);
			scalingFactor = new JunVoronoi2dDot(1.0d, 1.0d);

			return dots;
		}

		JunVoronoi2dDot delta = new JunVoronoi2dDot(minX, minY);

		for (int i = 0; i < dots.length; i++) {
			dots[i] = new JunVoronoi2dDot(dots[i].x() - delta.x(), dots[i].y() - delta.y());
		}

		final double width = Math.max(maxX - minX, ACCURACY);
		final double height = Math.max(maxY - minY, ACCURACY);
		final double factor = Math.max(width, height);
		final JunVoronoi2dDot factorDot = new JunVoronoi2dDot(factor, factor);

		for (int i = 0; i < dots.length; i++) {
			dots[i] = new JunVoronoi2dDot(dots[i].x() / factorDot.x(), dots[i].y() / factorDot.y());
		}

		translationDelta = delta;
		scalingFactor = factorDot;

		return dots;
	}

	/**
	 * displaying. make polygon by line (with width 1).
	 * 
	 * @param fromPoint
	 * @param toPoint
	 * @param lineWidth
	 * 
	 * @return java.awt.Polygon
	 */
	private Polygon polygonByWidth(Point fromPoint, Point toPoint, int lineWidth) {
		final double dx = fromPoint.x - toPoint.x;
		final double dy = fromPoint.y - toPoint.y;
		double distance = Math.sqrt((dx * dx) + (dy * dy));

		if (distance == 0.0) {
			distance = 1.0;
		}

		final double dxByWidth = (-dy / distance * lineWidth) / 2.0;
		final double dyByWidth = (dx / distance * lineWidth) / 2.0;
		final int[] xpoints = new int[5];
		final int[] ypoints = new int[5];
		xpoints[0] = (int) Math.ceil((double) fromPoint.x - dxByWidth);
		ypoints[0] = (int) Math.ceil((double) fromPoint.y - dyByWidth);
		xpoints[1] = (int) Math.ceil((double) fromPoint.x + dxByWidth);
		ypoints[1] = (int) Math.ceil((double) fromPoint.y + dyByWidth);
		xpoints[2] = (int) Math.ceil((double) toPoint.x + dxByWidth);
		ypoints[2] = (int) Math.ceil((double) toPoint.y + dyByWidth);
		xpoints[3] = (int) Math.ceil((double) toPoint.x - dxByWidth);
		ypoints[3] = (int) Math.ceil((double) toPoint.y - dyByWidth);
		xpoints[4] = xpoints[0];
		ypoints[4] = ypoints[0];

		return new Polygon(xpoints, ypoints, xpoints.length);
	}

	/**
	 * tracing.
	 * 
	 * @return DOCUMENT ME!
	 */
	private Graphics traceDisplayInitialize() {
		if (graphicsContext == null) {
			final Dimension dimension = new Dimension(400, 400);
			graphicsContextStImage = new StImage(400, 400);
			graphicsContextImage = graphicsContextStImage.image();
			graphicsContext = graphicsContextImage.getGraphics();
			gcMediumCanvas = new Canvas() {
				public void paint(Graphics g) {
					g.drawImage(graphicsContextImage, 0, 0, null);

					return;
				}
			};

			gcMediumCanvas.setBounds(0, 0, graphicsContextStImage.width(), graphicsContextStImage.height());
		}

		return graphicsContext;
	}
}
