package sra.jun.voronoi.twoD.diagram;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import sra.jun.geometry.basic.*;
import sra.jun.system.support.JunTestExamples;
import sra.smalltalk.*;

/**
 * JunVoronoi2dProcessorTestExamples class
 * 
 *  @author    NISHIHARA Satoshi
 *  @created   2000/01/6 (by NISHIHARA Satoshi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVoronoi2dProcessorTestExamples.java,v 6.0 2003/10/28 10:52:27 nisinaka Exp $
 */
public class JunVoronoi2dProcessorTestExamples extends JunTestExamples {

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example1() {
		//area: 0.9d
		Vector dots = new Vector();
		dots.addElement(new Jun2dPoint(0.0d, 0.0d));
		dots.addElement(new Jun2dPoint(0.1d, 1.0d));
		dots.addElement(new Jun2dPoint(1.0d, 0.1d));
		dots.addElement(new Jun2dPoint(1.0d, 1.0d));

		JunVoronoi2dProcessor voronoi = JunVoronoi2dProcessor.Dots_(dots);
		voronoi.trace_(true);
		JunVoronoi2dProcessorTestExamples.MakeTraceFrame(voronoi.canvas());
		voronoi.compute();
		System.out.println("area: " + voronoi.area());

		//return voronoi;
		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example2() {
		//area: 13.842356201099d
		Vector dots = new Vector();
		dots.addElement(new Jun2dPoint(-1.0709950047146d, -1.0d));
		dots.addElement(new Jun2dPoint(-1.3616787517522d, -0.55623057513983d));
		dots.addElement(new Jun2dPoint(-1.4749357023933d, 1.1021456892432e-16d));
		dots.addElement(new Jun2dPoint(-1.3616787517522d, 0.55623057513983d));
		dots.addElement(new Jun2dPoint(-1.0329942792654d, 1.0580134260987d));
		dots.addElement(new Jun2dPoint(-0.83909963117728d, 1.2088366880644d));
		dots.addElement(new Jun2dPoint(-0.3639702342662d, 1.4500662049207d));
		dots.addElement(new Jun2dPoint(-0.3290104887271d, 1.456230551298d));
		dots.addElement(new Jun2dPoint(0.0d, 1.4857898683076d));
		dots.addElement(new Jun2dPoint(0.3290104887271d, 1.456230551298d));
		dots.addElement(new Jun2dPoint(0.3639702342662d, 1.4500662049207d));
		dots.addElement(new Jun2dPoint(0.83909963117728d, 1.2088366880644d));
		dots.addElement(new Jun2dPoint(1.0d, 1.0836784193003d));
		dots.addElement(new Jun2dPoint(1.0d, 3.0d));
		dots.addElement(new Jun2dPoint(-3.0d, 3.0d));
		dots.addElement(new Jun2dPoint(-3.0d, -1.0d));

		JunVoronoi2dProcessor voronoi = JunVoronoi2dProcessor.Dots_(dots);
		voronoi.trace_(true);
		JunVoronoi2dProcessorTestExamples.MakeTraceFrame(voronoi.canvas());
		voronoi.compute();
		System.out.println("area: " + voronoi.area());

		Graphics gc = null;
		StImage anImage = new StImage(300, 300);

		try {
			Jun2dPoint[][] triangles = voronoi.triangles();
			gc = anImage.image().getGraphics();

			if (gc instanceof Graphics2D) {
				((Graphics2D) gc).setBackground(Color.white);
			}

			gc.setColor(Color.white);
			gc.fillRect(0, 0, anImage.width(), anImage.height());

			Point scale = new Point(50, 50);
			Point translation = new Point(200, 100);
			Rectangle box = new Rectangle(new Point(-3, -1), new Dimension(4, 4));
			box.x *= scale.x;
			box.y *= scale.y;
			box.width *= scale.x;
			box.height *= scale.y;
			box.x += translation.x;
			box.y += translation.y;
			gc.setColor(Color.red);
			gc.drawRect(box.x, box.y, box.width, box.height);

			for (int i = 0; i < triangles.length; i++) {
				Jun2dPoint[] points = triangles[i];
				int[] xPoints = new int[points.length + 1];
				int[] yPoints = new int[points.length + 1];

				for (int j = 0; j < points.length; j++) {
					Jun2dPoint point = points[j];

					if (point != null) {
						xPoints[j] = (int) (point.x() * scale.x + translation.x);
						yPoints[j] = (int) (point.y() * scale.y + translation.y);
					}
				}

				xPoints[points.length] = xPoints[0];
				yPoints[points.length] = yPoints[0];
				gc.setColor(Color.black);
				gc.drawPolyline(xPoints, yPoints, xPoints.length);
			}

			anImage._display();
		} finally {
			gc.dispose();
		}

		//return voronoi;
		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example3() {
		Vector dots = new Vector();
		Random randomStream = new Random();

		for (int i = 0; i < 50; i++) {
			dots.addElement(new Jun2dPoint(randomStream.nextDouble(), randomStream.nextDouble()));
		}

		JunVoronoi2dProcessor voronoi = JunVoronoi2dProcessor.Dots_(dots);
		voronoi.trace_(true);
		JunVoronoi2dProcessorTestExamples.MakeTraceFrame(voronoi.canvas());
		voronoi.compute();
		System.out.println("area: " + voronoi.area());

		Graphics gc = null;
		StImage anImage = new StImage(300, 300);

		try {
			Jun2dPoint[][] triangles = voronoi.triangles();
			gc = anImage.image().getGraphics();

			if (gc instanceof Graphics2D) {
				((Graphics2D) gc).setBackground(Color.white);
			}

			gc.setColor(Color.white);
			gc.fillRect(0, 0, anImage.width(), anImage.height());

			Point scale = new Point(200, 200);
			Point translation = new Point(50, 50);
			Rectangle box = new Rectangle(new Point(0, 0), new Dimension(1, 1));
			box.x *= scale.x;
			box.y *= scale.y;
			box.width *= scale.x;
			box.height *= scale.y;
			box.x += translation.x;
			box.y += translation.y;
			gc.setColor(Color.red);
			gc.drawRect(box.x, box.y, box.width, box.height);

			for (int i = 0; i < triangles.length; i++) {
				Jun2dPoint[] points = triangles[i];
				int[] xPoints = new int[points.length + 1];
				int[] yPoints = new int[points.length + 1];

				for (int j = 0; j < points.length; j++) {
					Jun2dPoint point = points[j];

					if (point != null) {
						xPoints[j] = (int) (point.x() * scale.x + translation.x);
						yPoints[j] = (int) (point.y() * scale.y + translation.y);
					}
				}

				xPoints[points.length] = xPoints[0];
				yPoints[points.length] = yPoints[0];
				gc.setColor(Color.black);
				gc.drawPolyline(xPoints, yPoints, xPoints.length);
			}

			anImage._display();
		} finally {
			gc.dispose();
		}

		//return voronoi;
		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example4() {
		//area: 3.1256671980047d
		final Vector dots = new Vector();
		dots.addElement(new Jun2dPoint(0.0d, 0.0d));

		StInterval interval = StInterval.From_to_by_(0, 360, 10);
		interval.do_(new StBlockClosure() {
			public Object value_(Object valueObject) {
				double degrees = ((Double) valueObject).doubleValue();
				final double x = Math.cos(JunAngle._DegreesToRadians(degrees));
				final double y = Math.sin(JunAngle._DegreesToRadians(degrees));
				dots.addElement(new Jun2dPoint(x, y));

				return null;
			}
		});

		JunVoronoi2dProcessor voronoi = JunVoronoi2dProcessor.Dots_(dots);
		voronoi.trace_(true);
		JunVoronoi2dProcessorTestExamples.MakeTraceFrame(voronoi.canvas());
		voronoi.compute();
		System.out.println("area: " + voronoi.area());

		Graphics gc = null;
		StImage anImage = new StImage(300, 300);

		try {
			Jun2dPoint[][] triangles = voronoi.triangles();
			gc = anImage.image().getGraphics();

			if (gc instanceof Graphics2D) {
				((Graphics2D) gc).setBackground(Color.white);
			}

			gc.setColor(Color.white);
			gc.fillRect(0, 0, anImage.width(), anImage.height());

			Point scale = new Point(100, 100);
			Point translation = new Point(150, 150);
			Point origin = new Point(-1, -1);
			Rectangle box = new Rectangle(origin, new Dimension(1 - origin.x, 1 - origin.y));
			box.x *= scale.x;
			box.y *= scale.y;
			box.width *= scale.x;
			box.height *= scale.y;
			box.x += translation.x;
			box.y += translation.y;
			gc.setColor(Color.red);
			gc.drawRect(box.x, box.y, box.width, box.height);

			for (int i = 0; i < triangles.length; i++) {
				Jun2dPoint[] points = triangles[i];
				int[] xPoints = new int[points.length + 1];
				int[] yPoints = new int[points.length + 1];

				for (int j = 0; j < points.length; j++) {
					Jun2dPoint point = points[j];

					if (point != null) {
						xPoints[j] = (int) (point.x() * scale.x + translation.x);
						yPoints[j] = (int) (point.y() * scale.y + translation.y);
					}
				}

				xPoints[points.length] = xPoints[0];
				yPoints[points.length] = yPoints[0];
				gc.setColor(Color.black);
				gc.drawPolyline(xPoints, yPoints, xPoints.length);
			}

			anImage._display();
		} finally {
			gc.dispose();
		}

		//return voronoi;
		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example5() {
		//area: 0.96987088262947d
		String string =
			"0.20254893600941d 0.30189919471741d"
				+ "0.90037482976913d 0.97756767272949d"
				+ "0.37021240592003d 0.7721865773201d"
				+ "0.73451060056686d 0.66534775495529d"
				+ "0.83000415563583d 0.35092878341675d"
				+ "0.20771345496178d 0.23709288239479d"
				+ "0.13294461369514d 0.44628903269768d"
				+ "0.585289478302d 0.81525200605392d"
				+ "0.25862556695938d 0.89500206708908d"
				+ "0.56111621856689d 0.95084547996521d"
				+ "0.61521035432816d 0.96237403154373d"
				+ "0.25764265656471d 0.62346524000168d"
				+ "0.039433568716049d 0.34228238463402d"
				+ "0.22588920593262d 0.24127447605133d"
				+ "0.79660141468048d 0.157176181674d"
				+ "0.445447742939d 0.64100790023804d"
				+ "0.58059144020081d 0.95807582139969d"
				+ "0.32448145747185d 0.087971679866314d"
				+ "0.51385253667831d 0.72457307577133d"
				+ "0.69972509145737d 0.27604332566261d"
				+ "0.14740525186062d 0.86483132839203d"
				+ "0.83648478984833d 0.32767179608345d"
				+ "0.46084132790565d 0.21558517217636d"
				+ "0.42863807082176d 0.79387754201889d"
				+ "0.86232405900955d 0.042140774428844d"
				+ "0.59863388538361d 0.654252409935d"
				+ "0.18858808279037d 0.038375675678253d"
				+ "0.89749270677567d 0.31695958971977d"
				+ "0.70493960380554d 0.32673886418343d"
				+ "0.30480632185936d 0.61873388290405d"
				+ "0.10525614023209d 0.45825073122978d"
				+ "0.22873802483082d 0.82488131523132d"
				+ "0.51198667287827d 0.41250312328339d"
				+ "0.50602251291275d 0.62927949428558d"
				+ "0.47615161538124d 0.59765928983688d"
				+ "0.40196585655212d 0.15437734127045d"
				+ "0.97734278440475d 0.85045397281647d"
				+ "0.61044561862946d 0.95119535923004d"
				+ "0.42372345924377d 0.43619325757027d"
				+ "0.25391086935997d 0.99932527542114d"
				+ "0.65202832221985d 0.048754684627056d"
				+ "0.88338190317154d 0.70693045854568d"
				+ "0.92756348848343d 0.81058722734451d"
				+ "0.47845065593719d 0.91074550151825d"
				+ "0.94420659542084d 0.27271136641502d"
				+ "0.44728028774261d 0.87607663869858d"
				+ "0.82440650463104d 0.41471886634827d"
				+ "0.62660557031631d 0.29680132865906d"
				+ "0.11918367445469d 0.64477300643921d"
				+ "0.28173261880875d 0.29536858201027d"
				+ "0.808129966259d 0.7996084690094d"
				+ "0.10653894394636d 0.4227988421917d"
				+ "0.29940858483315d 0.1445314437151d"
				+ "0.22347354888916d 0.65868389606476d"
				+ "0.42975425720215d 0.37341940402985d"
				+ "0.18355685472488d 0.4111869931221d"
				+ "0.46447312831879d 0.60872137546539d"
				+ "0.96638065576553d 0.51704287528992d"
				+ "0.097442731261253d 0.40145772695541d"
				+ "0.9801082611084d 0.24155768752098d"
				+ "0.45111203193665d 0.73122030496597d"
				+ "0.06147438660264d 0.27860891819d"
				+ "0.076501458883286d 0.0061724283732474d"
				+ "0.47578507661819d 0.75064557790756d"
				+ "0.95320284366608d 0.063048727810383d"
				+ "0.91691797971725d 0.2086880505085d"
				+ "0.48937943577766d 0.16715535521507d"
				+ "0.5073219537735d 0.6323282122612d"
				+ "0.52176594734192d 0.012369845993817d"
				+ "0.79801750183105d 0.42972928285599d"
				+ "0.31566846370697d 0.7211412191391d"
				+ "0.76859641075134d 0.43762597441673d"
				+ "0.56496459245682d 0.84448981285095d"
				+ "0.82722198963165d 0.92132443189621d"
				+ "0.3921032845974d 0.36200749874115d"
				+ "0.81062889099121d 0.57470220327377d"
				+ "0.34810495376587d 0.68185758590698d"
				+ "0.98412328958511d 0.52020823955536d"
				+ "0.4125614464283d 0.64077466726303d"
				+ "0.041582673788071d 0.30886298418045d"
				+ "0.99363601207733d 0.50406497716904d"
				+ "0.10545606166124d 0.92025822401047d"
				+ "0.92806333303452d 0.96560597419739d"
				+ "0.72676384449005d 0.7625572681427d"
				+ "0.4811495244503d 0.14784672856331d"
				+ "0.88509786128998d 0.67249476909637d"
				+ "0.34677216410637d 0.60180759429932d"
				+ "0.98862141370773d 0.91537690162659d"
				+ "0.64738023281097d 0.30708038806915d"
				+ "0.87406915426254d 0.18508121371269d"
				+ "0.9339941740036d 0.67182838916779d"
				+ "0.80674719810486d 0.60405665636063d"
				+ "0.18620574474335d 0.53278630971909d"
				+ "0.48053312301636d 0.72332364320755d"
				+ "0.8121782541275d 0.15526030957699d"
				+ "0.017875885590911d 0.52247399091721d"
				+ "0.64864641427994d 0.23312786221504d"
				+ "0.96979594230652d 0.40967097878456d"
				+ "0.96091628074646d 0.88883799314499d"
				+ "0.31588503718376d 0.22164931893349d"
				+ "0.44286546111107d 0.67341107130051d"
				+ "0.46430653333664d 0.22371511161327d"
				+ "0.21694293618202d 0.56643897294998d"
				+ "0.25179508328438d 0.10974594205618d"
				+ "0.83416908979416d 0.97608494758606d"
				+ "0.94365680217743d 0.0021907538175583d"
				+ "0.27413576841354d 0.73908370733261d"
				+ "0.23376926779747d 0.45206996798515d"
				+ "0.94500625133514d 0.12074135988951d"
				+ "0.24458143115044d 0.43897542357445d"
				+ "0.68351519107819d 0.81493544578552d"
				+ "0.52711367607117d 0.37107038497925d"
				+ "0.75496876239777d 0.94411492347717d"
				+ "0.060957934707403d 0.085089549422264d"
				+ "0.85324448347092d 0.059300292283297d"
				+ "0.25427737832069d 0.84633904695511d"
				+ "0.1007913351059d 0.14008329808712d"
				+ "0.94380676746368d 0.34869638085365d"
				+ "0.048629738390446d 0.5946272611618d"
				+ "0.39485213160515d 0.71461057662964d"
				+ "0.67635154724121d 0.25966680049896d"
				+ "0.30129113793373d 0.49510204792023d"
				+ "0.39211994409561d 0.40050813555717d"
				+ "0.78557270765305d 0.66976261138916d"
				+ "0.032669719308615d 0.71102875471115d"
				+ "0.39871719479561d 0.64675551652908d"
				+ "0.8633069396019d 0.31367763876915d"
				+ "0.12031653523445d 0.26281550526619d"
				+ "0.57790923118591d 0.75947523117065d"
				+ "0.3585339486599d 0.78324866294861d"
				+ "0.29892545938492d 0.02801332809031d"
				+ "0.95010411739349d 0.90193253755569d"
				+ "0.5773760676384d 0.52745521068573d"
				+ "0.16033320128918d 0.74131613969803d"
				+ "0.39285299181938d 0.094535611569881d"
				+ "0.68309873342514d 0.85241985321045d"
				+ "0.15351936221123d 0.99456059932709d"
				+ "0.64084964990616d 0.21483547985554d"
				+ "0.69610995054245d 0.92140775918961d"
				+ "0.58460640907288d 0.23672635853291d"
				+ "0.2859308719635d 0.99752604961395d"
				+ "0.49396085739136d 0.75482714176178d"
				+ "0.61685961484909d 0.77393585443497d"
				+ "0.77707624435425d 0.034443981945515d"
				+ "0.81134527921677d 0.2302290648222d"
				+ "0.27068722248077d 0.76945441961288d"
				+ "0.4204081594944d 0.77456891536713d"
				+ "0.24009995162487d 0.082307368516922d"
				+ "0.4236401617527d 0.24369013309479d"
				+ "0.37919199466705d 0.52402329444885d"
				+ "0.22920449078083d 0.90289878845215d"
				+ "0.81041234731674d 0.074194088578224d"
				+ "0.67383587360382d 0.44607245922089d"
				+ "0.084781341254711d 0.14098292589188d"
				+ "0.022840483114123d 0.99566012620926d"
				+ "0.18189087510109d 0.56112450361252d"
				+ "0.97009581327438d 0.10268221795559d"
				+ "0.5099042057991d 0.59992504119873d"
				+ "0.63805079460144d 0.74673050642014d"
				+ "0.90555602312088d 0.9512619972229d"
				+ "0.57772594690323d 0.33596834540367d"
				+ "0.63415241241455d 0.73758435249329d"
				+ "0.76871305704117d 0.70713037252426d"
				+ "0.38957101106644d 0.50991255044937d"
				+ "0.61917531490326d 0.12552270293236d"
				+ "0.29426071047783d 0.2478384077549d"
				+ "0.96584755182266d 0.28502291440964d"
				+ "0.89924198389053d 0.35952520370483d"
				+ "0.074035815894604d 0.30807995796204d"
				+ "0.18410661816597d 0.68170762062073d"
				+ "0.63761764764786d 0.74571430683136d"
				+ "0.55701792240143d 0.47969180345535d"
				+ "0.77904206514359d 0.57751768827438d"
				+ "0.85471051931381d 0.44735527038574d"
				+ "0.049329444766045d 0.21165347099304d"
				+ "0.34249064326286d 0.70714700222015d"
				+ "0.42807164788246d 0.48485630750656d"
				+ "0.71423572301865d 0.81008744239807d"
				+ "0.32343190908432d 0.66243231296539d"
				+ "0.092394836246967d 0.73576843738556d"
				+ "0.57214492559433d 0.43825906515121d"
				+ "0.027988338842988d 0.89235317707062d"
				+ "0.43951687216759d 0.93478548526764d"
				+ "0.500608086586d 0.11657642573118d"
				+ "0.6194252371788d 0.7030320763588d"
				+ "0.91841733455658d 0.6737442612648d"
				+ "0.23431903123856d 0.72259056568146d"
				+ "0.11815077066422d 0.25773426890373d"
				+ "0.83521866798401d 0.40162432193756d"
				+ "0.36511453986168d 0.99099540710449d"
				+ "0.4017159640789d 0.57686799764633d"
				+ "0.353186160326d 0.42454811930656d"
				+ "0.34197416901588d 0.51362764835358d"
				+ "0.20481465756893d 0.53798419237137d"
				+ "0.49272802472115d 0.90578091144562d"
				+ "0.47102040052414d 0.73946690559387d"
				+ "0.11928363144398d 0.87577676773071d"
				+ "0.13139525055885d 0.86573094129562d"
				+ "0.91551852226257d 0.97463554143906d"
				+ "0.59410244226456d 0.18208245933056d"
				+ "0.0038817159365863d 0.18194918334484d"
				+ "0.69587671756744d 0.38239899277687d"
				+ "0.93539357185364d 0.90588086843491d"
				+ "0.70202416181564d 0.58912950754166d"
				+ "0.68964600563049d 0.98316532373428d"
				+ "0.30642232298851d 0.35329446196556d"
				+ "0.67480218410492d 0.67910867929459d"
				+ "0.63152021169662d 0.65448564291d"
				+ "0.72759681940079d 0.68758851289749d"
				+ "0.22833819687366d 0.90086632966995d"
				+ "0.11333610862494d 0.13105373084545d"
				+ "0.076468139886856d 0.92917120456696d"
				+ "0.52589756250381d 0.56052476167679d"
				+ "0.58407330513d 0.0047063725069165d"
				+ "0.08773010969162d 0.95559352636337d"
				+ "0.5878883600235d 0.82134944200516d"
				+ "0.34985423088074d 0.72442317008972d"
				+ "0.35321947932243d 0.50154936313629d"
				+ "0.29186171293259d 0.70374846458435d"
				+ "0.57394421100616d 0.59632652997971d"
				+ "0.32191586494446d 0.15887546539307d"
				+ "0.37251144647598d 0.085272803902626d"
				+ "0.27675133943558d 0.78368180990219d"
				+ "0.29994168877602d 0.37655144929886d"
				+ "0.42167431116104d 0.70061641931534d"
				+ "0.33582672476768d 0.30689713358879d"
				+ "0.45056226849556d 0.46069970726967d"
				+ "0.88832986354828d 0.14161598682404d"
				+ "0.48586422204971d 0.043523531407118d"
				+ "0.79418575763702d 0.57458561658859d"
				+ "0.078600585460663d 0.85725116729736d"
				+ "0.31870055198669d 0.72825491428375d"
				+ "0.20836317539215d 0.7386172413826d"
				+ "0.15575176477432d 0.15364430844784d"
				+ "0.28331527113914d 0.95292794704437d"
				+ "0.42778840661049d 0.8303456902504d"
				+ "0.14019158482552d 0.19406080245972d"
				+ "0.68583089113235d 0.16652227938175d"
				+ "0.044298209249973d 0.58446478843689d"
				+ "0.90947103500366d 0.9989088177681d"
				+ "0.68951272964478d 0.67516034841537d"
				+ "0.50687211751938d 0.59281134605408d"
				+ "0.19828404486179d 0.44573926925659d"
				+ "0.31476885080338d 0.64210742712021d"
				+ "0.12163265049458d 0.30436486005783d"
				+ "0.59846729040146d 0.26924616098404d"
				+ "0.43915036320686d 0.087771758437157d"
				+ "0.051845066249371d 0.025247814133763d"
				+ "0.5590004324913d 0.061266139149666d"
				+ "0.79735112190247d 0.88970428705215d"
				+ "0.31791752576828d 0.91872555017471d"
				+ "0.38600581884384d 0.27077883481979d"
				+ "0.98120784759521d 0.78259891271591d"
				+ "0.79740107059479d 0.0052061639726162d"
				+ "0.24274885654449d 0.20390670001507d"
				+ "0.43970012664795d 0.35829237103462d"
				+ "0.22498959302902d 0.16224072873592d"
				+ "0.14963765442371d 0.023915035650134d"
				+ "0.47895044088364d 0.065764263272285d"
				+ "0.19251978397369d 0.12452311813831d"
				+ "0.9842232465744d 0.75121200084686d"
				+ "0.26222407817841d 0.21113702654839d"
				+ "0.14897125959396d 0.48389005661011d"
				+ "0.48119950294495d 0.26334860920906d"
				+ "0.80992919206619d 0.95767599344254d"
				+ "0.40046647191048d 0.68932110071182d"
				+ "0.23240315914154d 0.29501873254776d"
				+ "0.99961680173874d 0.32578924298286d"
				+ "0.11027072370052d 0.04693877696991d"
				+ "0.68681383132935d 0.43805915117264d"
				+ "0.56598085165024d 0.19302789866924d"
				+ "0.2987921833992d 0.72000831365585d"
				+ "0.15055394172668d 0.14144939184189d"
				+ "0.10085797309875d 0.29408580064774d"
				+ "0.84358185529709d 0.72893792390823d"
				+ "0.78688877820969d 0.71131193637848d"
				+ "0.053227823227644d 0.22079966962337d"
				+ "0.47933360934258d 0.95127862691879d"
				+ "0.61622655391693d 0.31091213226318d"
				+ "0.72921282052994d 0.422149091959d"
				+ "0.79788422584534d 0.12172428518534d"
				+ "0.51611828804016d 0.96065807342529d"
				+ "0.29207828640938d 0.20425656437874d"
				+ "0.24821324646473d 0.83211159706116d"
				+ "0.22125780582428d 0.53810077905655d"
				+ "0.76223242282867d 0.73038733005524d"
				+ "0.13644315302372d 0.53142023086548d"
				+ "0.32348188757896d 0.77793419361114d"
				+ "0.017226155847311d 0.02094960398972d"
				+ "0.62583923339844d 0.52577257156372d"
				+ "0.27177008986473d 0.27199500799179d"
				+ "0.79175341129303d 0.95349436998367d"
				+ "0.73680967092514d 0.97843396663666d"
				+ "0.37224489450455d 0.46926280856133d"
				+ "0.67765098810196d 0.2627155482769d"
				+ "0.34690546989441d 0.90981256961823d"
				+ "0.78817158937454d 0.67586004734039d"
				+ "0.1238983720541d 0.54044979810715d"
				+ "0.19082048535347d 0.197459384799d"
				+ "0.53995835781097d 0.055052060633898d"
				+ "0.4366180896759d 0.23567679524422d"
				+ "0.860391497612d 0.57606828212738d"
				+ "0.50515615940094d 0.62724697589874d"
				+ "0.77907538414001d 0.65451896190643d"
				+ "0.80459809303284d 0.63747602701187d"
				+ "0.41845896840096d 0.2699958384037d"
				+ "0.17167846858501d 0.9602415561676d"
				+ "0.3295626938343d 0.8306622505188d"
				+ "0.87170344591141d 0.71799248456955d"
				+ "0.49197834730148d 0.17325280606747d"
				+ "0.59855061769485d 0.46174928545952d"
				+ "0.31386920809746d 0.56307369470596d"
				+ "0.47466889023781d 0.17110370099545d"
				+ "0.6319699883461d 0.69400250911713d"
				+ "0.051078718155622d 0.2542190849781d"
				+ "0.71158683300018d 0.68848812580109d"
				+ "0.30737191438675d 0.54783004522324d"
				+ "0.24659724533558d 0.097551017999649d"
				+ "0.65171176195145d 0.31724280118942d"
				+ "0.35945022106171d 0.90078300237656d"
				+ "0.92083299160004d 0.2563348710537d"
				+ "0.60116618871689d 0.50634735822678d"
				+ "0.38004165887833d 0.48755517601967d"
				+ "0.95133692026138d 0.75097876787186d"
				+ "0.72321534156799d 0.56192421913147d"
				+ "0.81812578439713d 0.89998334646225d"
				+ "0.072802998125553d 0.45903372764587d"
				+ "0.038267388939857d 0.64723867177963d"
				+ "0.97982507944107d 0.58704704046249d"
				+ "0.87706786394119d 0.1151936724782d"
				+ "0.42387339472771d 0.7826988697052d"
				+ "0.028404831886292d 0.85486882925034d"
				+ "0.81311118602753d 0.31129530072212d"
				+ "0.61472719907761d 0.84585589170456d"
				+ "0.98427319526672d 0.86671388149261d"
				+ "0.18705539405346d 0.4963181912899d"
				+ "0.20266555249691d 0.57140356302261d"
				+ "0.72498124837875d 0.64299041032791d"
				+ "0.16216576099396d 0.97638481855392d"
				+ "0.6366680264473d 0.55117869377136d"
				+ "0.98522281646729d 0.061249479651451d"
				+ "0.75885045528412d 0.91476052999496d"
				+ "0.22285714745522d 0.23416076600552d"
				+ "0.35683465003967d 0.85618489980698d"
				+ "0.85466057062149d 0.33185338973999d"
				+ "0.12449812889099d 0.92647230625153d"
				+ "0.28879633545876d 0.61963349580765d"
				+ "0.18428987264633d 0.10521449148655d"
				+ "0.36199915409088d 0.79137861728668d"
				+ "0.087230317294598d 0.8005747795105d"
				+ "0.33957517147064d 0.96953767538071d"
				+ "0.81291127204895d 0.84928780794144d"
				+ "0.91540193557739d 0.70513117313385d"
				+ "0.76949602365494d 0.5166597366333d"
				+ "0.21192836761475d 0.97775095701218d"
				+ "0.79371929168701d 0.49656808376312d"
				+ "0.78017491102219d 0.19556018710136d"
				+ "0.15088713169098d 0.9114618897438d"
				+ "0.59973347187042d 0.19529362022877d"
				+ "0.53487712144852d 0.31236150860786d"
				+ "0.07876718044281d 0.242257386446d"
				+ "0.068138279020786d 0.67885881662369d"
				+ "0.054010827094316d 0.030329030007124d"
				+ "0.30169096589088d 0.4191170334816d"
				+ "0.79077053070068d 0.68195754289627d"
				+ "0.21512703597546d 0.36987090110779d"
				+ "0.98292380571365d 0.74816328287125d"
				+ "0.21660974621773d 0.79642647504807d"
				+ "0.75291961431503d 0.20853811502457d"
				+ "0.14287380874157d 0.39266139268875d"
				+ "0.65177839994431d 0.4712453186512d"
				+ "0.25922530889511d 0.28102457523346d"
				+ "0.65909206867218d 0.37301957607269d"
				+ "0.25954186916351d 0.012536442838609d"
				+ "0.18302373588085d 0.17916701734066d"
				+ "0.26627239584923d 0.56678885221481d"
				+ "0.060308203101158d 0.58356517553329d"
				+ "0.83043730258942d 0.35194501280785d"
				+ "0.55625158548355d 0.70866304636002d"
				+ "0.93162846565247d 0.20473968982697d"
				+ "0.36473137140274d 0.10548105090857d"
				+ "0.97800916433334d 0.3904789686203d"
				+ "0.60819655656815d 0.75361096858978d"
				+ "0.80631405115128d 0.60304039716721d"
				+ "0.8376676440239d 0.061216160655022d"
				+ "0.68184924125671d 0.96487295627594d"
				+ "0.032736361026764d 0.86503124237061d"
				+ "0.29849228262901d 0.026997083798051d"
				+ "0.60156601667404d 0.43036234378815d"
				+ "0.77869218587875d 0.76900458335876d"
				+ "0.38089129328728d 0.45108705759048d"
				+ "0.6734693646431d 0.59905874729156d"
				+ "0.63601833581924d 0.049654312431812d"
				+ "0.96241563558578d 0.35389420390129d"
				+ "0.060824654996395d 0.77708452939987d"
				+ "0.053694292902946d 0.29881715774536d"
				+ "0.77775925397873d 0.61296957731247d"
				+ "0.78403997421265d 0.12770512700081d"
				+ "0.33784255385399d 0.96547269821167d"
				+ "0.41875883936882d 0.96300709247589d"
				+ "0.72066640853882d 0.67132860422134d"
				+ "0.65172845125198d 0.35574343800545d"
				+ "0.33439400792122d 0.99584341049194d"
				+ "0.60539776086807d 0.28550603985786d"
				+ "0.015760099515319d 0.6328946352005d"
				+ "0.83078718185425d 0.16045814752579d"
				+ "0.030070804059505d 0.70493125915527d"
				+ "0.30748856067657d 0.81733447313309d"
				+ "0.071203663945198d 0.76297378540039d"
				+ "0.4436651468277d 0.52144104242325d"
				+ "0.26160767674446d 0.78661388158798d"
				+ "0.076051644980907d 0.96665555238724d"
				+ "0.15230320394039d 0.18401499092579d"
				+ "0.4699541926384d 0.27542689442635d"
				+ "0.7228821516037d 0.79191172122955d"
				+ "0.31925031542778d 0.99877548217773d"
				+ "0.38150769472122d 0.87561017274857d"
				+ "0.74638903141022d 0.11629321426153d"
				+ "0.96491461992264d 0.1289879232645d"
				+ "0.3023906648159d 0.036143273115158d"
				+ "0.73840898275375d 0.67449396848679d"
				+ "0.96684712171555d 0.59506040811539d"
				+ "0.39586839079857d 0.063148684799671d"
				+ "0.14792169630527d 0.058350685983896d"
				+ "0.059741772711277d 0.27454394102097d"
				+ "0.68234902620316d 0.1198917105794d"
				+ "0.28104954957962d 0.71684300899506d"
				+ "0.83543521165848d 0.90213245153427d"
				+ "0.039383590221405d 0.22678050398827d"
				+ "0.30105790495872d 0.95609331130981d"
				+ "0.74290710687637d 0.069662638008595d"
				+ "0.20166596770287d 0.26136609911919d"
				+ "0.22835485637188d 0.93936693668365d"
				+ "0.088279880583286d 0.22611412405968d"
				+ "0.76103287935257d 0.95834237337112d"
				+ "0.94049143791199d 0.68707203865051d"
				+ "0.034818824380636d 0.67760932445526d"
				+ "0.166463971138d 0.90954601764679d"
				+ "0.17216159403324d 0.076759681105614d"
				+ "0.60293209552765d 0.58741354942322d"
				+ "0.72408163547516d 0.56395667791367d"
				+ "0.51520198583603d 0.8431236743927d"
				+ "0.67017078399658d 0.97593504190445d"
				+ "0.59715116024017d 0.22769679129124d"
				+ "0.41859224438667d 0.57800084352493d"
				+ "0.97122865915298d 0.72072470188141d"
				+ "0.80608081817627d 0.064031653106213d"
				+ "0.18845480680466d 0.73037070035934d"
				+ "0.097942523658276d 0.55647647380829d"
				+ "0.22842149436474d 0.09336943179369d"
				+ "0.98805499076843d 0.60635566711426d"
				+ "0.49929195642471d 0.075027070939541d"
				+ "0.59886711835861d 0.19326114654541d"
				+ "0.83780092000961d 0.36922115087509d"
				+ "0.48139941692352d 0.72535610198975d"
				+ "0.50925445556641d 0.098400667309761d"
				+ "0.61524367332458d 0.039375260472298d"
				+ "0.20753020048141d 0.81358599662781d"
				+ "0.40856310725212d 0.40062475204468d"
				+ "0.05507704988122d 0.49436900019646d"
				+ "0.69809246063232d 0.50298207998276d"
				+ "0.60291546583176d 0.54891294240952d"
				+ "0.74913787841797d 0.4688962996006d"
				+ "0.83063721656799d 0.81395250558853d"
				+ "0.25557684898376d 0.84938776493073d"
				+ "0.14640566706657d 0.55479383468628d"
				+ "0.33985838294029d 0.62404829263687d"
				+ "0.3869554400444d 0.46531444787979d"
				+ "0.55300289392471d 0.20104123651981d"
				+ "0.81759268045425d 0.66796332597733d"
				+ "0.87460225820541d 0.41710120439529d"
				+ "0.13219492137432d 0.71376091241837d"
				+ "0.71281963586807d 0.53753435611725d"
				+ "0.45321115851402d 0.58229905366898d"
				+ "0.90438985824585d 0.25621825456619d"
				+ "0.33166182041168d 0.68174093961716d"
				+ "0.71461892127991d 0.69560182094574d"
				+ "0.74713867902756d 0.84882134199142d"
				+ "0.83738440275192d 0.4067055284977d"
				+ "0.10780508071184d 0.34884631633759d"
				+ "0.39513537287712d 0.3691211938858d"
				+ "0.25039565563202d 0.87569344043732d"
				+ "0.93889212608337d 0.99101209640503d"
				+ "0.44021657109261d 0.55181175470352d"
				+ "0.44824656844139d 0.10911286622286d"
				+ "0.37114536762238d 0.92822158336639d"
				+ "0.33136191964149d 0.98872971534729d"
				+ "0.1656309813261d 0.984514772892d"
				+ "0.42497292160988d 0.32374009490013d"
				+ "0.37469387054443d 0.12885464727879d"
				+ "0.99438565969467d 0.23659308254719d"
				+ "0.97792583703995d 0.19797584414482d"
				+ "0.73347771167755d 0.27830904722214d"
				+ "0.3834902048111d 0.45718449354172d"
				+ "0.76469802856445d 0.42847979068756d"
				+ "0.42812162637711d 0.60035818815231d"
				+ "0.63906705379486d 0.09526863694191d"
				+ "0.37712618708611d 0.74994587898254d"
				+ "0.33617660403252d 0.11541024595499d"
				+ "0.92438149452209d 0.45696792006493d"
				+ "";
		StringReader stream = new StringReader(string);
		StreamTokenizer streamTokenizer = new StreamTokenizer(stream);
		streamTokenizer.whitespaceChars('d', 'd');

		final Vector dots = new Vector();

		try {
			int token;

			while ((token = streamTokenizer.nextToken()) != StreamTokenizer.TT_EOF) {
				final double x = streamTokenizer.nval;
				token = streamTokenizer.nextToken();

				final double y = streamTokenizer.nval;
				dots.addElement(new Jun2dPoint(x, y));
			}
		} catch (Exception e) {
			e.printStackTrace();

			return false;
		}

		JunVoronoi2dProcessor voronoi = JunVoronoi2dProcessor.Dots_(dots);
		voronoi.trace_(true);
		JunVoronoi2dProcessorTestExamples.MakeTraceFrame(voronoi.canvas());
		voronoi.compute();
		System.out.println("area: " + voronoi.area());

		Graphics gc = null;
		StImage anImage = new StImage(300, 300);

		try {
			Jun2dPoint[][] triangles = voronoi.triangles();
			gc = anImage.image().getGraphics();

			if (gc instanceof Graphics2D) {
				((Graphics2D) gc).setBackground(Color.white);
			}

			gc.setColor(Color.white);
			gc.fillRect(0, 0, anImage.width(), anImage.height());

			Point scale = new Point(200, 200);
			Point translation = new Point(50, 50);
			Point origin = new Point(0, 0);
			Rectangle box = new Rectangle(origin, new Dimension(1, 1));
			box.x *= scale.x;
			box.y *= scale.y;
			box.width *= scale.x;
			box.height *= scale.y;
			box.x += translation.x;
			box.y += translation.y;
			gc.setColor(Color.red);
			gc.drawRect(box.x, box.y, box.width, box.height);

			for (int i = 0; i < triangles.length; i++) {
				Jun2dPoint[] points = triangles[i];
				int[] xPoints = new int[points.length + 1];
				int[] yPoints = new int[points.length + 1];

				for (int j = 0; j < points.length; j++) {
					Jun2dPoint point = points[j];

					if (point != null) {
						xPoints[j] = (int) (point.x() * scale.x + translation.x);
						yPoints[j] = (int) (point.y() * scale.y + translation.y);
					}
				}

				xPoints[points.length] = xPoints[0];
				yPoints[points.length] = yPoints[0];
				gc.setColor(Color.black);
				gc.drawPolyline(xPoints, yPoints, xPoints.length);
			}

			anImage._display();
		} finally {
			gc.dispose();
		}

		//return voronoi;
		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example6() {
		//area: 0.639997532195d
		System.out.println("This is a low precision example.");

		Vector dots = new Vector();
		dots.addElement(new Jun2dPoint(0.1d, 0.1d));
		dots.addElement(new Jun2dPoint(0.1d, 0.9d));
		dots.addElement(new Jun2dPoint(0.9d, 0.1d));
		dots.addElement(new Jun2dPoint(0.9d, 0.9d));
		dots.addElement(new Jun2dPoint(0.5d, 0.5d));
		dots.addElement(new Jun2dPoint(0.1d, 0.5d));
		dots.addElement(new Jun2dPoint(0.9d, 0.5d));
		dots.addElement(new Jun2dPoint(0.5d, 0.1d));
		dots.addElement(new Jun2dPoint(0.5d, 0.9d));

		JunVoronoi2dProcessor voronoi = JunVoronoi2dProcessor.Dots_(dots);
		voronoi.trace_(true);
		JunVoronoi2dProcessorTestExamples.MakeTraceFrame(voronoi.canvas());
		voronoi.compute();
		System.out.println("area : " + voronoi.area());

		Graphics gc = null;
		StImage anImage = new StImage(300, 300);

		try {
			Jun2dPoint[][] triangles = voronoi.triangles();
			gc = anImage.image().getGraphics();

			if (gc instanceof Graphics2D) {
				((Graphics2D) gc).setBackground(Color.white);
			}

			gc.setColor(Color.white);
			gc.fillRect(0, 0, anImage.width(), anImage.height());

			Point scale = new Point(200, 200);
			Point translation = new Point(50, 50);
			Point origin = new Point(0, 0);
			Rectangle box = new Rectangle(origin, new Dimension(1, 1));
			box.x *= scale.x;
			box.y *= scale.y;
			box.width *= scale.x;
			box.height *= scale.y;
			box.x += translation.x;
			box.y += translation.y;
			gc.setColor(Color.red);
			gc.drawRect(box.x, box.y, box.width, box.height);

			for (int i = 0; i < triangles.length; i++) {
				Jun2dPoint[] points = triangles[i];
				int[] xPoints = new int[points.length + 1];
				int[] yPoints = new int[points.length + 1];

				for (int j = 0; j < points.length; j++) {
					Jun2dPoint point = points[j];

					if (point != null) {
						xPoints[j] = (int) (point.x() * scale.x + translation.x);
						yPoints[j] = (int) (point.y() * scale.y + translation.y);
					}
				}

				xPoints[points.length] = xPoints[0];
				yPoints[points.length] = yPoints[0];
				gc.setColor(Color.black);
				gc.drawPolyline(xPoints, yPoints, xPoints.length);
			}

			anImage._display();
		} finally {
			gc.dispose();
		}

		//return voronoi;
		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example7() {
		//area: 0.99999368088055d
		System.out.println("This is a low precision example.");

		final Vector dots = new Vector();
		final int roundTimes = 10;
		int count = 0;

		for (int i = 0; i <= roundTimes; i++) {
			for (int j = 0; j <= roundTimes; j++) {
				if ((count % 2) == 0) {
					dots.addElement(new Jun2dPoint((double) j / 10, (double) i / 10));
				}

				count++;
			}
		}

		JunVoronoi2dProcessor voronoi = JunVoronoi2dProcessor.Dots_(dots);
		voronoi.trace_(true);
		JunVoronoi2dProcessorTestExamples.MakeTraceFrame(voronoi.canvas());
		voronoi.compute();
		System.out.println("area: " + voronoi.area());

		Graphics gc = null;
		StImage anImage = new StImage(300, 300);

		try {
			Jun2dPoint[][] triangles = voronoi.triangles();
			gc = anImage.image().getGraphics();

			if (gc instanceof Graphics2D) {
				((Graphics2D) gc).setBackground(Color.white);
			}

			gc.setColor(Color.white);
			gc.fillRect(0, 0, anImage.width(), anImage.height());

			Point scale = new Point(200, 200);
			Point translation = new Point(50, 50);
			Point origin = new Point(0, 0);
			Rectangle box = new Rectangle(origin, new Dimension(1 - origin.x, 1 - origin.y));
			box.x *= scale.x;
			box.y *= scale.y;
			box.width *= scale.x;
			box.height *= scale.y;
			box.x += translation.x;
			box.y += translation.y;
			gc.setColor(Color.red);
			gc.drawRect(box.x, box.y, box.width, box.height);

			for (int i = 0; i < triangles.length; i++) {
				Jun2dPoint[] points = triangles[i];
				int[] xPoints = new int[points.length + 1];
				int[] yPoints = new int[points.length + 1];

				for (int j = 0; j < points.length; j++) {
					Jun2dPoint point = points[j];

					if (point != null) {
						xPoints[j] = (int) (point.x() * scale.x + translation.x);
						yPoints[j] = (int) (point.y() * scale.y + translation.y);
					}
				}

				xPoints[points.length] = xPoints[0];
				yPoints[points.length] = yPoints[0];
				gc.setColor(Color.black);
				gc.drawPolyline(xPoints, yPoints, xPoints.length);
			}

			anImage._display();
		} finally {
			gc.dispose();
		}

		//return voronoi;
		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example8() {
		//area: 3.1256671980047d
		System.out.println("This is a low precision example.");

		final Vector dots = new Vector();
		final int step = 10;
		dots.addElement(new Jun2dPoint(0.0d, 0.0d));

		StInterval interval = StInterval.From_to_by_(0, 360, step);
		interval.do_(new StBlockClosure() {
			public Object value_(Object valueObject) {
				double degrees = ((Double) valueObject).doubleValue();
				double x = Math.cos(JunAngle._DegreesToRadians(degrees));
				double y = Math.sin(JunAngle._DegreesToRadians(degrees));
				dots.addElement(new Jun2dPoint(x, y));
				x = Math.cos(JunAngle._DegreesToRadians(degrees + (step / 2)));
				y = Math.sin(JunAngle._DegreesToRadians(degrees + (step / 2)));
				dots.addElement(new Jun2dPoint(x * 0.5, y * 0.5));

				return null;
			}
		});

		JunVoronoi2dProcessor voronoi = JunVoronoi2dProcessor.Dots_(dots);
		voronoi.trace_(true);
		JunVoronoi2dProcessorTestExamples.MakeTraceFrame(voronoi.canvas());
		voronoi.compute();
		System.out.println("area: " + voronoi.area());

		Graphics gc = null;
		StImage anImage = new StImage(300, 300);

		try {
			Jun2dPoint[][] triangles = voronoi.triangles();
			gc = anImage.image().getGraphics();

			if (gc instanceof Graphics2D) {
				((Graphics2D) gc).setBackground(Color.white);
			}

			gc.setColor(Color.white);
			gc.fillRect(0, 0, anImage.width(), anImage.height());

			Point scale = new Point(100, 100);
			Point translation = new Point(150, 150);
			Point origin = new Point(-1, -1);
			Rectangle box = new Rectangle(origin, new Dimension(1 - origin.x, 1 - origin.y));
			box.x *= scale.x;
			box.y *= scale.y;
			box.width *= scale.x;
			box.height *= scale.y;
			box.x += translation.x;
			box.y += translation.y;
			gc.setColor(Color.red);
			gc.drawRect(box.x, box.y, box.width, box.height);

			for (int i = 0; i < triangles.length; i++) {
				Jun2dPoint[] points = triangles[i];
				int[] xPoints = new int[points.length + 1];
				int[] yPoints = new int[points.length + 1];

				for (int j = 0; j < points.length; j++) {
					Jun2dPoint point = points[j];

					if (point != null) {
						xPoints[j] = (int) (point.x() * scale.x + translation.x);
						yPoints[j] = (int) (point.y() * scale.y + translation.y);
					}
				}

				xPoints[points.length] = xPoints[0];
				yPoints[points.length] = yPoints[0];
				gc.setColor(Color.black);
				gc.drawPolyline(xPoints, yPoints, xPoints.length);
			}

			anImage._display();
		} finally {
			gc.dispose();
		}

		//return voronoi;
		return true;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public static boolean Example9() {
		//area: 3.0d
		System.out.println("This is a low precision example.");

		final Vector dots = new Vector();
		dots.addElement(new Jun2dPoint(0.0d, 0.0d));
		dots.addElement(new Jun2dPoint(0.5d, 0.0d));
		dots.addElement(new Jun2dPoint(0.43301270189222d, 0.25d));
		dots.addElement(new Jun2dPoint(0.86602540378444d, 0.0d));
		dots.addElement(new Jun2dPoint(0.75d, 0.43301270189222d));
		dots.addElement(new Jun2dPoint(1.0d, 0.0d));
		dots.addElement(new Jun2dPoint(0.86602540378444d, 0.5d));
		dots.addElement(new Jun2dPoint(0.25d, 0.43301270189222d));
		dots.addElement(new Jun2dPoint(0.43301270189222d, 0.75d));
		dots.addElement(new Jun2dPoint(0.5d, 0.86602540378444d));
		dots.addElement(new Jun2dPoint(0.0d, 0.5d));
		dots.addElement(new Jun2dPoint(0.0d, 0.86602540378444d));
		dots.addElement(new Jun2dPoint(0.0d, 1.0d));
		dots.addElement(new Jun2dPoint(-0.25d, 0.43301270189222d));
		dots.addElement(new Jun2dPoint(-0.43301270189222d, 0.75d));
		dots.addElement(new Jun2dPoint(-0.5d, 0.86602540378444d));
		dots.addElement(new Jun2dPoint(-0.43301270189222d, 0.25d));
		dots.addElement(new Jun2dPoint(-0.75d, 0.43301270189222d));
		dots.addElement(new Jun2dPoint(-0.86602540378444d, 0.5d));
		dots.addElement(new Jun2dPoint(-0.5d, 0.0d));
		dots.addElement(new Jun2dPoint(-0.86602540378444d, 0.0d));
		dots.addElement(new Jun2dPoint(-1.0d, 0.0d));
		dots.addElement(new Jun2dPoint(-0.43301270189222d, -0.25d));
		dots.addElement(new Jun2dPoint(-0.75d, -0.43301270189222d));
		dots.addElement(new Jun2dPoint(-0.86602540378444d, -0.5d));
		dots.addElement(new Jun2dPoint(-0.25d, -0.43301270189222d));
		dots.addElement(new Jun2dPoint(-0.43301270189222d, -0.75d));
		dots.addElement(new Jun2dPoint(-0.5d, -0.86602540378444d));
		dots.addElement(new Jun2dPoint(0.0d, -0.5d));
		dots.addElement(new Jun2dPoint(0.0d, -0.86602540378444d));
		dots.addElement(new Jun2dPoint(0.0d, -1.0d));
		dots.addElement(new Jun2dPoint(0.25d, -0.43301270189222d));
		dots.addElement(new Jun2dPoint(0.43301270189222d, -0.75d));
		dots.addElement(new Jun2dPoint(0.5d, -0.86602540378444d));
		dots.addElement(new Jun2dPoint(0.43301270189222d, -0.25d));
		dots.addElement(new Jun2dPoint(0.75d, -0.43301270189222d));
		dots.addElement(new Jun2dPoint(0.86602540378444d, -0.5d));

		JunVoronoi2dProcessor voronoi = JunVoronoi2dProcessor.Dots_(dots);
		voronoi.trace_(true);
		JunVoronoi2dProcessorTestExamples.MakeTraceFrame(voronoi.canvas());
		voronoi.compute();
		System.out.println("area: " + voronoi.area());

		Graphics gc = null;
		StImage anImage = new StImage(300, 300);

		try {
			Jun2dPoint[][] triangles = voronoi.triangles();
			gc = anImage.image().getGraphics();

			if (gc instanceof Graphics2D) {
				((Graphics2D) gc).setBackground(Color.white);
			}

			gc.setColor(Color.white);
			gc.fillRect(0, 0, anImage.width(), anImage.height());

			Point scale = new Point(100, 100);
			Point translation = new Point(150, 150);
			Point origin = new Point(-1, -1);
			Rectangle box = new Rectangle(origin, new Dimension(1 - origin.x, 1 - origin.y));
			box.x *= scale.x;
			box.y *= scale.y;
			box.width *= scale.x;
			box.height *= scale.y;
			box.x += translation.x;
			box.y += translation.y;
			gc.setColor(Color.red);
			gc.drawRect(box.x, box.y, box.width, box.height);

			for (int i = 0; i < triangles.length; i++) {
				Jun2dPoint[] points = triangles[i];
				int[] xPoints = new int[points.length + 1];
				int[] yPoints = new int[points.length + 1];

				for (int j = 0; j < points.length; j++) {
					Jun2dPoint point = points[j];

					if (point != null) {
						xPoints[j] = (int) (point.x() * scale.x + translation.x);
						yPoints[j] = (int) (point.y() * scale.y + translation.y);
					}
				}

				xPoints[points.length] = xPoints[0];
				yPoints[points.length] = yPoints[0];
				gc.setColor(Color.black);
				gc.drawPolyline(xPoints, yPoints, xPoints.length);
			}

			anImage._display();
		} finally {
			gc.dispose();
		}

		//return voronoi;
		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 * 
	 * @throws Exception DOCUMENT ME!
	 */
	public static void main(String[] args) throws Exception {
		new JunVoronoi2dProcessorTestExamples();
	}

	/**
	 * Execute all examples.
	 * 
	 * @param canvas
	 */
	protected static void MakeTraceFrame(Canvas canvas) {
		Frame frame = new Frame("JunVoronoi2dProcessor Trace Display");
		frame.setSize(JunVoronoi2dProcessor.defaultDimension);
		frame.setResizable(false);
		frame.add(canvas);
		frame.pack();
		frame.show();
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				e.getWindow().dispose();
			}
		});
	}
}
