package sra.jun.voronoi.twoD.diagram;

import java.io.IOException;
import java.io.Writer;

/**
 * JunVoronoi2dSide class
 * 
 *  @author    NISHIHARA Satoshi
 *  @created   2000/01/25 (by NISHIHARA Satoshi)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVoronoi2dSide.java,v 6.0 2003/10/28 10:52:27 nisinaka Exp $
 */
public class JunVoronoi2dSide extends JunVoronoi2dObject {
	protected JunVoronoi2dPoint from = null;
	protected JunVoronoi2dPoint to = null;
	protected JunVoronoi2dDot dot1 = null;
	protected JunVoronoi2dDot dot2 = null;

	/**
	 * Create a new instance of Jun2dPolygon and initialize it with the array
	 * of points.
	 * 
	 * @param voronoi2dPoint1
	 * @param voronoi2dPoint2
	 */
	public JunVoronoi2dSide(JunVoronoi2dPoint voronoi2dPoint1, JunVoronoi2dPoint voronoi2dPoint2) {
		super();

		if (voronoi2dPoint1 != null) {
			this.from_(voronoi2dPoint1);
		}

		if (voronoi2dPoint2 != null) {
			this.to_(voronoi2dPoint2);
		}
	}

	/**
	 * Create a new instance of JunVoronoi2dSide.
	 * 
	 * @param voronoi2dPoint1
	 * @param voronoi2dPoint2
	 * 
	 * @return JunVoronoi2dSide
	 */
	public static JunVoronoi2dSide From_to_(JunVoronoi2dPoint voronoi2dPoint1, JunVoronoi2dPoint voronoi2dPoint2) {
		return new JunVoronoi2dSide(voronoi2dPoint1, voronoi2dPoint2);
	}

	/**
	 * accessor dot1.
	 * 
	 * @return JunVoronoi2dDot
	 */
	public JunVoronoi2dDot dot1() {
		return this.dot1;
	}

	/**
	 * accessor dot1:.
	 * 
	 * @param voronoi2dDot
	 */
	public void dot1_(JunVoronoi2dDot voronoi2dDot) {
		this.dot1 = voronoi2dDot;
	}

	/**
	 * accessor dot2.
	 * 
	 * @return JunVoronoi2dDot
	 */
	public JunVoronoi2dDot dot2() {
		return this.dot2;
	}

	/**
	 * accessor dot2:.
	 * 
	 * @param voronoi2dDot
	 */
	public void dot2_(JunVoronoi2dDot voronoi2dDot) {
		this.dot2 = voronoi2dDot;
	}

	/**
	 * accessor from.
	 * 
	 * @return JunVoronoi2dPoint
	 */
	public JunVoronoi2dPoint from() {
		return this.from;
	}

	/**
	 * accessor from:.
	 * 
	 * @param voronoi2dPoint
	 */
	public void from_(JunVoronoi2dPoint voronoi2dPoint) {
		this.from = voronoi2dPoint;
	}

	/**
	 * functions:.
	 * 
	 * @param voronoi2dSide
	 * 
	 * @return JunVoronoi2dPoint
	 */
	public JunVoronoi2dPoint intersectingPointWithSide_(JunVoronoi2dSide voronoi2dSide) {
		final double x1 = this.from().x();
		final double y1 = this.from().y();
		final double x2 = this.to().x();
		final double y2 = this.to().y();
		final double x3 = voronoi2dSide.from().x();
		final double y3 = voronoi2dSide.from().y();
		final double x4 = voronoi2dSide.to().x();
		final double y4 = voronoi2dSide.to().y();
		final double x21 = x2 - x1;
		final double y21 = y2 - y1;
		final double x43 = x4 - x3;
		final double y43 = y4 - y3;
		final double x31 = x3 - x1;
		final double y31 = y3 - y1;
		final double det = (x43 * y21) - (y43 * x21);

		if (Math.abs(det) < ACCURACY) {
			return null;
		}

		final double detinv = 1.0d / det;
		final double s = (x43 * y31 - y43 * x31) * detinv;
		final double t = (x21 * y31 - y21 * x31) * detinv;

		if ((s < 0.0d) || (1.0d < s) || (t < 0.0d) || (1.0d < t)) {
			return null;
		}

		final double x = x1 + (x21 * s);
		final double y = y1 + (y21 * s);

		return new JunVoronoi2dPoint(x, y);
	}

	/**
	 * testing:.
	 * 
	 * @return boolean
	 */
	public boolean isInvalid() {
		return !this.isValid();
	}

	/**
	 * testing:.
	 * 
	 * @return boolean
	 */
	public boolean isValid() {
		boolean bool = true;

		if (from == null) {
			bool = false;
		} else {
			if (!from.isValid()) {
				bool = false;
			}
		}

		if (to == null) {
			bool = false;
		} else {
			if (!to.isValid()) {
				bool = false;
			}
		}

		if (dot1 == null) {
			bool = false;
		}

		if (dot2 == null) {
			bool = false;
		}

		return bool;
	}

	/**
	 * testing:.
	 * 
	 * @return boolean
	 */
	public boolean isValid2() {
		boolean bool = true;

		if (from == null) {
			bool = false;
		}

		if (to == null) {
			bool = false;
		}

		if (dot1 == null) {
			bool = false;
		}

		if (dot2 == null) {
			bool = false;
		}

		return bool;
	}

	/**
	 * Print my string representation on aStream.
	 * 
	 * @param aWriter java.io.OutputStream
	 * 
	 * @throws IOException DOCUMENT ME!
	 */
	public void printOn_(Writer aWriter) throws IOException {
		aWriter.write("side ( ");
		this.from().printOn_(aWriter);
		aWriter.write(" , ");
		this.to().printOn_(aWriter);

		if (VoronoiDebugFlag) {
			aWriter.write(" , ");

			if (!(dot1 == null)) {
				this.dot1().printOn_(aWriter);
			} else {
				aWriter.write("invalid dot1");
			}

			aWriter.write(" , ");

			if (!(dot2 == null)) {
				this.dot2().printOn_(aWriter);
			} else {
				aWriter.write("invalid dot2");
			}
		}

		aWriter.write(" )");
		//}
		//catch (Exception e) {
		//}
	}

	/**
	 * accessor to.
	 * 
	 * @return JunVoronoi2dPoint
	 */
	public JunVoronoi2dPoint to() {
		return this.to;
	}

	/**
	 * accessor to:.
	 * 
	 * @param voronoi2dPoint
	 */
	public void to_(JunVoronoi2dPoint voronoi2dPoint) {
		this.to = voronoi2dPoint;
	}
}
