package sra.jun.vrml.node.geometry20;

import java.util.Hashtable;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.opengl.objects.JunOpenGL3dObject;
import sra.jun.vrml.field.sf.*;
import sra.jun.vrml.support.JunVrmlNodeVisitor;

/**
 * JunVrmlConeNode class
 * 
 *  @author    nisinaka
 *  @created   2000/04/17 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlConeNode.java,v 6.0 2003/10/28 10:52:29 nisinaka Exp $
 */
public class JunVrmlConeNode extends sra.jun.vrml.node.abstracts.JunVrmlAbstractGeometryNode {
	protected JunVrmlSFFloatField bottomRadiusField;
	protected JunVrmlSFFloatField heightField;
	protected JunVrmlSFBoolField sideField;
	protected JunVrmlSFBoolField bottomField;

	/**
	 * Create a new instance of JunVrmlConeNode and initialize it.
	 * 
	 * @param aDictionary java.util.Hashtable
	 */
	public JunVrmlConeNode(Hashtable aDictionary) {
		if (aDictionary.containsKey($("bottomRadius"))) {
			bottomRadiusField = new JunVrmlSFFloatField(aDictionary.get($("bottomRadius")));
		}

		if (aDictionary.containsKey($("height"))) {
			heightField = new JunVrmlSFFloatField(aDictionary.get($("height")));
		}

		if (aDictionary.containsKey($("side"))) {
			sideField = new JunVrmlSFBoolField(aDictionary.get($("side")));
		}

		if (aDictionary.containsKey($("bottom"))) {
			bottomField = new JunVrmlSFBoolField(aDictionary.get($("bottom")));
		}
	}

	/**
	 * Answer the default value of bottom.
	 * 
	 * @return boolean
	 */
	public static final boolean DefaultBottom() {
		return true;
	}

	/**
	 * Answer the default value of bottomRadius.
	 * 
	 * @return float
	 */
	public static final float DefaultBottomRadius() {
		return 1;
	}

	/**
	 * Answer the default value of height.
	 * 
	 * @return float
	 */
	public static final float DefaultHeight() {
		return 2;
	}

	/**
	 * Answer the default value of side.
	 * 
	 * @return boolean
	 */
	public static final boolean DefaultSide() {
		return true;
	}

	/**
	 * Convert to a JunOpenGL3dObject.
	 * 
	 * @return sra.jun.opengl.objects.JunOpenGL3dObject
	 */
	public JunOpenGL3dObject asJunOpenGL3dObject() {
		JunOpenGL3dObject aCone = JunOpenGL3dObject.Cone_radius_height_(10, this.bottomRadius(), this.height());
		Jun3dTransformation aTransformation = Jun3dTransformation.Translate_(new Jun3dPoint(0, 0, this.height() / -2));
		aTransformation = aTransformation.product_(Jun3dTransformation.RotateX_(JunAngle.FromDeg_(-90)));
		aCone = aCone.transform_(aTransformation);

		return aCone;
	}

	/**
	 * Answer the bottom.
	 * 
	 * @return boolean
	 */
	public boolean bottom() {
		return this.bottomField().value();
	}

	/**
	 * Answer the bottomRadius.
	 * 
	 * @return float
	 */
	public float bottomRadius() {
		return this.bottomRadiusField().value();
	}

	/**
	 * Answer the height.
	 * 
	 * @return float
	 */
	public float height() {
		return this.heightField().value();
	}

	/**
	 * Process myself with the generator.
	 * 
	 * @param aVisitor sra.jun.vrml.support.JunVrmlNodeVisitor
	 */
	public void nodeDo_(JunVrmlNodeVisitor aVisitor) {
		try {
			aVisitor.perform_with_("doCone_", this);
		} catch (Exception e) {
			super.nodeDo_(aVisitor);
		}
	}

	/**
	 * Answer the side.
	 * 
	 * @return boolean
	 */
	public boolean side() {
		return this.sideField().value();
	}

	/**
	 * Answer the bottom field.
	 * 
	 * @return sra.jun.vrml.field.sf.JunVrmlSFBoolField
	 */
	protected JunVrmlSFBoolField bottomField() {
		if (bottomField == null) {
			bottomField = new JunVrmlSFBoolField(DefaultBottom());
		}

		return bottomField;
	}

	/**
	 * Answer the bottomRadius.
	 * 
	 * @return sra.jun.vrml.field.sf.JunVrmlSFFloatField
	 */
	protected JunVrmlSFFloatField bottomRadiusField() {
		if (bottomRadiusField == null) {
			bottomRadiusField = new JunVrmlSFFloatField(DefaultBottomRadius());
		}

		return bottomRadiusField;
	}

	/**
	 * Answer the height.
	 * 
	 * @return sra.jun.vrml.field.sf.JunVrmlSFFloatField
	 */
	protected JunVrmlSFFloatField heightField() {
		if (heightField == null) {
			heightField = new JunVrmlSFFloatField(DefaultHeight());
		}

		return heightField;
	}

	/**
	 * Answer the side field.
	 * 
	 * @return sra.jun.vrml.field.sf.JunVrmlSFBoolField
	 */
	protected JunVrmlSFBoolField sideField() {
		if (sideField == null) {
			sideField = new JunVrmlSFBoolField(DefaultSide());
		}

		return sideField;
	}
}
