package sra.jun.vrml.node.property20;

import java.awt.Color;
import java.util.Hashtable;
import sra.jun.opengl.objects.JunOpenGL3dObject;
import sra.jun.vrml.field.mf.JunVrmlMFColorField;
import sra.jun.vrml.node.abstracts.JunVrmlAbstractPropertyNode;
import sra.jun.vrml.support.JunVrmlNodeVisitor;

/**
 * JunVrmlColorNode class
 * 
 *  @author    nisinaka
 *  @created   2000/03/29 (by nisinaka)
 *  @updated   N/A
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlColorNode.java,v 6.0 2003/10/28 10:52:31 nisinaka Exp $
 */
public class JunVrmlColorNode extends JunVrmlAbstractPropertyNode {

	protected JunVrmlMFColorField colorField;

	/**
	 * The default constructor.
	 * 
	 * @category Instance creation
	 */
	public JunVrmlColorNode() {
	}

	/**
	 * Create a new instance of JunVrmlColorNode and initialize it.
	 * 
	 * @param aDictionary java.util.Hashtable
	 * @category Instance creation
	 */
	public JunVrmlColorNode(Hashtable aDictionary) {
		if (aDictionary.containsKey($("color"))) {
			colorField = new JunVrmlMFColorField(aDictionary.get($("color")));
		}
	}

	/**
	 * Answer the default value of color.
	 * 
	 * @return float[]
	 * @category Defaults
	 */
	public static final float[] DefaultColor() {
		return new float[] {
		};
	}

	/**
	 * Answer the color value.
	 * 
	 * @return float[]
	 * @category accessing
	 */
	public float[] color() {
		return this.colorField().value();
	}

	/**
	 * Answer the color field.
	 * 
	 * @return sra.jun.vrml.field.mf.JunVrmlMFColorField
	 * @category accessing
	 */
	protected JunVrmlMFColorField colorField() {
		if (colorField == null) {
			colorField = new JunVrmlMFColorField(DefaultColor());
		}
		return colorField;
	}

	/**
	 * Set my properties on the JunOpenGL3dObject.
	 * 
	 * @param anOpenGL3dObject sra.jun.opengl.objects.JunOpenGL3dObject
	 * @category code generation
	 */
	public void propertyOn_(JunOpenGL3dObject anOpenGL3dObject) {
		anOpenGL3dObject.paint_(this.colorField().asColors()[0]);
	}

	/**
	 * Answer the color field values as an array of Color.
	 * 
	 * @return java.awt.Color
	 * @category code generation
	 */
	public Color[] colorAsColors() {
		return this.colorField().asColors();
	}

	/**
	 * Process myself with the generator.
	 * 
	 * @param aVisitor sra.jun.vrml.support.JunVrmlNodeVisitor
	 * @category enumerating
	 */
	public void nodeDo_(JunVrmlNodeVisitor aVisitor) {
		try {
			aVisitor.perform_with_("doColor_", this);
		} catch (Exception e) {
			super.nodeDo_(aVisitor);
		}
	}

}
