package sra.jun.vrml.support;

import java.util.Vector;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.opengl.display.*;
import sra.jun.opengl.objects.JunOpenGL3dObject;
import sra.jun.vrml.node.abstracts.*;
import sra.jun.vrml.node.geometry20.*;
import sra.jun.vrml.node.grouping20.*;
import sra.jun.vrml.node.property20.*;

/**
 * JunVrmlGenerator20 class
 * 
 *  @author    ASTI Beijing
 *  @created   UNKNOWN
 *  @updated   2003/07/02 (by nisinaka)
 *  @version   406b (with StPL6.0) based on JunXXX for Smalltalk
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2003 Information-technology Promotion Agency,Japan (IPA)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunVrmlGenerator20.java,v 6.0 2003/10/28 10:52:31 nisinaka Exp $
 */
public class JunVrmlGenerator20 extends JunVrmlGenerator {

	/**
	 * Process the Appearance node.
	 * 
	 * @param aNode sra.jun.vrml.node.property20.JunVrmlAppearanceNode
	 */
	public void doAppearance_(JunVrmlAppearanceNode aNode) {
		if (aNode.material() != null) {
			this.addPropertyNode_(aNode.material());
		}

		if (aNode.texture() != null) {
			this.informationAt_put_($("texture"), aNode.texture());
		}
	}

	/**
	 * Process the Group node.
	 * 
	 * @param aNode sra.jun.vrml.node.grouping20.JunVrmlGroupNode
	 */
	public void doGroup_(JunVrmlGroupNode aNode) {
		JunVrmlGenerator20 newGenerator = new JunVrmlGenerator20();
		JunVrmlNode[] children = aNode.children();

		for (int i = 0; i < children.length; i++) {
			children[i].nodeDo_(newGenerator);
		}

		JunOpenGL3dObject newObject = newGenerator.object();

		if (newObject != null) {
			newGenerator.propertyOn_(newObject);
			this.addObject_(newObject);
		}
	}

	/**
	 * Process the IndexedFaceSet node.
	 * 
	 * @param aNode sra.jun.vrml.node.geometry20.JunVrmlIndexedFaceSetNode20
	 */
	public void doIndexedFaceSet_(JunVrmlIndexedFaceSetNode20 aNode) {
		JunOpenGL3dObject a3dObject = aNode.asJunOpenGL3dObject();

		if (a3dObject != null) {
			JunVrmlAbstractPropertyNode textureNode = (JunVrmlAbstractPropertyNode) this.informationAt_($("texture"));

			if (textureNode != null) {
				textureNode.propertyOn_(a3dObject);
				aNode.texCoordOn_(a3dObject);
			}

			this.addObject_(a3dObject);
		}
	}

	/**
	 * Process the IndexedLineSet node.
	 * 
	 * @param aNode sra.jun.vrml.node.geometry20.JunVrmlIndexedLineSetNode20
	 */
	public void doIndexedLineSet_(JunVrmlIndexedLineSetNode20 aNode) {
		JunOpenGL3dObject a3dObject = aNode.asJunOpenGL3dObject();

		if (a3dObject != null) {
			this.addObject_(a3dObject);
		}
	}

	/**
	 * Process the NavigationInfo node.
	 * 
	 * @param aNode sra.jun.vrml.node.property20.JunVrmlNavigationInfoNode
	 */
	public void doNavigationInfo_(JunVrmlNavigationInfoNode aNode) {
		this.informationAt_put_($("navigationInfo"), aNode);
	}

	/**
	 * Process the Shape node.
	 * 
	 * @param aNode sra.jun.vrml.node.grouping20.JunVrmlShapeNode
	 */
	public void doShape_(JunVrmlShapeNode aNode) {
		if (aNode.geometry() == null) {
			return;
		}

		// The order of the processing is quite important.
		// Sometimes properties in the appearance node
		// are used to process the geometry node.
		JunVrmlGenerator newGenerator = new JunVrmlGenerator20();

		if (aNode.appearance() != null) {
			aNode.appearance().nodeDo_(newGenerator);
		}

		aNode.geometry().nodeDo_(newGenerator);

		JunOpenGL3dObject newObject = newGenerator.object();

		if (newObject != null) {
			newGenerator.propertyOn_(newObject);
			this.addObject_(newObject);
		}
	}

	/**
	 * Process the TextureCoordinate node.
	 * 
	 * @param aNode sra.jun.vrml.node.property20.JunVrmlTextureCoordinateNode
	 */
	public void doTextureCoordinate_(JunVrmlTextureCoordinateNode aNode) {
		this.informationAt_put_($("textureCoordinate"), aNode);
	}

	/**
	 * Process the Transform node.
	 * 
	 * @param aNode sra.jun.vrml.node.grouping20.JunVrmlTransformNode
	 */
	public void doTransform_(JunVrmlTransformNode aNode) {
		JunVrmlGenerator20 newGenerator = new JunVrmlGenerator20();
		JunVrmlNode[] children = aNode.children();

		for (int i = 0; i < children.length; i++) {
			children[i].nodeDo_(newGenerator);
		}

		JunOpenGL3dObject newObject = newGenerator.object();

		if (newObject != null) {
			newGenerator.propertyOn_(newObject);
			this.addObject_(newObject);
		}
	}

	/**
	 * Process the Viewpoint node.
	 * 
	 * @param aNode sra.jun.vrml.node.property20.JunVrmlViewpointNode
	 */
	public void doViewpoint_(JunVrmlViewpointNode aNode) {
		this.informationAt_put_($("viewpoint"), aNode);
	}

	/**
	 * Process the WorldInfo node.
	 * 
	 * @param aNode sra.jun.vrml.node.property20.JunVrmlWorldInfoNode
	 */
	public void doWorldInfo_(JunVrmlWorldInfoNode aNode) {
		this.informationAt_put_($("worldInfo"), aNode);
	}

	/**
	 * Set the lights on the JunOpenGLDisplayModel.
	 * 
	 * @param aDisplayModel sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	protected void setLights_(JunOpenGLDisplayModel aDisplayModel) {
		JunOpenGLDisplayLight[] displayLights = aDisplayModel.displayLights();
		for (int i = 0; i < displayLights.length; i++) {
			displayLights[i].beOff();
		}

		JunVrmlNavigationInfoNode navigationInfoNode = (JunVrmlNavigationInfoNode) this.informationAt_($("navigationInfo"));
		if ((navigationInfoNode == null) || (navigationInfoNode.headlight() == true)) {
			// Use the default light of the display model.
			aDisplayModel.displayLight1().beOn();
		}

		// Use the ambient light anyway.
		aDisplayModel.displayLight5().beOn();

		Vector lights = this.lights();
		if (lights == null) {
			return;
		}

		// displayLight2
		if (lights.size() > 0) {
			aDisplayModel.displayLights()[1] = (JunOpenGLDisplayLight) lights.elementAt(0);
			aDisplayModel.displayLights()[1].beOn();
		}

		// displayLight3
		if (lights.size() > 1) {
			aDisplayModel.displayLights()[2] = (JunOpenGLDisplayLight) lights.elementAt(1);
			aDisplayModel.displayLights()[2].beOn();
		}
		// displayLight4
		if (lights.size() > 2) {
			aDisplayModel.displayLights()[3] = (JunOpenGLDisplayLight) lights.elementAt(2);
			aDisplayModel.displayLights()[3].beOn();
		}
	}

	/**
	 * Set the projection to the JunOpenGLDisplayModel.
	 * 
	 * @param aDisplayModel sra.jun.opengl.display.JunOpenGLDisplayModel
	 */
	protected void setProjection_(JunOpenGLDisplayModel aDisplayModel) {
		Jun3dPoint eyePoint, sightPoint, upVector;
		double viewFactor, zoomHeight;

		JunVrmlViewpointNode viewpointNode = (JunVrmlViewpointNode) this.informationAt_($("viewpoint"));
		if (viewpointNode == null) {
			viewpointNode = new JunVrmlViewpointNode();
		}
		JunVrmlNavigationInfoNode navigationInfoNode = (JunVrmlNavigationInfoNode) this.informationAt_($("navigationInfo"));
		if (navigationInfoNode == null) {
			navigationInfoNode = new JunVrmlNavigationInfoNode();
		}

		eyePoint = viewpointNode.positionAs3dPoint();
		upVector = (new Jun3dPoint(0, 1, 0)).transform_(viewpointNode.orientationAs3dTransformation());

		float[] avatarSize = navigationInfoNode.avatarSize();
		if (avatarSize.length >= 6) {
			sightPoint = new Jun3dPoint(avatarSize[3], avatarSize[4], avatarSize[5]);
		} else {
			sightPoint = aDisplayModel.defaultSightPoint();
		}

		if (navigationInfoNode.visibilityLimit() > 0) {
			viewFactor = navigationInfoNode.visibilityLimit() / eyePoint.distance_(sightPoint);
		} else {
			viewFactor = aDisplayModel.defaultViewFactor();
		}

		zoomHeight = Math.tan(viewpointNode.fieldOfView() / 2) * eyePoint.distance_(sightPoint) * 2;

		aDisplayModel.defaultEyePoint_(eyePoint);
		aDisplayModel.defaultSightPoint_(sightPoint);
		aDisplayModel.defaultUpVector_(upVector);
		aDisplayModel.defaultViewFactor_(viewFactor);
		aDisplayModel.defaultZoomHeight_(zoomHeight);
		aDisplayModel.flushProjector();
	}
}
