package sra.smalltalk;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * StApplicationModel class
 * 
 *  @author    he weijie
 *  @created   1998/08/18 (by he weijie)
 *  @updated   1999/10/01 (by MATSUDA Ryouichi)
 *  @updated   2002/04/09 (by nisinaka)
 *  @version   6.0
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StApplicationModel.java,v 6.0 2003/10/28 10:52:32 nisinaka Exp $
 */
public abstract class StApplicationModel extends StModel implements ActionListener, ItemListener {

	/** view mode */
	public static final int VIEW_AWT = 0;
	public static final int VIEW_SWING = 1;
	public static final int VIEW_SWING_ONLY = 2;
	protected static int DefaultViewMode = VIEW_AWT;

	protected StUIBuilder builder = null;

	/**
	 * Create a new instance of the ApplicationModel and initialize it.
	 * 
	 * @category Instance creation
	 */
	public StApplicationModel() {
		super();
		this.initialize();
	}

	/**
	 * Create a JPanel with border.
	 * 
	 * @return javax.swing.JPanel
	 * @category Utilities
	 */
	public static JPanel _JPanelWithBorder() {
		return new JPanel() {
			public Insets getInsets() {
				return new Insets(2, 2, 2, 2);
			}

			public void paint(Graphics graphics) {
				super.paint(graphics);

				Dimension size = this.getSize();

				// Black lines
				graphics.setColor(Color.black);
				graphics.drawLine(1, 1, size.width - 3, 1);
				graphics.drawLine(1, 1, 1, size.height - 3);

				// Gray lines.
				graphics.setColor(Color.gray);
				graphics.drawLine(0, 0, size.width - 2, 0);
				graphics.drawLine(0, 0, 0, size.height - 2);

				// Light gray lines.
				graphics.setColor(Color.lightGray);
				graphics.drawLine(1, size.height - 2, size.width - 2, size.height - 2);
				graphics.drawLine(size.width - 2, 1, size.width - 2, size.height - 2);

				// White lines.
				graphics.setColor(Color.white);
				graphics.drawLine(0, size.height - 1, size.width - 1, size.height - 1);
				graphics.drawLine(size.width - 1, 0, size.width - 1, size.height - 1);
			}
		};
	}

	/**
	 * create a panel with the border.
	 * 
	 * @return java.awt.Panel
	 * @category Utilities
	 */
	public static Panel _PanelWithBorder() {
		Panel aPanel = new Panel() {
			public Insets getInsets() {
				return new Insets(2, 2, 2, 2);
			}

			public void paint(Graphics graphics) {
				super.paint(graphics);

				Dimension size = this.getSize();

				// Black lines
				graphics.setColor(Color.black);
				graphics.drawLine(1, 1, size.width - 3, 1);
				graphics.drawLine(1, 1, 1, size.height - 3);

				// Gray lines.
				graphics.setColor(Color.gray);
				graphics.drawLine(0, 0, size.width - 2, 0);
				graphics.drawLine(0, 0, 0, size.height - 2);

				// Light gray lines.
				graphics.setColor(Color.lightGray);
				graphics.drawLine(1, size.height - 2, size.width - 2, size.height - 2);
				graphics.drawLine(size.width - 2, 1, size.width - 2, size.height - 2);

				// White lines.
				graphics.setColor(Color.white);
				graphics.drawLine(0, size.height - 1, size.width - 1, size.height - 1);
				graphics.drawLine(size.width - 1, 0, size.width - 1, size.height - 1);
			}
		};

		return aPanel;
	}

	/**
	 * Show the window at the center of the display.
	 * 
	 * @param aWindow java.awt.Window
	 * @category Utilities
	 */
	public static void _ShowAtCenterPoint(Window aWindow) {
		Rectangle bounds = aWindow.getBounds();
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Point centerPoint = new Point(screenSize.width / 2, screenSize.height / 2);
		int originX = Math.max(0, Math.min(centerPoint.x - (bounds.width / 2), screenSize.width - bounds.width));
		int originY = Math.max(0, Math.min(centerPoint.y - (bounds.height / 2), screenSize.height - bounds.height));
		aWindow.setLocation(originX, originY);
		aWindow.setVisible(true);
	}

	/**
	 * Show the window at the current mouse point.
	 * 
	 * @param aWindow java.awt.Window
	 * @category Utilities
	 */
	public static void _ShowAtMousePoint(Window aWindow) {
		try {
			Point point = SystemInterface._MousePoint();
			Rectangle bounds = aWindow.getBounds();
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			int originX = Math.max(0, Math.min(point.x - (bounds.width / 2), screenSize.width - bounds.width));
			int originY = Math.max(0, Math.min(point.y - (bounds.height / 2), screenSize.height - bounds.height));
			aWindow.setLocation(originX, originY);
		} catch (UnsatisfiedLinkError e) {
		} catch (NoClassDefFoundError e) {
		}

		aWindow.setVisible(true);
	}

	/**
	 * Create a view from class name.
	 * 
	 * @param className java.lang.String
	 * @return sra.smalltalk.StView
	 * @throws SmalltalkException sra.smalltalk.SmalltalkException
	 * @deprecated since Jun340
	 * @category Utilities
	 */
	protected static final StView CreateView(String className) {
		try {
			return (StView) Class.forName(className).newInstance();
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Answer the default view mode.
	 * 
	 * @return int
	 * @category Defaults
	 */
	public static final int GetDefaultViewMode() {
		return DefaultViewMode;
	}

	/**
	 * Set the default view mode.
	 * 
	 * @param mode int
	 * @return boolean
	 * @category Defaults
	 */
	public static final boolean SetDefaultViewMode(int mode) {
		DefaultViewMode = mode;
		return true;
	}

	/**
	 * Is swing enabled.
	 * 
	 * @return boolean
	 * @deprecated since Jun340
	 * @category Testing
	 */
	public static final boolean IsSwingInstalled() {
		return true;
	}

	/**
	 * Initialize the ApplicationModel when created.
	 * 
	 * @category initialize-release
	 */
	protected void initialize() {
		// Nothing to do by default.
	}

	/**
	 * Answer my builder.
	 * 
	 * @return sra.smalltalk.StUIBuilder
	 * @category accessing
	 */
	public final StUIBuilder builder() {
		if (builder == null) {
			builder = new StUIBuilder(this);
		}
		return builder;
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 * @category defaults
	 */
	public abstract StView defaultView();

	/**
	 * Invoked when a menu item has been selected.
	 * 
	 * @param ev java.awt.event.ActionEvent
	 * @throws SmalltalkException if failed to perform.
	 * @category event handling
	 */
	public void actionPerformed(ActionEvent ev) {
		try {
			StMenuItem menuItem = (StMenuItem) ev.getSource();
			if (menuItem.getActionArgument() == null) {
				this.perform_(ev.getActionCommand());
			} else {
				this.perform_with_(ev.getActionCommand(), menuItem.getActionArgument());
			}
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * Invoked when a checkbox item has been selected.
	 * 
	 * @param ev java.awt.event.ActionEvent
	 * @throws sra.smalltalk.SmalltalkException
	 * @category event handling
	 */
	public void itemStateChanged(ItemEvent ev) {
		String command;

		if (ev.getSource() instanceof StCheckBoxMenuItem) {
			command = ((StCheckBoxMenuItem) ev.getSource()).getActionCommand();
		} else {
			command = ((CheckboxMenuItem) ev.getSource()).getActionCommand();
		}

		try {
			this.perform_(command);
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}
	}

	/**
	 * The application is checking to see if the application is in a state such that it can be closed.
	 * 
	 * @return boolean
	 * @category events
	 */
	protected boolean requestForWindowClose() {
		return true;
	}

	/**
	 * Answer whether a window resizable or not.
	 * 
	 * @return boolean
	 * @category testing
	 */
	protected boolean windowResizable() {
		return true;
	}

	/**
	 * Create a frame of the default view and open it.
	 * 
	 * @return java.awt.Frame
	 * @category interface opening
	 */
	public Frame open() {
		return this.openView_(this.defaultView());
	}

	/**
	 * Create a frame of the view and open it.
	 * 
	 * @param aView sra.smalltalk.StView
	 * @return java.awt.Frame
	 * @category interface opening
	 */
	public Frame openView_(StView aView) {
		Frame aFrame = this.allButOpenView_(aView);
		_ShowAtMousePoint(aFrame);
		return aFrame;
	}

	/**
	 * Create a Frame for the application model.
	 * Do everything but open the frame.
	 *
	 * @return java.awt.Frame
	 * @param aView sra.smalltalk.StView
	 * @category interface opening
	 */
	protected Frame allButOpenView_(StView aView) {
		aView.model_(this);
		Frame aFrame = aView._createFrame(this);
		return aFrame;
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return this.toString();
	}

	/**
	 * Close all windows of the application.
	 * 
	 * @category interface closing
	 */
	public void closeRequest() {
		Window aWindow = this.builder().window();
		if (aWindow != null) {
			aWindow.dispatchEvent(new WindowEvent(aWindow, WindowEvent.WINDOW_CLOSING));
		}
	}

	/**
	 * Invoked when a window is in the process of being closed.
	 * 
	 * @param e java.awt.event.WindowEvent
	 * @category interface closing
	 */
	public void noticeOfWindowClose(WindowEvent e) {
		Window[] windows = this.builder().windows();
		for (int i = 0; i < windows.length; i++) {
			windows[i].dispose();
		}
	}

	/**
	 * Invoked when all windows are closed.
	 *  
	 * @category interface closing
	 */
	public void allWindowsClosed() {
		builder = null;
	}

	/**
	 * A utility method to make all windows expand and raise.
	 * 
	 * @category utilities
	 */
	public void _windowExpandAndRaise() {
		Window[] windows = this.builder().windows();
		for (int i = 0; i < windows.length; i++) {
			if (windows[i] instanceof Frame) {
				((Frame) windows[i]).setState(Frame.NORMAL);
			}
			windows[i].show();
		}
	}

	/**
	 * Answer a menu model.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 * @category resources
	 */
	public StMenuModel menuModel() {
		return null;
	}

	/**
	 * Create a StCheckBoxMenuItem and set a command.
	 * 
	 * @return sra.smalltalk.StCheckBoxMenuItem
	 * @param displayString java.lang.String
	 * @param commandString java.lang.String
	 * @category menu utilities
	 */
	protected StCheckBoxMenuItem createCheckBoxMenuItem(String displayString, String commandString) {
		return this.createCheckBoxMenuItem(displayString, commandString, false);
	}

	/**
	 * Create a StCheckBoxMenuItem and set a command.
	 * 
	 * @return sra.smalltalk.StCheckBoxMenuItem
	 * @param displayString java.lang.String
	 * @param commandString java.lang.String
	 * @param isSelected boolean
	 * @category menu utilities
	 */
	protected StCheckBoxMenuItem createCheckBoxMenuItem(String displayString, String commandString, boolean isSelected) {
		StCheckBoxMenuItem item = new StCheckBoxMenuItem(displayString, isSelected);
		item.setActionCommand(commandString);
		item.addActionListener(this);
		return item;
	}

	/**
	 * Create a StMenuItem and set a command.
	 * 
	 * @param displayString java.lang.String
	 * @param commandString java.lang.String
	 * @return sra.smalltalk.StMenuItem
	 * @category menu utilities
	 */
	protected StMenuItem createMenuItem(String displayString, String commandString) {
		StMenuItem item = new StMenuItem(displayString);
		item.setActionCommand(commandString);
		item.addActionListener(this);
		return item;
	}

	/**
	 * Create a StRadioButtonMenuItem and set a command.
	 * 
	 * @param displayString java.lang.String
	 * @param commandString java.lang.String
	 * @param radioGroup sra.smalltalk.StMenuRadioButtonGroup
	 * @return sra.smalltalk.StRadioButtonMenuItem
	 * @category menu utilities
	 */
	protected StRadioButtonMenuItem createRadioButtonMenuItem(String displayString, String commandString, StButtonGroup radioGroup) {
		StRadioButtonMenuItem item = new StRadioButtonMenuItem(displayString);
		item.setActionCommand(commandString);
		//item.addItemListener(this);
		item.addActionListener(this);
		radioGroup.add(item);
		return item;
	}

}
