package sra.smalltalk;

/**
 * StBlockValue class
 * 
 *  @author    nisinaka
 *  @created   1998/10/15 (by nisinaka)
 *  @updated   1998/11/09 (by nisinaka)
 *  @updated   2003/03/07 (by nisinaka)
 *  @version   6.0
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StBlockValue.java,v 6.0 2003/10/28 10:52:32 nisinaka Exp $
 */
public class StBlockValue extends StComputedValue {

	/** The block closure to evaluate. */
	protected StBlockClosure block = null;

	/** The array of StBlockValued, each of the object has a dependency. */
	protected StBlockValued[] arguments = null;

	/** The number of arguments block takes. */
	protected int numArgs;

	/**
	 * Create a new instance of StBlockValue.
	 *
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public StBlockValue(StBlockClosure aBlock) {
		if (aBlock.numArgs() > 0) {
			throw new IllegalArgumentException();
		}

		this.setBlock_(aBlock);
	}

	/**
	 * Create a new instance of StBlockValue and initialize it.
	 *
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @param anArray sra.smalltalk.StBlockValued[]
	 */
	public StBlockValue(StBlockClosure aBlock, StBlockValued[] anArray) {
		this.setBlock_arguments_(aBlock, anArray);
	}

	/**
	 * Create a new instance of StBlockValue and initialize it.
	 *
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @param aValued sra.smalltalk.StBlockValued
	 */
	public StBlockValue(StBlockClosure aBlock, StBlockValued aValued) {
		this.setBlock_arguments_(aBlock, new StBlockValued[] { aValued });
	}

	/**
	 * Create a new StBlockValue with specified block and arguments.
	 * 
	 * @return sra.smalltalk.StBlockValue
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @param anArray sra.smalltalk.StBlockValued[]
	 * @deprecated since StPL5.0d, use the constructor.
	 */
	public static StBlockValue Block_arguments_(StBlockClosure aBlock, StBlockValued[] anArray) {
		return new StBlockValue(aBlock, anArray);
	}

	/**
	 * Create a new StBlockValue with specified block and arguments.
	 * 
	 * @return sra.smalltalk.StBlockValue
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @param anArray sra.smalltalk.StObject[]
	 * @deprecated since StPL5.0d, use StValued interface.
	 */
	public static StBlockValue Block_arguments_(StBlockClosure aBlock, StObject[] anArray) {
		class BlockValuedObject implements StBlockValued {
			StObject object;
			protected BlockValuedObject(StObject anObject) {
				object = anObject;
			}
			public void addDependent_(DependentListener aDependentListener) {
				object.addDependent_(aDependentListener);
			}
			public Object value() {
				try {
					return object.perform_("value");
				} catch (Exception e) {
					throw new SmalltalkException(e);
				}
			}
		}

		BlockValuedObject[] valuedObjects = new BlockValuedObject[anArray.length];
		for (int i = 0; i < anArray.length; i++) {
			valuedObjects[i] = new BlockValuedObject(anArray[i]);
		}

		return new StBlockValue(aBlock, valuedObjects);
	}

	/**
	 * Compute a value for the receiver.
	 * 
	 * @return java.lang.Object
	 */
	public Object computeValue() {
		switch (numArgs) {
			case 0 :
				return block.value();
			case 1 :
				return block.value_(arguments[0].value());
			case 2 :
				return block.value_value_(arguments[0].value(), arguments[1].value());
			case 3 :
				return block.value_value_value_(arguments[0].value(), arguments[1].value(), arguments[2].value());
		}

		Object[] anArray = new StObject[Math.min(numArgs, arguments.length)];
		for (int i = 0; i < anArray.length; i++) {
			anArray[i] = arguments[i].value();
		}
		return block.valueWithArguments_(anArray);
	}

	/**
	 * Set the receiver's BlockClosure.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	protected void setBlock_(StBlockClosure aBlock) {
		block = aBlock;
		numArgs = aBlock.numArgs();
		cachedValue = UnassignedValue;
		arguments = new StBlockValued[0];
	}

	/**
	 * Set the receiver's block and the arguments.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @param anArray sra.smalltalk.StBlockValued[]
	 */
	protected void setBlock_arguments_(StBlockClosure aBlock, StBlockValued[] anArray) {
		this.setBlock_(aBlock);
		arguments = anArray;
		for (int i = 0; i < arguments.length; i++) {
			arguments[i].addDependent_(this);
		}
	}
}
