package sra.smalltalk;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

/**
 * StCheckBoxMenuItem class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/09/16 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   6.0
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StCheckBoxMenuItem.java,v 6.0 2003/10/28 10:52:32 nisinaka Exp $
 */
public class StCheckBoxMenuItem extends StMenuItem implements ItemSelectable {
	private Vector itemListeners;
	protected boolean selected = false;

	/**
	 * Creates a menuItem with text.
	 * 
	 * @param text java.lang.String
	 */
	public StCheckBoxMenuItem(String text) {
		this(text, false);
	}

	/**
	 * Creates a menuItem with text.
	 * 
	 * @param text java.lang.String
	 * @param selected boolean
	 */
	public StCheckBoxMenuItem(String text, boolean selected) {
		super(text);
		this.selected = selected;
	}

	/**
	 * adds an ItemListener to the checkbox
	 * 
	 * @param l DOCUMENT ME!
	 */
	public synchronized void addItemListener(ItemListener l) {
		if (itemListeners == null) {
			itemListeners = new Vector();
		}

		itemListeners.addElement(l);
	}

	/**
	 * Set the indicator OFF.
	 */
	public void beOff() {
		selected = false;

		StMenuItem root = getRoot();

		if (root instanceof StMenuModel) {
			((StMenuModel) root).changed_with_($("deselected"), this);
		}
	}

	/**
	 * Set the indicator ON.
	 */
	public void beOn() {
		selected = true;

		StMenuItem root = getRoot();

		if (root instanceof StMenuModel) {
			((StMenuModel) root).changed_with_($("selected"), this);
		}
	}

	/**
	 * Returns the an array (length 1) containing the checkbox menu item label
	 * or null if the checkbox is not selected.
	 * 
	 * @see ItemSelectable
	 */
	public synchronized Object[] getSelectedObjects() {
		if (isSelected()) {
			Object[] items = new Object[1];

			items[0] = label;

			return items;
		}

		return null;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean isSelected() {
		return selected;
	}

	/**
	 * removes an ItemListener from the button
	 * 
	 * @param l DOCUMENT ME!
	 */
	public void removeItemListener(ItemListener l) {
		itemListeners.removeElement(l);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBoolean boolean
	 */
	public void setSelected(boolean aBoolean) {
		this.selected_(aBoolean);
	}

	/**
	 * Invoked when an item has been selected or deselected.
	 * 
	 * @param event DOCUMENT ME!
	 */
	protected void fireItemStateChanged(ItemEvent event) {
		if (itemListeners == null) {
			return;
		}

		Vector copy;
		synchronized (itemListeners) {
			copy = (Vector) itemListeners.clone();
		}

		for (int i = 0; i < copy.size(); i++) {
			ItemListener l = (ItemListener) copy.elementAt(i);

			l.itemStateChanged(event);
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aBoolean boolean
	 */
	protected void selected_(boolean aBoolean) {
		if (selected != aBoolean) {
			selected = aBoolean;

			StSymbol aspect = selected ? $("selected") : $("deselected");
			StMenuItem root = getRoot();

			if (root instanceof StMenuModel) {
				((StMenuModel) root).changed_with_(aspect, this);
			}

			this.fireItemStateChanged(new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, this, selected ? ItemEvent.SELECTED : ItemEvent.DESELECTED));
		}
	}
}
