package sra.smalltalk;

import java.awt.*;
import java.awt.event.*;

/**
 * StController class
 * 
 *  @author    nisinaka
 *  @created   1998/12/09 (by nisinaka)
 *  @updated   1998/12/10 (by He Weijie)
 *  @updated   2002/11/11 (by nisinaka)
 *  @version   6.0
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StController.java,v 6.0 2003/10/28 10:52:32 nisinaka Exp $
 */
public class StController extends StObject implements MouseListener {
	protected StView view;

	/**
	 * Invoked when component has been hidden.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void componentHidden(ComponentEvent e) {
	}

	/**
	 * Invoked when component has been moved.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void componentMoved(ComponentEvent e) {
	}

	/**
	 * Invoked when component has been resized.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void componentResized(ComponentEvent e) {
	}

	/**
	 * Invoked when component has been shown.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void componentShown(ComponentEvent e) {
	}

	/**
	 * Answer a Point indicating the cursor location.
	 * 
	 * @return java.awt.Point
	 */
	public Point cursorPoint() {
		Point point = SystemInterface._MousePoint();
		Point viewLocation = this.view().toComponent().getLocationOnScreen();
		point.x = point.x - viewLocation.x;
		point.y = point.y - viewLocation.y;
		return point;
	}

	/**
	 * Set aPoint to be the current cursor location.
	 * 
	 * @param aPoint java.awt.Point
	 */
	public void cursorPoint_(Point aPoint) {
		Point point = this.view().toComponent().getLocationOnScreen();
		point.translate(aPoint.x, aPoint.y);
		SystemInterface._SetMousePoint(point.x, point.y);
	}

	/**
	 * Invoked when a component gains the keyboard focus.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void focusGained(FocusEvent e) {
	}

	/**
	 * Invoked when a component loses the keyboard focus.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void focusLost(FocusEvent e) {
	}

	/**
	 * Invoked when a key has been pressed.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void keyPressed(KeyEvent e) {
	}

	/**
	 * Invoked when a key has been released.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void keyReleased(KeyEvent e) {
	}

	/**
	 * Invoked when a key has been typed. This event occurs when a key press is
	 * followed by a key release.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void keyTyped(KeyEvent e) {
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	public StModel model() {
		return this.view().model();
	}

	/**
	 * Invoked when the mouse has been clicked on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 */
	public void mouseClicked(MouseEvent e) {
		if (e.isPopupTrigger()) {
			this.view()._showPopupMenu(e.getX(), e.getY());
		}
	}

	/**
	 * Invoked when a mouse button is pressed on a component and then  dragged.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void mouseDragged(MouseEvent e) {
	}

	/**
	 * Invoked when the mouse enters a component.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void mouseEntered(MouseEvent e) {
	}

	/**
	 * Invoked when the mouse exits a component.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void mouseExited(MouseEvent e) {
	}

	/**
	 * Invoked when the mouse button has been moved on a component (with no
	 * buttons no down).
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void mouseMoved(MouseEvent e) {
	}

	/**
	 * Invoked when a mouse button has been pressed on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 */
	public void mousePressed(MouseEvent e) {
		if (e.isPopupTrigger()) {
			this.view()._showPopupMenu(e.getX(), e.getY());
		}
	}

	/**
	 * Invoked when a mouse button has been released on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 */
	public void mouseReleased(MouseEvent e) {
		if (e.isPopupTrigger()) {
			this.view()._showPopupMenu(e.getX(), e.getY());
		}
	}

	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return sra.mvctest.StView
	 */
	public StView view() {
		return view;
	}

	/**
	 * Set a view.
	 * 
	 * @param aView sra.mvctest.StView
	 */
	public void view_(StView aView) {
		view = aView;
		this.buildListener(aView);
	}

	/**
	 * Add myself as a listener of the view.
	 * 
	 * @param aView sra.mvctest.StView
	 */
	protected void buildListener(StView aView) {
		((Component) aView).addMouseListener(this);
	}

	/**
	 * distinguih button event is for menu or not.
	 * 
	 * @param event DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	protected final boolean isMenuButtonPressed_(MouseEvent event) {
		return event.isMetaDown();
	}

	/**
	 * distinguih button event is for menu or not.
	 * 
	 * @param event DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	protected final boolean isShiftKeyPressed_(MouseEvent event) {
		return event.isShiftDown();
	}
}
