package sra.smalltalk;

import java.awt.*;
import java.awt.event.*;
import java.io.File;

/**
 * StDialog class
 * 
 *  @author    Hirotsugu Kondo
 *  @created   1998/11/11 (by Hirotsugu Kondo)
 *  @updated   1999/07/02 (by nisinaka)
 *  @version   6.0
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StDialog.java,v 6.0 2003/10/28 10:52:32 nisinaka Exp $
 */
public class StDialog implements ActionListener {
	protected static String lastDirectory = null;
	protected Dialog dialog;
	protected boolean accepted;
	protected String value;
	protected TextField textField;

	/**
	 * Default constructor.
	 */
	public StDialog() {
		this(SystemResourceSupport.getFrame());
	}

	/**
	 * Default constructor.
	 * 
	 * @param aFrame DOCUMENT ME!
	 */
	public StDialog(Frame aFrame) {
		super();
		dialog = new Dialog(aFrame, "", true);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString java.lang.String
	 * 
	 * @return boolean
	 */
	public static boolean Confirm_(String messageString) {
		return Confirm_(messageString, _getDefaultPoint());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString java.lang.String
	 * @param point java.awt.Point
	 * 
	 * @return boolean
	 */
	public static boolean Confirm_(String messageString, Point point) {
		return Confirm_(messageString, true, point);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString java.lang.String
	 * @param initialAnswer boolean
	 * 
	 * @return boolean
	 */
	public static boolean Confirm_(String messageString, boolean initialAnswer) {
		return Confirm_(messageString, initialAnswer, _getDefaultPoint());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString java.lang.String
	 * @param initialAnswer boolean
	 * @param point java.awt.Point
	 * 
	 * @return boolean
	 */
	public static boolean Confirm_(String messageString, boolean initialAnswer, Point point) {
		StDialog aDialog = new StDialog();

		return aDialog.confirm_(messageString, initialAnswer, point);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.io.File
	 */
	public static File OpenFileDialog() {
		return OpenFileDialog_default_("", null);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString java.lang.String
	 * 
	 * @return java.io.File
	 */
	public static File OpenFileDialog_(String messageString) {
		return OpenFileDialog_default_(messageString, null);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString java.lang.String
	 * @param defaultFilename java.lang.String
	 * 
	 * @return java.io.File
	 */
	public static File OpenFileDialog_default_(String messageString, String defaultFilename) {
		FileDialog fileDialog = new FileDialog(SystemResourceSupport.getFrame(), messageString);

		if (lastDirectory != null) {
			fileDialog.setDirectory(lastDirectory);
		}

		if ((defaultFilename != null) && (defaultFilename.length() > 0)) {
			fileDialog.setFile(defaultFilename);
		}

		fileDialog.show();

		String directory = fileDialog.getDirectory();
		String filename = fileDialog.getFile();

		if ((directory == null) || (filename == null)) {
			return null;
		}

		lastDirectory = directory;

		return new File(directory, filename);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString java.lang.String
	 * 
	 * @return java.lang.String
	 */
	public static String Request_(String messageString) {
		return Request_(messageString, _getDefaultPoint());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString java.lang.String
	 * @param point java.awt.Point
	 * 
	 * @return java.lang.String
	 */
	public static String Request_(String messageString, Point point) {
		return Request_(messageString, "", point);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString java.lang.String
	 * @param initialAnswer java.lang.String
	 * 
	 * @return java.lang.String
	 */
	public static String Request_(String messageString, String initialAnswer) {
		return Request_(messageString, initialAnswer, _getDefaultPoint());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString java.lang.String
	 * @param initialAnswer java.lang.String
	 * @param point java.awt.Point
	 * 
	 * @return java.lang.String
	 */
	public static String Request_(String messageString, String initialAnswer, Point point) {
		StDialog aDialog = new StDialog();

		return aDialog.request_(messageString, initialAnswer, point);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.io.File
	 */
	public static File SaveFileDialog() {
		return SaveFileDialog_default_("", null);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString java.lang.String
	 * 
	 * @return java.io.File
	 */
	public static File SaveFileDialog_(String messageString) {
		return SaveFileDialog_default_(messageString, null);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString java.lang.String
	 * @param defaultFilename java.lang.String
	 * 
	 * @return java.io.File
	 */
	public static File SaveFileDialog_default_(String messageString, String defaultFilename) {
		FileDialog fileDialog = new FileDialog(SystemResourceSupport.getFrame(), "", FileDialog.SAVE);

		if (lastDirectory != null) {
			fileDialog.setDirectory(lastDirectory);
		}

		if ((defaultFilename != null) && (defaultFilename.length() > 0)) {
			fileDialog.setFile(defaultFilename);
		}

		fileDialog.show();

		String directory = fileDialog.getDirectory();
		String filename = fileDialog.getFile();

		if ((directory == null) || (filename == null)) {
			return null;
		}

		lastDirectory = directory;

		return new File(directory, filename);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString DOCUMENT ME!
	 */
	public static void Warn_(String messageString) {
		Warn_(messageString, _getDefaultPoint());
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString DOCUMENT ME!
	 * @param point DOCUMENT ME!
	 */
	public static void Warn_(String messageString, Point point) {
		StDialog aDialog = new StDialog();

		aDialog.warn_(messageString, point);
	}

	/**
	 * Answer the center point of the screen.
	 * 
	 * @return Point
	 */
	protected static Point _getCenterPoint() {
		Dimension dimension = Toolkit.getDefaultToolkit().getScreenSize();

		return new Point(dimension.width / 2, dimension.height / 2);
	}

	/**
	 * Answer the default point to show the dialog.
	 * 
	 * @return Point
	 */
	protected static Point _getDefaultPoint() {
		Point point = null;

		try {
			point = SystemInterface._MousePoint();
		} catch (UnsatisfiedLinkError e) {
			point = new Point(0, 0);
		} catch (NoClassDefFoundError e) {
			point = new Point(0, 0);
		}

		return point;
	}

	/**
	 * Invoked when an action occurs.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void actionPerformed(ActionEvent e) {
		String buttonLabel = e.getActionCommand();

		if (buttonLabel.equals("Yes") || buttonLabel.equals("OK")) {
			accepted = true;
			value = (textField != null) ? textField.getText() : null;
		} else if (buttonLabel.equals("No") || buttonLabel.equals("Cancel")) {
			accepted = false;
		}

		dialog.dispose();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString java.lang.String
	 * @param initialAnswer boolean
	 * @param point java.awt.Point
	 * 
	 * @return boolean
	 */
	public boolean confirm_(String messageString, boolean initialAnswer, Point point) {
		dialog.add("Center", new Label(messageString, Label.CENTER));
		dialog.add("South", this._yesNoButtons());
		this._showDialog(point);

		return accepted;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString java.lang.String
	 * @param initialAnswer java.lang.String
	 * @param point java.awt.Point
	 * 
	 * @return java.lang.String
	 */
	public String request_(String messageString, String initialAnswer, Point point) {
		dialog.setLayout(new GridLayout(3, 1));
		textField = new TextField(20);
		textField.setText(initialAnswer);
		dialog.add(new Label(messageString, Label.CENTER));
		dialog.add(textField);
		dialog.add(this._okCancelButtons());
		this._showDialog(point);

		return value;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param messageString DOCUMENT ME!
	 * @param point DOCUMENT ME!
	 */
	public void warn_(String messageString, Point point) {
		dialog.add("Center", new Label(messageString, Label.CENTER));
		dialog.add("South", this._closeOKButton());
		this._showDialog(point);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.awt.Button
	 */
	protected Button _closeCancelButton() {
		Button yesButton = new Button("Cancel");

		yesButton.addActionListener(this);

		return yesButton;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.awt.Button
	 */
	protected Button _closeOKButton() {
		Button yesButton = new Button("OK");

		yesButton.addActionListener(this);

		return yesButton;
	}

	/**
	 * Answer the panel of ok/cancel buttons.
	 * 
	 * @return java.awt.Panel
	 */
	protected Panel _okCancelButtons() {
		Button yesButton = new Button("OK");
		yesButton.addActionListener(this);

		Button noButton = new Button("Cancel");
		noButton.addActionListener(this);

		Panel panel = new Panel();
		panel.setLayout(new FlowLayout());
		panel.add(yesButton);
		panel.add(noButton);

		return panel;
	}

	/**
	 * Dialog standard handling for JAVA.
	 * 
	 * @param point java.awt.Point
	 */
	protected void _showDialog(Point point) {
		//dialog.setResizable(false);
		dialog.pack();

		Rectangle bounds = dialog.getBounds();
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		int originX = Math.max(0, Math.min(point.x - (bounds.width / 2), screenSize.width - bounds.width));
		int originY = Math.max(0, Math.min(point.y - (bounds.height / 2), screenSize.height - bounds.height));

		dialog.setBounds(originX, originY, bounds.width, bounds.height);
		dialog.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				e.getWindow().dispose();
			}
		});
		dialog.show();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return java.awt.Panel
	 */
	protected Panel _yesNoButtons() {
		Button yesButton = new Button("Yes");

		yesButton.addActionListener(this);

		Button noButton = new Button("No");

		noButton.addActionListener(this);

		//
		Panel panel = new Panel();

		panel.add("West", yesButton);
		panel.add("East", noButton);

		return panel;
	}
}
