package sra.smalltalk;

import java.awt.event.*;
import java.util.Vector;

/**
 * StMenuItem class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/09/16 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   6.0
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StMenuItem.java,v 6.0 2003/10/28 10:52:32 nisinaka Exp $
 */
public class StMenuItem extends StObject {

	private Vector actionlisteners;
	protected String label;
	protected String actionCommand;
	protected Object actionArgument;
	protected boolean enabled = true;
	protected StMenu parent;

	/**
	 * Creates a menuItem with label.
	 * 
	 * @param aLabel java.lang.String
	 */
	public StMenuItem(String aLabel) {
		super();
		label = aLabel;
	}

	/**
	 * DOCUMENT ME!
	 */
	public void action() {
		fireActionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, getActionCommand()));
	}

	/**
	 * adds an ActionListener to the button
	 * 
	 * @param l DOCUMENT ME!
	 */
	public synchronized void addActionListener(ActionListener l) {
		if (actionlisteners == null) {
			actionlisteners = new Vector();
		}

		actionlisteners.addElement(l);
	}

	/**
	 * Answer my action argument.
	 *
	 * @return java.lang.Object
	 */
	public Object getActionArgument() {
		return actionArgument;
	}

	/**
	 * Returns the command string associated with this action.
	 * 
	 * @return java.lang.String
	 */
	public String getActionCommand() {
		return actionCommand;
	}

	/**
	 * Answer the label string.
	 * 
	 * @return java.lang.String
	 */
	public String getLabel() {
		return label;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return boolean
	 */
	public boolean isEnabled() {
		return enabled;
	}

	/**
	 * removes an ActionListener from the button
	 * 
	 * @param l DOCUMENT ME!
	 */
	public void removeActionListener(ActionListener l) {
		actionlisteners.removeElement(l);
	}

	/**
	 * Set my new action argument.
	 *
	 * @param newActionArgument java.lang.Object
	 */
	public void setActionArgument(Object newActionArgument) {
		actionArgument = newActionArgument;
	}

	/**
	 * Sets the command name of the action event that is fired by this menu
	 * item.
	 * 
	 * @param command java.lang.String
	 */
	public void setActionCommand(String command) {
		actionCommand = command;
	}

	/**
	 * Enable or disable the menu item.
	 * 
	 * @param aBoolean boolean
	 */
	public void setEnabled(boolean aBoolean) {
		if (enabled != aBoolean) {
			enabled = aBoolean;

			StSymbol aspect = enabled ? $("enabled") : $("disable");
			StMenuItem root = getRoot();

			if (root instanceof StMenuModel) {
				((StMenuModel) root).changed_with_(aspect, this);
			}
		}
	}

	/**
	 * Set the new label string.
	 *
	 * @param newLabel java.lang.String
	 */
	public void setLabel(String newLabel) {
		label = newLabel;
	}

	/**
	 * Invoked when a menu item has been selected.
	 * 
	 * @param event java.awt.event.ActionEvent
	 */
	protected void fireActionPerformed(ActionEvent event) {
		if (actionlisteners == null) {
			return;
		}

		Vector copy;
		synchronized (actionlisteners) {
			copy = (Vector) actionlisteners.clone();
		}

		for (int i = 0; i < copy.size(); i++) {
			ActionListener l = (ActionListener) copy.elementAt(i);

			l.actionPerformed(event);
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu getParent() {
		return parent;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @return sra.smalltalk.StMenuModel
	 */
	protected StMenuItem getRoot() {
		StMenuItem current = this;
		StMenuItem parent = getParent();

		while (parent != null) {
			current = parent;
			parent = current.getParent();
		}

		return current;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aMenu sra.smalltalk.StMenu
	 */
	protected void setParent(StMenu aMenu) {
		parent = aMenu;
	}
}
