package sra.smalltalk;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

/**
 * StMenuViewForAwt class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/09/16 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   6.0
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StMenuViewForAwt.java,v 6.0 2003/10/28 10:52:32 nisinaka Exp $
 */
public class StMenuViewForAwt implements DependentListener, ActionListener, ItemListener {

	private transient boolean isBroadcasting = false;
	protected StMenu stRootMenu;
	protected MenuComponent uiRootMenu;
	protected Vector stMenus;
	protected Vector uiMenus;

	/**
	 * Create a new instance of StMenuViewForAwt and initialize it with a MenuBar.
	 * 
	 * @param menuBar java.awt.MenuBar
	 */
	public StMenuViewForAwt(MenuBar menuBar) {
		super();
		uiRootMenu = menuBar;
	}

	/**
	 * Create a new instance of StMenuViewForAwt and initialize it with a PopupMenu.
	 * 
	 * @param popupMenu java.awt.PopupMenu
	 */
	public StMenuViewForAwt(PopupMenu popupMenu) {
		super();
		uiRootMenu = popupMenu;
	}

	/**
	 * Replacement for the # notation of Smalltalk. :-)
	 * 
	 * @return sra.smalltalk.StSymbol
	 * @param aString java.lang.String
	 */
	public static final StSymbol $(String aString) {
		return StSymbol.Intern_(aString);
	}

	/**
	 * Create a frame for the application model with the view.
	 * 
	 * @return java.awt.Frame
	 * @param anApplicatonModel sra.smalltalk.StApplicationModel
	 * @param aView sra.smalltalk.StView
	 */
	public static final Frame CreateFrame(StApplicationModel anApplicatonModel, StView aView) {
		Frame aFrame = new Frame(anApplicatonModel.windowTitle());
		aFrame.add((Component) aView);

		// A workaround for a problem with Java2 SDK 1.3.1 & 1.4.1 on Linux.
		// If a window contains a TextField, the window is tried to doLayout() with its size (0, 0)
		// when the peer of the TextField is created and addNotify() of the TextField is called.
		// As a result, the content of the window will be resized to be fit on a window which size is (0, 0).
		Insets insets = aFrame.getInsets();
		Dimension size = aView.toComponent().getSize();
		aFrame.setSize(size.width + insets.left + insets.right, size.height + insets.top + insets.bottom);

		aFrame.pack();

		StMenuModel menuModel = anApplicatonModel.menuModel();
		if (menuModel != null) {
			MenuBar aMenuBar = new MenuBar();
			StMenuViewForAwt aMenuView = new StMenuViewForAwt(aMenuBar);
			aMenuView.setModel(menuModel);
			aFrame.setMenuBar(aMenuBar);
		}

		if (anApplicatonModel.windowResizable() == false) {
			aFrame.setResizable(false);
		}

		aFrame.pack();
		anApplicatonModel.builder().addWindow_(aFrame, aView);

		return aFrame;
	}

	/**
	 * Create a popup menu from StMenu.
	 * 
	 * @return java.awt.PopupMenu
	 * @param menu sra.smalltalk.StMenu
	 */
	public static final PopupMenu CreatePopupMenu(StMenu menu) {
		PopupMenu popupMenu = new PopupMenu();
		StMenuViewForAwt aMenuView = new StMenuViewForAwt(popupMenu);
		aMenuView.setModel(menu);
		return popupMenu;
	}

	/**
	 * Invoked when a menu item has been selected.
	 * 
	 * @param ev java.awt.event.ActionEvent
	 */
	public void actionPerformed(ActionEvent ev) {
		StMenuItem anItem = getMenu((MenuItem) ev.getSource());
		anItem.action();
	}

	/**
	 * Answer a model.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	public StMenu getModel() {
		return stRootMenu;
	}

	/**
	 * Invoked when an item has been selected or deselected.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void itemStateChanged(ItemEvent e) {
		if (isBroadcasting) {
			return;
		}

		StMenuItem menuItem = getMenu((MenuItem) e.getSource());
		boolean selected = (e.getStateChange() == e.SELECTED);
		ActionEvent action = new ActionEvent(e.getSource(), e.getID(), menuItem.getActionCommand());

		if (menuItem instanceof StRadioButtonMenuItem) {
			if (selected) {
				StRadioButtonMenuItem anItem = (StRadioButtonMenuItem) menuItem;
				anItem.setSelected(selected);
				anItem.action();
			} else {
				((CheckboxMenuItem) e.getSource()).setState(true);
			}
		} else {
			// StCheckBoxMenuItem
			StCheckBoxMenuItem anItem = (StCheckBoxMenuItem) menuItem;
			anItem.setSelected(selected);
			anItem.action();
		}
	}

	/**
	 * Set a model.
	 * 
	 * @param aModel sra.smalltalk.StMenuModel
	 */
	public void setModel(StMenu aModel) {
		stRootMenu = aModel;
		((StMenu) stRootMenu.getRoot()).addDependentListener(this);
		this.rebuildComponent();
	}

	/**
	 * Called when one of the dependents has been changed.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		StMenuItem stMenu = (StMenuItem) evt.getParameter();
		MenuItem awtMenu = getMenu(stMenu);
		StSymbol aspect = evt.getAspect();
		if (aspect == $("selected")) {
			isBroadcasting = true;
			((CheckboxMenuItem) awtMenu).setState(true);
			isBroadcasting = false;
		} else if (aspect == $("deselected")) {
			isBroadcasting = true;
			((CheckboxMenuItem) awtMenu).setState(false);
			isBroadcasting = false;
		} else if (aspect == $("enabled")) {
			awtMenu.setEnabled(true);
		} else if (aspect == $("disable")) {
			awtMenu.setEnabled(false);
		} else if (aspect == $("added")) {
			Menu menu = new Menu(stMenu.label);
			addMenu(stMenu, menu);
			makeSubMenu((StMenu) stMenu, menu);
			((Menu) getMenu(stMenu.getParent())).add(menu);
		}
	}

	/**
	 * Add a menu.
	 * 
	 * @param stMenu sra.smalltalk.StMenuItem
	 * @param uiMenu java.awt.MenuItem
	 */
	protected void addMenu(StMenuItem stMenu, MenuItem uiMenu) {
		stMenus.addElement(stMenu);
		uiMenus.addElement(uiMenu);
	}

	/**
	 * Clear the internal information.
	 */
	protected void clearMenuTable() {
		stMenus = new Vector();
		uiMenus = new Vector();
	}

	/**
	 * Answer the StMenuItem corresponds to the MenuItem.
	 * 
	 * @return sra.smalltalk.StMenuItem
	 * @param uiMenu
	 */
	protected StMenuItem getMenu(MenuItem uiMenu) {
		int i = uiMenus.indexOf(uiMenu);
		return (i < 0) ? null : (StMenuItem) stMenus.elementAt(i);
	}

	/**
	 * Answer the MenuItem corresponds to the StMenuItem.
	 * 
	 * @return java.awt.MenuItem
	 * @param stMenu sra.smalltalk.StMenuItem
	 */
	protected MenuItem getMenu(StMenuItem stMenu) {
		int i = stMenus.indexOf(stMenu);
		return (i < 0) ? null : (MenuItem) uiMenus.elementAt(i);
	}

	/**
	 * Create the sub menu.
	 * 
	 * @param stMenu sra.smalltalk.StMenu
	 * @param awtMenu java.awt.Menu
	 */
	protected void makeSubMenu(StMenu stMenu, Menu awtMenu) {
		for (int i = 0; i < stMenu.getItems().size(); i++) {
			StMenuItem stSubMenu = stMenu.getItem(i);
			if (stSubMenu == null) {
				// Separator
				awtMenu.addSeparator();
			} else if (stSubMenu instanceof StMenu) {
				Menu awtSubMenu = new Menu(stSubMenu.label);
				awtMenu.add(awtSubMenu);
				this.addMenu(stSubMenu, awtSubMenu);
				this.makeSubMenu((StMenu) stSubMenu, awtSubMenu);
			} else if (stSubMenu instanceof StRadioButtonMenuItem) {
				boolean isSelected = ((StRadioButtonMenuItem) stSubMenu).isSelected();
				CheckboxMenuItem awtSubMenu = new CheckboxMenuItem(stSubMenu.label, isSelected);
				awtSubMenu.setEnabled(stSubMenu.isEnabled());
				awtSubMenu.setActionCommand(stSubMenu.getActionCommand());
				awtSubMenu.addItemListener(this);
				awtMenu.add(awtSubMenu);
				this.addMenu(stSubMenu, awtSubMenu);
			} else if (stSubMenu instanceof StCheckBoxMenuItem) {
				boolean isSelected = ((StCheckBoxMenuItem) stSubMenu).isSelected();
				CheckboxMenuItem awtSubMenu = new CheckboxMenuItem(stSubMenu.label, isSelected);
				awtSubMenu.setEnabled(stSubMenu.isEnabled());
				awtSubMenu.setActionCommand(stSubMenu.getActionCommand());
				awtSubMenu.addItemListener(this);
				awtMenu.add(awtSubMenu);
				this.addMenu(stSubMenu, awtSubMenu);
			} else {
				MenuItem awtSubMenu = new MenuItem(stSubMenu.label);
				awtSubMenu.setEnabled(stSubMenu.isEnabled());
				awtSubMenu.setActionCommand(stSubMenu.getActionCommand());
				awtSubMenu.addActionListener(this);
				awtMenu.add(awtSubMenu);
				this.addMenu(stSubMenu, awtSubMenu);
			}
		}
	}

	/**
	 * Rebuild this component.
	 */
	protected void rebuildComponent() {
		this.clearMenuTable();

		if (uiRootMenu instanceof PopupMenu) {
			this.makeSubMenu(this.getModel(), (PopupMenu) uiRootMenu);
		} else {
			for (int i = 0; i < getModel().getItems().size(); i++) {
				StMenu stMenu = (StMenu) this.getModel().getItem(i);
				Menu awtMenu = new Menu(stMenu.label);
				((MenuBar) uiRootMenu).add(awtMenu);
				this.addMenu(stMenu, awtMenu);
				this.makeSubMenu(stMenu, awtMenu);
			}
		}
	}
}
