package sra.smalltalk;

import java.awt.event.*;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.*;

/**
 * StMenuViewForSwing class
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/10/01 (by MATSUDA Ryouichi)
 *  @updated   N/A
 *  @version   6.0
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StMenuViewForSwing.java,v 6.0 2003/10/28 10:52:32 nisinaka Exp $
 */
public class StMenuViewForSwing implements DependentListener, ActionListener, ItemListener {
	private transient boolean isBroadcasting = false;
	protected StMenu stRootMenu;
	protected JComponent uiRootMenu;
	protected Vector stMenus;
	protected Vector uiMenus;
	protected Hashtable groups;

	/**
	 * DOCUMENT ME!
	 * 
	 * @param menuBar javax.swing.JMenuBar
	 */
	public StMenuViewForSwing(JMenuBar menuBar) {
		super();
		uiRootMenu = menuBar;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param popupMenu javax.swing.JPopupMenu
	 */
	public StMenuViewForSwing(JPopupMenu popupMenu) {
		super();
		uiRootMenu = popupMenu;
	}

	/**
	 * Replacement for the # notation of Smalltalk. :-)
	 * 
	 * @param aString java.lang.String
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public static final StSymbol $(String aString) {
		return StSymbol.Intern_(aString);
	}

	/**
	 * Create a frame for the application model with the view.
	 * 
	 * @return javax.swing.JFrame
	 * @param anApplicatonModel sra.smalltalk.StApplicationModel
	 * @param aView sra.smalltalk.StView
	 */
	public static final JFrame CreateJFrame(StApplicationModel anApplicatonModel, StView aView) {
		JFrame aFrame = new JFrame(anApplicatonModel.windowTitle());
		aFrame.getContentPane().add((JComponent) aView);
		aFrame.pack();

		StMenuModel menuModel = anApplicatonModel.menuModel();
		if (menuModel != null) {
			JMenuBar aMenuBar = new JMenuBar();
			StMenuViewForSwing aMenuView = new StMenuViewForSwing(aMenuBar);
			aMenuView.setModel(menuModel);
			aFrame.setJMenuBar(aMenuBar);
		}

		if (anApplicatonModel.windowResizable() == false) {
			aFrame.setResizable(false);
		}

		aFrame.pack();
		aFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		anApplicatonModel.builder().addWindow_(aFrame, aView);

		return aFrame;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param menu sra.smalltalk.StMenu
	 * 
	 * @return javax.swing.JPopupMenu
	 */
	public static final JPopupMenu CreateJPopupMenu(StMenu menu) {
		JPopupMenu popupMenu = new JPopupMenu();
		StMenuViewForSwing aMenuView = new StMenuViewForSwing(popupMenu);

		aMenuView.setModel(menu);

		return popupMenu;
	}

	/**
	 * Invoked when a menu item has been selected.
	 * 
	 * @param ev java.awt.event.ActionEvent
	 */
	public void actionPerformed(ActionEvent ev) {
		StMenuItem anItem = getMenu((JMenuItem) ev.getSource());

		anItem.action();
	}

	/**
	 * Answer a model.
	 * 
	 * @return sra.smalltalk.StMenu
	 */
	public StMenu getModel() {
		return stRootMenu;
	}

	/**
	 * Invoked when an item has been selected or deselected.
	 * 
	 * @param e DOCUMENT ME!
	 */
	public void itemStateChanged(ItemEvent e) {
		if (isBroadcasting) {
			return;
		}

		StMenuItem menuItem = getMenu((JMenuItem) e.getSource());
		boolean selected = (e.getStateChange() == e.SELECTED);
		ActionEvent action = new ActionEvent(e.getSource(), e.getID(), menuItem.getActionCommand());

		if (menuItem instanceof StRadioButtonMenuItem) {
			StRadioButtonMenuItem anItem = (StRadioButtonMenuItem) menuItem;

			anItem.selected_(selected);
		} else {
			// StCheckBoxMenuItem
			StCheckBoxMenuItem anItem = (StCheckBoxMenuItem) menuItem;

			anItem.selected_(selected);
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aModel sra.smalltalk.StMenu
	 */
	public void setModel(StMenu aModel) {
		stRootMenu = aModel;
		((StMenu) stRootMenu.getRoot()).addDependentListener(this);
		this.rebuildComponent();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		StMenuItem stMenu = (StMenuItem) evt.getParameter();
		JMenuItem uiMenu = getMenu(stMenu);
		StSymbol aspect = evt.getAspect();

		if (aspect == $("selected")) {
			isBroadcasting = true;
			uiMenu.setSelected(true);
			isBroadcasting = false;
		} else if (aspect == $("deselected")) {
			isBroadcasting = true;
			uiMenu.setSelected(false);
			isBroadcasting = false;
		} else if (aspect == $("enabled")) {
			uiMenu.setEnabled(true);
		} else if (aspect == $("disable")) {
			uiMenu.setEnabled(false);
		} else if (aspect == $("added")) {
			JMenu menu = new JMenu(stMenu.label);

			addMenu(stMenu, menu);
			makeSubMenu((StMenu) stMenu, menu);
			((JMenu) getMenu(stMenu.getParent())).add(menu);
		}
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param stMenu sra.smalltalk.StMenuItem
	 * @param uiMenu javax.swing.JMenuItem
	 */
	protected void addMenu(StMenuItem stMenu, JMenuItem uiMenu) {
		stMenus.addElement(stMenu);
		uiMenus.addElement(uiMenu);
	}

	/**
	 * DOCUMENT ME!
	 */
	protected void clearMenuTable() {
		stMenus = new Vector();
		uiMenus = new Vector();
		groups = new Hashtable();
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param stGroup sra.smalltalk.StMenuItem
	 * 
	 * @return javax.swing.JMenuItem
	 */
	protected ButtonGroup getGroup(StButtonGroup stGroup) {
		ButtonGroup uiGroup = (ButtonGroup) groups.get(stGroup);

		if (uiGroup == null) {
			uiGroup = new ButtonGroup();
			groups.put(stGroup, uiGroup);
		}

		return uiGroup;
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param uiMenu
	 * 
	 * @return sra.smalltalk.StMenuItem
	 */
	protected StMenuItem getMenu(JMenuItem uiMenu) {
		int i = uiMenus.indexOf(uiMenu);

		return (i < 0) ? null : (StMenuItem) stMenus.elementAt(i);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param stMenu sra.smalltalk.StMenuItem
	 * 
	 * @return javax.swing.JMenuItem
	 */
	protected JMenuItem getMenu(StMenuItem stMenu) {
		int i = stMenus.indexOf(stMenu);

		return (i < 0) ? null : (JMenuItem) uiMenus.elementAt(i);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param stMenu sra.smalltalk.StMenu
	 * @param uiMenu javax.swing.JMenu
	 */
	protected void makeSubMenu(StMenu stMenu, JMenu uiMenu) {
		for (int i = 0; i < stMenu.getItems().size(); i++) {
			StMenuItem stSubMenu = stMenu.getItem(i);

			if (stSubMenu == null) {
				// Separator
				uiMenu.addSeparator();
			} else if (stSubMenu instanceof StMenu) {
				JMenu uiSubMenu = new JMenu(stSubMenu.label);

				uiMenu.add(uiSubMenu);
				addMenu(stSubMenu, uiSubMenu);

				//
				makeSubMenu((StMenu) stSubMenu, uiSubMenu);
			} else if (stSubMenu instanceof StRadioButtonMenuItem) {
				boolean isSelected = ((StRadioButtonMenuItem) stSubMenu).isSelected();
				JRadioButtonMenuItem uiSubMenu = new JRadioButtonMenuItem(stSubMenu.label, isSelected);

				uiSubMenu.setEnabled(stSubMenu.isEnabled());

				StButtonGroup stGroup = ((StRadioButtonMenuItem) stSubMenu).getGroup();

				getGroup(stGroup).add(uiSubMenu);
				uiSubMenu.setActionCommand(stSubMenu.getActionCommand());
				uiSubMenu.addItemListener(this);
				uiSubMenu.addActionListener(this);
				uiMenu.add(uiSubMenu);
				addMenu(stSubMenu, uiSubMenu);
			} else if (stSubMenu instanceof StCheckBoxMenuItem) {
				boolean isSelected = ((StCheckBoxMenuItem) stSubMenu).isSelected();
				JCheckBoxMenuItem uiSubMenu = new JCheckBoxMenuItem(stSubMenu.label, isSelected);

				uiSubMenu.setEnabled(stSubMenu.isEnabled());
				uiSubMenu.setActionCommand(stSubMenu.getActionCommand());
				uiSubMenu.addItemListener(this);
				uiSubMenu.addActionListener(this);
				uiMenu.add(uiSubMenu);
				addMenu(stSubMenu, uiSubMenu);
			} else {
				JMenuItem uiSubMenu = new JMenuItem(stSubMenu.label);

				uiSubMenu.setEnabled(stSubMenu.isEnabled());
				uiSubMenu.setActionCommand(stSubMenu.getActionCommand());
				uiSubMenu.addActionListener(this);
				uiMenu.add(uiSubMenu);
				addMenu(stSubMenu, uiSubMenu);
			}
		}
	}

	/**
	 * Rebuild this component.
	 */
	protected void rebuildComponent() {
		this.clearMenuTable();

		if (uiRootMenu instanceof JPopupMenu) {
			JPopupMenu uiMenu = (JPopupMenu) uiRootMenu;

			for (int i = 0; i < this.getModel().getItems().size(); i++) {
				StMenuItem stSubMenu = this.getModel().getItem(i);

				if (stSubMenu == null) {
					// Separator
					uiMenu.addSeparator();
				} else if (stSubMenu instanceof StMenu) {
					JMenu uiSubMenu = new JMenu(stSubMenu.label);

					uiMenu.add(uiSubMenu);
					addMenu(stSubMenu, uiSubMenu);

					//
					this.makeSubMenu((StMenu) stSubMenu, uiSubMenu);
				} else if (stSubMenu instanceof StRadioButtonMenuItem) {
					boolean isSelected = ((StRadioButtonMenuItem) stSubMenu).isSelected();
					JRadioButtonMenuItem uiSubMenu = new JRadioButtonMenuItem(stSubMenu.label, isSelected);
					StButtonGroup stGroup = ((StRadioButtonMenuItem) stSubMenu).getGroup();

					this.getGroup(stGroup).add(uiSubMenu);
					uiSubMenu.setActionCommand(stSubMenu.getActionCommand());
					uiSubMenu.addItemListener(this);
					uiSubMenu.addActionListener(this);
					uiMenu.add(uiSubMenu);
					this.addMenu(stSubMenu, uiSubMenu);
				} else if (stSubMenu instanceof StCheckBoxMenuItem) {
					boolean isSelected = ((StCheckBoxMenuItem) stSubMenu).isSelected();
					JCheckBoxMenuItem uiSubMenu = new JCheckBoxMenuItem(stSubMenu.label, isSelected);

					uiSubMenu.setEnabled(stSubMenu.isEnabled());
					uiSubMenu.setActionCommand(stSubMenu.getActionCommand());
					uiSubMenu.addItemListener(this);
					uiSubMenu.addActionListener(this);
					uiMenu.add(uiSubMenu);
					this.addMenu(stSubMenu, uiSubMenu);
				} else {
					JMenuItem uiSubMenu = new JMenuItem(stSubMenu.label);

					uiSubMenu.setEnabled(stSubMenu.isEnabled());
					uiSubMenu.setActionCommand(stSubMenu.getActionCommand());
					uiSubMenu.addActionListener(this);
					uiMenu.add(uiSubMenu);
					this.addMenu(stSubMenu, uiSubMenu);
				}
			}
		} else {
			for (int i = 0; i < getModel().getItems().size(); i++) {
				StMenu stMenu = (StMenu) getModel().getItem(i);
				JMenu uiMenu = new JMenu(stMenu.label);

				((JMenuBar) uiRootMenu).add(uiMenu);
				this.addMenu(stMenu, uiMenu);
				this.makeSubMenu(stMenu, uiMenu);
			}
		}
	}
}
