package sra.smalltalk;

import java.awt.*;
import java.awt.event.WindowEvent;

/**
 * StSimpleDialog class
 * 
 *  @author    nisinaka
 *  @created   2003/03/07 (by nisinaka)
 *  @updated   N/A
 *  @version   6.0
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StSimpleDialog.java,v 6.0 2003/10/28 10:52:32 nisinaka Exp $
 */
public class StSimpleDialog extends StApplicationModel {

	protected StValueHolder accept;
	protected StValueHolder cancel;
	protected StValueHolder close;
	protected boolean escapeIsCancel;

	/**
	 * StSimpleDialog constructor comment.
	 */
	public StSimpleDialog() {
		super();
	}

	/**
	 * Answer the value holder for the accept status.
	 *
	 * @return sra.smalltalk.StValueHolder
	 */
	public StValueHolder accept() {
		return accept;
	}

	/**
	 * Answer the value holder for the cancel status.
	 *
	 * @return sra.smalltalk.StValueHolder
	 */
	public StValueHolder cancel() {
		return cancel;
	}

	/**
	 * An utility method to open a dialog.
	 * Ask the user a question.
	 * Let the user pick from a row of buttons made up to match the labels collection.
	 * Return the response from the corresponding item from the values collection.
	 *
	 * @return java.lang.Object
	 * @param messageString java.lang.String
	 * @param labels java.lang.String[]
	 * @param values java.lang.Object[]
	 * @param defaultValue java.lang.Object
	 */
	public Object choose_labels_values_default_(String messageString, String[] labels, Object[] values, Object defaultValue) {
		return this.choose_labels_values_default_equalize_(messageString, labels, values, defaultValue, true);
	}

	/**
	 * An utility method to open a dialog.
	 * Ask the user a question.
	 * Let the user pick from a row of buttons made up to match the labels collection.
	 * Return the response from the corresponding item from the values collection.
	 *
	 * @return java.lang.Object
	 * @param messageString java.lang.String
	 * @param labels java.lang.String[]
	 * @param values java.lang.Object[]
	 * @param defaultValue java.lang.Object
	 * @param eqBoolean boolean
	 */
	public Object choose_labels_values_default_equalize_(String messageString, String[] labels, Object[] values, Object defaultValue, boolean eqBoolean) {
		this.escapeIsCancel_(labels.length < 2);
		StValueHolder result = new StValueHolder();
		if (labels.length == 1) {
			result.value_(defaultValue);
		}

		StSimpleDialogView aSimpleDialogView = (StSimpleDialogView) this.defaultView();
		aSimpleDialogView._initializeToChoose(messageString, labels, values, defaultValue, result, eqBoolean);
		Dialog aDialog = aSimpleDialogView.defaultDialog();
		this.builder().addWindow_(aDialog, aSimpleDialogView);
		this.preOpen();
		_ShowAtMousePoint(aDialog);
		return result.value();
	}

	/**
	 * Close the dialog.
	 */
	public void close() {
		close.value_(true);

		// This is done by ApplicationDialogController indirectly in Smalltalk by checking isFinishedWithEvent:.
		super.noticeOfWindowClose(null);
	}

	/**
	 * Called when the accept value is changed.
	 */
	public void closeAccept() {
		if (accept._booleanValue()) {
			if (this.requestForWindowClose() == false) {
				accept.value_(false);
				return;
			}
			this.close();
		}
	}

	/**
	 * Called when the accept value is changed.
	 */
	public void closeCancel() {
		this.close();
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new StSimpleDialogViewAwt(this) {
				public Insets getInsets() {
					return new Insets(5, 5, 5, 5);
				}
			};
		} else {
			return new StSimpleDialogViewSwing(this) {
				public Insets getInsets() {
					return new Insets(5, 5, 5, 5);
				}
			};
		}
	}

	/**
	 * Control whether the escape character cancels the dialog.
	 *
	 * @param aBoolean boolean
	 */
	public void escapeIsCancel_(boolean aBoolean) {
		escapeIsCancel = aBoolean;
	}

	/**
	 * Open a dialog.
	 *
	 * @return java.lang.Object
	 */
	public Object openDialog() {
		Dialog aDialog = this.allButOpenDialog();
		_ShowAtMousePoint(aDialog);
		return accept.value();
	}

	/**
	 * An utility method to open a dialog.
	 * Ask the user to enter a string and return it.
	 *
	 * @return java.lang.Object
	 * @param messageString java.lang.String
	 * @param aString java.lang.String
	 * @param aBlockOrNull sra.smalltalk.StBlockClosure
	 */
	public Object request_initialAnswer_onCancel_(String messageString, String aString, StBlockClosure aBlockOrNull) {
		StValueHolder string = new StValueHolder(aString);

		StSimpleDialogView aSimpleDialogView = (StSimpleDialogView) this.defaultView();
		aSimpleDialogView._initializeToRequest(messageString, string);
		Dialog aDialog = aSimpleDialogView.defaultDialog();
		this.builder().addWindow_(aDialog, aSimpleDialogView);
		this.preOpen();
		_ShowAtMousePoint(aDialog);

		Object result = accept._booleanValue() ? string.value() : (aBlockOrNull == null ? new String() : aBlockOrNull.value());
		return result;
	}

	/**
	 * Invoked when a window is in the process of being closed.
	 * Replacement for the ApplicationDialogController>>closeNoTerminate in Smalltalk.
	 * 
	 * @param e java.awt.event.WindowEvent
	 * @see sra.smalltalk.StApplicationModel#noticeOfWindowClose()
	 * @category interface closing
	 */
	public void noticeOfWindowClose(WindowEvent e) {
		super.noticeOfWindowClose(e);

		this.simulateEscapeKey();
	}

	/**
	 * Answer the string for the accept button label.
	 *
	 * @return java.lang.String
	 */
	protected String _acceptButtonLabelString() {
		return "OK";
	}

	/**
	 * Answer the string for the cancel button label.
	 *
	 * @return java.lang.String
	 */
	protected String _cancelButtonLabelString() {
		return "Cancel";
	}

	/**
	 * Do everything short of actually opening the window.
	 *
	 * @return java.awt.Dialog
	 */
	protected Dialog allButOpenDialog() {
		StSimpleDialogView aDialogView = (StSimpleDialogView) this.defaultView();
		Dialog aDialog = aDialogView.defaultDialog();
		this.builder().addWindow_(aDialog, aDialogView);
		this.preOpen();
		return aDialog;
	}

	/**
	 * Do the cancellation.
	 */
	protected void doCancel() {
		cancel.value_(true);
	}

	/**
	 * Initialize the ApplicationModel when created.
	 */
	protected void initialize() {
		super.initialize();

		accept = new StValueHolder(false);
		cancel = new StValueHolder(false);
		close = new StValueHolder(false);
		escapeIsCancel = true;
	}

	/**
	 * Do everything short of actually opening the window.
	 */
	protected void preOpen() {

		accept.onChangeSend_to_("closeAccept", this);
		cancel.onChangeSend_to_("closeCancel", this);
	}

	/**
	 * Simulate as if the escape key is pressed.
	 */
	protected void simulateEscapeKey() {
		if (escapeIsCancel) {
			this.doCancel();
		}
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "";
	}
}
