package sra.smalltalk;

import java.awt.*;
import java.awt.event.*;

/**
 * StSimpleDialogViewAwt class
 * 
 *  @author    nisinaka
 *  @created   2003/03/17 (by nisinaka)
 *  @updated   N/A
 *  @version   6.0
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StSimpleDialogViewAwt.java,v 6.0 2003/10/28 10:52:32 nisinaka Exp $
 */
public class StSimpleDialogViewAwt extends StViewPanel implements StSimpleDialogView {

	protected Button defaultButton;

	/**
	 * Create a new StSimpleDialogViewAwt and initialize it.
	 *
	 * @param aSimpleDialog sra.smalltalk.StSimpleDialog
	 */
	public StSimpleDialogViewAwt(StSimpleDialog aSimpleDialog) {
		super(aSimpleDialog);
	}

	/**
	 * Initialize and build the components for choosing.
	 *
	 * @param messageString java.lang.String
	 * @param labels java.lang.String[]
	 * @param values java.lang.Object[]
	 * @param defaultValue java.lang.Object
	 * @param result sra.smalltalk.StValueHolder
	 * @param eqBoolean boolean
	 */
	public void _initializeToChoose(final String messageString, String[] labels, Object[] values, Object defaultValue, final StValueHolder result, boolean eqBoolean) {
		this.removeAll();

		Canvas messageCanvas = new Canvas() {
			StComposedText composedText = new StComposedText(messageString);
			public Dimension getPreferredSize() {
				return composedText.extent();
			}
			public void paint(Graphics g) {
				super.paint(g);
				g.setColor(Color.black);
				int x = (this.getWidth() - composedText.width()) / 2;
				int y = (this.getHeight() - composedText.height()) / 2;
				composedText.displayOn_at_(g, new Point(x, y));
			}
		};

		Panel buttonPanel = new Panel();
		buttonPanel.setLayout(new FlowLayout());
		for (int i = 0; i < labels.length; i++) {
			Button aButton = new Button(labels[i]);
			final Object value = values[i];
			if (defaultValue == null) {
				if (value == null) {
					defaultButton = aButton;
				}
			} else if (defaultValue.equals(value)) {
				defaultButton = aButton;
			}
			aButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					result.value_(value);
					getSimpleDialog().close();
				}
			});
			buttonPanel.add(aButton);
		}

		this.setLayout(new GridLayout(2, 1));
		this.add(messageCanvas, BorderLayout.CENTER);
		this.add(buttonPanel, BorderLayout.SOUTH);
	}

	/**
	 * Initialize and build the components for requesting.
	 *
	 * @param messageString java.lang.String
	 * @param answerStringValue sra.smalltalk.StValueHolder
	 */
	public void _initializeToRequest(final String messageString, final StValueHolder answerStringValue) {
		this.removeAll();

		Canvas messageCanvas = new Canvas() {
			StComposedText composedText = new StComposedText(messageString);
			public Dimension getPreferredSize() {
				return composedText.extent();
			}
			public void paint(Graphics g) {
				super.paint(g);
				g.setColor(Color.black);
				int x = (this.getWidth() - composedText.width()) / 2;
				int y = (this.getHeight() - composedText.height()) / 2;
				composedText.displayOn_at_(g, new Point(x, y));
			}
		};

		final TextField textField = new TextField((String) answerStringValue.value()) {
			public Dimension getPreferredSize() {
				return new Dimension(200, this.getFont().getSize());
			}
		};
		textField.addTextListener(new TextListener() {
			public void textValueChanged(TextEvent e) {
				String newText = textField.getText();
				if (newText.equals(answerStringValue.value()) == false) {
					answerStringValue.value_(newText);
				}
			}
		});

		Panel buttonPanel = new Panel();
		buttonPanel.setLayout(new FlowLayout());
		Button acceptButton = new Button(this.getSimpleDialog()._acceptButtonLabelString());
		acceptButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				getSimpleDialog().accept().value_(true);
			}
		});
		buttonPanel.add(acceptButton);
		Button cancelButton = new Button(this.getSimpleDialog()._cancelButtonLabelString());
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				getSimpleDialog().cancel().value_(true);
			}
		});
		buttonPanel.add(cancelButton);
		defaultButton = acceptButton;

		this.setLayout(new GridLayout(3, 1));
		this.add(messageCanvas);
		this.add(textField);
		this.add(buttonPanel);
	}

	/**
	 * The view is realized, so gives the focus to the default button.
	 */
	public void addNotify() {
		super.addNotify();

		// Gives the initial focus to the default button.
		this.topComponent().addWindowListener(new WindowAdapter() {
			public void windowOpened(WindowEvent e) {
				super.windowOpened(e);

				if (defaultButton != null) {
					defaultButton.requestFocus();
				}
			}
		});
	}

	/**
	 * Answer the default dialog.
	 *
	 * @return java.awt.Dialog
	 */
	public Dialog defaultDialog() {
		Dialog aDialog = new Dialog(new Frame(), this.getSimpleDialog().windowTitle(), true);
		aDialog.add(this);

		// A workaround for a problem with Java2 SDK 1.3.1 & 1.4.1 on Linux.
		// If a dialog contains a TextField, the dialog is tried to doLayout() with its size (0, 0)
		// when the peer of the TextField is created and addNotify() of the TextField is called.
		// As a result, the content of the dialog will be resized to be fit on a dialog which size is (0, 0).
		Insets insets = aDialog.getInsets();
		Dimension size = this.getSize();
		aDialog.setSize(size.width + insets.left + insets.right, size.height + insets.top + insets.bottom);

		aDialog.pack();
		return aDialog;
	}

	/**
	 * Answer my model as StSimpleDialog.
	 *
	 * @return sra.smalltalk.StSimpleDialog
	 */
	public StSimpleDialog getSimpleDialog() {
		return (StSimpleDialog) this.model();
	}

	/**
	 * Initialize the receiver.
	 */
	protected void initialize() {
		super.initialize();
		defaultButton = null;
	}
}
