package sra.smalltalk;

import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * StSimpleDialogViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2003/03/17 (by nisinaka)
 *  @updated   N/A
 *  @version   6.0
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StSimpleDialogViewSwing.java,v 6.0 2003/10/28 10:52:32 nisinaka Exp $
 */
public class StSimpleDialogViewSwing extends StViewJPanel implements StSimpleDialogView {

	protected JButton defaultButton;

	/**
	 * Create a new StSimpleDialogViewSwing and initialize it.
	 *
	 * @param aSimpleDialog sra.smalltalk.StSimpleDialog
	 */
	public StSimpleDialogViewSwing(StSimpleDialog aSimpleDialog) {
		super(aSimpleDialog);
	}

	/**
	 * Initialize and build the components for choosing.
	 *
	 * @param messageString java.lang.String
	 * @param labels java.lang.String[]
	 * @param values java.lang.Object[]
	 * @param defaultValue java.lang.Object
	 * @param result sra.smalltalk.StValueHolder
	 * @param eqBoolean boolean
	 */
	public void _initializeToChoose(final String messageString, String[] labels, Object[] values, Object defaultValue, final StValueHolder result, boolean eqBoolean) {
		this.removeAll();

		JPanel messageCanvas = new JPanel() {
			StComposedText composedText = new StComposedText(messageString);
			public Dimension getPreferredSize() {
				return composedText.extent();
			}
			public void paint(Graphics g) {
				super.paint(g);
				g.setColor(Color.black);
				int x = (this.getWidth() - composedText.width()) / 2;
				int y = (this.getHeight() - composedText.height()) / 2;
				composedText.displayOn_at_(g, new Point(x, y));
			}
		};

		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new FlowLayout());
		for (int i = 0; i < labels.length; i++) {
			JButton aButton = new JButton(labels[i]);
			final Object value = values[i];
			if (defaultValue == null) {
				if (value == null) {
					defaultButton = aButton;
				}
			} else if (defaultValue.equals(value)) {
				defaultButton = aButton;
			}
			aButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					result.value_(value);
					getSimpleDialog().close();
				}
			});
			buttonPanel.add(aButton);
		}

		this.setLayout(new GridLayout(2, 1));
		this.add(messageCanvas, BorderLayout.CENTER);
		this.add(buttonPanel, BorderLayout.SOUTH);
	}

	/**
	 * Initialize and build the components for requesting.
	 *
	 * @param messageString java.lang.String
	 * @param answerStringValue sra.smalltalk.StValueHolder
	 */
	public void _initializeToRequest(final String messageString, final StValueHolder answerStringValue) {
		this.removeAll();

		JPanel messageCanvas = new JPanel() {
			StComposedText composedText = new StComposedText(messageString);
			public Dimension getPreferredSize() {
				return composedText.extent();
			}
			public void paint(Graphics g) {
				super.paint(g);
				g.setColor(Color.black);
				int x = (this.getWidth() - composedText.width()) / 2;
				int y = (this.getHeight() - composedText.height()) / 2;
				composedText.displayOn_at_(g, new Point(x, y));
			}
		};

		final JTextField textField = new JTextField((String) answerStringValue.value()) {
			public Dimension getPreferredSize() {
				return new Dimension(200, this.getFont().getSize());
			}
		};
		textField.addCaretListener(new CaretListener() {
			public void caretUpdate(CaretEvent e) {
				String newText = textField.getText();
				if (newText.equals(answerStringValue.value()) == false) {
					answerStringValue.value_(newText);
				}
			}
		});

		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new FlowLayout());
		JButton acceptButton = new JButton(this.getSimpleDialog()._acceptButtonLabelString());
		acceptButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				getSimpleDialog().accept().value_(true);
			}
		});
		buttonPanel.add(acceptButton);
		JButton cancelButton = new JButton(this.getSimpleDialog()._cancelButtonLabelString());
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				getSimpleDialog().cancel().value_(true);
			}
		});
		buttonPanel.add(cancelButton);
		defaultButton = acceptButton;

		this.setLayout(new GridLayout(3, 1));
		this.add(messageCanvas);
		this.add(textField);
		this.add(buttonPanel);
	}

	/**
	 * The view is realized, so gives the focus to the default button.
	 */
	public void addNotify() {
		super.addNotify();

		// Gives the initial focus to the default button.
		this.topComponent().addWindowListener(new WindowAdapter() {
			public void windowOpened(WindowEvent e) {
				super.windowOpened(e);

				if (defaultButton != null) {
					defaultButton.requestFocus();
				}
			}
		});
	}

	/**
	 * Answer the default dialog.
	 *
	 * @return java.awt.Dialog
	 */
	public Dialog defaultDialog() {
		JDialog aDialog = new JDialog(new Frame(), this.getSimpleDialog().windowTitle(), true);
		aDialog.getContentPane().add(this);
		aDialog.pack();
		return aDialog;
	}

	/**
	 * Answer my model as StSimpleDialog.
	 *
	 * @return sra.smalltalk.StSimpleDialog
	 */
	public StSimpleDialog getSimpleDialog() {
		return (StSimpleDialog) this.model();
	}

	/**
	 * Initialize the receiver.
	 */
	protected void initialize() {
		super.initialize();
		defaultButton = null;
	}
}
