package sra.smalltalk;

import java.util.Hashtable;

/**
 * StSymbol class
 * 
 *  @author    He Weijie
 *  @created   1998/09/16 (by HeWeijie)
 *  @updated   1998/11/04 (by nisinaka)
 *  @updated   1999/06/22 (by nisinaka)
 *  @version   6.0
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StSymbol.java,v 6.0 2003/10/28 10:52:32 nisinaka Exp $
 */
public class StSymbol extends StObject {
	//
	protected static Hashtable _symbols = new Hashtable();

	/** The string representation. */
	protected String _string;

	/**
	 * The instance of StSymbol should not be created with the default
	 * constructor.
	 */
	private StSymbol() {
	}

	/**
	 * Create a new instance of StSymbol.
	 * 
	 * @param aString java.lang.String
	 */
	private StSymbol(String aString) {
		_string = aString.intern();
		_symbols.put(_string, this);
	}

	/**
	 * Answer the symbol specified with the String.
	 * 
	 * @param aString
	 * 
	 * @return sra.smalltalk.StSymbol
	 */
	public static StSymbol Intern_(String aString) {
		StSymbol aSymbol = (StSymbol) _symbols.get(aString.intern());

		if (aSymbol == null) {
			aSymbol = new StSymbol(aString);
		}

		return aSymbol;
	}

	/**
	 * Answer true if the receiver is equal to the Object, otherwise false.
	 * 
	 * @param anObject java.lang.Object
	 * 
	 * @return boolean
	 */
	public boolean equals(Object anObject) {
		return (this == anObject);
	}

	/**
	 * DOCUMENT ME!
	 * 
	 * @param aSymbol sra.smalltalk.StSymbol
	 * 
	 * @return boolean
	 * 
	 * @deprecated replaced by <code>==</code>.
	 */
	public boolean isEqual(StSymbol aSymbol) {
		return (this == aSymbol);
	}

	/**
	 * Answer the size.
	 * 
	 * @return int
	 */
	public int size() {
		return _string.length();
	}

	/**
	 * Returns a string representation of the StString.
	 * 
	 * @return java.lang.String
	 */
	public String toString() {
		return _string;
	}
}
