package sra.smalltalk;

/**
 * StValueModel class
 * 
 *  @author    nisinaka
 *  @created   1998/10/09 (by nisinaka)
 *  @updated   2003/03/07 (by nisinaka)
 *  @version   6.0
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StValueModel.java,v 6.0 2003/10/28 10:52:32 nisinaka Exp $
 */
public class StValueModel extends StModel implements StBlockValued {

	/**
	 * Create a new instance and initialize it.
	 */
	public StValueModel() {
		super();
		this.initialize();
	}

	/**
	 * Convert this as StValueModel.
	 * 
	 * @return sra.smalltalk.StValueModel
	 */
	public StValueModel asValue() {
		return this;
	}

	/**
	 * Answer a StBlockValue that computes aBlock with the receiver's value as
	 * the argument.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * 
	 * @return sra.smalltalk.StBlockValue
	 */
	public StBlockValue compute_(StBlockClosure aBlock) {
		return new StBlockValue(aBlock, this);
	}

	/**
	 * Initialize the instance. Subclasses may extend this.
	 * 
	 * @return sra.smalltalk.StObject
	 */
	public StObject initialize() {
		return this;
	}

	/**
	 * Arrange to receive a message with aSymbol when the value aspect changes
	 * on anObject.
	 * 
	 * @param aSymbol java.lang.String
	 * @param anObject sra.smalltalk.StObject
	 */
	public void onChangeSend_to_(String aSymbol, StObject anObject) {
		this.expressInterestIn_for_sendBack_($("value"), anObject, aSymbol);
	}

	/**
	 * Remove references to objects that may refer to the receiver.
	 */
	public void release() {
		this.releaseParts();
		super.release();
	}

	/**
	 * Break the dependency links from any parts of myself to myself.
	 * Subclasses holding composite values will implement this in a
	 * non-trivial way.
	 */
	public void releaseParts() {
	}

	/**
	 * Undo a send of onChangeSend_to_().
	 * 
	 * @param anObject sra.smalltalk.StObject
	 */
	public void retractInterestsFor_(StObject anObject) {
		this.retractInterestIn_for_($("value"), anObject);
	}

	/**
	 * Set the currently stored value, without notifying dependents.
	 * 
	 * @param newValue java.lang.Object
	 */
	public void setValue_(Object newValue) {
		throw SmalltalkException.SubclassResponsibility();
	}

	/**
	 * Answer the currently stored value.
	 * 
	 * @return java.lang.Object
	 */
	public Object value() {
		throw SmalltalkException.SubclassResponsibility();
	}

	/**
	 * Set the currently stored value, and notify dependents.
	 * 
	 * @param newValue java.lang.Object
	 */
	public void value_(Object newValue) {
		this.setValue_(newValue);
		this.changed_($("value"));
	}
}
