package sra.smalltalk;

import java.awt.*;

/**
 * StView interface
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/06/03 (by MATSUDA Ryouichi)
 *  @updated   1999/11/11 (by MATSUDA Ryouichi)
 *  @updated   2002/01/23 (by nisinaka)
 *  @updated   2002/11/11 (by nisinaka)
 *  @version   6.0
 *  @copyright 1999-2003 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2003 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StView.java,v 6.0 2003/10/28 10:52:32 nisinaka Exp $
 */
public interface StView extends DependentListener {

	/**
	 * Create a Frame for the view.
	 *
	 * @return java.awt.Frame
	 * @param anApplicationModel sra.smalltalk.StApplicationModel
	 */
	public Frame _createFrame(StApplicationModel anApplicationModel);

	/**
	 * Show the popup menu at the specified point on the view.
	 *
	 * @param x int
	 * @param y int
	 */
	public void _showPopupMenu(int x, int y);

	/**
	 * Answer my controller.
	 * 
	 * @return sra.smalltalk.StController
	 */
	public StController controller();

	/**
	 * Set my controller.
	 * 
	 * @param newController sra.smalltalk.StController
	 */
	public void controller_(StController newController);

	/**
	 * Answer my model.
	 * 
	 * @return sra.smalltalk.StModel
	 */
	public StModel model();

	/**
	 * Set my model.
	 * 
	 * @param newModel sra.smalltalk.StModel
	 */
	public void model_(StModel newModel);

	/**
	 * Answer the view as a java component.
	 *
	 * @return java.awt.Component
	 */
	public Component toComponent();

	/**
	 * Answer the top component of the view.
	 *
	 * @return java.awt.Window
	 */
	public Window topComponent();

	/**
	 * Remove the receiver as a dependent of its model.
	 */
	public void release();

}
